#include "AReVi/Lib3D/transform3D.h"

#include "arrow.h"

AR_CLASS_DEF(Arrow, Object3D)

Arrow::Arrow(ArCW& arCW)
: Object3D(arCW),
  _link(),
  _sh(VrmlShape3D::NEW()),
  _motionFromLink(false) {
  ArRef<Transform3D> tr = Transform3D::NEW();
  tr->postScale(0.5, 0.5, 0.5);

  _sh->parseURL("arrow.wrl");
  _sh->writeTransformation(tr);

  setShape(_sh);
}

Arrow::~Arrow(void) {
}

void
Arrow::linkTo(ArRef<Base3D> base) {
  if(_link) {
    _link->removeMotionCB(thisRef(), &Arrow::_onLinkMotionCB);
  }

  _link = base;

  if(_link) {
    double x, y, z;
    _link->getPosition(x, y, z);
    setPosition(x, y, z);
    _link->addMotionCB(thisRef(), &Arrow::_onLinkMotionCB);
  }
}

ArRef<Base3D>
Arrow::getLinkedTo(void) {
  return _link;
}

ArConstRef<Shape3D>
Arrow::getShape(void) const {
  return _sh;
}

ArRef<Shape3D>
Arrow::accessShape(void) {
  return _sh;
}

void
Arrow::_onMotion(void) {
  if(!_motionFromLink && _link) {
    double x, y, z;
    getPosition(x, y, z);
    _link->setPosition(x, y, z);
  }
  Object3D::_onMotion();
}

void
Arrow::_onLinkMotionCB(const Base3D::MotionEvent& evt) {
  _motionFromLink = true;
  double x, y, z;
  evt.source->getPosition(x, y, z);
  setPosition(x, y, z);
  _motionFromLink = false;
}
