#ifndef PATH3D_H
#define PATH3D_H

#include "AReVi/arObject.h"
#include "AReVi/Lib3D/object3D.h"
#include "AReVi/Shapes/lineSet3D.h"
#include "AReVi/Shapes/shape3D.h"
#include "AReVi/VRML/vrmlShape3D.h"
using namespace AReVi;

#include "arrow.h"
#include "myObject3D.h"
#include "AReVi/Contrib/path.h"

class Path3D : public Path {
public :
  AR_CLASS(Path3D)
  AR_CONSTRUCTOR(Path3D)

  virtual
  void
  applyChanges(void);

  virtual
  void
  addWayPoint(int pos);

  virtual
  void
  removeWayPoint(int position);

  virtual
  ArConstRef<Object3D>
  getWayPoint(int position) const;

  virtual
  ArRef<Object3D>
  accessWayPoint(int position);

  virtual
  void
  setStartDerivate(const Vector3d& d);

  virtual
  void
  setEndDerivate(const Vector3d& d);

  virtual
  void
  setPrecision(int p);

  virtual
  double
  getPrecision(void) const;

  virtual
  void
  setDuration(double d);

  virtual
  double
  getDuration(void) const;

  virtual
  ArConstRef<Base3D> // base to attach stuffs
  evaluate(double p);

  //-------- Serialization operations -----------------------------

  virtual
  void
  serialize(SerializationDependencies& dependenciesOut,
            ArRef<AbstractOStream> output) const;

  virtual
  bool // success
  unserialize(const SerializationDependencies& dependencies,
              ArRef<AbstractIStream> input);

protected :
  virtual
  void
  _onWayPointTranslation(const MyObject3D::TranslationEvent& evt);

  virtual
  void
  _onArrowMotion(const Base3D::MotionEvent& evt);

  virtual
  void
  _updateLine(void);

protected :
  bool _updating;
  int _precision;

  ArRef<Object3D> _line;
  ArRef<Arrow>    _derivate[2]; // 0 => start - 1 => end
  double          _scale[2];

  ArRef<VrmlShape3D> _wayPointSh;
  ArRef<VrmlShape3D> _movingBaseSh;
  ArRef<Shape3D>     _lineSh;
  ArRef<LineSet3D>   _lineShp;

  double _duration;
};

#endif // PATH3D_H
