#include "AReVi/Contrib/File3D/file3D.h"
#include "AReVi/Lib3D/base3D.h"
#include "AReVi/Lib3D/boundingBox3D.h"
#include "AReVi/Lib3D/simpleInteractor.h"
#include "AReVi/Lib3D/transform3D.h"
#include "AReVi/Shapes/shape3D.h"
#include "AReVi/Utils/fileStream.h"
#include "AReVi/Utils/memoryBlock.h"
#include "AReVi/Utils/memoryStream.h"

#include "myObject3D.h"
#include "pathEditor.h"
#include "planeInteractor.h"

AR_CLASS_DEF(PathEditor, ArObject)

PathEditor::PathEditor(ArCW& arCW)
: ArObject(arCW),
  _sceneObj(),
  _scene(),
  _sceneFilename(),
  _selectedPath(),
  _selectedPathFilename((StlString*)0),
  _act(Activity::NEW(0.01)),
  _suspended(true) {
  _mainXml         = glade_xml_new("./patheditor.glade", "main", NULL);
  _newPathXml      = glade_xml_new("./patheditor.glade", "newPath", NULL);
  _editWayPointXml = glade_xml_new("./patheditor.glade", "editWayPoint", NULL);

  _main         = glade_xml_get_widget(_mainXml, "main");
  _newPath      = glade_xml_get_widget(_newPathXml, "newPath");
  _editWayPoint = glade_xml_get_widget(_editWayPointXml, "editWayPoint");

  _multiView  = MultiView::NEW();
  GtkWidget* rendererContainer = glade_xml_get_widget(_mainXml, "rendererContainer");
  gtk_box_pack_start(GTK_BOX(rendererContainer),
		     _multiView->getMultiViewWidget(),
		     true,
		     true,
		     4);
  gtk_box_pack_start(GTK_BOX(rendererContainer),
		     _multiView->getSingleViewWidget(),
		     true,
		     true,
		     4);
  gtk_widget_show(_multiView->getMultiViewWidget());
  _multiView->initViews();

  for(size_t i = 0; i < 5; i++) {
    if(i < 3) {
      _interactors[i] = PlaneInteractor::NEW();
    } else {
      _interactors[i] = SimpleInteractor::NEW();
    }
    _interactors[i]->setRenderer(_multiView->getWindow((MultiView::Views)i));
    _interactors[i]->setNavigation(true);
    _interactors[i]->setObjectInteraction(true);
  }

  GtkWidget* container;
  GtkTreeModel* model;
  GtkCellRenderer* renderer;
  GtkTreeViewColumn* col;

  model = GTK_TREE_MODEL(gtk_list_store_new(1, G_TYPE_STRING));
  _pathList = gtk_combo_box_new_with_model(model);
  renderer = gtk_cell_renderer_text_new();
  gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(_pathList), renderer, true);
  gtk_cell_layout_add_attribute(GTK_CELL_LAYOUT(_pathList),
				renderer, "text", 0);

  container = glade_xml_get_widget(_mainXml, "pathListContainer");
  gtk_box_pack_start_defaults(GTK_BOX(container), _pathList);
  gtk_widget_show(_pathList);

  model = GTK_TREE_MODEL(gtk_list_store_new(1, G_TYPE_STRING));
  _wayPointList = gtk_tree_view_new_with_model(model);
  gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(_wayPointList), false);
  renderer = gtk_cell_renderer_text_new();
  col = gtk_tree_view_column_new_with_attributes("Way points",
						 renderer,
						 "text",
						 0,
						 NULL);
  gtk_tree_view_append_column(GTK_TREE_VIEW(_wayPointList), col);
  container = glade_xml_get_widget(_mainXml, "wayPointTreeViewContainer");
  gtk_box_pack_start_defaults(GTK_BOX(container), _wayPointList);
  gtk_widget_show(_wayPointList);

  _camera = glade_xml_get_widget(_mainXml, "camera");

  // Main window
  g_signal_connect(_main, "delete-event",
		   G_CALLBACK(&PathEditor::_onQuit), NULL);

  // Main window -- Menus
  g_signal_connect(glade_xml_get_widget(_mainXml, "quit"), "activate",
		   G_CALLBACK(&PathEditor::_onFileQuit), this);
  g_signal_connect(glade_xml_get_widget(_mainXml, "open"), "activate",
		   G_CALLBACK(&PathEditor::_onSceneOpen), this);
  g_signal_connect(glade_xml_get_widget(_mainXml, "close"), "activate",
		   G_CALLBACK(&PathEditor::_onSceneClose), this);

  // Main window -- Path
  g_signal_connect(_pathList, "changed",
		   G_CALLBACK(&PathEditor::_onPathChanged), this);
  g_signal_connect(glade_xml_get_widget(_mainXml, "pathAdd"), "clicked",
		   G_CALLBACK(&PathEditor::_onPathAdd), this);
  g_signal_connect(glade_xml_get_widget(_mainXml, "pathRemove"), "clicked",
		   G_CALLBACK(&PathEditor::_onPathRemove), this);
  g_signal_connect(glade_xml_get_widget(_mainXml, "pathOpen"), "clicked",
		   G_CALLBACK(&PathEditor::_onPathFileOpen), this);
  g_signal_connect(glade_xml_get_widget(_mainXml, "pathFilenameSelect"), "clicked",
		   G_CALLBACK(&PathEditor::_onPathSelectFilename), this);
  g_signal_connect(glade_xml_get_widget(_mainXml, "pathSave"), "clicked",
		   G_CALLBACK(&PathEditor::_onPathFileSave), this);

  // Main window -- Control Points
  g_signal_connect(_wayPointList, "row-activated",
		   G_CALLBACK(&PathEditor::_onWayPointActivated), this);
  g_signal_connect(glade_xml_get_widget(_mainXml, "wayPointAdd"), "clicked",
		   G_CALLBACK(&PathEditor::_onWayPointAdd), this);
  g_signal_connect(glade_xml_get_widget(_mainXml, "wayPointRemove"), "clicked",
		   G_CALLBACK(&PathEditor::_onWayPointRemove), this);
  g_signal_connect(glade_xml_get_widget(_mainXml, "wayPointDrop"), "clicked",
		   G_CALLBACK(&PathEditor::_onWayPointDrop), this);
  GtkTreeSelection* selection =
    gtk_tree_view_get_selection(GTK_TREE_VIEW(_wayPointList));
  gtk_tree_selection_set_mode(selection, GTK_SELECTION_MULTIPLE);
  gtk_tree_selection_set_select_function(selection,
					 &PathEditor::_wayPointSelectFunc,
					 this,
					 NULL);

  // Main window -- Path testing
  g_signal_connect(glade_xml_get_widget(_mainXml, "precision"), "value-changed",
		   G_CALLBACK(&PathEditor::_onPrecisionChanged), this);
  g_signal_connect(glade_xml_get_widget(_mainXml, "duration"), "changed",
		   G_CALLBACK(&PathEditor::_onDurationChanged), this);
  g_signal_connect(glade_xml_get_widget(_mainXml, "stop"), "clicked",
		   G_CALLBACK(&PathEditor::_onPathStop), this);
  g_signal_connect(glade_xml_get_widget(_mainXml, "play"), "toggled",
		   G_CALLBACK(&PathEditor::_onPathTogglePlay), this);
  g_signal_connect(glade_xml_get_widget(_mainXml, "durationSlider"), "value-changed",
		   G_CALLBACK(&PathEditor::_onPathSliderChanged), this);
  g_signal_connect(glade_xml_get_widget(_mainXml, "camera"), "toggled",
		   G_CALLBACK(&PathEditor::_onPathToggleCamera), this);
  g_signal_connect(glade_xml_get_widget(_mainXml, "view"), "toggled",
		   G_CALLBACK(&PathEditor::_onPathToggleView), this);
  _act->setSuspended(true);
  _act->setBehavior(thisRef(), &PathEditor::_activate);

  // Edit way points
  g_signal_connect(glade_xml_get_widget(_editWayPointXml, "autoOrientation"), "toggled",
		   G_CALLBACK(&PathEditor::_onEditWayPointAutoOrientationToggle), this);
}

PathEditor::~PathEditor(void) {
}

void
PathEditor::_onQuit(GtkWidget*, GdkEvent*, gpointer) {
  ArSystem::leaveSimulation();
}

void
PathEditor::_onFileQuit(GtkMenuItem*, gpointer) {
  ArSystem::leaveSimulation();
}

void
PathEditor::_onSceneOpen(GtkMenuItem*, gpointer data) {
  PathEditor* pe = (PathEditor*)data;

  GtkFileFilter* vrmlFilter = gtk_file_filter_new();
  gtk_file_filter_set_name(vrmlFilter, "Fichiers Vrml");
  gtk_file_filter_add_pattern(vrmlFilter, "*.wrl");

  GtkFileFilter* troisdsFilter = gtk_file_filter_new();
  gtk_file_filter_set_name(troisdsFilter, "Fichiers 3DS");
  gtk_file_filter_add_pattern(troisdsFilter, "*.3ds");

  GtkFileFilter* objFilter = gtk_file_filter_new();
  gtk_file_filter_set_name(objFilter, "Fichiers Wavefront");
  gtk_file_filter_add_pattern(objFilter, "*.obj");

  GtkFileFilter* allFilter = gtk_file_filter_new();
  gtk_file_filter_set_name(allFilter, "Scene Sérialisée");
  gtk_file_filter_add_pattern(allFilter, "*");

  GtkWidget* fo = gtk_file_chooser_dialog_new("Open Scene",
					      NULL,
					      GTK_FILE_CHOOSER_ACTION_OPEN,
					      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
					      GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
					      NULL);
  if(pe->_sceneFilename != "") {
    gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(fo),
				  pe->_sceneFilename.c_str());
  }
  gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(fo), vrmlFilter);
  gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(fo), troisdsFilter);
  gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(fo), objFilter);
  gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(fo), allFilter);

  if(gtk_dialog_run(GTK_DIALOG(fo)) == GTK_RESPONSE_ACCEPT) {
    char* filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(fo));
    StlString ext = strExtension(filename);

    if(ext == ".wrl" ||
       ext == ".3ds" ||
       ext == ".obj") {
      ArRef<Shape3D> sh = File3D::load(filename);
      if(sh) {
	pe->_sceneObj.clear();
	pe->_sceneObj.push_back(Object3D::NEW());
	pe->_sceneObj.back()->setShape(sh);
	pe->_scene = sh;
	pe->_sceneFilename = StlString(filename);
      }
    } else {
      ArRef<FileIStream> fs = FileIStream::NEW(filename);

      SerializationDependencies dep;
      dep.readFromStream(fs);
      const StlVector<ArRef<ArObject> >& refs = dep.getReferences();

      pe->_sceneObj.clear();	
      for(size_t i = 0; i < refs.size(); i++) {
	refs[i]->unserialize(dep, fs);
	if(refs[i]->getClass()->isA(Object3D::CLASS())) {
	  pe->_sceneObj.push_back(ar_down_cast<Object3D>(refs[i]));
	}
      }
    }

    ArRef<BoundingBox3D> bbox = BoundingBox3D::NEW();
    for(size_t i = pe->_sceneObj.size(); i--;) {
      ArRef<BoundingBox3D> tmpBbox = BoundingBox3D::NEW();
      pe->_sceneObj[i]->computeBoundingBox(tmpBbox, Base3D::nullRef());
      bbox->addBox(tmpBbox);

      pe->_sceneObj[i]->setMouseButtonInteraction(false);
      pe->_sceneObj[i]->setKeyboardInteraction(false);
      pe->_sceneObj[i]->setMotionInteraction(false);
    }
    pe->_multiView->adjustViews(bbox);
    pe->_sceneFilename = StlString(filename);

    for(size_t i = 0; i < 5; i++) {
      pe->_interactors[i]->setLinearSpeed(bbox->getMaxSize() * 0.25);
    }

    g_free(filename);
  }

  gtk_widget_destroy(fo);
}

void
PathEditor::_onSceneClose(GtkMenuItem*, gpointer data) {
  PathEditor* pe = (PathEditor*)data;
  pe->_sceneObj.clear();
}

void
PathEditor::_onPathChanged(GtkComboBox*, gpointer data) {
  PathEditor* pe = (PathEditor*)data;
  int i = gtk_combo_box_get_active(GTK_COMBO_BOX(pe->_pathList));

  GtkWidget* entry;
  entry = glade_xml_get_widget(pe->_mainXml, "pathFilename");

  if(i != - 1) {
    pe->_selectedPath = pe->_path[i];
    pe->_selectedPathFilename = &pe->_pathFilename[i];
    gtk_entry_set_text(GTK_ENTRY(entry),
		       pe->_pathFilename[i].c_str());

    StlVector<ArRef<Base3D> >& wp = pe->_selectedPath->accessWayPoints();
    if(!wp.empty()) {
      ArRef<Base3D> w = pe->_multiView->getWindow(MultiView::PERSPECTIVE_VIEW);
      w->setLocation(wp[0]);
      w->cancelRotation();
      w->translate(- 1.0, 0.0, 0.0);
    }
  } else {
    pe->_selectedPath = Path3D::nullRef();
    pe->_selectedPathFilename = NULL;
    entry = glade_xml_get_widget(pe->_mainXml, "pathFilename");
    gtk_entry_set_text(GTK_ENTRY(entry), "");
  }

  pe->_refreshWayPointList();
  pe->_updateTestingTools();
}

void
PathEditor::_onPathAdd(GtkButton*, gpointer data) {
  PathEditor* pe = (PathEditor*)data;
  GtkWidget* entry = glade_xml_get_widget(pe->_newPathXml, "pathName");
  gtk_entry_set_text(GTK_ENTRY(entry), "noname");

  if(gtk_dialog_run(GTK_DIALOG(pe->_newPath)) == GTK_RESPONSE_OK) {
    const gchar* name = gtk_entry_get_text(GTK_ENTRY(entry));

    pe->_path.push_back(Path3D::NEW());
    pe->_path.back()->setName(StlString(name));
    ArRef<Object3D> mb =
      ar_down_cast<Object3D>(pe->_path.back()->accessMovingBase());
    mb->addMotionCB(ArRef<PathEditor>(pe), &PathEditor::_onMovingBaseMotion);
    pe->_interactors[MultiView::PERSPECTIVE_VIEW]->addIgnoredObject(mb);
    pe->_interactors[MultiView::PERSPECTIVE_VIEW_MULTI]->addIgnoredObject(mb);
    pe->_pathFilename.push_back(StlString());

    if(!pe->_sh.empty()) {
      StlVector<double> dist;
      StlVector<ArRef<Shape3D> > sh;


      mb->queryShapes(dist, sh);
      pe->_sh.push_back(sh.front());
      mb->setShape(Shape3D::nullRef());
    }

    GtkTreeModel* model = gtk_combo_box_get_model(GTK_COMBO_BOX(pe->_pathList));
    GtkTreeIter iter;
    gtk_list_store_append(GTK_LIST_STORE(model), &iter);
    gtk_list_store_set(GTK_LIST_STORE(model),
		       &iter,
		       0, pe->_path.back()->getName().c_str(),
		       - 1);
    gtk_combo_box_set_active(GTK_COMBO_BOX(pe->_pathList), pe->_path.size() - 1);
  }
  gtk_widget_hide(pe->_newPath);
}

void
PathEditor::_onPathRemove(GtkButton*, gpointer data) {
  PathEditor* pe = (PathEditor*)data;
  int id = gtk_combo_box_get_active(GTK_COMBO_BOX(pe->_pathList));
  if(id != - 1) {
    GtkWidget* dialog = gtk_message_dialog_new(NULL,
					       GTK_DIALOG_MODAL,
					       GTK_MESSAGE_WARNING,
					       GTK_BUTTONS_OK_CANCEL,
					       "Supprimer le chemin %s ?",
					       pe->_path[id]->getName().c_str());
    if(gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_OK) {
      GtkTreeModel* model = gtk_combo_box_get_model(GTK_COMBO_BOX(pe->_pathList));
      GtkTreeIter iter;
      gtk_combo_box_get_active_iter(GTK_COMBO_BOX(pe->_pathList), &iter);
      gtk_list_store_remove(GTK_LIST_STORE(model), &iter);

      StlVectorFastErase(pe->_path, id);
      StlVectorFastErase(pe->_pathFilename, id);

      if(!pe->_sh.empty()) {
	StlVectorFastErase(pe->_sh, id);
      }

      if(pe->_path.empty()) {
	gtk_combo_box_set_active(GTK_COMBO_BOX(pe->_pathList), - 1);
      } else {
	gtk_combo_box_set_active(GTK_COMBO_BOX(pe->_pathList), 0);
      }
    }
    gtk_widget_destroy(dialog);
  }
}

void
PathEditor::_onPathFileOpen(GtkButton*, gpointer data) {
  PathEditor* pe = (PathEditor*)data;

  GtkFileFilter* pathFilter = gtk_file_filter_new();
  gtk_file_filter_set_name(pathFilter, "Fichiers Path");
  gtk_file_filter_add_pattern(pathFilter, "*.path");

  GtkWidget* fo = gtk_file_chooser_dialog_new("Open Scene",
					      NULL,
					      GTK_FILE_CHOOSER_ACTION_SAVE,
					      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
					      GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
					      NULL);
  gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(fo), pathFilter);
  if(gtk_dialog_run(GTK_DIALOG(fo)) == GTK_RESPONSE_ACCEPT) {
    char* filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(fo));

    ArRef<FileIStream> input = FileIStream::NEW(filename);
    if(!input->fail()) {
      SerializationDependencies dependencies;
      if(dependencies.readFromStream(input)) {
	ArRef<Path3D> path = Path3D::NEW();
	if(path->unserialize(dependencies, input)) {
	  ArRef<Object3D> mb = ar_down_cast<Object3D>(path->accessMovingBase());
	  mb->addMotionCB(ArRef<PathEditor>(pe), &PathEditor::_onMovingBaseMotion);
	  pe->_interactors[MultiView::PERSPECTIVE_VIEW]->addIgnoredObject(mb);
	  pe->_interactors[MultiView::PERSPECTIVE_VIEW_MULTI]->addIgnoredObject(mb);
	  pe->_path.push_back(path);
	  pe->_pathFilename.push_back(StlString(filename));

	  if(!pe->_sh.empty()) {
	    StlVector<double> dist;
	    StlVector<ArRef<Shape3D> > sh;
	    mb->queryShapes(dist, sh);
	    pe->_sh.push_back(sh.front());
	    mb->setShape(Shape3D::nullRef());
	  }

	  GtkTreeModel* model = gtk_combo_box_get_model(GTK_COMBO_BOX(pe->_pathList));
	  GtkTreeIter iter;
	  gtk_list_store_append(GTK_LIST_STORE(model), &iter);
	  gtk_list_store_set(GTK_LIST_STORE(model),
			     &iter,
			     0, pe->_path.back()->getName().c_str(),
			     - 1);
	  gtk_combo_box_set_active(GTK_COMBO_BOX(pe->_pathList), pe->_path.size() - 1);
	}
      }
    }
    g_free(filename);
  }
  gtk_widget_destroy(fo);
}

void
PathEditor::_onPathSelectFilename(GtkButton*, gpointer data) {
  PathEditor* pe = (PathEditor*)data;

  if(!pe->_selectedPath) {
    return;
  }

  GtkFileFilter* pathFilter = gtk_file_filter_new();
  gtk_file_filter_set_name(pathFilter, "Fichiers Path");
  gtk_file_filter_add_pattern(pathFilter, "*.path");

  GtkWidget* fo = gtk_file_chooser_dialog_new("Open Scene",
					      NULL,
					      GTK_FILE_CHOOSER_ACTION_SAVE,
					      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
					      GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
					      NULL);

  if(pe->_selectedPathFilename) {
    gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(fo),
				  pe->_selectedPathFilename->c_str());
  }
  gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(fo), pathFilter);

  if(gtk_dialog_run(GTK_DIALOG(fo)) == GTK_RESPONSE_ACCEPT) {
    char* filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(fo));
    StlString newFilename(filename);
    if(strExtension(newFilename) != ".path") {
      newFilename += ".path";
    }
    *(pe->_selectedPathFilename) = newFilename;
    GtkWidget* entry = glade_xml_get_widget(pe->_mainXml, "pathFilename");
    gtk_entry_set_text(GTK_ENTRY(entry), newFilename.c_str());
    g_free(filename);
  }
  gtk_widget_destroy(fo);
}

void
PathEditor::_onPathFileSave(GtkButton*, gpointer data) {
  PathEditor* pe = (PathEditor*)data;
  GtkWidget* entry = glade_xml_get_widget(pe->_mainXml, "pathFilename");
  const gchar* filename = gtk_entry_get_text(GTK_ENTRY(entry));
  if(filename) {
    ArRef<FileOStream> output = FileOStream::NEW(filename, false);
    if(!output->fail()) {
      SerializationDependencies dependencies;
      ArRef<MemoryBlock>        data       = MemoryBlock::NEW();
      ArRef<MemoryOStream>      dataStream = MemoryOStream::NEW(data, 0);

      pe->_selectedPath->serialize(dependencies, dataStream);

      dependencies.writeToStream(output);
      output->writeData(data->getMemory(), dataStream->getMemoryUsage());
    }
  }
}

void
PathEditor::_onWayPointActivated(GtkTreeView* treeview,
				 GtkTreePath* path,
				 GtkTreeViewColumn*,
				 gpointer data) {
  PathEditor* pe = (PathEditor*)data;

  GtkTreeModel* model = gtk_tree_view_get_model(treeview);

  GtkTreeIter iter;
  gtk_tree_model_get_iter(model, &iter, path);
  const char* c; int pos;
  gtk_tree_model_get(model, &iter, 0, &c, - 1);
  _getIndiceFromName(c, pos);

  ArRef<Base3D> wp = pe->_selectedPath->accessWayPoints()[pos];

  double x, y, z;
  wp->getPosition(x, y, z);
  double roll, pitch, yaw;
  wp->extractOrientation(roll, pitch, yaw);

  GtkWidget* entry;
  GtkWidget* w;
  StlString s;

  entry = glade_xml_get_widget(pe->_editWayPointXml, "x");
  s = doubleToStr(x);
  gtk_entry_set_text(GTK_ENTRY(entry), s.c_str());
  entry = glade_xml_get_widget(pe->_editWayPointXml, "y");
  s = doubleToStr(y);
  gtk_entry_set_text(GTK_ENTRY(entry), s.c_str());
  entry = glade_xml_get_widget(pe->_editWayPointXml, "z");
  s = doubleToStr(z);
  gtk_entry_set_text(GTK_ENTRY(entry), s.c_str());

  entry = glade_xml_get_widget(pe->_editWayPointXml, "roll");
  s = doubleToStr(roll);
  gtk_entry_set_text(GTK_ENTRY(entry), s.c_str());
  entry = glade_xml_get_widget(pe->_editWayPointXml, "pitch");
  s = doubleToStr(pitch);
  gtk_entry_set_text(GTK_ENTRY(entry), s.c_str());
  entry = glade_xml_get_widget(pe->_editWayPointXml, "yaw");
  s = doubleToStr(yaw);
  gtk_entry_set_text(GTK_ENTRY(entry), s.c_str());

  w = glade_xml_get_widget(pe->_editWayPointXml, "orientationBox");
  GtkWidget* t = glade_xml_get_widget(pe->_editWayPointXml, "autoOrientation");
  if(pe->_selectedPath->accessAutoOrientations()[pos]) {
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(t), true);
    gtk_widget_set_sensitive(w, false);
  } else {
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(t), false);
    gtk_widget_set_sensitive(w, true);
  }

  int derivate; // 0 => none / 1 => start / 2 => end
  Vector3d d;
  double dx, dy, dz;

  if(pos == 0) {
    gtk_widget_show(glade_xml_get_widget(pe->_editWayPointXml, "sd"));
    gtk_widget_hide(glade_xml_get_widget(pe->_editWayPointXml, "ed"));

    d = pe->_selectedPath->getStartDerivate();
    derivate = 1;

    entry = glade_xml_get_widget(pe->_editWayPointXml, "sdx");
    s = doubleToStr(d.x());
    gtk_entry_set_text(GTK_ENTRY(entry), s.c_str());
    entry = glade_xml_get_widget(pe->_editWayPointXml, "sdy");
    s = doubleToStr(d.y());
    gtk_entry_set_text(GTK_ENTRY(entry), s.c_str());
    entry = glade_xml_get_widget(pe->_editWayPointXml, "sdz");
    s = doubleToStr(d.z());
    gtk_entry_set_text(GTK_ENTRY(entry), s.c_str());

  } else if(pos == (int)pe->_selectedPath->accessWayPoints().size() - 1) {
    gtk_widget_hide(glade_xml_get_widget(pe->_editWayPointXml, "sd"));
    gtk_widget_show(glade_xml_get_widget(pe->_editWayPointXml, "ed"));

    d = pe->_selectedPath->getEndDerivate();
    derivate = 2;

    entry = glade_xml_get_widget(pe->_editWayPointXml, "edx");
    s = doubleToStr(d.x());
    gtk_entry_set_text(GTK_ENTRY(entry), s.c_str());
    entry = glade_xml_get_widget(pe->_editWayPointXml, "edy");
    s = doubleToStr(d.y());
    gtk_entry_set_text(GTK_ENTRY(entry), s.c_str());
    entry = glade_xml_get_widget(pe->_editWayPointXml, "edz");
    s = doubleToStr(d.z());
    gtk_entry_set_text(GTK_ENTRY(entry), s.c_str());

  } else {
    gtk_widget_hide(glade_xml_get_widget(pe->_editWayPointXml, "sd"));
    gtk_widget_hide(glade_xml_get_widget(pe->_editWayPointXml, "ed"));

    derivate = 0;
  }

  if(gtk_dialog_run(GTK_DIALOG(pe->_editWayPoint)) == GTK_RESPONSE_OK) {
    entry = glade_xml_get_widget(pe->_editWayPointXml, "x");
    c = gtk_entry_get_text(GTK_ENTRY(entry));
    if(!strToDouble(x, c)) return;
    entry = glade_xml_get_widget(pe->_editWayPointXml, "y");
    c = gtk_entry_get_text(GTK_ENTRY(entry));
    if(!strToDouble(y, c)) return;
    entry = glade_xml_get_widget(pe->_editWayPointXml, "z");
    c = gtk_entry_get_text(GTK_ENTRY(entry));
    if(!strToDouble(z, c)) return;

    bool autoOrientation;
    w = glade_xml_get_widget(pe->_editWayPointXml, "autoOrientation");
    if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w))) {
      autoOrientation = true;
    } else {
      autoOrientation = false;

      entry = glade_xml_get_widget(pe->_editWayPointXml, "roll");
      c = gtk_entry_get_text(GTK_ENTRY(entry));
      if(!strToDouble(roll, c)) return;
      entry = glade_xml_get_widget(pe->_editWayPointXml, "pitch");
      c = gtk_entry_get_text(GTK_ENTRY(entry));
      if(!strToDouble(pitch, c)) return;
      entry = glade_xml_get_widget(pe->_editWayPointXml, "yaw");
      c = gtk_entry_get_text(GTK_ENTRY(entry));
      if(!strToDouble(yaw, c)) return;
    }

    if(autoOrientation) {
      pe->_selectedPath->accessAutoOrientations()[pos] = true;
    } else {
      pe->_selectedPath->accessAutoOrientations()[pos] = false;
      wp->setOrientation(roll, pitch, yaw);
    }
    wp->setPosition(x, y, z);

    switch(derivate) {
    case 1:
      entry = glade_xml_get_widget(pe->_editWayPointXml, "sdx");
      c = gtk_entry_get_text(GTK_ENTRY(entry));
      if(!strToDouble(dx, c)) return;
      entry = glade_xml_get_widget(pe->_editWayPointXml, "sdy");
      c = gtk_entry_get_text(GTK_ENTRY(entry));
      if(!strToDouble(dy, c)) return;
      entry = glade_xml_get_widget(pe->_editWayPointXml, "sdz");
      c = gtk_entry_get_text(GTK_ENTRY(entry));
      if(!strToDouble(dz, c)) return;
      pe->_selectedPath->setStartDerivate(Vector3d(dx, dy, dz));
      break;
    case 2:
      entry = glade_xml_get_widget(pe->_editWayPointXml, "edx");
      c = gtk_entry_get_text(GTK_ENTRY(entry));
      if(!strToDouble(dx, c)) return;
      entry = glade_xml_get_widget(pe->_editWayPointXml, "edy");
      c = gtk_entry_get_text(GTK_ENTRY(entry));
      if(!strToDouble(dy, c)) return;
      entry = glade_xml_get_widget(pe->_editWayPointXml, "edz");
      c = gtk_entry_get_text(GTK_ENTRY(entry));
      if(!strToDouble(dz, c)) return;
      pe->_selectedPath->setEndDerivate(Vector3d(dx, dy, dz));
      break;
    default :
      break;
    }
  }

  gtk_widget_hide(pe->_editWayPoint);
}

void
PathEditor::_onWayPointAdd(GtkButton*, gpointer data) {
  PathEditor* pe = (PathEditor*)data;

  if(!pe->_selectedPath) {
    return;
  }

  GtkTreeModel* model;
  GtkTreeSelection* sel;
  GList* selectedWayPoints;
  GtkTreeIter iter;
  StlString n;
  char* s;
  int p = - 1;

  model = gtk_tree_view_get_model(GTK_TREE_VIEW(pe->_wayPointList));
  sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(pe->_wayPointList));
  selectedWayPoints = gtk_tree_selection_get_selected_rows(sel, NULL);

  // Get a reference on last element of the selection
  GList* last = g_list_last(selectedWayPoints);

  // Free the list containing selected path
  GList* e = g_list_first(selectedWayPoints);
  while(e) {
    if(e == last) {
      gtk_tree_model_get_iter(model, &iter, (GtkTreePath*)e->data);
      gtk_tree_model_get(model, &iter, 0, &s, - 1);
      _getIndiceFromName(s, p);
    }
    gtk_tree_path_free((GtkTreePath*)e->data);
    e = g_list_next(e);
  }
  g_list_free(selectedWayPoints);

  n = "Waypoint_" + intToStr(p + 1);
  pe->_selectedPath->addWayPoint(p + 1);
  gtk_list_store_insert(GTK_LIST_STORE(model), &iter, p + 1);
  gtk_list_store_set(GTK_LIST_STORE(model),
		     &iter,
		     0, n.c_str(),
		     - 1);

  while(gtk_tree_model_iter_next(model, &iter)) {
    p++;
    n = "Waypoint_" + intToStr(p + 1);
    gtk_list_store_set(GTK_LIST_STORE(model),
		       &iter,
		       0, n.c_str(),
		       - 1);
  }
}

void
PathEditor::_onWayPointRemove(GtkButton*, gpointer data) {
  PathEditor* pe = (PathEditor*)data;

  if(!pe->_selectedPath) {
    return;
  }

  GtkTreeSelection* sel;
  GtkTreeModel* model;
  GtkTreeIter iter;

  model = gtk_tree_view_get_model(GTK_TREE_VIEW(pe->_wayPointList));
  sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(pe->_wayPointList));

  GList* selectedWayPoints = gtk_tree_selection_get_selected_rows(sel, NULL);
  GList* wp = g_list_first(selectedWayPoints);
  char* s; int p;

  StlVector<GtkTreeRowReference*> refs;
  StlVector<int> indices;

  while(wp) {
    refs.push_back(gtk_tree_row_reference_new(model, (GtkTreePath*)wp->data));
    gtk_tree_model_get_iter(model, &iter, (GtkTreePath*)wp->data);
    gtk_tree_model_get(model, &iter, 0, &s, - 1);
    _getIndiceFromName(s, p);
    indices.push_back(p);
    gtk_tree_path_free((GtkTreePath*)wp->data);
    wp = g_list_next(wp);
  }

  g_list_free(selectedWayPoints);

  sort(indices.begin(), indices.end());

  while(!indices.empty()) {
    ArRef<Object3D> curWp = pe->_selectedPath->accessWayPoint(indices.back());
    for(size_t i = 0; i < indices.size(); i++) {
      ArRef<Object3D> curWp2 = pe->_selectedPath->accessWayPoint(indices[i]);
      if(curWp != curWp2) {
	curWp->attachTo(curWp2,Base3D::ATTACH_NONE);
	curWp2->attachTo(curWp,Base3D::ATTACH_NONE);
      }
    }
    pe->_selectedPath->removeWayPoint(indices.back());
    indices.pop_back();
  }

  while(!refs.empty()) {
    GtkTreePath* path = gtk_tree_row_reference_get_path(refs.back());
    gtk_tree_model_get_iter(model, &iter, path);
    gtk_list_store_remove(GTK_LIST_STORE(model), &iter);
    gtk_tree_row_reference_free(refs.back());
    refs.pop_back();
  }

  p = 0;
  StlString n;
  if(gtk_tree_model_get_iter_first(model, &iter)) {
    do {
      n = "Waypoint_" + intToStr(p);
      gtk_list_store_set(GTK_LIST_STORE(model),
			 &iter,
			 0, n.c_str(),
			 - 1);
      p++;
    } while(gtk_tree_model_iter_next(model, &iter));
  }
}

void
PathEditor::_onWayPointDrop(GtkButton*, gpointer data) {
  PathEditor* pe = (PathEditor*)data;

  if(!pe->_selectedPath) {
    return;
  }

  StlVector<ArRef<Base3D> >& wp = pe->_selectedPath->accessWayPoints();
  int pos = wp.size();

  GtkTreeModel* model = gtk_tree_view_get_model(GTK_TREE_VIEW(pe->_wayPointList));
  GtkTreeIter iter;
  StlString n = "Waypoint_" + intToStr(pos);
  gtk_list_store_append(GTK_LIST_STORE(model), &iter);
  gtk_list_store_set(GTK_LIST_STORE(model),
		     &iter,
		     0, n.c_str(),
		     -1);

  pe->_selectedPath->addWayPoint(pos);
  wp.back()->setLocation(pe->_multiView->getWindow(MultiView::PERSPECTIVE_VIEW));
}

void
PathEditor::_refreshWayPointList(void) {
  GtkTreeModel* model = gtk_tree_view_get_model(GTK_TREE_VIEW(_wayPointList));
  gtk_list_store_clear(GTK_LIST_STORE(model));

  if(!_selectedPath) {
    return;
  }

  GtkTreeIter iter;
  StlString n;
  const StlVector<ArRef<Base3D> >& wp = _selectedPath->getWayPoints();
  for(size_t i = 0; i < wp.size(); i++) {
    n = "Waypoint_" + intToStr(i);
    gtk_list_store_append(GTK_LIST_STORE(model), &iter);
    gtk_list_store_set(GTK_LIST_STORE(model),
		       &iter,
		       0, n.c_str(),
		       -1);
  }
}

gboolean
PathEditor::_wayPointSelectFunc(GtkTreeSelection* selection,
				GtkTreeModel* model,
				GtkTreePath* path,
				gboolean currentlySelected,
				gpointer data) {
  PathEditor* pe = (PathEditor*)data;
  GtkTreeIter iter;
  char* c; int p;

  gtk_tree_model_get_iter(model, &iter, path);
  gtk_tree_model_get(model, &iter, 0, &c, - 1);
  _getIndiceFromName(c, p);
  ArRef<Object3D> currentObj = pe->_selectedPath->accessWayPoint(p);

  GList* selectedWayPoints = gtk_tree_selection_get_selected_rows(selection, NULL);
  GList* wp = g_list_first(selectedWayPoints);
  StlVector<ArRef<Object3D> > obj;

  while(wp) {
    gtk_tree_model_get_iter(model, &iter, (GtkTreePath*)wp->data);
    gtk_tree_model_get(model, &iter, 0, &c, - 1);
    _getIndiceFromName(c, p);
    ArRef<Object3D> o = pe->_selectedPath->accessWayPoint(p);
    if(o != currentObj) {
      obj.push_back(o);
    }
    gtk_tree_path_free((GtkTreePath*)wp->data);
    wp = g_list_next(wp);
  }

  g_list_free(selectedWayPoints);

  ArRef<Scene3D> scene = Scene3D::accessGlobalScene();

  if(currentlySelected) {
    scene->unselectObject(currentObj);
    for(size_t i = obj.size(); i--;) {
      currentObj->attachTo(obj[i],Base3D::ATTACH_NONE);
      obj[i]->attachTo(currentObj,Base3D::ATTACH_NONE);
    }
  } else {
    scene->selectObject(currentObj);
    for(size_t i = obj.size(); i--;) {
      currentObj->attachTo(obj[i]);
      obj[i]->attachTo(currentObj);
    }
  }

  return true;
}

void
PathEditor::_getIndiceFromName(const char* c, int& i) {
  StlString s(c);
  s.erase(0, s.find_last_of('_') + 1);
  strToInt(i, s);
}

void
PathEditor::_onPrecisionChanged(GtkSpinButton*, gpointer data) {
  PathEditor* pe = (PathEditor*)data;
  GtkWidget* w;
  double d;
  int p;

  w = glade_xml_get_widget(pe->_mainXml, "precision");
  p = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(w));

  w = glade_xml_get_widget(pe->_mainXml, "duration");
  strToDouble(d, gtk_entry_get_text(GTK_ENTRY(w)));

  w = glade_xml_get_widget(pe->_mainXml, "durationSlider");
  gtk_range_set_range(GTK_RANGE(w), 0, d);
  gtk_range_set_increments(GTK_RANGE(w), p / d, 10.0 * p / d);

  for(size_t i = 0; i < pe->_path.size(); i++) {
    pe->_path[i]->setPrecision(p);
  }
}

void
PathEditor::_onDurationChanged(GtkEditable*, gpointer data) {
  PathEditor* pe = (PathEditor*)data;
  GtkWidget* w = glade_xml_get_widget(pe->_mainXml, "duration");
  double d;
  int p;
  strToDouble(d, gtk_entry_get_text(GTK_ENTRY(w)));
  if(d > 0) {
    pe->_selectedPath->setDuration(d);

    w = glade_xml_get_widget(pe->_mainXml, "precision");
    p = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(w));

    w = glade_xml_get_widget(pe->_mainXml, "durationSlider");
    gtk_range_set_range(GTK_RANGE(w), 0, d);
    gtk_range_set_increments(GTK_RANGE(w), p / d, 10.0 * p / d);
  }
}

void
PathEditor::_onPathStop(GtkButton*, gpointer data) {
  PathEditor* pe = (PathEditor*)data;
  GtkWidget* b = glade_xml_get_widget(pe->_mainXml, "play");
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(b), false);
  pe->_act->setSuspended(true);
  pe->_suspended = true;

  GtkWidget* slider = glade_xml_get_widget(pe->_mainXml, "durationSlider");
  gtk_range_set_value(GTK_RANGE(slider), 0.0);
}

void
PathEditor::_onPathTogglePlay(GtkToggleButton* button, gpointer data) {
  PathEditor* pe = (PathEditor*)data;
  GtkWidget* icon = glade_xml_get_widget(pe->_mainXml, "playIcon");

  if(gtk_toggle_button_get_active(button)) {
    gtk_image_set_from_file(GTK_IMAGE(icon), "player_pause.png");
    pe->_act->setSuspended(false);
  } else {
    gtk_image_set_from_file(GTK_IMAGE(icon), "player_play.png");
    pe->_act->setSuspended(true);
    pe->_suspended = true;
  }
}

void
PathEditor::_onPathSliderChanged(GtkRange* slider, gpointer data) {
  PathEditor* pe = (PathEditor*)data;
  double t = gtk_range_get_value(slider);
  pe->_selectedPath->evaluate(t);
}

void
PathEditor::_onPathToggleCamera(GtkToggleButton* button, gpointer data) {
  PathEditor* pe = (PathEditor*)data;

  if(gtk_toggle_button_get_active(button)) {
    pe->_sh.clear();

    for(size_t i = 0; i < pe->_path.size(); i++) {
      StlVector<double> dist;
      StlVector<ArRef<Shape3D> > sh;

      ArRef<Object3D> mb =
	ar_down_cast<Object3D>(pe->_path[i]->accessMovingBase());
      mb->queryShapes(dist, sh);
      pe->_sh.push_back(sh.front());
      mb->setShape(Shape3D::nullRef());
    }

    GtkRange* slider = GTK_RANGE(glade_xml_get_widget(pe->_mainXml, "durationSlider"));
    pe->_selectedPath->evaluate(gtk_range_get_value(slider));
  } else {
    for(size_t i = 0; i < pe->_path.size(); i++) {
      ArRef<Object3D> mb =
	ar_down_cast<Object3D>(pe->_path[i]->accessMovingBase());
      mb->setShape(pe->_sh[i]);
    }
    pe->_sh.clear();
  }
}

void
PathEditor::_onPathToggleView(GtkToggleButton* button, gpointer data) {
  PathEditor* pe = (PathEditor*)data;
  GtkWidget* icon =
    glade_xml_get_widget(pe->_mainXml, "viewIcon");

  if(gtk_toggle_button_get_active(button)) {
    gtk_image_set_from_file(GTK_IMAGE(icon), "view_multicolumn.png");
    gtk_widget_show(pe->_multiView->getSingleViewWidget());
    gtk_widget_hide(pe->_multiView->getMultiViewWidget());
  } else {
    gtk_image_set_from_file(GTK_IMAGE(icon), "view_remove.png");
    gtk_widget_show(pe->_multiView->getMultiViewWidget());
    gtk_widget_hide(pe->_multiView->getSingleViewWidget());
  }
}

void
PathEditor::_onMovingBaseMotion(const Base3D::MotionEvent& evt) {
  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_camera))) {
    _multiView->getWindow(MultiView::PERSPECTIVE_VIEW)->setLocation(evt.source);
  }
}

void
PathEditor::_updateTestingTools(void) {
  GtkWidget* w;

  int precision;
  double duration;

  if(_selectedPath) {
    w = glade_xml_get_widget(_mainXml, "precision");
    precision = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(w));
    duration = _selectedPath->getDuration();

    w = glade_xml_get_widget(_mainXml, "duration");
    gtk_entry_set_text(GTK_ENTRY(w), doubleToStr(duration).c_str());
    gtk_widget_set_sensitive(w, true);

    w = glade_xml_get_widget(_mainXml, "stop");
    gtk_widget_set_sensitive(w, true);

    w = glade_xml_get_widget(_mainXml, "play");
    gtk_widget_set_sensitive(w, true);

    w = glade_xml_get_widget(_mainXml, "durationSlider");
    gtk_range_set_range(GTK_RANGE(w), 0, duration);
    gtk_range_set_increments(GTK_RANGE(w),
			     precision / duration,
			     10.0 * precision / duration);
    gtk_range_set_value(GTK_RANGE(w), 0.0);
    gtk_widget_set_sensitive(w, true);

    w = glade_xml_get_widget(_mainXml, "camera");
    gtk_widget_set_sensitive(w, true);
  } else {
    w = glade_xml_get_widget(_mainXml, "duration");
    gtk_widget_set_sensitive(w, false);

    w = glade_xml_get_widget(_mainXml, "stop");
    gtk_widget_set_sensitive(w, false);

    w = glade_xml_get_widget(_mainXml, "play");
    gtk_widget_set_sensitive(w, false);

    w = glade_xml_get_widget(_mainXml, "durationSlider");
    gtk_widget_set_sensitive(w, false);

    w = glade_xml_get_widget(_mainXml, "camera");
    gtk_widget_set_sensitive(w, false);
  }
}

bool
PathEditor::_activate(ArRef<Activity>, double dt) {
  GtkWidget* slider = glade_xml_get_widget(_mainXml, "durationSlider");
  double p = gtk_range_get_value(GTK_RANGE(slider));

  if(!_selectedPath.valid()) {
    _act->setSuspended(true);
    return true;
  }

  if(_suspended) {
    _suspended = false;
    dt = 0.0;
  }

  p += dt;
  if(p > _selectedPath->getDuration()) {
    p -= _selectedPath->getDuration();
  }

  gtk_range_set_value(GTK_RANGE(slider), p);

  return true;
}

void
PathEditor::_onEditWayPointAutoOrientationToggle(GtkToggleButton* button,
						 gpointer data) {
  PathEditor* pe = (PathEditor*)data;
  GtkWidget* w = glade_xml_get_widget(pe->_editWayPointXml, "orientationBox");
  if(gtk_toggle_button_get_active(button)) {
    gtk_widget_set_sensitive(w, false);
  } else {
    gtk_widget_set_sensitive(w, true);
  }
}

