#ifndef PATH_EDITOR_H
#define PATH_EDITOR_H

#include <gtk/gtk.h>
#include <glade/glade.h>

#include "AReVi/activity.h"
#include "AReVi/arObject.h"
#include "AReVi/Lib3D/embeddedWindow3D.h"
#include "AReVi/Lib3D/object3D.h"
#include "AReVi/Lib3D/scene3D.h"
#include "AReVi/Lib3D/simpleInteractor.h"
#include "AReVi/VRML/vrmlShape3D.h"
#include "AReVi/Contrib/path.h"
using namespace AReVi;

#include "gtkWindowPlugger.h"
#include "multiView.h"
#include "path3D.h"

class PathEditor : public ArObject {
public :
  AR_CLASS(PathEditor)
  AR_CONSTRUCTOR(PathEditor)

protected :
  static
  void
  _onQuit(GtkWidget*, GdkEvent*, gpointer);

  // Main window -- Menus
  static
  void
  _onFileQuit(GtkMenuItem*, gpointer);

  static
  void
  _onSceneOpen(GtkMenuItem*, gpointer);

  static
  void
  _onSceneClose(GtkMenuItem*, gpointer);

  // Main window -- Path

  static
  void
  _onPathChanged(GtkComboBox*, gpointer);

  static
  void
  _onPathAdd(GtkButton*, gpointer);

  static
  void
  _onPathRemove(GtkButton*, gpointer);

  static
  void
  _onPathFileOpen(GtkButton*, gpointer);

  static
  void
  _onPathSelectFilename(GtkButton*, gpointer);

  static
  void
  _onPathFileSave(GtkButton*, gpointer);

  // Main window -- Control Points

  static
  void
  _onWayPointActivated(GtkTreeView*, GtkTreePath*,
		       GtkTreeViewColumn*, gpointer);

  static
  void
  _onWayPointAdd(GtkButton*, gpointer);

  static
  void
  _onWayPointRemove(GtkButton*, gpointer);

  static
  void
  _onWayPointDrop(GtkButton*, gpointer);

  virtual
  void
  _refreshWayPointList(void);

  static
  gboolean
  _wayPointSelectFunc(GtkTreeSelection*, GtkTreeModel*,
		      GtkTreePath*, gboolean, gpointer);

  static
  void
  _getIndiceFromName(const char* c, int& i);

  // Main window -- Path testing

  static
  void
  _onPrecisionChanged(GtkSpinButton*, gpointer);

  static
  void
  _onDurationChanged(GtkEditable*, gpointer);

  static
  void
  _onPathStop(GtkButton*, gpointer);

  static
  void
  _onPathTogglePlay(GtkToggleButton*, gpointer);

  static
  void
  _onPathSliderChanged(GtkRange*, gpointer);

  static
  void
  _onPathToggleCamera(GtkToggleButton*, gpointer);

  static
  void
  _onPathToggleView(GtkToggleButton*, gpointer);

  virtual
  void
  _onMovingBaseMotion(const Base3D::MotionEvent&);

  virtual
  void
  _updateTestingTools(void);

  virtual
  bool
  _activate(ArRef<Activity>, double);

  // Edit Control Point Window

  static
  void
  _onEditWayPointAutoOrientationToggle(GtkToggleButton*, gpointer);

protected :
  // IHM components
  GladeXML*  _mainXml;
  GladeXML*  _newPathXml;
  GladeXML*  _editWayPointXml;

  GtkWidget* _main;
  GtkWidget* _newPath;
  GtkWidget* _editWayPoint;

  GtkWidget* _pathList;
  GtkWidget* _wayPointList;
  GtkWidget* _camera;

  // Rendering

  //ArRef<GtkWindowPlugger> _plugger;
  ArRef<MultiView> _multiView;
  ArRef<SimpleInteractor> _interactors[5];

  StlVector<ArRef<Object3D> > _sceneObj;
  ArRef<Shape3D>  _scene;
  StlString       _sceneFilename;

  StlVector<ArRef<Shape3D> > _sh;

  // Path
  StlVector<ArRef<Path3D> > _path;
  StlVector<StlString>      _pathFilename;

  ArRef<Path3D> _selectedPath;
  StlString* _selectedPathFilename;

  ArRef<Activity> _act;
  bool _suspended;
};

#endif // PATH_EDITOR_H
