#include "AReVi/arSystem.h"
#include "AReVi/Lib3D/object3D.h"
#include "AReVi/Lib3D/viewer3D.h"

#include "planeInteractor.h"

AR_CLASS_DEF(PlaneInteractor, SimpleInteractor)

#define NAVIGATION_DT 20
#define ROTATION_DT 20
#define ROTATION_TRESHOLD 100

PlaneInteractor::PlaneInteractor(ArCW& arCW)
: SimpleInteractor(arCW) {
}

PlaneInteractor::~PlaneInteractor(void) {
}

void
PlaneInteractor::_onTimeOut(const Renderer3D::TimeOutEvent & evt)
{                                                                 // Protected
if(evt.timeOut==_timeOut)
  {
  if(_canRotate)
    {
    unsigned int dt=ArSystem::getTime()-_lastTime;
    _lastTime+=dt;
    double dx=_lastDx;
    double dy=_lastDy;
    double dz=_lastDz;
    evt.source->globalToLocalVector(dx,dy,dz);
    Util3D::Base & loc=_setupLocation(_renderer);
    loc.translate(_lastXc,0.0,0.0);
    //loc.rotate(dx,dy,dz,_lastAngle*dt*0.25);
    loc.translate(-_lastXc,0.0,0.0);
    _timeOut=evt.source->postTimeOut(ROTATION_DT);
    evt.source->onMotionInteraction(thisRef(),_newLocation,
                                    true,true); // motion begins and ends !
    }
  else if(_pendingNavigation)
    {
    unsigned int dt=ArSystem::getTime()-_lastTime;
    _lastTime+=dt;
    unsigned int w,h;
    evt.source->getRenderSize(w,h);
    double xRatio=(2.0*(_mouseX-_clickX))/w;
    double yRatio=(2.0*(_mouseY-_clickY))/h;
    Util3D::Base & loc=_setupLocation(_renderer);
    switch(_mouseBtn)
      {
      case 1:
        {
	  //loc.yaw(-xRatio*_angularSpeed*dt*1e-3);
        loc.translate(-yRatio*_linearSpeed*dt*1e-3,0.0,0.0);
        break;
        }
      case 2:
      case 3:
        {
        loc.translate(0.0,-xRatio*_linearSpeed*dt*1e-3,
                          -yRatio*_linearSpeed*dt*1e-3);
        break;
        }
      }
    _timeOut=evt.source->postTimeOut(NAVIGATION_DT);
    evt.source->onMotionInteraction(thisRef(),_newLocation,
                                    false,false);
    }
  }
}
