// LI2/ENIB - AReVi-Tools - ArClass stub generator
//----------------------------------------------------------------------------
// file        : Tools/StubGenerator/arstub.cpp
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
//
//  This tool uses the Doxygen source code analyzer (http://www.doxygen.org).
//
//  It reads the XML output produced by doxygen to generate ArClass stubs.
//
//----------------------------------------------------------------------------

#include "AReVi/arSystem.h"
#include "AReVi/Utils/fileStream.h"
#include "AReVi/Utils/process.h"
using namespace AReVi;

#include "xmlClassLoader.h"

#include <sys/stat.h>
#include <sys/types.h>
#include <errno.h>

//----------------------------------------------------------------------------

void
emptyDir(const StlString & dirName)
{
StlVector<StlString> content;
ArSystem::getDirContent(dirName,content);
for(unsigned int i=0;i<content.size();i++)
  {
  StlString entry=dirName+"/"+content[i];
  StlString kind;
  if(ArSystem::getFileKind(entry,kind)&&(kind=="DIR"))
    {
    emptyDir(entry);
    ArSystem::removeDir(entry);
    }
  else
    {
    ArSystem::removeFile(entry);
    }
  }
}

bool
parseCmdLine(StlString & doxyCmd,
             StlString & stubName,
             StlString & outDir,
             StlVector<StlString> & incPaths,
             StlVector<StlString> & stripPaths,
             StlVector<StlString> & enums)
{
bool showUsage=false;
const StlVector<StlString> & args=ArSystem::getCommandLine();
for(unsigned int i=1;i<args.size();i++)
  {
  if(args[i]=="-d")
    {
    ++i;
    if((i<args.size())&&(args[i][0]!='-'))
      {
      if(doxyCmd.size())
        {
        cerr << "overiding `-d " << doxyCmd
             << "' with `-d " << args[i] << "'" << endl;
        }
      doxyCmd=args[i];
      }
    else
      {
      showUsage=true;
      }
    }
  else if(args[i]=="-f")
    {
    ++i;
    if((i<args.size())&&(args[i][0]!='-'))
      {
      if(stubName.size())
        {
        cerr << "overiding `-f " << stubName
             << "' with `-f " << args[i] << "'" << endl;
        }
      stubName=args[i];
      }
    else
      {
      showUsage=true;
      }
    }
  else if(args[i]=="-o")
    {
    ++i;
    if((i<args.size())&&(args[i][0]!='-'))
      {
      if(outDir.size())
        {
        cerr << "overiding `-o " << outDir
             << "' with `-o " << args[i] << "'" << endl;
        }
      outDir=args[i];
      }
    else
      {
      showUsage=true;
      }
    }
  else if(args[i]=="-i")
    {
    ++i;
    if((i<args.size())&&(args[i][0]!='-'))
      {
      incPaths.push_back(args[i]);
      }
    else
      {
      showUsage=true;
      }
    }
  else if(args[i]=="-s")
    {
    ++i;
    if((i<args.size())&&(args[i][0]!='-'))
      {
      stripPaths.push_back(args[i]);
      }
    else
      {
      showUsage=true;
      }
    }
  else if(args[i]=="-e")
    {
    ++i;
    if((i<args.size())&&(args[i][0]!='-'))
      {
      enums.push_back(args[i]);
      }
    else
      {
      showUsage=true;
      }
    }
  else
    {
    showUsage=true;
    }
  }
if(!stubName.size()||!outDir.size()||incPaths.empty())
  {
  showUsage=true;
  }
if(showUsage)
  {
  cerr << endl
       << "usage: " << args[0] << " [options]" << endl
       << endl
       << "  Available options:" << endl
       << "    -d doxygen_command        "
          "optional (one occurence)" << endl
       << "    -f stub_function_name     "
          "required (one occurence)" << endl
       << "    -o output_dir             "
          "required (one occurence)" << endl
       << "    -i include_path           "
          "required (multiple occurence allowed)" << endl
       << "    -s strip_include_path     "
          "optional (multiple occurence allowed)" << endl
       << "    -e enum_name              "
          "optional (multiple occurence allowed)" << endl
       << endl;
  return(false);
  }
else
  {
  return(true);
  }
}

ArRef<Scheduler>
simulationInit(void)
{
unsigned int i;
StlString doxyCmd;
StlString stubName;
StlString outDir;
StlVector<StlString> incPaths;
StlVector<StlString> stripPaths;
StlVector<StlString> enums;
if(!parseCmdLine(doxyCmd,stubName,outDir,incPaths,stripPaths,enums))
  {
  return(Scheduler::nullRef());
  }
ArRef<XmlClassLoader> loader=XmlClassLoader::NEW();
if(loader->fail())
  {
  cerr << loader->getErrorMessage() << endl;
  return(Scheduler::nullRef());
  }
StlString kind;
if(ArSystem::getFileKind(outDir,kind))
  {
  if(kind!="DIR")
    {
    cerr << "`" << outDir << "' is not a directory" << endl;
    return(Scheduler::nullRef());
    }
  }
else if(ArSystem::createDir(outDir)==-1)
  {
  cerr << "Cannot create `" << outDir << "' directory : "
       << ArSystem::printError(errno) << endl;
  return(Scheduler::nullRef());
  }
emptyDir(outDir);
StlString doxyFile=outDir+"/arstub.doxy";
ArRef<FileOStream> output=FileOStream::NEW(doxyFile,false);
if(output->fail())
  {
  cerr << output->getErrorMessage() << endl;
  return(Scheduler::nullRef());
  }
ostream & out=output->stream();
out << endl
    << "BRIEF_MEMBER_DESC      = NO" << endl
    << "REPEAT_BRIEF           = NO" << endl
    << "ABBREVIATE_BRIEF       =" << endl
    << "FULL_PATH_NAMES        = NO" << endl
    << "STRIP_FROM_PATH        =" << endl
    << "STRIP_FROM_INC_PATH    =";
for(i=0;i<stripPaths.size();i++)
  {
  out << " \"" << stripPaths[i] << "\"" << endl;
  }
out << endl
    << "INHERIT_DOCS           = NO" << endl
    << "EXTRACT_ALL            = YES" << endl
    << "EXTRACT_STATIC         = YES" << endl
    << "HIDE_FRIEND_COMPOUNDS  = YES" << endl
    << "SHOW_INCLUDE_FILES     = NO" << endl
    << "SORT_MEMBER_DOCS       = NO" << endl
    << "GENERATE_TODOLIST      = NO" << endl
    << "GENERATE_TESTLIST      = NO" << endl
    << "GENERATE_BUGLIST       = NO" << endl
    << "GENERATE_DEPRECATEDLIST= NO" << endl
    << "SHOW_USED_FILES        = NO" << endl
    << "SHOW_DIRECTORIES       = NO" << endl
    << "QUIET                  = YES" << endl
    << "WARN_IF_UNDOCUMENTED   = NO" << endl
    << "WARN_IF_DOC_ERROR      = NO" << endl
    << "INPUT                  =";
for(i=0;i<incPaths.size();i++)
  {
  out << " \"" << incPaths[i] << "\"" << endl;
  }
out << endl
    << "FILE_PATTERNS          = *.h" << endl
    << "RECURSIVE              = YES" << endl
    << "REFERENCED_BY_RELATION = NO" << endl
    << "REFERENCES_RELATION    = NO" << endl
    << "VERBATIM_HEADERS       = NO" << endl
    << "GENERATE_HTML          = NO" << endl
    << "GENERATE_LATEX         = NO" << endl
    << "GENERATE_XML           = YES" << endl
    << "XML_OUTPUT             = " << outDir << "/xml" << endl
    << "XML_PROGRAMLISTING     = NO" << endl
    << "MACRO_EXPANSION        = YES" << endl
    << "EXPAND_ONLY_PREDEF     = YES" << endl
    << "SEARCH_INCLUDES        = NO" << endl
    << "PREDEFINED             = \"AR_CLASS(a)=a AR__CLASS(void);\" \\" << endl
    << "                         \"AR_CONSTRUCTOR(a)=static ArRef<a> AR__CONSTRUCTOR(void);\" \\" << endl
    << "                         \"AR_CONSTRUCTOR_1(a,b,c)=static ArRef<a> AR__CONSTRUCTOR(b c);\" \\" << endl
    << "                         \"AR_CONSTRUCTOR_2(a,b,c,d,e)=static ArRef<a> AR__CONSTRUCTOR(b c,d e);\" \\" << endl
    << "                         \"AR_CONSTRUCTOR_3(a,b,c,d,e,f,g)=static ArRef<a> AR__CONSTRUCTOR(b c,d e,f g);\" \\" << endl
    << "                         \"AR_CONSTRUCTOR_4(a,b,c,d,e,f,g,h,i)=static ArRef<a> AR__CONSTRUCTOR(b c,d e,f g,h i);\" \\" << endl
    << "                         \"AR_CONSTRUCTOR_5(a,b,c,d,e,f,g,h,i,j,k)=static ArRef<a> AR__CONSTRUCTOR(b c,d e,f g,h i,j k);\" \\" << endl
    << "                         \"AR_CONSTRUCTOR_6(a,b,c,d,e,f,g,h,i,j,k,l,m)=static ArRef<a> AR__CONSTRUCTOR(b c,d e,f g,h i,j k,l m);\" \\" << endl
    << "                         \"AR_CONSTRUCTOR_7(a,b,c,d,e,f,g,h,i,j,k,l,m,n,o)=static ArRef<a> AR__CONSTRUCTOR(b c,d e,f g,h i,j k,l m,n o);\" \\" << endl
    << "                         \"AR_CONSTRUCTOR_8(a,b,c,d,e,f,g,h,i,j,k,l,m,n,o,p,q)=static ArRef<a> AR__CONSTRUCTOR(b c,d e,f g,h i,j k,l m,n o,p q);\" \\" << endl
    << "                         \"AR_CALLBACK(a,b,c,d)=b AR__CALLBACK(void); struct b##Event { ArRef<a> source; c };\" \\" << endl
    << "                         \"APIENTRY=\" \\" << endl
    << "                         \"virtual=\" \\" << endl
    << "                         \"inline=\"" << endl
    << "SKIP_FUNCTION_MACROS   = NO" << endl
    << "CLASS_DIAGRAMS         = NO" << endl
    << endl;
output=FileOStream::nullRef();
if(!doxyCmd.size())
  {
  doxyCmd="doxygen";
  }
doxyCmd+=" "+doxyFile;
cout << "======== Executing `" << doxyCmd << "' ========" << endl;
#ifndef __CYGWIN__
Process::execute(doxyCmd);
#else // __CYGWIN__
system(doxyCmd.c_str()); // FIXME: check why Process::execute() fails
#endif // __CYGWIN__
StlString xmlDir=outDir+"/xml";
cout << "======== Analysing `" << xmlDir
     << "/class*.xml' ========" << endl;
StlVector<StlString> content;
ArSystem::getDirContent(xmlDir,content);
for(unsigned int i=0;i<content.size();i++)
  {
  StlString fileName=xmlDir+"/"+content[i];
  StlString kind;
  if(ArSystem::getFileKind(fileName,kind)&&
     (kind=="REG")&&
     (content[i].find("class")==0)&&
     (strExtension(content[i])==".xml"))
    {
    loader->loadXmlClass(fileName);
    }
  }
loader->checkClasses(enums);
cout << "======== Generating source code for `" << stubName
     << "()' ========" << endl;
loader->generate(stubName,outDir);
return(Scheduler::nullRef());
}

void registerClasses(void);

int
main(int argc,
     char ** argv)
{
ArSystem arevi(argc,argv,false);
registerClasses();
ArSystem::loadPlugin("XmlParser");
ArSystem::simulationLoop(&simulationInit);
return(0);
}

//----------------------------------------------------------------------------
