// LI2/ENIB - AReVi-Tools - ArClass stub generator
//----------------------------------------------------------------------------
// file        : Tools/StubGenerator/xmlClassLoader.cpp
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "xmlClassLoader.h"

#include "AReVi/Utils/regex.h"
#include "AReVi/Utils/fileStream.h"
#include "AReVi/Contrib/xmlParser.h"

template <typename T> inline
void
insertInfoByLine(T * info,
             StlVector<T *> & vectInOut)
{
for(unsigned int i=0;i<vectInOut.size();i++)
  {
  if(info->line<vectInOut[i]->line)
    {
    vectInOut.insert(vectInOut.begin()+i,info);
    return;
    }
  }
vectInOut.push_back(info);
}

//----------------------------------------------------------------------------
// Class XmlClassLoader
//----------------------------------------------------------------------------

AR_CLASS_DEF(XmlClassLoader,ArObject)

//-------- Construction / Destruction ----------------------------------------

XmlClassLoader::XmlClassLoader(ArCW & arCW)
: ArObject(arCW),
  _classes(),
  _voidRE(),
  _signedRE(),
  _unsignedRE(),
  _otherRE(),
  _compoundRE(),
  _namespaceRE()
{
StlString start="^ *(const)? *";
StlString end=" *(&)? *$";
StlString ident="[a-zA-Z_][a-zA-Z_0-9]*";
StlString voidRE="^ *void *$";
StlString signedRE=start
                  +"(unsigned|signed)? *(char|short|int|long long)"
                  +end;
StlString unsignedRE=start
                    +"(bool|float|double|StlString)"
                    +end;
StlString otherRE=start
                 +"((::)?"+ident+"(::"+ident+")*)"
                 +end;
StlString compoundRE=start
                    +"((Ar(Const)?(Ref|Ptr))|StlVector) *<(.*)>"
                    +end;
StlString namespaceRE="namespace[ \t]+((::)?"+ident+"(::"+ident+")*)";
_voidRE=RegularExpression::NEW(voidRE);
if(_voidRE->fail())
  {
  setErrorMessage(_voidRE->getErrorMessage());
  return;
  }
_signedRE=RegularExpression::NEW(signedRE);
if(_signedRE->fail())
  {
  setErrorMessage(_signedRE->getErrorMessage());
  return;
  }
_unsignedRE=RegularExpression::NEW(unsignedRE);
if(_unsignedRE->fail())
  {
  setErrorMessage(_unsignedRE->getErrorMessage());
  return;
  }
_otherRE=RegularExpression::NEW(otherRE);
if(_otherRE->fail())
  {
  setErrorMessage(_otherRE->getErrorMessage());
  return;
  }
_compoundRE=RegularExpression::NEW(compoundRE);
if(_compoundRE->fail())
  {
  setErrorMessage(_compoundRE->getErrorMessage());
  return;
  }
_namespaceRE=RegularExpression::NEW(namespaceRE);
if(_namespaceRE->fail())
  {
  setErrorMessage(_namespaceRE->getErrorMessage());
  return;
  }
}

XmlClassLoader::~XmlClassLoader(void)
{
while(!_classes.empty())
  {
  delete _classes.back();
  _classes.pop_back();
  }
}

//-------- Analyze -----------------------------------------------------------

bool
XmlClassLoader::loadXmlClass(const StlString & url)
{
ArRef<XmlParser> parser=XmlParser::NEW();
if(!parser->parseFile(url,true))
  {
  cerr << "Error while parsing `" << url << "'" << endl;
  return(false);
  }
ArRef<XmlNode> rootNode=parser->getRoot();
if(!rootNode||(rootNode->getName()!="doxygen"))
  {
  cerr << "`" << url << "' should have a `doxygen' root node" << endl;
  return(false);
  }
for(ArRef<XmlNode> rootChild=rootNode->getFirstChild();
    rootChild;rootChild=rootChild->getNext())
  {
  if(rootChild->getName()=="compounddef")
    {
    StlString compoundKind;
    StlString compoundProt;
    if(rootChild->getPropertyString("kind",compoundKind)&&
       (compoundKind=="class")&&
       rootChild->getPropertyString("prot",compoundProt)&&
       (compoundProt=="public"))
      {
      XmlClassLoader::ClassInfo * clsInfo=new XmlClassLoader::ClassInfo;
      for(ArRef<XmlNode> classChild=rootChild->getFirstChild();
          classChild;classChild=classChild->getNext())
        {
        if(classChild->getName()=="compoundname")
          {
          StlString fullName;
          if(classChild->getTextString(fullName))
            {
            clsInfo->fullName=fullName;
            if(fullName=="AReVi::ArObject")
              {
              clsInfo->name="ArObject";
              }
            }
          }
        else if(classChild->getName()=="includes")
          {
          StlString header;
          if(classChild->getTextString(header))
            {
            clsInfo->header=header;
            }
          }
        else if(classChild->getName()=="sectiondef")
          {
          StlString sectiondefKind;
          classChild->getPropertyString("kind",sectiondefKind);
          for(ArRef<XmlNode> sectionChild=classChild->getFirstChild();
              sectionChild;sectionChild=sectionChild->getNext())
            {
            if(sectionChild->getName()=="memberdef")
              {
              StlString memberKind;
              StlString memberProt;
              StlString memberStatic;
              sectionChild->getPropertyString("kind",memberKind);
              sectionChild->getPropertyString("prot",memberProt);
              sectionChild->getPropertyString("static",memberStatic);
              if(sectiondefKind=="public-type")
                {
                if((memberKind=="enum")&&(memberProt=="public"))
                  {
                  XmlClassLoader::EnumInfo * enumInfo=
                    new XmlClassLoader::EnumInfo(clsInfo);
                  for(ArRef<XmlNode> memberChild=sectionChild->getFirstChild();
                      memberChild;memberChild=memberChild->getNext())
                    {
                    if(memberChild->getName()=="name")
                      {
                      memberChild->getTextString(enumInfo->name);
                      }
                    else if(memberChild->getName()=="enumvalue")
                      {
                      ArRef<XmlNode> nameNode=memberChild->getChild("name");
                      if(nameNode)
                        {
                        StlString name;
                        nameNode->getTextString(name);
                        enumInfo->values.push_back(name);
                        }
                      }
                    else if(memberChild->getName()=="location")
                      {
                      int line;
                      if(memberChild->getPropertyInteger("line",line))
                        {
                        enumInfo->line=line;
                        }
                      }
                    }
                  if(enumInfo->values.empty())
                    {
                    delete enumInfo;
                    }
                  else
                    {
                    insertInfoByLine(enumInfo,clsInfo->enums);
                    }
                  }
                }
              else if(sectiondefKind=="public-static-attrib")
                {
                if((memberKind=="variable")&&
                   (memberProt=="public")&&
                   (memberStatic=="yes"))
                  {
                  XmlClassLoader::ConstantInfo * cstInfo=
                    new XmlClassLoader::ConstantInfo(clsInfo);
                  for(ArRef<XmlNode> memberChild=sectionChild->getFirstChild();
                      memberChild;memberChild=memberChild->getNext())
                    {
                    if(memberChild->getName()=="type")
                      {
                      cstInfo->typeStr=memberChild->flattenText();
                      if(cstInfo->typeStr.substr(0,7)=="static ")
                        {
                        cstInfo->typeStr=cstInfo->typeStr.substr(7);
                        }
                      if(cstInfo->typeStr.substr(0,6)=="const ")
                        {
                        cstInfo->typeStr=cstInfo->typeStr.substr(6);
                        }
                      else // no const property in this memberdef node !?!?
                        {
                        cstInfo->typeStr.clear();
                        }
                      }
                    else if(memberChild->getName()=="name")
                      {
                      memberChild->getTextString(cstInfo->name);
                      }
                    else if(memberChild->getName()=="location")
                      {
                      int line;
                      if(memberChild->getPropertyInteger("line",line))
                        {
                        cstInfo->line=line;
                        }
                      }
                    }
                  if(cstInfo->typeStr.size()&&
                     ((int)cstInfo->typeStr.find("AR_UNSTUBBED")<0))
                    {
                    insertInfoByLine(cstInfo,clsInfo->constants);
                    }
                  else
                    {
                    delete cstInfo;
                    }
                  }
                }
              else if((sectiondefKind=="public-func")||
                      (sectiondefKind=="public-static-func"))
                {
                if((memberKind=="function")&&(memberProt=="public"))
                  {
                  XmlClassLoader::MethodInfo * mthInfo=
                    new XmlClassLoader::MethodInfo(clsInfo);
                  if(memberStatic=="yes")
                    {
                    mthInfo->isStatic=true;
                    }
                  StlString isConst;
                  if(sectionChild->getPropertyString("const",isConst)&&
                     (isConst=="yes"))
                    {
                    mthInfo->isConst=true;
                    }
                  for(ArRef<XmlNode> memberChild=sectionChild->getFirstChild();
                      memberChild;memberChild=memberChild->getNext())
                    {
                    if(memberChild->getName()=="templateparamlist")
                      {
                      for(ArRef<XmlNode> templChild=
                                         memberChild->getFirstChild();
                          templChild;templChild=templChild->getNext())
                        {
                        ArRef<XmlNode> kindNode=templChild->getChild("type");
                        ArRef<XmlNode> nameNode=templChild
                                                ->getChild("declname");
                        if(kindNode&&nameNode)
                          {
                          StlString kind;
                          kindNode->getTextString(kind);
                          mthInfo->templKind.push_back(kind);
                          StlString name;
                          nameNode->getTextString(name);
                          mthInfo->templName.push_back(name);
                          }
                        }
                      }
                    else if(memberChild->getName()=="type")
                      {
                      mthInfo->typeStr=memberChild->flattenText();
                      if(mthInfo->isStatic&&
                         (mthInfo->typeStr.substr(0,7)=="static "))
                        {
                        mthInfo->typeStr=mthInfo->typeStr.substr(7);
                        }
                      }
                    else if(memberChild->getName()=="name")
                      {
                      memberChild->getTextString(mthInfo->name);
                      }
                    else if(memberChild->getName()=="param")
                      {
                      ArRef<XmlNode> node=memberChild->getChild("type");
                      if(node)
                        {
                        StlString typeStr=node->flattenText();
                        mthInfo->argTypeStr.push_back(typeStr);
                        node=memberChild->getChild("declname");
                        StlString name;
                        if(!(node&&node->getTextString(name)))
                          {
                          name="#"+uintToStr(mthInfo->argTypeStr.size());
                          }
                        mthInfo->argName.push_back(name);
                        }
                      }
                    else if(memberChild->getName()=="location")
                      {
                      int line;
                      if(memberChild->getPropertyInteger("line",line))
                        {
                        mthInfo->line=line;
                        }
                      }
                    }
                  if((int)mthInfo->typeStr.find("AR_UNSTUBBED")>=0)
                    {
                    delete mthInfo;
                    }
                  else if(mthInfo->name=="AR__CLASS")
                    {
                    clsInfo->name=mthInfo->typeStr;
                    mthInfo->isStatic=true;
                    mthInfo->typeStr="ArConstRef< ArClass >";
                    mthInfo->name="CLASS";
                    insertInfoByLine(mthInfo,clsInfo->methods);
                    }
                  else if(mthInfo->name=="AR__CONSTRUCTOR")
                    {
                    mthInfo->name="AR_CONSTRUCTOR";
                    insertInfoByLine(mthInfo,clsInfo->constructors);
                    }
                  else if(mthInfo->name=="AR__CALLBACK")
                    {
                    mthInfo->name="AR_CALLBACK";
                    if(_loadXmlCallback(url,mthInfo))
                      {
                      insertInfoByLine(mthInfo,clsInfo->callbacks);
                      }
                    else
                      {
                      delete mthInfo;
                      }
                    }
                  else
                    {
                    if(((clsInfo->name=="ArObject")&&
                        ((mthInfo->name=="thisRef")||
                         (mthInfo->name=="thisPtr")||
                         (mthInfo->name=="nullRef")||
                         (mthInfo->name=="nullPtr")||
                         (mthInfo->name=="setUserData")||
                         (mthInfo->name=="getUserData")))||
                       ((clsInfo->name=="ArClass")&&
                        (mthInfo->name=="registerClass")))
                      {
                      delete mthInfo;
                      }
                    else
                      {
                      insertInfoByLine(mthInfo,clsInfo->methods);
                      }
                    }
                  }
                }
              }
            }
          }
        else if(classChild->getName()=="location")
          {
          StlString file;
          if(classChild->getPropertyString("file",file))
            {
            clsInfo->absHeader=file;
            }
          int line;
          if(classChild->getPropertyInteger("line",line))
            {
            clsInfo->line=line;
            }
          }
        }
      if(clsInfo->name.size())
        {
        if(clsInfo->name=="ArObject")
          {
          XmlClassLoader::MethodInfo * mthInfo=
            new XmlClassLoader::MethodInfo(clsInfo);
          mthInfo->isStatic=true;
          mthInfo->typeStr="ArRef<ArObject>";
          mthInfo->name="AR_CONSTRUCTOR";
          mthInfo->line=clsInfo->line;
          insertInfoByLine(mthInfo,clsInfo->constructors);
          }
        _classes.push_back(clsInfo);
        }
      else
        {
        delete clsInfo;
        }
      }
    }
  }
return(true);
}

void
XmlClassLoader::checkClasses(const StlVector<StlString> & enums)
{
unsigned int i;
unsigned int j;
for(i=0;i<_classes.size();i++)
  {
  XmlClassLoader::ClassInfo * clsInfo=_classes[i];
  for(j=0;j<clsInfo->constants.size();j++)
    {
    XmlClassLoader::ConstantInfo * cstInfo=clsInfo->constants[j];
    bool keep=true;
    cstInfo->type=_parseType(cstInfo->typeStr);
    if(cstInfo->type)
      {
      if((cstInfo->type->kind==TypeInfo::AR_OTHER)&&
         !_checkIfEnum(cstInfo->type,enums))
        {
        cerr << cstInfo->owner->header << ", line "
             << cstInfo->line << "," << endl
             << "    unknown type (" << cstInfo->typeStr
             << ") for constant `" << cstInfo->owner->name << "::"
             << cstInfo->name << "'," << endl
             << "    if `" << cstInfo->type->baseName
             << "' is an enum (ie an int), use the `-e' option" << endl
             << "    (else use `AR_UNSTUBBED' to avoid this warning)." << endl;
        keep=false;
        }
      }
    else
      {
      cerr << cstInfo->owner->header << ", line "
           << cstInfo->line << "," << endl
           << "    Cannot handle constant `" << cstInfo->owner->name << "::"
           << cstInfo->name << "' type: " << cstInfo->typeStr << endl;
      keep=false;
      }
    if(!keep)
      {
      delete cstInfo;
      clsInfo->constants.erase(clsInfo->constants.begin()+j);
      j--;
      }
    }
  for(j=0;j<clsInfo->constructors.size();j++)
    {
    if(!_checkTypes(clsInfo->constructors[j],enums,true))
      {
      delete clsInfo->constructors[j];
      clsInfo->constructors.erase(clsInfo->constructors.begin()+j);
      j--;
      }
    }
  for(j=0;j<clsInfo->methods.size();j++)
    {
    if(!_checkTypes(clsInfo->methods[j],enums,true))
      {
      delete clsInfo->methods[j];
      clsInfo->methods.erase(clsInfo->methods.begin()+j);
      j--;
      }
    }
  for(j=0;j<clsInfo->callbacks.size();j++)
    {
    if(!_checkTypes(clsInfo->callbacks[j],enums,false))
      {
      delete clsInfo->callbacks[j];
      clsInfo->callbacks.erase(clsInfo->callbacks.begin()+j);
      j--;
      }
    }
  if(!(clsInfo->enums.empty()&&clsInfo->constants.empty()&&
       clsInfo->constructors.empty()&&clsInfo->methods.empty()))
    {
    clsInfo->generate=true;
    }
  }
}

//-------- Generation --------------------------------------------------------

void
XmlClassLoader::generate(const StlString & stubName,
                         const StlString & directory) const
{
unsigned int i,j;
StlVector<StlString> headers;
StlVector<StlString> absHeaders;
for(i=0;i<_classes.size();i++)
  {
  if(_classes[i]->generate)
    {
    bool found=false;
    for(j=headers.size();j--;)
      {
      if(headers[j]==_classes[i]->header)
        {
        found=true;
        break;
        }
      }
    if(!found)
      {
      headers.push_back(_classes[i]->header);
      absHeaders.push_back(_classes[i]->absHeader);
      }
    }
  }
ArRef<FileOStream> output;
for(i=0;i<headers.size();i++)
  {
  StlVector<StlString> namespaces;
  ArRef<FileIStream> input=FileIStream::NEW(absHeaders[i]);
  if(!input->fail())
    {
    StlString line;
    StlVector<int> start;
    StlVector<int> end;
    while(input->readTxtLine(line))
      {
      if(_namespaceRE->detailledMatch(line,start,end)&&(start[1]>=0))
        {
        StlString name=line.substr(start[1],end[1]-start[1]);
        bool found=false;
        for(j=namespaces.size();j--;)
          {
          if(namespaces[j]==name)
            {
            found=true;
            break;
            }
          }
        if(!found)
          {
          namespaces.push_back(name);
          }
        }
      }
    }
  input=FileIStream::nullRef();
  StlString fileName=uintToStr(i+1);
  while(fileName.size()<3)
    {
    fileName="0"+fileName;
    }
  fileName=directory+"/arstub_"+fileName+".cpp";
  output=FileOStream::NEW(fileName,false);
  if(output->fail())
    {
    cerr << output->getErrorMessage() << endl;
    }
  else
    {
    ostream & out=output->stream();
    _generateHeader(out);
    out << "#include \"" << headers[i] << "\"" << endl
        << endl;
    for(j=0;j<namespaces.size();j++)
      {
      out << "using namespace " << namespaces[j] << ";" << endl;
      }
    out << endl
        << "namespace AReVi {" << endl
        << endl;
    for(j=0;j<_classes.size();j++)
      {
      if((_classes[j]->generate)&&(_classes[j]->header==headers[i]))
        {
        _generateClass(_classes[j],out);
        }
      }
    out << "} // namespace AReVi" << endl
        << endl;
    }
  }
StlString fileName=directory+"/arstub_000.cpp";
output=FileOStream::NEW(fileName,false);
if(output->fail())
  {
  cerr << output->getErrorMessage() << endl;
  }
else
  {
  ostream & out=output->stream();
  _generateHeader(out);
  out << "namespace AReVi {" << endl
      << endl;
  for(i=0;i<_classes.size();i++)
    {
    if(_classes[i]->generate)
      {
      out << "void arClassConstantStub_" << _classes[i]->name
          << "(ArConstRef<ArClass> cls);" << endl
          << "void arClassMethodStub_" << _classes[i]->name
          << "(ArConstRef<ArClass> cls);" << endl;
      }
    }
  out << endl
      << "} // namespace AReVi" << endl
      << endl
      << "void" << endl
      << stubName << "(void)" << endl
      << "{" << endl;
  for(i=0;i<_classes.size();i++)
    {
    if(_classes[i]->generate)
      {
      out << "AReVi::ArStubHelper::initCls(\"" << _classes[i]->name
          << "\",&AReVi::arClassMethodStub_" << _classes[i]->name
          << ",&AReVi::arClassConstantStub_" << _classes[i]->name
          << ");" << endl;
      }
    }
  out << "}" << endl
      << endl;
  }
}

void
XmlClassLoader::display(ostream & out) const
{
for(unsigned int i=0;i<_classes.size();i++)
  {
  cerr << "================================"
       << "================================" << endl;
  _displayClass(_classes[i],out);
  }
}

//-------- Forbidden operations ----------------------------------------------

XmlClassLoader::TypeInfo *
XmlClassLoader::_parseType(const StlString & typeStr) const // Protected
{
XmlClassLoader::TypeInfo * typeInfo=(XmlClassLoader::TypeInfo *)0;
StlVector<int> start;
StlVector<int> end;
StlString constStr;
StlString byRefStr;
if(_voidRE->detailledMatch(typeStr,start,end))
  {
  typeInfo=new XmlClassLoader::TypeInfo;
  }
else if(_signedRE->detailledMatch(typeStr,start,end))
  {
  if(start[1]>=0)
    {
    constStr=typeStr.substr(start[1],end[1]-start[1]);
    }
  StlString signStr;
  if(start[2]>=0)
    {
    signStr=typeStr.substr(start[2],end[2]-start[2]);
    }
  StlString baseStr;
  if(start[3]>=0)
    {
    baseStr=typeStr.substr(start[3],end[3]-start[3]);
    }
  if(start[4]>=0)
    {
    byRefStr=typeStr.substr(start[4],end[4]-start[4]);
    }
  typeInfo=new XmlClassLoader::TypeInfo;
  if(signStr=="unsigned")
    {
    if(baseStr=="char")
      {
      typeInfo->kind=TypeInfo::AR_UCHAR;
      }
    else if(baseStr=="short")
      {
      typeInfo->kind=TypeInfo::AR_USHORT;
      }
    else if(baseStr=="int")
      {
      typeInfo->kind=TypeInfo::AR_UINT;
      }
    else if(baseStr=="long long")
      {
      typeInfo->kind=TypeInfo::AR_ULLONG;
      }
    }
  else
    {
    if(baseStr=="char")
      {
      typeInfo->kind=TypeInfo::AR_CHAR;
      }
    else if(baseStr=="short")
      {
      typeInfo->kind=TypeInfo::AR_SHORT;
      }
    else if(baseStr=="int")
      {
      typeInfo->kind=TypeInfo::AR_INT;
      }
    else if(baseStr=="long long")
      {
      typeInfo->kind=TypeInfo::AR_LLONG;
      }
    }
  }
else if(_unsignedRE->detailledMatch(typeStr,start,end))
  {
  if(start[1]>=0)
    {
    constStr=typeStr.substr(start[1],end[1]-start[1]);
    }
  StlString baseStr;
  if(start[2]>=0)
    {
    baseStr=typeStr.substr(start[2],end[2]-start[2]);
    }
  if(start[3]>=0)
    {
    byRefStr=typeStr.substr(start[3],end[3]-start[3]);
    }
  typeInfo=new XmlClassLoader::TypeInfo;
  if(baseStr=="bool")
    {
    typeInfo->kind=TypeInfo::AR_BOOL;
    }
  else if(baseStr=="float")
    {
    typeInfo->kind=TypeInfo::AR_FLOAT;
    }
  else if(baseStr=="double")
    {
    typeInfo->kind=TypeInfo::AR_DOUBLE;
    }
  else if(baseStr=="StlString")
    {
    typeInfo->kind=TypeInfo::AR_STRING;
    }
  }
else if(_otherRE->detailledMatch(typeStr,start,end))
  {
  if(start[1]>=0)
    {
    constStr=typeStr.substr(start[1],end[1]-start[1]);
    }
  StlString baseStr;
  if(start[2]>=0)
    {
    baseStr=typeStr.substr(start[2],end[2]-start[2]);
    }
  if(start[5]>=0)
    {
    byRefStr=typeStr.substr(start[5],end[5]-start[5]);
    }
  typeInfo=new XmlClassLoader::TypeInfo;
  typeInfo->kind=TypeInfo::AR_OTHER;
  typeInfo->baseName=baseStr;
  }
else if(_compoundRE->detailledMatch(typeStr,start,end))
  {
  if(start[1]>=0)
    {
    constStr=typeStr.substr(start[1],end[1]-start[1]);
    }
  StlString compoundStr;
  if(start[2]>=0)
    {
    compoundStr=typeStr.substr(start[2],end[2]-start[2]);
    }
  StlString innerStr;
  if(start[6]>=0)
    {
    innerStr=typeStr.substr(start[6],end[6]-start[6]);
    }
  if(start[7]>=0)
    {
    byRefStr=typeStr.substr(start[7],end[7]-start[7]);
    }
  typeInfo=_parseType(innerStr);
  if(typeInfo)
    {
    if(compoundStr=="StlVector")
      {
      if(typeInfo->isConst||typeInfo->byRef||
         (typeInfo->kind==TypeInfo::AR_VOID))
        {
        delete typeInfo;
        typeInfo=(XmlClassLoader::TypeInfo *)0;
        }
      else
        {
        typeInfo->dimension++;
        }
      }
    else
      {
      if(typeInfo->isConst||typeInfo->byRef||typeInfo->dimension||
         (typeInfo->kind!=TypeInfo::AR_OTHER))
        {
        delete typeInfo;
        typeInfo=(XmlClassLoader::TypeInfo *)0;
        }
      else
        {
        if(compoundStr=="ArRef")
          {
          typeInfo->kind=TypeInfo::AR_REF;
          }
        else if(compoundStr=="ArConstRef")
          {
          typeInfo->kind=TypeInfo::AR_CONST_REF;
          }
        else if(compoundStr=="ArPtr")
          {
          typeInfo->kind=TypeInfo::AR_PTR;
          }
        else if(compoundStr=="ArConstPtr")
          {
          typeInfo->kind=TypeInfo::AR_CONST_PTR;
          }
        }
      }
    }
  }
if(typeInfo)
  {
  if(constStr=="const")
    {
    typeInfo->isConst=true;
    }
  if(byRefStr=="&")
    {
    typeInfo->byRef=true;
    }
  if(typeInfo->dimension>2)
    {
    delete typeInfo;
    typeInfo=(XmlClassLoader::TypeInfo *)0;
    }
  }
return(typeInfo);
}

bool
XmlClassLoader::_checkTypes(XmlClassLoader::MethodInfo * mthInfo,
                            const StlVector<StlString> & enums,
                            bool checkResult) const // Protected
{
bool result=true;
if(!mthInfo->templKind.empty())
  {
  cerr << _methodLocation(mthInfo) << endl
       << "    cannot handle template method" << endl
       << "    (use `AR_UNSTUBBED' to avoid this warning)." << endl;
  result=false;
  return(result);
  }
if(checkResult)
  {
  mthInfo->type=_parseType(mthInfo->typeStr);
  if(mthInfo->type)
    {
    if((mthInfo->type->kind==TypeInfo::AR_OTHER)&&
       !_checkIfEnum(mthInfo->type,enums))
      {
      cerr << _methodLocation(mthInfo) << endl
           << "    unknown return type (" << mthInfo->typeStr << ")," << endl
           << "    if `" << mthInfo->type->baseName
           << "' is an enum (ie an int), use the `-e' option" << endl
           << "    (else use `AR_UNSTUBBED' to avoid this warning)." << endl;
      result=false;
      }
    }
  else
    {
    cerr << _methodLocation(mthInfo) << endl
         << "    Cannot handle return type: " << mthInfo->typeStr << endl;
    result=false;
    }
  }
for(unsigned int i=0;i<mthInfo->argTypeStr.size();i++)
  {
  mthInfo->argType.push_back(_parseType(mthInfo->argTypeStr[i]));
  if(mthInfo->argType[i])
    {
    if((mthInfo->argType[i]->kind==TypeInfo::AR_OTHER)&&
       !_checkIfEnum(mthInfo->argType[i],enums))
      {
      cerr << _methodLocation(mthInfo) << endl
           << "    unknown type (" << mthInfo->argTypeStr[i]
           << ") for argument `" << mthInfo->argName[i] << "'," << endl
           << "    if `" << mthInfo->argType[i]->baseName
           << "' is an enum (ie an int), use the `-e' option" << endl
           << "    (else use `AR_UNSTUBBED' to avoid this warning)." << endl;
      result=false;
      }
    }
  else
    {
    cerr << _methodLocation(mthInfo) << endl
         << "    Cannot handle `" << mthInfo->argName[i]
         << "' type: " << mthInfo->argTypeStr[i] << endl;
    result=false;
    }
  }
if((mthInfo->argType.size()==1)&&
   mthInfo->argType[0]&&
   mthInfo->argType[0]->kind==XmlClassLoader::TypeInfo::AR_VOID)
  {
  delete mthInfo->argType[0];
  mthInfo->argType.clear();
  mthInfo->argTypeStr.clear();
  mthInfo->argName.clear();
  }
return(result);
}

bool
XmlClassLoader::_checkIfEnum(const XmlClassLoader::TypeInfo * typeInfo,
                             const StlVector<StlString> & enums) const
{
unsigned int i;
for(i=_classes.size();i--;)
  {
  for(unsigned int j=_classes[i]->enums.size();j--;)
    {
    if(typeInfo->baseName==_classes[i]->name+"::"+_classes[i]->enums[j]->name)
      {
      return(true);
      }
    }
  }
for(i=enums.size();i--;)
  {
  if(enums[i]==typeInfo->baseName)
    {
    return(true);
    }
  }
return(false);
}

StlString
XmlClassLoader::_methodLocation(const XmlClassLoader::MethodInfo * mthInfo)
                                                            const // Protected
{
StlString result=mthInfo->owner->header;
result+=", line ";
result+=uintToStr(mthInfo->line);
result+=",\n    in ";
result+=mthInfo->owner->name;
result+="::";
if(mthInfo->name=="AR_CALLBACK")
  {
  result+=mthInfo->typeStr;
  result+="Event";
  }
else
  {
  result+=mthInfo->name;
  if((mthInfo->name=="AR_CONSTRUCTOR")&&
     (!mthInfo->argTypeStr.empty()))
    {
    result+="_";
    result+=uintToStr(mthInfo->argTypeStr.size());
    }
  result+="()";
  }
result+=",";
return(result);
}

bool
XmlClassLoader::_loadXmlCallback(const StlString & clsUrl,
                                 XmlClassLoader::MethodInfo * mthInfo)
{                                                                 // Protected
StlString base=strBasename(clsUrl);
if(base.find("class")!=0)
  {
  cerr << "`" << clsUrl << "' should begin with `class'" << endl;
  return(false);
  }
StlString ext=strExtension(base);
StlString url=strDirname(clsUrl);
url+="struct";
url+=base.substr(5,base.size()-5-ext.size());
url+="_1_1";
#ifndef __APPLE__
url+=mthInfo->typeStr;
url+="Event";
#else // __APPLE__
for(unsigned int i=0;i<mthInfo->typeStr.size();i++)
  {
  char c=mthInfo->typeStr[i];
  if((c>='A')&&(c<='Z'))
    {
    url+='_';
    url+=(c+('a'-'A'));
    }
  else
    {
    url+=c;
    }
  }
url+="_event";
#endif // __APPLE__
url+=ext;
ArRef<XmlParser> parser=XmlParser::NEW();
if(!parser->parseFile(url,true))
  {
  cerr << "Error while parsing `" << url << "'" << endl;
  return(false);
  }
ArRef<XmlNode> rootNode=parser->getRoot();
if(!rootNode||(rootNode->getName()!="doxygen"))
  {
  cerr << "`" << url << "' should have a `doxygen' root node" << endl;
  return(false);
  }
for(ArRef<XmlNode> rootChild=rootNode->getFirstChild();
    rootChild;rootChild=rootChild->getNext())
  {
  if(rootChild->getName()=="compounddef")
    {
    StlString compoundKind;
    StlString compoundProt;
    if(rootChild->getPropertyString("kind",compoundKind)&&
       (compoundKind=="struct")&&
       rootChild->getPropertyString("prot",compoundProt)&&
       (compoundProt=="public"))
      {
      for(ArRef<XmlNode> structChild=rootChild->getFirstChild();
          structChild;structChild=structChild->getNext())
        {
        if(structChild->getName()=="compoundname")
          {
          StlString goodName=mthInfo->owner->fullName;
          goodName+="::";
          goodName+=mthInfo->typeStr;
          goodName+="Event";
          StlString structName;
          structChild->getTextString(structName);
          if(structName!=goodName)
            {
            cerr << "`" << url << "' should describe `" << goodName
                 << "' instead of `" << structName << "'" << endl;
            return(false);
            }
          }
        else if(structChild->getName()=="sectiondef")
          {
          StlString sectiondefKind;
          if(structChild->getPropertyString("kind",sectiondefKind)&&
             (sectiondefKind=="public-attrib"))
            {
            for(ArRef<XmlNode> sectionChild=structChild->getFirstChild();
                sectionChild;sectionChild=sectionChild->getNext())
              {
              if(sectionChild->getName()=="memberdef")
                {
                StlString memberKind;
                StlString memberProt;
                StlString memberStatic;
                if(sectionChild->getPropertyString("kind",memberKind)&&
                   (memberKind=="variable")&&
                   sectionChild->getPropertyString("prot",memberProt)&&
                   (memberProt=="public")&&
                   sectionChild->getPropertyString("static",memberStatic)&&
                   (memberStatic=="no"))
                  {
                  ArRef<XmlNode> typeNode=sectionChild->getChild("type");
                  ArRef<XmlNode> nameNode=sectionChild->getChild("name");
                  if(typeNode&&nameNode)
                    {
                    StlString typeStr=typeNode->flattenText();
                    StlString name;
                    nameNode->getTextString(name);
                    mthInfo->argTypeStr.push_back(typeStr);
                    mthInfo->argName.push_back(name);
                    }
                  }
                }
              }
            }
          }
        }
      }
    }
  }
return(true);
}

void
XmlClassLoader::_generateHeader(ostream & out) const // Protected
{
out << endl
    << "/****" << endl
    << "This file is generated by the arstub tool" << endl
    << "Do not edit or your changes will be lost !" << endl
    << "****/" << endl
    << endl
    << "#include \"AReVi/arStubHelper_priv.h\"" << endl
    << endl;
}

void
XmlClassLoader::_generateClass(const XmlClassLoader::ClassInfo * clsInfo,
                               ostream & out) const // Protected
{
unsigned int i;
for(i=0;i<clsInfo->constructors.size();i++)
  {
  _generateStub(clsInfo->constructors[i],true,out);
  }
for(i=0;i<clsInfo->methods.size();i++)
  {
  _generateStub(clsInfo->methods[i],false,out);
  }
out << "void" << endl
    << "arClassConstantStub_" << clsInfo->name
    << "(ArConstRef<ArClass>";
if(clsInfo->enums.empty()&&clsInfo->constants.empty())
  {
  out << ") {}" << endl;
  }
else
  {
  out << " cls)" << endl
      << "{" << endl;
  out << "ArRef<ArValue> v;" << endl;
  for(i=0;i<clsInfo->enums.size();i++)
    {
    _generateEnum(clsInfo->enums[i],out);
    }
  for(i=0;i<clsInfo->constants.size();i++)
    {
    _generateConstant(clsInfo->constants[i],out);
    }
  out << "}" << endl;
  }
out << endl
    << "void" << endl
    << "arClassMethodStub_" << clsInfo->name
    << "(ArConstRef<ArClass>";
if(clsInfo->constructors.empty()&&clsInfo->methods.empty())
  {
  out << ") {}" << endl;
  }
else
  {
   out << " cls)" << endl
       << "{" << endl;
  out << "ArMethod * m=(ArMethod *)0;" << endl;
  for(i=0;i<clsInfo->constructors.size();i++)
    {
    _generateMethod(clsInfo->constructors[i],out);
    }
  for(i=0;i<clsInfo->methods.size();i++)
    {
    _generateMethod(clsInfo->methods[i],out);
    }
  out << "}" << endl
      << endl;
  }
}

void
XmlClassLoader::_generateStub(const XmlClassLoader::MethodInfo * mthInfo,
                              bool constructor,
                              ostream & out) const // Protected
{
bool isVoid=(mthInfo->type->kind==XmlClassLoader::TypeInfo::AR_VOID);
out << "static" << endl
    << "void" << endl
    << "arMethodStub_" << ptrToStr(mthInfo) << "(ArRef<ArObject> "
    << (mthInfo->isStatic ? "/* o */," : "o,") << endl
    << "                        ArRef<ArValue> "
    << (isVoid ? "/* r */," : "r,") << endl
    << "                        const StlVector<ArRef<ArValue> > & "
    << (mthInfo->argType.empty() ? "/* a */)" : "a)") << endl
    << "{" << endl;
if(!isVoid)
  {
  out << _writeAccess(mthInfo->type,"r") << "=" << endl;
  }
if(mthInfo->isStatic)
  {
  if(constructor)
    {
    out << mthInfo->owner->name << "::NEW(";
    }
  else
    {
    out << mthInfo->owner->name << "::" << mthInfo->name << "(";
    }
  }
else
  {
  out << "ar_down_cast<" << mthInfo->owner->name
      << ">(o)->" << mthInfo->name << "(";
  }
for(unsigned int i=0;i<mthInfo->argType.size();i++)
  {
  out << (i ? "," : "") << endl
      << "  " << _writeAccess(mthInfo->argType[i],"a["+intToStr(i)+"]") ;
  }
out << ");" << endl
    << "}" << endl
    << endl;
}

void
XmlClassLoader::_generateEnum(const XmlClassLoader::EnumInfo * enumInfo,
                              ostream & out) const // Protected
{
StlString spc(23,' ');
for(unsigned int i=0;i<enumInfo->values.size();i++)
  {
  out << "v=ArValue::NEW(ArType::DT_INT,0);" << endl
      << "(" << enumInfo->owner->name << "::" << enumInfo->name
      << " &)(v->accessInt())=" << enumInfo->owner->name << "::"
      << enumInfo->values[i] << ";" << endl
      << "ArStubHelper::constant(cls,\""
      << enumInfo->owner->header << "\"," << enumInfo->line << "," << endl
      << spc << "\"" << enumInfo->values[i] << "\",v);" << endl;
  }
}

void
XmlClassLoader::_generateConstant(const XmlClassLoader::ConstantInfo * cstInfo,
                                  ostream & out) const // Protected
{
StlString spc(23,' ');
out << "v=ArValue::NEW(" << _writeType(cstInfo->type,false) << ");" << endl
    << _writeAccess(cstInfo->type,"v") << "=" << cstInfo->owner->name << "::"
    << cstInfo->name << ";" << endl
    << "ArStubHelper::constant(cls,\""
    << cstInfo->owner->header << "\"," << cstInfo->line << "," << endl
    << spc << "\"" << cstInfo->name << "\",v);" << endl;
}

void
XmlClassLoader::_generateMethod(const XmlClassLoader::MethodInfo * mthInfo,
                                ostream & out) const // Protected
{
StlString spc(21,' ');
out << "ArStubHelper::method(m,cls,\""
    << mthInfo->owner->header << "\"," << mthInfo->line << "," << endl
    << spc << "\"" << mthInfo->name << "\""
    << (mthInfo->isStatic ? ",true" : ",false")
    << (mthInfo->isConst ? ",true," : ",false,") << endl
    << spc << "&arMethodStub_" <<  ptrToStr(mthInfo) << ");" << endl;
if(mthInfo->type->kind!=XmlClassLoader::TypeInfo::AR_VOID)
  {
  out << "ArStubHelper::result(m,"
      << _writeType(mthInfo->type,true) << ");" << endl;
  }
for(unsigned int i=0;i<mthInfo->argType.size();i++)
  {
  out << "ArStubHelper::arg(m,\"" << mthInfo->argName[i] << "\","
      << _writeType(mthInfo->argType[i],true) << ");" << endl;
  }
}

StlString
XmlClassLoader::_writeType(const XmlClassLoader::TypeInfo * typeInfo,
                           bool withPassingMode) const // Protected
{
StlString result;
switch(typeInfo->kind)
  {
  case XmlClassLoader::TypeInfo::AR_BOOL:
    {
    result="ArType::DT_BOOL";
    break;
    }
  case XmlClassLoader::TypeInfo::AR_CHAR:
    {
    result="ArType::DT_CHAR";
    break;
    }
  case XmlClassLoader::TypeInfo::AR_UCHAR:
    {
    result="ArType::DT_UCHAR";
    break;
    }
  case XmlClassLoader::TypeInfo::AR_SHORT:
    {
    result="ArType::DT_SHORT";
    break;
    }
  case XmlClassLoader::TypeInfo::AR_USHORT:
    {
    result="ArType::DT_USHORT";
    break;
    }
  case XmlClassLoader::TypeInfo::AR_INT:
    {
    result="ArType::DT_INT";
    break;
    }
  case XmlClassLoader::TypeInfo::AR_UINT:
    {
    result="ArType::DT_UINT";
    break;
    }
  case XmlClassLoader::TypeInfo::AR_LLONG:
    {
    result="ArType::DT_LLONG";
    break;
    }
  case XmlClassLoader::TypeInfo::AR_ULLONG:
    {
    result="ArType::DT_ULLONG";
    break;
    }
  case XmlClassLoader::TypeInfo::AR_FLOAT:
    {
    result="ArType::DT_FLOAT";
    break;
    }
  case XmlClassLoader::TypeInfo::AR_DOUBLE:
    {
    result="ArType::DT_DOUBLE";
    break;
    }
  case XmlClassLoader::TypeInfo::AR_STRING:
    {
    result="ArType::DT_STRING";
    break;
    }
  case XmlClassLoader::TypeInfo::AR_REF:
    {
    result="ArType::IT_REF,\""+typeInfo->baseName+"\"";
    break;
    }
  case XmlClassLoader::TypeInfo::AR_CONST_REF:
    {
    result="ArType::IT_CONST_REF,\""+typeInfo->baseName+"\"";
    break;
    }
  case XmlClassLoader::TypeInfo::AR_PTR:
    {
    result="ArType::IT_PTR,\""+typeInfo->baseName+"\"";
    break;
    }
  case XmlClassLoader::TypeInfo::AR_CONST_PTR:
    {
    result="ArType::IT_CONST_PTR,\""+typeInfo->baseName+"\"";
    break;
    }
  case XmlClassLoader::TypeInfo::AR_OTHER:
    {
    result="ArType::DT_INT";
    break;
    }
  case XmlClassLoader::TypeInfo::AR_VOID:
    {
    result="ArType::DT_NONE"; // should not happen
    break;
    }
  }
result+=","+uintToStr(typeInfo->dimension);
if(withPassingMode)
  {
  if(typeInfo->byRef)
    {
    if(typeInfo->isConst)
      {
      result+=",ArType::PM_CONST_REF";
      }
    else
      {
      result+=",ArType::PM_REF";
      }
    }
  else
    {
    result+=",ArType::PM_COPY";
    }
  }
return(result);
}

StlString
XmlClassLoader::_writeAccess(const XmlClassLoader::TypeInfo * typeInfo,
                             const StlString & var) const // Protected
{
StlString cast;
StlString access;
bool addSpace=false;
switch(typeInfo->kind)
  {
  case XmlClassLoader::TypeInfo::AR_BOOL:
    {
    cast="bool";
    access="Bool";
    break;
    }
  case XmlClassLoader::TypeInfo::AR_CHAR:
    {
    cast="char";
    access="Char";
    break;
    }
  case XmlClassLoader::TypeInfo::AR_UCHAR:
    {
    cast="unsigned char";
    access="UChar";
    break;
    }
  case XmlClassLoader::TypeInfo::AR_SHORT:
    {
    cast="short";
    access="Short";
    break;
    }
  case XmlClassLoader::TypeInfo::AR_USHORT:
    {
    cast="unsigned short";
    access="UShort";
    break;
    }
  case XmlClassLoader::TypeInfo::AR_INT:
    {
    cast="int";
    access="Int";
    break;
    }
  case XmlClassLoader::TypeInfo::AR_UINT:
    {
    cast="unsigned int";
    access="UInt";
    break;
    }
  case XmlClassLoader::TypeInfo::AR_LLONG:
    {
    cast="long long";
    access="LLong";
    break;
    }
  case XmlClassLoader::TypeInfo::AR_ULLONG:
    {
    cast="unsigned long long";
    access="ULLong";
    break;
    }
  case XmlClassLoader::TypeInfo::AR_FLOAT:
    {
    cast="float";
    access="Float";
    break;
    }
  case XmlClassLoader::TypeInfo::AR_DOUBLE:
    {
    cast="double";
    access="Double";
    break;
    }
  case XmlClassLoader::TypeInfo::AR_STRING:
    {
    cast="StlString";
    access="String";
    break;
    }
  case XmlClassLoader::TypeInfo::AR_REF:
    {
    cast="ArRef<"+typeInfo->baseName+">";
    access="Ref";
    addSpace=true;
    break;
    }
  case XmlClassLoader::TypeInfo::AR_CONST_REF:
    {
    cast="ArConstRef<"+typeInfo->baseName+">";
    access="ConstRef";
    addSpace=true;
    break;
    }
  case XmlClassLoader::TypeInfo::AR_PTR:
    {
    cast="ArPtr<"+typeInfo->baseName+">";
    access="Ptr";
    addSpace=true;
    break;
    }
  case XmlClassLoader::TypeInfo::AR_CONST_PTR:
    {
    cast="ArConstPtr<"+typeInfo->baseName+">";
    access="ConstPtr";
    addSpace=true;
    break;
    }
  case XmlClassLoader::TypeInfo::AR_OTHER:
    {
    cast=typeInfo->baseName;
    access="Int";
    break;
    }
  case XmlClassLoader::TypeInfo::AR_VOID:
    {
    cast="void";
    access="Void"; // should not happen
    break;
    }
  }
switch(typeInfo->dimension)
  {
  case 0:
    {
    return("("+cast+" &)("+var+"->access"+access+"())");
    }
  case 1:
    {
    return("(StlVector<"+cast+(addSpace ? " " : "")+"> &)("+
           var+"->access"+access+"V())");
    }
  case 2:
    {
    return("(StlVector<StlVector<"+cast+(addSpace ? " " : "")+"> > &)("+
           var+"->access"+access+"VV())");
    }
  default:
    {
    return("");
    }
  }
}

StlString
XmlClassLoader::_formatType(const XmlClassLoader::TypeInfo * typeInfo) const
{                                                                 // Protected
StlString result;
if(typeInfo->isConst)
  {
  result+="const ";
  }
unsigned int i;
for(i=0;i<typeInfo->dimension;i++)
  {
  result+="StlVector<";
  }
bool addSpace=false;
switch(typeInfo->kind)
  {
  case XmlClassLoader::TypeInfo::AR_VOID:
    {
    result+="void";
    break;
    }
  case XmlClassLoader::TypeInfo::AR_BOOL:
    {
    result+="bool";
    break;
    }
  case XmlClassLoader::TypeInfo::AR_CHAR:
    {
    result+="char";
    break;
    }
  case XmlClassLoader::TypeInfo::AR_UCHAR:
    {
    result+="unsigned char";
    break;
    }
  case XmlClassLoader::TypeInfo::AR_SHORT:
    {
    result+="short";
    break;
    }
  case XmlClassLoader::TypeInfo::AR_USHORT:
    {
    result+="unsigned short";
    break;
    }
  case XmlClassLoader::TypeInfo::AR_INT:
    {
    result+="int";
    break;
    }
  case XmlClassLoader::TypeInfo::AR_UINT:
    {
    result+="unsigned int";
    break;
    }
  case XmlClassLoader::TypeInfo::AR_LLONG:
    {
    result+="long long";
    break;
    }
  case XmlClassLoader::TypeInfo::AR_ULLONG:
    {
    result+="unsigned long long";
    break;
    }
  case XmlClassLoader::TypeInfo::AR_FLOAT:
    {
    result+="float";
    break;
    }
  case XmlClassLoader::TypeInfo::AR_DOUBLE:
    {
    result+="double";
    break;
    }
  case XmlClassLoader::TypeInfo::AR_STRING:
    {
    result+="StlString";
    break;
    }
  case XmlClassLoader::TypeInfo::AR_REF:
    {
    result+="ArRef<"+typeInfo->baseName+">";
    addSpace=true;
    break;
    }
  case XmlClassLoader::TypeInfo::AR_CONST_REF:
    {
    result+="ArConstRef<"+typeInfo->baseName+">";
    addSpace=true;
    break;
    }
  case XmlClassLoader::TypeInfo::AR_PTR:
    {
    result+="ArPtr<"+typeInfo->baseName+">";
    addSpace=true;
    break;
    }
  case XmlClassLoader::TypeInfo::AR_CONST_PTR:
    {
    result+="ArConstPtr<"+typeInfo->baseName+">";
    addSpace=true;
    break;
    }
  case XmlClassLoader::TypeInfo::AR_OTHER:
    {
    result+=typeInfo->baseName;
    break;
    }
  }
for(i=0;i<typeInfo->dimension;i++)
  {
  if(addSpace)
    {
    result+=" ";
    }
  result+=">";
  addSpace=true;
  }
if(typeInfo->byRef)
  {
  result+=" &";
  }
return(result);
}

void
XmlClassLoader::_displayEnum(const XmlClassLoader::EnumInfo * enumInfo,
                             ostream & out) const // Protected
{
out << "    ";
out << enumInfo->name << " {";
for(unsigned int i=0;i<enumInfo->values.size();i++)
  {
  if(i)
    {
    out << ",";
    }
  out << enumInfo->values[i];
  }
out << "}; // line " << enumInfo->line << endl;
}

void
XmlClassLoader::_displayConstant(const XmlClassLoader::ConstantInfo * cstInfo,
                                 ostream & out) const // Protected
{
out << "    " << cstInfo->typeStr << " " << cstInfo->name
    << "; // line " << cstInfo->line << endl;
}

void
XmlClassLoader::_displayMethod(const XmlClassLoader::MethodInfo * mthInfo,
                               ostream & out) const // Protected
{
out << "    ";
if(mthInfo->isStatic)
  {
  out << "static ";
  }
out << mthInfo->typeStr << " " << mthInfo->name << "(";
if(mthInfo->argTypeStr.empty())
  {
  out << "void";
  }
else
  {
  for(unsigned int i=0;i<mthInfo->argTypeStr.size();i++)
    {
    if(i)
      {
      out << ",";
      }
    out << mthInfo->argTypeStr[i] << " " << mthInfo->argName[i];
    }
  }
out << ")";
if(mthInfo->isConst)
  {
  out << " const";
  }
out << "; // line " << mthInfo->line << endl;
}

void
XmlClassLoader::_displayClass(const XmlClassLoader::ClassInfo * clsInfo,
                              ostream & out) const // Protected
{
unsigned int i;
out << "Class: " << clsInfo->fullName << " (" << clsInfo->name << ") // "
    << clsInfo->header << ", line " << clsInfo->line << endl;
out << "  Header location: " << clsInfo->absHeader << endl;
if(!clsInfo->enums.empty())
  {
  out << "  Enums:" << endl;
  for(i=0;i<clsInfo->enums.size();i++)
    {
    _displayEnum(clsInfo->enums[i],out);
    }
  }
if(!clsInfo->constants.empty())
  {
  out << "  Constants:" << endl;
  for(i=0;i<clsInfo->constants.size();i++)
    {
    _displayConstant(clsInfo->constants[i],out);
    }
  }
if(!clsInfo->constructors.empty())
  {
  out << "  Constructors:" << endl;
  for(i=0;i<clsInfo->constructors.size();i++)
    {
    _displayMethod(clsInfo->constructors[i],out);
    }
  }
if(!clsInfo->methods.empty())
  {
  out << "  Methods:" << endl;
  for(i=0;i<clsInfo->methods.size();i++)
    {
    _displayMethod(clsInfo->methods[i],out);
    }
  }
if(!clsInfo->callbacks.empty())
  {
  out << "  Callbacks:" << endl;
  for(i=0;i<clsInfo->callbacks.size();i++)
    {
    _displayMethod(clsInfo->callbacks[i],out);
    }
  }
}

//----------------------------------------------------------------------------
