// LI2/ENIB - AReVi-Tools - ArClass stub generator
//----------------------------------------------------------------------------
// file        : Tools/StubGenerator/xmlClassLoader.h
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef XMLCLASS_LOADER_H
#define XMLCLASS_LOADER_H 1

#include "AReVi/arObject.h"

using namespace AReVi;

namespace AReVi {
class RegularExpression;
}

//----------------------------------------------------------------------------
// Class XmlClassLoader
//----------------------------------------------------------------------------

class XmlClassLoader : public ArObject
{
//-------- Type information --------------------------------------------------
public:

  AR_CLASS(XmlClassLoader)

//-------- Construction / Destruction ----------------------------------------

  AR_CONSTRUCTOR(XmlClassLoader)

//-------- Analyze -----------------------------------------------------------

  virtual
  bool
  loadXmlClass(const StlString & url);

  virtual
  void
  checkClasses(const StlVector<StlString> & enums);

//-------- Generation --------------------------------------------------------

  virtual
  void
  generate(const StlString & pluginName,
           const StlString & directory) const;

  virtual
  void
  display(ostream & out) const;

//-------- Forbidden operations ----------------------------------------------
protected:

  struct EnumInfo;
  struct TypeInfo;
  struct ConstantInfo;
  struct MethodInfo;
  struct ClassInfo;

  virtual
  XmlClassLoader::TypeInfo *
  _parseType(const StlString & typeStr) const;

  virtual
  bool
  _checkTypes(XmlClassLoader::MethodInfo * mthInfo,
              const StlVector<StlString> & enums,
              bool checkResult) const;

  virtual
  bool
  _checkIfEnum(const XmlClassLoader::TypeInfo * typeInfo,
               const StlVector<StlString> & enums) const;

  virtual
  StlString
  _methodLocation(const XmlClassLoader::MethodInfo * mthInfo) const;

  virtual
  bool
  _loadXmlCallback(const StlString & clsUrl,
                   XmlClassLoader::MethodInfo * mthInfo);

  virtual
  void
  _generateHeader(ostream & out) const;

  virtual
  void
  _generateClass(const XmlClassLoader::ClassInfo * clsInfo,
                 ostream & out) const;

  virtual
  void
  _generateStub(const XmlClassLoader::MethodInfo * mthInfo,
                bool constructor,
                ostream & out) const;

  virtual
  void
  _generateEnum(const XmlClassLoader::EnumInfo * enumInfo,
                ostream & out) const;

  virtual
  void
  _generateConstant(const XmlClassLoader::ConstantInfo * cstInfo,
                    ostream & out) const;

  virtual
  void
  _generateMethod(const XmlClassLoader::MethodInfo * mthInfo,
                  ostream & out) const;

  virtual
  StlString
  _writeType(const XmlClassLoader::TypeInfo * typeInfo,
             bool withPassingMode) const;

  virtual
  StlString
  _writeAccess(const XmlClassLoader::TypeInfo * typeInfo,
               const StlString & var) const;

  virtual
  StlString
  _formatType(const XmlClassLoader::TypeInfo * typeInfo) const;

  virtual
  void
  _displayEnum(const XmlClassLoader::EnumInfo * enumInfo,
               ostream & out) const;

  virtual
  void
  _displayConstant(const XmlClassLoader::ConstantInfo * cstInfo,
                   ostream & out) const;

  virtual
  void
  _displayMethod(const XmlClassLoader::MethodInfo * mthInfo,
                 ostream & out) const;

  virtual
  void
  _displayClass(const XmlClassLoader::ClassInfo * clsInfo,
                ostream & out) const;

//-------- Member attributes -------------------------------------------------
protected:

  struct EnumInfo
    {
    XmlClassLoader::ClassInfo * owner;
    int line;
    StlString name;
    StlVector<StlString> values;
    EnumInfo(XmlClassLoader::ClassInfo * cls)
    : owner(cls), line(0), name(), values() {}
    };

  struct TypeInfo
    {
    enum Kind { AR_VOID, AR_BOOL, AR_CHAR, AR_UCHAR, AR_SHORT, AR_USHORT,
                AR_INT, AR_UINT, AR_LLONG, AR_ULLONG, AR_FLOAT, AR_DOUBLE,
                AR_STRING, AR_REF, AR_CONST_REF, AR_PTR, AR_CONST_PTR,
                AR_OTHER };

    TypeInfo::Kind kind;
    bool byRef;
    bool isConst;
    unsigned int dimension;
    StlString baseName;
    TypeInfo(void)
    : kind(AR_VOID), byRef(false), isConst(false),
      dimension(0), baseName() {}
    };

  struct ConstantInfo
    {
    XmlClassLoader::ClassInfo * owner;
    int line;
    XmlClassLoader::TypeInfo * type;
    StlString typeStr;
    StlString name;
    ConstantInfo(XmlClassLoader::ClassInfo * cls)
    : owner(cls), line(0),
      type((XmlClassLoader::TypeInfo *)0), typeStr(), name() {}
    ~ConstantInfo(void)
      {
      if(type)
        {
        delete type;
        }
      }
    };

  struct MethodInfo
    {
    XmlClassLoader::ClassInfo * owner;
    int line;
    bool isStatic;
    bool isConst;
    StlVector<StlString> templKind;
    StlVector<StlString> templName;
    XmlClassLoader::TypeInfo * type;
    StlString typeStr;
    StlString name;
    StlVector<XmlClassLoader::TypeInfo *> argType;
    StlVector<StlString> argTypeStr;
    StlVector<StlString> argName;
    MethodInfo(XmlClassLoader::ClassInfo * cls)
    : owner(cls), line(0), isStatic(false), isConst(false),
      templKind(), templName(),
      type((XmlClassLoader::TypeInfo *)0), typeStr(), name(),
      argType(), argTypeStr(), argName() {}
    ~MethodInfo(void)
      {
      if(type)
        {
        delete type;
        }
      while(!argType.empty())
        {
        if(argType.back())
          {
          delete argType.back();
          }
        argType.pop_back();
        }
      }
    };

  struct ClassInfo
    {
    StlString header;
    StlString absHeader;
    int line;
    StlString name;
    StlString fullName;
    StlVector<XmlClassLoader::EnumInfo *> enums;
    StlVector<XmlClassLoader::ConstantInfo *> constants;
    StlVector<XmlClassLoader::MethodInfo *> constructors;
    StlVector<XmlClassLoader::MethodInfo *> methods;
    StlVector<XmlClassLoader::MethodInfo *> callbacks;
    bool generate;
    ClassInfo(void)
    : header(), absHeader(), line(0), name(), fullName(), enums(),
      constructors(), methods(), callbacks(), generate(false) {}
    ~ClassInfo(void)
      {
      while(!enums.empty())
        {
        delete enums.back();
        enums.pop_back();
        }
      while(!constants.empty())
        {
        delete constants.back();
        constants.pop_back();
        }
      while(!constructors.empty())
        {
        delete constructors.back();
        constructors.pop_back();
        }
      while(!methods.empty())
        {
        delete methods.back();
        methods.pop_back();
        }
      while(!callbacks.empty())
        {
        delete callbacks.back();
        callbacks.pop_back();
        }
      }
    };

  StlVector<XmlClassLoader::ClassInfo *> _classes;
  ArRef<RegularExpression> _voidRE;
  ArRef<RegularExpression> _signedRE;
  ArRef<RegularExpression> _unsignedRE;
  ArRef<RegularExpression> _otherRE;
  ArRef<RegularExpression> _compoundRE;
  ArRef<RegularExpression> _namespaceRE;
};

#endif // XMLCLASS_LOADER_H

//----------------------------------------------------------------------------
