// LI2/ENIB - AReVi-Tools - Texture converter
//----------------------------------------------------------------------------
// file        : Tools/TextureConverter/textureConverter.cpp
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/arSystem.h"
#include "AReVi/Utils/memoryBlock.h"
#include "AReVi/Lib3D/llw_priv.h"

using namespace AReVi;

//----------------------------------------------------------------------------

int
main(int argc,
     char ** argv)
{
ArSystem arevi(argc,argv);
ArSystem::loadPlugin("Imlib2ImageLoader");
ArSystem::loadPlugin("MagickImageLoader");
if(ArSystem::getCommandLine().size()<3)
  {
  cerr << endl;
  cerr << "usage: " << ArSystem::getCommandLine()[0]
       << " input_texture output_texture [output_format]" << endl;
  cerr << endl;
  return(-1);
  }
unsigned int componentsOut=0;
if((ArSystem::getCommandLine().size()>3)&&
   (!strToUint(componentsOut,ArSystem::getCommandLine()[3])||
    (componentsOut<1)||(componentsOut>4)))
  {
  cerr << "Output format should be 1 (Grey), 2 (Grey+Alpha),"
       << " 3 (RGB), or 4 (RGBA)" << endl;
  return(-1);
  }
if(ArSystem::getCommandLine()[1]==ArSystem::getCommandLine()[2])
  {
  cerr << "Same input and output files !" << endl;
  return(-1);
  }
ArRef<MemoryBlock> inputBlock=MemoryBlock::NEW();
unsigned int width,height,componentsIn;
if(!LLW::loadImage(ArSystem::getCommandLine()[1],width,height,componentsIn,
                   inputBlock))
  {
  cerr << "Cannot load texture '" << ArSystem::getCommandLine()[1]
       << "'" << endl;
  return(-1);
  }
ArRef<MemoryBlock> outputBlock;
if(!componentsOut||(componentsIn==componentsOut))
  {
  outputBlock=inputBlock;
  }
else
  {
  outputBlock=MemoryBlock::NEW();
  if(!outputBlock->ensureMinSize(width*height*componentsOut))
    {
    cerr << "Cannot allocate memory" << endl;
    return(-1);
    }
  const unsigned char * in=(const unsigned char *)inputBlock->getMemory();
  unsigned char * out=(unsigned char *)outputBlock->accessMemory();
  unsigned int nb=width*height;
  in+=nb*componentsIn;
  out+=nb*componentsOut;
#define MOVE_POINTERS {in-=componentsIn;out-=componentsOut;}
  switch(componentsIn)
    {
    case 1:
      {
      switch(componentsOut)
        {
        case 2:
          {
          cerr << "Duplicating color to alpha" << endl;
          while(nb--)
            {
            MOVE_POINTERS
            out[0]=out[1]=in[0];
            }
          break;
          }
        case 3:
          {
          while(nb--)
            {
            MOVE_POINTERS
            out[0]=out[1]=out[2]=in[0];
            }
          break;
          }
        case 4:
          {
          cerr << "Duplicating color to alpha" << endl;
          while(nb--)
            {
            MOVE_POINTERS
            out[0]=out[1]=out[2]=out[3]=in[0];
            }
          break;
          }
        }
      break;
      }
    case 2:
      {
      switch(componentsOut)
        {
        case 1:
          {
          while(nb--)
            {
            MOVE_POINTERS
            out[0]=in[0];
            }
          break;
          }
        case 3:
          {
          while(nb--)
            {
            MOVE_POINTERS
            out[0]=out[1]=out[2]=in[0];
            }
          break;
          }
        case 4:
          {
          while(nb--)
            {
            MOVE_POINTERS
            out[0]=out[1]=out[2]=in[0];
            out[3]=in[1];
            }
          break;
          }
        }
      break;
      }
    case 3:
      {
      switch(componentsOut)
        {
        case 1:
          {
          while(nb--)
            {
            MOVE_POINTERS
            unsigned int accum=in[0];
            accum+=in[1];
            accum+=in[2];
            out[0]=(unsigned char)(accum/3);
            }
          break;
          }
        case 2:
          {
          cerr << "Duplicating color to alpha" << endl;
          while(nb--)
            {
            MOVE_POINTERS
            unsigned int accum=in[0];
            accum+=in[1];
            accum+=in[2];
            out[0]=out[1]=(unsigned char)(accum/3);
            }
          break;
          }
        case 4:
          {
          cerr << "Duplicating color to alpha" << endl;
          while(nb--)
            {
            unsigned int accum=out[0]=in[0];
            accum+=out[1]=in[1];
            accum+=out[2]=in[2];
            out[3]=(unsigned char)(accum/3);
            }
          break;
          }
        }
      break;
      }
    case 4:
      {
      switch(componentsOut)
        {
        case 1:
          {
          while(nb--)
            {
            MOVE_POINTERS
            unsigned int accum=in[0];
            accum+=in[1];
            accum+=in[2];
            out[0]=(unsigned char)(accum/3);
            }
          break;
          }
        case 2:
          {
          while(nb--)
            {
            MOVE_POINTERS
            unsigned int accum=in[0];
            accum+=in[1];
            accum+=in[2];
            out[0]=(unsigned char)(accum/3);
            out[1]=in[3];
            }
          break;
          }
        case 3:
          {
          while(nb--)
            {
            for(unsigned int x=width;x--;)
              {
              MOVE_POINTERS
              out[0]=in[0];
              out[1]=in[1];
              out[2]=in[2];
              }
            }
          break;
          }
        }
      break;
      }
    }
  }
if(!LLW::saveImage(ArSystem::getCommandLine()[2],width,height,componentsOut,
                   outputBlock))
  {
  cerr << "Cannot save texture to '" << ArSystem::getCommandLine()[2]
       << "'" << endl;
  return(-1);
  }
cerr << "'" << ArSystem::getCommandLine()[1] << "' ("
     << width << "x" << height << "x" << componentsIn
     << ") converted to '" << ArSystem::getCommandLine()[2] << "' ("
     << width << "x" << height << "x" << componentsOut << ")" << endl;
return(0);
}

//----------------------------------------------------------------------------
