#ifndef _AR_MATH_LINE2D_H_
#define _AR_MATH_LINE2D_H_

namespace AReVi {

class Line2d {
public :
  /** Constructor / Destructor
   */
  inline Line2d(double a = 0, double b = 0, double c = 0);
  inline Line2d(const Point2d& p1, const Point2d& p2);
  inline Line2d(const Point2d& p, const Vector2d& v);
  inline Line2d(const Line2d& l);

  /** Accessors
   */
  inline double a(void) const;
  inline double b(void) const;
  inline double c(void) const;

  /** Test is the point belongs to the line
   */
  inline bool isOn(const Point2d& p) const;

  /** Get a unit vector from the line
   */
  inline Vector2d vec(void) const;

  /** Get a normal unit vector from the line
   */
  inline Vector2d normal(void) const;
  inline Line2d   normal(const Point2d& p) const;

  /** Get intersection from 2 lines
   */
  inline bool intersect(const Line2d& l, Point2d& result) const;

  /** Compute the minimal distance between p and this
   */
  inline double distance(const Point2d& p) const;

private :
  double _a, _b, _c;
};

inline ostream& operator<<(ostream& o, const Line2d& l);

} // namespace AReVi

#endif // _AR_MATH_LINE2D_H_
