#ifndef _AR_MATH_LINE3D_H_
#define _AR_MATH_LINE3D_H_

namespace AReVi {

class Line3d {
public :
  /** Constructor / destuctor
   */
  inline Line3d(double x = 0, double x0 = 0,
                double y = 0, double y0 = 0,
                double z = 0, double z0 = 0);
  inline Line3d(const Line3d& l);
  inline Line3d(const Point3d& p, const Vector3d& v);
  inline Line3d(const Point3d& p1, const Point3d& p2);

  /** Accessors
   */
  inline double x(void) const;
  inline double y(void) const;
  inline double z(void) const;

  inline double x0(void) const;
  inline double y0(void) const;
  inline double z0(void) const;

  /** Test is a point belongs to the line
   */
  inline bool isOn(const Point3d& p) const;

  /** Get a unit vector from the line
   */
  inline Vector3d vec(void) const;

  /** Get a plane normal to this line
   */
  inline bool normal(const Point3d& p, Planed& result) const;

  /** Get the intersection between 2 lines
   */
  inline bool intersect(const Line3d& l, Point3d& result) const;

  /** Get the minimal distance between p and this
   */
  inline double distance(const Point3d& p) const;

private :
  double _x, _x0;
  double _y, _y0;
  double _z, _z0;
};

inline ostream& operator<<(ostream& o, const Line3d& l);

} // namespace AReVi

#endif // _AR_MATH_LINE3D_H_
