#ifndef _AR_MATH_MATRIX3X3_H_
#define _AR_MATH_MATRIX3X3_H_

namespace AReVi {

class Quaterniond;

class Matrix3x3d {
public :

  enum Matrix3x3Index {
    R00 = 0, R01 = 3, R02 = 6,
    R10 = 1, R11 = 4, R12 = 7,
    R20 = 2, R21 = 5, R22 = 8
  };

  /** Constructor / Destructor
   */
  inline Matrix3x3d(void);
  inline Matrix3x3d(double m00, double m01, double m02,
	 	    double m10, double m11, double m12,
		    double m20, double m21, double m22);
  inline Matrix3x3d(double a);
  inline Matrix3x3d(const Vector3d& axe, double angle);
  inline Matrix3x3d(const Quaterniond& q);
  inline Matrix3x3d(const Vector3d& scale);
  inline Matrix3x3d(const Matrix3x3d& m);
  inline Matrix3x3d(const Vector3d& u1, const Vector3d& u2, const Vector3d& u3);

  /** Accessor
   */
  inline double at(Matrix3x3Index i) const;
  inline double at(size_t row, size_t col) const;
  inline double operator[](Matrix3x3Index i) const;

  /** Affectation
   */
  inline Matrix3x3d& operator=(const Matrix3x3d& m);

  /** Add / sum / mul / div
   */
  inline Matrix3x3d& operator+=(const Matrix3x3d& m);
  inline Matrix3x3d  operator+ (const Matrix3x3d& m) const;
  inline Matrix3x3d& operator-=(const Matrix3x3d& m);
  inline Matrix3x3d  operator- (const Matrix3x3d& m) const;
  inline Matrix3x3d& operator*=(const double& a);
  inline Matrix3x3d  operator* (const double& a) const;
  inline Matrix3x3d& operator*=(const Matrix3x3d& m);
  inline Matrix3x3d  operator* (const Matrix3x3d& m) const;
  inline Matrix3x3d& operator/=(const double& a);
  inline Matrix3x3d  operator/ (const double& a);

  inline Point3d  operator*(const Point3d& p) const;
  inline Vector3d operator*(const Vector3d& v) const;

  /** Inverse / Transpose
   */
  inline bool invert(void);
  inline void transpose(void);

  /** Matrix3x3 operations
   */
  inline Matrix3x3d& set(const double* m);
  inline Matrix3x3d& set(double m00, double m01, double m02,
                           double m10, double m11, double m12,
                           double m20, double m21, double m22);
  inline Matrix3x3d& setIdentity(void);
  inline Matrix3x3d& setXRotation(double angle);
  inline Matrix3x3d& setYRotation(double angle);
  inline Matrix3x3d& setZRotation(double angle);
  inline Matrix3x3d& setRotation(const Vector3d& axe, double angle);
  inline Matrix3x3d& setRotation(const Quaterniond& q);
  inline Matrix3x3d& setScale(const Vector3d& scale);

  /** Determinant
   */
  inline double determinant(void) const;

  /** Cast
   */
  inline operator double* (void);
  inline operator const double* (void) const;
  inline       double* accessMatrix(void);
  inline const double* getMatrix(void) const;

private :
  double _v[9];
};

inline Matrix3x3d operator*(double a, const Matrix3x3d& m);

inline Matrix3x3d operator/(double a, const Matrix3x3d& m);

inline ostream& operator<<(ostream& o, const Matrix3x3d& m);

} // namespace AReVi

#endif // _AR_MATH_MATRIX3X3_H_
