#ifndef _AR_MATH_PLANE_H_
#define _AR_MATH_PLANE_H_

namespace AReVi {

class Planed {
public :
  /** Constructor / Destructor
   */
  inline Planed(double a = 0, double b = 0, double c = 0, double d = 0);
  inline Planed(const Planed& p);
  inline Planed(const Point3d& p1,
	       const Point3d& p2,
	       const Point3d& p3);
  inline Planed(const Point3d& p,
	       const Vector3d& v1,
	       const Vector3d& v2);
  inline Planed(const Point3d& p,
               const Vector3d& n);
  /*
  inline Planed(const Line3D<T>& l1,
	       const Line3D<T>& l2);
  */
  /** Accessors
   */
  inline double a(void) const;
  inline double b(void) const;
  inline double c(void) const;
  inline double d(void) const;

  /** Get a normal vector to the plane
   */
  inline Vector3d normal(void) const;

  /** Test is a point is on the plane
   */
  inline bool isOn(const Point3d& p) const;

  /** Get the distance from a point to the plane
   */
  inline double distance(const Point3d& p) const;

  /** Get the intersection between two planes
   */
  inline bool intersect(const Planed& p, Line3d& result) const;

  /** Get the intersection between the line and the plane
   */
  inline bool intersect(const Line3d& l, Point3d& result) const;

private :
  double _a, _b, _c, _d;
};

inline ostream& operator<<(ostream& o, const Planed& p);

} // namespace AReVi

#endif // _AR_MATH_PLANE_H_
