#ifndef _AR_MATH_VECTOR2D_H_
#define _AR_MATH_VECTOR2D_H_

namespace AReVi {

class Vector2d {
public :
  /** Construction / Destruction
   */
  inline Vector2d(double x = 0, double y = 0);
  inline Vector2d(const Point2d& p1, const Point2d& p2);
  inline Vector2d(const Vector2d& v);

  /** Accessors
   */
  inline double x(void) const;
  inline double y(void) const;

  /** Affectation
   */
  inline Vector2d& operator=(const Vector2d& v);

  /** Comparison
   */
  inline bool operator==(const Vector2d& v) const;
  inline bool operator!=(const Vector2d& v) const;

  /** Add / Sum functions
   */
  inline Vector2d& operator+=(const Vector2d& v);
  inline Vector2d  operator+ (const Vector2d& v) const;
  inline Vector2d& operator-=(const Vector2d& v);
  inline Vector2d  operator- (const Vector2d& v) const;

  /** Dot product
   */
  inline double operator*(const Vector2d& v) const;

  /** Scale
   */
  inline Vector2d& operator*=(const double& a);
  inline Vector2d  operator* (const double& a) const;
  inline Vector2d& operator/=(const double& a);
  inline Vector2d  operator/ (const double& a) const;

  /** Normalize / get length
   */
  inline void       normalize(void);
  inline Vector2d unit(void) const;
  inline double          squareLength(void) const;
  inline double          length(void) const;

  /** Get a normal vector to this vector
   */
  inline Vector2d normal(void) const;

  /** Cast opetator
   */
  inline operator double* (void);
  inline operator const double* (void) const;
  inline       double* accessVector(void);
  inline const double* getVector(void) const;

private :
  double _v[2];
};

inline Vector2d operator*(double a, const Vector2d& v);

inline Vector2d operator/(double a, const Vector2d& v);

inline Vector2d lerp(const Vector2d& v1, const Vector2d& v2, double p);

/** Output of the form {x,y}
 */
inline ostream& operator<<(ostream& os, const Vector2d& v);

} // namespace AReVi

#endif // _AR_MATH_VECTOR2D_H_
