#ifndef _AR_MATH_VECTOR3D_H_
#define _AR_MATH_VECTOR3D_H_

namespace AReVi {

class Planed;

class Vector3d {
public :
  /** Construction / Destruction
   */
  inline Vector3d(double x = 0, double y = 0, double z = 0);
  inline Vector3d(const Point3d& p1, const Point3d& p2);
  inline Vector3d(const Vector3d& v);
  inline Vector3d(const Util3D::Dbl3& u);
  inline Vector3d(const Util3D::Dbl2& u);

  /** Accessors
   */
  inline double x(void) const;
  inline double y(void) const;
  inline double z(void) const;

  /** Affectation
   */
  inline Vector3d& operator=(const Vector3d& v);

  /** Comparison
   */
  inline bool operator==(const Vector3d& v) const;
  inline bool operator!=(const Vector3d& v) const;

  /** Add / Sum functions
   */
  inline Vector3d& operator+=(const Vector3d& v);
  inline Vector3d  operator+ (const Vector3d& v) const;
  inline Vector3d& operator-=(const Vector3d& v);
  inline Vector3d  operator- (const Vector3d& v) const;

  /** Dot product
   */
  inline double operator*(const Vector3d& v) const;
  inline void dotMulEqual(const Vector3d&);
  inline Vector3d dotMul(const Vector3d&) const;
  inline void dotDivEqual(const Vector3d&);
  inline Vector3d dotDiv(const Vector3d&) const;

  /** Cross product
   */
  inline Vector3d& operator^=(const Vector3d& v);
  inline Vector3d  operator^ (const Vector3d& v) const;

  /** Scale
   */
  inline Vector3d& operator*=(const double& a);
  inline Vector3d  operator* (const double& a) const;
  inline Vector3d& operator/=(const double& a);
  inline Vector3d  operator/ (const double& a) const;

  /** Normalize / get length
   */
  inline void        normalize(void);
  inline Vector3d    unit(void) const;
  inline double      squareLength(void) const;
  inline double      length(void) const;

  /** Get a normal plane to this vector
   */
  inline Planed normal(const Point3d& p) const;

  /** Returns the angle in radians between this vector and the vector
   *  parameter; the return value is constrained to the range [0,PI].
   */
  inline double angle(const Vector3d& v) const;

  /** Cast operator
   */
  inline operator double* (void);
  inline operator const double* (void) const;
  inline       double* accessVector(void);
  inline const double* getVector(void) const;

private :
  double _v[3];
};

inline Vector3d operator*(double a, const Vector3d& v);

inline Vector3d operator/(double a, const Vector3d& v);

inline Vector3d lerp(const Vector3d& v1, const Vector3d& v2, double p);

/** Output of the form {x,y,z}
 */
inline ostream& operator<<(ostream& os, const Vector3d& v);

} // namespace AReVi

#endif // _AR_MATH_VECTOR3D_H_
