#ifndef _AR_MATH_VECTOR4D_H_
#define _AR_MATH_VECTOR4D_H_

namespace AReVi {

template <class T> class Plane;

template <class T>
class Vector4 {
public :
  /** Construction / Destruction
   */
  inline Vector4(T x = 0, T y = 0, T z = 0, T w = 0);
  inline Vector4(const Point3<T>& p, T w=1);
  inline Vector4(const Vector3<T>& v);
  inline Vector4(const Vector4<T>& v);

  /** Accessors
   */
  inline T x(void) const;
  inline T y(void) const;
  inline T z(void) const;
  inline T w(void) const;

  /** Affectation
   */
  inline Vector4<T>& operator=(const Vector4<T>& v);

  /** Comparison
   */
  inline bool operator==(const Vector4<T>& v) const;
  inline bool operator!=(const Vector4<T>& v) const;

  /** Add / Sum functions
   */
  inline Vector4<T>& operator+=(const Vector4<T>& v);
  inline Vector4<T>  operator+ (const Vector4<T>& v) const;
  inline Vector4<T>& operator-=(const Vector4<T>& v);
  inline Vector4<T>  operator- (const Vector4<T>& v) const;

  /** Dot product
   */
  inline T operator*(const Vector4<T>& v) const;

  /** Cross product
   */
  inline Vector3<T>  operator^ (const Vector4<T>& v) const;

  /** Scale
   */
  inline Vector4<T>& operator*=(const T& a);
  inline Vector4<T>  operator* (const T& a) const;
  inline Vector4<T>& operator/=(const T& a);
  inline Vector4<T>  operator/ (const T& a) const;

  /** Normalize / get length
   */
  inline void        normalize(void);
  inline Vector4<T>  unit(void) const;
  inline T           squareLength(void) const;
  inline T           length(void) const;

  /** Cast opetator
   */
  inline operator T* (void);
  inline operator const T* (void) const;
  inline       T* accessVector(void);
  inline const T* getVector(void) const;

private :
  T _v[4];
};

template <class T> inline
Vector4<T> operator*(T a, const Vector4<T>& v);

template <class T> inline
Vector4<T> operator/(T a, const Vector4<T>& v);

/** Output of the form {x,y,z}
 */
template <class T> inline
ostream& operator<<(ostream& os, const Vector4<T>& v);

} // namespace AReVi

#endif // _AR_MATH_VECTOR4D_H_
