#ifndef _AR_MATH_VECTORND_H_
#define _AR_MATH_VECTORND_H_

#include "AReVi/Utils/stlVector_priv.h"

namespace AReVi {

// T is the type of data
// N is the dimension of vector (ie 3 <=> Vector3)
template <class T, int N>
class VectorN {
public :
//-------- Construction / Destruction ----------------------------------------

  inline VectorN(void);
  inline VectorN(const VectorN<T,N>& v);

//-------- Accessors ---------------------------------------------------------
  inline       T *accessVector(void);
  inline const T *getVector(void) const;

  inline unsigned getDimension(void) const; // return N

  inline T &operator[](unsigned int index); // assert if index>=N
  inline const T &operator[](unsigned int index) const;// assert if index>=N


//-------- Affectation -------------------------------------------------------

  inline VectorN<T,N>& operator=(const VectorN<T,N>& v);

//-------- Comparaison -------------------------------------------------------

  inline bool operator==(const VectorN<T,N>& v) const;
  inline bool operator!=(const VectorN<T,N>& v) const;

//-------- Operators Add/Sub -------------------------------------------------

  inline VectorN<T,N>& operator+=(const VectorN<T,N>& v);
  inline VectorN<T,N>  operator+ (const VectorN<T,N>& v) const;
  inline VectorN<T,N>& operator-=(const VectorN<T,N>& v);
  inline VectorN<T,N>  operator- (const VectorN<T,N>& v) const;

//-------- Operators Dot product ---------------------------------------------

  inline T operator*(const VectorN<T,N>& v) const;

//-------- Operators Scale ---------------------------------------------------

  inline VectorN<T,N>& operator*=(const T& a);
  inline VectorN<T,N>  operator* (const T& a) const;
  inline VectorN<T,N>& operator/=(const T& a);
  inline VectorN<T,N>  operator/ (const T& a) const;

//-------- Normalize/Length ---------------------------------------------------

  inline void         normalize(void);
  inline VectorN<T,N> unit(void) const;
  inline T            squareLength(void) const;
  inline T            length(void) const;
  inline void         reset(void);


private :
  T _v[N];
};

template <class T, int N> inline
VectorN<T,N> operator*(T a, const VectorN<T,N>& v);

template <class T, int N> inline
VectorN<T,N> operator/(T a, const VectorN<T,N>& v);

template <class T, int N> inline
VectorN<T,N>
lerp(const VectorN<T,N>& v1, const VectorN<T,N>& v2, T p);

/** Output of the form {x,y,z,w,a...}
 */
template <class T, int N> inline
ostream& operator<<(ostream& os, const VectorN<T,N>& v);

} // namespace AReVi

#endif // _AR_MATH_VECTORND_H_
