// LI2/ENIB - AReVi-Contrib - 3D file loader/saver
//----------------------------------------------------------------------------
// file        : AReVi/Contrib/File3D/3ds.h
// author(s)   : C. Septseault
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVI_CONTRIB_FILE3D_3DS_H
#define AREVI_CONTRIB_FILE3D_3DS_H 1

#include "AReVi/arObject.h"
#include "AReVi/Lib3D/util3D_priv.h"

namespace AReVi
{
	class Shape3D;
	class ShapePart3D;
	class Transform3D;
	class Mesh3D;
	class Surface3D;
	class FDIStream;
	class FDOStream;
	class MemoryOStream;
	class URLTexture;
	class Material3D;
	class Transform2D;

typedef enum  {
  NULL_CHUNK             =0x0000,
  M3DMAGIC               =0x4D4D,    /*3DS file*/
  SMAGIC                 =0x2D2D,
  LMAGIC                 =0x2D3D,
  MLIBMAGIC              =0x3DAA,    /*MLI file*/
  MATMAGIC               =0x3DFF,
  CMAGIC                 =0xC23D,    /*PRJ file*/
  M3D_VERSION            =0x0002,
  M3D_KFVERSION          =0x0005,

  COLOR_F                =0x0010,
  COLOR_24               =0x0011,
  LIN_COLOR_24           =0x0012,
  LIN_COLOR_F            =0x0013,
  INT_PERCENTAGE         =0x0030,
  FLOAT_PERCENTAGE       =0x0031,

  MDATA                  =0x3D3D,
  MESH_VERSION           =0x3D3E,
  MASTER_SCALE           =0x0100,
  LO_SHADOW_BIAS         =0x1400,
  HI_SHADOW_BIAS         =0x1410,
  SHADOW_MAP_SIZE        =0x1420,
  SHADOW_SAMPLES         =0x1430,
  SHADOW_RANGE           =0x1440,
  SHADOW_FILTER          =0x1450,
  RAY_BIAS               =0x1460,
  O_CONSTS               =0x1500,
  AMBIENT_LIGHT          =0x2100,
  BIT_MAP                =0x1100,
  SOLID_BGND             =0x1200,
  V_GRADIENT             =0x1300,
  USE_BIT_MAP            =0x1101,
  USE_SOLID_BGND         =0x1201,
  USE_V_GRADIENT         =0x1301,
  FOG                    =0x2200,
  FOG_BGND               =0x2210,
  LAYER_FOG              =0x2302,
  DISTANCE_CUE           =0x2300,
  DCUE_BGND              =0x2310,
  USE_FOG                =0x2201,
  USE_LAYER_FOG          =0x2303,
  USE_DISTANCE_CUE       =0x2301,

  MAT_ENTRY              =0xAFFF,
  MAT_NAME               =0xA000,
  MAT_AMBIENT            =0xA010,
  MAT_DIFFUSE            =0xA020,
  MAT_SPECULAR           =0xA030,
  MAT_SHININESS          =0xA040,
  MAT_SHIN2PCT           =0xA041,
  MAT_TRANSPARENCY       =0xA050,
  MAT_XPFALL             =0xA052,
  MAT_USE_XPFALL         =0xA240,
  MAT_REFBLUR            =0xA053,
  MAT_SHADING            =0xA100,
  MAT_USE_REFBLUR        =0xA250,
  MAT_SELF_ILLUM         =0xA080,
  MAT_TWO_SIDE           =0xA081,
  MAT_DECAL              =0xA082,
  MAT_ADDITIVE           =0xA083,
  MAT_WIRE               =0xA085,
  MAT_FACEMAP            =0xA088,
  MAT_PHONGSOFT          =0xA08C,
  MAT_WIREABS            =0xA08E,
  MAT_WIRE_SIZE          =0xA087,
  MAT_TEXMAP             =0xA200,
  MAT_SXP_TEXT_DATA      =0xA320,
  MAT_TEXMASK            =0xA33E,
  MAT_SXP_TEXTMASK_DATA  =0xA32A,
  MAT_TEX2MAP            =0xA33A,
  MAT_SXP_TEXT2_DATA     =0xA321,
  MAT_TEX2MASK           =0xA340,
  MAT_SXP_TEXT2MASK_DATA =0xA32C,
  MAT_OPACMAP            =0xA210,
  MAT_SXP_OPAC_DATA      =0xA322,
  MAT_OPACMASK           =0xA342,
  MAT_SXP_OPACMASK_DATA  =0xA32E,
  MAT_BUMPMAP            =0xA230,
  MAT_SXP_BUMP_DATA      =0xA324,
  MAT_BUMPMASK           =0xA344,
  MAT_SXP_BUMPMASK_DATA  =0xA330,
  MAT_SPECMAP            =0xA204,
  MAT_SXP_SPEC_DATA      =0xA325,
  MAT_SPECMASK           =0xA348,
  MAT_SXP_SPECMASK_DATA  =0xA332,
  MAT_SHINMAP            =0xA33C,
  MAT_SXP_SHIN_DATA      =0xA326,
  MAT_SHINMASK           =0xA346,
  MAT_SXP_SHINMASK_DATA  =0xA334,
  MAT_SELFIMAP           =0xA33D,
  MAT_SXP_SELFI_DATA     =0xA328,
  MAT_SELFIMASK          =0xA34A,
  MAT_SXP_SELFIMASK_DATA =0xA336,
  MAT_REFLMAP            =0xA220,
  MAT_REFLMASK           =0xA34C,
  MAT_SXP_REFLMASK_DATA  =0xA338,
  MAT_ACUBIC             =0xA310,
  MAT_MAPNAME            =0xA300,
  MAT_MAP_TILING         =0xA351,
  MAT_MAP_TEXBLUR        =0xA353,
  MAT_MAP_USCALE         =0xA354,
  MAT_MAP_VSCALE         =0xA356,
  MAT_MAP_UOFFSET        =0xA358,
  MAT_MAP_VOFFSET        =0xA35A,
  MAT_MAP_ANG            =0xA35C,
  MAT_MAP_COL1           =0xA360,
  MAT_MAP_COL2           =0xA362,
  MAT_MAP_RCOL           =0xA364,
  MAT_MAP_GCOL           =0xA366,
  MAT_MAP_BCOL           =0xA368,

  NAMED_OBJECT           =0x4000,
  N_DIRECT_LIGHT         =0x4600,
  DL_OFF                 =0x4620,
  DL_OUTER_RANGE         =0x465A,
  DL_INNER_RANGE         =0x4659,
  DL_MULTIPLIER          =0x465B,
  DL_EXCLUDE             =0x4654,
  DL_ATTENUATE           =0x4625,
  DL_SPOTLIGHT           =0x4610,
  DL_SPOT_ROLL           =0x4656,
  DL_SHADOWED            =0x4630,
  DL_LOCAL_SHADOW2       =0x4641,
  DL_SEE_CONE            =0x4650,
  DL_SPOT_RECTANGULAR    =0x4651,
  DL_SPOT_ASPECT         =0x4657,
  DL_SPOT_PROJECTOR      =0x4653,
  DL_SPOT_OVERSHOOT      =0x4652,
  DL_RAY_BIAS            =0x4658,
  DL_RAYSHAD             =0x4627,
  N_CAMERA               =0x4700,
  CAM_SEE_CONE           =0x4710,
  CAM_RANGES             =0x4720,
  OBJ_HIDDEN             =0x4010,
  OBJ_VIS_LOFTER         =0x4011,
  OBJ_DOESNT_CAST        =0x4012,
  OBJ_DONT_RECVSHADOW    =0x4017,
  OBJ_MATTE              =0x4013,
  OBJ_FAST               =0x4014,
  OBJ_PROCEDURAL         =0x4015,
  OBJ_FROZEN             =0x4016,
  N_TRI_OBJECT           =0x4100,
  POINT_ARRAY            =0x4110,
  POINT_FLAG_ARRAY       =0x4111,
  FACE_ARRAY             =0x4120,
  MSH_MAT_GROUP          =0x4130,
  SMOOTH_GROUP           =0x4150,
  MSH_BOXMAP             =0x4190,
  TEX_VERTS              =0x4140,
  MESH_MATRIX            =0x4160,
  MESH_COLOR             =0x4165,
  MESH_TEXTURE_INFO      =0x4170,

  KFDATA                 =0xB000,
  KFHDR                  =0xB00A,
  KFSEG                  =0xB008,
  KFCURTIME              =0xB009,
  AMBIENT_NODE_TAG       =0xB001,
  OBJECT_NODE_TAG        =0xB002,
  CAMERA_NODE_TAG        =0xB003,
  TARGET_NODE_TAG        =0xB004,
  LIGHT_NODE_TAG         =0xB005,
  L_TARGET_NODE_TAG      =0xB006,
  SPOTLIGHT_NODE_TAG     =0xB007,
  NODE_ID                =0xB030,
  NODE_HDR               =0xB010,
  PIVOT                  =0xB013,
  INSTANCE_NAME          =0xB011,
  MORPH_SMOOTH           =0xB015,
  BOUNDBOX               =0xB014,
  POS_TRACK_TAG          =0xB020,
  COL_TRACK_TAG          =0xB025,
  ROT_TRACK_TAG          =0xB021,
  SCL_TRACK_TAG          =0xB022,
  MORPH_TRACK_TAG        =0xB026,
  FOV_TRACK_TAG          =0xB023,
  ROLL_TRACK_TAG         =0xB024,
  HOT_TRACK_TAG          =0xB027,
  FALL_TRACK_TAG         =0xB028,
  HIDE_TRACK_TAG         =0xB029,

  POLY_2D                = 0x5000,
  SHAPE_OK               = 0x5010,
  SHAPE_NOT_OK           = 0x5011,
  SHAPE_HOOK             = 0x5020,
  PATH_3D                = 0x6000,
  PATH_MATRIX            = 0x6005,
  SHAPE_2D               = 0x6010,
  M_SCALE                = 0x6020,
  M_TWIST                = 0x6030,
  M_TEETER               = 0x6040,
  M_FIT                  = 0x6050,
  M_BEVEL                = 0x6060,
  XZ_CURVE               = 0x6070,
  YZ_CURVE               = 0x6080,
  INTERPCT               = 0x6090,
  DEFORM_LIMIT           = 0x60A0,

  USE_CONTOUR            = 0x6100,
  USE_TWEEN              = 0x6110,
  USE_SCALE              = 0x6120,
  USE_TWIST              = 0x6130,
  USE_TEETER             = 0x6140,
  USE_FIT                = 0x6150,
  USE_BEVEL              = 0x6160,

  DEFAULT_VIEW           = 0x3000,
  VIEW_TOP               = 0x3010,
  VIEW_BOTTOM            = 0x3020,
  VIEW_LEFT              = 0x3030,
  VIEW_RIGHT             = 0x3040,
  VIEW_FRONT             = 0x3050,
  VIEW_BACK              = 0x3060,
  VIEW_USER              = 0x3070,
  VIEW_CAMERA            = 0x3080,
  VIEW_WINDOW            = 0x3090,

  VIEWPORT_LAYOUT_OLD    = 0x7000,
  VIEWPORT_DATA_OLD      = 0x7010,
  VIEWPORT_LAYOUT        = 0x7001,
  VIEWPORT_DATA          = 0x7011,
  VIEWPORT_DATA_3        = 0x7012,
  VIEWPORT_SIZE          = 0x7020,
  NETWORK_VIEW           = 0x7030
} Chunks3DS;

class Loader3DS
{
	protected:
	friend class File3D;
	bool
	read3DSFile(ArRef<FDIStream>,ArRef<Shape3D> shapeOut);
	
	void
	setUrlDir(const StlString& dir);
	
	const StlString &
	getMessage();

	
	protected:
	
	bool
	checkMesh();
	
	unsigned int
	read3DSChunk(ArRef<FDIStream> stream,unsigned short chunkId);

	unsigned int
	read3dsInt(ArRef<AbstractIStream> stream);

	short
	read3dsShort(ArRef<AbstractIStream> stream);
	
	double
	read3dsFloat(ArRef<AbstractIStream> stream);
	
	unsigned int
	readColor(ArRef<AbstractIStream> stream,double &r,double &g, double &b);
	
	unsigned int
	readPercentage(ArRef<AbstractIStream> stream,double &p);
	
	protected:
	bool _warning;
	StlString _message;
	StlString _urlDir;
	StlVector<Util3D::Dbl3> * _vertices;
	StlVector<Util3D::Face> * _faces;
	StlVector<Util3D::Dbl2> *  _textures;
	ArRef<Mesh3D> _mesh;
	string _objectName;
	ArRef<ShapePart3D> _object;
	StlMap<StlString,ArRef<ShapePart3D> > _objects;
	StlMap<StlString,ArRef<Transform3D> > _transf;
	ArRef<FDIStream> _stream;
	ArRef<Shape3D> _shape;

	StlMap<string,pair<ArRef<URLTexture>,ArRef<URLTexture> > > _urlTextures;
	StlMap<string,pair<ArRef<Transform2D>,ArRef<Transform2D> > > _urlTexturesTransform;
	StlMap<string,ArRef<Material3D> > _materials;
	StlMap<string,bool> _doubleSided;
	StlMap<string,Util3D::Dbl3> _color;
	
	string _lastMaterialName;
	ArRef<Material3D> _lastMaterial;

	string _lastHierarchyObject;
	int _lastHierValue;
	StlVector<pair<int, string > > _hierarchy;
	Util3D::Dbl3 _lastTr;
	StlVector<int> _lastNumFaces;
	StlVector<Util3D::Face> _lastFaces;
	bool _firstTexMap;
	map<string,pair<bool,bool> > _noTiling;
};


class Saver3DS
{
	protected:
	friend class File3D;
	void
	save3DS(ArRef<FDOStream> stream,ArConstRef<Shape3D> shape);
	
	void
	saveSurface3D(ArRef<MemoryOStream> stream,ArConstRef<Surface3D> surf);
	void saveSurfaces(ArConstRef<ShapePart3D> shapePart,ArConstRef<Transform3D> parentTr);

	ArRef<MemoryOStream> createChunkM3DMAGIC();

	
	ArRef<MemoryOStream> createChunkMDATA();
	ArRef<MemoryOStream> createChunkMESH_VERSION();
	ArRef<MemoryOStream> createChunkMASTER_SCALE(double scale);
	ArRef<MemoryOStream> createChunkMAT_ENTRY(ArConstRef<Surface3D> surf3D);
	ArRef<MemoryOStream> createChunkMAT_NAME(const StlString& name);
	ArRef<MemoryOStream> createChunkMAT_TEXMAP(ArConstRef<URLTexture> texture,ArConstRef<Transform2D> tr2D);
	ArRef<MemoryOStream> createChunkMAT_TEX2MAP(ArConstRef<URLTexture> texture,ArConstRef<Transform2D> tr2D);
	ArRef<MemoryOStream> createChunkMAT_MAPNAME(const StlString& name);
	ArRef<MemoryOStream> createChunkTexTransform(ArConstRef<Transform2D> tr2D);
	ArRef<MemoryOStream> createChunkMAT_MAP_TILING();
	ArRef<MemoryOStream> createChunkNAMED_OBJECT(ArConstRef<Surface3D> surface,ArConstRef<Transform3D> tr);
	ArRef<MemoryOStream> createChunkN_TRI_OBJECT(ArConstRef<Surface3D> surface,ArConstRef<Transform3D> tr);
	ArRef<MemoryOStream> createChunkPOINT_ARRAY(StlMap<size_t,Util3D::Dbl3> & vertices);
	ArRef<MemoryOStream> createChunkFACE_ARRAY(const StlVector<Util3D::Face> & faces,const StlString& name);
	ArRef<MemoryOStream> createChunkMSH_MAT_GROUP(const StlVector<Util3D::Face> & faces,const StlString &name);
	ArRef<MemoryOStream> createChunkTEX_VERTS(const StlVector<Util3D::Dbl2> & textures);
	ArRef<MemoryOStream> newChunk(unsigned short chunkId);
	void chunk_set_size(ArRef<MemoryOStream> stream);
	
	ArRef<MemoryOStream> createChunkColor(unsigned short id, double r, double g,double b);
	ArRef<MemoryOStream> createChunkPercentage(unsigned short id, unsigned int percentage);
	ArRef<MemoryOStream> createChunkMAT_SHADING();
	ArRef<MemoryOStream> createChunkMAT_TWO_SIDE();
	protected:
//	map<StlString,pair<ArConstRef<Material3D>, pair<ArConstRef<Texture>,ArConstRef<Texture> > > > _materials;
	StlVector<ArConstRef<Surface3D> > _surfaces;
	StlVector<ArRef<Transform3D> > _transform;
};

} // namespace AReVi

#endif // AREVI_CONTRIB_FILE3D_3DS_H

//----------------------------------------------------------------------------
