// LI2/ENIB - AReVi-Contrib - 3D file loader/saver
//----------------------------------------------------------------------------
// file        : AReVi/Contrib/File3D/file3D.h
// author(s)   : C. Septseault
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVI_CONTRIB_FILE3D_FILE3D_H
#define AREVI_CONTRIB_FILE3D_FILE3D_H 1

#include "AReVi/arObject.h"

namespace AReVi
{
class Shape3D;
class ShapePart3D;

class File3D
{
	public:

	static
	ArRef<Shape3D>
	load(const string& name);
	
	static
	ArRef<Shape3D>
	load3DS(const string& name);

	static
	ArRef<Shape3D>
	loadWaveFront(const string& name);

	static
	ArRef<Shape3D>
	loadLightWave(const string& name);

	static
	ArRef<Shape3D>
	loadImagine(const string& name);

	static
	ArRef<Shape3D>
	loadAutocad(const string& name);

	static
	bool
	save3DS(const string& name, ArConstRef<Shape3D> shapeIn);

	static
	bool
	saveWaveFront(const string& name, ArConstRef<Shape3D> shapeIn);

	static
	bool
	saveLightWave(const string& name, ArConstRef<Shape3D> shapeIn);

	static
	bool
	saveImagine(const string& name, ArConstRef<Shape3D> shapeIn);

	static
	bool
	saveAutocad(const string& name, ArConstRef<Shape3D> shapeIn);

	static
	bool
	saveVrml1(const string& name, ArConstRef<Shape3D> shapeIn);

	static
	bool
	saveVrml2(const string& name, ArConstRef<Shape3D> shapeIn);
	
	static
	bool fail();
	
	static
	StlString
	readMessage();
	
	//Utilities
	static
	StlVector<ArRef<Shape3D> >
	divideShape(ArRef<Shape3D> shape);
	
        static
        ArRef<Shape3D>
        mergeShapes(StlVector<ArRef<Shape3D> > shapes);

	static
	void
	smooth(ArRef<Shape3D> shape);
	
	protected:
	static
	void
	_smoothShapePart(ArRef<ShapePart3D> shp);
	
	static bool _failed;
	static StlString _message;
	
};

} // namespace AReVi

#endif // AREVI_CONTRIB_FILE3D_FILE3D_H

//----------------------------------------------------------------------------
