// LI2/ENIB - AReVi-Contrib - 3D file loader/saver
//----------------------------------------------------------------------------
// file        : AReVi/Contrib/File3D/wavefront.h
// author(s)   : C. Septseault
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVI_CONTRIB_FILE3D_WAVEFRONT_H
#define AREVI_CONTRIB_FILE3D_WAVEFRONT_H 1

#include "AReVi/Lib3D/util3D_priv.h"
#include "AReVi/Shapes/mesh3D.h"

#ifdef __sgi
#define IOSTREAMH // avoid inclusion of iostream.h from FlexLexer.h
#else // not __sgi
#define _BACKWARD_IOSTREAM_H // same thing ...
#define _CPP_BACKWARD_IOSTREAM_H // same thing ...
#endif // not __sgi

#define yyFlexLexer _WFOFlexLexer
#include <FlexLexer.h>
#undef yyFlexLexer

namespace AReVi
{
typedef struct
{
	int x,y,z;
}Int3;


class WFOParser : protected _WFOFlexLexer
{

// Construction / Destruction

  protected:
  friend class File3D;
  WFOParser(istream * input);	
  virtual ~WFOParser();

// Parsing

  virtual
  int
  parse(void);

// Errors Messages

  virtual
  void
  yyerror(const char * text);

// Others Errors
  inline
  bool
  fail(){return _failed;}

  inline
  const StlString&
  getMessage(){return _message;}

// Parameters

  void
  setShape(ArRef<Shape3D> sh);

  void
  setUrlDir(const StlString & dir);


// Utilities

  protected:

  int yylex(void) {return(0);} // prevent SGI compiler from complaining

  virtual
  int
  yylex(void * arg);

  virtual
  void
  _nextLine(void);

  virtual
  void
  _store(const char * text);
  void _warningNotImplemented();
protected:

  int _nbErrors;
  int _nbLines;
  StlString _line;
  StlVector<Util3D::Dbl3>* vertices ;
  StlVector<Util3D::Dbl3>* normals ;
  StlVector<Util3D::Dbl2>* vtex ;
  StlVector<Util3D::Face>* faces;
  StlVector<Util3D::Face>* textureFaces;
  StlVector<Util3D::Face>* normalFaces;
  ArRef<Shape3D> _shape;
  ArRef<Mesh3D> _mesh;
  string _message;
  bool _failed;
  StlString _urlDir;
};

class SaverWaveFront
{
        public:
        bool
        saveWaveFront(FILE * file, ArConstRef<Shape3D> shapeIn);

        const StlString &
        getMessage();

        protected:

        StlString _message;

};

} // namespace AReVi

#endif // AREVI_CONTRIB_FILE3D_WAVEFRONT_H

//----------------------------------------------------------------------------
