// LI2/ENIB - AReVi-Contrib - ArConsole ???
//----------------------------------------------------------------------------
// file        : AReVi/Contrib/arConsole.h
// author(s)   : T. Jourdan
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVI_CONTRIB_ARCONSOLE_H
#define AREVI_CONTRIB_ARCONSOLE_H 1

#include <stack>
#include <queue>
using namespace std;

#include "AReVi/arSystem.h"
#include "AReVi/Lib3D/osdText.h"
#include "AReVi/Lib3D/window3D.h"

namespace AReVi {

class ArConsoleAttr {
public :
  double r, g, b, a;
  StlString font;
  /** TO DO : add attributes like underlined, bold, blink... */
};

class ArConsole : public OSDText {
public :
  AR_CLASS(ArConsole)
  AR_CONSTRUCTOR_1(ArConsole,
		   bool, readOnly)

  // ----- Console management

  virtual
  void
  setBackgroundColor(double r, double g, double b, double a);

  virtual
  void
  getBackgroundColor(double &rOut, double &gOut, double &bOut, double &aOut) const;

  virtual
  void
  setSize(int nblines);

  virtual
  int
  getSize(void) const;

  virtual
  void
  setBufferSize(int nbLines);

  virtual
  int
  getBufferSize(void) const;

  virtual
  void
  clear(void);

  virtual
  void
  scroll(int nbLines);

  // ----- Event management

  virtual
  void
  setReleaseKeyboardFocusKey(const StlString& key);

  virtual
  void
  grabKeyboardFocus(ArRef<Window3D> window);

  virtual
  void
  releaseKeyboardFocus(void);

  AR_UNSTUBBED virtual
  void
  releaseKeyboardFocus(const Renderer3D::KeyboardEvent& evt);

  // ----- Text attributes

  virtual
  void
  setColor(double r, double g, double b, double a);

  virtual
  void
  getColor(double &rOut, double &gOut, double &bOut, double &aOut) const;

  virtual
  bool
  setFont(const StlString& fontName);

  AR_UNSTUBBED virtual
  void
  pushAttrib(ArConsoleAttr attr);

  virtual
  void
  popAttrib(void);

  virtual
  void
  flushAttrib(void);

  virtual
  void
  pushDefaultAttrib(void);

  // ----- Text management

  virtual
  void
  storeCharacter(char c);

  virtual
  void
  push(const StlString& text);

  friend ostream& operator<<(ostream& output, const ArRef<ArConsole>& console);

//-------- Serialization operations ------------------------------------------

  AR_UNSTUBBED virtual
  void
  serialize(SerializationDependencies & dependenciesOut,
            ArRef<AbstractOStream> output) const;

  AR_UNSTUBBED virtual
  bool // success
  unserialize(const SerializationDependencies & dependencies,
              ArRef<AbstractIStream> input);

//----------------------------Event management -------------------------------

  AR_CALLBACK(ArConsole,BackgroundChanged,_backgroundChangedCB,
              /* no field */)

  AR_CALLBACK(ArConsole,ForegroundChanged,_foregroundChangedCB,
              /* no field */)

  AR_CALLBACK(ArConsole,ScrollText,_scrollTextCB,
              int nbLines;)

  AR_CALLBACK(ArConsole,BufferSetup,_bufferSetupCB,
              /* no field */)

  AR_CALLBACK(ArConsole,InputText,_inputTextCB,
              StlString text;)

  AR_CALLBACK(ArConsole,PushText,_pushTextCB,
              StlString text;)

  AR_CALLBACK(ArConsole,StoreCharacter,_storeCharacterCB,
              char c;)

  AR_CALLBACK(ArConsole,ClearText,_clearTextCB,
              /* no field */)

  AR_UNSTUBBED
  AR_CALLBACK(ArConsole,PushAttrib,_pushAttribCB,
              ArConsoleAttr attr;)

  AR_CALLBACK(ArConsole,PopAttrib,_popAttribCB,
              /* no field */)

  AR_CALLBACK(ArConsole,FlushAttribs,_flushAttribsCB,
              /* no field */)

protected :

  // ----- Forbidden operations

  ArConsole(ArCW & arCW);

  virtual
  void
  _adjustSize(void);

  virtual
  void
  _render(unsigned int width, unsigned int height) const;

  virtual
  void
  _keyboardCB(const Renderer3D::KeyboardEvent& evt);

protected :
  StlString _focusKey;
  bool _up, _ro;
  ArConsoleAttr _bg;
  int _bufferSize, _displaySize, _pos;
  StlString _inputBuffer;
  stack<ArConsoleAttr> _attr;
  StlList<pair<ArConsoleAttr, StlString> > _text;
  StlList<pair<ArConsoleAttr, StlString> >::iterator _currentPos;
  queue<StlString> _textStream;
  ArRef<Window3D> _window;

  CallbackManager<ArConsole, BackgroundChangedEvent> _backgroundChangedCB;
  CallbackManager<ArConsole, ForegroundChangedEvent> _foregroundChangedCB;
  CallbackManager<ArConsole, ScrollTextEvent> _scrollTextCB;
  CallbackManager<ArConsole, BufferSetupEvent> _bufferSetupCB;
  CallbackManager<ArConsole, InputTextEvent> _inputTextCB;
  CallbackManager<ArConsole, PushTextEvent> _pushTextCB;
  CallbackManager<ArConsole, StoreCharacterEvent> _storeCharacterCB;
  CallbackManager<ArConsole, ClearTextEvent> _clearTextCB;
  CallbackManager<ArConsole, PushAttribEvent> _pushAttribCB;
  CallbackManager<ArConsole, PopAttribEvent> _popAttribCB;
  CallbackManager<ArConsole, FlushAttribsEvent> _flushAttribsCB;
};

} // namespace AReVi

#endif // AREVI_CONTRIB_ARCONSOLE_H

//----------------------------------------------------------------------------
