// LI2/ENIB - AReVi-Contrib - Lua bindings
//----------------------------------------------------------------------------
// file        : AReVi/Contrib/arLua.h
// author(s)   : D. Herviou
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVI_CONTRIB_ARLUA_H
#define AREVI_CONTRIB_ARLUA_H 1

#include "AReVi/arObject.h"

using namespace std;

using namespace AReVi;

namespace AReVi{


#define MARSHALLAREVILUA "MarshallAReViLua"

#define CHECK_RETRIEVE_ARREF(_LUASTATE,_OBJECT,_CLASS,_INDEX)                               \
  ArRef<ArObject> _CLASS##_OBJECT = LuaBind::retrieveArRef(_LUASTATE,_INDEX);                  \
  if (!_CLASS##_OBJECT.valid()) {                                                              \
    StlString errMsg = StlString("argument ")+intToStr(_INDEX-1)+                              \
                       StlString(" is not an AReVi Object");                                   \
    LuaInfo::dumpLuaStack(_LUASTATE);                                                          \
    Lua::luaL_error(_LUASTATE,errMsg.c_str());                                                 \
    return 0;                                                                                  \
  }                                                                                            \
  if (!_CLASS##_OBJECT->getClass()->isA(_CLASS::CLASS())){                                     \
    StlString errMsg = _CLASS##_OBJECT->getAlias()+                                            \
                       StlString(" is not a kind of ")+_CLASS::CLASS()->getName();             \
    Lua::luaL_error(_LUASTATE,errMsg.c_str());                                                 \
    return 0;                                                                                  \
  }                                                                                            \
  ArRef<_CLASS> _OBJECT = ar_down_cast<_CLASS>(_CLASS##_OBJECT);




//arg must be one of thoses :
//number  cfunction  function  userdata  lightuserdata string boolean
//table  nil  none  noneornil


#define CHECK_LUA_ARG(_LUASTATE, _ARG, _INDEX)                       \
  if (!Lua::lua_is##_ARG(_LUASTATE,_INDEX)) {                        \
    StlString errMsg = StlString("argument ")+intToStr(_INDEX-1)+    \
                       StlString(" is not a ")+StlString(#_ARG);     \
    Lua::luaL_error(_LUASTATE,errMsg.c_str());                       \
    return 0;                                                        \
  }


extern "C" bool LuaWrapper_init(const StlVector<StlString> *);
extern "C" void LuaWrapper_destroy(void);

typedef int (*arlua_CFunction)(void*);

typedef struct {
    const char *name;
    arlua_CFunction func;
}arluaL_reg;


class LuaExport;

/*! \class Lua
  \brief Lua : une classe de wrap entre Lua et AReVi

  contient un ensemble de fonction et d'numration dont la convention
  de nommage fait que les noms de fonctions Lua porte le mme nom en
  AReVi avec le prfixe de classe Lua::

  Un ensemble de pointeur de fonction sont l pour initialiser le plugin

*/
class Lua:public ArObject{
public:
  AR_UNSTUBBED AR_CLASS(Lua)

  /**
   * based on lua.h declaration.
   * Explicit affectation is willing
   */
  enum ARLUA_ERROR{
    NONE=0,
    YIELD=1,
    ERRRUN=2,
    ERRSYNTAX=3,
    ERRMEM=4,
    ERRERR=5
  };


  /** Lua index table (two particular case) */
  enum ARLUA_INDEX{
    LUA_REGISTRYINDEX = -10000,
    LUA_ENVIRONINDEX  = -10001,
    LUA_GLOBALSINDEX  = -10002
  };

  /** */
  enum ARLUA_LUA_TYPE{
    LUA_TNONE      = -1,

    LUA_TNIL             =   0,
    LUA_TBOOLEAN         =   1,
    LUA_TLIGHTUSERDATA   =   2,
    LUA_TNUMBER          =   3,
    LUA_TSTRING          =   4,
    LUA_TTABLE           =   5,
    LUA_TFUNCTION        =   6,
    LUA_TUSERDATA        =   7,
    LUA_TTHREAD          =   8

  };

  /** */
  enum ARLUA_MISC{
    LUA_MULTRET = -1
  };


  typedef void * (*lua_Allocator) (void *ud,
                                       void *ptr,
                                       size_t osize,
                                       size_t nsize);

  /*----------  lua basic function ----------*/

  /** ouvre un tat lua : deprecated */
  static  void* lua_open(void);

  /** ouvre un tat lua */
  static  void* lua_newstate(lua_Allocator f, void* ud);

  /** ferme l'tat lua */
  static  void  lua_close(void* L);

  /*---------- opening function ----------*/
  /** ouvre les fonctions de bases de Lua  */
  static  int luaopen_base(void* L);

  /** ouvre les fonctions sur les tables */
  static  int luaopen_table(void* L);

  /** ouvre les fonctions sur les entres-sorties */
  static  int luaopen_io(void* L);

  /** ouvre les fonctions de manipulation de chaines de caractres*/
  static  int luaopen_string(void* L);

  /** ouvre les fonctions mathmatiques */
  static  int luaopen_math(void* L);

  /** ouvre les fonctions de link dynamique */
  //static  int luaopen_loadlib(void* L);

  /** ouvre les fonctions de debuggage */
  static  int luaopen_debug(void* L);

  /*---------- execution stack ----------*/
  /** fait un appel non protg  la fonction sur la pile
      avec nargs arguments et nresults resultats */
  static  void lua_call (void* L, int nargs, int nresults);

  /** sauvegarde la pile en cas d'chec de l'execution @see lua_call*/
  static  int  lua_pcall (void* L, int nargs, int nresults, int errfunc);

  /** */
  static  int  lua_cpcall (void* L, int(*func)(void*), void *ud);

  /*---------- error ----------*/
  /** verifie l'erreur d'execution qui vient d'avoir lieu et l'enlve
      de la pile */
  static  int  lua_error (void* L);

  /** */
  static  void lua_concat (void* L, int n);

  /*---------- divers ----------*/
  /** */
  static  void lua_getfield(void *L , int index, const char* k);
  /** */
  static  void lua_getglobal(void* L,const char* s);
  /** */
  static  void lua_setglobal(void* L,const char* s);
  /** */
  static  void lua_getregistry(void* L);

  /*---------- auxiliar library ----------*/
  /** ouvre une librairie de fonction  */
  static  void luaL_openlib(void* L,
			    const char* libname,
			    const void *l,
			    int nup);


#ifdef LUA_JIT
  static int luaopen_jit(void* L);
#endif

  /** ouvre toutes les librairies de fonction  */
  static  void luaL_openlibs(void* L);

  /** charge du code Lua  partir d'un fichier */
  static  int luaL_loadfile (void* L, const char *filename);

  /** */
  static  int luaL_loadbuffer (void* L,
			       const char *buff,
			       size_t sz,
			       const char *name);

  static  int luaL_dofile(void* L, const char* filename);
  static  int luaL_dostring(void* L, const char* str);
  /**< */
  //static  int lua_dobuffer(void* L, const char* buff,
  //				 size_t sz, const char* n);



  /**< */
  static  int  luaL_newmetatable(void* L, const char* name);

  /**< */
  static  void luaL_getmetatable(void* L, const char* name);

  /**< */
  static  int luaL_error (void *L, const char *fmt, ...);

  /*---------- table manipulation ----------*/
  /** */
  static  void lua_setmetatable(void* L, int index);
  /** */
  static  void lua_newtable(void* L);
  /** */
  static  void lua_createtable(void* L, int narr, int nrec);
  /** */
  static  void lua_gettable(void* L,int index);
  /** */
  static  void lua_settable(void* L,int index);
  /** */
  static  void lua_rawget(void* L, int idx);
  /** */
  static  void lua_rawgeti(void* L, int idx, int n);
  /** */
  static  void lua_rawset(void* L, int idx);
  /** */
  static  void lua_rawseti(void* L, int idx, int n);
  /** */
  static  int  lua_next(void* L, int idx);

  /*---------- pushing element on stack ----------*/
  /** mettre nul sur la pile*/
  static  void lua_pushnil(void* L);

  /** mettre un boolen sur la pile */
  static  void lua_pushboolean(void* L, int b);

  /** mettre un nombre sur la pile */
  static  void lua_pushnumber(void* L, double n);

  /** mettre une chaine de caractres sur la pile */
  static  void lua_pushlstring(void* L,
			       const char *s,
			       size_t length);

  /** mettre une chaine de caractres sur la pile */
  static  void  lua_pushstring (void* L, const char *s);

  /** */
  static  void  lua_pushliteral (void* L, const char *s);

  /** */
  static  void  lua_pushlightuserdata(void* L, void* data);

  /** */
  static  void  lua_pushcfunction(void* L, int(*cfuntion)(void*) );

  /** */
  static  void* lua_newuserdata(void* L, int nBytes);

  /*---------- query stack ----------*/
  /** */
  static  int              lua_toboolean(void* L, int index);

  /** */
  static  double           lua_tonumber(void* L, int index);

  /** */
  static  const char*      lua_tostring(void* L, int index);

  /** */
  static  void*            lua_touserdata(void* L, int index);

  /** */
  static  size_t           lua_strlen(void* L, int index);

  /** */
  static  arlua_CFunction  lua_tocfunction(void* L, int index);

  /*---------- testing element ----------*/
  /** */
  static  int lua_isnumber(void* L, int index);

  /** */
  static  int lua_iscfunction(void* L, int index);

  /** */
  static  int lua_isfunction(void* L, int index);

  /** */
  static  int lua_isuserdata(void* L, int index);

  /** */
  static  int lua_islightuserdata(void* L, int index);

  /** */
  static  int lua_isstring(void* L, int index);

  /** */
  static  int lua_isboolean(void* L, int index);

  /** */
  static  int lua_istable(void* L, int index);

  /** */
  static  int lua_isnil(void* L, int index);

  /** */
  static  int lua_isnone(void* L, int index);

  /** */
  static  int lua_isnoneornil(void* L, int index);

  /** */
  static  int lua_type(void* L, int index);

  /*---------- stack manipulation ----------*/
  /** */
  static  int  lua_gettop(void* L);

  /** */
  static  void lua_settop(void* L, int index);

  /** */
  static  void lua_pop(void* L, int index);

  /** */
  static  void lua_pushvalue(void* L, int index);

  /** */
  static  void lua_remove(void* L, int index);

  /** */
  static  void lua_insert(void* L, int index);

  /** */
  static  void lua_replace(void* L, int index);

  /** */
  static  int  lua_checkstack(void* L, int sz);

  /*---------- garbage manipulation ----------*/
  /** */
  static void luaC_collectgarbage(void* L);

  /** */
  static int  lua_getgcthreshold(void* L);

  /** */
  static void lua_setgcthreshold(void* L,int threshold);

  /** */
  static int lua_gc (void *L, int what, int data);



private:

  Lua(ArCW& arCW);

  /**
   * lua basic function
   */
  static  void* (*_arlua_lua_open)(void);
  static  void  (*_arlua_lua_close)(void* L);
  static  void* (*_arlua_lua_newstate)(lua_Allocator allocator, void* ud);


  /**
   * opening function
   */
  static  int (*_arlua_luaopen_base)(void* L);
  static  int (*_arlua_luaopen_table)(void* L);
  static  int (*_arlua_luaopen_io)(void* L);
  static  int (*_arlua_luaopen_string)(void* L);
  static  int (*_arlua_luaopen_math)(void* L);
  //  static  int (*_arlua_luaopen_loadlib)(void* L);
  static  int (*_arlua_luaopen_debug)(void* L);

  /**
   * execution stack
   */
  static  void (*_arlua_lua_call) (void* L, int nargs, int nresults);
  static  int  (*_arlua_lua_pcall) (void* L, int nargs, int nresults, int errfunc);
  static  int  (*_arlua_lua_cpcall) (void* L, int(*func)(void*), void *ud);

  /**
   * error
   */
  static  int  (*_arlua_lua_error) (void* L);
  static  void (*_arlua_lua_concat) (void* L, int n);

  /**
   * divers
   */
  static  void (*_arlua_lua_getglobal)(void* L,const char* s);
  static  void (*_arlua_lua_getfield)(void* L,int index, const char* s);
  static  void (*_arlua_lua_setglobal)(void* L,const char* s);
  static  void (*_arlua_lua_getregistry)(void* L);

  /**
   * auxiliar library
   */
  static  void (*_arlua_luaL_openlib)(void* L,
				  const char* libname,
				  const void *l,
				  int nup);
#ifdef LUA_JIT
  static int (*_arlua_luaopen_jit)(void *L);
#endif

  static  void (*_arlua_luaL_openlibs)(void* L);

  static  int (*_arlua_luaL_loadfile )(void* L, const char *filename);
  static  int (*_arlua_luaL_loadbuffer) (void* L,
				     const char *buff,
				     size_t sz,
				     const char *name);

  static  int (*_arlua_luaL_dofile)(void* L, const char* filename);
  static  int (*_arlua_luaL_dostring)(void* L, const char* str);
/*   static  int (*_arlua_lua_dobuffer)(void* L, const char* buff, */
/* 				 size_t sz, const char* n); */

  static  int  (*_arlua_luaL_newmetatable)(void* L, const char* name);
  static  void (*_arlua_luaL_getmetatable)(void* L, const char* name);
  static  int  (*_arlua_luaL_error)(void *L, const char *fmt, ...);


  /**
   * table manipulation
   */
  static  void (*_arlua_lua_setmetatable)(void* L, int index);
  static  void (*_arlua_lua_newtable)(void* L);
  static  void (*_arlua_lua_createtable)(void* L, int narr, int nrec);
  static  void (*_arlua_lua_gettable)(void* L,int index);
  static  void (*_arlua_lua_settable)(void* L,int index);
  static  void (*_arlua_lua_rawget)(void* L, int idx);
  static  void (*_arlua_lua_rawgeti)(void* L, int idx, int n);
  static  void (*_arlua_lua_rawset)(void* L, int idx);
  static  void (*_arlua_lua_rawseti)(void* L, int idx, int n);
  static  int  (*_arlua_lua_next)(void* L, int idx);


  /**
   * pushing element on stack
   */
  static  void (*_arlua_lua_pushnil)(void* L);
  static  void (*_arlua_lua_pushboolean)(void* L, int b);
  static  void (*_arlua_lua_pushnumber)(void* L, double n);
  static  void (*_arlua_lua_pushlstring)(void* L,
				     const char *s,
				     size_t length);

  static  void  (*_arlua_lua_pushstring )(void* L, const char *s);
  static  void  (*_arlua_lua_pushliteral )(void* L, const char *s);
  static  void  (*_arlua_lua_pushlightuserdata)(void* L, void* data);
  static  void  (*_arlua_lua_pushcfunction)(void* L, int(*cfuntion)(void*) );
  static  void* (*_arlua_lua_newuserdata)(void* L, int nBytes);

  /**
   * query stack
   */
  static  int              (*_arlua_lua_toboolean)(void* L, int index);
  static  double           (*_arlua_lua_tonumber)(void* L, int index);
  static  const char*      (*_arlua_lua_tostring)(void* L, int index);
  static  void*            (*_arlua_lua_touserdata)(void* L, int index);
  static  size_t           (*_arlua_lua_strlen)(void* L, int index);
  static  arlua_CFunction  (*_arlua_lua_tocfunction)(void* L, int index);

  /**
   * testing element
   */
  static  int (*_arlua_lua_isnumber)(void* L, int index);
  static  int (*_arlua_lua_iscfunction)(void* L, int index);
  static  int (*_arlua_lua_isfunction)(void* L, int index);
  static  int (*_arlua_lua_isuserdata)(void* L, int index);
  static  int (*_arlua_lua_islightuserdata)(void* L, int index);
  static  int (*_arlua_lua_isstring)(void* L, int index);
  static  int (*_arlua_lua_isboolean)(void* L, int index);
  static  int (*_arlua_lua_istable)(void* L, int index);
  static  int (*_arlua_lua_isnil)(void* L, int index);
  static  int (*_arlua_lua_isnone)(void* L, int index);
  static  int (*_arlua_lua_isnoneornil)(void* L, int index);
  static  int (*_arlua_lua_type)(void* L, int index);

  /**
   * stack manipulation
   */
  static  int  (*_arlua_lua_gettop)(void* L);
  static  void (*_arlua_lua_settop)(void* L, int index);
  static  void (*_arlua_lua_pop)(void* L, int index);
  static  void (*_arlua_lua_pushvalue)(void* L, int index);
  static  void (*_arlua_lua_remove)(void* L, int index);
  static  void (*_arlua_lua_insert)(void* L, int index);
  static  void (*_arlua_lua_replace)(void* L, int index);
  static  int  (*_arlua_lua_checkstack)(void* L, int sz);

  /**
   * garbage manipulation
   */
  static void (*_arlua_luaC_collectgarbage)(void* L);
  static void (*_arlua_lua_setgcthreshold)(void* L,int threshold);
  static int  (*_arlua_lua_getgcthreshold)(void* L);
  static int  (*_arlua_lua_gc)(void* L, int what, int data);

private:

  /**
   * plugin initialisation
   */
  friend bool LuaWrapper_init(const StlVector<StlString> *);

  /**
   * plugin destruction
   */
  friend void LuaWrapper_destroy(void);

  /**
   * plugin initialization succeed?
   */
  static bool _initSuccess;

}; /**< end of class Lua */




class LuaBind:public ArObject{


public:
  AR_UNSTUBBED AR_CLASS(LuaBind)

  struct lua_data_share{
    ArObject* data;
    // bool cg; //collect garbage
    ArRef<ArObject>* obj;
  };

  struct ltdatashare
  {
    bool operator()(ArObject * obj1,ArObject * obj2) const
    {
      return obj1 < obj2;
    }
  };


  /**< helpfull functions */
  static  ArRef<ArObject> retrieveArRef(void* L,int index);

  static  int             shareArRef   (void* L, ArRef<ArObject> objShare/*, bool garbageCollected=true*/);

  static  void            setLookUp  (void* L, int index);

  static  StlString       findMetaTable (void* L, int index);

  static  StlString       findMetaTable (void* L, StlString& clsName);

  /**< helpfull data sharing*/
  static  int shareVectorArRef(void* L, StlVector<ArRef<ArObject> >& );

  static  int shareVectorDouble(void* L, StlVector<double>& );

  static  int shareVectorInt   (void* L, StlVector<int>& );

  static  int shareVectorStlString(void* L, StlVector<StlString>& );

  /**< needed to register destructor in lua bindings object*/
  static int destroy(void* L);


  /**< class exports*/
  static void exportClass(void* L,
			  const char* clsName,
			  const arluaL_reg* methods ,
			  const arluaL_reg* constructor);

  /**< marshalling Lua table creation */
  static void createMarshallArea(void *L);


protected:
  friend class LuaExport;

  /**< marshalling area */
  static map<void*, StlString> _objectsTable;

  /**< map<className,exportedMethodsOfTheClass> */
  static map<StlString,const arluaL_reg*> _lookUp;

private:

  LuaBind(ArCW& arCW);


  /**
   * plugin initialisation
   */
  friend bool LuaWrapper_init(const StlVector<StlString> *);

  /**
   * plugin destruction
   */
  friend void LuaWrapper_destroy(void);

  /**
   * plugin initialization succeed?
   */
  static bool _initSuccess;

}; /**< end of class LuaBind */


class LuaInfo:public ArObject{

public:
  AR_UNSTUBBED AR_CLASS(LuaInfo)

  static void checkLuaType(void* L, int index);

  static void dumpLuaStack(void* L);

  static void sizeofLuaStack(void* L);

private:

  LuaInfo(ArCW& arCW);

  /**
   * plugin initialisation
   */
  friend bool LuaWrapper_init(const StlVector<StlString> *);

  /**
   * plugin destruction
   */
  friend void LuaWrapper_destroy(void);

  /**
   * plugin initialization succeed?
   */
  static bool _initSuccess;


}; /**< end of class LuaDebug */


}/** namespace AReVi */


#endif // AREVI_CONTRIB_ARLUA_H

//----------------------------------------------------------------------------
