// LI2/ENIB - AReVi-Contrib - ArTree ???
//----------------------------------------------------------------------------
// file        : AReVi/Contrib/arTree.h
// author(s)   : T. Jourdan
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVI_CONTRIB_ARTREE_H
#define AREVI_CONTRIB_ARTREE_H

#include "AReVi/arObject.h"

namespace AReVi {

class ArTreeNode;

// --------------------------------------------------------------------
// ArTree
// --------------------------------------------------------------------

class ArTree : public ArObject {
// ----- Construction / Destruction
public :
  AR_UNSTUBBED AR_CLASS(ArTree)
  AR_CONSTRUCTOR(ArTree)

// ----- Node management
  virtual
  ArRef<ArTreeNode>
  getRoot(void);

  virtual
  void
  clear(void);

// ----- Member attributes
private :
  ArRef<ArTreeNode> _root;
};

typedef StlList<ArRef<ArTreeNode> >::iterator ArTreeNodeIter;

// --------------------------------------------------------------------
// ArTreeNode : basic element of the tree
//
//   A node can have many childrens but can't store data
// --------------------------------------------------------------------

class ArTreeNode : public ArObject {
// ----- Construction / Destruction
public :
  AR_UNSTUBBED AR_CLASS(ArTreeNode)
  AR_CONSTRUCTOR(ArTreeNode)

  // ----- Name
  virtual
  void
  setName(const StlString& name);

  virtual
  const StlString&
  name(void);

// ----- Accessibility
  virtual
  ArRef<ArTreeNode>
  parent(void);

  virtual
  ArTreeNodeIter
  find(ArRef<ArTreeNode> child);

  virtual
  size_t
  size(void);

  virtual
  ArTreeNodeIter
  begin(void);

  virtual
  ArTreeNodeIter
  end(void);

// ----- Insertion / Deletion management
  virtual
  void
  insert(ArRef<ArTreeNode> child);

  virtual
  bool
  erase(ArRef<ArTreeNode> child);

  virtual
  void
  clear(void);

// ----- Member attributes
private :
  StlString _name;
  StlList<ArRef<ArTreeNode> > _nodes;
  ArRef<ArTreeNode> _parent;
};

// --------------------------------------------------------------------
// ArTreeLeaf : specific node
//
//   A leaf is a special node that can't have childrens but it can
//   store a data
// --------------------------------------------------------------------

class ArTreeLeaf : public ArTreeNode {
// ----- Construction / Destruction
public :
  AR_UNSTUBBED AR_CLASS(ArTreeLeaf)
  AR_CONSTRUCTOR(ArTreeLeaf)

// ----- Data management
  virtual
  void
  setData(ArRef<ArObject> data);

  virtual
  ArRef<ArObject>
  getData(void);

// ----- Insertion / Deletion management
  virtual
  void
  insert(ArRef<ArTreeNode> child);

// ----- Member attributes
private :
  ArRef<ArObject> _data;
};

} // namespace AReVi

#endif // AREVI_CONTRIB_ARTREE_H

//----------------------------------------------------------------------------
