// LI2/ENIB - AReVi-Contrib - ArTweaker ???
//----------------------------------------------------------------------------
// file        : AReVi/Contrib/arTweaker.h
// author(s)   : T. Jourdan
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVI_CONTRIB_ARTWEAKER_H
#define AREVI_CONTRIB_ARTWEAKER_H 1

#include "AReVi/arObject.h"
#include "AReVi/Contrib/arConsole.h"

#ifndef CLAMP
#define CLAMP(v, a, b) (v < a) ? a : ((v > b) ? b : v)
#endif

namespace AReVi {

class ArTree;
class ArTreeNode;
class Viewer3D;

class TV : public ArObject {
public :
  AR_UNSTUBBED AR_CLASS(TV)
  AR_CONSTRUCTOR(TV)

  AR_CALLBACK(TV,Value,_valueCB,
              /* no field */)

  virtual void set(const StlString& /* input */) {};
  virtual void incr(void) {};
  virtual void decr(void) {};

  virtual void setDescription(const StlString& desc) { _desc = desc; };
  virtual const StlString& getDescription(void) const { return _desc; };

  friend ostream& operator<<(ostream& output, const ArRef<TV>& ref);

protected :
  virtual void _onValueChanged(void);

private :
  StlString _desc;
  CallbackManager<TV, ValueEvent> _valueCB;
};

class TVBool : public TV {
public :
  AR_UNSTUBBED AR_CLASS(TVBool)
  AR_CONSTRUCTOR_1(TVBool,
		   bool&, var);
  virtual void set(const StlString& input);
  virtual bool value(void);
  virtual void incr(void);
  virtual void decr(void);
  friend ostream& operator<<(ostream& output, const ArRef<TVBool>& ref);
private :
  bool* _data;
};

class TVChar : public TV {
public :
  AR_UNSTUBBED AR_CLASS(TVChar)
  AR_CONSTRUCTOR_1(TVChar,
		   char&, var);
  virtual void set(const StlString& input);
  virtual char value(void);
  virtual void incr(void);
  virtual void decr(void);
  friend ostream& operator<<(ostream& output, const ArRef<TVChar>& ref);
private :
  char* _data;
};

class TVInt : public TV {
public :
  AR_UNSTUBBED AR_CLASS(TVInt)
  AR_CONSTRUCTOR_1(TVInt,
		   int&, var)
  AR_CONSTRUCTOR_2(TVInt,
		   int&, var,
		   int, incr)
  AR_CONSTRUCTOR_4(TVInt,
		   int&, var,
		   int, incr,
		   int, lowerBound,
		   int, upperBound)
  virtual void set(const StlString& input);
  virtual int value(void);
  virtual void incr(void);
  virtual void decr(void);
  friend ostream& operator<<(ostream& output, const ArRef<TVInt>& ref);
private :
  int* _data;
  int _incr, _low, _up;
};

class TVFloat : public TV {
public :
  AR_UNSTUBBED AR_CLASS(TVFloat)
  AR_CONSTRUCTOR_1(TVFloat,
		   float&, var)
  AR_CONSTRUCTOR_2(TVFloat,
		   float&, var,
		   float, incr)
  AR_CONSTRUCTOR_4(TVFloat,
		   float&, var,
		   float, incr,
		   float, lowerBound,
		   float, upperBound)
  virtual void set(const StlString& input);
  virtual float value(void);
  virtual void incr(void);
  virtual void decr(void);
  friend ostream& operator<<(ostream& output, const ArRef<TVFloat>& ref);
private :
  float* _data;
  float _incr, _low, _up;
};

class TVDouble : public TV {
public :
  AR_UNSTUBBED AR_CLASS(TVDouble)
  AR_CONSTRUCTOR_1(TVDouble,
		   double&, var)
  AR_CONSTRUCTOR_2(TVDouble,
		   double&, var,
		   double, incr)
  AR_CONSTRUCTOR_4(TVDouble,
		   double&, var,
		   double, incr,
		   double, lowerBound,
		   double, upperBound)
  virtual void set(const StlString& input);
  virtual double value(void);
  virtual void incr(void);
  virtual void decr(void);
  friend ostream& operator<<(ostream& output, const ArRef<TVDouble>& ref);
private :
  double* _data;
  double _incr, _low, _up;
};

class TVStlString : public TV {
public :
  AR_UNSTUBBED AR_CLASS(TVStlString)
  AR_CONSTRUCTOR_1(TVStlString,
		   StlString&, var);
  virtual void set(const StlString& input);
  virtual StlString value(void);
  friend ostream& operator<<(ostream& output, const ArRef<TVStlString>& ref);
private :
  StlString* _data;
};

// ---------------------------
// Class ArTweaker
// ---------------------------

class ArTweaker : public ArConsole {
public :
  AR_UNSTUBBED AR_CLASS(ArTweaker)
  AR_CONSTRUCTOR(ArTweaker)

// ----- Management
  virtual
  bool
  insert(const StlString& path, ArRef<TV> var = TV::nullRef());

  virtual
  bool
  erase(const StlString& path);

// ----- Forbidden operations
protected :

  virtual
  ArRef<ArTreeNode>
  _lookup(const StlString& elems);

  virtual
  StlString
  _eatBasePath(const StlString& path);

  virtual
  void
  _extractPath(const StlString& path, StlVector<StlString>& elems);

  virtual
  void
  _buildText(void);

  virtual
  void
  _keyboardCB(const Renderer3D::KeyboardEvent& evt);

// ----- Member attributes
protected :
  ArRef<ArTree> _tree;
  ArRef<ArTreeNode> _selection;
  bool _editing;
  StlString _buffer;
};

} // namespace AReVi

#endif // AREVI_CONTRIB_ARTWEAKER

//----------------------------------------------------------------------------
