// LI2/ENIB - AReVi-Contrib - Lua bindings
//----------------------------------------------------------------------------
// file        : AReVi/Contrib/arevi_lua_bindings.h
// author(s)   : D. Herviou
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVI_CONTRIB_AREVI_LUA_BINDINGS
#define AREVI_CONTRIB_AREVI_LUA_BINDINGS

#include "AReVi/activity.h"
#include "AReVi/arObject.h"

#include "AReVi/Contrib/arLua.h"

#include <set>
#include <map>
using namespace std;

namespace AReVi{

class LuaExport:public ArObject{
 public:
  AR_UNSTUBBED AR_CLASS(LuaExport)

    /**< open and register some bindings of arevi Object */
  static int luaopen_arevi_lua_bindings(void* L);

private:
  LuaExport(ArCW& arCW);

/*   static map<StlString,const arluaL_reg*> _lookUp; */

};


class LuaInterpreter:public ArObject{
public:
  AR_UNSTUBBED AR_CLASS(LuaInterpreter)

  AR_CONSTRUCTOR(LuaInterpreter)

public:
  int interpret(void* L, StlString& buffer);

  const StlString& getError(void);

protected:
  int _doBuffer(void * L,const char *buff,
		size_t size,
		const char *name);

  int _aux_do(void* L,int status);

  void _callalert(void* L,int status);

  StlString _errorMsg;
};


typedef int (*ptrOpenlib)(void* L);

class LuaService : public ArObject{
  AR_UNSTUBBED AR_CLASS(LuaService)

  AR_CONSTRUCTOR_1(LuaService,StlString, file)

    /**
     * activity in charge of parsing file if needed
     * and push sharing elements on lua stack
     * and execute the call
     */
  virtual bool act(ArRef<Activity> , double);

  /**
   * open more functions in this lua_State
   * by default the lua_State is loaded
   * with a full of functionnality
   * but it can be usefull for user add on library
   */
  void openlib(ptrOpenlib); /*to add more stuff to the current lua state*/

  /**
   * adding more arguments on lua stack
   * this one is called each time a lua call has to be done
   * consequently to ensure a proper call
   * this method returns the number of args it has pushed on stack
   */
  virtual int addMoreArg(void);


  /**
   * initialitize _LS and open default library
   * @see constructor and _checkRecompile
   */
  virtual void init(void);

  /**
   * check if file has been edited since last
   * compilation.
   * close _LS --> destroy share object
   * use init() to open library
   * @return : true means that a compiling has been done
   * @see init
   */
  bool checkRecompile(void);

  /**
   * print readable message
   * when error(s) have occured during lua execution
   */
  void checkError(int luaCodeErr);


protected:

  /**
   * pointer to lua_State*
   */
  void* _LS;


  /**
   * the file parse and byte compile by lua
   * this one must have a 'run' function with
   * a minimum of two arguments
   * to be call from the activity of this object
   */
  StlString _fileName;


  /**
   * last modification of the file _fileName
   */
  time_t _last_mod;

  /**
   * added library when openlib has been called
   * used when _checkRecompile close and open _LS
   */
  set<ptrOpenlib> _lib;


  /**
   * boolean needed to check if lua is in error mode:
   * so we can show the error message only once
   */
  bool _isOnError;


  /**
   * gives the number of elements push on stack til last compiling
   */
  int _totalArg, _topCalling;


  /**
   * a function handler onError has been found
   */
  bool _handler;

};


class LuaObject: public ArObject{
public:
  AR_UNSTUBBED AR_CLASS(LuaObject)

  AR_CONSTRUCTOR_1(LuaObject,StlString, file)

    /**
     * activity in charge of parsing file if needed
     * and push sharing elements on lua stack
     * and execute the call
     */
  virtual bool act(ArRef<Activity> , double);

  /**
   * open more functions in this lua_State
   * by default the lua_State is loaded
   * with a full of functionnality
   * but it can be usefull for user add on library
   */
  void openlib(ptrOpenlib); /*to add more stuff to the current lua state*/

  /**
   * adding more arguments on lua stack
   * this one is called each time a lua call has to be done
   * consequently to ensure a proper call
   * this method returns the number of args it has pushed on stack
   */
  virtual int addMoreArg(void);


  /**
   * initialitize _LS and open default library
   * @see constructor and _checkRecompile
   */
  virtual void init(void);

  /**
   * check if file has been edited since last
   * compilation.
   * close _LS --> destroy share object
   * use init() to open library
   * @return : true means that a compiling has been done
   * @see init
   */
  bool checkRecompile(void);

  /**
   * print readable message
   * when error(s) have occured during lua execution
   */
  void checkError(int luaCodeErr);


  /**< return the activity _act */
  inline ArRef<Activity> getActivity(void){
    return _act;
  }



protected:

  /**
   * pointer to lua_State*
   */
  void* _LS;


  /**
   * the file parse and byte compile by lua
   * this one must have a 'run' function with
   * a minimum of two arguments
   * to be call from the activity of this object
   */
  StlString _fileName;

  /**
   * activity in charge of parsing file if needed
   * and push sharing elements on lua stack
   * and execute the call
   */
  ArRef<Activity> _act;

  /**
   * last modification of the file _fileName
   */
  time_t _last_mod;

  /**
   * added library when openlib has been called
   * used when _checkRecompile close and open _LS
   */
  set<ptrOpenlib> _lib;


  /**
   * boolean needed to check if lua is in error mode:
   * so we can show the error message only once
   */
  bool _isOnError;

};


}//namespace
#endif
