// LI2/ENIB - AReVi-Contrib - AVI recorder
//----------------------------------------------------------------------------
// file        : AReVi/Contrib/aviRecorder.h
// author(s)   : T. Jourdan, B. Muguet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVI_CONTRIB_AVIRECORDER_H
#define AREVI_CONTRIB_AVIRECORDER_H 1

#include "AReVi/Utils/abstractStream.h"
#include "AReVi/Lib3D/urlSound3D.h"

namespace AReVi {

extern "C" bool AudioVideoEncoder_init(const StlVector<StlString> *);
extern "C" void AudioVideoEncoder_destroy(void);

class FileOStream;

class MemoryBlock;
class SoundEncoder;

struct AviRecorderData;


//----------------------------------------------------------------------------
// Class AviRecorder
//----------------------------------------------------------------------------

class AviRecorder : public ArObject
{

 public:

struct AudioParameters
{
  unsigned int bitRate;
  unsigned int sampleRate;
  unsigned int nbChannels;
};

//-------- Type information --------------------------------------------------
public:

  AR_CLASS(AviRecorder);

//-------- Construction / Destruction ----------------------------------------

  AR_CONSTRUCTOR_5(AviRecorder,
                   const StlString &, fileName,
                   double, frameRate,
                   unsigned int, bitRate,
                   unsigned int, frameWidth,
                   unsigned int, frameHeight);

  AR_UNSTUBBED
  AR_CONSTRUCTOR_7(AviRecorder,
		   const StlString &, fileName,
		   double, frameRate,
		   unsigned int, bitRate,
		   unsigned int, frameWidth,
		   unsigned int, frameHeight,
		   bool,useAudio,
		   AudioParameters,audioParameters);

  AR_CONSTRUCTOR_6(AviRecorder,
                   ArRef<AbstractOStream> , stream,
                   const StlString &, format,
                   double, frameRate,
                   unsigned int, bitRate,
                   unsigned int, frameWidth,
                   unsigned int, frameHeight);

//-------- Recording management ----------------------------------------------

  virtual
    bool // success
    addRGBFrame(unsigned int frameWidth,
		unsigned int frameHeight,
		ArConstRef<MemoryBlock> frameData);

  virtual
    bool // success
    addDataSound(ArRef<MemoryBlock> dataSound);


  virtual
    double // record duration
    getDuration(void) const;

  virtual
    ArConstRef<AbstractOStream>  // output stream if any
    getStream(void) const;

  virtual
    const StlString &   // output file name if any
    getFileName(void) const;

  virtual
    const StlString &   // output format
    getFormat(void) const;

  virtual
    double // frame rate
    getFrameRate(void) const;

  virtual
    unsigned int // video bit rate
    getVideoBitRate(void) const;

  virtual
    unsigned int // audio bit rate
    getAudioBitRate(void) const;

  virtual
    void
    getFrameSize(unsigned int & frameWidthOut,
		 unsigned int & frameHeightOut) const;

//-------- Public attributes ---------------------------------------------

//-------- Codecs properties --------------------------------------------------

  AR_UNSTUBBED
  StlList<StlString> getListOfAvailableCodecs();

//-------- Forbidden operations ----------------------------------------------
protected:

  //using plugin
  friend bool AudioVideoEncoder_init(const StlVector<StlString> *);
  friend void AudioVideoEncoder_destroy(void);

  friend void AviRecorder_start(AviRecorder *);
  friend void AviRecorder_stop(AviRecorder *);
  friend bool AviRecorder_addVideoFrame(AviRecorder *,
					unsigned int,
					unsigned int,
					ArConstRef<MemoryBlock>);
  friend bool AviRecorder_addAudioFrame(AviRecorder *,
					ArConstRef<MemoryBlock>);
  friend int AviRecorder_getAudioFrameSize(AviRecorder * avi);

  //-------- Member attributes -------------------------------------------------
 protected:

  ArRef<AbstractOStream> _stream;
  StlString _fileName;
  StlString _format;

  double _frameRate;
  unsigned int _videoBitRate;
  unsigned int _frameWidth;
  unsigned int _frameHeight;
  unsigned int _dataSize;
  unsigned int _nbFrames;
  AviRecorderData * _aviData;

  // time scheduler to maintain AMAP the frame rate
  double _lastTime;
  double _startTime;

  // audio parameters
  unsigned int _sampleRate;
  unsigned int _audioBitRate;
  unsigned int _channels;

  ArRef<MemoryBlock> _soundBuffer;
  int _soundBufferSize;

    //selected modes
  bool _useAudio;

  AudioParameters _audioParameters;

  //-------- Static attributes -------------------------------------------------
protected:

  static bool _initSuccess;
  static void (* _startFnct)(AviRecorder *);
  static void (* _stopFnct)(AviRecorder *);

  static bool (* _addVideoFrameFnct)(AviRecorder *,
				     unsigned int,
				     unsigned int,
				     ArConstRef<MemoryBlock>);
  static bool (* _addAudioFrameFnct)(AviRecorder *,
				     ArConstRef<MemoryBlock>);
  static int (* _getAudioFrameSizeFnct)(AviRecorder *);

  static StlList<StlString> (* _getListOfAvailableCodecsFnct)();

};

} // namespace AReVi

#endif // AREVI_CONTRIB_AVIRECORDER_H

//----------------------------------------------------------------------------
