// LI2/ENIB - AReVi-Contrib - CgFilter ???
//----------------------------------------------------------------------------
// file        : AReVi/Contrib/cgFilter.h
// author(s)   : D. Herviou
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVI_CONTRIB_CGFILTER_H
#define AREVI_CONTRIB_CGFILTER_H 1

#include "AReVi/Contrib/cgContext.h"

namespace AReVi {

class Scene3D;
class Renderer3D;
class Viewer3D;
class RenderTexture;
class TextureRenderer3D;
class OSDSprite;

//-------------------------------------------------------------
// Class CgFilter : construction d'un filtre Cg
//-------------------------------------------------------------
class CgFilter : public ArObject{
public :

  AR_CLASS(CgFilter)
  AR_CONSTRUCTOR_5(CgFilter,
		   const StlString&, CGPName,   //CG prog name
		   ArRef<CgContext>, CGContext, //Cg Context
		   CgContext::Profile, CGProfile, //Cg Profile
		   int, width,
		   int, height)

  //----Enable/disable Filter------------------------------------

  virtual void generateFilter( void );

  virtual void activateFilter( void );

  virtual void deactivateFilter( void ) ; // disable filter after rendering

  //----Set Cg Program Attributes-----(*MORE METHOD WILL BE GOOD*)

  virtual void setParameter(const StlString& name, float  value);

  virtual void setParameter(const StlString& name, double  value);

  virtual void setParameter4x4(const StlString& name,CgContext::StateMatrixType matrix);

  virtual void setParameter(const StlString& name, unsigned int TextureID);

  //----Filter name----------------------------------------------

  inline  const StlString& getName(void) {return _CGPName;} ;

  virtual void addProgPath(const StlString& path);

  //----Private attributes---------------------------------------
private :
  StlString _CGPName;
  CgContext::Program _CGProg;
  ArRef<CgContext> _CGContext;
  CgContext::Profile _CGProfile;
  StlList<StlString> _pathProg;
  int _width, _height;
};



//-------------------------------------------------------------
//  Class CgPipeline : rendering pipeline for applying filter
//-------------------------------------------------------------
class CgPipeline : public ArObject{
public:
  AR_CLASS(CgPipeline)
  AR_CONSTRUCTOR_1(CgPipeline, int, nbSlot)


  //----CgPipeline standard operation----------------------------

  virtual void addFilter( ArRef<CgFilter> filter );

  virtual void removeFilter( const StlString& filterName );

  virtual void activateFilter( const StlString& filterName);

  virtual void deactivateFilter( const StlString& filterName );

  virtual void initInternalRenderers( ArRef<Viewer3D> viewer );

  virtual void setRenderSize(unsigned int w, unsigned int h);

  virtual void setFarDistance(float far);

  virtual void setNearDistance(float near);

  virtual void render(int output);

  virtual void render(int input, int output);

  virtual ArRef<RenderTexture> getTextureNb(int nb);

  virtual ArRef<CgFilter> getFilter(const StlString& name);

  inline ArRef<CgContext> getCgContext(void){return _context;};

  inline ArRef<Viewer3D> getViewer(void){return _viewer3D;};

  //----Forbidden operations-------------------------------------

private :

  virtual void _setTexture(int nb, ArRef<RenderTexture> texture);

  virtual void _beforeShowResult(ArRef<RenderTexture> ,
                                 ArRef<Renderer3D> viewer);

  virtual void _afterShowResult(ArRef<Renderer3D> viewer);

  virtual void _captureBuffer(int TextureOutputBuffer);

  virtual ArRef<TextureRenderer3D> _getInternalRenderer(int nb);


  //----Private attributes---------------------------------------

private:

  StlList<pair<StlString , ArRef<CgFilter> > > _filters; //catalog of filters
  StlList<pair<int , ArRef<RenderTexture> > > _textures; //textures Buffers
  StlList<pair<int , ArRef<TextureRenderer3D> > > _internalRenderers;//catalog of offScreen Renderers
  ArRef<Viewer3D> _viewer3D;
  ArRef<CgContext> _context;
  ArRef<OSDSprite> _osdSprite;
  ArRef<Scene3D> _sceneTempo;
  ArRef<Scene3D> _sceneBackUp;
};

} // namespace AReVi

#endif // AREVI_CONTRIB_CGFILTER_H

//----------------------------------------------------------------------------
