// LI2/ENIB - AReVi-Contrib - Free form deformation
//----------------------------------------------------------------------------
// file        : AReVi/Contrib/ffdInteractor.h
// author(s)   : T. Jourdan
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVI_CONTRIB_FFDINTERACTOR_H
#define AREVI_CONTRIB_FFDINTERACTOR_H 1

#include "AReVi/arObject.h"

#include "AReVi/Shapes/shapePart3D.h"

namespace AReVi {

class BoundingBox3D;
class Mesh3D;
class Shape3D;

using namespace AReVi;

//----------------------------------------------------------------------------
// Class FfdInteractor
//----------------------------------------------------------------------------

class FfdInteractor : public ArObject
{
//-------- Type information --------------------------------------------------
public :

  AR_CLASS(FfdInteractor)

//-------- Construction / Destruction ----------------------------------------

  AR_CONSTRUCTOR(FfdInteractor)

//-------- Changes -----------------------------------------------------------

  virtual
  void
  applyChanges(void);

  AR_CALLBACK(FfdInteractor, Rebuild, _rebuildCB,
              /* no field */)

//-------- Deformation Box ---------------------------------------------------

  virtual
  void // Deformation box expressed in shape location !
  writeDeformationBox(ArRef<BoundingBox3D> dbox);

  virtual
  void
  readDeformationBox(ArRef<BoundingBox3D> dboxOut);

// ------- Control points ----------------------------------------------------

  virtual
  void
  setDivision(unsigned int nbx,
	      unsigned int nby,
	      unsigned int nbz);

  virtual
  void
  getDivision(unsigned int& nbx,
	      unsigned int& nby,
	      unsigned int& nbz);

  AR_UNSTUBBED virtual
  Util3D::Dbl3&
  accessControlPoint(int idx,
		     int idy,
		     int idz);

//-------- Tree structure ----------------------------------------------------

  virtual
  void
  addSubFfdInteractor(ArRef<FfdInteractor> ffd);

  virtual
  bool
  removeSubFfdInteractor(ArRef<FfdInteractor> ffd);

  virtual
  unsigned int
  getNbSubFfdInteractors(void) const;

  virtual
  ArConstRef<FfdInteractor>
  getSubFfdInteractor(unsigned int index) const;

  virtual
  ArRef<FfdInteractor>
  accessSubFfdInteractor(unsigned int index);

  virtual
  ArConstRef<FfdInteractor>
  getParentFfdInteractor(void) const;

  virtual
  ArRef<FfdInteractor>
  accessParentFfdInteractor(void);

  AR_CALLBACK(FfdInteractor, StructureChange, _structureChangeCB,
              /* no field */)

//-------- Mesh management ---------------------------------------------------

  virtual
  void
  setShape(ArConstRef<Shape3D> shape);

  virtual
  ArConstRef<Shape3D>
  getShape(void) const;

  virtual
  bool
  addMesh(ArRef<Mesh3D> mesh);

  virtual
  bool
  removeMesh(ArRef<Mesh3D> mesh);

  virtual
  unsigned int
  getNbMeshes(void) const;

  virtual
  ArConstRef<Mesh3D>
  getMesh(unsigned int index) const;

  virtual
  ArRef<Mesh3D>
  accessMesh(unsigned int index);

//-------- Serialization operations ------------------------------------------

  AR_UNSTUBBED virtual
  void
  extractDependencies(SerializationDependencies & dependenciesOut,
                      bool recursive=true) const;

  AR_UNSTUBBED virtual
  void
  serialize(SerializationDependencies & dependenciesOut,
            ArRef<AbstractOStream> output) const;

  AR_UNSTUBBED virtual
  bool // success
  unserialize(const SerializationDependencies & dependencies,
              ArRef<AbstractIStream> input);

protected :

//------- Callbacks ---------------------------------------------------------

  virtual
  void
  _onMeshTransformationCB(const ShapePart3D::TransformationEvent& evt);

  virtual
  void
  _onMeshStructureChangeCB(const ShapePart3D::StructureChangeEvent& evt);

//------- Utilities functions -----------------------------------------------

  virtual
  void
  _onStructureChange(void);

  virtual
  void
  _updateShape(ArConstRef<Shape3D> parent);

  virtual
  void
  _updateDeformationBox(void);

  virtual
  void
  _computeCoordinates(unsigned int index);

  struct MeshEntry {
    ArRef<Mesh3D> mesh;
    StlVector<Util3D::Dbl3>* verticesPtr;
    StlVector<Util3D::Dbl3>* normalsPtr;
    // Default vertices & normals
    StlVector<Util3D::Dbl3> vertices;
    StlVector<Util3D::Dbl3> normals;
    // Default vertices & normals in shape location
    StlVector<Util3D::Dbl3> transformedVertices;
    StlVector<Util3D::Dbl3> transformedNormals;
  };

  virtual
  void
  _rebuild(StlVector<MeshEntry*>& meshes);

  virtual
  void
  _onRebuild(void);

  virtual
  void
  _invertLocation(void);

protected :
  // Parts
  ArConstRef<Shape3D> _shape;
  StlVector<MeshEntry> _meshes;

  // Tree structure
  ArPtr<FfdInteractor> _parentFfd;
  StlVector<ArRef<FfdInteractor> > _subFfd;
  CallbackManager<FfdInteractor, RebuildEvent> _rebuildCB;
  CallbackManager<FfdInteractor, StructureChangeEvent> _structureChangeCB;

  // Deformation box
  Util3D::BoundingBox _dbox;
  int _l, _m, _n;
  StlVector<StlVector<StlVector<Util3D::Dbl3> > > _ctrlPoints;
  double* _combinL;
  double* _combinM;
  double* _combinN;
};

} // namespace AReVi

#endif // AREVI_CONTRIB_FFDINTERACTOR_H
