// LI2/ENIB - AReVi-Contrib - Nurbs shapes
//----------------------------------------------------------------------------
// file        : AReVi/Contrib/nurbsCurve3D.h
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVI_CONTRIB_NURBSCURVE3D_H
#define AREVI_CONTRIB_NURBSCURVE3D_H 1

#include "AReVi/Shapes/shapePart3D.h"
#include "AReVi/Contrib/arNurbs.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class NurbsCurve3D
//----------------------------------------------------------------------------

class NurbsCurve3D : public ShapePart3D
{
//-------- Type information --------------------------------------------------
public:

  AR_CLASS(NurbsCurve3D)

//-------- Construction / Destruction ----------------------------------------

  AR_CONSTRUCTOR(NurbsCurve3D)

//-------- Changes -----------------------------------------------------------

  virtual
  bool // success
  applyChanges(bool checkConsistency);

//-------- Appearance --------------------------------------------------------

  virtual
  bool // success
  setThickness(unsigned int thickness);

  virtual
  unsigned int // lines thickness
  getThickness(void) const;

  virtual
  bool // success
  setPrecision(unsigned int precision);

  virtual
  unsigned int // nurbs interpolation precision
  getPrecision(void) const;

//-------- Vertices ----------------------------------------------------------

  AR_UNSTUBBED virtual
  const ArNurbs::Curve3d & // nurbs curve
  getCurve(void) const;

  AR_UNSTUBBED virtual
  ArNurbs::Curve3d & // nurbs curve
  accessCurve(void);

//-------- Colors ------------------------------------------------------------

  virtual
  void
  setColor(double r,
           double g,
           double b);

  virtual
  void
  getColor(double & rOut,
           double & gOut,
           double & bOut) const;

//-------- Serialization operations ------------------------------------------

  AR_UNSTUBBED virtual
  void
  serialize(SerializationDependencies & dependenciesOut,
            ArRef<AbstractOStream> output) const;

  AR_UNSTUBBED virtual
  bool // success
  unserialize(const SerializationDependencies & dependencies,
              ArRef<AbstractIStream> input);

//-------- Forbidden operations ----------------------------------------------
protected:

  virtual
  void
  _setAppearance(void) const;

  virtual
  void
  _resetAppearance(void) const;

  virtual
  void
  _drawGeometry(void) const;

  virtual
  void
  _rebuildIfNeeded(void);

//-------- Member attributes -------------------------------------------------
protected:

  unsigned int _thickness;
  unsigned int _precision;
  ArNurbs::Curve3d _curve;
  Util3D::Dbl3 _color;
  StlVector<Util3D::Dbl3> _vertices;
};

} // namespace AReVi

#endif // AREVI_CONTRIB_NURBSCURVE3D_H

//----------------------------------------------------------------------------
