// LI2/ENIB - AReVi-Contrib - Nurbs shapes
//----------------------------------------------------------------------------
// file        : AReVi/Contrib/nurbsSurface3D.h
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVI_CONTRIB_NURBSSURFACE3D_H
#define AREVI_CONTRIB_NURBSSURFACE3D_H 1

#include "AReVi/Shapes/surface3D.h"
#include "AReVi/Contrib/arNurbs.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class NurbsSurface3D
//----------------------------------------------------------------------------

class NurbsSurface3D : public Surface3D
{
//-------- Type information --------------------------------------------------
public:

  AR_CLASS(NurbsSurface3D)

//-------- Construction / Destruction ----------------------------------------

  AR_CONSTRUCTOR(NurbsSurface3D)

//-------- Changes -----------------------------------------------------------

  virtual
  bool // success
  applyChanges(bool checkConsistency);

//-------- Appearance --------------------------------------------------------

  virtual
  bool // success
  setPrecision(unsigned int precisionU,
               unsigned int precisionV);

  virtual
  void
  getPrecision(unsigned int & precisionUOut,
               unsigned int & precisionVOut) const;

//-------- Vertices ----------------------------------------------------------

  AR_UNSTUBBED virtual
  const ArNurbs::Surface3d & // nurbs surface
  getSurface(void) const;

  AR_UNSTUBBED virtual
  ArNurbs::Surface3d & // nurbs surface
  accessSurface(void);

//-------- Serialization operations ------------------------------------------

  AR_UNSTUBBED virtual
  void
  serialize(SerializationDependencies & dependenciesOut,
            ArRef<AbstractOStream> output) const;

  AR_UNSTUBBED virtual
  bool // success
  unserialize(const SerializationDependencies & dependencies,
              ArRef<AbstractIStream> input);

//-------- Forbidden operations ----------------------------------------------
protected:

  virtual
  void
  _rebuildIfNeeded(void);

//-------- Member attributes -------------------------------------------------
protected:

  unsigned int _precisionU;
  unsigned int _precisionV;
  ArNurbs::Surface3d _surface;
};

} // namespace AReVi

#endif // AREVI_CONTRIB_NURBSSURFACE3D_H

//----------------------------------------------------------------------------
