// LI2/ENIB - AReVi-Contrib - Path ???
//----------------------------------------------------------------------------
// file        : AReVi/Contrib/path.h
// author(s)   : T. Jourdan
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVI_CONTRIB_PATH_H
#define AREVI_CONTRIB_PATH_H 1

#include "AReVi/arObject.h"
#include "AReVi/Lib3D/base3D.h"
#include "AReVi/Contrib/arNurbs.h"

namespace AReVi {

class Path : public ArObject {
public :
  AR_CLASS(Path)
  AR_CONSTRUCTOR(Path)

  virtual
  void
  applyChanges(void);

  //-------- Name -------------------------------------------------

  virtual
  void
  setName(const StlString& name);

  virtual
  const StlString&
  getName(void) const;

  //-------- Control Points ---------------------------------------

  virtual
  const StlVector<ArRef<Base3D> >&
  getWayPoints(void) const;

  virtual
  StlVector<ArRef<Base3D> >&
  accessWayPoints(void);

  virtual
  const StlVector<bool>&
  getAutoOrientations(void) const;

  virtual
  StlVector<bool>&
  accessAutoOrientations(void);

  AR_UNSTUBBED virtual
  void
  setStartDerivate(const Vector3d& d);

  AR_UNSTUBBED virtual
  Vector3d
  getStartDerivate(void) const;

  AR_UNSTUBBED virtual
  void
  setEndDerivate(const Vector3d& d);

  AR_UNSTUBBED virtual
  Vector3d
  getEndDerivate(void) const;

  //-------- Path parameters --------------------------------------

  virtual
  double
  getLength(void) const;

  virtual
  double
  getArcLength(double p, bool fromEnd = false);

  virtual
  double
  getParameter(double length, bool fromEnd = false);

  virtual
  double
  getPrecision(void) const;

  virtual
  void
  setPrecision(double p);

  //-------- Interpolation ----------------------------------------

  virtual
  ArConstRef<Base3D> // base to attach stuffs
  getMovingBase(void) const;

  virtual
  ArRef<Base3D>
  accessMovingBase(void);

  virtual
  ArConstRef<Base3D> // base to attach stuffs
  evaluate(double p);

  //-------- Nurbs curve ------------------------------------------

  AR_UNSTUBBED virtual
  ArNurbs::Curve3d& // nurbs curve
  getCurve(void);

  //-------- Serialization operations -----------------------------

  AR_UNSTUBBED virtual
  void
  serialize(SerializationDependencies& dependenciesOut,
            ArRef<AbstractOStream> output) const;

  AR_UNSTUBBED virtual
  bool // success
  unserialize(const SerializationDependencies& dependencies,
              ArRef<AbstractIStream> input);

protected :
  virtual
  void
  _computeParameters(void);

  virtual
  void
  _computeKnotVector(void);

  virtual
  void
  _solveTridiagonal(StlVector<Vector3d>& ctrlPoints);

  virtual
  pair<Vector3d, Quaterniond>
  _evaluate(double p);

  virtual
  void
  _precompLength(void);

  virtual
  double
  _arcLength(double a, double b, double step);

  virtual
  double
  _fastArcLength(double a, double b, double step);

  virtual
  double
  _getP(double s);

  virtual
  double
  _speed(double s);

protected :
  StlString _name;
  double _precision;
  ArRef<Base3D> _movingBase;
  Vector3d _sd;
  Vector3d _ed;
  StlVector<ArRef<Base3D> > _wayPoints;
  StlVector<bool> _autoOrientations;
  // internal
  StlVector<Vector3d>    _points;
  StlVector<Quaterniond> _orientations;
  StlVector<double>      _parameters;
  StlVector<double>      _pLength;
  ArNurbs::Curve3d _curve;
};

} // namespace AReVi

#endif // AREVI_CONTRIB_PATH_H

//----------------------------------------------------------------------------
