// LI2/ENIB - AReVi-Contrib - Sound encoder
//----------------------------------------------------------------------------
// file        : AReVi/Contrib/soundEncoder.h
// author(s)   : B. Muguet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVI_CONTRIB_SOUNDENCODER_H
#define AREVI_CONTRIB_SOUNDENCODER_H 1

namespace AReVi {

extern "C" bool AudioVideoEncoder_init(const StlVector<StlString> *);
extern "C" void AudioVideoEncoder_destroy(void);

class MemoryBlock;
class Activity;
class FileOStream;

struct SoundEncoderData;


//----------------------------------------------------------------------------
// CLASS       : SoundEncoder
//----------------------------------------------------------------------------
class SoundEncoder : public ArObject
{

//-------- Type information --------------------------------------------------
 public:
  AR_CLASS(SoundEncoder)

 //-------- Construction / Destruction ----------------------------------------

    AR_CONSTRUCTOR(SoundEncoder)
		
    AR_CONSTRUCTOR_3(SoundEncoder,
		     int, bitRate,
		     int, sampleRate,
		     int, nbChannels)

    AR_CONSTRUCTOR_4(SoundEncoder,
		     StlString, codecName,
		     int, bitRate,
		     int, sampleRate,
		     int, nbChannels)

//-------- Encoding management ----------------------------------------------

  ArRef<MemoryBlock> encodeDataSound(ArRef<MemoryBlock> dataSoundToEncode);
  ArRef<MemoryBlock> decodeDataSound(ArRef<MemoryBlock> dataSoundToDecode);

  //Encoding and recording in a file
  bool openStream(StlString filename);
  bool addSoundFrame(ArRef<MemoryBlock> dataSoundToEncode);
  bool closeStream();

//-------- Codecs properties --------------------------------------------------

  AR_UNSTUBBED StlList<StlString> getListOfAvailableCodecs();

  StlString getCodecName();
  void setCodecName(StlString codecName);

//-------- Sound properties --------------------------------------------------

  int getBitRate();
  int getSampleRate();
  int getNbChannels();

  void setBitRate(int bitRate);
  void setSampleRate(int sampleRate);
  void setNbChannels(int nbChannels);

  void setEncoderParameters(StlString codecName, int bitRate, int sampleRate, int nbChannels);

//-------- Forbidden operations ----------------------------------------------
protected:

  friend bool AudioVideoEncoder_init(const StlVector<StlString> *);
  friend void AudioVideoEncoder_destroy(void);
  friend void SoundEncoder_start(SoundEncoder *, const char*, int, int, int);
  friend void SoundEncoder_stop(SoundEncoder *);
  friend int SoundEncoder_encodeSoundSample(SoundEncoder *,unsigned char *, short*);
  friend int SoundEncoder_decodeSoundSample(SoundEncoder *,unsigned int *, int*, unsigned char *, int);
  friend int SoundEncoder_getFrameSize(SoundEncoder *);

//-------- Member attributes -------------------------------------------------

 protected:

  StlString _codecName;
  int _bitRate;
  int _sampleRate;
  int _nbChannels;

  ArRef<MemoryBlock> _soundBuffer;
  int _soundBufferSize;

  SoundEncoderData * _soundEncoderData;

  ArRef<FileOStream> _fileStream;

//-------- Static attributes -------------------------------------------------
 protected:

  static bool _initSuccess;
  static void (* _startFnct)(SoundEncoder *, const char*, int, int, int);
  static void (* _stopFnct)(SoundEncoder *);

  static int (* _encodeSoundSampleFnct)(SoundEncoder *,
					unsigned char *,short*);

  static int (* _decodeSoundSampleFnct)(SoundEncoder *,
					unsigned int *,int*,
					unsigned char *, int);

  static int (* _getFrameSizeFnct)(SoundEncoder *);

  static int (* _getMaxAudioFrameSizeFnct)();

  static StlList<StlString> (* _getListOfAvailableCodecsFnct)();

};

} // namespace AReVi

#endif
//----------------------------------------------------------------------------
