// LI2/ENIB - AReVi-Contrib - Sound recorder
//----------------------------------------------------------------------------
// file        : AReVi/Contrib/soundRecorder.h
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVI_CONTRIB_SOUNDRECORDER_H
#define AREVI_CONTRIB_SOUNDRECORDER_H 1

#include "AReVi/arObject.h"
#include "AReVi/scheduler.h"
#include "AReVi/Utils/utilSound.h"

namespace AReVi {

extern "C" bool SoundPlayer_init(const StlVector<StlString> *);
extern "C" void SoundPlayer_destroy(void);

class MemoryBlock;

//----------------------------------------------------------------------------
// Class SoundRecorder
//----------------------------------------------------------------------------

class SoundRecorder : public ArObject
{
//-------- Type information --------------------------------------------------

public:

  AR_CLASS(SoundRecorder)

  AR_CONSTRUCTOR_2(SoundRecorder,
                   UtilSound::SoundFormat, format,
                   unsigned int, rate)

  AR_CONSTRUCTOR_3(SoundRecorder,
                   UtilSound::SoundFormat, format,
                   unsigned int, rate,
                   bool, recordOutput)

//-------- Sound properties --------------------------------------------------

  virtual
  UtilSound::SoundFormat // sound format
  getFormat(void) const;

  virtual
  unsigned int // sound rate (commons rate are 11025, 22050, 44100)
  getRate(void) const;

//-------- Playback control --------------------------------------------------

  virtual
  bool
  start(bool backupData); // Recorder backup buffers

  virtual
  bool
  stop(void);

  virtual
  bool // sound is recording
  isRecording(void) const;

//-------- Data management  --------------------------------------------------

  // No-empty only if start was called with backupData=true!
  AR_UNSTUBBED virtual
  const StlList<ArRef<MemoryBlock> > &
  getBuffers(void) const;

  // No-empty only if start was called with backupData=true!
  AR_UNSTUBBED virtual
  StlList<ArRef<MemoryBlock> > &
  accessBuffers(void);

  // Return all recorded but not yet read buffers in queue list
  AR_UNSTUBBED virtual
  bool // success
  flushPendingBuffers(StlList<ArRef<MemoryBlock> > &buffersOut);

//-------- Event management --------------------------------------------------

  AR_CALLBACK(SoundRecorder,BufferReady,_bufferReadyCB,
              ArRef<MemoryBlock> buffer;)

  AR_CALLBACK(SoundRecorder,Control,_controlCB,
              /* no field */) // start, stop

//-------- Forbidden operations ----------------------------------------------
protected:

  friend bool SoundPlayer_init(const StlVector<StlString> *);
  friend void SoundPlayer_destroy(void);

  virtual void
  _onPolling(const Scheduler::PollingEvent & evt);

  virtual
  void
  _onControl(void);

//-------- Member attributes -------------------------------------------------
protected:

  int _deviceID;
  void *_pluginData;

  bool _recording;
  bool _backupData;

  UtilSound::SoundFormat _soundFormat;
  unsigned _sampleRate;

  StlList<ArRef<MemoryBlock> > _buffers;

  CallbackManager<SoundRecorder,BufferReadyEvent> _bufferReadyCB;
  CallbackManager<SoundRecorder,ControlEvent> _controlCB;

//-------- Static attributes -------------------------------------------------
protected:
  static bool _initSuccess;

  static void *(* _openSoundInputFnct)(SoundRecorder *, const StlString &, int &);
  static void (* _closeSoundInputFnct)(void *);
  static bool (* _setRecordFnct)(void *, bool );
  static bool (* _popPendingBufferFnct)(void *, ArRef<MemoryBlock> );
  static unsigned (* _getNbPendingBuffersFnct)(const void *);
};

} // namespace AReVi

#endif // AREVI_CONTRIB_SOUNDRECORDER_H

//----------------------------------------------------------------------------
