// LI2/ENIB - AReVi - 3D services
//----------------------------------------------------------------------------
// file        : AReVi/Lib3D/abstractInteractor.h
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVI_LIB3D_ABSTRACTINTERACTOR_H
#define AREVI_LIB3D_ABSTRACTINTERACTOR_H 1

#include "AReVi/Lib3D/renderer3D.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class AbstractInteractor (abstract)
//----------------------------------------------------------------------------

class AbstractInteractor : public ArObject
{
//-------- Type information --------------------------------------------------
public:

  AR_CLASS(AbstractInteractor)

//-------- Renderer management -----------------------------------------------

  virtual
  void
  setRenderer(ArRef<Renderer3D> renderer);

  virtual
  ArConstRef<Renderer3D> // interacted renderer
  getRenderer(void) const;

  virtual
  ArRef<Renderer3D> // interacted renderer
  accessRenderer(void);

//-------- Object interaction ------------------------------------------------

  virtual
  void
  addIgnoredObject(ArRef<Object3D> object);

  virtual
  void
  removeIgnoredObject(ArRef<Object3D> object);

  virtual
  bool // object is ignored for interactions
  ignoresObject(ArConstRef<Object3D> object) const;

  virtual
  void
  getIgnoredObjects(StlVector<ArRef<Object3D> > & objectsOut) const;

//-------- OSD interaction ---------------------------------------------------

  virtual
  void
  addIgnoredOSD(ArRef<OSD> osd);

  virtual
  void
  removeIgnoredOSD(ArRef<OSD> osd);

  virtual
  bool // object is ignored for interactions
  ignoresOSD(ArConstRef<OSD> osd) const;

  virtual
  void
  getIgnoredOSD(StlVector<ArRef<OSD> > & osdOut) const;

//------- Forbidden operations -----------------------------------------------
protected:

  AbstractInteractor(ArCW & arCW);

  virtual
  void
  _clearInteraction(void);

  virtual
  void
  _onMouseButton(const Renderer3D::MouseButtonEvent & evt);

  virtual
  void
  _onMouseMotion(const Renderer3D::MouseMotionEvent & evt);

  virtual
  void
  _onKeyboard(const Renderer3D::KeyboardEvent & evt);

  virtual
  void
  _onTimeOut(const Renderer3D::TimeOutEvent & evt);

  virtual
  ArRef<Object3D> // found object
  _findObject(Util3D::Dbl3 & localPointOut,
              ArRef<ShapePart3D> & interPartOut,
              unsigned int & triIndexOut,
              int xMouse,
              int yMouse,
              bool onlyMainScene);

  virtual
  ArRef<OSD> // found osd
  _findOSD(int xMouse,
	   int yMouse);

//-------- Member attributes -------------------------------------------------
protected:

  ArRef<Renderer3D> _renderer;
  StlVector<ArPtr<Object3D> > _ignoredObjects;
  StlVector<ArPtr<OSD> > _ignoredOSD;
};

} // namespace AReVi

#endif // AREVI_LIB3D_ABSTRACTINTERACTOR_H

//----------------------------------------------------------------------------
