// LI2/ENIB - AReVi - 3D services
//----------------------------------------------------------------------------
// file        : AReVi/Lib3D/boundingBox3D.h
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVI_LIB3D_BOUNDINGBOX3D_H
#define AREVI_LIB3D_BOUNDINGBOX3D_H 1

#include "AReVi/arObject.h"

#include "AReVi/Lib3D/util3D_priv.h"

namespace AReVi {

class Transform3D;

//----------------------------------------------------------------------------
// Class BoundingBox3D
//----------------------------------------------------------------------------

class BoundingBox3D : public ArObject
{
//-------- Type information --------------------------------------------------
public:

  AR_CLASS(BoundingBox3D)

//-------- Construction / Destruction ----------------------------------------

  AR_CONSTRUCTOR(BoundingBox3D)

//-------- Geometry ----------------------------------------------------------

  virtual
  void
  invalidate(void);

  virtual
  bool // bounds are significant
  isValid(void) const;

  virtual
  void
  addXYZ(double x,
         double y,
         double z);

  virtual
  void
  addBox(ArConstRef<BoundingBox3D> bbox);

  virtual
  void
  addTransformedBox(ArConstRef<Transform3D> transform,
                    ArConstRef<BoundingBox3D> bbox);

  virtual
  bool // bbox is valid
  getBounds(double & xminOut,
            double & yminOut,
            double & zminOut,
            double & xmaxOut,
            double & ymaxOut,
            double & zmaxOut) const;

  virtual
  bool // bbox is valid
  getSize(double & sizeXOut,
          double & sizeYOut,
          double & sizeZOut) const;

  virtual
  double // size of smallest side
  getMinSize(void) const;

  virtual
  double // size of biggest side
  getMaxSize(void) const;

  virtual
  bool // bbox is valid
  getCenter(double & centerXOut,
            double & centerYOut,
            double & centerZOut) const;

//-------- Data access (internal) --------------------------------------------

  AR_UNSTUBBED inline virtual
  const Util3D::BoundingBox &
  getBoundingBox(void) const;

  AR_UNSTUBBED inline virtual
  Util3D::BoundingBox &
  accessBoundingBox(void);

//-------- Member attributes -------------------------------------------------
protected:

  Util3D::BoundingBox _bbox;
};

//-------- Inline code -------------------------------------------------------

inline
const Util3D::BoundingBox &
BoundingBox3D::getBoundingBox(void) const
{
return(_bbox);
}

inline
Util3D::BoundingBox &
BoundingBox3D::accessBoundingBox(void)
{
return(_bbox);
}

} // namespace AReVi

#endif // AREVI_LIB3D_BOUNDINGBOX3D_H

//----------------------------------------------------------------------------
