// LI2/ENIB - AReVi - 3D services
//----------------------------------------------------------------------------
// file        : AReVi/Lib3D/dataTexture.h
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVI_LIB3D_DATATEXTURE_H
#define AREVI_LIB3D_DATATEXTURE_H 1

#include "AReVi/Lib3D/texture.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class DataTexture
//----------------------------------------------------------------------------

class DataTexture : public Texture
{
//-------- Type information --------------------------------------------------
public:

  AR_CLASS(DataTexture)

//-------- Construction / Destruction ----------------------------------------

  AR_CONSTRUCTOR_6(DataTexture,
                   unsigned int, width,
                   unsigned int, height,
                   unsigned int, components,
                   ArRef<MemoryBlock>, data,
                   bool, checkAlpha,
                   bool, useMipmap)

  AR_CONSTRUCTOR_7(DataTexture,
                   unsigned int, width,
                   unsigned int, height,
                   unsigned int, components,
                   ArRef<MemoryBlock>, data,
                   bool, checkAlpha,
                   bool, useMipmap,
                   bool, useCubemap)

//-------- Texture changes ---------------------------------------------------

  virtual
  ArRef<MemoryBlock> // data filled from bottom to top
  accessData(void);

  virtual
  void
  setBlending(bool needsBlending);

  virtual
  void
  setAlphaTest(bool needsAlphaTest);

  virtual
  bool // success
  reload(bool checkAlpha,
         bool useMipmap);

//-------- Serialization operations ------------------------------------------

  AR_UNSTUBBED virtual
  void
  serialize(SerializationDependencies & dependenciesOut,
            ArRef<AbstractOStream> output) const;

  AR_UNSTUBBED virtual
  bool // success
  unserialize(const SerializationDependencies & dependencies,
              ArRef<AbstractIStream> input);

//-------- Event management --------------------------------------------------

  AR_CALLBACK(DataTexture,Reload,_reloadCB,
              /* no field */)

  AR_CALLBACK(DataTexture,Setup,_setupCB,
              /* no field */) // on set Alpha/Blend

//-------- Forbidden operations ----------------------------------------------
protected:

  DataTexture(ArCW & arCW); // only for serialisation

  virtual
  bool // success
  _loadDataTexture(unsigned int width,
                   unsigned int height,
                   unsigned int components,
                   ArRef<MemoryBlock> data,
                   bool checkAlpha,
                   bool useMipmap,
                   bool useCubemap);

  virtual
  void
  _onSetup(void);

//-------- Member attributes -------------------------------------------------
protected:

  CallbackManager<DataTexture,ReloadEvent> _reloadCB;
  CallbackManager<DataTexture,SetupEvent> _setupCB;

//-------- Static attributes -------------------------------------------------
public:

  static
  unsigned int
  getQuality(void); // FIXME: really here ?

  static
  void
  setQuality(unsigned int quality); // FIXME: really here ?

protected:

  static unsigned int _quality; // [0-100] => 100 means no compression
};

} // namespace AReVi

#endif // AREVI_LIB3D_DATATEXTURE_H

//----------------------------------------------------------------------------
