// LI2/ENIB - AReVi - 3D services
//----------------------------------------------------------------------------
// file        : AReVi/Lib3D/gl_priv.h
// author(s)   : F. Harrouet, T. Jourdan, E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVI_LIB3D_GL_PRIV_H
#define AREVI_LIB3D_GL_PRIV_H 1

#include "AReVi/Utils/stlString_priv.h"

#include <GL/gl.h>
#include <GL/glu.h>

#ifndef __sgi
# include <GL/glext.h>
#endif // not __sgi

#ifndef APIENTRY
# define APIENTRY
#endif

namespace AReVi {

//----------------------------------------------------------------------------
// Class GL (abstract)
//----------------------------------------------------------------------------

class GL
{
//-------- OpenGL initialization ---------------------------------------------
public :

  static
  void
  init(void);

  static
  bool // success
  hasExtension(const char * extension,
               const char * extensionList);

  template <typename Fnct> inline static
  void
  loadFunction(Fnct & fnct,
               const char * procName,
               const char * extSuffix);

//-------- Properties --------------------------------------------------------

  inline static
  unsigned int // max number of OpenGL lights
  getNbLights(void);

  inline static
  unsigned int // max texture size
  getMaxTextureSize(void);

//-------- Utilities ---------------------------------------------------------

  inline static
  const char * // last GL error message if any
  lastError(void);

  inline static
  void
  selectTexture(unsigned int textureID,
                bool repeatS,
                bool repeatT,
                bool modulate);

  inline static
  void
  selectCubeTexture(unsigned int textureID,
                    bool repeatS,
                    bool repeatT,
                    bool modulate,
                    bool useCubeMapReflection);

//-------- GL_EXT_separate_specular_color ------------------------------------

#ifndef GL_LIGHT_MODEL_COLOR_CONTROL_EXT
# define GL_LIGHT_MODEL_COLOR_CONTROL_EXT 0x81F8
#endif

#ifndef GL_SEPARATE_SPECULAR_COLOR_EXT
# define GL_SEPARATE_SPECULAR_COLOR_EXT 0x81FA
#endif

  inline static
  bool // separate specular color is supported
  hasSeparateSpecularColor(void);

//-------- GL_ARB_texture_compression ----------------------------------------

#ifndef GL_COMPRESSED_LUMINANCE_ARB
# define GL_COMPRESSED_LUMINANCE_ARB 0x84EA
#endif

#ifndef GL_COMPRESSED_LUMINANCE_ALPHA_ARB
# define GL_COMPRESSED_LUMINANCE_ALPHA_ARB 0x84EB
#endif

#ifndef GL_COMPRESSED_RGB_ARB
# define GL_COMPRESSED_RGB_ARB 0x84ED
#endif

#ifndef GL_COMPRESSED_RGBA_ARB
# define GL_COMPRESSED_RGBA_ARB 0x84EE
#endif

  inline static
  bool // texture compression is supported
  hasTextureCompression(void);

//-------- GL_EXT_texture_cube_map -------------------------------------------

  inline static
  bool // texture cube map is supported
  hasTextureCubeMap(void);

#ifndef GL_NORMAL_MAP_EXT
# define GL_NORMAL_MAP_EXT 0x8511
#endif

#ifndef GL_REFLECTION_MAP_EXT
# define GL_REFLECTION_MAP_EXT 0x8512
#endif

#ifndef GL_TEXTURE_CUBE_MAP_EXT
# define GL_TEXTURE_CUBE_MAP_EXT 0x8513
#endif

#ifndef GL_TEXTURE_BINDING_CUBE_MAP_EXT
# define GL_TEXTURE_BINDING_CUBE_MAP_EXT 0x8514
#endif

#ifndef GL_TEXTURE_CUBE_MAP_POSITIVE_X_EXT
# define GL_TEXTURE_CUBE_MAP_POSITIVE_X_EXT 0x8515
#endif

#ifndef GL_TEXTURE_CUBE_MAP_NEGATIVE_X_EXT
# define GL_TEXTURE_CUBE_MAP_NEGATIVE_X_EXT 0x8516
#endif

#ifndef GL_TEXTURE_CUBE_MAP_POSITIVE_Y_EXT
# define GL_TEXTURE_CUBE_MAP_POSITIVE_Y_EXT 0x8517
#endif

#ifndef GL_TEXTURE_CUBE_MAP_NEGATIVE_Y_EXT
# define GL_TEXTURE_CUBE_MAP_NEGATIVE_Y_EXT 0x8518
#endif

#ifndef GL_TEXTURE_CUBE_MAP_POSITIVE_Z_EXT
# define GL_TEXTURE_CUBE_MAP_POSITIVE_Z_EXT 0x8519
#endif

#ifndef GL_TEXTURE_CUBE_MAP_NEGATIVE_Z_EXT
# define GL_TEXTURE_CUBE_MAP_NEGATIVE_Z_EXT 0x851A
#endif

#ifndef GL_PROXY_TEXTURE_CUBE_MAP_EXT
# define GL_PROXY_TEXTURE_CUBE_MAP_EXT 0x851B
#endif

#ifndef GL_MAX_CUBE_MAP_TEXTURE_SIZE_EXT
# define GL_MAX_CUBE_MAP_TEXTURE_SIZE_EXT 0x851C
#endif

//-------- GL_ARB_multitexture -----------------------------------------------

#ifndef GL_MAX_TEXTURE_UNITS_ARB
# define GL_MAX_TEXTURE_UNITS_ARB 0x84E2
#endif

#ifndef GL_TEXTURE0_ARB
# define GL_TEXTURE0_ARB 0x84C0
#endif

#ifndef GL_TEXTURE1_ARB
# define GL_TEXTURE1_ARB 0x84C1
#endif

#ifndef GL_TEXTURE2_ARB
# define GL_TEXTURE2_ARB 0x84C2
#endif

#ifndef GL_TEXTURE3_ARB
# define GL_TEXTURE3_ARB 0x84C3
#endif

  inline static
  unsigned int // max number of OpenGL texture units
  getNbTextureUnits(void);

  static
  void
  (APIENTRY * glActiveTexture)(GLenum texture);

  static
  void
  (APIENTRY * glClientActiveTexture)(GLenum texture);

  static
  void
  (APIENTRY * glMultiTexCoord2dv)(GLenum texture,
                                  const GLdouble * coords);

//-------- GL_SGIS_generate_mipmap -------------------------------------------

#ifndef GL_GENERATE_MIPMAP_SGIS
# define GL_GENERATE_MIPMAP_SGIS 0x8191
#endif

  inline static
  bool // automatic mipmap generation is supported
  hasMipmapGeneration(void);

//-------- GL_EXT_draw_range_elements ----------------------------------------

  static
  void
  (APIENTRY * glDrawRangeElements)(GLenum mode,
                                   GLuint start,
                                   GLuint end,
                                   GLsizei count,
                                   GLenum type,
                                   const GLvoid * indices);

//-------- GL_ARB_shading_language_100 ---------------------------------------

#ifndef GL_SHADING_LANGUAGE_VERSION_ARB
# define GL_SHADING_LANGUAGE_VERSION_ARB 0x8B8C
#endif

  typedef unsigned int SLhandle; // avoid dependency with GLhandleARB
  typedef char SLchar; // avoid dependency with GLcharARB

  inline static
  bool // GLSL is supported
  hasShadingLanguage(void);

//-------- GL_ARB_vertex_shader ----------------------------------------------

#ifndef GL_VERTEX_SHADER_ARB
# define GL_VERTEX_SHADER_ARB 0x8B31
#endif

#ifndef GL_MAX_VERTEX_UNIFORM_COMPONENTS_ARB
# define GL_MAX_VERTEX_UNIFORM_COMPONENTS_ARB 0x8B4A
#endif

  inline static
  unsigned int
  getNbVertexUniformComponents(void);

#ifndef GL_MAX_VARYING_FLOATS_ARB
# define GL_MAX_VARYING_FLOATS_ARB 0x8B4B
#endif

  inline static
  unsigned int
  getNbVaryingFloats(void);

#ifndef GL_MAX_VERTEX_ATTRIBS_ARB
# define GL_MAX_VERTEX_ATTRIBS_ARB 0x8869
#endif

  inline static
  unsigned int
  getNbVertexAttribs(void);

#ifndef GL_MAX_TEXTURE_IMAGE_UNITS_ARB
# define GL_MAX_TEXTURE_IMAGE_UNITS_ARB 0x8872
#endif

  inline static
  unsigned int
  getNbTextureImageUnits(void);

#ifndef GL_MAX_VERTEX_TEXTURE_IMAGE_UNITS_ARB
# define GL_MAX_VERTEX_TEXTURE_IMAGE_UNITS_ARB 0x8B4C
#endif

  inline static
  unsigned int
  getNbVertexTextureImageUnits(void);

#ifndef GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS_ARB
# define GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS_ARB 0x8B4D
#endif

  inline static
  unsigned int
  getNbCombinedTextureImageUnits(void);

#ifndef GL_MAX_TEXTURE_COORDS_ARB
# define GL_MAX_TEXTURE_COORDS_ARB 0x8871
#endif

  inline static
  unsigned int
  getNbTextureCoords(void);

#ifndef GL_VERTEX_PROGRAM_POINT_SIZE_ARB
# define GL_VERTEX_PROGRAM_POINT_SIZE_ARB 0x8642
#endif

#ifndef GL_VERTEX_PROGRAM_TWO_SIDE_ARB
# define GL_VERTEX_PROGRAM_TWO_SIDE_ARB 0x8643
#endif

#ifndef GL_OBJECT_ACTIVE_ATTRIBUTES_ARB
# define GL_OBJECT_ACTIVE_ATTRIBUTES_ARB 0x8B89
#endif

#ifndef GL_OBJECT_ACTIVE_ATTRIBUTE_MAX_LENGTH_ARB
# define GL_OBJECT_ACTIVE_ATTRIBUTE_MAX_LENGTH_ARB 0x8B8A
#endif

#ifndef GL_VERTEX_ATTRIB_ARRAY_ENABLED_ARB
# define GL_VERTEX_ATTRIB_ARRAY_ENABLED_ARB 0x8622
#endif

#ifndef GL_VERTEX_ATTRIB_ARRAY_SIZE_ARB
# define GL_VERTEX_ATTRIB_ARRAY_SIZE_ARB 0x8623
#endif

#ifndef GL_VERTEX_ATTRIB_ARRAY_STRIDE_ARB
# define GL_VERTEX_ATTRIB_ARRAY_STRIDE_ARB 0x8624
#endif

#ifndef GL_VERTEX_ATTRIB_ARRAY_TYPE_ARB
# define GL_VERTEX_ATTRIB_ARRAY_TYPE_ARB 0x8625
#endif

#ifndef GL_VERTEX_ATTRIB_ARRAY_NORMALIZED_ARB
# define GL_VERTEX_ATTRIB_ARRAY_NORMALIZED_ARB 0x886A
#endif

#ifndef GL_CURRENT_VERTEX_ATTRIB_ARB
# define GL_CURRENT_VERTEX_ATTRIB_ARB 0x8626
#endif

#ifndef GL_VERTEX_ATTRIB_ARRAY_POINTER_ARB
# define GL_VERTEX_ATTRIB_ARRAY_POINTER_ARB 0x8645
#endif

#ifndef GL_FLOAT
# define GL_FLOAT 0x1406
#endif

#ifndef GL_FLOAT_VEC2_ARB
# define GL_FLOAT_VEC2_ARB 0x8B50
#endif

#ifndef GL_FLOAT_VEC3_ARB
# define GL_FLOAT_VEC3_ARB 0x8B51
#endif

#ifndef GL_FLOAT_VEC4_ARB
# define GL_FLOAT_VEC4_ARB 0x8B52
#endif

#ifndef GL_FLOAT_MAT2_ARB
# define GL_FLOAT_MAT2_ARB 0x8B5A
#endif

#ifndef GL_FLOAT_MAT3_ARB
# define GL_FLOAT_MAT3_ARB 0x8B5B
#endif

#ifndef GL_FLOAT_MAT4_ARB
# define GL_FLOAT_MAT4_ARB 0x8B5C
#endif

  static
  void
  (APIENTRY * glVertexAttrib1f)(GLuint index,
                                GLfloat v0);

  static
  void
  (APIENTRY * glVertexAttrib1s)(GLuint index,
                                GLshort v0);

  static
  void
  (APIENTRY * glVertexAttrib1d)(GLuint index,
                                GLdouble v0);

  static
  void
  (APIENTRY * glVertexAttrib2f)(GLuint index,
                                GLfloat v0,
                                GLfloat v1);

  static
  void
  (APIENTRY * glVertexAttrib2s)(GLuint index,
                                GLshort v0,
                                GLshort v1);

  static
  void
  (APIENTRY * glVertexAttrib2d)(GLuint index,
                                GLdouble v0,
                                GLdouble v1);

  static
  void
  (APIENTRY * glVertexAttrib3f)(GLuint index,
                                GLfloat v0,
                                GLfloat v1,
                                GLfloat v2);

  static
  void
  (APIENTRY * glVertexAttrib3s)(GLuint index,
                                GLshort v0,
                                GLshort v1,
                                GLshort v2);

  static
  void
  (APIENTRY * glVertexAttrib3d)(GLuint index,
                                GLdouble v0,
                                GLdouble v1,
                                GLdouble v2);

  static
  void
  (APIENTRY * glVertexAttrib4f)(GLuint index,
                                GLfloat v0,
                                GLfloat v1,
                                GLfloat v2,
                                GLfloat v3);

  static
  void
  (APIENTRY * glVertexAttrib4s)(GLuint index,
                                GLshort v0,
                                GLshort v1,
                                GLshort v2,
                                GLshort v3);

  static
  void
  (APIENTRY * glVertexAttrib4d)(GLuint index,
                                GLdouble v0,
                                GLdouble v1,
                                GLdouble v2,
                                GLdouble v3);

  static
  void
  (APIENTRY * glVertexAttrib4Nub)(GLuint index,
                                  GLubyte v0,
                                  GLubyte v1,
                                  GLubyte v2,
                                  GLubyte v3);

  static
  void
  (APIENTRY * glVertexAttrib1fv)(GLuint index,
                                 const GLfloat *v);

  static
  void
  (APIENTRY * glVertexAttrib1sv)(GLuint index,
                                 const GLshort *v);

  static
  void
  (APIENTRY * glVertexAttrib1dv)(GLuint index,
                                 const GLdouble *v);

  static
  void
  (APIENTRY * glVertexAttrib2fv)(GLuint index,
                                 const GLfloat *v);

  static
  void
  (APIENTRY * glVertexAttrib2sv)(GLuint index,
                                 const GLshort *v);

  static
  void
  (APIENTRY * glVertexAttrib2dv)(GLuint index,
                                 const GLdouble *v);

  static
  void
  (APIENTRY * glVertexAttrib3fv)(GLuint index,
                                 const GLfloat *v);

  static
  void
  (APIENTRY * glVertexAttrib3sv)(GLuint index,
                                 const GLshort *v);

  static
  void
  (APIENTRY * glVertexAttrib3dv)(GLuint index,
                                 const GLdouble *v);

  static
  void
  (APIENTRY * glVertexAttrib4fv)(GLuint index,
                                 const GLfloat *v);

  static
  void
  (APIENTRY * glVertexAttrib4sv)(GLuint index,
                                 const GLshort *v);

  static
  void
  (APIENTRY * glVertexAttrib4dv)(GLuint index,
                                 const GLdouble *v);

  static
  void
  (APIENTRY * glVertexAttrib4iv)(GLuint index,
                                 const GLint *v);

  static
  void
  (APIENTRY * glVertexAttrib4bv)(GLuint index,
                                 const GLbyte *v);

  static
  void
  (APIENTRY * glVertexAttrib4ubv)(GLuint index,
                                  const GLubyte *v);

  static
  void
  (APIENTRY * glVertexAttrib4usv)(GLuint index,
                                  const GLushort *v);

  static
  void
  (APIENTRY * glVertexAttrib4uiv)(GLuint index,
                                  const GLuint *v);

  static
  void
  (APIENTRY * glVertexAttrib4Nbv)(GLuint index,
                                  const GLbyte *v);

  static
  void
  (APIENTRY * glVertexAttrib4Nsv)(GLuint index,
                                  const GLshort *v);

  static
  void
  (APIENTRY * glVertexAttrib4Niv)(GLuint index,
                                  const GLint *v);

  static
  void
  (APIENTRY * glVertexAttrib4Nubv)(GLuint index,
                                   const GLubyte *v);

  static
  void
  (APIENTRY * glVertexAttrib4Nusv)(GLuint index,
                                   const GLushort *v);

  static
  void
  (APIENTRY * glVertexAttrib4Nuiv)(GLuint index,
                                   const GLuint *v);

  static
  void
  (APIENTRY * glVertexAttribPointer)(GLuint index,
                                     GLint size,
                                     GLenum type,
                                     GLboolean normalized,
                                     GLsizei stride,
                                     const GLvoid *pointer);

  static
  void
  (APIENTRY * glEnableVertexAttribArray)(GLuint index);

  static
  void
  (APIENTRY * glDisableVertexAttribArray)(GLuint index);

  static
  void
  (APIENTRY * glBindAttribLocation)(GL::SLhandle program,
                                    GLuint index,
                                    const GL::SLchar * name);

  static
  void
  (APIENTRY * glGetActiveAttrib)(GL::SLhandle program,
                                 GLuint index,
                                 GLsizei maxLength,
                                 GLsizei *length,
                                 GLint *size,
                                 GLenum *type,
                                 GL::SLchar * name);

  static
  GLint
  (APIENTRY * glGetAttribLocation)(GL::SLhandle program,
                                   const GL::SLchar * name);

  static
  void
  (APIENTRY * glGetVertexAttribfv)(GLuint index,
                                   GLenum pname,
                                   GLfloat *params);

  static
  void
  (APIENTRY * glGetVertexAttribiv)(GLuint index,
                                   GLenum pname,
                                   GLint *params);

  static
  void
  (APIENTRY * glGetVertexAttribdv)(GLuint index,
                                   GLenum pname,
                                   GLdouble *params);

  static
  void
  (APIENTRY * glGetVertexAttribPointerv)(GLuint index,
                                         GLenum pname,
                                         GLvoid **pointer);

//-------- GL_ARB_fragment_shader -------------------------------------------

#ifndef GL_FRAGMENT_SHADER_ARB
# define GL_FRAGMENT_SHADER_ARB 0x8B30
#endif

#ifndef GL_MAX_FRAGMENT_UNIFORM_COMPONENTS_ARB
# define GL_MAX_FRAGMENT_UNIFORM_COMPONENTS_ARB 0x8B49
#endif

  inline static
  unsigned int
  getNbFragmentUniformComponents(void);

#ifndef GL_FRAGMENT_SHADER_DERIVATIVE_HINT_ARB
# define GL_FRAGMENT_SHADER_DERIVATIVE_HINT_ARB 0x8B8B
#endif

//-------- GL_ARB_shader_objects ---------------------------------------------

#ifndef GL_PROGRAM_OBJECT_ARB
# define GL_PROGRAM_OBJECT_ARB 0x8B40
#endif

#ifndef GL_OBJECT_TYPE_ARB
# define GL_OBJECT_TYPE_ARB 0x8B4E
#endif

#ifndef GL_OBJECT_SUBTYPE_ARB
# define GL_OBJECT_SUBTYPE_ARB 0x8B4F
#endif

#ifndef GL_OBJECT_DELETE_STATUS_ARB
# define GL_OBJECT_DELETE_STATUS_ARB 0x8B80
#endif

#ifndef GL_OBJECT_COMPILE_STATUS_ARB
# define GL_OBJECT_COMPILE_STATUS_ARB 0x8B81
#endif

#ifndef GL_OBJECT_LINK_STATUS_ARB
# define GL_OBJECT_LINK_STATUS_ARB 0x8B82
#endif

#ifndef GL_OBJECT_VALIDATE_STATUS_ARB
# define GL_OBJECT_VALIDATE_STATUS_ARB 0x8B83
#endif

#ifndef GL_OBJECT_INFO_LOG_LENGTH_ARB
# define GL_OBJECT_INFO_LOG_LENGTH_ARB 0x8B84
#endif

#ifndef GL_OBJECT_ATTACHED_OBJECTS_ARB
# define GL_OBJECT_ATTACHED_OBJECTS_ARB 0x8B85
#endif

#ifndef GL_OBJECT_ACTIVE_UNIFORMS_ARB
# define GL_OBJECT_ACTIVE_UNIFORMS_ARB 0x8B86
#endif

#ifndef GL_OBJECT_ACTIVE_UNIFORM_MAX_LENGTH_ARB
# define GL_OBJECT_ACTIVE_UNIFORM_MAX_LENGTH_ARB 0x8B87
#endif

#ifndef GL_OBJECT_SHADER_SOURCE_LENGTH_ARB
# define GL_OBJECT_SHADER_SOURCE_LENGTH_ARB 0x8B88
#endif

#ifndef GL_SHADER_OBJECT_ARB
# define GL_SHADER_OBJECT_ARB 0x8B48
#endif

#ifndef GL_FLOAT
# define GL_FLOAT 0x1406
#endif

#ifndef GL_FLOAT_VEC2_ARB
# define GL_FLOAT_VEC2_ARB 0x8B50
#endif

#ifndef GL_FLOAT_VEC3_ARB
# define GL_FLOAT_VEC3_ARB 0x8B51
#endif

#ifndef GL_FLOAT_VEC4_ARB
# define GL_FLOAT_VEC4_ARB 0x8B52
#endif

#ifndef GL_INT
# define GL_INT 0x1404
#endif

#ifndef GL_INT_VEC2_ARB
# define GL_INT_VEC2_ARB 0x8B53
#endif

#ifndef GL_INT_VEC3_ARB
# define GL_INT_VEC3_ARB 0x8B54
#endif

#ifndef GL_INT_VEC4_ARB
# define GL_INT_VEC4_ARB 0x8B55
#endif

#ifndef GL_BOOL_ARB
# define GL_BOOL_ARB 0x8B56
#endif

#ifndef GL_BOOL_VEC2_ARB
# define GL_BOOL_VEC2_ARB 0x8B57
#endif

#ifndef GL_BOOL_VEC3_ARB
# define GL_BOOL_VEC3_ARB 0x8B58
#endif

#ifndef GL_BOOL_VEC4_ARB
# define GL_BOOL_VEC4_ARB 0x8B59
#endif

#ifndef GL_FLOAT_MAT2_ARB
# define GL_FLOAT_MAT2_ARB 0x8B5A
#endif

#ifndef GL_FLOAT_MAT3_ARB
# define GL_FLOAT_MAT3_ARB 0x8B5B
#endif

#ifndef GL_FLOAT_MAT4_ARB
# define GL_FLOAT_MAT4_ARB 0x8B5C
#endif

#ifndef GL_SAMPLER_1D_ARB
# define GL_SAMPLER_1D_ARB 0x8B5D
#endif

#ifndef GL_SAMPLER_2D_ARB
# define GL_SAMPLER_2D_ARB 0x8B5E
#endif

#ifndef GL_SAMPLER_3D_ARB
# define GL_SAMPLER_3D_ARB 0x8B5F
#endif

#ifndef GL_SAMPLER_CUBE_ARB
# define GL_SAMPLER_CUBE_ARB 0x8B60
#endif

#ifndef GL_SAMPLER_1D_SHADOW_ARB
# define GL_SAMPLER_1D_SHADOW_ARB 0x8B61
#endif

#ifndef GL_SAMPLER_2D_SHADOW_ARB
# define GL_SAMPLER_2D_SHADOW_ARB 0x8B62
#endif

#ifndef GL_SAMPLER_2D_RECT_ARB
# define GL_SAMPLER_2D_RECT_ARB 0x8B63
#endif

#ifndef GL_SAMPLER_2D_RECT_SHADOW_ARB
# define GL_SAMPLER_2D_RECT_SHADOW_ARB 0x8B64
#endif

  static
  void
  (APIENTRY * glDeleteObject)(GL::SLhandle object);

  static
  GL::SLhandle
  (APIENTRY *  glGetHandle)(GLenum pname);

  static
  void
  (APIENTRY * glDetachObject)(GL::SLhandle container,
                              GL::SLhandle attached);

  static
  GL::SLhandle
  (APIENTRY * glCreateShaderObject)(GLenum shaderType);

  static
  void
  (APIENTRY * glShaderSource)(GL::SLhandle shader,
                              GLsizei nstrings,
                              const GL::SLchar ** strings,
                              const GLint *lengths);

  static
  void
  (APIENTRY * glCompileShader)(GL::SLhandle shader);

  static
  GL::SLhandle
  (APIENTRY * glCreateProgramObject)(void);

  static
  void
  (APIENTRY * glAttachObject)(GL::SLhandle program,
                              GL::SLhandle shader);

  static
  void
  (APIENTRY * glLinkProgram)(GL::SLhandle program);

  static
  void
  (APIENTRY * glUseProgramObject)(GL::SLhandle program);

  static
  void
  (APIENTRY * glValidateProgram)(GL::SLhandle program);

  static
  void
  (APIENTRY * glUniform1f)(GLint location,
                           GLfloat v0);

  static
  void
  (APIENTRY * glUniform2f)(GLint location,
                           GLfloat v0,
                           GLfloat v1);

  static
  void
  (APIENTRY * glUniform3f)(GLint location,
                           GLfloat v0,
                           GLfloat v1,
                           GLfloat v2);

  static
  void
  (APIENTRY * glUniform4f)(GLint location,
                           GLfloat v0,
                           GLfloat v1,
                           GLfloat v2,
                           GLfloat v3);

  static
  void
  (APIENTRY * glUniform1i)(GLint location,
                           GLint v0);

  static
  void
  (APIENTRY * glUniform2i)(GLint location,
                           GLint v0,
                           GLint v1);

  static
  void
  (APIENTRY * glUniform3i)(GLint location,
                           GLint v0,
                           GLint v1,
                           GLint v2);

  static
  void
  (APIENTRY * glUniform4i)(GLint location,
                           GLint v0,
                           GLint v1,
                           GLint v2,
                           GLint v3);

  static
  void
  (APIENTRY * glUniform1fv)(GLint location,
                            GLsizei count,
                            GLfloat *value);

  static
  void
  (APIENTRY * glUniform2fv)(GLint location,
                            GLsizei count,
                            GLfloat *value);

  static
  void
  (APIENTRY * glUniform3fv)(GLint location,
                            GLsizei count,
                            GLfloat *value);

  static
  void
  (APIENTRY * glUniform4fv)(GLint location,
                            GLsizei count,
                            GLfloat *value);

  static
  void
  (APIENTRY * glUniform1iv)(GLint location,
                            GLsizei count,
                            GLint *value);

  static
  void
  (APIENTRY * glUniform2iv)(GLint location,
                            GLsizei count,
                            GLint *value);

  static
  void
  (APIENTRY * glUniform3iv)(GLint location,
                            GLsizei count,
                            GLint *value);

  static
  void
  (APIENTRY * glUniform4iv)(GLint location,
                            GLsizei count,
                            GLint *value);

  static
  void
  (APIENTRY * glUniformMatrix2fv)(GLint location,
                                  GLsizei count,
                                  GLboolean transpose,
                                  GLfloat *value);

  static
  void
  (APIENTRY * glUniformMatrix3fv)(GLint location,
                                  GLsizei count,
                                  GLboolean transpose,
                                  GLfloat *value);

  static
  void
  (APIENTRY * glUniformMatrix4fv)(GLint location,
                                  GLsizei count,
                                  GLboolean transpose,
                                  GLfloat *value);

  static
  void
  (APIENTRY * glGetObjectParameterfv)(GL::SLhandle object,
                                      GLenum pname,
                                      GLfloat *params);

  static
  void
  (APIENTRY * glGetObjectParameteriv)(GL::SLhandle object,
                                      GLenum pname,
                                      GLint *params);

  static
  void
  (APIENTRY * glGetInfoLog)(GL::SLhandle object,
                            GLsizei maxLength,
                            GLsizei *length,
                            GL::SLchar * infoLog);

  static
  void
  (APIENTRY * glGetAttachedObjects)(GL::SLhandle program,
                                    GLsizei maxCount,
                                    GLsizei *count,
                                    GL::SLhandle *objects);

  static
  GLint
  (APIENTRY * glGetUniformLocation)(GL::SLhandle program,
                                    const GL::SLchar * name);

  static
  void
  (APIENTRY * glGetActiveUniform)(GL::SLhandle program,
                                  GLuint index,
                                  GLsizei maxLength,
                                  GLsizei *length,
                                  GLint *size,
                                  GLenum *type,
                                  GL::SLchar * name);

  static
  void
  (APIENTRY * glGetUniformfv)(GL::SLhandle program,
                              GLint location,
                              GLfloat *params);

  static
  void
  (APIENTRY * glGetUniformiv)(GL::SLhandle program,
                              GLint location,
                              GLint *params);

  static
  void
  (APIENTRY * glGetShaderSource)(GL::SLhandle shader,
                                 GLsizei maxLength,
                                 GLsizei *length,
                                 GL::SLchar * source);

//-------- GL_EXT_framebuffer_object -----------------------------------------

  inline static
  bool
  hasFrameBufferObject(void);

#ifndef GL_FRAMEBUFFER_EXT
# define GL_FRAMEBUFFER_EXT 0x8D40
#endif

#ifndef GL_RENDERBUFFER_EXT
# define GL_RENDERBUFFER_EXT 0x8D41
#endif

#ifndef GL_COLOR_ATTACHMENT0_EXT
# define GL_COLOR_ATTACHMENT0_EXT 0x8CE0
#endif

#ifndef GL_DEPTH_ATTACHMENT_EXT
# define GL_DEPTH_ATTACHMENT_EXT 0x8D00
#endif

#ifndef GL_DEPTH_COMPONENT24
# define GL_DEPTH_COMPONENT24 0x81A6
#endif

#ifndef GL_FRAMEBUFFER_COMPLETE_EXT
# define GL_FRAMEBUFFER_COMPLETE_EXT 0x8CD5
#endif

  static
  void
  (APIENTRY * glGenFramebuffers)(GLsizei n,
                                 GLuint * framebuffers);

  static
  void
  (APIENTRY * glDeleteFramebuffers)(GLsizei n,
                                    const GLuint * framebuffers);

  static
  void
  (APIENTRY * glGenRenderbuffers)(GLsizei n,
                                  GLuint * renderbuffers);

  static
  void
  (APIENTRY * glDeleteRenderbuffers)(GLsizei n,
                                     const GLuint * renderbuffers);

  static
  void
  (APIENTRY * glBindFramebuffer)(GLenum target,
                                 GLuint framebuffer);

  static
  void
  (APIENTRY * glFramebufferTexture2D)(GLenum target,
                                      GLenum attachment,
                                      GLenum textarget,
                                      GLuint texture,
                                      GLint level);

  static
  void
  (APIENTRY * glBindRenderbuffer)(GLenum target,
                                  GLuint renderbuffer);

  static
  void
  (APIENTRY * glRenderbufferStorage)(GLenum target,
                                     GLenum internalformat,
                                     GLsizei width,
                                     GLsizei height);

  static
  void
  (APIENTRY * glFramebufferRenderbuffer)(GLenum target,
                                         GLenum attachment,
                                         GLenum renderbuffertarget,
                                         GLuint renderbuffer);

  static
  GLenum
  (APIENTRY * glCheckFramebufferStatus)(GLenum target);

//-------- GL_ARB_vertex_buffer_object ---------------------------------------

  typedef ptrdiff_t VBOintptr; // avoid dependency with GLintptrARB
  typedef ptrdiff_t VBOsizeiptr; // avoid dependency with GLsizeiptrARB

  inline static
  bool
  hasVertexBufferObject(void);

#ifndef GL_ARRAY_BUFFER_ARB
# define GL_ARRAY_BUFFER_ARB 0x8892
#endif

#ifndef GL_ELEMENT_ARRAY_BUFFER_ARB
# define GL_ELEMENT_ARRAY_BUFFER_ARB 0x8893
#endif

#ifndef GL_ARRAY_BUFFER_BINDING_ARB
# define GL_ARRAY_BUFFER_BINDING_ARB 0x8894
#endif

#ifndef GL_ELEMENT_ARRAY_BUFFER_BINDING_ARB
# define GL_ELEMENT_ARRAY_BUFFER_BINDING_ARB 0x8895
#endif

#ifndef GL_VERTEX_ARRAY_BUFFER_BINDING_ARB
# define GL_VERTEX_ARRAY_BUFFER_BINDING_ARB 0x8896
#endif

#ifndef GL_NORMAL_ARRAY_BUFFER_BINDING_ARB
# define GL_NORMAL_ARRAY_BUFFER_BINDING_ARB 0x8897
#endif

#ifndef GL_COLOR_ARRAY_BUFFER_BINDING_ARB
# define GL_COLOR_ARRAY_BUFFER_BINDING_ARB 0x8898
#endif

#ifndef GL_INDEX_ARRAY_BUFFER_BINDING_ARB
# define GL_INDEX_ARRAY_BUFFER_BINDING_ARB 0x8899
#endif

#ifndef GL_TEXTURE_COORD_ARRAY_BUFFER_BINDING_ARB
# define GL_TEXTURE_COORD_ARRAY_BUFFER_BINDING_ARB 0x889A
#endif

#ifndef GL_EDGE_FLAG_ARRAY_BUFFER_BINDING_ARB
# define GL_EDGE_FLAG_ARRAY_BUFFER_BINDING_ARB 0x889B
#endif

#ifndef GL_SECONDARY_COLOR_ARRAY_BUFFER_BINDING_ARB
# define GL_SECONDARY_COLOR_ARRAY_BUFFER_BINDING_ARB 0x889C
#endif

#ifndef GL_FOG_COORDINATE_ARRAY_BUFFER_BINDING_ARB
# define GL_FOG_COORDINATE_ARRAY_BUFFER_BINDING_ARB 0x889D
#endif

#ifndef GL_WEIGHT_ARRAY_BUFFER_BINDING_ARB
# define GL_WEIGHT_ARRAY_BUFFER_BINDING_ARB 0x889E
#endif

#ifndef GL_VERTEX_ATTRIB_ARRAY_BUFFER_BINDING_ARB
# define GL_VERTEX_ATTRIB_ARRAY_BUFFER_BINDING_ARB 0x889F
#endif

#ifndef GL_STREAM_READ_ARB
# define GL_STREAM_DRAW_ARB 0x88E0
#endif

#ifndef GL_STREAM_READ_ARB
# define GL_STREAM_READ_ARB 0x88E1
#endif

#ifndef GL_STREAM_COPY_ARB
# define GL_STREAM_COPY_ARB 0x88E2
#endif

#ifndef GL_STATIC_DRAW_ARB
# define GL_STATIC_DRAW_ARB 0x88E4
#endif

#ifndef GL_STATIC_READ_ARB
# define GL_STATIC_READ_ARB 0x88E5
#endif

#ifndef GL_STATIC_COPY_ARB
# define GL_STATIC_COPY_ARB 0x88E6
#endif

#ifndef GL_DYNAMIC_DRAW_ARB
# define GL_DYNAMIC_DRAW_ARB 0x88E8
#endif

#ifndef GL_DYNAMIC_READ_ARB
# define GL_DYNAMIC_READ_ARB 0x88E9
#endif

#ifndef GL_DYNAMIC_COPY_ARB
# define GL_DYNAMIC_COPY_ARB 0x88EA
#endif

#ifndef GL_READ_ONLY_ARB
# define GL_READ_ONLY_ARB 0x88B8
#endif

#ifndef GL_WRITE_ONLY_ARB
# define GL_WRITE_ONLY_ARB 0x88B9
#endif

#ifndef GL_READ_WRITE_ARB
# define GL_READ_WRITE_ARB 0x88BA
#endif

#ifndef GL_BUFFER_SIZE_ARB
# define GL_BUFFER_SIZE_ARB 0x8764
#endif

#ifndef GL_BUFFER_USAGE_ARB
# define GL_BUFFER_USAGE_ARB 0x8765
#endif

#ifndef GL_BUFFER_ACCESS_ARB
# define GL_BUFFER_ACCESS_ARB 0x88BB
#endif

#ifndef GL_BUFFER_MAPPED_ARB
# define GL_BUFFER_MAPPED_ARB 0x88BC
#endif

#ifndef GL_BUFFER_MAP_POINTER_ARB
# define GL_BUFFER_MAP_POINTER_ARB 0x88BD
#endif

  static
  void
  (APIENTRY * glBindBuffer)(GLenum target,
			    GLuint buffer);

  static
  void
  (APIENTRY * glDeleteBuffers)(GLsizei n,
			       const GLuint * buffers);

  static
  void
  (APIENTRY * glGenBuffers)(GLsizei n,
			    GLuint * buffers);

  static
  GLboolean
  (APIENTRY * glIsBuffer)(GLuint buffer);

  static
  void
  (APIENTRY * glBufferData)(GLenum target,
			    VBOsizeiptr size,
			    const GLvoid * data,
			    GLenum usage);

  static
  void
  (APIENTRY * glBufferSubData)(GLenum target,
			       VBOintptr offset,
			       VBOsizeiptr size,
			       const GLvoid * data);

  static
  void
  (APIENTRY * glGetBufferSubData)(GLenum target,
				  VBOintptr offset,
				  VBOsizeiptr size,
				  GLvoid * data);

  static
  GLvoid*
  (APIENTRY * glMapBuffer)(GLenum target,
			   GLenum access);

  static
  GLboolean
  (APIENTRY * glUnmapBuffer)(GLenum target);

  static
  void
  (APIENTRY * glGetBufferParameteriv)(GLenum target,
				      GLenum pname,
				      GLint * params);

  static
  void
  (APIENTRY * glGetBufferPointerv)(GLenum target,
				   GLenum pname,
				   GLvoid ** params);

//-------- GL_ARB_point_sprite -----------------------------------------------

  inline static
  bool
  hasPointSprite(void);

#ifndef GL_POINT_SPRITE_ARB
# define GL_POINT_SPRITE_ARB 0x8861
#endif

#ifndef GL_COORD_REPLACE_ARB
# define GL_COORD_REPLACE_ARB 0x8862
#endif

//-------- GL_ARB_point_parameters -------------------------------------------

#ifndef POINT_SIZE_MIN_ARB
# define POINT_SIZE_MIN_ARB 0x8126
#endif

#ifndef POINT_SIZE_MAX_AR
# define POINT_SIZE_MAX_ARB 0x8127
#endif

#ifndef POINT_FADE_THRESHOLD_SIZE_ARB
# define POINT_FADE_THRESHOLD_SIZE_ARB 0x8128
#endif

#ifndef POINT_DISTANCE_ATTENUATION_ARB
# define POINT_DISTANCE_ATTENUATION_ARB 0x8129
#endif

  static
  void
  (APIENTRY * glPointParameterf)(GLenum pname,
				 GLfloat param);

  static
  void
  (APIENTRY * glPointParameterfv)(GLenum pname,
				  GLfloat * params);

//-------- Forbidden operations ----------------------------------------------
private:

  GL(void); // no implementation

  GL(const GL &); // no implementation

  GL & // itself
  operator=(const GL &); // no implementation

  virtual
  ~GL(void); // no implementation

  static
  void * // GL function pointer
  _getFunctionPtr(const char * procName);

  static
  void
  (*(*_getProcAddress)(const char *))(void);

//-------- Static attributes -------------------------------------------------
private:

  static unsigned int _nbLights;
  static unsigned int _maxTextureSize;
  static bool _hasSeparateSpecularColor;
  static bool _hasTextureCompression;
  static bool _hasTextureCubeMap;
  static unsigned int _nbTextureUnits;
  static bool _hasMipmapGeneration;
  static bool _hasShadingLanguage;
  static unsigned int _nbVertexUniformComponents;
  static unsigned int _nbVaryingFloats;
  static unsigned int _nbVertexAttribs;
  static unsigned int _nbTextureImageUnits;
  static unsigned int _nbVertexTextureImageUnits;
  static unsigned int _nbCombinedTextureImageUnits;
  static unsigned int _nbTextureCoords;
  static unsigned int _nbFragmentUniformComponents;
  static bool _hasFrameBufferObject;
  static bool _hasVertexBufferObject;
  static bool _hasPointSprite;
};

} // namespace AReVi

//-------- Inline source code ------------------------------------------------

#include "AReVi/Lib3D/gl_priv.Ci"

#endif // AREVI_LIB3D_GL_PRIV_H

//----------------------------------------------------------------------------
