// LI2/ENIB - AReVi - 3D services
//----------------------------------------------------------------------------
// file        : AReVi/Lib3D/light3D.h
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVI_LIB3D_LIGHT3D_H
#define AREVI_LIB3D_LIGHT3D_H 1

#include "AReVi/Lib3D/base3D.h"

namespace AReVi {

class Scene3D;
class ShapePartRenderer;

//----------------------------------------------------------------------------
// Class Light3D
//----------------------------------------------------------------------------

class Light3D : public Base3D
{
//-------- Type information --------------------------------------------------
public:

  AR_CLASS(Light3D)

//-------- Construction / Destruction ----------------------------------------

  AR_CONSTRUCTOR(Light3D)

//-------- Switching ---------------------------------------------------------

  virtual
  void
  setSwitch(bool onOff);

  virtual
  bool // light is switched on
  getSwitch(void) const;

//-------- Light model -------------------------------------------------------

  virtual
  void
  setDirectional(bool directional);

  virtual
  bool // light is directional
  getDirectional(void) const;

  virtual
  void
  setAttenuation(double constant,
                 double linear,
                 double quadratic);

  virtual
  void
  getAttenuation(double & constantOut,
                 double & linearOut,
                 double & quadraticOut) const;

  virtual
  void
  setSpotParameters(double exponent,
                    double cutOff);

  virtual
  void
  getSpotParameters(double & exponentOut,
                    double & cutOffOut) const;

//-------- Color -------------------------------------------------------------

  virtual
  void
  setColor(double red,
           double green,
           double blue);

  virtual
  void
  getColor(double & redOut,
           double & greenOut,
           double & blueOut) const;

//-------- Serialization operations ------------------------------------------

  AR_UNSTUBBED virtual
  void
  serialize(SerializationDependencies & dependenciesOut,
            ArRef<AbstractOStream> output) const;

  AR_UNSTUBBED virtual
  bool // success
  unserialize(const SerializationDependencies & dependencies,
              ArRef<AbstractIStream> input);

//-------- Event management --------------------------------------------------

  AR_CALLBACK(Light3D,Setup,_setupCB,
              /* no field */)

//-------- Forbidden operations ----------------------------------------------
protected:

  friend class Renderer3D;
  friend class Scene3D;
  friend class ShapePartRenderer;

  virtual
  void
  _onMotion(void);

  virtual
  void
  _addScene(ArRef<Scene3D> scene);

  virtual
  void
  _removeScene(ArRef<Scene3D> scene);

  virtual
  void
  _postRedisplay(void);

  virtual
  void
  _render(unsigned int lightNumber) const;

  static
  void
  _renderHeadLight(unsigned int lightNumber,
                   const Util3D::Base & base,
                   double red,
                   double green,
                   double blue);

  virtual
  void
  _onSetup(void);

//-------- Member attributes -------------------------------------------------
protected:

  StlVector<ArPtr<Scene3D> > _scenes;
  bool _on;
  bool _directional;
  double _constantAtt;
  double _linearAtt;
  double _quadraticAtt;
  double _spotExponent;
  double _spotCutOff;
  Util3D::Dbl3 _color;
  CallbackManager<Light3D,SetupEvent> _setupCB;
};

} // namespace AReVi

#endif // AREVI_LIB3D_LIGHT3D_H

//----------------------------------------------------------------------------
