// LI2/ENIB - AReVi - 3D services
//----------------------------------------------------------------------------
// file        : AReVi/Lib3D/llw_priv.h
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVI_LIB3D_LLW_PRIV_H
#define AREVI_LIB3D_LLW_PRIV_H 1

#include "AReVi/Utils/memoryBlock.h"

#include "AReVi/Lib3D/util3D_priv.h"

namespace AReVi {

extern "C" bool JpegLib_init(const StlVector<StlString> *);
extern "C" void JpegLib_destroy(void);

//----------------------------------------------------------------------------
// Class LLW (abstract)
//----------------------------------------------------------------------------

class LLW
{
//-------- Data types --------------------------------------------------------
public:

  typedef void * AreaID;

  typedef LLW::AreaID DrawID;

  typedef LLW::DrawID WinID;

  typedef LLW::DrawID PixID;

  typedef LLW::AreaID GLBufID;

  typedef void * FontID;

  typedef void * GLFontID;

  typedef void * BMFontID;

  enum EventType
    {
    EVT_NONE=0,
    EVT_CLOSE=1,
    EVT_EXPOSE=2,
    EVT_GEOMETRY=4,
    EVT_MOUSE_BUTTON=8,
    EVT_MOUSE_MOTION=16,
    EVT_MOUSE_INOUT=32,
    EVT_KEYBOARD=64,
    EVT_FOCUS=128,
    EVT_TIMEOUT=256,
    EVT_ALL=511, // all previous event types
    EVT_MOUSE_BUTTON_UNGRAB=512 // changes EVT_MOUSE_BUTTON's behavior
    };

  typedef int EventMask; // bitwise operations with enums gives an int !

  enum Modifier
    {
    MDF_NONE=0,
    MDF_CONTROL=1,
    MDF_SHIFT=2,
    MDF_CAPSLOCK=4,
    MDF_NUMLOCK=8,
    MDF_LEFT=16,
    MDF_MIDDLE=32,
    MDF_RIGHT=64
    };

  enum Button
    {
    BTN_NONE=0,
    BTN_LEFT=1,
    BTN_MIDDLE=2,
    BTN_RIGHT=3,
    BTN_FORWARD=4,
    BTN_BACKWARD=5
    };

  struct Event
    {
    LLW::AreaID area;
    LLW::EventType type;
    LLW::Button button;
    StlString key;
    bool onOff;
    unsigned int time;
    int x,y;
    int xScreen,yScreen;
    LLW::Modifier modifiers;
    };

  enum CursorID
    {
    CSR_EMPTY,
    CSR_ARROW,
    CSR_NORTH,
    CSR_SOUTH,
    CSR_EAST,
    CSR_WEST,
    CSR_NORTHEAST,
    CSR_NORTHWEST,
    CSR_SOUTHEAST,
    CSR_SOUTHWEST,
    CSR_MOVE,
    CSR_CROSS,
    CSR_WAIT,
    CSR_TEXT,
    CSR_HAND,
    CSR_HELP,
    CSR_UP,
    CSR_NO
    };

//-------- Initialization ----------------------------------------------------

  static
  void
  init(void);

  static
  void
  destroy(void);

  static
  bool
  initFail(void);

//-------- Window creation/destruction ---------------------------------------

  static
  LLW::WinID
  createTopWindow(const StlString & title,
                  bool managed,
                  int x,
                  int y,
                  unsigned int width,
                  unsigned int height);
  static
  LLW::WinID
  createChildWindow(LLW::WinID parent,
                    int x,
                    int y,
                    unsigned int width,
                    unsigned int height);

  static
  LLW::WinID
  createEmbeddedWindow(int externalWindowID,
                       int x,
                       int y,
                       unsigned int width,
                       unsigned int height);

  static
  void
  destroyWindow(LLW::WinID window,
                bool recursive);

  static
  int // platform-dependent window ID
  getSpecificWindowID(LLW::WinID window);

//-------- Pixmap creation/destruction ---------------------------------------

  static
  LLW::PixID
  createPixmap(unsigned int width,
               unsigned int height);

  static
  LLW::PixID
  createImagePixmap(const StlString & url);

  static
  void
  destroyPixmap(LLW::PixID pixmap);

//-------- GLBuffer creation/destruction -------------------------------------

  static
  LLW::GLBufID
  createGLBuffer(unsigned int width,
                 unsigned int height);

  static
  void
  destroyGLBuffer(LLW::GLBufID buffer);

//-------- Area identification -----------------------------------------------

  static
  bool // area is a drawable
  areaIsDrawable(LLW::AreaID area);

  static
  bool // area is a window
  areaIsWindow(LLW::AreaID area);

  static
  bool // area is a pixmap
  areaIsPixmap(LLW::AreaID area);

  static
  bool // area is a GL buffer
  areaIsGLBuffer(LLW::AreaID area);

//-------- Window hierarchy --------------------------------------------------

  static
  LLW::WinID // this window's parent
  getParentWindow(LLW::WinID window);

  static
  LLW::WinID // this window's top window
  getTopWindow(LLW::WinID window);

  static
  bool // success
  containsChildWindow(LLW::WinID window,
                      LLW::WinID child);

//-------- Window management -------------------------------------------------

  static
  void
  setTopWindowTitle(LLW::WinID window,
                    const StlString & title);

  static
  const StlString & // window title
  getTopWindowTitle(LLW::WinID window);

  static
  void
  mapWindow(LLW::WinID window);

  static
  void
  unmapWindow(LLW::WinID window);

  static
  void
  iconifyTopWindow(LLW::WinID window);

  static
  bool // window is mapped
  isWindowMapped(LLW::WinID window);

  static
  bool // window and its ancestors are mapped
  isWindowViewable(LLW::WinID window);

  static
  void
  setWindowBackgroundColor(LLW::WinID window,
                           unsigned int color);

  static
  unsigned int // background color
  getWindowBackgroundColor(LLW::WinID window);

  static
  void
  setWindowBackgroundPixmap(LLW::WinID window,
                            LLW::PixID pixmap);

  static
  LLW::PixID // background pixmap
  getWindowBackgroundPixmap(LLW::WinID window);

//-------- Cursor management -------------------------------------------------

  static
  void
  setWindowCursor(LLW::WinID window,
                  LLW::CursorID cursor);

  static
  LLW::CursorID // cursor for this window
  getWindowCursor(LLW::WinID window);

  static
  void
  moveCursor(int dx,
             int dy);

//-------- Geometry ----------------------------------------------------------

  static
  void
  setWindowGeometry(LLW::WinID window,
                    int x,
                    int y,
                    unsigned int width,
                    unsigned int height);

  static
  void
  getWindowGeometry(LLW::WinID window,
                    int & xOut,
                    int & yOut,
                    unsigned int & widthOut,
                    unsigned int & heightOut);

  static
  void
  setWindowPosition(LLW::WinID window,
                    int x,
                    int y);

  static
  void
  getWindowPosition(LLW::WinID window,
                    int & xOut,
                    int & yOut);

  static
  void
  setWindowSize(LLW::WinID window,
                unsigned int width,
                unsigned int height);

  static
  void
  getWindowSize(LLW::WinID window,
                unsigned int & widthOut,
                unsigned int & heightOut);

  static
  void
  setWindowMinSize(LLW::WinID window,
                   unsigned int minWidth,
                   unsigned int minHeight);

  static
  void
  getWindowMinSize(LLW::WinID window,
                   unsigned int & minWidthOut,
                   unsigned int & minHeightOut);

  static
  void
  getPixmapSize(LLW::PixID pixmap,
                unsigned int & widthOut,
                unsigned int & heightOut);

  static
  void
  getGLBufferSize(LLW::GLBufID buffer,
                  unsigned int & widthOut,
                  unsigned int & heightOut);

  static
  void
  getAreaSize(LLW::AreaID area,
              unsigned int & widthOut,
              unsigned int & heightOut);

  static
  void
  getScreenSize(unsigned int & widthOut,
                unsigned int & heightOut);

//-------- Color management --------------------------------------------------

  static
  unsigned int // color pixel value
  makeDoubleColor(double red,
                  double green,
                  double blue);

  static
  unsigned int // color pixel value
  makeByteColor(unsigned char red,
                unsigned char green,
                unsigned char blue);

//-------- Font management ---------------------------------------------------

  static
  LLW::FontID // loaded font
  loadFont(const StlString & fontName);

  static
  void
  freeFont(LLW::FontID font);

  static
  void
  getFontHeight(LLW::FontID font,
                unsigned int & ascentOut,
                unsigned int & descentOut);

  static
  unsigned int // text width
  getTextWidth(LLW::FontID font,
               const StlString & text);

  static
  void
  drawText(LLW::DrawID drawable,
           LLW::FontID font,
           unsigned int color,
           int x,
           int y,
           const StlString & text);

//-------- Drawing -----------------------------------------------------------

  static
  void
  drawPoint(LLW::DrawID drawable,
            unsigned int color,
            int x,
            int y);

  static
  void
  drawLine(LLW::DrawID drawable,
           unsigned int color,
           int x1,
           int y1,
           int x2,
           int y2);

  static
  void
  drawRectangle(LLW::DrawID drawable,
                unsigned int color,
                int x,
                int y,
                unsigned int width,
                unsigned int height);

  static
  void
  fillRectangle(LLW::DrawID drawable,
                unsigned int color,
                int x,
                int y,
                unsigned int width,
                unsigned int height);

  static
  void
  drawEllipse(LLW::DrawID drawable,
              unsigned int color,
              int x,
              int y,
              unsigned int width,
              unsigned int height);

  static
  void
  fillEllipse(LLW::DrawID drawable,
              unsigned int color,
              int x,
              int y,
              unsigned int width,
              unsigned int height);

  static
  void
  drawArc(LLW::DrawID drawable,
          unsigned int color,
          int x,
          int y,
          unsigned int width,
          unsigned int height,
          double startAngle,
          double endAngle);

  static
  void
  fillArc(LLW::DrawID drawable,
          unsigned int color,
          int x,
          int y,
          unsigned int width,
          unsigned int height,
          double startAngle,
          double endAngle);

  static
  void
  drawImage(LLW::DrawID dest,
            int destX,
            int destY,
            ArConstRef<MemoryBlock> image,
            unsigned int width,
            unsigned int height,
            unsigned int components);

  static
  void
  copyDrawable(LLW::DrawID dest,
               int destX,
               int destY,
               LLW::DrawID source);

  static
  void
  copyDrawablePart(LLW::DrawID dest,
                   int destX,
                   int destY,
                   LLW::DrawID source,
                   int sourceX,
                   int sourceY,
                   unsigned int width,
                   unsigned int height);

  static
  void
  clearWindow(LLW::WinID window);

//-------- OpenGL management -------------------------------------------------

  static
  void
  beginGLRendering(LLW::AreaID area);

  static
  void
  endGLRendering(void);

  static
  unsigned int // texture of buffer
  getGLBufferTexture(LLW::GLBufID buffer);

  static
  unsigned int // texture components in buffer
  getGLBufferTextureComponents(LLW::GLBufID buffer);

//-------- OpenGL text management --------------------------------------------

  static
  LLW::GLFontID // loaded OpenGL font
  loadGLFont(const StlString & url);

  static
  void
  freeGLFont(LLW::GLFontID font);

  static
  void
  getGLFontHeight(LLW::GLFontID font,
                  double & ascentOut,
                  double & descentOut);

  static
  double // text width
  getGLTextWidth(LLW::GLFontID font,
                 double symbolSpace,
                 double whiteSpace,
                 const StlString & text);

  static
  void
  buildGLText(LLW::GLFontID font,
              double symbolSpace,
              double whiteSpace,
              const StlString & text,
              double extrusion,
              StlVector<Util3D::Dbl3> & verticesInOut,
              StlVector<Util3D::Face> & facesVInOut,
              StlVector<Util3D::Dbl3> & normalsInOut,
              StlVector<Util3D::Face> & facesNInOut);

  static
  void
  drawGLText(LLW::GLFontID font,
             double symbolSpace,
             double whiteSpace,
             const StlString & text);

//-------- Bitmap text management --------------------------------------------

  static
  LLW::BMFontID // loaded bitmap font
  loadBMFont(const StlString & url);

  static
  void
  freeBMFont(LLW::BMFontID font);

  static
  void
  getBMFontHeight(LLW::BMFontID font,
                  unsigned int & ascentOut,
                  unsigned int & descentOut);

  static
  unsigned int // symbol width
  getBMSymbolWidth(LLW::BMFontID font,
                   char symbol);

  static
  unsigned int // text width
  getBMTextWidth(LLW::BMFontID font,
                 const StlString & text);

  static
  void
  drawBMSymbol(LLW::BMFontID font,
               char symbol);

  static
  void
  drawBMText(LLW::BMFontID font,
             const StlString & text);

//-------- Time management ---------------------------------------------------

  static
  unsigned int // expiration time
  postTimeOut(LLW::AreaID area,
              unsigned int delay);

  static
  void
  cancelTimeOut(LLW::AreaID area,
                unsigned int timeOut);

//-------- Event management --------------------------------------------------

  static
  void
  setWindowEventMask(LLW::WinID window,
                     LLW::EventMask eventMask);

  static
  LLW::EventMask // event mask for this window
  getWindowEventMask(LLW::WinID window);

  static
  void
  setWindowKeyRepeat(LLW::WinID window,
                     bool keyRepeat);

  static
  bool // key repeat is enabled for this window
  getWindowKeyRepeat(LLW::WinID window);

  static
  const LLW::Event *
  nextEvent(bool blocking);

  static
  StlString
  printEvent(const LLW::Event * evt);

  static
  void
  flush(void);

//-------- Beep --------------------------------------------------------------

  static
  void
  beep(void);

//-------- Image management --------------------------------------------------

  static
  bool // success (dataOut filled from bottom to top)
  loadImage(const StlString & url,
            unsigned int & widthOut,
            unsigned int & heightOut,
            unsigned int & componentsOut,
            ArRef<MemoryBlock> dataOut);

  static
  bool // success
  saveImage(const StlString & fileName,
            unsigned int width,
            unsigned int height,
            unsigned int components,
            ArConstRef<MemoryBlock> data);

  static
  bool // success (dataOut filled from bottom to top)
  dumpArea(LLW::AreaID area,
           unsigned int & widthOut,
           unsigned int & heightOut,
           unsigned int & componentsOut,
           ArRef<MemoryBlock> dataOut);

  static
  bool // success
  loadRGBImageFromJPEG(ArConstRef<MemoryBlock> dataIn,
                       unsigned int & widthOut,
                       unsigned int & heightOut,
                       ArRef<MemoryBlock> dataOut);

  static
  bool // success
  saveRGBImageToJPEG(ArConstRef<MemoryBlock> dataIn,
                     unsigned int width,
                     unsigned int height,
                     unsigned int quality, // [0-100]
                     ArRef<MemoryBlock> dataOut);

//-------- Forbidden operations ----------------------------------------------
private:

  LLW(void); // no implementation

  LLW(const LLW &); // no implementation

  LLW & // itself
  operator=(const LLW &); // no implementation

  virtual
  ~LLW(void); // no implementation

//-------- Static attributes -------------------------------------------------
private:

  friend bool ImageLoader_init(const StlVector<StlString> *);
  friend void ImageLoader_destroy(void);

  typedef bool (* ImageLoaderFnct)(const char *,
                                   unsigned int &,
                                   unsigned int &,
                                   unsigned int &,
                                   ArRef<MemoryBlock>);

  typedef StlVector<LLW::ImageLoaderFnct> ImageLoaderFnctVect;

  static LLW::ImageLoaderFnctVect * _imageLoaderFncts;

  typedef bool (* ImageSaverFnct)(const char *,
                                  unsigned int,
                                  unsigned int,
                                  unsigned int,
                                  const char *);

  typedef StlVector<LLW::ImageSaverFnct> ImageSaverFnctVect;

  static LLW::ImageSaverFnctVect * _imageSaverFncts;

  // JpegLib plugin
  friend bool JpegLib_init(const StlVector<StlString> *);
  friend void JpegLib_destroy(void);

  static bool _initJpegLibSuccess;
  static bool (* _compressRGBImageToJPEG)(ArConstRef<MemoryBlock>,
                                          ArRef<MemoryBlock> ,
                                          unsigned int ,
                                          unsigned int ,
                                          unsigned int );
  static bool (* _uncompressRGBImageFromJPEG)(ArConstRef<MemoryBlock>,
                                              ArRef<MemoryBlock>,
                                              unsigned int &,
                                              unsigned int &);
};

} // namespace AReVi

#endif // AREVI_LIB3D_LLW_PRIV_H

//----------------------------------------------------------------------------
