// LI2/ENIB - AReVi - 3D services
//----------------------------------------------------------------------------
// file        : AReVi/Lib3D/material3D.h
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVI_LIB3D_MATERIAL3D_H
#define AREVI_LIB3D_MATERIAL3D_H 1

#include "AReVi/arObject.h"

#include "AReVi/Lib3D/util3D_priv.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class Material3D
//----------------------------------------------------------------------------

class Material3D : public ArObject
{
//-------- Type information --------------------------------------------------
public:

  AR_CLASS(Material3D)

//-------- Construction / Destruction ----------------------------------------

  AR_CONSTRUCTOR(Material3D)

//-------- Material ----------------------------------------------------------

  virtual
  void
  setLighting(bool lighting);

  virtual
  bool // lighting is on
  getLighting(void) const;

  virtual
  void
  setTransparency(double transparency);

  virtual
  double
  getTransparency(void) const;

  virtual
  void
  setShininess(double shininess);

  virtual
  double
  getShininess(void) const;

  virtual
  void
  setAmbientIntensity(double ambientIntensity);

  virtual
  double
  getAmbientIntensity(void) const;

  virtual
  void
  setDiffuseColor(double r,
                  double g,
                  double b);

  virtual
  void
  getDiffuseColor(double & rOut,
                  double & gOut,
                  double & bOut) const;

  virtual
  void
  setEmissiveColor(double r,
                   double g,
                   double b);

  virtual
  void
  getEmissiveColor(double & rOut,
                   double & gOut,
                   double & bOut) const;

  virtual
  void
  setSpecularColor(double r,
                   double g,
                   double b);

  virtual
  void
  getSpecularColor(double & rOut,
                   double & gOut,
                   double & bOut) const;

//-------- Stream operations -------------------------------------------------

  virtual
  void
  writeToStream(ArRef<AbstractOStream> output) const;

  virtual
  bool // success
  readFromStream(ArRef<AbstractIStream> input);

//-------- Data access (internal) --------------------------------------------

  AR_UNSTUBBED inline virtual
  const Util3D::Material &
  getMaterial(void) const;

  AR_UNSTUBBED inline virtual
  Util3D::Material &
  accessMaterial(void);

//-------- Member attributes -------------------------------------------------
protected:

  Util3D::Material _material;
};

//-------- Inline code -------------------------------------------------------

inline
const Util3D::Material &
Material3D::getMaterial(void) const
{
return(_material);
}

inline
Util3D::Material &
Material3D::accessMaterial(void)
{
return(_material);
}

} // namespace AReVi

#endif // AREVI_LIB3D_MATERIAL3D_H

//----------------------------------------------------------------------------
