// LI2/ENIB - AReVi - 3D services
//----------------------------------------------------------------------------
// file        : AReVi/Lib3D/osdSprite.h
// author(s)   : T. Jourdan
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVI_LIB3D_OSDSPRITE_H
#define AREVI_LIB3D_OSDSPRITE_H 1

#include "AReVi/Lib3D/osd.h"

namespace AReVi {

class Texture;

//----------------------------------------------------------------------------
// Class OSDSprite
//----------------------------------------------------------------------------

class OSDSprite : public OSD
{
//-------- Type information --------------------------------------------------
public:

  AR_CLASS(OSDSprite)

//-------- Construction / Destruction ----------------------------------------

  AR_CONSTRUCTOR_1(OSDSprite,
		   ArConstRef<Texture>, texture)

//-------- Aspect ------------------------------------------------------------

  virtual
  void
  resizeWithRenderer(bool resize);

  virtual
  bool // resized with renderer
  isResizedWithRenderer(void) const;

  virtual
  void
  setRatio(double ratio);

  virtual
  double // > 0.0 => use ratio, else no ratio
  getRatio(void) const;

  /* if resizeWithRenderer = false => no effect
     else
       width  = renderer width * x
       heigth = renderer height * y
     corrected with ratio if specified
   */
  virtual
  void
  setScale(double x,
           double y);

  virtual
  void
  getScale(double & xOut,
           double & yOut) const;

  /* if resizeWithRenderer = true => no effect
     else set width and height in pixel according to ratio
   */
  virtual
  void
  setSize(unsigned int width,
          unsigned int height);

  // renderer can be nullRef if resizeWithRenderer = false
  virtual
  bool
  getSize(ArRef<Renderer3D> renderer,
	  unsigned int & widthOut,
	  unsigned int & heightOut);

  virtual
  ArConstRef<Texture>
  getTexture(void) const;

  virtual
  void
  setTexture(ArConstRef<Texture> texture);

  virtual
  void
  setColorFilter(double r,
                 double g,
                 double b);

  virtual
  void
  getColorFilter(double & rOut,
                 double & gOut,
                 double & bOut) const;

//-------- Serialization operations ------------------------------------------

  AR_UNSTUBBED virtual
  void
  extractDependencies(SerializationDependencies & dependenciesOut,
                      bool recursive=true) const;

  AR_UNSTUBBED virtual
  void
  serialize(SerializationDependencies & dependenciesOut,
            ArRef<AbstractOStream> output) const;

  AR_UNSTUBBED virtual
  bool // success
  unserialize(const SerializationDependencies & dependencies,
              ArRef<AbstractIStream> input);

//-------- Forbidden operations ----------------------------------------------
protected:

  OSDSprite(ArCW & arCW); // only for serialisation

  virtual
  void
  _getSize(unsigned int rendererWidth,
	   unsigned int rendererHeight,
	   unsigned int & widthOut,
	   unsigned int & heightOut) const;

  virtual
  void
  _render(unsigned int width,
          unsigned int height) const;

//-------- Member attributes -------------------------------------------------
protected:

  bool _resize;
  double _ratio;
  Util3D::Dbl2 _scale;
  Util3D::Dbl3 _color; // filter texture color
  size_t _w,_h;
  ArConstRef<Texture> _texture;
};

} // namespace AReVi

#endif // AREVI_LIB3D_OSDSPRITE_H

//----------------------------------------------------------------------------
