// LI2/ENIB - AReVi - 3D services
//----------------------------------------------------------------------------
// file        : AReVi/Lib3D/osdText.h
// author(s)   : T. Jourdan
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVI_LIB3D_OSDTEXT_H
#define AREVI_LIB3D_OSDTEXT_H 1

#include "AReVi/Lib3D/osd.h"

#include "AReVi/Lib3D/llw_priv.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class OSDSprite
//----------------------------------------------------------------------------

class OSDText : public OSD
{
//-------- Type information --------------------------------------------------
public:

  AR_CLASS(OSDText)

//-------- Construction / Destruction ----------------------------------------

  AR_CONSTRUCTOR(OSDText)

//-------- Aspect ------------------------------------------------------------

  enum Justification { JUSTIF_LEFT,
                       JUSTIF_CENTER,
                       JUSTIF_RIGHT };

  virtual
  bool // success
  setFont(const StlString & fontName);

  virtual
  const StlString &
  getFont(void) const;

  virtual
  void
  setJustification(OSDText::Justification justification);

  virtual
  OSDText::Justification
  getJustification(void) const;

  virtual
  void
  setText(const StlString & text);

  virtual
  StlString
  getText(void) const;

  virtual
  void
  setColor(double r,
           double g,
           double b,
           double a);

  virtual
  void
  getColor(double & rOut,
           double & gOut,
           double & bOut,
           double & aOut) const;

  virtual
  void
  setOrientation(double orientation);

//-------- Event management --------------------------------------------------

  AR_CALLBACK(OSDText,TextChanged,_textChangedCB,
              /* no field */)

//-------- Serialization operations ------------------------------------------

  AR_UNSTUBBED virtual
  void
  serialize(SerializationDependencies & dependenciesOut,
            ArRef<AbstractOStream> output) const;

  AR_UNSTUBBED virtual
  bool // success
  unserialize(const SerializationDependencies & dependencies,
              ArRef<AbstractIStream> input);

//-------- Forbidden operations ----------------------------------------------
protected:

  virtual
  LLW::BMFontID
  _loadFont(const StlString & fontName);

  virtual
  void
  _getSize(unsigned int rendererWidth,
	   unsigned int rendererHeight,
	   unsigned int & widthOut,
	   unsigned int & heightOut) const;

  virtual
  void
  _render(unsigned int width,
          unsigned int height) const;

  virtual
  void
  _onTextChanged(void);

//-------- Member attributes -------------------------------------------------
protected :

  StlString _fontName;
  LLW::BMFontID _curFont;
  StlVector<StlString> _text;
  Util3D::Dbl4 _color;
  OSDText::Justification _justification;
  StlVector<unsigned int> _lineWidth;
  CallbackManager<OSDText,TextChangedEvent> _textChangedCB;

//-------- Static attributes -------------------------------------------------
protected:

  static StlVector<StlString> _fontDir;
  static StlMap<StlString,LLW::BMFontID> _font;
};

} // namespace AReVi

#endif // AREVI_LIB3D_OSDTEXT_H

//----------------------------------------------------------------------------
