// LI2/ENIB - AReVi - 3D services
//----------------------------------------------------------------------------
// file        : AReVi/Lib3D/renderTexture.h
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVI_LIB3D_RENDERTEXTURE_H
#define AREVI_LIB3D_RENDERTEXTURE_H 1

#include "AReVi/Lib3D/texture.h"

namespace AReVi {

class TextureRenderer3D;
class TextureCubeRenderer3D;

//----------------------------------------------------------------------------
// Class RenderTexture
//----------------------------------------------------------------------------

class RenderTexture : public Texture
{
//-------- Type information --------------------------------------------------
public:

  AR_CLASS(RenderTexture)

//-------- Renderer management -----------------------------------------------

  virtual
  ArConstRef<TextureRenderer3D> // texture source
  getRenderer(void) const;

  virtual
  ArRef<TextureRenderer3D> // texture source
  accessRenderer(void);

//-------- Texture changes ---------------------------------------------------

  virtual
  void
  setBlending(bool needsBlending);

  virtual
  void
  setAlphaTest(bool needsAlphaTest);

//-------- Serialization operations ------------------------------------------

  AR_UNSTUBBED virtual
  void
  extractDependencies(SerializationDependencies & dependenciesOut,
                      bool recursive=true) const;

  AR_UNSTUBBED virtual
  void
  serialize(SerializationDependencies & dependenciesOut,
            ArRef<AbstractOStream> output) const;

  AR_UNSTUBBED virtual
  bool // success
  unserialize(const SerializationDependencies & dependencies,
              ArRef<AbstractIStream> input);

//-------- Event management --------------------------------------------------

  AR_CALLBACK(RenderTexture,Setup,_setupCB,
              /* no field */) // on set Alpha/Blend

//-------- Forbidden operations ----------------------------------------------
protected:

  friend class TextureRenderer3D;
  friend class TextureCubeRenderer3D;

  RenderTexture(ArCW & arCW); // only for serialisation

  RenderTexture(ArCW & arCW,
                ArRef<TextureRenderer3D> renderer,
                bool useCubeMap);

  virtual
  void
  _onSetup(void);

//-------- Member attributes -------------------------------------------------
protected:

  ArRef<TextureRenderer3D> _renderer;
  CallbackManager<RenderTexture,SetupEvent> _setupCB;
};

} // namespace AReVi

#endif // AREVI_LIB3D_RENDERTEXTURE_H

//----------------------------------------------------------------------------
