// LI2/ENIB - AReVi - 3D services
//----------------------------------------------------------------------------
// file        : AReVi/Lib3D/scene3D.h
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVI_LIB3D_SCENE3D_H
#define AREVI_LIB3D_SCENE3D_H 1

#include "AReVi/arObject.h"

#include "AReVi/Lib3D/util3D_priv.h"

namespace AReVi {

class Base3D;
class Object3D;
class ShapePart3D;
class Light3D;
class ParticleSystem;
class Sound3D;
class BoundingBox3D;
class Transform2D;
class Texture;
class Renderer3D;
class Listener3D;

//----------------------------------------------------------------------------
// Class Scene3D
//----------------------------------------------------------------------------

class Scene3D : public ArObject
{
//-------- Type information --------------------------------------------------
public:

  AR_CLASS(Scene3D)

//-------- Construction / Destruction ----------------------------------------

  AR_CONSTRUCTOR(Scene3D)

//-------- Global scene ------------------------------------------------------

  static
  ArConstRef<Scene3D> // global scene
  getGlobalScene(void);

  static
  ArRef<Scene3D> // global scene
  accessGlobalScene(void);

//-------- Object3D management -----------------------------------------------

  virtual
  void
  addObject(ArRef<Object3D> obj);

  virtual
  void
  removeObject(ArRef<Object3D> obj);

  virtual
  bool // object is contained
  hasObject(ArConstRef<Object3D> obj) const;

  virtual
  unsigned int // number of objects contained
  getNbObjects(void) const;

  virtual
  ArConstRef<Object3D> // index'th object contained
  getObject(unsigned int index) const;

  virtual
  ArRef<Object3D> // index'th object contained
  accessObject(unsigned int index);

//-------- Light3D management ------------------------------------------------

  virtual
  void
  addLight(ArRef<Light3D> light);

  virtual
  void
  removeLight(ArRef<Light3D> light);

  virtual
  bool // light is contained
  hasLight(ArConstRef<Light3D> light) const;

  virtual
  unsigned int // number of lights contained
  getNbLights(void) const;

  virtual
  ArConstRef<Light3D> // index'th light contained
  getLight(unsigned int index) const;

  virtual
  ArRef<Light3D> // index'th light contained
  accessLight(unsigned int index);

//-------- ParticleSystem management -----------------------------------------

  virtual
  void
  addParticleSystem(ArRef<ParticleSystem> particleSystem);

  virtual
  void
  removeParticleSystem(ArRef<ParticleSystem> particleSystem);

  virtual
  bool // particle system is contained
  hasParticleSystem(ArConstRef<ParticleSystem> particleSystem) const;

  virtual
  unsigned int // number of particle systems contained
  getNbParticleSystems(void) const;

  virtual
  ArConstRef<ParticleSystem> // index'th particle system contained
  getParticleSystem(unsigned int index) const;

  virtual
  ArRef<ParticleSystem> // index'th particle system contained
  accessParticleSystem(unsigned int index);

//-------- Sound3D management ------------------------------------------------

  virtual
  void
  addSound(ArRef<Sound3D> sound);

  virtual
  void
  removeSound(ArRef<Sound3D> sound);

  virtual
  bool // sound is contained
  hasSound(ArConstRef<Sound3D> sound) const;

  virtual
  unsigned int // number of sounds contained
  getNbSounds(void) const;

  virtual
  ArConstRef<Sound3D> // index'th sound contained
  getSound(unsigned int index) const;

  virtual
  ArRef<Sound3D> // index'th sound contained
  accessSound(unsigned int index);

//-------- Selection management ----------------------------------------------

  virtual
  void
  selectObject(ArRef<Object3D> obj);

  virtual
  void
  unselectObject(ArRef<Object3D> obj);

  virtual
  bool // object is selected
  isSelected(ArConstRef<Object3D> obj) const;

  virtual
  unsigned int // number of objects selected
  getSelectionSize(void) const;

  virtual
  ArConstRef<Object3D> // index'th object selected
  getSelectedObject(unsigned int index) const;

  virtual
  ArRef<Object3D> // index'th object selected
  accessSelectedObject(unsigned int index);

//-------- Fog management ----------------------------------------------------

  virtual
  void
  setFogColor(double red,
              double green,
              double blue);

  virtual
  void
  getFogColor(double & redOut,
              double & greenOut,
              double & blueOut) const;

  virtual
  void
  setFog(unsigned int degree,
         double density);

  virtual
  void
  getFog(unsigned int & degreeOut,
         double & densityOut) const;

//-------- Background management ---------------------------------------------

  virtual
  void
  setBackground(ArRef<Texture> texture);

  virtual
  ArConstRef<Texture> // background texture
  getBackground(void) const;

  virtual
  ArRef<Texture> // background texture
  accessBackground(void);

  virtual
  void
  writeBackgroundTransformation(ArConstRef<Transform2D> transform);

  virtual
  void
  readBackgroundTransformation(ArRef<Transform2D> transformInOut) const;

//-------- Detection ---------------------------------------------------------

  virtual
  void
  computeBoundingBox(ArRef<BoundingBox3D> bboxOut,
                     ArConstRef<Base3D> viewPoint) const;

  virtual
  void
  getAll(ArConstRef<ArClass> objectClass,
         StlVector<ArRef<Object3D> > & objectsOut) const;

  virtual
  void
  viewFirst(ArConstRef<Base3D> origin,
            double yawVision,
            double pitchVision,
            double range,
            ArConstRef<ArClass> objectClass,
            ArRef<Object3D> & objectOut,
            double & yawAngleOut,
            double & pitchAngleOut,
            double & distanceOut) const;

  virtual
  void
  view(ArConstRef<Base3D> origin,
       double yawVision,
       double pitchVision,
       double range,
       ArConstRef<ArClass> objectClass,
       StlVector<ArRef<Object3D> > & objectsOut,
       StlVector<double> & yawAnglesOut,
       StlVector<double> & pitchAnglesOut,
       StlVector<double> & distancesOut) const;

  virtual
  void
  firstPicking(ArConstRef<Renderer3D> viewPoint,
               int xMouse,
               int yMouse,
               unsigned int level,
               bool findNearest,
               ArConstRef<ArClass> objectClass,
               ArRef<Object3D> & objectOut,
               ArRef<ShapePart3D> & interPartOut,
               unsigned int & triIndexOut,
               double & distanceOut) const;

  virtual
  void
  firstRayIntersection(ArConstRef<Base3D> ray,
                       unsigned int level,
                       bool findNearest,
                       ArConstRef<ArClass> objectClass,
                       ArRef<Object3D> & objectOut,
                       ArRef<ShapePart3D> & interPartOut,
                       unsigned int & triIndexOut,
                       double & distanceOut,
                       ArConstRef<Base3D> viewPoint) const;

  virtual
  void
  firstSimpleRayIntersection(double sourceX,
                             double sourceY,
                             double sourceZ,
                             double unitDirX,
                             double unitDirY,
                             double unitDirZ,
                             unsigned int level,
                             bool findNearest,
                             ArConstRef<ArClass> objectClass,
                             ArRef<Object3D> & objectOut,
                             ArRef<ShapePart3D> & interPartOut,
                             unsigned int & triIndexOut,
                             double & distanceOut) const;

  virtual
  void
  picking(ArConstRef<Renderer3D> viewPoint,
          int xMouse,
          int yMouse,
          unsigned int level,
          bool findNearest,
          ArConstRef<ArClass> objectClass,
          StlVector<ArRef<Object3D> > & objectsOut,
          StlVector<ArRef<ShapePart3D> > & interPartsOut,
          StlVector<unsigned int> & triIndicesOut,
          StlVector<double> & distancesOut) const;

  virtual
  void
  rayIntersection(ArConstRef<Base3D> ray,
                  unsigned int level,
                  bool findNearest,
                  ArConstRef<ArClass> objectClass,
                  StlVector<ArRef<Object3D> > & objectsOut,
                  StlVector<ArRef<ShapePart3D> > & interPartsOut,
                  StlVector<unsigned int> & triIndicesOut,
                  StlVector<double> & distancesOut,
                  ArConstRef<Base3D> viewPoint) const;

  virtual
  void
  simpleRayIntersection(double sourceX,
                        double sourceY,
                        double sourceZ,
                        double unitDirX,
                        double unitDirY,
                        double unitDirZ,
                        unsigned int level,
                        bool findNearest,
                        ArConstRef<ArClass> objectClass,
                        StlVector<ArRef<Object3D> > & objectsOut,
                        StlVector<ArRef<ShapePart3D> > & interPartsOut,
                        StlVector<unsigned int> & triIndicesOut,
                        StlVector<double> & distancesOut) const;

  virtual
  bool // collision occured
  testCollision(ArConstRef<Object3D> object,
                unsigned int level,
                ArConstRef<ArClass> objectClass,
                StlVector<ArRef<Object3D> > & objectsOut,
                ArConstRef<Base3D> viewPoint) const;

  AR_UNSTUBBED virtual
  bool // collision occured
  intersection(ArConstRef<Object3D> object,
               ArConstRef<ArClass> objectClass,
               StlVector<ArRef<Object3D> > & objectsOut,
               StlVector<StlVector<Util3D::TriInter> > & itOut,
               ArConstRef<Base3D> viewPoint) const;

//-------- Serialization operations ------------------------------------------

  AR_UNSTUBBED virtual
  void
  extractDependencies(SerializationDependencies & dependenciesOut,
                      bool recursive=true) const;

  AR_UNSTUBBED virtual
  void
  serialize(SerializationDependencies & dependenciesOut,
            ArRef<AbstractOStream> output) const;

  AR_UNSTUBBED virtual
  bool // success
  unserialize(const SerializationDependencies & dependencies,
              ArRef<AbstractIStream> input);

//-------- Event management --------------------------------------------------

  AR_CALLBACK(Scene3D,AddObject,_addObjectCB,
              ArPtr<Object3D> object;)

  AR_CALLBACK(Scene3D,RemoveObject,_removeObjectCB,
              ArPtr<Object3D> object;)

  AR_CALLBACK(Scene3D,AddLight,_addLightCB,
              ArPtr<Light3D> light;)

  AR_CALLBACK(Scene3D,RemoveLight,_removeLightCB,
              ArPtr<Light3D> light;)

  AR_CALLBACK(Scene3D,AddParticleSystem,_addParticleSystemCB,
              ArPtr<ParticleSystem> particleSystem;)

  AR_CALLBACK(Scene3D,RemoveParticleSystem,_removeParticleSystemCB,
              ArPtr<ParticleSystem> particleSystem;)

  AR_CALLBACK(Scene3D,AddSound,_addSoundCB,
              ArPtr<Sound3D> sound;)

  AR_CALLBACK(Scene3D,RemoveSound,_removeSoundCB,
              ArPtr<Sound3D> sound;)

  AR_CALLBACK(Scene3D,SceneSetup,_sceneSetupCB,
              /* no field */) // Fog,Bkgnd(text&transform)

  AR_CALLBACK(Scene3D,SelectObject,_selectObjectCB,
              ArPtr<Object3D> object;)

  AR_CALLBACK(Scene3D,UnselectObject,_unselectObjectCB,
              ArPtr<Object3D> object;)

//-------- Forbidden operations ----------------------------------------------
protected:

  friend class ArSystem;
  friend class Renderer3D;
  friend class Listener3D;
  friend class Object3D;
  friend class Light3D;
  friend class Texture;
  friend class ParticleSystem;
  friend class Sound3D;

  static
  void
  _init(void);

  static
  void
  _destroy(void);

  virtual
  void
  _rebuildShapesIfNeeded(void);

  virtual
  void
  _removeObject(ArRef<Object3D> obj);

  virtual
  void
  _removeLight(ArRef<Light3D> light);

  virtual
  void
  _removeParticleSystem(ArRef<ParticleSystem> particleSystem);

  virtual
  void
  _removeSound(ArRef<Sound3D> sound);

  virtual
  void
  _addRenderer(ArPtr<Renderer3D> renderer);

  virtual
  void
  _removeRenderer(ArPtr<Renderer3D> renderer);

  virtual
  void
  _addListener(ArPtr<Listener3D> listener);

  virtual
  void
  _removeListener(ArPtr<Listener3D> listener);

  virtual
  void
  _postRedisplay(void);

  virtual
  void
  _askVisibleParts(Renderer3D * renderer) const;

  virtual
  void
  _onSceneSetup(void);

//-------- Member attributes -------------------------------------------------
protected:

  StlVector<ArPtr<Object3D> > _objects;
  StlVector<ArPtr<Light3D> > _lights;
  StlVector<ArPtr<ParticleSystem> > _particleSystems;
  StlVector<ArPtr<Sound3D> > _sounds;
  StlVector<ArPtr<Renderer3D> > _renderers;
  StlVector<ArPtr<Listener3D> > _listeners;
  StlVector<ArPtr<Object3D> > _selection;

  Util3D::Dbl3 _fogColor;
  unsigned int _fogDegree;
  double _fogDensity;
  ArRef<Texture> _backTexture;
  Util3D::Transform _backTransform;
  CallbackManager<Scene3D,AddObjectEvent> _addObjectCB;
  CallbackManager<Scene3D,RemoveObjectEvent> _removeObjectCB;
  CallbackManager<Scene3D,AddLightEvent> _addLightCB;
  CallbackManager<Scene3D,RemoveLightEvent> _removeLightCB;
  CallbackManager<Scene3D,AddParticleSystemEvent> _addParticleSystemCB;
  CallbackManager<Scene3D,RemoveParticleSystemEvent> _removeParticleSystemCB;
  CallbackManager<Scene3D,AddSoundEvent> _addSoundCB;
  CallbackManager<Scene3D,RemoveSoundEvent> _removeSoundCB;
  CallbackManager<Scene3D,SceneSetupEvent> _sceneSetupCB;
  CallbackManager<Scene3D,SelectObjectEvent> _selectObjectCB;
  CallbackManager<Scene3D,UnselectObjectEvent> _unselectObjectCB;

//-------- Static attributes -------------------------------------------------
protected:

  static ArRef<Scene3D> _globalScene;
};

} // namespace AReVi

#endif // AREVI_LIB3D_SCENE3D_H

//----------------------------------------------------------------------------
