// LI2/ENIB - AReVi - 3D services
//----------------------------------------------------------------------------
// file        : AReVi/Lib3D/simpleInteractor.h
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVI_LIB3D_SIMPLEINTERACTOR_H
#define AREVI_LIB3D_SIMPLEINTERACTOR_H 1

#include "AReVi/Lib3D/abstractInteractor.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class SimpleInteractor
//----------------------------------------------------------------------------

class SimpleInteractor : public AbstractInteractor
{
//-------- Type information --------------------------------------------------
public:

  AR_CLASS(SimpleInteractor)

//-------- Construction / Destruction ----------------------------------------

  AR_CONSTRUCTOR(SimpleInteractor)

//-------- Interaction with objects ------------------------------------------

  virtual
  void
  setObjectInteraction(bool useObjectInteraction);

  virtual
  bool // object interaction is used
  getObjectInteraction(void) const;

  virtual
  void
  setMainSceneRestriction(bool restrictToMainScene);

  virtual
  bool // object interaction is restricted to main scene
  getMainSceneRestriction(void) const;

//-------- Interaction with osd ------------------------------------------

  virtual
  void
  setOSDInteraction(bool useOSDInteraction);

  virtual
  bool // osd interaction is used
  getOSDInteraction(void) const;

//-------- Usual shortcuts ---------------------------------------------------

  virtual
  void
  setShortcuts(bool useShortcuts);

  virtual
  bool // keyboard shortcuts are used
  getShortcuts(void) const;

//-------- Scene examination -------------------------------------------------

  virtual
  void
  setExamination(bool useExamination);

  virtual
  bool // examination is used
  getExamination(void) const;

//-------- Scene navigation --------------------------------------------------

  virtual
  void
  setNavigation(bool useNavigation);

  virtual
  bool // navigation is used
  getNavigation(void) const;

  virtual
  void
  setLinearSpeed(double linearSpeed);

  virtual
  double // half-screen linear speed in 1/s
  getLinearSpeed(void) const;

  virtual
  void
  setAngularSpeed(double angularSpeed);

  virtual
  double // half-screen angular speed in rad/s
  getAngularSpeed(void) const;

//------- Forbidden operations -----------------------------------------------
protected:

  virtual
  Util3D::Base & // current location
  _setupLocation(ArRef<Base3D> base);

  virtual
  void
  _chooseCursor(void);

  virtual
  void
  _clearTimeOut(void);

  virtual
  void
  _clearInteraction(void);

  virtual
  void
  _onMouseButton(const Renderer3D::MouseButtonEvent & evt);

  virtual
  void
  _onMouseMotion(const Renderer3D::MouseMotionEvent & evt);

  virtual
  void
  _onKeyboard(const Renderer3D::KeyboardEvent & evt);

  virtual
  void
  _onTimeOut(const Renderer3D::TimeOutEvent & evt);

//-------- Member attributes -------------------------------------------------
protected:

  bool _useObjectInteraction;
  bool _useOSDInteraction;
  bool _useShortcuts;
  bool _useExamination;
  bool _useNavigation;
  int _mouseX,_mouseY,_mouseBtn;
  Util3D::Dbl3 _localPoint;
  bool _restrictToMainScene;
  bool _canRotate;
  unsigned int _timeOut;
  unsigned int _lastTime;
  ArRef<Base3D> _newLocation;
  int _newX, _newY;
  double _lastXc,_lastDx,_lastDy,_lastDz,_lastAngle;
  double _linearSpeed;
  double _angularSpeed;
  int _clickX,_clickY;
  bool _pendingInteraction;
  ArRef<Object3D> _objInteracted;
  ArRef<ShapePart3D> _partInteracted;
  unsigned int _triInteracted;
  ArRef<OSD> _osdInteracted;
  bool _pendingExamination;
  bool _pendingNavigation;
};

} // namespace AReVi

#endif // AREVI_LIB3D_SIMPLEINTERACTOR_H

//----------------------------------------------------------------------------
