// LI2/ENIB - AReVi - 3D services
//----------------------------------------------------------------------------
// file        : AReVi/Lib3D/tesselator.h
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVI_LIB3D_TESSELATOR_H
#define AREVI_LIB3D_TESSELATOR_H 1

#include "AReVi/arObject.h"

#include "AReVi/Lib3D/util3D_priv.h"
#include "AReVi/Lib3D/gl_priv.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class Tesselator
//----------------------------------------------------------------------------

class Tesselator : public ArObject
{
//-------- Type information --------------------------------------------------
public:

  AR_UNSTUBBED AR_CLASS(Tesselator)

//-------- Construction / Destruction ----------------------------------------

  AR_CONSTRUCTOR(Tesselator)

//-------- Tesselation -------------------------------------------------------

  virtual
  bool // success
  tesselation3D(const StlVector<Util3D::Dbl3> & vertices,
                const StlVector<int> & contour,
                StlVector<Util3D::Face> & trianglesOut);

  virtual
  bool // success
  tesselation2D(const StlVector<Util3D::Dbl2> & vertices,
                const StlVector<int> & contour,
                StlVector<Util3D::Face> & trianglesOut);

//-------- Forbidden operations ----------------------------------------------
protected:

  void
  _init(void);

  static
  void
  _edgeFlagCB(GLboolean flag,
              Tesselator * tesselator);

  static
  void
  _errorCB(GLenum errorCode,
           Tesselator * tesselator);

  static
  void
  _beginCB(GLenum primitive,
           Tesselator * tesselator);

  static
  void
  _vertexCB(unsigned int index,
            Tesselator * tesselator);

  static
  void
  _endCB(Tesselator * tesselator);

//-------- Member attributes -------------------------------------------------
protected:

  bool _fail;
  GLUtesselator * _tess;
  unsigned int _vertexCount;
  Util3D::Face _currentFace;
  StlVector<Util3D::Face> * _faces;
};

} // namespace AReVi

#endif // AREVI_LIB3D_TESSELATOR_H

//----------------------------------------------------------------------------
