// LI2/ENIB - AReVi - 3D services
//----------------------------------------------------------------------------
// file        : AReVi/Lib3D/textureRenderer3D.h
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVI_LIB3D_TEXTURERENDERER3D_H
#define AREVI_LIB3D_TEXTURERENDERER3D_H 1

#include "AReVi/Lib3D/renderer3D.h"

namespace AReVi {

class RenderTexture;

//----------------------------------------------------------------------------
// Class TextureRenderer3D
//----------------------------------------------------------------------------

class TextureRenderer3D : public Renderer3D
{
//-------- Type information --------------------------------------------------
public:

  AR_CLASS(TextureRenderer3D)

//-------- Construction / Destruction ----------------------------------------

  AR_CONSTRUCTOR(TextureRenderer3D)

//-------- Rendering ---------------------------------------------------------

  virtual
  void
  setRenderSize(unsigned int width,
                unsigned int height);

  virtual
  void
  setAutoRender(bool autoRender);

  virtual
  bool // texture is generated automatically
  getAutoRender(void) const;

  virtual
  void
  render(void);

  virtual
  ArRef<RenderTexture> // created texture
  buildTexture(void);

//-------- Serialization operations ------------------------------------------

  AR_UNSTUBBED virtual
  void
  extractDependencies(SerializationDependencies & dependenciesOut,
                      bool recursive=true) const;

  AR_UNSTUBBED virtual
  void
  serialize(SerializationDependencies & dependenciesOut,
            ArRef<AbstractOStream> output) const;

  AR_UNSTUBBED virtual
  bool // success
  unserialize(const SerializationDependencies & dependencies,
              ArRef<AbstractIStream> input);

//-------- Event management --------------------------------------------------
  // only on non autoRender mode
  AR_CALLBACK(TextureRenderer3D,Render,_renderCB,
              /* no field */)

//-------- Forbidden operations ----------------------------------------------
protected:

  friend class RenderTexture;

  virtual
  void
  _render(void);

//-------- Member attributes -------------------------------------------------
protected:

  ArPtr<RenderTexture> _texture;
  bool _autoRender;

  CallbackManager<TextureRenderer3D,RenderEvent> _renderCB;
};

} // namespace AReVi

#endif // AREVI_LIB3D_TEXTURERENDERER3D_H

//----------------------------------------------------------------------------
