// LI2/ENIB - AReVi - 3D services
//----------------------------------------------------------------------------
// file        : AReVi/Lib3D/transform2D.h
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVI_LIB3D_TRANSFORM2D_H
#define AREVI_LIB3D_TRANSFORM2D_H 1

#include "AReVi/arObject.h"

#include "AReVi/Lib3D/util3D_priv.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class Transform2D
//----------------------------------------------------------------------------

class Transform2D : public ArObject
{
//-------- Type information --------------------------------------------------
public:

  AR_CLASS(Transform2D)

//-------- Construction / Destruction ----------------------------------------

  AR_CONSTRUCTOR(Transform2D)

//-------- Transformations ---------------------------------------------------

  virtual
  void
  invert(void);

  virtual
  void
  apply(double & xOut,
        double & yOut,
        double xIn,
        double yIn) const;

  virtual
  void
  identity(void);

  virtual
  bool // transformation has no effect
  isIdentity(bool checkCoefs) const;

  virtual
  void
  preTransform(ArConstRef<Transform2D> tr);

  virtual
  void
  postTransform(ArConstRef<Transform2D> tr);

  virtual
  void
  preTranslate(double tx,
               double ty);

  virtual
  void
  postTranslate(double tx,
                double ty);

  virtual
  void
  preRotate(double angle);

  virtual
  void
  postRotate(double angle);

  virtual
  void
  preScale(double sx,
           double sy);

  virtual
  void
  postScale(double sx,
            double sy);

//-------- Data access (internal) --------------------------------------------

  AR_UNSTUBBED inline virtual
  const Util3D::Transform &
  getTransformation(void) const;

  AR_UNSTUBBED inline virtual
  Util3D::Transform &
  accessTransformation(void);

//-------- Member attributes -------------------------------------------------
protected:

  Util3D::Transform _transform;
};

//-------- Inline code -------------------------------------------------------

inline
const Util3D::Transform &
Transform2D::getTransformation(void) const
{
return(_transform);
}

inline
Util3D::Transform &
Transform2D::accessTransformation(void)
{
return(_transform);
}

} // namespace AReVi

#endif // AREVI_LIB3D_TRANSFORM2D_H

//----------------------------------------------------------------------------
