// LI2/ENIB - AReVi - 3D services
//----------------------------------------------------------------------------
// file        : AReVi/Lib3D/urlSound3D.h
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVI_LIB3D_URLSOUND3D_H
#define AREVI_LIB3D_URLSOUND3D_H 1

#include "AReVi/Lib3D/sound3D.h"

namespace AReVi {
//----------------------------------------------------------------------------
// Class URLSound3D
//----------------------------------------------------------------------------

class URLSound3D : public Sound3D
{
//-------- Type information --------------------------------------------------
public:

  AR_CLASS(URLSound3D)

//-------- Construction / Destruction ----------------------------------------

  AR_CONSTRUCTOR_2(URLSound3D,
                   const StlString &,url,
                   bool ,useCache)

//-------- Sound properties --------------------------------------------------

  virtual
  const StlString & // loaded URL
  getURL(void) const;

//-------- Serialization operations ------------------------------------------

  AR_UNSTUBBED virtual
  void
  extractDependencies(SerializationDependencies & dependenciesOut,
                      bool recursive=true) const;

  AR_UNSTUBBED virtual
  void
  serialize(SerializationDependencies & dependenciesOut,
            ArRef<AbstractOStream> output) const;

  AR_UNSTUBBED virtual
  bool // success
  unserialize(const SerializationDependencies & dependencies,
              ArRef<AbstractIStream> input);

//-------- Forbidden operations ----------------------------------------------
protected:

  friend bool Sound3DPlayer_init(const StlVector<StlString> *);
  friend void Sound3DPlayer_destroy(void);

  URLSound3D(ArCW & arCW); // only for serialisation

  static
  void
  _init(void);

  static
  void
  _destroy(void);

  virtual
  bool // success
  _loadURL(const StlString & url,
           bool useCache);

  static
  StlVector<Sound3D::SoundData> * // NULL if fail
  _loadBuffers(const StlString & url,
               bool useCache);

//-------- Member attributes -------------------------------------------------
protected:

  StlString _url;
  bool _useCache;

//-------- Static attributes -------------------------------------------------
protected:

  static StlMap<StlString,StlVector<Sound3D::SoundData> *> * _cache;
};

} // namespace AReVi

#endif // AREVI_LIB3D_URLSOUND3D_H

//----------------------------------------------------------------------------
