// LI2/ENIB - AReVi - 3D services
//----------------------------------------------------------------------------
// file        : AReVi/Lib3D/urlTexture.h
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVI_LIB3D_URLTEXTURE_H
#define AREVI_LIB3D_URLTEXTURE_H 1

#include "AReVi/Lib3D/texture.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class URLTexture
//----------------------------------------------------------------------------

class URLTexture : public Texture
{
//-------- Type information --------------------------------------------------
public:

  AR_CLASS(URLTexture)

//-------- Construction / Destruction ----------------------------------------

  AR_CONSTRUCTOR_3(URLTexture,
                   const StlString &, url,
                   bool, useCache,
                   bool, useMipmap);

  AR_CONSTRUCTOR_4(URLTexture,
                   const StlString &, url,
                   bool, useCache,
                   bool, useMipmap,
                   bool, useCubeMap);

//-------- Inspection --------------------------------------------------------

  virtual
  const StlString & // texture URL
  getURL(void) const;

  virtual
  bool // cache is used for same texture URLs
  usesCache(void) const;

//-------- Serialization operations ------------------------------------------

  AR_UNSTUBBED virtual
  void
  extractDependencies(SerializationDependencies & dependenciesOut,
                      bool recursive=true) const;

  AR_UNSTUBBED virtual
  void
  serialize(SerializationDependencies & dependenciesOut,
            ArRef<AbstractOStream> output) const;

  AR_UNSTUBBED virtual
  bool // success
  unserialize(const SerializationDependencies & dependencies,
              ArRef<AbstractIStream> input);

//-------- Forbidden operations ----------------------------------------------
protected:

  friend class ArSystem;

  URLTexture(ArCW & arCW); // only for serialisation

  static
  void
  _init(void);

  static
  void
  _destroy(void);

  virtual
  bool // success
  _loadURL(const StlString & url,
           bool useCache,
           bool useMipmap,
           bool useCubemap);

//-------- Member attributes -------------------------------------------------
protected:

  StlString _url;
  bool _useCache;

//-------- Static attributes -------------------------------------------------
protected:

  static StlMap<StlString,Texture::TexData *> * _cache;
};

} // namespace AReVi

#endif // AREVI_LIB3D_URLTEXTURE_H

//----------------------------------------------------------------------------
