// LI2/ENIB - AReVi - 3D services
//----------------------------------------------------------------------------
// file        : AReVi/Lib3D/viewer3D.h
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVI_LIB3D_VIEWER3D_H
#define AREVI_LIB3D_VIEWER3D_H 1

#include "AReVi/Lib3D/window3D.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class Viewer3D
//----------------------------------------------------------------------------

class Viewer3D : public Window3D
{
//-------- Type information --------------------------------------------------
public:

  AR_CLASS(Viewer3D)

//-------- Construction / Destruction ----------------------------------------

  AR_CONSTRUCTOR(Viewer3D)

//-------- Title -------------------------------------------------------------

  virtual
  void
  setTitle(const StlString & title);

  virtual
  const StlString & // window bar title
  getTitle(void) const;

//-------- Visibility --------------------------------------------------------

  virtual
  void
  setMapped(bool mapped);

  virtual
  bool // viewer is mapped
  isMapped(void) const;

  virtual
  bool // viewer has focus
  hasFocus(void) const;

//-------- Decoration --------------------------------------------------------

  virtual
  void
  setDecoration(bool decoration);

  virtual
  bool // viewer has decoration
  getDecoration(void) const;

//-------- Window geometry ---------------------------------------------------

  virtual
  void
  setWindowGeometry(int x,
                    int y,
                    unsigned int width,
                    unsigned int height);

  virtual
  void
  getWindowGeometry(int & xOut,
                    int & yOut,
                    unsigned int & widthOut,
                    unsigned int & heightOut) const;

  virtual
  void
  setWindowPosition(int x,
                    int y);

  virtual
  void
  getWindowPosition(int & xOut,
                    int & yOut) const;

  virtual
  void
  setWindowSize(unsigned int width,
                unsigned int height);

  virtual
  void
  getWindowSize(unsigned int & widthOut,
                unsigned int & heightOut) const;

  virtual
  void
  setRenderSize(unsigned int renderWidth,
                unsigned int renderHeight);

//-------- Window ID ---------------------------------------------------------

  virtual
  int // platform-specific window ID
  getWindowID(void) const;

//-------- Event management --------------------------------------------------

  AR_CALLBACK(Viewer3D,Focus,_focusCB,
              bool hasFocus;)

  AR_CALLBACK(Viewer3D,Appearance,_appearanceCB,
              /* no field */)

//-------- Forbidden operations ----------------------------------------------
protected:

  virtual
  void
  _toInnerSize(unsigned int & widthInOut,
               unsigned int & heightInOut) const;

  virtual
  void
  _toOuterSize(unsigned int & widthInOut,
               unsigned int & heightInOut) const;

  virtual
  void
  _selectColors(bool light);

  virtual
  void
  _drawBorderUp(int x,
                int y,
                unsigned int w,
                unsigned int h);

  virtual
  void
  _drawBorderDown(int x,
                  int y,
                  unsigned int w,
                  unsigned int h);

  virtual
  void
  _drawLineUp(int x,
              int y,
              unsigned int w,
              unsigned int h);

  virtual
  void
  _drawLineDown(int x,
                int y,
                unsigned int w,
                unsigned int h);

  virtual
  void
  _drawHLineUp(int x,
               int y,
               unsigned int w);

  virtual
  void
  _drawHLineDown(int x,
                 int y,
                 unsigned int w);

  virtual
  void
  _drawVLineUp(int x,
               int y,
               unsigned int h);

  virtual
  void
  _drawVLineDown(int x,
                 int y,
                 unsigned int h);

  virtual
  void
  _drawTitleBar(int x,
                int y,
                unsigned int w,
                unsigned int h);

  virtual
  void
  _drawCloseButton(int x,
                   int y);

  virtual
  void
  _drawMaxButton(int x,
                 int y,
                 bool pressed);

  virtual
  void
  _drawMinButton(int x,
                 int y);

  virtual
  void
  _redrawDecoration(void);

  enum WindowPart
    {
    PART_NONE=0,
    PART_CLOSE=1, PART_MIN=2, PART_MAX=3, PART_BAR=4, PART_INNER=5,
    PART_NW=6, PART_NE=7, PART_SW=8, PART_SE=9,
    PART_N=10, PART_S=11, PART_W=12, PART_E=13
    };

  virtual
  Viewer3D::WindowPart
  _findWindowPart(LLW::Modifier modifiers,
                  int x,
                  int y) const;

  virtual
  void
  _chooseCursor(LLW::Modifier modifiers,
                Viewer3D::WindowPart part);

  virtual
  void
  _manageClick(int x,
               int y);

  virtual
  void
  _manageMotion(int xScreen,
                int yScreen);

  virtual
  void
  _shade(void);

  virtual
  void
  _unshade(void);

  virtual
  bool // event processed
  _processEvent(const LLW::Event * evt);

//-------- Member attributes -------------------------------------------------
protected:

  LLW::WinID _window;
  LLW::PixID _pix;
  LLW::FontID _font;
  unsigned int _blackColor;
  unsigned int _darkColor;
  unsigned int _backColor;
  unsigned int _lightColor;
  unsigned int _whiteColor;
  int _dx1,_dx2,_dy1,_dy2;
  bool _focus;
  bool _decoration;
  bool _shaded;
  bool _max;
  int _saveX,_saveY;
  unsigned int _saveWidth,_saveHeight;
  Viewer3D::WindowPart _clickPart;
  int _clickDx,_clickDy;
  int _clickX,_clickY;
  unsigned int _clickWidth,_clickHeight;
  unsigned int _clickDelay;
  unsigned int _clickTime;
  CallbackManager<Viewer3D,FocusEvent> _focusCB;
  CallbackManager<Viewer3D,AppearanceEvent> _appearanceCB;
};

} // namespace AReVi

#endif // AREVI_LIB3D_VIEWER3D_H

//----------------------------------------------------------------------------
