// LI2/ENIB - AReVi - 3D services
//----------------------------------------------------------------------------
// file        : AReVi/Lib3D/window3D.h
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVI_LIB3D_WINDOW3D_H
#define AREVI_LIB3D_WINDOW3D_H 1

#include "AReVi/Lib3D/renderer3D.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class Window3D (abstract)
//----------------------------------------------------------------------------

class Window3D : public Renderer3D
{
//-------- Type information --------------------------------------------------
public:

  AR_CLASS(Window3D)

//-------- Window geometry ---------------------------------------------------

  virtual
  void
  setWindowGeometry(int x,
                    int y,
                    unsigned int width,
                    unsigned int height);

  virtual
  void
  getWindowGeometry(int & xOut,
                    int & yOut,
                    unsigned int & widthOut,
                    unsigned int & heightOut) const;

  virtual
  void
  setWindowPosition(int x,
                    int y);

  virtual
  void
  getWindowPosition(int & xOut,
                    int & yOut) const;

  virtual
  void
  setWindowSize(unsigned int width,
                unsigned int height);

  virtual
  void
  getWindowSize(unsigned int & widthOut,
                unsigned int & heightOut) const;

  virtual
  void
  setRenderSize(unsigned int width,
                unsigned int height);

//-------- Cursor management -------------------------------------------------

  enum Cursor
    {
    CSR_EMPTY=LLW::CSR_EMPTY,
    CSR_ARROW=LLW::CSR_ARROW,
    CSR_NORTH=LLW::CSR_NORTH,
    CSR_SOUTH=LLW::CSR_SOUTH,
    CSR_EAST=LLW::CSR_EAST,
    CSR_WEST=LLW::CSR_WEST,
    CSR_NORTHEAST=LLW::CSR_NORTHEAST,
    CSR_NORTHWEST=LLW::CSR_NORTHWEST,
    CSR_SOUTHEAST=LLW::CSR_SOUTHEAST,
    CSR_SOUTHWEST=LLW::CSR_SOUTHWEST,
    CSR_MOVE=LLW::CSR_MOVE,
    CSR_CROSS=LLW::CSR_CROSS,
    CSR_WAIT=LLW::CSR_WAIT,
    CSR_TEXT=LLW::CSR_TEXT,
    CSR_HAND=LLW::CSR_HAND,
    CSR_HELP=LLW::CSR_HELP,
    CSR_UP=LLW::CSR_UP,
    CSR_NO=LLW::CSR_NO
    };

  virtual
  void
  setWindowCursor(Window3D::Cursor cursor);

  virtual
  Window3D::Cursor // used cursor
  getWindowCursor(void) const;

//-------- Event management --------------------------------------------------

  AR_CALLBACK(Window3D,Geometry,_geometryCB,
              /* no field */)

  AR_CALLBACK(Window3D,Cursor,_cursorCB,
              Window3D::Cursor cursor;)

  virtual
  void
  setKeyRepeat(bool keyRepeat);

  virtual
  bool // key repeat is enabled
  getKeyRepeat(void) const;

//-------- Window ID ---------------------------------------------------------

  virtual
  int // platform-specific render window ID
  getRenderWindowID(void) const;

//-------- Appearance --------------------------------------------------------

  virtual
  void
  setBackgroundColor(double red,
                     double green,
                     double blue);

//-------- Forbidden operations ----------------------------------------------
protected:

  friend class ArConsole; // FIXME: really needed ?

  Window3D(ArCW & arCW);

  virtual
  void
  _afterRenderAreaSetup(void);

  virtual
  void
  _render(void);

  virtual
  bool // event processed
  _processEvent(const LLW::Event * evt);

//-------- Member attributes -------------------------------------------------
protected:

  CallbackManager<Window3D,GeometryEvent> _geometryCB;
  CallbackManager<Window3D,CursorEvent> _cursorCB;
};

} // namespace AReVi

#endif // AREVI_LIB3D_WINDOW3D_H

//----------------------------------------------------------------------------
