// LI2/ENIB - AReVi - 3D shapes
//----------------------------------------------------------------------------
// file        : AReVi/Shapes/bumping.h
// author(s)   : P.A Beal and F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVI_SHAPES_BUMPING_H
#define AREVI_SHAPES_BUMPING_H 1

#include "AReVi/Shapes/surface3D.h"

namespace AReVi {

//-----------------------------------------------------------------------------
// Class Bumping
//-----------------------------------------------------------------------------

class Bumping : public Surface3D
{
//-------- Type information --------------------------------------------------
public :

  AR_CLASS(Bumping)

//-------- Construction / Destruction ----------------------------------------

  AR_CONSTRUCTOR_1(Bumping,
                   ArRef<Texture>, elevationTexture)

//-------- Geometry ----------------------------------------------------------

  virtual
  void
  setHeight(double minHeight,
            double maxHeight);

  virtual
  void
  getHeight(double & minHeightOut,
            double & maxHeightOut) const;

  virtual
  void
  setBounds(double xMin,
            double xMax,
            double yMin,
            double yMax);

  virtual
  void
  getBounds(double & xMinOut,
            double & xMaxOut,
            double & yMinOut,
            double & yMaxOut) const;

//-------- Elevation ---------------------------------------------------------

  virtual
  ArConstRef<Texture>
  getElevationTexture(void) const;

  virtual
  void
  setPrecision(unsigned int xPrecision,
               unsigned int yPrecision);

  virtual
  void
  getPrecision(unsigned int & xPrecisionOut,
               unsigned int & yPrecisionOut) const;

//-------- Forbidden operations ----------------------------------------------
protected:

  virtual
  void
  _rebuildIfNeeded(void);

//-------- Member attributes -------------------------------------------------
protected:

  ArRef<Texture> _elevation;
  double _xMin,_xMax;
  double _yMin,_yMax;
  double _zMin,_zMax;
  unsigned int _xPrecision;
  unsigned int _yPrecision;
};

} // namespace AReVi

#endif // AREVI_SHAPES_BUMPING_H

//----------------------------------------------------------------------------
