// LI2/ENIB - AReVi - 3D shapes
//----------------------------------------------------------------------------
// file        : AReVi/Shapes/cylinder3D.h
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVI_SHAPES_CYLINDER3D_H
#define AREVI_SHAPES_CYLINDER3D_H 1

#include "AReVi/Shapes/surface3D.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class Cylinder3D
//----------------------------------------------------------------------------

class Cylinder3D : public Surface3D
{
//-------- Type information --------------------------------------------------
public:

  AR_CLASS(Cylinder3D)

//-------- Construction / Destruction ----------------------------------------

  AR_CONSTRUCTOR(Cylinder3D)

//-------- Geometry ----------------------------------------------------------

  virtual
  bool // success
  setRadius(double radius);

  virtual
  double // radius
  getRadius(void) const;

  virtual
  bool // success
  setHeight(double height);

  virtual
  double // height
  getHeight(void) const;

  virtual
  void
  setSide(bool buildSide);

  virtual
  bool // side is built
  getSide(void) const;

  virtual
  void
  setTop(bool buildTop);

  virtual
  bool // top is built
  getTop(void) const;

  virtual
  void
  setBottom(bool buildBottom);

  virtual
  bool // bottom is built
  getBottom(void) const;

  virtual
  void
  setCCW(bool ccw);

  virtual
  bool // counter clock wise surface
  getCCW(void) const;

  virtual
  bool // success
  setPrecision(unsigned int precision);

  virtual
  unsigned int // precision
  getPrecision(void) const;

//-------- Serialization operations ------------------------------------------

  AR_UNSTUBBED virtual
  void
  serialize(SerializationDependencies & dependenciesOut,
            ArRef<AbstractOStream> output) const;

  AR_UNSTUBBED virtual
  bool // success
  unserialize(const SerializationDependencies & dependencies,
              ArRef<AbstractIStream> input);

//-------- Forbidden operations ----------------------------------------------
protected:

  virtual
  void
  _rebuildIfNeeded(void);

//-------- Member attributes -------------------------------------------------
protected:

  double _radius;
  double _height;
  bool _side;
  bool _top;
  bool _bottom;
  bool _ccw;
  unsigned int _precision;
};

} // namespace AReVi

#endif // AREVI_SHAPES_CYLINDER3D_H

//----------------------------------------------------------------------------
