// LI2/ENIB - AReVi - 3D shapes
//----------------------------------------------------------------------------
// file        : AReVi/Shapes/mesh3D.h
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVI_SHAPES_MESH3D_H
#define AREVI_SHAPES_MESH3D_H 1

#include "AReVi/Shapes/surface3D.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class Mesh3D
//----------------------------------------------------------------------------

class Mesh3D : public Surface3D
{
//-------- Type information --------------------------------------------------
public:

  AR_CLASS(Mesh3D)

//-------- Construction / Destruction ----------------------------------------

  AR_CONSTRUCTOR(Mesh3D)

//-------- Changes -----------------------------------------------------------

  virtual
  bool // success
  applyChanges(bool checkConsistency);

//-------- Vertices ----------------------------------------------------------

  AR_UNSTUBBED virtual
  StlVector<Util3D::Dbl3> & // vertex coords
  accessVertices(void);

  AR_UNSTUBBED virtual
  StlVector<Util3D::Face> & // vertex faces
  accessVertexFaces(void);

  virtual
  bool // success
  checkVertexConsistency(void) const;

//-------- Normals -----------------------------------------------------------

  AR_UNSTUBBED virtual
  StlVector<Util3D::Dbl3> & // normal coords
  accessNormals(void);

  AR_UNSTUBBED virtual
  StlVector<Util3D::Face> & // normal faces
  accessNormalFaces(void);

  virtual
  void
  computeDefaultNormals(bool smooth);

  virtual
  bool // success
  checkNormalConsistency(void) const;

//-------- Textures ----------------------------------------------------------

  AR_UNSTUBBED virtual
  StlVector<Util3D::Dbl2> & // texture coords
  accessTextures(void);

  AR_UNSTUBBED virtual
  StlVector<Util3D::Face> & // texture faces
  accessTextureFaces(void);

  virtual
  bool // success
  checkTextureConsistency(void) const;

  AR_UNSTUBBED virtual
  StlVector<Util3D::Dbl2> & // second texture coords
  accessSecondTextures(void);

  AR_UNSTUBBED virtual
  StlVector<Util3D::Face> & // second texture faces
  accessSecondTextureFaces(void);

  virtual
  bool // success
  checkSecondTextureConsistency(void) const;

  AR_UNSTUBBED virtual
  StlVector<Util3D::Dbl2> & // third texture coords
  accessThirdTextures(void);

  AR_UNSTUBBED virtual
  StlVector<Util3D::Face> & // third texture faces
  accessThirdTextureFaces(void);

  virtual
  bool // success
  checkThirdTextureConsistency(void) const;

  AR_UNSTUBBED virtual
  StlVector<Util3D::Dbl2> & // fourth texture coords
  accessFourthTextures(void);

  AR_UNSTUBBED virtual
  StlVector<Util3D::Face> & // fourth texture faces
  accessFourthTextureFaces(void);

  virtual
  bool // success
  checkFourthTextureConsistency(void) const;

//-------- Colors ------------------------------------------------------------

  AR_UNSTUBBED virtual
  StlVector<Util3D::Dbl3> & // color components
  accessColors(void);

  AR_UNSTUBBED virtual
  StlVector<Util3D::Face> & // color faces
  accessColorFaces(void);

  virtual
  bool // success
  checkColorConsistency(void) const;

//-------- Serialization operations ------------------------------------------

  AR_UNSTUBBED virtual
  void
  serialize(SerializationDependencies & dependenciesOut,
            ArRef<AbstractOStream> output) const;

  AR_UNSTUBBED virtual
  bool // success
  unserialize(const SerializationDependencies & dependencies,
              ArRef<AbstractIStream> input);

};

} // namespace AReVi

#endif // AREVI_SHAPES_MESH3D_H

//----------------------------------------------------------------------------
