// LI2/ENIB - AReVi - 3D shapes
//----------------------------------------------------------------------------
// file        : AReVi/Shapes/metaball.h
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVI_SHAPES_METABALL_H
#define AREVI_SHAPES_METABALL_H 1

#include "AReVi/Shapes/surface3D.h"
#include "AReVi/Contrib/arMath.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class Metaball
//----------------------------------------------------------------------------

class Metaball : public Surface3D
{
public:
  enum FieldFunction {FIELD_FUNCTION_LINEAR, FIELD_FUNCTION_SOFTOBJECTS};
  enum MetaballType {METABALL_SPHERE, METABALL_CUBE, METABALL_TORUS};

  typedef struct MetaballInfo
    {
    double radius;
    double innerSize; //second size info (ie : inner rad for torus)
    double potential;
    Point3d position;
    MetaballType type;
    Util3D::BoundingBox bbox;
    } MetaballInfo;

//-------- Type information --------------------------------------------------
public:

  AR_CLASS(Metaball)

//-------- Construction / Destruction ----------------------------------------

  AR_CONSTRUCTOR(Metaball)

//-------- Changes -----------------------------------------------------------

  virtual
  bool // success
  applyChanges(void);

//-------- Metaballs ---------------------------------------------------------

  AR_UNSTUBBED virtual
  void
  addMetaball(const MetaballInfo &metaball);

  virtual
  void
  removeMetaball(unsigned int index);

  virtual
  unsigned int
  getNbMetaballs(void) const;

  AR_UNSTUBBED virtual
  const MetaballInfo &
  getMetaball(unsigned int index) const;

  AR_UNSTUBBED virtual
  MetaballInfo &
  accessMetaball(unsigned int index);

//-------- Surface Setup -----------------------------------------------------

  virtual
  bool // success
  setIsoLevel(double isolevel);

  virtual
  double // iso level defining the contour
  getIsoLevel(void) const;

  virtual
  bool // success
  setPrecision(unsigned int precision);

  virtual
  unsigned int // precision
  getPrecision(void) const;

  virtual
  void
  setCCW(bool ccw);

  virtual
  bool // counter clock wise surface
  getCCW(void) const;

  virtual
  void
  setSmoothed(bool smooth);

  virtual
  bool // Faces are smoothed or not
  getSmoothed(void) const;

  virtual
  void
  setFieldFunction(Metaball::FieldFunction function);

  virtual
  Metaball::FieldFunction // Select Field Fonctions
  getFieldFunction(void) const;

//-------- Serialization operations ------------------------------------------

  AR_UNSTUBBED virtual
  void
  serialize(SerializationDependencies & depsendenciesOut,
            ArRef<AbstractOStream> output) const;

  AR_UNSTUBBED virtual
  bool // success
  unserialize(const SerializationDependencies & dependencies,
              ArRef<AbstractIStream> input);

//-------- Forbidden operations ----------------------------------------------
protected:

  bool
  _rebuildGeometry(void);

  double
  _evalFieldLinear(double x,double y,double z);

  double
  _evalFieldSoftObjects(double x,double y,double z);

  void
  _processCubie(unsigned xsub,unsigned  ysub,unsigned  zsub,unsigned  index);

   void
   _computeIsoSurfaceNormals(bool smooth);

  virtual
  void
  _rebuildIfNeeded(void);

//-------- Member attributes -------------------------------------------------
protected:

  double _isolevel;
  bool _ccw,_smooth;
  unsigned int _precision;
  FieldFunction _fieldFunction;
  StlVector<MetaballInfo> _metaballs;

// "On the fly" variables
  unsigned int _sideSize;
  StlVector<double> _cubeFields;
  StlVector<Point3d> _cubePoints;
  StlVector<int> _xeverts,_yeverts,_zeverts;
  double _minX,_minY,_minZ;
  double _maxX,_maxY,_maxZ;
};

} // namespace AReVi

#endif // AREVI_SHAPES_METABALL_H

//----------------------------------------------------------------------------
