// LI2/ENIB - AReVi - 3D shapes
//----------------------------------------------------------------------------
// file        : AReVi/Shapes/pointSet3D.h
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVI_SHAPES_POINTSET3D_H
#define AREVI_SHAPES_POINTSET3D_H 1

#include "AReVi/Shapes/shapePart3D.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class PointSet3D
//----------------------------------------------------------------------------

class PointSet3D : public ShapePart3D
{
//-------- Type information --------------------------------------------------
public:

  AR_CLASS(PointSet3D)

//-------- Construction / Destruction ----------------------------------------

  AR_CONSTRUCTOR(PointSet3D)

//-------- Changes -----------------------------------------------------------

  virtual
  bool // success
  applyChanges(bool checkConsistency);

//-------- Appearance --------------------------------------------------------

  virtual
  bool // success
  setThickness(unsigned int thickness);

  virtual
  unsigned int // points thickness
  getThickness(void) const;

//-------- Vertices ----------------------------------------------------------

  AR_UNSTUBBED virtual
  const StlVector<Util3D::Dbl3> & // vertex coords
  getVertices(void) const;

  AR_UNSTUBBED virtual
  StlVector<Util3D::Dbl3> & // vertex coords
  accessVertices(void);

//-------- Colors ------------------------------------------------------------

  AR_UNSTUBBED virtual
  const StlVector<Util3D::Dbl3> & // color components
  getColors(void) const;

  AR_UNSTUBBED virtual
  StlVector<Util3D::Dbl3> & // color components
  accessColors(void);

//-------- Serialization operations ------------------------------------------

  AR_UNSTUBBED virtual
  void
  serialize(SerializationDependencies & dependenciesOut,
            ArRef<AbstractOStream> output) const;

  AR_UNSTUBBED virtual
  bool // success
  unserialize(const SerializationDependencies & dependencies,
              ArRef<AbstractIStream> input);

//-------- Forbidden operations ----------------------------------------------
protected:

  virtual
  void
  _setAppearance(void) const;

  virtual
  void
  _resetAppearance(void) const;

  virtual
  void
  _drawGeometry(void) const;

  virtual
  void
  _rebuildIfNeeded(void);

//-------- Member attributes -------------------------------------------------
protected:

  unsigned int _thickness;
  StlVector<Util3D::Dbl3> _vertices;
  StlVector<Util3D::Dbl3> _colors;
};

} // namespace AReVi

#endif // AREVI_SHAPES_POINTSET3D_H

//----------------------------------------------------------------------------
