// LI2/ENIB - AReVi - 3D shapes
//----------------------------------------------------------------------------
// file        : AReVi/Shapes/shader.h
// author(s)   : T. Jourdan, F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVI_SHAPES_SHADER_H
#define AREVI_SHAPES_SHADER_H 1

#include "AReVi/Shapes/shapePart3D.h"

namespace AReVi {

class ShaderProgram;

//----------------------------------------------------------------------------
// Class Shader (abstract)
//----------------------------------------------------------------------------

class Shader : public ArObject
{
//-------- Type information --------------------------------------------------
public:

  AR_CLASS(Shader)

//-------- Source code management --------------------------------------------

  virtual
  bool // success
  load(const StlString & code);

  virtual
  bool // success
  loadURL(const StlString & url);

  virtual
  bool // success
  loadStream(ArRef<AbstractIStream> input);

  unsigned int // OpenGL shader handle
  getGLHandle(void) const;

//-------- Serialization operations ------------------------------------------

  AR_UNSTUBBED virtual
  void
  serialize(SerializationDependencies & dependenciesOut,
            ArRef<AbstractOStream> output) const;

  AR_UNSTUBBED virtual
  bool // success
  unserialize(const SerializationDependencies & dependencies,
              ArRef<AbstractIStream> input);

//-------- Forbidden operations ----------------------------------------------
protected:

  friend class ShaderProgram;

  Shader(ArCW & arCW);

  virtual
  void
  _addShaderProgram(ShaderProgram * shaderProgram);

  virtual
  void
  _removeShaderProgram(ShaderProgram * shaderProgram);

  virtual
  void
  _linkShaders(void);

//-------- Member attributes -------------------------------------------------
protected:

  unsigned int _handle;
  StlString _code;
  StlVector<ShaderProgram *> _programs;
};


//----------------------------------------------------------------------------
// Class VertexShader
//----------------------------------------------------------------------------

class VertexShader : public Shader
{
//-------- Type information --------------------------------------------------
public:

  AR_CLASS(VertexShader)

//-------- Construction / Destruction ----------------------------------------

  AR_CONSTRUCTOR(VertexShader)
};


//----------------------------------------------------------------------------
// Class FragmentShader
//----------------------------------------------------------------------------

class FragmentShader : public Shader
{
//-------- Type information --------------------------------------------------
public:

  AR_CLASS(FragmentShader)

//-------- Construction / Destruction ----------------------------------------

  AR_CONSTRUCTOR(FragmentShader)
};


//----------------------------------------------------------------------------
// Class ShaderProgram
//----------------------------------------------------------------------------

class ShaderProgram : public ShapePartRenderer
{
//-------- Type information --------------------------------------------------
public:

  AR_CLASS(ShaderProgram)

//-------- Construction / Destruction ----------------------------------------

  AR_CONSTRUCTOR(ShaderProgram)

//-------- Shader management -------------------------------------------------

  virtual
  bool // link success
  setShaders(ArRef<VertexShader> vertexShader,
             ArRef<FragmentShader> fragmentShader);

  virtual
  ArConstRef<VertexShader>
  getVertexShader(void) const;

  virtual
  ArRef<VertexShader>
  accessVertexShader(void);

  virtual
  ArConstRef<FragmentShader>
  getFragmentShader(void) const;

  virtual
  ArRef<FragmentShader>
  accessFragmentShader(void);

  unsigned int // OpenGL program handle
  getGLHandle(void) const;

//-------- Serialization operations ------------------------------------------

  AR_UNSTUBBED virtual
  void
  extractDependencies(SerializationDependencies & dependenciesOut,
                      bool recursive=true) const;

  AR_UNSTUBBED virtual
  void
  serialize(SerializationDependencies & dependenciesOut,
            ArRef<AbstractOStream> output) const;

  AR_UNSTUBBED virtual
  bool // success
  unserialize(const SerializationDependencies & dependencies,
              ArRef<AbstractIStream> input);

//-------- Forbidden operations ----------------------------------------------
protected:

  friend class Shader;

  virtual
  void
  _render(const ShapePart3D * part,
          const Object3D * object,
          const Renderer3D * renderer) const;

  void
  _renderWithoutAppearance(const ShapePart3D * part,
                           const Object3D * object,
                           const Renderer3D * renderer) const;

  virtual
  bool // success
  _linkShaders(void);

//-------- Member attributes -------------------------------------------------
protected:

  unsigned int _handle;
  bool _linked;
  ArRef<VertexShader> _vertexShader;
  ArRef<FragmentShader> _fragmentShader;
};

} // namespace AReVi

#endif // AREVI_SHAPES_SHADER_H

//----------------------------------------------------------------------------
