// LI2/ENIB - AReVi - 3D shapes
//----------------------------------------------------------------------------
// file        : AReVi/Shapes/surface3D.h
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVI_SHAPES_SURFACE3D_H
#define AREVI_SHAPES_SURFACE3D_H 1

#include "AReVi/Shapes/shapePart3D.h"

namespace AReVi {

class Material3D;
class Transform2D;
class Texture;

//----------------------------------------------------------------------------
// Class Surface3D (abstract)
//----------------------------------------------------------------------------

class Surface3D : public ShapePart3D
{
//-------- Type information --------------------------------------------------
public:

  AR_CLASS(Surface3D)

//-------- Appearance --------------------------------------------------------

  virtual
  void
  writeMaterial(ArConstRef<Material3D> material);

  virtual
  void
  readMaterial(ArRef<Material3D> materialOut) const;

  virtual
  void
  setDoubleSided(bool doubleSided);

  virtual
  bool // double sided surface
  getDoubleSided(void) const;

//-------- Texture management ------------------------------------------------

  virtual
  void
  setTexture(ArRef<Texture> texture);

  virtual
  ArConstRef<Texture> // texture
  getTexture(void) const;

  virtual
  ArRef<Texture> // texture
  accessTexture(void);

  virtual
  void
  setTextureRepeat(bool repeatX,
                   bool repeatY);

  virtual
  void
  getTextureRepeat(bool & repeatXOut,
                   bool & repeatYOut) const;

  virtual
  void
  writeTextureTransformation(ArConstRef<Transform2D> transform);

  virtual
  void
  readTextureTransformation(ArRef<Transform2D> transformOut) const;

  virtual
  void
  setTextureReflection(bool reflectMode); // Consistent only for CubeMapTextures

  virtual
  bool
  getTextureReflection(void) const;

  virtual
  void
  setSecondTexture(ArRef<Texture> texture);

  virtual
  ArConstRef<Texture> // second texture
  getSecondTexture(void) const;

  virtual
  ArRef<Texture> // second texture
  accessSecondTexture(void);

  virtual
  void
  setSecondTextureRepeat(bool repeatX,
                         bool repeatY);

  virtual
  void
  getSecondTextureRepeat(bool & repeatXOut,
                         bool & repeatYOut) const;

  virtual
  void
  writeSecondTextureTransformation(ArConstRef<Transform2D> transform);

  virtual
  void
  readSecondTextureTransformation(ArRef<Transform2D> transformOut) const;

  virtual
  void
  setSecondTextureReflection(bool reflectMode); // Consistent only for CubeMapTextures

  virtual
  bool
  getSecondTextureReflection(void) const;

  virtual
  void
  setThirdTexture(ArRef<Texture> texture);

  virtual
  ArConstRef<Texture> // third texture
  getThirdTexture(void) const;

  virtual
  ArRef<Texture> // third texture
  accessThirdTexture(void);

  virtual
  void
  setThirdTextureRepeat(bool repeatX,
                        bool repeatY);

  virtual
  void
  getThirdTextureRepeat(bool & repeatXOut,
                        bool & repeatYOut) const;

  virtual
  void
  writeThirdTextureTransformation(ArConstRef<Transform2D> transform);

  virtual
  void
  readThirdTextureTransformation(ArRef<Transform2D> transformOut) const;

  virtual
  void
  setThirdTextureReflection(bool reflectMode); // Consistent only for CubeMapTextures

  virtual
  bool
  getThirdTextureReflection(void) const;


  virtual
  void
  setFourthTexture(ArRef<Texture> texture);

  virtual
  ArConstRef<Texture> // fourth texture
  getFourthTexture(void) const;

  virtual
  ArRef<Texture> // fourth texture
  accessFourthTexture(void);

  virtual
  void
  setFourthTextureRepeat(bool repeatX,
                         bool repeatY);

  virtual
  void
  getFourthTextureRepeat(bool & repeatXOut,
                         bool & repeatYOut) const;

  virtual
  void
  writeFourthTextureTransformation(ArConstRef<Transform2D> transform);

  virtual
  void
  readFourthTextureTransformation(ArRef<Transform2D> transformOut) const;

    virtual
  void
  setFourthTextureReflection(bool reflectMode); // Consistent only for CubeMapTextures

  virtual
  bool
  getFourthTextureReflection(void) const;

//-------- Vertices ----------------------------------------------------------

  AR_UNSTUBBED virtual
  const StlVector<Util3D::Dbl3> & // vertex coords
  getVertices(void) const;

  AR_UNSTUBBED virtual
  const StlVector<Util3D::Face> & // vertex faces
  getVertexFaces(void) const;

//-------- Normals -----------------------------------------------------------

  AR_UNSTUBBED virtual
  const StlVector<Util3D::Dbl3> & // normal coords
  getNormals(void) const;

  AR_UNSTUBBED virtual
  const StlVector<Util3D::Face> & // normal faces
  getNormalFaces(void) const;

//-------- Textures ----------------------------------------------------------

  AR_UNSTUBBED virtual
  const StlVector<Util3D::Dbl2> & // texture coords
  getTextures(void) const;

  AR_UNSTUBBED virtual
  const StlVector<Util3D::Face> & // texture faces
  getTextureFaces(void) const;

  AR_UNSTUBBED virtual
  const StlVector<Util3D::Dbl2> & // second texture coords
  getSecondTextures(void) const;

  AR_UNSTUBBED virtual
  const StlVector<Util3D::Face> & // second texture faces
  getSecondTextureFaces(void) const;

  AR_UNSTUBBED virtual
  const StlVector<Util3D::Dbl2> & // third texture coords
  getThirdTextures(void) const;

  AR_UNSTUBBED virtual
  const StlVector<Util3D::Face> & // third texture faces
  getThirdTextureFaces(void) const;

  AR_UNSTUBBED virtual
  const StlVector<Util3D::Dbl2> & // fourth texture coords
  getFourthTextures(void) const;

  AR_UNSTUBBED virtual
  const StlVector<Util3D::Face> & // fourth texture faces
  getFourthTextureFaces(void) const;

//-------- Colors ------------------------------------------------------------

  AR_UNSTUBBED virtual
  const StlVector<Util3D::Dbl3> & // color components
  getColors(void) const;

  AR_UNSTUBBED virtual
  const StlVector<Util3D::Face> & // color faces
  getColorFaces(void) const;

//-------- Serialization operations ------------------------------------------

  AR_UNSTUBBED virtual
  void
  extractDependencies(SerializationDependencies & dependenciesOut,
                      bool recursive=true) const;

  AR_UNSTUBBED virtual
  void
  serialize(SerializationDependencies & dependenciesOut,
            ArRef<AbstractOStream> output) const;

  AR_UNSTUBBED virtual
  bool // success
  unserialize(const SerializationDependencies & dependencies,
              ArRef<AbstractIStream> input);

//-------- Forbidden operations ----------------------------------------------
protected:

  Surface3D(ArCW & arCW);

  virtual
  void
  _setAppearance(void) const;

  virtual
  void
  _resetAppearance(void) const;

  virtual
  void
  _drawGeometry(void) const;

  virtual
  void
  _rebuildIfNeeded(void);

  virtual
  void
  _computeDefaultNormals(bool smooth);

//-------- Member attributes -------------------------------------------------
protected:

  Util3D::Material _material;
  bool _doubleSided;
  ArRef<Texture> _texture;
  ArRef<Texture> _texture2;
  ArRef<Texture> _texture3;
  ArRef<Texture> _texture4;
  Util3D::Transform _texTransform;
  Util3D::Transform _texTransform2;
  Util3D::Transform _texTransform3;
  Util3D::Transform _texTransform4;
  bool _repeatX,_repeatY,_cubeMapReflect;
  bool _repeatX2,_repeatY2,_cubeMapReflect2;
  bool _repeatX3,_repeatY3,_cubeMapReflect3;
  bool _repeatX4,_repeatY4,_cubeMapReflect4;
  StlVector<Util3D::Dbl3> _vertices;
  StlVector<Util3D::Face> _facesV;
  StlVector<Util3D::Dbl3> _normals;
  StlVector<Util3D::Face> _facesN;
  StlVector<Util3D::Dbl2> _textures;
  StlVector<Util3D::Face> _facesT;
  StlVector<Util3D::Dbl2> _textures2;
  StlVector<Util3D::Face> _facesT2;
  StlVector<Util3D::Dbl2> _textures3;
  StlVector<Util3D::Face> _facesT3;
  StlVector<Util3D::Dbl2> _textures4;
  StlVector<Util3D::Face> _facesT4;
  StlVector<Util3D::Dbl3> _colors;
  StlVector<Util3D::Face> _facesC;
};

} // namespace AReVi

#endif // AREVI_SHAPES_SURFACE3D_H

//----------------------------------------------------------------------------
