// LI2/ENIB - AReVi - 3D shapes
//----------------------------------------------------------------------------
// file        : AReVi/Shapes/text3D.h
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVI_SHAPES_TEXT3D_H
#define AREVI_SHAPES_TEXT3D_H 1

#include "AReVi/Shapes/surface3D.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class Text3D
//----------------------------------------------------------------------------

class Text3D : public Surface3D
{
//-------- Type information --------------------------------------------------
public:

  AR_CLASS(Text3D)

//-------- Construction / Destruction ----------------------------------------

  AR_CONSTRUCTOR(Text3D)

//-------- Text --------------------------------------------------------------

  virtual
  bool // success
  setFont(const StlString & fontURL);

  virtual
  const StlString & // font URL
  getFont(void) const;

  virtual
  bool // success
  setSpaces(double symbolSpace,
            double whiteSpace);

  virtual
  void
  getSpaces(double & symbolSpaceOut,
            double & whiteSpaceOut) const;

  virtual
  void
  setText(const StlString & text);

  virtual
  const StlString &
  getText(void) const;

  virtual
  bool // success
  setExtrusion(double extrusion);

  virtual
  double // extrusion thickness
  getExtrusion(void) const;

//-------- Serialization operations ------------------------------------------

  AR_UNSTUBBED virtual
  void
  extractDependencies(SerializationDependencies & dependenciesOut,
                      bool recursive=true) const;

  AR_UNSTUBBED virtual
  void
  serialize(SerializationDependencies & dependenciesOut,
            ArRef<AbstractOStream> output) const;

  AR_UNSTUBBED virtual
  bool // success
  unserialize(const SerializationDependencies & dependencies,
              ArRef<AbstractIStream> input);

//-------- Forbidden operations ----------------------------------------------
protected:

  virtual
  void
  _rebuildIfNeeded(void);

//-------- Member attributes -------------------------------------------------
protected:

  StlString _fontURL;
  void * _font;
  double _symbolSpace;
  double _whiteSpace;
  StlString _text;
  double _extrusion;
};

} // namespace AReVi

#endif // AREVI_SHAPES_TEXT3D_H

//----------------------------------------------------------------------------
