// LI2/ENIB - AReVi - Utility services
//----------------------------------------------------------------------------
// file        : AReVi/Utils/abstractStream.h
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef  AREVI_UTILS_ABSTRACTSTREAM_H
#define  AREVI_UTILS_ABSTRACTSTREAM_H 1

#include "AReVi/arObject.h"

namespace AReVi {

class AbstractOStream;
class MemoryBlock;

//----------------------------------------------------------------------------
// Class AbstractIStream (abstract)
//----------------------------------------------------------------------------

class AbstractIStream : public ArObject
{
//-------- Type information --------------------------------------------------
public:

  AR_CLASS(AbstractIStream)

//-------- Stream control ----------------------------------------------------

  virtual
  unsigned int // number of bytes currently available in the input buffer
  bufferContent(void) const;

  AR_UNSTUBBED virtual
  istream & // embedded C++ stream
  stream(void);

//-------- Text input --------------------------------------------------------

  virtual
  bool // success
  readTxtChar(char & cOut);

  virtual
  bool // success
  readTxtUChar(unsigned char & ucOut);

  virtual
  bool // success
  readTxtShort(short & sOut);

  virtual
  bool // success
  readTxtUShort(unsigned short & usOut);

  virtual
  bool // success
  readTxtInt(int & iOut);

  virtual
  bool // success
  readTxtUInt(unsigned int & uiOut);

  virtual
  bool // success
  readTxtLLong(long long & llOut);

  virtual
  bool // success
  readTxtULLong(unsigned long long & ullOut);

  virtual
  bool // success
  readTxtFloat(float & fOut);

  virtual
  bool // success
  readTxtDouble(double & dOut);

  virtual
  bool // success
  readTxtWord(StlString & wOut);

  virtual
  bool // success
  readTxtLine(StlString & lOut);

//-------- Binary input ------------------------------------------------------

  AR_UNSTUBBED virtual
  unsigned int // nb read
  readData(void * buffer,
           unsigned int size);

  virtual
  bool // success
  copyDataToOutput(ArRef<AbstractOStream> output);

//-------- Network format binary input ---------------------------------------

  virtual
  bool // success
  readChar(char & cOut);

  virtual
  bool // success
  readUChar(unsigned char & ucOut);

  virtual
  bool // success
  readShort(short & sOut);

  virtual
  bool // success
  readUShort(unsigned short & usOut);

  virtual
  bool // success
  readInt(int & iOut);

  virtual
  bool // success
  readUInt(unsigned int & uiOut);

  virtual
  bool // success
  readLLong(long long & llOut);

  virtual
  bool // success
  readULLong(unsigned long long & ullOut);

  virtual
  bool // success
  readFloat(float & fOut);

  virtual
  bool // success
  readDouble(double & dOut);

  virtual
  bool // success
  readString(StlString & sOut);

//-------- Error management --------------------------------------------------

  virtual
  bool // problem occured
  fail(void) const;

//-------- Forbidden operations ----------------------------------------------
protected:

  AbstractIStream(ArCW & arCW);

  AbstractIStream(ArCW & arCW,
                  ArConstRef<MemoryBlock> memory,
                  unsigned int offset);

  virtual
  int // >0: nb read   <=0: error
  _readBuffer(char * buffer,
              unsigned int size);

//-------- Member attributes -------------------------------------------------
protected:

  class IBuffer : public streambuf
  {
  public:
    IBuffer(AbstractIStream * stream);
    IBuffer(AbstractIStream * stream,
            ArConstRef<MemoryBlock> memory,
            unsigned int offset);
    virtual ~IBuffer(void);
    virtual unsigned int getMemoryUsage(void) const;
    virtual unsigned int getOffset(void) const;
  protected:
    virtual int underflow(void);
    enum { ISTREAM_BUFFERSIZE=0x400 } ;
    char * _buffer;
    ArConstRef<MemoryBlock> _memory;
    unsigned int _offset;
    AbstractIStream * _stream;
  };

  friend class AbstractIStream::IBuffer;

  AbstractIStream::IBuffer * _iBuffer;
  istream _stream;
};


//----------------------------------------------------------------------------
// Class AbstractOStream (abstract)
//----------------------------------------------------------------------------

class AbstractOStream : public ArObject
{
//-------- Type information --------------------------------------------------
public:

  AR_CLASS(AbstractOStream)

//-------- Stream control ----------------------------------------------------

  virtual
  bool // success
  flush(void);

  AR_UNSTUBBED virtual
  ostream & // embedded C++ stream
  stream(void);

//-------- Text output -------------------------------------------------------

  virtual
  bool // success
  writeTxtChar(char c);

  virtual
  bool // success
  writeTxtUChar(unsigned char uc);

  virtual
  bool // success
  writeTxtShort(short s);

  virtual
  bool // success
  writeTxtUShort(unsigned short us);

  virtual
  bool // success
  writeTxtInt(int i);

  virtual
  bool // success
  writeTxtUInt(unsigned int ui);

  virtual
  bool // success
  writeTxtLLong(long long ll);

  virtual
  bool // success
  writeTxtULLong(unsigned long long ull);

  virtual
  bool // success
  writeTxtFloat(float f);

  virtual
  bool // success
  writeTxtDouble(double d);

  virtual
  bool // success
  writeTxtString(const StlString & s);

//-------- Binary output -----------------------------------------------------

  AR_UNSTUBBED virtual
  bool // success
  writeData(const void * buffer,
            unsigned int size);

  virtual
  bool // success
  copyDataFromInput(ArRef<AbstractIStream> input);

//-------- Network format binary output --------------------------------------

  virtual
  bool // success
  writeChar(char c);

  virtual
  bool // success
  writeUChar(unsigned char uc);

  virtual
  bool // success
  writeShort(short s);

  virtual
  bool // success
  writeUShort(unsigned short us);

  virtual
  bool // success
  writeInt(int i);

  virtual
  bool // success
  writeUInt(unsigned int ui);

  virtual
  bool // success
  writeLLong(long long ll);

  virtual
  bool // success
  writeULLong(unsigned long long ull);

  virtual
  bool // success
  writeFloat(float f);

  virtual
  bool // success
  writeDouble(double d);

  virtual
  bool // success
  writeString(const StlString & s);

//-------- Error management --------------------------------------------------

  virtual
  bool // problem occured
  fail(void) const;

//-------- Forbidden operations ----------------------------------------------
protected:

  AbstractOStream(ArCW & arCW);

  AbstractOStream(ArCW & arCW,
                  ArRef<MemoryBlock> memory,
                  unsigned int offset);

  virtual
  int // >=0: nb written   -1: error
  _writeBuffer(const char * buffer,
               unsigned int size);

//-------- Member attributes -------------------------------------------------
protected:

  class OBuffer : public streambuf
  {
  public:
    OBuffer(AbstractOStream * stream);
    OBuffer(AbstractOStream * stream,
            ArRef<MemoryBlock> memory,
            unsigned int offset);
    virtual ~OBuffer(void);
    virtual unsigned int getMemoryUsage(void) const;
    virtual unsigned int getOffset(void) const;
  protected:
    virtual int sync(void);
    virtual int overflow(int c=EOF);
    enum { OSTREAM_BUFFERSIZE=0x400 } ;
    char * _buffer;
    ArRef<MemoryBlock> _memory;
    unsigned int _offset;
    AbstractOStream * _stream;
  };

  friend class AbstractOStream::OBuffer;

  AbstractOStream::OBuffer * _oBuffer;
  ostream _stream;
};

} // namespace AReVi

#endif //  AREVI_UTILS_ABSTRACTSTREAM_H

//----------------------------------------------------------------------------
