// LI2/ENIB - AReVi - Utility services
//----------------------------------------------------------------------------
// file        : AReVi/Utils/callback_priv.h
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVI_UTILS_CALLBACK_PRIV_H
#define AREVI_UTILS_CALLBACK_PRIV_H 1

#include "AReVi/arObject.h"

namespace AReVi {

#define AR_CALLBACK(_SRC,_EVT_PREFIX,_ATTR,_EVT_FIELDS)    \
  struct _EVT_PREFIX##Event                                \
  {                                                        \
  ArRef<_SRC> source;                                      \
  _EVT_FIELDS                                              \
  };                                                       \
  template <class _T1,class _T2> inline                    \
  void                                                     \
  add##_EVT_PREFIX##CB(ArRef<_T1> target,                  \
    void (_T2::*cb)(const _SRC::_EVT_PREFIX##Event & evt)) \
    { _ATTR.add(target,cb); } /* NOT virtual ! */          \
  template <class _T1,class _T2> inline                    \
  void                                                     \
  remove##_EVT_PREFIX##CB(ArRef<_T1> target,               \
    void (_T2::*cb)(const _SRC::_EVT_PREFIX##Event & evt)) \
    { _ATTR.remove(target,cb); } /* NOT virtual ! */       \
  template <class _T1,class _T2> inline                    \
  void                                                     \
  pushMasking##_EVT_PREFIX##CB(ArRef<_T1> target,          \
    void (_T2::*cb)(const _SRC::_EVT_PREFIX##Event & evt)) \
    { _ATTR.pushMasking(target,cb); } /* NOT virtual ! */  \
  template <class _T1,class _T2> inline                    \
  void                                                     \
  popMasking##_EVT_PREFIX##CB(ArRef<_T1> target,           \
    void (_T2::*cb)(const _SRC::_EVT_PREFIX##Event & evt)) \
    { _ATTR.popMasking(target,cb); } /* NOT virtual ! */

//----------------------------------------------------------------------------
// Class CallbackManager<SRC,EVT>
//----------------------------------------------------------------------------

// Methods of CallbackManager are not virtual ! (due to template members)
//   --> do NOT inherit from CallbackManager

template <class SRC,class EVT>
class CallbackManager
{
//-------- Construction / Destruction ----------------------------------------
public:

  inline
  CallbackManager(ArRef<SRC> source);

  inline
  ~CallbackManager(void); // NOT virtual !

//-------- Callback management -----------------------------------------------

  template <class T1,class T2> inline
  void
  add(ArRef<T1> target,
      void (T2::*cb)(const EVT &)); // NOT virtual !

  template <class T1,class T2> inline
  void
  remove(ArRef<T1> target,
         void (T2::*cb)(const EVT &)); // NOT virtual !

  template <class T1,class T2> inline
  void
  pushMasking(ArRef<T1> target,
              void (T2::*cb)(const EVT &)); // NOT virtual !

  template <class T1,class T2> inline
  void
  popMasking(ArRef<T1> target,
             void (T2::*cb)(const EVT &)); // NOT virtual !

  inline
  bool
  empty(void) const; // NOT virtual !

  inline
  void
  fire(EVT & event); // NOT virtual !

//-------- Forbidden operations ----------------------------------------------
private:

  inline
  CallbackManager(const CallbackManager<SRC,EVT> & mgr); // no implementation

  inline
  CallbackManager & // itself
  operator=(const CallbackManager<SRC,EVT> & mgr); // no implementation

//-------- Member attributes -------------------------------------------------
private:

  typedef void (ArObject::*CB)(const EVT &);

  struct CBInfo
    {
    ArPtr<ArObject> target;
    typename CallbackManager<SRC,EVT>::CB cb;

    inline
    CBInfo(ArPtr<ArObject> target,
           CallbackManager<SRC,EVT>::CB cb);
    };

  ArPtr<SRC> _source;
  StlVector<typename CallbackManager<SRC,EVT>::CBInfo> _infos;
  StlVector<typename CallbackManager<SRC,EVT>::CBInfo> _masking;
};

} // namespace AReVi

//-------- Inline source code ------------------------------------------------

#include "AReVi/Utils/callback_priv.Ci"

#endif // AREVI_UTILS_CALLBACK_PRIV_H

//----------------------------------------------------------------------------
