// LI2/ENIB - AReVi - Utility services
//----------------------------------------------------------------------------
// file        : AReVi/Utils/javaObject.h
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVI_UTILS_JAVAOBJECT_H
#define AREVI_UTILS_JAVAOBJECT_H 1

#include "AReVi/arObject.h"

#define DECLARE_JAVA_AREVI_APP(_INITFNCT,_UPDATEFNCT,_DESTROYFNCT)            \
  namespace AReVi {                                                           \
  int JAVA_InitAReViApp(void *,bool (*)(int,char **),                         \
                        bool (*)(void),void (*)(void));                       \
  }                                                                           \
  extern "C" {                                                                \
  int JNI_OnLoad(void * javaVM,void *)                                        \
    { return(AReVi::JAVA_InitAReViApp(javaVM,                                 \
                                      _INITFNCT,_UPDATEFNCT,_DESTROYFNCT)); } \
  unsigned char JAVA_native_initApp(void *,void *,void *);                    \
  unsigned char JAVA_native_updateApp(void *,void *);                         \
  void JAVA_native_destroyApp(void *,void *);                                 \
  void * JAVA_native_toAReVi(void *,void *,void *);                           \
  int JAVA_native_windowID(void *,void *,void *);                             \
  unsigned char Java_AReViObject_initApp(void * env,void * cls,void * args)   \
    { return(JAVA_native_initApp(env,cls,args)); }                            \
  unsigned char Java_AReViObject_updateApp(void * env,void * cls)             \
    { return(JAVA_native_updateApp(env,cls)); }                               \
  void Java_AReViObject_destroyApp(void * env,void * cls)                     \
    { JAVA_native_destroyApp(env,cls); }                                      \
  void * Java_AReViObject_toAReVi(void * env,void * self,void * array)        \
    { return(JAVA_native_toAReVi(env,self,array)); }                          \
  int Java_AReViObject_windowID(void * env,void * cls,void * component)       \
    { return(JAVA_native_windowID(env,cls,component)); }                      \
  }

namespace AReVi {

//----------------------------------------------------------------------------
// Class JavaData
//----------------------------------------------------------------------------

class JavaData : public ArObject
{
//-------- Type information --------------------------------------------------
public:

  AR_CLASS(JavaData)

//-------- Construction / Destruction ----------------------------------------

  AR_CONSTRUCTOR(JavaData)

//-------- Value management --------------------------------------------------

  virtual
  unsigned int
  getNbValues(void) const;

  enum ValueType { J_BOOLEAN ,  // unsigned 8 bits
                   J_BYTE    ,  // signed 8 bits
                   J_CHAR    ,  // unsigned 16 bits
                   J_SHORT   ,  // signed 16 bits
                   J_INT     ,  // signed 32 bits
                   J_LONG    ,  // signed 64 bits
                   J_FLOAT   ,  // 32 bits
                   J_DOUBLE  ,  // 64 bits
                   J_STRING  }; // string

  virtual
  JavaData::ValueType
  getType(unsigned int index) const;

  virtual
  void
  addJBoolean(unsigned char value);

  virtual
  unsigned char // java boolean value
  readJBoolean(unsigned int index) const;

  virtual
  void
  addJByte(char value);

  virtual
  char // java byte value
  readJByte(unsigned int index) const;

  virtual
  void
  addJChar(unsigned short value);

  virtual
  unsigned short // java char value
  readJChar(unsigned int index) const;

  virtual
  void
  addJShort(short value);

  virtual
  short // java short value
  readJShort(unsigned int index) const;

  virtual
  void
  addJInt(int value);

  virtual
  int // java int value
  readJInt(unsigned int index) const;

  virtual
  void
  addJLong(long long value);

  virtual
  long long // java long value
  readJLong(unsigned int index) const;

  virtual
  void
  addJFloat(float value);

  virtual
  float // java float value
  readJFloat(unsigned int index) const;

  virtual
  void
  addJDouble(double value);

  virtual
  double // java double value
  readJDouble(unsigned int index) const;

  virtual
  void
  addJString(const StlString & value);

  virtual
  const StlString & // java string value
  readJString(unsigned int index) const;

//-------- Member attributes -------------------------------------------------
protected:

  struct Value
  {
  JavaData::ValueType t;
  union
    {
    unsigned char Z;
    char B;
    unsigned short C;
    short S;
    int I;
    long long J;
    float F;
    double D;
    StlString * str;
    } v;
  };

  StlVector<JavaData::Value> _values;
};

//----------------------------------------------------------------------------
// Class JavaObject
//----------------------------------------------------------------------------

class JavaObject : public ArObject
{
//-------- Type information --------------------------------------------------
public:

  AR_CLASS(JavaObject)

//-------- Construction / Destruction ----------------------------------------

  AR_CONSTRUCTOR(JavaObject)

//-------- Communication -----------------------------------------------------

  virtual
  ArRef<JavaData> // result
  toJava(ArConstRef<JavaData> data);

  virtual
  ArRef<JavaData> // result
  fromJava(ArConstRef<JavaData> data);

  static
  bool // application is embeded in Java
  inJavaApp(void);

//-------- Forbidden operations ----------------------------------------------
protected:

  static
  void *
  _nativeToAReVi(void * self,
                 void * array);

  static
  int
  _nativeWindowID(void * cls,
                  void * component);

//-------- Member attributes -------------------------------------------------
protected:

  void * _javaRef;

//-------- Static attributes -------------------------------------------------
protected:

  friend bool JAVA_init(void);
  friend void JAVA_destroy(void);
  static bool _initSuccess;
  static void * (*_newFnct)(JavaObject *);
  static void (*_deleteFnct)(void *);
  static ArRef<JavaData> (*_toJavaFnct)(void *,ArConstRef<JavaData>);
};

} // namespace AReVi

#endif // AREVI_UTILS_JAVAOBJECT_H

//----------------------------------------------------------------------------
