// LI2/ENIB - AReVi - Utility services
//----------------------------------------------------------------------------
// file        : AReVi/Utils/math_priv.h
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVI_UTILS_MATH_PRIV_H
#define AREVI_UTILS_MATH_PRIV_H 1

#include "AReVi/Utils/platform_priv.h"

#include <math.h>

namespace AReVi {

//-------- Angles ------------------------------------------------------------

#ifndef M_PI // pi
#define M_PI 3.14159265358979323846
#endif

#ifndef M_PI_2 // pi/2
#define M_PI_2 1.57079632679489661923
#endif

#ifndef M_2PI // 2*pi
#define M_2PI 6.28318530717958647692
#endif

#ifndef M_180_PI // 180/pi
#define M_180_PI 57.2957795130823228646
#endif

#ifndef M_PI_180 // pi/180
#define M_PI_180 0.01745329251994329547
#endif

double
betweenPi(double angle);

double
degreeToRadian(double angle);

double
radianToDegree(double angle);

//-------- Cartesian 2D <---> Polar ------------------------------------------

double
cart2DToAngle(double x,
              double y);

double
cart2DToDistance(double x,
                 double y);

void
cart2DToPolar(double & distanceOut,
              double & angleOut,
              double x,
              double y);

void
polarToCart2D(double & xOut,
              double & yOut,
              double distance,
              double angle);

//-------- Cartesian 3D <---> Spherical --------------------------------------

void
cart3DToAngles(double & yawOut,
               double & pitchOut,
               double x,
               double y,
               double z);

double
cart3DToDistance(double x,
                 double y,
                 double z);

void
cart3DToSpherical(double & distanceOut,
                  double & yawOut,
                  double & pitchOut,
                  double x,
                  double y,
                  double z);

void
sphericalToCart3D(double & xOut,
                  double & yOut,
                  double & zOut,
                  double distance,
                  double yaw,
                  double pitch);

//-------- Sign extensions ---------------------------------------------------

double
signedSquare(double x); // keeps the sign of x

double
signedSqrt(double x); // keeps the sign of x

//-------- Interpolation -----------------------------------------------------

void
interpolationIndex(unsigned int & previousIndexOut,
                   double & nextPreviousRatioOut,
                   unsigned int nbElems,
                   double cursor);

} // namespace AReVi

#endif // AREVI_UTILS_MATH_PRIV_H

//----------------------------------------------------------------------------
