// LI2/ENIB - AReVi - Utility services
//----------------------------------------------------------------------------
// file        : AReVi/Utils/memoryBlock.h
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVI_UTILS_MEMORYBLOCK_H
#define AREVI_UTILS_MEMORYBLOCK_H 1

#include "AReVi/arObject.h"

namespace AReVi {

extern "C" bool ZLib_init(const StlVector<StlString> *);
extern "C" void ZLib_destroy(void);

//----------------------------------------------------------------------------
// Class MemoryBlock
//----------------------------------------------------------------------------

class MemoryBlock : public ArObject
{
//-------- Type information --------------------------------------------------
public:

  AR_CLASS(MemoryBlock)

//-------- Construction / Destruction ----------------------------------------

  AR_CONSTRUCTOR(MemoryBlock)

//-------- Memory management -------------------------------------------------

  AR_UNSTUBBED virtual
  const char * // memory area start
  getMemory(void) const;

  AR_UNSTUBBED virtual
  char * // memory area start
  accessMemory(void);

  virtual
  bool // success
  setSize(unsigned int size);

  virtual
  bool // success
  ensureMinSize(unsigned int size);

  virtual
  unsigned int // memory area size
  getSize(void) const;

  virtual
  bool // success
  swapBlock(ArRef<MemoryBlock> block);

  virtual
  bool // success
  compressFrom(ArConstRef<MemoryBlock> uncompressedBlock);

  // level [-1,9] => -1 is default compression
  //                  0  no compression
  //                  1-9 compression level (1-faster,9-slower)
  virtual
  bool // success
  compressFrom(ArConstRef<MemoryBlock> uncompressedBlock, int level);

  // memoryBlock size must be large enough to store uncompressed data!!!
  virtual
  bool // success
  uncompressFrom(ArConstRef<MemoryBlock> compressedBlock);

//-------- Member attributes -------------------------------------------------
protected:
  friend bool ZLib_init(const StlVector<StlString> *);
  friend void ZLib_destroy(void);

  unsigned int _size;
  char * _memory;
  bool _canSwap;

//-------- Static attributes -------------------------------------------------
protected:
  static bool _initSuccess;
  static bool (* _compress)(ArConstRef<MemoryBlock>, ArRef<MemoryBlock>);
  static bool (* _compress2)(ArConstRef<MemoryBlock>, ArRef<MemoryBlock>, int level);
  static bool (* _uncompress)(ArConstRef<MemoryBlock>, ArRef<MemoryBlock>);
};


//----------------------------------------------------------------------------
// Class RigidMemoryBlock
//----------------------------------------------------------------------------

class RigidMemoryBlock : public MemoryBlock
{
//-------- Type information --------------------------------------------------
public:

  AR_CLASS(RigidMemoryBlock)

//-------- Construction / Destruction ----------------------------------------

  AR_UNSTUBBED
  AR_CONSTRUCTOR_3(RigidMemoryBlock,
                   char *, memory,
                   unsigned int, size,
                   bool, writable)

//-------- Memory management -------------------------------------------------

  AR_UNSTUBBED virtual
  char * // memory area start (if writable)
  accessMemory(void);

  virtual
  bool // success (always fails)
  setSize(unsigned int size);

  virtual
  bool // success (always fails)
  ensureMinSize(unsigned int size);

  virtual
  bool // can write into memory area
  isWritable(void) const;

//-------- Member attributes -------------------------------------------------
protected:

  bool _writable;
};

} // namespace AReVi

#endif // AREVI_UTILS_MEMORYBLOCK_H

//----------------------------------------------------------------------------
