// LI2/ENIB - AReVi - Utility services
//----------------------------------------------------------------------------
// file        : AReVi/Utils/memoryStream.h
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVI_UTILS_MEMORYSTREAM_H
#define AREVI_UTILS_MEMORYSTREAM_H 1

#include "AReVi/Utils/abstractStream.h"

namespace AReVi {

class MemoryBlock;

//----------------------------------------------------------------------------
// Class MemoryIStream
//----------------------------------------------------------------------------

class MemoryIStream : public AbstractIStream
{
//-------- Type information --------------------------------------------------
public:

  AR_CLASS(MemoryIStream)

//-------- Construction / Destruction ----------------------------------------

  AR_CONSTRUCTOR_2(MemoryIStream,
                   ArConstRef<MemoryBlock>, memory,
                   unsigned int, offset);

//-------- Memory informations -----------------------------------------------

  virtual
  ArConstRef<MemoryBlock> // input memory block
  getMemoryBlock(void) const;

  virtual
  unsigned int // amount of extracted memory
  getMemoryUsage(void) const;

  virtual
  unsigned int // where extraction begins
  getOffset(void) const;

//-------- Member attributes -------------------------------------------------
protected:

  ArConstRef<MemoryBlock> _memory;
};


//----------------------------------------------------------------------------
// Class MemoryOStream
//----------------------------------------------------------------------------

class MemoryOStream : public AbstractOStream
{
//-------- Type information --------------------------------------------------
public:

  AR_CLASS(MemoryOStream)

//-------- Construction / Destruction ----------------------------------------

  AR_CONSTRUCTOR_2(MemoryOStream,
                   ArRef<MemoryBlock>, memory,
                   unsigned int, offset)

//-------- Memory informations -----------------------------------------------

  virtual
  ArRef<MemoryBlock> // output memory block
  getMemoryBlock(void) const;

  virtual
  unsigned int // amount of filled memory
  getMemoryUsage(void) const;

  virtual
  unsigned int // where fill begins
  getOffset(void) const;

//-------- Member attributes -------------------------------------------------
protected:

  ArRef<MemoryBlock> _memory;
};

} // namespace AReVi

#endif // AREVI_UTILS_MEMORYSTREAM_H

//----------------------------------------------------------------------------
