// LI2/ENIB - AReVi - Utility services
//----------------------------------------------------------------------------
// file        : AReVi/Utils/process.h
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVI_UTILS_PROCESS_H
#define AREVI_UTILS_PROCESS_H 1

#include "AReVi/arSystem.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class Process
//----------------------------------------------------------------------------

class Process : public ArObject
{
//-------- Type information --------------------------------------------------
public:

  AR_CLASS(Process)

//-------- Construction / Destruction ----------------------------------------

  AR_CONSTRUCTOR_4(Process,
                   const StlVector<StlString> &, commandLine,
                   bool, redirectInput,
                   bool, redirectOutput,
                   bool, redirectError);

  AR_CONSTRUCTOR_4(Process,
                   const StlString &, shellCommand,
                   bool, redirectInput,
                   bool, redirectOutput,
                   bool, redirectError);

//-------- Inspection --------------------------------------------------------

  virtual
  ArSystem::ProcessID // process identifier
  getID(void) const;

//-------- I/O redirection ---------------------------------------------------

  virtual
  int // >=0: input file descriptor  -1: not redirected
  getInputFD(void) const;

  virtual
  void
  closeInputFD(void);

  virtual
  int // >=0: output file descriptor  -1: not redirected
  getOutputFD(void) const;

  virtual
  void
  closeOutputFD(void);

  virtual
  int // >=0: error file descriptor  -1: not redirected
  getErrorFD(void) const;

  virtual
  void
  closeErrorFD(void);

//-------- Ending ------------------------------------------------------------

  virtual
  bool // process end reached
  isEnded(void) const;

  virtual
  int // status after process end
  waitStatus(void);

//-------- External commands embedding ---------------------------------------

  static
  int // status
  execute(const StlString & shellCommand);

  static
  int // status
  executeI(const StlString & shellCommand,
           const StlString & input);

  static
  int // status
  executeO(const StlString & shellCommand,
           StlString & outputOut);

  static
  int // status
  executeE(const StlString & shellCommand,
           StlString & errorOut);

  static
  int // status
  executeIO(const StlString & shellCommand,
            const StlString & input,
            StlString & outputOut);

  static
  int // status
  executeIE(const StlString & shellCommand,
            const StlString & input,
            StlString & errorOut);

  static
  int // status
  executeOE(const StlString & shellCommand,
            StlString & outputOut,
            StlString & errorOut);

  static
  int // status
  executeIOE(const StlString & shellCommand,
             const StlString & input,
             StlString & outputOut,
             StlString & errorOut);

//-------- Forbidden operations ----------------------------------------------
protected:

  virtual
  void
  _start(const StlVector<StlString> & commandLine,
         bool redirectInput,
         bool redirectOutput,
         bool redirectError);

//-------- Member attributes -------------------------------------------------
protected:

  ArSystem::ProcessID _id;
  bool _ended;
  int _result;
  int _inputPipe[2];
  int _outputPipe[2];
  int _errorPipe[2];
};

} // namespace AReVi

#endif // AREVI_UTILS_PROCESS_H

//----------------------------------------------------------------------------
