// LI2/ENIB - AReVi - Utility services
//----------------------------------------------------------------------------
// file        : AReVi/Utils/regex.h
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVI_UTILS_REGEX_H
#define AREVI_UTILS_REGEX_H 1

#include "AReVi/arObject.h"

#include <regex.h>

namespace AReVi {

//----------------------------------------------------------------------------
// Class RegularExpression
//----------------------------------------------------------------------------

class RegularExpression : public ArObject
{
//-------- Type information --------------------------------------------------
public:

  AR_CLASS(RegularExpression)

//-------- Construction / Destruction ----------------------------------------

  AR_CONSTRUCTOR_1(RegularExpression,
                   const StlString &, pattern)

//-------- Pattern matching --------------------------------------------------

  virtual
  const StlString & // pattern to match
  getPattern(void) const;

  virtual
  bool // success
  match(const StlString & input,
        unsigned int & startOut,
        unsigned int & endOut) const;

  virtual
  bool // success
  detailledMatch(const StlString & input,
                 StlVector<int> & startOut,
                 StlVector<int> & endOut) const;

//-------- String management -------------------------------------------------

  virtual
  unsigned int // number of replacements done
  replace(StlString & strInOut,
          const StlString & replacement,
          unsigned int maxReplacement) const;

  virtual
  void
  split(const StlString & input,
        StlVector<StlString> & subStringsOut,
        unsigned int maxSplit) const;

//-------- Member attributes -------------------------------------------------
protected:

  regex_t _regex;
  regmatch_t * _match;
  StlString _pattern;
};

} // namespace AReVi

#endif // AREVI_UTILS_REGEX_H

//----------------------------------------------------------------------------
