// LI2/ENIB - AReVi - Utility services
//----------------------------------------------------------------------------
// file        : AReVi/Utils/spaceMouse.h
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVI_UTILS_SPACEMOUSE_H
#define AREVI_UTILS_SPACEMOUSE_H 1

#include "AReVi/Utils/inputDevice.h"

#include <termios.h>

namespace AReVi {

//----------------------------------------------------------------------------
// Class SpaceMouse
//----------------------------------------------------------------------------

class SpaceMouse : public InputDevice
{
//-------- Type information --------------------------------------------------
public:

  AR_CLASS(SpaceMouse)

//-------- Construction / Destruction ----------------------------------------

  AR_CONSTRUCTOR_1(SpaceMouse,
                   const StlString &, device)

//-------- Device management -------------------------------------------------

  virtual
  bool // success
  reset(const StlString & device);

  virtual
  void
  update(void);

//-------- Translation / Rotation  -------------------------------------------

  virtual
  void
  getTranslation(double & txOut,
                 double & tyOut,
                 double & tzOut) const;

  virtual
  void
  getRotation(double & rollOut,
              double & pitchOut,
              double & yawOut) const;

//-------- Forbidden operations ----------------------------------------------
protected:

  virtual
  bool // success
  _send(const char * msg,
        unsigned int timeOut);

  virtual
  bool // line available
  _readLine(void);

//-------- Member attributes -------------------------------------------------
protected:

  int _fd;
  struct termios _savedTermios;
  char _line[0x400];
  unsigned int _lineSize;
  unsigned int _lineLimit;
};

} // namespace AReVi

#endif // AREVI_UTILS_SPACEMOUSE_H

//----------------------------------------------------------------------------
