// LI2/ENIB - AReVi - Utility services
//----------------------------------------------------------------------------
// file        : AReVi/Utils/stlEnumerator_priv.h
// author(s)   : L. Coquelle
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVI_UTILS_STLENUMERATOR_PRIV_H
#define AREVI_UTILS_STLENUMERATOR_PRIV_H 1

namespace AReVi {

template <class Container>
class StlEnumerator
{
public:

  inline
  StlEnumerator(Container & container);

  inline
  StlEnumerator(typename Container::iterator begin,
                typename Container::iterator end);

  inline
  void
  init(Container & container);

  inline
  void
  init(typename Container::iterator begin,
       typename Container::iterator end);

  inline
  bool
  moveNext(void);

  inline
  typename Container::value_type &
  current(void) const;

  inline
  typename Container::value_type &
  operator()(void) const;

private:
  typename Container::iterator _cur;
  typename Container::iterator _it;
  typename Container::iterator _end;
};


template <class Container>
class StlConstEnumerator
{
public:

  inline
  StlConstEnumerator(const Container & container);

  inline
  StlConstEnumerator(typename Container::const_iterator begin,
                     typename Container::const_iterator end);

  inline
  void
  init(const Container & container);

  inline
  void
  init(typename Container::const_iterator begin,
       typename Container::const_iterator end);

  inline
  bool
  moveNext(void);

  inline
  const typename Container::value_type &
  current(void);

  inline
  const typename Container::value_type &
  operator()(void);

private:
  typename Container::const_iterator _cur;
  typename Container::const_iterator _it;
  typename Container::const_iterator _end;
};


template <class Container> inline
StlEnumerator<Container>::StlEnumerator(Container & container)
: _cur(container.begin()),
  _it(container.begin()),
  _end(container.end())
{
}

template <class Container> inline
StlEnumerator<Container>::StlEnumerator(typename Container::iterator begin,
                                        typename Container::iterator end)
: _cur(begin),
  _it(begin),
  _end(end)
{
}

template <class Container> inline
void
StlEnumerator<Container>::init(Container & container)
{
_cur=_it=container.begin();
_end=container.end();
}

template <class Container> inline
void
StlEnumerator<Container>::init(typename Container::iterator begin,
                               typename Container::iterator end)
{
_cur=_it=begin;
_end=end;
}

template <class Container> inline
bool
StlEnumerator<Container>::moveNext(void)
{
if(_it==_end)
  {
  return(false);
  }
else
  {
  _cur=_it++;
  return(true);
  }
}

template <class Container> inline
typename Container::value_type &
StlEnumerator<Container>::current(void) const
{
return(*_cur);
}

template <class Container> inline
typename Container::value_type &
StlEnumerator<Container>::operator()(void) const
{
return(current());
}

template <class Container> inline
StlConstEnumerator<Container>::StlConstEnumerator(const Container & container)
: _cur(container.begin()),
  _it(container.begin()),
  _end(container.end())
{
}

template <class Container> inline
StlConstEnumerator<Container>::StlConstEnumerator(
                             typename Container::const_iterator begin,
                             typename Container::const_iterator end)
: _cur(begin),
  _it(begin),
  _end(end)
{
}

template <class Container> inline
void
StlConstEnumerator<Container>::init(const Container & container)
{
_cur=_it=container.begin();
_end=container.end();
}

template <class Container> inline
void
StlConstEnumerator<Container>::init(typename Container::const_iterator begin,
                                    typename Container::const_iterator end)
{
_cur=_it=begin;
_end=end;
}

template <class Container> inline
bool
StlConstEnumerator<Container>::moveNext(void)
{
if(_it==_end)
  {
  return(false);
  }
else
  {
  _cur=_it++;
  return(true);
  }
}

template <class Container> inline
const typename Container::value_type &
StlConstEnumerator<Container>::current(void)
{
return(*_cur);
}

template <class Container> inline
const typename Container::value_type &
StlConstEnumerator<Container>::operator()(void)
{
return(current());
}

} // namespace AReVi

#endif // AREVI_UTILS_STLENUMERATOR_PRIV_H

//----------------------------------------------------------------------------
