// LI2/ENIB - AReVi - Utility services
//----------------------------------------------------------------------------
// file        : AReVi/Utils/stlString_priv.h
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVI_UTILS_STLSTRING_PRIV_H
#define AREVI_UTILS_STLSTRING_PRIV_H 1

#include "AReVi/Utils/stlMisc_priv.h"

#include <string>

namespace AReVi {

typedef string StlString;

//-------- Write like a string constant --------------------------------------

void
strDump(StlString & resultOut,
        const StlString & str);

//-------- UTF8 conversions --------------------------------------------------

StlString // UTF8 string
strToUTF8(const StlString & str);

StlString // ascii string
strFromUTF8(const StlString & utf8str);

//-------- Suppress spaces ---------------------------------------------------

StlString // corrected string
strNoTrailingSpace(const StlString & str);

StlString // corrected string
strNoLeadingSpace(const StlString & str);

//-------- File name operations ----------------------------------------------

StlString // extracted base name
strBasename(const StlString & fileName);

StlString // extracted directory name
strDirname(const StlString & fileName);

StlString // extracted extension
strExtension(const StlString & fileName);

//-------- StlString <---> pointer -------------------------------------------

bool // success
strToPtr(void * & pOut,
         const StlString & str);

StlString // formated string
ptrToStr(const void * p);

//-------- StlString <---> char ----------------------------------------------

bool // success
strToChar(char & cOut,
          const StlString & str);

StlString // formated string
charToStr(char c);

StlString // formated string
charToHexaStr(char c);

//-------- StlString <---> unsigned char -------------------------------------

bool // success
strToUchar(unsigned char & ucOut,
           const StlString & str);

StlString // formated string
ucharToStr(unsigned char uc);

StlString // formated string
ucharToHexaStr(unsigned char uc);

//-------- StlString <---> short ---------------------------------------------

bool // success
strToShort(short & sOut,
           const StlString & str);

StlString // formated string
shortToStr(short s);

StlString // formated string
shortToHexaStr(short s);

//-------- StlString <---> unsigned short ------------------------------------

bool // success
strToUshort(unsigned short & usOut,
            const StlString & str);

StlString // formated string
ushortToStr(unsigned short us);

StlString // formated string
ushortToHexaStr(unsigned short us);

//-------- StlString <---> int -----------------------------------------------

bool // success
strToInt(int & iOut,
         const StlString & str);

StlString // formated string
intToStr(int i);

StlString // formated string
intToHexaStr(int i);

//-------- StlString <---> unsigned int --------------------------------------

bool // success
strToUint(unsigned int & uiOut,
          const StlString & str);

StlString // formated string
uintToStr(unsigned int ui);

StlString // formated string
uintToHexaStr(unsigned int ui);

//-------- StlString <---> long long -----------------------------------------

bool // success
strToLlong(long long & llOut,
           const StlString & str);

StlString // formated string
llongToStr(long long ll);

StlString // formated string
llongToHexaStr(long long ll);

//-------- StlString <---> unsigned long long --------------------------------

bool // success
strToUllong(unsigned long long & ullOut,
            const StlString & str);

StlString // formated string
ullongToStr(unsigned long long ull);

StlString // formated string
ullongToHexaStr(unsigned long long ull);

//-------- StlString <---> float ---------------------------------------------

bool // success
strToFloat(float & fOut,
           const StlString & str);

StlString // formated string
floatToStr(float f,
           bool showReal=false);

//-------- StlString <---> double --------------------------------------------

bool // success
strToDouble(double & dOut,
            const StlString & str);

StlString // formated string
doubleToStr(double d,
            bool showReal=false);

//-------- StlString <---> IP Address ----------------------------------------

bool // success
strToIPAddr(unsigned int & addrOut,
            const StlString & str);

StlString // formated string
ipAddrToStr(unsigned int addr);

} // namespace AReVi

#endif // AREVI_UTILS_STLSTRING_PRIV_H

//----------------------------------------------------------------------------
