// LI2/ENIB - AReVi - Utility services
//----------------------------------------------------------------------------
// file        : AReVi/Utils/tclCommand.h
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVI_UTILS_TCLCOMMAND_H
#define AREVI_UTILS_TCLCOMMAND_H 1

#include "AReVi/arObject.h"

#define DECLARE_TCL_AREVI_APP(_INITFNCT,_UPDATEFNCT,_DESTROYFNCT)      \
  namespace AReVi {                                                    \
  int TCL_InitAReViApp(void *,bool (*)(int,char **),                   \
                       bool (*)(void),void (*)(void));                 \
  }                                                                    \
  extern "C" int Areviapp_Init(void * tclInterp)                       \
  {                                                                    \
  return(AReVi::TCL_InitAReViApp(tclInterp,                            \
                                 _INITFNCT,_UPDATEFNCT,_DESTROYFNCT)); \
  }


namespace AReVi {

struct _TclCommandInfo;

//----------------------------------------------------------------------------
// Class TclCommand (abstract)
//----------------------------------------------------------------------------

class TclCommand : public ArObject
{
//-------- Type information --------------------------------------------------
public:

  AR_CLASS(TclCommand)

//-------- Tcl invocation ----------------------------------------------------

  static
  bool // command is complete
  isComplete(const StlString & command);

  static
  bool // success
  eval(const StlString & command,
       StlString & resultOut);

  static
  bool // success
  evalURL(const StlString & url,
          StlString & resultOut);

//-------- Command management ------------------------------------------------

  template <class T1,class T2> inline static
  bool // success
  add(const StlString & commandName,
      ArRef<T1> obj,
      bool (T2::*method)(const StlVector<StlString> &,StlString &));

  static
  void
  remove(const StlString & commandName);

  static
  void
  getCommandNames(StlVector<StlString> & commandNamesOut);

  template <class T1,class T2> inline static
  bool // success
  getCommand(const StlString & commandName,
             ArRef<T1> & objOut,
             bool (T2::* & methodOut)(const StlVector<StlString> &,
                                      StlString &));

  static
  bool // application is embeded in TCL
  inTclApp(void);

//-------- Forbidden operations ----------------------------------------------
protected:

  TclCommand(ArCW & arCW);

//-------- Static attributes -------------------------------------------------
protected:

  friend bool TCL_init(void);
  friend void TCL_destroy(void);

  static bool _initSuccess;
  static StlVector<_TclCommandInfo *> * _commands;
  static bool (*_isCompleteFnct)(const StlString &);
  static bool (*_evalFnct)(const StlString &,StlString &);
  static bool (*_evalURLFnct)(const StlString &,StlString &);
  static void (*_addFnct)(_TclCommandInfo *);
  static void (*_removeFnct)(_TclCommandInfo *);
};

} // namespace AReVi

//-------- Inline source code ------------------------------------------------

#include "AReVi/Utils/tclCommand.Ci"

#endif // AREVI_UTILS_TCLCOMMAND_H

//----------------------------------------------------------------------------
