// LI2/ENIB - AReVi - Utility services
//----------------------------------------------------------------------------
// file        : AReVi/Utils/thread.h
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVI_UTILS_THREAD_H
#define AREVI_UTILS_THREAD_H 1

#include "AReVi/arObject.h"

#include <pthread.h>

namespace AReVi {

//----------------------------------------------------------------------------
// Class Thread
//----------------------------------------------------------------------------

class Thread : public ArObject
{
//-------- Type information --------------------------------------------------
public:

  AR_CLASS(Thread)

//-------- Construction / Destruction ----------------------------------------

  AR_CONSTRUCTOR(Thread)

//-------- Execution control -------------------------------------------------

  template <class T1,class T2> inline
  bool // success
  run(ArRef<T1> obj,
      void (T2::*method)(void)); // template --> NOT virtual !

  virtual
  bool // thread is running
  isRunning(void) const;

  virtual
  bool // success
  waitEnd(void) const;

//-------- Forbidden operations ----------------------------------------------
protected:

  bool // success
  _start(void);

  static
  void *
  _threadTask(void *);

//-------- Member attributes -------------------------------------------------
protected:

  bool _started;
  bool _ended;
  ArRef<ArObject> _obj;
  void (ArObject::*_method)(void);
  pthread_t _th;
};

template <class T1,class T2> inline
bool // success
Thread::run(ArRef<T1> obj,
            void (T2::*method)(void)) // template --> NOT virtual !
{
if(_started&&!_ended)
  {
  return(false);
  }
else
  {
  T2 * dummy=(T1 *)0; // ensure T1 inherits from T2
  (void)dummy;
  _obj=obj;
  _method=static_cast<void (ArObject::*)(void)>(method);
  return(_start());
  }
}


//----------------------------------------------------------------------------
// Class ThreadMutex
//----------------------------------------------------------------------------

class ThreadMutex : public ArObject
{
//-------- Type information --------------------------------------------------
public:

  AR_CLASS(ThreadMutex)

//-------- Construction / Destruction ----------------------------------------

  AR_CONSTRUCTOR(ThreadMutex);

//-------- Execution control -------------------------------------------------

  virtual
  void
  lock(void);

  virtual
  void
  unlock(void);

  virtual
  bool // lock success
  tryLock(void);

//-------- Member attributes -------------------------------------------------
protected:

  pthread_mutex_t _mtx;
};


//----------------------------------------------------------------------------
// Class RecursiveThreadMutex
//----------------------------------------------------------------------------

class RecursiveThreadMutex : public ThreadMutex
{
//-------- Type information --------------------------------------------------
public:

  AR_CLASS(RecursiveThreadMutex)

//-------- Construction / Destruction ----------------------------------------

  AR_CONSTRUCTOR(RecursiveThreadMutex);

//-------- Execution control -------------------------------------------------

  virtual
  void
  lock(void);

  virtual
  void
  unlock(void);

  virtual
  bool // lock success
  tryLock(void);

//-------- Member attributes -------------------------------------------------
protected:

  unsigned int _count;
  pthread_t _th;
  pthread_cond_t _cond;
};


//----------------------------------------------------------------------------
// Class ThreadCondition
//----------------------------------------------------------------------------

class ThreadCondition : public ArObject
{
//-------- Type information --------------------------------------------------
public:

  AR_CLASS(ThreadCondition)

//-------- Construction / Destruction ----------------------------------------

  AR_CONSTRUCTOR(ThreadCondition);

//-------- Execution control -------------------------------------------------

  virtual
  void
  enter(void);

  virtual
  void
  leave(void);

  virtual
  void
  wait(void);

  virtual
  void
  broadcast(void);

//-------- Member attributes -------------------------------------------------
protected:

  pthread_cond_t _cond;
  pthread_mutex_t _mtx;
};

} // namespace AReVi

#endif // AREVI_UTILS_THREAD_H

//----------------------------------------------------------------------------
