// LI2/ENIB - AReVi - VRML services
//----------------------------------------------------------------------------
// file        : AReVi/VRML/appearanceNode_priv.h
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVI_VRML_APPEARANCENODE_PRIV_H
#define AREVI_VRML_APPEARANCENODE_PRIV_H 1

#include "AReVi/VRML/vrmlNode_priv.h"

namespace AReVi {

class MaterialNode;
class TextureNode;
class TextureTransformNode;

//----------------------------------------------------------------------------
// Class AppearanceNode
//----------------------------------------------------------------------------

class AppearanceNode : public VrmlNode
{
friend class VrmlNode;
friend class ShapeNode;

//-------- Construction / Destruction ----------------------------------------
public:

  virtual
  ~AppearanceNode(void);

//-------- Field management --------------------------------------------------

  virtual
  void
  getNodeFieldNames(StlVector<StlString> & nodeFieldNamesOut) const;

  virtual
  const VrmlNode * // node at field
  getNodeField(const StlString & fieldName) const;

  virtual
  void
  addField(const StlString & fieldName,
           VrmlValue * fieldValue);

  virtual
  void
  check(void);

//-------- Rendering ---------------------------------------------------------

  virtual
  void
  postMaterialRebuild(void);

  virtual
  void
  postTextureRebuild(const TextureNode * from);

  virtual
  void
  postTextureTransformRebuild(const TextureTransformNode * from);

//-------- Forbidden operations ----------------------------------------------
protected:

  virtual
  void
  _generateFields(ostream & output,
                  unsigned int indent,
                  bool showDefaultValues,
                  StlVector<const VrmlNode *> & defNodesInOut) const;

  AppearanceNode(const StlString & nodeType,
                 VrmlShape3D * shape);

private:

  AppearanceNode(void); // no implementation

  AppearanceNode(const AppearanceNode & obj); // no implementation

  AppearanceNode & // itself
  operator=(const AppearanceNode & obj); // no implementation

//-------- Member attributes -------------------------------------------------
protected:

  MaterialNode * _materialField;
  TextureNode * _textureField;
  TextureTransformNode * _textureTransformField;
  TextureNode * _texture2Field;
  TextureTransformNode * _textureTransform2Field;
  TextureNode * _texture3Field;
  TextureTransformNode * _textureTransform3Field;
  TextureNode * _texture4Field;
  TextureTransformNode * _textureTransform4Field;
};

} // namespace AReVi

#endif // AREVI_VRML_APPEARANCENODE_PRIV_H

//----------------------------------------------------------------------------
