// LI2/ENIB - AReVi - VRML services
//----------------------------------------------------------------------------
// file        : AReVi/VRML/groupNode_priv.h
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVI_VRML_GROUPNODE_PRIV_H
#define AREVI_VRML_GROUPNODE_PRIV_H 1

#include "AReVi/VRML/groupChildNode_priv.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class GroupNode
//----------------------------------------------------------------------------

class GroupNode : public GroupChildNode
{
friend class VrmlNode;

//-------- Construction / Destruction ----------------------------------------
public:

  virtual
  ~GroupNode(void);

//-------- Inspection --------------------------------------------------------

  virtual
  bool // is a group
  isGroup(void) const;

//-------- Field management --------------------------------------------------

  virtual
  void
  addField(const StlString & fieldName,
           VrmlValue * fieldValue);

  virtual
  void
  check(void);

  virtual
  void
  clearChildren(void);

  virtual
  unsigned int
  getNbChildren(void) const;

  virtual
  const GroupChildNode *
  getChild(unsigned int childIndex) const;

  virtual
  GroupChildNode *
  accessChild(unsigned int childIndex);

//-------- Rendering ---------------------------------------------------------

  virtual
  void
  buildParts(const StlVector<TransformNode *> & transformPath);

  virtual
  void
  rebuild(void);

  virtual
  void
  postTransformation(void);

//-------- Forbidden operations ----------------------------------------------
protected:

  virtual
  void
  _generateFields(ostream & output,
                  unsigned int indent,
                  bool showDefaultValues,
                  StlVector<const VrmlNode *> & defNodesInOut) const;

  GroupNode(const StlString & nodeType,
            VrmlShape3D * shape);

private:

  GroupNode(void); // no implementation

  GroupNode(const GroupNode & obj); // no implementation

  GroupNode & // itself
  operator=(const GroupNode & obj); // no implementation

//-------- Member attributes -------------------------------------------------
protected:

  StlVector<GroupChildNode *> _childrenField;
  Util3D::Dbl3 _bboxCenterField;
  Util3D::Dbl3 _bboxSizeField;
};

} // namespace AReVi

#endif // AREVI_VRML_GROUPNODE_PRIV_H

//----------------------------------------------------------------------------
