// LI2/ENIB - AReVi - VRML services
//----------------------------------------------------------------------------
// file        : AReVi/VRML/indexedFaceSetNode_priv.h
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVI_VRML_INDEXEDFACESETNODE_PRIV_H
#define AREVI_VRML_INDEXEDFACESETNODE_PRIV_H 1

#include "AReVi/VRML/geometryNode_priv.h"

namespace AReVi {

class ColorNode;
class CoordinateNode;
class NormalNode;
class TextureCoordinateNode;

//----------------------------------------------------------------------------
// Class IndexedFaceSetNode
//----------------------------------------------------------------------------

class IndexedFaceSetNode : public GeometryNode
{
friend class VrmlNode;
friend class VrmlIndexedFaceSetInteractor;

//-------- Construction / Destruction ----------------------------------------
public:

  virtual
  ~IndexedFaceSetNode(void);

//-------- Field management --------------------------------------------------

  virtual
  void
  getNodeFieldNames(StlVector<StlString> & nodeFieldNamesOut) const;

  virtual
  const VrmlNode * // node at field
  getNodeField(const StlString & fieldName) const;

  virtual
  void
  addField(const StlString & fieldName,
           VrmlValue * fieldValue);

  virtual
  void
  check(void);

//-------- Rendering ---------------------------------------------------------

  virtual
  ArRef<ShapePart3D> // built part
  buildPart(Util3D::Transform & initTransformOut) const;

  virtual
  void
  updatePart(ArRef<ShapePart3D> part,
             const MaterialNode * material) const;

//-------- Forbidden operations ----------------------------------------------
protected:

  virtual
  void
  _generateFields(ostream & output,
                  unsigned int indent,
                  bool showDefaultValues,
                  StlVector<const VrmlNode *> & defNodesInOut) const;

  IndexedFaceSetNode(const StlString & nodeType,
                     VrmlShape3D * shape);

private:

  IndexedFaceSetNode(void); // no implementation

  IndexedFaceSetNode(const IndexedFaceSetNode & obj); // no implementation

  IndexedFaceSetNode & // itself
  operator=(const IndexedFaceSetNode & obj); // no implementation

//-------- Member attributes -------------------------------------------------
protected:

  ColorNode * _colorField;
  CoordinateNode * _coordField;
  NormalNode * _normalField;
  TextureCoordinateNode * _texCoordField;
  TextureCoordinateNode * _texCoord2Field;
  TextureCoordinateNode * _texCoord3Field;
  TextureCoordinateNode * _texCoord4Field;
  bool _ccwField;
  StlVector<StlVector<int> > _colorIndexField;
  bool _colorPerVertexField;
  bool _convexField;
  StlVector<StlVector<int> > _coordIndexField;
  double _creaseAngleField;
  StlVector<StlVector<int> > _normalIndexField;
  bool _normalPerVertexField;
  bool _solidField;
  StlVector<StlVector<int> > _texCoordIndexField;
  StlVector<StlVector<int> > _texCoordIndex2Field;
  StlVector<StlVector<int> > _texCoordIndex3Field;
  StlVector<StlVector<int> > _texCoordIndex4Field;
};

} // namespace AReVi

#endif // AREVI_VRML_INDEXEDFACESETNODE_PRIV_H

//----------------------------------------------------------------------------
