// LI2/ENIB - AReVi - VRML services
//----------------------------------------------------------------------------
// file        : AReVi/VRML/materialNode_priv.h
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVI_VRML_MATERIALNODE_PRIV_H
#define AREVI_VRML_MATERIALNODE_PRIV_H 1

#include "AReVi/VRML/vrmlNode_priv.h"

namespace AReVi {

class Material3D;
class Surface3D;

//----------------------------------------------------------------------------
// Class MaterialNode
//----------------------------------------------------------------------------

class MaterialNode : public VrmlNode
{
friend class VrmlNode;
friend class VrmlMaterialInteractor;
friend class PointSetNode;
friend class IndexedLineSetNode;

//-------- Construction / Destruction ----------------------------------------
public:

  virtual
  ~MaterialNode(void);

//-------- Field management --------------------------------------------------

  virtual
  void
  addField(const StlString & fieldName,
           VrmlValue * fieldValue);

  virtual
  void
  check(void);

//-------- Rendering ---------------------------------------------------------

  virtual
  void
  postRebuild(void);

  virtual
  void
  rebuild(void);

  virtual
  void
  updatePart(ArRef<Surface3D> part) const;

//-------- Forbidden operations ----------------------------------------------
protected:

  virtual
  void
  _generateFields(ostream & output,
                  unsigned int indent,
                  bool showDefaultValues,
                  StlVector<const VrmlNode *> & defNodesInOut) const;

  MaterialNode(const StlString & nodeType,
               VrmlShape3D * shape);

private:

  MaterialNode(void); // no implementation

  MaterialNode(const MaterialNode & obj); // no implementation

  MaterialNode & // itself
  operator=(const MaterialNode & obj); // no implementation

//-------- Member attributes -------------------------------------------------
protected:

  double _ambientIntensityField;
  Util3D::Dbl3 _diffuseColorField;
  Util3D::Dbl3 _emissiveColorField;
  double _shininessField;
  Util3D::Dbl3 _specularColorField;
  double _transparencyField;
  ArRef<Material3D> _material;
};

} // namespace AReVi

#endif // AREVI_VRML_MATERIALNODE_PRIV_H

//----------------------------------------------------------------------------
