// LI2/ENIB - AReVi - VRML services
//----------------------------------------------------------------------------
// file        : AReVi/VRML/pointSetNode_priv.h
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVI_VRML_POINTSETNODE_PRIV_H
#define AREVI_VRML_POINTSETNODE_PRIV_H 1

#include "AReVi/VRML/geometryNode_priv.h"

namespace AReVi {

class ColorNode;
class CoordinateNode;

//----------------------------------------------------------------------------
// Class PointSetNode
//----------------------------------------------------------------------------

class PointSetNode : public GeometryNode
{
friend class VrmlNode;

//-------- Construction / Destruction ----------------------------------------
public:

  virtual
  ~PointSetNode(void);

//-------- Inspection --------------------------------------------------------

  virtual
  bool // made of triangles
  isSurface(void) const;

//-------- Field management --------------------------------------------------

  virtual
  void
  getNodeFieldNames(StlVector<StlString> & nodeFieldNamesOut) const;

  virtual
  const VrmlNode * // node at field
  getNodeField(const StlString & fieldName) const;

  virtual
  void
  addField(const StlString & fieldName,
           VrmlValue * fieldValue);

  virtual
  void
  check(void);

//-------- Rendering ---------------------------------------------------------

  virtual
  ArRef<ShapePart3D> // built part
  buildPart(Util3D::Transform & initTransformOut) const;

  virtual
  void
  updatePart(ArRef<ShapePart3D> part,
             const MaterialNode * material) const;

//-------- Forbidden operations ----------------------------------------------
protected:

  virtual
  void
  _generateFields(ostream & output,
                  unsigned int indent,
                  bool showDefaultValues,
                  StlVector<const VrmlNode *> & defNodesInOut) const;

  PointSetNode(const StlString & nodeType,
               VrmlShape3D * shape);

private:

  PointSetNode(void); // no implementation

  PointSetNode(const PointSetNode & obj); // no implementation

  PointSetNode & // itself
  operator=(const PointSetNode & obj); // no implementation

//-------- Member attributes -------------------------------------------------
protected:

  ColorNode * _colorField;
  CoordinateNode * _coordField;
};

} // namespace AReVi

#endif // AREVI_VRML_POINTSETNODE_PRIV_H

//----------------------------------------------------------------------------
