// LI2/ENIB - AReVi - VRML services
//----------------------------------------------------------------------------
// file        : AReVi/VRML/shapeNode_priv.h
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVI_VRML_SHAPENODE_PRIV_H
#define AREVI_VRML_SHAPENODE_PRIV_H 1

#include "AReVi/VRML/groupChildNode_priv.h"

namespace AReVi {

class ShapePart3D;
class Transform3D;
class Material3D;
class AppearanceNode;
class GeometryNode;

//----------------------------------------------------------------------------
// Class ShapeNode
//----------------------------------------------------------------------------

class ShapeNode : public GroupChildNode
{
friend class VrmlNode;

//-------- Construction / Destruction ----------------------------------------
public:

  virtual
  ~ShapeNode(void);

//-------- Field management --------------------------------------------------

  virtual
  void
  getNodeFieldNames(StlVector<StlString> & nodeFieldNamesOut) const;

  virtual
  const VrmlNode * // node at field
  getNodeField(const StlString & fieldName) const;

  virtual
  void
  addField(const StlString & fieldName,
           VrmlValue * fieldValue);

  virtual
  void
  check(void);

//-------- Rendering ---------------------------------------------------------

  virtual
  void
  buildParts(const StlVector<TransformNode *> & transformPath);

  virtual
  void
  getParts(StlVector<ArConstRef<ShapePart3D> > & partsOut) const;

  virtual
  void
  accessParts(StlVector<ArRef<ShapePart3D> > & partsOut);

  virtual
  void
  postMaterialRebuild(void);

  virtual
  void
  postTextureRebuild(unsigned int which);

  virtual
  void
  postTextureTransformRebuild(unsigned int which);

  virtual
  void
  postGeometryRebuild(void);

  virtual
  void
  rebuild(void);

  virtual
  void
  postTransformation(void);

  virtual
  void
  applyTransformations(void);

//-------- Forbidden operations ----------------------------------------------
protected:

  virtual
  void
  _generateFields(ostream & output,
                  unsigned int indent,
                  bool showDefaultValues,
                  StlVector<const VrmlNode *> & defNodesInOut) const;

  ShapeNode(const StlString & nodeType,
            VrmlShape3D * shape);

private:

  ShapeNode(void); // no implementation

  ShapeNode(const ShapeNode & obj); // no implementation

  ShapeNode & // itself
  operator=(const ShapeNode & obj); // no implementation

//-------- Member attributes -------------------------------------------------
protected:

  struct PartInfo
    {
    ArRef<ShapePart3D> part;
    Util3D::Transform initTransform;
    StlVector<TransformNode *> transformPath;
    inline PartInfo(ArRef<ShapePart3D> p,
                    const Util3D::Transform & t,
                    const StlVector<TransformNode *> & tp)
      : part(p), initTransform(t), transformPath(tp) {}
    };

  AppearanceNode * _appearanceField;
  GeometryNode * _geometryField;
  StlVector<ShapeNode::PartInfo> _partInfos;
  ArRef<Transform3D> _transform;
  ArRef<Material3D> _lightOff;
  bool _mustBuildMaterial;
  bool _mustBuildTexture;
  bool _mustBuildTextureTransform;
  bool _mustBuildTexture2;
  bool _mustBuildTextureTransform2;
  bool _mustBuildTexture3;
  bool _mustBuildTextureTransform3;
  bool _mustBuildTexture4;
  bool _mustBuildTextureTransform4;
  bool _mustBuildGeometry;
  bool _mustTransform;
};

} // namespace AReVi

#endif // AREVI_VRML_SHAPENODE_PRIV_H

//----------------------------------------------------------------------------
