// LI2/ENIB - AReVi - VRML services
//----------------------------------------------------------------------------
// file        : AReVi/VRML/textureNode_priv.h
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVI_VRML_TEXTURENODE_PRIV_H
#define AREVI_VRML_TEXTURENODE_PRIV_H 1

#include "AReVi/VRML/vrmlNode_priv.h"

namespace AReVi {

class Texture;
class Surface3D;

//----------------------------------------------------------------------------
// Class TextureNode
//----------------------------------------------------------------------------

class TextureNode : public VrmlNode
{
//-------- Construction / Destruction ----------------------------------------
public:

  virtual
  ~TextureNode(void);

//-------- Inspection --------------------------------------------------------

  virtual
  bool // is a texture
  isTexture(void) const;

//-------- Field management --------------------------------------------------

  virtual
  void
  addField(const StlString & fieldName,
           VrmlValue * fieldValue);

//-------- Rendering ---------------------------------------------------------

  virtual
  void
  postTextureRebuild(bool regenerate);

  virtual
  void
  rebuild(void)=0;

  virtual
  void
  updatePart(ArRef<Surface3D> part,
             unsigned int which) const;

//-------- Forbidden operations ----------------------------------------------
protected:

  virtual
  void
  _generateFields(ostream & output,
                  unsigned int indent,
                  bool showDefaultValues,
                  StlVector<const VrmlNode *> & defNodesInOut) const;

  TextureNode(const StlString & nodeType,
              VrmlShape3D * shape);

private:

  TextureNode(void); // no implementation

  TextureNode(const TextureNode & obj); // no implementation

  TextureNode & // itself
  operator=(const TextureNode & obj); // no implementation

//-------- Member attributes -------------------------------------------------
protected:

  bool _repeatSField;
  bool _repeatTField;
  ArRef<Texture> _texture;
};

} // namespace AReVi

#endif // AREVI_VRML_TEXTURENODE_PRIV_H

//----------------------------------------------------------------------------
