// LI2/ENIB - AReVi - VRML services
//----------------------------------------------------------------------------
// file        : AReVi/VRML/textureTransformNode_priv.h
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVI_VRML_TEXTURETRANSFORMNODE_PRIV_H
#define AREVI_VRML_TEXTURETRANSFORMNODE_PRIV_H 1

#include "AReVi/VRML/vrmlNode_priv.h"

namespace AReVi {

class Transform2D;
class Surface3D;

//----------------------------------------------------------------------------
// Class TextureTransformNode
//----------------------------------------------------------------------------

class TextureTransformNode : public VrmlNode
{
friend class VrmlNode;
friend class VrmlTextureTransformInteractor;
friend class ShapeNode;

//-------- Construction / Destruction ----------------------------------------
public:

  virtual
  ~TextureTransformNode(void);

//-------- Field management --------------------------------------------------

  virtual
  void
  addField(const StlString & fieldName,
           VrmlValue * fieldValue);

  virtual
  void
  check(void);

//-------- Rendering ---------------------------------------------------------

  virtual
  void
  postRebuild(void);

  virtual
  void
  rebuild(void);

  virtual
  void
  updatePart(ArRef<Surface3D> part,
             unsigned int which) const;

//-------- Forbidden operations ----------------------------------------------
protected:

  virtual
  void
  _generateFields(ostream & output,
                  unsigned int indent,
                  bool showDefaultValues,
                  StlVector<const VrmlNode *> & defNodesInOut) const;

  TextureTransformNode(const StlString & nodeType,
                       VrmlShape3D * shape);

private:

  TextureTransformNode(void); // no implementation

  TextureTransformNode(const TextureTransformNode & obj); // no implementation

  TextureTransformNode & // itself
  operator=(const TextureTransformNode & obj); // no implementation

//-------- Member attributes -------------------------------------------------
protected:

  Util3D::Dbl2 _centerField;
  double _rotationField;
  Util3D::Dbl2 _scaleField;
  Util3D::Dbl2 _translationField;
  ArRef<Transform2D> _transform;
};

} // namespace AReVi

#endif // AREVI_VRML_TEXTURETRANSFORMNODE_PRIV_H

//----------------------------------------------------------------------------
