// LI2/ENIB - AReVi - VRML services
//----------------------------------------------------------------------------
// file        : AReVi/VRML/vrmlInteractor.h
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVI_VRML_VRMLINTERACTOR_H
#define AREVI_VRML_VRMLINTERACTOR_H 1

#include "AReVi/arObject.h"

namespace AReVi {

class VrmlShape3D;
class VrmlNode;

//----------------------------------------------------------------------------
// Class VrmlInteractor
//----------------------------------------------------------------------------

class VrmlInteractor : public ArObject
{
//-------- Type information --------------------------------------------------
public:

  AR_CLASS(VrmlInteractor)

//-------- Construction / Destruction ----------------------------------------

  AR_CONSTRUCTOR(VrmlInteractor)

//-------- Inspection --------------------------------------------------------

  virtual
  ArConstRef<VrmlShape3D> // connected shape
  getShape(void) const;

  virtual
  ArRef<VrmlShape3D> // connected shape
  accessShape(void);

  virtual
  const StlString & // type of connected node
  getNodeType(void) const;

  virtual
  const StlString & // name of connected node
  getNodeName(void) const;

  virtual
  int // occurence of named node
  getNodeOccurence(void) const;

//-------- Connection --------------------------------------------------------

  virtual
  bool // a shape/node is connected
  isConnected(void) const;

  virtual
  bool // success
  connectToNode(ArRef<VrmlShape3D> shape,
                const StlString & nodeName,
                int occurence=-1);

  virtual
  bool // success
  connectTo(ArConstRef<VrmlInteractor> interactor);

  virtual
  void
  disconnect(void);

//-------- Code generation ---------------------------------------------------

  virtual
  void
  generateString(StlString & vrmlStringOut,
                 unsigned int indent,
                 bool showDefaultValues) const;

  virtual
  void
  generateStream(ArRef<AbstractOStream> output,
                 unsigned int indent,
                 bool showDefaultValues) const;

//-------- Forbidden operations ----------------------------------------------
protected:

  friend class VrmlShape3D;
  friend class VrmlGenericInteractor;

  virtual
  void
  _shapeRemoved(void);

//-------- Member attributes -------------------------------------------------
protected:

  ArRef<VrmlShape3D> _shape;
  int _occurence;
  VrmlNode * _node;
  StlString _dummyStr;
};

} // namespace AReVi

#endif // AREVI_VRML_VRMLINTERACTOR_H

//----------------------------------------------------------------------------
