// LI2/ENIB - AReVi - VRML services
//----------------------------------------------------------------------------
// file        : AReVi/VRML/vrmlMaterialInteractor.h
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVI_VRML_VRMLMATERIALINTERACTOR_H
#define AREVI_VRML_VRMLMATERIALINTERACTOR_H 1

#include "AReVi/VRML/vrmlInteractor.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class VrmlMaterialInteractor
//----------------------------------------------------------------------------

class VrmlMaterialInteractor : public VrmlInteractor
{
//-------- Type information --------------------------------------------------
public:

  AR_CLASS(VrmlMaterialInteractor)

//-------- Construction / Destruction ----------------------------------------

  AR_CONSTRUCTOR(VrmlMaterialInteractor)

//-------- Connection --------------------------------------------------------

  virtual
  bool // success
  connectToNode(ArRef<VrmlShape3D> shape,
                const StlString & nodeName,
                int occurence=-1);

  virtual
  bool // success
  connectTo(ArConstRef<VrmlInteractor> interactor);

//-------- Parameters --------------------------------------------------------

  virtual
  bool // success
  readAmbientIntensityField(double & ambientIntensityOut) const;

  virtual
  bool // success
  writeAmbientIntensityField(double ambientIntensity);

  virtual
  bool // success
  readDiffuseColorField(double & rOut,
                        double & gOut,
                        double & bOut) const;

  virtual
  bool // success
  writeDiffuseColorField(double r,
                         double g,
                         double b);

  virtual
  bool // success
  readEmissiveColorField(double & rOut,
                         double & gOut,
                         double & bOut) const;

  virtual
  bool // success
  writeEmissiveColorField(double r,
                          double g,
                          double b);

  virtual
  bool // success
  readShininessField(double & shininessOut) const;

  virtual
  bool // success
  writeShininessField(double shininess);

  virtual
  bool // success
  readSpecularColorField(double & rOut,
                         double & gOut,
                         double & bOut) const;

  virtual
  bool // success
  writeSpecularColorField(double r,
                          double g,
                          double b);

  virtual
  bool // success
  readTransparencyField(double & transparencyOut) const;

  virtual
  bool // success
  writeTransparencyField(double transparency);

  virtual
  bool // success
  interpolate(ArRef<VrmlMaterialInteractor> previous,
              ArRef<VrmlMaterialInteractor> next,
              double nextPreviousRatio);
};

} // namespace AReVi

#endif // AREVI_VRML_VRMLMATERIALINTERACTOR_H

//----------------------------------------------------------------------------
