// LI2/ENIB - AReVi - VRML services
//----------------------------------------------------------------------------
// file        : AReVi/VRML/vrmlNode_priv.h
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVI_VRML_VRMLNODE_PRIV_H
#define AREVI_VRML_VRMLNODE_PRIV_H 1

#include "AReVi/arObject.h"

#include "AReVi/Lib3D/util3D_priv.h"

namespace AReVi {

class VrmlShape3D;
class VrmlValue;

//----------------------------------------------------------------------------
// Class VrmlNode
//----------------------------------------------------------------------------

class VrmlNode
{
//-------- Construction / Destruction ----------------------------------------
public:

  static
  VrmlNode * // new node
  create(const StlString & nodeType,
         VrmlShape3D * shape);

  virtual
  ~VrmlNode(void);

//-------- Inspection --------------------------------------------------------

  virtual
  bool // problem occured
  fail(void) const;

  virtual
  bool // valid child for grouping node
  isGroupChild(void) const;

  virtual
  bool // is a group
  isGroup(void) const;

  virtual
  bool // is a texture
  isTexture(void) const;

  virtual
  bool // is a geometry
  isGeometry(void) const;

//-------- Naming ------------------------------------------------------------

  virtual
  const StlString & // node type
  getType(void) const;

  virtual
  void
  setName(const StlString & nodeName);

  virtual
  const StlString & // node name
  getName(void) const;

//-------- Parent management -------------------------------------------------

  virtual
  void
  addParent(VrmlNode * parent);

  virtual
  void
  removeParent(VrmlNode * parent);

  virtual
  void
  destroyIfNoParent(void);

//-------- Field management --------------------------------------------------

  virtual
  void
  getNodeFieldNames(StlVector<StlString> & nodeFieldNamesOut) const;

  virtual
  const VrmlNode * // node at field
  getNodeField(const StlString & fieldName) const;

  virtual
  VrmlNode * // node at field
  accessNodeField(const StlString & fieldName) const;

  virtual
  void
  addField(const StlString & fieldName,
           VrmlValue * fieldValue);

  virtual
  void
  check(void);

//-------- Code generation ---------------------------------------------------

  virtual
  void
  generateCode(ostream & output,
               unsigned int indent,
               bool showDefaultValues,
               StlVector<const VrmlNode *> & defNodesInOut) const;

//-------- Rendering ---------------------------------------------------------

  virtual
  void
  postRebuild(void);

  virtual
  void
  postRebuildToTop(void);

  virtual
  void
  rebuild(void);

//-------- Forbidden operations ----------------------------------------------
protected:

  virtual
  void
  _generateFields(ostream & output,
                  unsigned int indent,
                  bool showDefaultValues,
                  StlVector<const VrmlNode *> & defNodesInOut) const;

  virtual
  void
  _indent(ostream & output,
          unsigned int indent) const;

  virtual
  void
  _generateSFBool(ostream & output,
                  unsigned int indent,
                  const StlString & fieldName,
                  bool field) const;

  virtual
  void
  _generateSFColor(ostream & output,
                   unsigned int indent,
                   const StlString & fieldName,
                   const Util3D::Dbl3 & field) const;

  virtual
  void
  _generateMFColor(ostream & output,
                   unsigned int indent,
                   const StlString & fieldName,
                   const StlVector<Util3D::Dbl3> & field) const;

  virtual
  void
  _generateSFFloat(ostream & output,
                   unsigned int indent,
                   const StlString & fieldName,
                   double field) const;

  virtual
  void
  _generateMFFloat(ostream & output,
                   unsigned int indent,
                   const StlString & fieldName,
                   const StlVector<double> & field) const;

  virtual
  void
  _generateSFImage(ostream & output,
                   unsigned int indent,
                   const StlString & fieldName,
                   const StlVector<unsigned int> & field) const;

  virtual
  void
  _generateSFInt32(ostream & output,
                   unsigned int indent,
                   const StlString & fieldName,
                   int field) const;

  virtual
  void
  _generateMFInt32(ostream & output,
                   unsigned int indent,
                   const StlString & fieldName,
                   const StlVector<StlVector<int> > & field) const;

  virtual
  void
  _generateSFNode(ostream & output,
                  unsigned int indent,
                  const StlString & fieldName,
                  const VrmlNode * field,
                  bool showDefaultValues,
                  StlVector<const VrmlNode *> & defNodesInOut) const;

  virtual
  void
  _generateMFNode(ostream & output,
                  unsigned int indent,
                  const StlString & fieldName,
                  const StlVector<VrmlNode *> & field,
                  bool showDefaultValues,
                  StlVector<const VrmlNode *> & defNodesInOut) const;

  virtual
  void
  _generateSFRotation(ostream & output,
                      unsigned int indent,
                      const StlString & fieldName,
                      const Util3D::Dbl4 & field) const;

  virtual
  void
  _generateMFRotation(ostream & output,
                      unsigned int indent,
                      const StlString & fieldName,
                      const StlVector<Util3D::Dbl4> & field) const;

  virtual
  void
  _generateSFString(ostream & output,
                    unsigned int indent,
                    const StlString & fieldName,
                    const StlString & field) const;

  virtual
  void
  _generateMFString(ostream & output,
                    unsigned int indent,
                    const StlString & fieldName,
                    const StlVector<StlString> & field) const;

  virtual
  void
  _generateSFTime(ostream & output,
                  unsigned int indent,
                  const StlString & fieldName,
                  double field) const;

  virtual
  void
  _generateMFTime(ostream & output,
                  unsigned int indent,
                  const StlString & fieldName,
                  const StlVector<double> & field) const;

  virtual
  void
  _generateSFVec2f(ostream & output,
                   unsigned int indent,
                   const StlString & fieldName,
                   const Util3D::Dbl2 & field) const;

  virtual
  void
  _generateMFVec2f(ostream & output,
                   unsigned int indent,
                   const StlString & fieldName,
                   const StlVector<Util3D::Dbl2> & field) const;

  virtual
  void
  _generateSFVec3f(ostream & output,
                   unsigned int indent,
                   const StlString & fieldName,
                   const Util3D::Dbl3 & field) const;

  virtual
  void
  _generateMFVec3f(ostream & output,
                   unsigned int indent,
                   const StlString & fieldName,
                   const StlVector<Util3D::Dbl3> & field) const;

  VrmlNode(const StlString & nodeType,
           VrmlShape3D * shape);

private:

  VrmlNode(void); // no implementation

  VrmlNode(const VrmlNode & obj); // no implementation

  VrmlNode & // itself
  operator=(const VrmlNode & obj); // no implementation

//-------- Member attributes -------------------------------------------------
protected:

  StlString _type;
  VrmlShape3D * _shape;
  StlVector<VrmlNode *> _parents;
  StlString _name;
  bool _fail;
  bool _needsRebuild;
  bool _needsRebuildUnder;
};

} // namespace AReVi

#endif // AREVI_VRML_VRMLNODE_PRIV_H

//----------------------------------------------------------------------------
