// LI2/ENIB - AReVi - VRML services
//----------------------------------------------------------------------------
// file        : AReVi/VRML/vrmlParser_priv.h
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVI_VRML_VRMLPARSER_PRIV_H
#define AREVI_VRML_VRMLPARSER_PRIV_H 1

#include "AReVi/VRML/vrmlNode_priv.h"

#ifdef __sgi
#define IOSTREAMH // avoid inclusion of iostream.h from FlexLexer.h
#else // not __sgi
#define _BACKWARD_IOSTREAM_H // same thing ...
#define _CPP_BACKWARD_IOSTREAM_H // same thing ...
#endif // not __sgi

#define yyFlexLexer _VrmlFlexLexer
#include <FlexLexer.h>
#undef yyFlexLexer

namespace AReVi {

//----------------------------------------------------------------------------
// Class VrmlParser
//----------------------------------------------------------------------------

class VrmlParser : protected _VrmlFlexLexer
{
friend class VrmlShape3D;

//-------- Construction / Destruction ----------------------------------------
protected:

  VrmlParser(istream * input,
             VrmlShape3D * shape);

  virtual
  ~VrmlParser(void);

//-------- Parsing -----------------------------------------------------------

  virtual
  int
  parse(void);

//-------- Error messages ----------------------------------------------------

  virtual
  void
  yyerror(const char * text);

//-------- Utilities ---------------------------------------------------------
protected:

  int yylex(void) {return(0);} // prevent SGI compiler from complaining

  virtual
  int
  yylex(void * arg);

  virtual
  StlString *
  _saveString(StlString * str);

  virtual
  StlString *
  _relaxString(StlString * str);

  virtual
  VrmlValue *
  _saveValue(VrmlValue * val);

  virtual
  VrmlValue *
  _relaxValue(VrmlValue * val);

  virtual
  VrmlNode *
  _saveNode(VrmlNode * node);

  virtual
  VrmlNode *
  _relaxNode(VrmlNode * node);

  virtual
  void
  _nextLine(void);

  virtual
  void
  _store(const char * text);

//-------- Member attributes -------------------------------------------------
protected:

  int _nbErrors;
  int _nbLines;
  StlString _line;
  VrmlShape3D * _shape;
  int _insideProto;
  StlVector<StlString *> _curStrings;
  StlVector<VrmlValue *> _curValues;
  StlVector<VrmlNode *> _curNodes;
};

} // namespace AReVi

#endif // AREVI_VRML_VRMLPARSER_PRIV_H

//----------------------------------------------------------------------------
