// LI2/ENIB - AReVi - VRML services
//----------------------------------------------------------------------------
// file        : AReVi/VRML/vrmlShape3D.h
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVI_VRML_VRMLSHAPE3D_H
#define AREVI_VRML_VRMLSHAPE3D_H 1

#include "AReVi/Shapes/shape3D.h"

namespace AReVi {

class VrmlNode;
class GroupChildNode;
class ShapeNode;
class VrmlParser;
class VrmlInteractor;

//----------------------------------------------------------------------------
// Class VrmlShape3D
//----------------------------------------------------------------------------

class VrmlShape3D : public Shape3D
{
//-------- Type information --------------------------------------------------
public:

  AR_CLASS(VrmlShape3D)

//-------- Construction / Destruction ----------------------------------------

  AR_CONSTRUCTOR(VrmlShape3D)

//-------- Parsing -----------------------------------------------------------

  virtual
  bool // success
  parseURL(const StlString & url);

  virtual
  bool // success
  parseString(const StlString & vrmlString);

  virtual
  bool // success
  parseStream(ArRef<AbstractIStream> input);

  virtual
  const StlString & // messages when parsing
  readParserMessages(void) const;

  virtual
  void
  forgetAll(void);

//-------- URL management ----------------------------------------------------

  virtual
  const StlString & // URL
  getURL(void) const;

  virtual
  const StlString & // directory of URL
  getURLDirectory(void) const;

  virtual
  void
  setURLDirectory(const StlString & urlDir);

//-------- Code generation ---------------------------------------------------

  virtual
  void
  generateString(StlString & vrmlStringOut,
                 unsigned int indent,
                 bool showDefaultValues) const;

  virtual
  void
  generateStream(ArRef<AbstractOStream> output,
                 unsigned int indent,
                 bool showDefaultValues) const;

//-------- Forbidden operations ----------------------------------------------
protected:

  friend class VrmlNode;
  friend class AnchorNode;
  friend class AppearanceNode;
  friend class BillboardNode;
  friend class BoxNode;
  friend class CollisionNode;
  friend class ColorNode;
  friend class ConeNode;
  friend class CoordinateNode;
  friend class CylinderNode;
  friend class ElevationGridNode;
  friend class ExtrusionNode;
  friend class GeometryNode;
  friend class GroupChildNode;
  friend class GroupNode;
  friend class ImageTextureNode;
  friend class IndexedFaceSetNode;
  friend class IndexedLineSetNode;
  friend class InlineNode;
  friend class MaterialNode;
  friend class NormalNode;
  friend class PixelTextureNode;
  friend class PointSetNode;
  friend class ShapeNode;
  friend class SphereNode;
  friend class TextureNode;
  friend class TextureCoordinateNode;
  friend class TextureTransformNode;
  friend class TransformNode;
  friend class WorldInfoNode;
  friend class VrmlParser;
  friend class VrmlInteractor;
  friend class VrmlGenericInteractor;

  virtual
  void
  _ignoreNodeMsg(const StlString & nodeType);

  virtual
  void
  _ignoreFieldMsg(const StlString & fieldName,
                  const StlString & nodeType);

  virtual
  void
  _badFieldMsg(const StlString & fieldName,
               const StlString & nodeType);

  virtual
  void
  _errorMsg(const StlString & msg);

  virtual
  void
  _addRootNode(GroupChildNode * node);

  virtual
  unsigned int
  _getNbRootNodes(void) const;

  virtual
  const GroupChildNode *
  _getRootNode(unsigned int index) const;

  virtual
  GroupChildNode *
  _accessRootNode(unsigned int index);

  virtual
  void
  _addNamedNode(VrmlNode * node);

  virtual
  void
  _removeNamedNode(VrmlNode * node);

  virtual
  const VrmlNode * // found node
  _getNamedNode(const StlString & nodeName,
                int occurence=-1) const;

  virtual
  VrmlNode * // found node
  _accessNamedNode(const StlString & nodeName,
                   int occurence=-1);

  virtual
  void
  _buildParts(void);

  virtual
  void
  _rebuildIfNeeded(void);

  virtual
  void
  _postShapeTransformation(ShapeNode * shape);

  virtual
  void
  _addInteractor(VrmlInteractor * interactor);

  virtual
  void
  _removeInteractor(VrmlInteractor * interactor);

//-------- Member attributes -------------------------------------------------
protected:

  StlString _msg;
  StlVector<GroupChildNode *> _rootNodes;
  StlVector<VrmlNode *> _namedNodes;
  StlVector<ShapeNode *> _transformShapes;
  StlString _url;
  StlString _urlDir;
  VrmlParser * _parser;
  StlVector<VrmlInteractor *> _interactors;
};

} // namespace AReVi

#endif // AREVI_VRML_VRMLSHAPE3D_H

//----------------------------------------------------------------------------
