// LI2/ENIB - AReVi - VRML services
//----------------------------------------------------------------------------
// file        : AReVi/VRML/vrmlValue_priv.h
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVI_VRML_VRMLVALUE_PRIV_H
#define AREVI_VRML_VRMLVALUE_PRIV_H 1

#include "AReVi/Utils/stlString_priv.h"
#include "AReVi/Utils/stlVector_priv.h"

namespace AReVi {

class VrmlNode;

//----------------------------------------------------------------------------
// Class VrmlValue
//----------------------------------------------------------------------------

class VrmlValue
{
//-------- Construction / Destruction ----------------------------------------
public:

  VrmlValue(void);

  virtual
  ~VrmlValue(void);

//-------- Type checking -----------------------------------------------------

  virtual
  bool // success
  checkSFBool(void);

  virtual
  bool // success
  checkSFColor(void);

  virtual
  bool // success
  checkMFColor(void);

  virtual
  bool // success
  checkSFFloat(void);

  virtual
  bool // success
  checkMFFloat(void);

  virtual
  bool // success
  checkSFImage(void);

  virtual
  bool // success
  checkSFInt32(void);

  virtual
  bool // success
  checkMFInt32(void);

  virtual
  bool // success
  checkSFNode(void);

  virtual
  bool // success
  checkMFNode(void);

  virtual
  bool // success
  checkSFRotation(void);

  virtual
  bool // success
  checkMFRotation(void);

  virtual
  bool // success
  checkSFString(void);

  virtual
  bool // success
  checkMFString(void);

  virtual
  bool // success
  checkSFTime(void);

  virtual
  bool // success
  checkSFVec2f(void);

  virtual
  bool // success
  checkMFVec2f(void);

  virtual
  bool // success
  checkSFVec3f(void);

  virtual
  bool // success
  checkMFVec3f(void);

//-------- Boolean values ----------------------------------------------------

  virtual
  void
  addBool(bool boolVal);

  virtual
  bool // bool at index
  getBool(int index) const;

//-------- Integer values ----------------------------------------------------

  virtual
  void
  addInteger(int integerVal);

  virtual
  int // integer at index
  getInteger(int index) const;

//-------- Real values -------------------------------------------------------

  virtual
  void
  addReal(double realVal);

  virtual
  double // real at index
  getReal(int index) const;

//-------- Node values -------------------------------------------------------

  virtual
  void
  addNode(VrmlNode * nodeVal);

  virtual
  const VrmlNode * // node at index
  getNode(int index) const;

  virtual
  VrmlNode * // node at index
  accessNode(int index);

//-------- String values -----------------------------------------------------

  virtual
  void
  addString(const StlString & stringVal);

  virtual
  const StlString & // string at index
  getString(int index) const;

//-------- Multiple values ---------------------------------------------------

  virtual
  unsigned int // number of contained values
  getSize(void) const;

  virtual
  void
  setBrackets(void);

//-------- Forbidden operations ----------------------------------------------
protected:

  enum Type { BOOL_T, INT_T, REAL_T, NODE_T, STRING_T, UNKNOWN_T };

  virtual
  void
  _init(VrmlValue::Type type);

  virtual
  void
  _destroy(void);

  virtual
  void
  _switchTo(VrmlValue::Type type);

private:

  VrmlValue(const VrmlValue & obj); // no implementation

  VrmlValue & // itself
  operator=(const VrmlValue & obj); // no implementation

//-------- Member attributes -------------------------------------------------
protected:

  VrmlValue::Type _type;
  bool _bracket;
  union
    {
    StlVector<bool> * b;
    StlVector<int> * i;
    StlVector<double> * r;
    StlVector<VrmlNode *> * n;
    StlVector<StlString> * s;
    } _val;
};

} // namespace AReVi

#endif // AREVI_VRML_VRMLVALUE_PRIV_H

//----------------------------------------------------------------------------
