// LI2/ENIB - AReVi - Core services
//----------------------------------------------------------------------------
// file        : AReVi/activity.h
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVI_ACTIVITY_H
#define AREVI_ACTIVITY_H 1

#include "AReVi/arObject.h"

namespace AReVi {

class Scheduler;

//----------------------------------------------------------------------------
// Class Activity
//----------------------------------------------------------------------------

class Activity : public ArObject
{
friend class Scheduler;

//-------- Type information --------------------------------------------------
public:

  AR_CLASS(Activity)

//-------- Construction / Destruction ----------------------------------------

  AR_CONSTRUCTOR_1(Activity,
                   double, interval)

//-------- Behavior ----------------------------------------------------------

  template <class T1,class T2,class T3> inline
  bool // success
  setBehavior(ArRef<T1> obj,                     // template --> NOT virtual !
              bool (T2::*method)(ArRef<T3>,double));

  template <class T1,class T2,class T3> inline
  bool // success
  getBehavior(ArRef<T1> & objOut,                // template --> NOT virtual !
              bool (T2::* & methodOut)(ArRef<T3>,double)) const;

//-------- Scheduling properties ---------------------------------------------

  virtual
  double // current time
  getTime(void) const;

  virtual
  void
  setInterval(double interval);

  virtual
  double // activation interval
  getInterval(void) const;

  virtual
  void
  setPriority(int priority);

  virtual
  int // priority ( -1 0 +1 )
  getPriority(void) const;

//-------- Scheduling control ------------------------------------------------

  virtual
  void
  setSuspended(bool suspended);

  virtual
  bool // suspended
  isSuspended(void) const;

  virtual
  void
  abort(void);

  virtual
  bool // ended
  isEnded(void) const;

//-------- Forbidden operations ----------------------------------------------
protected:

  virtual
  bool // re-activate
  _activate(double dt);

//-------- Member attributes -------------------------------------------------
protected:

  ArRef<ArObject> _obj;
  bool (ArObject::*_method)(ArRef<Activity>,double);
  ArConstRef<ArClass> _methodArgClass;
  double _interval;
  int _priority;
  double _lastTime;
  double _nextTime;
  bool _suspended;
  bool _ended;
};

} // namespace AReVi

//-------- Inline source code ------------------------------------------------

#include "AReVi/activity.Ci"

#endif // AREVI_ACTIVITY_H

//----------------------------------------------------------------------------
