// LI2/ENIB - AReVi - Core services
//----------------------------------------------------------------------------
// file        : AReVi/arClass.h
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVI_ARCLASS_H
#define AREVI_ARCLASS_H 1

#include "AReVi/arObject.h"

namespace AReVi {

class ArValue;
class ArMethod;
class ArConstant;
class ArStubHelper;

//----------------------------------------------------------------------------
// Class ArClass
//----------------------------------------------------------------------------

// Classes derived from ``ArObject'' MUST use, in their implementation
// (i.e. the .ccp file) one of the following macros.
//
#define AR_CLASS_DEF(_CLASS,_PARENT)                        \
  ArCW _CLASS::_CLS=(ArCW)0;                                \
  void                                                      \
  _CLASS::REGISTER_CLASS(void) /* Static */                 \
    {                                                       \
    if(!_CLASS::_CLS)                                       \
      {                                                     \
      _PARENT::REGISTER_CLASS();                            \
      ArClass::registerClass(_CLASS::_CLS,#_CLASS,#_PARENT, \
                             &_CLASS::_makeInstance);       \
      }                                                     \
    }                                                       \
  ArConstRef<ArClass> /* ArClass objet for _CLASS */        \
  _CLASS::CLASS(void) /* Static */                          \
    {                                                       \
    if(!_CLASS::_CLS)                                       \
      {                                                     \
      cerr << "!!! Don't forget to call "                   \
              #_CLASS "::REGISTER_CLASS() !!!" << endl;     \
      _CLASS::REGISTER_CLASS();                             \
      }                                                     \
    return(ArConstRef<ArClass>(_CLASS::_CLS));              \
    }                                                       \
  ArObject * /* created instance */                         \
  _CLASS::_makeInstance(void) /* Static, Private */         \
  { return(new _CLASS(_CLASS::_CLS)); }

#define AR_CLASS_NOVOID_DEF(_CLASS,_PARENT)                 \
  ArCW _CLASS::_CLS=(ArCW)0;                                \
  void                                                      \
  _CLASS::REGISTER_CLASS(void) /* Static */                 \
    {                                                       \
    if(!_CLASS::_CLS)                                       \
      {                                                     \
      _PARENT::REGISTER_CLASS();                            \
      ArClass::registerClass(_CLASS::_CLS,#_CLASS,#_PARENT, \
                             &_CLASS::_makeInstance);       \
      }                                                     \
    }                                                       \
  ArConstRef<ArClass> /* ArClass objet for _CLASS */        \
  _CLASS::CLASS(void) /* Static */                          \
    {                                                       \
    if(!_CLASS::_CLS)                                       \
      {                                                     \
      cerr << "!!! Don't forget to call "                   \
              #_CLASS "::REGISTER_CLASS() !!!" << endl;     \
      _CLASS::REGISTER_CLASS();                             \
      }                                                     \
    return(ArConstRef<ArClass>(_CLASS::_CLS));              \
    }                                                       \
  ArObject * /* no instance created ! */                    \
  _CLASS::_makeInstance(void) /* Static, Private */         \
  { return((_CLASS *)0); }

// Methods in class ``ArClass'' are NOT virtual because providing
// a derived class has been made IMPOSSIBLE (to preserve consistency)

class ArClass : public ArObject
{
//-------- Type information --------------------------------------------------
public:

  AR_CLASS(ArClass)

  static
  void
  registerClass(ArCW & wrapper,
                const StlString & className,
                const StlString & parentName,
                ArObject * (*instantiateFnct)(void));

//-------- Class hierarchy ---------------------------------------------------
public:

  const StlString & // class name
  getName(void) const;

  bool // equals to or derived from arClass
  isA(ArConstRef<ArClass> arClass) const;

  ArConstRef<ArClass> // parent class or 0
  getParent(void) const;

  void
  getAncestors(StlVector<ArConstRef<ArClass> > & ancestorsOut) const;

  void
  getChildren(StlVector<ArConstRef<ArClass> > & childrenOut) const;

  void
  getDescendants(StlVector<ArConstRef<ArClass> > & descendantsOut) const;

  static
  ArConstRef<ArClass> // found class or 0
  find(const StlString & className);

//-------- Instance management -----------------------------------------------

  template <class T1> inline
  void
  getInstances(StlVector<ArRef<T1> > & instancesOut,
               bool subClasses) const;

  unsigned int
  getNbInstances(bool subClasses) const;

  template <class T1> inline
  void
  getInstance(unsigned int id,
              ArRef<T1> & instanceOut) const;

  void
  getAliases(StlVector<StlString> & aliasesOut,
             bool subClasses) const;

  template <class T1> inline
  void
  findAlias(const StlString & alias,
            ArRef<T1> & instanceOut) const;

  template <class T1> inline
  void
  newInstance(ArRef<T1> & instanceOut) const;

//-------- Reflection --------------------------------------------------------

  unsigned int // number of methods in this class only
  getNbMethods(void) const;

  ArConstRef<ArMethod> // index'th method in this class only
  getMethod(unsigned int index) const;

  ArConstRef<ArMethod> // found constructor-method in this class only
  findConstructor(const StlVector<ArRef<ArValue> > & args) const;

  ArConstRef<ArMethod> // found constructor-method in this class only
  findConstructorWithSignature(const StlString & argSignature) const;

  bool // found constructor-methods in this class only
  findConstructors(StlVector<ArConstRef<ArMethod> > & methodsOut) const;

  ArConstRef<ArMethod> // found static-method in base classes
  findStaticMethod(const StlString & methodName,
                   const StlVector<ArRef<ArValue> > & args) const;

  ArConstRef<ArMethod> // found static-method in base classes
  findStaticMethodWithSignature(const StlString & methodName,
                                const StlString & argSignature) const;

  bool // found static-methods in base classes
  findStaticMethods(StlVector<ArConstRef<ArMethod> > & methodsOut,
                    const StlString & methodName) const;

  ArConstRef<ArMethod> // found method in base classes
  findMethod(const StlString & methodName,
             const StlVector<ArRef<ArValue> > & args) const;

  ArConstRef<ArMethod> // found method in base classes
  findMethodWithSignature(const StlString & methodName,
                          const StlString & argSignature) const;

  bool // found methods in base classes
  findMethods(StlVector<ArConstRef<ArMethod> > & methodsOut,
              const StlString & methodName) const;

  ArConstRef<ArMethod> // found const-method in base classes
  findConstMethod(const StlString & methodName,
                  const StlVector<ArRef<ArValue> > & args) const;

  ArConstRef<ArMethod> // found const-method in base classes
  findConstMethodWithSignature(const StlString & methodName,
                               const StlString & argSignature) const;

  bool // found const-methods in base classes
  findConstMethods(StlVector<ArConstRef<ArMethod> > & methodsOut,
                   const StlString & methodName) const;

  unsigned int // number of constants in this class only
  getNbConstants(void) const;

  ArConstRef<ArConstant> // index'th constant in this class only
  getConstant(unsigned int index) const;

  ArConstRef<ArConstant> // found constant in base classes
  findConstant(const StlString & constantName) const;

//-------- Forbidden operations ----------------------------------------------
private:

  friend class ArObject;
  friend class ArMethod;
  friend class ArConstant;
  friend class ArStubHelper;

  ArClass(ArCW & wrapper,
          const StlString & className,
          ArConstRef<ArClass> superClass,
          ArObject * (*instantiateFnct)(void));

  void
  _manageObject(ArObject * obj);

  void
  _unmanageObject(ArObject * obj);

  void
  _getDescendants(StlVector<ArConstRef<ArClass> > & descendantsOut) const;

  void
  _getInstances(StlVector<ArRef<ArObject> > & instancesOut,
                bool subClasses) const;

  void
  _getAliases(StlVector<StlString> & aliasesOut,
              bool subClasses) const;

  bool // success
  _findAlias(ArConstRef<ArClass> cls,
             const StlString & alias,
             ArRef<ArObject> & instanceOut) const;

//-------- Member attributes -------------------------------------------------
private:
  StlString _className;
  ArObject * (*_instantiateFnct)(void);
  ArConstRef<ArClass> _super;
  StlVector<ArConstRef<ArClass> > _sub;
  unsigned int _count;
  bool _countLoop;
  StlVector<ArPtr<ArObject> > _instances;
  void (*_initMethodsFnct)(ArConstRef<ArClass>);
  StlVector<ArRef<ArMethod> > _methods;
  void (*_initConstantsFnct)(ArConstRef<ArClass>);
  StlVector<ArRef<ArConstant> > _constants;

//-------- Static attributes -------------------------------------------------
protected:
  static StlVector<ArClass *> * _toBeManaged;
  static StlMap<StlString,ArClass *> * _classMap;
};

} // namespace AReVi

//-------- Inline source code ------------------------------------------------

#include "AReVi/arClass.Ci"

#endif // AREVI_ARCLASS_H

//----------------------------------------------------------------------------
