// LI2/ENIB - AReVi - Core services
//----------------------------------------------------------------------------
// file        : AReVi/arMethod.h
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVI_ARMETHOD_H
#define AREVI_ARMETHOD_H 1

#include "AReVi/arType.h"

namespace AReVi {

class ArValue;

//----------------------------------------------------------------------------
// Class ArMethod
//----------------------------------------------------------------------------

// Methods in class ``ArMethod'' are NOT virtual because providing
// a derived class has been made IMPOSSIBLE (to preserve consistency)

class ArMethod : public ArObject
{
//-------- Type information --------------------------------------------------
public:

  AR_CLASS(ArMethod)

//-------- Inspection --------------------------------------------------------

  ArConstRef<ArClass> // class declaring this method
  getDeclarationClass(void) const;

  const StlString & // declaration header file of this method
  getDeclarationFile(void) const;

  unsigned int // declaration line number of this method
  getDeclarationLine(void) const;

  const StlString & // name of this method
  getName(void) const;

  bool // this method is a constructor
  isConstructor(void) const;

  bool // this method is static
  isStaticMethod(void) const;

  bool // this method is const
  isConstMethod(void) const;

  ArConstRef<ArType> // returned type if any
  getResultType(void) const;

  unsigned int // number of arguments
  getNbArgs(void) const;

  ArConstRef<ArType> // type of index'th argument
  getArgType(unsigned int index) const;

  const StlString & // name of index'th argument
  getArgName(unsigned int index) const;

  const StlString & // C++ arguments
  getArgSignature(void) const;

  bool // C++ arguments match (ignoring spaces)
  matchesArgSignature(const StlString & argSignature) const;

//-------- Invocation --------------------------------------------------------

  bool // success
  invoke(ArRef<ArObject> obj,
         ArRef<ArValue> resultOut,
         const StlVector<ArRef<ArValue> > & argsInOut,
         bool check=true) const;

  bool // success
  invokeConst(ArConstRef<ArObject> obj,
              ArRef<ArValue> resultOut,
              const StlVector<ArRef<ArValue> > & argsInOut,
              bool check=true) const;

  bool // success
  invokeStatic(ArRef<ArValue> resultOut,
               const StlVector<ArRef<ArValue> > & argsInOut,
               bool check=true) const;

//-------- Forbidden operations ----------------------------------------------
private:

  friend class ArStubHelper;

  typedef void (*Stub)(ArRef<ArObject>,
                       ArRef<ArValue>,
                       const StlVector<ArRef<ArValue> > &);

  ArMethod(ArCW & arCW,
           ArConstRef<ArClass> declCls,
           const StlString & declFile,
           unsigned int declLine,
           const StlString & mthName,
           bool isStatic,
           bool isConst,
           ArMethod::Stub stub);

  void
  _setResult(ArType::DirectType type,
             unsigned int dimension,
             ArType::PassingMode passing);

  void
  _setResult(ArType::IndirectType type,
             ArConstRef<ArClass> cls,
             unsigned int dimension,
             ArType::PassingMode passing);

  void
  _addArg(const StlString & name,
          ArType::DirectType type,
          unsigned int dimension,
          ArType::PassingMode passing);

  void
  _addArg(const StlString & name,
          ArType::IndirectType type,
          ArConstRef<ArClass> cls,
          unsigned int dimension,
          ArType::PassingMode passing);

//-------- Member attributes -------------------------------------------------
private:
  ArConstRef<ArClass> _declCls;
  StlString _declFile;
  unsigned int _declLine;
  StlString _mthName;
  bool _constructor;
  bool _isStatic;
  bool _isConst;
  ArMethod::Stub _stub;
  ArConstRef<ArType> _result;
  StlVector<ArConstRef<ArType> > _argTypes;
  StlVector<StlString> _argNames;
  StlString _argSignature;
};

} // namespace AReVi

#endif // AREVI_ARMETHOD_H

//----------------------------------------------------------------------------
