// LI2/ENIB - AReVi - Core services
//----------------------------------------------------------------------------
// file        : AReVi/arType.h
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVI_ARTYPE_H
#define AREVI_ARTYPE_H 1

#include "AReVi/arObject.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class ArType
//----------------------------------------------------------------------------

// Methods in class ``ArType'' are NOT virtual because providing
// a derived class has been made IMPOSSIBLE (to preserve consistency)

class ArType : public ArObject
{
//-------- Type information --------------------------------------------------
public:

  AR_CLASS(ArType)

//-------- Inspection --------------------------------------------------------

  inline
  bool // type is direct (not an ArClass)
  isDirect(void) const;

  enum DirectType { DT_NONE, DT_BOOL, DT_CHAR, DT_UCHAR, DT_SHORT, DT_USHORT,
                    DT_INT, DT_UINT, DT_LLONG, DT_ULLONG, DT_FLOAT, DT_DOUBLE,
                    DT_STRING };

  inline
  ArType::DirectType // kind of direct type or DT_NONE
  getDirectType(void) const;

  enum IndirectType { IT_NONE, IT_REF, IT_CONST_REF, IT_PTR, IT_CONST_PTR };

  inline
  ArType::IndirectType // kind of indirect type or IT_NONE
  getIndirectType(void) const;

  inline
  ArConstRef<ArClass> // class type if indirect
  getIndirectClass(void) const;

  inline
  unsigned int // 0: simple object  1: StlVector<>  2: StlVector<StlVector<> >
  getDimension(void) const;

  enum PassingMode { PM_COPY, PM_REF, PM_CONST_REF };

  inline
  ArType::PassingMode // kind of passing mode
  getPassingMode(void) const;

  inline
  bool // same type except passing mode
  isSameType(ArConstRef<ArType> type) const;

  inline
  const StlString & // C++ representation of this type
  getCppType(void) const;

//-------- Forbidden operations ----------------------------------------------
private:

  friend class ArValue;
  friend class ArMethod;

  ArType(ArCW & arCW,
         ArType::DirectType type,
         unsigned int dimension,
         ArType::PassingMode passing);

  ArType(ArCW & arCW,
         ArType::IndirectType type,
         ArConstRef<ArClass> cls,
         unsigned int dimension,
         ArType::PassingMode passing);

//-------- Member attributes -------------------------------------------------
private:
  bool _isDirect;
  int _type;
  ArConstRef<ArClass> _indirectClass;
  unsigned int _dimension;
  ArType::PassingMode _passingMode;
  StlString _cppType;
};

} // namespace AReVi

//-------- Inline source code ------------------------------------------------

#include "AReVi/arType.Ci"

#endif // AREVI_ARTYPE_H

//----------------------------------------------------------------------------
