// LI2/ENIB - AReVi - Core services
//----------------------------------------------------------------------------
// file        : AReVi/arValue.h
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVI_ARVALUE_H
#define AREVI_ARVALUE_H 1

#include "AReVi/arType.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class ArValue
//----------------------------------------------------------------------------

// Methods in class ``ArValue'' are NOT virtual because providing
// a derived class has been made IMPOSSIBLE (to preserve consistency)

class ArValue : public ArObject
{
//-------- Type information --------------------------------------------------
public:

  AR_CLASS(ArValue)

//-------- Construction / Destruction ----------------------------------------

  AR_CONSTRUCTOR_1(ArValue,
                   ArConstRef<ArType>, type)

  AR_CONSTRUCTOR_2(ArValue,
                   ArType::DirectType, type,
                   unsigned int, dimension)

  AR_CONSTRUCTOR_3(ArValue,
                   ArType::IndirectType, type,
                   ArConstRef<ArClass>, cls,
                   unsigned int, dimension)

//-------- Value management --------------------------------------------------

  inline
  ArConstRef<ArType> // value type
  getType(void) const;

  inline
  bool // correct type
  isBool(void) const;

  inline
  const bool & // null if bad type
  getBool(void) const;

  inline
  bool & // null if bad type
  accessBool(void);

  inline
  bool // correct type
  isBoolV(void) const;

  inline
  const StlVector<bool> & // null if bad type
  getBoolV(void) const;

  inline
  StlVector<bool> & // null if bad type
  accessBoolV(void);

  inline
  bool // correct type
  isBoolVV(void) const;

  inline
  const StlVector<StlVector<bool> > & // null if bad type
  getBoolVV(void) const;

  inline
  StlVector<StlVector<bool> > & // null if bad type
  accessBoolVV(void);

  inline
  bool // correct type
  isChar(void) const;

  inline
  const char & // null if bad type
  getChar(void) const;

  inline
  char & // null if bad type
  accessChar(void);

  inline
  bool // correct type
  isCharV(void) const;

  inline
  const StlVector<char> & // null if bad type
  getCharV(void) const;

  inline
  StlVector<char> & // null if bad type
  accessCharV(void);

  inline
  bool // correct type
  isCharVV(void) const;

  inline
  const StlVector<StlVector<char> > & // null if bad type
  getCharVV(void) const;

  inline
  StlVector<StlVector<char> > & // null if bad type
  accessCharVV(void);

  inline
  bool // correct type
  isUChar(void) const;

  inline
  const unsigned char & // null if bad type
  getUChar(void) const;

  inline
  unsigned char & // null if bad type
  accessUChar(void);

  inline
  bool // correct type
  isUCharV(void) const;

  inline
  const StlVector<unsigned char> & // null if bad type
  getUCharV(void) const;

  inline
  StlVector<unsigned char> & // null if bad type
  accessUCharV(void);

  inline
  bool // correct type
  isUCharVV(void) const;

  inline
  const StlVector<StlVector<unsigned char> > & // null if bad type
  getUCharVV(void) const;

  inline
  StlVector<StlVector<unsigned char> > & // null if bad type
  accessUCharVV(void);

  inline
  bool // correct type
  isShort(void) const;

  inline
  const short & // null if bad type
  getShort(void) const;

  inline
  short & // null if bad type
  accessShort(void);

  inline
  bool // correct type
  isShortV(void) const;

  inline
  const StlVector<short> & // null if bad type
  getShortV(void) const;

  inline
  StlVector<short> & // null if bad type
  accessShortV(void);

  inline
  bool // correct type
  isShortVV(void) const;

  inline
  const StlVector<StlVector<short> > & // null if bad type
  getShortVV(void) const;

  inline
  StlVector<StlVector<short> > & // null if bad type
  accessShortVV(void);

  inline
  bool // correct type
  isUShort(void) const;

  inline
  const unsigned short & // null if bad type
  getUShort(void) const;

  inline
  unsigned short & // null if bad type
  accessUShort(void);

  inline
  bool // correct type
  isUShortV(void) const;

  inline
  const StlVector<unsigned short> & // null if bad type
  getUShortV(void) const;

  inline
  StlVector<unsigned short> & // null if bad type
  accessUShortV(void);

  inline
  bool // correct type
  isUShortVV(void) const;

  inline
  const StlVector<StlVector<unsigned short> > & // null if bad type
  getUShortVV(void) const;

  inline
  StlVector<StlVector<unsigned short> > & // null if bad type
  accessUShortVV(void);

  inline
  bool // correct type
  isInt(void) const;

  inline
  const int & // null if bad type
  getInt(void) const;

  inline
  int & // null if bad type
  accessInt(void);

  inline
  bool // correct type
  isIntV(void) const;

  inline
  const StlVector<int> & // null if bad type
  getIntV(void) const;

  inline
  StlVector<int> & // null if bad type
  accessIntV(void);

  inline
  bool // correct type
  isIntVV(void) const;

  inline
  const StlVector<StlVector<int> > & // null if bad type
  getIntVV(void) const;

  inline
  StlVector<StlVector<int> > & // null if bad type
  accessIntVV(void);

  inline
  bool // correct type
  isUInt(void) const;

  inline
  const unsigned int & // null if bad type
  getUInt(void) const;

  inline
  unsigned int & // null if bad type
  accessUInt(void);

  inline
  bool // correct type
  isUIntV(void) const;

  inline
  const StlVector<unsigned int> & // null if bad type
  getUIntV(void) const;

  inline
  StlVector<unsigned int> & // null if bad type
  accessUIntV(void);

  inline
  bool // correct type
  isUIntVV(void) const;

  inline
  const StlVector<StlVector<unsigned int> > & // null if bad type
  getUIntVV(void) const;

  inline
  StlVector<StlVector<unsigned int> > & // null if bad type
  accessUIntVV(void);

  inline
  bool // correct type
  isLLong(void) const;

  inline
  const long long & // null if bad type
  getLLong(void) const;

  inline
  long long & // null if bad type
  accessLLong(void);

  inline
  bool // correct type
  isLLongV(void) const;

  inline
  const StlVector<long long> & // null if bad type
  getLLongV(void) const;

  inline
  StlVector<long long> & // null if bad type
  accessLLongV(void);

  inline
  bool // correct type
  isLLongVV(void) const;

  inline
  const StlVector<StlVector<long long> > & // null if bad type
  getLLongVV(void) const;

  inline
  StlVector<StlVector<long long> > & // null if bad type
  accessLLongVV(void);

  inline
  bool // correct type
  isULLong(void) const;

  inline
  const unsigned long long & // null if bad type
  getULLong(void) const;

  inline
  unsigned long long & // null if bad type
  accessULLong(void);

  inline
  bool // correct type
  isULLongV(void) const;

  inline
  const StlVector<unsigned long long> & // null if bad type
  getULLongV(void) const;

  inline
  StlVector<unsigned long long> & // null if bad type
  accessULLongV(void);

  inline
  bool // correct type
  isULLongVV(void) const;

  inline
  const StlVector<StlVector<unsigned long long> > & // null if bad type
  getULLongVV(void) const;

  inline
  StlVector<StlVector<unsigned long long> > & // null if bad type
  accessULLongVV(void);

  inline
  bool // correct type
  isFloat(void) const;

  inline
  const float & // null if bad type
  getFloat(void) const;

  inline
  float & // null if bad type
  accessFloat(void);

  inline
  bool // correct type
  isFloatV(void) const;

  inline
  const StlVector<float> & // null if bad type
  getFloatV(void) const;

  inline
  StlVector<float> & // null if bad type
  accessFloatV(void);

  inline
  bool // correct type
  isFloatVV(void) const;

  inline
  const StlVector<StlVector<float> > & // null if bad type
  getFloatVV(void) const;

  inline
  StlVector<StlVector<float> > & // null if bad type
  accessFloatVV(void);

  inline
  bool // correct type
  isDouble(void) const;

  inline
  const double & // null if bad type
  getDouble(void) const;

  inline
  double & // null if bad type
  accessDouble(void);

  inline
  bool // correct type
  isDoubleV(void) const;

  inline
  const StlVector<double> & // null if bad type
  getDoubleV(void) const;

  inline
  StlVector<double> & // null if bad type
  accessDoubleV(void);

  inline
  bool // correct type
  isDoubleVV(void) const;

  inline
  const StlVector<StlVector<double> > & // null if bad type
  getDoubleVV(void) const;

  inline
  StlVector<StlVector<double> > & // null if bad type
  accessDoubleVV(void);

  inline
  bool // correct type
  isString(void) const;

  inline
  const StlString & // null if bad type
  getString(void) const;

  inline
  StlString & // null if bad type
  accessString(void);

  inline
  bool // correct type
  isStringV(void) const;

  inline
  const StlVector<StlString> & // null if bad type
  getStringV(void) const;

  inline
  StlVector<StlString> & // null if bad type
  accessStringV(void);

  inline
  bool // correct type
  isStringVV(void) const;

  inline
  const StlVector<StlVector<StlString> > & // null if bad type
  getStringVV(void) const;

  inline
  StlVector<StlVector<StlString> > & // null if bad type
  accessStringVV(void);

  inline
  bool // correct type
  isRef(void) const;

  inline
  const ArRef<ArObject> & // null if bad type
  getRef(void) const;

  inline
  ArRef<ArObject> & // null if bad type
  accessRef(void);

  inline
  bool // correct type
  isRefV(void) const;

  inline
  const StlVector<ArRef<ArObject> > & // null if bad type
  getRefV(void) const;

  inline
  StlVector<ArRef<ArObject> > & // null if bad type
  accessRefV(void);

  inline
  bool // correct type
  isRefVV(void) const;

  inline
  const StlVector<StlVector<ArRef<ArObject> > > & // null if bad type
  getRefVV(void) const;

  inline
  StlVector<StlVector<ArRef<ArObject> > > & // null if bad type
  accessRefVV(void);

  inline
  bool // correct type
  isConstRef(void) const;

  inline
  const ArConstRef<ArObject> & // null if bad type
  getConstRef(void) const;

  inline
  ArConstRef<ArObject> & // null if bad type
  accessConstRef(void);

  inline
  bool // correct type
  isConstRefV(void) const;

  inline
  const StlVector<ArConstRef<ArObject> > & // null if bad type
  getConstRefV(void) const;

  inline
  StlVector<ArConstRef<ArObject> > & // null if bad type
  accessConstRefV(void);

  inline
  bool // correct type
  isConstRefVV(void) const;

  inline
  const StlVector<StlVector<ArConstRef<ArObject> > > & // null if bad type
  getConstRefVV(void) const;

  inline
  StlVector<StlVector<ArConstRef<ArObject> > > & // null if bad type
  accessConstRefVV(void);

  inline
  bool // correct type
  isPtr(void) const;

  inline
  const ArPtr<ArObject> & // null if bad type
  getPtr(void) const;

  inline
  ArPtr<ArObject> & // null if bad type
  accessPtr(void);

  inline
  bool // correct type
  isPtrV(void) const;

  inline
  const StlVector<ArPtr<ArObject> > & // null if bad type
  getPtrV(void) const;

  inline
  StlVector<ArPtr<ArObject> > & // null if bad type
  accessPtrV(void);

  inline
  bool // correct type
  isPtrVV(void) const;

  inline
  const StlVector<StlVector<ArPtr<ArObject> > > & // null if bad type
  getPtrVV(void) const;

  inline
  StlVector<StlVector<ArPtr<ArObject> > > & // null if bad type
  accessPtrVV(void);

  inline
  bool // correct type
  isConstPtr(void) const;

  inline
  const ArConstPtr<ArObject> & // null if bad type
  getConstPtr(void) const;

  inline
  ArConstPtr<ArObject> & // null if bad type
  accessConstPtr(void);

  inline
  bool // correct type
  isConstPtrV(void) const;

  inline
  const StlVector<ArConstPtr<ArObject> > & // null if bad type
  getConstPtrV(void) const;

  inline
  StlVector<ArConstPtr<ArObject> > & // null if bad type
  accessConstPtrV(void);

  inline
  bool // correct type
  isConstPtrVV(void) const;

  inline
  const StlVector<StlVector<ArConstPtr<ArObject> > > & // null if bad type
  getConstPtrVV(void) const;

  inline
  StlVector<StlVector<ArConstPtr<ArObject> > > & // null if bad type
  accessConstPtrVV(void);

//-------- Member attributes -------------------------------------------------
private:
  ArConstRef<ArType> _t;
  void * _v;
};

} // namespace AReVi

//-------- Inline source code ------------------------------------------------

#include "AReVi/arValue.Ci"

#endif // AREVI_ARVALUE_H

//----------------------------------------------------------------------------
