// LI2/ENIB - AReVi - Core services
//----------------------------------------------------------------------------
// file        : AReVi/message.h
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVI_MESSAGE_H
#define AREVI_MESSAGE_H 1

#include "AReVi/arObject.h"

namespace AReVi {

class TCPListener;
class TCPConnection;
class UDPTransmitter;
class MessageService;

//----------------------------------------------------------------------------
// Class Message
//----------------------------------------------------------------------------

class Message : public ArObject
{
friend class MessageService;

//-------- Type information --------------------------------------------------
public:

  AR_CLASS(Message)

//-------- Construction / Destruction ----------------------------------------

  AR_CONSTRUCTOR_1(Message,
                   ArRef<ArObject>, emitter)

//-------- Identification ----------------------------------------------------

  virtual
  void
  setEmitter(ArRef<ArObject> emitter);

  virtual
  ArRef<ArObject> // reference on emitter if known
  getEmitter(void) const;

  virtual
  void
  setEmitterAlias(const StlString & alias);

  virtual
  const StlString & // emitter's alias
  getEmitterAlias(void) const;

  ArRef<MessageService> // message service that delivered this message
  getService(void) const;

//-------- Content -----------------------------------------------------------

  virtual
  void
  setText(const StlString & text);

  virtual
  const StlString & // message content
  getText(void) const;

//-------- Stream operations -------------------------------------------------

  virtual
  void
  writeToStream(ArRef<AbstractOStream> output) const;

  virtual
  bool // success
  readFromStream(ArRef<AbstractIStream> input);

//-------- Forbidden operations ----------------------------------------------
protected:

  Message(ArCW & arCW);

//-------- Member attributes -------------------------------------------------
protected:

  ArPtr<ArObject> _emitter;
  StlString _emitterAlias;
  ArPtr<MessageService> _service;
  StlString _text;
};


//----------------------------------------------------------------------------
// Class MessageService
//----------------------------------------------------------------------------

class MessageService : public ArObject
{
//-------- Type information --------------------------------------------------
public:

  AR_CLASS(MessageService)

//-------- Construction / Destruction ----------------------------------------

  AR_CONSTRUCTOR(MessageService)

//-------- Peer to peer ------------------------------------------------------

  virtual
  void
  sendTo(ArRef<ArObject> receiver,
         ArRef<Message> message);

  virtual
  void
  sendToAlias(const StlString & receiver,
              ArRef<Message> message);

//-------- Broadcast ---------------------------------------------------------

  virtual
  void
  subscribeBroadcast(ArConstRef<ArClass> messageType,
                     ArRef<ArObject> receiver);

  virtual
  void
  unsubscribeBroadcast(ArConstRef<ArClass> messageType,
                       ArRef<ArObject> receiver);

  virtual
  void
  broadcast(ArRef<Message> message);

//-------- Member attributes -------------------------------------------------
protected:

  struct BroadcastSubscription
  {
  ArPtr<ArObject> receiver;
  StlVector<ArConstRef<ArClass> > messageTypes;
  };

  StlVector<BroadcastSubscription> _bdcast;
};


//----------------------------------------------------------------------------
// Class NetworkMessageService
//----------------------------------------------------------------------------

class NetworkMessageService : public MessageService
{
//-------- Type information --------------------------------------------------
public:

  AR_CLASS(NetworkMessageService)

//-------- Construction / Destruction ----------------------------------------

  AR_CONSTRUCTOR_1(NetworkMessageService,
                   int, udpPortNumber)

  AR_CONSTRUCTOR_2(NetworkMessageService,
                   int, udpPortNumber,
                   int, tcpPortNumber)

//-------- Session management ------------------------------------------------

  virtual
  const StlString & // session ID
  getSessionID(void) const;

  virtual
  void
  tryToReachLocalSessions(void);

  virtual
  void
  tryToReachSession(unsigned int distantIPAddress,
                    int distantUdpPortNumber);

  virtual
  void
  getDistantSessions(StlVector<StlString> & sessionsOut) const;

//-------- Identification ----------------------------------------------------

  virtual
  void
  extractLocalAlias(const StlString & alias,
                    StlString & localAliasOut) const;

  virtual
  void
  extractNetworkSession(const StlString & alias,
                        StlString & sessionOut) const;

  virtual
  void
  makeNetworkAlias(StlString & aliasInOut) const;

//-------- Peer to peer ------------------------------------------------------

  virtual
  void
  sendToAlias(const StlString & receiver,
              ArRef<Message> message);

  virtual
  void
  sendToSameAlias(const StlString & receiver,
                  ArRef<Message> message);

//-------- Broadcast ---------------------------------------------------------

  virtual
  void
  broadcast(ArRef<Message> message);

//-------- Network watching --------------------------------------------------

  virtual
  void
  watchNetwork(void);

//-------- Member attributes -------------------------------------------------
protected:

  struct ConnectedService
  {
  StlString session;
  ArRef<TCPConnection> connection;
  };

  ArRef<UDPTransmitter> _transmitter;
  ArRef<TCPListener> _listener;
  StlString _sessionID;
  StlVector<ConnectedService> _services;
};

} // namespace AReVi

#endif // AREVI_MESSAGE_H

//----------------------------------------------------------------------------
