// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/Controlers/nsControler.h
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVINS_NSCONSTROLER_H
#define AREVINS_NSCONSTROLER_H 1

//--- Imported classes -------------------------------------------------------

#include "AReViNS/networkService.h"



//----------------------------------------------------------------------------
// CLASS       : NSListener (Server side)
//----------------------------------------------------------------------------
namespace AReVi {
  class LocalDepService;
  class RemoteDepService;

/*! \class NSListener
  * Listener for server shared objects. Intercept all objects CB events and
  * transmit them to the correspondant Interactor on the client(remote) side.
  */
class NSListener : public ArObject
{
public:
  AR_UNSTUBBED AR_CLASS(NSListener)
//---------------------- Constructors / Destructor ---------------------------
  AR_CONSTRUCTOR(NSListener)

//---------------------------- Public methods --------------------------------
public:
  //! Controler will sign to all object events and will be able to transmit
  //! them. (getPendingEvents());
  virtual void
  init(ArRef<LocalDepService> depService, ArRef<ArObject> object);

  //! Return events backup (eg : motion, setup change ...)
  virtual void
  getPendingEvents(StlList<ArRef<NSEvent> > &events);

//------------------------- Forbiden operations ------------------------------
protected:
//----------------------------- Attributes -----------------------------------
protected:
  ArPtr<ArObject> _object;
  ArPtr<LocalDepService> _depService;
};

//----------------------------------------------------------------------------
// CLASS       : NSInteractor (Client Side)
//----------------------------------------------------------------------------
/*! \class NSInteractor
  * Interactor for remote shared objects. Receive all objects events sended by
  * the server listener and modify remote reference as consequence.
  */
class NSInteractor : public ArObject
{
public:
  AR_UNSTUBBED AR_CLASS(NSInteractor)
//---------------------- Constructors / Destructor ---------------------------
  AR_CONSTRUCTOR(NSInteractor)

//---------------------------- Public methods --------------------------------
public:
  //! Controler will sign to all object network events and will be able to
  //! affect remote object with.
  virtual void
  init(ArRef<RemoteDepService> depService, ArRef<ArObject> object);

  //! Return events backup (eg : motion, setup change ...)
  NSEvent::EventProcess
  fireEvent(ArConstRef<NSEvent> event);

//------------------------- Forbiden operations ------------------------------
protected:
  //! Register events
  virtual bool
  _registerEvent(const StlString &eventClass,
                 NetworkService::EventMethod method);

  //! Unregister events
  virtual bool
  _unregisterEvent(const StlString &eventClass);

//----------------------------- Attributes -----------------------------------
protected:
  StlMap<StlString, NetworkService::EventMethod> _eventsTable;

  ArPtr<ArObject> _object;
  ArPtr<RemoteDepService> _depService;
  ArConstPtr<NSEvent> _waitingForEvent;

  bool _ready; // Remote reference is ready (unserialized).
};

} // end of namespace AReVi
#endif // end of AREVINS_NSCONSTROLER_H
//---End-Of-File---------------------------------------------------------------
