// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/Controlers/nsCtrlArConsole.h
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVINS_NSCONTROLERARCONSOLE_H
#define AREVINS_NSCONTROLERARCONSOLE_H 1

//--- Imported classes -------------------------------------------------------
#include "AReViNS/Controlers/nsCtrlOsdText.h"
#include "AReVi/Contrib/arConsole.h"

namespace AReVi {
//----------------------------------------------------------------------------
// CLASS       : NSListenerArConsole
//----------------------------------------------------------------------------
/*! \class NSListenerArConsole
  * Network Service Event class
  */
class NSListenerArConsole : public NSListenerOSDText
{
public:
  AR_UNSTUBBED AR_CLASS(NSListenerArConsole)
//---------------------- Constructors / Destructor ---------------------------
  AR_CONSTRUCTOR(NSListenerArConsole)

//---------------------------- Public methods --------------------------------
public:
  //! Controler will sign to all object events and will be able to transmit
  //! them. (getPendingEvents());
  virtual void
  init(ArRef<LocalDepService> service, ArRef<ArObject> object);

  //! Return events backup (eg : motion, setup change ...)
  virtual void
  getPendingEvents(StlList<ArRef<NSEvent> > &events);


//------------------------- Forbiden operations ------------------------------
protected:

//! Background changed (setBackground).
  virtual void
  _onBackgroundChangedCB(const ArConsole::BackgroundChangedEvent &event);

//! Foreground changed (setColor, set Font).
  virtual void
  _onForegroundChangedCB(const ArConsole::ForegroundChangedEvent &event);

//! Text Scrolled (scroll)
  virtual void
  _onScrollTextCB(const ArConsole::ScrollTextEvent &event);

//! Buffer setup(setSize, setBufferSize, clear)
  virtual void
  _onBufferSetupCB(const ArConsole::BufferSetupEvent &event);

//! Push Text (push)
  virtual void
  _onPushTextCB(const ArConsole::PushTextEvent &event);

//! Store character (storeCharacter)
  virtual void
  _onStoreCharacterCB(const ArConsole::StoreCharacterEvent &event);

//! Clear text (clear)
  virtual void
  _onClearTextCB(const ArConsole::ClearTextEvent &event);

//! Push attrib (pushAttrib)
  virtual void
  _onPushAttribCB(const ArConsole::PushAttribEvent &event);

//! Pop attrib (popAttrib)
  virtual void
  _onPopAttribCB(const ArConsole::PopAttribEvent &event);

//! Flush attrib (flushAttrib)
  virtual void
  _onFlushAttribCB(const ArConsole::FlushAttribsEvent &event);

//----------------------------- Attributes -----------------------------------
protected:
  ArPtr<ArConsole> _arConsole;

  StlList<ArRef<NSEvent> > _events;
  bool _backgroundFlag;
};

//----------------------------------------------------------------------------
// CLASS       : NSInteractorArConsole
//----------------------------------------------------------------------------
/*! \class NSInteractorArConsole
  * Network Service Event class
  */
class NSInteractorArConsole : public NSInteractorOSDText
{
public:
  AR_UNSTUBBED AR_CLASS(NSInteractorArConsole)
//---------------------- Constructors / Destructor ---------------------------
  AR_CONSTRUCTOR(NSInteractorArConsole)

//---------------------------- Public methods --------------------------------
public:
  //! Controler will sign to all object network events and will be able to
  //! affect remote object with.
  virtual void
  init(ArRef<RemoteDepService> service, ArRef<ArObject> object);

//------------------------- Forbiden operations ------------------------------
protected:

//! Background changed (setBackground).
  virtual NSEvent::EventProcess
  _onBackgroundChangedEvent(ArConstRef<NSEvent> event);

//! Foreground changed (setColor, set Font).
  virtual NSEvent::EventProcess
  _onForegroundChangedEvent(ArConstRef<NSEvent> event);

//! Text Scrolled (scroll)
  virtual NSEvent::EventProcess
  _onScrollTextEvent(ArConstRef<NSEvent> event);

//! Buffer setup(setSize, setBufferSize, clear)
  virtual NSEvent::EventProcess
  _onBufferSetupEvent(ArConstRef<NSEvent> event);

//! Push Text (push)
  virtual NSEvent::EventProcess
  _onPushTextEvent(ArConstRef<NSEvent> event);

//! Store character (storeCharacter)
  virtual NSEvent::EventProcess
  _onStoreCharacterEvent(ArConstRef<NSEvent> event);

//! Clear Text (clear)
  virtual NSEvent::EventProcess
  _onClearTextEvent(ArConstRef<NSEvent> event);

//! Push attrib (pushAttrib)
  virtual NSEvent::EventProcess
  _onPushAttribEvent(ArConstRef<NSEvent> event);

//! Pop attrib (popAttrib)
  virtual NSEvent::EventProcess
  _onPopAttribEvent(ArConstRef<NSEvent> event);

//! Flush attrib (flushAttrib)
  virtual NSEvent::EventProcess
  _onFlushAttribEvent(ArConstRef<NSEvent> event);

//----------------------------- Attributes -----------------------------------
protected:
  ArPtr<ArConsole> _arConsole;
};

} // end of namespace AReVi
#endif // end of AREVINS_NSCONTROLERARCONSOLE_H
//---End-Of-File---------------------------------------------------------------
