// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/Controlers/nsCtrlBase3D.h
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef NETWORKDSERVICE_NSCTRLBASE3D_H
#define NETWORKDSERVICE_NSCTRLBASE3D_H 1

//--- Imported classes -------------------------------------------------------
#include "AReViNS/Controlers/nsControler.h"
#include "AReVi/Lib3D/base3D.h"



namespace AReVi {
//----------------------------------------------------------------------------
// CLASS       : NSListenerBase3D
//----------------------------------------------------------------------------

/*! \class NSListenerBase3D
  * Receive Motion notification and transmit them
  */
class NSListenerBase3D : public NSListener
{
public:
  AR_UNSTUBBED AR_CLASS(NSListenerBase3D)
//---------------------- Constructors / Destructor ---------------------------
  AR_CONSTRUCTOR(NSListenerBase3D)

//---------------------------- Public methods --------------------------------
public:
  //! Controler will sign to all object events and will be able to transmit
  //! them. (getPendingEvents());
  virtual void
  init(ArRef<LocalDepService> service, ArRef<ArObject> object);

  //! Return events backup (eg : motion, setup change ...)
  virtual void
  getPendingEvents(StlList<ArRef<NSEvent> > &events);

//------------------------- Forbiden operations ------------------------------
protected:
  virtual void
  _onMotionCB(const Base3D::MotionEvent &event);

//----------------------------- Attributes -----------------------------------
protected:
  ArPtr<Base3D> _base;
  bool _motionFlag;
};

//----------------------------------------------------------------------------
// CLASS       : NSInteractorBase3D
//----------------------------------------------------------------------------

/*! \class NSInteractorBase3D
  * Receive Motion notification from server and apply them to remote ref.
  */
class NSInteractorBase3D : public NSInteractor
{
public:
  AR_UNSTUBBED AR_CLASS(NSInteractorBase3D)
//---------------------- Constructors / Destructor ---------------------------
  AR_CONSTRUCTOR(NSInteractorBase3D)

//---------------------------- Public methods --------------------------------
public:
  //! Controler will sign to all object network events and will be able to
  //! affect remote object with.
  virtual void
  init(ArRef<RemoteDepService> service, ArRef<ArObject> object);

//------------------------- Forbiden operations ------------------------------
protected:
  virtual NSEvent::EventProcess
  _onMotionEvent(ArConstRef<NSEvent> event);

//----------------------------- Attributes -----------------------------------
protected:
  ArPtr<Base3D> _base;
};

} // end of namespace AReVi
#endif // end of NETWORKDSERVICE_NSCTRLBASE3D_H
//---End-Of-File---------------------------------------------------------------
