// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/Controlers/nsCtrlObject3D.h
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVINS_NSCONTROLEROBJECT3D_H
#define AREVINS_NSCONTROLEROBJECT3D_H 1

//--- Imported classes -------------------------------------------------------
#include "AReViNS/Controlers/nsCtrlBase3D.h"
#include "AReVi/Lib3D/object3D.h"

namespace AReVi {
//----------------------------------------------------------------------------
// CLASS       : NSListenerObject3D
//----------------------------------------------------------------------------

/*! \class NSListenerObject3D
  * Network Service Event class
  */
class NSListenerObject3D : public NSListenerBase3D
{
public:
  AR_UNSTUBBED AR_CLASS(NSListenerObject3D)
//---------------------- Constructors / Destructor ---------------------------
  AR_CONSTRUCTOR(NSListenerObject3D)

//---------------------------- Public methods --------------------------------
public:
  //! Controler will sign to all object events and will be able to transmit
  //! them. (getPendingEvents());
  virtual void
  init(ArRef<LocalDepService> service, ArRef<ArObject> object);

  //! Return events backup (eg : motion, setup change ...)
  virtual void
  getPendingEvents(StlList<ArRef<NSEvent> > &events);

//------------------------- Forbiden operations ------------------------------
protected:
  virtual void
  _onShapeChangeCB(const Object3D::ShapeChangeEvent &event);

  virtual void
  _onAppearanceCB(const Object3D::AppearanceEvent &event);

//----------------------------- Attributes -----------------------------------
protected:
  ArPtr<Object3D> _object3D;
  bool _shapeChangeFlag;
  bool _appearanceFlag;
};

//----------------------------------------------------------------------------
// CLASS       : NSInteractorObject3D
//----------------------------------------------------------------------------

/*! \class NSInteractorObject3D
  * Network Service Event class
  */
class NSInteractorObject3D : public NSInteractorBase3D
{
public:
  AR_UNSTUBBED AR_CLASS(NSInteractorObject3D)
//---------------------- Constructors / Destructor ---------------------------
  AR_CONSTRUCTOR(NSInteractorObject3D)

//---------------------------- Public methods --------------------------------
public:
  //! Controler will sign to all object network events and will be able to
  //! affect remote object with.
  virtual void
  init(ArRef<RemoteDepService> service, ArRef<ArObject> object);

//------------------------- Forbiden operations ------------------------------
protected:
  virtual NSEvent::EventProcess
  _onShapeChangeEvent(ArConstRef<NSEvent> event);

  virtual NSEvent::EventProcess
  _onAppearanceEvent(ArConstRef<NSEvent> event);

//----------------------------- Attributes -----------------------------------
protected:
  ArPtr<Object3D> _object3D;
};

} // end of namespace AReVi
#endif // end of AREVINS_NSCONTROLEROBJECT3D_H
//---End-Of-File---------------------------------------------------------------
