// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/Controlers/nsCtrlOsd.h
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVINS_NSCONTROLEROSDTEXT_H
#define AREVINS_NSCONTROLEROSDTEXT_H 1

//--- Imported classes -------------------------------------------------------
#include "AReViNS/Controlers/nsCtrlOsd.h"
#include "AReVi/Lib3D/osdText.h"

namespace AReVi {
//----------------------------------------------------------------------------
// CLASS       : NSListenerOSDText
//----------------------------------------------------------------------------
/*! \class NSListenerOSDText
  * Network Service Event class
  */
class NSListenerOSDText : public NSListenerOSD
{
public:
  AR_UNSTUBBED AR_CLASS(NSListenerOSDText)
//---------------------- Constructors / Destructor ---------------------------
  AR_CONSTRUCTOR(NSListenerOSDText)

//---------------------------- Public methods --------------------------------
public:
  //! Controler will sign to all object events and will be able to transmit
  //! them. (getPendingEvents());
  virtual void
  init(ArRef<LocalDepService> service, ArRef<ArObject> object);

  //! Return events backup (eg : motion, setup change ...)
  virtual void
  getPendingEvents(StlList<ArRef<NSEvent> > &events);


//------------------------- Forbiden operations ------------------------------
protected:
//! Text changed (setText).
  virtual void
  _onTextChangedCB(const OSDText::TextChangedEvent &event);

//----------------------------- Attributes -----------------------------------
protected:
  ArPtr<OSDText> _osdText;
  bool _textFlag;
};

//----------------------------------------------------------------------------
// CLASS       : NSInteractorOSDText
//----------------------------------------------------------------------------
/*! \class NSInteractorOSDText
  * Network Service Event class
  */
class NSInteractorOSDText : public NSInteractorOSD
{
public:
  AR_UNSTUBBED AR_CLASS(NSInteractorOSDText)
//---------------------- Constructors / Destructor ---------------------------
  AR_CONSTRUCTOR(NSInteractorOSDText)

//---------------------------- Public methods --------------------------------
public:
  //! Controler will sign to all object network events and will be able to
  //! affect remote object with.
  virtual void
  init(ArRef<RemoteDepService> service, ArRef<ArObject> object);

//------------------------- Forbiden operations ------------------------------
protected:

//! OSD Setup (set padding, size ...)
  virtual NSEvent::EventProcess
  _onSetupEvent(ArConstRef<NSEvent> event);

//! Text changed (setText).
  virtual NSEvent::EventProcess
  _onTextChangedEvent(ArConstRef<NSEvent> event);

//----------------------------- Attributes -----------------------------------
protected:
  ArPtr<OSDText> _osdText;
};

} // end of namespace AReVi
#endif // end of AREVINS_NSCONTROLEROSDTEXT_H
//---End-Of-File---------------------------------------------------------------
