// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/Controlers/nsCtrlParticleSystem.h
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVINS_NSCONTROLERPARTICLESYSTEM_H
#define AREVINS_NSCONTROLERPARTICLESYSTEM_H 1

//--- Imported classes -------------------------------------------------------
#include "AReViNS/Controlers/nsCtrlBase3D.h"
#include "AReVi/Lib3D/particleSystem.h"

namespace AReVi {
//----------------------------------------------------------------------------
// CLASS       : NSListenerParticleSystem
//----------------------------------------------------------------------------

/*! \class NSListenerParticleSystem
  * Network Service Event class
  */
class NSListenerParticleSystem : public NSListenerBase3D
{
public:
  AR_UNSTUBBED AR_CLASS(NSListenerParticleSystem)
//---------------------- Constructors / Destructor ---------------------------
  AR_CONSTRUCTOR(NSListenerParticleSystem)

//---------------------------- Public methods --------------------------------
public:
  //! Controler will sign to all object events and will be able to transmit
  //! them. (getPendingEvents());
  virtual void
  init(ArRef<LocalDepService> service, ArRef<ArObject> object);

  //! Return events backup (eg : motion, setup change ...)
  virtual void
  getPendingEvents(StlList<ArRef<NSEvent> > &events);

//------------------------- Forbiden operations ------------------------------
protected:
  virtual void
  _onRenderingCB(const ParticleSystem::RenderingEvent &event);
  virtual void
  _onUpdateCB(const ParticleSystem::UpdateEvent &event);
  virtual void
  _onEmissionCB(const ParticleSystem::EmissionEvent &event);
  virtual void
  _onEvolutionCB(const ParticleSystem::EvolutionEvent &event);
  virtual void
  _onAddColliderCB(const ParticleSystem::AddColliderEvent &event);
  virtual void
  _onRemoveColliderCB(const ParticleSystem::RemoveColliderEvent &event);

//----------------------------- Attributes -----------------------------------
protected:
  ArPtr<ParticleSystem> _part;
  bool _renderingFlag;
  bool _emissionFlag;
  bool _evolutionFlag;
  //double _dt;
  StlList<ArRef<NSEvent> > _events;
};

//----------------------------------------------------------------------------
// CLASS       : NSInteractorParticleSystem
//----------------------------------------------------------------------------

/*! \class NSInteractorParticleSystem
  * Network Service Event class
  */
class NSInteractorParticleSystem : public NSInteractorBase3D
{
public:
  AR_UNSTUBBED AR_CLASS(NSInteractorParticleSystem)
//---------------------- Constructors / Destructor ---------------------------
  AR_CONSTRUCTOR(NSInteractorParticleSystem)

//---------------------------- Public methods --------------------------------
public:
  //! Controler will sign to all object network events and will be able to
  //! affect remote object with.
  virtual void
  init(ArRef<RemoteDepService> service, ArRef<ArObject> object);

//------------------------- Forbiden operations ------------------------------
protected:
  virtual NSEvent::EventProcess
  _onRenderingEvent(ArConstRef<NSEvent> event);

  virtual NSEvent::EventProcess
  _onUpdateEvent(ArConstRef<NSEvent> event);

  virtual NSEvent::EventProcess
  _onEmissionEvent(ArConstRef<NSEvent> event);

  virtual NSEvent::EventProcess
  _onEvolutionEvent(ArConstRef<NSEvent> event);

  virtual NSEvent::EventProcess
  _onAddColliderEvent(ArConstRef<NSEvent> event);

  virtual NSEvent::EventProcess
  _onRemoveColliderEvent(ArConstRef<NSEvent> event);

//----------------------------- Attributes -----------------------------------
protected:
  ArPtr<ParticleSystem> _part;
};

} // end of namespace AReVi
#endif // end of AREVINS_NSCONTROLERPARTICLESYSTEM_H
//---End-Of-File---------------------------------------------------------------
