// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/Controlers/nsCtrlRenderer3D.h
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVINS_NSCONTROLERRENDERER3D_H
#define AREVINS_NSCONTROLERRENDERER3D_H 1

//--- Imported classes -------------------------------------------------------
#include "AReViNS/Controlers/nsCtrlBase3D.h"
#include "AReVi/Lib3D/renderer3D.h"

namespace AReVi {
//----------------------------------------------------------------------------
// CLASS       : NSListenerRenderer3D
//----------------------------------------------------------------------------

/*! \class NSListenerRenderer3D
  * Network Service Event class
  */
class NSListenerRenderer3D : public NSListenerBase3D
{
public:
  AR_UNSTUBBED AR_CLASS(NSListenerRenderer3D)
//---------------------- Constructors / Destructor ---------------------------
  AR_CONSTRUCTOR(NSListenerRenderer3D)

//---------------------------- Public methods --------------------------------
public:
  //! Controler will sign to all object events and will be able to transmit
  //! them. (getPendingEvents());
  virtual void
  init(ArRef<LocalDepService> service, ArRef<ArObject> object);

  //! Return events backup (eg : motion, setup change ...)
  virtual void
  getPendingEvents(StlList<ArRef<NSEvent> > &events);

//------------------------- Forbiden operations ------------------------------
protected:
  virtual void
  _onViewParametersCB(const Renderer3D::ViewParametersEvent &event);

  virtual void
  _onRenderingSetupCB(const Renderer3D::RenderingSetupEvent &event);

  virtual void
  _onAppearanceSetupCB(const Renderer3D::AppearanceSetupEvent &event);

  virtual void
  _onAddOSDCB(const Renderer3D::AddOSDEvent &event);

  virtual void
  _onRemoveOSDCB(const Renderer3D::RemoveOSDEvent &event);

//----------------------------- Attributes -----------------------------------
protected:
  ArPtr<Renderer3D> _renderer;
  bool _viewParametersFlag;
  bool _renderingSetupFlag;
  bool _appearanceSetupFlag;
  StlList<ArRef<NSEvent> > _events;
};

//----------------------------------------------------------------------------
// CLASS       : NSInteractorRenderer3D
//----------------------------------------------------------------------------

/*! \class NSInteractorRenderer3D
  * Network Service Event class
  */
class NSInteractorRenderer3D : public NSInteractorBase3D
{
public:
  AR_UNSTUBBED AR_CLASS(NSInteractorRenderer3D)
//---------------------- Constructors / Destructor ---------------------------
  AR_CONSTRUCTOR(NSInteractorRenderer3D)

//---------------------------- Public methods --------------------------------
public:
  //! Controler will sign to all object network events and will be able to
  //! affect remote object with.
  virtual void
  init(ArRef<RemoteDepService> service, ArRef<ArObject> object);

//------------------------- Forbiden operations ------------------------------
protected:
  virtual NSEvent::EventProcess
  _onViewParametersEvent(ArConstRef<NSEvent> event);

  virtual NSEvent::EventProcess
  _onRenderingSetupEvent(ArConstRef<NSEvent> event);

  virtual NSEvent::EventProcess
  _onAppearanceSetupEvent(ArConstRef<NSEvent> event);

  virtual NSEvent::EventProcess
  _onAddOSDEvent(ArConstRef<NSEvent> event);

  virtual NSEvent::EventProcess
  _onRemoveOSDEvent(ArConstRef<NSEvent> event);

//----------------------------- Attributes -----------------------------------
protected:
  ArPtr<Renderer3D> _renderer;
};

} // end of namespace AReVi
#endif // end of AREVINS_NSCONTROLERRENDERER3D_H
//---End-Of-File---------------------------------------------------------------
