// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/Controlers/nsCtrlScene3D.h
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVINS_NSCONTROLERSCENE3D_H
#define AREVINS_NSCONTROLERSCENE3D_H 1

//--- Imported classes -------------------------------------------------------
#include "AReViNS/Controlers/nsControler.h"
#include "AReVi/Lib3D/scene3D.h"

namespace AReVi {
//----------------------------------------------------------------------------
// CLASS       : NSListenerScene3D
//----------------------------------------------------------------------------
/*! \class NSListenerScene3D
  * Network Service Event class
  */
class NSListenerScene3D : public NSListener
{
public:
  AR_UNSTUBBED AR_CLASS(NSListenerScene3D)
//---------------------- Constructors / Destructor ---------------------------
  AR_CONSTRUCTOR(NSListenerScene3D)

//---------------------------- Public methods --------------------------------
public:
  //! Controler will sign to all object events and will be able to transmit
  //! them. (getPendingEvents());
  virtual void
  init(ArRef<LocalDepService> service, ArRef<ArObject> object);

  //! Return events backup (eg : motion, setup change ...)
  virtual void
  getPendingEvents(StlList<ArRef<NSEvent> > &events);


//------------------------- Forbiden operations ------------------------------
protected:
//! Scene Setup (fog, background, backTransform...)
  virtual void
  _onSceneSetupCB(const Scene3D::SceneSetupEvent &event);

//! Add Object
  virtual void
  _onAddObjectCB(const Scene3D::AddObjectEvent &event);

//! Remove Object
  virtual void
  _onRemoveObjectCB(const Scene3D::RemoveObjectEvent &event);

//! Select Object
  virtual void
  _onSelectObjectCB(const Scene3D::SelectObjectEvent &event);

//! Unselect Object
  virtual void
  _onUnselectObjectCB(const Scene3D::UnselectObjectEvent &event);

//! Add Light
  virtual void
  _onAddLightCB(const Scene3D::AddLightEvent &event);

//! Remove Light
  virtual void
  _onRemoveLightCB(const Scene3D::RemoveLightEvent &event);

//! Add ParticleSystem
  virtual void
  _onAddParticleSystemCB(const Scene3D::AddParticleSystemEvent &event);

//! Remove ParticleSystem
  virtual void
  _onRemoveParticleSystemCB(const Scene3D::RemoveParticleSystemEvent &event);

//! Add Sound
  virtual void
  _onAddSoundCB(const Scene3D::AddSoundEvent &event);

//! Remove Sound
  virtual void
  _onRemoveSoundCB(const Scene3D::RemoveSoundEvent &event);

//----------------------------- Attributes -----------------------------------
protected:
  ArPtr<Scene3D> _scene;
  bool _setupFlag;
  StlList<ArRef<NSEvent> > _events;
};

//----------------------------------------------------------------------------
// CLASS       : NSInteractorScene3D
//----------------------------------------------------------------------------
/*! \class NSInteractorScene3D
  * Network Service Event class
  */
class NSInteractorScene3D : public NSInteractor
{
public:
  AR_UNSTUBBED AR_CLASS(NSInteractorScene3D)
//---------------------- Constructors / Destructor ---------------------------
  AR_CONSTRUCTOR(NSInteractorScene3D)

//---------------------------- Public methods --------------------------------
public:
  //! Controler will sign to all object network events and will be able to
  //! affect remote object with.
  virtual void
  init(ArRef<RemoteDepService> service, ArRef<ArObject> object);

//------------------------- Forbiden operations ------------------------------
protected:
//! Viewer Setup (fog, background, backTransform...)
  virtual NSEvent::EventProcess
  _onSceneSetupEvent(ArConstRef<NSEvent> event);

//! Add Object
  virtual NSEvent::EventProcess
  _onAddObjectEvent(ArConstRef<NSEvent> event);

//! Remove Object
  virtual NSEvent::EventProcess
  _onRemoveObjectEvent(ArConstRef<NSEvent> event);

//! Select Object
  virtual NSEvent::EventProcess
  _onSelectObjectEvent(ArConstRef<NSEvent> event);

//! Unselect Object
  virtual NSEvent::EventProcess
  _onUnselectObjectEvent(ArConstRef<NSEvent> event);

//! Add Light
  virtual NSEvent::EventProcess
  _onAddLightEvent(ArConstRef<NSEvent> event);

//! Remove Light
  virtual NSEvent::EventProcess
  _onRemoveLightEvent(ArConstRef<NSEvent> event);

//! Add ParticleSystem
  virtual NSEvent::EventProcess
  _onAddParticleSystemEvent(ArConstRef<NSEvent> event);

//! Remove ParticleSystem
  virtual NSEvent::EventProcess
  _onRemoveParticleSystemEvent(ArConstRef<NSEvent> event);

//! Add Sound
  virtual NSEvent::EventProcess
  _onAddSoundEvent(ArConstRef<NSEvent> event);

//! Remove Sound
  virtual NSEvent::EventProcess
  _onRemoveSoundEvent(ArConstRef<NSEvent> event);
//----------------------------- Attributes -----------------------------------
protected:
  ArPtr<Scene3D> _scene;
};

} // end of namespace AReVi
#endif // end of AREVINS_NSCONTROLERSCENE3D_H
//---End-Of-File---------------------------------------------------------------
