// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/Controlers/nsCtrlShapePart3D.h
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVINS_CONTROLERSHAPEPART3D_H
#define AREVINS_CONTROLERSHAPEPART3D_H 1

//--- Imported classes -------------------------------------------------------
#include "AReViNS/Controlers/nsControler.h"
#include "AReVi/Shapes/shapePart3D.h"

namespace AReVi {

//----------------------------------------------------------------------------
// CLASS       : NSListenerShapePart3D
//----------------------------------------------------------------------------
/*! \class NSListenerShapePart3D
  * Network Service Event class
  */
class NSListenerShapePart3D : public NSListener
{
public:
  AR_UNSTUBBED AR_CLASS(NSListenerShapePart3D)
//---------------------- Constructors / Destructor ---------------------------
  AR_CONSTRUCTOR(NSListenerShapePart3D)

//---------------------------- Public methods --------------------------------
public:
  //! Controler will sign to all object events and will be able to transmit
  //! them. (getPendingEvents());
  virtual void
  init(ArRef<LocalDepService> service, ArRef<ArObject> object);

  //! Return events backup (eg : motion, setup change ...)
  virtual void
  getPendingEvents(StlList<ArRef<NSEvent> > &events);

//------------------------- Forbiden operations ------------------------------
protected:
  virtual void
  _onTransformationCB(const ShapePart3D::TransformationEvent &event);

  virtual void
  _onStructureChangeCB(const ShapePart3D::StructureChangeEvent &event);


  virtual void
  _onAppearanceRebuildCB(const ShapePart3D::AppearanceRebuildEvent &event);

  virtual void
  _onGeometryRebuildCB(const ShapePart3D::GeometryRebuildEvent &event);

//----------------------------- Attributes -----------------------------------
protected:
  ArPtr<ShapePart3D> _part;
  bool _geometryRebuildFlag;
  bool _appearanceRebuildFlag;
  bool _transformationFlag;
  bool _structureChangeFlag;
};

//----------------------------------------------------------------------------
// CLASS       : NSInteractorShapePart3D
//----------------------------------------------------------------------------
/*! \class NSInteractorShapePart3D
  * Network Service Event class
  */
class NSInteractorShapePart3D : public NSInteractor
{
public:
  AR_UNSTUBBED AR_CLASS(NSInteractorShapePart3D)
//---------------------- Constructors / Destructor ---------------------------
  AR_CONSTRUCTOR(NSInteractorShapePart3D)

//---------------------------- Public methods --------------------------------
public:
  //! Controler will sign to all object network events and will be able to
  //! affect remote object with.
  virtual void
  init(ArRef<RemoteDepService> service, ArRef<ArObject> object);

//------------------------- Forbiden operations ------------------------------
protected:
  virtual NSEvent::EventProcess
  _onTransformationEvent(ArConstRef<NSEvent> event);

  virtual NSEvent::EventProcess
  _onStructureChangeEvent(ArConstRef<NSEvent> event);

  virtual NSEvent::EventProcess
  _onAppearanceRebuildEvent(ArConstRef<NSEvent> event);

  virtual NSEvent::EventProcess
  _onGeometryRebuildEvent(ArConstRef<NSEvent> event);

//----------------------------- Attributes -----------------------------------
protected:
  ArPtr<ShapePart3D> _part;
};

} // end of namespace AReVi
#endif // end of AREVINS_CONTROLERSHAPEPART3D_H
//---End-Of-File---------------------------------------------------------------
