// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/Events/nsEvent.h
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVINS_NSEVENT_H
#define AREVINS_NSEVENT_H 1

//--- Imported classes -------------------------------------------------------
#include "AReVi/arObject.h"

//----------------------------------------------------------------------------
// CLASS       : NSEvent
//----------------------------------------------------------------------------
namespace AReVi {
class NetworkService;

/*! \class NSEvent
  * Abstract Event
  */
class NSEvent : public ArObject
{
  friend class NetworkService;

public:
  enum EventProcess
        { EP_OK,   // Event process success
          EP_FAIL, // Event process fail => event will not be processed again
          EP_LATER,// Event process will be done later(pause events process)
          EP_POST};// Event process will be done later(push back event and continue)

public:
  AR_UNSTUBBED AR_CLASS(NSEvent)
//---------------------- Constructors / Destructor ---------------------------
  AR_CONSTRUCTOR(NSEvent)

//---------------------------- Public methods --------------------------------
public:
  inline int
  getServiceId(void) const;

  inline void
  setCompress(bool compress);

  inline bool
  getCompress(void) const;

//-------------------------- Stream operations -------------------------------
  virtual void
  writeToStream(ArRef<AbstractOStream> output) const;

  virtual bool // success
  readFromStream(ArRef<AbstractIStream> input);

//------------------------- Forbiden operations ------------------------------
protected:
  void
  _setNetworkService(ArRef<NetworkService> service, const int &senderId);

//----------------------------- Attributes -----------------------------------
protected:
  ArRef<NetworkService> _service;
  int _serviceId;//sender
  bool _compress;//event can be compressed
};

//----------------------------------------------------------------------------
// Inline implementations
//----------------------------------------------------------------------------
inline int
NSEvent::getServiceId(void) const
{
  return _serviceId;
}

inline void
NSEvent::setCompress(bool compress)
{
  _compress=compress;
}

inline bool
NSEvent::getCompress(void) const
{
  return _compress;
}


//----------------------------------------------------------------------------
// CLASS       : NSEventConnection
//----------------------------------------------------------------------------
/*! \class NSEventConnection
  * Inform server of a connection
  */
class NSEventConnection : public NSEvent
{
public:
  AR_UNSTUBBED AR_CLASS(NSEventConnection)
//---------------------- Constructors / Destructor ---------------------------
  AR_CONSTRUCTOR(NSEventConnection)
};


//----------------------------------------------------------------------------
// CLASS       : NSEventPingRequest=> request a ping message
//----------------------------------------------------------------------------
/*! \class NSEventPingRequest  */
class NSEventPingRequest : public NSEvent
{
public:
  AR_UNSTUBBED AR_CLASS(NSEventPingRequest)
//---------------------- Constructors / Destructor ---------------------------
  AR_CONSTRUCTOR(NSEventPingRequest)
};

//----------------------------------------------------------------------------
// CLASS       : NSEventPing
//----------------------------------------------------------------------------
/*! \class NSEventPing  */
class NSEventPing : public NSEvent
{
public:
  AR_UNSTUBBED AR_CLASS(NSEventPing)
//---------------------- Constructors / Destructor ---------------------------
  AR_CONSTRUCTOR(NSEventPing)
};


//----------------------------------------------------------------------------
// CLASS       : NSEventRef
//----------------------------------------------------------------------------
/*! \class NSEventRef
  * Event related to a reference ID is based on server Alias!
  */
class NSEventRef : public NSEvent
{
public:
  AR_UNSTUBBED AR_CLASS(NSEventRef)
//---------------------- Constructors / Destructor ---------------------------
  AR_CONSTRUCTOR(NSEventRef)

//---------------------------- Public methods --------------------------------
public:
  //! Get Set ID of object in server/local part
  const StlString &
  getID(void) const;

  void
  setID(const StlString &id);

//-------------------------- Stream operations -------------------------------
  virtual void
  writeToStream(ArRef<AbstractOStream> output) const;

  virtual bool // success
  readFromStream(ArRef<AbstractIStream> input);

//------------------------- Forbiden operations ------------------------------
protected:
//----------------------------- Attributes -----------------------------------
  StlString _ID;
};

//----------------------------------------------------------------------------
// CLASS       : NSEventSessionName
//----------------------------------------------------------------------------
/*! \class NSEventSessionName  */
class NSEventSessionName : public NSEventRef
{
public:
  AR_UNSTUBBED AR_CLASS(NSEventSessionName)
//---------------------- Constructors / Destructor ---------------------------
  AR_CONSTRUCTOR(NSEventSessionName)
};


//----------------------------------------------------------------------------
// CLASS       : NSEventArRef
//----------------------------------------------------------------------------
/*! \class NSEventArRef
  * Event related to a reference ID is based on server Alias!
  * Here className is gived.
  */
class NSEventArRef : public NSEventRef
{
public:
  AR_UNSTUBBED AR_CLASS(NSEventArRef)
//---------------------- Constructors / Destructor ---------------------------
  AR_CONSTRUCTOR(NSEventArRef)

//---------------------------- Public methods --------------------------------
public:
  //! Get Class Name of object in server/local part
  const StlString &
  getClassName(void) const;

  virtual void
  setRef(ArConstRef<ArObject> ref);

//-------------------------- Stream operations -------------------------------
  virtual void
  writeToStream(ArRef<AbstractOStream> output) const;

  virtual bool // success
  readFromStream(ArRef<AbstractIStream> input);

//------------------------- Forbiden operations ------------------------------
protected:
//----------------------------- Attributes -----------------------------------
  StlString _className;
};


} // end of namespace AReVi
#endif // end of AREVINS_NSEVENT_H
//---End-Of-File---------------------------------------------------------------
