// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/Events/nsEventArConsole.h
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVINS_NSEVENTARCONSOLE_H
#define AREVINS_NSEVENTARCONSOLE_H

//--- Imported classes -------------------------------------------------------
#include "AReVi/Lib3D/util3D_priv.h"
#include "AReViNS/Events/nsEvent.h"
#include "AReViNS/remoteDepService.h"
#include "AReVi/Contrib/arConsole.h"

namespace AReVi {
//----------------------------------------------------------------------------
// CLASS       : NSEventArConsoleAttrib
//----------------------------------------------------------------------------
/*! \class NSEventArConsoleAttrib
  *
  */
class NSEventArConsoleAttrib : public NSEvent
{
public:
  AR_UNSTUBBED AR_CLASS(NSEventArConsoleAttrib)
//---------------------- Constructors / Destructor ---------------------------
  AR_CONSTRUCTOR(NSEventArConsoleAttrib)

//---------------------------- Public methods --------------------------------
public:
  //! Read/Write views parameters
  virtual void
  readAttrib(ArConsoleAttr &attribOut) const;

  virtual void
  writeAttrib(const ArConsoleAttr& attrib);

  virtual void
  writeColor(double r, double g,double b,double a);

  virtual void
  writeFont(const StlString &font);

//-------------------------- Stream operations -------------------------------
  virtual void
  writeToStream(ArRef<AbstractOStream> output) const;

  virtual bool // success
  readFromStream(ArRef<AbstractIStream> input);

//------------------------- Forbiden operations ------------------------------
protected:
//----------------------------- Attributes -----------------------------------
  double _r,_g,_b,_a;
  StlString _font;
};


//----------------------------------------------------------------------------
// CLASS       : NSEventArConsoleBackground
//----------------------------------------------------------------------------
/*! \class NSEventArConsoleBackground
  *
  */
class NSEventArConsoleBackground : public NSEventArConsoleAttrib
{
public:
  AR_UNSTUBBED AR_CLASS(NSEventArConsoleBackground)
//---------------------- Constructors / Destructor ---------------------------
  AR_CONSTRUCTOR(NSEventArConsoleBackground)
};

//----------------------------------------------------------------------------
// CLASS       : NSEventArConsoleForeground
//----------------------------------------------------------------------------
/*! \class NSEventArConsoleForeground
  *
  */
class NSEventArConsoleForeground : public NSEventArConsoleAttrib
{
public:
  AR_UNSTUBBED AR_CLASS(NSEventArConsoleForeground)
//---------------------- Constructors / Destructor ---------------------------
  AR_CONSTRUCTOR(NSEventArConsoleForeground)
};

//----------------------------------------------------------------------------
// CLASS       : NSEventArConsolePopAttrib
//----------------------------------------------------------------------------
/*! \class NSEventArConsolePopAttrib
  *
  */
class NSEventArConsolePopAttrib : public NSEvent
{
public:
  AR_UNSTUBBED AR_CLASS(NSEventArConsolePopAttrib)
//---------------------- Constructors / Destructor ---------------------------
  AR_CONSTRUCTOR(NSEventArConsolePopAttrib)
};

//----------------------------------------------------------------------------
// CLASS       : NSEventArConsoleFlushAttribs
//----------------------------------------------------------------------------
/*! \class NSEventArConsoleFlushAttribs
  *
  */
class NSEventArConsoleFlushAttribs : public NSEvent
{
public:
  AR_UNSTUBBED AR_CLASS(NSEventArConsoleFlushAttribs)
//---------------------- Constructors / Destructor ---------------------------
  AR_CONSTRUCTOR(NSEventArConsoleFlushAttribs)
};


//----------------------------------------------------------------------------
// CLASS       : NSEventArConsoleClearText
//----------------------------------------------------------------------------
/*! \class NSEventArConsoleClearText
  *
  */
class NSEventArConsoleClearText : public NSEvent
{
public:
  AR_UNSTUBBED AR_CLASS(NSEventArConsoleClearText)
//---------------------- Constructors / Destructor ---------------------------
  AR_CONSTRUCTOR(NSEventArConsoleClearText)
};

//----------------------------------------------------------------------------
// CLASS       : NSEventArConsoleBufferSetup
//----------------------------------------------------------------------------
/*! \class NSEventArConsoleBufferSetup
  *
  */
class NSEventArConsoleBufferSetup : public NSEvent
{
public:
  AR_UNSTUBBED AR_CLASS(NSEventArConsoleBufferSetup)
//---------------------- Constructors / Destructor ---------------------------
  AR_CONSTRUCTOR(NSEventArConsoleBufferSetup)

//---------------------------- Public methods --------------------------------
public:
  //! Read/Write views parameters
  virtual void
  readArConsole(ArRef<ArConsole> arConsole) const;

  virtual void
  writeArConsole(ArConstRef<ArConsole> arConsole);

//-------------------------- Stream operations -------------------------------
  virtual void
  writeToStream(ArRef<AbstractOStream> output) const;

  virtual bool // success
  readFromStream(ArRef<AbstractIStream> input);

//------------------------- Forbiden operations ------------------------------
protected:
//----------------------------- Attributes -----------------------------------
  int _bufferSize;
  int _displaySize;
};

//----------------------------------------------------------------------------
// CLASS       : NSEventArConsoleScrollText
//----------------------------------------------------------------------------
/*! \class NSEventArConsoleScrollText
  *
  */
class NSEventArConsoleScrollText : public NSEvent
{
public:
  AR_UNSTUBBED AR_CLASS(NSEventArConsoleScrollText)
//---------------------- Constructors / Destructor ---------------------------
  AR_CONSTRUCTOR(NSEventArConsoleScrollText)

//---------------------------- Public methods --------------------------------
public:
  //! Read/Write views parameters
  virtual void
  readArConsole(ArRef<ArConsole> arConsole) const;

  virtual void
  setNbLines(int nbLines);

//-------------------------- Stream operations -------------------------------
  virtual void
  writeToStream(ArRef<AbstractOStream> output) const;

  virtual bool // success
  readFromStream(ArRef<AbstractIStream> input);

//------------------------- Forbiden operations ------------------------------
protected:
//----------------------------- Attributes -----------------------------------
  int _nbLines;
};

//----------------------------------------------------------------------------
// CLASS       : NSEventArConsolePushText
//----------------------------------------------------------------------------
/*! \class NSEventArConsolePushText
  *
  */
class NSEventArConsolePushText : public NSEvent
{
public:
  AR_UNSTUBBED AR_CLASS(NSEventArConsolePushText)
//---------------------- Constructors / Destructor ---------------------------
  AR_CONSTRUCTOR(NSEventArConsolePushText)

//---------------------------- Public methods --------------------------------
public:
  //! Read/Write views parameters
  virtual void
  readArConsole(ArRef<ArConsole> arConsole) const;

  virtual void
  setText(const StlString &text);

//-------------------------- Stream operations -------------------------------
  virtual void
  writeToStream(ArRef<AbstractOStream> output) const;

  virtual bool // success
  readFromStream(ArRef<AbstractIStream> input);

//------------------------- Forbiden operations ------------------------------
protected:
//----------------------------- Attributes -----------------------------------
  StlString _text;
};

//----------------------------------------------------------------------------
// CLASS       : NSEventArConsoleStoreCharacter
//----------------------------------------------------------------------------
/*! \class NSEventArConsoleStoreCharacter
  *
  */
class NSEventArConsoleStoreCharacter : public NSEvent
{
public:
  AR_UNSTUBBED AR_CLASS(NSEventArConsoleStoreCharacter)
//---------------------- Constructors / Destructor ---------------------------
  AR_CONSTRUCTOR(NSEventArConsoleStoreCharacter)

//---------------------------- Public methods --------------------------------
public:
  //! Read/Write views parameters
  virtual void
  readArConsole(ArRef<ArConsole> arConsole) const;

  virtual void
  setChar(char c);

//-------------------------- Stream operations -------------------------------
  virtual void
  writeToStream(ArRef<AbstractOStream> output) const;

  virtual bool // success
  readFromStream(ArRef<AbstractIStream> input);

//------------------------- Forbiden operations ------------------------------
protected:
//----------------------------- Attributes -----------------------------------
  char _c;
};


} // end of namespace AReVi
#endif // end of AREVINS_NSEVENTARCONSOLE_H
//---End-Of-File---------------------------------------------------------------
