// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/Events/nsEventControler.h
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVINS_NSEVENTCONTOLER_H
#define AREVINS_NSEVENTCONTOLER_H 1

//--- Imported classes -------------------------------------------------------
#include "AReViNS/Events/nsEvent.h"

namespace AReVi {
class MemoryBlock;

//----------------------------------------------------------------------------
// CLASS       : NSEventControler
//----------------------------------------------------------------------------
/*! \class NSEventControler
  * Send an event from a Listener to an Interactor
  */
class NSEventControler : public NSEventRef
{
public:
  AR_UNSTUBBED AR_CLASS(NSEventControler)
//---------------------- Constructors / Destructor ---------------------------
  AR_CONSTRUCTOR(NSEventControler)

//---------------------------- Public methods --------------------------------
public:
  void
  setEvent(ArRef<NSEvent> event);

  ArConstRef<NSEvent>
  getEvent(void) const;

//-------------------------- Stream operations -------------------------------
  virtual void
  writeToStream(ArRef<AbstractOStream> output) const;

  virtual bool // success
  readFromStream(ArRef<AbstractIStream> input);

//------------------------- Forbiden operations ------------------------------
protected:
//----------------------------- Attributes -----------------------------------
  ArRef<NSEvent>    _event;
};

//----------------------------------------------------------------------------
// CLASS       : NSEventControlerPack
//----------------------------------------------------------------------------
/*! \class NSEventControlerPack
  * This contain a pack of messages
  */
class NSEventControlerPack : public NSEvent
{
public:
  AR_UNSTUBBED AR_CLASS(NSEventControlerPack)
//---------------------- Constructors / Destructor ---------------------------
  AR_CONSTRUCTOR(NSEventControlerPack)

//---------------------------- Public methods --------------------------------
public:
  void
  setEvents(StlList<ArRef<NSEvent> > &event);

  const StlList<ArRef<NSEvent> > &
  getEvents(void) const;

//-------------------------- Stream operations -------------------------------
  virtual void
  writeToStream(ArRef<AbstractOStream> output) const;

  virtual bool // success
  readFromStream(ArRef<AbstractIStream> input);

//------------------------- Forbiden operations ------------------------------
protected:
//----------------------------- Attributes -----------------------------------
  StlList<ArRef<NSEvent> > _events;
};

//----------------------------------------------------------------------------
// CLASS       : NSEventRemoteDepReady
//----------------------------------------------------------------------------
/*! \class NSEventRemoteDepReady
  * This remote dependencies service have processed all previous messages
  */
class NSEventRemoteDepReady : public NSEvent
{
public:
  AR_UNSTUBBED AR_CLASS(NSEventRemoteDepReady)
//---------------------- Constructors / Destructor ---------------------------
  AR_CONSTRUCTOR(NSEventRemoteDepReady)
};

} // end of namespace AReVi
#endif // end of AREVINS_NSEVENTCONTOLER_H
//---End-Of-File---------------------------------------------------------------
