// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/Events/nsEventFile.h
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVINS_NSEVENTFILE_H
#define AREVINS_NSEVENTFILE_H 1

//--- Imported classes -------------------------------------------------------
#include "AReViNS/Events/nsEvent.h"

namespace AReVi
{
  class MemoryBlock;
}

namespace AReVi {
//----------------------------------------------------------------------------
// CLASS       : NSEventFileRequest
//----------------------------------------------------------------------------
/*! \class NSEventFileRequest
  * Request serialisation of specified file url from client to server
  */
class NSEventFileRequest : public NSEvent
{
public:
  AR_UNSTUBBED AR_CLASS(NSEventFileRequest)
//---------------------- Constructors / Destructor ---------------------------
  AR_CONSTRUCTOR(NSEventFileRequest)

//---------------------------- Public methods --------------------------------
public:
  //! Get/Set URL of file
  const StlString &
  getURL(void) const;

  virtual void
  setURL(const StlString &url);

//-------------------------- Stream operations -------------------------------
  virtual void
  writeToStream(ArRef<AbstractOStream> output) const;

  virtual bool // success
  readFromStream(ArRef<AbstractIStream> input);

//------------------------- Forbiden operations ------------------------------
protected:
//----------------------------- Attributes -----------------------------------
  StlString _url;
};


//----------------------------------------------------------------------------
// CLASS       : NSEventFile
//----------------------------------------------------------------------------
/*! \class NSEventFile
  * Transmit serialized file
  */
class NSEventFile : public NSEventFileRequest
{
public:
  AR_UNSTUBBED AR_CLASS(NSEventFile)
//---------------------- Constructors / Destructor ---------------------------
  AR_CONSTRUCTOR(NSEventFile)

//---------------------------- Public methods --------------------------------
public:
  ArConstRef<MemoryBlock>
  getData(void) const;

  virtual void
  setURL(const StlString &url);

//-------------------------- Stream operations -------------------------------
  virtual void
  writeToStream(ArRef<AbstractOStream> output) const;

  virtual bool // success
  readFromStream(ArRef<AbstractIStream> input);

//------------------------- Forbiden operations ------------------------------
protected:
//----------------------------- Attributes -----------------------------------
  ArRef<MemoryBlock> _data;
};
} // end of namespace AReVi
#endif // end of AREVINS_NSEVENTFILE_H
//---End-Of-File---------------------------------------------------------------
