// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/Events/nsEventObject3D.h
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVINS_NSEVENTOBJECT3D_H
#define AREVINS_NSEVENTOBJECT3D_H 1

//--- Imported classes -------------------------------------------------------
#include "AReViNS/Events/nsEvent.h"
#include "AReViNS/remoteDepService.h"
#include "AReVi/Lib3D/util3D_priv.h"

//----------------------------------------------------------------------------
// CLASS       : NSEventObject3DShapeChange
//----------------------------------------------------------------------------
namespace AReVi {
class Object3D;
class Material3D;

/*! \class NSEventObject3DShapeChange
  * Object3D setShape (LODShape event)
  */
class NSEventObject3DShapeChange : public NSEvent
{
public:
  AR_UNSTUBBED AR_CLASS(NSEventObject3DShapeChange)
//---------------------- Constructors / Destructor ---------------------------
  AR_CONSTRUCTOR(NSEventObject3DShapeChange)

//---------------------------- Public methods --------------------------------
public:
  virtual void
  writeObject3D(ArConstRef<Object3D> object);

  virtual bool // success => if fail, write error msg Zin "object"
  readObject3D(ArRef<Object3D> object,
               ArRef<RemoteDepService> depService) const;

//-------------------------- Stream operations -------------------------------
  virtual void
  writeToStream(ArRef<AbstractOStream> output) const;

  virtual bool // success
  readFromStream(ArRef<AbstractIStream> input);

//------------------------- Forbiden operations ------------------------------
protected:
//----------------------------- Attributes -----------------------------------
  StlVector<RemoteRef> _shapes;
  StlVector<double> _distances;
};


//----------------------------------------------------------------------------
// CLASS       : NSEventObject3DAppearance
//----------------------------------------------------------------------------
/*! \class NSEventObject3DAppearance
  * Object3D appearance changed
  */
class NSEventObject3DAppearance : public NSEvent
{
public:
  AR_UNSTUBBED AR_CLASS(NSEventObject3DAppearance)
//---------------------- Constructors / Destructor ---------------------------
  AR_CONSTRUCTOR(NSEventObject3DAppearance)

//---------------------------- Public methods --------------------------------
public:
  virtual void
  writeObject3D(ArConstRef<Object3D> object);

  virtual void
  readObject3D(ArRef<Object3D> object) const;

//-------------------------- Stream operations -------------------------------
  virtual void
  writeToStream(ArRef<AbstractOStream> output) const;

  virtual bool // success
  readFromStream(ArRef<AbstractIStream> input);

//------------------------- Forbiden operations ------------------------------
protected:
//----------------------------- Attributes -----------------------------------
  bool _wireFrame;
  bool _drawShapeBBox;
  bool _drawShapePartBBox;
  ArRef<Material3D> _material;
};

} // end of namespace AReVi
#endif // end of AREVINS_NSEVENTOBJECT3D_H
//---End-Of-File---------------------------------------------------------------
