// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/Events/nsEventObjectRef.h
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVINS_NSEVENTOBJECTREF_H
#define AREVINS_NSEVENTOBJECTREF_H 1

//--- Imported classes -------------------------------------------------------
#include "AReViNS/Events/nsEvent.h"

namespace AReVi
{
  class MemoryBlock;
}

namespace AReVi {
//----------------------------------------------------------------------------
// CLASS       : NSEventObjectRefRequest
//----------------------------------------------------------------------------
/*! \class NSEventObjectRefRequest
  * Request serialisation of specified reference from client to server
  */
class NSEventObjectRefRequest : public NSEventRef
{
public:
  AR_UNSTUBBED AR_CLASS(NSEventObjectRefRequest)
//---------------------- Constructors / Destructor ---------------------------
  AR_CONSTRUCTOR(NSEventObjectRefRequest)
};


//----------------------------------------------------------------------------
// CLASS       : NSEventObjectRef
//----------------------------------------------------------------------------
/*! \class NSEventObjectRef
  * Transmit serialized object
  */
class NSEventObjectRef : public NSEventArRef
{
public:
  AR_UNSTUBBED AR_CLASS(NSEventObjectRef)
//---------------------- Constructors / Destructor ---------------------------
  AR_CONSTRUCTOR(NSEventObjectRef)

//---------------------------- Public methods --------------------------------
public:
  virtual void
  setRef(ArConstRef<ArObject> object);

  const StlVector<StlString> &
  getURLs(void) const;

  const StlVector<StlString> &
  getRefs(void) const;

  const StlVector<StlString> &
  getClassRefs(void) const;

  ArConstRef<MemoryBlock>
  getData(void) const;

//-------------------------- Stream operations -------------------------------
  virtual void
  writeToStream(ArRef<AbstractOStream> output) const;

  virtual bool // success
  readFromStream(ArRef<AbstractIStream> input);

//------------------------- Forbiden operations ------------------------------
protected:
//----------------------------- Attributes -----------------------------------
  StlVector<StlString>      _urls;
  StlVector<StlString>      _refs;
  StlVector<StlString>      _classRefs;
  ArRef<MemoryBlock>        _objectData;
};

//----------------------------------------------------------------------------
// CLASS       : NSEventArRefList
//----------------------------------------------------------------------------
/*! \class NSEventArRefList
  * Transmit list of references
  */
class NSEventArRefList : public NSEvent
{
public:
  AR_UNSTUBBED AR_CLASS(NSEventArRefList)
//---------------------- Constructors / Destructor ---------------------------
  AR_CONSTRUCTOR(NSEventArRefList)

//---------------------------- Public methods --------------------------------
public:
  virtual void
  setRefs(const StlVector<ArRef<ArObject> > &refs);

  virtual void
  addRef(ArConstRef<ArObject> ref);

  const StlVector<StlString> &
  getRefs(void) const;

  const StlVector<StlString> &
  getClassRefs(void) const;

//-------------------------- Stream operations -------------------------------
  virtual void
  writeToStream(ArRef<AbstractOStream> output) const;

  virtual bool // success
  readFromStream(ArRef<AbstractIStream> input);

//------------------------- Forbiden operations ------------------------------
protected:
//----------------------------- Attributes -----------------------------------
  StlVector<StlString>      _refs;
  StlVector<StlString>      _classRefs;
};

//----------------------------------------------------------------------------
// CLASS       : NSEventDownCastMap
//----------------------------------------------------------------------------
/*! \class NSEventDownCastMap
  * Transmit downcastMap
  */
class NSEventDownCastMap : public NSEvent
{
public:
  AR_UNSTUBBED AR_CLASS(NSEventDownCastMap)
//---------------------- Constructors / Destructor ---------------------------
  AR_CONSTRUCTOR(NSEventDownCastMap)

//---------------------------- Public methods --------------------------------
public:
  virtual void
  setMap(const StlMap<StlString,StlString> &map);

  const StlMap<StlString,StlString> &
  getMap(void) const;

//-------------------------- Stream operations -------------------------------
  virtual void
  writeToStream(ArRef<AbstractOStream> output) const;

  virtual bool // success
  readFromStream(ArRef<AbstractIStream> input);

//------------------------- Forbiden operations ------------------------------
protected:
//----------------------------- Attributes -----------------------------------
  StlMap<StlString,StlString>      _map;
};

//----------------------------------------------------------------------------
// CLASS       : NSEventObjectListenerRequest
//----------------------------------------------------------------------------
/*! \class NSEventObjectListenerRequest
  * Request instanciation of a listener on server.
  */
class NSEventObjectListenerRequest : public NSEventRef
{
public:
  AR_UNSTUBBED AR_CLASS(NSEventObjectListenerRequest)
//---------------------- Constructors / Destructor ---------------------------
  AR_CONSTRUCTOR(NSEventObjectListenerRequest)
};


} // end of namespace AReVi
#endif // end of AREVINS_NSEVENTOBJECTREF_H
//---End-Of-File---------------------------------------------------------------
