// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/Events/nsEventOSD.h
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVINS_NSEVENTOSD_H
#define AREVINS_NSEVENTOSD_H 1

//--- Imported classes -------------------------------------------------------
#include "AReViNS/Events/nsEvent.h"
#include "AReViNS/remoteDepService.h"

//----------------------------------------------------------------------------
// CLASS       : NSEventOSDSetup
//----------------------------------------------------------------------------
namespace AReVi {
class OSD;
class OSDText;
class OSDSprite;

/*! \class NSEventOSDSetup
  *
  */
class NSEventOSDSetup : public NSEvent
{
public:
  AR_UNSTUBBED AR_CLASS(NSEventOSDSetup)
//---------------------- Constructors / Destructor ---------------------------
  AR_CONSTRUCTOR(NSEventOSDSetup)

//---------------------------- Public methods --------------------------------
public:
  //! Read/Write views parameters
  virtual void
  readOSD(ArRef<OSD> osd) const;

  virtual void
  writeOSD(ArConstRef<OSD> osd);

//-------------------------- Stream operations -------------------------------
  virtual void
  writeToStream(ArRef<AbstractOStream> output) const;

  virtual bool // success
  readFromStream(ArRef<AbstractIStream> input);

//------------------------- Forbiden operations ------------------------------
protected:
//----------------------------- Attributes -----------------------------------
  int _location;
  double _offsetX;
  double _offsetY;
  double _depth;
  double _alpha;
  double _padding[2];
  int _px;
  int _py;
  double _orientation;
};

//----------------------------------------------------------------------------
// CLASS       : NSEventOSDTextSetup
//----------------------------------------------------------------------------
/*! \class NSEventOSDTextSetup
  *
  */
class NSEventOSDTextSetup : public NSEventOSDSetup
{
public:
  AR_UNSTUBBED AR_CLASS(NSEventOSDTextSetup)
//---------------------- Constructors / Destructor ---------------------------
  AR_CONSTRUCTOR(NSEventOSDTextSetup)

//---------------------------- Public methods --------------------------------
public:
  //! Read/Write views parameters
  virtual void
  readOSDText(ArRef<OSDText> osd) const;

  virtual void
  writeOSDText(ArConstRef<OSDText> osd);

//-------------------------- Stream operations -------------------------------
  virtual void
  writeToStream(ArRef<AbstractOStream> output) const;

  virtual bool // success
  readFromStream(ArRef<AbstractIStream> input);

//------------------------- Forbiden operations ------------------------------
protected:
//----------------------------- Attributes -----------------------------------
  StlString _fontName;
  double _color[4];
  int _justification;
  double _padding[2];
};

//----------------------------------------------------------------------------
// CLASS       : NSEventOSDText
//----------------------------------------------------------------------------
/*! \class NSEventOSDText
  *
  */
class NSEventOSDText : public NSEvent
{
public:
  AR_UNSTUBBED AR_CLASS(NSEventOSDText)
//---------------------- Constructors / Destructor ---------------------------
  AR_CONSTRUCTOR(NSEventOSDText)

//---------------------------- Public methods --------------------------------
public:
  //! Read/Write views parameters
  virtual void
  readOSDText(ArRef<OSDText> osd) const;

  virtual void
  writeOSDText(ArConstRef<OSDText> osd);

//-------------------------- Stream operations -------------------------------
  virtual void
  writeToStream(ArRef<AbstractOStream> output) const;

  virtual bool // success
  readFromStream(ArRef<AbstractIStream> input);

//------------------------- Forbiden operations ------------------------------
protected:
//----------------------------- Attributes -----------------------------------
  StlString _text;
};

//----------------------------------------------------------------------------
// CLASS       : NSEventOSDSpriteSetup
//----------------------------------------------------------------------------
/*! \class NSEventOSDSpriteSetup
  *
  */
class NSEventOSDSpriteSetup : public NSEventOSDSetup
{
public:
  AR_UNSTUBBED AR_CLASS(NSEventOSDSpriteSetup)
//---------------------- Constructors / Destructor ---------------------------
  AR_CONSTRUCTOR(NSEventOSDSpriteSetup)

//---------------------------- Public methods --------------------------------
public:
  //! Read/Write views parameters
  virtual bool
  readOSDSprite(ArRef<OSDSprite> osd,ArRef<RemoteDepService> depService) const;

  virtual void
  writeOSDSprite(ArConstRef<OSDSprite> osd);

//-------------------------- Stream operations -------------------------------
  virtual void
  writeToStream(ArRef<AbstractOStream> output) const;

  virtual bool // success
  readFromStream(ArRef<AbstractIStream> input);

//------------------------- Forbiden operations ------------------------------
protected:
//----------------------------- Attributes -----------------------------------
  double _scale[2];
  double _color[3];
  double _ratio;
  RemoteRef _texture;
  unsigned _w,_h;
};

} // end of namespace AReVi
#endif // end of AREVINS_NSEVENTOSD_H
//---End-Of-File---------------------------------------------------------------
