// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/Events/nsEventParticleSystem.h
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVINS_NSEVENTPARTICLESYSTEM_H
#define AREVINS_NSEVENTPARTICLESYSTEM_H 1

//--- Imported classes -------------------------------------------------------
#include "AReViNS/Events/nsEventShapePart3D.h"
#include "AReViNS/remoteDepService.h"

namespace AReVi
{
  class ParticleSystem;
}

namespace AReVi {

//----------------------------------------------------------------------------
// CLASS       : NSEventParticleSystemUpdate
//----------------------------------------------------------------------------
/*! \class NSEventParticleSystemUpdate
  *
  */
class NSEventParticleSystemUpdate : public NSEvent
{
public:
  AR_UNSTUBBED AR_CLASS(NSEventParticleSystemUpdate)
//---------------------- Constructors / Destructor ---------------------------
  AR_CONSTRUCTOR(NSEventParticleSystemUpdate)

//---------------------------- Public methods --------------------------------
public:
//-------------------------- Stream operations -------------------------------
  virtual void
  writeToStream(ArRef<AbstractOStream> output) const;

  virtual bool // success
  readFromStream(ArRef<AbstractIStream> input);

  double dt;
};


//----------------------------------------------------------------------------
// CLASS       : NSEventParticleSystemRendering
//----------------------------------------------------------------------------
/*! \class NSEventParticleSystemRendering
  *
  */
class NSEventParticleSystemRendering : public NSEvent
{
public:
  AR_UNSTUBBED AR_CLASS(NSEventParticleSystemRendering)
//---------------------- Constructors / Destructor ---------------------------
  AR_CONSTRUCTOR(NSEventParticleSystemRendering)

//---------------------------- Public methods --------------------------------
public:
//! Read/Write ParticleSystem parameters
  virtual void
  readParticleSystem(ArRef<ParticleSystem> part,
                     ArRef<RemoteDepService> depService) const;

  virtual void
  writeParticleSystem(ArConstRef<ParticleSystem> part);

//-------------------------- Stream operations -------------------------------
  virtual void
  writeToStream(ArRef<AbstractOStream> output) const;

  virtual bool // success
  readFromStream(ArRef<AbstractIStream> input);

//------------------------- Forbiden operations ------------------------------
protected:
//----------------------------- Attributes -----------------------------------
  bool _lines;
  unsigned int _thickness;
  RemoteRef _refTexture;
  double _initAngle;
  double _initAngleDisp;
  double _angularSpeed;
  double _angularSpeedDisp;
  double _size[2];
  double _sizeDisp[2];
  double _color[4];
  bool _lighting;
};

//----------------------------------------------------------------------------
// CLASS       : NSEventParticleSystemEmission
//----------------------------------------------------------------------------
/*! \class NSEventParticleSystemEmission
  *
  */
class NSEventParticleSystemEmission : public NSEvent
{
public:
  AR_UNSTUBBED AR_CLASS(NSEventParticleSystemEmission)
//---------------------- Constructors / Destructor ---------------------------
  AR_CONSTRUCTOR(NSEventParticleSystemEmission)

//---------------------------- Public methods --------------------------------
public:
//! Read/Write ParticleSystem parameters
  virtual void
  readParticleSystem(ArRef<ParticleSystem> part) const;

  virtual void
  writeParticleSystem(ArConstRef<ParticleSystem> part);

//-------------------------- Stream operations -------------------------------
  virtual void
  writeToStream(ArRef<AbstractOStream> output) const;

  virtual bool // success
  readFromStream(ArRef<AbstractIStream> input);

//------------------------- Forbiden operations ------------------------------
protected:
//----------------------------- Attributes -----------------------------------
  double _width;
  double _height;
  double _widthAngle;
  double _heightAngle;
  bool _rounded;
  double _particleRate;
  double _emissionSpeed;
  double _emissionSpeedDisp;
};


//----------------------------------------------------------------------------
// CLASS       : NSEventParticleSystemEvolution
//----------------------------------------------------------------------------
/*! \class NSEventParticleSystemEvolution
  *
  */
class NSEventParticleSystemEvolution : public NSEvent
{
public:
  AR_UNSTUBBED AR_CLASS(NSEventParticleSystemEvolution)
//---------------------- Constructors / Destructor ---------------------------
  AR_CONSTRUCTOR(NSEventParticleSystemEvolution)

//---------------------------- Public methods --------------------------------
public:
//! Read/Write ParticleSystem parameters
  virtual void
  readParticleSystem(ArRef<ParticleSystem> part) const;

  virtual void
  writeParticleSystem(ArConstRef<ParticleSystem> part);

//-------------------------- Stream operations -------------------------------
  virtual void
  writeToStream(ArRef<AbstractOStream> output) const;

  virtual bool // success
  readFromStream(ArRef<AbstractIStream> input);

//------------------------- Forbiden operations ------------------------------
protected:
//----------------------------- Attributes -----------------------------------
  double _particleDuration;
  double _gravity[3];
};

//----------------------------------------------------------------------------
// CLASS       : NSEventParticleSystemAddCollider
//----------------------------------------------------------------------------
/*! \class NSEventParticleSystemAddCollider
  *
  */
class NSEventParticleSystemAddCollider : public NSEventArRef
{
public:
  AR_UNSTUBBED AR_CLASS(NSEventParticleSystemAddCollider)
//---------------------- Constructors / Destructor ---------------------------
  AR_CONSTRUCTOR(NSEventParticleSystemAddCollider)
};


//----------------------------------------------------------------------------
// CLASS       : NSEventParticleSystemRemoveCollider
//----------------------------------------------------------------------------
/*! \class NSEventParticleSystemRemoveCollider
  *
  */
class NSEventParticleSystemRemoveCollider : public NSEventRef
{
public:
  AR_UNSTUBBED AR_CLASS(NSEventParticleSystemRemoveCollider)
//---------------------- Constructors / Destructor ---------------------------
  AR_CONSTRUCTOR(NSEventParticleSystemRemoveCollider)
};


} // end of namespace AReVi
#endif // end of AREVINS_NSEVENTPARTICLESYSTEM_H
//---End-Of-File---------------------------------------------------------------
