// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/Events/nsEventRenderer3D.h
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVINS_NSEVENTRENDERER3D_H
#define AREVINS_NSEVENTRENDERER3D_H 1

//--- Imported classes -------------------------------------------------------
#include "AReViNS/Events/nsEvent.h"

namespace AReVi
{
  class Renderer3D;
}

//----------------------------------------------------------------------------
// CLASS       : NSEventRenderer3DViewParameters
//----------------------------------------------------------------------------
namespace AReVi {

/*! \class NSEventRenderer3DViewParameters
  *
  */
class NSEventRenderer3DViewParameters : public NSEvent
{
public:
  AR_UNSTUBBED AR_CLASS(NSEventRenderer3DViewParameters)
//---------------------- Constructors / Destructor ---------------------------
  AR_CONSTRUCTOR(NSEventRenderer3DViewParameters)

//---------------------------- Public methods --------------------------------
public:
  //! Read/Write views parameters
  virtual void
  readRendererViewsParameters(ArRef<Renderer3D> renderer) const;

  virtual void
  writeRendererViewsParameters(ArConstRef<Renderer3D> renderer);

//-------------------------- Stream operations -------------------------------
  virtual void
  writeToStream(ArRef<AbstractOStream> output) const;

  virtual bool // success
  readFromStream(ArRef<AbstractIStream> input);

//------------------------- Forbiden operations ------------------------------
protected:
//----------------------------- Attributes -----------------------------------
  double _fov;
  double _near;
  double _far;
  double _nearYOffset;
  double _nearZOffset;
  double _fixedRatio;
  unsigned int _stereoMode;
  double _leftColor[3];
  double _rightColor[3];
  unsigned int _stereoMaxWidth;
  unsigned int _stereoMaxHeight;
  double _stereoViewDistance;
  double _stereoEyeSeparation;
  bool _wireFrame;
  bool _headLight;
  bool _originVisible;
  bool _drawShapeBBox;
  bool _drawShapePartBBox;
  bool _stereo;
  bool _drawStereoGlance;
};


/*! \class NSEventRenderer3DAppearanceSetup
  *
  */
class NSEventRenderer3DAppearanceSetup : public NSEvent
{
public:
  AR_UNSTUBBED AR_CLASS(NSEventRenderer3DAppearanceSetup)
//---------------------- Constructors / Destructor ---------------------------
  AR_CONSTRUCTOR(NSEventRenderer3DAppearanceSetup)

//---------------------------- Public methods --------------------------------
public:
  //! Read/Write rendering setup
  virtual void
  readRendererAppearanceSetup(ArRef<Renderer3D> renderer) const;

  virtual void
  writeRendererAppearanceSetup(ArConstRef<Renderer3D> renderer);

//-------------------------- Stream operations -------------------------------
  virtual void
  writeToStream(ArRef<AbstractOStream> output) const;

  virtual bool // success
  readFromStream(ArRef<AbstractIStream> input);

//------------------------- Forbiden operations ------------------------------
protected:
//----------------------------- Attributes -----------------------------------
  double _backgroundColor[3];
  double _headLightColor[3];
  double _bboxColor[3];
};

/*! \class NSEventRenderer3DRenderingSetup
  *
  */
class NSEventRenderer3DRenderingSetup : public NSEvent
{
public:
  AR_UNSTUBBED AR_CLASS(NSEventRenderer3DRenderingSetup)
//---------------------- Constructors / Destructor ---------------------------
  AR_CONSTRUCTOR(NSEventRenderer3DRenderingSetup)

//---------------------------- Public methods --------------------------------
public:
  //! Get Set Scenes
  inline void
  setMainScene(const StlString &mainScene, const StlString &className);

  inline void
  setBackScene(const StlString &backScene, const StlString &className);

  inline void
  setFrontScene(const StlString &frontScene, const StlString &className);

  inline void
  getScenes(StlString &mainSceneOut,
            StlString &frontSceneOut,
            StlString &backSceneOut) const;

  inline const StlString &
  getMainSceneClass() const;

  inline const StlString &
  getBackSceneClass() const;

  inline const StlString &
  getFrontSceneClass() const;

//-------------------------- Stream operations -------------------------------
  virtual void
  writeToStream(ArRef<AbstractOStream> output) const;

  virtual bool // success
  readFromStream(ArRef<AbstractIStream> input);

//------------------------- Forbiden operations ------------------------------
protected:
//----------------------------- Attributes -----------------------------------
  StlString _mainScene,_mainClass;
  StlString _frontScene,_frontClass;
  StlString _backScene,_backClass;
};

// Inline implementation
inline const StlString &
NSEventRenderer3DRenderingSetup::getMainSceneClass() const
{
  return _mainClass;
}

inline const StlString &
NSEventRenderer3DRenderingSetup::getBackSceneClass() const
{
  return _backClass;
}

inline const StlString &
NSEventRenderer3DRenderingSetup::getFrontSceneClass() const
{
  return _frontClass;
}

inline void
NSEventRenderer3DRenderingSetup::getScenes(StlString &mainSceneOut,
                                StlString &frontSceneOut,
                                StlString &backSceneOut) const
{
  mainSceneOut=_mainScene;
  frontSceneOut=_frontScene;
  backSceneOut=_backScene;
}

inline void
NSEventRenderer3DRenderingSetup::setFrontScene(const StlString &scene,const StlString &className)
{
  _frontScene=scene;
  _frontClass=className;
}

inline void
NSEventRenderer3DRenderingSetup::setMainScene(const StlString &scene,const StlString &className)
{
  _mainScene=scene;
  _mainClass=className;
}

inline void
NSEventRenderer3DRenderingSetup::setBackScene(const StlString &scene,const StlString &className)
{
  _backScene=scene;
  _backClass=className;
}

//----------------------------------------------------------------------------
// CLASS       : NSEventRenderer3DAddOSD
//----------------------------------------------------------------------------
/*! \class NSEventRenderer3DAddOSD
  * OSD add in renderer3D
  */
class NSEventRenderer3DAddOSD : public NSEventArRef
{
public:
  AR_UNSTUBBED AR_CLASS(NSEventRenderer3DAddOSD)
//---------------------- Constructors / Destructor ---------------------------
  AR_CONSTRUCTOR(NSEventRenderer3DAddOSD)

//---------------------------- Public methods --------------------------------
public:
};

//----------------------------------------------------------------------------
// CLASS       : NSEventRenderer3DRemoveOSD
//----------------------------------------------------------------------------
/*! \class NSEventRenderer3DRemoveOSD
  * OSD removed in renderer3D
  */
class NSEventRenderer3DRemoveOSD : public NSEventRef
{
public:
  AR_UNSTUBBED AR_CLASS(NSEventRenderer3DRemoveOSD)
//---------------------- Constructors / Destructor ---------------------------
  AR_CONSTRUCTOR(NSEventRenderer3DRemoveOSD)

//---------------------------- Public methods --------------------------------
};


} // end of namespace AReVi
#endif // end of AREVINS_NSEVENTRENDERER3D_H
//---End-Of-File---------------------------------------------------------------
