// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/Events/nsEventScene3D.h
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVINS_NSEVENTSCENE3D_H
#define AREVINS_NSEVENTSCENE3D_H 1

//--- Imported classes -------------------------------------------------------
#include "AReViNS/Events/nsEventObjectRef.h"
#include "AReViNS/remoteDepService.h"

namespace AReVi
{
  class Scene3D;
  class Transform2D;
}

namespace AReVi {

//----------------------------------------------------------------------------
// CLASS       : NSEventScene3DSetup
//----------------------------------------------------------------------------
/*! \class NSEventScene3DSetup
  * Setup change in scene3D
  */
class NSEventScene3DSetup : public NSEvent
{
public:
  AR_UNSTUBBED AR_CLASS(NSEventScene3DSetup)
//---------------------- Constructors / Destructor ---------------------------
  AR_CONSTRUCTOR(NSEventScene3DSetup)

//---------------------------- Public methods --------------------------------
public:
  //! Read/Write sound parameters
  virtual bool
  readScene3D(ArRef<Scene3D> scene,
              ArRef<RemoteDepService> depService) const;

  virtual void
  writeScene3D(ArConstRef<Scene3D> scene);

//-------------------------- Stream operations -------------------------------
  virtual void
  writeToStream(ArRef<AbstractOStream> output) const;

  virtual bool // success
  readFromStream(ArRef<AbstractIStream> input);

//------------------------- Forbiden operations ------------------------------
protected:
//----------------------------- Attributes -----------------------------------
  double _fogColor[3];
  unsigned int _fogDegree;
  double _fogDensity;
  ArRef<Transform2D> _transform;
  RemoteRef  _backgroundTex;
};

//----------------------------------------------------------------------------
// CLASS       : NSEventScene3DAddObject
//----------------------------------------------------------------------------
/*! \class NSEventScene3DAddObject
  * Object3D add in scene3D
  */
class NSEventScene3DAddObject : public NSEventArRef
{
public:
  AR_UNSTUBBED AR_CLASS(NSEventScene3DAddObject)
//---------------------- Constructors / Destructor ---------------------------
  AR_CONSTRUCTOR(NSEventScene3DAddObject)

//---------------------------- Public methods --------------------------------
public:
};

//----------------------------------------------------------------------------
// CLASS       : NSEventScene3RemoveObject
//----------------------------------------------------------------------------
/*! \class NSEventScene3DRemoveObject
  * Object3D add removed in scene3D
  */
class NSEventScene3DRemoveObject : public NSEventRef
{
public:
  AR_UNSTUBBED AR_CLASS(NSEventScene3DRemoveObject)
//---------------------- Constructors / Destructor ---------------------------
  AR_CONSTRUCTOR(NSEventScene3DRemoveObject)

//---------------------------- Public methods --------------------------------
};

//----------------------------------------------------------------------------
// CLASS       : NSEventScene3DSelectObject
//----------------------------------------------------------------------------
/*! \class NSEventScene3DSelectObject
  * Object3D selected in scene3D
  */
class NSEventScene3DSelectObject : public NSEventArRef
{
public:
  AR_UNSTUBBED AR_CLASS(NSEventScene3DSelectObject)
//---------------------- Constructors / Destructor ---------------------------
  AR_CONSTRUCTOR(NSEventScene3DSelectObject)
};

//----------------------------------------------------------------------------
// CLASS       : NSEventScene3DUnselectObject
//----------------------------------------------------------------------------
/*! \class NSEventScene3DUnselectObject
  * Object3D unselected in scene3D
  */
class NSEventScene3DUnselectObject : public NSEventRef
{
public:
  AR_UNSTUBBED AR_CLASS(NSEventScene3DUnselectObject)
//---------------------- Constructors / Destructor ---------------------------
  AR_CONSTRUCTOR(NSEventScene3DUnselectObject)
};

//----------------------------------------------------------------------------
// CLASS       : NSEventScene3AddParticleSystem
//----------------------------------------------------------------------------
/*! \class NSEventScene3DAddParticleSystem
  * Object3D add in scene3D
  */
class NSEventScene3DAddParticleSystem : public NSEventArRef
{
public:
  AR_UNSTUBBED AR_CLASS(NSEventScene3DAddParticleSystem)
//---------------------- Constructors / Destructor ---------------------------
  AR_CONSTRUCTOR(NSEventScene3DAddParticleSystem)

//---------------------------- Public methods --------------------------------
public:
};

//----------------------------------------------------------------------------
// CLASS       : NSEventScene3RemoveParticleSystem
//----------------------------------------------------------------------------
/*! \class NSEventScene3DRemoveParticleSystem
  * ParticleSystem add removed in scene3D
  */
class NSEventScene3DRemoveParticleSystem : public NSEventRef
{
public:
  AR_UNSTUBBED AR_CLASS(NSEventScene3DRemoveParticleSystem)
//---------------------- Constructors / Destructor ---------------------------
  AR_CONSTRUCTOR(NSEventScene3DRemoveParticleSystem)

//---------------------------- Public methods --------------------------------
};

//----------------------------------------------------------------------------
// CLASS       : NSEventScene3AddSound
//----------------------------------------------------------------------------
/*! \class NSEventScene3DAddSound
  * Sound3D add in scene3D
  */
class NSEventScene3DAddSound : public NSEventArRef
{
public:
  AR_UNSTUBBED AR_CLASS(NSEventScene3DAddSound)
//---------------------- Constructors / Destructor ---------------------------
  AR_CONSTRUCTOR(NSEventScene3DAddSound)

//---------------------------- Public methods --------------------------------
public:
};

//----------------------------------------------------------------------------
// CLASS       : NSEventScene3RemoveSound
//----------------------------------------------------------------------------
/*! \class NSEventScene3DRemoveSound
  * Sound3D add removed from scene3D
  */
class NSEventScene3DRemoveSound : public NSEventRef
{
public:
  AR_UNSTUBBED AR_CLASS(NSEventScene3DRemoveSound)
//---------------------- Constructors / Destructor ---------------------------
  AR_CONSTRUCTOR(NSEventScene3DRemoveSound)

//---------------------------- Public methods --------------------------------
};

//----------------------------------------------------------------------------
// CLASS       : NSEventScene3AddLight
//----------------------------------------------------------------------------
/*! \class NSEventScene3DAddLight
  * Light3D add in scene3D
  */
class NSEventScene3DAddLight : public NSEventArRef
{
public:
  AR_UNSTUBBED AR_CLASS(NSEventScene3DAddLight)
//---------------------- Constructors / Destructor ---------------------------
  AR_CONSTRUCTOR(NSEventScene3DAddLight)

//---------------------------- Public methods --------------------------------
public:
};

//----------------------------------------------------------------------------
// CLASS       : NSEventScene3RemoveLight
//----------------------------------------------------------------------------
/*! \class NSEventScene3DRemoveLight
  * Light3D add removed from scene3D
  */
class NSEventScene3DRemoveLight : public NSEventRef
{
public:
  AR_UNSTUBBED AR_CLASS(NSEventScene3DRemoveLight)
//---------------------- Constructors / Destructor ---------------------------
  AR_CONSTRUCTOR(NSEventScene3DRemoveLight)

//---------------------------- Public methods --------------------------------
};

} // end of namespace AReVi
#endif // end of AREVINS_NSEVENTSCENE3D_H
//---End-Of-File---------------------------------------------------------------
