// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/Events/nsEventWindow3D.h
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVINS_NSEVENTWINDOW3D_H
#define AREVINS_NSEVENTWINDOW3D_H 1

//--- Imported classes -------------------------------------------------------
#include "AReViNS/Events/nsEvent.h"
#include "AReVi/Lib3D/util3D_priv.h"

//----------------------------------------------------------------------------
// CLASS       : NSEventWindow3DCursor
//----------------------------------------------------------------------------
namespace AReVi {


/*! \class NSEventObject3DMotion
  * Object3D on motion event
  */
class NSEventWindow3DGeometry : public NSEvent
{
public:
  AR_UNSTUBBED AR_CLASS(NSEventWindow3DGeometry)
//---------------------- Constructors / Destructor ---------------------------
  AR_CONSTRUCTOR(NSEventWindow3DGeometry)

//---------------------------- Public methods --------------------------------
public:
  //! Get Set Window Geometry
  virtual
  void
  setWindowGeometry(int x,
                    int y,
                    unsigned int width,
                    unsigned int height);

  virtual
  void
  getWindowGeometry(int & xOut,
                    int & yOut,
                    unsigned int & widthOut,
                    unsigned int & heightOut) const;

//-------------------------- Stream operations -------------------------------
  virtual void
  writeToStream(ArRef<AbstractOStream> output) const;

  virtual bool // success
  readFromStream(ArRef<AbstractIStream> input);

//------------------------- Forbiden operations ------------------------------
protected:
//----------------------------- Attributes -----------------------------------
  int _x,_y;
  unsigned _w,_h;
};


/*! \class NSEventWindow3DMouseMotion
  *
  */
class NSEventWindow3DMouseMotion : public NSEvent
{
public:
  AR_UNSTUBBED AR_CLASS(NSEventWindow3DMouseMotion)
//---------------------- Constructors / Destructor ---------------------------
  AR_CONSTRUCTOR(NSEventWindow3DMouseMotion)

//---------------------------- Public methods --------------------------------
public:
  virtual void
  setPosition(int x,int y);

  virtual void
  setModifiers(unsigned int modifiers);

  inline int
  x(void) const {return _x;}

  inline int
  y(void) const  {return _y;}

  inline unsigned int
  modifiers(void) const  {return _modifiers;}

//-------------------------- Stream operations -------------------------------
  virtual void
  writeToStream(ArRef<AbstractOStream> output) const;

  virtual bool // success
  readFromStream(ArRef<AbstractIStream> input);

//------------------------- Forbiden operations ------------------------------
protected:
//----------------------------- Attributes -----------------------------------
  int _x,_y;
  unsigned int _modifiers;
};

/*! \class NSEventWindow3DMouseButton
  *
  */
class NSEventWindow3DMouseButton : public NSEventWindow3DMouseMotion
{
public:
  AR_UNSTUBBED AR_CLASS(NSEventWindow3DMouseButton)
//---------------------- Constructors / Destructor ---------------------------
  AR_CONSTRUCTOR(NSEventWindow3DMouseButton)

//---------------------------- Public methods --------------------------------
public:

  virtual void
  setButton(int button,bool pressed);

  inline int
  button(void) const {return _button;}

  inline bool
  pressed(void) const  {return _pressed;}

//-------------------------- Stream operations -------------------------------
  virtual void
  writeToStream(ArRef<AbstractOStream> output) const;

  virtual bool // success
  readFromStream(ArRef<AbstractIStream> input);

//------------------------- Forbiden operations ------------------------------
protected:
//----------------------------- Attributes -----------------------------------
  int _button;
  bool _pressed;
};

/*! \class NSEventWindow3DMouseButton
  *
  */
class NSEventWindow3DKeyboard : public NSEventWindow3DMouseMotion
{
public:
  AR_UNSTUBBED AR_CLASS(NSEventWindow3DKeyboard)
//---------------------- Constructors / Destructor ---------------------------
  AR_CONSTRUCTOR(NSEventWindow3DKeyboard)

//---------------------------- Public methods --------------------------------
public:

  virtual void
  setKey(const StlString &key,bool pressed);

  inline const StlString &
  key(void) const {return _key;}

  inline bool
  pressed(void) const { return _pressed;}

//-------------------------- Stream operations -------------------------------
  virtual void
  writeToStream(ArRef<AbstractOStream> output) const;

  virtual bool // success
  readFromStream(ArRef<AbstractIStream> input);

//------------------------- Forbiden operations ------------------------------
protected:
//----------------------------- Attributes -----------------------------------
  StlString  _key;
  bool _pressed;
};

/*! \class NSEventWindow3DClose
  *
  */
class NSEventWindow3DClose : public NSEvent
{
public:
  AR_UNSTUBBED AR_CLASS(NSEventWindow3DClose)
//---------------------- Constructors / Destructor ---------------------------
  AR_CONSTRUCTOR(NSEventWindow3DClose)

//---------------------------- Public methods --------------------------------
public:
};


/*! \class NSEventWindow3DCursor
  *
  */
class NSEventWindow3DCursor : public NSEvent
{
public:
  AR_UNSTUBBED AR_CLASS(NSEventWindow3DCursor)
//---------------------- Constructors / Destructor ---------------------------
  AR_CONSTRUCTOR(NSEventWindow3DCursor)

//---------------------------- Public methods --------------------------------
public:
  virtual void
  setCursor(unsigned int cursorID);

  inline unsigned int
  getCursor(void) const  {return _cursorID;}

//-------------------------- Stream operations -------------------------------
  virtual void
  writeToStream(ArRef<AbstractOStream> output) const;

  virtual bool // success
  readFromStream(ArRef<AbstractIStream> input);

//------------------------- Forbiden operations ------------------------------
protected:
//----------------------------- Attributes -----------------------------------
  unsigned int _cursorID;
};
} // end of namespace AReVi
#endif // end of AREVINS_NSEVENTWINDOW3D_H
//---End-Of-File---------------------------------------------------------------
