// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/localDepService.h
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVINS_LOCALDEPSERVICE_H
#define AREVINS_LOCALDEPSERVICE_H 1

//--- Imported classes -------------------------------------------------------
#include "AReViNS/localService.h"

//--- Constants --------------------------------------------------------------
// Listener fail to associated with shared ArObject
// - Listener class not exist
// - Listener class have not default constructor
#define NS_ERROR_LISTENER_FAIL      (NS_ERROR_LOCALSERVICE_ID+1)

#define NS_ERROR_LOCALDEPSERVICE_ID (NS_ERROR_LOCALSERVICE_ID+10) // Derived class use this as base

//----------------------------------------------------------------------------
// CLASS       : LocalDepService
//----------------------------------------------------------------------------
namespace AReVi {
  class NSListener;

/*! \class LocalDepService
  * Class managing local service
  */
class LocalDepService : public LocalService
{
public:
  AR_UNSTUBBED AR_CLASS(LocalDepService)
//---------------------- Constructors / Destructor ---------------------------
  // Service will be local (isRemote=false)
  AR_CONSTRUCTOR(LocalDepService) // let system specify port number

  AR_CONSTRUCTOR_1(LocalDepService,
                  const int &, port)

//---------------------------- Public methods --------------------------------
public:
  //! send object to specified serviceID or all if serviceID=-1
  bool
  shareObject(ArRef<ArObject> sharedObject, int serviceID=-1);

  //! Register down casted shared object (and all subclass if true).
  //!   => client class is a parent of the server class
  virtual bool // success if clientClass is parent of serverClass
  registerDownCast(const StlString &serverClass,
                   const StlString &clientClass,
                   bool subclass);

  //! send file pointed by url to specified serviceID or all if serviceID=-1
  bool
  shareURL(const StlString &url, int serviceID=-1);

//----------------------------Event management -------------------------------

//------------------------  Controlers management ----------------------------
//! Register controlers (and all subclass boolean is true).
  virtual bool
  registerListener(const StlString &objectClass,
                   const StlString &ctrlClass,
                   bool subclass);

//------------------------- Forbiden operations ------------------------------
protected:
  //! CB : inform that a client need a reference
  virtual NSEvent::EventProcess
  _onRequestRemoteReference(ArConstRef<NSEvent> eventObj);

  //! CB : inform that a client need an URL
  virtual NSEvent::EventProcess
  _onRequestRemoteFile(ArConstRef<NSEvent> eventObj);

  //! CB : inform that a client need a listener for gived reference
  virtual NSEvent::EventProcess
  _onRequestRemoteReferenceListener(ArConstRef<NSEvent> eventObj);

  //! CB : inform that a shared object is destroyed
  virtual void
  _onSharedObjectDestroyed(const ArObject::DestroyEvent &event);

  //! CB On new remote DepService connect to localDepService
  virtual void
  _onNewService(const LocalService::InformNewRemoteServiceEvent &evt);

  //! CB When a remoteDepService deconnect!
  virtual void
  _onDestroyService(const NetworkService::ServiceDeconnectEvent &evt);

  virtual
  bool
  _attachListener(ArRef<ArObject> sharedObject);

  virtual bool
  _updateNetwork(ArRef<Activity> act, double dt);

  virtual bool
  _flushPendingEvents(void);

  //! CB : inform that a client have processed all his messages
  virtual NSEvent::EventProcess
  _onRemoteDepServiceReady(ArConstRef<NSEvent> eventObj);

private:
  //! Init localDepService instance (only called by constructor!)
  void
  _initLocalDepService(void);

//----------------------------- Attributes -----------------------------------
protected:
  // Link shared object class with controlers class
  StlMap<StlString, StlString> _listenersMap;

  // Map serverClass to clientClass (serverClass must derive clientClass!)
  StlMap<StlString, StlString> _downCastMap;

  // Map of shared objects => object controler
  StlMap<StlString, ArRef<NSListener> > _listeners;

  // List of in waiting clients
  StlVector<int> _waitingClients;

};
} // end of namespace AReVi
#endif // end of AREVINS_LOCALSERVICE_H
//---End-Of-File---------------------------------------------------------------
