// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/localService.h
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVINS_LOCALSERVICE_H
#define AREVINS_LOCALSERVICE_H 1

//--- Imported classes -------------------------------------------------------
#include "AReViNS/networkService.h"

//--- Constants --------------------------------------------------------------
#define NS_ERROR_LOCALSERVICE_ID NS_ERROR_NETWORKSERVICE_ID // Derived class use this as base
//----------------------------------------------------------------------------
// CLASS       : LocalService
//----------------------------------------------------------------------------
namespace AReVi {
/*! \class LocalService
  * Class managing local service
  */
class LocalService : public NetworkService
{
public:
  AR_UNSTUBBED AR_CLASS(LocalService)
//---------------------- Constructors / Destructor ---------------------------
  AR_CONSTRUCTOR(LocalService) // let system specify port number

  AR_CONSTRUCTOR_1(LocalService,
                  const int &, port)

//---------------------------- Public methods --------------------------------
public:
  //! Service allow connection on specified port number (-1=>stop listenning)
  virtual bool
  openSlot(int portNumber);

  //! Port number of listenner for new connections.
  virtual int
  getListenningPortNumber(void) const;

  //! Port number of service.
  virtual void
  getServicePortNumber(int serviceId, int &readPort, int &writePort) const;

  //! IP of service.
  virtual unsigned int
  getServiceIPAdress(int serviceId) const;

  //! Distant port number of service.
  virtual void
  getServiceDistantPortNumber(int serviceId, int &readPort, int &writePort) const;

  //! Service session name
  virtual StlString  // "" on error
  getServiceName(int serviceId) const;

  //! Service host name
  virtual StlString  // "" on error
  getServiceHostName(int serviceId) const;

//------------------------- Network Statistics ------------------------------
  //! Read data from this service
  virtual unsigned int
  getDataReadCount(int serviceId) const;

  //! Write data to this service
  virtual unsigned int
  getDataWriteCount(int serviceId) const;

//----------------------------Event management -------------------------------
  AR_CALLBACK(LocalService,InformNewRemoteService,_informNewRemoteServiceCB,
              int serviceID;)

//------------------------- Forbiden operations ------------------------------
protected:

  //! Surdefined polling cb
  virtual void
  _onPolling(const Scheduler::PollingEvent & evt);

  //! watch network (look for news messages).
  virtual bool
  _getNetworkConnections(void);

//----------------------------- Attributes -----------------------------------
protected:
  //! Listener for incoming connections
  ArRef<TCPListener> _listener;

  CallbackManager<LocalService,InformNewRemoteServiceEvent> _informNewRemoteServiceCB;
};
} // end of namespace AReVi
#endif // end of AREVINS_LOCALSERVICE_H
//---End-Of-File---------------------------------------------------------------
