// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/localViewer3D.h
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
// LI2/ENIB - AReVi- Contrib - Network Services
//----------------------------------------------------------------------------
// package     : NetworkScene3D
// file        : serverViewer3D.h => declaration of LocalViewer3D
// author(s)   : CAZEAUX Eric
// mail        : cazeaux@enib.fr
//----------------------------------------------------------------------------
// last update : 21/02/2004
//----------------------------------------------------------------------------
#ifndef AREVINS_LOCALVIEWER3D_H
#define AREVINS_LOCALVIEWER3D_H 1

//--- Imported classes -------------------------------------------------------
#include "AReVi/Lib3D/viewer3D.h"
#include "AReViNS/localDepService.h"

//--- Constants --------------------------------------------------------------
// Service that intent to connect is already connected.
#define NS_ERROR_SERVICE_ALREADYCONNECTED      (NS_ERROR_LOCALSERVICE_ID+1)

// CB performer that is given to LocalViewer3D is no more valid
#define NS_ERROR_SETUPREMOTEVIEWER3D           (NS_ERROR_LOCALSERVICE_ID+2)

// Derived class use this as base
#define NS_ERROR_LOCALVIEWER3D_ID             (NS_ERROR_LOCALSERVICE_ID+10)

//----------------------------------------------------------------------------
// CLASS       : LocalViewer3D
//----------------------------------------------------------------------------

namespace AReVi {
class Listener3D;
class NSListenerRenderer3D;

/*! \class NetworkScene3D
  * Class managing local(server) Viewer3D
  */
class LocalViewer3D : public LocalService
{
public:
  typedef ArRef<Viewer3D>
          (ArObject::*ViewerSetupMethod)(unsigned int serviceID);

public:
  AR_UNSTUBBED AR_CLASS(LocalViewer3D)

  AR_CONSTRUCTOR_2(LocalViewer3D,
                  int              ,port,
                  ArRef<LocalDepService> ,depService)

//---------------------------- Public methods --------------------------------
public:
  ArRef<LocalDepService>
  accessDepService(void);

  ArRef<Viewer3D>
  accessViewer(int serviceID);

  ArRef<Listener3D>
  accessListener(int serviceID);

  bool
  setListener(int serviceID, ArRef<Listener3D> listener);

  //! Register and unregister events
  //! Method must return a new valid viewer else connection will be closed
  virtual bool
  setViewerSetup(ArRef<ArObject> performer,
                 ViewerSetupMethod  method);

//------------------------- Forbiden operations ------------------------------
protected:
  void
  _initLocalViewer3D(void);

  virtual ArRef<Viewer3D>
  _onViewerSetup(unsigned int serviceID);

  virtual bool
  _updateNetwork(ArRef<Activity> act, double dt);

  virtual bool
  _findServiceByViewer(const StlString &viewerAlias, int &serviceIdOut);

//--------------------- Local scene3D ----------------------------------------
  virtual void
  _onNewRemoteViewer3D(const LocalService::InformNewRemoteServiceEvent &);

  virtual void
  _onViewer3DDeconnect(const NetworkService::ServiceDeconnectEvent &);

  //! Viewer destroyed => transmit to client
  virtual void
  _onDestroyCB(const ArObject::DestroyEvent &evt);

//----------------- Messages from client -------------------------------------
  virtual NSEvent::EventProcess
  _onGeometryEvent(ArConstRef<NSEvent> event);

  // On Keyboards events
  NSEvent::EventProcess
  _onKeyboardEvent(ArConstRef<NSEvent> eventObj);

  // On MOUSE events
  NSEvent::EventProcess
  _onMouseMotionEvent(ArConstRef<NSEvent> eventObj);

  // On MOUSE Buttons events
  NSEvent::EventProcess
  _onMouseButtonEvent(ArConstRef<NSEvent> eventObj);

  // VIEWER CLOSE
  NSEvent::EventProcess
  _onCloseEvent(ArConstRef<NSEvent> eventObj);

//----------------------------- Attributes -----------------------------------
protected:
  class ClientInfos
  {
    public :
      ArRef<Viewer3D> _viewer;
      ArRef<Listener3D> _soundListener;
      ArRef<NSListenerRenderer3D> _viewerListener;
  };

  StlMap<int, ClientInfos *> _infos;

  ArRef<LocalDepService> _depService;

  struct ViewerSetupPerformer
  {
    ArPtr<ArObject>       performer;
    ViewerSetupMethod     method;
    bool                  autoref;
  };

  ViewerSetupPerformer _viewerSetup;
};

} // end of namespace AReVi
#endif // end of AREVINS_LOCALVIEWER3D_H
//---End-Of-File---------------------------------------------------------------
