// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/remoteService.h
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef AREVINS_REMOTESERVICE_H
#define AREVINS_REMOTESERVICE_H 1

//--- Imported classes -------------------------------------------------------
#include "AReViNS/networkService.h"

//--- Constants --------------------------------------------------------------
#define NS_ERROR_REMOTESERVICE_ID NS_ERROR_NETWORKSERVICE_ID // Derived class use this as base

//----------------------------------------------------------------------------
// CLASS       : RemoteService
//----------------------------------------------------------------------------
namespace AReVi {
/*! \class RemoteService
  * Class managing remote service
  */
class RemoteService : public NetworkService
{
public:
  AR_UNSTUBBED AR_CLASS(RemoteService)
//---------------------- Constructors / Destructor ---------------------------
  AR_CONSTRUCTOR(RemoteService) // NO CONNECTION

//---------------------------- Public methods --------------------------------
public:
  //! Connection method (only if deconnected)
  virtual bool
  connect(unsigned int distantIPAddress,
          int           distantPort);

  //! fill port number of CONNECTION (-1 if not connected)
  virtual void
  getPortNumber(int &readPort, int &writePort) const;

  // Server session name
  virtual StlString  // "" on error
  getServerName(void) const;

  // Server host name
  virtual StlString  // "" on error
  getServerHostName(void) const;

//------------------------- Network Statistics ------------------------------
  //! Read data from server
  virtual unsigned int
  getDataReadCount(void) const;

  //! Write data to server
  virtual unsigned int
  getDataWriteCount(void) const;

  //! Ping server
  virtual bool
  ping(void);

//----------------------------Event management -------------------------------
  AR_CALLBACK(RemoteService,Ping,_pingCB,
              unsigned int dt;) // in millisecond

protected:

  //! CB : reception of a ping Event
  virtual NSEvent::EventProcess
  _onPingEvent(ArConstRef<NSEvent> eventObj);

  CallbackManager<RemoteService,PingEvent> _pingCB;
};

} // end of namespace AReVi
#endif // end of AREVINS_REMOTESERVICE_H
//---End-Of-File---------------------------------------------------------------
