// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/remoteViewer3D.h
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#ifndef NETWORKSERVICE_REMOTEVIEWER3D_H
#define NETWORKSERVICE_REMOTEVIEWER3D_H 1

//--- Imported classes -------------------------------------------------------
#include "AReVi/Lib3D/viewer3D.h"
#include "AReViNS/remoteDepService.h"

//--- Constants --------------------------------------------------------------
// Derived class use this as base
#define NS_ERROR_REMOTEVIEWER3D_ID      (NS_ERROR_REMOTESERVICE_ID+10)

//----------------------------------------------------------------------------
// CLASS       : RemoteViewer3D
//----------------------------------------------------------------------------
namespace AReVi {
class NSInteractorRenderer3D;

/*! \class RemoteViewer3D
  * Class managing local(server) Viewer3D
  */
class RemoteViewer3D : public RemoteService
{
public:
  AR_UNSTUBBED AR_CLASS(RemoteViewer3D)

  AR_CONSTRUCTOR_1(RemoteViewer3D,
                   ArRef<RemoteDepService> ,depService)

//---------------------------- Public methods --------------------------------
public:
  ArRef<Viewer3D>
  accessViewer(void);

  ArRef<RemoteDepService>
  accessDepService(void);

  //! Connection method (only if deconnected)
  virtual bool // success
  connect(unsigned int distantIPAddress,
          int           distantPort);

//------------------------- Forbiden operations ------------------------------
protected:
  RemoteViewer3D(ArCW &arCw,
                 ArRef<RemoteDepService> depService,
                 ArRef<Viewer3D> viewer);


  virtual bool
  _updateNetwork(ArRef<Activity> act, double dt);

//--- NSEvents
  virtual NSEvent::EventProcess
  _onViewerInteractorEvent(ArConstRef<NSEvent> event);

  virtual NSEvent::EventProcess
  _onCloseEvent(ArConstRef<NSEvent> event);

  virtual NSEvent::EventProcess
  _onListenerEvent(ArConstRef<NSEvent> event);

//--- CB
// Geometry (Resize/Move widget?)
  virtual void
  _onGeometryCB(const Viewer3D::GeometryEvent &event);

// On Keyboards events
  virtual void
  _onKeyboardCB(const Viewer3D::KeyboardEvent &event);

// On MOUSE events
  virtual void
  _onMouseMotionCB(const Viewer3D::MouseMotionEvent &event);

// On MOUSE Buttons events
  virtual void
  _onMouseButtonCB(const Viewer3D::MouseButtonEvent &event);

// VIEWER CLOSE
  virtual void
  _onCloseCB(const Viewer3D::CloseEvent &event);

// VIEWER DESTROY
  virtual void
  _onDestroyCB(const ArObject::DestroyEvent &event);

//----------------------------- Attributes -----------------------------------
protected:
  ArRef<Viewer3D> _viewer;

  ArRef<NSInteractorRenderer3D> _viewerInteractor;
  ArRef<RemoteDepService> _depService;

  // CB DATA
  StlList<ArRef<NSEvent> > _events;
  bool _geometryFlag;
  bool _closeFlag;
};

} // end of namespace AReVi
#endif // end of NETWORKSERVICE_REMOTEVIEWER3D_H
//---End-Of-File---------------------------------------------------------------
