## LI2/ENIB - AReVi - Tcl wrapper library
##----------------------------------------------------------------------------
## file        : lib/AReViApp.tcl
## author(s)   : F. Harrouet
##----------------------------------------------------------------------------
## Copyright (C) 2002-2006  LI2/ENIB
##   Laboratoire d'Ingenierie Informatique (LI2)
##   Ecole Nationale d'Ingenieurs de Brest (ENIB)
##   CS 73862 - 29238 BREST Cedex 3 - France
##   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
##
## This software package is free software; you can redistribute it and/or
## modify it under the terms of the GNU Lesser General Public License as
## published by the Free Software Foundation; either version 2.1 of the
## License, or (at your option) any later version.
## This software package is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
## General Public License (file LICENSE-LGPL) for more details.
##----------------------------------------------------------------------------

namespace eval AReViApp {}

proc ::AReViApp::load {libName {convertLibName true}} {
  if {[string is true $convertLibName]} {
    global tcl_platform
    if {![string eq -nocase "windows" $tcl_platform(platform)]} {
      set libName lib$libName
    }
    append libName [info sharedlibextension]
  }
  if {[catch {::load $libName AReViApp} errMsg]} {
    puts stderr "While loading $libName ..."
    puts stderr "  $errMsg"
    exit -1
  }
  set exitStatus 0
  foreach cmd {init update destroy invoke constant} {
    set cmd ::AReViApp::$cmd
    if {$cmd ne [info commands $cmd]} {
      puts stderr "Cannot find $cmd command"
      set exitStatus -1
    }
  }
  if {$exitStatus} {
    exit $exitStatus
  }
}

proc ::AReViApp::makeBrowser {{className ArObject}} {
  if {[info globals tk_version] eq ""} {
    package require Tk
    wm withdraw .
  }
  set id [format "%x" [clock clicks]]
  set browser ::AReViApp::Browser_$id
  uplevel #0 "array set $browser {
    name $browser
    ivk ::AReViApp::invoke
    close ::AReViApp::_browser_close_$id
    getMethods ::AReViApp::_browser_getMethods_$id
    getConstants ::AReViApp::_browser_getConstants_$id
    update ::AReViApp::_browser_update_$id
    change ::AReViApp::_browser_change_$id
    choose ::AReViApp::_browser_choose_$id
    win ._AReViBrowser_$id
    cls $className
    reUse 1
    exact 1
    tab M
  }"
  upvar #0 $browser br

  proc $br(close) {} [concat "upvar #0 $browser br ;" {
    ::destroy $br(win)
    rename $br(getMethods) ""
    rename $br(getConstants) ""
    rename $br(update) ""
    rename $br(change) ""
    rename $br(choose) ""
    rename $br(close) ""
    uplevel #0 "unset $br(name)"
  }]

  proc $br(getMethods) {cls mthList} [concat "upvar #0 $browser br ;" {
    upvar $mthList result
    set clsName [$br(ivk) $cls "getName()"]
    if {[llength $result]} {
      lappend result "~~~~~~~~ Inherited from $clsName ~~~~~~~~"
    }
    set nb [$br(ivk) $cls "getNbMethods()"]
    for {set i 0} {$i<$nb} {incr i} {
      set mth [$br(ivk) $cls "getMethod(unsigned int)" $i]
      set construct [$br(ivk) $mth "isConstructor()"]
      if {!$construct||($br(cls) eq $clsName)} {
        if {$construct} {
          set str $clsName
        } elseif {[$br(ivk) $mth "isStaticMethod()"]} {
          set str $clsName\::[$br(ivk) $mth "getName()"]
        } else {
          set str [$br(ivk) $mth "getName()"]
        }
        set str $str\([$br(ivk) $mth "getArgSignature()"])
        if {[lsearch $result $str]<0} {
          lappend result $str
        }
      }
    }
    if {!$br(exact)} {
      set parent [$br(ivk) $cls "getParent()"]
      if {$parent ne ""} {
        $br(getMethods) $parent result
      }
    }
  }]

  proc $br(getConstants) {cls cstList} [concat "upvar #0 $browser br ;" {
    upvar $cstList result
    set clsName [$br(ivk) $cls "getName()"]
    set nb [$br(ivk) $cls "getNbConstants()"]
    for {set i 0} {$i<$nb} {incr i} {
      set cst [$br(ivk) $cls "getConstant(unsigned int)" $i]
      set str $clsName\::[$br(ivk) $cst "getName()"]
      lappend result $str
    }
    if {!$br(exact)} {
      set parent [$br(ivk) $cls "getParent()"]
      if {$parent ne ""} {
        $br(getConstants) $parent result
      }
    }
  }]

  proc $br(update) {} [concat "upvar #0 $browser br ;" {
    if {$br(cls) eq ""} {
      uplevel #0 "set $br(name)\(cls) ArObject"
    }
    wm title $br(win) "AReVi Browser: $br(cls)"
    $br(win).mid.lst delete 0 end
    set cls [$br(ivk) -s "ArClass::find(const StlString &)" $br(cls)]
    if {$cls eq ""} {
      $br(win).top.clsLbl configure -text "UNKNOWN Class:"
    } else {
      $br(ivk) $cls "getAncestors(StlVector<ArConstRef<ArClass> > &)" cList
      $br(win).mbar.up delete 0 end
      set cnt 0
      foreach c $cList {
        set clsName [$br(ivk) $c "getName()"]
        $br(win).mbar.up add command -label $clsName \
                         -columnbreak [expr [incr cnt]%20 ? 0 : 1] \
                         -command "$br(change) $clsName"
      }
      $br(ivk) $cls "getChildren(StlVector<ArConstRef<ArClass> > &)" cList
      $br(win).mbar.down delete 0 end
      set cnt 0
      foreach c $cList {
        set clsName [$br(ivk) $c "getName()"]
        $br(win).mbar.down add command -label $clsName \
                           -columnbreak [expr [incr cnt]%20 ? 0 : 1] \
                           -command "$br(change) $clsName"
      }
      $br(win).top.clsLbl configure -text "Class:"
      switch $br(tab) {
        M {
          $br(win).mid configure -text Methods
          set mthList {}
          $br(getMethods) $cls mthList
          foreach mth $mthList {
            $br(win).mid.lst insert end $mth
          }
        }
        C {
          $br(win).mid configure -text Constants
          set cstList {}
          $br(getConstants) $cls cstList
          foreach cst $cstList {
            $br(win).mid.lst insert end $cst
          }
        }
        I {
          $br(win).mid configure -text Instances
          set instList {}
          $br(ivk) $cls "getAliases(StlVector<StlString> &,bool)" \
                        instList [expr !$br(exact)]
          foreach inst $instList {
            $br(win).mid.lst insert end "$inst"
          }
        }
      }
    }
  }]

  proc $br(change) {cls} [concat "upvar #0 $browser br ;" {
    if {$br(reUse)} {
      uplevel #0 "set $br(name)\(cls) $cls"
      catch $br(update)
    } else {
      ::AReViApp::makeBrowser $cls
    }
  }]

  proc $br(choose) {x y} [concat "upvar #0 $browser br ;" {
    if {$br(tab) ne "M"} return
    set s [$br(win).mid.lst curselection]
    if {$s eq ""} return
    set s [$br(win).mid.lst get $s]
    if {[string index $s 0] eq "~"} return
    if {![regexp {^((.+)::)?([^(]+)\(([^)]+)\)} $s m0 m1 m2 m3 m4]} return
    if {$m2 ne ""} {
      set cls [$br(ivk) -s "ArClass::find(const StlString &)" $m2]
      if {$cls eq ""} return
      set mth [$br(ivk) $cls \
              "findStaticMethodWithSignature(const StlString &, \
                                             const StlString &)" $m3 $m4]
    } elseif {$m3 eq $br(cls)} {
      set cls [$br(ivk) -s "ArClass::find(const StlString &)" $m3]
      if {$cls eq ""} return
      set mth [$br(ivk) $cls \
              "findConstructorWithSignature(const StlString &)" $m4]
    } else {
      set cls [$br(ivk) -s "ArClass::find(const StlString &)" $br(cls)]
      if {$cls eq ""} return
      set mth [$br(ivk) $cls \
              "findMethodWithSignature(const StlString &, \
                                       const StlString &)" $m3 $m4]
    }
    if {$mth eq ""} return
    set mthName [$br(ivk) [$br(ivk) $mth "getDeclarationClass()"] "getName()"]
    append mthName ::[$br(ivk) $mth "getName()"]
    set inf [toplevel .mthInfo[format "%x" [clock clicks]]]
    wm title $inf "$mthName"
    set fnt "Courier -12"
    text $inf.msg -relief flat -wrap none -font $fnt
    $inf.msg tag configure italic -font "$fnt italic"
    $inf.msg tag configure bold -font "$fnt bold"
    $inf.msg tag configure boldItalic -font "$fnt bold italic"
    set nbLines 1
    if {[$br(ivk) $mth "isStaticMethod()"]} {
      $inf.msg insert end "static\n" bold
      incr nbLines
    }
    set r [$br(ivk) $mth "getResultType()"]
    if {$r eq ""} {
      $inf.msg insert end "void\n"
    } else {
      $inf.msg insert end "[$br(ivk) $r getCppType()]\n"
    }
    incr nbLines
    $inf.msg insert end $mthName bold
    $inf.msg insert end "("
    set indent [string length $mthName]
    incr indent
    set nbArgs [$br(ivk) $mth "getNbArgs()"]
    if {$nbArgs} {
      for {set i 0} {$i<$nbArgs} {incr i} {
        if {$i} {
          $inf.msg insert end ",\n[string repeat { } $indent ]"
          incr nbLines
        }
        set r [$br(ivk) $mth "getArgType(unsigned int)" $i]
        $inf.msg insert end "[$br(ivk) $r getCppType()] "
        $inf.msg insert end \
                 [$br(ivk) $mth "getArgName(unsigned int)" $i] boldItalic
      }
    } else {
      $inf.msg insert end "void"
    }
    if {[$br(ivk) $mth "isConstMethod()"]} {
      $inf.msg insert end ") const;"
    } else {
      $inf.msg insert end ");"
    }
    incr nbLines
    set maxWidth 0
    for {set i 1} {$i<=$nbLines} {incr i} {
      set w [string length [$inf.msg get $i.0 $i.end]]
      if {$w>$maxWidth} {
        set maxWidth $w
      }
    }
    $inf.msg configure -state disable -height $nbLines -width $maxWidth
    $inf configure -background [$inf.msg cget -background]
    pack $inf.msg -expand yes -fill both -padx 10 -pady {10 0}
    wm protocol $inf WM_DELETE_WINDOW "::destroy $inf"
    if {[incr x -12]<0} { set x 0 }
    if {[incr y -12]<0} { set y 0 }
    wm geometry $inf +$x+$y
    wm deiconify $inf
  }]

  toplevel $br(win)

  menu $br(win).mbar
  $br(win) configure -menu $br(win).mbar
  menu $br(win).mbar.view
  $br(win).mbar add cascade -label "View" -menu $br(win).mbar.view
  $br(win).mbar.view add radio -label Methods \
                               -variable $browser\(tab) -value M \
                               -command "catch $br(update)"
  $br(win).mbar.view add radio -label Constants \
                               -variable $browser\(tab) -value C \
                               -command "catch $br(update)"
  $br(win).mbar.view add radio -label Instances \
                               -variable $browser\(tab) -value I \
                               -command "catch $br(update)"
  $br(win).mbar.view add separator
  $br(win).mbar.view add command -label "Console" \
                                 -command "::AReViApp::makeConsole"
  menu $br(win).mbar.up
  $br(win).mbar add cascade -label "Base classes" -menu $br(win).mbar.up
  menu $br(win).mbar.down
  $br(win).mbar add cascade -label "Derived classes" -menu $br(win).mbar.down

  frame $br(win).top
  pack $br(win).top -side top -fill x -padx 4 -pady 4
  label $br(win).top.clsLbl -text "Class:"
  entry $br(win).top.clsTxt -textvariable $browser\(cls)
  bind $br(win).top.clsTxt <Return> "catch $br(update)"
  pack $br(win).top.clsLbl -side left
  pack $br(win).top.clsTxt -side left -fill both -expand true

  frame $br(win).bottom
  pack $br(win).bottom -side bottom -fill x -padx 4 -pady 4
  checkbutton $br(win).bottom.reuse -text Re-use \
              -variable $browser\(reUse) \
              -command "catch $br(update)"
  checkbutton $br(win).bottom.exact -text Exact \
              -variable $browser\(exact) \
              -command "catch $br(update)"
  button $br(win).bottom.close -text Close -command "$br(close)"
  pack $br(win).bottom.reuse -side left
  pack $br(win).bottom.exact -side left
  pack $br(win).bottom.close -side right

  labelframe $br(win).mid -text Methods -padx 4
  pack $br(win).mid -side top -expand true -fill both -padx 4
  listbox $br(win).mid.lst -yscroll "$br(win).mid.scr set" \
          -activestyle none -height 8
  bind $br(win).mid.lst <ButtonPress-1> "focus %W"
  bind $br(win).mid.lst <Double-1> "$br(choose) %X %Y"
  bind $br(win).mid.lst <Return> "$br(choose) %X %Y"
  scrollbar $br(win).mid.scr -orient vertical \
            -command "$br(win).mid.lst yview"
  pack $br(win).mid.scr -side right -fill y -pady {0 4}
  pack $br(win).mid.lst -fill both -expand true -pady {0 4}

  catch $br(update)

  wm protocol $br(win) WM_DELETE_WINDOW "$br(close)"
  wm geometry $br(win) 400x300
  wm deiconify $br(win)
}

proc ::AReViApp::makeConsole {} {
  if {[info globals tk_version] eq ""} {
    package require Tk
    wm withdraw .
  }
  set id [format "%x" [clock clicks]]
  set console ::AReViApp::Console_$id
  uplevel #0 "array set $console {
    name $console
    ivk ::AReViApp::invoke
    new_puts ::AReViApp::_console_new_puts_$id
    new_exit ::AReViApp::_console_new_exit_$id
    valid ::AReViApp::_console_valid_$id
    cancel ::AReViApp::_console_cancel_$id
    recall ::AReViApp::_console_recall_$id
    getMethods ::AReViApp::_console_getMethods_$id
    getConstants ::AReViApp::_console_getConstants_$id
    completion ::AReViApp::_console_completion_$id
    key ::AReViApp::_console_key_$id
    copy ::AReViApp::_console_copy_$id
    paste ::AReViApp::_console_paste_$id
    close ::AReViApp::_console_close_$id
    saved_puts ::AReViApp::_console_saved_puts_$id
    saved_exit ::AReViApp::_console_saved_exit_$id
    win ._AReViConsole_$id
    prompt {tcl> }
    usePrompt 1
    history [list {}]
    curLine 0
    cmd {}
    escTime [clock clicks -milliseconds]
  }"
  upvar #0 $console cs

  proc $cs(close) {} [concat "upvar #0 $console cs ;" {
    ::destroy $cs(win)
    rename $cs(new_puts) ""
    rename $cs(new_exit) ""
    rename $cs(valid) ""
    rename $cs(cancel) ""
    rename $cs(recall) ""
    rename $cs(getMethods) ""
    rename $cs(getConstants) ""
    rename $cs(completion) ""
    rename $cs(key) ""
    rename $cs(copy) ""
    rename $cs(paste) ""
    rename $cs(close) ""
    uplevel #0 "unset $cs(name)"
  }]

  proc $cs(new_puts) {args} [concat "upvar #0 $console cs ;" {
    set nl 1
    set channel stdout
    switch [llength $args] {
      1 {
        set txt [lindex $args 0]
      }
      2 {
        if {[lindex $args 0] eq "-nonewline"} {
          set nl 0
        } else {
          set channel [lindex $args 0]
        }
        set txt [lindex $args 1]
      }
      3 {
        if {[lindex $args 0] eq "-nonewline"} {
          set nl 0
          set channel [lindex $args 1]
          set txt [lindex $args 2]
        } else {
          return [eval $cs(saved_puts) $args]
        }
      }
      default {
        return [eval $cs(saved_puts) $args]
      }
    }
    switch $channel {
      stdout {
        if {$nl} {append txt "\n"}
        $cs(win).txt insert insert $txt stdoutTag
      }
      stderr {
        if {$nl} {append txt "\n"}
        $cs(win).txt insert insert $txt stderrTag
      }
      default {
        return [eval $cs(saved_puts) $args]
      }
    }
  }]

  proc $cs(new_exit) {{n 0}} [concat "upvar #0 $console cs ;" {
    error $cs(new_exit)
  }]

  proc $cs(valid) {} [concat "upvar #0 $console cs ;" {
    set goOn 1
    set line [$cs(win).txt get beginMark end-1c]
    set cs(history) [lreplace $cs(history) end end $line]
    if {$line ne ""} {
      lappend cs(history) ""
    }
    set cs(curLine) [expr [llength $cs(history)] - 1]
    append cs(cmd) "$line\n"
    $cs(win).txt mark set insert end
    $cs(win).txt insert insert "\n"
    if {[info complete $cs(cmd)]} {
      rename ::puts $cs(saved_puts)
      rename ::exit $cs(saved_exit)
      rename $cs(new_puts) ::puts
      rename $cs(new_exit) ::exit
      if {[catch {uplevel #0 $cs(cmd)} result]} {
        if {$result eq $cs(new_exit)} {
          set goOn 0
        }
        $cs(win).txt insert insert $result errorTag
      } else {
        $cs(win).txt insert insert $result resultTag
      }
      rename ::puts $cs(new_puts)
      rename ::exit $cs(new_exit)
      rename $cs(saved_puts) ::puts
      rename $cs(saved_exit) ::exit
      set cs(usePrompt) 1
      set cs(cmd) {}
    } else {
      set cs(usePrompt) 0
    }
    if {[$cs(win).txt compare insert != "insert linestart"]} {
      $cs(win).txt insert insert "\n"
    }
    if {$cs(usePrompt)} {
      $cs(win).txt insert insert $cs(prompt) promptTag
    }
    $cs(win).txt mark set beginMark insert
    return $goOn
  }]

  proc $cs(cancel) {} [concat "upvar #0 $console cs ;" {
    set cs(cmd) {}
    lreplace $cs(history) end end ""
    set cs(curLine) [expr [llength $cs(history)] - 1]
    $cs(win).txt mark set insert end
    $cs(win).txt insert insert "\n"
    $cs(win).txt insert insert $cs(prompt) promptTag
    $cs(win).txt mark set beginMark insert
    $cs(win).txt mark set savedMark insert
    $cs(win).txt see insert
    return -code break
  }]

  proc $cs(recall) {offset} [concat "upvar #0 $console cs ;" {
    set old $cs(curLine)
    set last [expr [llength $cs(history)] - 1]
    incr cs(curLine) $offset
    if {$cs(curLine)<0} {
      set cs(curLine) 0
    } elseif {$cs(curLine)>$last} {
      set cs(curLine) $last
    }
    if {$cs(curLine)!=$old} {
      if {$old==$last} {
        set cs(history) [lreplace $cs(history) $last $last \
                         [$cs(win).txt get beginMark end-1c]]
      }
      $cs(win).txt delete "insert linestart" end
      $cs(win).txt mark set insert end
      $cs(win).txt insert insert "\n"
      if {$cs(usePrompt)} {
        $cs(win).txt insert insert $cs(prompt) promptTag
      }
      $cs(win).txt mark set beginMark insert
      $cs(win).txt insert insert \
                   [lindex $cs(history) $cs(curLine)] stdinTag
    }
  }]

  proc $cs(getMethods) {cls static mthList} [concat "upvar #0 $console cs ;" {
    upvar $mthList result
    set nb [$cs(ivk) $cls "getNbMethods()"]
    for {set i 0} {$i<$nb} {incr i} {
      set mth [$cs(ivk) $cls "getMethod(unsigned int)" $i]
      if {([$cs(ivk) $mth "isStaticMethod()"]==$static)&&
           ![$cs(ivk) $mth "isConstructor()"]} {
        set str [$cs(ivk) $mth "getName()"]
        if {[lsearch $result $str]<0} {
          lappend result $str
        }
      }
    }
    set parent [$cs(ivk) $cls "getParent()"]
    if {$parent ne ""} {
      $cs(getMethods) $parent $static result
    }
  }]

  proc $cs(getConstants) {cls cstList} [concat "upvar #0 $console cs ;" {
    upvar $cstList result
    set nb [$cs(ivk) $cls "getNbConstants()"]
    for {set i 0} {$i<$nb} {incr i} {
      set cst [$cs(ivk) $cls "getConstant(unsigned int)" $i]
      lappend result [$cs(ivk) $cst "getName()"]
    }
    set parent [$cs(ivk) $cls "getParent()"]
    if {$parent ne ""} {
      $cs(getConstants) $parent result
    }
  }]


  proc $cs(completion) {} [concat "upvar #0 $console cs ;" {
    set line [$cs(win).txt get beginMark insert]
    set i [string length $line]
    while {$i>0} {
      set pos [expr $i - 1]
      set c [string range $line $pos $pos]
      if {[string is space $c]} break
      switch -- $c {
        "\{" break
        "\}" break
        "\[" break
        "\]" break
        "\"" break
        "\$" break
        ";" break
      }
      incr i -1
    }
    set p [string range $line $i end]
    while {$i>0} {
      set pos [expr $i - 1]
      set c [string range $line $pos $pos]
      if {![string is space $c]} break
      incr i -1
    }
    set j $i
    while {$j>0} {
      set pos [expr $j - 1]
      set c [string range $line $pos $pos]
      if {[string is space $c]} break
      switch -- $c {
        "\{" break
        "\}" break
        "\[" break
        "\]" break
        "\"" break
        "\$" break
        ";" break
      }
      incr j -1
    }
    set obj [string range $line $j [expr $i - 1]]
    set nmspc {}
    set maxLen 0
    set l {}
    if {![catch {$cs(ivk) $obj "getClass()"} cls]} {
      set cmpLen [string length $p]
      set methods {}
      catch {$cs(getMethods) $cls 0 methods}
      foreach i $methods {
        if {[string equal -length $cmpLen $i $p]} {
          set i "::$i"
          set len [string length $i]
          if {$len>$maxLen} {set maxLen $len}
          lappend l "$i "
        }
      }
    } else {
      regsub -line {^:+} $p {} p
      set i [string last "::" $p]
      if {$i>0} {
        set nmspc [string range $p 0 [expr $i - 1]]
        set p [string range $p [expr $i + 2] end]
      }
      foreach i [uplevel #0 info commands $nmspc\::$p*] {
        set len [string length $i]
        if {$len>$maxLen} {set maxLen $len}
        lappend l "$i "
      }
      foreach i [uplevel #0 info vars $nmspc\::$p*] {
        set len [string length $i]
        if {$len>$maxLen} {set maxLen $len}
        lappend l "$i "
      }
      if {[namespace exists ::$nmspc]} {
        foreach i [uplevel #0 namespace children ::$nmspc $p*] {
          append i "::"
          set len [string length $i]
          if {$len>$maxLen} {set maxLen $len}
          lappend l "$i"
        }
      }
      if {$nmspc eq ""} {
        if {![catch {$cs(ivk) -s "ArObject::CLASS()"} cls]} {
          set cmpLen [string length $p]
          set classes {}
          catch {$cs(ivk) $cls \
                 "getDescendants(StlVector<ArConstRef<ArClass> > &)" classes}
          lappend classes $cls
          foreach i $classes {
            if {[catch {$cs(ivk) $i "getName()"} i]} {
              break
            }
            if {[string equal -length $cmpLen $i $p]} {
              set i "::$i"
              set len [string length $i]
              if {$len>$maxLen} {set maxLen $len}
              lappend l "$i"
            }
          }
          set objs {}
          catch {$cs(ivk) $cls \
                 "getAliases(StlVector<StlString> &,bool)" objs true}
          foreach i $objs {
            if {[string equal -length $cmpLen $i $p]} {
              set i "::$i"
              set len [string length $i]
              if {$len>$maxLen} {set maxLen $len}
              lappend l "$i "
            }
          }
        }
      } else {
        if {![catch {$cs(ivk) -s "$nmspc\::CLASS()"} cls]} {
          set cmpLen [string length $p]
          set elems {}
          catch {$cs(getMethods) $cls 1 elems}
          catch {$cs(getConstants) $cls elems}
          foreach i $elems {
            if {[string equal -length $cmpLen $i $p]} {
              set i "::$nmspc\::$i"
              set len [string length $i]
              if {$len>$maxLen} {set maxLen $len}
              lappend l "$i "
            }
          }
        }
      }
    }
    if {[llength $l]} {
      set p $nmspc\::$p
      set o [string first $p [lindex $l 0]]
      incr o [string length $p]
      set str {}
      while {1} {
        set i [lindex $l 0]
        if {[string length $i]<=$o} break
        set goOn 1
        set c [string index $i $o]
        foreach i $l {
          if {([string length $i]<=$o)||([string index $i $o] ne $c)} {
            set goOn 0
            break
          }
        }
        if {$goOn} {incr o; append str $c} else break
      }
      if {$str eq ""} {
        $cs(win).txt mark set savedMark insert
        $cs(win).txt mark set insert "insert linestart"
        set line [$cs(win).txt get beginMark end]
        if {$cs(usePrompt)} {
          $cs(win).txt insert insert $cs(prompt) promptTag
        }
        $cs(win).txt insert insert $line stdinTag
        incr maxLen -1 ; # trailing " " instead of leading "::"
        set w [winfo width $cs(win).txt]
        incr w -[expr 2 * [$cs(win).txt cget -borderwidth]]
        set w [expr $w / [font measure [$cs(win).txt cget -font] "0"]]
        set nbCol [expr $w / $maxLen]
        if {!$nbCol} {incr nbCol}
        set c 0
        set l [lsort $l]
        foreach i $l {
          set i [string range $i 2 end]
          $cs(win).txt insert insert $i resultTag
          if {[incr c]%$nbCol} {
            $cs(win).txt insert insert [string repeat " " \
                                [expr $maxLen - [string length $i]]]
          } else {
            $cs(win).txt insert insert "\n"
          }
        }
        if {[$cs(win).txt compare insert != "insert linestart"]} {
          $cs(win).txt insert insert "\n"
        }
        if {$cs(usePrompt)} {
          $cs(win).txt mark set beginMark insert+[string length $cs(prompt)]c
        } else {
          $cs(win).txt mark set beginMark insert
        }
        $cs(win).txt mark set insert savedMark
      } else {
        $cs(win).txt insert insert "$str" stdinTag
      }
    }
  }]

  proc $cs(key) {k a} [concat "upvar #0 $console cs ;" {
    set see 1
    if {($a ne "")&&[string is print $a]} {
      $cs(win).txt insert insert $a stdinTag
    } else {
      switch -regexp $k {
        Return {
          if {![$cs(valid)]} {
            $cs(close)
            return -code break
          }
        }
        BackSpace {
          if {[$cs(win).txt compare insert > beginMark]} {
            $cs(win).txt delete insert-1c
          }
        }
        Delete {
          if {[$cs(win).txt compare insert < end]} {
            $cs(win).txt delete insert
          }
        }
        Left {
          if {[$cs(win).txt compare insert > beginMark]} {
            $cs(win).txt mark set insert insert-1c
          }
        }
        Right {
          if {[$cs(win).txt compare insert < end]} {
            $cs(win).txt mark set insert insert+1c
          }
        }
        Home {
          $cs(win).txt mark set insert beginMark
        }
        End {
          $cs(win).txt mark set insert end
        }
        Up {
          $cs(recall) -1
        }
        Down {
          $cs(recall) 1
        }
        Prior {
          $cs(recall) -10
        }
        Next {
          $cs(recall) 10
        }
        Tab {
          $cs(completion)
        }
        Escape {
          set t [clock clicks -milliseconds]
          if {($t - $cs(escTime))<500} {
            $cs(completion)
          }
          set cs(escTime) $t
        }
        default {
          set see 0
        }
      }
    }
    $cs(win).txt mark set savedMark insert
    if {$see} {
      $cs(win).txt see insert
    }
    return -code break
  }]

  proc $cs(copy) {} [concat "upvar #0 $console cs ;" {
    if {[$cs(win).txt tag nextrange sel 1.0 end] ne ""} {
      clipboard clear -displayof $cs(win).txt
      clipboard append -displayof $cs(win).txt \
                       [selection get -displayof $cs(win).txt]
    }
    $cs(win).txt mark set insert savedMark
    return -code break
  }]

  proc $cs(paste) {} [concat "upvar #0 $console cs ;" {
    if {![catch {selection get -displayof $cs(win).txt \
                              -selection PRIMARY} selTxt]||
        ![catch {clipboard get -displayof $cs(win).txt} selTxt]} {
      set lines [split $selTxt "\n"]
      set nb [llength $lines]
      for {set i 0} {$i<$nb} {incr i} {
        if {$i&&![$cs(valid)]} {
          $cs(close)
          return -code break
        }
        $cs(win).txt insert insert [lindex $lines $i] stdinTag
      }
    }
    $cs(win).txt mark set savedMark insert
    $cs(win).txt see insert
    return -code break
  }]

  toplevel $cs(win)

  frame $cs(win).bottom
  pack $cs(win).bottom -side bottom -fill x -padx 4 -pady 4
  button $cs(win).bottom.browser -text Browser \
                                 -command "::AReViApp::makeBrowser"
  button $cs(win).bottom.close -text Close -command "$cs(close)"
  pack $cs(win).bottom.browser -side left
  pack $cs(win).bottom.close -side right

  set fnt "Courier -12"
  text $cs(win).txt -yscrollcommand "$cs(win).scr set" \
                    -font $fnt -width 80 -height 25
  scrollbar $cs(win).scr -orient vertical -command "$cs(win).txt yview"
  pack $cs(win).scr -side left -fill y -pady {4 0} -padx {4 0}
  pack $cs(win).txt -side left -expand true -fill both -pady {4 0} -padx {0 4}

  $cs(win).txt tag configure promptTag -font "$fnt italic"
  $cs(win).txt tag configure stdinTag -font "$fnt bold"
  $cs(win).txt tag configure stdoutTag -font "$fnt"
  $cs(win).txt tag configure stderrTag -foreground #BB0000
  $cs(win).txt tag configure resultTag -font "$fnt italic"
  $cs(win).txt tag configure errorTag -font "$fnt italic" -foreground #BB0000
  $cs(win).txt insert insert $cs(prompt) promptTag
  $cs(win).txt mark set beginMark insert
  $cs(win).txt mark set savedMark insert
  $cs(win).txt mark gravity beginMark left
  focus $cs(win).txt

  bindtags $cs(win).txt [list $cs(win).txt]
  bind $cs(win).txt <Key> "$cs(key) %K %A"
  bind $cs(win).txt <Control-Key-c> "$cs(cancel)"
  bind $cs(win).txt <Control-Key-v> "$cs(paste)"
  bind $cs(win).txt <Control-Key-d> "$cs(close)"
  bind $cs(win).txt <ButtonRelease-2> "$cs(paste)"
  bind $cs(win).txt <Shift-Key-Prior> "%W yview scroll -1 pages"
  bind $cs(win).txt <Button-4> "%W yview scroll -5 units"
  bind $cs(win).txt <Shift-Button-4> "%W yview scroll -1 units"
  bind $cs(win).txt <Shift-Key-Next> "%W yview scroll 1 pages"
  bind $cs(win).txt <Button-5> "%W yview scroll 5 units"
  bind $cs(win).txt <Shift-Button-5> "%W yview scroll 1 units"
  bind $cs(win).txt <MouseWheel> \
                    "%W yview scroll \[expr -(%D/120)*5] units"
  bind $cs(win).txt <Shift-MouseWheel> \
                    "%W yview scroll \[expr -%D/120] units"
  foreach b {<Button-1> <B1-Motion> <Double-Button-1> <Triple-Button-1>} {
    bind $cs(win).txt $b "[bind Text $b] ; $cs(copy)"
  }

  wm title $cs(win) "AReVi Console"
  wm protocol $cs(win) WM_DELETE_WINDOW "$cs(close)"
  wm deiconify $cs(win)
}

##----------------------------------------------------------------------------
