// LI2/ENIB - AReVi-Contrib - 3D file loader/saver
//----------------------------------------------------------------------------
// file        : AReVi/Contrib/File3D/3ds.cpp
// author(s)   : C. Septseault
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/Shapes/shape3D.h"
#include "AReVi/Utils/fdStream.h"
#include "AReVi/Utils/memoryStream.h"
#include "AReVi/Utils/memoryBlock.h"
#include "AReVi/Lib3D/urlTexture.h"
#include "AReVi/Shapes/mesh3D.h"
#include "AReVi/arSystem.h"
#include "AReVi/Lib3D/transform3D.h"
#include "AReVi/Lib3D/transform2D.h"
#include "AReVi/Lib3D/material3D.h"
#include "AReVi/Lib3D/util3D_priv.h"

#include "AReVi/Contrib/File3D/3ds.h"

namespace AReVi {


void Loader3DS::setUrlDir(const StlString& dir)
{
	_urlDir=dir;
}
const StlString& Loader3DS::getMessage()
{
	return _message;
}
bool Loader3DS::checkMesh()
{
	if(_mesh && !_mesh->applyChanges(true))
	{
		_message+="Warning : Consistance problem !!!!!!!!!\n";
		if(!_mesh->checkVertexConsistency())
			_message+="vertex Consistance failed\n";
		if(!_mesh->checkNormalConsistency())
			_message+="normal Consistance failed\n";
		if(!_mesh->checkTextureConsistency())
			_message+="texture Consistance failed\n";
		if(!_mesh->checkColorConsistency())
			_message+="color Consistance failed\n";
		_warning=true;
		return false;
	}
	else return true;
}
bool Loader3DS::read3DSFile(ArRef<FDIStream> stream,ArRef<Shape3D> shapeOut)
{
	_mesh=Mesh3D::nullRef();
	_object=ShapePart3D::nullRef();
	_objects.clear();
	_transf.clear();
	_urlTextures.clear();
	_materials.clear();
	_doubleSided.clear();
	_color.clear();
	_lastHierValue=-1;
	_hierarchy.clear();
	_lastNumFaces.clear();
	_lastFaces.clear();
	_warning=false;
	_message="";	

	_shape=shapeOut;
	if (!read3DSChunk(stream,0))
	{
		_message="Unexpected end of file\n";
		_warning=true;
	}
	if(_mesh)
	{
			    if(_lastNumFaces.size())
			    {
				    for(size_t i=0;i<_lastNumFaces.size();i++)
				    {
					    _faces->push_back(_lastFaces[_lastNumFaces[i]]);
					    _mesh->accessTextureFaces().push_back(_lastFaces[_lastNumFaces[i]]);
				    }
			    }
			    else
			    {
				    _mesh->accessVertexFaces().insert(_mesh->accessVertexFaces().begin(),_lastFaces.begin(),_lastFaces.end());
				    _mesh->accessTextureFaces().insert(_mesh->accessTextureFaces().begin(),_lastFaces.begin(),_lastFaces.end());

			    }
		_mesh->computeDefaultNormals(true);
		checkMesh();
	}
	return (!_warning);
	//...
}
unsigned int Loader3DS::read3dsInt(ArRef<AbstractIStream> stream)
{
	unsigned int i;
	stream->readData(&i,sizeof(int));
#if BYTE_ORDER==BIG_ENDIAN
        ArSystem::invert32(&i); // 3DS files are little endian
#endif
	return i;
}
short Loader3DS::read3dsShort(ArRef<AbstractIStream> stream)
{
	short s;
	stream->readData(&s,sizeof(short));
#if BYTE_ORDER==BIG_ENDIAN
        ArSystem::invert16(&s); // 3DS files are little endian
#endif
	return s;
}
double Loader3DS::read3dsFloat(ArRef<AbstractIStream> stream)
{
	float f;
	stream->readData(&f,sizeof(float));
#if BYTE_ORDER==BIG_ENDIAN
        ArSystem::invert32(&f); // 3DS files are little endian
#endif
	return f;
}

unsigned int Loader3DS::readColor(ArRef<AbstractIStream> stream,double &r,double &g, double &b)
{
	unsigned short id=read3dsShort(stream);
	unsigned int size=read3dsInt(stream);
	char c;
	switch(id)
	{
		case COLOR_F:
		case LIN_COLOR_F:
			r=read3dsFloat(stream);
			g=read3dsFloat(stream);
			b=read3dsFloat(stream);
			break;
		case COLOR_24:
		case LIN_COLOR_24:
			stream->readChar(c);
			r=((unsigned char)c)/255.;
			stream->readChar(c);
			g=((unsigned char)c)/255.;
			stream->readChar(c);
			b=((unsigned char)c)/255.;
			break;
		default:
			{
			for(int i=size;i--;)
				stream->readChar(c);
			}
	}
	return size;
}
unsigned int Loader3DS::readPercentage(ArRef<AbstractIStream> stream,double &p)
{
	unsigned short id=read3dsShort(stream);
	unsigned int size=read3dsInt(stream);
	char c;
	short s;
	switch(id)
	{
		case INT_PERCENTAGE: //int percentage
			s=read3dsShort(stream);
			p=s/100.;
			break;
		case FLOAT_PERCENTAGE: //float percentage
			p=read3dsFloat(stream);
			break;
		default:
			{
			for(int i=size;i--;)
				stream->readChar(c);
			}
	}
	return size;

}
void error()
{
	return;
}
unsigned int Loader3DS::read3DSChunk(ArRef<FDIStream> stream,unsigned short chunkId)
{
//    short   chunkId;
    unsigned int chunkLength;

    unsigned int     index = 6;  // index into current chunk
    unsigned int     j;
    char name[120];
    unsigned short numFaces;
    unsigned short numV;
    unsigned short numP;
    unsigned short numT;
    unsigned short shortValue;

    if(!chunkId)
    	chunkId=read3dsShort(stream);
    if (stream->fail())
	    {error();return 0;}
    chunkLength=read3dsInt(stream);
    switch (chunkId)
    {
	    case M3DMAGIC:    // first chunk
		    do {
			    shortValue=read3dsShort(stream);
                            index+=2;
		   		if (stream->fail())
					{error();return 0;}
		    }while(shortValue!=0x3D3D && shortValue!=0xB000);
		    index+=read3DSChunk(stream,shortValue);
		    if (stream->fail())
			    {error();return 0;}
		    break;
	    case MDATA:    // editor data
		    break;

	    case NAMED_OBJECT:    // object description
		    //Save last Mesh
		    if(_mesh)
		    {
			    if(_lastNumFaces.size())
			    {
				    for(size_t i=0;i<_lastNumFaces.size();i++)
				    {
					    _faces->push_back(_lastFaces[_lastNumFaces[i]]);
					    _mesh->accessTextureFaces().push_back(_lastFaces[_lastNumFaces[i]]);
				    }
			    }
			    else
			    {
				    _mesh->accessVertexFaces().insert(_mesh->accessVertexFaces().begin(),_lastFaces.begin(),_lastFaces.end());
				    _mesh->accessTextureFaces().insert(_mesh->accessTextureFaces().begin(),_lastFaces.begin(),_lastFaces.end());

			    }
			    _mesh->computeDefaultNormals(true);
		    }
		    checkMesh();

		

		
		     // get past string description
		    j=0;
		    do {
			    stream->readChar(name[j]);
			    if (stream->fail())
				    {error();return 0;}
		    }while(name[j++]!=0);
		    index+=j;
		    // create new mesh;
		    _object=_mesh=Mesh3D::NEW();
		    _objects[name]=_object;
		    _object->setAlias(name);
		    _objectName=name;
		    _shape->addRootPart(_object);
		    _vertices=&_mesh->accessVertices();
		    _textures=&_mesh->accessTextures();
		    _faces=&_mesh->accessVertexFaces();
		    _lastFaces.clear();
		    _lastNumFaces.clear();
		    break;
	    case N_TRI_OBJECT:    // triangular polygon list
		    break;
	    case POINT_ARRAY:    // vertex list
		    {
			    double x,y,z;
			    numV=read3dsShort(stream);
			    if (stream->fail())
				    {error();return 0;}
			    index+=2;
			    for(j=0;j<numV;j++)
			    {
				    x=read3dsFloat(stream);
				    y=read3dsFloat(stream);
				    z=read3dsFloat(stream);
				    _vertices->push_back(Util3D::Dbl3(x,y,z));
				    index+=12;
			    }
		    }
		    break;
	    case FACE_ARRAY: // Faces points
		    {
			    unsigned short p1,p2,p3;
			    numP=read3dsShort(stream);
			    if (stream->fail())
				    {error();return 0;}
			    index+=2;
			    for(j=0;j<numP;j++)
			    {
				    p1=read3dsShort(stream);
				    p2=read3dsShort(stream);
				    p3=read3dsShort(stream);
				    read3dsShort(stream); //flags
				
				    _lastFaces.push_back(Util3D::Face(p1,p2,p3));
				    index+=8;
			    }
		    }

		    break;
	    case MSH_MAT_GROUP: //Face Material
		    {
			    if (_lastNumFaces.size())
			    {
				    if(_mesh)
				    {
					    for(size_t i=0;i<_lastNumFaces.size();i++)
					    {
						    _faces->push_back(_lastFaces[_lastNumFaces[i]]);
						    _mesh->accessTextureFaces().push_back(_lastFaces[_lastNumFaces[i]]);
					    }
					    _mesh->computeDefaultNormals(true);
				    }
				    checkMesh();
				    ArRef<Mesh3D> mesh2=Mesh3D::NEW();
				    _object->addSubPart(mesh2);
				    mesh2->accessVertices().insert(mesh2->accessVertices().begin(),_vertices->begin(),_vertices->end());
				
				    mesh2->accessTextures().insert(mesh2->accessTextures().begin(),_textures->begin(),_textures->end());

				    _mesh=mesh2;
				    _vertices=&_mesh->accessVertices();
				    _textures=&_mesh->accessTextures();
				    _faces=&_mesh->accessVertexFaces();
				    _lastNumFaces.clear();
			    }

			    j=0;
			    do {
				    stream->readChar(name[j]);
				    if (stream->fail())
					    {error();return 0;}
			    }while(name[j++]!=0);
			    index+=j;

			    if (!_mesh->getTextures().empty())
			    {
				    _mesh->setTexture(_urlTextures[name].first);
				    if (_noTiling[name].first)
					    _mesh->setTextureRepeat(false,false);
				    if (_urlTexturesTransform[name].first)
					    _mesh->writeTextureTransformation(_urlTexturesTransform[name].first);
				    if (_urlTextures[name].second)		
				    {
					    _mesh->setSecondTexture(_urlTextures[name].second);
					    if (_noTiling[name].second)
						    _mesh->setSecondTextureRepeat(false,false);
					    if (_urlTexturesTransform[name].second)
						    _mesh->writeSecondTextureTransformation(_urlTexturesTransform[name].second);
				    }
			    }
			    if (_materials.find(name)!=_materials.end())
				    _mesh->writeMaterial(_materials[name]);
			    _mesh->setDoubleSided(_doubleSided[name]);
//			    _mesh->setDoubleSided(true);
			    numFaces=read3dsShort(stream);
			    if (stream->fail())
				    {error();return 0;}
			    index+=2;
			    for(j=0;j<numFaces;j++)
			    {
				    shortValue=read3dsShort(stream);
				    if (stream->fail())
					    {error();return 0;}
				    index+=2;
				    _lastNumFaces.push_back(shortValue);
			    }
			
		    }
		    break;
		
	    case TEX_VERTS:	//Mapping coordinates (textures points)
		    {
			    double x,y;
			    numT=read3dsShort(stream);
			    if (stream->fail())
				    {error();return 0;}
			    index+=2;
			    for(int j=0;j<numT;j++)
			    {
				    x=read3dsFloat(stream);
				    y=read3dsFloat(stream);
				    if (stream->fail())
					    {error();return 0;}
				    _textures->push_back(Util3D::Dbl2(x,y));
				    index+=8;
			    }

		    }
		
		
		    break;
	    case MESH_MATRIX:	//Transformation
		    {
			ArRef<Transform3D> tr3D=Transform3D::NEW();
			_mesh->readTransformation(tr3D);
			
			Util3D::Transform& tr=tr3D->accessTransformation();
		
			tr.matrix[0]=read3dsFloat(stream);
			tr.matrix[1]=read3dsFloat(stream);
			tr.matrix[2]=read3dsFloat(stream);
			
			tr.matrix[4]=read3dsFloat(stream);
			tr.matrix[5]=read3dsFloat(stream);
			tr.matrix[6]=read3dsFloat(stream);
			
			tr.matrix[8]=read3dsFloat(stream);
			tr.matrix[9]=read3dsFloat(stream);
			tr.matrix[10]=read3dsFloat(stream);
			tr.matrix[12]=read3dsFloat(stream);
			tr.matrix[13]=read3dsFloat(stream);
			tr.matrix[14]=read3dsFloat(stream);
			index+=12*4;
			if (stream->fail())
				{error();return 0;}
			
		ArRef<Transform3D> tr3D_2=Transform3D::NEW();
			tr3D_2->identity();
			
			for(int ii=0;ii<16;ii++)
			_transf[_objectName]=tr3D;
		    }
		    break;
	    case MAT_ENTRY: //Materials
		    break;
	    case MAT_NAME: //Material Name
			    j=0;
			    do {
				    stream->readChar(name[j]);
				    if (stream->fail())
					    {error();return 0;}
			    }while(name[j++]!=0);
			    index+=j;
			    _lastMaterial=Material3D::NEW();
			    _lastMaterialName=name;
			    _materials[name]=_lastMaterial;		
		    break;
		
	    case MAT_AMBIENT: //Ambiant color
		    {
		    double r,g,b;
		    index+=readColor(stream,r,g,b);
		    if (stream->fail())
			    {error();return 0;}
		    }
		    break;
		
	    case MAT_DIFFUSE: //Diffuse color
		    {
		    double r,g,b;
		    index+=readColor(stream,r,g,b);
		    if (stream->fail())
			    {error();return 0;}
		    _lastMaterial->setDiffuseColor(r,g,b);
		    }
		    break;
	    case MAT_SPECULAR: //Specular color
		    {
		    double r,g,b;
		    index+=readColor(stream,r,g,b);
		    if (stream->fail())
			    {error();return 0;}
		    _lastMaterial->setSpecularColor(r,g,b);
		    }
		    break;
	    case MAT_SHININESS: //MAT_SHININESS 0xA040
		    {
			    double p;
			    index+=readPercentage(stream,p);
			    if (stream->fail())
				    {error();return 0;}
			    _lastMaterial->setShininess(p);
		    }
		    break;
	    case MAT_TRANSPARENCY: //MAT_TRANSPARENCY 0xA050
		    {
			    double p;
			    index+=readPercentage(stream,p);
			    if (stream->fail())
				    {error();return 0;}
			    _lastMaterial->setTransparency(p);
		    }
		    break;
	    case MAT_TWO_SIDE: //Two Sided
		    _doubleSided[_lastMaterialName]=true;
		    break;
	    case MAT_TEXMAP: //DIFFUSE MAT
		    _firstTexMap=true;
		    break;
	    case MAT_TEX2MAP: //Second MAP
		    _firstTexMap=false;
		    break;
	    case MAT_MAPNAME: // MAP_MATNAME
		    {
			    j=0;
			    do {
				    stream->readChar(name[j]);
				    if (stream->fail())
					    {error();return 0;}
			    }while(name[j++]!=0);
			    index+=j;
			
			    ArRef<URLTexture> texture=URLTexture::NEW(_urlDir+name,true,true);
			    if (texture->fail())
			    {
				    _message+="Cannot load texture "+texture->getURL()+" for material "+_lastMaterialName+"\n";
				    _warning=true;
			    }
			    if (_firstTexMap)
				    _urlTextures[_lastMaterialName].first=texture;
			    else
				    _urlTextures[_lastMaterialName].second=texture;
			}
		    break;
	    case MAT_MAP_TILING:
		    {
			unsigned short s;
			s=read3dsShort(stream);
			index+=2;
			if (s&0x10) //desactivate tiling
			{
				if (_firstTexMap)
					_noTiling[_lastMaterialName].first=true;
				else
					_noTiling[_lastMaterialName].second=true;
			}
			if (s&0x8) //negation
			{
			    if (_firstTexMap)
			    {
				    if(!_urlTexturesTransform[_lastMaterialName].first)
					    _urlTexturesTransform[_lastMaterialName].first=Transform2D::NEW();
				    _urlTexturesTransform[_lastMaterialName].first->invert();
			    }
			    else
			    {
				    if(!_urlTexturesTransform[_lastMaterialName].second)
					    _urlTexturesTransform[_lastMaterialName].second=Transform2D::NEW();
				    _urlTexturesTransform[_lastMaterialName].second->invert();
			    }
			}	
		    }
		    break;
	    case MAT_MAP_USCALE:
		    {
			    double f;
			    f=read3dsFloat(stream);
			    index+=4;
			    if (_firstTexMap)
			    {
				    if(!_urlTexturesTransform[_lastMaterialName].first)
					    _urlTexturesTransform[_lastMaterialName].first=Transform2D::NEW();
				    _urlTexturesTransform[_lastMaterialName].first->preScale(f,1.);
			    }
			    else
			    {
				    if(!_urlTexturesTransform[_lastMaterialName].second)
					    _urlTexturesTransform[_lastMaterialName].second=Transform2D::NEW();
				    _urlTexturesTransform[_lastMaterialName].second->preScale(f,1.);
			    }
		    }	
		    break;
	    case MAT_MAP_VSCALE:
		    {
			    double f;
			    f=read3dsFloat(stream);
			    index+=4;
			    if (_firstTexMap)
			    {
				    if(!_urlTexturesTransform[_lastMaterialName].first)
					    _urlTexturesTransform[_lastMaterialName].first=Transform2D::NEW();
				    _urlTexturesTransform[_lastMaterialName].first->preScale(1,f);
			    }
			    else
			    {
				    if(!_urlTexturesTransform[_lastMaterialName].second)
					    _urlTexturesTransform[_lastMaterialName].second=Transform2D::NEW();
				    _urlTexturesTransform[_lastMaterialName].second->preScale(1.,f);
			    }
		    }	
		    break;
	    case MAT_MAP_UOFFSET:
		    {
			    double f;
			    f=read3dsFloat(stream);
			    index+=4;
			    if (_firstTexMap)
			    {
				    if(!_urlTexturesTransform[_lastMaterialName].first)
					    _urlTexturesTransform[_lastMaterialName].first=Transform2D::NEW();
				    _urlTexturesTransform[_lastMaterialName].first->preTranslate(f,0.);
			    }
			    else
			    {
				    if(!_urlTexturesTransform[_lastMaterialName].second)
					    _urlTexturesTransform[_lastMaterialName].second=Transform2D::NEW();
				    _urlTexturesTransform[_lastMaterialName].second->preTranslate(f,0.);
			    }
		    }	
		    break;
	    case MAT_MAP_VOFFSET:
		    {
			    double f;
			    f=read3dsFloat(stream);
			    index+=4;
			    if (_firstTexMap)
			    {
				    if(!_urlTexturesTransform[_lastMaterialName].first)
					    _urlTexturesTransform[_lastMaterialName].first=Transform2D::NEW();
				    _urlTexturesTransform[_lastMaterialName].first->preTranslate(0.,f);
			    }
			    else
			    {
				    if(!_urlTexturesTransform[_lastMaterialName].second)
					    _urlTexturesTransform[_lastMaterialName].second=Transform2D::NEW();
				    _urlTexturesTransform[_lastMaterialName].second->preTranslate(0.,f);
			    }
		    }	
		    break;
	    case MAT_MAP_ANG:
		    {
			    double f;
			    f=read3dsFloat(stream);
			    index+=4;
			    if (_firstTexMap)
			    {
				    if(!_urlTexturesTransform[_lastMaterialName].first)
					    _urlTexturesTransform[_lastMaterialName].first=Transform2D::NEW();
				    _urlTexturesTransform[_lastMaterialName].first->preRotate(f);
			    }
			    else
			    {
				    if(!_urlTexturesTransform[_lastMaterialName].second)
					    _urlTexturesTransform[_lastMaterialName].second=Transform2D::NEW();
				    _urlTexturesTransform[_lastMaterialName].second->preRotate(f);
			    }
		    }	
		    break;
	    case KFDATA:
		    break;
	    case OBJECT_NODE_TAG:
		    break;

	    case NODE_HDR: //Hierarchy
		    {
			    j=0;
			    do {
				    stream->readChar(name[j]);
				    if (stream->fail())
				    {error();return 0;}
			    }while(name[j++]!=0);
			    index+=j;
			    read3dsShort(stream);
			    read3dsShort(stream);
			    short hierValue=read3dsShort(stream);
			    index+=6;
			    if (stream->fail())
			    {error();return 0;}
			    if (string(name)!="$$$DUMMY")
			    {
				    if (hierValue>_lastHierValue)
				    {
					    if (_lastHierarchyObject!="")		
						    _hierarchy.push_back(make_pair<int,string >(_lastHierValue,_lastHierarchyObject));
				    }

				    else if (hierValue<_lastHierValue)
				    {
					    do
					    {
						    if (_hierarchy.empty())
							    _lastHierValue=-1;
						    else {
							    _lastHierValue=_hierarchy.back().first;
							    _hierarchy.pop_back();
						    }
					    }while(_lastHierValue!=hierValue);
				    }
				    if(hierValue==_lastHierValue)
				    {
					    if (_hierarchy.empty())
						    _lastHierarchyObject="";
					    else
						    _lastHierarchyObject=_hierarchy.back().second;
				    }

				    ArRef<ShapePart3D> o=_objects[name];
				    ArRef<Transform3D> tr=Transform3D::NEW();
				    ArRef<Transform3D> trLast=Transform3D::NEW();
				    if (_lastHierarchyObject!="")
				    {
					    if (_transf.find(_lastHierarchyObject)!=_transf.end())
						    tr=_transf[_lastHierarchyObject];
					    _objects[_lastHierarchyObject]->readTransformation(trLast);
				    }

				    if (o)
				    {
					    tr->preTransform(trLast);

					    _lastHierarchyObject=name;
					    _lastHierValue=hierValue;
				    }
				    else _lastHierarchyObject="";

			    }
			    else _lastHierarchyObject="";
		    }

		    break;
#if 1
	    case PIVOT: //Pivot point
		    {
			    double x=read3dsFloat(stream);
			    double y=read3dsFloat(stream);
			    double z=read3dsFloat(stream);
			    index+=3*4;
			    if (stream->fail())
			    {error();return 0;}
			    if (_lastHierarchyObject!="")
			    {
				    ArRef<Transform3D> tr=Transform3D::NEW();
				    ArRef<Transform3D> tr2=Transform3D::NEW();
				    //create invert matrix

				    tr->preTransform(_transf[_lastHierarchyObject]);
				    tr->invert();
				    //			
				    tr2->preTransform(tr);
				    tr2->preTranslate(-x,-y,-z);
				    tr2->preTransform(_transf[_lastHierarchyObject]);

				    //parent
				    /*				    if (!_hierarchy.empty())
								    {
								    tr2->preTransform(_transf[_hierarchy.back().second]);
								    }
								    */				
				    _objects[_lastHierarchyObject]->writeTransformation(tr2);
			    }
		    }
		    break;
#endif
#if 0	//EN TEST

		case BOUNDBOX:
			    {
			     double x=read3dsFloat(stream);
			     double y=read3dsFloat(stream);
			     double z=read3dsFloat(stream);
			     x+=read3dsFloat(stream);
			     y+=read3dsFloat(stream);
			     z+=read3dsFloat(stream);
			     index+=6*4;
			     x/=2.;
			     y/=2.;
			     z/=2.;
			    cerr<<"bb x :  "<<x<<", y :"<<y<<", z:"<<z<<endl;
			     if (_lastHierarchyObject!="")
			     {
				    ArRef<Transform3D> tr=Transform3D::NEW();
				    _objects[_lastHierarchyObject]->readTransformation(tr);
				    tr->preTranslate(x,y,z);
				    tr->preTransform(_transf[_lastHierarchyObject]);
				    _objects[_lastHierarchyObject]->writeTransformation(tr);
			     }

			    }			
			    break;
		case POS_TRACK_TAG: //pos
			    {
				    read3dsShort(stream); //flags
				    read3dsShort(stream);//unknown
				    read3dsShort(stream);
				    read3dsShort(stream);
				    read3dsShort(stream);
				    unsigned short nbKeys=read3dsShort(stream);
				    read3dsShort(stream);
				    unsigned short framenum=read3dsShort(stream);
				    cerr<<"Frame : "<<framenum<<endl;
				    read3dsInt(stream);
				    double x=read3dsFloat(stream);
				    double y=read3dsFloat(stream);
				    double z=read3dsFloat(stream);
				    cerr<<"--!-- "<<x<<","<<y<<","<<z<<" -- keys :"<<nbKeys<<", frame: "<<framenum<<endl;
				    for(int i=nbKeys-1;i--;)
				    {
					    read3dsShort(stream);
					    read3dsInt(stream);
					    read3dsFloat(stream);
					    read3dsFloat(stream);
					    read3dsFloat(stream);
				    }
				    if (_objects.find(_lastHierarchyObject)!=_objects.end())
				    {
					    cerr<<"################################################################"<<endl;
				    ArRef<Transform3D> tr=Transform3D::NEW();
				    _objects[_lastHierarchyObject]->readTransformation(tr);
				    tr->preTranslate(x,y,z);
				    _objects[_lastHierarchyObject]->writeTransformation(tr);
				    }
				
			    }

			    break;
			
			case ROT_TRACK_TAG: //rot
			    {
				    read3dsShort(stream); //flags
				    read3dsShort(stream);//unknown
				    read3dsShort(stream);
				    read3dsShort(stream);
				    read3dsShort(stream);
				    unsigned short nbKeys=read3dsShort(stream);
				    read3dsShort(stream);
				    unsigned short framenum=read3dsShort(stream);
				    read3dsInt(stream);
				    cerr<<"Frame : "<<framenum<<endl;
				    double a=read3dsFloat(stream);
				    double x=read3dsFloat(stream);
				    double y=read3dsFloat(stream);
				    double z=read3dsFloat(stream);
				    cerr<<"--!-- "<<x<<","<<y<<","<<z<<","<<a<<" -- nbKeys :"<<nbKeys<<", frame: "<<framenum<<endl;
				    for(int i=nbKeys-1;i--;)
					
				    {
					    read3dsShort(stream);
					    read3dsInt(stream);
					    read3dsFloat(stream);
					    read3dsFloat(stream);
					    read3dsFloat(stream);
					    read3dsFloat(stream);
				    }
				    cerr<<"name : "<<_lastHierarchyObject<<endl;
				    if (_objects.find(_lastHierarchyObject)!=_objects.end())
				    {
					    cerr<<"################################################################"<<endl;
					
					    ArRef<Transform3D> tr=Transform3D::NEW();
					    _objects[_lastHierarchyObject]->readTransformation(tr);
					    tr->preRotate(x,y,z,a);
//				    tr->preTransform(_transf[_lastHierarchyObject]);
				    _objects[_lastHierarchyObject]->writeTransformation(tr);
				    }
				
			    }
			    break;
			case SCL_TRACK_TAG:
			    {
				    read3dsShort(stream); //flags
				    read3dsShort(stream);//unknown
				    read3dsShort(stream);
				    read3dsShort(stream);
				    read3dsShort(stream);
				    unsigned short nbKeys=read3dsShort(stream);
				    read3dsShort(stream);
				    unsigned short framenum=read3dsShort(stream);
				    read3dsInt(stream);
				    double x=read3dsFloat(stream);
				    double y=read3dsFloat(stream);
				    double z=read3dsFloat(stream);
				    cerr<<"--!-- "<<x<<","<<y<<","<<z<<" -- nbKeys :"<<nbKeys<<", frame: "<<framenum<<endl;
				    for(int i=nbKeys-1;i--;)
					
				    {
					    read3dsShort(stream);
					    read3dsInt(stream);
					    read3dsFloat(stream);
					    read3dsFloat(stream);
					    read3dsFloat(stream);
				    }
				    if (_objects.find(_lastHierarchyObject)!=_objects.end())
				    {
					    cerr<<"################################################################"<<endl;
					    ArRef<Transform3D> tr=Transform3D::NEW();
					    _objects[_lastHierarchyObject]->readTransformation(tr);
					    tr->preScale(x,y,z);
//					    _objects[_lastHierarchyObject]->writeTransformation(tr);
//					    tr->preTransform(_transf[_lastHierarchyObject]);
						
				    _objects[_lastHierarchyObject]->writeTransformation(tr);
				    }	
			    }
			
			    break;
#endif
	    default:
		    {
			    if (chunkLength<index)
			    	{error();return 0;}
			    char * data=new char[chunkLength-index];
			    stream->readData(data,chunkLength-index);
			    if (stream->fail())
				    {error();return 0;}
			    delete [] data;
			    index = chunkLength;    // skips over rest of chunk (ignores it)
		    }
		    break;
    }

    // read child chunks
    while (index < chunkLength){
	    index += read3DSChunk(stream,0);
    if (stream->fail())
	    {error();return 0;}
    }
    return chunkLength;
}

void
Saver3DS::save3DS(ArRef<FDOStream> fstream,ArConstRef<Shape3D> shape)
{
	StlVector<ArRef<MemoryOStream> > streams;
	ArConstRef<ShapePart3D> shp;
	ArConstRef<Surface3D> surf3D;
//	_materials.clear();
	_surfaces.clear();
	_transform.clear();
	
	ArRef<Transform3D> tr=Transform3D::NEW();
	shape->readTransformation(tr);
	
	for (size_t i=0;i<shape->getNbRootParts();i++)
	{
		shp=shape->getRootPart(i);

		saveSurfaces(shp,tr);	
	}

	ArRef<MemoryOStream> stream=createChunkM3DMAGIC();
	
	fstream->writeData(stream->getMemoryBlock()->getMemory(),stream->getMemoryUsage());
	
}
void Saver3DS::saveSurfaces(ArConstRef<ShapePart3D> shapePart,ArConstRef<Transform3D> parentTr)
{
	ArConstRef<Surface3D> surf3D;
	if (shapePart->getClass()->isA(Surface3D::CLASS()))
		surf3D= ar_down_cast<Surface3D>(shapePart);
	ArRef<Transform3D> tr=Transform3D::NEW();
	if (shapePart)	
	{
		shapePart->readTransformation(tr);
		tr->preTransform(parentTr);
		
	}
	if (surf3D)
	{
		ArRef<Material3D> material=Material3D::NEW();
		surf3D->readMaterial(material);
		_surfaces.push_back (surf3D);
		_transform.push_back(tr);
	}
	for (size_t i=0;i<shapePart->getNbSubParts();i++)
	{
		saveSurfaces(shapePart->getSubPart(i),tr);
	}
	
}

void write3dsUInt(ArRef<MemoryOStream> stream,unsigned int value)
{
#if BYTE_ORDER==BIG_ENDIAN
        ArSystem::invert32(&value); // 3DS files are little endian
#endif
	stream->writeData(&value,sizeof(unsigned int));
}
void write3dsUShort(ArRef<MemoryOStream> stream,unsigned short value)
{
#if BYTE_ORDER==BIG_ENDIAN
        ArSystem::invert16(&value); // 3DS files are little endian
#endif
	stream->writeData(&value,sizeof(unsigned short));
}
void write3dsFloat(ArRef<MemoryOStream> stream,double value)
{
        float f=value;
#if BYTE_ORDER==BIG_ENDIAN
        ArSystem::invert32(&f); // 3DS files are little endian
#endif
	stream->writeData(&f,sizeof(float));
}
void write3dsString(ArRef<MemoryOStream> stream,const StlString str)
{
	stream->writeData(str.c_str(),str.size()+1);
}
ArRef<MemoryOStream> Saver3DS::createChunkM3DMAGIC()
{
	ArRef<MemoryOStream> stream = newChunk(M3DMAGIC);
	ArRef<MemoryOStream> tempStream;
	tempStream=createChunkMDATA();
	stream->writeData(tempStream->getMemoryBlock()->getMemory(),tempStream->getMemoryUsage());
	chunk_set_size(stream);
	return stream;
}
ArRef<MemoryOStream> Saver3DS::createChunkMDATA()
{
	ArRef<MemoryOStream> stream = newChunk(MDATA);
	ArRef<MemoryOStream> tempStream;
	tempStream=createChunkMESH_VERSION();
	stream->writeData(tempStream->getMemoryBlock()->getMemory(),tempStream->getMemoryUsage());
//	map<string,ArConstRef<Surface3D>  >::iterator it;
//	for (it=_surfaces.begin();it!=_surfaces.end();it++)
	for(int i=_surfaces.size();i--;)
	{
		tempStream=createChunkMAT_ENTRY(_surfaces[i]);
		stream->writeData(tempStream->getMemoryBlock()->getMemory(),tempStream->getMemoryUsage());
	}
	tempStream=createChunkMASTER_SCALE(1.0);
	for(int i=_surfaces.size();i--;)
	{
		if (_surfaces[i]->getVertexFaces().size())
		{
			tempStream=createChunkNAMED_OBJECT(_surfaces[i],_transform[i]);
			stream->writeData(tempStream->getMemoryBlock()->getMemory(),tempStream->getMemoryUsage());
		}
	}
	chunk_set_size(stream);
	return stream;
}
ArRef<MemoryOStream> Saver3DS::createChunkMESH_VERSION()
{
	unsigned int version=0x00000003;
	ArRef<MemoryOStream> stream = newChunk(MESH_VERSION);
	write3dsUInt(stream,version);
	chunk_set_size(stream);
	return stream;
	
}
ArRef<MemoryOStream> Saver3DS::createChunkMASTER_SCALE(double scale)
{
	ArRef<MemoryOStream> stream = newChunk(MASTER_SCALE);
	write3dsFloat(stream,scale);
	chunk_set_size(stream);
	return stream;
	
}
ArRef<MemoryOStream> Saver3DS::createChunkMAT_ENTRY(ArConstRef<Surface3D> surf3D)
{
	ArRef<MemoryOStream> stream = newChunk(MAT_ENTRY);
	ArRef<MemoryOStream> tempStream;
	ArRef<Material3D> mat=Material3D::NEW();
	surf3D->readMaterial(mat);
	ArConstRef<URLTexture> texmap;
	if (surf3D->getTexture() && surf3D->getTexture()->getClass()->isA(URLTexture::CLASS()))
		texmap=ar_down_cast<URLTexture>(surf3D->getTexture());
#if 0
	ArConstRef<URLTexture> tex2map;
	if (surf3D->getSecondTexture() && surf3D->getSecondTexture()->getClass()->isA(URLTexture::CLASS()))
		tex2map=ar_down_cast<URLTexture>(surf3D->getSecondTexture());
#endif	
	ArRef<Transform2D> tr=Transform2D::NEW();
	surf3D->readTextureTransformation(tr);
	ArRef<Transform2D> tr2=Transform2D::NEW();
	surf3D->readSecondTextureTransformation(tr2);
	tempStream=createChunkMAT_NAME(surf3D->getAlias()+"_MAT");
	stream->writeData(tempStream->getMemoryBlock()->getMemory(),tempStream->getMemoryUsage());
	double r,g,b;
	unsigned int p;
	r=g=b=mat->getAmbientIntensity();
	tempStream=createChunkColor(MAT_AMBIENT,r,g,b);
	stream->writeData(tempStream->getMemoryBlock()->getMemory(),tempStream->getMemoryUsage());
	mat->getDiffuseColor(r,g,b);
	tempStream=createChunkColor(MAT_DIFFUSE,r,g,b);
	stream->writeData(tempStream->getMemoryBlock()->getMemory(),tempStream->getMemoryUsage());
	mat->getSpecularColor(r,g,b);
	tempStream=createChunkColor(MAT_SPECULAR,r,g,b);
	stream->writeData(tempStream->getMemoryBlock()->getMemory(),tempStream->getMemoryUsage());
	p=(int)(mat->getShininess()*100.);
	tempStream=createChunkPercentage(MAT_SHININESS,p);
	stream->writeData(tempStream->getMemoryBlock()->getMemory(),tempStream->getMemoryUsage());
	tempStream=createChunkPercentage(MAT_SHIN2PCT,p);
	stream->writeData(tempStream->getMemoryBlock()->getMemory(),tempStream->getMemoryUsage());
	p=(int)(mat->getTransparency()*100.);
	tempStream=createChunkPercentage(MAT_TRANSPARENCY,p);
	stream->writeData(tempStream->getMemoryBlock()->getMemory(),tempStream->getMemoryUsage());
	tempStream=createChunkMAT_SHADING();
	stream->writeData(tempStream->getMemoryBlock()->getMemory(),tempStream->getMemoryUsage());
	if(surf3D->getDoubleSided())
	{
		tempStream=createChunkMAT_TWO_SIDE();
		stream->writeData(tempStream->getMemoryBlock()->getMemory(),tempStream->getMemoryUsage());
	}
	if (texmap)
	{
		tempStream=createChunkMAT_TEXMAP(texmap,tr);
		stream->writeData(tempStream->getMemoryBlock()->getMemory(),tempStream->getMemoryUsage());
	}
#if 0 //NO SECOND TEXTURE
	if (tex2map)
	{
		tempStream=createChunkMAT_TEX2MAP(tex2map,tr2);
		stream->writeData(tempStream->getMemoryBlock()->getMemory(),tempStream->getMemoryUsage());
	}
#endif
	chunk_set_size(stream);
	return stream;
	
}
ArRef<MemoryOStream> Saver3DS::createChunkMAT_NAME(const StlString& name)
{
	ArRef<MemoryOStream> stream = newChunk(MAT_NAME);
	write3dsString(stream,name);
	chunk_set_size(stream);
	return stream;
	
}
ArRef<MemoryOStream> Saver3DS::createChunkTexTransform(ArConstRef<Transform2D> tr2D)
{
	const double * matrix=tr2D->getTransformation().matrix;
	double scaleX,scaleY,transX,transY,angle;
	scaleX=sqrt(matrix[Util3D::R00]*matrix[Util3D::R00]+matrix[Util3D::R01]*matrix[Util3D::R01]+matrix[Util3D::R02]*matrix[Util3D::R02]);
	scaleY=sqrt(matrix[Util3D::R10]*matrix[Util3D::R10]+matrix[Util3D::R11]*matrix[Util3D::R11]+matrix[Util3D::R12]*matrix[Util3D::R12]);
	transX=matrix[Util3D::TX];
	transY=matrix[Util3D::TY];
	angle= atan2(matrix[Util3D::R10],matrix[Util3D::R00]);
	
	ArRef<MemoryOStream> stream = newChunk(MAT_MAP_USCALE);
	write3dsFloat(stream,scaleX);
	chunk_set_size(stream);

	write3dsUShort(stream,MAT_MAP_VSCALE);
	write3dsUInt(stream,10);
	write3dsFloat(stream,scaleY);
	
	write3dsUShort(stream,MAT_MAP_ANG);
	write3dsUInt(stream,10);
	write3dsFloat(stream,angle);
	
	write3dsUShort(stream,MAT_MAP_UOFFSET);
	write3dsUInt(stream,10);
	write3dsFloat(stream,transX);
	write3dsUShort(stream,MAT_MAP_VOFFSET);
	write3dsUInt(stream,10);
	write3dsFloat(stream,transY);
	
	return stream;
}
ArRef<MemoryOStream> Saver3DS::createChunkColor(unsigned short id, double r, double g,double b)
{
	ArRef<MemoryOStream> stream = newChunk(id);
	write3dsUShort(stream,COLOR_F);
	write3dsUInt(stream,18);
	write3dsFloat(stream,r);
	write3dsFloat(stream,g);
	write3dsFloat(stream,b);
	chunk_set_size(stream);
	return stream;
	
}
ArRef<MemoryOStream> Saver3DS::createChunkPercentage(unsigned short id, unsigned int percentage)
{
	ArRef<MemoryOStream> stream = newChunk(id);
	write3dsUShort(stream,INT_PERCENTAGE);
	write3dsUInt(stream,8);
	write3dsUShort(stream,percentage);
	chunk_set_size(stream);
	return stream;
	
}
ArRef<MemoryOStream> Saver3DS::createChunkMAT_SHADING()
{
	ArRef<MemoryOStream> stream = newChunk(MAT_SHADING);
	write3dsUShort(stream,3); //Phong
	chunk_set_size(stream);
	return stream;
	
}
ArRef<MemoryOStream> Saver3DS::createChunkMAT_TWO_SIDE()
{
	ArRef<MemoryOStream> stream = newChunk(MAT_TWO_SIDE);
	return stream;
	
}

ArRef<MemoryOStream> Saver3DS::createChunkMAT_TEXMAP(ArConstRef<URLTexture> texture,ArConstRef<Transform2D> tr2D)
{
	int percentage=100;
	ArRef<MemoryOStream> stream = newChunk(MAT_TEXMAP);
	ArRef<MemoryOStream> tempStream;
	write3dsUShort(stream,INT_PERCENTAGE);
	write3dsUInt(stream,8);
	write3dsUShort(stream,percentage);
	tempStream=createChunkMAT_MAPNAME(texture->getURL());
	stream->writeData(tempStream->getMemoryBlock()->getMemory(),tempStream->getMemoryUsage());
	if (tr2D)
	{
		tempStream=createChunkTexTransform(tr2D);
		stream->writeData(tempStream->getMemoryBlock()->getMemory(),tempStream->getMemoryUsage());
	}
	tempStream=createChunkMAT_MAP_TILING();
	stream->writeData(tempStream->getMemoryBlock()->getMemory(),tempStream->getMemoryUsage());
	chunk_set_size(stream);
	return stream;
}
ArRef<MemoryOStream> Saver3DS::createChunkMAT_TEX2MAP(ArConstRef<URLTexture> texture,ArConstRef<Transform2D> tr2D)
{
	int percentage=100;
	ArRef<MemoryOStream> stream = newChunk(MAT_TEX2MAP);
	ArRef<MemoryOStream> tempStream;
	write3dsUShort(stream,INT_PERCENTAGE);
	write3dsUInt(stream,8);
	write3dsUShort(stream,percentage);
	tempStream=createChunkMAT_MAPNAME(texture->getURL());
	stream->writeData(tempStream->getMemoryBlock()->getMemory(),tempStream->getMemoryUsage());
	if (tr2D)
	{
		tempStream=createChunkTexTransform(tr2D);
		stream->writeData(tempStream->getMemoryBlock()->getMemory(),tempStream->getMemoryUsage());
	}
	tempStream=createChunkMAT_MAP_TILING();
	stream->writeData(tempStream->getMemoryBlock()->getMemory(),tempStream->getMemoryUsage());
	chunk_set_size(stream);
	return stream;
}
ArRef<MemoryOStream> Saver3DS::createChunkMAT_MAPNAME(const StlString& name)
{
	ArRef<MemoryOStream> stream = newChunk(MAT_MAPNAME);
	write3dsString(stream,name);
	chunk_set_size(stream);
	return stream;
	
}
ArRef<MemoryOStream> Saver3DS::createChunkMAT_MAP_TILING()
{
	ArRef<MemoryOStream> stream = newChunk(MAT_MAP_TILING);
	write3dsUShort(stream,0);
	chunk_set_size(stream);
	return stream;
}
ArRef<MemoryOStream> Saver3DS::createChunkNAMED_OBJECT(ArConstRef<Surface3D> surface,ArConstRef<Transform3D> tr)
{
	ArRef<MemoryOStream> stream = newChunk(NAMED_OBJECT);
	ArRef<MemoryOStream> tempStream;
	write3dsString(stream,surface->getAlias());
	tempStream=createChunkN_TRI_OBJECT(surface,tr);
	stream->writeData(tempStream->getMemoryBlock()->getMemory(),tempStream->getMemoryUsage());
	chunk_set_size(stream);
	return stream;
}

#define VertexType <int,Util3D::Dbl3>
#define MakeVertex(x,y) make_pair VertexType(x,y)
ArRef<MemoryOStream> Saver3DS::createChunkN_TRI_OBJECT(ArConstRef<Surface3D> surface,ArConstRef<Transform3D> tr)
{
	ArRef<MemoryOStream> stream = newChunk(N_TRI_OBJECT);
	ArRef<MemoryOStream> tempStream;
	/* Readjust textures and vertices*/
	StlMap<size_t,Util3D::Dbl3 > newVertices;
	StlMap<size_t,size_t > newVertexFacesIndex;
	StlVector<Util3D::Dbl2> textures;
	StlVector<Util3D::Face> textureFaces;
	textures=surface->getTextures();		
	textureFaces=surface->getTextureFaces();		
	if (textureFaces.empty())
		textureFaces=surface->getVertexFaces();
	bool isTextureFaces=(!textureFaces.empty())&&(!textures.empty());
	const StlVector<Util3D::Dbl3>& vertices=surface->getVertices();
	
	if (isTextureFaces)
	{
//		newVertices.assign((surface->getTextures().size()>surface->getVertices().size())?surface->getTextures().size():surface->getVertices().size() ,Util3D::Dbl3(0.,0.,0.));
		size_t index=textures.size();
		for (size_t i=surface->getVertexFaces().size();i--;)
		{
			/*
			const Util3D::Face& texFace=surface->getTextureFaces()[i];
			const Util3D::Face& vFace=surface->getVertexFaces()[i];
			double x,y,z;
			tr->apply(x,y,z,vertices[vFace.p1].x,vertices[vFace.p1].y,vertices[vFace.p1].z);
			newVertices[texFace.p1]=Util3D::Dbl3(x,y,z);
			tr->apply(x,y,z,vertices[vFace.p2].x,vertices[vFace.p2].y,vertices[vFace.p2].z);
			newVertices[texFace.p2]=Util3D::Dbl3(x,y,z);
			tr->apply(x,y,z,vertices[vFace.p3].x,vertices[vFace.p3].y,vertices[vFace.p3].z);
			newVertices[texFace.p3]=Util3D::Dbl3(x,y,z);
			*/
			Util3D::Face& texFace=textureFaces[i];
			const Util3D::Face& vFace=surface->getVertexFaces()[i];
			double x,y,z;
			tr->apply(x,y,z,vertices[vFace.p1].x,vertices[vFace.p1].y,vertices[vFace.p1].z);
			if (newVertices.find(texFace.p1)!=newVertices.end() && newVertexFacesIndex[texFace.p1]!=vFace.p1)
			{
				textures.push_back(textures[texFace.p1]);
				texFace.p1=index;
				index++;
			}
			newVertices[texFace.p1]=Util3D::Dbl3(x,y,z);
			newVertexFacesIndex[texFace.p1]=vFace.p1;
			tr->apply(x,y,z,vertices[vFace.p2].x,vertices[vFace.p2].y,vertices[vFace.p2].z);
			if (newVertices.find(texFace.p2)!=newVertices.end() && newVertexFacesIndex[texFace.p2]!=vFace.p2)
			{
				textures.push_back(textures[texFace.p2]);
				texFace.p2=index;
				index++;
			}
			newVertices[texFace.p2]=Util3D::Dbl3(x,y,z);
			newVertexFacesIndex[texFace.p2]=vFace.p2;
			tr->apply(x,y,z,vertices[vFace.p3].x,vertices[vFace.p3].y,vertices[vFace.p3].z);
			if (newVertices.find(texFace.p3)!=newVertices.end() && newVertexFacesIndex[texFace.p3]!=vFace.p3)
			{
				textures.push_back(textures[texFace.p3]);
				texFace.p3=index;
				index++;
			}
			newVertices[texFace.p3]=Util3D::Dbl3(x,y,z);
			newVertexFacesIndex[texFace.p3]=vFace.p3;
		}
	}
	else
	{
		newVertices.clear();
		for(size_t i=0;i<vertices.size();i++)
		{
			double x,y,z;
			tr->apply(x,y,z,vertices[i].x,vertices[i].y,vertices[i].z);
			newVertices[i]=Util3D::Dbl3(x,y,z);
		}
	}
	tempStream=createChunkPOINT_ARRAY(newVertices);
	stream->writeData(tempStream->getMemoryBlock()->getMemory(),tempStream->getMemoryUsage());
	if (isTextureFaces)
	{
		tempStream=createChunkTEX_VERTS(textures);
		stream->writeData(tempStream->getMemoryBlock()->getMemory(),tempStream->getMemoryUsage());
	}
	tempStream=createChunkFACE_ARRAY((isTextureFaces)?textureFaces:surface->getVertexFaces(),surface->getAlias());
	stream->writeData(tempStream->getMemoryBlock()->getMemory(),tempStream->getMemoryUsage());
	chunk_set_size(stream);
	return stream;
}
ArRef<MemoryOStream> Saver3DS::createChunkPOINT_ARRAY(StlMap<size_t,Util3D::Dbl3> & vertices)
{
	ArRef<MemoryOStream> stream = newChunk(POINT_ARRAY);
	size_t size=(--vertices.end())->first+1;
	write3dsUShort(stream,size);
	for (size_t i=0;i<size;i++)
	{
		write3dsFloat(stream,vertices[i].x);
		write3dsFloat(stream,vertices[i].y);
		write3dsFloat(stream,vertices[i].z);
	}
	chunk_set_size(stream);
	return stream;
}
ArRef<MemoryOStream> Saver3DS::createChunkFACE_ARRAY(const StlVector<Util3D::Face> & faces,const StlString& name)
{
	ArRef<MemoryOStream> stream = newChunk(FACE_ARRAY);
	ArRef<MemoryOStream> tempStream;
	write3dsUShort(stream,faces.size());
	for (size_t i=0;i<faces.size();i++)
	{
		write3dsUShort(stream,faces[i].p1);
		write3dsUShort(stream,faces[i].p2);
		write3dsUShort(stream,faces[i].p3);
		write3dsUShort(stream,0x8);
	}
	tempStream=createChunkMSH_MAT_GROUP(faces,name);
	stream->writeData(tempStream->getMemoryBlock()->getMemory(),tempStream->getMemoryUsage());
	chunk_set_size(stream);
	return stream;
}
ArRef<MemoryOStream> Saver3DS::createChunkMSH_MAT_GROUP(const StlVector<Util3D::Face> & faces,const StlString &name)
{
	ArRef<MemoryOStream> stream = newChunk(MSH_MAT_GROUP);
	write3dsString(stream,name+"_MAT");
	write3dsUShort(stream,faces.size());
	for (size_t i=0;i<faces.size();i++)
	{
		
		write3dsUShort(stream,i);
	}
	chunk_set_size(stream);
	return stream;
	
}
ArRef<MemoryOStream> Saver3DS::createChunkTEX_VERTS(const StlVector<Util3D::Dbl2> & textures)
{
	ArRef<MemoryOStream> stream = newChunk(TEX_VERTS);
	write3dsUShort(stream,textures.size());
	for (size_t i=0;i<textures.size();i++)
	{
		write3dsFloat(stream,textures[i].x);
		write3dsFloat(stream,textures[i].y);
	}
	chunk_set_size(stream);
	return stream;
}

ArRef<MemoryOStream> Saver3DS::newChunk(unsigned short chunkId)
{
	ArRef<MemoryBlock> mem=MemoryBlock::NEW();
	ArRef<MemoryOStream> stream=MemoryOStream::NEW(mem,0);
	write3dsUShort(stream,chunkId);
	write3dsUInt(stream,6);
	return stream;
	
}
void Saver3DS::chunk_set_size(ArRef<MemoryOStream> stream)
{
	unsigned int size=stream->getMemoryUsage();
#if BYTE_ORDER==BIG_ENDIAN
        ArSystem::invert32(&size); // 3DS files are little endian
#endif
	memcpy(stream->getMemoryBlock()->accessMemory()+2,&size,sizeof(size));
}

} // namespace AReVi

//----------------------------------------------------------------------------
