/* LI2/ENIB - AReVi-Contrib - 3D file loader/saver
//----------------------------------------------------------------------------
// file        : AReVi/Contrib/File3D/wavefront.l
// author(s)   : C. Septseault
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
// Class WFOParser
//--------------------------------------------------------------------------*/

%{
#undef YY_DECL
#define YY_DECL int WFOParser::yylex(void * arg)
#define yylval (*((YYSTYPE*)arg))
%}

%option c++
%option 8bit
%option noyywrap
%option prefix="_WFO"
%option yyclass="WFOParser"

%x strEnv


space		[ \t\r]+
nl		[\n]
integer		[0-9]+
negInteger	-[0-9]+
hexaInteger	0[Xx][0-9A-Fa-f]+
r1		[0-9]*\.[0-9]+
r2		[0-9]+\.[0-9]*
r_exp		[eE][+-]?[0-9]+
real		[+\-]?(({r1}|{r2}){r_exp}?|{integer}{r_exp})
id1		[^\x00-\x20"#',./[\\\]{}\x7f+\-0-9]
id2		[^\x00-\x20"#',/[\\\]{}\x7f]
ident		{id1}{id2}*

%%



"/"	{
	_store(yytext);
	return(SLASH);
	}
"//"	{
	_store(yytext);
	return(SLASH2);
	}
	/* keyword */
"v"	{
	_store(yytext);
	return (VERTEX_KW);
	}

"vp"	{
	_store(yytext);
	return (VERTEX_PARAM_KW);
	}
"vn"	{
	_store(yytext);
	return (VERTEX_NORMAL_KW);
	}
"vt"	{
	_store(yytext);
	return (VERTEX_TEX_KW);
	}

	/* Curves and free Form */
"cstype"	{
		_store(yytext);
		return (CSTYPE_KW);
		}
"deg"	{
		_store(yytext);
		return (DEG_KW);
		}
"bmat"	{
		_store(yytext);
		return (BMAT_KW);
		}
"step"	{
		_store(yytext);
		return (STEP_KW);
		}
	/* Referencing groups and vertices */
"p"	{
		_store(yytext);
		return (POINT_KW);
		}
"l"	{
		_store(yytext);
		return (LINE_KW);
		}
"f"	{
		_store(yytext);
		return (FACE_KW);
		}


"curv"	{
		_store(yytext);
		return (CURV_KW);
		}
"curv2"	{
		_store(yytext);
		return (CURV2_KW);
		}
"surf"	{
		_store(yytext);
		return (SURF_KW);
		}
	/* Grouping */
"g"	{
		_store(yytext);
		return (GROUP_KW);
		}
"s"	{
		_store(yytext);
		return (S_KW);
		}
"mg"	{
		_store(yytext);
		return (MG_KW);
		}
"o"	{
		_store(yytext);
		return (OBJECT_KW);
		}
"mtllib"	{
		_store(yytext);
		return (MTLLIB_KW);
		}
"usemtl"	{
		_store(yytext);
		return (USEMTL_KW);
		}

	/*... */

"off"	{
		_store(yytext);
		yylval.integer=0;
		return (INTEGER_VAL);
	}
"end"	{
		_store(yytext);
		return (END_KW);
	}

{integer}	{
                _store(yytext);
                sscanf(yytext,"%d",&yylval.integer);
                return(INTEGER_VAL);
                }
{negInteger}	{
                _store(yytext);
                sscanf(yytext,"%d",&yylval.integer);
                return(NEG_INTEGER_VAL);
                }
{real}		{
                _store(yytext);
                sscanf(yytext,"%lg",&yylval.real);
                return(REAL_VAL);
                }

{ident}		{
                _store(yytext);
                yylval.str=new StlString(yytext);
                return(IDENT);
                }

"#".*{nl}    	{ /* comment */_nextLine(); }
{space}		{ _store(yytext); }
{nl}		{ _nextLine();return (NL);}
.		{
                _store(yytext);
                yyerror("lexical error");
                }
%%

/*--------------------------------------------------------------------------*/
