// LI2/ENIB - AReVi-Contrib - 3D file loader/saver
//----------------------------------------------------------------------------
// file        : AReVi/Contrib/File3D/wavefront.y
// author(s)   : C. Septseault
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// Class WFOParser
//----------------------------------------------------------------------------

%{
#include "AReVi/Contrib/File3D/wavefront.h"

#include "AReVi/Shapes/shape3D.h"
#include "AReVi/Shapes/mesh3D.h"

#include <stdio.h>
#include <stdlib.h>

using namespace AReVi;

#define YYINCLUDED_STDLIB_H
#define yyparse WFOParser::parse
%}

%pure_parser

%token SLASH
%token SLASH2
/* Keywords */
%token VERTEX_KW VERTEX_PARAM_KW VERTEX_NORMAL_KW VERTEX_TEX_KW
%token CSTYPE_KW DEG_KW BMAT_KW STEP_KW
%token POINT_KW LINE_KW FACE_KW CURV_KW CURV2_KW SURF_KW
%token GROUP_KW S_KW MG_KW OBJECT_KW
%token MTLLIB_KW USEMTL_KW
%token NL
%token END_KW



%union
  {
  int integer;
  double real;
  StlString * str;
  Int3 i3;
  };

%token<integer> INTEGER_VAL
%token<integer> NEG_INTEGER_VAL
%token<real>    REAL_VAL
%token<str>     IDENT

%type<real> real optReal
%type<i3> int3
%type<integer> integerVertex integerTexture integerNormal

%start wfo

%%

wfo:
	wfoLines {
			if (_mesh->getVertices().empty())
			{
				_message+="Error : 0 Vertices\n";	
				_message+="Maybe not a WaveFront file\n";
			}
			
			if (_mesh->getNormals().empty())
				_mesh->computeDefaultNormals(false);
			if (!_mesh->applyChanges(true))
			{
			_message+="Warning : Consistance problem !!!!!!!!!\n";
			if(!_mesh->checkVertexConsistency())
				_message+="vertex Consistance failed\n";
			if(!_mesh->checkNormalConsistency())
				_message+="normal Consistance failed\n";
			if(!_mesh->checkTextureConsistency())
				_message+="texture Consistance failed\n";
			if(!_mesh->checkColorConsistency())
				_message+="color Consistance failed\n";
			}
				}
;
wfoLines:
	 // empty
	| wfoLines wfoLine
;
wfoLine:
	command NL
	|NL
	|freeFormGroups
	|others
;
others:
      IDENT othersParams NL {}
;
othersParams:
	
	| othersParams IDENT
	| othersParams real
	| othersParams SLASH
;	
freeFormGroups:
	freeFormKeyWord freeFormParams END_KW
;
freeFormKeyWord:
	 CURV_KW
	| CURV2_KW
	| SURF_KW
;
freeFormParams:

	|freeFormParams IDENT
	|freeFormParams  real
	|freeFormParams  SLASH
	|freeFormParams NL
;
real:
	REAL_VAL
	|NEG_INTEGER_VAL {$$=$1;}
	|INTEGER_VAL {$$=$1;}
;
optReal:
       {$$=0.;}
       | real {$$=$1;}
;
command:
	vertexData
	| freeFormAttributes
	| refGroupsData
	| groupsData
	| renderData
;
vertexData:
	vertex
	| vertexParam
	| vertexNormal
	| vertexTexture
;
vertex:
	VERTEX_KW real real real optReal
		{
		vertices->push_back(Util3D::Dbl3($2,$3,$4));	
		}
;
vertexParam:
	VERTEX_PARAM_KW real  optReal //optReal
		{
		_warningNotImplemented()
			//TODO
		}
;
vertexNormal:
	VERTEX_NORMAL_KW real real real
		{
			normals->push_back(Util3D::Dbl3($2,$3,$4));
		}
;
vertexTexture:
	VERTEX_TEX_KW real optReal //optReal
		{
			vtex->push_back(Util3D::Dbl2($2,$3)); // IGNORE $4
		}
;
freeFormAttributes:
	cstype
	| deg
	| bmat
	| step
;
cstype:
	CSTYPE_KW IDENT {_warningNotImplemented()}
;
deg:
	DEG_KW INTEGER_VAL {_warningNotImplemented()}
	|DEG_KW INTEGER_VAL INTEGER_VAL {_warningNotImplemented()}

;
integers:
	
	|integers INTEGER_VAL {}
	|integers NEG_INTEGER_VAL {}

;
bmat:
	BMAT_KW IDENT integers {_warningNotImplemented()}
;
step:
	STEP_KW INTEGER_VAL {_warningNotImplemented()}
;

refGroupsData:
	point
	| line
	| face
;
point:
	POINT_KW {_warningNotImplemented()}
;
line:
	LINE_KW {_warningNotImplemented()}
;

integerVertex:
       INTEGER_VAL
       |NEG_INTEGER_VAL {$$=(int)faces->size()+$1+1;}
;
integerTexture:
       INTEGER_VAL
       |NEG_INTEGER_VAL {$$=(int)textureFaces->size()+$1+1;}
;
integerNormal:
       INTEGER_VAL
       |NEG_INTEGER_VAL {$$=(int)normalFaces->size()+$1+1;}
;

int3:
	integerVertex  SLASH integerTexture SLASH integerNormal {$$.x=$1;$$.y=$3;$$.z=$5;}
	|integerVertex  SLASH integerTexture {$$.x=$1 ; $$.y=3;$$.z=0;}
	|integerVertex  SLASH2 integerNormal {$$.x=$1; $$.y=0;$$.z=$3;}
	|integerVertex  {$$.x=$1; $$.y=$$.z=0;}
;	
face:
	FACE_KW int3 int3 int3 {
					
						faces->push_back(Util3D::Face($2.x-1,$3.x-1,$4.x-1));
						textureFaces->push_back(Util3D::Face($2.y-1,$3.y-1,$4.y-1));
						normalFaces->push_back(Util3D::Face($2.z-1,$3.z-1,$4.z-1));


					}
	|FACE_KW int3 int3 int3 int3{
						faces->push_back(Util3D::Face($2.x-1,$3.x-1,$4.x-1));
						textureFaces->push_back(Util3D::Face($2.y-1,$3.y-1,$4.y-1));
						normalFaces->push_back(Util3D::Face($2.z-1,$3.z-1,$4.z-1));
						faces->push_back(Util3D::Face($2.x-1,$4.x-1,$5.x-1));
						textureFaces->push_back(Util3D::Face($2.y-1,$4.y-1,$5.y-1));
						normalFaces->push_back(Util3D::Face($2.z-1,$4.z-1,$5.z-1));
					}
;
floatsAndOthers:
	
	|floatsAndOthers real
	|floatsAndOthers SLASH
;
/*
curv:
    	CURV_KW
;
curv2:
    	CURV2_KW
;
surf:
    	SURF_KW
;
*/

groupsData:
	group
	| groupNumber
	| groupNumber2
	| object
;
idents:
      |idents IDENT
;
group:
     GROUP_KW idents {_warningNotImplemented()}
;
groupNumber: /*Smoothing group statement */
	   S_KW INTEGER_VAL {_warningNotImplemented()}
;
groupNumber2:
	    MG_KW real floatsAndOthers {_warningNotImplemented()}
;
object:
      OBJECT_KW IDENT {_warningNotImplemented()}
;

renderData:
	  mtllib
	|usemtl
;
mtllib:
      MTLLIB_KW IDENT
;
usemtl:
      USEMTL_KW IDENT
;
%%
namespace AReVi {

WFOParser::WFOParser(istream * input)
	:_WFOFlexLexer(input)
	
{
}
WFOParser::~WFOParser()
{
}
void
WFOParser::yyerror(const char * text)
{
StlString msg=text;
if(_line.size())
  {
  msg+='\n';
  msg+=_line;
  msg+=" <---";
  }
  	_message+="Error :"+msg+"\n";
  _failed=true;
}

void
WFOParser::_nextLine(void)
{
_nbLines++;
_line.erase();
}
void
WFOParser::_store(const char * text)
{
_line+=text;
}
void
WFOParser::_warningNotImplemented()
{
	_message+=intToStr(_nbLines)+"Warning : function "+_line+" not implemented\n";
	_failed=true;
}

void WFOParser::setShape(ArRef<Shape3D> shape)
{
_shape=shape;
 _mesh=Mesh3D::NEW();
 _shape->addRootPart(_mesh);
 vertices=&(_mesh->accessVertices());
 normals=&(_mesh->accessNormals());
 vtex=&(_mesh->accessTextures());
 faces=&(_mesh->accessVertexFaces());
 textureFaces=&(_mesh->accessTextureFaces());
 normalFaces=&(_mesh->accessNormalFaces());
}
void WFOParser::setUrlDir(const StlString & dir)
{
	_urlDir=dir;
}


bool
SaverWaveFront::saveWaveFront(FILE * file, ArConstRef<Shape3D> shapeIn)
{
	_message="";
	unsigned int base_v=1;
	unsigned int base_t=1;
	unsigned int base_n=1;
	if (!file)
	{
		_message="Cannot open file\n";
		return false;	
	}
	
		for(size_t i=0;i<shapeIn->getNbRootParts();i++)
		{
			ArConstRef<Surface3D> surf3D=ar_down_cast<Surface3D>(shapeIn->getRootPart(i));
			if(surf3D)
			{
				fprintf(file,"# Shape %s\n",surf3D->getAlias().c_str());
				StlVector<Util3D::Dbl3>::const_iterator itf3;
				/* Save Vertices */
				for(itf3=surf3D->getVertices().begin();itf3!=surf3D->getVertices().end();itf3++)
				{
					fprintf(file,"v %f %f %f\n",itf3->x,itf3->y,itf3->z);
				}
				StlVector<Util3D::Dbl2>::const_iterator itf2;
				/* Save Textures */
				for(itf2=surf3D->getTextures().begin();itf2!=surf3D->getTextures().end();itf2++)
				{
					fprintf(file,"vt %f %f\n",itf2->x,itf2->y);
				}
#if 0 //EN TEST				
				/* Save Normals */
				for(itf3=surf3D->getNormals().begin();itf3!=surf3D->getNormals().end();itf3++)
				{
					fprintf(file,"vn %f %f %f\n",itf3->x,itf3->y,itf3->z);
				}
#endif
				/* Save Object Name */
				fprintf(file,"o %s\n",surf3D->getAlias().c_str());
				/* Save Faces */
				
				size_t nbVFaces=surf3D->getVertexFaces().size();
				
				if (nbVFaces==surf3D->getNormalFaces().size() && nbVFaces==surf3D->getTextureFaces().size() && 0) //Do not take normal now;
				for(size_t i=0;i<nbVFaces;i++)
				{
					const Util3D::Face & vf=surf3D->getVertexFaces()[i];
					const Util3D::Face & tf=surf3D->getTextureFaces()[i];
					const Util3D::Face & nf=surf3D->getNormalFaces()[i];
					fprintf(file,"f %d/%d/%d %d/%d/%d %d/%d/%d\n",vf.p1+base_v,tf.p1+base_t,nf.p1+base_n,
										vf.p2+base_v,tf.p2+base_t,nf.p2+base_n,
										vf.p3+base_v,tf.p3+base_t,nf.p3+base_n);
				}
				else if (nbVFaces==surf3D->getNormalFaces().size() && 0) // Do not take normal now
				{
					for(size_t i=0;i<nbVFaces;i++)
					{
						const Util3D::Face & vf=surf3D->getVertexFaces()[i];
						const Util3D::Face & nf=surf3D->getNormalFaces()[i];
						fprintf(file,"f %d//%d %d//%d %d//%d\n",vf.p1+base_v,nf.p1+base_n,vf.p2+base_v,nf.p2+base_n,vf.p3+base_v,nf.p3+base_n);
					}
				}
				else if (nbVFaces==surf3D->getTextureFaces().size() )
				{
					for(size_t i=0;i<nbVFaces;i++)
					{
						const Util3D::Face & vf=surf3D->getVertexFaces()[i];
						const Util3D::Face & tf=surf3D->getTextureFaces()[i];
						fprintf(file,"f %d/%d %d/%d %d/%d\n",vf.p1+base_v,tf.p1+base_t,vf.p2+base_v,tf.p2+base_t,vf.p3+base_v,tf.p3+base_t);
					}
				}
				else
				{
					const Util3D::Face & vf=surf3D->getVertexFaces()[i];
					fprintf(file,"f %d %d %d\n",vf.p1+base_v,vf.p2+base_v,vf.p3+base_v);
					/*
					//  Pour le moment : pas de sauvegarde des coords de textures & normales  //
					if (surf3D->getNormals().empty() && surf3D->getTextures().empty())
					for(size_t i=0;i<nbVFaces;i++)
					{
						const Util3D::Face & vf=surf3D->getVertexFaces()[i];
						fprintf(file,"f %d %d %d\n",vf.p1+1,vf.p2+1,vf.p3+1);
					}
					else if (surf3D->getNormals().empty())
					for(size_t i=0;i<nbVFaces;i++)
					{
						const Util3D::Face & vf=surf3D->getVertexFaces()[i];
						fprintf(file,"f %d/%d %d/%d %d/%d\n",vf.p1+1,vf.p1+1,vf.p2+1,vf.p2+1,vf.p3+1,vf.p3+1);
					}
					else  if (surf3D->getTextures().empty())
					for(size_t i=0;i<nbVFaces;i++)
					{
						const Util3D::Face & vf=surf3D->getVertexFaces()[i];
							fprintf(file,"f %d//%d %d//%d %d//%d\n",vf.p1+1,vf.p1+1,vf.p2+1,vf.p2+1,vf.p3+1,vf.p3+1);
					}
					else
					for(size_t i=0;i<nbVFaces;i++)
					{
						const Util3D::Face & vf=surf3D->getVertexFaces()[i];
							fprintf(file,"f %d/%d/%d %d/%d/%d %d/%d/%d\n",vf.p1+1,vf.p1+1,vf.p1+1,
										vf.p2+1,vf.p2+1,vf.p2+1, vf.p3+1,vf.p3+1,vf.p3+1);
					}
					*/
				}
				base_v+=surf3D->getVertices().size();
				base_t+=surf3D->getTextures().size();
				base_n+=surf3D->getNormals().size();

			}
		}
		_message="";
		return true;
	
}

const StlString & SaverWaveFront::getMessage()
{
	return _message;
}

} // namespace AReVi

//----------------------------------------------------------------------------
