// LI2/ENIB - AReVi-Contrib - ArConsole ???
//----------------------------------------------------------------------------
// file        : AReVi/Contrib/arConsole.cpp
// author(s)   : T. Jourdan
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/Contrib/arConsole.h"

#include "AReVi/Lib3D/gl_priv.h"
#include "AReVi/Lib3D/window3D.h"
#include "AReVi/Utils/abstractStream.h"

namespace AReVi {

// -------------------------------------------------------------------
// ArConsole
// -------------------------------------------------------------------

AR_CLASS_DEF(ArConsole, OSDText)

ArConsole::ArConsole(ArCW& arCW)
: OSDText(arCW),
  _focusKey("\x1b"),
  _ro(false),
  _bufferSize(1000),
  _displaySize(20),
  _pos(0),
  _inputBuffer("> "),
  _window(Window3D::nullRef()),
  _backgroundChangedCB(thisPtr()),
  _foregroundChangedCB(thisPtr()),
  _scrollTextCB(thisPtr()),
  _bufferSetupCB(thisPtr()),
  _inputTextCB(thisPtr()),
  _pushTextCB(thisPtr()),
  _storeCharacterCB(thisPtr()),
  _clearTextCB(thisPtr()),
  _pushAttribCB(thisPtr()),
  _popAttribCB(thisPtr()),
  _flushAttribsCB(thisPtr())   {
  ArConsoleAttr attr;
  attr.r = 1.f;
  attr.g = 1.f;
  attr.b = 1.f;
  attr.a = 1.f;
  attr.font = "9x15bold";
  _attr.push(attr);

  _bg.r = 0.3;
  _bg.g = 0.3;
  _bg.b = 0.8;
  _bg.a = 0.3;

  _currentPos = _text.end();
}

ArConsole::ArConsole(ArCW& arCW, bool readOnly)
: OSDText(arCW),
  _focusKey("\x1b"),
  _ro(readOnly),
  _bufferSize(1000),
  _displaySize(20),
  _pos(0),
  _inputBuffer("> "),
  _window(Window3D::nullRef()),
  _backgroundChangedCB(thisPtr()),
  _foregroundChangedCB(thisPtr()),
  _scrollTextCB(thisPtr()),
  _bufferSetupCB(thisPtr()),
  _inputTextCB(thisPtr()),
  _pushTextCB(thisPtr()),
  _storeCharacterCB(thisPtr()),
  _clearTextCB(thisPtr()),
  _pushAttribCB(thisPtr()),
  _popAttribCB(thisPtr()),
  _flushAttribsCB(thisPtr())   {
  ArConsoleAttr attr;
  attr.r = 1.f;
  attr.g = 1.f;
  attr.b = 1.f;
  attr.a = 1.f;
  attr.font = "9x15bold";
  _attr.push(attr);

  _bg.r = 0.3;
  _bg.g = 0.3;
  _bg.b = 0.8;
  _bg.a = 0.3;

  _currentPos = _text.end();
}

ArConsole::~ArConsole() {
}

void
ArConsole::setBackgroundColor(double r, double g, double b, double a) {
  _bg.r = r;
  _bg.g = g;
  _bg.b = b;
  _bg.a = a;

if(!_backgroundChangedCB.empty())
  {
  BackgroundChangedEvent evt;
  _backgroundChangedCB.fire(evt);
  }
_postRedisplay();
}

void
ArConsole::getBackgroundColor(double &r, double &g, double &b, double &a) const{
  r =_bg.r;
  g = _bg.g;
  b = _bg.b;
  a = _bg.a;
}

void
ArConsole::setSize(int nblines) {
  if(nblines > 0) {
    _displaySize = nblines;

    if(!_bufferSetupCB.empty()) {
      BufferSetupEvent evt;
      _bufferSetupCB.fire(evt);
    }
  }
  _postRedisplay();
}

int
ArConsole::getSize(void) const {
  return _displaySize;
}

void
ArConsole::setBufferSize(int nblines) {
  if(nblines > 0) {
    _bufferSize = nblines;

    if(!_bufferSetupCB.empty()) {
      BufferSetupEvent evt;
      _bufferSetupCB.fire(evt);
    }
  }
  _adjustSize();
}

int
ArConsole::getBufferSize(void) const {
  return _bufferSize;
}


void
ArConsole::clear(void) {
  while(!_attr.empty()) {
    _attr.pop();
  }
  ArConsoleAttr attr;
  attr.r = 1.f;
  attr.g = 1.f;
  attr.b = 1.f;
  attr.a = 1.f;
  _attr.push(attr);

  _text.clear();

  while(!_textStream.empty()) {
    _textStream.pop();
  }

  _pos = 0;
  _currentPos = _text.end();

  if(!_clearTextCB.empty()) {
      ClearTextEvent evt;
      _clearTextCB.fire(evt);
    }

  _postRedisplay();
}

void
ArConsole::scroll(int nbLines) {
  int min = _displaySize < (int)_text.size() ? _displaySize : _text.size();
  if(_pos + nbLines < min - 1) {
    nbLines = min - 1 - _pos;
  }

  bool decr = nbLines < 0 ? true : false;
  for(int i = 0; i < abs(nbLines); i++) {
    if(decr) {
      if(_currentPos != _text.begin()) {
	_currentPos--;
      }
    } else {
      if(_currentPos != _text.end()) {
	_currentPos++;
      }
    }
  }

  _pos += nbLines;
  if(_pos < 0) _pos = 0;
  if(_pos > (int)_text.size()) _pos = _text.size();

  if(!_scrollTextCB.empty()) {
      ScrollTextEvent evt;
      evt.nbLines=nbLines;
      _scrollTextCB.fire(evt);
    }
  _postRedisplay();
}

void
ArConsole::setReleaseKeyboardFocusKey(const StlString& key) {
  _focusKey = key;
}

void
ArConsole::grabKeyboardFocus(ArRef<Window3D> window) {
  releaseKeyboardFocus();
  _window = window;
  _window->pushMaskingKeyboardCB(thisRef(), &ArConsole::_keyboardCB);
}

void
ArConsole::releaseKeyboardFocus(void) {
  if(_window != Window3D::nullRef()) {
    _window->popMaskingKeyboardCB(thisRef(), &ArConsole::_keyboardCB);
  }
}

void
ArConsole::releaseKeyboardFocus(const Renderer3D::KeyboardEvent& evt) {
  if(_window != Window3D::nullRef()) {
    _window->popMaskingKeyboardCB(thisRef(), &ArConsole::_keyboardCB);
    Renderer3D::KeyboardEvent evtCopy(evt);
    _window->_keyboardCB.fire(evtCopy);
  }
}

void
ArConsole::setColor(double r, double g, double b, double a) {
  ArConsoleAttr& attr = _attr.top();
  attr.r = r;
  attr.g = g;
  attr.b = b;
  attr.a = a;

  if(!_foregroundChangedCB.empty()) {
      ForegroundChangedEvent evt;
      _foregroundChangedCB.fire(evt);
    }
}

void
ArConsole::getColor(double &rOut, double &gOut, double &bOut, double &aOut) const
{
const ArConsoleAttr& attr = _attr.top();
rOut=attr.r;
gOut=attr.g;
bOut=attr.b;
aOut=attr.a;
}


bool
ArConsole::setFont(const StlString& font) {
  OSDText::setFont(font);
  ArConsoleAttr& attr = _attr.top();
  attr.font = font;

  if(!_foregroundChangedCB.empty()) {
      ForegroundChangedEvent evt;
      _foregroundChangedCB.fire(evt);
    }
  return true;
}

void
ArConsole::pushAttrib(ArConsoleAttr attr) {
if(!_pushAttribCB.empty()) {
      PushAttribEvent evt;
      evt.attr=attr;
      _pushAttribCB.fire(evt);
    }
  _attr.push(attr);
}

void
ArConsole::popAttrib(void) {
if(!_popAttribCB.empty()) {
      PopAttribEvent evt;
      _popAttribCB.fire(evt);
    }
  _attr.pop();
}

void
ArConsole::flushAttrib(void) {
  while(!_attr.empty()) {
    _attr.pop();
  }
  ArConsoleAttr attr;
  attr.r = 1.f;
  attr.g = 1.f;
  attr.b = 1.f;
  attr.a = 1.f;
  _attr.push(attr);

  if(!_flushAttribsCB.empty()) {
      FlushAttribsEvent evt;
      _flushAttribsCB.fire(evt);
    }
}

void
ArConsole::pushDefaultAttrib(void) {
  ArConsoleAttr attr;
  attr.r = 1.f;
  attr.g = 1.f;
  attr.b = 1.f;
  attr.a = 1.f;
  pushAttrib(attr);
}

void
ArConsole::storeCharacter(char c) {
  if(_ro) {
    return;
  }

  switch(c) {
  case 8 :
    if(_inputBuffer.size() > 2) {
      _inputBuffer.erase(_inputBuffer.size() - 1, 1);
    }
    break;
  case 10 : {
    bool incr = false;
    if(_currentPos == _text.end()) {
      incr = true;
    }
    _inputBuffer.erase(0, 2);
    _text.push_back(make_pair(_attr.top(), _inputBuffer));
    // user input 1 line : send InputText event
    if(!_inputTextCB.empty()) {
      InputTextEvent evt;
      evt.text = _inputBuffer;
      _inputTextCB.fire(evt);
    }
   _inputBuffer = "> ";
    if(incr) {
      _pos++;
      _currentPos = _text.end();
    }
    break;
  }
  default :
    _inputBuffer.push_back(c);
    break;
  }

  if(!_storeCharacterCB.empty()) {
      StoreCharacterEvent evt;
      evt.c=c;
      _storeCharacterCB.fire(evt);
  }
  _adjustSize();
  _postRedisplay();
}

void
ArConsole::push(const StlString& text) {
  bool incr;
  StlString line;
  for(size_t i = 0; i < text.size(); i++) {
    if(text[i] == 10 ||
       text[i] == '\0') {
      incr = false;
      if(_currentPos == _text.end()) {
	incr = true;
      }
      _text.push_back(make_pair(_attr.top(), line));
      _textStream.push(line);
      line.clear();
      if(incr) {
	_pos++;
	_currentPos = _text.end();
      }
    } else {
      line.push_back(text[i]);
    }
  }

  incr = false;
  if(_currentPos == _text.end()) {
    incr = true;
  }
  _text.push_back(make_pair(_attr.top(), line));
  _textStream.push(line);
  if(incr) {
    _pos++;
    _currentPos = _text.end();
  }

  if(!_pushTextCB.empty()) {
      PushTextEvent evt;
      evt.text=text;
      _pushTextCB.fire(evt);
  }

  _adjustSize();
  _postRedisplay();
}

ostream& operator<<(ostream& output, const ArRef<ArConsole>& console) {
  while(!console->_textStream.empty()) {
    output << console->_textStream.front() << endl;
    console->_textStream.pop();
  }

  return output;
}

//-------- Serialization operations ------------------------------------------

void
ArConsole::serialize(SerializationDependencies & dependenciesOut,
          ArRef<AbstractOStream> output) const
{
OSDText::serialize(dependenciesOut,output);

unsigned readOnly((_ro?1:0));

output->writeUInt(readOnly);

// Background
output->writeDouble(_bg.r);
output->writeDouble(_bg.g);
output->writeDouble(_bg.b);
output->writeDouble(_bg.a);
output->writeString(_bg.font);

// Buffers
output->writeInt(_bufferSize);
output->writeInt(_displaySize);
output->writeInt(_pos);

output->writeString(_inputBuffer);


stack<ArConsoleAttr> attr(_attr);

// Attributes Stack
output->writeUInt(_attr.size());
while(!attr.empty())
  {
  output->writeDouble(attr.top().r);
  output->writeDouble(attr.top().g);
  output->writeDouble(attr.top().b);
  output->writeDouble(attr.top().a);
  output->writeString(attr.top().font);
  attr.pop();
  }

// Text
unsigned currentPos(_text.size()),current(0);
output->writeUInt(_text.size());

StlList<pair<ArConsoleAttr, StlString> >::const_iterator it=_text.begin();
while(it!=_text.end())
  {
  output->writeDouble(it->first.r);
  output->writeDouble(it->first.g);
  output->writeDouble(it->first.b);
  output->writeDouble(it->first.a);
  output->writeString(it->first.font);
  output->writeString(it->second);

  if (it==_currentPos)
    currentPos=current;

  it++;
  current++;
  }

output->writeUInt(currentPos);

// TextStream
output->writeUInt(_textStream.size());

queue<StlString> textStream(_textStream);

while(!textStream.empty())
  {
  output->writeString(textStream.front());
  textStream.pop();
  }
}


bool // success
ArConsole::unserialize(const SerializationDependencies & dependencies,
            ArRef<AbstractIStream> input)
{
unsigned readOnly, attrSize, textSize, streamSize,currentPos;
StlList<pair<ArConsoleAttr, StlString> >::iterator current;
ArConsoleAttr bg;
int bufferSize, displaySize, pos;
StlString inputBuffer;
stack<ArConsoleAttr> attr;
StlList<pair<ArConsoleAttr, StlString> > text;
queue<StlString> textStream;

if (!OSDText::unserialize(dependencies,input) ||
    !input->readUInt(readOnly) ||
    !input->readDouble(bg.r) || // Background
    !input->readDouble(bg.g) ||
    !input->readDouble(bg.b) ||
    !input->readDouble(bg.a) ||
    !input->readString(bg.font) ||
    !input->readInt(bufferSize) || // Buffers
    !input->readInt(displaySize) ||
    !input->readInt(pos) ||
    !input->readString(inputBuffer) ||
    !input->readUInt(attrSize))
  {
  return(false);
  }

// Attributes Stack
ArConsoleAttr attrib;

for(unsigned int cpt=attrSize;cpt--;)
  {
  if (!input->readDouble(attrib.r) ||
      !input->readDouble(attrib.g) ||
      !input->readDouble(attrib.b) ||
      !input->readDouble(attrib.a) ||
      !input->readString(attrib.font))
    {
    return(false);
    }
  attr.push(attrib);
  }

// Text
if (!input->readUInt(textSize))
  {
  return(false);
  }

for(unsigned int cpt=textSize;cpt--;)
  {
  StlString texte;
  if (!input->readDouble(attrib.r) ||
      !input->readDouble(attrib.g) ||
      !input->readDouble(attrib.b) ||
      !input->readDouble(attrib.a) ||
      !input->readString(attrib.font) ||
      !input->readString(texte))
    {
    return(false);
    }

  text.push_back(make_pair(attrib,texte));
  }

if (!input->readUInt(currentPos))
  {
  return(false);
  }

// TextStream

if (!input->readUInt(streamSize))
  {
  return(false);
  }
for(unsigned int cpt=streamSize;cpt--;)
  {
  StlString texte;
  if (!input->readString(texte))
    {
    return(false);
    }
  textStream.push(texte);
  }

_ro=readOnly;

_bufferSize=bufferSize;
_displaySize=displaySize;
_pos=pos;

_inputBuffer=inputBuffer;
_attr=attr;
_text=text;
_textStream=textStream;
_bg=bg;

_currentPos=_text.begin();
while(currentPos--)
  _currentPos++;

_adjustSize();
_postRedisplay();

return true;
}


void
ArConsole::_adjustSize(void) {
  while((int)_text.size() > _bufferSize) {
    if(_currentPos == _text.begin()) {
      _currentPos++;
    }
    _pos--;
    _text.pop_front();
  }

  if(_pos < 0) _pos = 0;

  while((int)_textStream.size() > _bufferSize) {
    _textStream.pop();
  }
}

void
ArConsole::_render(unsigned int width, unsigned int height) const {
  if(!_curFont) {
    return;
  }

  StlList<pair<ArConsoleAttr, StlString> >::iterator low, up;
  low = _currentPos;
  up = _currentPos;

  int min, max, padding;
  min = _pos - (_displaySize - 1) < 0 ? 0 : _pos - (_displaySize - 1);
  max = _pos;
  padding = (_displaySize - 1) > (int)_text.size() ? _text.size() - (max - min) : (_displaySize - 1) - (max - min);
  max = _pos + padding > (int)_text.size() ? _text.size() : _pos + padding;

  for(int i = 0; i < _pos - min; i++) {
    low--;
  }

  for(int i = 0; i < max - _pos; i++) {
    up++;
  }

  bool blend = glIsEnabled(GL_BLEND);
  glEnable(GL_BLEND);
  bool lighting = glIsEnabled(GL_LIGHTING);
  glDisable(GL_LIGHTING);

  glPushMatrix();
  glLoadIdentity();

  int curHeight = 0;
  LLW::BMFontID curFont;
  unsigned int ascent, descent;

  for(StlList<pair<ArConsoleAttr, StlString> >::iterator it = low;
      it != up;
      it++)
    {
    StlMap<StlString, LLW::BMFontID>::iterator itFont = _font.find(it->first.font);
    if(itFont != _font.end())
      {
      curFont = itFont->second;
      }
    else
      {
      curFont = _curFont;
      }
    LLW::getBMFontHeight(curFont, ascent, descent);
    curHeight += ascent + descent + 4;
    glColor4f(it->first.r, it->first.g, it->first.b, it->first.a);
    glRasterPos2i(5, height - curHeight);

    for(size_t i = 0; i < it->second.size(); i++)
      {
      LLW::drawBMSymbol(curFont, it->second[i]);
      }
    }
  if(!_ro) {
    LLW::getBMFontHeight(_curFont, ascent, descent);
    curHeight += ascent + descent + 4;
    glColor4f(1.f, 1.f, 1.f, 1.f);
    glRasterPos2i(5, height - curHeight);
    for(size_t i = 0; i < _inputBuffer.size(); i++) {
      LLW::drawBMSymbol(_curFont, _inputBuffer[i]);
    }
  }

  glBegin(GL_QUADS);
    glColor4f(_bg.r, _bg.g, _bg.b, _bg.a);
    glVertex2i(0, height - curHeight - 5);
    glVertex2i(width, height - curHeight - 5);
    glVertex2i(width, height);
    glVertex2i(0, height);
  glEnd();

  if(!blend) {
    glDisable(GL_BLEND);
  }

  if(lighting) {
    glEnable(GL_LIGHTING);
  }

  glPopMatrix();
}

void
ArConsole::_keyboardCB(const Renderer3D::KeyboardEvent& evt) {
  if(evt.pressed) {
    if(evt.key == _focusKey) {
      releaseKeyboardFocus(evt);
    } else if(evt.key == "Up") {
      scroll(-1);
    } else if(evt.key == "Down") {
      scroll(1);
    } else if(evt.key == "Prior") {
      scroll(-5);
    } else if(evt.key == "Next") {
      scroll(5);
    } else if(evt.key == "\x1b"     ||
	      evt.key == "Left"     ||
	      evt.key == "Right"    ||
	      evt.key == "Home"     ||
	      evt.key == "End"      ||
	      evt.key == "Begin"    ||
	      evt.key == "Insert"   ||
	      evt.key == "Delete"   ||
	      evt.key == "Shift"    ||
	      evt.key == "Control"  ||
	      evt.key == "CapsLock" ||
	      evt.key == "NumLock"  ||
	      evt.key == "F1"       ||
	      evt.key == "F2"       ||
	      evt.key == "F3"       ||
	      evt.key == "F4"       ||
	      evt.key == "F5"       ||
	      evt.key == "F6"       ||
	      evt.key == "F7"       ||
	      evt.key == "F8"       ||
	      evt.key == "F9"       ||
	      evt.key == "F10"      ||
	      evt.key == "F11"      ||
	      evt.key == "F12") {
      // Do Nothing
    } else {
      storeCharacter(evt.key[0]);
    }
  }
}

} // namespace AReVi

//----------------------------------------------------------------------------
