// LI2/ENIB - AReVi-Contrib - nurbs
//----------------------------------------------------------------------------
// file        : AReVi/Contrib/arNurbs.cpp
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/Contrib/arNurbs.h"

#include "AReVi/Utils/abstractStream.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class ArNurbs::Curve3d
//----------------------------------------------------------------------------
void
ArNurbs::Curve3d::writeToStream(ArRef<AbstractOStream> output) const
{
output->writeUInt(_degree);
output->writeUInt(_knots.size());
output->writeUInt(_ctrlPoints.size());
output->writeUInt(_weights.size());

for (unsigned i=_knots.size();i--;)
  output->writeDouble(_knots[i]);

for (unsigned i=_ctrlPoints.size();i--;)
  {
  output->writeDouble(_ctrlPoints[i].x());
  output->writeDouble(_ctrlPoints[i].y());
  output->writeDouble(_ctrlPoints[i].z());
  }

for (unsigned i=_weights.size();i--;)
  output->writeDouble(_weights[i]);
}


bool // success
ArNurbs::Curve3d::readFromStream(ArRef<AbstractIStream> input)
{
unsigned int degree;
unsigned int nbKnots,nbCtrlPoints,nbWeights;
StlVector<double> knots,weights;
StlVector<Vector3d> ctrlPoints;

if (!input->readUInt(degree) ||
    !input->readUInt(nbKnots) ||
    !input->readUInt(nbCtrlPoints) ||
    !input->readUInt(nbWeights))
    return false;

knots.resize(nbKnots);
for (unsigned i=nbKnots;i--;)
  if (!input->readDouble(knots[i]))
    return false;

ctrlPoints.resize(nbCtrlPoints);
for (unsigned i=nbCtrlPoints;i--;)
  {
  double x,y,z;
  if (!input->readDouble(x) ||
      !input->readDouble(y) ||
      !input->readDouble(z))
    return false;
  ctrlPoints[i]=Vector3d(x,y,z);
  }

weights.resize(nbWeights);
for (unsigned i=nbWeights;i--;)
  if (!input->readDouble(weights[i]))
    return false;

_degree=degree;
_knots=knots;
_weights=weights;
_ctrlPoints=ctrlPoints;
return true;
}

//----------------------------------------------------------------------------
// Class ArNurbs::Surface3d
//----------------------------------------------------------------------------
void
ArNurbs::Surface3d::writeToStream(ArRef<AbstractOStream> output) const
{
output->writeUInt(_uDegree);
output->writeUInt(_vDegree);

output->writeUInt(_uKnots.size());
output->writeUInt(_vKnots.size());
output->writeUInt(_ctrlPoints.sizeX());
output->writeUInt(_ctrlPoints.sizeY());
output->writeUInt(_weights.size()); //=> Equal to ctrlPoint.size or 0

for (unsigned i=_uKnots.size();i--;)
  output->writeDouble(_uKnots[i]);
for (unsigned i=_vKnots.size();i--;)
  output->writeDouble(_vKnots[i]);

for (unsigned i=_ctrlPoints.size();i--;)
  {
  output->writeDouble(_ctrlPoints.getData()[i].x());
  output->writeDouble(_ctrlPoints.getData()[i].y());
  output->writeDouble(_ctrlPoints.getData()[i].z());
  }

for (unsigned i=_weights.size();i--;)
  output->writeDouble(_weights.getData()[i]);
}


bool // success
ArNurbs::Surface3d::readFromStream(ArRef<AbstractIStream> input)
{
unsigned int uDegree,vDegree;
unsigned int nbUKnots,nbVKnots,nbUCtrlPoints,nbVCtrlPoints,nbWeights;
StlVector<double> uKnots,vKnots;
Tab<double> weights(0,0);
Tab<Vector3d> ctrlPoints;
//cerr << "deser" << endl;
if (!input->readUInt(uDegree) ||
    !input->readUInt(vDegree) ||
    !input->readUInt(nbUKnots) ||
    !input->readUInt(nbVKnots) ||
    !input->readUInt(nbUCtrlPoints) ||
    !input->readUInt(nbVCtrlPoints) ||
    !input->readUInt(nbWeights))
    return false;

uKnots.resize(nbUKnots);
for (unsigned i=nbUKnots;i--;)
  if (!input->readDouble(uKnots[i]))
    return false;

vKnots.resize(nbVKnots);
for (unsigned i=nbVKnots;i--;)
  if (!input->readDouble(vKnots[i]))
    return false;

ctrlPoints.resize(nbUCtrlPoints,nbVCtrlPoints);
for (unsigned i=ctrlPoints.size();i--;)
  {
  double x,y,z;
  if (!input->readDouble(x) ||
      !input->readDouble(y) ||
      !input->readDouble(z))
    return false;

  ctrlPoints.accessData()[i]=Vector3d(x,y,z);
  }

  //cerr << "nbWeight=" <<  nbWeights << endl;
if (nbWeights)
  {
    weights.resize(nbUCtrlPoints,nbVCtrlPoints);

    for (unsigned i=nbWeights;i--;)
      if (!input->readDouble(weights.accessData()[i]))
        return false;
  }

_uDegree=uDegree;
_vDegree=vDegree;
_uKnots=uKnots;
_vKnots=vKnots;
_weights=weights;
_ctrlPoints=ctrlPoints;
return true;
}

} // namespace AReVi

//----------------------------------------------------------------------------
