// LI2/ENIB - AReVi-Contrib - ArTree ???
//----------------------------------------------------------------------------
// file        : AReVi/Contrib/arTree.h
// author(s)   : T. Jourdan
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/Contrib/arTree.h"

namespace AReVi {

// --------------------------------------------------------------------
// ArTree
// --------------------------------------------------------------------

AR_CLASS_DEF(ArTree, ArObject)

ArTree::ArTree(ArCW& arCW)
: ArObject(arCW),
  _root(ArTreeNode::NEW()) {
}

ArTree::~ArTree(void) {
}

ArRef<ArTreeNode> ArTree::getRoot(void) {
  return _root;
}

void ArTree::clear(void) {
  _root->clear();
}

// --------------------------------------------------------------------
// ArTreeNode
// --------------------------------------------------------------------

AR_CLASS_DEF(ArTreeNode, ArObject)

ArTreeNode::ArTreeNode(ArCW& arCW)
: ArObject(arCW),
  _name("noname"),
  _parent(ArTreeNode::nullRef()) {
}

ArTreeNode::~ArTreeNode(void) {
}

void ArTreeNode::setName(const StlString& name) {
  _name = name;
}

const StlString& ArTreeNode::name(void) {
  return _name;
}

ArRef<ArTreeNode> ArTreeNode::parent(void) {
  return _parent;
}

ArTreeNodeIter ArTreeNode::find(ArRef<ArTreeNode> child) {
  ArTreeNodeIter it = _nodes.begin();
  do {
    if(*it == child) {
      return it;
    } else {
      it++;
    }
  } while(it != _nodes.end());
  return it;
}

size_t ArTreeNode::size(void) {
  return _nodes.size();
}

ArTreeNodeIter ArTreeNode::begin(void) {
  return _nodes.begin();
}

ArTreeNodeIter ArTreeNode::end(void) {
  return _nodes.end();
}

void ArTreeNode::insert(ArRef<ArTreeNode> child) {
  child->_parent = thisRef();
  _nodes.push_back(child);
}

bool ArTreeNode::erase(ArRef<ArTreeNode> child) {
  ArTreeNodeIter it = find(child);
  // does not compile on gcc 4.1 !!! meaning of erase()==0 ?!?!
  // return _nodes.erase(it) == 0 ? false : true;
  _nodes.erase(it);
  return true;
}

void ArTreeNode::clear(void) {
  for(StlList<ArRef<ArTreeNode> >::iterator it = _nodes.begin();
      it != _nodes.end();
      it++) {
    it->clear();
  }
  _nodes.clear();
}

// --------------------------------------------------------------------
// ArTreeLeaf
// --------------------------------------------------------------------

AR_CLASS_DEF(ArTreeLeaf, ArTreeNode)

ArTreeLeaf::ArTreeLeaf(ArCW& arCW)
: ArTreeNode(arCW) {
}

ArTreeLeaf::~ArTreeLeaf(void) {
}

void ArTreeLeaf::setData(ArRef<ArObject> data) {
  _data = data;
}

ArRef<ArObject> ArTreeLeaf::getData(void) {
  return _data;
}

void ArTreeLeaf::insert(ArRef<ArTreeNode> /* child */) {
  return;
}

} // namespace AReVi

//----------------------------------------------------------------------------
