// LI2/ENIB - AReVi-Contrib - Lua bindings
//----------------------------------------------------------------------------
// file        : AReVi/Contrib/arevi_lua_bindings.cpp
// author(s)   : D. Herviou
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
//
//  This files need the luaWrapper installed.
//
//----------------------------------------------------------------------------


#include "AReVi/arSystem.h"
#include "AReVi/arObject.h"
#include "AReVi/Lib3D/object3D.h"
#include "AReVi/Lib3D/boundingBox3D.h"
#include "AReVi/Lib3D/viewer3D.h"
#include "AReVi/Lib3D/simpleInteractor.h"
#include "AReVi/Shapes/shape3D.h"
#include "AReVi/Shapes/box3D.h"
#include "AReVi/activity.h"
#include "AReVi/Contrib/arLua.h"
#include "AReVi/Contrib/arevi_lua_bindings.h"

//Reflection
#include "AReVi/arValue.h"
#include "AReVi/arMethod.h"

#include <sys/stat.h>
#include <sys/types.h>


namespace AReVi{

//============================================================
// Class LuaExport
//============================================================

AR_CLASS_DEF(LuaExport,ArObject)


LuaExport::LuaExport(ArCW& arCW):
ArObject(arCW){
}

LuaExport::~LuaExport(){
}
//============================================================
// Class LuaInterpreter
//============================================================

AR_CLASS_DEF(LuaInterpreter,ArObject)


LuaInterpreter::LuaInterpreter(ArCW& arCW):
ArObject(arCW){
  _errorMsg=StlString("");
}

LuaInterpreter::~LuaInterpreter(){}

int
LuaInterpreter::interpret(void* L, StlString& buffer){
  _errorMsg="";
  return _doBuffer(L,buffer.c_str(),buffer.size(),NULL);
}

int
LuaInterpreter::_doBuffer(void * L,const char *buff,
			 size_t size,
			 const char *name) {
  return _aux_do(L,Lua::luaL_loadbuffer(L,buff, size, name));
}


const StlString&
LuaInterpreter::getError(){
  return _errorMsg;
}

int
LuaInterpreter::_aux_do(void* L,int status) {
  if (status == 0) {  /* parse OK? */
    status = Lua::lua_pcall(L, 0, Lua::LUA_MULTRET, 0);  /* call main */
  }

  _callalert(L,status);
  return status;
}

void
LuaInterpreter::_callalert(void* L,int status){
  if (status != 0) {
    Lua::lua_getglobal(L, "_ALERT");
    if (Lua::lua_isfunction(L, -1)) {
      Lua::lua_insert(L, -2);
      Lua::lua_call(L, 1, 0);
    }
    else if(Lua::lua_isstring(L,-2)){  /* no _ALERT function; print it on stderr */
      _errorMsg=StlString(Lua::lua_tostring(L, -2));
      Lua::lua_pop(L, 2);  /* remove error message and _ALERT */
    }
  }
}
//============================================================
// Class LuaService
//============================================================
AR_CLASS_NOVOID_DEF(LuaService,ArObject)

LuaService::LuaService(ArCW& arCW,StlString file):
ArObject(arCW),
_fileName(file),
_last_mod(0),
_isOnError(false),
_handler(false){

  init();
}

LuaService::~LuaService(){
  Lua::lua_close(_LS);
}

void
LuaService::init(){
  _LS = Lua::lua_open();   /* opens Lua */
  if(_LS){

    Lua::luaL_openlibs(_LS);

#ifdef LUA_JIT
    Lua::lua_pushcfunction(_LS, Lua::luaopen_jit);
    Lua::lua_pushstring(_LS, "luaopen_jit");
    Lua::lua_call(_LS, 1, 0);
#endif
    Lua::lua_pushcfunction(_LS,Lua::luaopen_debug);
    Lua::lua_pushstring(_LS,"luaopen_debug");
    Lua::lua_call(_LS,1,0);


    LuaExport::luaopen_arevi_lua_bindings(_LS);/* opens the default bindings */
    set<ptrOpenlib>::const_iterator it=_lib.begin();

    while(it!=_lib.end()){
      openlib(*it);
      it++;
    }
  }
}

bool
LuaService::act(ArRef<Activity> act, double dt){

//   bool recompiled = checkRecompile();
  checkRecompile();

  if (_isOnError) return true; //on error nothing to do

  //double time = ((double)clock())/CLOCKS_PER_SEC;


  //  if (recompiled){
  int top = 0;

  Lua::lua_pushstring(_LS,"onError"); //error handler
  Lua::lua_gettable(_LS,Lua::LUA_GLOBALSINDEX);

  if (Lua::lua_isfunction(_LS,-1)){
    top = Lua::lua_gettop(_LS);
    _handler = true;
    //cerr<<"LuaService::act top "<<top<<endl;
  }else{
    Lua::lua_pop(_LS,-1);
  }
  _topCalling = top;


  Lua::lua_pushstring(_LS,"run"); //must have a run function in fileName

  Lua::lua_gettable(_LS,Lua::LUA_GLOBALSINDEX);

  if(!Lua::lua_isfunction(_LS,-1)) {
    //cerr<<"no function 'run' in file "<<_fileName<<endl;
    Lua::lua_pop(_LS,-1);
    return true;
  }

    //share object with lua
    //nb1 elements push on stack (0 if fails 1 instead)  cerr<<" act : "<<_LS<<endl;

  int nb1 = LuaBind::shareArRef(_LS,thisRef());

    //add a second argument
  int nb2 =LuaBind::shareArRef(_LS,act);

    //add a third argument
  Lua::lua_pushnumber(_LS,dt);

  int nbArgSupp = addMoreArg();

  _totalArg = 1+nb1+nb2+nbArgSupp;

    //1+nb1+nb2+nbArgSupp is the number of argument push on stack
  int err = Lua::lua_pcall(_LS,_totalArg,0,_topCalling);
//    int err = Lua::lua_pcall(_LS,0,0,0);

    //Lua::lua_call(_LS,_totalArg,0);

  if(_handler){//an error function has been found
    Lua::lua_pop(_LS,-1); //the error function
  }

  checkError(err);

  // cerr<< "lua cost time : "<<getAlias()<<" dt "<<dt<<" ";
//   cerr<<" - "<<act->getInterval()<<"  "<<((double)clock())/CLOCKS_PER_SEC - time <<endl;
  return true;
}

int
LuaService::addMoreArg(void){
  //nothing to be add here
  //but when deriving from this object
  //it is possible to add more arguments
  //to the lua function
  return 0; //the number of argument added on stack here 0
}

bool
LuaService::checkRecompile(){
  struct stat buff;
  int errorCode;
  buff.st_mtime = _last_mod;
  stat(_fileName.c_str(),&buff);
  if(buff.st_mtime >  _last_mod ){

    Lua::lua_close(_LS);

    init();

#ifdef LUA_5_0
    errorCode = Lua::lua_dofile(_LS,_fileName.c_str());
#else
    errorCode = Lua::luaL_dofile(_LS,_fileName.c_str());
#endif

    _last_mod = buff.st_mtime;

    checkError(errorCode);

    return true;
  }
  else{
    return false; //no compiling
  }
}

void
LuaService::checkError(int luaCodeErr){

  StlString errMsg;

  //  int top = Lua::lua_gettop(_LS);

  if (luaCodeErr!=Lua::NONE){
    if(Lua::lua_isstring(_LS,-1)){
      errMsg = Lua::lua_tostring(_LS,-1);
    }
    cerr<<"-------# lua error #----"<<_LS<<endl<<errMsg<<endl;
    ArRef<LuaInterpreter> luaI = LuaInterpreter::NEW();
    StlString code = "print(\"traceBack : \",debug.traceback())";
    //StlString code = "debug.debug()";
    //StlString code = "info = debug.getinfo(2); print(info.name,info.namewhat,info.what,info.source,info.currentline) ";
    luaI->interpret(_LS,code);
    cerr<<"--------------------"<<endl;
    _isOnError=true;
  }else{
    _isOnError=false;
  }
}

void
LuaService::openlib(ptrOpenlib ptr){
  _lib.insert(ptr);
  ptr(_LS);
}
//============================================================
// Class LuaObject
//============================================================
AR_CLASS_NOVOID_DEF(LuaObject,ArObject)

LuaObject::LuaObject(ArCW& arCW,StlString file):
ArObject(arCW),
_fileName(file),
_act(Activity::NEW(0.04)),
_last_mod(0),
_isOnError(false){

  init();
  _act->setBehavior(thisRef(),&LuaObject::act);

}

LuaObject::~LuaObject(){
  Lua::lua_close(_LS);
}

void
LuaObject::init(){
  _LS = Lua::lua_open();   /* opens Lua */
  if(_LS){
     Lua::luaL_openlibs(_LS);
#ifdef LUA_JIT
     Lua::lua_pushcfunction(_LS, Lua::luaopen_jit);
     Lua::lua_pushstring(_LS, "luaopen_jit");
     Lua::lua_call(_LS, 1, 0);
#endif
    LuaExport::luaopen_arevi_lua_bindings(_LS);/* opens the default bindings and marshall area*/
    set<ptrOpenlib>::const_iterator it=_lib.begin();


    while(it!=_lib.end()){
      openlib(*it);
      it++;
    }
  }
}

bool
LuaObject::act(ArRef<Activity> act, double dt){

  checkRecompile();

  if (_isOnError) return true; //on error nothing to do

  Lua::lua_pushstring(_LS,"run"); //must have a run function in fileName

  Lua::lua_gettable(_LS,Lua::LUA_GLOBALSINDEX);


  if(!Lua::lua_isfunction(_LS,-1)) {
    //cerr<<"no function 'run' in file "<<_fileName<<endl;
     Lua::lua_pop(_LS,-1);
     return true;
  }

  //share object with lua
  //nb1 elements push on stack (0 if fails 1 instead)
  int nb1 = LuaBind::shareArRef(_LS,thisRef());

  //add a second argument
  int nb2 =LuaBind::shareArRef(_LS,act);

  //add a third argument
  Lua::lua_pushnumber(_LS,dt);

  int nbArgSupp = addMoreArg();

  //1+nb1+nb2+nbArgSupp is the number of argument push on stack
  int err = Lua::lua_pcall(_LS,1+nb1+nb2+nbArgSupp,0,0);

  checkError(err);

  return true;
}

int
LuaObject::addMoreArg(void){
  //nothing to be add here
  //but when deriving from this object
  //it is possible to add more arguments
  //to the lua function
  return 0; //the number of argument added on stack here 0
}

bool
LuaObject::checkRecompile(){
  struct stat buff;
  int errorCode;
  buff.st_mtime = _last_mod;
  stat(_fileName.c_str(),&buff);
  if(buff.st_mtime >  _last_mod ){

    Lua::lua_close(_LS);

    init();

#ifdef LUA_5_0
    errorCode = Lua::lua_dofile(_LS,_fileName.c_str());
#else
    errorCode = Lua::luaL_dofile(_LS,_fileName.c_str());
#endif

    _last_mod = buff.st_mtime;

    checkError(errorCode);

    return true;
  }
  else{
    return false; //no compiling
  }
}

void
LuaObject::checkError(int luaCodeErr){

  StlString errMsg;

  int top = Lua::lua_gettop(_LS);

  if (luaCodeErr!=Lua::NONE){
    if(Lua::lua_isstring(_LS,-1)){
      errMsg = Lua::lua_tostring(_LS,-1);
    }
    cerr<<"-------# lua error #----"<<endl<<errMsg<<endl;
    ArRef<LuaInterpreter> luaI = LuaInterpreter::NEW();
    //    StlString code = "print(\"traceBack : \",debug.traceback())";
    StlString code = "info = debug.debug(2) \n print(info.name,info.namewhat,info.what,info.source,info.currentline) ";
    luaI->interpret(_LS,code);
    cerr<<"--------------------"<<endl;
    _isOnError=true;
  }else{
    _isOnError=false;
  }

  switch(luaCodeErr){

  case Lua::ERRRUN:
    cerr<<"lua runtime error : "<<errMsg<<endl;
    Lua::lua_pop(_LS,top);
    break;
  case Lua::ERRMEM:
    cerr<<"lua mem error : "<<errMsg<<endl;
    Lua::lua_error(_LS);
    Lua::lua_pop(_LS,top);
    break;
  case Lua::ERRERR:
    cerr<<"lua handler error : "<<errMsg<<endl;
    Lua::lua_error(_LS);
    Lua::lua_pop(_LS,top);
    break;
  default:
    break;
  }

}

void
LuaObject::openlib(ptrOpenlib ptr){
  _lib.insert(ptr);
  ptr(_LS);
}





//============================================================
//============================================================
//WRAP DES FONCTIONS
//============================================================
//============================================================


//============================================================
// Add On for Lua
//============================================================
int arevi_lua_addOn_doPackage(void * L){
  unsigned int nbArg = Lua::lua_gettop(L);
  if(nbArg>2){
    Lua::luaL_error(L,"addOn_doPackage error, %d arg instead of 1",nbArg);
    return 0;
  }else if(!Lua::lua_isstring(L,-1)){
    Lua::luaL_error(L,"addOn_doPackage error, need a string has argument",nbArg);
    return 0;
  }
  else{
    ArRef<LuaInterpreter> interpreter = LuaInterpreter::NEW();
    StlString buffer = " \
          local P = {}   -- package \
          if _REQUIREDNAME == nil then " +
              StlString(Lua::lua_tostring(L,-1)) + "= P \
          else \
              _G[_REQUIREDNAME] = P \
          end ";
      if(interpreter->interpret(L,buffer)){
	cerr<<"error while parsing buffer in doPackage"<<endl;
	cerr<<"error is : "<<interpreter->getError()<<endl;
      }
    return 0;
  }
}

//simple clone for a table
//doesn't work if table contains loop
int arevi_lua_addOn_cloneTable(void *L){
  int top = Lua::lua_gettop(L);

  CHECK_LUA_ARG(L,table,top) //top of the stack must be a table
  //stack : table

  Lua::lua_newtable(L);
  //stack : table - cloneTable

  Lua::lua_pushnil(L); //first key
  //stack : table - cloneTable - table - key

  cerr<<"coucou"<<endl;
  while (Lua::lua_next(L, top) != 0) { // table to be cloned is at index 1
    //stack : table - cloneTable - key - value

    Lua::lua_pushvalue(L,-2);
    //stack : table - cloneTable - key - value -key

    Lua::lua_pushvalue(L,-2);
    //stack : table - cloneTable - key - value - key -value

//     if (Lua::lua_type(L,-1)==Lua::LUA_TTABLE){ //if the value is a table clone it

//       arevi_lua_addOn_cloneTable(L);
//       //stack : table - cloneTable - key - value - key - value - valueCloned

//       Lua::lua_remove(L,-2);
//       //stack : table - cloneTable - key - value - key - valueCloned

//     }
//     else{
//       //nothing to do
//     }

    Lua::lua_settable(L,top+1);
    //stack : table - cloneTableUpdate - key - value

    Lua::lua_pop(L, 1);  /* removes `value'; keeps `key' for next iteration */
    //stack : table - cloneTableUpdate - key

  }

  Lua::lua_pop(L,1);
  //stack : table - cloneTableUpdate

  return 1; //one element added on stack

}



//============================================================
// AReVi utilities
//============================================================


//todo : take into consideration the
//function that return something
//some other function as create or destroy

int arevi_lua_aReVi_invoke(void* L){
  unsigned int nbArg = Lua::lua_gettop(L);

  if(nbArg<3){
    cerr<<"must use invoke with a minimum of 3 arguments"<<endl;
    return 0;
  }
  else{
    ArRef<ArObject> obj;
    StlString mthToCall("");
    if(!Lua::lua_isuserdata(L,1)){
      cerr<<"must use an ArObject in arg 1"<<endl;
      return 0;
    }
    else if(Lua::lua_isuserdata(L,1)){
      obj = LuaBind::retrieveArRef(L,1);
    }
    if(!Lua::lua_isstring(L,2)){
      cerr<<"must use a  AReVi string that defines a method in arg 2 "<<endl;
      return 0;
    }
    else{
      mthToCall=Lua::lua_tostring(L,2);
    }

    StlVector<ArRef<ArValue> > args;
    StlVector<ArConstRef<ArMethod> > methodsOut;

    obj->getClass()->findMethods(methodsOut,mthToCall);
    //    cerr<<"method find for "<<mthToCall<<endl;
    for(unsigned int j = 0; j<methodsOut.size();j++){
      //cerr<<j+1<<"eme mth found has signature :"<<endl;
      for(unsigned int k = 0; k<methodsOut[j]->getNbArgs();k++){
	//cerr<<methodsOut[j]->getArgType(k)->getIndirectType()<<" ";
      }
      //cerr<<" |||| ";
      for(unsigned int k = 0; k<methodsOut[j]->getNbArgs();k++){
	//cerr<<methodsOut[j]->getArgType(k)->getDirectType()<<" ";
      }
      //cerr<<endl;
    }
    //cerr<<endl;

    if(methodsOut.empty()){
      cerr<<"no method "<<mthToCall<<" on "<<obj->getAlias()<<endl;
      return 0;
    }

    for(unsigned int i=3; i<=nbArg; i++){

      if(Lua::lua_isuserdata(L,i)){
	ArRef<ArObject> o = LuaBind::retrieveArRef(L,i);
	ArType::IndirectType it = methodsOut[0]->getArgType(i-3)->getIndirectType();
	args.push_back(ArValue::NEW(it,o->getClass(),0));
	if(it==ArType::IT_REF){
	  args.back()->accessRef()=o;
	}else if(it==ArType::IT_CONST_REF){
	  args.back()->accessConstRef()=o;
	}else if(it==ArType::IT_PTR){
	  args.back()->accessPtr()=o;
	}else if(it==ArType::IT_CONST_PTR){
	  args.back()->accessConstPtr()=o;
	}
	//cerr<<i<<"eme arg is a userdata"<<endl;
      }else if(Lua::lua_isnone(L,i)){
	//cerr<<"none"<<endl;
      }else if(Lua::lua_isnil(L,i)){
	//cerr<<"nil"<<endl;
      }else if(Lua::lua_istable(L,i)){
	//cerr<<"table"<<endl;
      }else if(Lua::lua_isfunction(L,i)){
	//cerr<<"function"<<endl;
      }else if(Lua::lua_isnumber(L,i)){
	double d = Lua::lua_tonumber(L,i);
	ArType::DirectType dt = methodsOut[0]->getArgType(i-3)->getDirectType();
	args.push_back(ArValue::NEW(dt,0));
 	if(dt==ArType::DT_INT){
	  args.back()->accessInt()=(int)d;
	}else if(dt==ArType::DT_FLOAT){
	  args.back()->accessFloat()=(float)d;
	}else if(dt==ArType::DT_DOUBLE){
	  args.back()->accessDouble()=(double)d;
	}else if(dt==ArType::DT_BOOL){
	  args.back()->accessBool()=(bool)d;
	}
	//cerr<<i<<"eme arg is a number"<<endl;
      }else if(Lua::lua_isstring(L,i)){
	StlString s(Lua::lua_tostring(L,i));
	args.push_back(ArValue::NEW(ArType::DT_STRING,0));
	args.back()->accessString()=s;
	//cerr<<i<<"eme arg is a string"<<endl;
      }
    }

  ArConstRef<ArMethod> mth=obj->getClass()->findMethod(mthToCall,args);
  ArRef<ArValue> result;
  if(!mth) return 1;
  else{
    if(mth->getResultType())
      {
	result=ArValue::NEW(mth->getResultType());
      }

    StlString mthName=mth->getDeclarationClass()->getName()
      +"::"+mth->getName();
    if(mth->invoke(obj,result,args))        {
      //cerr << "Invocation of `" << mthName << "' done" << endl;
    }
    else {
      //cerr << "Cannot invoke `" << mthName << "' !" << endl;
    }
    if(result){


    }

  }


  }
  return 0;
}

//============================================================
// ArClass
//============================================================


int arevi_lua_arClass_getInstances(void* L){
  CHECK_LUA_ARG(L,string,1)

  StlString clssName = Lua::lua_tostring(L,-1);
  ArConstRef<ArClass> cls = ArClass::find(clssName);

  if(!cls) {
    Lua::luaL_error(L,"no class %s in AReVi",clssName.c_str());
    return 0;
  }

  StlVector<ArRef<ArObject> > instances;
  cls->getInstances(instances,true);
  LuaBind::shareVectorArRef(L,instances);
  return 1;
}

int arevi_lua_arClass_getObject(void* L){
  CHECK_LUA_ARG(L,string,1)

  StlString objName = Lua::lua_tostring(L,1);
  ArRef<ArObject> instanceOut;
  ArObject::CLASS()->findAlias(objName,instanceOut);

  if(instanceOut){
    return LuaBind::shareArRef(L,instanceOut/*,false*/);
  }
  else{
    StlString error = StlString("no object")+objName+StlString(" in AReVi");
    Lua::luaL_error(L,error.c_str());
    return 0;
  }
}


//============================================================
// ArObject
//============================================================

int arevi_lua_arObject_new(void* L){
  ArRef<ArObject> b = ArObject::NEW();
  int nb = LuaBind::shareArRef(L,b); //push one element on stack
  return nb;
}

int arevi_lua_arObject_getAlias(void* L){
  CHECK_RETRIEVE_ARREF(L,arObject,ArObject,1)

  StlString s = arObject->getAlias();
  Lua::lua_pushstring(L,s.c_str());
  return 1;
}

int arevi_lua_arObject_getNbInstances(void* L){
  CHECK_RETRIEVE_ARREF(L,arObject,ArObject,1)

  int nb = arObject->getClass()->getNbInstances(true);
  Lua::lua_pushnumber(L,nb);
  return 1;
}


int arevi_lua_arObject_isA(void* L){
  CHECK_RETRIEVE_ARREF(L,obj,ArObject,1)
  CHECK_LUA_ARG(L,string,2)

  StlString clssName = Lua::lua_tostring(L,-1);
  ArConstRef<ArClass> cls = ArClass::find(clssName);

  bool isA = obj->getClass()->isA(cls);
  Lua::lua_pushboolean(L,isA);
  return 1;
}

int arevi_lua_arObject_getClassName(void* L){
  CHECK_RETRIEVE_ARREF(L,obj,ArObject,1)

  StlString name(obj->getClass()->getName());

  Lua::lua_pushstring(L,name.c_str());
  return 1;

}

//============================================================
// LuaObject
//============================================================
int arevi_lua_luaObject_new(void *L){
  CHECK_LUA_ARG(L,string,2)

  StlString filename = Lua::lua_tostring(L,2);

  ArRef<LuaObject> lo = LuaObject::NEW(filename);

  return LuaBind::shareArRef(L,lo);
}

//============================================================
// BoundingBox
//============================================================
int arevi_lua_boundingBox3D_new(void* L){
  ArRef<BoundingBox3D> bbox = BoundingBox3D::NEW();
  return LuaBind::shareArRef(L,bbox);
}

int arevi_lua_boundingBox3D_getSize(void* L){
  CHECK_RETRIEVE_ARREF(L,bbox,BoundingBox3D,1)
  double xs,ys,zs;
  if(bbox->getSize(xs,ys,zs)){
    Lua::lua_pushnumber(L,xs);
    Lua::lua_pushnumber(L,ys);
    Lua::lua_pushnumber(L,zs);
    return 3;
  }else{
    return 0;
  }
}

int arevi_lua_boundingBox3D_getCenter(void* L){
  CHECK_RETRIEVE_ARREF(L,bbox,BoundingBox3D,1)
  double xc,yc,zc;
  if(bbox->getCenter(xc,yc,zc)){
    Lua::lua_pushnumber(L,xc);
    Lua::lua_pushnumber(L,yc);
    Lua::lua_pushnumber(L,zc);
    return 3;
  }else{
    return 0;
  }
}

int arevi_lua_boundingBox3D_getBounds(void* L){
  CHECK_RETRIEVE_ARREF(L,bbox,BoundingBox3D,1)
  double xmin,ymin,zmin,xmax,ymax,zmax;
  if(bbox->getBounds(xmin,ymin,zmin,xmax,ymax,zmax)){
    Lua::lua_pushnumber(L,xmin);
    Lua::lua_pushnumber(L,ymin);
    Lua::lua_pushnumber(L,zmin);
    Lua::lua_pushnumber(L,xmax);
    Lua::lua_pushnumber(L,ymax);
    Lua::lua_pushnumber(L,zmax);
    return 6;
  }else{
    return 0;
  }
}


//============================================================
// Base3D
//============================================================

int arevi_lua_base3D_new(void* L){
  ArRef<Base3D> b = Base3D::NEW();

  return LuaBind::shareArRef(L,b);
}


int arevi_lua_base3D_setLocation(void* L){
  CHECK_RETRIEVE_ARREF(L,base,Base3D,1)
  CHECK_RETRIEVE_ARREF(L,destination,Base3D,2)

  base->setLocation(destination);
  return 0;
}

int arevi_lua_base3D_globalToLocalPosition(void* L){
  CHECK_RETRIEVE_ARREF(L,base,Base3D,1)
  CHECK_LUA_ARG(L,number,2)
  CHECK_LUA_ARG(L,number,3)
  CHECK_LUA_ARG(L,number,4)

  double x = Lua::lua_tonumber(L,2);
  double y = Lua::lua_tonumber(L,3);
  double z = Lua::lua_tonumber(L,4);
  base->globalToLocalPosition(x,y,z);
  Lua::lua_pushnumber(L,x);
  Lua::lua_pushnumber(L,y);
  Lua::lua_pushnumber(L,z);
  return 3;
}

int arevi_lua_base3D_localToGlobalPosition(void* L){
  CHECK_RETRIEVE_ARREF(L,base,Base3D,1)
  CHECK_LUA_ARG(L,number,2)
  CHECK_LUA_ARG(L,number,3)
  CHECK_LUA_ARG(L,number,4)

  double x = Lua::lua_tonumber(L,2);
  double y = Lua::lua_tonumber(L,3);
  double z = Lua::lua_tonumber(L,4);

  base->localToGlobalPosition(x,y,z);

  Lua::lua_pushnumber(L,x);
  Lua::lua_pushnumber(L,y);
  Lua::lua_pushnumber(L,z);
  return 3;
}

int arevi_lua_base3D_globalToLocalOrientation(void* L){
  CHECK_RETRIEVE_ARREF(L,base,Base3D,1)
  CHECK_LUA_ARG(L,number,2)
  CHECK_LUA_ARG(L,number,3)
  CHECK_LUA_ARG(L,number,4)

  double x = Lua::lua_tonumber(L,2);
  double y = Lua::lua_tonumber(L,3);
  double z = Lua::lua_tonumber(L,4);

  base->globalToLocalOrientation(x,y,z);

  Lua::lua_pushnumber(L,x);
  Lua::lua_pushnumber(L,y);
  Lua::lua_pushnumber(L,z);
  return 3;
}

int arevi_lua_base3D_localToGlobalOrientation(void* L){
  CHECK_RETRIEVE_ARREF(L,base,Base3D,1)
  CHECK_LUA_ARG(L,number,2)
  CHECK_LUA_ARG(L,number,3)
  CHECK_LUA_ARG(L,number,4)

  double x = Lua::lua_tonumber(L,2);
  double y = Lua::lua_tonumber(L,3);
  double z = Lua::lua_tonumber(L,4);

  base->localToGlobalOrientation(x,y,z);

  Lua::lua_pushnumber(L,x);
  Lua::lua_pushnumber(L,y);
  Lua::lua_pushnumber(L,z);
  return 3;
}


int arevi_lua_base3D_getPosition(void* L){
  CHECK_RETRIEVE_ARREF(L,base,Base3D,1)

  double x,y,z;

  base->getPosition(x,y,z);

  Lua::lua_pushnumber(L,x);
  Lua::lua_pushnumber(L,y);
  Lua::lua_pushnumber(L,z);
  return 3;
}
int arevi_lua_base3D_setPosition(void* L){
  CHECK_RETRIEVE_ARREF(L,base,Base3D,1)
  CHECK_LUA_ARG(L,number,2)
  CHECK_LUA_ARG(L,number,3)
  CHECK_LUA_ARG(L,number,4)

  double x = Lua::lua_tonumber(L,2);
  double y = Lua::lua_tonumber(L,3);
  double z = Lua::lua_tonumber(L,4);

  base->setPosition(x,y,z);

  return 0;
}


int arevi_lua_base3D_extractOrientation(void* L){
  CHECK_RETRIEVE_ARREF(L,base,Base3D,1)

  double r,p,y;

  base->extractOrientation(r,p,y);

  Lua::lua_pushnumber(L,r);
  Lua::lua_pushnumber(L,p);
  Lua::lua_pushnumber(L,y);
  return 3;
}

int arevi_lua_base3D_setOrientation(void* L){
  CHECK_RETRIEVE_ARREF(L,base,Base3D,1)
  CHECK_LUA_ARG(L,number,2)
  CHECK_LUA_ARG(L,number,3)
  CHECK_LUA_ARG(L,number,4)

  double r = Lua::lua_tonumber(L,2);
  double p = Lua::lua_tonumber(L,3);
  double y = Lua::lua_tonumber(L,4);
  base->setOrientation(r,p,y);
  return 0;
}


int arevi_lua_base3D_translate(void* L){
  CHECK_RETRIEVE_ARREF(L,base,Base3D,1)
  CHECK_LUA_ARG(L,number,2)
  CHECK_LUA_ARG(L,number,3)
  CHECK_LUA_ARG(L,number,4)

  double x = Lua::lua_tonumber(L,2);
  double y = Lua::lua_tonumber(L,3);
  double z = Lua::lua_tonumber(L,4);

  base->translate(x,y,z);

  return 0;
}


int arevi_lua_base3D_yaw(void* L){
  CHECK_RETRIEVE_ARREF(L,base,Base3D,1)
  CHECK_LUA_ARG(L,number,2)

  double x=Lua::lua_tonumber(L,2);

  base->yaw(x);

  return 0;
}

int arevi_lua_base3D_roll(void* L){
  CHECK_RETRIEVE_ARREF(L,base,Base3D,1)
  CHECK_LUA_ARG(L,number,2)

  double x=Lua::lua_tonumber(L,2);

  base->roll(x);

  return 0;
}

int arevi_lua_base3D_pitch(void* L){
  CHECK_RETRIEVE_ARREF(L,base,Base3D,1)
  CHECK_LUA_ARG(L,number,2)

  double x=Lua::lua_tonumber(L,2);

  base->pitch(x);

  return 0;
}

int arevi_lua_base3D_attachTo(void* L){
  CHECK_RETRIEVE_ARREF(L,base,Base3D,1)
  CHECK_RETRIEVE_ARREF(L,baseToAttachTo,Base3D,2)

  CHECK_LUA_ARG(L,number,3)

  int attachment = (int)Lua::lua_tonumber(L,3);

  base->attachTo(baseToAttachTo,attachment);

  return 0;
}

//============================================================
// Window3D
//============================================================

int arevi_lua_window3D_setRenderSize(void* L){
  CHECK_RETRIEVE_ARREF(L,w,Window3D,1)
  CHECK_LUA_ARG(L,number,2)
  CHECK_LUA_ARG(L,number,3)
  w->setRenderSize((unsigned int)Lua::lua_tonumber(L,2),(unsigned int)Lua::lua_tonumber(L,3));
  return 0;
}

//============================================================
// Viewer3D
//============================================================
int arevi_lua_viewer3D_new(void* L){
  ArRef<Viewer3D> v = Viewer3D::NEW();
  return LuaBind::shareArRef(L,v);
}

int arevi_lua_viewer3D_setFarDistance(void *L){
  CHECK_RETRIEVE_ARREF(L,v,Viewer3D,1)
  CHECK_LUA_ARG(L,number,2)

  double far = Lua::lua_tonumber(L,2);

  v->setFarDistance(far);

  return 0;
}

int arevi_lua_viewer3D_setNearDistance(void *L){
  CHECK_RETRIEVE_ARREF(L,v,Viewer3D,1)
  CHECK_LUA_ARG(L,number,2)

  double near = Lua::lua_tonumber(L,2);

  v->setNearDistance(near);

  return 0;
}

int arevi_lua_viewer3D_setFov(void *L){
  CHECK_RETRIEVE_ARREF(L,v,Viewer3D,1)
  CHECK_LUA_ARG(L,number,2)

  double fov = Lua::lua_tonumber(L,2);
  v->setFieldOfView(fov);
  return 0;
}

//============================================================
// Object3D
//============================================================

int arevi_lua_object3D_new(void* L){
  ArRef<Object3D> b = Object3D::NEW();
  int nb = LuaBind::shareArRef(L,b);
  return nb;
}

int arevi_lua_object3D_computeBoundingBox(void *L){
  CHECK_RETRIEVE_ARREF(L,obj,Object3D,1)
  CHECK_RETRIEVE_ARREF(L,bbox,BoundingBox3D,2)

  obj->computeBoundingBox(bbox,obj);

  return 0;
}

int arevi_lua_object3D_setWireFrame(void *L){
  CHECK_RETRIEVE_ARREF(L,obj,Object3D,1)
  CHECK_LUA_ARG(L,boolean,2)

  obj->setWireFrame(Lua::lua_toboolean(L,2));
  return 0;
}

int arevi_lua_object3D_getWireFrame(void *L){
  CHECK_RETRIEVE_ARREF(L,obj,Object3D,1)
  Lua::lua_pushboolean(L,obj->getWireFrame());
  return 1;
}

int arevi_lua_object3D_testCollision(void* L){
  CHECK_RETRIEVE_ARREF(L,object,Object3D,1)
  CHECK_RETRIEVE_ARREF(L,target,Object3D,2)
  CHECK_LUA_ARG(L,number,3)
  CHECK_RETRIEVE_ARREF(L,viewPoint,Object3D,4)

  Lua::lua_pushboolean(L,object->testCollision(target,(unsigned int)Lua::lua_tonumber(L,3),viewPoint));
  return 1;
}

//============================================================
// Activity
//============================================================

int arevi_lua_activity_setInterval(void* L){
  CHECK_RETRIEVE_ARREF(L,act,Activity,1)
  CHECK_LUA_ARG(L,number,2)

  double interval = Lua::lua_tonumber(L,2);
  act->setInterval(interval);
  return 0;
}

int arevi_lua_activity_getInterval(void* L){
  CHECK_RETRIEVE_ARREF(L,act,Activity,1)

  double interval =  act->getInterval();
  Lua::lua_pushnumber(L,interval);
  return 1;
}

int arevi_lua_activity_setPriority(void* L){
  CHECK_RETRIEVE_ARREF(L,act,Activity,1)
  CHECK_LUA_ARG(L,number,2)

  int priority = (int)Lua::lua_tonumber(L,2);
  act->setPriority(priority);
  return 0;
}

int arevi_lua_activity_getPriority(void* L){
  CHECK_RETRIEVE_ARREF(L,act,Activity,1)

  int priority =  act->getPriority();
  Lua::lua_pushnumber(L,priority);
  return 1;
}

int arevi_lua_activity_setSuspended(void* L){
  CHECK_RETRIEVE_ARREF(L,act,Activity,1)
  CHECK_LUA_ARG(L,number,2)

  int suspended = (int)Lua::lua_tonumber(L,2);

  act->setSuspended(suspended);
  return 0;
}

int arevi_lua_activity_isSuspended(void* L){
  CHECK_RETRIEVE_ARREF(L,act,Activity,1)

  int suspended = (int)act->isSuspended();

  Lua::lua_pushnumber(L,suspended);
  return 1;
}

int arevi_lua_activity_getTime(void* L){
  CHECK_RETRIEVE_ARREF(L,act,Activity,1)

  double time =  act->getTime();
  Lua::lua_pushnumber(L,time);
  return 1;
}


//============================================================
//============================================================
// DECLARATION DES STRUCTURES
//============================================================
//============================================================
//============================================================
// addOn for Lua
//============================================================

const arluaL_reg arevi_lua_addOn_function[]={
  {"doPackage",arevi_lua_addOn_doPackage},
  {"cloneTable",arevi_lua_addOn_cloneTable},
  {NULL,NULL}
};

//============================================================
// AReVi
//============================================================
const arluaL_reg arevi_lua_aReVi_function[]={
  {"invoke",arevi_lua_aReVi_invoke},
  {NULL,NULL}
};


//============================================================
// ArClass
//============================================================
const arluaL_reg arevi_lua_arClass_function[]={
  {"getInstances",arevi_lua_arClass_getInstances},
  {"getObject",arevi_lua_arClass_getObject},
  {NULL,NULL}
};

//============================================================
// ArObject
//============================================================

const arluaL_reg arevi_lua_arObject_method[]={
  {"getAlias",arevi_lua_arObject_getAlias},
  {"getNbInstances",arevi_lua_arObject_getNbInstances},
  {"isA",arevi_lua_arObject_isA},
  {"getClassName",arevi_lua_arObject_getClassName},
  {NULL,NULL}
};

const arluaL_reg arevi_lua_arObject_function[]={
  {"new_arObject",arevi_lua_arObject_new},
  {NULL,NULL}
};

//============================================================
// BoundingBox3D
//============================================================
const arluaL_reg arevi_lua_boundingBox3D_method[]={
  {"getSize",arevi_lua_boundingBox3D_getSize},
  {"getBounds",arevi_lua_boundingBox3D_getBounds},
  {"getCenter",arevi_lua_boundingBox3D_getCenter},
  {NULL,NULL}
};

const arluaL_reg arevi_lua_boundingBox3D_function[]={
  {"new",arevi_lua_boundingBox3D_new},
  {NULL,NULL}
};
//============================================================
// Base3D
//============================================================

const arluaL_reg arevi_lua_base3D_method[]={
  {"roll",arevi_lua_base3D_roll},
  {"pitch",arevi_lua_base3D_pitch},
  {"yaw",arevi_lua_base3D_yaw},
  {"translate",arevi_lua_base3D_translate},
  {"getPosition",arevi_lua_base3D_getPosition},
  {"setPosition",arevi_lua_base3D_setPosition},
  {"extractOrientation",arevi_lua_base3D_extractOrientation},
  {"setOrientation",arevi_lua_base3D_setOrientation},
  {"localToGlobalPosition",arevi_lua_base3D_localToGlobalPosition},
  {"globalToLocalPosition",arevi_lua_base3D_globalToLocalPosition},
  {"localToGlobalOrientation",arevi_lua_base3D_localToGlobalOrientation},
  {"globalToLocalOrientation",arevi_lua_base3D_globalToLocalOrientation},
  {"attachTo",arevi_lua_base3D_attachTo},
  {"setLocation",arevi_lua_base3D_setLocation},
  {NULL,NULL}
};

const arluaL_reg arevi_lua_base3D_function[]={
  {"new",arevi_lua_base3D_new},
  {NULL,NULL}
};

//============================================================
// Window3D
//============================================================
const arluaL_reg arevi_lua_window3D_method[]={
  {"setRenderSize",arevi_lua_window3D_setRenderSize},
  {NULL,NULL}
};

const arluaL_reg arevi_lua_window3D_function[]={
  {NULL,NULL}
};

//============================================================
// Viewer3D
//============================================================
const arluaL_reg arevi_lua_viewer3D_method[]={
  {"setFarDistance",arevi_lua_viewer3D_setFarDistance},
  {"setNearDistance",arevi_lua_viewer3D_setNearDistance},
  {"setFov",arevi_lua_viewer3D_setFov},
  {NULL,NULL}
};

const arluaL_reg arevi_lua_viewer3D_function[]={
  {"new",arevi_lua_viewer3D_new},
  {NULL,NULL}
};

//============================================================
// Object3D
//============================================================
const arluaL_reg arevi_lua_object3D_method[]={
  {"computeBoundingBox",arevi_lua_object3D_computeBoundingBox},
  {"setWireFrame",arevi_lua_object3D_setWireFrame},
  {"getWireFrame",arevi_lua_object3D_getWireFrame},
  {"testCollision",arevi_lua_object3D_testCollision},
  {NULL,NULL}
};

const arluaL_reg arevi_lua_object3D_function[]={
  {"new",arevi_lua_object3D_new},
  {NULL,NULL}
};


//============================================================
// Activity
//============================================================

const arluaL_reg arevi_lua_activity_method[]={
  {"setInterval",arevi_lua_activity_setInterval},
  {"getInterval",arevi_lua_activity_getInterval},
  {"setPriority",arevi_lua_activity_setPriority},
  {"getPriority",arevi_lua_activity_getPriority},
  {"getTime",arevi_lua_activity_getTime},
  {"isSuspended",arevi_lua_activity_isSuspended},
  {"setSuspended",arevi_lua_activity_setSuspended},
  {NULL,NULL}
};

const arluaL_reg arevi_lua_activity_function[]={
  {NULL,NULL}
};

//============================================================
// LuaObject
//============================================================
const arluaL_reg arevi_lua_luaObject_method[]={
  {NULL,NULL}
};

const arluaL_reg arevi_lua_luaObject_function[]={
  {"new",arevi_lua_luaObject_new},
  {NULL,NULL}
};


int LuaExport::luaopen_arevi_lua_bindings(void* L){
   /* stack : input status */
  //LuaBind::createMarshallArea(L);


   //========================================
   // Lua::lua_pushcfunction(L, arevi_lua_addOn_doPackage); //register only doPackage function
//    Lua::lua_setglobal(L, "doPackage");
   Lua::luaL_openlib(L,"AddOn",arevi_lua_addOn_function, 0); //register addOn function

   //========================================

   Lua::luaL_openlib(L,"ArClass",arevi_lua_arClass_function, 0); //register as arClass

   //========================================

   Lua::luaL_openlib(L,"AReVi",arevi_lua_aReVi_function, 0); //register as arObject

   //========================================
   //Must declare class in Hierarchical order from Mother to child
   //========================================
   LuaBind::exportClass(L,"ArObject",
 		       arevi_lua_arObject_method,
 		       arevi_lua_arObject_function);

   //========================================
   LuaBind::exportClass(L,"BoundingBox3D",
 		       arevi_lua_boundingBox3D_method,
 		       arevi_lua_boundingBox3D_function);

   //========================================
   LuaBind::exportClass(L,"LuaObject",
 		       arevi_lua_luaObject_method,
 		       arevi_lua_luaObject_function);

   //========================================
   LuaBind::exportClass(L,"Base3D",
 		       arevi_lua_base3D_method,
 		       arevi_lua_base3D_function);

 //========================================
   LuaBind::exportClass(L,"Window3D",
 		       arevi_lua_window3D_method,
 		       arevi_lua_window3D_function);

   //========================================
   LuaBind::exportClass(L,"Viewer3D",
 		       arevi_lua_viewer3D_method,
 		       arevi_lua_viewer3D_function);

   //========================================
   LuaBind::exportClass(L,"Object3D",
 		       arevi_lua_object3D_method,
 		       arevi_lua_object3D_function);

   //========================================
   LuaBind::exportClass(L,"Activity",
 		       arevi_lua_activity_method,
 		       arevi_lua_activity_function);

   //LuaDebug::dumpLuaStack(L);
   //   cerr<<endl<<"=========end stack "<<endl;

   // cerr<<"=========================================="<<endl;
   return 0;
}


}//namespace AReVi




//************************************************************
// EXPLANATION NOTES : why doing so ?
//************************************************************
/*
we would like to write lua like this:

  b = Base3D:new()
  b:translate(10,0,0)
  x,y,_=b:getPosition()

but b is a user data create consequently doesn't have
a metatable (see Lua book). To obtain a metatable for a userdata
we have to allocate memory with luanewuserdata(lua_State*) that
return a memory block adress. As we want to share object create in
AReVi one solution offers is to create a structure with a field
that is a pointer to an "ArRef<>.c_ptr". Consequently, when allocating
memory with lua_newuserdata, we allocate this structure. Next, we
set the field of the structure to the c_ptr() to our sharing object.
Now the sharing structure have a metatable and embed a pointer to
our data.

to write thinks in object style (ie b:translate(10,0,0)) we have
to do declaration in C++ in a certain way.
b:translate(b,x,y,z) is equivalent to b.translate(b,x,y,z)
but b evaluate only once. So, lua checks if b have an object method
translate (b doesn't have object method because it is a user data)
so lua checks in the field  __index of its metatable and find that
the metatable for the object is itself. Consequently
the search is done is this metatable and lua find a function translate
in this "Base3D" metatable thus:
  b.translate(b,10,0,0) become Base3D.translate(b,10,0,0)


As our objects shared with lua have metatable they can be manage
by the garbage collector of lua. To do so, we have to had a __gc
field to the metatable, this event is trigger when the garbage decides
to liberate the memory occupied by this object.


A sort of inheritance can also be done. Assume, we have a Base3D
which is a child for ArObject. Actually, all method of ArObject will
be visible in Base3D (unless they are private). The mecanism to such
an inherance in lua is to fill the metatable of Base3D with is own
method but also with the method of ArObject.
That is done when registering the ''class'' Base3D:

  luaL_newmetatable(L,"Base3D"); //create a metatable for Base3D the one in which the lookup will be done
  int metatable = lua_gettop(L); //get created table index on stack

  lua_pushstring(L,__index); //push index on top of the stack
  lua_pushvalue(L,metatable); //push metatable on top of the stack
  lua_setvalue(L,metatable); //set metatable.__index = metatable

  luaL_openlib(L,NULL,arevi_lua_arObject_method, 0); //fill metatable with method of ArObject
  luaL_openlib(L,NULL,arevi_lua_Base3D_method, 0); //fill metatable with method of Base3D
  luaL_openlib(L,"Base3D",arevi_lua_Base3D_function,0); //create table for Base3D (we can create object with Base3D:new() )


In a nutshell to bind an object : we need to encapsulate data
in a structure allocate by lua. We need a desallocation mecanism
if we want garbage collection. We have to register our methods
from C++.

Why a cg boolean in the shared structured:
as explain above we need metatable for our objects consequently
the lightuserdata of lua can't be used. Sometimes it is important
that lua doesn't destroy share object, for example when the object
has been created in AReVi and is passed to lua as an argument. Sometimes
lua has to delete user object, for example when they are declare and
instancied from lua. So, by default when offering the creation
of an user object for lua you have to let the possibility to be
garbage collected, but when sharing data from arevi to lua we need
a mecanism that disable garbage collection. An easy solution
is just to add an information that tells if the shared object has
to be garbage collected.



*/



