// LI2/ENIB - AReVi-Contrib - AVI recorder
//----------------------------------------------------------------------------
// file        : AReVi/Contrib/aviRecorder.cpp
// author(s)   : T. Jourdan, B. Muguet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/Contrib/aviRecorder.h"
#include "AReVi/Contrib/soundEncoder.h"

#include "AReVi/Utils/memoryBlock.h"
#include "AReVi/Utils/fileStream.h"

#include "AReVi/arSystem.h"
#include "AReVi/scheduler.h"

#include "AReVi/Lib3D/urlSound3D.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class AviRecorder
//----------------------------------------------------------------------------

AR_CLASS_NOVOID_DEF(AviRecorder,ArObject)

  bool AviRecorder::_initSuccess=false;

  void (* AviRecorder::_startFnct)(AviRecorder *)=
    (void (*)(AviRecorder *))0;

  void (* AviRecorder::_stopFnct)(AviRecorder *)=
    (void (*)(AviRecorder *))0;

  bool (* AviRecorder::_addVideoFrameFnct)(AviRecorder *,
					   unsigned int,
					   unsigned int,
					   ArConstRef<MemoryBlock>)=
    (bool (*)(AviRecorder *,
              unsigned int,
              unsigned int,
              ArConstRef<MemoryBlock>))0;

  bool (* AviRecorder::_addAudioFrameFnct)(AviRecorder *,
					   ArConstRef<MemoryBlock>)=
    (bool (*)(AviRecorder *,
	      ArConstRef<MemoryBlock>))0;

  int (* AviRecorder::_getAudioFrameSizeFnct)(AviRecorder *) =
    (int (*)(AviRecorder *))0;

  StlList<StlString> (* AviRecorder::_getListOfAvailableCodecsFnct)() = (StlList<StlString> (*)())0;

//-------- Construction / Destruction ----------------------------------------

  AviRecorder::AviRecorder(ArCW & arCW,
			   const StlString & fileName,
			   double frameRate,
			   unsigned int bitRate,
			   unsigned int frameWidth,
			   unsigned int frameHeight)
: ArObject(arCW),
  _stream(),
  _fileName(fileName),
  _format(),
  _frameRate(frameRate),
  _videoBitRate(bitRate),
  _frameWidth(frameWidth),
  _frameHeight(frameHeight),
  _dataSize(3*_frameWidth*_frameHeight),
  _nbFrames(0),
  _aviData((AviRecorderData *)0),
  _lastTime(ArSystem::getTime()),
  _startTime(ArSystem::getTime()),
  _useAudio(false)
{

if(!_fileName.size())
  {
  setErrorMessage("File name should not be empty");
  return;
  }
if(_frameRate<=0.0)
  {
  setErrorMessage("Frame rate should be greater than zero");
  return;
  }
if(!_dataSize)
  {
  setErrorMessage("Frame size should be greater than zero");
  return;
  }
if(!AviRecorder::_initSuccess)
  {
  setErrorMessage("Plugin not initialized");
  return;
  }
 (*AviRecorder::_startFnct)(this);
}

  AviRecorder::AviRecorder(ArCW & arCW,
			   const StlString & fileName,
			   double frameRate,
			   unsigned int bitRate,
			   unsigned int frameWidth,
			   unsigned int frameHeight,
			   bool useAudio,
			   struct AudioParameters audioParameters)
: ArObject(arCW),
  _stream(),
  _fileName(fileName),
  _format(),
  _frameRate(frameRate),
  _videoBitRate(bitRate),
  _frameWidth(frameWidth),
  _frameHeight(frameHeight),
  _dataSize(3*_frameWidth*_frameHeight),
  _nbFrames(0),
  _aviData((AviRecorderData *)0),
  _lastTime(ArSystem::getTime()),
  _startTime(ArSystem::getTime()),
  _useAudio(useAudio)
{

  if(_useAudio)
    {
      _audioParameters = audioParameters;
      //---audio parameters---//
      _audioBitRate = _audioParameters.bitRate;
      _sampleRate = _audioParameters.sampleRate;
      _channels = _audioParameters.nbChannels;
      //---------------------//
   }

if(!_fileName.size())
  {
  setErrorMessage("File name should not be empty");
  return;
  }
if(_frameRate<=0.0)
  {
  setErrorMessage("Frame rate should be greater than zero");
  return;
  }
if(!_dataSize)
  {
  setErrorMessage("Frame size should be greater than zero");
  return;
  }
if(!AviRecorder::_initSuccess)
  {
  setErrorMessage("Plugin not initialized");
  return;
  }
 (*AviRecorder::_startFnct)(this);

 if(_useAudio)
   {
     int frame_size = (*AviRecorder::_getAudioFrameSizeFnct)(this);

     _soundBuffer = MemoryBlock::NEW();
     _soundBuffer->setSize(frame_size*2); // sizeofshort == 2
     _soundBufferSize = 0;
   }

}


AviRecorder::AviRecorder(ArCW & arCW,
			 ArRef<AbstractOStream> stream,
			 const StlString &format,
			 double frameRate,
			 unsigned int bitRate,
			 unsigned int frameWidth,
			 unsigned int frameHeight)
: ArObject(arCW),
  _stream(stream),
  _fileName(),
  _format(format),
  _frameRate(frameRate),
  _videoBitRate(bitRate),
  _frameWidth(frameWidth),
  _frameHeight(frameHeight),
  _dataSize(3*_frameWidth*_frameHeight),
  _nbFrames(0),
  _aviData((AviRecorderData *)0),
  _lastTime(ArSystem::getTime()),
  _startTime(ArSystem::getTime()),
  _useAudio(false)
{

if(_stream->fail())
  {
  setErrorMessage("Invalid stream : '" + _stream->getErrorMessage()+"'");
  return;
  }
if(_frameRate<=0.0)
  {
  setErrorMessage("Frame rate should be greater than zero");
   return;
  }
if(!_dataSize)
  {
  setErrorMessage("Frame size should be greater than zero");
  return;
  }
if(!AviRecorder::_initSuccess)
  {
  setErrorMessage("Plugin not initialized");
  return;
  }
(*AviRecorder::_startFnct)(this);
}

AviRecorder::~AviRecorder(void) // Protected
{
if(_aviData)
  {
  (*AviRecorder::_stopFnct)(this);
  }
}

  //-------- Recording management ----------------------------------------------

bool // success
AviRecorder::addRGBFrame(unsigned int frameWidth,
			   unsigned int frameHeight,
			   ArConstRef<MemoryBlock> frameData)
{
  if(!_aviData) return false;

  if(!_useAudio)
    {
      return (*AviRecorder::_addVideoFrameFnct)(this,frameWidth,frameHeight,frameData);
    }
  else
    {
      //Must maintain the video frame rate,
      // in order to keep synchronisation between video and audio

      if(!_aviData) return false;

      double dt = (ArSystem::getTime() - _lastTime) / 1000;

      int nbFramesToAppend = (int) (dt / (1.0/(double)_frameRate));

      _lastTime += 1000 * nbFramesToAppend * (1.0/(double)_frameRate);
	
      if(nbFramesToAppend < 0 || nbFramesToAppend > _frameRate)
	return false;

      ArRef<MemoryBlock> frame = MemoryBlock::NEW();
      frame->setSize(frameData->getSize());
      memcpy(frame->accessMemory(),frameData->getMemory(),frame->getSize());

      // add the same frame n times to synchronize with timer
      while(nbFramesToAppend)
	{
	  (*AviRecorder::_addVideoFrameFnct)(this,frameWidth,frameHeight,frameData);
	  nbFramesToAppend--;
	}
      return true;
    }
}

bool // success
AviRecorder::addDataSound(ArRef<MemoryBlock> dataSound)
{

  if(!AviRecorder::_initSuccess)
    return false;

  if(!_useAudio)
    return false;

    /* the codec gives us the frame size, in samples */
  int frame_size = (*AviRecorder::_getAudioFrameSizeFnct)(this);

  int input_size = dataSound->getSize();

  char * input =  (char*)(dataSound->accessMemory());

  ArRef<MemoryBlock> sample = MemoryBlock::NEW();
  sample->setSize(2*frame_size);

  // if sound buffer contains dataSound
      if(_soundBufferSize>0)
	{
	  if(dataSound->getSize() + _soundBufferSize < _soundBuffer->getSize())
	    {
	      memcpy(_soundBuffer->accessMemory()+_soundBufferSize,dataSound->getMemory(),dataSound->getSize());
	      _soundBufferSize += dataSound->getSize();
	      input_size=0;
	    }
	  else
	    {
	      //when the buffer contains a sample
	      memcpy(_soundBuffer->accessMemory()+_soundBufferSize,dataSound->getMemory(),_soundBuffer->getSize() - _soundBufferSize);
	
	      input += (_soundBuffer->getSize() - _soundBufferSize);
	      input_size -= (_soundBuffer->getSize() - _soundBufferSize);
	
	      // compressing the sample in the soundBuffer
	      memcpy(sample->accessMemory(),(char *)(_soundBuffer->getMemory()),frame_size*2);
	      (*AviRecorder::_addAudioFrameFnct)(this,sample);

	      //flushing the soundBuffer
	      _soundBufferSize = 0;
	    }
	}

      int i;
      for(i=0;i<(input_size-2*frame_size);i+=(2*frame_size))
	{
  	  memcpy(sample->accessMemory(),input+i,frame_size*2);

	  /* encode the samples */
	  if(!((*AviRecorder::_addAudioFrameFnct)(this, sample)))
	    return false;
	
	}

      if(i<input_size)
	{
	  // stock the overflow in the sound buffer
	  input += i;
	  memcpy(_soundBuffer->accessMemory(), input, (input_size-i));
	  _soundBufferSize = (input_size-i);
	  //cerr << "resting ..." << _soundBufferSize << endl;
	}


      return true;
}

double // record duration
AviRecorder::getDuration(void) const
{
return(_aviData ? _nbFrames/_frameRate : 0.0);
}

ArConstRef<AbstractOStream> // output stream if any
AviRecorder::getStream(void) const
{
return(_stream);
}


const StlString & // output file name if any
AviRecorder::getFileName(void) const
{
return(_fileName);
}

const StlString & // output format
AviRecorder::getFormat(void) const
{
return(_format);
}

//-------- Codecs properties --------------------------------------------------

StlList<StlString> AviRecorder::getListOfAvailableCodecs()
{

  StlList<StlString> codecsList;

  if(!AviRecorder::_initSuccess)
    return codecsList;

  codecsList = (*AviRecorder::_getListOfAvailableCodecsFnct)();

  return codecsList;
}

double // frame rate
AviRecorder::getFrameRate(void) const
{
return(_frameRate);
}

unsigned int // video bit rate
AviRecorder::getVideoBitRate(void) const
{
return(_videoBitRate);
}

unsigned int // audio bit rate
AviRecorder::getAudioBitRate(void) const
{
  if(!_useAudio)
    {
      cerr << "audio stream not initialized" << endl;
      return 0;
    }
  else
    return(_audioBitRate);
}


void
AviRecorder::getFrameSize(unsigned int & frameWidthOut,
			       unsigned int & frameHeightOut) const
{
frameWidthOut=_frameWidth;
frameHeightOut=_frameHeight;
}

//-------- Forbidden operations ----------------------------------------------

} // namespace AReVi

//----------------------------------------------------------------------------
