// LI2/ENIB - AReVi-Contrib - CgFilter ???
//----------------------------------------------------------------------------
// file        : AReVi/Contrib/cgFilter.cpp
// author(s)   : D. Herviou
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/Contrib/cgFilter.h"

#include "AReVi/Lib3D/renderTexture.h"
#include "AReVi/Lib3D/textureRenderer3D.h"
#include "AReVi/Lib3D/viewer3D.h"
#include "AReVi/Lib3D/scene3D.h"
#include "AReVi/Lib3D/osdSprite.h"

namespace AReVi {

//-------------------------------------------------------------
// CgFilter implementation
//-------------------------------------------------------------

AR_CLASS_NOVOID_DEF(CgFilter, ArObject)

CgFilter::CgFilter(ArCW & arCW,
		   const StlString& CGPName,
		   ArRef<CgContext> CGContext,
		   CgContext::Profile CGProfile,
		   int width,
		   int height):
ArObject(arCW),
_CGPName(CGPName),
_CGContext(CGContext),
_width(width),
_height(height)
{
  _CGProfile = CGProfile;
  _CGPName = StlString(CGPName);
  _pathProg.push_back("");
}
//-------------------------------------------------------------

CgFilter::~CgFilter(void){
}

//-------------------------------------------------------------
void
CgFilter::addProgPath(const StlString& name){
   _pathProg.push_back(name);
}

//-------------------------------------------------------------
void
CgFilter::generateFilter(void){
   StlList<StlString>::iterator it;
   StlString pathName ;


   //search in all path if _CGPName is present
   for (it = _pathProg.begin(); it!=_pathProg.end();){
     pathName = (*it)+_CGPName;
     _CGProg = _CGContext -> createProgramFromURL(pathName, _CGProfile, "main", NULL);
     if ( _CGProg != 0 ){
       _CGContext -> loadProgram(_CGProg);
       return ;
     }else{
       it++  ;
     }
   }
   cerr<<"perhaps a mistake in "<<_CGPName<<"try a single compilation with cgc"<<endl;
   cerr<<"or verify the paths to "<<_CGPName<<endl;
   cerr<<"search path where :"<<endl;
   for (it = _pathProg.begin(); it!=_pathProg.end();){
     pathName = StlString((*it)+_CGPName);
     cerr<<"---->"<<pathName<<"<----"<<endl;
     it++  ;
   }

}

//-------------------------------------------------------------
void
CgFilter::activateFilter(void){
  _CGContext->bindProgram(_CGProg);
  _CGContext->enableProfile(_CGProfile);
}

//-------------------------------------------------------------
void
CgFilter::deactivateFilter(void){
  _CGContext->disableProfile(_CGProfile);
}

//-------------------------------------------------------------
void
CgFilter::setParameter(const StlString& name, float value){
  CgContext::Param _param = _CGContext->getNamedParameter(_CGProg, name);
  _CGContext->setParameter1f(_param,value);
}

//-------------------------------------------------------------
void
CgFilter::setParameter(const StlString& name, double value){
  CgContext::Param _param = _CGContext->getNamedParameter(_CGProg, name);
  _CGContext->setParameter1d(_param,value);
}

//-------------------------------------------------------------
void
CgFilter::setParameter4x4(const StlString& name, CgContext::StateMatrixType matrix){
  CgContext::Param _param = _CGContext->getNamedParameter(_CGProg, name);
  _CGContext->setStateMatrixParameter(_param, matrix, CgContext::MATRIX_IDENTITY);
}

//-------------------------------------------------------------
void
CgFilter::setParameter(const StlString& name, unsigned int textureID){
  CgContext::Param _param = _CGContext->getNamedParameter(_CGProg, name);
  _CGContext->setTextureParameter(_param,textureID);
  _CGContext->enableTextureParameter(_param);
}


//-------------------------------------------------------------
// CgPipeline implementation
//-------------------------------------------------------------

AR_CLASS_NOVOID_DEF(CgPipeline,ArObject)

CgPipeline::CgPipeline(ArCW& arCW, int nbSlot):
  ArObject(arCW),
  _context(CgContext::NEW())
{

  for(int i=0 ; i < nbSlot ; i++){
    _textures.push_back(make_pair(i, RenderTexture::nullRef() ));
    _internalRenderers.push_back(make_pair(i, TextureRenderer3D::NEW()));
  }
}

//-------------------------------------------------------------
CgPipeline::~CgPipeline(void){
}

//-------------------------------------------------------------
void
CgPipeline::initInternalRenderers( ArRef<Viewer3D> viewer ){

   unsigned int w, h;
   Viewer3D::getScreenSize(w,h);

  StlList<pair<int, ArRef<TextureRenderer3D> > >::iterator it;
  for (it=_internalRenderers.begin(); it!=_internalRenderers.end(); ){
  (it->second) -> setLocation (viewer);
  (it->second) -> attachTo(viewer);
  (it->second) -> setAutoRender(false);
  (it->second) -> setNearDistance(1);
  (it->second) -> setFarDistance(50);
  it++;
  }
  _viewer3D = viewer ;
}

//-------------------------------------------------------------
void
CgPipeline::addFilter( ArRef<CgFilter> filter ){
  _filters.push_back(make_pair( filter->getName() , filter));
}

//-------------------------------------------------------------
void
CgPipeline::removeFilter( const StlString& name ){

  StlList<pair<StlString, ArRef<CgFilter> > >::iterator it;
  for (it =_filters.begin(); it!= _filters.end(); ){
    if (it->first == name ){
      _filters.erase(it);
      return ;
    }
    it++;
  }
}

//-------------------------------------------------------------
ArRef<RenderTexture>
CgPipeline::getTextureNb(int nb){
  StlList<pair<int, ArRef<RenderTexture> > >::iterator it;
  for(it = _textures.begin(); it != _textures.end(); ){
    if( it->first == nb ){
      return it->second;
    }
    it++;
  }
  return RenderTexture::nullRef();
}


//-------------------------------------------------------------
ArRef<CgFilter>
CgPipeline::getFilter(const StlString& name){
  StlList<pair<StlString, ArRef<CgFilter> > >::iterator it;
  for( it = _filters.begin() ; it != _filters.end() ; ){
    if( it->first == name ){
      return it -> second;
    }
    it++;
  }
  return CgFilter::nullRef();
}


//-------------------------------------------------------------
void
CgPipeline::activateFilter( const StlString& name){
  getFilter(name)->activateFilter();
}

//-------------------------------------------------------------
void
CgPipeline::deactivateFilter( const StlString& name ){
  getFilter(name)->deactivateFilter();
}

//-------------------------------------------------------------
void
CgPipeline::setRenderSize(unsigned int w, unsigned int h){
  StlList<pair<int, ArRef<TextureRenderer3D> > >::iterator it;

  for(it = _internalRenderers.begin(); it!= _internalRenderers.end(); ){
    (it->second)->setRenderSize(w,h);
    it++;
  }
}

//-------------------------------------------------------------
void
CgPipeline::setFarDistance(float far){
  StlList<pair<int, ArRef<TextureRenderer3D> > >::iterator it;

  for(it = _internalRenderers.begin(); it!= _internalRenderers.end(); ){
    (it->second)->setFarDistance(far);
    it++;
  }
}

//-------------------------------------------------------------
void
CgPipeline::setNearDistance(float near){
  StlList<pair<int, ArRef<TextureRenderer3D> > >::iterator it;
  for(it = _internalRenderers.begin(); it!= _internalRenderers.end(); ){
    (it->second)->setNearDistance(near);
    it++;
  }
}

//-------------------------------------------------------------
void
CgPipeline::render(int nb){
  _getInternalRenderer(nb)->render();
  _captureBuffer(nb); //dump result in texture slot nb
}

//-------------------------------------------------------------
void
CgPipeline::render(int input, int output){
  _beforeShowResult(getTextureNb(input),_getInternalRenderer(output));
  render(output); // dump result in slot 1
  _afterShowResult(_getInternalRenderer(output));
}


//-------------------------------------------------------------
ArRef<TextureRenderer3D>
CgPipeline::_getInternalRenderer(int nb){
  StlList<pair<int, ArRef<TextureRenderer3D> > >::iterator it;
  for(it = _internalRenderers.begin(); it != _internalRenderers.end(); ){
    if( it->first == nb ){
      return it->second;
    }
    it++;
  }
  return TextureRenderer3D::nullRef();
}

//-------------------------------------------------------------
void
CgPipeline::_setTexture(int nb, ArRef<RenderTexture> texture){
   StlList<pair<int, ArRef<RenderTexture> > >::iterator it;

   for ( it = _textures.begin(); it != _textures.end(); ){
     if ( it->first == nb ){
       _textures.push_back(make_pair(nb,texture));
       _textures.erase(it);
       return;
     }
     it++;
   }
   return;
}

//-------------------------------------------------------------
void
CgPipeline::_beforeShowResult(ArRef<RenderTexture> textureInput, ArRef<Renderer3D> viewer){
  _osdSprite = OSDSprite::NEW(textureInput );
  _osdSprite->setRatio(0.0);
  _osdSprite->setScale(1.0,1.0);
  _osdSprite->setLocation(OSD::LOC_CENTER);
  _sceneBackUp = viewer->accessScene();
  _sceneTempo = Scene3D::NEW();
  viewer->selectScene(_sceneTempo);//empty scene
  viewer->addOSD(_osdSprite); //only one OSD to render
}


//-------------------------------------------------------------
void
CgPipeline::_afterShowResult(ArRef<Renderer3D> viewer){
     viewer->removeOSD(_osdSprite);
     viewer->selectScene(_sceneBackUp); //restore scene
}


//-------------------------------------------------------------
void
CgPipeline::_captureBuffer(int TextureOutputBuffer){
  _setTexture(TextureOutputBuffer, (_getInternalRenderer(TextureOutputBuffer))->buildTexture());
}

} // namespace AReVi

//----------------------------------------------------------------------------
