// LI2/ENIB - AReVi-Contrib - Sound encoder
//----------------------------------------------------------------------------
// file        : AReVi/Contrib/soundEncoder.cpp
// author(s)   : B. Muguet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/arSystem.h"
#include "AReVi/Utils/memoryBlock.h"

#include "AReVi/Contrib/soundEncoder.h"
#include "AReVi/Utils/fileStream.h"

namespace AReVi {

//----------------------------------------------------------------------------
// CLASS       : VideoConf
//----------------------------------------------------------------------------

AR_CLASS_NOVOID_DEF(SoundEncoder,ArObject)

  bool SoundEncoder::_initSuccess=false;

  void (* SoundEncoder::_startFnct)(SoundEncoder *,
				    const char* ,int, int, int)=
  (void (*)(SoundEncoder *, const char*, int, int, int))0;

  void (* SoundEncoder::_stopFnct)(SoundEncoder *)=
  (void (*)(SoundEncoder *))0;

  int (* SoundEncoder::_encodeSoundSampleFnct)(SoundEncoder *,
					       unsigned char *,short*)=
  (int (*)(SoundEncoder *, unsigned char*, short*))0;

  int (* SoundEncoder::_decodeSoundSampleFnct)(SoundEncoder *,
					       unsigned int *, int*,
					       unsigned char *, int)=
  (int (*)(SoundEncoder *, unsigned int *, int*, unsigned char*, int))0;

  int (* SoundEncoder::_getFrameSizeFnct)(SoundEncoder *) =
  (int (*)(SoundEncoder *))0;

  int (* SoundEncoder::_getMaxAudioFrameSizeFnct)() = (int (*)())0;

  StlList<StlString> (* SoundEncoder::_getListOfAvailableCodecsFnct)() = (StlList<StlString> (*)())0;

 //-------- Construction / Destruction ----------------------------------------

SoundEncoder::SoundEncoder(ArCW & arCW)
  : ArObject(arCW),
    _soundBuffer(MemoryBlock::NEW())
{

  // default values
  _codecName = "MP2";
  _bitRate = 64000;
  _sampleRate = 22050;
  _nbChannels = 1;  // mode mono

   if(!SoundEncoder::_initSuccess)
  {
    setErrorMessage("Plugin not initialized");
    return;
  }

   // Initialization of codecs
  (*SoundEncoder::_startFnct)(this,_codecName.c_str(),_bitRate,_sampleRate,_nbChannels);

  int frame_size = (*SoundEncoder::_getFrameSizeFnct)(this);
  _soundBuffer->setSize(frame_size*2);
  _soundBufferSize = 0;
}


 SoundEncoder::SoundEncoder(ArCW & arCW,
			    int bitRate,
			    int sampleRate,
			    int nbChannels)
   : ArObject(arCW),
     _bitRate(bitRate),
     _sampleRate(sampleRate),
     _nbChannels(nbChannels),
     _soundBuffer(MemoryBlock::NEW())
  {

    _codecName = "MP2"; //default codec

    if(!SoundEncoder::_initSuccess)
  {
    setErrorMessage("Plugin not initialized");
    return;
  }

  (*SoundEncoder::_startFnct)(this,_codecName.c_str(),_bitRate,_sampleRate,_nbChannels);

  int frame_size = (*SoundEncoder::_getFrameSizeFnct)(this);
  _soundBuffer->setSize(frame_size*2);
  _soundBufferSize = 0;
}


SoundEncoder::SoundEncoder(ArCW & arCW,
			   StlString codecName,
			   int bitRate,
			   int sampleRate,
			   int nbChannels)
  : ArObject(arCW),
    _codecName(codecName),
    _bitRate(bitRate),
    _sampleRate(sampleRate),
    _nbChannels(nbChannels),
    _soundBuffer(MemoryBlock::NEW())
{

  if(!SoundEncoder::_initSuccess)
    {
      setErrorMessage("Plugin not initialized");
      return;
    }

  (*SoundEncoder::_startFnct)(this,_codecName.c_str(),_bitRate,_sampleRate,_nbChannels);

  int frame_size = (*SoundEncoder::_getFrameSizeFnct)(this);
  _soundBuffer->setSize(frame_size*2);
  _soundBufferSize = 0;

}

SoundEncoder::~SoundEncoder(void)
{
  if(!SoundEncoder::_initSuccess)
    return;

  if(_soundEncoderData)
  {
  (*SoundEncoder::_stopFnct)(this);
  }
}

//-------- Encoding management ----------------------------------------------

ArRef<MemoryBlock> SoundEncoder::encodeDataSound(ArRef<MemoryBlock> dataSoundToEncode)
{

  ArRef<MemoryBlock> encodedDataSound = MemoryBlock::NEW();

  if(!SoundEncoder::_initSuccess)
    return encodedDataSound;

  int i, out_size, outbuf_size;
  short *samples;

  unsigned char *outbuf;
  outbuf = (unsigned char *)malloc(10000);
  outbuf_size = 10000;
  memset(outbuf, 0, outbuf_size);

  /* the codec gives us the frame size, in samples */
      int frame_size = (*SoundEncoder::_getFrameSizeFnct)(this);

      int input_size = dataSoundToEncode->getSize();

      char * input =  (char*)(dataSoundToEncode->accessMemory());

      samples = (short *)malloc(frame_size * sizeof(short) * _nbChannels);

      // if sound buffer contains dataSound
      if(_soundBufferSize>0)
	{
	  if(dataSoundToEncode->getSize() + _soundBufferSize < _soundBuffer->getSize())
	    {
	      memcpy(_soundBuffer->accessMemory()+_soundBufferSize,dataSoundToEncode->getMemory(),dataSoundToEncode->getSize());
	      _soundBufferSize += dataSoundToEncode->getSize();
	      input_size=0;
	    }
	  else
	    { //when the buffer contains a sample
	      memcpy(_soundBuffer->accessMemory()+_soundBufferSize,dataSoundToEncode->getMemory(),_soundBuffer->getSize() - _soundBufferSize);
	
	      input += (_soundBuffer->getSize() - _soundBufferSize);
	      input_size -= (_soundBuffer->getSize() - _soundBufferSize);
	
	      // compressing the sample in the soundBuffer
	      memcpy((char *)samples,(char *)(_soundBuffer->getMemory()),frame_size*2);

	      out_size = (*SoundEncoder::_encodeSoundSampleFnct)(this,outbuf, samples);

	      encodedDataSound->setSize(encodedDataSound->getSize() + out_size);
	      memcpy(encodedDataSound->accessMemory(), outbuf, out_size);

	      //flushing the soundBuffer
	      _soundBufferSize = 0;
	    }
	}

      for(i=0;i<(input_size-2*frame_size);i+=(2*frame_size))
	{
  	  memcpy((char*)samples,input+i,frame_size*2);
	
 	  /* encode the samples */
	  out_size = (*SoundEncoder::_encodeSoundSampleFnct)(this,outbuf, samples);

	  int pos = encodedDataSound->getSize();
	  encodedDataSound->setSize(encodedDataSound->getSize() + out_size);
	  memcpy(encodedDataSound->accessMemory() + pos, outbuf, out_size);
	}

      if(i<input_size)
	{
	  // stock the overflow in the sound buffer
	  input += i;
	  memcpy(_soundBuffer->accessMemory(), input, (input_size-i));
	  _soundBufferSize = ((input_size-i));
	  //cerr << "resting ..." << _soundBufferSize << endl;
	}

     free(outbuf);
     free(samples);

     return encodedDataSound;
}

ArRef<MemoryBlock> SoundEncoder::decodeDataSound(ArRef<MemoryBlock> dataSoundToDecode)
{

  ArRef<MemoryBlock> decodedDataSound = MemoryBlock::NEW();

  if(!SoundEncoder::_initSuccess)
    return decodedDataSound;

  int out_size, size, len;
  unsigned int *outbuf;

  int outbuf_size = (*SoundEncoder::_getMaxAudioFrameSizeFnct)();

  outbuf = (unsigned int *)malloc(outbuf_size);

  unsigned char *inbuf, *inbuf_ptr;
  inbuf = (unsigned char*)malloc(dataSoundToDecode->getSize());
  inbuf_ptr = inbuf;

  memcpy(inbuf,dataSoundToDecode->getMemory(),dataSoundToDecode->getSize());
  size = dataSoundToDecode->getSize();

  // decoding the sample
  while(size>0) {
      len = (*SoundEncoder::_decodeSoundSampleFnct)(this,outbuf, &out_size, inbuf_ptr, size);

    if (len < 0) {
      fprintf(stderr, "Error while decoding\n");
      exit(1);
    }

    if (out_size > 0) {
      /* if a frame has been decoded, output it */
      decodedDataSound->setSize(out_size);
      memcpy(decodedDataSound->accessMemory(), (char*)outbuf, out_size);
    }

    size -= len;
    inbuf_ptr += len;
  }

  free(outbuf);
  free(inbuf);

  return decodedDataSound;
}

//Encoding and recording in a file
bool SoundEncoder::openStream(StlString filename)
{
  _fileStream = FileOStream::NEW(filename,false);
  if (!_fileStream) {
    cerr << "could not open " <<  filename << endl;
    return false;
  }

  return true;
}

bool SoundEncoder::addSoundFrame(ArRef<MemoryBlock> dataSoundToEncode)
{
  ArRef<MemoryBlock> encodedSound = encodeDataSound(dataSoundToEncode);

  if(encodedSound->getSize())
    _fileStream->writeData((char*)(encodedSound->accessMemory()),encodedSound->getSize());

  return true;
}


bool SoundEncoder::closeStream()
{
  _fileStream->flush();

  return true;
}

//-------- Codecs properties --------------------------------------------------

StlList<StlString> SoundEncoder::getListOfAvailableCodecs()
{

  StlList<StlString> codecsList;

  if(!SoundEncoder::_initSuccess)
    return codecsList;

  codecsList = (*SoundEncoder::_getListOfAvailableCodecsFnct)();

  return codecsList;

}

StlString SoundEncoder::getCodecName()
{
  return _codecName;
}

void SoundEncoder:: setCodecName(StlString codecName)
{
  if(!SoundEncoder::_initSuccess)
    return;

  if(codecName != _codecName)
    {
      // stop (if necessary) and restart the encoder to apply changes
      if(_soundEncoderData)
	{
	  (*SoundEncoder::_stopFnct)(this);
	}

      _codecName = codecName;

      (*SoundEncoder::_startFnct)(this,_codecName.c_str(),_bitRate,_sampleRate,_nbChannels);
    }
}

//-------- Sound properties --------------------------------------------------

int SoundEncoder::getBitRate()
{
  return _bitRate;
}

int SoundEncoder::getSampleRate()
{
  return _sampleRate;
}

int SoundEncoder::getNbChannels()
{
  return _nbChannels;
}


void SoundEncoder::setBitRate(int bitRate)
{
  if(!SoundEncoder::_initSuccess)
    return;

  if(bitRate != _bitRate)
    {
      // stop (if necessary) and restart the encoder to apply changes
      if(_soundEncoderData)
	{
	  (*SoundEncoder::_stopFnct)(this);
	}

      _bitRate = bitRate;

      (*SoundEncoder::_startFnct)(this,_codecName.c_str(),_bitRate,_sampleRate,_nbChannels);
    }
}

void SoundEncoder::setSampleRate(int sampleRate)
{
  if(!SoundEncoder::_initSuccess)
    return;

  if(sampleRate != _sampleRate)
    {
      // stop (if necessary) and restart the encoder to apply changes
      if(_soundEncoderData)
	{
	  (*SoundEncoder::_stopFnct)(this);
	}

      _sampleRate = sampleRate;

      (*SoundEncoder::_startFnct)(this,_codecName.c_str(),_bitRate,_sampleRate,_nbChannels);
    }
}

void SoundEncoder::setNbChannels(int nbChannels)
{
  if(!SoundEncoder::_initSuccess)
    return;

  if(nbChannels != _nbChannels)
    {
      // stop (if necessary) and restart the encoder to apply changes
      if(_soundEncoderData)
	{
	  (*SoundEncoder::_stopFnct)(this);
	}

      _nbChannels = nbChannels;

      (*SoundEncoder::_startFnct)(this,_codecName.c_str(),_bitRate,_sampleRate,_nbChannels);
    }
}


void SoundEncoder::setEncoderParameters(StlString codecName, int bitRate, int sampleRate, int nbChannels)
{
  if(!SoundEncoder::_initSuccess)
    return;

  if((codecName != _codecName) ||
     (bitRate != _bitRate) ||
     (sampleRate != _sampleRate) ||
     (nbChannels != _nbChannels))
    {
      // stop (if necessary) and restart the encoder to apply changes
      if(_soundEncoderData)
	{
	  (*SoundEncoder::_stopFnct)(this);
	}

      _codecName = codecName;
      _bitRate = bitRate;
      _sampleRate = sampleRate;
      _nbChannels = nbChannels;

      (*SoundEncoder::_startFnct)(this,_codecName.c_str(),_bitRate,_sampleRate,_nbChannels);
    }
}


} // namespace AReVi

//----------------------------------------------------------------------------
