// LI2/ENIB - AReVi-Contrib - Text To Speech service
//----------------------------------------------------------------------------
// file        : AReVi/Contrib/text2Speech.cpp
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/Contrib/text2Speech.h"

#include "AReVi/arSystem.h"
#include "AReVi/Lib3D/dataSound3D.h"
#include "AReVi/Contrib/soundPlayer.h"

namespace AReVi {
//----------------------------------------------------------------------------
// Class Text2Speech
//----------------------------------------------------------------------------
AR_CLASS_DEF(Text2Speech,DataSound3D)

//----------------------------------------------------------------------------
// Static class data initialization (for plugin)
//----------------------------------------------------------------------------
bool Text2Speech::_initSuccess=false;
void *Text2Speech::_serverHandle=NULL;
void *Text2Speech::_dispatchHandle=NULL;


// Init of functions
ArRef<MemoryBlock>  (*Text2Speech::_sayTextToMemory)(TTSCHANELID, const StlString &, const StlString &, unsigned int & ,UtilSound::SoundFormat &)=
    (ArRef<MemoryBlock>  (*)(TTSCHANELID, const StlString &, const StlString &, unsigned int &,UtilSound::SoundFormat & ))0;

bool (*Text2Speech::_sayTextToMemoryAsync)(void *, const StlString &, const StlString &)=
    (bool (*)(void *, const StlString &,const StlString &))0;

void *(*Text2Speech::_sayTextToMemoryAsyncInit)(TTSCHANELID, TTSSayTextToMemoryPerformer , TTSSayTextToMemoryMethod , void *)=
    (void *(*)(TTSCHANELID, TTSSayTextToMemoryPerformer , TTSSayTextToMemoryMethod , void *))0;

bool   (*Text2Speech::_getVoices)(StlVector<StlString> &,const int &,const int &)        =
        (bool   (*)(StlVector<StlString> &,const int &,const int &))0;

TTSCHANELID (*Text2Speech::_openVoice)(void *, void *, const StlString &)     =
        (TTSCHANELID (*)(void *, void *, const StlString &))0;

bool   (*Text2Speech::_closeVoice)(void *, TTSCHANELID)                       =
        (bool   (*)(void *, TTSCHANELID))0;

bool   (*Text2Speech::_getVoiceSettings)(const StlString &, unsigned int &, UtilSound::SoundFormat &, unsigned int &)=
        (bool   (*)(const StlString &, unsigned int &, UtilSound::SoundFormat &, unsigned int &))0;

bool   (*Text2Speech::_setVoiceParams)(TTSCHANELID, double ,double ,double )=
        (bool   (*)(TTSCHANELID, double,double,double))0;

bool   (*Text2Speech::_getVoiceParams)(TTSCHANELID, double &,double &,double &)=
        (bool   (*)(TTSCHANELID, double &,double &,double &))0;

bool   (*Text2Speech::_continueProcessing)(void *, int )=(bool   (*)(void *, int ))0;

bool   (*Text2Speech::_abortProcessing)(void *)=(bool   (*)(void *))0;

void   (*Text2Speech::_getEncodingSupportList)(StlVector<StlString> &)=(void   (*)(StlVector<StlString> &))0;

//-------- Construction / Destruction ----------------------------------------

Text2Speech::Text2Speech(ArCW & arCW) // Protected
: ArObject(arCW),
  _channelID((TTSCHANELID)-1),
  _onCancel(false),
  _useSoundPlayer(false),
  _useSound3D(false),
  _dataSound(),
  _soundPlayer(),
  _encoding("ISO-8859-1"),
  _currentVoice("None"),
  _voicePitch(1), _voiceVolume(0.5), _voiceSpeed(1),
  _pInstanceData(NULL)
{

if(Text2Speech::_initSuccess)
  {
  StlVector<StlString> voices;
  getVoices(voices);
  if (voices.size()) // Initialize with the last voice (better?)
    {
    selectVoice(voices[voices.size()-1]);
    }
  else
    setErrorMessage("Text2Speech error : No voice found!");
  }
else
  setErrorMessage("Text2Speech plugin not initialized");
}

Text2Speech::Text2Speech(ArCW & arCW, bool soundPlayer, bool sound3D) // Protected
: ArObject(arCW),
  _channelID((TTSCHANELID)-1),
  _onCancel(false),
  _useSoundPlayer(soundPlayer),
  _useSound3D(sound3D),
  _dataSound(),
  _soundPlayer(),
  _encoding("ISO-8859-1"),
  _currentVoice("None"),
  _voicePitch(1), _voiceVolume(0.5), _voiceSpeed(1),
  _pInstanceData(NULL)
{
if(Text2Speech::_initSuccess)
  {
  StlVector<StlString> voices;
  getVoices(voices);

  if (sound3D)
    {
    _dataSound=DataSound3D::NEW();
    if (_dataSound->fail())
      {
      setErrorMessage(_dataSound->getErrorMessage());
      _dataSound.clear();
      _useSound3D=false;
      }
    }

  if (voices.size()) // Initialize with the last voice (better?)
    {
    selectVoice(voices[voices.size()-1]);

    if (soundPlayer)
      {
      unsigned int frequency, languageId;
      UtilSound::SoundFormat format(UtilSound::MONO16);

      if (getVoiceSettings(_currentVoice,frequency, format, languageId))
        {
        _soundPlayer=SoundPlayer::NEW(format,frequency);
        if (_soundPlayer->fail())
          {
          setErrorMessage(_soundPlayer->getErrorMessage());
          _soundPlayer.clear();
          _useSoundPlayer=false;
          }
        else
          _soundPlayer->setAutoPop(true);
        }
      }
    }
  else
    setErrorMessage("Text2Speech error : No voice found!");
  }
else
  setErrorMessage("Text2Speech plugin not initialized");
}


Text2Speech::~Text2Speech(void) // Protected
{
if(Text2Speech::_initSuccess)
  {
  // Destroy PInstanceData made by plugin
  _pInstanceData=(*_sayTextToMemoryAsyncInit)((TTSCHANELID)-1,(Text2Speech::TTSSayTextToMemoryPerformer)thisRef(),
                                                            (Text2Speech::TTSSayTextToMemoryMethod)&Text2Speech::_onSayTextToMemory,
                                                            _pInstanceData);
  }
_closeAll();
}

// Close all executions channels
bool //  virtual protected
Text2Speech::_closeAll(void)
{
if(Text2Speech::_initSuccess && _channelID!=(TTSCHANELID)-1)
  {
  _closeVoice(Text2Speech::_serverHandle,_channelID);
  _channelID=(TTSCHANELID)-1;
  }
return true;
}

//--------  Say Text with current voice and settings ----------------------------------------
// sync=false =>Non blocking call(return true if request is successed)
bool
Text2Speech::sayText(const StlString & str)
{
if(Text2Speech::_initSuccess && _pInstanceData)
  {
  if (!(*_sayTextToMemoryAsync)(_pInstanceData, str, _encoding))
    return false;

  if (_dataSound.valid()) // Here PushBack text ...
    {
//     _dataSound->stop();
//     _dataSound->popBuffers();
    _dataSound->play();
    }

  if (_soundPlayer.valid())
    {
//     _soundPlayer->popBuffers(true);
    if (!_soundPlayer->getPlay())
      {
      _soundPlayer->play();
      }
    }
  _onCancel=false;

  if (!_speakingAct)
    {
    _speakingAct=Activity::NEW(0.01);
    _speakingAct->setBehavior(thisRef(),&Text2Speech::_onSpeaking);
    }
  return(true);
  }
else
  {
  return(false);
  }
}

// Return if TTS is speaking
bool
Text2Speech::isSpeaking(void) const
{
if (Text2Speech::_initSuccess)
  {
  if (_speakingAct.valid()) // Generating text
    return(true);

  if (_dataSound.valid() && _dataSound->getNbPendingBuffers())
    return(true);

  if (_soundPlayer.valid() && _soundPlayer->isPlaying())
    return(true);
  }

return(false);
}

// Abort speaking
void
Text2Speech::abortSpeaking(void)
{
if(Text2Speech::_initSuccess && !_onCancel)
  {
  if (_speakingAct && _pInstanceData)
    {
    _onCancel=true;
    (*_abortProcessing)(_pInstanceData);
    _onSpeaking(_speakingAct,0);
    _speakingAct.clear();
    }
  if (_dataSound.valid())
    {
    _dataSound->stop();
    }

  if (_soundPlayer.valid())
    {
    _soundPlayer->stop();
    _soundPlayer->popBuffers(true);
    }
  }
}

//--------  Store Generated Text To MemoryBlock  ----------------------------------------
ArRef<MemoryBlock>
Text2Speech::sayTextToMemory(const StlString & str, unsigned int &freqOut , UtilSound::SoundFormat &formatOut)
{
if(Text2Speech::_initSuccess && !_speakingAct.valid())
  {
  _onCancel=false;

  if (selectVoice(_currentVoice))
    {
    return (*_sayTextToMemory)(_channelID, str,_encoding, freqOut,formatOut);
    }
  else
    return  (ArRef<MemoryBlock>)NULL;
  }
else
  return  (ArRef<MemoryBlock>)NULL;
}

//---------- CB pour signaler la fin d'un traitement asynchrone--------------------------
bool //virtual protected
Text2Speech::_onSayTextToMemory(ArRef<MemoryBlock> mem,
                                unsigned int freq,
                                UtilSound::SoundFormat format,
                                bool finished)
{
if (finished)
  {
  _speakingAct.clear();
  _onCancel=false;
//   _pInstanceData=NULL;
  }
else if (mem.valid() && !_onCancel)
  {
  if (_dataSound.valid())
    {
    _dataSound->pushBuffer(mem,int(freq),format);
    }

  if (_soundPlayer.valid())
    {
    if(freq==_soundPlayer->getRate() && format==_soundPlayer->getFormat())
      {
      _soundPlayer->pushBuffer(mem);
      }
    else
      {
      setErrorMessage("Bad sample format for selected soundPlayer");
      abortSpeaking();
      return(false);
      }
    }
  }
return(true);
}

// CB speaking
bool // virtual protected
Text2Speech::_onSpeaking(ArRef<Activity> , double )
{
if(Text2Speech::_initSuccess)
  {
  if (_speakingAct || _onCancel)
    return (*_continueProcessing)(Text2Speech::_dispatchHandle,0);
  }

  return  false;
}

//--------  Static : Get availables voices   ------------------------------------
bool
Text2Speech::getVoices(StlVector<StlString> & voicesOut,
                        const int &specificLanguage/*=-1*/,
                        const int &specificFrequency/*=-1*/) //static
{
if(Text2Speech::_initSuccess)
  {
  return (*_getVoices)(voicesOut,specificLanguage,specificFrequency);
  }
else
  return  false;
}

bool
Text2Speech::getVoiceSettings(const StlString&voiceID, unsigned int &frequencyOut,  UtilSound::SoundFormat &formatOut, unsigned int &langageIdOut) //static
{
if(Text2Speech::_initSuccess)
  {
  return (*_getVoiceSettings)(voiceID,frequencyOut,  formatOut, langageIdOut);
  }
else
  return  false;
}


//--------  Select voice   ----------------------------------------
bool
Text2Speech::selectVoice(const StlString& voiceName)
{
if(Text2Speech::_initSuccess && (!isSpeaking()))
  {
  if (_channelID!=(TTSCHANELID)-1)
    (*_closeVoice)(Text2Speech::_serverHandle,_channelID);

  if (voiceName!="")
    {
    _channelID=(*_openVoice)(Text2Speech::_serverHandle,Text2Speech::_dispatchHandle,voiceName);

    if (_channelID==(TTSCHANELID)-1)
      {
      setErrorMessage("Text2Speech error : Unable to open voice!");
      return(false);
      }
    _currentVoice=voiceName;
    _setVoiceParams(_channelID,_voicePitch,_voiceVolume,_voiceSpeed);

    _pInstanceData=(*_sayTextToMemoryAsyncInit)(_channelID, (Text2Speech::TTSSayTextToMemoryPerformer)thisRef(),
                                                            (Text2Speech::TTSSayTextToMemoryMethod)&Text2Speech::_onSayTextToMemory,
                                                            _pInstanceData);

    if (_useSoundPlayer)
      {
      unsigned int frequency, languageId;
      UtilSound::SoundFormat format(UtilSound::MONO16);

      if (getVoiceSettings(_currentVoice,frequency, format, languageId))
        {
        if((!_soundPlayer.valid()) || (_soundPlayer->getFormat()!=format) || (_soundPlayer->getRate()!=frequency))
          {
          _soundPlayer=SoundPlayer::NEW(format,frequency);
          _soundPlayer->setAutoPop(true);
          }
        else
          {
          _soundPlayer->stop();
          _soundPlayer->popBuffers(true);
          }
        }
      else
        return(false);
      }
    return(true);
    }
  else
    {
    _channelID=(TTSCHANELID)-1;
    return true;
    }
  }
else
  return  false;
}

//--------  Get Current voice   ----------------------------------------
bool
Text2Speech::getCurrentVoice(StlString &voiceNameOut) const
{
if(Text2Speech::_initSuccess && (_channelID!=(TTSCHANELID)-1))
  {
  voiceNameOut=_currentVoice;
  return(true);
  }
else
  return(false);
}


//--------  Get Current voice settings  ----------------------------------------

double
Text2Speech::getVoicePitch(void) const
{
if(Text2Speech::_initSuccess && _channelID!=(TTSCHANELID)-1 /*&& !_pInstanceData*/)
  {
  double  ttspitch,vol,speed;
  if (_getVoiceParams(_channelID, ttspitch,vol,speed))
                return (ttspitch);
  }
return -1;
}

bool
Text2Speech::setVoicePitch(double ttspitch)
{
if(Text2Speech::_initSuccess && _channelID!=(TTSCHANELID)-1/* && !_pInstanceData*/)
  {
  if (ttspitch>1) ttspitch=1;
  else if (ttspitch<0) ttspitch=0;
  _voicePitch=ttspitch;
  return _setVoiceParams(_channelID,_voicePitch,-1,-1);
  }
return false;
}

double
Text2Speech::getVoiceVolume(void) const
{
if(Text2Speech::_initSuccess && _channelID!=(TTSCHANELID)-1/* && !_pInstanceData*/)
  {
  double pitch,vol,speed;
  if (_getVoiceParams(_channelID, pitch,vol,speed))
             return (vol);
  }
return -1;
}

bool
Text2Speech::setVoiceVolume(double volume)
{
if(Text2Speech::_initSuccess && _channelID!=(TTSCHANELID)-1/* && !_pInstanceData*/)
  {
  if (volume>1) volume=1;
  else if (volume<0) volume=0;
  _voiceVolume=volume;
  return _setVoiceParams(_channelID, -1,volume,-1);
  }
return false;
}

double
Text2Speech::getVoiceSpeed(void) const
{
if(Text2Speech::_initSuccess && _channelID!=(TTSCHANELID)-1/* && !_pInstanceData*/)
  {
  double pitch,vol,speed;
  if (_getVoiceParams(_channelID, pitch,vol,speed))
             return speed;
  }
return -1;
}

bool
Text2Speech::setVoiceSpeed(double speed)
{
if(Text2Speech::_initSuccess && _channelID!=(TTSCHANELID)-1/* && !_pInstanceData*/)
  {
  if (speed>1) speed=1;
  else if (speed<0) speed=0;
  _voiceSpeed=speed;
  return _setVoiceParams(_channelID, -1,-1,speed);
  }
return false;
}


// Select input text to say encoding
// default is ISO-8859-1
bool
Text2Speech::setTextEncoding(const StlString &encoding)
{
if(Text2Speech::_initSuccess)
  {
  StlVector<StlString> list;
  _getEncodingSupportList(list);
  if(find(list.begin(),list.end(),encoding)!=list.end())
    {
    _encoding=encoding;
    return(true);
    }
  }
return false;
}

// get Selected text encoding
StlString
Text2Speech::getTextEncoding(void) const
{
return(_encoding);
}

//-------- Sound Speaker -----------------------------------------------------

// All generated sounds will be played by this dataSound3D
ArConstRef<DataSound3D> // Current 3D player
Text2Speech::getSound3D(void) const
{
return(_dataSound);
}

ArRef<DataSound3D> // Current 3D player
Text2Speech::accessSound3D(void) const
{
return(_dataSound);
}

// All generated sounds will be played by this soundPlayer
ArConstRef<SoundPlayer>
Text2Speech::getSoundPlayer(void) const
{
return(_soundPlayer);
}

ArRef<SoundPlayer>
Text2Speech::accessSoundPlayer(void) const
{
return(_soundPlayer);
}

} // namespace AReVi

//----------------------------------------------------------------------------
