// LI2/ENIB - AReVi - 3D services
//----------------------------------------------------------------------------
// file        : AReVi/Lib3D/base3D.cpp
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/Lib3D/base3D.h"

#include "AReVi/Lib3D/index3D.h"
#include "AReVi/Utils/abstractStream.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class Base3D
//----------------------------------------------------------------------------

AR_CLASS_DEF(Base3D,ArObject)

//-------- Construction / Destruction ----------------------------------------

Base3D::Base3D(ArCW & arCW) // Protected
: ArObject(arCW),
  _base(true),
  _motionCB(thisRef()),
  _motionMask(0),
  _moving(false),
  _attachInfos(),
  _oldPos(0.0,0.0,0.0),
  _indexes()
{
  // Nothing to be done
}

Base3D::~Base3D(void) // Protected
{
while(!_attachInfos.empty())
  {
  if(_attachInfos.back()->src.c_ptr()==this)
    {
    _attachInfos.back()->dst->attachTo(thisRef(),Base3D::ATTACH_NONE);
    }
  else
    {
    attachTo(_attachInfos.back()->src,Base3D::ATTACH_NONE);
    }
  }
while(!_indexes.empty())
  {
  _indexes.back().index->removeElement(thisRef());
  }
}

//-------- Transformations ---------------------------------------------------

void
Base3D::globalToLocalLocation(ArRef<Base3D> baseInOut) const
{
_base.globalToLocalLocation(baseInOut->_base);
}

void
Base3D::localToGlobalLocation(ArRef<Base3D> baseInOut) const
{
_base.localToGlobalLocation(baseInOut->_base);
}

void
Base3D::globalToLocalPosition(double & xInOut,
                              double & yInOut,
                              double & zInOut) const
{
_base.globalToLocalPosition(xInOut,yInOut,zInOut);
}

void
Base3D::localToGlobalPosition(double & xInOut,
                              double & yInOut,
                              double & zInOut) const
{
_base.localToGlobalPosition(xInOut,yInOut,zInOut);
}

void
Base3D::globalToLocalVector(double & vxInOut,
                            double & vyInOut,
                            double & vzInOut) const
{
_base.globalToLocalVector(vxInOut,vyInOut,vzInOut);
}

void
Base3D::localToGlobalVector(double & vxInOut,
                            double & vyInOut,
                            double & vzInOut) const
{
_base.localToGlobalVector(vxInOut,vyInOut,vzInOut);
}

void
Base3D::globalToLocalOrientation(double & rollInOut,
                                 double & pitchInOut,
                                 double & yawInOut) const
{
_base.globalToLocalOrientation(rollInOut,pitchInOut,yawInOut);
}

void
Base3D::localToGlobalOrientation(double & rollInOut,
                                 double & pitchInOut,
                                 double & yawInOut) const
{
_base.localToGlobalOrientation(rollInOut,pitchInOut,yawInOut);
}

void
Base3D::getLocalX(double & xOut,
                  double & yOut,
                  double & zOut) const
{
_base.getLocalX(xOut,yOut,zOut);
}

void
Base3D::getLocalY(double & xOut,
                  double & yOut,
                  double & zOut) const
{
_base.getLocalY(xOut,yOut,zOut);
}

void
Base3D::getLocalZ(double & xOut,
                  double & yOut,
                  double & zOut) const
{
_base.getLocalZ(xOut,yOut,zOut);
}

//-------- Location ----------------------------------------------------------

void
Base3D::setLocation(ArConstRef<Base3D> base)
{
_base.setMatrix(base->_base.matrix);
_onMotion();
}

void
Base3D::move(ArConstRef<Base3D> base)
{
_base.move(base->_base);
_onMotion();
}

void
Base3D::invertLocation(void)
{
_base.invert();
_onMotion();
}

bool // success
Base3D::adjustOrthonormality(void)
{
bool result=_base.adjustOrthonormality();
_onMotion();
return(result);
}

//-------- Position ----------------------------------------------------------

void
Base3D::getPosition(double & pxOut,
                    double & pyOut,
                    double & pzOut) const
{
_base.getPosition(pxOut,pyOut,pzOut);
}

void
Base3D::setPosition(double px,
                    double py,
                    double pz)
{
_base.setPosition(px,py,pz);
_onMotion();
}

void
Base3D::translate(double dx,
                  double dy,
                  double dz)
{
_base.translate(dx,dy,dz);
_onMotion();
}

//-------- Orientation -------------------------------------------------------

void
Base3D::extractOrientation(double & rollOut,
                           double & pitchOut,
                           double & yawOut) const
{
_base.extractOrientation(rollOut,pitchOut,yawOut);
}

void
Base3D::extractRotation(double & xOut,
                        double & yOut,
                        double & zOut,
                        double & angleOut) const
{
_base.extractRotation(xOut,yOut,zOut,angleOut);
}

void
Base3D::cancelRotation(void)
{
_base.cancelRotation();
_onMotion();
}

void
Base3D::setOrientation(double roll,
                       double pitch,
                       double yaw)
{
_base.setOrientation(roll,pitch,yaw);
_onMotion();
}

void
Base3D::setRotation(double x,
                    double y,
                    double z,
                    double angle)
{
_base.setRotation(x,y,z,angle);
_onMotion();
}

void
Base3D::rotate(double x,
               double y,
               double z,
               double dAngle)
{
_base.rotate(x,y,z,dAngle);
_onMotion();
}

void
Base3D::roll(double dRoll)
{
_base.roll(dRoll);
_onMotion();
}

void
Base3D::pitch(double dPitch)
{
_base.pitch(dPitch);
_onMotion();
}

void
Base3D::yaw(double dYaw)
{
_base.yaw(dYaw);
_onMotion();
}

//-------- Event management --------------------------------------------------

void
Base3D::maskMotion(void)
{
++_motionMask;
}

void
Base3D::unmaskMotion(void)
{
--_motionMask;
_onMotion();
}

//-------- Attachment --------------------------------------------------------

void
Base3D::attachTo(ArRef<Base3D> base,
                 int attachment)
{
if(base.c_ptr()!=this)
  {
  attachment&=Base3D::ATTACH_ALL;
  Base3D::AttachInfo * info=(Base3D::AttachInfo *)0;
  unsigned int i=_attachInfos.size();
  while(i--)
    {
    if(_attachInfos[i]->src==base)
      {
      info=_attachInfos[i];
      break;
      }
    }
  if(info)
    {
    if(attachment)
      {
      info->attachment=attachment;
      }
    else
      {
      StlVectorFastErase(_attachInfos,i);
      for(i=base->_attachInfos.size();i--;)
        {
        if(base->_attachInfos[i]==info)
          {
          StlVectorFastErase(base->_attachInfos,i);
          break;
          }
        }
      delete info;
      }
    }
  else if(attachment)
    {
    Base3D::AttachInfo * info=new Base3D::AttachInfo(base,thisPtr(),
                                                     attachment);
    _attachInfos.push_back(info);
    base->_attachInfos.push_back(info);
    }
  }
}

int // attachement to base
Base3D::getAttachmentTo(ArConstRef<Base3D> base) const
{
for(unsigned int i=_attachInfos.size();i--;)
  {
  if(_attachInfos[i]->src==base)
    {
    return(_attachInfos[i]->attachment);
    }
  }
return(Base3D::ATTACH_NONE);
}

void
Base3D::getAttachments(StlVector<ArRef<Base3D> > & srcOut,
                       StlVector<ArRef<Base3D> > & dstOut) const
{
srcOut.clear();
dstOut.clear();
for(unsigned int i=_attachInfos.size();i--;)
  {
  if(_attachInfos[i]->src.c_ptr()==this)
    {
    dstOut.push_back(_attachInfos[i]->dst);
    }
  else
    {
    srcOut.push_back(_attachInfos[i]->src);
    }
  }
}

//-------- Detection ---------------------------------------------------------

void
Base3D::viewFirst(double yawVision,
                  double pitchVision,
                  double range,
                  const StlVector<ArRef<Base3D> > & candidates,
                  ArRef<Base3D> & elemOut,
                  double & yawAngleOut,
                  double & pitchAngleOut,
                  double & distanceOut) const
{
elemOut.clear();
yawAngleOut=0.0;
pitchAngleOut=0.0;
distanceOut=-1.0;
yawVision=(yawVision>0.0 ? yawVision*0.5 : M_PI);
pitchVision=(pitchVision>0.0 ? pitchVision*0.5 : M_PI_2);
range=(range>0.0 ? range*range : DBL_MAX);
for(unsigned int l=candidates.size();l--;)
  {
  ArRef<Base3D> elem=candidates[l];
  if(elem.c_ptr()!=this)
    {
    double x,y,z;
    elem->_base.getPosition(x,y,z);
    globalToLocalPosition(x,y,z);
    double d=x*x+y*y+z*z;
    if((d<range)&&(!elemOut||(d<distanceOut)))
      {
      double dxy=sqrt(d-z*z);
      double yaw=(dxy ? atan2(y,x) : 0.0);
      if(fabs(yaw)<=yawVision)
        {
        double pitch=(d ? -atan2(z,dxy) : 0.0);
        if(fabs(pitch)<=pitchVision)
          {
          elemOut=elem->thisRef();
          yawAngleOut=yaw;
          pitchAngleOut=pitch;
          distanceOut=d;
          }
        }
      }
    }
  }
if(elemOut)
  {
  distanceOut=sqrt(distanceOut);
  }
}

void
Base3D::view(double yawVision,
             double pitchVision,
             double range,
             const StlVector<ArRef<Base3D> > & candidates,
             StlVector<ArRef<Base3D> > & elemsOut,
             StlVector<double> & yawAnglesOut,
             StlVector<double> & pitchAnglesOut,
             StlVector<double> & distancesOut) const
{
elemsOut.clear();
yawAnglesOut.clear();
pitchAnglesOut.clear();
distancesOut.clear();
yawVision=(yawVision>0.0 ? yawVision*0.5 : M_PI);
pitchVision=(pitchVision>0.0 ? pitchVision*0.5 : M_PI_2);
range=(range>0.0 ? range*range : DBL_MAX);
for(unsigned int l=candidates.size();l--;)
  {
  ArRef<Base3D> elem=candidates[l];
  if(elem.c_ptr()!=this)
    {
    double x,y,z;
    elem->_base.getPosition(x,y,z);
    globalToLocalPosition(x,y,z);
    double d=x*x+y*y+z*z;
    if(d<range)
      {
      double dxy=sqrt(d-z*z);
      double yaw=(dxy ? atan2(y,x) : 0.0);
      if(fabs(yaw)<=yawVision)
        {
        double pitch=(d ? -atan2(z,dxy) : 0.0);
        if(fabs(pitch)<=pitchVision)
          {
          d=sqrt(d);
          unsigned int min=0;
          unsigned int max=distancesOut.size();
          while(min!=max)
            {
            unsigned int mid=(min+max)>>1;
            if(d<distancesOut[mid])
              {
              max=mid;
              }
            else
              {
              min=mid+1;
              }
            }
          yawAnglesOut.insert(yawAnglesOut.begin()+min,yaw);
          pitchAnglesOut.insert(pitchAnglesOut.begin()+min,pitch);
          distancesOut.insert(distancesOut.begin()+min,d);
          elemsOut.insert(elemsOut.begin()+min,elem);
          }
        }
      }
    }
  }
}

//-------- Serialization operations ------------------------------------------

void
Base3D::serialize(SerializationDependencies & dependenciesOut,
                  ArRef<AbstractOStream> output) const
{
ArObject::serialize(dependenciesOut,output);
for(unsigned int i=16;i--;)
  {
  output->writeDouble(_base.matrix[i]);
  }
}

bool // success
Base3D::unserialize(const SerializationDependencies & dependencies,
                    ArRef<AbstractIStream> input)
{
if(!ArObject::unserialize(dependencies,input))
  {
  return(false);
  }
double matrix[16];
for(unsigned int i=16;i--;)
  {
  if(!input->readDouble(matrix[i]))
    {
    return(false);
    }
  }
_base.setMatrix(matrix);
_onMotion();
return(true);
}

//-------- Forbidden operations ----------------------------------------------

void
Base3D::_onMotion(void) // Protected
{
if(!_motionMask)
  {
  ArRef<Base3D> flag(this);
  _moving=true;
  if(!_motionCB.empty())
    {
    Base3D::MotionEvent evt;
    _motionCB.fire(evt);
    }
  if(flag&&!_attachInfos.empty())
    {
    for(unsigned int i=_attachInfos.size();i--;)
      {
      if(i<_attachInfos.size()) // not removed
        {
        Base3D::AttachInfo & info=*_attachInfos[i];
        if((info.src.c_ptr()==this)&&!info.dst->_moving)
          {
          info.propagating=true;
          info.dst->maskMotion();
          info.dst->_base=_base;
          info.dst->_base.translate(info.relT.x,info.relT.y,info.relT.z);
          info.dst->_base.yaw(info.relR.z);
          info.dst->_base.pitch(info.relR.y);
          info.dst->_base.roll(info.relR.x);
          if(info.attachment!=Base3D::ATTACH_ALL)
            {
            if((~info.attachment)&Base3D::ATTACH_TRANSLATION)
              {
              double x,y,z;
	      info.dst->_base.getPosition(x,y,z);
              if(!(info.attachment&Base3D::ATTACH_TX))
                {
                x=info.absT.x;
                }
              if(!(info.attachment&Base3D::ATTACH_TY))
                {
                y=info.absT.y;
                }
              if(!(info.attachment&Base3D::ATTACH_TZ))
                {
                z=info.absT.z;
                }
              info.dst->_base.setPosition(x,y,z);
              }
            if((~info.attachment)&Base3D::ATTACH_ROTATION)
              {
              double roll,pitch,yaw;
	      info.dst->_base.extractOrientation(roll,pitch,yaw);
              if(!(info.attachment&Base3D::ATTACH_ROLL))
                {
                roll=info.absR.x;
                }
              if(!(info.attachment&Base3D::ATTACH_PITCH))
                {
                pitch=info.absR.y;
                }
              if(!(info.attachment&Base3D::ATTACH_YAW))
                {
                yaw=info.absR.z;
                }
              info.dst->_base.setOrientation(roll,pitch,yaw);
              }
            }
          info.dst->unmaskMotion();
          if(flag)
            {
            info.propagating=false;
            }
          else
            {
            break;
            }
          }
        else if(!info.propagating)
          {
          info.dst->_base.getPosition(info.absT.x,
                                      info.absT.y,
                                      info.absT.z);
          info.dst->_base.extractOrientation(info.absR.x,
                                             info.absR.y,
                                             info.absR.z);
          info.relT=info.absT;
          info.relR=info.absR;
          info.src->_base.globalToLocalPosition(info.relT.x,
                                                info.relT.y,
                                                info.relT.z);
          info.src->_base.globalToLocalOrientation(info.relR.x,
                                                   info.relR.y,
                                                   info.relR.z);
          }
        }
      }
    }
  if(flag)
    {
    if(!_indexes.empty())
      {
      for(unsigned int i=_indexes.size();i--;)
        {
        Base3D::IndexInfo & info=_indexes[i];
        _indexes[i].index->_onMotion(this,info.gridData,
                                     info.factor,info.level);
        }
      _base.getPosition(_oldPos.x,_oldPos.y,_oldPos.z);
      }
    _moving=false;
    }
  }
}

void
Base3D::_addIndex(ArRef<Index3D> index,
                  void * gridData,
                  double precision,
                  double factor,
                  int level)
{
_indexes.push_back(Base3D::IndexInfo(index,gridData,precision,factor,level));
}

void
Base3D::_removeIndex(ArRef<Index3D> index)
{
for(unsigned int i=_indexes.size();i--;)
  {
  if(_indexes[i].index==index)
    {
    StlVectorFastErase(_indexes,i);
    break;
    }
  }
}

Base3D::AttachInfo::AttachInfo(ArPtr<Base3D> s,
                               ArRef<Base3D> d,
                               int a)
: src(s),
  dst(d),
  attachment(a),
  propagating(false),
  absT(),
  absR(),
  relT(),
  relR()
{
dst->_base.getPosition(absT.x,absT.y,absT.z);
dst->_base.extractOrientation(absR.x,absR.y,absR.z);
relT=absT;
relR=absR;
src->_base.globalToLocalPosition(relT.x,relT.y,relT.z);
src->_base.globalToLocalOrientation(relR.x,relR.y,relR.z);
}

} // namespace AReVi

//----------------------------------------------------------------------------
