// LI2/ENIB - AReVi - 3D services
//----------------------------------------------------------------------------
// file        : AReVi/Lib3D/boundingBox3D.cpp
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/Lib3D/boundingBox3D.h"

#include "AReVi/Lib3D/transform3D.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class BoundingBox3D
//----------------------------------------------------------------------------

AR_CLASS_DEF(BoundingBox3D,ArObject)

//-------- Construction / Destruction ----------------------------------------

BoundingBox3D::BoundingBox3D(ArCW & arCW) // Protected
: ArObject(arCW),
  _bbox()
{
  // Nothing to be done
}

BoundingBox3D::~BoundingBox3D(void) // Protected
{
  // Nothing to be done
}

//-------- Geometry ----------------------------------------------------------

void
BoundingBox3D::invalidate(void)
{
_bbox.valid=false;
}

bool // bounds are significant
BoundingBox3D::isValid(void) const
{
return(_bbox.valid);
}

void
BoundingBox3D::addXYZ(double x,
                      double y,
                      double z)
{
_bbox.addXYZ(x,y,z);
}

void
BoundingBox3D::addBox(ArConstRef<BoundingBox3D> bbox)
{
_bbox.addBox(bbox->_bbox);
}

void
BoundingBox3D::addTransformedBox(ArConstRef<Transform3D> transform,
                                 ArConstRef<BoundingBox3D> bbox)
{
_bbox.addTransformedBox(transform->getTransformation(),bbox->_bbox);
}

bool // bbox is valid
BoundingBox3D::getBounds(double & xminOut,
                         double & yminOut,
                         double & zminOut,
                         double & xmaxOut,
                         double & ymaxOut,
                         double & zmaxOut) const
{
if(_bbox.valid)
  {
  xminOut=_bbox.xmin;
  yminOut=_bbox.ymin;
  zminOut=_bbox.zmin;
  xmaxOut=_bbox.xmax;
  ymaxOut=_bbox.ymax;
  zmaxOut=_bbox.zmax;
  return(true);
  }
else return(false);
}

bool // bbox is valid
BoundingBox3D::getSize(double & sizeXOut,
                       double & sizeYOut,
                       double & sizeZOut) const
{
if(_bbox.valid)
  {
  sizeXOut=_bbox.sizeX();
  sizeYOut=_bbox.sizeY();
  sizeZOut=_bbox.sizeZ();
  return(true);
  }
else return(false);
}

double // size of smallest side
BoundingBox3D::getMinSize(void) const
{
if(_bbox.valid)
  {
  double minSize=_bbox.sizeX();
  double size=_bbox.sizeY();
  if(size<minSize) minSize=size;
  size=_bbox.sizeZ();
  if(size<minSize) minSize=size;
  return(minSize);
  }
else return(0.0);
}

double // size of biggest side
BoundingBox3D::getMaxSize(void) const
{
if(_bbox.valid)
  {
  double maxSize=_bbox.sizeX();
  double size=_bbox.sizeY();
  if(size>maxSize) maxSize=size;
  size=_bbox.sizeZ();
  if(size>maxSize) maxSize=size;
  return(maxSize);
  }
else return(0.0);
}

bool // bbox is valid
BoundingBox3D::getCenter(double & centerXOut,
                         double & centerYOut,
                         double & centerZOut) const
{
if(_bbox.valid)
  {
  centerXOut=_bbox.centerX();
  centerYOut=_bbox.centerY();
  centerZOut=_bbox.centerZ();
  return(true);
  }
else return(false);
}

} // namespace AReVi

//----------------------------------------------------------------------------
