// LI2/ENIB - AReVi - 3D services
//----------------------------------------------------------------------------
// file        : AReVi/Lib3D/dataSound3D.cpp
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/Lib3D/dataSound3D.h"

#include "AReVi/arSystem.h"
#include "AReVi/Utils/memoryBlock.h"
#include "AReVi/Utils/abstractStream.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class DataSound3D
//----------------------------------------------------------------------------

AR_CLASS_DEF(DataSound3D,Sound3D)

//-------- Construction / Destruction ----------------------------------------

DataSound3D::DataSound3D(ArCW & arCW) // Protected
: Sound3D(arCW),
  _buffers(),
  _pushBufferCB(thisPtr()),
  _popBufferCB(thisPtr())
{
_soundData=new StlVector<Sound3D::SoundData>();
}

DataSound3D::~DataSound3D(void) // Protected
{
// Nothing to be done
}

//-------- Sound properties --------------------------------------------------

bool
DataSound3D::pushBuffer(ArConstRef<MemoryBlock> buffer, // uncompressed data!
                         unsigned int freq,
                         UtilSound::SoundFormat format)
{
if(Sound3D::_pushBuffer(buffer,freq,format))
  {
  // Backup memoryBlock
  ArRef<MemoryBlock> mem=MemoryBlock::NEW();
  mem->setSize(buffer->getSize());
  memcpy(mem->accessMemory(),buffer->getMemory(),buffer->getSize());
  _buffers.push_back(mem);

  if(!_pushBufferCB.empty())
  {
  DataSound3D::PushBufferEvent evt;
  evt.buffer=mem;
  evt.frequency=freq;
  evt.format=format;
  _pushBufferCB.fire(evt);
  }
  return true;
  }
else
  return false;
}

unsigned int // nb played buffers
DataSound3D::popBuffers(void)
{
unsigned nbBuffers=_freeBuffers();

for (unsigned i=nbBuffers;i--;)
  _buffers.erase(_buffers.begin());

if(!_popBufferCB.empty())
  {
  DataSound3D::PopBufferEvent evt;
  _popBufferCB.fire(evt);
  }

return nbBuffers;
}

unsigned int // nb pending buffers
DataSound3D::getNbPendingBuffers(void) const
{
int nbBuffers,nbProcessed;
(*Sound3D::_getSourceBuffersStateFnct)(_sourceID,nbBuffers,nbProcessed);
return nbBuffers-nbProcessed;
}

//-------- Serialization operations ------------------------------------------

void
DataSound3D::serialize(SerializationDependencies & dependenciesOut,
                   ArRef<AbstractOStream> output) const
{
Sound3D::serialize(dependenciesOut,output);

output->writeUInt(_buffers.size());
for (unsigned i=0;i<_buffers.size();i++)
  {
  // compress data?
  output->writeUInt(_buffers[i]->getSize());
  output->writeData(_buffers[i]->getMemory(),_buffers[i]->getSize());

  output->writeUInt((*_soundData)[i].freq);
  output->writeUInt((unsigned int)((*_soundData)[i].format));
  }
}

bool // success
DataSound3D::unserialize(const SerializationDependencies & dependencies,
                     ArRef<AbstractIStream> input)
{
unsigned int nbBuffers;
// Destroy all olders buffers.
stop();
popBuffers();

// begin deserialization
if(!Sound3D::unserialize(dependencies,input) ||
   !input->readUInt(nbBuffers))
  {
  return(false);
  }

for (unsigned i=nbBuffers;i--;)
  {
  // compress data?
  unsigned dataSize,frequency,format;
  ArRef<MemoryBlock> mem=MemoryBlock::NEW();

  if (!input->readUInt(dataSize))
    return false;

  mem->setSize(dataSize);
  if (!input->readData(mem->accessMemory(),dataSize) ||
      !input->readUInt(frequency) ||
      !input->readUInt(format))
    return false;

  // no call to pushbuffer => no need to recopy!!!
  if(Sound3D::_pushBuffer(mem,frequency,(UtilSound::SoundFormat)format))
    {
    _buffers.push_back(mem);

    if(!_pushBufferCB.empty())
      {
      DataSound3D::PushBufferEvent evt;
      evt.buffer=mem;
      evt.frequency=frequency;
      evt.format=(UtilSound::SoundFormat)format;
      _pushBufferCB.fire(evt);
      }
    }
  else
    return false;
  }

return(true);
}

} // namespace AReVi

//----------------------------------------------------------------------------
