// LI2/ENIB - AReVi - 3D services
//----------------------------------------------------------------------------
// file        : AReVi/Lib3D/llw_priv.cpp
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/Lib3D/llw_priv.h"
#include "AReVi/Lib3D/gl_priv.h"

#include "AReVi/arSystem.h"
#include "AReVi/Utils/urlStream.h"

#include <stdio.h>
#include <string.h>

namespace AReVi {

//----------------------------------------------------------------------------
// Class LLW (abstract) - X11/Win32 common part
//----------------------------------------------------------------------------

typedef StlList<LLW::Event *>
        LLW_EventList;

struct LLW_TimeOut
{
unsigned int limit;
LLW::AreaID area;
};

typedef StlList<LLW_TimeOut *>
        LLW_TimeOutList;

struct LLW_AreaData
{
LLW_AreaData(void) { }
virtual ~LLW_AreaData(void) { }
virtual bool isDrawable(void) const=0;
virtual bool isWindow(void) const=0;
virtual bool isPixmap(void) const=0;
virtual bool isGLBuffer(void) const=0;
unsigned int width;
unsigned int height;
};

struct LLW_DrawData : public LLW_AreaData
{
LLW_DrawData(void) { }
virtual ~LLW_DrawData(void) { }
virtual bool isDrawable(void) const { return(true); }
virtual bool isWindow(void) const=0;
virtual bool isPixmap(void) const=0;
virtual bool isGLBuffer(void) const { return(false); }
};

struct LLW_PixData;

struct LLW_WinData : public LLW_DrawData
{
LLW_WinData(void) : LLW_DrawData() { }
virtual ~LLW_WinData(void) { }
virtual bool isWindow(void) const { return(true); }
virtual bool isPixmap(void) const { return(false); }
LLW_WinData * parent;
StlVector<LLW_WinData *> children;
LLW::CursorID cursor;
StlString title;
LLW::EventMask mask;
bool keyRepeat;
bool mapped;
int x,y;
unsigned int minWidth,minHeight;
unsigned int backColor;
LLW_PixData * backPixmap;
int extWin;
};

struct LLW_PixData : public LLW_DrawData
{
LLW_PixData(void) : LLW_DrawData() { }
virtual ~LLW_PixData(void) { }
virtual bool isWindow(void) const { return(false); }
virtual bool isPixmap(void) const { return(true); }
};

struct LLW_GLBufData : public LLW_AreaData
{
LLW_GLBufData(void) { }
virtual ~LLW_GLBufData(void) { }
virtual bool isDrawable(void) const { return(false); }
virtual bool isWindow(void) const { return(false); }
virtual bool isPixmap(void) const { return(false); }
virtual bool isGLBuffer(void) const { return(true); }
unsigned int tex;
unsigned int fb;
unsigned int depth;
};

struct LLW_GLFontSymbol
{
unsigned char nbVertices;
unsigned char nbTriangles;
unsigned char nbLines;
double * vertices;
unsigned char * triangles;
unsigned char * lines;
double xmin,xmax,ymin,ymax;
};

struct LLW_GLFontData
{
StlString url;
LLW_GLFontSymbol * symbols[256];
double ascent,descent;
unsigned int count;
};

typedef StlMap<StlString,LLW_GLFontData *,StlLess<StlString> >
        LLW_GLFontDataMap;

struct LLW_BMFontSymbol
{
unsigned int width;
unsigned char * pixels;
};

struct LLW_BMFontData
{
StlString url;
LLW_BMFontSymbol * symbols[256];
unsigned int ascent,descent;
unsigned int count;
};

typedef StlMap<StlString,LLW_BMFontData *,StlLess<StlString> >
        LLW_BMFontDataMap;

enum LLW_InitState { LLW_DOWN, LLW_GOING_UP, LLW_UP, LLW_GOING_DOWN };

static LLW_InitState LLW_initState=LLW_DOWN;

LLW::ImageLoaderFnctVect * LLW::_imageLoaderFncts=
                           (LLW::ImageLoaderFnctVect *)0;

LLW::ImageSaverFnctVect * LLW::_imageSaverFncts=
                          (LLW::ImageSaverFnctVect *)0;

bool LLW::_initJpegLibSuccess=false;

bool (* LLW::_compressRGBImageToJPEG)(ArConstRef<MemoryBlock>,
                                      ArRef<MemoryBlock>,
                                      unsigned int,
                                      unsigned int,
                                      unsigned int)=
    (bool (*)(ArConstRef<MemoryBlock>,ArRef<MemoryBlock>,
              unsigned int,unsigned int,unsigned int))0;

bool (* LLW::_uncompressRGBImageFromJPEG)(ArConstRef<MemoryBlock>,
                                          ArRef<MemoryBlock>,
                                          unsigned int &,
                                          unsigned int &)=
    (bool (*)(ArConstRef<MemoryBlock>,ArRef<MemoryBlock>,
              unsigned int &,unsigned int &))0;

static LLW_EventList * LLW_pendingEvent=(LLW_EventList *)0;

static LLW_EventList * LLW_freeEvent=(LLW_EventList *)0;

static LLW_TimeOutList * LLW_timeOutList=(LLW_TimeOutList *)0;

static LLW_AreaData * LLW_currentGLTarget=(LLW_AreaData *)0;

static LLW_WinData * LLW_mouseWindow=(LLW_WinData *)0;

static LLW_GLFontDataMap * LLW_glFontDataMap=(LLW_GLFontDataMap *)0;

static LLW_BMFontDataMap * LLW_bmFontDataMap=(LLW_BMFontDataMap *)0;

inline static
LLW_GLFontData *
LLW_getGLFontData(const StlString & url)
{
LLW_GLFontDataMap::iterator it=LLW_glFontDataMap->find(url);
if(it==LLW_glFontDataMap->end())
  {
  return((LLW_GLFontData *)0);
  }
else
  {
  return((*it).second);
  }
}

inline static
LLW_BMFontData *
LLW_getBMFontData(const StlString & url)
{
LLW_BMFontDataMap::iterator it=LLW_bmFontDataMap->find(url);
if(it==LLW_bmFontDataMap->end())
  {
  return((LLW_BMFontData *)0);
  }
else
  {
  return((*it).second);
  }
}

//-------- Window creation/destruction ---------------------------------------

static
LLW::WinID
LLW_createWindow(LLW::WinID parent,
                 int externalWindowID,
                 const StlString & title,
                 bool managed,
                 int x,
                 int y,
                 unsigned int width,
                 unsigned int height);

LLW::WinID
LLW::createTopWindow(const StlString & title,
                     bool managed,
                     int x,
                     int y,
                     unsigned int width,
                     unsigned int height) // Static
{
return(LLW_createWindow((LLW::WinID)0,0,title,managed,
                        x,y,width,height));
}

LLW::WinID
LLW::createChildWindow(LLW::WinID parent,
                       int x,
                       int y,
                       unsigned int width,
                       unsigned int height) // Static
{
return(LLW_createWindow(parent,0,"",false,
                        x,y,width,height));
}

LLW::WinID
LLW::createEmbeddedWindow(int externalWindowID,
                          int x,
                          int y,
                          unsigned int width,
                          unsigned int height) // Static
{
return(LLW_createWindow((LLW::WinID)0,externalWindowID,"",false,
                        x,y,width,height));
}

static
void
LLW_destroyWindow(LLW::WinID window);

void
LLW::destroyWindow(LLW::WinID window,
                   bool recursive)
{
LLW_WinData * winData=(LLW_WinData *)window;
if(winData==LLW_mouseWindow)
  {
  LLW_mouseWindow=(LLW_WinData *)0;
  }
if(winData->parent)
  {
  for(unsigned int i=winData->parent->children.size();i--;)
    {
    if(winData->parent->children[i]==winData)
      {
      StlVectorFastErase(winData->parent->children,i);
      break;
      }
    }
  }
while(!winData->children.empty())
  {
  if(recursive)
    {
    LLW::destroyWindow(winData->children.back(),true);
    }
  else
    {
    cerr << "!!!!  LLW::WinID " << ptrToStr(winData->children.back())
         << " is loosing its parent  !!!!" << endl;
    winData->children.back()->parent=(LLW_WinData *)0;
    winData->children.pop_back();
    }
  }
LLW_destroyWindow(window);
}

//-------- Pixmap creation/destruction ---------------------------------------

LLW::PixID
LLW::createImagePixmap(const StlString & url) // Static
{
unsigned int width,height,components;
ArRef<MemoryBlock> data=MemoryBlock::NEW();
if(!LLW::loadImage(url,width,height,components,data))
  {
  return((LLW::PixID)0);
  }
char * ptr=data->accessMemory();
LLW::PixID pixmap=LLW::createPixmap(width,height);
if(pixmap)
  {
  switch(components)
    {
    case 1:
      {
      for(unsigned int y=height;y--;)
        {
        for(unsigned int x=width;x--;)
          {
          unsigned int index=x+(width*y);
          unsigned int color=LLW::makeByteColor(ptr[index],
                                                ptr[index],
                                                ptr[index]);
          LLW::drawPoint(pixmap,color,x,height-1-y);
          }
        }
      break;
      }
    case 2:
      {
      for(unsigned int y=height;y--;)
        {
        for(unsigned int x=width;x--;)
          {
          unsigned int index=2*(x+(width*y));
          unsigned int color=LLW::makeByteColor(ptr[index],
                                                ptr[index],
                                                ptr[index]);
          LLW::drawPoint(pixmap,color,x,height-1-y);
          }
        }
      break;
      }
    case 3:
      {
      for(unsigned int y=height;y--;)
        {
        for(unsigned int x=width;x--;)
          {
          unsigned int index=3*(x+(width*y));
          unsigned int color=LLW::makeByteColor(ptr[index],
                                                ptr[index+1],
                                                ptr[index+2]);
          LLW::drawPoint(pixmap,color,x,height-1-y);
          }
        }
      break;
      }
    case 4:
      {
      for(unsigned int y=height;y--;)
        {
        for(unsigned int x=width;x--;)
          {
          unsigned int index=4*(x+(width*y));
          unsigned int color=LLW::makeByteColor(ptr[index],
                                                ptr[index+1],
                                                ptr[index+2]);
          LLW::drawPoint(pixmap,color,x,height-1-y);
          }
        }
      break;
      }
    }
  }
return(pixmap);
}

//-------- GLBuffer creation/destruction -------------------------------------

static
LLW_GLBufData *
LLW_createGLBuffer(unsigned int width,
                   unsigned int height);

LLW::GLBufID
LLW::createGLBuffer(unsigned int width,
                    unsigned int height)
{
unsigned int oldWidth=width;
width=1;
while((width<<1)<=oldWidth) // (width=2^N) <= oldWidth
  {
  width<<=1;
  }
if(width<oldWidth) // (width=2^N) >= oldWidth
  {
  width<<=1;
  }
unsigned int oldHeight=height;
height=1;
while((height<<1)<=oldHeight) // (height=2^N) <= oldHeight
  {
  height<<=1;
  }
if(height<oldHeight) // (height=2^N) >= oldHeight
  {
  height<<=1;
  }
if(width>GL::getMaxTextureSize())
  {
  width=GL::getMaxTextureSize();
  }
if(height>GL::getMaxTextureSize())
  {
  height=GL::getMaxTextureSize();
  }
if(GL::hasFrameBufferObject())
  {
  LLW_GLBufData * bufferData=new LLW_GLBufData;
  bufferData->width=width;
  bufferData->height=height;
  glGenTextures(1,&bufferData->tex);
  glBindTexture(GL_TEXTURE_2D,bufferData->tex);
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
  glTexImage2D(GL_TEXTURE_2D,0,GL_RGBA,
               bufferData->width,bufferData->height,
               0,GL_RGBA,GL_UNSIGNED_BYTE,(const void *)0);
  GL::glGenFramebuffers(1,&bufferData->fb);
  GL::glGenRenderbuffers(1,&bufferData->depth);
  GL::glBindFramebuffer(GL_FRAMEBUFFER_EXT,bufferData->fb);
  GL::glFramebufferTexture2D(GL_FRAMEBUFFER_EXT,GL_COLOR_ATTACHMENT0_EXT,
                             GL_TEXTURE_2D,bufferData->tex,0);
  GL::glBindRenderbuffer(GL_RENDERBUFFER_EXT,bufferData->depth);
  GL::glRenderbufferStorage(GL_RENDERBUFFER_EXT,GL_DEPTH_COMPONENT24,
                            bufferData->width,bufferData->height);
  GL::glFramebufferRenderbuffer(GL_FRAMEBUFFER_EXT,GL_DEPTH_ATTACHMENT_EXT,
                                GL_RENDERBUFFER_EXT,bufferData->depth);
  if(GL::glCheckFramebufferStatus(GL_FRAMEBUFFER_EXT)
     ==GL_FRAMEBUFFER_COMPLETE_EXT)
    {
    GL::glBindFramebuffer(GL_FRAMEBUFFER_EXT,0);
    return(bufferData);
    }
  else
    {
    GL::glBindFramebuffer(GL_FRAMEBUFFER_EXT,0);
    GL::glDeleteRenderbuffers(1,&bufferData->depth);
    GL::glDeleteFramebuffers(1,&bufferData->fb);
    glDeleteTextures(1,&bufferData->tex);
    delete bufferData;
    }
  }
LLW_GLBufData * bufferData=LLW_createGLBuffer(width,height);
if(bufferData)
  {
  glGenTextures(1,&bufferData->tex);
  glBindTexture(GL_TEXTURE_2D,bufferData->tex);
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
  glTexImage2D(GL_TEXTURE_2D,0,GL_RGB,
               bufferData->width,bufferData->height,
               0,GL_RGB,GL_UNSIGNED_BYTE,(const void *)0);
  bufferData->fb=0;
  bufferData->depth=0;
  }
return(bufferData);
}

static
void
LLW_destroyGLBuffer(LLW::GLBufID buffer);

void
LLW::destroyGLBuffer(LLW::GLBufID buffer)
{
LLW_GLBufData * bufferData=(LLW_GLBufData *)buffer;
if(bufferData->fb)
  {
  GL::glDeleteRenderbuffers(1,&bufferData->depth);
  GL::glDeleteFramebuffers(1,&bufferData->fb);
  glDeleteTextures(1,&bufferData->tex);
  delete bufferData;
  }
else
  {
  glDeleteTextures(1,&bufferData->tex);
  LLW_destroyGLBuffer(bufferData);
  }
}

//-------- Area identification -----------------------------------------------

bool // area is a drawable
LLW::areaIsDrawable(LLW::AreaID area)
{
LLW_AreaData * areaData=(LLW_AreaData *)area;
return(areaData->isDrawable());
}

bool // area is a window
LLW::areaIsWindow(LLW::AreaID area)
{
LLW_AreaData * areaData=(LLW_AreaData *)area;
return(areaData->isWindow());
}

bool // area is a pixmap
LLW::areaIsPixmap(LLW::AreaID area)
{
LLW_AreaData * areaData=(LLW_AreaData *)area;
return(areaData->isPixmap());
}

bool // area is a GL buffer
LLW::areaIsGLBuffer(LLW::AreaID area)
{
LLW_AreaData * areaData=(LLW_AreaData *)area;
return(areaData->isGLBuffer());
}

//-------- Window hierarchy --------------------------------------------------

LLW::WinID // this window's parent
LLW::getParentWindow(LLW::WinID window) // Static
{
LLW_WinData * winData=(LLW_WinData *)window;
return((LLW::WinID)winData->parent);
}

LLW::WinID // this window's top window
LLW::getTopWindow(LLW::WinID window) // Static
{
LLW_WinData * winData=(LLW_WinData *)window;
while(winData->parent)
  {
  winData=winData->parent;
  }
return((LLW::WinID)winData);
}

bool // success
LLW::containsChildWindow(LLW::WinID window,
                         LLW::WinID child)
{
LLW_WinData * winData=(LLW_WinData *)child;
do
  {
  winData=winData->parent;
  if(winData==(LLW_WinData *)window)
    {
    return(true);
    }
  } while(winData);
return(false);
}

//-------- Window management -------------------------------------------------

static
void
LLW_updateWindowTitle(LLW::WinID window);

void
LLW::setTopWindowTitle(LLW::WinID window,
                       const StlString & title)
{
LLW_WinData * winData=(LLW_WinData *)window;
if(!winData->parent)
  {
  winData->title=title;
  LLW_updateWindowTitle(window);
  }
}

const StlString & // window title
LLW::getTopWindowTitle(LLW::WinID window) // Static
{
LLW_WinData * winData=(LLW_WinData *)window;
return(winData->title);
}

bool // window is mapped
LLW::isWindowMapped(LLW::WinID window) // Static
{
LLW_WinData * winData=(LLW_WinData *)window;
return(winData->mapped);
}

bool // window and its ancestors are mapped
LLW::isWindowViewable(LLW::WinID window) // Static
{
LLW_WinData * winData=(LLW_WinData *)window;
while(winData->mapped)
  {
  winData=winData->parent;
  if(!winData)
    {
    return(true);
    }
  }
return(false);
}

static
void
LLW_updateWindowBackground(LLW::WinID window);

void
LLW::setWindowBackgroundColor(LLW::WinID window,
                              unsigned int color) // Static
{
LLW_WinData * winData=(LLW_WinData *)window;
winData->backColor=color;
winData->backPixmap=(LLW_PixData *)0;
LLW_updateWindowBackground(window);
LLW::clearWindow(window);
}

unsigned int // background color
LLW::getWindowBackgroundColor(LLW::WinID window) // Static
{
LLW_WinData * winData=(LLW_WinData *)window;
return(winData->backColor);
}

void
LLW::setWindowBackgroundPixmap(LLW::WinID window,
                               LLW::PixID pixmap) // Static
{
LLW_WinData * winData=(LLW_WinData *)window;
winData->backPixmap=(LLW_PixData *)pixmap;
LLW_updateWindowBackground(window);
LLW::clearWindow(window);
}

LLW::PixID // background pixmap
LLW::getWindowBackgroundPixmap(LLW::WinID window) // Static
{
LLW_WinData * winData=(LLW_WinData *)window;
return((LLW::PixID)winData->backPixmap);
}

//-------- Cursor management -------------------------------------------------

LLW::CursorID // cursor for this window
LLW::getWindowCursor(LLW::WinID window) // Static
{
LLW_WinData * winData=(LLW_WinData *)window;
return(winData->cursor);
}

//-------- Geometry ----------------------------------------------------------

static
void
LLW_updateWindowGeometry(LLW::WinID window);

void
LLW::setWindowGeometry(LLW::WinID window,
                       int x,
                       int y,
                       unsigned int width,
                       unsigned int height) // Static
{
LLW_WinData * winData=(LLW_WinData *)window;
winData->x=x;
winData->y=y;
winData->width=(width<winData->minWidth ? winData->minWidth : width);
winData->height=(height<winData->minHeight ? winData->minHeight : height);
LLW_updateWindowGeometry(window);
}

void
LLW::getWindowGeometry(LLW::WinID window,
                       int & xOut,
                       int & yOut,
                       unsigned int & widthOut,
                       unsigned int & heightOut) // Static
{
LLW_WinData * winData=(LLW_WinData *)window;
xOut=winData->x;
yOut=winData->y;
widthOut=winData->width;
heightOut=winData->height;
}

void
LLW::setWindowPosition(LLW::WinID window,
                       int x,
                       int y) // Static
{
LLW_WinData * winData=(LLW_WinData *)window;
winData->x=x;
winData->y=y;
LLW_updateWindowGeometry(window);
}

void
LLW::getWindowPosition(LLW::WinID window,
                       int & xOut,
                       int & yOut) // Static
{
LLW_WinData * winData=(LLW_WinData *)window;
xOut=winData->x;
yOut=winData->y;
}

void
LLW::setWindowSize(LLW::WinID window,
                   unsigned int width,
                   unsigned int height) // Static
{
LLW_WinData * winData=(LLW_WinData *)window;
winData->width=(width<winData->minWidth ? winData->minWidth : width);
winData->height=(height<winData->minHeight ? winData->minHeight : height);
LLW_updateWindowGeometry(window);
}

void
LLW::getWindowSize(LLW::WinID window,
                   unsigned int & widthOut,
                   unsigned int & heightOut) // Static
{
LLW_WinData * winData=(LLW_WinData *)window;
widthOut=winData->width;
heightOut=winData->height;
}

void
LLW::setWindowMinSize(LLW::WinID window,
                      unsigned int minWidth,
                      unsigned int minHeight) // Static
{
LLW_WinData * winData=(LLW_WinData *)window;
winData->minWidth=minWidth;
winData->minHeight=minHeight;
bool update=false;
if(winData->width<winData->minWidth)
  {
  winData->width=winData->minWidth;
  update=true;
  }
if(winData->height<winData->minHeight)
  {
  winData->height=winData->minHeight;
  update=true;
  }
if(update)
  {
  LLW_updateWindowGeometry(window);
  }
}

void
LLW::getWindowMinSize(LLW::WinID window,
                      unsigned int & minWidthOut,
                      unsigned int & minHeightOut) // Static
{
LLW_WinData * winData=(LLW_WinData *)window;
minWidthOut=winData->minWidth;
minHeightOut=winData->minHeight;
}

void
LLW::getPixmapSize(LLW::PixID pixmap,
                   unsigned int & widthOut,
                   unsigned int & heightOut) // Static
{
LLW_PixData * pixData=(LLW_PixData *)pixmap;
widthOut=pixData->width;
heightOut=pixData->height;
}

void
LLW::getGLBufferSize(LLW::GLBufID buffer,
                     unsigned int & widthOut,
                     unsigned int & heightOut)
{
LLW_GLBufData * bufferData=(LLW_GLBufData *)buffer;
widthOut=bufferData->width;
heightOut=bufferData->height;
}

void
LLW::getAreaSize(LLW::AreaID area,
                 unsigned int & widthOut,
                 unsigned int & heightOut) // Static
{
LLW_AreaData * areaData=(LLW_AreaData *)area;
widthOut=areaData->width;
heightOut=areaData->height;
}

//-------- Font management ---------------------------------------------------

static
void
LLW_parseFontName(const StlString & fontName,
                  StlString & nameOut,
                  unsigned int & sizeOut,
                  bool & boldOut,
                  char & slantOut)
{
nameOut.erase();
sizeOut=0;
boldOut=false;
slantOut='r';
if(fontName.size())
  {
  char * str=new char[fontName.size()+1];
  strcpy(str,fontName.c_str());
  bool stop=false;
  char * p;
  char * p1=str;
  char * p2=p1;
  while(*p2&&(*p2!=',')) // seek ',' or '\0'
    {
    ++p2;
    }
  if(*p2)
    {
    *p2='\0';
    }
  else
    {
    stop=true;
    }
  for(p=p1;*p;p++)
    {
    *p=tolower(*p);
    }
  if(p1!=p2)
    {
    nameOut=p1;
    }
  while(!stop)
    {
    p1=++p2;
    while(*p2&&(*p2!=',')) // seek ',' or '\0'
      {
      ++p2;
      }
    if(*p2)
      {
      *p2='\0';
      }
    else
      {
      stop=true;
      }
    for(p=p1;*p;p++)
      {
      *p=tolower(*p);
      }
    unsigned int sz;
    if((sscanf(p1,"%u",&sz)==1)&&(sz>sizeOut))
      {
      sizeOut=sz;
      }
    else if(!strcmp(p1,"bold")||!strcmp(p1,"b"))
      {
      boldOut=true;
      }
    else if(!strcmp(p1,"italic")||!strcmp(p1,"i"))
      {
      slantOut='i';
      }
    else if(!strcmp(p1,"oblique")||!strcmp(p1,"o"))
      {
      slantOut='o';
      }
    }
  delete[] str;
  }
}

//-------- Drawing -----------------------------------------------------------

void
LLW::drawImage(LLW::DrawID dest,
               int destX,
               int destY,
               ArConstRef<MemoryBlock> image,
               unsigned int width,
               unsigned int height,
               unsigned int components) // Static
{
if(image->getSize()>=components*width*height)
  {
  const char * ptr=image->getMemory();
  switch(components)
    {
    case 1: // GRAY
      {
      for(unsigned int y=height;y--;)
        {
        for(unsigned int x=width;x--;)
          {
          unsigned int index=x+(width*y);
          unsigned int color=LLW::makeByteColor(ptr[index],
                                                ptr[index],
                                                ptr[index]);
          LLW::drawPoint(dest,color,destX+x,destY+height-1-y);
          }
        }
      break;
      }
    case 2: // GRAY - ALPHA
      {
      for(unsigned int y=height;y--;)
        {
        for(unsigned int x=width;x--;)
          {
          unsigned int index=2*(x+(width*y));
          if(ptr[index+1])
            {
            unsigned int color=LLW::makeByteColor(ptr[index],
                                                  ptr[index],
                                                  ptr[index]);
            LLW::drawPoint(dest,color,destX+x,destY+height-1-y);
            }
          }
        }
      break;
      }
    case 3: // RED - GREEN - BLUE
      {
      for(unsigned int y=height;y--;)
        {
        for(unsigned int x=width;x--;)
          {
          unsigned int index=3*(x+(width*y));
          unsigned int color=LLW::makeByteColor(ptr[index],
                                                ptr[index+1],
                                                ptr[index+2]);
          LLW::drawPoint(dest,color,destX+x,destY+height-1-y);
          }
        }
      break;
      }
    case 4: // RED - GREEN - BLUE - ALPHA
      {
      for(unsigned int y=height;y--;)
        {
        for(unsigned int x=width;x--;)
          {
          unsigned int index=4*(x+(width*y));
          if(ptr[index+3])
            {
            unsigned int color=LLW::makeByteColor(ptr[index],
                                                  ptr[index+1],
                                                  ptr[index+2]);
            LLW::drawPoint(dest,color,destX+x,destY+height-1-y);
            }
          }
        }
      break;
      }
    }
  }
}

void
LLW::copyDrawable(LLW::DrawID dest,
                  int destX,
                  int destY,
                  LLW::DrawID source) // Static
{
unsigned int w,h;
LLW::getAreaSize(source,w,h);
LLW::copyDrawablePart(dest,destX,destY,source,0,0,w,h);
}

//-------- OpenGL management -------------------------------------------------

static
void
LLW_setCurrentGLWindow(void);

static
void
LLW_setCurrentGLBuffer(void);

void
LLW::beginGLRendering(LLW::AreaID area) // Static
{
LLW_currentGLTarget=(LLW_AreaData *)area;
if(LLW_currentGLTarget->isWindow())
  {
  LLW_setCurrentGLWindow();
  }
else if(LLW_currentGLTarget->isGLBuffer())
  {
  LLW_GLBufData * bufferData=(LLW_GLBufData *)LLW_currentGLTarget;
  if(bufferData->fb)
    {
    GL::glBindFramebuffer(GL_FRAMEBUFFER_EXT,bufferData->fb);
    }
  else
    {
    LLW_setCurrentGLBuffer();
    }
  }
else
  {
  cerr << "!!! Cannot use LLW::beginGLRendering() with a pixmap !!!" << endl;
  }
}

static
void
LLW_swapGLDoubleBuffers(void);

void
LLW::endGLRendering(void) // Static
{
if(LLW_currentGLTarget->isWindow())
  {
  LLW_swapGLDoubleBuffers();
  }
else if(LLW_currentGLTarget->isGLBuffer())
  {
  LLW_GLBufData * bufferData=(LLW_GLBufData *)LLW_currentGLTarget;
  if(bufferData->fb)
    {
    GL::glBindFramebuffer(GL_FRAMEBUFFER_EXT,0);
    }
  else
    {
    glBindTexture(GL_TEXTURE_2D,bufferData->tex);
    // glReadBuffer() already set at context creation
    glCopyTexSubImage2D(GL_TEXTURE_2D,0,0,0,0,0,
                        bufferData->width,bufferData->height);
    // glCopyTexImage2D(GL_TEXTURE_2D,0,GL_RGB,0,0,
    //                  bufferData->width,bufferData->height,0);
    }
  }
else
  {
  cerr << "!!! Cannot use LLW::endGLRendering() with a pixmap !!!" << endl;
  }
}

unsigned int // texture of buffer
LLW::getGLBufferTexture(LLW::GLBufID buffer) // Static
{
LLW_GLBufData * bufferData=(LLW_GLBufData *)buffer;
return(bufferData->tex);
}

unsigned int // texture components in buffer
LLW::getGLBufferTextureComponents(LLW::GLBufID buffer) // Static
{
LLW_GLBufData * bufferData=(LLW_GLBufData *)buffer;
return(bufferData->fb ? 4 : 3);
}

//-------- OpenGL text management --------------------------------------------

LLW::GLFontID // loaded OpenGL font
LLW::loadGLFont(const StlString & url) // Static
{
LLW_GLFontData * glFontData=LLW_getGLFontData(url);
if(glFontData)
  {
  ++(glFontData->count);
  return((LLW::GLFontID)glFontData);
  }
else
  {
  glFontData=new LLW_GLFontData;
  }
glFontData->url=url;
glFontData->ascent=0.0;
glFontData->descent=0.0;
glFontData->count=1;
unsigned int i;
for(i=256;i--;)
  {
  glFontData->symbols[i]=(LLW_GLFontSymbol *)0;
  }
ArRef<URLStream> input=URLStream::NEW(glFontData->url);
if(input->fail())
  {
  goto loadError;
  }
unsigned char header;
unsigned char nbSymbols;
if(!input->readUChar(header)||(header!='A')||
   !input->readUChar(header)||(header!='r')||
   !input->readUChar(header)||(header!='G')||
   !input->readUChar(header)||(header!='L')||
   !input->readUChar(header)||(header!='F')||
   !input->readUChar(nbSymbols))
  {
  goto loadError;
  }
while(nbSymbols--)
  {
  unsigned char code;
  if(!input->readUChar(code))
    {
    goto loadError;
    }
  LLW_GLFontSymbol * symb=new LLW_GLFontSymbol;
  symb->vertices=(double *)0;
  symb->triangles=(unsigned char *)0;
  symb->lines=(unsigned char *)0;
  glFontData->symbols[code]=symb;
  if(!input->readUChar(symb->nbVertices)||
     !input->readUChar(symb->nbTriangles)||
     !input->readUChar(symb->nbLines))
    {
    goto loadError;
    }
  symb->vertices=new double[2*symb->nbVertices];
  symb->xmin=symb->xmax=symb->ymin=symb->ymax=0.0;
  for(i=0;i<symb->nbVertices;i++)
    {
    float x,y;
    if(!input->readFloat(x)||
       !input->readFloat(y))
      {
      goto loadError;
      }
    symb->vertices[2*i]=x;
    symb->vertices[2*i+1]=y;
    if(i)
      {
      if(symb->vertices[2*i]<symb->xmin)
        {
        symb->xmin=symb->vertices[2*i];
        }
      else if(symb->vertices[2*i]>symb->xmax)
        {
        symb->xmax=symb->vertices[2*i];
        }
      if(symb->vertices[2*i+1]<symb->ymin)
        {
        symb->ymin=symb->vertices[2*i+1];
        }
      else if(symb->vertices[2*i+1]>symb->ymax)
        {
        symb->ymax=symb->vertices[2*i+1];
        }
      }
    else
      {
      symb->xmin=symb->xmax=symb->vertices[2*i];
      symb->ymin=symb->ymax=symb->vertices[2*i+1];
      }
    }
  if(symb->ymin<-glFontData->descent)
    {
    glFontData->descent=-symb->ymin;
    }
  else if(symb->ymax>glFontData->ascent)
    {
    glFontData->ascent=symb->ymax;
    }
  symb->triangles=new unsigned char[3*symb->nbTriangles];
  for(i=0;i<symb->nbTriangles;i++)
    {
    if(!input->readUChar(symb->triangles[3*i])||
       !input->readUChar(symb->triangles[3*i+1])||
       !input->readUChar(symb->triangles[3*i+2]))
      {
      goto loadError;
      }
    }
  symb->lines=new unsigned char[symb->nbLines];
  for(i=0;i<symb->nbLines;i++)
    {
    if(!input->readUChar(symb->lines[i]))
      {
      goto loadError;
      }
    }
  }
LLW_glFontDataMap->insert(LLW_GLFontDataMap::value_type(glFontData->url,
                                                        glFontData));
return((LLW::GLFontID)glFontData);
loadError: // PB while loading font !
LLW::freeGLFont((LLW::GLFontID)glFontData);
return((LLW::GLFontID)0);
}

void
LLW::freeGLFont(LLW::GLFontID font) // Static
{
LLW_GLFontData * glFontData=(LLW_GLFontData *)font;
if(!(--(glFontData->count)))
  {
  for(unsigned int i=256;i--;)
    {
    if(glFontData->symbols[i])
      {
      if(glFontData->symbols[i]->vertices)
        {
        delete[] glFontData->symbols[i]->vertices;
        }
      if(glFontData->symbols[i]->triangles)
        {
        delete[] glFontData->symbols[i]->triangles;
        }
      if(glFontData->symbols[i]->lines)
        {
        delete[] glFontData->symbols[i]->lines;
        }
      delete glFontData->symbols[i];
      }
    }
  LLW_glFontDataMap->erase(glFontData->url);
  delete glFontData;
  }
}

void
LLW::getGLFontHeight(LLW::GLFontID font,
                     double & ascentOut,
                     double & descentOut) // Static
{
const LLW_GLFontData * glFontData=(LLW_GLFontData *)font;
ascentOut=glFontData->ascent;
descentOut=glFontData->descent;
}

double // text width
LLW::getGLTextWidth(LLW::GLFontID font,
                    double symbolSpace,
                    double whiteSpace,
                    const StlString & text) // Static
{
const LLW_GLFontData * glFontData=(LLW_GLFontData *)font;
double width=0.0;
unsigned int len=text.size();
for(unsigned int i=0;i<len;i++)
  {
  if(i)
    {
    width+=symbolSpace;
    }
  unsigned char c=(unsigned char)text[i];
  const LLW_GLFontSymbol * symb=glFontData->symbols[c];
  if((!symb)||(c==' '))
    {
    width+=whiteSpace;
    }
  else
    {
    width+=(symb->xmax-symb->xmin);
    }
  }
return(width);
}

void
LLW::buildGLText(LLW::GLFontID font,
                 double symbolSpace,
                 double whiteSpace,
                 const StlString & text,
                 double extrusion,
                 StlVector<Util3D::Dbl3> & verticesInOut,
                 StlVector<Util3D::Face> & facesVInOut,
                 StlVector<Util3D::Dbl3> & normalsInOut,
                 StlVector<Util3D::Face> & facesNInOut) // Static
{
const LLW_GLFontData * glFontData=(LLW_GLFontData *)font;
unsigned int o1=verticesInOut.size();
unsigned int on=normalsInOut.size();
normalsInOut.push_back(Util3D::Dbl3(1.0,0.0,0.0));
unsigned int i;
unsigned int len=text.size();
double thickness=0.5*extrusion;
double offset=0.0;
unsigned int n;
for(n=0;n<len;n++) // top
  {
  if(n)
    {
    offset+=symbolSpace;
    }
  unsigned char c=text[n];
  const LLW_GLFontSymbol * symb=glFontData->symbols[c];
  if(!symb||(c==' '))
    {
    offset+=whiteSpace;
    }
  else
    {
    offset-=symb->xmin;
    unsigned int o=verticesInOut.size();
    for(i=0;i<symb->nbVertices;i++)
      {
      verticesInOut.push_back(Util3D::Dbl3(thickness,
                                           offset+symb->vertices[2*i],
                                           symb->vertices[2*i+1]));
      }
    for(i=0;i<symb->nbTriangles;i++)
      {
      facesVInOut.push_back(Util3D::Face(o+symb->triangles[3*i],
                                       o+symb->triangles[3*i+1],
                                       o+symb->triangles[3*i+2]));
      facesNInOut.push_back(Util3D::Face(on,on,on));
      }
    offset+=symb->xmax;
    }
  }
if(thickness) // bottom and side
  {
  ++on;
  normalsInOut.push_back(Util3D::Dbl3(-1.0,0.0,0.0));
  unsigned int o2=verticesInOut.size();
  for(i=o1;i<o2;i++) // duplicate top vertices to bottom
    {
    const Util3D::Dbl3 & v=verticesInOut[i];
    verticesInOut.push_back(Util3D::Dbl3(-v.x,v.y,v.z));
    }
  unsigned int nbV=facesVInOut.size();
  for(i=0;i<nbV;i++) // duplicate top faces to bottom
    {
    const Util3D::Face & f=facesVInOut[i];
    facesVInOut.push_back(Util3D::Face(o2+f.p3,o2+f.p2,o2+f.p1));
    facesNInOut.push_back(Util3D::Face(on,on,on));
    }
  StlVector<Util3D::Dbl3> tmpN;
  for(n=0;n<len;n++) // side
    {
    unsigned char c=text[n];
    const LLW_GLFontSymbol * symb=glFontData->symbols[c];
    if(symb&&(c!=' '))
      {
      bool newLine=true;
      unsigned char line=0;
      unsigned int first=0;
      for(i=0;i<symb->nbVertices;i++)
        {
        if(newLine)
          {
          first=i;
          tmpN.clear();
          newLine=false;
          }
        else
          {
          facesVInOut.push_back(Util3D::Face(o1+(i-1),o2+(i-1),o1+i));
          facesVInOut.push_back(Util3D::Face(o2+(i-1),o2+i,o1+i));
          tmpN.push_back(Util3D::Dbl3(0.0,
                         verticesInOut[facesVInOut.back().p2].z-
                         verticesInOut[facesVInOut.back().p1].z,
                         verticesInOut[facesVInOut.back().p1].y-
                         verticesInOut[facesVInOut.back().p2].y));
          }
        if(symb->lines[line]==i)
          {
          facesVInOut.push_back(Util3D::Face(o1+i,o2+i,o1+first));
          facesVInOut.push_back(Util3D::Face(o2+i,o2+first,o1+first));
          tmpN.push_back(Util3D::Dbl3(0.0,
                         verticesInOut[facesVInOut.back().p2].z-
                         verticesInOut[facesVInOut.back().p1].z,
                         verticesInOut[facesVInOut.back().p1].y-
                         verticesInOut[facesVInOut.back().p2].y));
          unsigned int o=normalsInOut.size();
          unsigned int nbN=tmpN.size();
          for(unsigned int i=0;i<nbN;i++)
            {
            unsigned int prev=(i ? i-1 : nbN-1);
            unsigned int next=(i+1)%nbN;
            normalsInOut.push_back(Util3D::Dbl3(0.0,
                                                0.5*(tmpN[i].y+tmpN[prev].y),
                                                0.5*(tmpN[i].z+tmpN[prev].z)));
            facesNInOut.push_back(Util3D::Face(o+i,o+i,o+next));
            facesNInOut.push_back(Util3D::Face(o+i,o+next,o+next));
            }
          if(++line<symb->nbLines)
            {
            newLine=true;
            }
          else
            {
            break; // last line
            }
          }
        }
      o1+=symb->nbVertices;
      o2+=symb->nbVertices;
      }
    }
  }
}

void
LLW::drawGLText(LLW::GLFontID font,
                double symbolSpace,
                double whiteSpace,
                const StlString & text) // Static
{
const LLW_GLFontData * glFontData=(const LLW_GLFontData *)font;
double offset=0.0;
unsigned int len=text.size();
glBegin(GL_TRIANGLES);
for(unsigned int i=0;i<len;i++)
  {
  if(i)
    {
    offset+=symbolSpace;
    }
  unsigned char c=(unsigned char)text[i];
  const LLW_GLFontSymbol * symb=glFontData->symbols[c];
  if(!symb||(c==' '))
    {
    offset+=whiteSpace;
    }
  else
    {
    offset-=symb->xmin;
    unsigned char * pt;
    double * pv;
    for(unsigned int j=symb->nbTriangles;j--;)
      {
      pt=symb->triangles+(3*j);
      pv=symb->vertices+(2*pt[0]);
      glVertex2f(offset+pv[0],pv[1]);
      pv=symb->vertices+(2*pt[1]);
      glVertex2f(offset+pv[0],pv[1]);
      pv=symb->vertices+(2*pt[2]);
      glVertex2f(offset+pv[0],pv[1]);
      }
    offset+=symb->xmax;
    }
  }
glEnd();
}

//-------- Bitmap text management --------------------------------------------

LLW::BMFontID // loaded bitmap font
LLW::loadBMFont(const StlString & url) // Static
{
LLW_BMFontData * bmFontData=LLW_getBMFontData(url);
if(bmFontData)
  {
  ++(bmFontData->count);
  return((LLW::BMFontID)bmFontData);
  }
else
  {
  bmFontData=new LLW_BMFontData;
  }
bmFontData->url=url;
bmFontData->ascent=0;
bmFontData->descent=0;
bmFontData->count=1;
unsigned int i;
for(i=256;i--;)
  {
  bmFontData->symbols[i]=(LLW_BMFontSymbol *)0;
  }
ArRef<URLStream> input=URLStream::NEW(bmFontData->url);
if(input->fail())
  {
  goto loadError;
  }
unsigned char header;
unsigned char ascent;
unsigned char descent;
if(!input->readUChar(header)||(header!='A')||
   !input->readUChar(header)||(header!='r')||
   !input->readUChar(header)||(header!='B')||
   !input->readUChar(header)||(header!='M')||
   !input->readUChar(header)||(header!='F')||
   !input->readUChar(ascent)||
   !input->readUChar(descent))
  {
  goto loadError;
  }
bmFontData->ascent=ascent;
bmFontData->descent=descent;
for(unsigned int code=0;code<256;code++)
  {
  LLW_BMFontSymbol * symb=new LLW_BMFontSymbol;
  symb->pixels=(unsigned char *)0;
  bmFontData->symbols[code]=symb;
  unsigned char width;
  unsigned char dataSize;
  if(!input->readUChar(width)||
     !input->readUChar(dataSize))
    {
    goto loadError;
    }
  symb->width=width;
  symb->pixels=new unsigned char[dataSize];
  dataSize*=sizeof(unsigned char);
  if(input->readData(symb->pixels,dataSize)!=dataSize)
    {
    goto loadError;
    }
  }
LLW_bmFontDataMap->insert(LLW_BMFontDataMap::value_type(bmFontData->url,
                                                        bmFontData));
return((LLW::BMFontID)bmFontData);
loadError: // PB while loading font !
LLW::freeBMFont((LLW::BMFontID)bmFontData);
return((LLW::BMFontID)0);
}

void
LLW::freeBMFont(LLW::BMFontID font) // Static
{
LLW_BMFontData * bmFontData=(LLW_BMFontData *)font;
if(!(--(bmFontData->count)))
  {
  for(unsigned int i=256;i--;)
    {
    if(bmFontData->symbols[i])
      {
      if(bmFontData->symbols[i]->pixels)
        {
        delete[] bmFontData->symbols[i]->pixels;
        }
      delete bmFontData->symbols[i];
      }
    }
  LLW_bmFontDataMap->erase(bmFontData->url);
  delete bmFontData;
  }
}

void
LLW::getBMFontHeight(LLW::BMFontID font,
                     unsigned int & ascentOut,
                     unsigned int & descentOut) // Static
{
const LLW_BMFontData * bmFontData=(const LLW_BMFontData *)font;
ascentOut=bmFontData->ascent;
descentOut=bmFontData->descent;
}

unsigned int // symbol width
LLW::getBMSymbolWidth(LLW::BMFontID font,
                      char symbol) // Static
{
const LLW_BMFontData * bmFontData=(const LLW_BMFontData *)font;
return(bmFontData->symbols[(unsigned char)symbol]->width);
}

unsigned int // text width
LLW::getBMTextWidth(LLW::BMFontID font,
                    const StlString & text) // Static
{
const LLW_BMFontData * bmFontData=(const LLW_BMFontData *)font;
unsigned int width=0;
unsigned int len=text.size();
for(unsigned int i=0;i<len;i++)
  {
  unsigned char c=(unsigned char)text[i];
  const LLW_BMFontSymbol * symb=bmFontData->symbols[c];
  width+=symb->width;
  }
return(width);
}

void
LLW::drawBMSymbol(LLW::BMFontID font,
                  char symbol) // Static
{
const LLW_BMFontData * bmFontData=(const LLW_BMFontData *)font;
unsigned int height=bmFontData->ascent+bmFontData->descent;
const LLW_BMFontSymbol * symb=bmFontData->symbols[(unsigned char)symbol];
glBitmap(symb->width,height,
         0,bmFontData->descent,
         symb->width,0,
         symb->pixels);
}

void
LLW::drawBMText(LLW::BMFontID font,
                const StlString & text) // Static
{
const LLW_BMFontData * bmFontData=(const LLW_BMFontData *)font;
unsigned int len=text.size();
unsigned int height=bmFontData->ascent+bmFontData->descent;
for(unsigned int i=0;i<len;i++)
  {
  unsigned char c=(unsigned char)text[i];
  const LLW_BMFontSymbol * symb=bmFontData->symbols[c];
  glBitmap(symb->width,height,
           0,bmFontData->descent,
           symb->width,0,
           symb->pixels);
  }
}

//-------- Time management ---------------------------------------------------

unsigned int // expiration time
LLW::postTimeOut(LLW::AreaID area,
                 unsigned int delay) // Static
{
unsigned int limit=ArSystem::getTime()+delay;
if(!limit)
  {
  limit=1; // keep zero as an invalid time out
  }
LLW_TimeOutList::iterator it=LLW_timeOutList->begin();
for(;it!=LLW_timeOutList->end();it++)
  {
  if((*it)->limit>limit) break;
  }
LLW_TimeOut * t=new LLW_TimeOut;
t->area=area;
t->limit=limit;
LLW_timeOutList->insert(it,t);
return(limit);
}

void
LLW::cancelTimeOut(LLW::AreaID area,
                   unsigned int timeOut) // Static
{
LLW_TimeOutList::iterator it=LLW_timeOutList->begin();
for(;it!=LLW_timeOutList->end();it++)
  {
  if(((*it)->limit==timeOut)&&
     ((*it)->area==area))
    {
    delete *it;
    LLW_timeOutList->erase(it);
    break;
    }
  if((*it)->limit>timeOut)
    {
    break;
    }
  }
}

//-------- Event management --------------------------------------------------

static
void
LLW_updateWindowEventMask(LLW::WinID window);

void
LLW::setWindowEventMask(LLW::WinID window,
                        LLW::EventMask eventMask) // Static
{
LLW_WinData * winData=(LLW_WinData *)window;
if(eventMask&LLW::EVT_MOUSE_BUTTON_UNGRAB)
  {
  eventMask|=LLW::EVT_MOUSE_BUTTON;
  }
winData->mask=eventMask;
LLW_updateWindowEventMask(window);
}

LLW::EventMask // event mask for this window
LLW::getWindowEventMask(LLW::WinID window) // Static
{
LLW_WinData * winData=(LLW_WinData *)window;
return(winData->mask);
}

void
LLW::setWindowKeyRepeat(LLW::WinID window,
                        bool keyRepeat) // Static
{
LLW_WinData * winData=(LLW_WinData *)window;
winData->keyRepeat=keyRepeat;
}

bool // key repeat is enabled for this window
LLW::getWindowKeyRepeat(LLW::WinID window) // Static
{
LLW_WinData * winData=(LLW_WinData *)window;
return(winData->keyRepeat);
}

static
void
LLW_updateMouseAndModifiers(LLW::Event * evt);

inline static
bool // event has been posted
LLW_postEvent(LLW_AreaData * areaData,
              LLW::EventType type,
              LLW::Button button,
              const char * key,
              bool onOff,
              unsigned int time)
{
bool posted=false;
if((!areaData->isWindow())||
   (((LLW_WinData *)areaData)->mask&type))
  {
  LLW::Event * evt=(LLW::Event *)0;
  if((type==LLW::EVT_MOUSE_MOTION)||
     (type==LLW::EVT_EXPOSE)||
     (type==LLW::EVT_GEOMETRY))
    {
    LLW_EventList::iterator it=LLW_pendingEvent->begin();
    for(;it!=LLW_pendingEvent->end();it++)
      {
      if(((*it)->type==type)&&((*it)->area==(LLW::AreaID)areaData))
        {
        evt=*it;
        break;
        }
      }
    }
  if(!evt)
    {
    if(LLW_freeEvent->size())
      {
      evt=LLW_freeEvent->front();
      LLW_freeEvent->pop_front();
      }
    else
      {
      evt=new LLW::Event;
      }
    LLW_pendingEvent->push_back(evt);
    }
  evt->area=(LLW::AreaID)areaData;
  evt->type=type;
  evt->button=button;
  if(key)
    {
    evt->key=key;
    }
  else
    {
    evt->key.erase();
    }
  evt->onOff=onOff;
  evt->time=time;
  if(type==LLW::EVT_TIMEOUT)
    {
    evt->modifiers=LLW::MDF_NONE;
    evt->x=evt->xScreen=0;
    evt->y=evt->yScreen=0;
    }
  else
    {
    LLW_updateMouseAndModifiers(evt);
    }
  posted=true;
  }
return(posted);
}

static
void
LLW_extractEvents(bool blocking);

inline static
void
LLW_checkTimeOut(void)
{
while(LLW_timeOutList->size()&&
      (LLW_timeOutList->front()->limit<=ArSystem::getTime()))
  {
  LLW_postEvent((LLW_AreaData *)LLW_timeOutList->front()->area,
                LLW::EVT_TIMEOUT,
                LLW::BTN_NONE,
                (const char *)0,
                false,
                LLW_timeOutList->front()->limit);
  delete LLW_timeOutList->front();
  LLW_timeOutList->pop_front();
  }
}

const LLW::Event *
LLW::nextEvent(bool blocking) // Static
{
if(blocking)
  {
  while(!LLW_pendingEvent->size())
    {
    if(LLW_timeOutList->size())
      {
      LLW_checkTimeOut();
      LLW_extractEvents(false);
      if(!LLW_pendingEvent->size())
        {
        ArSystem::yield();
        }
      }
    else
      {
      LLW_extractEvents(true);
      }
    }
  }
else
  {
  LLW_checkTimeOut();
  LLW_extractEvents(false);
  }
if(LLW_pendingEvent->size())
  {
  LLW_freeEvent->push_back(LLW_pendingEvent->front());
  LLW_pendingEvent->pop_front();
  return(LLW_freeEvent->back());
  }
else
  {
  return((const LLW::Event *)0);
  }
}

StlString
LLW::printEvent(const LLW::Event * evt)
{
char buffer[0x80];
StlString result="area: ";
result+=ptrToStr(evt->area);
result+="\ntype: ";
switch(evt->type)
  {
  case LLW::EVT_NONE:
    {
    result+="NONE\n";
    break;
    }
  case LLW::EVT_CLOSE:
    {
    result+="CLOSE\n";
    break;
    }
  case LLW::EVT_EXPOSE:
    {
    result+="EXPOSE\n";
    break;
    }
  case LLW::EVT_GEOMETRY:
    {
    const LLW::WinID window=(const LLW::WinID)evt->area;
    result+="GEOMETRY";
    int x,y;
    unsigned int w,h;
    LLW::getWindowPosition(window,x,y);
    LLW::getWindowSize(window,w,h);
    sprintf(buffer," x=%d y=%d w=%u h=%u",x,y,w,h);
    result+=buffer;
    if(LLW::isWindowViewable(window))
      {
      result+=" (viewable)";
      }
    else if(LLW::isWindowMapped(window))
      {
      result+=" (mapped)";
      }
    else
      {
      result+=" (unmapped)";
      }
    result+="\n";
    break;
    }
  case LLW::EVT_MOUSE_BUTTON:
    {
    result+="MOUSE_BUTTON\n";
    result+="button: ";
    switch(evt->button)
      {
      case LLW::BTN_NONE:
        {
        break;
        }
      case LLW::BTN_LEFT:
        {
        result+="LEFT";
        break;
        }
      case LLW::BTN_MIDDLE:
        {
        result+="MIDDLE";
        break;
        }
      case LLW::BTN_RIGHT:
        {
        result+="RIGHT";
        break;
        }
      case LLW::BTN_FORWARD:
        {
        result+="FORWARD";
        break;
        }
      case LLW::BTN_BACKWARD:
        {
        result+="BACKWARD";
        break;
        }
      }
    result+="\nonOff: ";
    if(evt->onOff)
      {
      result+=" true (pressed)\n";
      }
    else
      {
      result+=" false (released)\n";
      }
    break;
    }
  case LLW::EVT_MOUSE_MOTION:
    {
    result+="MOUSE_MOTION\n";
    break;
    }
  case LLW::EVT_MOUSE_INOUT:
    {
    result+="MOUSE_INOUT\n";
    result+="onOff: ";
    if(evt->onOff)
      {
      result+=" true (enter)\n";
      }
    else
      {
      result+=" false (leave)\n";
      }
    break;
    }
  case LLW::EVT_KEYBOARD:
    {
    result+="KEYBOARD\n";
    result+="key: [";
    result+=evt->key;
    result+="]";
    if(evt->key.size()<=1)
      {
      sprintf(buffer," (ascii: %d 0x%.2X)",
                     (int)evt->key[0],(int)evt->key[0]);
      result+=buffer;
      }
    result+="\n";
    break;
    }
  default:
    break;
  }
result+="time: ";
result+=uintToStr(evt->time);
result+="\n";
result+="modifiers:";
if(evt->modifiers&LLW::MDF_CONTROL)
  {
  result+=" CONTROL";
  }
if(evt->modifiers&LLW::MDF_SHIFT)
  {
  result+=" SHIFT";
  }
if(evt->modifiers&LLW::MDF_CAPSLOCK)
  {
  result+=" CAPSLOCK";
  }
if(evt->modifiers&LLW::MDF_NUMLOCK)
  {
  result+=" NUMLOCK";
  }
if(evt->modifiers&LLW::MDF_LEFT)
  {
  result+=" LEFT";
  }
if(evt->modifiers&LLW::MDF_MIDDLE)
  {
  result+=" MIDDLE";
  }
if(evt->modifiers&LLW::MDF_RIGHT)
  {
  result+=" RIGHT";
  }
result+="\n";
sprintf(buffer,"mouse: x=%d y=%d\n",evt->x,evt->y);
result+=buffer;
return(result);
}

//-------- Image management --------------------------------------------------

static
bool // success (dataOut filled from bottom to top)
LLW_loadTGAImage(const char * fileName,
                 unsigned int & widthOut,
                 unsigned int & heightOut,
                 unsigned int & componentsOut,
                 ArRef<MemoryBlock> dataOut)
{
unsigned char header[18];
FILE * file=fopen(fileName,"rb");
if(!file)
  {
  return(false);
  }
if(fread(header,sizeof(unsigned char),18,file)!=18)
  {
  fclose(file);
  return(false);
  }
int idLength=header[0];
int colormapType=header[1];
int dataTypeCode=header[2];
int colormapLength=header[6]*256+header[5];
widthOut=header[13]*256+header[12];
heightOut=header[15]*256+header[14];
int bitsPerPixel=header[16];
bool flipV=(header[17]&(unsigned char)0x20);
bool flipH=(header[17]&(unsigned char)0x10);
if((dataTypeCode!=2)||
   ((bitsPerPixel!=24)&&(bitsPerPixel!=32))||
   ((colormapType!=0)&&(colormapType!=1))||
   (!widthOut)||(!heightOut))
  {
  fclose(file);
  return(false);
  }
if(fseek(file,idLength+colormapType*colormapLength,SEEK_CUR)==-1)
  {
  fclose(file);
  return(false);
  }
unsigned int i;
componentsOut=(bitsPerPixel==24 ? 3 : 4);
unsigned int lineSize=widthOut*componentsOut;
unsigned int dataSize=heightOut*lineSize;
if(!dataOut->ensureMinSize(dataSize))
  {
  fclose(file);
  return(false);
  }
char * ptr=dataOut->accessMemory();
if(flipV)
  {
  if(flipH)
    {
    for(char * dest=ptr+dataSize-componentsOut;dest>=ptr;dest-=componentsOut)
      {
      if(fread(dest,sizeof(char),componentsOut,file)!=componentsOut)
        {
        fclose(file);
        return(false);
        }
      }
    }
  else
    {
    for(char * line=ptr+dataSize-lineSize;line>=ptr;line-=lineSize)
      {
      if(fread(line,sizeof(char),lineSize,file)!=lineSize)
        {
        fclose(file);
        return(false);
        }
      }
    }
  }
else
  {
  if(flipH)
    {
    for(unsigned int i=0;i<heightOut;i++)
      {
      char * line=ptr+i*lineSize;
      for(char * row=line+lineSize-componentsOut;row>=line;row-=componentsOut)
        {
        if(fread(row,sizeof(char),componentsOut,file)!=componentsOut)
          {
          fclose(file);
          return(false);
          }
        }
      }
    }
  else
    {
    if(fread(ptr,sizeof(char),dataSize,file)!=dataSize)
      {
      fclose(file);
      return(false);
      }
    }
  }
fclose(file);
for(i=0;i<dataSize;i+=componentsOut)
  {
  char tmp=ptr[i]; // swap red and blue
  ptr[i]=ptr[i+2];
  ptr[i+2]=tmp;
  }
return(true);
}

bool // success (dataOut filled from bottom to top)
LLW::loadImage(const StlString & url,
               unsigned int & widthOut,
               unsigned int & heightOut,
               unsigned int & componentsOut,
               ArRef<MemoryBlock> dataOut) // Static
{
bool result=false;
ArRef<URLStream> input=URLStream::NEW(url);
if(input->fail())
  {
  return(result);
  }
if(input->getProtocol()=="file")
  {
  StlString fileName=input->getFileName();
  input.clear();
  for(unsigned int i=0;i<LLW::_imageLoaderFncts->size();++i)
    {
    LLW::ImageLoaderFnct fnct=(*LLW::_imageLoaderFncts)[i];
    result=(*fnct)(fileName.c_str(),widthOut,heightOut,componentsOut,dataOut);
    if(result)
      {
      break;
      }
    }
  }
else
  {
  StlString tmpName;
  int tmpFD=ArSystem::createTmpFile("./LLW_img_",tmpName);
  if(tmpFD==-1)
    {
    return(result);
    }
  ArRef<FDOStream> output=FDOStream::NEW(tmpFD,true);
  if(!output->fail())
    {
    output->copyDataFromInput(input);
    input.clear();
    output.clear();
    for(unsigned int i=0;i<LLW::_imageLoaderFncts->size();++i)
      {
      LLW::ImageLoaderFnct fnct=(*LLW::_imageLoaderFncts)[i];
      result=(*fnct)(tmpName.c_str(),widthOut,heightOut,componentsOut,dataOut);
      if(result)
        {
        break;
        }
      }
    }
  ArSystem::removeFile(tmpName);
  }
return(result);
}

static
bool // success
LLW_saveTGAImage(const char * fileName,
                 unsigned int width,
                 unsigned int height,
                 unsigned int components,
                 const char * data)
{
if((strExtension(fileName)!=".tga")||
   ((components!=3)&&(components!=4)))
  {
  return(false);
  }
FILE * file=fopen(fileName,"wb");
if(!file)
  {
  return(false);
  }
unsigned char header[18];
memset(header,0,18);
header[2]=2; // uncompressed RGB
header[12]=width&0x000000FF;
header[13]=(width>>8)&0x000000FF;
header[14]=height&0x000000FF;
header[15]=(height>>8)&0x000000FF;
header[16]=components*8;
if(fwrite(header,sizeof(unsigned char),18,file)!=18)
  {
  fclose(file);
  return(false);
  }
for(unsigned int y=0;y<height;y++)
  {
  for(unsigned int x=0;x<width;x++)
    {
    unsigned int i=components*(y*width+x);
    if(fwrite(data+i+2,sizeof(char),1,file)!=1) // blue
      {
      fclose(file);
      return(false);
      }
    if(fwrite(data+i+1,sizeof(char),1,file)!=1) // green
      {
      fclose(file);
      return(false);
      }
    if(fwrite(data+i,sizeof(char),1,file)!=1) // red
      {
      fclose(file);
      return(false);
      }
    if((components==4)&&
       (fwrite(data+i+3,sizeof(char),1,file)!=1)) // alpha
      {
      fclose(file);
      return(false);
      }
    }
  }
fclose(file);
return(true);
}

bool // success
LLW::saveImage(const StlString & fileName,
               unsigned int width,
               unsigned int height,
               unsigned int components,
               ArConstRef<MemoryBlock> data) // Static
{
bool result=false;
unsigned int size=components*height*width;
if(size&&(data->getSize()>=size))
  {
  for(unsigned int i=LLW::_imageSaverFncts->size();i--;)
    {
    LLW::ImageSaverFnct fnct=(*LLW::_imageSaverFncts)[i];
    result=(*fnct)(fileName.c_str(),width,height,components,data->getMemory());
    if(result)
      {
      break;
      }
    }
  }
return(result);
}

bool // success
LLW::loadRGBImageFromJPEG(ArConstRef<MemoryBlock> dataIn,
                          unsigned int & widthOut,
                          unsigned int & heightOut,
                          ArRef<MemoryBlock> dataOut)
{
if(LLW::_initJpegLibSuccess)
  {
  return(LLW::_uncompressRGBImageFromJPEG(dataIn,dataOut,widthOut,heightOut));
  }
else
  {
  dataOut->setErrorMessage("JpegLib plugin not initialized");
  return(false);
  }
}

bool // success
LLW::saveRGBImageToJPEG(ArConstRef<MemoryBlock> dataIn,
                        unsigned int width,
                        unsigned int height,
                        unsigned int quality, // [0-100]
                        ArRef<MemoryBlock> dataOut)
{
if(LLW::_initJpegLibSuccess)
  {
  return(LLW::_compressRGBImageToJPEG(dataIn,dataOut,width,height,quality));
  }
else
  {
  dataOut->setErrorMessage("JpegLib plugin not initialized");
  return(false);
  }
}

static
void
LLW_dumpDrawable(LLW::DrawID drawable,
                 unsigned int width,
                 unsigned int height,
                 char * dataOut);

bool // success (dataOut filled from bottom to top)
LLW::dumpArea(LLW::AreaID area,
              unsigned int & widthOut,
              unsigned int & heightOut,
              unsigned int & componentsOut,
              ArRef<MemoryBlock> dataOut) // Static
{
LLW_AreaData * areaData=(LLW_AreaData *)area;
LLW::getAreaSize(areaData,widthOut,heightOut);
componentsOut=3; // RGB
unsigned int size=componentsOut*heightOut*widthOut;
if(!dataOut->ensureMinSize(size))
  {
  return(false);
  }
char * ptr=dataOut->accessMemory();
if(areaData->isPixmap())
  {
  LLW_dumpDrawable(areaData,widthOut,heightOut,ptr);
  }
else
  {
  LLW::beginGLRendering(areaData);
  // glReadBuffer() already set at context creation
  glReadPixels(0,0,widthOut,heightOut,GL_RGB,GL_UNSIGNED_BYTE,ptr);
  LLW::endGLRendering();
  }
return(true);
}

} // namespace AReVi

//----------------------------------------------------------------------------
// Class LLW (abstract) - X11/Win32 specific part
//----------------------------------------------------------------------------

#ifdef __CYGWIN__
# include "llw_w32_priv.cpp"
#else // not __CYGWIN__
# include "llw_x11_priv.cpp"
#endif // not __CYGWIN__

//----------------------------------------------------------------------------
