// LI2/ENIB - AReVi - 3D services
//----------------------------------------------------------------------------
// file        : AReVi/Lib3D/llw_w32_priv.cpp
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include <windows.h>

#ifndef WGL_DRAW_TO_PBUFFER_ARB
# define WGL_DRAW_TO_PBUFFER_ARB 0x202D
#endif

#ifndef WGL_SUPPORT_OPENGL_ARB
# define WGL_SUPPORT_OPENGL_ARB 0x2010
#endif

#ifndef WGL_DOUBLE_BUFFER_ARB
# define WGL_DOUBLE_BUFFER_ARB 0x2011
#endif

#ifndef WGL_STEREO_ARB
# define WGL_STEREO_ARB 0x2012
#endif

#ifndef WGL_PIXEL_TYPE_ARB
# define WGL_PIXEL_TYPE_ARB 0x2013
#endif

#ifndef WGL_TYPE_RGBA_ARB
# define WGL_TYPE_RGBA_ARB 0x202B
#endif

#ifndef WGL_RED_BITS_ARB
# define WGL_RED_BITS_ARB 0x2015
#endif

#ifndef WGL_GREEN_BITS_ARB
# define WGL_GREEN_BITS_ARB 0x2017
#endif

#ifndef WGL_BLUE_BITS_ARB
# define WGL_BLUE_BITS_ARB  0x2019
#endif

#ifndef WGL_ALPHA_BITS_ARB
# define WGL_ALPHA_BITS_ARB 0x201B
#endif

#ifndef WGL_DEPTH_BITS_ARB
# define WGL_DEPTH_BITS_ARB 0x2022
#endif

namespace AReVi {

//----------------------------------------------------------------------------
// Class LLW (abstract) - Win32 specific part
//----------------------------------------------------------------------------

struct LLW_WinData_W32 : public LLW_WinData
{
LLW_WinData_W32(void) : LLW_WinData () { }
virtual ~LLW_WinData_W32(void) { }
HWND win;
HDC hdc;
bool mini;
};

struct LLW_PixData_W32 : public LLW_PixData
{
LLW_PixData_W32(void) : LLW_PixData () { }
virtual ~LLW_PixData_W32(void) { }
HBITMAP pix;
HDC hdc;
};

struct LLW_GLBufData_W32 : public LLW_GLBufData
{
LLW_GLBufData_W32(void) : LLW_GLBufData () { }
virtual ~LLW_GLBufData_W32(void) { }
HWND pbuffer;
HDC hdc;
};

typedef StlMap<HWND,LLW_WinData_W32 *,StlLess<HWND> >
        LLW_WinDataMap;

typedef StlMap<HBITMAP,LLW_PixData_W32 *,StlLess<HBITMAP> >
        LLW_PixDataMap;

typedef StlMap<HWND,LLW_GLBufData_W32 *,StlLess<HWND> >
        LLW_GLBufDataMap;

struct LLW_FontData
{
StlString fontName;
HFONT font;
unsigned int count;
};

typedef StlMap<StlString,LLW_FontData *,StlLess<StlString> >
        LLW_FontDataMap;

static HINSTANCE LLW_hInstance=(HINSTANCE)0;

static LLW_WinData_W32 * LLW_commonWindow=(LLW_WinData_W32 *)0;

static HGLRC LLW_commonWinCtx=(HGLRC)0;

static LLW_GLBufData_W32 * LLW_commonPBuffer=(LLW_GLBufData_W32 *)0;

static HGLRC LLW_commonPBufCtx=(HGLRC)0;

static LLW_WinDataMap * LLW_winDataMap=(LLW_WinDataMap *)0;

static LLW_PixDataMap * LLW_pixDataMap=(LLW_PixDataMap *)0;

static LLW_GLBufDataMap * LLW_glBufDataMap=(LLW_GLBufDataMap *)0;

static LLW_FontDataMap * LLW_fontDataMap=(LLW_FontDataMap *)0;

static HCURSOR LLW_cursor[18];

static LLW::CursorID LLW_lastCursor=(LLW::CursorID)-1;

//---- WGL_ARB_pixel_format ----

static
BOOL
(APIENTRY * LLW_wglChoosePixelFormat)(HDC,
                                      const int *,
                                      const FLOAT *,
                                      UINT,
                                      int *,
                                      UINT *)=
  (BOOL (APIENTRY *)(HDC,const int *,const FLOAT *,UINT,int *,UINT *))0;

//---- WGL_ARB_pbuffer ----

static
HWND
(APIENTRY * LLW_wglCreatePbuffer)(HDC,
                                  int,
                                  int,
                                  int,
                                  const int *)=
  (HWND (APIENTRY *)(HDC,int,int,int,const int *))0;

static
HDC
(APIENTRY * LLW_wglGetPbufferDC)(HWND)=
  (HDC (APIENTRY *)(HWND))0;

static
int
(APIENTRY * LLW_wglReleasePbufferDC)(HWND,HDC)=
  (int (APIENTRY *)(HWND,HDC))0;

static
BOOL
(APIENTRY * LLW_wglDestroyPbuffer)(HWND)=
  (BOOL (APIENTRY *)(HWND))0;

static bool LLW_hasPBuffer=false;

inline static
LLW_WinData_W32 *
LLW_getWinData(HWND window)
{
LLW_WinDataMap::iterator it=LLW_winDataMap->find(window);
if(it==LLW_winDataMap->end())
  {
  return((LLW_WinData_W32 *)0);
  }
else
  {
  return((*it).second);
  }
}

inline static
LLW_PixData_W32 *
LLW_getPixData(HBITMAP pixmap)
{
LLW_PixDataMap::iterator it=LLW_pixDataMap->find(pixmap);
if(it==LLW_pixDataMap->end())
  {
  return((LLW_PixData_W32 *)0);
  }
else
  {
  return((*it).second);
  }
}

inline static
LLW_GLBufData_W32 *
LLW_getGLBufData(HWND buffer)
{
LLW_GLBufDataMap::iterator it=LLW_glBufDataMap->find(buffer);
if(it==LLW_glBufDataMap->end())
  {
  return((LLW_GLBufData_W32 *)0);
  }
else
  {
  return((*it).second);
  }
}

inline static
LLW_FontData *
LLW_getFontData(const StlString & fontName)
{
LLW_FontDataMap::iterator it=LLW_fontDataMap->find(fontName);
if(it==LLW_fontDataMap->end())
  {
  return((LLW_FontData *)0);
  }
else
  {
  return((*it).second);
  }
}

static
LRESULT CALLBACK
LLW_wndProc(HWND hWnd,
            UINT uMsg,
            WPARAM wParam,
            LPARAM lParam);

//-------- Initialization ----------------------------------------------------

static
void
LLW_initWGL(void)
{
const char * (*wglExtFnct)(HDC);
GL::loadFunction(wglExtFnct,"wglGetExtensionsString","EXT");
if(!wglExtFnct) // should suffix be EXT or ARB ?
  {
  GL::loadFunction(wglExtFnct,"wglGetExtensionsString","ARB");
  }
const char * wglExt=(wglExtFnct
                     ? (*wglExtFnct)(((LLW_WinData_W32 *)LLW_commonWindow)->hdc)
                     : (const char *)0);
//---- ChoosePixelFormat ----
if(GL::hasExtension("WGL_ARB_pixel_format",wglExt))
  {
  GL::loadFunction(LLW_wglChoosePixelFormat,"wglChoosePixelFormat","ARB");
  }
//---- pbuffer ----
if(GL::hasExtension("WGL_ARB_pbuffer",wglExt))
  {
  GL::loadFunction(LLW_wglCreatePbuffer,"wglCreatePbuffer","ARB");
  GL::loadFunction(LLW_wglGetPbufferDC,"wglGetPbufferDC","ARB");
  GL::loadFunction(LLW_wglReleasePbufferDC,"wglReleasePbufferDC","ARB");
  GL::loadFunction(LLW_wglDestroyPbuffer,"wglDestroyPbuffer","ARB");
  }
LLW_hasPBuffer=false;
if(LLW_wglChoosePixelFormat&&
   LLW_wglCreatePbuffer&&
   LLW_wglGetPbufferDC&&
   LLW_wglReleasePbufferDC&&
   LLW_wglDestroyPbuffer)
  {
  LLW_hasPBuffer=true;
  }
else
  {
  cerr << "!!! WGL_ARB_pbuffer extension is not supported !!!"
       << endl;
  }
}

void
LLW::init(void) // Static
{
if(LLW_initState==LLW_DOWN)
  {
  LLW_initState=LLW_GOING_UP;
  LLW::_imageLoaderFncts=new LLW::ImageLoaderFnctVect;
  LLW::_imageLoaderFncts->push_back(&LLW_loadTGAImage);
  LLW::_imageSaverFncts=new LLW::ImageSaverFnctVect;
  LLW::_imageSaverFncts->push_back(&LLW_saveTGAImage);
  LLW_pendingEvent=new LLW_EventList;
  LLW_freeEvent=new LLW_EventList;
  LLW_timeOutList=new LLW_TimeOutList;
  LLW_winDataMap=new LLW_WinDataMap;
  LLW_pixDataMap=new LLW_PixDataMap;
  LLW_glBufDataMap=new LLW_GLBufDataMap;
  LLW_fontDataMap=new LLW_FontDataMap;
  LLW_glFontDataMap=new LLW_GLFontDataMap;
  LLW_bmFontDataMap=new LLW_BMFontDataMap;
  WNDCLASS wc;
  LLW_hInstance=(HINSTANCE)GetModuleHandle((LPCTSTR)0);
  if(!LLW_hInstance)
    {
    goto endInit;
    }
  LLW_cursor[LLW::CSR_EMPTY]=(HCURSOR)0;
  LLW_cursor[LLW::CSR_ARROW]=LoadCursor((HINSTANCE)0,IDC_ARROW);
  LLW_cursor[LLW::CSR_NORTH]=LoadCursor((HINSTANCE)0,IDC_SIZENS);
  LLW_cursor[LLW::CSR_SOUTH]=LoadCursor((HINSTANCE)0,IDC_SIZENS);
  LLW_cursor[LLW::CSR_EAST]=LoadCursor((HINSTANCE)0,IDC_SIZEWE);
  LLW_cursor[LLW::CSR_WEST]=LoadCursor((HINSTANCE)0,IDC_SIZEWE);
  LLW_cursor[LLW::CSR_NORTHEAST]=LoadCursor((HINSTANCE)0,IDC_SIZENESW);
  LLW_cursor[LLW::CSR_NORTHWEST]=LoadCursor((HINSTANCE)0,IDC_SIZENWSE);
  LLW_cursor[LLW::CSR_SOUTHEAST]=LoadCursor((HINSTANCE)0,IDC_SIZENWSE);
  LLW_cursor[LLW::CSR_SOUTHWEST]=LoadCursor((HINSTANCE)0,IDC_SIZENESW);
  LLW_cursor[LLW::CSR_MOVE]=LoadCursor((HINSTANCE)0,IDC_SIZEALL);
  LLW_cursor[LLW::CSR_CROSS]=LoadCursor((HINSTANCE)0,IDC_CROSS);
  LLW_cursor[LLW::CSR_WAIT]=LoadCursor((HINSTANCE)0,IDC_WAIT);
  LLW_cursor[LLW::CSR_TEXT]=LoadCursor((HINSTANCE)0,IDC_IBEAM);
  LLW_cursor[LLW::CSR_HAND]=LoadCursor((HINSTANCE)0,IDC_HAND);
  LLW_cursor[LLW::CSR_HELP]=LoadCursor((HINSTANCE)0,IDC_HELP);
  LLW_cursor[LLW::CSR_UP]=LoadCursor((HINSTANCE)0,IDC_UPARROW);
  LLW_cursor[LLW::CSR_NO]=LoadCursor((HINSTANCE)0,IDC_NO);
  wc.style=CS_OWNDC;
  wc.lpfnWndProc=LLW_wndProc;
  wc.cbClsExtra=0;
  wc.cbWndExtra=0;
  wc.hInstance=LLW_hInstance;
  wc.hIcon=LoadIcon((HINSTANCE)0,IDI_APPLICATION);
  wc.hCursor=(HCURSOR)0;
  wc.hbrBackground=(HBRUSH)0;
  wc.lpszMenuName=(LPCTSTR)0;
  wc.lpszClassName="AReVi-OpenGL";
  if(!::RegisterClass(&wc))
    {
    goto endInit;
    }
  LLW_commonWindow=(LLW_WinData_W32 *)LLW::createTopWindow("",false,0,0,16,16);
  if(!LLW_commonWindow)
    {
    goto endInit;
    }
/***
  PIXELFORMATDESCRIPTOR pfd;
  DescribePixelFormat(LLW_commonWindow->hdc,
                      GetPixelFormat(LLW_commonWindow->hdc),
                      sizeof(PIXELFORMATDESCRIPTOR),&pfd);
  LLW_redBits=pfd.cRedBits;
  LLW_greenBits=pfd.cGreenBits;
  LLW_blueBits=pfd.cBlueBits;
  LLW_alphaBits=pfd.cAlphaBits;
  LLW_accumRedBits=pfd.cAccumRedBits;
  LLW_accumGreenBits=pfd.cAccumGreenBits;
  LLW_accumBlueBits=pfd.cAccumBlueBits;
  LLW_accumAlphaBits=pfd.cAccumAlphaBits;
***/
  LLW_commonWinCtx=wglCreateContext(LLW_commonWindow->hdc);
  if(!LLW_commonWinCtx)
    {
    goto endInit;
    }
  wglMakeCurrent(LLW_commonWindow->hdc,LLW_commonWinCtx);
  GL::init();
  glReadBuffer(GL_FRONT);
  glDrawBuffer(GL_BACK);
  LLW_initWGL();
  if(!LLW_hasPBuffer)
    {
    goto endInit;
    }
  LLW_commonPBuffer=(LLW_GLBufData_W32 *)LLW_createGLBuffer(16,16);
  if(!LLW_commonPBuffer)
    {
    goto endInit;
    }
  LLW_commonPBufCtx=wglCreateContext(LLW_commonPBuffer->hdc);
  if(!LLW_commonPBufCtx)
    {
    goto endInit;
    }
  wglShareLists(LLW_commonWinCtx,LLW_commonPBufCtx);
  wglMakeCurrent(LLW_commonPBuffer->hdc,LLW_commonPBufCtx);
  GL::init();
  glReadBuffer(GL_FRONT);
  glDrawBuffer(GL_FRONT);
  wglMakeCurrent(LLW_commonWindow->hdc,LLW_commonWinCtx);
  endInit:
  if(!LLW_commonPBufCtx)
    {
    LLW_hasPBuffer=false;
    }
  LLW_initState=LLW_UP;
  }
}

void
LLW::destroy(void) // Static
{
if(LLW_initState==LLW_UP)
  {
  LLW_initState=LLW_GOING_DOWN;
  while(!LLW_glBufDataMap->empty())
    {
    LLW::destroyGLBuffer((LLW::GLBufID)(*LLW_glBufDataMap->begin()).second);
    }
  delete LLW_glBufDataMap;
  LLW_glBufDataMap=(LLW_GLBufDataMap *)0;
  while(!LLW_pixDataMap->empty())
    {
    LLW::destroyPixmap((LLW::PixID)(*LLW_pixDataMap->begin()).second);
    }
  delete LLW_pixDataMap;
  LLW_pixDataMap=(LLW_PixDataMap *)0;
  while(!LLW_winDataMap->empty())
    {
    LLW_WinData_W32 * winData=(*LLW_winDataMap->begin()).second;
    if(winData==LLW_commonWindow)
      {
      LLW_winDataMap->erase(winData->win);
      }
    else
      {
      LLW::destroyWindow((LLW::WinID)winData,true);
      }
    }
  if(LLW_commonWindow)
    {
    LLW::destroyWindow((LLW::WinID)LLW_commonWindow,true);
    }
  delete LLW_winDataMap;
  LLW_winDataMap=(LLW_WinDataMap *)0;
  LLW_commonWindow=(LLW_WinData_W32 *)0;
  LLW_commonPBuffer=(LLW_GLBufData_W32 *)0;
  LLW_currentGLTarget=(LLW_AreaData *)0;
  LLW_mouseWindow=(LLW_WinData *)0;
  while(!LLW_fontDataMap->empty())
    {
    LLW::freeFont((LLW::FontID)(*LLW_fontDataMap->begin()).second);
    }
  delete LLW_fontDataMap;
  LLW_fontDataMap=(LLW_FontDataMap *)0;
  while(!LLW_glFontDataMap->empty())
    {
    LLW::freeGLFont((LLW::GLFontID)(*LLW_glFontDataMap->begin()).second);
    }
  delete LLW_glFontDataMap;
  LLW_glFontDataMap=(LLW_GLFontDataMap *)0;
  while(!LLW_bmFontDataMap->empty())
    {
    LLW::freeBMFont((LLW::BMFontID)(*LLW_bmFontDataMap->begin()).second);
    }
  delete LLW_bmFontDataMap;
  LLW_bmFontDataMap=(LLW_BMFontDataMap *)0;
  while(!LLW_pendingEvent->empty())
    {
    delete LLW_pendingEvent->front();
    LLW_pendingEvent->pop_front();
    }
  delete LLW_pendingEvent;
  LLW_pendingEvent=(LLW_EventList *)0;
  while(!LLW_freeEvent->empty())
    {
    delete LLW_freeEvent->front();
    LLW_freeEvent->pop_front();
    }
  delete LLW_freeEvent;
  LLW_freeEvent=(LLW_EventList *)0;
  while(!LLW_timeOutList->empty())
    {
    delete LLW_timeOutList->front();
    LLW_timeOutList->pop_front();
    }
  delete LLW_timeOutList;
  LLW_timeOutList=(LLW_TimeOutList *)0;
  delete LLW::_imageLoaderFncts;
  LLW::_imageLoaderFncts=(LLW::ImageLoaderFnctVect *)0;
  delete LLW::_imageSaverFncts;
  LLW::_imageSaverFncts=(LLW::ImageSaverFnctVect *)0;
  if(LLW_commonPBufCtx)
    {
    wglDeleteContext(LLW_commonPBufCtx);
    LLW_commonPBufCtx=(HGLRC)0;
    }
  if(LLW_commonWinCtx)
    {
    wglDeleteContext(LLW_commonWinCtx);
    LLW_commonWinCtx=(HGLRC)0;
    }
  UnregisterClass("AReVi-OpenGL",LLW_hInstance);
  LLW_hInstance=(HINSTANCE)0;
  LLW_lastCursor=(LLW::CursorID)-1;
  LLW_initState=LLW_DOWN;
  }
}

bool
LLW::initFail(void) // Static
{
if(LLW_initState==LLW_UP)
  {
  return(!LLW_commonWinCtx);
  }
return(true);
}

//-------- Window creation/destruction ---------------------------------------

static
LLW::WinID
LLW_createWindow(LLW::WinID parent,
                 int externalWindowID,
                 const StlString & title,
                 bool managed,
                 int x,
                 int y,
                 unsigned int width,
                 unsigned int height)
{
LLW_WinData_W32 * parentData=(LLW_WinData_W32 *)parent;
DWORD dwStyle=WS_CLIPSIBLINGS | WS_CLIPCHILDREN;
DWORD dwExStyle=0;
if(parentData||externalWindowID)
  {
  dwStyle|=WS_CHILD;
  }
else
  {
  dwStyle|=WS_POPUP | WS_SYSMENU | WS_MINIMIZEBOX | WS_MAXIMIZEBOX;
  if(!managed)
    {
    dwExStyle|=WS_EX_TOOLWINDOW;
    }
  }
HWND window=CreateWindowEx(dwExStyle,
                           "AReVi-OpenGL",
                           title.c_str(),
                           dwStyle,
                           x,y,width,height,
                           (parentData ? parentData->win
                                       : (HWND)externalWindowID),
                           (HMENU)0,
                           LLW_hInstance,
                           (LPVOID)0);
if(!window)
  {
  return((LLW::WinID)0);
  }
PIXELFORMATDESCRIPTOR pfd;
pfd.nSize=sizeof(PIXELFORMATDESCRIPTOR);
pfd.nVersion=1;
pfd.dwFlags=PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL | PFD_DOUBLEBUFFER;
pfd.iPixelType=PFD_TYPE_RGBA;
pfd.cColorBits=32;
pfd.cRedBits=0;
pfd.cRedShift=0;
pfd.cGreenBits=0;
pfd.cGreenShift=0;
pfd.cBlueBits=0;
pfd.cBlueShift=0;
pfd.cAlphaBits=0;
pfd.cAlphaShift=0;
pfd.cAccumBits=0;
pfd.cAccumRedBits=0;
pfd.cAccumGreenBits=0;
pfd.cAccumBlueBits=0;
pfd.cAccumAlphaBits=0;
pfd.cDepthBits=24;
pfd.cStencilBits=0;
pfd.cAuxBuffers=0;
pfd.iLayerType=PFD_MAIN_PLANE;
pfd.bReserved=0;
pfd.dwLayerMask=0;
pfd.dwVisibleMask=0;
pfd.dwDamageMask=0;
HDC hdc=GetDC(window);
int pixelFormat=ChoosePixelFormat(hdc,&pfd);
if(!pixelFormat||!SetPixelFormat(hdc,pixelFormat,&pfd))
  {
  ReleaseDC(window,hdc);
  DestroyWindow(window);
  return((LLW::WinID)0);
  }
SetTextAlign(hdc,TA_BASELINE);
SetBkMode(hdc,TRANSPARENT); // avoid text background
LLW_WinData_W32 * winData=new LLW_WinData_W32;
winData->parent=parentData;
if(parentData)
  {
  parentData->children.push_back(winData);
  }
winData->extWin=externalWindowID;
winData->title=title;
winData->mask=LLW::EVT_NONE;
winData->mapped=false;
winData->x=x;
winData->y=y;
winData->width=width;
winData->height=height;
winData->minWidth=0;
winData->minHeight=0;
winData->backColor=LLW::makeByteColor(128,128,128);
winData->backPixmap=(LLW_PixData *)0;
winData->win=window;
winData->hdc=hdc;
winData->mini=false;
winData->cursor=(LLW::CursorID)-1;
LLW::setWindowCursor((LLW::WinID)winData,LLW::CSR_ARROW);
LLW_updateWindowTitle((LLW::WinID)winData);
LLW_updateWindowBackground((LLW::WinID)winData);
LLW_updateWindowEventMask((LLW::WinID)winData);
LLW_updateWindowGeometry((LLW::WinID)winData);
LLW_winDataMap->insert(LLW_WinDataMap::value_type(winData->win,winData));
return((LLW::WinID)winData);
}

static
void
LLW_destroyWindow(LLW::WinID window)
{
LLW_WinData_W32 * winData=(LLW_WinData_W32 *)window;
if(winData==LLW_currentGLTarget)
  {
  if(winData==LLW_commonWindow)
    {
    wglMakeCurrent((HDC)0,(HGLRC)0);
    }
  else
    {
    LLW::beginGLRendering((LLW::AreaID)LLW_commonWindow);
    }
  }
ReleaseDC(winData->win,winData->hdc);
DestroyWindow(winData->win);
LLW_winDataMap->erase(winData->win);
delete winData;
}

int // platform-dependent window ID
LLW::getSpecificWindowID(LLW::WinID window) // Static
{
LLW_WinData_W32 * winData=(LLW_WinData_W32 *)window;
return((int)winData->win);
}

//-------- Pixmap creation/destruction ---------------------------------------

LLW::PixID
LLW::createPixmap(unsigned int width,
                  unsigned int height) // Static
{
HDC hdc=CreateCompatibleDC(((LLW_WinData_W32 *)LLW_commonWindow)->hdc);
if(!hdc)
  {
  return((LLW::PixID)0);
  }
HBITMAP pixmap=CreateCompatibleBitmap(((LLW_WinData_W32 *)LLW_commonWindow)
                                                          ->hdc,
                                        width,height);
if(!pixmap)
  {
  DeleteDC(hdc);
  return((LLW::PixID)0);
  }
SelectObject(hdc,pixmap);
SetTextAlign(hdc,TA_BASELINE);
SetBkMode(hdc,TRANSPARENT); // avoid text background
LLW_PixData_W32 * pixData=new LLW_PixData_W32;
pixData->width=width;
pixData->height=height;
pixData->pix=pixmap;
pixData->hdc=hdc;
LLW_pixDataMap->insert(LLW_PixDataMap::value_type(pixData->pix,pixData));
return((LLW::PixID)pixData);
}

void
LLW::destroyPixmap(LLW::PixID pixmap) // Static
{
LLW_PixData_W32 * pixData=(LLW_PixData_W32 *)pixmap;
DeleteDC(pixData->hdc);
DeleteObject(pixData->pix);
LLW_pixDataMap->erase(pixData->pix);
delete pixData;
}

//-------- GLBuffer creation/destruction -------------------------------------

static
LLW_GLBufData *
LLW_createGLBuffer(unsigned int width,
                   unsigned int height)
{
if(!LLW_hasPBuffer)
  {
  return((LLW_GLBufData *)0);
  }
int iAttr[]={ WGL_SUPPORT_OPENGL_ARB,  true,
              WGL_DRAW_TO_PBUFFER_ARB, true,
              WGL_DOUBLE_BUFFER_ARB, false,
              WGL_STEREO_ARB, false,
              WGL_PIXEL_TYPE_ARB, WGL_TYPE_RGBA_ARB,
              WGL_DEPTH_BITS_ARB, 24,
              WGL_RED_BITS_ARB, 8,
              WGL_GREEN_BITS_ARB, 8,
              WGL_BLUE_BITS_ARB, 8,
              WGL_ALPHA_BITS_ARB, 8,
              0 };
float fAttr[]={ 0.0 };
int format;
unsigned int nb=0;
if(!LLW_wglChoosePixelFormat(((LLW_WinData_W32 *)LLW_commonWindow)->hdc,
                             iAttr,fAttr,1,&format,&nb)||
   !nb)
  {
  return((LLW_GLBufData *)0);
  }
iAttr[0]=0;
HWND pbuffer=LLW_wglCreatePbuffer(((LLW_WinData_W32 *)LLW_commonWindow)->hdc,
                                  format,width,height,iAttr);
if(!pbuffer)
  {
  return((LLW_GLBufData *)0);
  }
HDC hdc=LLW_wglGetPbufferDC(pbuffer);
LLW_GLBufData_W32 * glBufData=new LLW_GLBufData_W32;
glBufData->width=width;
glBufData->height=height;
glBufData->pbuffer=pbuffer;
glBufData->hdc=hdc;
LLW_glBufDataMap->insert(LLW_GLBufDataMap::value_type(glBufData->pbuffer,
                                                      glBufData));
return(glBufData);
}

static
void
LLW_destroyGLBuffer(LLW::GLBufID buffer)
{
LLW_GLBufData_W32 * glBufData=(LLW_GLBufData_W32 *)buffer;
if(glBufData==LLW_currentGLTarget)
  {
  LLW::beginGLRendering((LLW::AreaID)LLW_commonWindow);
  }
LLW_wglReleasePbufferDC(glBufData->pbuffer,glBufData->hdc);
LLW_wglDestroyPbuffer(glBufData->pbuffer);
LLW_glBufDataMap->erase(glBufData->pbuffer);
delete glBufData;
}

//-------- Window management -------------------------------------------------

static
void
LLW_updateWindowTitle(LLW::WinID window)
{
LLW_WinData_W32 * winData=(LLW_WinData_W32 *)window;
SetWindowText(winData->win,winData->title.c_str());
}

void
LLW::mapWindow(LLW::WinID window) // Static
{
LLW_WinData_W32 * winData=(LLW_WinData_W32 *)window;
winData->mini=false;
winData->mapped=true;
ShowWindow(winData->win,SW_SHOWNORMAL);
SetForegroundWindow(winData->win);
}

void
LLW::unmapWindow(LLW::WinID window) // Static
{
LLW_WinData_W32 * winData=(LLW_WinData_W32 *)window;
winData->mapped=false;
winData->mini=false;
ShowWindow(winData->win,SW_HIDE);
}

void
LLW::iconifyTopWindow(LLW::WinID window) // Static
{
LLW_WinData_W32 * winData=(LLW_WinData_W32 *)window;
if(!winData->parent&&!winData->extWin)
  {
  winData->mapped=false;
  winData->mini=true;
  ShowWindow(winData->win,SW_MINIMIZE);
  }
}

static
void
LLW_updateWindowBackground(LLW::WinID /* window */)
{
  // Window$ does nothing, everything is done in LLW::clearWindow()
}

//-------- Cursor management -------------------------------------------------

void
LLW::setWindowCursor(LLW::WinID window,
                     LLW::CursorID cursor) // Static
{
LLW_WinData_W32 * winData=(LLW_WinData_W32 *)window;
if(winData->cursor!=cursor)
  {
  winData->cursor=cursor;
  if((winData==LLW_mouseWindow)&&
     (LLW_lastCursor!=winData->cursor))
    {
    LLW_lastCursor=winData->cursor;
    SetCursor(LLW_cursor[LLW_lastCursor]);
    }
  }
}

void
LLW::moveCursor(int dx,
                int dy) // Static
{
if(dx||dy)
  {
  POINT pt;
  GetCursorPos(&pt);
  SetCursorPos(pt.x+dx,pt.y+dy);
  }
}

//-------- Geometry ----------------------------------------------------------

static
void
LLW_updateWindowGeometry(LLW::WinID window)
{
LLW_WinData_W32 * winData=(LLW_WinData_W32 *)window;
MoveWindow(winData->win,
           winData->x,winData->y,
           winData->width,winData->height,
           TRUE);
}

void
LLW::getScreenSize(unsigned int & widthOut,
                   unsigned int & heightOut) // Static
{
RECT screen;
SystemParametersInfo(SPI_GETWORKAREA,0,&screen,0);
widthOut=screen.right-screen.left;
heightOut=screen.bottom-screen.top;
}

//-------- Color management --------------------------------------------------

unsigned int // color pixel value
LLW::makeDoubleColor(double red,
                     double green,
                     double blue) // Static
{
return(RGB(255.0*red,255.0*green,255.0*blue));
}

unsigned int // color pixel value
LLW::makeByteColor(unsigned char red,
                   unsigned char green,
                   unsigned char blue) // Static
{
return(RGB(red,green,blue));
}

//-------- Font management ---------------------------------------------------

LLW::FontID // loaded font
LLW::loadFont(const StlString & fontName) // Static
{
LLW_FontData * fontData=LLW_getFontData(fontName);
if(fontData)
  {
  ++(fontData->count);
  return((LLW::FontID)fontData);
  }
StlString name;
unsigned int size;
bool bold;
char slant;
LLW_parseFontName(fontName,name,size,bold,slant);
// perform well known substitutions
if((name=="fixed")||(name=="courier")) name="courier new";
else if((name=="variable")||(name=="helvetica")) name="arial";
else if(name=="times") name="times new roman";
else if(name=="new century schoolbook") name="century schoolbook";
else if((name=="lucida")||(name=="lucida sans")) name="lucida sans unicode";
else if(name=="lucidatypewriter") name="lucida console";
HFONT font=CreateFont(size,
                      0,0,0,
                      (bold ? FW_BOLD : FW_DONTCARE),
                      (slant=='r' ? FALSE : TRUE),
                      FALSE,FALSE,DEFAULT_CHARSET,
                      OUT_TT_PRECIS,CLIP_DEFAULT_PRECIS,
                      PROOF_QUALITY,DEFAULT_PITCH|FF_DONTCARE,
                      name.c_str());
if(!font) // second chance: load as is !
  {
  char buffer[32];
  strncpy(buffer,fontName.c_str(),32); // limitation in CreateFont() !
  buffer[31]='\0';
  font=CreateFont(0,
                  0,0,0,
                  FW_DONTCARE,
                  FALSE,
                  FALSE,FALSE,DEFAULT_CHARSET,
                  OUT_TT_PRECIS,CLIP_DEFAULT_PRECIS,
                  PROOF_QUALITY,DEFAULT_PITCH|FF_DONTCARE,
                  buffer);
  }
if(!font)
  {
  return((LLW::FontID)0);
  }
fontData=new LLW_FontData;
fontData->fontName=fontName;
fontData->font=font;
fontData->count=1;
LLW_fontDataMap->insert(LLW_FontDataMap::value_type(fontData->fontName,
                                                    fontData));
return((LLW::FontID)fontData);
}

void
LLW::freeFont(LLW::FontID font) // Static
{
LLW_FontData * fontData=(LLW_FontData *)font;
if(!(--(fontData->count)))
  {
  DeleteObject(fontData->font);
  LLW_fontDataMap->erase(fontData->fontName);
  delete fontData;
  }
}

void
LLW::getFontHeight(LLW::FontID font,
                   unsigned int & ascentOut,
                   unsigned int & descentOut) // Static
{
LLW_FontData * fontData=(LLW_FontData *)font;
HGDIOBJ oldFont=SelectObject(((LLW_WinData_W32 *)LLW_commonWindow)->hdc,
                             fontData->font);
TEXTMETRIC tm;
GetTextMetrics(((LLW_WinData_W32 *)LLW_commonWindow)->hdc,&tm);
SelectObject(((LLW_WinData_W32 *)LLW_commonWindow)->hdc,oldFont);
ascentOut=tm.tmAscent;
descentOut=tm.tmDescent;
}

unsigned int // text width
LLW::getTextWidth(LLW::FontID font,
                  const StlString & text) // Static
{
LLW_FontData * fontData=(LLW_FontData *)font;
HGDIOBJ oldFont=SelectObject(((LLW_WinData_W32 *)LLW_commonWindow)->hdc,
                             fontData->font);
SIZE sz;
GetTextExtentPoint32(((LLW_WinData_W32 *)LLW_commonWindow)->hdc,
                     text.c_str(),text.size(),&sz);
SelectObject(((LLW_WinData_W32 *)LLW_commonWindow)->hdc,oldFont);
return(sz.cx);
}

void
LLW::drawText(LLW::DrawID drawable,
              LLW::FontID font,
              unsigned int color,
              int x,
              int y,
              const StlString & text) // Static
{
HDC hdc=(((LLW_DrawData *)drawable)->isWindow()
         ? ((LLW_WinData_W32 *)drawable)->hdc
         : ((LLW_PixData_W32 *)drawable)->hdc);
HGDIOBJ oldFont=SelectObject(hdc,((LLW_FontData *)font)->font);
SetTextColor(hdc,color);
TextOut(hdc,x,y,text.c_str(),text.size());
SelectObject(hdc,oldFont);
}

//-------- Drawing -----------------------------------------------------------

void
LLW::drawPoint(LLW::DrawID drawable,
               unsigned int color,
               int x,
               int y) // Static
{
HDC hdc=(((LLW_DrawData *)drawable)->isWindow()
         ? ((LLW_WinData_W32 *)drawable)->hdc
         : ((LLW_PixData_W32 *)drawable)->hdc);
SetPixelV(hdc,x,y,color);
}

void
LLW::drawLine(LLW::DrawID drawable,
              unsigned int color,
              int x1,
              int y1,
              int x2,
              int y2) // Static
{
HDC hdc=(((LLW_DrawData *)drawable)->isWindow()
         ? ((LLW_WinData_W32 *)drawable)->hdc
         : ((LLW_PixData_W32 *)drawable)->hdc);
HPEN pen=CreatePen(PS_SOLID,0,color);
HGDIOBJ oldPen=SelectObject(hdc,pen);
MoveToEx(hdc,x1,y1,NULL);
LineTo(hdc,x2,y2);
SetPixelV(hdc,x2,y2,color);
SelectObject(hdc,oldPen);
DeleteObject(pen);
}

void
LLW::drawRectangle(LLW::DrawID drawable,
                   unsigned int color,
                   int x,
                   int y,
                   unsigned int width,
                   unsigned int height) // Static
{
HDC hdc=(((LLW_DrawData *)drawable)->isWindow()
         ? ((LLW_WinData_W32 *)drawable)->hdc
         : ((LLW_PixData_W32 *)drawable)->hdc);
HPEN pen=CreatePen(PS_SOLID,0,color);
HGDIOBJ oldPen=SelectObject(hdc,pen);
int x2=x+width-1;
int y2=y+height-1;
MoveToEx(hdc,x,y,NULL);
LineTo(hdc,x2,y);
LineTo(hdc,x2,y2);
LineTo(hdc,x,y2);
LineTo(hdc,x,y);
SelectObject(hdc,oldPen);
DeleteObject(pen);
}

void
LLW::fillRectangle(LLW::DrawID drawable,
                   unsigned int color,
                   int x,
                   int y,
                   unsigned int width,
                   unsigned int height) // Static
{
HDC hdc=(((LLW_DrawData *)drawable)->isWindow()
         ? ((LLW_WinData_W32 *)drawable)->hdc
         : ((LLW_PixData_W32 *)drawable)->hdc);
HPEN pen=CreatePen(PS_SOLID,0,color);
HGDIOBJ oldPen=SelectObject(hdc,pen);
HBRUSH brush=CreateSolidBrush(color);
HGDIOBJ oldBrush=SelectObject(hdc,brush);
Rectangle(hdc,x,y,x+width,y+height);
SelectObject(hdc,oldPen);
DeleteObject(pen);
SelectObject(hdc,oldBrush);
DeleteObject(brush);
}

void
LLW::drawEllipse(LLW::DrawID drawable,
                 unsigned int color,
                 int x,
                 int y,
                 unsigned int width,
                 unsigned int height) // Static
{
HDC hdc=(((LLW_DrawData *)drawable)->isWindow()
         ? ((LLW_WinData_W32 *)drawable)->hdc
         : ((LLW_PixData_W32 *)drawable)->hdc);
HPEN pen=CreatePen(PS_SOLID,0,color);
HGDIOBJ oldPen=SelectObject(hdc,pen);
Arc(hdc,x,y,x+width,y+height,
    x+width,y+(height>>1),
    x+width,y+(height>>1));
SelectObject(hdc,oldPen);
DeleteObject(pen);
}

void
LLW::fillEllipse(LLW::DrawID drawable,
                 unsigned int color,
                 int x,
                 int y,
                 unsigned int width,
                 unsigned int height) // Static
{
HDC hdc=(((LLW_DrawData *)drawable)->isWindow()
         ? ((LLW_WinData_W32 *)drawable)->hdc
         : ((LLW_PixData_W32 *)drawable)->hdc);
HPEN pen=CreatePen(PS_SOLID,0,color);
HGDIOBJ oldPen=SelectObject(hdc,pen);
HBRUSH brush=CreateSolidBrush(color);
HGDIOBJ oldBrush=SelectObject(hdc,brush);
Ellipse(hdc,x,y,x+width,y+height);
SelectObject(hdc,oldPen);
DeleteObject(pen);
SelectObject(hdc,oldBrush);
DeleteObject(brush);
}

void
LLW::drawArc(LLW::DrawID drawable,
             unsigned int color,
             int x,
             int y,
             unsigned int width,
             unsigned int height,
             double startAngle,
             double angle) // Static
{
HDC hdc=(((LLW_DrawData *)drawable)->isWindow()
         ? ((LLW_WinData_W32 *)drawable)->hdc
         : ((LLW_PixData_W32 *)drawable)->hdc);
HPEN pen=CreatePen(PS_SOLID,0,color);
HGDIOBJ oldPen=SelectObject(hdc,pen);
if(!(width%2)) --width;
if(!(height%2)) --height;
unsigned int w2=width>>1;
unsigned int h2=height>>1;
Arc(hdc,x,y,x+width,y+height,
    x+w2+(int)(cos(startAngle)*w2),
    y+h2-(int)(sin(startAngle)*h2),
    x+w2+(int)(cos(angle+startAngle)*w2),
    y+h2-(int)(sin(angle+startAngle)*h2));
SelectObject(hdc,oldPen);
DeleteObject(pen);
}

void
LLW::fillArc(LLW::DrawID drawable,
             unsigned int color,
             int x,
             int y,
             unsigned int width,
             unsigned int height,
             double startAngle,
             double angle) // Static
{
HDC hdc=(((LLW_DrawData *)drawable)->isWindow()
         ? ((LLW_WinData_W32 *)drawable)->hdc
         : ((LLW_PixData_W32 *)drawable)->hdc);
HPEN pen=CreatePen(PS_SOLID,0,color);
HGDIOBJ oldPen=SelectObject(hdc,pen);
HBRUSH brush=CreateSolidBrush(color);
HGDIOBJ oldBrush=SelectObject(hdc,brush);
if(!(width%2)) --width;
if(!(height%2)) --height;
unsigned int w2=width>>1;
unsigned int h2=height>>1;
Pie(hdc,x,y,x+width,y+height,
    x+w2+(int)(cos(startAngle)*w2),
    y+h2-(int)(sin(startAngle)*h2),
    x+w2+(int)(cos(angle+startAngle)*w2),
    y+h2-(int)(sin(angle+startAngle)*h2));
SelectObject(hdc,oldPen);
DeleteObject(pen);
SelectObject(hdc,oldBrush);
DeleteObject(brush);
}

void
LLW::copyDrawablePart(LLW::DrawID dest,
                      int destX,
                      int destY,
                      LLW::DrawID source,
                      int sourceX,
                      int sourceY,
                      unsigned int width,
                      unsigned int height) // Static
{
HDC hdcTo=(((LLW_DrawData *)dest)->isWindow()
           ? ((LLW_WinData_W32 *)dest)->hdc
           : ((LLW_PixData_W32 *)dest)->hdc);
HDC hdcFrom=(((LLW_DrawData *)source)->isWindow()
             ? ((LLW_WinData_W32 *)source)->hdc
             : ((LLW_PixData_W32 *)source)->hdc);
BitBlt(hdcTo,destX,destY,width,height,hdcFrom,sourceX,sourceY,SRCCOPY);
}

void
LLW::clearWindow(LLW::WinID window) // Static
{
LLW_WinData_W32 * winData=(LLW_WinData_W32 *)window;
if(winData->backPixmap)
  {
  LLW_PixData_W32 * pixData=(LLW_PixData_W32 *)winData->backPixmap;
  for(unsigned int y=0;y<winData->height;y+=pixData->height)
    {
    for(unsigned int x=0;x<winData->width;x+=pixData->width)
      {
      BitBlt(winData->hdc,x,y,pixData->width,pixData->height,
             pixData->hdc,0,0,SRCCOPY);
      }
    }
  }
else
  {
  LLW::fillRectangle(window,winData->backColor,
                     0,0,winData->width,winData->height);
  }
}

//-------- OpenGL management -------------------------------------------------

static
void
LLW_setCurrentGLWindow(void)
{
LLW_WinData_W32 * winData=(LLW_WinData_W32 *)LLW_currentGLTarget;
wglMakeCurrent(winData->hdc,LLW_commonWinCtx);
}

static
void
LLW_setCurrentGLBuffer(void)
{
LLW_GLBufData_W32 * glBufData=(LLW_GLBufData_W32 *)LLW_currentGLTarget;
wglMakeCurrent(glBufData->hdc,LLW_commonPBufCtx);
}

static
void
LLW_swapGLDoubleBuffers(void)
{
SwapBuffers(((LLW_WinData_W32 *)LLW_currentGLTarget)->hdc);
}

//-------- Event management --------------------------------------------------

static
void
LLW_updateWindowEventMask(LLW::WinID /* window */)
{
  // Nothing to be done
  // Window$ sends thousands of events even if you are not interested in !
}

static
void
LLW_updateMouseAndModifiers(LLW::Event * evt)
{
POINT pt;
GetCursorPos(&pt);
evt->xScreen=pt.x;
evt->yScreen=pt.y;
if(((LLW_AreaData *)evt->area)->isWindow())
  {
  ScreenToClient(((LLW_WinData_W32 *)evt->area)->win,&pt);
  }
evt->x=pt.x;
evt->y=pt.y;
int mod=LLW::MDF_NONE;
if(GetKeyState(VK_CONTROL)&0x8000) mod|=LLW::MDF_CONTROL;
if(GetKeyState(VK_SHIFT)&0x8000)   mod|=LLW::MDF_SHIFT;
if(GetKeyState(VK_CAPITAL)&0x0001) mod|=LLW::MDF_CAPSLOCK;
if(GetKeyState(VK_NUMLOCK)&0x0001) mod|=LLW::MDF_NUMLOCK;
if(GetKeyState(VK_LBUTTON)&0x8000) mod|=LLW::MDF_LEFT;
if(GetKeyState(VK_MBUTTON)&0x8000) mod|=LLW::MDF_MIDDLE;
if(GetKeyState(VK_RBUTTON)&0x8000) mod|=LLW::MDF_RIGHT;
evt->modifiers=(LLW::Modifier)mod;
}

LRESULT CALLBACK
LLW_wndProc(HWND hWnd,
            UINT uMsg,
            WPARAM wParam,
            LPARAM lParam)
{
// Ugly hack: Window$ doesn't know enter/leave events !
//            Trying to track the mouse in order to generate fake ones.
if(uMsg==WM_NCHITTEST)
  {
  LRESULT hitTest=DefWindowProc(hWnd,uMsg,wParam,lParam);
  if(hitTest==HTCLIENT)
    {
    LLW_WinData_W32 * old=(LLW_WinData_W32 *)LLW_mouseWindow;
    if(old&&(hWnd!=old->win))
      {
      LLW_postEvent(old,
                    LLW::EVT_MOUSE_INOUT,
                    LLW::BTN_NONE,
                    (const char *)0,
                    false,
                    ArSystem::getTime());
      }
    LLW_mouseWindow=(old&&(old->win==hWnd) ? old : LLW_getWinData(hWnd));
    if(LLW_mouseWindow&&(LLW_mouseWindow!=old))
      {
      LLW_postEvent(LLW_mouseWindow,
                    LLW::EVT_MOUSE_INOUT,
                    LLW::BTN_NONE,
                    (const char *)0,
                    true,
                    ArSystem::getTime());
      }
    }
  else
    {
    if(LLW_mouseWindow)
      {
      LLW_postEvent(LLW_mouseWindow,
                    LLW::EVT_MOUSE_INOUT,
                    LLW::BTN_NONE,
                    (const char *)0,
                    false,
                    ArSystem::getTime());
      LLW_mouseWindow=(LLW_WinData *)0;
      }
    }
  return(hitTest);
  }
LLW_WinData_W32 * winData=LLW_getWinData(hWnd);
if(winData)
  {
  if((winData==LLW_mouseWindow)&&
     (LLW_lastCursor!=winData->cursor))
    {
    LLW_lastCursor=winData->cursor;
    SetCursor(LLW_cursor[LLW_lastCursor]);
    }
  switch(uMsg)
    {
    case WM_MOUSEMOVE:
      {
      LLW_postEvent(winData,
                    LLW::EVT_MOUSE_MOTION,
                    LLW::BTN_NONE,
                    (const char *)0,
                    false,
                    ArSystem::getTime());
      return(0);
      }
    case WM_ERASEBKGND:
      {
      LLW::clearWindow((LLW::WinID)winData);
      return(1); // nonzero when painted
      }
    case WM_PAINT:
      {
      LLW_postEvent(winData,
                    LLW::EVT_EXPOSE,
                    LLW::BTN_NONE,
                    (const char *)0,
                    false,
                    ArSystem::getTime());
      break; // use default procedure too !
      }
    case WM_SETFOCUS:
      {
      if(!winData->parent&&!winData->extWin) // only for top windows
        {
        LLW_postEvent(winData,
                      LLW::EVT_FOCUS,
                      LLW::BTN_NONE,
                      (const char *)0,
                      true,
                      ArSystem::getTime());
        }
      return(0);
      }
    case WM_KILLFOCUS:
      {
      if(!winData->parent&&!winData->extWin) // only for top windows
        {
        LLW_postEvent(winData,
                      LLW::EVT_FOCUS,
                      LLW::BTN_NONE,
                      (const char *)0,
                      false,
                      ArSystem::getTime());
        }
      return(0);
      }
    case WM_LBUTTONDOWN:
    case WM_MBUTTONDOWN:
    case WM_RBUTTONDOWN:
    case WM_LBUTTONUP:
    case WM_MBUTTONUP:
    case WM_RBUTTONUP:
    case WM_MOUSEWHEEL:
      {
      LLW::Button button=LLW::BTN_NONE;
      bool onOff=false;
      switch(uMsg)
        {
        case WM_LBUTTONDOWN:
          {
          button=LLW::BTN_LEFT;
          onOff=true;
          break;
          }
        case WM_MBUTTONDOWN:
          {
          button=LLW::BTN_MIDDLE;
          onOff=true;
          break;
          }
        case WM_RBUTTONDOWN:
          {
          button=LLW::BTN_RIGHT;
          onOff=true;
          break;
          }
        case WM_LBUTTONUP:
          {
          button=LLW::BTN_LEFT;
          onOff=false;
          break;
          }
        case WM_MBUTTONUP:
          {
          button=LLW::BTN_MIDDLE;
          onOff=false;
          break;
          }
        case WM_RBUTTONUP:
          {
          button=LLW::BTN_RIGHT;
          onOff=false;
          break;
          }
       case WM_MOUSEWHEEL:
          {
          button=((short)(HIWORD(wParam))>0 ? LLW::BTN_FORWARD
                                            : LLW::BTN_BACKWARD);
          onOff=true;
          // Ugly hack: Window$ considers the mouse wheel as the keyboard !
          if(LLW_mouseWindow)
            {
            winData=(LLW_WinData_W32 *)LLW_mouseWindow;
            }
          else
            {
            return(0);
            }
          break;
          }
        }
      if(LLW_postEvent(winData,
                       LLW::EVT_MOUSE_BUTTON,
                       button,
                       (const char *)0,
                       onOff,
                       ArSystem::getTime()))
        {
        if(!(winData->mask&LLW::EVT_MOUSE_BUTTON_UNGRAB))
          {
          if(onOff)
            {
            SetCapture(winData->win);
            }
          else
            {
            ReleaseCapture();
            }
          }
        }
      return(0);
      }
    case WM_KEYDOWN:
    case WM_KEYUP:
      {
      if(!winData->keyRepeat&&(uMsg==WM_KEYDOWN)&&(lParam&(1<<30)))
        {
        return(0); // bit 30 --> key was already down, ignore
        }
      char asciiKey[2]={'\0','\0'};
      const char * key=(const char *)0;
      switch(wParam)
        {
        case VK_UP:      key="Up";       break;
        case VK_DOWN:    key="Down";     break;
        case VK_LEFT:    key="Left";     break;
        case VK_RIGHT:   key="Right";    break;
        case VK_PRIOR:   key="Prior";    break;
        case VK_NEXT:    key="Next";     break;
        case VK_HOME:    key="Home";     break;
        case VK_END:     key="End";      break;
        case VK_CLEAR:   key="Begin";    break;
        case VK_INSERT:  key="Insert";   break;
        case VK_DELETE:  key="Delete";   break;
        case VK_SHIFT:   key="Shift";    break;
        case VK_CONTROL: key="Control";  break;
        case VK_CAPITAL: key="CapsLock"; break;
        case VK_NUMLOCK: key="NumLock";  break;
        case VK_F1:      key="F1";       break;
        case VK_F2:      key="F2";       break;
        case VK_F3:      key="F3";       break;
        case VK_F4:      key="F4";       break;
        case VK_F5:      key="F5";       break;
        case VK_F6:      key="F6";       break;
        case VK_F7:      key="F7";       break;
        case VK_F8:      key="F8";       break;
        case VK_F9:      key="F9";       break;
        case VK_F10:     key="F10";      break;
        case VK_F11:     key="F11";      break;
        case VK_F12:     key="F12";      break;
        default:
          {
          BYTE keyboardState[256];
          if(GetKeyboardState(keyboardState))
            {
            keyboardState[VK_CONTROL]=0;  // ignore [Control] key
            keyboardState[VK_LCONTROL]=0;
            keyboardState[VK_RCONTROL]=0;
            WORD buffer=0;
            if(ToAscii(wParam,lParam,keyboardState,&buffer,0)==1)
              {
              asciiKey[0]=buffer;
              if(asciiKey[0]=='\r')
                {
                asciiKey[0]='\n';
                }
              key=asciiKey;
              }
            }
          }
        }
      if(key)
        {
        if(LLW_mouseWindow&&
           (LLW::getTopWindow(LLW_mouseWindow)==LLW::getTopWindow(winData)))
          {
          LLW_postEvent(LLW_mouseWindow,
                        LLW::EVT_KEYBOARD,
                        LLW::BTN_NONE,
                        key,
                        (uMsg==WM_KEYDOWN ? true : false),
                        ArSystem::getTime());
          }
        }
      return(0);
      }
    case WM_MOVE:
      {
      if(winData->mini) break; // ignore if iconified (see below)
      winData->x=(short)LOWORD(lParam);
      winData->y=(short)HIWORD(lParam);
      LLW_postEvent(winData,
                    LLW::EVT_GEOMETRY,
                    LLW::BTN_NONE,
                    (const char *)0,
                    false,
                    ArSystem::getTime());
      return(0);
      }
    case WM_SIZE:
      {
      if(winData->mini) break; // ignore if iconified (see below)
      winData->width=LOWORD(lParam);
      winData->height=HIWORD(lParam);
      bool update=false;
      if(winData->width<winData->minWidth)
        {
        winData->width=winData->minWidth;
        update=true;
        }
      if(winData->height<winData->minHeight)
        {
        winData->height=winData->minHeight;
        update=true;
        }
      if(update)
        {
        LLW_updateWindowGeometry((LLW::WinID)winData);
        }
      else
        {
        LLW_postEvent(winData,
                      LLW::EVT_GEOMETRY,
                      LLW::BTN_NONE,
                      (const char *)0,
                      false,
                      ArSystem::getTime());
        }
      return(0);
      }
    case WM_SYSCOMMAND:
      {
      // Ugly hack: Window$ signals minimization with WM_SYSACTIVATE
      //            after sending irrelevant WM_MOVE and WM_SIZE.
      bool mapped=winData->mapped;
      switch(wParam)
        {
        case SC_MINIMIZE:
          {
          winData->mini=true;
          winData->mapped=false;
          break;
          }
        case SC_MAXIMIZE:
        case SC_RESTORE:
          {
          winData->mini=false;
          winData->mapped=true;
          break;
          }
        }
      if(winData->mapped!=mapped)
        {
        LLW_postEvent(winData,
                      LLW::EVT_GEOMETRY,
                      LLW::BTN_NONE,
                      (const char *)0,
                      false,
                      ArSystem::getTime());
        }
      break; // use default procedure too !
      }
    case WM_GETMINMAXINFO:
      {
      MINMAXINFO * info=(MINMAXINFO *)lParam;
      unsigned int width,height;
      LLW::getScreenSize(width,height);
      info->ptMaxSize.x=width;
      info->ptMaxSize.y=height;
      return(0);
      }
    case WM_CLOSE:
      {
      LLW_postEvent(winData,
                    LLW::EVT_CLOSE,
                    LLW::BTN_NONE,
                    (const char *)0,
                    false,
                    ArSystem::getTime());
      return(0);
      }
    }
  }
return(DefWindowProc(hWnd,uMsg,wParam,lParam));
}

static
void
LLW_extractEvents(bool blocking)
{
MSG msg;
while(PeekMessage(&msg,(HWND)0,0,0,PM_REMOVE))
  {
  TranslateMessage(&msg);
  DispatchMessage(&msg);
  blocking=false;
  }
if(blocking)
  {
  GetMessage(&msg,(HWND)0,0,0);
  TranslateMessage(&msg);
  DispatchMessage(&msg);
  }
}

void
LLW::flush(void) // Static
{
GdiFlush();
}

//-------- Beep --------------------------------------------------------------

void
LLW::beep(void) // Static
{
MessageBeep(0xFFFFFFFF);
}

//-------- Image management --------------------------------------------------

static
void
LLW_recursiveDump(LLW::DrawID drawable,
                  unsigned int width,
                  unsigned int height,
                  int xOffset,
                  int yOffset,
                  char * dataOut)
{
HDC hdc=(((LLW_DrawData *)drawable)->isWindow()
         ? ((LLW_WinData_W32 *)drawable)->hdc
         : ((LLW_PixData_W32 *)drawable)->hdc);
unsigned int w,h;
LLW::getAreaSize(drawable,w,h);
int xmin=(xOffset>0 ? 0 : -xOffset);
int ymin=(yOffset>0 ? 0 : -yOffset);
int xmax=(xOffset+w<width ? w : width-xOffset)-1;
int ymax=(yOffset+h<height ? h : height-yOffset)-1;
for(int y=ymin;y<=ymax;y++)
  {
  unsigned int raw=height-1-(y+yOffset);
  for(int x=xmin;x<=xmax;x++)
    {
    unsigned int pixel=GetPixel(hdc,x,y);
    unsigned int index=3*(raw*width+x+xOffset);
    dataOut[index]=static_cast<char>(0x000000FF&pixel);
    dataOut[index+1]=static_cast<char>(0x000000FF&(pixel>>8));
    dataOut[index+2]=static_cast<char>(0x000000FF&(pixel>>16));
    }
  }
if(((LLW_DrawData *)drawable)->isWindow())
  {
  LLW_WinData * winData=(LLW_WinData *)drawable;
  for(unsigned int i=winData->children.size();i--;)
    {
    int x,y;
    LLW::getWindowPosition(winData->children[i],x,y);
    LLW_recursiveDump(winData->children[i],
                      width,height,xOffset+x,yOffset+y,dataOut);
    }
  }
}

static
void
LLW_dumpDrawable(LLW::DrawID drawable,
                 unsigned int width,
                 unsigned int height,
                 char * dataOut)
{
LLW_recursiveDump(drawable,width,height,0,0,dataOut);
}

} // namespace AReVi

//----------------------------------------------------------------------------
