// LI2/ENIB - AReVi - 3D services
//----------------------------------------------------------------------------
// file        : AReVi/Lib3D/llw_x11_priv.cpp
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include <GL/glx.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/keysym.h>
#include <X11/cursorfont.h>

namespace AReVi {

typedef void * LLW_GLXFBConfig; // avoid dependency with GLXFBConfigSGIX
typedef XID LLW_GLXPbuffer; // avoid dependency with GLXPbufferSGIX

//----------------------------------------------------------------------------
// Class LLW (abstract) - X11 specific part
//----------------------------------------------------------------------------

struct LLW_WinData_X11 : public LLW_WinData
{
LLW_WinData_X11(void) : LLW_WinData () { }
virtual ~LLW_WinData_X11(void) { }
Window win;
bool mini;
};

struct LLW_PixData_X11 : public LLW_PixData
{
LLW_PixData_X11(void) : LLW_PixData () { }
virtual ~LLW_PixData_X11(void) { }
Pixmap pix;
};

struct LLW_GLBufData_X11 : public LLW_GLBufData
{
LLW_GLBufData_X11(void) : LLW_GLBufData () { }
virtual ~LLW_GLBufData_X11(void) { }
LLW_GLXPbuffer pbuffer;
};

typedef StlMap<Window,LLW_WinData_X11 *,StlLess<Window> >
        LLW_WinDataMap;

typedef StlMap<Pixmap,LLW_PixData_X11 *,StlLess<Pixmap> >
        LLW_PixDataMap;

typedef StlMap<LLW_GLXPbuffer,LLW_GLBufData_X11 *,StlLess<LLW_GLXPbuffer> >
        LLW_GLBufDataMap;

struct LLW_FontData
{
StlString fontName;
XFontStruct * font;
unsigned int count;
};

typedef StlMap<StlString,LLW_FontData *,StlLess<StlString> >
        LLW_FontDataMap;

static Display * LLW_display=(Display *)0;

static XVisualInfo * LLW_commonWinVisual=(XVisualInfo *)0;

static int LLW_redShift=0,LLW_greenShift=0,LLW_blueShift=0;

static int LLW_redCut=0,LLW_greenCut=0,LLW_blueCut=0;

static double LLW_redFactor=0.0,LLW_greenFactor=0.0,LLW_blueFactor=0.0;

static Colormap LLW_colormap=(Colormap)0;

static GC LLW_gc=(GC)0;

static LLW_WinData_X11 * LLW_commonWindow=(LLW_WinData_X11 *)0;

static GLXContext LLW_commonWinCtx=(GLXContext)0;

static XVisualInfo * LLW_commonPBufVisual=(XVisualInfo *)0;

static LLW_GLBufData_X11 * LLW_commonPBuffer=(LLW_GLBufData_X11 *)0;

static GLXContext LLW_commonPBufCtx=(GLXContext)0;

static LLW_WinDataMap * LLW_winDataMap=(LLW_WinDataMap *)0;

static LLW_PixDataMap * LLW_pixDataMap=(LLW_PixDataMap *)0;

static LLW_GLBufDataMap * LLW_glBufDataMap=(LLW_GLBufDataMap *)0;

static LLW_FontDataMap * LLW_fontDataMap=(LLW_FontDataMap *)0;

static Cursor LLW_cursor[18];

//---- GLX_SGIX_fbconfig ----

static
LLW_GLXFBConfig
(APIENTRY * LLW_glXGetFBConfigFromVisual)(Display *,
                                          XVisualInfo *)=
  (LLW_GLXFBConfig (APIENTRY *)(Display *,XVisualInfo *))0;

//---- GLX_SGIX_pbuffer ----

static
LLW_GLXPbuffer
(APIENTRY * LLW_glXCreateGLXPbuffer)(Display *,
                                     LLW_GLXFBConfig,
                                     unsigned int,
                                     unsigned int,
                                     int *)=
  (LLW_GLXPbuffer (APIENTRY *)(Display *,LLW_GLXFBConfig,
                               unsigned int,unsigned int,int *))0;

static
void
(APIENTRY * LLW_glXDestroyGLXPbuffer)(Display *,
                                      LLW_GLXPbuffer)=
  (void (APIENTRY *)(Display *,LLW_GLXPbuffer))0;

static bool LLW_hasPBuffer=false;

inline static
LLW_WinData_X11 *
LLW_getWinData(Window window)
{
LLW_WinDataMap::iterator it=LLW_winDataMap->find(window);
if(it==LLW_winDataMap->end())
  {
  return((LLW_WinData_X11 *)0);
  }
else
  {
  return((*it).second);
  }
}

inline static
LLW_PixData_X11 *
LLW_getPixData(Pixmap pixmap)
{
LLW_PixDataMap::iterator it=LLW_pixDataMap->find(pixmap);
if(it==LLW_pixDataMap->end())
  {
  return((LLW_PixData_X11 *)0);
  }
else
  {
  return((*it).second);
  }
}

inline static
LLW_GLBufData_X11 *
LLW_getGLBufData(LLW_GLXPbuffer buffer)
{
LLW_GLBufDataMap::iterator it=LLW_glBufDataMap->find(buffer);
if(it==LLW_glBufDataMap->end())
  {
  return((LLW_GLBufData_X11 *)0);
  }
else
  {
  return((*it).second);
  }
}

inline static
LLW_FontData *
LLW_getFontData(const StlString & fontName)
{
LLW_FontDataMap::iterator it=LLW_fontDataMap->find(fontName);
if(it==LLW_fontDataMap->end())
  {
  return((LLW_FontData *)0);
  }
else
  {
  return((*it).second);
  }
}

static
int
LLW_ignoreXErrors(Display * /* display */,
                  XErrorEvent * /* event */)
{
return(True);
}

static
bool // success
LLW_checkVisual(XVisualInfo * info,
                bool doubleBuffer,
                bool stereo,
                bool accum,
                bool stencil)
{
if((info->screen!=DefaultScreen(LLW_display))||
   ((info->c_class!=TrueColor)&&(info->c_class!=DirectColor))||
   (!info->red_mask)||(!info->green_mask)||(!info->blue_mask))
  {
  return(false);
  }
int value;
if(glXGetConfig(LLW_display,info,GLX_USE_GL,&value)||!value)
  {
  return(false);
  }
if(glXGetConfig(LLW_display,info,GLX_RGBA,&value)||!value)
  {
  return(false);
  }
if(glXGetConfig(LLW_display,info,GLX_RED_SIZE,&value)||!value)
  {
  return(false);
  }
if(glXGetConfig(LLW_display,info,GLX_GREEN_SIZE,&value)||!value)
  {
  return(false);
  }
if(glXGetConfig(LLW_display,info,GLX_BLUE_SIZE,&value)||!value)
  {
  return(false);
  }
/* sometimes value is 0 even if alpha is supported !
if(glXGetConfig(LLW_display,info,GLX_ALPHA_SIZE,&value)||!value)
  {
  return(false);
  }
*/
if(glXGetConfig(LLW_display,info,GLX_DEPTH_SIZE,&value)||!value)
  {
  return(false);
  }
if(doubleBuffer)
  {
  if(glXGetConfig(LLW_display,info,GLX_DOUBLEBUFFER,&value)||!value)
    {
    return(false);
    }
  }
else
  {
  if(glXGetConfig(LLW_display,info,GLX_DOUBLEBUFFER,&value)||value)
    {
    return(false);
    }
  }
if(stereo)
  {
  if(glXGetConfig(LLW_display,info,GLX_STEREO,&value)||!value)
    {
    return(false);
    }
  }
else
  {
  if(glXGetConfig(LLW_display,info,GLX_STEREO,&value)||value)
    {
    return(false);
    }
  }
if(accum)
  {
  if(glXGetConfig(LLW_display,info,GLX_ACCUM_RED_SIZE,&value)||!value)
    {
    return(false);
    }
  if(glXGetConfig(LLW_display,info,GLX_ACCUM_GREEN_SIZE,&value)||!value)
    {
    return(false);
    }
  if(glXGetConfig(LLW_display,info,GLX_ACCUM_BLUE_SIZE,&value)||!value)
    {
    return(false);
    }
  if(glXGetConfig(LLW_display,info,GLX_ACCUM_ALPHA_SIZE,&value)||!value)
    {
    return(false);
    }
  }
if(stencil)
  {
  if(glXGetConfig(LLW_display,info,GLX_STENCIL_SIZE,&value)||!value)
    {
    return(false);
    }
  }
return(true);
}

static
XVisualInfo *
LLW_chooseVisual(bool doubleBuffer,
                 bool stereo,
                 bool accum,
                 bool stencil)
{
XVisualInfo * best=(XVisualInfo *)0;
StlVector<int> attrib;
attrib.push_back(GLX_RGBA);
attrib.push_back(GLX_RED_SIZE);
attrib.push_back(1);
attrib.push_back(GLX_GREEN_SIZE);
attrib.push_back(1);
attrib.push_back(GLX_BLUE_SIZE);
attrib.push_back(1);
// attrib.push_back(GLX_ALPHA_SIZE); // bad side effects under MacOsX ?
// attrib.push_back(1);
attrib.push_back(GLX_DEPTH_SIZE);
attrib.push_back(1);
if(doubleBuffer)
  {
  attrib.push_back(GLX_DOUBLEBUFFER);
  }
if(stereo)
  {
  attrib.push_back(GLX_STEREO);
  }
if(accum)
  {
  attrib.push_back(GLX_ACCUM_RED_SIZE);
  attrib.push_back(1);
  attrib.push_back(GLX_ACCUM_GREEN_SIZE);
  attrib.push_back(1);
  attrib.push_back(GLX_ACCUM_BLUE_SIZE);
  attrib.push_back(1);
  // attrib.push_back(GLX_ACCUM_ALPHA_SIZE); // bad side effects under MacOsX ?
  // attrib.push_back(1);
  }
if(stencil)
  {
  attrib.push_back(GLX_STENCIL_SIZE);
  attrib.push_back(1);
  }
attrib.push_back(None);
best=glXChooseVisual(LLW_display,DefaultScreen(LLW_display),&attrib[0]);
if(best)
  {
  if(!LLW_checkVisual(best,doubleBuffer,stereo,accum,stencil))
    {
    XFree(best); // reject configurations we don't want to use
    best=(XVisualInfo *)0;
    }
  }
if(!best) // try our own visual selection algorithm
  {
  int nbInfo;
  XVisualInfo * info=XGetVisualInfo(LLW_display,VisualNoMask,
                                    (XVisualInfo *)0,&nbInfo);
  if(info)
    {
    for(unsigned int i=nbInfo;i--;)
      {
      if(LLW_checkVisual(&info[i],doubleBuffer,stereo,accum,stencil))
        {
        if(!best) // first match
          {
          best=info+i;
          }
        else if(info[i].depth>best->depth) // better depth
          {
          best=info+i;
          }
        else if((info[i].depth==best->depth)&& // prefer TrueColor
                (info[i].c_class==TrueColor))  // to DirectColor
          {
          best=info+i;
          }
        }
      }
    if(best)
      {
      XVisualInfo tmp;
      tmp.screen=best->screen;
      tmp.visualid=best->visualid; // get only this visual
      best=XGetVisualInfo(LLW_display,VisualScreenMask|VisualIDMask,
                          &tmp,&nbInfo);
      }
    XFree(info);
    }
  }
if(!best&&!doubleBuffer) // try double-buffer if single is not availlable
  {
  best=LLW_chooseVisual(true,stereo,accum,stencil);
  }
return(best);
}

static
void
LLW_calibratePixel(void)
{
unsigned int mask=LLW_commonWinVisual->red_mask;
for(LLW_redShift=0;!(mask&0x00000001);mask>>=1)
  {
  ++LLW_redShift;
  }
LLW_redFactor=(double)mask;
for(LLW_redCut=8;mask;mask>>=1)
  {
  --LLW_redCut;
  }
mask=LLW_commonWinVisual->green_mask;
for(LLW_greenShift=0;!(mask&0x00000001);mask>>=1)
  {
  ++LLW_greenShift;
  }
LLW_greenFactor=(double)mask;
for(LLW_greenCut=8;mask;mask>>=1)
  {
  --LLW_greenCut;
  }
mask=LLW_commonWinVisual->blue_mask;
for(LLW_blueShift=0;!(mask&0x00000001);mask>>=1)
  {
  ++LLW_blueShift;
  }
LLW_blueFactor=(double)mask;
for(LLW_blueCut=8;mask;mask>>=1)
  {
  --LLW_blueCut;
  }
}

//-------- Initialization ----------------------------------------------------

static
void
LLW_initGLX(void)
{
const char *glxExt=glXQueryExtensionsString(LLW_display,
                                            DefaultScreen(LLW_display));
//---- fbconfig ----
if(GL::hasExtension("GLX_SGIX_fbconfig",glxExt))
  {
  GL::loadFunction(LLW_glXGetFBConfigFromVisual,
                   "glXGetFBConfigFromVisual","SGIX");
  }
//---- pbuffer ----
if(GL::hasExtension("GLX_SGIX_pbuffer",glxExt))
  {
  GL::loadFunction(LLW_glXCreateGLXPbuffer,"glXCreateGLXPbuffer","SGIX");
  GL::loadFunction(LLW_glXDestroyGLXPbuffer,"glXDestroyGLXPbuffer","SGIX");
  }
LLW_hasPBuffer=false;
if(LLW_glXGetFBConfigFromVisual&&
   LLW_glXCreateGLXPbuffer&&
   LLW_glXDestroyGLXPbuffer)
  {
  LLW_hasPBuffer=true;
  }
else
  {
  cerr << "!!! GLX_SGIX_pbuffer extension is not supported !!!"
       << endl;
  }
}

void
LLW::init(void) // Static
{
if(LLW_initState==LLW_DOWN)
  {
  LLW_initState=LLW_GOING_UP;
  LLW::_imageLoaderFncts=new LLW::ImageLoaderFnctVect;
  LLW::_imageLoaderFncts->push_back(&LLW_loadTGAImage);
  LLW::_imageSaverFncts=new LLW::ImageSaverFnctVect;
  LLW::_imageSaverFncts->push_back(&LLW_saveTGAImage);
  LLW_pendingEvent=new LLW_EventList;
  LLW_freeEvent=new LLW_EventList;
  LLW_timeOutList=new LLW_TimeOutList;
  LLW_winDataMap=new LLW_WinDataMap;
  LLW_pixDataMap=new LLW_PixDataMap;
  LLW_glBufDataMap=new LLW_GLBufDataMap;
  LLW_fontDataMap=new LLW_FontDataMap;
  LLW_glFontDataMap=new LLW_GLFontDataMap;
  LLW_bmFontDataMap=new LLW_BMFontDataMap;
  LLW_display=XOpenDisplay("");
  LLW_GLXFBConfig cfg=(LLW_GLXFBConfig)0;
  if(!LLW_display)
    {
    goto endInit;
    }
  // XSynchronize(LLW_display,True);
  {
  char blankData[1]={0};
  XColor dummyColor;
  Pixmap blankPixmap=XCreateBitmapFromData(LLW_display,
                                           DefaultRootWindow(LLW_display),
                                           blankData,1,1);
  LLW_cursor[LLW::CSR_EMPTY]=XCreatePixmapCursor(LLW_display,
                                                 blankPixmap,blankPixmap,
                                                 &dummyColor,&dummyColor,0,0);
  XFreePixmap(LLW_display,blankPixmap);
  }
  LLW_cursor[LLW::CSR_ARROW]=XCreateFontCursor(LLW_display,
                                               XC_left_ptr);
  LLW_cursor[LLW::CSR_NORTH]=XCreateFontCursor(LLW_display,
                                               XC_top_side);
  LLW_cursor[LLW::CSR_SOUTH]=XCreateFontCursor(LLW_display,
                                               XC_bottom_side);
  LLW_cursor[LLW::CSR_EAST]=XCreateFontCursor(LLW_display,
                                              XC_right_side);
  LLW_cursor[LLW::CSR_WEST]=XCreateFontCursor(LLW_display,
                                              XC_left_side);
  LLW_cursor[LLW::CSR_NORTHEAST]=XCreateFontCursor(LLW_display,
                                                   XC_top_right_corner);
  LLW_cursor[LLW::CSR_NORTHWEST]=XCreateFontCursor(LLW_display,
                                                   XC_top_left_corner);
  LLW_cursor[LLW::CSR_SOUTHEAST]=XCreateFontCursor(LLW_display,
                                                   XC_bottom_right_corner);
  LLW_cursor[LLW::CSR_SOUTHWEST]=XCreateFontCursor(LLW_display,
                                                   XC_bottom_left_corner);
  LLW_cursor[LLW::CSR_MOVE]=XCreateFontCursor(LLW_display,
                                              XC_fleur);
  LLW_cursor[LLW::CSR_CROSS]=XCreateFontCursor(LLW_display,
                                               XC_crosshair);
  LLW_cursor[LLW::CSR_WAIT]=XCreateFontCursor(LLW_display,
                                              XC_watch);
  LLW_cursor[LLW::CSR_TEXT]=XCreateFontCursor(LLW_display,
                                              XC_xterm);
  LLW_cursor[LLW::CSR_HAND]=XCreateFontCursor(LLW_display,
                                              XC_hand2);
  LLW_cursor[LLW::CSR_HELP]=XCreateFontCursor(LLW_display,
                                              XC_question_arrow);
  LLW_cursor[LLW::CSR_UP]=XCreateFontCursor(LLW_display,
                                            XC_center_ptr);
  LLW_cursor[LLW::CSR_NO]=XCreateFontCursor(LLW_display,
                                            XC_circle);
  LLW_commonWinVisual=LLW_chooseVisual(true,false,false,false);
  if(!LLW_commonWinVisual)
    {
    goto endInit;
    }
  LLW_calibratePixel();
  LLW_colormap=XCreateColormap(LLW_display,
                               DefaultRootWindow(LLW_display),
                               LLW_commonWinVisual->visual,
                               AllocNone);
  if(!LLW_colormap)
    {
    goto endInit;
    }
  LLW_commonWindow=(LLW_WinData_X11 *)LLW::createTopWindow("",false,0,0,16,16);
  if(!LLW_commonWindow)
    {
    goto endInit;
    }
  LLW_gc=XCreateGC(LLW_display,((LLW_WinData_X11 *)LLW_commonWindow)->win,
                   0,(XGCValues *)0);
  if(!LLW_gc)
    {
    goto endInit;
    }
  LLW_commonWinCtx=glXCreateContext(LLW_display,
                                    LLW_commonWinVisual,
                                    (GLXContext)0,true);
  if(!LLW_commonWinCtx)
    {
    goto endInit;
    }
  glXMakeCurrent(LLW_display,LLW_commonWindow->win,LLW_commonWinCtx);
  GL::init();
  glReadBuffer(GL_FRONT);
  glDrawBuffer(GL_BACK);
  LLW_initGLX();
  if(!LLW_hasPBuffer)
    {
    goto endInit;
    }
  LLW_commonPBufVisual=LLW_chooseVisual(false,false,false,false);
  if(!LLW_commonPBufVisual)
    {
    goto endInit;
    }
  cfg=LLW_glXGetFBConfigFromVisual(LLW_display,LLW_commonPBufVisual);
  if(!cfg)
    {
    goto endInit;
    }
  LLW_commonPBuffer=(LLW_GLBufData_X11 *)LLW_createGLBuffer(16,16);
  if(!LLW_commonPBuffer)
    {
    goto endInit;
    }
  LLW_commonPBufCtx=glXCreateContext(LLW_display,LLW_commonPBufVisual,
                                     LLW_commonWinCtx,true);
  if(!LLW_commonPBufCtx)
    {
    goto endInit;
    }
  glXMakeCurrent(LLW_display,LLW_commonPBuffer->pbuffer,LLW_commonPBufCtx);
  GL::init();
  glReadBuffer(GL_FRONT);
  glDrawBuffer(GL_FRONT);
  glXMakeCurrent(LLW_display,LLW_commonWindow->win,LLW_commonWinCtx);
  endInit:
  if(!LLW_commonPBufCtx)
    {
    LLW_hasPBuffer=false;
    }
  LLW_initState=LLW_UP;
  }
}

void
LLW::destroy(void) // Static
{
if(LLW_initState==LLW_UP)
  {
  LLW_initState=LLW_GOING_DOWN;
  while(!LLW_glBufDataMap->empty())
    {
    LLW::destroyGLBuffer((LLW::GLBufID)(*LLW_glBufDataMap->begin()).second);
    }
  delete LLW_glBufDataMap;
  LLW_glBufDataMap=(LLW_GLBufDataMap *)0;
  while(!LLW_pixDataMap->empty())
    {
    LLW::destroyPixmap((LLW::PixID)(*LLW_pixDataMap->begin()).second);
    }
  delete LLW_pixDataMap;
  LLW_pixDataMap=(LLW_PixDataMap *)0;
  while(!LLW_winDataMap->empty())
    {
    LLW_WinData_X11 * winData=(*LLW_winDataMap->begin()).second;
    if(winData==LLW_commonWindow)
      {
      LLW_winDataMap->erase(winData->win);
      }
    else
      {
      LLW::destroyWindow((LLW::WinID)winData,true);
      }
    }
  if(LLW_commonWindow)
    {
    LLW::destroyWindow((LLW::WinID)LLW_commonWindow,true);
    }
  delete LLW_winDataMap;
  LLW_winDataMap=(LLW_WinDataMap *)0;
  LLW_commonWindow=(LLW_WinData_X11 *)0;
  LLW_commonPBuffer=(LLW_GLBufData_X11 *)0;
  LLW_currentGLTarget=(LLW_AreaData *)0;
  LLW_mouseWindow=(LLW_WinData *)0;
  while(!LLW_fontDataMap->empty())
    {
    LLW::freeFont((LLW::FontID)(*LLW_fontDataMap->begin()).second);
    }
  delete LLW_fontDataMap;
  LLW_fontDataMap=(LLW_FontDataMap *)0;
  while(!LLW_glFontDataMap->empty())
    {
    LLW::freeGLFont((LLW::GLFontID)(*LLW_glFontDataMap->begin()).second);
    }
  delete LLW_glFontDataMap;
  LLW_glFontDataMap=(LLW_GLFontDataMap *)0;
  while(!LLW_bmFontDataMap->empty())
    {
    LLW::freeBMFont((LLW::BMFontID)(*LLW_bmFontDataMap->begin()).second);
    }
  delete LLW_bmFontDataMap;
  LLW_bmFontDataMap=(LLW_BMFontDataMap *)0;
  while(!LLW_pendingEvent->empty())
    {
    delete LLW_pendingEvent->front();
    LLW_pendingEvent->pop_front();
    }
  delete LLW_pendingEvent;
  LLW_pendingEvent=(LLW_EventList *)0;
  while(!LLW_freeEvent->empty())
    {
    delete LLW_freeEvent->front();
    LLW_freeEvent->pop_front();
    }
  delete LLW_freeEvent;
  LLW_freeEvent=(LLW_EventList *)0;
  while(!LLW_timeOutList->empty())
    {
    delete LLW_timeOutList->front();
    LLW_timeOutList->pop_front();
    }
  delete LLW_timeOutList;
  LLW_timeOutList=(LLW_TimeOutList *)0;
  delete LLW::_imageLoaderFncts;
  LLW::_imageLoaderFncts=(LLW::ImageLoaderFnctVect *)0;
  delete LLW::_imageSaverFncts;
  LLW::_imageSaverFncts=(LLW::ImageSaverFnctVect *)0;
  if(LLW_commonPBufCtx)
    {
    glXDestroyContext(LLW_display,LLW_commonPBufCtx);
    LLW_commonPBufCtx=(GLXContext)0;
    }
  if(LLW_commonPBufVisual)
    {
    XFree(LLW_commonPBufVisual);
    LLW_commonPBufVisual=(XVisualInfo *)0;
    }
  if(LLW_gc)
    {
    XFreeGC(LLW_display,LLW_gc);
    LLW_gc=(GC)0;
    }
  if(LLW_colormap)
    {
    XFreeColormap(LLW_display,LLW_colormap);
    LLW_colormap=(Colormap)0;
    }
  if(LLW_commonWinCtx)
    {
    glXDestroyContext(LLW_display,LLW_commonWinCtx);
    LLW_commonWinCtx=(GLXContext)0;
    }
  if(LLW_commonWinVisual)
    {
    XFree(LLW_commonWinVisual);
    LLW_commonWinVisual=(XVisualInfo *)0;
    LLW_redShift=LLW_greenShift=LLW_blueShift=0;
    LLW_redCut=LLW_greenCut=LLW_blueCut=0;
    LLW_redFactor=LLW_greenFactor=LLW_blueFactor=0.0;
    }
  if(LLW_display)
    {
    XFreeCursor(LLW_display,LLW_cursor[LLW::CSR_EMPTY]);
    XFreeCursor(LLW_display,LLW_cursor[LLW::CSR_ARROW]);
    XFreeCursor(LLW_display,LLW_cursor[LLW::CSR_NORTH]);
    XFreeCursor(LLW_display,LLW_cursor[LLW::CSR_SOUTH]);
    XFreeCursor(LLW_display,LLW_cursor[LLW::CSR_EAST]);
    XFreeCursor(LLW_display,LLW_cursor[LLW::CSR_WEST]);
    XFreeCursor(LLW_display,LLW_cursor[LLW::CSR_NORTHEAST]);
    XFreeCursor(LLW_display,LLW_cursor[LLW::CSR_NORTHWEST]);
    XFreeCursor(LLW_display,LLW_cursor[LLW::CSR_SOUTHEAST]);
    XFreeCursor(LLW_display,LLW_cursor[LLW::CSR_SOUTHWEST]);
    XFreeCursor(LLW_display,LLW_cursor[LLW::CSR_MOVE]);
    XFreeCursor(LLW_display,LLW_cursor[LLW::CSR_CROSS]);
    XFreeCursor(LLW_display,LLW_cursor[LLW::CSR_WAIT]);
    XFreeCursor(LLW_display,LLW_cursor[LLW::CSR_TEXT]);
    XFreeCursor(LLW_display,LLW_cursor[LLW::CSR_HAND]);
    XFreeCursor(LLW_display,LLW_cursor[LLW::CSR_HELP]);
    XFreeCursor(LLW_display,LLW_cursor[LLW::CSR_UP]);
    XFreeCursor(LLW_display,LLW_cursor[LLW::CSR_NO]);
    XCloseDisplay(LLW_display);
    LLW_display=(Display *)0;
    }
  LLW_initState=LLW_DOWN;
  }
}

bool
LLW::initFail(void) // Static
{
if(LLW_initState==LLW_UP)
  {
  return(!LLW_commonWinCtx);
  }
return(true);
}

//-------- Window creation/destruction ---------------------------------------

static
LLW::WinID
LLW_createWindow(LLW::WinID parent,
                 int externalWindowID,
                 const StlString & title,
                 bool managed,
                 int x,
                 int y,
                 unsigned int width,
                 unsigned int height) // Static
{
LLW_WinData_X11 * parentData=(LLW_WinData_X11 *)parent;
XSetWindowAttributes attr;
attr.colormap=LLW_colormap;
attr.override_redirect=(managed ? False : True);
attr.background_pixel=WhitePixel(LLW_display,DefaultScreen(LLW_display));
attr.border_pixel=BlackPixel(LLW_display,DefaultScreen(LLW_display));
Window parentWin=(parentData ? parentData->win
                  : (externalWindowID ? (Window)externalWindowID
                     : DefaultRootWindow(LLW_display)));
Window window=XCreateWindow(LLW_display,
                            parentWin,
                            x,y,width,height,0,
                            LLW_commonWinVisual->depth,
                            InputOutput,
                            LLW_commonWinVisual->visual,
                            CWColormap|CWOverrideRedirect|
                            CWBackPixel|CWBorderPixel,
                            &attr);
if(!window)
  {
  return((LLW::WinID)0);
  }
Atom ha=XInternAtom(LLW_display,"_MOTIF_WM_HINTS",False);
Atom ta;
int f;
unsigned long n,b;
long ** hints;
XGetWindowProperty(LLW_display,window,ha,0,5,False,AnyPropertyType,
                   &ta,&f,&n,&b,(unsigned char **)(void *)&hints);
if(ta==None)
  {
  long newHints[5];
  newHints[0]=(1L<<1); // flags: MWM_HINTS_DECORATIONS
  newHints[2]=0L; // no decoration !
  XChangeProperty(LLW_display,window,ha,ha,32,PropModeReplace,
                  (unsigned char *)&newHints,5);
  }
else // change existing hints
  {
  (*hints)[0]|=(1L<<1); // flags: MWM_HINTS_DECORATIONS
  (*hints)[2]=0L; // no decoration !
  XChangeProperty(LLW_display,window,ha,ha,32,PropModeReplace,
                  (unsigned char *)hints,5);
  XFree(hints);
  }
LLW_WinData_X11 * winData=new LLW_WinData_X11;
winData->parent=parentData;
if(parentData)
  {
  parentData->children.push_back(winData);
  }
winData->extWin=externalWindowID;
winData->title=title;
winData->mask=LLW::EVT_NONE;
winData->keyRepeat=false;
winData->mapped=false;
winData->x=x;
winData->y=y;
winData->width=width;
winData->height=height;
winData->minWidth=0;
winData->minHeight=0;
winData->backColor=LLW::makeByteColor(128,128,128);
winData->backPixmap=(LLW_PixData *)0;
winData->win=window;
winData->mini=false;
winData->cursor=(LLW::CursorID)-1;
LLW::setWindowCursor((LLW::WinID)winData,LLW::CSR_ARROW);
LLW_updateWindowTitle((LLW::WinID)winData);
LLW_updateWindowBackground((LLW::WinID)winData);
LLW_updateWindowEventMask((LLW::WinID)winData);
LLW_updateWindowGeometry((LLW::WinID)winData);
LLW_winDataMap->insert(LLW_WinDataMap::value_type(winData->win,winData));
return((LLW::WinID)winData);
}

static
void
LLW_destroyWindow(LLW::WinID window)
{
LLW_WinData_X11 * winData=(LLW_WinData_X11 *)window;
LLW::flush();
if(winData==LLW_currentGLTarget)
  {
  if(winData==LLW_commonWindow)
    {
    glXMakeCurrent(LLW_display,(GLXDrawable)0,(GLXContext)0);
    }
  else
    {
    LLW::beginGLRendering((LLW::AreaID)LLW_commonWindow);
    }
  }
XDestroyWindow(LLW_display,winData->win);
LLW_winDataMap->erase(winData->win);
delete winData;
}

int // platform-dependent window ID
LLW::getSpecificWindowID(LLW::WinID window) // Static
{
LLW_WinData_X11 * winData=(LLW_WinData_X11 *)window;
return((int)winData->win);
}

//-------- Pixmap creation/destruction ---------------------------------------

LLW::PixID
LLW::createPixmap(unsigned int width,
                  unsigned int height) // Static
{
Pixmap pixmap=XCreatePixmap(LLW_display,
                            ((LLW_WinData_X11 *)LLW_commonWindow)->win,
                            width,height,
                            LLW_commonWinVisual->depth);
if(!pixmap)
  {
  return((LLW::PixID)0);
  }
LLW_PixData_X11 * pixData=new LLW_PixData_X11;
pixData->width=width;
pixData->height=height;
pixData->pix=pixmap;
LLW_pixDataMap->insert(LLW_PixDataMap::value_type(pixData->pix,pixData));
return((LLW::PixID)pixData);
}

void
LLW::destroyPixmap(LLW::PixID pixmap) // Static
{
LLW_PixData_X11 * pixData=(LLW_PixData_X11 *)pixmap;
XFreePixmap(LLW_display,pixData->pix);
LLW_pixDataMap->erase(pixData->pix);
delete pixData;
}

//-------- GLBuffer creation/destruction -------------------------------------

static
LLW_GLBufData *
LLW_createGLBuffer(unsigned int width,
                   unsigned int height)
{
if(!LLW_hasPBuffer)
  {
  return((LLW_GLBufData *)0);
  }
LLW_GLXFBConfig cfg=LLW_glXGetFBConfigFromVisual(LLW_display,
                                                 LLW_commonPBufVisual);
int (*old)(Display *,XErrorEvent *)=XSetErrorHandler(&LLW_ignoreXErrors);
LLW_GLXPbuffer pbuffer=LLW_glXCreateGLXPbuffer(LLW_display,cfg,
                                               width,height,(int *)0);
XSetErrorHandler(old);
if(!pbuffer)
  {
  return((LLW_GLBufData *)0);
  }
LLW_GLBufData_X11 * glBufData=new LLW_GLBufData_X11;
glBufData->width=width;
glBufData->height=height;
glBufData->fb=0;
glBufData->pbuffer=pbuffer;
LLW_glBufDataMap->insert(LLW_GLBufDataMap::value_type(glBufData->pbuffer,
                                                      glBufData));
return(glBufData);
}

static
void
LLW_destroyGLBuffer(LLW::GLBufID buffer)
{
LLW_GLBufData_X11 * glBufData=(LLW_GLBufData_X11 *)buffer;
if(glBufData==LLW_currentGLTarget)
  {
  LLW::beginGLRendering((LLW::AreaID)LLW_commonWindow);
  }
LLW_glXDestroyGLXPbuffer(LLW_display,glBufData->pbuffer);
LLW_glBufDataMap->erase(glBufData->pbuffer);
delete glBufData;
}

//-------- Window management -------------------------------------------------

static
void
LLW_updateWindowTitle(LLW::WinID window)
{
LLW_WinData_X11 * winData=(LLW_WinData_X11 *)window;
const char * str=winData->title.c_str();
XTextProperty textProperty;
XStringListToTextProperty((char **)&str,1,&textProperty);
XSetWMName(LLW_display,winData->win,&textProperty);
XSetWMIconName(LLW_display,winData->win,&textProperty);
}

void
LLW::mapWindow(LLW::WinID window) // Static
{
LLW_WinData_X11 * winData=(LLW_WinData_X11 *)window;
if(winData->mapped)
  {
  if(!winData->parent&&!winData->extWin)
    {
    XRaiseWindow(LLW_display,winData->win);
    }
  }
else
  {
  if(!winData->parent&&!winData->extWin)
    {
    XSizeHints * szHints=XAllocSizeHints();
    szHints->flags=USPosition|USSize|PWinGravity;
    szHints->x=winData->x;
    szHints->y=winData->y;
    szHints->width=winData->width;
    szHints->height=winData->height;
    szHints->win_gravity=StaticGravity;
    XSetWMNormalHints(LLW_display,winData->win,szHints);
    XFree(szHints);
    XWMHints * wmHints=XAllocWMHints();
    wmHints->flags=StateHint;
    wmHints->initial_state=NormalState;
    XSetWMHints(LLW_display,winData->win,wmHints);
    XFree(wmHints);
    }
  winData->mapped=true;
  winData->mini=false;
  XMapRaised(LLW_display,winData->win);
  }
}

void
LLW::unmapWindow(LLW::WinID window) // Static
{
LLW_WinData_X11 * winData=(LLW_WinData_X11 *)window;
if(winData->parent||winData->extWin)
  {
  winData->mapped=false;
  XUnmapWindow(LLW_display,winData->win);
  }
else
  {
  winData->mapped=false;
  if(winData->mini)
    {
    winData->mini=false;
    XWithdrawWindow(LLW_display,winData->win,DefaultScreen(LLW_display));
    }
  else
    {
    XUnmapWindow(LLW_display,winData->win);
    }
  }
}

void
LLW::iconifyTopWindow(LLW::WinID window) // Static
{
LLW_WinData_X11 * winData=(LLW_WinData_X11 *)window;
if(!winData->parent&&!winData->extWin)
  {
  if(winData->mapped)
    {
    winData->mini=true;
    winData->mapped=false;
    XIconifyWindow(LLW_display,winData->win,DefaultScreen(LLW_display));
    }
  else
    {
    if(!winData->mini)
      {
      winData->mini=true;
      XSizeHints * szHints=XAllocSizeHints();
      szHints->flags=USPosition|USSize;
      szHints->x=winData->x;
      szHints->y=winData->y;
      szHints->width=winData->width;
      szHints->height=winData->height;
      XSetWMNormalHints(LLW_display,winData->win,szHints);
      XFree(szHints);
      XWMHints * wmHints=XAllocWMHints();
      wmHints->flags=StateHint;
      wmHints->initial_state=IconicState;
      XSetWMHints(LLW_display,winData->win,wmHints);
      XFree(wmHints);
      XMapWindow(LLW_display,winData->win);
      }
    }
  }
}

static
void
LLW_updateWindowBackground(LLW::WinID window)
{
LLW_WinData_X11 * winData=(LLW_WinData_X11 *)window;
if(winData->backPixmap)
  {
  LLW_PixData_X11 * pixData=(LLW_PixData_X11 *)winData->backPixmap;
  XSetWindowBackgroundPixmap(LLW_display,winData->win,pixData->pix);
  }
else
  {
  XSetWindowBackground(LLW_display,winData->win,winData->backColor);
  }
}

//-------- Cursor management -------------------------------------------------

void
LLW::setWindowCursor(LLW::WinID window,
                     LLW::CursorID cursor) // Static
{
LLW_WinData_X11 * winData=(LLW_WinData_X11 *)window;
if(winData->cursor!=cursor)
  {
  winData->cursor=cursor;
  XDefineCursor(LLW_display,winData->win,LLW_cursor[cursor]);
  }
}

void
LLW::moveCursor(int dx,
                int dy) // Static
{
if(dx||dy)
  {
  XWarpPointer(LLW_display,None,None,0,0,0,0,dx,dy);
  }
}

//-------- Geometry ----------------------------------------------------------

static
void
LLW_updateWindowGeometry(LLW::WinID window)
{
LLW_WinData_X11 * winData=(LLW_WinData_X11 *)window;
XMoveResizeWindow(LLW_display,winData->win,
                  winData->x,winData->y,
                  winData->width,winData->height);
}

void
LLW::getScreenSize(unsigned int & widthOut,
                   unsigned int & heightOut) // Static
{
int screen=DefaultScreen(LLW_display);
widthOut=DisplayWidth(LLW_display,screen);
heightOut=DisplayHeight(LLW_display,screen);
}

//-------- Color management --------------------------------------------------

unsigned int // color pixel value
LLW::makeDoubleColor(double red,
                     double green,
                     double blue) // Static
{
return((((unsigned int)(red*LLW_redFactor))<<LLW_redShift)|
       (((unsigned int)(green*LLW_greenFactor))<<LLW_greenShift)|
       (((unsigned int)(blue*LLW_blueFactor))<<LLW_blueShift));
}

unsigned int // color pixel value
LLW::makeByteColor(unsigned char red,
                   unsigned char green,
                   unsigned char blue) // Static
{
return(((((unsigned int)red)>>LLW_redCut)<<LLW_redShift)|
       ((((unsigned int)green)>>LLW_greenCut)<<LLW_greenShift)|
       ((((unsigned int)blue)>>LLW_blueCut)<<LLW_blueShift));
}

//-------- Font management ---------------------------------------------------

inline static
StlString
LLW_makeXFontName(const StlString & name,
                  int size,
                  bool bold,
                  char slant)
{
StlString result="-*-";
result+=name;
result+=(bold ? "-bold-" : "-medium-");
switch(slant)
  {
  case 'i': result+="i-*-*-"; break;
  case 'o': result+="o-*-*-"; break;
  default:  result+="r-*-*-"; break;
  }
result+=(size>0 ? intToStr(size) : StlString("*"));
result+="-*-*-*-*-*-iso8859-1";
return(result);
}

LLW::FontID // loaded font
LLW::loadFont(const StlString & fontName) // Static
{
LLW_FontData * fontData=LLW_getFontData(fontName);
if(fontData)
  {
  ++(fontData->count);
  return((LLW::FontID)fontData);
  }
XFontStruct * fs=XLoadQueryFont(LLW_display,fontName.c_str());
if(!fs)
  {
  StlString name;
  unsigned int size;
  bool bold;
  char slant;
  LLW_parseFontName(fontName,name,size,bold,slant);
  StlString substName; // perform well known substitutions
  if(name=="courier new") substName="courier";
  else if((name=="arial")||(name=="variable")) substName="helvetica";
  else if(name=="times new roman") substName="times";
  else if(name=="century schoolbook") substName="new century schoolbook";
  else if(name=="lucida sans unicode") substName="lucida sans";
  else if(name=="lucida console") substName="lucidatypewriter";
  while(!fs)
    {
    StlString xFont=LLW_makeXFontName(name,size,bold,slant);
    fs=XLoadQueryFont(LLW_display,xFont.c_str());
    if(!fs&&bold) // try without 'bold'
      {
      xFont=LLW_makeXFontName(name,size,false,slant);
      fs=XLoadQueryFont(LLW_display,xFont.c_str());
      }
    if(!fs&&(slant!='r')) // try without 'slant'
      {
      xFont=LLW_makeXFontName(name,size,bold,'r');
      fs=XLoadQueryFont(LLW_display,xFont.c_str());
      }
    if(!fs&&bold&&(slant!='r')) // try without 'bold' and 'slant'
      {
      xFont=LLW_makeXFontName(name,size,false,'r');
      fs=XLoadQueryFont(LLW_display,xFont.c_str());
      }
    if(!fs&&substName.size()) // try another font name
      {
      xFont=LLW_makeXFontName(substName,size,bold,slant);
      fs=XLoadQueryFont(LLW_display,xFont.c_str());
      if(!fs&&bold) // try without 'bold'
        {
        xFont=LLW_makeXFontName(substName,size,false,slant);
        fs=XLoadQueryFont(LLW_display,xFont.c_str());
        }
      if(!fs&&(slant!='r')) // try without 'slant'
        {
        xFont=LLW_makeXFontName(substName,size,bold,'r');
        fs=XLoadQueryFont(LLW_display,xFont.c_str());
        }
      if(!fs&&bold&&(slant!='r')) // try without 'bold' and 'slant'
        {
        xFont=LLW_makeXFontName(substName,size,false,'r');
        fs=XLoadQueryFont(LLW_display,xFont.c_str());
        }
      }
    if(!(size--)) break; // try a smaller size if not 0
    }
  }
if(!fs)
  {
  return((LLW::FontID)0);
  }
fontData=new LLW_FontData;
fontData->fontName=fontName;
fontData->font=fs;
fontData->count=1;
LLW_fontDataMap->insert(LLW_FontDataMap::value_type(fontData->fontName,
                                                    fontData));
return((LLW::FontID)fontData);
}

void
LLW::freeFont(LLW::FontID font) // Static
{
LLW_FontData * fontData=(LLW_FontData *)font;
if(!(--(fontData->count)))
  {
  XFreeFont(LLW_display,fontData->font);
  LLW_fontDataMap->erase(fontData->fontName);
  delete fontData;
  }
}

void
LLW::getFontHeight(LLW::FontID font,
                   unsigned int & ascentOut,
                   unsigned int & descentOut) // Static
{
LLW_FontData * fontData=(LLW_FontData *)font;
ascentOut=fontData->font->ascent;
descentOut=fontData->font->descent;
}

unsigned int // text width
LLW::getTextWidth(LLW::FontID font,
                  const StlString & text) // Static
{
LLW_FontData * fontData=(LLW_FontData *)font;
return(XTextWidth(fontData->font,text.c_str(),text.size()));
}

void
LLW::drawText(LLW::DrawID drawable,
              LLW::FontID font,
              unsigned int color,
              int x,
              int y,
              const StlString & text) // Static
{
Drawable draw=(((LLW_DrawData *)drawable)->isWindow()
               ? ((LLW_WinData_X11 *)drawable)->win
               : ((LLW_PixData_X11 *)drawable)->pix);
XGCValues values;
values.font=((LLW_FontData *)font)->font->fid;
values.foreground=color;
XChangeGC(LLW_display,LLW_gc,GCFont|GCForeground,&values);
XDrawString(LLW_display,draw,LLW_gc,x,y,text.c_str(),text.size());
}

//-------- Drawing -----------------------------------------------------------

void
LLW::drawPoint(LLW::DrawID drawable,
               unsigned int color,
               int x,
               int y) // Static
{
Drawable draw=(((LLW_DrawData *)drawable)->isWindow()
               ? ((LLW_WinData_X11 *)drawable)->win
               : ((LLW_PixData_X11 *)drawable)->pix);
XSetForeground(LLW_display,LLW_gc,color);
XDrawPoint(LLW_display,draw,LLW_gc,x,y);
}

void
LLW::drawLine(LLW::DrawID drawable,
              unsigned int color,
              int x1,
              int y1,
              int x2,
              int y2) // Static
{
Drawable draw=(((LLW_DrawData *)drawable)->isWindow()
               ? ((LLW_WinData_X11 *)drawable)->win
               : ((LLW_PixData_X11 *)drawable)->pix);
XSetForeground(LLW_display,LLW_gc,color);
XDrawLine(LLW_display,draw,LLW_gc,x1,y1,x2,y2);
}

void
LLW::drawRectangle(LLW::DrawID drawable,
                   unsigned int color,
                   int x,
                   int y,
                   unsigned int width,
                   unsigned int height) // Static
{
Drawable draw=(((LLW_DrawData *)drawable)->isWindow()
               ? ((LLW_WinData_X11 *)drawable)->win
               : ((LLW_PixData_X11 *)drawable)->pix);
XSetForeground(LLW_display,LLW_gc,color);
XDrawRectangle(LLW_display,draw,LLW_gc,x,y,width-1,height-1);
}

void
LLW::fillRectangle(LLW::DrawID drawable,
                   unsigned int color,
                   int x,
                   int y,
                   unsigned int width,
                   unsigned int height) // Static
{
Drawable draw=(((LLW_DrawData *)drawable)->isWindow()
               ? ((LLW_WinData_X11 *)drawable)->win
               : ((LLW_PixData_X11 *)drawable)->pix);
XSetForeground(LLW_display,LLW_gc,color);
XFillRectangle(LLW_display,draw,LLW_gc,x,y,width,height);
}

void
LLW::drawEllipse(LLW::DrawID drawable,
                 unsigned int color,
                 int x,
                 int y,
                 unsigned int width,
                 unsigned int height) // Static
{
Drawable draw=(((LLW_DrawData *)drawable)->isWindow()
               ? ((LLW_WinData_X11 *)drawable)->win
               : ((LLW_PixData_X11 *)drawable)->pix);
XSetForeground(LLW_display,LLW_gc,color);
XDrawArc(LLW_display,draw,LLW_gc,x,y,width,height,0,23040);
}

void
LLW::fillEllipse(LLW::DrawID drawable,
                 unsigned int color,
                 int x,
                 int y,
                 unsigned int width,
                 unsigned int height) // Static
{
Drawable draw=(((LLW_DrawData *)drawable)->isWindow()
               ? ((LLW_WinData_X11 *)drawable)->win
               : ((LLW_PixData_X11 *)drawable)->pix);
XSetForeground(LLW_display,LLW_gc,color);
XFillArc(LLW_display,draw,LLW_gc,x,y,width,height,0,23040);
}

void
LLW::drawArc(LLW::DrawID drawable,
             unsigned int color,
             int x,
             int y,
             unsigned int width,
             unsigned int height,
             double startAngle,
             double angle) // Static
{
Drawable draw=(((LLW_DrawData *)drawable)->isWindow()
               ? ((LLW_WinData_X11 *)drawable)->win
               : ((LLW_PixData_X11 *)drawable)->pix);
XSetForeground(LLW_display,LLW_gc,color);
XDrawArc(LLW_display,draw,LLW_gc,x,y,width,height,
         (int)(startAngle*M_180_PI*64.0),
         (int)(angle*M_180_PI*64.0));
}

void
LLW::fillArc(LLW::DrawID drawable,
             unsigned int color,
             int x,
             int y,
             unsigned int width,
             unsigned int height,
             double startAngle,
             double angle) // Static
{
Drawable draw=(((LLW_DrawData *)drawable)->isWindow()
               ? ((LLW_WinData_X11 *)drawable)->win
               : ((LLW_PixData_X11 *)drawable)->pix);
XSetForeground(LLW_display,LLW_gc,color);
XFillArc(LLW_display,draw,LLW_gc,x,y,width,height,
         (int)(startAngle*M_180_PI*64.0),
         (int)(angle*M_180_PI*64.0));
}

void
LLW::copyDrawablePart(LLW::DrawID dest,
                      int destX,
                      int destY,
                      LLW::DrawID source,
                      int sourceX,
                      int sourceY,
                      unsigned int width,
                      unsigned int height) // Static
{
Drawable drawTo=(((LLW_DrawData *)dest)->isWindow()
                 ? ((LLW_WinData_X11 *)dest)->win
                 : ((LLW_PixData_X11 *)dest)->pix);
Drawable drawFrom=(((LLW_DrawData *)source)->isWindow()
                   ? ((LLW_WinData_X11 *)source)->win
                   : ((LLW_PixData_X11 *)source)->pix);
XCopyArea(LLW_display,drawFrom,drawTo,LLW_gc,
          sourceX,sourceY,width,height,destX,destY);
}

void
LLW::clearWindow(LLW::WinID window) // Static
{
LLW_WinData_X11 * winData=(LLW_WinData_X11 *)window;
XClearWindow(LLW_display,winData->win);
}

//-------- OpenGL management -------------------------------------------------

static
void
LLW_setCurrentGLWindow(void)
{
LLW_WinData_X11 * winData=(LLW_WinData_X11 *)LLW_currentGLTarget;
glXMakeCurrent(LLW_display,winData->win,LLW_commonWinCtx);
}

static
void
LLW_setCurrentGLBuffer(void)
{
LLW_GLBufData_X11 * glBufData=(LLW_GLBufData_X11 *)LLW_currentGLTarget;
glXMakeCurrent(LLW_display,glBufData->pbuffer,LLW_commonPBufCtx);
}

static
void
LLW_swapGLDoubleBuffers(void)
{
glXSwapBuffers(LLW_display,
               ((LLW_WinData_X11 *)LLW_currentGLTarget)->win);
}

//-------- Event management --------------------------------------------------

static
void
LLW_updateWindowEventMask(LLW::WinID window)
{
LLW_WinData_X11 * winData=(LLW_WinData_X11 *)window;
long evtMask=StructureNotifyMask               // always watch geometry
            |EnterWindowMask|LeaveWindowMask;  // and enter/leave
if(winData->mask&LLW::EVT_EXPOSE)
  {
  evtMask|=ExposureMask;
  }
if(winData->mask&LLW::EVT_MOUSE_BUTTON)
  {
  evtMask|=ButtonPressMask|ButtonReleaseMask;
  }
if(winData->mask&LLW::EVT_MOUSE_BUTTON_UNGRAB)
  {
  evtMask|=OwnerGrabButtonMask;
  }
if(winData->mask&LLW::EVT_MOUSE_MOTION)
  {
  evtMask|=PointerMotionMask;
  }
if(winData->mask&LLW::EVT_KEYBOARD)
  {
  evtMask|=KeyPressMask|KeyReleaseMask;
  }
if(winData->mask&LLW::EVT_FOCUS)
  {
  evtMask|=FocusChangeMask;
  }
XSelectInput(LLW_display,winData->win,evtMask);
if(winData->mask&LLW::EVT_CLOSE)
  {
  Atom protocol;
  protocol=XInternAtom(LLW_display,"WM_DELETE_WINDOW",False);
  XSetWMProtocols(LLW_display,winData->win,&protocol,1);
  }
}

static
void
LLW_updateMouseAndModifiers(LLW::Event * evt)
{
Window win=(((LLW_AreaData *)evt->area)->isWindow()
            ? ((LLW_WinData_X11 *)evt->area)->win
            : DefaultRootWindow(LLW_display));
Window r,c;
unsigned int mask;
int mod=LLW::MDF_NONE;
if(XQueryPointer(LLW_display,win,&r,&c,
                 &evt->xScreen,&evt->yScreen,
                 &evt->x,&evt->y,&mask))
  {
  if(mask&ControlMask) mod|=LLW::MDF_CONTROL;
  if(mask&ShiftMask)   mod|=LLW::MDF_SHIFT;
  if(mask&LockMask)    mod|=LLW::MDF_CAPSLOCK;
  if(mask&Mod2Mask)    mod|=LLW::MDF_NUMLOCK;
  if(mask&Button1Mask) mod|=LLW::MDF_LEFT;
  if(mask&Button2Mask) mod|=LLW::MDF_MIDDLE;
  if(mask&Button3Mask) mod|=LLW::MDF_RIGHT;
  }
evt->modifiers=(LLW::Modifier)mod;
}

void
LLW_printEvt(int type)
{
switch(type)
  {
  case KeyPress: cerr << "KeyPress" << endl; break;
  case KeyRelease: cerr << "KeyRelease" << endl; break;
  case ButtonPress: cerr << "ButtonPress" << endl; break;
  case ButtonRelease: cerr << "ButtonRelease" << endl; break;
  case MotionNotify: cerr << "MotionNotify" << endl; break;
  case EnterNotify: cerr << "EnterNotify" << endl; break;
  case LeaveNotify: cerr << "LeaveNotify" << endl; break;
  case FocusIn: cerr << "FocusIn" << endl; break;
  case FocusOut: cerr << "FocusOut" << endl; break;
  case KeymapNotify: cerr << "KeymapNotify" << endl; break;
  case Expose: cerr << "Expose" << endl; break;
  case GraphicsExpose: cerr << "GraphicsExpose" << endl; break;
  case NoExpose: cerr << "NoExpose" << endl; break;
  case VisibilityNotify: cerr << "VisibilityNotify" << endl; break;
  case CreateNotify: cerr << "CreateNotify" << endl; break;
  case DestroyNotify: cerr << "DestroyNotify" << endl; break;
  case UnmapNotify: cerr << "UnmapNotify" << endl; break;
  case MapNotify: cerr << "MapNotify" << endl; break;
  case MapRequest: cerr << "MapRequest" << endl; break;
  case ReparentNotify: cerr << "ReparentNotify" << endl; break;
  case ConfigureNotify: cerr << "ConfigureNotify" << endl; break;
  case ConfigureRequest: cerr << "ConfigureRequest" << endl; break;
  case GravityNotify: cerr << "GravityNotify" << endl; break;
  case ResizeRequest: cerr << "ResizeRequest" << endl; break;
  case CirculateNotify: cerr << "CirculateNotify" << endl; break;
  case CirculateRequest: cerr << "CirculateRequest" << endl; break;
  case PropertyNotify: cerr << "PropertyNotify" << endl; break;
  case SelectionClear: cerr << "SelectionClear" << endl; break;
  case SelectionRequest: cerr << "SelectionRequest" << endl; break;
  case SelectionNotify: cerr << "SelectionNotify" << endl; break;
  case ColormapNotify: cerr << "ColormapNotify" << endl; break;
  case ClientMessage: cerr << "ClientMessage" << endl; break;
  case MappingNotify: cerr << "MappingNotify" << endl; break;
  }
}

static
void
LLW_translateEvent(XEvent * evt)
{
LLW_WinData_X11 * winData=LLW_getWinData(evt->xany.window);
if(!winData) return;
switch(evt->type)
  {
  case MotionNotify:
    {
    while(XCheckTypedWindowEvent(LLW_display,winData->win,
                                 MotionNotify,evt));
    LLW_postEvent(winData,
                  LLW::EVT_MOUSE_MOTION,
                  LLW::BTN_NONE,
                  (const char *)0,
                  false,
                  ArSystem::getTime());
    break;
    }
  case EnterNotify:
    {
    if((evt->xcrossing.mode!=NotifyGrab)&&
       (evt->xcrossing.detail!=NotifyVirtual)&& // avoid virtual events
       (evt->xcrossing.detail!=NotifyNonlinearVirtual)&&
       (LLW_mouseWindow!=winData))
      {
      if(LLW_mouseWindow)
        {
        LLW_postEvent(LLW_mouseWindow,
                      LLW::EVT_MOUSE_INOUT,
                      LLW::BTN_NONE,
                      (const char *)0,
                      false,
                      ArSystem::getTime());
        }
      LLW_mouseWindow=winData;
      LLW_postEvent(winData,
                    LLW::EVT_MOUSE_INOUT,
                    LLW::BTN_NONE,
                    (const char *)0,
                    true,
                    ArSystem::getTime());
      }
    break;
    }
  case LeaveNotify:
    {
    if((evt->xcrossing.mode!=NotifyUngrab)&&
       (evt->xcrossing.detail!=NotifyVirtual)&& // avoid virtual events
       (evt->xcrossing.detail!=NotifyNonlinearVirtual)&&
       (LLW_mouseWindow==winData))
      {
      LLW_postEvent(LLW_mouseWindow,
                    LLW::EVT_MOUSE_INOUT,
                    LLW::BTN_NONE,
                    (const char *)0,
                    false,
                    ArSystem::getTime());
      LLW_mouseWindow=(LLW_WinData *)0;
      }
    break;
    }
  case FocusIn:
    {
    if(!winData->parent&&!winData->extWin) // only for top windows
      {
      LLW_postEvent(winData,
                    LLW::EVT_FOCUS,
                    LLW::BTN_NONE,
                    (const char *)0,
                    true,
                    ArSystem::getTime());
      }
    break;
    }
  case FocusOut:
    {
    if(!winData->parent&&!winData->extWin) // only for top windows
      {
      LLW_postEvent(winData,
                    LLW::EVT_FOCUS,
                    LLW::BTN_NONE,
                    (const char *)0,
                    false,
                    ArSystem::getTime());
      }
    break;
    }
  case Expose:
    {
    while(XCheckTypedWindowEvent(LLW_display,winData->win,
                                 Expose,evt));
    LLW_postEvent(winData,
                  LLW::EVT_EXPOSE,
                  LLW::BTN_NONE,
                  (const char *)0,
                  false,
                  ArSystem::getTime());
    break;
    }
  case ButtonPress:
  case ButtonRelease:
    {
    LLW::Button button=LLW::BTN_NONE;
    switch(evt->xbutton.button)
      {
      case Button1:
        {
        button=LLW::BTN_LEFT;
        break;
        }
      case Button2:
        {
        button=LLW::BTN_MIDDLE;
        break;
        }
      case Button3:
        {
        button=LLW::BTN_RIGHT;
        break;
        }
      case Button4:
        {
        if(evt->type==ButtonPress)
          {
          button=LLW::BTN_FORWARD;
          }
        break;
        }
      case Button5:
        {
        if(evt->type==ButtonPress)
          {
          button=LLW::BTN_BACKWARD;
          }
        break;
        }
      }
    if(button!=LLW::BTN_NONE)
      {
      LLW_postEvent(winData,
                    LLW::EVT_MOUSE_BUTTON,
                    button,
                    (const char *)0,
                    (evt->type==ButtonPress ? true : false),
                    ArSystem::getTime());
      }
    break;
    }
  case KeyPress:
  case KeyRelease:
    {
    static XComposeStatus compose={ (XPointer)0, 0 };
    if(!winData->keyRepeat&&(evt->type==KeyRelease)&&XPending(LLW_display))
      {
      XEvent nextEvent;
      XPeekEvent(LLW_display,&nextEvent);
      if((nextEvent.type==KeyPress)&&
         (nextEvent.xkey.window==evt->xkey.window)&&
         (nextEvent.xkey.keycode==evt->xkey.keycode)&&
         (nextEvent.xkey.time==evt->xkey.time))
        {
        XNextEvent(LLW_display,&nextEvent);
        break; // ignore the KeyRelease/KeyPress pair
        }
      }
    char buffer[0x20];
    const_cast<XEvent *>(evt)->xkey.state&=~ControlMask; // ignore [Control]
    KeySym keysym;
    int nb=XLookupString(&evt->xkey,buffer,0x20,
                         &keysym,&compose);
    if((keysym==XK_Delete)&&    // Sometimes BackSpace is converted in Delete
       (evt->xkey.keycode==22)) // keycode for BackSpace
      {
      keysym=XK_BackSpace;
      buffer[0]='\b';
      nb=1;
      }
    const char * key=(const char *)0;
    switch(keysym)
      {
      case XK_Up:
      case XK_KP_Up:     key="Up";       break;
      case XK_Down:
      case XK_KP_Down:   key="Down";     break;
      case XK_Left:
      case XK_KP_Left:   key="Left";     break;
      case XK_Right:
      case XK_KP_Right:  key="Right";    break;
      case XK_Prior:
      case XK_KP_Prior:  key="Prior";    break;
      case XK_Next:
      case XK_KP_Next:   key="Next";     break;
      case XK_Home:
      case XK_KP_Home:   key="Home";     break;
      case XK_End:
      case XK_KP_End:    key="End";      break;
      case XK_Begin:
      case XK_KP_Begin:  key="Begin";    break;
      case XK_Insert:
      case XK_KP_Insert: key="Insert";   break;
      case XK_Delete:
      case XK_KP_Delete: key="Delete";   break;
      case XK_Shift_L:
      case XK_Shift_R:   key="Shift";    break;
      case XK_Control_L:
      case XK_Control_R: key="Control";  break;
      case XK_Caps_Lock: key="CapsLock"; break;
      case XK_Num_Lock:  key="NumLock";  break;
      case XK_F1:        key="F1";       break;
      case XK_F2:        key="F2";       break;
      case XK_F3:        key="F3";       break;
      case XK_F4:        key="F4";       break;
      case XK_F5:        key="F5";       break;
      case XK_F6:        key="F6";       break;
      case XK_F7:        key="F7";       break;
      case XK_F8:        key="F8";       break;
      case XK_F9:        key="F9";       break;
      case XK_F10:       key="F10";      break;
      case XK_F11:       key="F11";      break;
      case XK_F12:       key="F12";      break;
      }
    if(!key&&(nb==1))
      {
      if(buffer[0]=='\r') buffer[0]='\n';
      buffer[1]='\0';
      key=buffer;
      }
    if(key)
      {
      if(LLW_mouseWindow&&
         (LLW::getTopWindow(LLW_mouseWindow)==LLW::getTopWindow(winData)))
        {
        LLW_postEvent(LLW_mouseWindow,
                      LLW::EVT_KEYBOARD,
                      LLW::BTN_NONE,
                      key,
                      (evt->type==KeyPress ? true : false),
                      ArSystem::getTime());
        }
      }
    break;
    }
  case ConfigureNotify:
    {
    while(XCheckTypedWindowEvent(LLW_display,winData->win,
                                 ConfigureNotify,evt));
    if(winData->parent||winData->extWin)
      {
      winData->x=evt->xconfigure.x;
      winData->y=evt->xconfigure.y;
      }
    else
      {
      Window r;
      XTranslateCoordinates(LLW_display,winData->win,
                            DefaultRootWindow(LLW_display),
                            0,0,&winData->x,&winData->y,&r);
      }
    winData->width=evt->xconfigure.width;
    winData->height=evt->xconfigure.height;
    bool update=false;
    if(winData->width<winData->minWidth)
      {
      winData->width=winData->minWidth;
      update=true;
      }
    if(winData->height<winData->minHeight)
      {
      winData->height=winData->minHeight;
      update=true;
      }
    if(update)
      {
      LLW_updateWindowGeometry((LLW::WinID)winData);
      }
    else
      {
      LLW_postEvent(winData,
                    LLW::EVT_GEOMETRY,
                    LLW::BTN_NONE,
                    (const char *)0,
                    false,
                    ArSystem::getTime());
      }
    break;
    }
  case MapNotify:
    {
    winData->mini=false;
    winData->mapped=true;
    LLW_postEvent(winData,
                  LLW::EVT_GEOMETRY,
                  LLW::BTN_NONE,
                  (const char *)0,
                  false,
                  ArSystem::getTime());
    break;
    }
  case UnmapNotify:
    {
    if(winData->mapped) // `mapped' flag not cleared by our own code
      {                 // --> iconification from the Window Manager
      winData->mini=true;
      }
    winData->mapped=false;
    LLW_postEvent(winData,
                  LLW::EVT_GEOMETRY,
                  LLW::BTN_NONE,
                  (const char *)0,
                  false,
                  ArSystem::getTime());
    break;
    }
  case ClientMessage:
    {
    if(winData->mask&LLW::EVT_CLOSE)
      {
      char * atomName=XGetAtomName(LLW_display,evt->xclient.data.l[0]);
      if(!strcmp(atomName,"WM_DELETE_WINDOW"))
        {
        LLW_postEvent(winData,
                      LLW::EVT_CLOSE,
                      LLW::BTN_NONE,
                      (const char *)0,
                      false,
                      ArSystem::getTime());
        }
      XFree(atomName);
      }
    break;
    }
  default:
    // LLW_printEvt(evt->type);
    break;
  }
}

static
void
LLW_extractEvents(bool blocking)
{
XEvent evt;
while(XPending(LLW_display))
  {
  XNextEvent(LLW_display,&evt);
  LLW_translateEvent(&evt);
  blocking=false;
  }
if(blocking)
  {
  XNextEvent(LLW_display,&evt);
  LLW_translateEvent(&evt);
  }
}

void
LLW::flush(void) // Static
{
glXWaitGL();
XSync(LLW_display,False);
}

//-------- Beep --------------------------------------------------------------

void
LLW::beep(void) // Static
{
XBell(LLW_display,0);
}

//-------- Image management --------------------------------------------------

static
void
LLW_dumpDrawable(LLW::DrawID drawable,
                 unsigned int width,
                 unsigned int height,
                 char * dataOut)
{
Drawable draw=(((LLW_DrawData *)drawable)->isWindow()
               ? ((LLW_WinData_X11 *)drawable)->win
               : ((LLW_PixData_X11 *)drawable)->pix);
XImage * image=XGetImage(LLW_display,draw,0,0,width,height,AllPlanes,ZPixmap);
for(unsigned int y=0;y<height;y++)
  {
  for(unsigned int x=0;x<width;x++)
    {
    unsigned int pixel=XGetPixel(image,x,height-(y+1));
    *(dataOut++)=static_cast<char>
                 (0x000000FF&((pixel>>LLW_redShift)<<LLW_redCut));
    *(dataOut++)=static_cast<char>
                 (0x000000FF&((pixel>>LLW_greenShift)<<LLW_greenCut));
    *(dataOut++)=static_cast<char>
                 (0x000000FF&((pixel>>LLW_blueShift)<<LLW_blueCut));
    }
  }
XDestroyImage(image);
}

} // namespace AReVi

//----------------------------------------------------------------------------
