// LI2/ENIB - AReVi - 3D services
//----------------------------------------------------------------------------
// file        : AReVi/Lib3D/material3D.cpp
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/Lib3D/material3D.h"

#include "AReVi/Utils/abstractStream.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class Material3D
//----------------------------------------------------------------------------

AR_CLASS_DEF(Material3D,ArObject)

//-------- Construction / Destruction ----------------------------------------

Material3D::Material3D(ArCW & arCW) // Protected
: ArObject(arCW),
  _material()
{
  // Nothing to be done
}

Material3D::~Material3D(void) // Protected
{
  // Nothing to be done
}

//-------- Material ----------------------------------------------------------

void
Material3D::setLighting(bool lighting)
{
_material.setLighting(lighting);
}

bool // lighting is on
Material3D::getLighting(void) const
{
return(_material.getLighting());
}

void
Material3D::setTransparency(double transparency)
{
_material.setTransparency(transparency<0.0 ? 0.0 :
                          (transparency>1.0 ? 1.0 : transparency));
}

double
Material3D::getTransparency(void) const
{
return(_material.getTransparency());
}

void
Material3D::setShininess(double shininess)
{
_material.setShininess(shininess<0.0 ? 0.0 :
                       (shininess>1.0 ? 1.0 : shininess));
}

double
Material3D::getShininess(void) const
{
return(_material.getShininess());
}

void
Material3D::setAmbientIntensity(double ambientIntensity)
{
_material.setAmbientIntensity(ambientIntensity<0.0 ? 0.0 :
                              (ambientIntensity>1.0 ? 1.0 : ambientIntensity));
}

double
Material3D::getAmbientIntensity(void) const
{
return(_material.getAmbientIntensity());
}

void
Material3D::setDiffuseColor(double r,
                            double g,
                            double b)
{
_material.setDiffuseColor(r<0.0 ? 0.0 : (r>1.0 ? 1.0 : r),
                          g<0.0 ? 0.0 : (g>1.0 ? 1.0 : g),
                          b<0.0 ? 0.0 : (b>1.0 ? 1.0 : b));
}

void
Material3D::getDiffuseColor(double & rOut,
                            double & gOut,
                            double & bOut) const
{
const float * diffuse=_material.getDiffuseColor();
rOut=diffuse[0];
gOut=diffuse[1];
bOut=diffuse[2];
}

void
Material3D::setEmissiveColor(double r,
                             double g,
                             double b)
{
_material.setEmissiveColor(r<0.0 ? 0.0 : (r>1.0 ? 1.0 : r),
                           g<0.0 ? 0.0 : (g>1.0 ? 1.0 : g),
                           b<0.0 ? 0.0 : (b>1.0 ? 1.0 : b));
}

void
Material3D::getEmissiveColor(double & rOut,
                             double & gOut,
                             double & bOut) const
{
const float * emissive=_material.getEmissiveColor();
rOut=emissive[0];
gOut=emissive[1];
bOut=emissive[2];
}

void
Material3D::setSpecularColor(double r,
                             double g,
                             double b)
{
_material.setSpecularColor(r<0.0 ? 0.0 : (r>1.0 ? 1.0 : r),
                           g<0.0 ? 0.0 : (g>1.0 ? 1.0 : g),
                           b<0.0 ? 0.0 : (b>1.0 ? 1.0 : b));
}

void
Material3D::getSpecularColor(double & rOut,
                             double & gOut,
                             double & bOut) const
{
const float * specular=_material.getSpecularColor();
rOut=specular[0];
gOut=specular[1];
bOut=specular[2];
}

//-------- Stream operations -------------------------------------------------

void
Material3D::writeToStream(ArRef<AbstractOStream> output) const
{
ArObject::writeToStream(output);
output->writeUInt((unsigned int)_material.getLighting());
output->writeDouble(_material.getShininess());
output->writeDouble(_material.getTransparency());
output->writeDouble(_material.getAmbientIntensity());
output->writeDouble(_material.getDiffuseColor()[0]);
output->writeDouble(_material.getDiffuseColor()[1]);
output->writeDouble(_material.getDiffuseColor()[2]);
output->writeDouble(_material.getEmissiveColor()[0]);
output->writeDouble(_material.getEmissiveColor()[1]);
output->writeDouble(_material.getEmissiveColor()[2]);
output->writeDouble(_material.getSpecularColor()[0]);
output->writeDouble(_material.getSpecularColor()[1]);
output->writeDouble(_material.getSpecularColor()[2]);
}

bool // success
Material3D::readFromStream(ArRef<AbstractIStream> input)
{
unsigned int lighting;
double shininess,transparency,ambientIntensity;
double diffuse[3];
double emissive[3];
double specular[3];
if(!ArObject::readFromStream(input)||
   !input->readUInt(lighting)||
   !input->readDouble(shininess)||
   !input->readDouble(transparency)||
   !input->readDouble(ambientIntensity)||
   !input->readDouble(diffuse[0])||
   !input->readDouble(diffuse[1])||
   !input->readDouble(diffuse[2])||
   !input->readDouble(emissive[0])||
   !input->readDouble(emissive[1])||
   !input->readDouble(emissive[2])||
   !input->readDouble(specular[0])||
   !input->readDouble(specular[1])||
   !input->readDouble(specular[2]))
  {
  return(false);
  }
_material.setLighting(lighting);
_material.setShininess(shininess);
_material.setTransparency(transparency);
_material.setAmbientIntensity(ambientIntensity);
_material.setDiffuseColor(diffuse[0],diffuse[1],diffuse[2]);
_material.setEmissiveColor(emissive[0],emissive[1],emissive[2]);
_material.setSpecularColor(specular[0],specular[1],specular[2]);
return(true);
}

} // namespace AReVi

//----------------------------------------------------------------------------
