// LI2/ENIB - AReVi - 3D services
//----------------------------------------------------------------------------
// file        : AReVi/Lib3D/renderTexture.cpp
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/Lib3D/renderTexture.h"

#include "AReVi/Utils/memoryBlock.h"
#include "AReVi/Utils/abstractStream.h"

#include "AReVi/Lib3D/textureRenderer3D.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class RenderTexture
//----------------------------------------------------------------------------

AR_CLASS_DEF(RenderTexture,Texture)

//-------- Construction / Destruction ----------------------------------------

RenderTexture::RenderTexture(ArCW & arCW) // Protected
: Texture(arCW),
  _renderer(TextureRenderer3D::nullRef()),
  _setupCB(thisPtr())
{
_texData=Texture::_emptyTexData;
}

RenderTexture::RenderTexture(ArCW & arCW,
                             ArRef<TextureRenderer3D> renderer,
                             bool useCubeMap) // Protected
: Texture(arCW),
  _renderer(renderer),
  _setupCB(thisPtr())
{
unsigned int width,height;
_renderer->getRenderSize(width,height);
if(useCubeMap)
  {
  height=height*6;
  _texData=new Texture::TexData(0, // cube map --> no framebuffer as texture
               width,height,width,height,
               LLW::getGLBufferTextureComponents(_renderer->_renderArea),
               false,true,
               MemoryBlock::nullRef());
  }
else
  {
  _texData=new Texture::TexData(LLW::getGLBufferTexture(_renderer->_renderArea),
               width,height,width,height,
               LLW::getGLBufferTextureComponents(_renderer->_renderArea),
               false,false,
               MemoryBlock::nullRef());
  }
_renderer->render();
}

RenderTexture::~RenderTexture(void) // Protected
{
  // Nothing to be done
}

//-------- Renderer management -----------------------------------------------

ArConstRef<TextureRenderer3D> // texture source
RenderTexture::getRenderer(void) const
{
return(_renderer);
}

ArRef<TextureRenderer3D> // texture source
RenderTexture::accessRenderer(void)
{
return(_renderer);
}

//-------- Texture changes ---------------------------------------------------

void
RenderTexture::setBlending(bool needsBlending)
{
_texData->needsBlending=needsBlending;
_onSetup();
}

void
RenderTexture::setAlphaTest(bool needsAlphaTest)
{
_texData->needsAlphaTest=needsAlphaTest;
_onSetup();
}

//-------- Serialization operations ------------------------------------------

void
RenderTexture::extractDependencies(SerializationDependencies & dependenciesOut,
                                   bool recursive) const
{
Texture::extractDependencies(dependenciesOut,recursive);
dependenciesOut.addReference(_renderer);
}

void
RenderTexture::serialize(SerializationDependencies & dependenciesOut,
                         ArRef<AbstractOStream> output) const
{
Texture::serialize(dependenciesOut,output);
output->writeUInt(dependenciesOut.addReference(_renderer));
unsigned int flags=(_texData->needsBlending  ? 1<<0 : 0)+
                   (_texData->needsAlphaTest ? 1<<1 : 0)+
                   (_texData->useCubeMap     ? 1<<2 : 0);
output->writeUInt(flags);
}

bool // success
RenderTexture::unserialize(const SerializationDependencies & dependencies,
                           ArRef<AbstractIStream> input)
{
unsigned int ref,flags;
if (!Texture::unserialize(dependencies,input)||
    !input->readUInt(ref) ||
    !input->readUInt(flags))
  {
  return(false);
  }
_texData=(Texture::TexData *)0; // avoid destruction of cached data
_renderer=ar_down_cast<TextureRenderer3D>(dependencies.findReference(ref));
if (!_renderer)
  {
  return(false);
  }
unsigned int width,height;
_renderer->getRenderSize(width,height);
_texData=new Texture::TexData(0,width,height,width,height,3, // FIXME: RGBA ?
                              false,flags&(1<<2),MemoryBlock::nullRef());
_renderer->render();
setBlending(flags&(1<<0));
setAlphaTest(flags&(1<<1));
_postAppearanceRebuild();
_postRedisplay();
return(true);
}

void
RenderTexture::_onSetup(void) // Protected
{
if(!_setupCB.empty())
  {
  RenderTexture::SetupEvent evt;
  _setupCB.fire(evt);
  }
}

} // namespace AReVi

//----------------------------------------------------------------------------
