// LI2/ENIB - AReVi - 3D services
//----------------------------------------------------------------------------
// file        : AReVi/Lib3D/sound3D.cpp
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/Lib3D/sound3D.h"

#include "AReVi/arSystem.h"
#include "AReVi/Utils/urlStream.h"
#include "AReVi/Utils/memoryBlock.h"
#include "AReVi/Lib3D/scene3D.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class Sound3D
//----------------------------------------------------------------------------

AR_CLASS_DEF(Sound3D,Base3D)

ArPtr<Listener3D>
Sound3D::_currentListener;

bool Sound3D::_initSuccess=false;

int (* Sound3D::_loadBufferFnct)(const char *)=
    (int (*)(const char *))0;

int (* Sound3D::_createBufferFromMemoryFnct)(const char *,
                                             unsigned int,unsigned int,
                                             UtilSound::SoundFormat)=
    (int (*)(const char *,unsigned int,unsigned int,UtilSound::SoundFormat))0;

void (* Sound3D::_freeBufferFnct)(int)=
     (void (*)(int))0;

int (* Sound3D::_createSourceFnct)(void)=
    (int (*)(void))0;

void (* Sound3D::_destroySourceFnct)(int)=
     (void (*)(int))0;

void (* Sound3D::_setSourceBufferFnct)(int,int)=
     (void (*)(int,int))0;

void (* Sound3D::_addSourceBufferFnct)(int,int)=
     (void (*)(int,int))0;

void (* Sound3D::_removeSourceBufferFnct)(int, int, StlVector<unsigned int> &)=
     (void (*)(int, int, StlVector<unsigned int> &))0;

void (* Sound3D::_getSourceBuffersStateFnct)(int, int &, int &)=
     (void (*)(int, int &, int &))0;

void (* Sound3D::_setSourceGainFnct)(int,double)=
     (void (*)(int,double))0;

void (* Sound3D::_setSourcePitchFnct)(int,double)=
     (void (*)(int,double))0;

void (* Sound3D::_setSourceRepeatFnct)(int,bool)=
     (void (*)(int,bool))0;

void (* Sound3D::_setSourceRelativeFnct)(int,bool)=
     (void (*)(int,bool))0;

void (* Sound3D::_setSourcePositionFnct)(int,float[3])=
     (void (*)(int,float[3]))0;

void (* Sound3D::_setSourceVelocityFnct)(int,float[3])=
     (void (*)(int,float[3]))0;

void (* Sound3D::_playSourcesFnct)(int *,unsigned int)=
     (void (*)(int *,unsigned int))0;

void (* Sound3D::_pauseSourcesFnct)(int *,unsigned int)=
     (void (*)(int *,unsigned int))0;

void (* Sound3D::_stopSourcesFnct)(int *,unsigned int)=
     (void (*)(int *,unsigned int))0;

int (* Sound3D::_getSourceStateFnct)(int)=
    (int (*)(int))0;

void (* Sound3D::_setListenerPositionFnct)(float[3])=
     (void (*)(float[3]))0;

void (* Sound3D::_setListenerVelocityFnct)(float[3])=
     (void (*)(float[3]))0;

void (* Sound3D::_setListenerOrientationFnct)(float[6])=
     (void (*)(float[6]))0;

void (* Sound3D::_setListenerGainFnct)(double)=
     (void (*)(double))0;

//-------- Construction / Destruction ----------------------------------------

Sound3D::Sound3D(ArCW & arCW) // Protected
: Base3D(arCW),
  _scenes(),
  _sourceID(-1),
  _soundData((StlVector<SoundData> *)NULL),
  _repeat(true),
  _gain(1.0),
  _pitch(1.0),
  _minDistance(1.0),
  _maxDistance(100.0),
  _directional(false),
  _innerAngle(M_PI_2),
  _cosInnerAngle(1.0),
  _outerAngle(M_PI),
  _cosOuterAngle(1.0),
  _outerGain(0.25),
  _stopped(true),
  _paused(false),
  _playMode(false),
  _isMute(false),
  _isAttenuated(true),
  _setupCB(thisRef()),
  _controlCB(thisRef())
{
if(Sound3D::_initSuccess)
  {
  _sourceID=(*Sound3D::_createSourceFnct)();
  }
else
  {
  setErrorMessage("Plugin not initialized");
  }
_mute(true);
setDirectionalAttenuation(_innerAngle,_outerAngle,_outerGain);
setDistanceAttenuation(_minDistance,_maxDistance);
setRepeat(_repeat);
setPitch(_pitch);
setAttenuated(_isAttenuated);
Scene3D::_globalScene->addSound(thisRef());
}

Sound3D::~Sound3D(void) // Protected
{
while(!_scenes.empty())
  {
  _scenes.back()->_removeSound(thisRef());
  }
if(Sound3D::_initSuccess)
  {
  (*Sound3D::_destroySourceFnct)(_sourceID);

  if (_soundData)
    {
    for (unsigned cpt=_soundData->size();cpt--;)
      Sound3D::_freeBuffer((*_soundData)[cpt].bufferID);

    delete _soundData;
    _soundData=(StlVector<SoundData> *)NULL;
    }

  }
}

//-------- Sound properties --------------------------------------------------

void
Sound3D::setRepeat(bool repeat)
{
_repeat=repeat;
if(Sound3D::_initSuccess)
  {
  (*Sound3D::_setSourceRepeatFnct)(_sourceID,_repeat);
  _onSetup();
  }
}

bool // sound repeats
Sound3D::getRepeat(void) const
{
return(_repeat);
}

void
Sound3D::setGain(double gain)
{
_gain=(gain<=0.0 ? 0.0 : gain);
_updateSoundLocation();
_onSetup();
}

double // gain
Sound3D::getGain(void) const
{
return(_gain);
}

void
Sound3D::setPitch(double pitch)
{
_pitch=(pitch<=0.0 ? _pitch : pitch);
if(Sound3D::_initSuccess)
  {
  (*Sound3D::_setSourcePitchFnct)(_sourceID,_pitch);
  _onSetup();
  }
}

bool // pitch
Sound3D::getPitch(void) const
{
return(_pitch);
}

unsigned int // nb pending buffers
Sound3D::getNbBuffers(void) const
{
if (_soundData)
  return _soundData->size();
else
  return 0;
}

//-------- Attenuation -------------------------------------------------------

void
Sound3D::setAttenuated(bool attenuated)
{
_isAttenuated=attenuated;
_updateSoundLocation();
_onSetup();
}

bool
Sound3D::getAttenuated(void) const
{
return _isAttenuated;
}


void
Sound3D::setDistanceAttenuation(double minDistance,
                                double maxDistance)
{
if((minDistance>=0.0)&&(maxDistance>minDistance))
  {
  _minDistance=minDistance;
  _maxDistance=maxDistance;
  _updateSoundLocation();
  _onSetup();
  }
}

void
Sound3D::getDistanceAttenuation(double & minDistanceOut,
                                double & maxDistanceOut) const
{
minDistanceOut=_minDistance;
maxDistanceOut=_maxDistance;
}

void
Sound3D::setDirectional(bool directional)
{
_directional=directional;
_updateSoundLocation();
_onSetup();
}

bool // sound is directional
Sound3D::getDirectional(void) const
{
return(_directional);
}

void
Sound3D::setDirectionalAttenuation(double innerAngle,
                                   double outerAngle,
                                   double outerGain)
{
if((innerAngle>=0.0)&&(innerAngle<=M_2PI)&&
   (outerAngle>=innerAngle)&&(outerAngle<=M_2PI)&&
   (outerGain>=0.0)&&(outerGain<=1.0))
  {
  _innerAngle=innerAngle;
  _cosInnerAngle=cos(0.5*_innerAngle);
  _outerAngle=outerAngle;
  _cosOuterAngle=cos(0.5*_outerAngle);
  _outerGain=outerGain;
  _updateSoundLocation();
  _onSetup();
  }
}

void
Sound3D::getDirectionalAttenuation(double & innerAngleOut,
                                   double & outerAngleOut,
                                   double & outerGainOut) const
{
innerAngleOut=_innerAngle;
outerAngleOut=_outerAngle;
outerGainOut=_outerGain;
}

//-------- Playback control --------------------------------------------------

void
Sound3D::play(void)
{
if(_soundData && Sound3D::_initSuccess)
  {
  _paused=false;
  _stopped=false;
  _playMode=true;

  (*Sound3D::_playSourcesFnct)(&_sourceID,1);
  _onControl();
  }
}

void
Sound3D::stop(void)
{
if(_soundData)
  {
  _stopped=true;
  _playMode=false;
  if(Sound3D::_initSuccess)
    {
    (*Sound3D::_stopSourcesFnct)(&_sourceID,1);
    _onControl();
    }
  }
}

void
Sound3D::pause(void)
{
if(_soundData)
  {
  _queryState();
  if(!_stopped)
    {
    _paused=true;
    _playMode=false;
    if(Sound3D::_initSuccess)
      {
      (*Sound3D::_pauseSourcesFnct)(&_sourceID,1);
      _onControl();
      }
    }
  }
}

bool // sound is in playing mode
Sound3D::getPlayMode(void) const
{
  return _playMode;
}

bool // sound is stopped
Sound3D::isStopped(void) const
{
if(_soundData)
  {
  const_cast<Sound3D *>(this)->_queryState();
  }
return(_stopped);
}

bool // sound is paused
Sound3D::isPaused(void) const
{
if(_soundData)
  {
  const_cast<Sound3D *>(this)->_queryState();
  }
return(_paused);
}

void
Sound3D::playSounds(const StlVector<ArRef<Sound3D> > & sounds) // Static
{
StlVector<int> sourceIDs;
for(unsigned int i=sounds.size();i--;)
  {
  if(sounds[i]->_soundData)
    {
    sounds[i]->_paused=false;
    sounds[i]->_stopped=false;
    sounds[i]->_playMode=true;
    sourceIDs.push_back(sounds[i]->_sourceID);

    // TODO : Send a kind of sync event to synchronise sounds
    sounds[i]->_onControl();
    }
  }
if(Sound3D::_initSuccess&&!sourceIDs.empty())
  {
  (*Sound3D::_playSourcesFnct)(&sourceIDs.front(),sourceIDs.size());
  }
}

void
Sound3D::stopSounds(const StlVector<ArRef<Sound3D> > & sounds) // Static
{
StlVector<int> sourceIDs;
for(unsigned int i=sounds.size();i--;)
  {
  if(sounds[i]->_soundData)
    {
    sounds[i]->_stopped=true;
    sounds[i]->_paused=false;
    sounds[i]->_playMode=false;
    sourceIDs.push_back(sounds[i]->_sourceID);

    // TODO : Send a kind of sync event to synchronise sounds
    sounds[i]->_onControl();
    }
  }
if(Sound3D::_initSuccess&&!sourceIDs.empty())
  {
  (*Sound3D::_stopSourcesFnct)(&sourceIDs.front(),sourceIDs.size());
  }
}

void
Sound3D::pauseSounds(const StlVector<ArRef<Sound3D> > & sounds) // Static
{
StlVector<int> sourceIDs;
for(unsigned int i=sounds.size();i--;)
  {
  if(sounds[i]->_soundData)
    {
    sounds[i]->_queryState();
    if(!sounds[i]->_stopped)
      {
      sounds[i]->_paused=true;
      sounds[i]->_playMode=false;
      sourceIDs.push_back(sounds[i]->_sourceID);

      // TODO : Send a kind of sync event to synchronise sounds
      sounds[i]->_onControl();
      }
    }
  }
if(Sound3D::_initSuccess&&!sourceIDs.empty())
  {
  (*Sound3D::_pauseSourcesFnct)(&sourceIDs.front(),sourceIDs.size());
  }
}

//-------- Serialization operations ------------------------------------------

void
Sound3D::serialize(SerializationDependencies & dependenciesOut,
                   ArRef<AbstractOStream> output) const
{
Base3D::serialize(dependenciesOut,output);
const_cast<Sound3D *>(this)->_queryState();

output->writeDouble(_gain);
output->writeDouble(_pitch);
output->writeDouble(_minDistance);
output->writeDouble(_maxDistance);
output->writeDouble(_innerAngle);
output->writeDouble(_outerAngle);
output->writeDouble(_outerGain);
unsigned int flags=(_repeat      ? 1<<0 : 0)+
                   (_directional ? 1<<1 : 0)+
                   (_stopped     ? 1<<2 : 0)+
                   (_paused      ? 1<<3 : 0)+
                   (_playMode    ? 1<<4 : 0)+
                   (_isMute      ? 1<<5 : 0)+
                   (_isAttenuated? 1<<6 : 0);
output->writeUInt(flags);
}

bool // success
Sound3D::unserialize(const SerializationDependencies & dependencies,
                     ArRef<AbstractIStream> input)
{
double gain,pitch,minDistance,maxDistance,innerAngle,outerAngle,outerGain;
unsigned int flags;//,url;
if(!Base3D::unserialize(dependencies,input)||
   !input->readDouble(gain)||
   !input->readDouble(pitch)||
   !input->readDouble(minDistance)||
   !input->readDouble(maxDistance)||
   !input->readDouble(innerAngle)||
   !input->readDouble(outerAngle)||
   !input->readDouble(outerGain)||
   !input->readUInt(flags))
  {
  return(false);
  }
setDirectionalAttenuation(innerAngle,outerAngle,outerGain);
setDistanceAttenuation(minDistance,maxDistance);
setRepeat(flags&(1<<0));
setDirectional(flags&(1<<1));
setPitch(pitch);
setGain(gain);
//bool stopped=flags&(1<<2);
//bool paused=flags&(1<<3);
bool playMode=flags&(1<<4);
bool isMute=flags&(1<<5);


setAttenuated(flags&(1<<6));

if(!Sound3D::_currentListener)
  {
  isMute=true;
  }
_mute(isMute);

_playMode=playMode;
_onControl();
return(true);
}

//-------- Forbidden operations ----------------------------------------------

void
Sound3D::_onMotion(void) // Protected
{
if(!_motionMask)
  {
  _updateSoundLocation();
  Base3D::_onMotion();
  }
}

void
Sound3D::_mute(bool mute)
{
_isMute=mute;
if(_isMute)
  {
  if(Sound3D::_initSuccess)
    {
    (*Sound3D::_setSourceGainFnct)(_sourceID,0.0);
    }
  }
else
  {
  _updateSoundLocation();
  }
}

void
Sound3D::_updateSoundLocation(void) const
{
if(Sound3D::_initSuccess&&!_isMute)
  {
  const double * mtx=_base.matrix;
  float pos[3];
  if (_isAttenuated)
    {
    pos[0]=mtx[Util3D::TX];
    pos[1]=mtx[Util3D::TY];
    pos[2]=mtx[Util3D::TZ];
    }
  else
    {
    pos[0]=0.0; // linked to listener
    pos[1]=0.0;
    pos[2]=0.0;
    }
  (*Sound3D::_setSourcePositionFnct)(_sourceID,pos);
  (*Sound3D::_setSourceRelativeFnct)(_sourceID,!_isAttenuated);
  double gain=_gain;
  if(gain&&_isAttenuated)
    {
    const double * lmtx=Sound3D::_currentListener->_base.matrix;
    double dx=lmtx[Util3D::TX]-mtx[Util3D::TX];
    double dy=lmtx[Util3D::TY]-mtx[Util3D::TY];
    double dz=lmtx[Util3D::TZ]-mtx[Util3D::TZ];
    double dist=sqrt(dx*dx+dy*dy+dz*dz);
    if(dist>=_maxDistance)
      {
      gain=0.0;
      }
    else
      {
      if(dist>_minDistance)
        {
        gain*=(_maxDistance-dist)/(_maxDistance-_minDistance);
        }
      if(_directional&&dist)
        {
        double cos=(mtx[Util3D::R00]*dx+
                    mtx[Util3D::R10]*dy+
                    mtx[Util3D::R20]*dz)/dist;
        if(cos<=_cosOuterAngle)
          {
          gain*=_outerGain;
          }
        else
          {
          if(cos<_cosInnerAngle)
            {
            double a=(1.0-_outerGain)/(_cosInnerAngle-_cosOuterAngle);
            double b=1.0-a*_cosInnerAngle;
            gain*=b+a*cos;
            }
          }
        }
      }
    }
  (*Sound3D::_setSourceGainFnct)(_sourceID,gain);
  }
}

void
Sound3D::_addScene(ArRef<Scene3D> scene) // Protected
{
_scenes.push_back(scene);
if(_isMute&&
   Sound3D::_currentListener&&
   (Sound3D::_currentListener->_scene==scene))
  {
  _mute(false);
  }
}

void
Sound3D::_removeScene(ArRef<Scene3D> scene) // Protected
{
for(unsigned int i=_scenes.size();i--;)
  {
  if(_scenes[i]==scene)
    {
    if(Sound3D::_currentListener&&
       (Sound3D::_currentListener->_scene==scene))
      {
      _mute(true);
      }
    StlVectorFastErase(_scenes,i);
    break;
    }
  }
}

void
Sound3D::_queryState(void) // Protected
{
if(Sound3D::_initSuccess)
  {
  switch((*Sound3D::_getSourceStateFnct)(_sourceID))
    {
    case -1:
      _stopped=true;
      _paused=false;
      break;
    case 0:
      _stopped=false;
      _paused=true;
      break;
    case 1:
      _stopped=false;
      _paused=false;
      break;
    }
  }
}

void
Sound3D::_freeBuffer(int bufferID) // Protected, Static
{
if(Sound3D::_initSuccess)
  {
  (*Sound3D::_freeBufferFnct)(bufferID);
  }
}

bool
Sound3D::_pushBuffer(ArConstRef<MemoryBlock> buffer, // uncompressed data!
                     unsigned int freq,
                     UtilSound::SoundFormat format)
{
bool result=false;
if(Sound3D::_initSuccess&&buffer)
  {
  SoundData data;
  data.bufferID=(buffer->getSize()>0
                 ? (*Sound3D::_createBufferFromMemoryFnct)
                   (buffer->getMemory(),buffer->getSize(),freq,format)
                 : -1);
  if(data.bufferID!=-1)
    {
    data.freq=(int)freq;
    data.format=format;
    _soundData->push_back(data);


    // Push buffer to queue list.
    (*Sound3D::_addSourceBufferFnct)(_sourceID,data.bufferID);
    //if(!_stopped&&!_paused)
    if(_playMode)
      {
      _queryState();
      if (_stopped || _paused)
        {
        _paused=false;
        _stopped=false;
        (*Sound3D::_playSourcesFnct)(&_sourceID,1);
        }
      }
    result=true;
    }
  }
return(result);
}

unsigned int// nb deleted buffers
Sound3D::_freeBuffers(void)
{
if(_soundData)
  {
  int nbBuffers,nbProcessed;
  (*Sound3D::_getSourceBuffersStateFnct)(_sourceID,nbBuffers,nbProcessed);

  StlVector<unsigned int> buffers;
  (*Sound3D::_removeSourceBufferFnct)(_sourceID,nbProcessed,buffers);

  nbBuffers=(buffers.size()<_soundData->size())
             ? buffers.size() : _soundData->size();
  for (unsigned cpt=nbBuffers;cpt--;)
    {
    if ((*_soundData)[cpt].bufferID==(int)buffers[cpt])
      {
      Sound3D::_freeBuffer(buffers[cpt]);
      (*_soundData).erase((*_soundData).begin()+cpt);
      }
    else
      {
      cerr << "Sound3D::Incoherent Buffers ID!" << endl;
      }
    }

  return(nbBuffers);
  }
return 0;
}



void
Sound3D::_onSetup(void) // Protected
{
if(!_setupCB.empty())
  {
  Sound3D::SetupEvent evt;
  _setupCB.fire(evt);
  }
}

void
Sound3D::_onControl(void) // Protected
{
if(!_controlCB.empty())
  {
  Sound3D::ControlEvent evt;
  _controlCB.fire(evt);
  }
}


//----------------------------------------------------------------------------
// Class Listener3D
//----------------------------------------------------------------------------

AR_CLASS_DEF(Listener3D,Base3D)

//-------- Construction / Destruction ----------------------------------------

Listener3D::Listener3D(ArCW & arCW) // Protected
: Base3D(arCW),
  _scene(Scene3D::_globalScene),
  _gain(1.0),
  _setupCB(thisRef())
{
_scene->_addListener(thisPtr());
setGain(_gain);
}

Listener3D::~Listener3D(void) // Protected
{
if(Sound3D::_currentListener.c_ptr()==this)
  {
  for(unsigned int i=_scene->_sounds.size();i--;)
    {
    _scene->_sounds[i]->_mute(true);
    }
  }
_scene->_removeListener(thisPtr());
}

//-------- Rendering ---------------------------------------------------------

void
Listener3D::selectScene(ArRef<Scene3D> scene)
{
if(!scene)
  {
  scene=Scene3D::_globalScene;
  }
if(scene!=_scene)
  {
  if(Sound3D::_currentListener.c_ptr()==this)
    {
    for(unsigned int i=_scene->_sounds.size();i--;)
      {
      _scene->_sounds[i]->_mute(true);
      }
    }
  _scene->_removeListener(thisPtr());
  _scene=scene;
  _scene->_addListener(thisPtr());
  if(Sound3D::_currentListener.c_ptr()==this)
    {
    for(unsigned int i=_scene->_sounds.size();i--;)
      {
      _scene->_sounds[i]->_mute(false);
      }
    }
  _onSetup();
  }
}

ArConstRef<Scene3D> // eared scene
Listener3D::getScene(void) const
{
return(_scene);
}

ArRef<Scene3D> // eared scene
Listener3D::accessScene(void)
{
return(_scene);
}

void
Listener3D::setCurrent(ArRef<Listener3D> listener) // Static
{
StlVector<ArPtr<Sound3D> > * oldS=(StlVector<ArPtr<Sound3D> > *)0;
StlVector<ArPtr<Sound3D> > * newS=(StlVector<ArPtr<Sound3D> > *)0;
if(Sound3D::_currentListener)
  {
  oldS=&(Sound3D::_currentListener->_scene->_sounds);
  }
if(listener)
  {
  newS=&(listener->_scene->_sounds);
  }
if(oldS&&(oldS!=newS))
  {
  for(unsigned int i=oldS->size();i--;)
    {
    (*oldS)[i]->_mute(true);
    }
  }
if(listener!=Sound3D::_currentListener)
  {
  Sound3D::_currentListener=listener;
  if(Sound3D::_currentListener)
    {
    Sound3D::_currentListener->setGain(Sound3D::_currentListener->_gain);
    Sound3D::_currentListener->_updateListenerLocation();
    }
  }
if(newS&&(oldS!=newS))
  {
  for(unsigned int i=newS->size();i--;)
    {
    (*newS)[i]->_mute(false);
    }
  }
}

ArConstRef<Listener3D> // current listener
Listener3D::getCurrent(void) // Static
{
return(Sound3D::_currentListener);
}

ArRef<Listener3D> // current listener
Listener3D::accessCurrent(void) // Static
{
return(Sound3D::_currentListener);
}

//-------- Sound properties --------------------------------------------------

void
Listener3D::setGain(double gain)
{
_gain=(gain<=0.0 ? 0.0 : gain);
if(Sound3D::_initSuccess)
  {
  if(Sound3D::_currentListener.c_ptr()==this)
    {
    (*Sound3D::_setListenerGainFnct)(_gain);
    }
  _onSetup();
  }
}

double // gain
Listener3D::getGain(void) const
{
return(_gain);
}

//-------- Serialization operations ------------------------------------------

void
Listener3D::extractDependencies(SerializationDependencies & dependenciesOut,
                                bool recursive) const
{
Base3D::extractDependencies(dependenciesOut,recursive);
dependenciesOut.addReference(_scene);
}

void
Listener3D::serialize(SerializationDependencies & dependenciesOut,
                      ArRef<AbstractOStream> output) const
{
Base3D::serialize(dependenciesOut,output);
output->writeDouble(_gain);
output->writeUInt(dependenciesOut.addReference(_scene));
}

bool // success
Listener3D::unserialize(const SerializationDependencies & dependencies,
                        ArRef<AbstractIStream> input)
{
double gain;
unsigned int scene;
if(!Base3D::unserialize(dependencies,input)||
   !input->readDouble(gain)||
   !input->readUInt(scene))
  {
  return(false);
  }
setGain(gain);
selectScene(ar_down_cast<Scene3D>(dependencies.findReference(scene)));
return(true);
}

//-------- Forbidden operations ----------------------------------------------

void
Listener3D::_onMotion(void) // Protected
{
if(!_motionMask)
  {
  if(Sound3D::_currentListener.c_ptr()==this)
    {
    _updateListenerLocation();
    }
  Base3D::_onMotion();
  }
}

void
Listener3D::_updateListenerLocation(void) const
{
if(Sound3D::_initSuccess)
  {
  const double * mtx=_base.matrix;
  float v[6];
  v[0]=mtx[Util3D::TX];
  v[1]=mtx[Util3D::TY];
  v[2]=mtx[Util3D::TZ];
  (*Sound3D::_setListenerPositionFnct)(v);
  v[0]=v[1]=v[2]=0.0;
  (*Sound3D::_setListenerVelocityFnct)(v);
  v[0]=mtx[Util3D::R00]; // local x direction
  v[1]=mtx[Util3D::R10];
  v[2]=mtx[Util3D::R20];
  v[3]=mtx[Util3D::R02]; // local z direction
  v[4]=mtx[Util3D::R12];
  v[5]=mtx[Util3D::R22];
  (*Sound3D::_setListenerOrientationFnct)(v);
  for(unsigned int i=_scene->_sounds.size();i--;)
    {
    _scene->_sounds[i]->_updateSoundLocation();
    }
  }
}

void
Listener3D::_onSetup(void) // Protected
{
if(!_setupCB.empty())
  {
  Listener3D::SetupEvent evt;
  _setupCB.fire(evt);
  }
}

} // namespace AReVi

//----------------------------------------------------------------------------
