// LI2/ENIB - AReVi - 3D services
//----------------------------------------------------------------------------
// file        : AReVi/Lib3D/tesselator.cpp
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/Lib3D/tesselator.h"

namespace AReVi {

#ifdef __CYGWIN__
typedef void (__attribute__ ((__stdcall__)) * GLUTessCB)();
#else // not __CYGWIN__
typedef void (*GLUTessCB)();
#endif // not __CYGWIN__

//----------------------------------------------------------------------------
// Class Tesselator
//----------------------------------------------------------------------------

AR_CLASS_DEF(Tesselator,ArObject)

//-------- Construction / Destruction ----------------------------------------

Tesselator::Tesselator(ArCW & arCW) // Protected
: ArObject(arCW),
  _fail(false),
  _tess((GLUtesselator *)0),
  _vertexCount(0),
  _currentFace(),
  _faces((StlVector<Util3D::Face> *)0)
{
  // Nothing to be done
}

Tesselator::~Tesselator(void) // Protected
{
if(_tess)
  {
  gluDeleteTess(_tess);
  }
}

//-------- Tesselation -------------------------------------------------------

bool // success
Tesselator::tesselation3D(const StlVector<Util3D::Dbl3> & vertices,
                          const StlVector<int> & contour,
                          StlVector<Util3D::Face> & trianglesOut)
{
trianglesOut.clear();
_init();
_fail=false;
_faces=&trianglesOut;
unsigned int contourSize=contour.size();
StlVector<double> dummy;
dummy.reserve(3*contourSize);
gluTessBeginPolygon(_tess,this);
gluTessBeginContour(_tess);
for(unsigned int i=0;i<contourSize;i++)
  {
  unsigned int index=contour[i];
  dummy.push_back(vertices[index].x);
  dummy.push_back(vertices[index].y);
  dummy.push_back(vertices[index].z);
  gluTessVertex(_tess,&dummy[3*i],reinterpret_cast<void *>(i));
  }
gluTessEndContour(_tess);
gluTessEndPolygon(_tess);
return(!_fail);
}

bool // success
Tesselator::tesselation2D(const StlVector<Util3D::Dbl2> & vertices,
                          const StlVector<int> & contour,
                          StlVector<Util3D::Face> & trianglesOut)
{
trianglesOut.clear();
_init();
_fail=false;
_faces=&trianglesOut;
unsigned int contourSize=contour.size();
StlVector<double> dummy;
dummy.reserve(3*contourSize);
gluTessBeginPolygon(_tess,this);
gluTessBeginContour(_tess);
for(unsigned int i=0;i<contourSize;i++)
  {
  unsigned int index=contour[i];
  dummy.push_back(vertices[index].x);
  dummy.push_back(vertices[index].y);
  dummy.push_back(0.0);
  gluTessVertex(_tess,&dummy[3*i],reinterpret_cast<void *>(i));
  }
gluTessEndContour(_tess);
gluTessEndPolygon(_tess);
return(!_fail);
}

//-------- Forbidden operations ----------------------------------------------

void
Tesselator::_init(void) // Protected
{
if(!_tess)
  {
  _tess=gluNewTess();
  gluTessCallback(_tess,(GLenum)GLU_TESS_BEGIN_DATA,
                  reinterpret_cast<GLUTessCB>(&Tesselator::_beginCB));
  gluTessCallback(_tess,(GLenum)GLU_TESS_VERTEX_DATA,
                  reinterpret_cast<GLUTessCB>(&Tesselator::_vertexCB));
  gluTessCallback(_tess,(GLenum)GLU_TESS_END_DATA,
                  reinterpret_cast<GLUTessCB>(&Tesselator::_endCB));
  gluTessCallback(_tess,(GLenum)GLU_TESS_EDGE_FLAG_DATA,
                  reinterpret_cast<GLUTessCB>(&Tesselator::_edgeFlagCB));
  gluTessCallback(_tess,(GLenum)GLU_TESS_ERROR_DATA,
                  reinterpret_cast<GLUTessCB>(&Tesselator::_errorCB));
  }
}

void
Tesselator::_edgeFlagCB(GLboolean /* flag */,
                        Tesselator * /* tesselator */) // Static, Protected
{
  // Nothing to be done
}

void
Tesselator::_errorCB(GLenum /* errorCode */,
                     Tesselator * tesselator) // Static, Protected
{
tesselator->_fail=true;
}

void
Tesselator::_beginCB(GLenum primitive,
                     Tesselator * tesselator) // Static, Protected
{
if(primitive!=GL_TRIANGLES)
  {
  tesselator->_fail=true;
  }
}

void
Tesselator::_vertexCB(unsigned int index,
                      Tesselator * tesselator) // Static, Protected
{
if(!tesselator->_fail)
  {
  switch((tesselator->_vertexCount++)%3)
    {
    case 0:
      {
      tesselator->_currentFace.p1=index;
      break;
      }
    case 1:
      {
      tesselator->_currentFace.p2=index;
      break;
      }
    case 2:
      {
      tesselator->_currentFace.p3=index;
      tesselator->_faces->push_back(tesselator->_currentFace);
      break;
      }
    }
  }
}

void
Tesselator::_endCB(Tesselator * /* tesselator */) // Static, Protected
{
  // Nothing to be done
}

} // namespace AReVi

//----------------------------------------------------------------------------
