// LI2/ENIB - AReVi - 3D services
//----------------------------------------------------------------------------
// file        : AReVi/Lib3D/textureCubeRenderer3D.cpp
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/Lib3D/textureCubeRenderer3D.h"

#include "AReVi/Lib3D/renderTexture.h"

#include "AReVi/Utils/abstractStream.h"

#include "AReVi/Lib3D/gl_priv.h"

#define TEXTURECUBE_FRONT   0
#define TEXTURECUBE_BACK    1
#define TEXTURECUBE_LEFT    3
#define TEXTURECUBE_RIGHT   2
#define TEXTURECUBE_TOP     4
#define TEXTURECUBE_BOTTOM  5

namespace AReVi {

//----------------------------------------------------------------------------
// Class TextureCubeRenderer3D
//----------------------------------------------------------------------------

AR_CLASS_DEF(TextureCubeRenderer3D,TextureRenderer3D)

//-------- Construction / Destruction ----------------------------------------

TextureCubeRenderer3D::TextureCubeRenderer3D(ArCW & arCW) // Protected
: TextureRenderer3D(arCW),
  _currentRendererFace(0),
  _swapRenderArea(0)
{
setRenderSize(32,32); // destroy and rebuild area made by TextureRenderer3D
setFieldOfView(M_PI*0.5);
}

TextureCubeRenderer3D::~TextureCubeRenderer3D(void) // Protected
{
// Nothing to be done here!
}

void
TextureCubeRenderer3D::setRenderSize(unsigned int width,
                                 unsigned int height)
{
if(_swapRenderArea)
  {
  LLW::destroyGLBuffer(_swapRenderArea);
  }

_swapRenderArea=LLW::createGLBuffer(width,height);
if(_swapRenderArea)
  {
  TextureRenderer3D::setRenderSize(width,height);
  }
else
  {
  setErrorMessage("Rendering to texture is not supported under this platform");
  }
}


//-------- Rendering ---------------------------------------------------------


void
TextureCubeRenderer3D::render(void)
{
if(_texture)
  {
  Util3D::Base oldBase(_base);
//   LLW::AreaID tmpRenderArea(_renderArea);
//   _renderArea=_swapRenderArea;

  // Render front
  _currentRendererFace=TEXTURECUBE_FRONT;
  _base.rotate(1,0,0,M_PI*0.5);
  Renderer3D::_render();

 // Render back
  _currentRendererFace=TEXTURECUBE_BACK;
  _base.rotate(1,0,0,-M_PI);
  _base.rotate(0,1,0,M_PI);
  Renderer3D::_render();

  _base=oldBase;

// Render right
  _currentRendererFace=TEXTURECUBE_RIGHT;
  _base.rotate(0,0,1,M_PI*0.5);
  Renderer3D::_render();

  // Render left
  _currentRendererFace=TEXTURECUBE_LEFT;
  _base.rotate(0,0,1,M_PI);
  _base.rotate(1,0,0,M_PI);
  Renderer3D::_render();


  _base=oldBase;

// Render top
  _currentRendererFace=TEXTURECUBE_TOP;
  _base.rotate(1,0,0,M_PI*0.5);
  _base.rotate(0,0,1,M_PI*0.5);
  Renderer3D::_render();


// Render bottom
  _currentRendererFace=TEXTURECUBE_BOTTOM;
  _base.rotate(0,0,1,M_PI);
  Renderer3D::_render();

  _base=oldBase; // Reposition to front.
//   _renderArea=tmpRenderArea;
  _texture->_postRedisplay();
  _texture->_postAppearanceRebuild();

  if(!_renderCB.empty()&&!_autoRender)
    {
    TextureCubeRenderer3D::RenderEvent event;
    _renderCB.fire(event);
    }
  }
}

ArRef<RenderTexture> // created texture
TextureCubeRenderer3D::buildTexture(void)
{
ArRef<RenderTexture> texture=_texture;
if(!texture&&_renderArea)
  {
  texture=ArRef<RenderTexture>(new RenderTexture(RenderTexture::_CLS,
                                                 thisRef(),
                                                 true));
  texture->setTransient(true);
  _texture=texture;
  }
return(texture);
}

void
TextureCubeRenderer3D::_postRender(void) // Protected
{
unsigned width,height;
LLW::getAreaSize(_renderArea,width,height);
TextureRenderer3D::_postRender();
glBindTexture(GL_TEXTURE_CUBE_MAP_EXT,_texture->_texData->texId);
glCopyTexImage2D(GL_TEXTURE_CUBE_MAP_POSITIVE_X_EXT+_currentRendererFace,
                 0,GL_RGB,0,0,width,height,0);
}

} // namespace AReVi

//----------------------------------------------------------------------------
