// LI2/ENIB - AReVi - 3D services
//----------------------------------------------------------------------------
// file        : AReVi/Lib3D/textureRenderer3D.cpp
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/Lib3D/textureRenderer3D.h"

#include "AReVi/Lib3D/renderTexture.h"

#include "AReVi/Utils/abstractStream.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class TextureRenderer3D
//----------------------------------------------------------------------------

AR_CLASS_DEF(TextureRenderer3D,Renderer3D)

//-------- Construction / Destruction ----------------------------------------

TextureRenderer3D::TextureRenderer3D(ArCW & arCW) // Protected
: Renderer3D(arCW),
  _texture(),
  _autoRender(false),
  _renderCB(thisPtr())
{
setRenderSize(32,32);
}

TextureRenderer3D::~TextureRenderer3D(void) // Protected
{
if(_texture)
  {
  _texture.destroy(); // UGLY !
  }
if(_renderArea)
  {
  LLW::destroyGLBuffer(_renderArea);
  }
}

//-------- Rendering ---------------------------------------------------------

void
TextureRenderer3D::setRenderSize(unsigned int width,
                                 unsigned int height)
{
if(_renderArea)
  {
  LLW::destroyGLBuffer(_renderArea);
  }
_renderArea=LLW::createGLBuffer(width,height);
if(_renderArea)
  {
  _afterRenderAreaSetup();
  _onRenderingSetup();
  }
else
  {
  setErrorMessage("Rendering to texture is not supported under this platform");
  }
}

void
TextureRenderer3D::setAutoRender(bool autoRender)
{
_autoRender=autoRender;
_onRenderingSetup();
if(_autoRender)
  {
  render();
  }
}

bool // texture is generated automatically
TextureRenderer3D::getAutoRender(void) const
{
return(_autoRender);
}

void
TextureRenderer3D::render(void)
{
if(_texture)
  {
  unsigned int width,height;
  LLW::getAreaSize(_renderArea,width,height);
  _texture->_texData->width=width;
  _texture->_texData->height=height;
  Renderer3D::_render();
  _texture->_postRedisplay();
  if(!_renderCB.empty()&&!_autoRender)
    {
    TextureRenderer3D::RenderEvent event;
    _renderCB.fire(event);
    }
  }
}

ArRef<RenderTexture> // created texture
TextureRenderer3D::buildTexture(void)
{
ArRef<RenderTexture> texture=_texture;
if(!texture&&_renderArea)
  {
  texture=ArRef<RenderTexture>(new RenderTexture(RenderTexture::_CLS,
                                                 thisRef(),
                                                 false));
  texture->setTransient(true);
  _texture=texture;
  }
return(texture);
}

//-------- Serialization operations ------------------------------------------

void
TextureRenderer3D::extractDependencies(SerializationDependencies &
                                       dependenciesOut,
                                       bool recursive) const
{
Renderer3D::extractDependencies(dependenciesOut,recursive);
dependenciesOut.addReference(_texture);
}

void
TextureRenderer3D::serialize(SerializationDependencies & dependenciesOut,
                             ArRef<AbstractOStream> output) const
{
Renderer3D::serialize(dependenciesOut,output);
output->writeUInt(dependenciesOut.addReference(_texture));
output->writeChar((char)_autoRender);
}

bool // success
TextureRenderer3D::unserialize(const SerializationDependencies & dependencies,
                               ArRef<AbstractIStream> input)
{
unsigned int ref;
char autoRender;
if (!Renderer3D::unserialize(dependencies,input)||
    !input->readUInt(ref)||
    !input->readChar(autoRender))
  {
  return(false);
  }
_texture=ar_down_cast<RenderTexture>(dependencies.findReference(ref));
if (!_texture)
  {
  return(false);
  }
setAutoRender(autoRender);
return(true);
}

//-------- Forbidden operations ----------------------------------------------

void
TextureRenderer3D::_render(void)
{
if(_autoRender)
  {
  render();
  }
}

} // namespace AReVi

//----------------------------------------------------------------------------
