// LI2/ENIB - AReVi - 3D services
//----------------------------------------------------------------------------
// file        : AReVi/Lib3D/transform2D.cpp
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/Lib3D/transform2D.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class Transform2D
//----------------------------------------------------------------------------

AR_CLASS_DEF(Transform2D,ArObject)

//-------- Construction / Destruction ----------------------------------------

Transform2D::Transform2D(ArCW & arCW) // Protected
: ArObject(arCW),
  _transform(true)
{
  // Nothing to be done
}

Transform2D::~Transform2D(void) // Protected
{
  // Nothing to be done
}

//-------- Transformations ---------------------------------------------------

void
Transform2D::invert(void)
{
_transform.invert();
}

void
Transform2D::apply(double & xOut,
                   double & yOut,
                   double xIn,
                   double yIn) const
{
double z=0.0;
_transform.apply(xOut,yOut,z,xIn,yIn,z);
}

void
Transform2D::identity(void)
{
_transform.identity();
}

bool // transformation has no effect
Transform2D::isIdentity(bool checkCoefs) const
{
return(_transform.isIdentity(checkCoefs));
}

void
Transform2D::preTransform(ArConstRef<Transform2D> tr)
{
_transform.preTransform(tr->_transform);
}

void
Transform2D::postTransform(ArConstRef<Transform2D> tr)
{
_transform.postTransform(tr->_transform);
}

void
Transform2D::preTranslate(double tx,
                          double ty)
{
_transform.preTranslate(tx,ty,0.0);
}

void
Transform2D::postTranslate(double tx,
                           double ty)
{
_transform.postTranslate(tx,ty,0.0);
}

void
Transform2D::preRotate(double angle)
{
_transform.preYaw(angle);
}

void
Transform2D::postRotate(double angle)
{
_transform.postYaw(angle);
}

void
Transform2D::preScale(double sx,
                      double sy)
{
_transform.preScale(sx,sy,1.0);
}

void
Transform2D::postScale(double sx,
                       double sy)
{
_transform.postScale(sx,sy,1.0);
}

} // namespace AReVi

//----------------------------------------------------------------------------
