// LI2/ENIB - AReVi - 3D services
//----------------------------------------------------------------------------
// file        : AReVi/Lib3D/transform3D.cpp
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/Lib3D/transform3D.h"

#include "AReVi/Lib3D/base3D.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class Transform3D
//----------------------------------------------------------------------------

AR_CLASS_DEF(Transform3D,ArObject)

//-------- Construction / Destruction ----------------------------------------

Transform3D::Transform3D(ArCW & arCW) // Protected
: ArObject(arCW),
  _transform(true)
{
  // Nothing to be done
}

Transform3D::~Transform3D(void) // Protected
{
  // Nothing to be done
}

//-------- Transformations ---------------------------------------------------

void
Transform3D::invert(void)
{
_transform.invert();
}

void
Transform3D::apply(double & xOut,
                   double & yOut,
                   double & zOut,
                   double xIn,
                   double yIn,
                   double zIn) const
{
_transform.apply(xOut,yOut,zOut,xIn,yIn,zIn);
}

void
Transform3D::identity(void)
{
_transform.identity();
}

bool // transformation has no effect
Transform3D::isIdentity(bool checkCoefs) const
{
return(_transform.isIdentity(checkCoefs));
}

void
Transform3D::preTransform(ArConstRef<Transform3D> tr)
{
_transform.preTransform(tr->_transform);
}

void
Transform3D::postTransform(ArConstRef<Transform3D> tr)
{
_transform.postTransform(tr->_transform);
}

void
Transform3D::preMove(ArConstRef<Base3D> base)
{
_transform.preMove(base->getBase());
}

void
Transform3D::postMove(ArConstRef<Base3D> base)
{
_transform.postMove(base->getBase());
}

void
Transform3D::preTranslate(double tx,
                          double ty,
                          double tz)
{
_transform.preTranslate(tx,ty,tz);
}

void
Transform3D::postTranslate(double tx,
                           double ty,
                           double tz)
{
_transform.postTranslate(tx,ty,tz);
}

void
Transform3D::preRotate(double x,
                       double y,
                       double z,
                       double angle)
{
_transform.preRotate(x,y,z,angle);
}

void
Transform3D::postRotate(double x,
                        double y,
                        double z,
                        double angle)
{
_transform.postRotate(x,y,z,angle);
}

void
Transform3D::preRoll(double roll)
{
_transform.preRoll(roll);
}

void
Transform3D::postRoll(double roll)
{
_transform.postRoll(roll);
}

void
Transform3D::prePitch(double pitch)
{
_transform.prePitch(pitch);
}

void
Transform3D::postPitch(double pitch)
{
_transform.postPitch(pitch);
}

void
Transform3D::preYaw(double yaw)
{
_transform.preYaw(yaw);
}

void
Transform3D::postYaw(double yaw)
{
_transform.postYaw(yaw);
}

void
Transform3D::preScale(double sx,
                      double sy,
                      double sz)
{
_transform.preScale(sx,sy,sz);
}

void
Transform3D::postScale(double sx,
                       double sy,
                       double sz)
{
_transform.postScale(sx,sy,sz);
}

} // namespace AReVi

//----------------------------------------------------------------------------
