// LI2/ENIB - AReVi - 3D services
//----------------------------------------------------------------------------
// file        : AReVi/Lib3D/urlSound3D.cpp
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/Lib3D/urlSound3D.h"

#include "AReVi/arSystem.h"
#include "AReVi/Utils/urlStream.h"
#include "AReVi/Utils/memoryBlock.h"

namespace AReVi {

//-------- Initialization ----------------------------------------------------

//static ArSystem::InitState _initState=ArSystem::AR_DOWN;

void
URLSound3D::_init(void) // Static, Protected
{
// FIXME : Init Cache from ArSystem or Plugin??

if (!Sound3D::_initSuccess && !URLSound3D::_cache)
  {
  URLSound3D::_cache=new StlMap<StlString,StlVector<Sound3D::SoundData> *>;
  }
/*
if (_initState==ArSystem::AR_DOWN && !URLSound3D::_cache)
  {
  _initState=ArSystem::AR_GOING_UP;
  URLSound3D::_cache=new StlMap<StlString,StlVector<Sound3D::SoundData> *>;
  _initState=ArSystem::AR_UP;
  }
*/
}

void
URLSound3D::_destroy(void) // Static, Protected
{
// FIXME : Destroy Cache from ArSystem or Plugin??

//if(_initState==ArSystem::AR_UP)
if (Sound3D::_initSuccess)
  {
  //_initState=ArSystem::AR_GOING_DOWN;
  while(!URLSound3D::_cache->empty())
    {
    StlVector<Sound3D::SoundData> * soundData=(*URLSound3D::_cache->begin()).second;

    for (unsigned cpt=soundData->size();cpt--;)
      Sound3D::_freeBuffer((*soundData)[cpt].bufferID);

    delete soundData;
    URLSound3D::_cache->erase(URLSound3D::_cache->begin());
    }
  delete URLSound3D::_cache;
  URLSound3D::_cache=(StlMap<StlString,StlVector<Sound3D::SoundData> *> *)0;
  //_initState=ArSystem::AR_DOWN;
  }
}


//----------------------------------------------------------------------------
// Class URLSound3D
//----------------------------------------------------------------------------

AR_CLASS_DEF(URLSound3D,Sound3D)

StlMap<StlString,StlVector<Sound3D::SoundData> *> *
URLSound3D::_cache=(StlMap<StlString,StlVector<Sound3D::SoundData> *> *)0;

//-------- Construction / Destruction ----------------------------------------

URLSound3D::URLSound3D(ArCW & arCW,
                       const StlString &url,
                       bool useCache) // Protected
: Sound3D(arCW),
  _url(""),
  _useCache(false)
{
if (!fail())
  {
  if (!_loadURL(url,useCache))
    setErrorMessage("Fail to load url=" + url);
  }
}

URLSound3D::URLSound3D(ArCW & arCW) // Protected
: Sound3D(arCW),
  _url(""),
  _useCache(false)
{
// Nothing to be done here!
}

URLSound3D::~URLSound3D(void) // Protected
{
if(_useCache)
  {
  _soundData=(StlVector<SoundData> *)0;
  }
}

//-------- Sound properties --------------------------------------------------

bool // success
URLSound3D::_loadURL(const StlString & soundURL, bool useCache)
{
bool result=false;
if(Sound3D::_initSuccess)
  {
  if(soundURL==_url)
    {
    return(true);
    }

  if(_soundData) // Free old...
    {
    _queryState();
    (*URLSound3D::_stopSourcesFnct)(&_sourceID,1);
    StlVector<unsigned int> buffers;
    (*Sound3D::_removeSourceBufferFnct)(_sourceID,getNbBuffers(),buffers);

    if (!_useCache)
      {
      for (unsigned cpt=_soundData->size();cpt--;)
        Sound3D::_freeBuffer((*_soundData)[cpt].bufferID);

      delete _soundData;
      }
    _soundData=(StlVector<SoundData> *)NULL;
    _url.erase();
    _useCache=false;
    }
  _soundData=URLSound3D::_loadBuffers(soundURL,useCache);
  if (!_soundData)
    {
      _stopped=true;
      _paused=false;
    }
  else
    {
    _useCache=useCache;
    _url=soundURL;


    for (unsigned i=0;i<_soundData->size();i++)
      {
      (*Sound3D::_addSourceBufferFnct)(_sourceID,(*_soundData)[i].bufferID);
      //(*Sound3D::_setSourceBufferFnct)(_sourceID,(*_soundData)[i].bufferID);
      }

    //if(!_stopped&&!_paused)
    if(_playMode)
      {
      (*URLSound3D::_playSourcesFnct)(&_sourceID,1);
      }
    result=true;
    }
    _onSetup();
  }
return(result);
}

const StlString & // loaded URL
URLSound3D::getURL(void) const
{
return(_url);
}


//-------- Serialization operations ------------------------------------------

void
URLSound3D::extractDependencies(SerializationDependencies & dependenciesOut,
                             bool recursive) const
{
Sound3D::extractDependencies(dependenciesOut,recursive);
dependenciesOut.addURL(_url);
}

void
URLSound3D::serialize(SerializationDependencies & dependenciesOut,
                   ArRef<AbstractOStream> output) const
{
Sound3D::serialize(dependenciesOut,output);
unsigned int flags=(_useCache      ? 1<<0 : 0);

output->writeUInt(flags);
output->writeUInt(dependenciesOut.addURL(_url));
}

bool // success
URLSound3D::unserialize(const SerializationDependencies & dependencies,
                     ArRef<AbstractIStream> input)
{
unsigned int flags,url;
if(!Sound3D::unserialize(dependencies,input)||
   !input->readUInt(flags)||
   !input->readUInt(url))
  {
  return(false);
  }

return _loadURL(dependencies.findURL(url),flags&1);
}

//-------- Forbidden operations ----------------------------------------------

StlVector<Sound3D::SoundData> * // NULL if fail
URLSound3D::_loadBuffers(const StlString & soundURL,
                        bool useCache) // Protected, Static
{
if (useCache)
  {
  StlMap<StlString,StlVector<Sound3D::SoundData> *>::iterator it=_cache->find(soundURL);
  if(it!=URLSound3D::_cache->end())
    {
    return it->second;
    }
  }

Sound3D::SoundData data;
StlVector<Sound3D::SoundData> *soundData=(StlVector<Sound3D::SoundData> *)0;

data.bufferID=-1;
data.freq=0;

ArRef<URLStream> input=URLStream::NEW(soundURL);
if(input->fail())
  {
  return(soundData);
  }
if(input->getProtocol()=="file")
  {
  StlString fileName=input->getFileName();
  input.clear();

  data.bufferID=(*URLSound3D::_loadBufferFnct)(fileName.c_str());
  }
else
  {
  StlString tmpName;
  int tmpFD=ArSystem::createTmpFile("./snd_",tmpName);
  if(tmpFD==-1)
    {
    return(soundData);
    }
  ArRef<FDOStream> output=FDOStream::NEW(tmpFD,true);
  if(!output->fail())
    {
    input->copyDataToOutput(output);
    input.clear();
    output.clear();
    data.bufferID=(*URLSound3D::_loadBufferFnct)(tmpName.c_str());
    }
  ArSystem::removeFile(tmpName);
  }
if(data.bufferID!=-1)
  {
  soundData=new StlVector<Sound3D::SoundData>();
  soundData->push_back(data);

  if (useCache)
    (*_cache)[soundURL]=soundData;

  return soundData;
  }
return(soundData);
}

} // namespace AReVi

//----------------------------------------------------------------------------
