// LI2/ENIB - AReVi - 3D services
//----------------------------------------------------------------------------
// file        : AReVi/Lib3D/util3D_priv.cpp
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/Lib3D/util3D_priv.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class Util3D (abstract)
//----------------------------------------------------------------------------

double Util3D::_idMtx[16]={ 1.0 , 0.0 , 0.0 , 0.0 ,
                            0.0 , 1.0 , 0.0 , 0.0 ,
                            0.0 , 0.0 , 1.0 , 0.0 ,
                            0.0 , 0.0 , 0.0 , 1.0 };

//-------- Bounding box collision --------------------------------------------

int // 0: no collision   1: collision   -1: possible collision
Util3D::BoundingBox::testHalfApproxCollision(
                                       const Util3D::Transform & transform,
                                       const Util3D::BoundingBox & bbox) const
{
Util3D::Dbl3 p[8];
transform.apply(p[0].x,p[0].y,p[0].z,bbox.xmin,bbox.ymin,bbox.zmin);
transform.apply(p[1].x,p[1].y,p[1].z,bbox.xmax,bbox.ymin,bbox.zmin);
transform.apply(p[2].x,p[2].y,p[2].z,bbox.xmax,bbox.ymax,bbox.zmin);
transform.apply(p[6].x,p[6].y,p[6].z,bbox.xmax,bbox.ymax,bbox.zmax);
Util3D::Dbl3 v[3];
v[0].x=p[1].x-p[0].x; v[0].y=p[1].y-p[0].y; v[0].z=p[1].z-p[0].z;
v[1].x=p[2].x-p[1].x; v[1].y=p[2].y-p[1].y; v[1].z=p[2].z-p[1].z;
v[2].x=p[6].x-p[2].x; v[2].y=p[6].y-p[2].y; v[2].z=p[6].z-p[2].z;
p[3].x=p[0].x+v[1].x; p[3].y=p[0].y+v[1].y; p[3].z=p[0].z+v[1].z;
p[4].x=p[0].x+v[2].x; p[4].y=p[0].y+v[2].y; p[4].z=p[0].z+v[2].z;
p[5].x=p[1].x+v[2].x; p[5].y=p[1].y+v[2].y; p[5].z=p[1].z+v[2].z;
p[7].x=p[6].x-v[0].x; p[7].y=p[6].y-v[0].y; p[7].z=p[6].z-v[0].z;
unsigned int outFront=8;
unsigned int outBack=8;
unsigned int outLeft=8;
unsigned int outRight=8;
unsigned int outUp=8;
unsigned int outDown=8;
for(unsigned int i=8;i--;)
  {
  const Util3D::Dbl3 & pp=p[i];
  Util3D::VertexLocation loc=locateVertex(pp.x,pp.y,pp.z);
  if(loc==Util3D::MIDDLE_LOC)
    {
    return(1); // a vertex in the box --> collision
    }
  if(loc&Util3D::XPOS_LOC)
    {
    --outFront;
    }
  else if(loc&Util3D::XNEG_LOC)
    {
    --outBack;
    }
  if(loc&Util3D::YPOS_LOC)
    {
    --outLeft;
    }
  else if(loc&Util3D::YNEG_LOC)
    {
    --outRight;
    }
  if(loc&Util3D::ZPOS_LOC)
    {
    --outUp;
    }
  else if(loc&Util3D::ZNEG_LOC)
    {
    --outDown;
    }
  }
return(outFront&&outBack&&outLeft&&outRight&&outUp&&outDown
       ? -1 // not on the same side --> possible collision
       : 0);
}

int // 0: no collision   1: collision   -1: possible collision
Util3D::BoundingBox::testApproxCollision(const Util3D::Transform & transform,
                                         const Util3D::BoundingBox & bbox)
                                                                         const
{
switch(testHalfApproxCollision(transform,bbox))
  {
  case 0:
    return(0);
  case 1:
    return(1);
  default:
    {
    Util3D::Transform invert(transform);
    invert.invert();
    return(bbox.testHalfApproxCollision(invert,*this));
    }
  }
}

static
bool // collision occured
Util3D_halfBBoxCollision(const Util3D::BoundingBox & bbox1,
                         const Util3D::Transform & transform,
                         const Util3D::BoundingBox & bbox2)
{
Util3D::Dbl3 p[8];
transform.apply(p[0].x,p[0].y,p[0].z,bbox2.xmin,bbox2.ymin,bbox2.zmin);
transform.apply(p[1].x,p[1].y,p[1].z,bbox2.xmax,bbox2.ymin,bbox2.zmin);
transform.apply(p[2].x,p[2].y,p[2].z,bbox2.xmax,bbox2.ymax,bbox2.zmin);
transform.apply(p[6].x,p[6].y,p[6].z,bbox2.xmax,bbox2.ymax,bbox2.zmax);
Util3D::Dbl3 v[3];
v[0].x=p[1].x-p[0].x; v[0].y=p[1].y-p[0].y; v[0].z=p[1].z-p[0].z;
v[1].x=p[2].x-p[1].x; v[1].y=p[2].y-p[1].y; v[1].z=p[2].z-p[1].z;
v[2].x=p[6].x-p[2].x; v[2].y=p[6].y-p[2].y; v[2].z=p[6].z-p[2].z;
p[3].x=p[0].x+v[1].x; p[3].y=p[0].y+v[1].y; p[3].z=p[0].z+v[1].z;
p[4].x=p[0].x+v[2].x; p[4].y=p[0].y+v[2].y; p[4].z=p[0].z+v[2].z;
p[5].x=p[1].x+v[2].x; p[5].y=p[1].y+v[2].y; p[5].z=p[1].z+v[2].z;
p[7].x=p[6].x-v[0].x; p[7].y=p[6].y-v[0].y; p[7].z=p[6].z-v[0].z;
unsigned int outFront=8;
unsigned int outBack=8;
unsigned int outLeft=8;
unsigned int outRight=8;
unsigned int outUp=8;
unsigned int outDown=8;
for(unsigned int i=8;i--;)
  {
  const Util3D::Dbl3 & pp=p[i];
  Util3D::VertexLocation loc=bbox1.locateVertex(pp.x,pp.y,pp.z);
  if(loc==Util3D::MIDDLE_LOC)
    {
    return(true); // a vertex in the box
    }
  if(loc&Util3D::XPOS_LOC)
    {
    --outFront;
    }
  else if(loc&Util3D::XNEG_LOC)
    {
    --outBack;
    }
  if(loc&Util3D::YPOS_LOC)
    {
    --outLeft;
    }
  else if(loc&Util3D::YNEG_LOC)
    {
    --outRight;
    }
  if(loc&Util3D::ZPOS_LOC)
    {
    --outUp;
    }
  else if(loc&Util3D::ZNEG_LOC)
    {
    --outDown;
    }
  }
if(outFront&&outBack&&
   outLeft&&outRight&&
   outUp&&outDown) // not on the same side
  {
  double x,y,z,k;
  unsigned int index[3][4]={ { 0 , 3 , 7 , 4 } ,
                             { 0 , 4 , 5 , 1 } ,
                             { 0 , 1 , 2 , 3 } };
  for(unsigned int i=3;i--;)
    {
    const Util3D::Dbl3 & vv=v[i];
    for(unsigned int j=4;j--;)
      {
      const Util3D::Dbl3 & pp=p[index[i][j]];
      if(vv.x)
        {
        k=(bbox1.xmin-pp.x)/vv.x;
        if((k>=0.0)&&(k<=1.0))
          {
          y=pp.y+k*vv.y;
          if((y>=bbox1.ymin)&&(y<=bbox1.ymax))
            {
            z=pp.z+k*vv.z;
            if((z>=bbox1.zmin)&&(z<=bbox1.zmax))
              {
              return(true); // the edge crosses the xmin face
              }
            }
          }
        k=(bbox1.xmax-pp.x)/vv.x;
        if((k>=0.0)&&(k<=1.0))
          {
          y=pp.y+k*vv.y;
          if((y>=bbox1.ymin)&&(y<=bbox1.ymax))
            {
            z=pp.z+k*vv.z;
            if((z>=bbox1.zmin)&&(z<=bbox1.zmax))
              {
              return(true); // the edge crosses the xmax face
              }
            }
          }
        }
      if(vv.y)
        {
        k=(bbox1.ymin-pp.y)/vv.y;
        if((k>=0.0)&&(k<=1.0))
          {
          x=pp.x+k*vv.x;
          if((x>=bbox1.xmin)&&(x<=bbox1.xmax))
            {
            z=pp.z+k*vv.z;
            if((z>=bbox1.zmin)&&(z<=bbox1.zmax))
              {
              return(true); // the edge crosses the ymin face
              }
            }
          }
        k=(bbox1.ymax-pp.y)/vv.y;
        if((k>=0.0)&&(k<=1.0))
          {
          x=pp.x+k*vv.x;
          if((x>=bbox1.xmin)&&(x<=bbox1.xmax))
            {
            z=pp.z+k*vv.z;
            if((z>=bbox1.zmin)&&(z<=bbox1.zmax))
              {
              return(true); // the edge crosses the ymax face
              }
            }
          }
        }
      if(vv.z)
        {
        k=(bbox1.zmin-pp.z)/vv.z;
        if((k>=0.0)&&(k<=1.0))
          {
          y=pp.y+k*vv.y;
          if((y>=bbox1.ymin)&&(y<=bbox1.ymax))
            {
            x=pp.x+k*vv.x;
            if((x>=bbox1.xmin)&&(x<=bbox1.xmax))
              {
              return(true); // the edge crosses the zmin face
              }
            }
          }
        k=(bbox1.zmax-pp.z)/vv.z;
        if((k>=0.0)&&(k<=1.0))
          {
          y=pp.y+k*vv.y;
          if((y>=bbox1.ymin)&&(y<=bbox1.ymax))
            {
            x=pp.x+k*vv.x;
            if((x>=bbox1.xmin)&&(x<=bbox1.xmax))
              {
              return(true); // the edge crosses the zmax face
              }
            }
          }
        }
      }
    }
  }
return(false);
}

bool // collision occured
Util3D::BoundingBox::testCollision(const Util3D::Transform & transform,
                                   const Util3D::BoundingBox & bbox) const
{
if(Util3D_halfBBoxCollision(*this,transform,bbox))
  {
  return(true);
  }
else
  {
  Util3D::Transform invert(transform);
  invert.invert();
  return(Util3D_halfBBoxCollision(bbox,invert,*this));
  }
}

//-------- Triangulation -----------------------------------------------------

inline static
Util3D::Dbl3 // center is x,y  radius is z
Util3D_computeCircle(const Util3D::Dbl3 & p1,
                     const Util3D::Dbl3 & p2,
                     const Util3D::Dbl3 & p3,
                     double precision)
{
double cx,cy;
if(fabs(p2.y-p1.y)<precision)
  {
  double mx2=0.5*(p2.x+p3.x);
  double my2=0.5*(p2.y+p3.y);
  cx=0.5*(p2.x+p1.x);
  cy=my2-(p3.x-p2.x)*(cx-mx2)/(p3.y-p2.y);
  }
else if(fabs(p3.y-p2.y)<precision)
  {
  double mx1=0.5*(p1.x+p2.x);
  double my1=0.5*(p1.y+p2.y);
  cx=0.5*(p3.x+p2.x);
  cy=my1-(p2.x-p1.x)*(cx-mx1)/(p2.y-p1.y);
  }
else
  {
  double m1=-(p2.x-p1.x)/(p2.y-p1.y);
  double m2=-(p3.x-p2.x)/(p3.y-p2.y);
  double mx1=0.5*(p1.x+p2.x);
  double mx2=0.5*(p2.x+p3.x);
  double my1=0.5*(p1.y+p2.y);
  double my2=0.5*(p2.y+p3.y);
  cx=(m1*mx1-m2*mx2+my2-my1)/(m1-m2);
  cy=m1*(cx-mx1)+my1;
  }
double dx=p2.x-cx;
double dy=p2.y-cy;
return(Util3D::Dbl3(cx,cy,sqrt(dx*dx+dy*dy)));
}

void
Util3D::triangulateXY(StlVector<Util3D::Dbl3> & vertexInOut,
                      StlVector<Util3D::Face> & triangleOut,
                      double precision) // Static
{
unsigned int i,j,k;
triangleOut.clear();
unsigned int nb=vertexInOut.size();
if(!nb)
  {
  return;
  }
double xmin,xmax,ymin,ymax; // compute bounding box
xmin=xmax=vertexInOut.back().x;
ymin=ymax=vertexInOut.back().y;
Util3D::Dbl3 tmp(0.0,0.0,0.0);
for(i=0;i<nb-1;i++) // remove multiple vertices and compute bounding box
  {
  unsigned int nbSame=0;
  for(j=i+1;j<nb;j++)
    {
    if((fabs(vertexInOut[j].x-vertexInOut[i].x)<precision)&&
       (fabs(vertexInOut[j].y-vertexInOut[i].y)<precision))
      {
      if(!nbSame)
        {
        tmp=vertexInOut[i];
        nbSame++;
        }
      tmp.x+=vertexInOut[j].x;
      tmp.y+=vertexInOut[j].y;
      tmp.z+=vertexInOut[j].z;
      nbSame++;
      StlVectorFastErase(vertexInOut,j);
      j--;
      nb--;
      }
    }
  if(nbSame)
    {
    vertexInOut[i].x=tmp.x/nbSame;
    vertexInOut[i].y=tmp.y/nbSame;
    vertexInOut[i].z=tmp.z/nbSame;
    }
  if(vertexInOut[i].x<xmin)
    {
    xmin=vertexInOut[i].x;
    }
  else if(vertexInOut[i].x>xmax)
    {
    xmax=vertexInOut[i].x;
    }
  if(vertexInOut[i].y<ymin)
    {
    ymin=vertexInOut[i].y;
    }
  else if(vertexInOut[i].y>ymax)
    {
    ymax=vertexInOut[i].y;
    }
  }
if(vertexInOut.back().x<xmin)
  {
  xmin=vertexInOut.back().x;
  }
else if(vertexInOut.back().x>xmax)
  {
  xmax=vertexInOut.back().x;
  }
if(vertexInOut.back().y<ymin)
  {
  ymin=vertexInOut.back().y;
  }
else if(vertexInOut.back().y>ymax)
  {
  ymax=vertexInOut.back().y;
  }
double dx=xmax-xmin;
double dy=ymax-ymin;
bool xMain=(dx>=dy);
double dmax=(xMain ? dx : dy);
if(dmax<precision)
  {
  return;
  }
double xmid=0.5*(xmax+xmin);
double ymid=0.5*(ymax+ymin);
if(xMain)
  {
  for(i=0;i<nb-1;i++) // sort along x coordinate
    {
    k=i;
    for(j=i+1;j<nb;j++)
      {
      if(vertexInOut[j].x<vertexInOut[k].x)
        {
        k=j;
        }
      }
    if(k!=i)
      {
      Util3D::Dbl3 tmp(vertexInOut[k]);
      vertexInOut[k]=vertexInOut[i];
      vertexInOut[i]=tmp;
      }
    }
  }
else
  {
  for(i=0;i<nb-1;i++) // sort along y coordinate
    {
    k=i;
    for(j=i+1;j<nb;j++)
      {
      if(vertexInOut[j].y<vertexInOut[k].y)
        {
        k=j;
        }
      }
    if(k!=i)
      {
      Util3D::Dbl3 tmp(vertexInOut[k]);
      vertexInOut[k]=vertexInOut[i];
      vertexInOut[i]=tmp;
      }
    }
  }
vertexInOut.push_back(Util3D::Dbl3(xmid-2.0*dmax, // add bounding
                                   ymid-dmax,     //   triangle vertices
                                   0.0));
vertexInOut.push_back(Util3D::Dbl3(xmid+2.0*dmax,
                                   ymid-dmax,
                                   0.0));
vertexInOut.push_back(Util3D::Dbl3(xmid,
                                   ymid+2.0*dmax,
                                   0.0));
StlVector<Util3D::Dbl3> circle;
StlVector<bool> complete;
StlVector<Util3D::Face> edge;
triangleOut.push_back(Util3D::Face(nb,nb+1,nb+2)); // add bounding triangle
complete.push_back(false);
circle.push_back(Util3D_computeCircle(vertexInOut[nb],
                                      vertexInOut[nb+1],
                                      vertexInOut[nb+2],
                                      precision));
for(i=0;i<nb;i++) // include each vertex
  {
  for(j=0;j<triangleOut.size();j++)
    {
    if(!complete[j])
      {
      if(xMain)
        {
        if(circle[j].x+circle[j].z<vertexInOut[i].x)
          {
          complete[j]=true;
          }
        }
      else
        {
        if(circle[j].y+circle[j].z<vertexInOut[i].y)
          {
          complete[j]=true;
          }
        }
      double dx=vertexInOut[i].x-circle[j].x;
      double dy=vertexInOut[i].y-circle[j].y;
      if(dx*dx+dy*dy<=circle[j].z*circle[j].z)
        {
        edge.push_back(Util3D::Face(triangleOut[j].p1, // add edges
                                    triangleOut[j].p2,
                                    1));
        edge.push_back(Util3D::Face(triangleOut[j].p2,
                                    triangleOut[j].p3,
                                    1));
        edge.push_back(Util3D::Face(triangleOut[j].p3,
                                    triangleOut[j].p1,
                                    1));
        StlVectorFastErase(triangleOut,j); // remove triangle
        StlVectorFastErase(complete,j);
        StlVectorFastErase(circle,j);
        j--;
        }
      }
    }
  for(j=0;j<edge.size()-1;j++) // remove multiple edge
    {
    for(k=j+1;k<edge.size();k++)
      {
      if((edge[j].p1==edge[k].p2)&&(edge[j].p2==edge[k].p1))
        {
        edge[j].p3=edge[k].p3=0;
        }
      }
    }
  while(edge.size()) // build new triangles with edges
    {
    if(edge.back().p3)
      {
      triangleOut.push_back(Util3D::Face(edge.back().p1,edge.back().p2,i));
      complete.push_back(false);
      circle.push_back(Util3D_computeCircle(vertexInOut[triangleOut.back().p1],
                                            vertexInOut[triangleOut.back().p2],
                                            vertexInOut[triangleOut.back().p3],
                                            precision));
      }
    edge.pop_back();
    }
  }
for(i=0;i<triangleOut.size();i++) // remove bounding triangle
  {
  if((triangleOut[i].p1>=nb)||
     (triangleOut[i].p2>=nb)||
     (triangleOut[i].p3>=nb))
    {
    StlVectorFastErase(triangleOut,i);
    i--;
    }
  }
vertexInOut.pop_back(); // remove bounding triangle vertices
vertexInOut.pop_back();
vertexInOut.pop_back();
}

} // namespace AReVi

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

/* Triangle/triangle intersection test routine,
 * by Tomas Moller, 1997.
 * See article "A Fast Triangle-Triangle Intersection Test",
 * Journal of Graphics Tools, 2(2), 1997
 * updated: 2001-06-20 (added line of intersection)
 *
 * int tri_tri_intersect(double V0[3],double V1[3],double V2[3],
 *                       double U0[3],double U1[3],double U2[3])
 *
 * parameters: vertices of triangle 1: V0,V1,V2
 *             vertices of triangle 2: U0,U1,U2
 * result    : returns 1 if the triangles intersect, otherwise 0
 *
 * Here is a version withouts divisions (a little faster)
 * int NoDivTriTriIsect(double V0[3],double V1[3],double V2[3],
 *                      double U0[3],double U1[3],double U2[3]);
 *
 * This version computes the line of intersection as well (if they are not
 * coplanar):
 * int tri_tri_intersect_with_isectline(double V0[3],double V1[3],double V2[3],
 *                                      double U0[3],double U1[3],double U2[3],
 *                                      int *coplanar,
 *                                      double isectpt1[3],double isectpt2[3]);
 * coplanar returns whether the tris are coplanar
 * isectpt1, isectpt2 are the endpoints of the line of intersection
 */

#include <math.h>

#define FABS(x) (fabs(x)) /* implement as is fastest on your machine */

/* if USE_EPSILON_TEST is true then we do a check:
         if |dv|<EPSILON then dv=0.0;
   else no check is done (which is less robust)
*/
#define USE_EPSILON_TEST TRUE
#define EPSILON 1e-9

/* some macros */
#define CROSS(dest,v1,v2) {                    \
              dest[0]=v1[1]*v2[2]-v1[2]*v2[1]; \
              dest[1]=v1[2]*v2[0]-v1[0]*v2[2]; \
              dest[2]=v1[0]*v2[1]-v1[1]*v2[0]; }

#define DOT(v1,v2) (v1[0]*v2[0]+v1[1]*v2[1]+v1[2]*v2[2])

#define SUB(dest,v1,v2) {    \
        dest[0]=v1[0]-v2[0]; \
        dest[1]=v1[1]-v2[1]; \
        dest[2]=v1[2]-v2[2]; }

#define ADD(dest,v1,v2) {    \
        dest[0]=v1[0]+v2[0]; \
        dest[1]=v1[1]+v2[1]; \
        dest[2]=v1[2]+v2[2]; }

#define MULT(dest,v,factor) { \
        dest[0]=factor*v[0];  \
        dest[1]=factor*v[1];  \
        dest[2]=factor*v[2]; }

#define SET(dest,src) { \
        dest[0]=src[0]; \
        dest[1]=src[1]; \
        dest[2]=src[2]; }

/* sort so that a<=b */
#define SORT(a,b) {      \
             if(a>b)     \
             {           \
               double c; \
               c=a;      \
               a=b;      \
               b=c;      \
             } }

#define ISECT(VV0,VV1,VV2,D0,D1,D2,isect0,isect1) { \
              isect0=VV0+(VV1-VV0)*D0/(D0-D1);      \
              isect1=VV0+(VV2-VV0)*D0/(D0-D2); }

#define COMPUTE_INTERVALS(VV0,VV1,VV2,D0,D1,D2,D0D1,D0D2,isect0,isect1) \
  if(D0D1>0.0f)                                         \
  {                                                     \
    /* here we know that D0D2<=0.0 */                   \
    /* that is D0, D1 are on the same side, */          \
    /* D2 on the other or on the plane */               \
    ISECT(VV2,VV0,VV1,D2,D0,D1,isect0,isect1);          \
  }                                                     \
  else if(D0D2>0.0f)                                    \
  {                                                     \
    /* here we know that d0d1<=0.0 */                   \
    ISECT(VV1,VV0,VV2,D1,D0,D2,isect0,isect1);          \
  }                                                     \
  else if(D1*D2>0.0f || D0!=0.0f)                       \
  {                                                     \
    /* here we know that d0d1<=0.0 or that D0!=0.0 */   \
    ISECT(VV0,VV1,VV2,D0,D1,D2,isect0,isect1);          \
  }                                                     \
  else if(D1!=0.0f)                                     \
  {                                                     \
    ISECT(VV1,VV0,VV2,D1,D0,D2,isect0,isect1);          \
  }                                                     \
  else if(D2!=0.0f)                                     \
  {                                                     \
    ISECT(VV2,VV0,VV1,D2,D0,D1,isect0,isect1);          \
  }                                                     \
  else                                                  \
  {                                                     \
    /* triangles are coplanar */                        \
    return coplanar_tri_tri(N1,V0,V1,V2,U0,U1,U2);      \
  }

/* this edge to edge test is based on Franlin Antonio's gem:
   "Faster Line Segment Intersection", in Graphics Gems III,
   pp. 199-202 */
#define EDGE_EDGE_TEST(V0,U0,U1) {                    \
  Bx=U0[i0]-U1[i0];                                   \
  By=U0[i1]-U1[i1];                                   \
  Cx=V0[i0]-U0[i0];                                   \
  Cy=V0[i1]-U0[i1];                                   \
  f=Ay*Bx-Ax*By;                                      \
  d=By*Cx-Bx*Cy;                                      \
  if((f>0 && d>=0 && d<=f) || (f<0 && d<=0 && d>=f))  \
  {                                                   \
    e=Ax*Cy-Ay*Cx;                                    \
    if(f>0)                                           \
    {                                                 \
      if(e>=0 && e<=f) return 1;                      \
    }                                                 \
    else                                              \
    {                                                 \
      if(e<=0 && e>=f) return 1;                      \
    }                                                 \
  } }

#define EDGE_AGAINST_TRI_EDGES(V0,V1,U0,U1,U2) \
{                                              \
  double Ax,Ay,Bx,By,Cx,Cy,e,d,f;              \
  Ax=V1[i0]-V0[i0];                            \
  Ay=V1[i1]-V0[i1];                            \
  /* test edge U0,U1 against V0,V1 */          \
  EDGE_EDGE_TEST(V0,U0,U1);                    \
  /* test edge U1,U2 against V0,V1 */          \
  EDGE_EDGE_TEST(V0,U1,U2);                    \
  /* test edge U2,U1 against V0,V1 */          \
  EDGE_EDGE_TEST(V0,U2,U0);                    \
}

#define POINT_IN_TRI(V0,U0,U1,U2)           \
{                                           \
  double a,b,c,d0,d1,d2;                    \
  /* is T1 completly inside T2? */          \
  /* check if V0 is inside tri(U0,U1,U2) */ \
  a=U1[i1]-U0[i1];                          \
  b=-(U1[i0]-U0[i0]);                       \
  c=-a*U0[i0]-b*U0[i1];                     \
  d0=a*V0[i0]+b*V0[i1]+c;                   \
                                            \
  a=U2[i1]-U1[i1];                          \
  b=-(U2[i0]-U1[i0]);                       \
  c=-a*U1[i0]-b*U1[i1];                     \
  d1=a*V0[i0]+b*V0[i1]+c;                   \
                                            \
  a=U0[i1]-U2[i1];                          \
  b=-(U0[i0]-U2[i0]);                       \
  c=-a*U2[i0]-b*U2[i1];                     \
  d2=a*V0[i0]+b*V0[i1]+c;                   \
  if(d0*d1>0.0)                             \
  {                                         \
    if(d0*d2>0.0) return 1;                 \
  }                                         \
}

int
coplanar_tri_tri(double N[3],double V0[3],double V1[3],double V2[3],
                 double U0[3],double U1[3],double U2[3])
{
   double A[3];
   short i0,i1;
   /* first project onto an axis-aligned plane, that maximizes the area */
   /* of the triangles, compute indices: i0,i1. */
   A[0]=fabs(N[0]);
   A[1]=fabs(N[1]);
   A[2]=fabs(N[2]);
   if(A[0]>A[1])
   {
      if(A[0]>A[2])
      {
          i0=1;      /* A[0] is greatest */
          i1=2;
      }
      else
      {
          i0=0;      /* A[2] is greatest */
          i1=1;
      }
   }
   else   /* A[0]<=A[1] */
   {
      if(A[2]>A[1])
      {
          i0=0;      /* A[2] is greatest */
          i1=1;
      }
      else
      {
          i0=0;      /* A[1] is greatest */
          i1=2;
      }
    }

    /* test all edges of triangle 1 against the edges of triangle 2 */
    EDGE_AGAINST_TRI_EDGES(V0,V1,U0,U1,U2);
    EDGE_AGAINST_TRI_EDGES(V1,V2,U0,U1,U2);
    EDGE_AGAINST_TRI_EDGES(V2,V0,U0,U1,U2);

    /* finally, test if tri1 is totally contained in tri2 or vice versa */
    POINT_IN_TRI(V0,U0,U1,U2);
    POINT_IN_TRI(U0,V0,V1,V2);

    return 0;
}

int
tri_tri_intersect(double V0[3],double V1[3],double V2[3],
                  double U0[3],double U1[3],double U2[3])
{
  double E1[3],E2[3];
  double N1[3],N2[3],d1,d2;
  double du0,du1,du2,dv0,dv1,dv2;
  double D[3];
  double isect1[2], isect2[2];
  double du0du1,du0du2,dv0dv1,dv0dv2;
  short index;
  double vp0,vp1,vp2;
  double up0,up1,up2;
  double b,c,max;

  /* compute plane equation of triangle(V0,V1,V2) */
  SUB(E1,V1,V0);
  SUB(E2,V2,V0);
  CROSS(N1,E1,E2);
  d1=-DOT(N1,V0);
  /* plane equation 1: N1.X+d1=0 */

  /* put U0,U1,U2 into plane equation 1 to compute */
  /* signed distances to the plane*/
  du0=DOT(N1,U0)+d1;
  du1=DOT(N1,U1)+d1;
  du2=DOT(N1,U2)+d1;

  /* coplanarity robustness check */
#if USE_EPSILON_TEST==TRUE
  if(fabs(du0)<EPSILON) du0=0.0;
  if(fabs(du1)<EPSILON) du1=0.0;
  if(fabs(du2)<EPSILON) du2=0.0;
#endif
  du0du1=du0*du1;
  du0du2=du0*du2;

  if(du0du1>0.0f && du0du2>0.0f) /* same sign on all of them + not equal 0 ? */
    return 0;                    /* no intersection occurs */

  /* compute plane of triangle (U0,U1,U2) */
  SUB(E1,U1,U0);
  SUB(E2,U2,U0);
  CROSS(N2,E1,E2);
  d2=-DOT(N2,U0);
  /* plane equation 2: N2.X+d2=0 */

  /* put V0,V1,V2 into plane equation 2 */
  dv0=DOT(N2,V0)+d2;
  dv1=DOT(N2,V1)+d2;
  dv2=DOT(N2,V2)+d2;

#if USE_EPSILON_TEST==TRUE
  if(fabs(dv0)<EPSILON) dv0=0.0;
  if(fabs(dv1)<EPSILON) dv1=0.0;
  if(fabs(dv2)<EPSILON) dv2=0.0;
#endif

  dv0dv1=dv0*dv1;
  dv0dv2=dv0*dv2;

  if(dv0dv1>0.0f && dv0dv2>0.0f) /* same sign on all of them + not equal 0 ? */
    return 0;                    /* no intersection occurs */

  /* compute direction of intersection line */
  CROSS(D,N1,N2);

  /* compute and index to the largest component of D */
  max=fabs(D[0]);
  index=0;
  b=fabs(D[1]);
  c=fabs(D[2]);
  if(b>max) max=b,index=1;
  if(c>max) max=c,index=2;

  /* this is the simplified projection onto L*/
  vp0=V0[index];
  vp1=V1[index];
  vp2=V2[index];

  up0=U0[index];
  up1=U1[index];
  up2=U2[index];

  /* compute interval for triangle 1 */
  COMPUTE_INTERVALS(vp0,vp1,vp2,dv0,dv1,dv2,dv0dv1,dv0dv2,isect1[0],isect1[1]);

  /* compute interval for triangle 2 */
  COMPUTE_INTERVALS(up0,up1,up2,du0,du1,du2,du0du1,du0du2,isect2[0],isect2[1]);

  SORT(isect1[0],isect1[1]);
  SORT(isect2[0],isect2[1]);

  if(isect1[1]<isect2[0] || isect2[1]<isect1[0]) return 0;
  return 1;
}

#define NEWCOMPUTE_INTERVALS(VV0,VV1,VV2,D0,D1,D2,D0D1,D0D2,A,B,C,X0,X1) \
{ \
        if(D0D1>0.0f) \
        { \
          /* here we know that D0D2<=0.0 */ \
          /* that is D0, D1 are on the same side, */ \
          /* D2 on the other or on the plane */ \
                A=VV2; B=(VV0-VV2)*D2; C=(VV1-VV2)*D2; X0=D2-D0; X1=D2-D1; \
        } \
        else if(D0D2>0.0f)\
        { \
                /* here we know that d0d1<=0.0 */ \
            A=VV1; B=(VV0-VV1)*D1; C=(VV2-VV1)*D1; X0=D1-D0; X1=D1-D2; \
        } \
        else if(D1*D2>0.0f || D0!=0.0f) \
        { \
                /* here we know that d0d1<=0.0 or that D0!=0.0 */ \
                A=VV0; B=(VV1-VV0)*D0; C=(VV2-VV0)*D0; X0=D0-D1; X1=D0-D2; \
        } \
        else if(D1!=0.0f) \
        { \
                A=VV1; B=(VV0-VV1)*D1; C=(VV2-VV1)*D1; X0=D1-D0; X1=D1-D2; \
        } \
        else if(D2!=0.0f) \
        { \
                A=VV2; B=(VV0-VV2)*D2; C=(VV1-VV2)*D2; X0=D2-D0; X1=D2-D1; \
        } \
        else \
        { \
                /* triangles are coplanar */ \
                return coplanar_tri_tri(N1,V0,V1,V2,U0,U1,U2); \
        } \
}

int
NoDivTriTriIsect(double V0[3],double V1[3],double V2[3],
                 double U0[3],double U1[3],double U2[3])
{
  double E1[3],E2[3];
  double N1[3],N2[3],d1,d2;
  double du0,du1,du2,dv0,dv1,dv2;
  double D[3];
  double isect1[2], isect2[2];
  double du0du1,du0du2,dv0dv1,dv0dv2;
  short index;
  double vp0,vp1,vp2;
  double up0,up1,up2;
  double bb,cc,max;
  double a,b,c,x0,x1;
  double d,e,f,y0,y1;
  double xx,yy,xxyy,tmp;

  /* compute plane equation of triangle(V0,V1,V2) */
  SUB(E1,V1,V0);
  SUB(E2,V2,V0);
  CROSS(N1,E1,E2);
  d1=-DOT(N1,V0);
  /* plane equation 1: N1.X+d1=0 */

  /* put U0,U1,U2 into plane equation 1 to compute signed */
  /* distances to the plane*/
  du0=DOT(N1,U0)+d1;
  du1=DOT(N1,U1)+d1;
  du2=DOT(N1,U2)+d1;

  /* coplanarity robustness check */
#if USE_EPSILON_TEST==TRUE
  if(FABS(du0)<EPSILON) du0=0.0;
  if(FABS(du1)<EPSILON) du1=0.0;
  if(FABS(du2)<EPSILON) du2=0.0;
#endif
  du0du1=du0*du1;
  du0du2=du0*du2;

  if(du0du1>0.0f && du0du2>0.0f) /* same sign on all of them + not equal 0 ? */
    return 0;                    /* no intersection occurs */

  /* compute plane of triangle (U0,U1,U2) */
  SUB(E1,U1,U0);
  SUB(E2,U2,U0);
  CROSS(N2,E1,E2);
  d2=-DOT(N2,U0);
  /* plane equation 2: N2.X+d2=0 */

  /* put V0,V1,V2 into plane equation 2 */
  dv0=DOT(N2,V0)+d2;
  dv1=DOT(N2,V1)+d2;
  dv2=DOT(N2,V2)+d2;

#if USE_EPSILON_TEST==TRUE
  if(FABS(dv0)<EPSILON) dv0=0.0;
  if(FABS(dv1)<EPSILON) dv1=0.0;
  if(FABS(dv2)<EPSILON) dv2=0.0;
#endif

  dv0dv1=dv0*dv1;
  dv0dv2=dv0*dv2;

  if(dv0dv1>0.0f && dv0dv2>0.0f) /* same sign on all of them + not equal 0 ? */
    return 0;                    /* no intersection occurs */

  /* compute direction of intersection line */
  CROSS(D,N1,N2);

  /* compute and index to the largest component of D */
  max=FABS(D[0]);
  index=0;
  bb=FABS(D[1]);
  cc=FABS(D[2]);
  if(bb>max) max=bb,index=1;
  if(cc>max) max=cc,index=2;

  /* this is the simplified projection onto L*/
  vp0=V0[index];
  vp1=V1[index];
  vp2=V2[index];

  up0=U0[index];
  up1=U1[index];
  up2=U2[index];

  /* compute interval for triangle 1 */
  NEWCOMPUTE_INTERVALS(vp0,vp1,vp2,dv0,dv1,dv2,dv0dv1,dv0dv2,a,b,c,x0,x1);

  /* compute interval for triangle 2 */
  NEWCOMPUTE_INTERVALS(up0,up1,up2,du0,du1,du2,du0du1,du0du2,d,e,f,y0,y1);

  xx=x0*x1;
  yy=y0*y1;
  xxyy=xx*yy;

  tmp=a*xxyy;
  isect1[0]=tmp+b*x1*yy;
  isect1[1]=tmp+c*x0*yy;

  tmp=d*xxyy;
  isect2[0]=tmp+e*xx*y1;
  isect2[1]=tmp+f*xx*y0;

  SORT(isect1[0],isect1[1]);
  SORT(isect2[0],isect2[1]);

  if(isect1[1]<isect2[0] || isect2[1]<isect1[0]) return 0;
  return 1;
}

/* sort so that a<=b */
#define SORT2(a,b,smallest) { \
             if(a>b)          \
             {                \
               double c;      \
               c=a;           \
               a=b;           \
               b=c;           \
               smallest=1;    \
             }                \
             else smallest=0; }

inline
void
isect2(double VTX0[3],double VTX1[3],double VTX2[3],
       double VV0,double VV1,double VV2,
       double D0,double D1,double D2,
       double *isect0,double *isect1,
       double isectpoint0[3],double isectpoint1[3])
{
  double tmp=D0/(D0-D1);
  double diff[3];
  *isect0=VV0+(VV1-VV0)*tmp;
  SUB(diff,VTX1,VTX0);
  MULT(diff,diff,tmp);
  ADD(isectpoint0,diff,VTX0);
  tmp=D0/(D0-D2);
  *isect1=VV0+(VV2-VV0)*tmp;
  SUB(diff,VTX2,VTX0);
  MULT(diff,diff,tmp);
  ADD(isectpoint1,VTX0,diff);
}

#if 0
#define ISECT2(VTX0,VTX1,VTX2,VV0,VV1,VV2,D0,D1,D2, \
               isect0,isect1,isectpoint0,isectpoint1) { \
        tmp=D0/(D0-D1);                                 \
        isect0=VV0+(VV1-VV0)*tmp;                       \
        SUB(diff,VTX1,VTX0);                            \
        MULT(diff,diff,tmp);                            \
        ADD(isectpoint0,diff,VTX0);                     \
        tmp=D0/(D0-D2);
/*        isect1=VV0+(VV2-VV0)*tmp;                       \ */
/*        SUB(diff,VTX2,VTX0);                            \ */
/*        MULT(diff,diff,tmp);                            \ */
/*        ADD(isectpoint1,VTX0,diff);                       */
#endif

inline
int
compute_intervals_isectline(double VERT0[3],double VERT1[3],double VERT2[3],
                            double VV0,double VV1,double VV2,
                            double D0,double D1,double D2,
                            double D0D1,double D0D2,
                            double *isect0,double *isect1,
                            double isectpoint0[3],double isectpoint1[3])
{
  if(D0D1>0.0f)
  {
    /* here we know that D0D2<=0.0 */
    /* that is D0, D1 are on the same side, D2 on the other or on the plane */
    isect2(VERT2,VERT0,VERT1,VV2,VV0,VV1,D2,D0,D1,
           isect0,isect1,isectpoint0,isectpoint1);
  }
  else if(D0D2>0.0f)
    {
    /* here we know that d0d1<=0.0 */
    isect2(VERT1,VERT0,VERT2,VV1,VV0,VV2,D1,D0,D2,
           isect0,isect1,isectpoint0,isectpoint1);
  }
  else if(D1*D2>0.0f || D0!=0.0f)
  {
    /* here we know that d0d1<=0.0 or that D0!=0.0 */
    isect2(VERT0,VERT1,VERT2,VV0,VV1,VV2,D0,D1,D2,
           isect0,isect1,isectpoint0,isectpoint1);
  }
  else if(D1!=0.0f)
  {
    isect2(VERT1,VERT0,VERT2,VV1,VV0,VV2,D1,D0,D2,
           isect0,isect1,isectpoint0,isectpoint1);
  }
  else if(D2!=0.0f)
  {
    isect2(VERT2,VERT0,VERT1,VV2,VV0,VV1,D2,D0,D1,
           isect0,isect1,isectpoint0,isectpoint1);
  }
  else
  {
    /* triangles are coplanar */
    return 1;
  }
  return 0;
}

#define COMPUTE_INTERVALS_ISECTLINE(VERT0,VERT1,VERT2,VV0,VV1,VV2, \
                                    D0,D1,D2,D0D1,D0D2, \
                                    isect0,isect1,isectpoint0,isectpoint1) \
  if(D0D1>0.0f)                                         \
  {                                                     \
    /* here we know that D0D2<=0.0 */                   \
    /* that is D0, D1 are on the same side, */          \
    /* D2 on the other or on the plane */               \
    isect2(VERT2,VERT0,VERT1,VV2,VV0,VV1,D2,D0,D1,      \
           &isect0,&isect1,isectpoint0,isectpoint1);    \
  }
#if 0
  else if(D0D2>0.0f)                                    \
  {                                                     \
    /* here we know that d0d1<=0.0 */                   \
    isect2(VERT1,VERT0,VERT2,VV1,VV0,VV2,D1,D0,D2,      \
           &isect0,&isect1,isectpoint0,isectpoint1);    \
  }                                                     \
  else if(D1*D2>0.0f || D0!=0.0f)                       \
  {                                                     \
    /* here we know that d0d1<=0.0 or that D0!=0.0 */   \
    isect2(VERT0,VERT1,VERT2,VV0,VV1,VV2,D0,D1,D2,      \
           &isect0,&isect1,isectpoint0,isectpoint1);    \
  }                                                     \
  else if(D1!=0.0f)                                     \
  {                                                     \
    isect2(VERT1,VERT0,VERT2,VV1,VV0,VV2,D1,D0,D2,      \
           &isect0,&isect1,isectpoint0,isectpoint1);    \
  }                                                     \
  else if(D2!=0.0f)                                     \
  {                                                     \
    isect2(VERT2,VERT0,VERT1,VV2,VV0,VV1,D2,D0,D1,      \
           &isect0,&isect1,isectpoint0,isectpoint1);    \
  }                                                     \
  else                                                  \
  {                                                     \
    /* triangles are coplanar */                        \
    coplanar=1;                                         \
    return coplanar_tri_tri(N1,V0,V1,V2,U0,U1,U2);      \
  }
#endif

int
tri_tri_intersect_with_isectline(double V0[3],double V1[3],double V2[3],
                                 double U0[3],double U1[3],double U2[3],
                                 int *coplanar,
                                 double isectpt1[3],double isectpt2[3])
{
  double E1[3],E2[3];
  double N1[3],N2[3],d1,d2;
  double du0,du1,du2,dv0,dv1,dv2;
  double D[3];
  double isect1[2]={0.0,0.0}, isect2[2]={0.0,0.0};
  double isectpointA1[3]={0.0,0.0,0.0},isectpointA2[3]={0.0,0.0,0.0};
  double isectpointB1[3]={0.0,0.0,0.0},isectpointB2[3]={0.0,0.0,0.0};
  double du0du1,du0du2,dv0dv1,dv0dv2;
  short index;
  double vp0,vp1,vp2;
  double up0,up1,up2;
  double b,c,max;
  /* double tmp,diff[3]; */
  int smallest1,smallest2;

  /* compute plane equation of triangle(V0,V1,V2) */
  SUB(E1,V1,V0);
  SUB(E2,V2,V0);
  CROSS(N1,E1,E2);
  d1=-DOT(N1,V0);
  /* plane equation 1: N1.X+d1=0 */

  /* put U0,U1,U2 into plane equation 1 to compute */
  /* signed distances to the plane*/
  du0=DOT(N1,U0)+d1;
  du1=DOT(N1,U1)+d1;
  du2=DOT(N1,U2)+d1;

  /* coplanarity robustness check */
#if USE_EPSILON_TEST==TRUE
  if(fabs(du0)<EPSILON) du0=0.0;
  if(fabs(du1)<EPSILON) du1=0.0;
  if(fabs(du2)<EPSILON) du2=0.0;
#endif
  du0du1=du0*du1;
  du0du2=du0*du2;

  if(du0du1>0.0f && du0du2>0.0f) /* same sign on all of them + not equal 0 ? */
    return 0;                    /* no intersection occurs */

  /* compute plane of triangle (U0,U1,U2) */
  SUB(E1,U1,U0);
  SUB(E2,U2,U0);
  CROSS(N2,E1,E2);
  d2=-DOT(N2,U0);
  /* plane equation 2: N2.X+d2=0 */

  /* put V0,V1,V2 into plane equation 2 */
  dv0=DOT(N2,V0)+d2;
  dv1=DOT(N2,V1)+d2;
  dv2=DOT(N2,V2)+d2;

#if USE_EPSILON_TEST==TRUE
  if(fabs(dv0)<EPSILON) dv0=0.0;
  if(fabs(dv1)<EPSILON) dv1=0.0;
  if(fabs(dv2)<EPSILON) dv2=0.0;
#endif

  dv0dv1=dv0*dv1;
  dv0dv2=dv0*dv2;

  if(dv0dv1>0.0f && dv0dv2>0.0f) /* same sign on all of them + not equal 0 ? */
    return 0;                    /* no intersection occurs */

  /* compute direction of intersection line */
  CROSS(D,N1,N2);

  /* compute and index to the largest component of D */
  max=fabs(D[0]);
  index=0;
  b=fabs(D[1]);
  c=fabs(D[2]);
  if(b>max) max=b,index=1;
  if(c>max) max=c,index=2;

  /* this is the simplified projection onto L*/
  vp0=V0[index];
  vp1=V1[index];
  vp2=V2[index];

  up0=U0[index];
  up1=U1[index];
  up2=U2[index];

  /* compute interval for triangle 1 */
  *coplanar=compute_intervals_isectline(V0,V1,V2,vp0,vp1,vp2,dv0,dv1,dv2,
                                        dv0dv1,dv0dv2,
                                        &isect1[0],&isect1[1],
                                        isectpointA1,isectpointA2);
  if(*coplanar) return coplanar_tri_tri(N1,V0,V1,V2,U0,U1,U2);


  /* compute interval for triangle 2 */
  compute_intervals_isectline(U0,U1,U2,up0,up1,up2,du0,du1,du2,
                              du0du1,du0du2,
                              &isect2[0],&isect2[1],
                              isectpointB1,isectpointB2);

  SORT2(isect1[0],isect1[1],smallest1);
  SORT2(isect2[0],isect2[1],smallest2);

  if(isect1[1]<isect2[0] || isect2[1]<isect1[0]) return 0;

  /* at this point, we know that the triangles intersect */

  if(isect2[0]<isect1[0])
  {
    if(smallest1==0) { SET(isectpt1,isectpointA1); }
    else { SET(isectpt1,isectpointA2); }

    if(isect2[1]<isect1[1])
    {
      if(smallest2==0) { SET(isectpt2,isectpointB2); }
      else { SET(isectpt2,isectpointB1); }
    }
    else
    {
      if(smallest1==0) { SET(isectpt2,isectpointA2); }
      else { SET(isectpt2,isectpointA1); }
    }
  }
  else
  {
    if(smallest2==0) { SET(isectpt1,isectpointB1); }
    else { SET(isectpt1,isectpointB2); }

    if(isect2[1]>isect1[1])
    {
      if(smallest1==0) { SET(isectpt2,isectpointA2); }
      else { SET(isectpt2,isectpointA1); }
    }
    else
    {
      if(smallest2==0) { SET(isectpt2,isectpointB2); }
      else { SET(isectpt2,isectpointB1); }
    }
  }
  return 1;
}

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

/********************************************************/
/* AABB-triangle overlap test code                      */
/* by Tomas Akenine-Moller                              */
/* Function: int triBoxOverlap(double boxcenter[3],     */
/*        double boxhalfsize[3],double triverts[3][3]); */
/* History:                                             */
/*   2001-03-05: released the code in its first version */
/*   2001-06-18: changed the order of the tests, faster */
/*                                                      */
/* Acknowledgement: Many thanks to Pierre Terdiman for  */
/* suggestions and discussions on how to optimize code. */
/* Thanks to David Hunt for finding a ">="-bug!         */
/********************************************************/

#define X 0
#define Y 1
#define Z 2

#define FINDMINMAX(x0,x1,x2,min,max) { \
  min = max = x0;                      \
  if(x1<min) min=x1;                   \
  if(x1>max) max=x1;                   \
  if(x2<min) min=x2;                   \
  if(x2>max) max=x2; }

int planeBoxOverlap(double normal[3],double d, double maxbox[3])
{
  int q;
  double vmin[3],vmax[3];
  for(q=X;q<=Z;q++)
  {
    if(normal[q]>0.0f)
    {
      vmin[q]=-maxbox[q];
      vmax[q]=maxbox[q];
    }
    else
    {
      vmin[q]=maxbox[q];
      vmax[q]=-maxbox[q];
    }
  }
  if(DOT(normal,vmin)+d>0.0f) return 0;
  if(DOT(normal,vmax)+d>=0.0f) return 1;

  return 0;
}

/*======================== X-tests ========================*/
#define AXISTEST_X01(a, b, fa, fb) {                       \
        p0 = a*v0[Y] - b*v0[Z];                            \
        p2 = a*v2[Y] - b*v2[Z];                            \
        if(p0<p2) {min=p0; max=p2;} else {min=p2; max=p0;} \
        rad = fa * boxhalfsize[Y] + fb * boxhalfsize[Z];   \
        if(min>rad || max<-rad) return 0; }

#define AXISTEST_X2(a, b, fa, fb) {                        \
        p0 = a*v0[Y] - b*v0[Z];                            \
        p1 = a*v1[Y] - b*v1[Z];                            \
        if(p0<p1) {min=p0; max=p1;} else {min=p1; max=p0;} \
        rad = fa * boxhalfsize[Y] + fb * boxhalfsize[Z];   \
        if(min>rad || max<-rad) return 0; }

/*======================== Y-tests ========================*/
#define AXISTEST_Y02(a, b, fa, fb) {                       \
        p0 = -a*v0[X] + b*v0[Z];                           \
        p2 = -a*v2[X] + b*v2[Z];                           \
        if(p0<p2) {min=p0; max=p2;} else {min=p2; max=p0;} \
        rad = fa * boxhalfsize[X] + fb * boxhalfsize[Z];   \
        if(min>rad || max<-rad) return 0; }

#define AXISTEST_Y1(a, b, fa, fb) {                        \
        p0 = -a*v0[X] + b*v0[Z];                           \
        p1 = -a*v1[X] + b*v1[Z];                           \
        if(p0<p1) {min=p0; max=p1;} else {min=p1; max=p0;} \
        rad = fa * boxhalfsize[X] + fb * boxhalfsize[Z];   \
        if(min>rad || max<-rad) return 0; }

/*======================== Z-tests ========================*/
#define AXISTEST_Z12(a, b, fa, fb) {                       \
        p1 = a*v1[X] - b*v1[Y];                            \
        p2 = a*v2[X] - b*v2[Y];                            \
        if(p2<p1) {min=p2; max=p1;} else {min=p1; max=p2;} \
        rad = fa * boxhalfsize[X] + fb * boxhalfsize[Y];   \
        if(min>rad || max<-rad) return 0; }

#define AXISTEST_Z0(a, b, fa, fb) {                        \
        p0 = a*v0[X] - b*v0[Y];                            \
        p1 = a*v1[X] - b*v1[Y];                            \
        if(p0<p1) {min=p0; max=p1;} else {min=p1; max=p0;} \
        rad = fa * boxhalfsize[X] + fb * boxhalfsize[Y];   \
        if(min>rad || max<-rad) return 0; }

int
triBoxOverlap(double boxcenter[3],double boxhalfsize[3],double triverts[3][3])
{

/* use separating axis theorem to test overlap between triangle and box */
/* need to test for overlap in these directions: */
/* 1) the {x,y,z}-directions (actually, since we use the AABB of the */
/*    triangle we do not even need to test these) */
/* 2) normal of the triangle */
/* 3) crossproduct(edge from tri, {x,y,z}-directin) */
/*    this gives 3x3=9 more tests */
   double v0[3],v1[3],v2[3];
   /* double axis[3]; */
   double min,max,d,p0,p1,p2,rad,fex,fey,fez;
   double normal[3],e0[3],e1[3],e2[3];

   /* This is the fastest branch on Sun */
   /* move everything so that the boxcenter is in (0,0,0) */
   SUB(v0,triverts[0],boxcenter);
   SUB(v1,triverts[1],boxcenter);
   SUB(v2,triverts[2],boxcenter);

   /* compute triangle edges */
   SUB(e0,v1,v0);      /* tri edge 0 */
   SUB(e1,v2,v1);      /* tri edge 1 */
   SUB(e2,v0,v2);      /* tri edge 2 */

   /* Bullet 3:  */
   /*  test the 9 tests first (this was faster) */
   fex = fabs(e0[X]);
   fey = fabs(e0[Y]);
   fez = fabs(e0[Z]);
   AXISTEST_X01(e0[Z], e0[Y], fez, fey);
   AXISTEST_Y02(e0[Z], e0[X], fez, fex);
   AXISTEST_Z12(e0[Y], e0[X], fey, fex);

   fex = fabs(e1[X]);
   fey = fabs(e1[Y]);
   fez = fabs(e1[Z]);
   AXISTEST_X01(e1[Z], e1[Y], fez, fey);
   AXISTEST_Y02(e1[Z], e1[X], fez, fex);
   AXISTEST_Z0(e1[Y], e1[X], fey, fex);

   fex = fabs(e2[X]);
   fey = fabs(e2[Y]);
   fez = fabs(e2[Z]);
   AXISTEST_X2(e2[Z], e2[Y], fez, fey);
   AXISTEST_Y1(e2[Z], e2[X], fez, fex);
   AXISTEST_Z12(e2[Y], e2[X], fey, fex);

   /* Bullet 1: */
   /*  first test overlap in the {x,y,z}-directions */
   /*  find min, max of the triangle each direction, and test for overlap in */
   /*  that direction -- this is equivalent to testing a minimal AABB around */
   /*  the triangle against the AABB */

   /* test in X-direction */
   FINDMINMAX(v0[X],v1[X],v2[X],min,max);
   if(min>boxhalfsize[X] || max<-boxhalfsize[X]) return 0;

   /* test in Y-direction */
   FINDMINMAX(v0[Y],v1[Y],v2[Y],min,max);
   if(min>boxhalfsize[Y] || max<-boxhalfsize[Y]) return 0;

   /* test in Z-direction */
   FINDMINMAX(v0[Z],v1[Z],v2[Z],min,max);
   if(min>boxhalfsize[Z] || max<-boxhalfsize[Z]) return 0;

   /* Bullet 2: */
   /*  test if the box intersects the plane of the triangle */
   /*  compute plane equation of triangle: normal*x+d=0 */
   CROSS(normal,e0,e1);
   d=-DOT(normal,v0);  /* plane eq: normal.x+d=0 */
   if(!planeBoxOverlap(normal,d,boxhalfsize)) return 0;

   return 1;   /* box and triangle overlaps */
}

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

/* Ray-Triangle Intersection Test Routines          */
/* Different optimizations of my and Ben Trumbore's */
/* code from journals of graphics tools (JGT)       */
/* http://www.acm.org/jgt/                          */
/* by Tomas Moller, May 2000                        */

/* the original jgt code */
int
intersect_triangle(double orig[3], double dir[3],
                   double vert0[3], double vert1[3], double vert2[3],
                   double *t, double *u, double *v)
{
   double edge1[3], edge2[3], tvec[3], pvec[3], qvec[3];
   double det,inv_det;

   /* find vectors for two edges sharing vert0 */
   SUB(edge1, vert1, vert0);
   SUB(edge2, vert2, vert0);

   /* begin calculating determinant - also used to calculate U parameter */
   CROSS(pvec, dir, edge2);

   /* if determinant is near zero, ray lies in plane of triangle */
   det = DOT(edge1, pvec);

   if (det > -EPSILON && det < EPSILON)
     return 0;
   inv_det = 1.0 / det;

   /* calculate distance from vert0 to ray origin */
   SUB(tvec, orig, vert0);

   /* calculate U parameter and test bounds */
   *u = DOT(tvec, pvec) * inv_det;
   if (*u < 0.0 || *u > 1.0)
     return 0;

   /* prepare to test V parameter */
   CROSS(qvec, tvec, edge1);

   /* calculate V parameter and test bounds */
   *v = DOT(dir, qvec) * inv_det;
   if (*v < 0.0 || *u + *v > 1.0)
     return 0;

   /* calculate t, ray intersects triangle */
   *t = DOT(edge2, qvec) * inv_det;

   return 1;
}


/* code rewritten to do tests on the sign of the determinant */
/* the division is at the end in the code                    */
int
intersect_triangle1(double orig[3], double dir[3],
                    double vert0[3], double vert1[3], double vert2[3],
                    double *t, double *u, double *v)
{
   double edge1[3], edge2[3], tvec[3], pvec[3], qvec[3];
   double det,inv_det;

   /* find vectors for two edges sharing vert0 */
   SUB(edge1, vert1, vert0);
   SUB(edge2, vert2, vert0);

   /* begin calculating determinant - also used to calculate U parameter */
   CROSS(pvec, dir, edge2);

   /* if determinant is near zero, ray lies in plane of triangle */
   det = DOT(edge1, pvec);

   if (det > EPSILON)
   {
      /* calculate distance from vert0 to ray origin */
      SUB(tvec, orig, vert0);

      /* calculate U parameter and test bounds */
      *u = DOT(tvec, pvec);
      if (*u < 0.0 || *u > det)
        return 0;

      /* prepare to test V parameter */
      CROSS(qvec, tvec, edge1);

      /* calculate V parameter and test bounds */
      *v = DOT(dir, qvec);
      if (*v < 0.0 || *u + *v > det)
        return 0;

   }
   else if(det < -EPSILON)
   {
      /* calculate distance from vert0 to ray origin */
      SUB(tvec, orig, vert0);

      /* calculate U parameter and test bounds */
      *u = DOT(tvec, pvec);
/*      printf("*u=%f\n",*u); */
/*      printf("det=%f\n",det); */
      if (*u > 0.0 || *u < det)
        return 0;

      /* prepare to test V parameter */
      CROSS(qvec, tvec, edge1);

      /* calculate V parameter and test bounds */
      *v = DOT(dir, qvec) ;
      if (*v > 0.0 || *u + *v < det)
        return 0;
   }
   else return 0;  /* ray is parallell to the plane of the triangle */


   inv_det = 1.0 / det;

   /* calculate t, ray intersects triangle */
   *t = DOT(edge2, qvec) * inv_det;
   (*u) *= inv_det;
   (*v) *= inv_det;

   return 1;
}

/* code rewritten to do tests on the sign of the determinant */
/* the division is before the test of the sign of the det    */
int
intersect_triangle2(double orig[3], double dir[3],
                    double vert0[3], double vert1[3], double vert2[3],
                    double *t, double *u, double *v)
{
   double edge1[3], edge2[3], tvec[3], pvec[3], qvec[3];
   double det,inv_det;

   /* find vectors for two edges sharing vert0 */
   SUB(edge1, vert1, vert0);
   SUB(edge2, vert2, vert0);

   /* begin calculating determinant - also used to calculate U parameter */
   CROSS(pvec, dir, edge2);

   /* if determinant is near zero, ray lies in plane of triangle */
   det = DOT(edge1, pvec);

   /* calculate distance from vert0 to ray origin */
   SUB(tvec, orig, vert0);
   inv_det = 1.0 / det;

   if (det > EPSILON)
   {
      /* calculate U parameter and test bounds */
      *u = DOT(tvec, pvec);
      if (*u < 0.0 || *u > det)
        return 0;

      /* prepare to test V parameter */
      CROSS(qvec, tvec, edge1);

      /* calculate V parameter and test bounds */
      *v = DOT(dir, qvec);
      if (*v < 0.0 || *u + *v > det)
        return 0;

   }
   else if(det < -EPSILON)
   {
      /* calculate U parameter and test bounds */
      *u = DOT(tvec, pvec);
      if (*u > 0.0 || *u < det)
        return 0;

      /* prepare to test V parameter */
      CROSS(qvec, tvec, edge1);

      /* calculate V parameter and test bounds */
      *v = DOT(dir, qvec) ;
      if (*v > 0.0 || *u + *v < det)
        return 0;
   }
   else return 0;  /* ray is parallell to the plane of the triangle */

   /* calculate t, ray intersects triangle */
   *t = DOT(edge2, qvec) * inv_det;
   (*u) *= inv_det;
   (*v) *= inv_det;

   return 1;
}

/* code rewritten to do tests on the sign of the determinant */
/* the division is before the test of the sign of the det    */
/* and one CROSS has been moved out from the if-else if-else */
int
intersect_triangle3(double orig[3], double dir[3],
                    double vert0[3], double vert1[3], double vert2[3],
                    double *t, double *u, double *v)
{
   double edge1[3], edge2[3], tvec[3], pvec[3], qvec[3];
   double det,inv_det;

   /* find vectors for two edges sharing vert0 */
   SUB(edge1, vert1, vert0);
   SUB(edge2, vert2, vert0);

   /* begin calculating determinant - also used to calculate U parameter */
   CROSS(pvec, dir, edge2);

   /* if determinant is near zero, ray lies in plane of triangle */
   det = DOT(edge1, pvec);

   /* calculate distance from vert0 to ray origin */
   SUB(tvec, orig, vert0);
   inv_det = 1.0 / det;

   CROSS(qvec, tvec, edge1);

   if (det > EPSILON)
   {
      *u = DOT(tvec, pvec);
      if (*u < 0.0 || *u > det)
        return 0;

      /* calculate V parameter and test bounds */
      *v = DOT(dir, qvec);
      if (*v < 0.0 || *u + *v > det)
        return 0;

   }
   else if(det < -EPSILON)
   {
      /* calculate U parameter and test bounds */
      *u = DOT(tvec, pvec);
      if (*u > 0.0 || *u < det)
        return 0;

      /* calculate V parameter and test bounds */
      *v = DOT(dir, qvec) ;
      if (*v > 0.0 || *u + *v < det)
        return 0;
   }
   else return 0;  /* ray is parallell to the plane of the triangle */

   *t = DOT(edge2, qvec) * inv_det;
   (*u) *= inv_det;
   (*v) *= inv_det;

   return 1;
}

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

/*
 * A function for creating a rotation matrix that rotates a vector called
 * "from" into another vector called "to".
 * Input : from[3], to[3] which both must be *normalized* non-zero vectors
 * Output: mtx[3][3] -- a 3x3 matrix in colum-major form
 * Author: Tomas Moller, 1999
 */

void fromToRotation(double from[3],double to[3],double mtx[3][3])
{
  double v[3];
  double e,h;
  CROSS(v,from,to);
  e=DOT(from,to);
  if(e>1.0-EPSILON)     /* "from" almost or equal to "to"-vector? */
  {
    /* return identity */
    mtx[0][0]=1.0; mtx[0][1]=0.0; mtx[0][2]=0.0;
    mtx[1][0]=0.0; mtx[1][1]=1.0; mtx[1][2]=0.0;
    mtx[2][0]=0.0; mtx[2][1]=0.0; mtx[2][2]=1.0;
  }
  else if(e<-1.0+EPSILON) /* "from" almost or equal to negated "to"? */
  {
    double up[3],left[3];
    double invlen;
    double fxx,fyy,fzz,fxy,fxz,fyz;
    double uxx,uyy,uzz,uxy,uxz,uyz;
    double lxx,lyy,lzz,lxy,lxz,lyz;
    /* left=CROSS(from, (1,0,0)) */
    left[0]=0.0; left[1]=from[2]; left[2]=-from[1];
    /* was left=CROSS(from,(1,0,0)) a good choice? */
    if(DOT(left,left)<EPSILON)
    {
      /* here we now that left = CROSS(from, (1,0,0)) will be a good choice */
      left[0]=-from[2]; left[1]=0.0; left[2]=from[0];
    }
    /* normalize "left" */
    invlen=1.0/sqrt(DOT(left,left));
    left[0]*=invlen;
    left[1]*=invlen;
    left[2]*=invlen;
    CROSS(up,left,from);
    /* now we have a coordinate system, i.e., a basis;    */
    /* M=(from, up, left), and we want to rotate to:      */
    /* N=(-from, up, -left). This is done with the matrix:*/
    /* N*M^T where M^T is the transpose of M              */
    fxx=-from[0]*from[0]; fyy=-from[1]*from[1]; fzz=-from[2]*from[2];
    fxy=-from[0]*from[1]; fxz=-from[0]*from[2]; fyz=-from[1]*from[2];

    uxx=up[0]*up[0]; uyy=up[1]*up[1]; uzz=up[2]*up[2];
    uxy=up[0]*up[1]; uxz=up[0]*up[2]; uyz=up[1]*up[2];

    lxx=-left[0]*left[0]; lyy=-left[1]*left[1]; lzz=-left[2]*left[2];
    lxy=-left[0]*left[1]; lxz=-left[0]*left[2]; lyz=-left[1]*left[2];
    /* symmetric matrix */
    mtx[0][0]=fxx+uxx+lxx; mtx[0][1]=fxy+uxy+lxy; mtx[0][2]=fxz+uxz+lxz;
    mtx[1][0]=mtx[0][1];   mtx[1][1]=fyy+uyy+lyy; mtx[1][2]=fyz+uyz+lyz;
    mtx[2][0]=mtx[0][2];   mtx[2][1]=mtx[1][2];   mtx[2][2]=fzz+uzz+lzz;
  }
  else  /* the most common case, unless "from"="to", or "from"=-"to" */
  {
#if 0
    /* unoptimized version - a good compiler will optimize this. */
    h=(1.0-e)/DOT(v,v);
    mtx[0][0]=e+h*v[0]*v[0];
    mtx[0][1]=h*v[0]*v[1]-v[2];
    mtx[0][2]=h*v[0]*v[2]+v[1];
    mtx[1][0]=h*v[0]*v[1]+v[2];
    mtx[1][1]=e+h*v[1]*v[1];
    mtx[1][2]=h*v[1]*v[2]-v[0];
    mtx[2][0]=h*v[0]*v[2]-v[1];
    mtx[2][1]=h*v[1]*v[2]+v[0];
    mtx[2][2]=e+h*v[2]*v[2];
#else
    /* ...otherwise use this hand optimized version (9 mults less) */
    double hvx,hvz,hvxy,hvxz,hvyz;
    h=(1.0-e)/DOT(v,v);
    hvx=h*v[0];
    hvz=h*v[2];
    hvxy=hvx*v[1];
    hvxz=hvx*v[2];
    hvyz=hvz*v[1];
    mtx[0][0]=e+hvx*v[0]; mtx[0][1]=hvxy-v[2];     mtx[0][2]=hvxz+v[1];
    mtx[1][0]=hvxy+v[2];  mtx[1][1]=e+h*v[1]*v[1]; mtx[1][2]=hvyz-v[0];
    mtx[2][0]=hvxz-v[1];  mtx[2][1]=hvyz+v[0];     mtx[2][2]=e+hvz*v[2];
#endif
  }
}

//----------------------------------------------------------------------------
