// LI2/ENIB - AReVi - 3D services
//----------------------------------------------------------------------------
// file        : AReVi/Lib3D/viewer3D.cpp
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/Lib3D/viewer3D.h"

#include "AReVi/arSystem.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class Viewer3D
//----------------------------------------------------------------------------

AR_CLASS_DEF(Viewer3D,Window3D)

//-------- Construction / Destruction ----------------------------------------

Viewer3D::Viewer3D(ArCW & arCW) // Protected
: Window3D(arCW),
  _window((LLW::WinID)0),
  _pix((LLW::PixID)0),
  _font(LLW::loadFont("helvetica,14,b")),
  _blackColor(0),
  _darkColor(0),
  _backColor(0),
  _lightColor(0),
  _whiteColor(0),
  _dx1(6),
  _dx2(6),
  _dy1(30),
  _dy2(6),
  _focus(false),
  _decoration(true),
  _shaded(false),
  _max(false),
  _saveX(0),
  _saveY(0),
  _saveWidth(0),
  _saveHeight(0),
  _clickPart(Viewer3D::PART_NONE),
  _clickDx(0),
  _clickDy(0),
  _clickX(0),
  _clickY(0),
  _clickWidth(0),
  _clickHeight(0),
  _clickDelay(250),
  _clickTime(0),
  _focusCB(thisRef()),
  _appearanceCB(thisRef())
{
if(!_font)
  {
  _font=LLW::loadFont("fixed");
  }
unsigned int minWidth=102;
unsigned int minHeight=minWidth;
_window=LLW::createTopWindow(getAlias(),true,0,0,minWidth,minHeight);
LLW::setWindowMinSize(_window,minWidth,minHeight);
minWidth-=_dx1+_dx2;
minHeight-=_dy1+_dy2;
_renderArea=LLW::createChildWindow(_window,_dx1,_dy1,minWidth,minHeight);
LLW::setWindowMinSize(_renderArea,minWidth,minHeight);
_afterRenderAreaSetup();
_selectColors(false);
LLW::setWindowBackgroundColor(_window,_backColor);
LLW::setWindowEventMask(_window,LLW::EVT_GEOMETRY |
                                LLW::EVT_MOUSE_BUTTON |
                                LLW::EVT_MOUSE_MOTION |
                                LLW::EVT_FOCUS |
                                LLW::EVT_CLOSE);
}

Viewer3D::~Viewer3D(void) // Protected
{
LLW::destroyWindow(_renderArea,false);
LLW::destroyWindow(_window,false);
if(_pix)
  {
  LLW::destroyPixmap(_pix);
  }
LLW::freeFont(_font);
}

//-------- Title -------------------------------------------------------------

void
Viewer3D::setTitle(const StlString & title)
{
LLW::setTopWindowTitle(_window,title);
_redrawDecoration();
if(!_appearanceCB.empty())
  {
  Viewer3D::AppearanceEvent event;
  _appearanceCB.fire(event);
  }
}

const StlString & // window bar title
Viewer3D::getTitle(void) const
{
return(LLW::getTopWindowTitle(_window));
}

//-------- Visibility --------------------------------------------------------

void
Viewer3D::setMapped(bool mapped)
{
if(mapped)
  {
  LLW::mapWindow(_window);
  }
else
  {
  LLW::unmapWindow(_window);
  }
}

bool // viewer is mapped
Viewer3D::isMapped(void) const
{
return(LLW::isWindowMapped(_window));
}

bool // viewer has focus
Viewer3D::hasFocus(void) const
{
return(_focus);
}

//-------- Decoration --------------------------------------------------------

void
Viewer3D::setDecoration(bool decoration)
{
if(decoration!=_decoration)
  {
  _decoration=decoration;
  if(_decoration)
    {
    unsigned int width,height;
    LLW::getWindowSize(_window,width,height);
    _toInnerSize(width,height);
    LLW::setWindowGeometry(_renderArea,_dx1,_dy1,width,height);
    _redrawDecoration();
    }
  else
    {
    if(_shaded)
      {
      _unshade();
      }
    unsigned int width,height;
    LLW::getWindowSize(_window,width,height);
    LLW::setWindowGeometry(_renderArea,0,0,width,height);
    }
  if(!_appearanceCB.empty())
    {
    Viewer3D::AppearanceEvent event;
    _appearanceCB.fire(event);
    }
  }
}

bool // viewer has decoration
Viewer3D::getDecoration(void) const
{
return(_decoration);
}

//-------- Window geometry ---------------------------------------------------

void
Viewer3D::setWindowGeometry(int x,
                            int y,
                            unsigned int width,
                            unsigned int height)
{
_max=false;
if(_decoration)
  {
  _toInnerSize(width,height);
  }
LLW::setWindowSize(_renderArea,width,height);
LLW::getWindowSize(_renderArea,width,height); // min size ?
if(_decoration)
  {
  _toOuterSize(width,height);
  }
if(_shaded)
  {
  LLW::setWindowGeometry(_window,x,y,width,_dy1);
  }
else
  {
  LLW::setWindowGeometry(_window,x,y,width,height);
  }
}

void
Viewer3D::getWindowGeometry(int & xOut,
                            int & yOut,
                            unsigned int & widthOut,
                            unsigned int & heightOut) const
{
getWindowPosition(xOut,yOut);
getWindowSize(widthOut,heightOut);
}

void
Viewer3D::setWindowPosition(int x,
                            int y)
{
LLW::setWindowPosition(_window,x,y);
}

void
Viewer3D::getWindowPosition(int & xOut,
                            int & yOut) const
{
LLW::getWindowPosition(_window,xOut,yOut);
}

void
Viewer3D::setWindowSize(unsigned int width,
                        unsigned int height)
{
_max=false;
if(_decoration)
  {
  _toInnerSize(width,height);
  }
LLW::setWindowSize(_renderArea,width,height);
LLW::getWindowSize(_renderArea,width,height); // min size ?
if(_decoration)
  {
  _toOuterSize(width,height);
  }
if(_shaded)
  {
  LLW::setWindowSize(_window,width,_dy1);
  }
else
  {
  LLW::setWindowSize(_window,width,height);
  }
}

void
Viewer3D::getWindowSize(unsigned int & widthOut,
                        unsigned int & heightOut) const
{
// _window may be shaded --> use _renderArea
LLW::getWindowSize(_renderArea,widthOut,heightOut);
if(_decoration)
  {
  _toOuterSize(widthOut,heightOut);
  }
}

void
Viewer3D::setRenderSize(unsigned int renderWidth,
                        unsigned int renderHeight)
{
_max=false;
LLW::setWindowSize(_renderArea,renderWidth,renderHeight);
LLW::getWindowSize(_renderArea,renderWidth,renderHeight); // min size ?
if(_decoration)
  {
  _toOuterSize(renderWidth,renderHeight);
  }
if(_shaded)
  {
  LLW::setWindowSize(_window,renderWidth,_dy1);
  }
else
  {
  LLW::setWindowSize(_window,renderWidth,renderHeight);
  }
}

//-------- Window ID ---------------------------------------------------------

int // platform-specific window ID
Viewer3D::getWindowID(void) const
{
return(LLW::getSpecificWindowID(_window));
}

//-------- Forbidden operations ----------------------------------------------

void
Viewer3D::_toInnerSize(unsigned int & widthInOut,
                       unsigned int & heightInOut) const // Protected
{
unsigned int dx=_dx1+_dx2;
unsigned int dy=_dy1+_dy2;
widthInOut=(widthInOut>dx ? widthInOut-dx : 0); // avoid negative width
heightInOut=(heightInOut>dy ? heightInOut-dy : 0); // avoid negative height
}

void
Viewer3D::_toOuterSize(unsigned int & widthInOut,
                       unsigned int & heightInOut) const // Protected
{
widthInOut+=_dx1+_dx2;
heightInOut+=_dy1+_dy2;
}

void
Viewer3D::_selectColors(bool light) // Protected
{
if(light)
  {
  _blackColor=LLW::makeByteColor(0,0,0);
  _darkColor=LLW::makeByteColor(80,80,80);
  _backColor=LLW::makeByteColor(160,160,160);
  _lightColor=LLW::makeByteColor(186,186,186);
  _whiteColor=LLW::makeByteColor(224,224,224);
  }
else
  {
  _blackColor=LLW::makeByteColor(0,0,0);
  _darkColor=LLW::makeByteColor(64,64,64);
  _backColor=LLW::makeByteColor(128,128,128);
  _lightColor=LLW::makeByteColor(150,150,150);
  _whiteColor=LLW::makeByteColor(180,180,180);
  }
}

void
Viewer3D::_drawBorderUp(int x,
                        int y,
                        unsigned int w,
                        unsigned int h) // Protected
{
LLW::drawLine(_pix,_whiteColor, x     , y     , x+w-1 , y     );
LLW::drawLine(_pix,_whiteColor, x     , y     , x     , y+h-1 );
LLW::drawLine(_pix,_lightColor, x+1   , y+1   , x+w-2 , y+1   );
LLW::drawLine(_pix,_lightColor, x+1   , y+1   , x+1   , y+h-2 );
LLW::drawLine(_pix,_blackColor, x+1   , y+h-1 , x+w-1 , y+h-1 );
LLW::drawLine(_pix,_blackColor, x+w-1 , y+1   , x+w-1 , y+h-1 );
LLW::drawLine(_pix,_darkColor,  x+2   , y+h-2 , x+w-2 , y+h-2 );
LLW::drawLine(_pix,_darkColor,  x+w-2 , y+2   , x+w-2 , y+h-2 );
}

void
Viewer3D::_drawBorderDown(int x,
                          int y,
                          unsigned int w,
                          unsigned int h) // Protected
{
LLW::drawLine(_pix,_darkColor,  x     , y     , x+w-1 , y     );
LLW::drawLine(_pix,_darkColor,  x     , y     , x     , y+h-1 );
LLW::drawLine(_pix,_blackColor, x+1   , y+1   , x+w-2 , y+1   );
LLW::drawLine(_pix,_blackColor, x+1   , y+1   , x+1   , y+h-2 );
LLW::drawLine(_pix,_lightColor, x+1   , y+h-1 , x+w-1 , y+h-1 );
LLW::drawLine(_pix,_lightColor, x+w-1 , y+1   , x+w-1 , y+h-1 );
LLW::drawLine(_pix,_whiteColor, x+2   , y+h-2 , x+w-2 , y+h-2 );
LLW::drawLine(_pix,_whiteColor, x+w-2 , y+2   , x+w-2 , y+h-2 );
}

void
Viewer3D::_drawLineUp(int x,
                      int y,
                      unsigned int w,
                      unsigned int h) // Protected
{
LLW::drawRectangle(_pix,_whiteColor, x   , y   , w-1 , h-1 );
LLW::drawRectangle(_pix,_blackColor, x+1 , y+1 , w-1 , h-1 );
LLW::drawPoint(_pix,_darkColor , x     , y+h-1 );
LLW::drawPoint(_pix,_darkColor , x+w-1 , y     );
LLW::drawPoint(_pix,_lightColor, x+1   , y+h-2 );
LLW::drawPoint(_pix,_lightColor, x+w-2 , y+1   );
}

void
Viewer3D::_drawLineDown(int x,
                        int y,
                        unsigned int w,
                        unsigned int h) // Protected
{
LLW::drawRectangle(_pix,_blackColor, x   , y   , w-1 , h-1 );
LLW::drawRectangle(_pix,_whiteColor, x+1 , y+1 , w-1 , h-1 );
LLW::drawPoint(_pix,_lightColor, x     , y+h-1 );
LLW::drawPoint(_pix,_lightColor, x+w-1 , y     );
LLW::drawPoint(_pix,_darkColor , x+1   , y+h-2 );
LLW::drawPoint(_pix,_darkColor , x+w-2 , y+1   );
}

void
Viewer3D::_drawHLineUp(int x,
                       int y,
                       unsigned int w) // Protected
{
LLW::drawLine(_pix,_whiteColor, x , y   , x+w-1 , y   );
LLW::drawLine(_pix,_blackColor, x , y+1 , x+w-1 , y+1 );
}

void
Viewer3D::_drawHLineDown(int x,
                         int y,
                         unsigned int w) // Protected
{
LLW::drawLine(_pix,_blackColor, x , y   , x+w-1 , y   );
LLW::drawLine(_pix,_whiteColor, x , y+1 , x+w-1 , y+1 );
}

void
Viewer3D::_drawVLineUp(int x,
                       int y,
                       unsigned int h) // Protected
{
LLW::drawLine(_pix,_whiteColor, x   , y , x   , y+h-1 );
LLW::drawLine(_pix,_blackColor, x+1 , y , x+1 , y+h-1 );
}

void
Viewer3D::_drawVLineDown(int x,
                         int y,
                         unsigned int h) // Protected
{
LLW::drawLine(_pix,_blackColor, x   , y , x   , y+h-1 );
LLW::drawLine(_pix,_whiteColor, x+1 , y , x+1 , y+h-1 );
}

void
Viewer3D::_drawTitleBar(int x,
                        int y,
                        unsigned int w,
                        unsigned int h) // Protected
{
unsigned int ascent,descent;
LLW::getFontHeight(_font,ascent,descent);
int textY=(h-ascent-descent)/2+ascent;
LLW::drawText(_pix,_font,_blackColor, x+4 , y+textY ,
              LLW::getTopWindowTitle(_window));
LLW::drawRectangle(_pix,_backColor, x+w-4 , y+2 , 2 , h-4 );
_drawLineDown(x,y,w,h);
}

void
Viewer3D::_drawCloseButton(int x,
                           int y) // Protected
{
_drawLineDown( x , y , 22 , 22 );
LLW::drawLine(_pix,_blackColor, x+4 , y+5  , x+15 , y+16 );
LLW::drawLine(_pix,_blackColor, x+5 , y+5  , x+16 , y+16 );
LLW::drawLine(_pix,_blackColor, x+4 , y+16 , x+15 , y+5  );
LLW::drawLine(_pix,_blackColor, x+5 , y+16 , x+16 , y+5  );
}

void
Viewer3D::_drawMaxButton(int x,
                         int y,
                         bool pressed) // Protected
{
_drawLineDown( x , y , 22 , 22 );
if(pressed)
  {
  LLW::drawLine(_pix,_blackColor, x+8 , y+8  , x+12 , y+8  );
  LLW::drawLine(_pix,_blackColor, x+8 , y+13 , x+12 , y+13 );
  LLW::drawRectangle(_pix,_blackColor, x+7 , y+7 , 7 , 8 );
  }
else
  {
  LLW::drawLine(_pix,_blackColor, x+5 , y+5  , x+15 , y+5  );
  LLW::drawLine(_pix,_blackColor, x+5 , y+16 , x+15 , y+16 );
  LLW::drawRectangle(_pix,_blackColor, x+4 , y+4 , 13 , 14 );
  }
}

void
Viewer3D::_drawMinButton(int x,
                         int y) // Protected
{
_drawLineDown( x , y , 22 , 22 );
LLW::drawLine(_pix,_blackColor, x+5 , y+16 , x+15 , y+16 );
LLW::drawLine(_pix,_blackColor, x+5 , y+17 , x+15 , y+17 );
}

void
Viewer3D::_redrawDecoration(void) // Protected
{
unsigned int w,h;
LLW::getWindowSize(_window,w,h);
if(_pix)
  {
  unsigned int pw,ph;
  LLW::getPixmapSize(_pix,pw,ph);
  if((pw!=w)||(ph!=h))
    {
    LLW::destroyPixmap(_pix);
    _pix=(LLW::PixID)0;
    }
  }
if(!_pix)
  {
  _pix=LLW::createPixmap(w,h);
  }
_selectColors(_focus||_clickPart);
LLW::fillRectangle(_pix,_backColor, 0 , 0 , w , h );
_drawTitleBar( 28 , 4 , w-80 , 22 );
_drawCloseButton( 4 , 4 );
LLW::fillRectangle(_pix,_backColor, w-52 , 4 , 50 , 22 );
_drawMaxButton( w-26 , 4 , _max );
_drawMinButton( w-50 , 4 );
_drawBorderUp( 0 , 0 , w , h );
if(!_shaded)
  {
  _drawBorderDown( 4 , 28 , w-8 , h-32 );
  }
LLW::setWindowBackgroundPixmap(_window,_pix);
}

Viewer3D::WindowPart
Viewer3D::_findWindowPart(LLW::Modifier modifiers,
                          int x,
                          int y) const // Protected
{
unsigned int w,h;
LLW::getWindowSize(_window,w,h);
if((x>=0)&&(x<(int)w)&&(y>=0)&&(y<(int)h))
  {
  if(modifiers&(LLW::MDF_MIDDLE|LLW::MDF_RIGHT))
    {
    return(Viewer3D::PART_BAR);
    }
  if(y<6)
    {
    if(_shaded)
      {
      return(Viewer3D::PART_BAR);
      }
    if(x<24)
      {
      return(Viewer3D::PART_NW);
      }
    if(x>=(int)w-24)
      {
      return(Viewer3D::PART_NE);
      }
    return(Viewer3D::PART_N);
    }
  else if(y>=(int)h-6)
    {
    if(_shaded)
      {
      return(Viewer3D::PART_BAR);
      }
    if(x<24)
      {
      return(Viewer3D::PART_SW);
      }
    if(x>=(int)w-24)
      {
      return(Viewer3D::PART_SE);
      }
    return(Viewer3D::PART_S);
    }
  else
    {
    if(x<6)
      {
      if(_shaded)
        {
        return(Viewer3D::PART_BAR);
        }
      if(y<24)
        {
        return(Viewer3D::PART_NW);
        }
      if(y>=(int)h-24)
        {
        return(Viewer3D::PART_SW);
        }
      return(Viewer3D::PART_W);
      }
    else if(x>=(int)w-6)
      {
      if(_shaded)
        {
        return(Viewer3D::PART_BAR);
        }
      if(y<24)
        {
        return(Viewer3D::PART_NE);
        }
      if(y>=(int)h-24)
        {
        return(Viewer3D::PART_SE);
        }
      return(Viewer3D::PART_E);
      }
    else
      {
      if(y<30)
        {
        if(y<24)
          {
          if(x<24)
            {
            return(Viewer3D::PART_CLOSE);
            }
          if((x>=(int)w-46)&&(x<(int)w-30))
            {
            return(Viewer3D::PART_MIN);
            }
          if(x>=(int)w-24)
            {
            return(Viewer3D::PART_MAX);
            }
          return(Viewer3D::PART_BAR);
          }
        else return(Viewer3D::PART_BAR);
        }
      else return(Viewer3D::PART_INNER);
      }
    }
  }
return(Viewer3D::PART_NONE);
}

void
Viewer3D::_chooseCursor(LLW::Modifier modifiers,
                        Viewer3D::WindowPart part) // Protected
{
LLW::CursorID cursor=LLW::CSR_ARROW;
switch(part)
  {
  case Viewer3D::PART_CLOSE:
  case Viewer3D::PART_MIN:
    {
    cursor=LLW::CSR_HAND;
    break;
    }
  case Viewer3D::PART_MAX:
    {
    cursor=(_shaded ? LLW::CSR_NO : LLW::CSR_HAND);
    break;
    }
  case Viewer3D::PART_BAR:
    {
    if(modifiers&(LLW::MDF_LEFT|LLW::MDF_MIDDLE|LLW::MDF_RIGHT))
      {
      cursor=LLW::CSR_MOVE;
      }
    break;
    }
  case Viewer3D::PART_N:
    {
    cursor=LLW::CSR_NORTH;
    break;
    }
  case Viewer3D::PART_S:
    {
    cursor=LLW::CSR_SOUTH;
    break;
    }
  case Viewer3D::PART_E:
    {
    cursor=LLW::CSR_EAST;
    break;
    }
  case Viewer3D::PART_W:
    {
    cursor=LLW::CSR_WEST;
    break;
    }
  case Viewer3D::PART_NW:
    {
    cursor=LLW::CSR_NORTHWEST;
    break;
    }
  case Viewer3D::PART_NE:
    {
    cursor=LLW::CSR_NORTHEAST;
    break;
    }
  case Viewer3D::PART_SW:
    {
    cursor=LLW::CSR_SOUTHWEST;
    break;
    }
  case Viewer3D::PART_SE:
    {
    cursor=LLW::CSR_SOUTHEAST;
    break;
    }
  default:
    break;
  }
LLW::setWindowCursor(_window,cursor);
}

void
Viewer3D::_manageClick(int x,
                       int y) // Protected
{
LLW::getWindowGeometry(_window,_clickX,_clickY,_clickWidth,_clickHeight);
switch(_clickPart)
  {
  case Viewer3D::PART_BAR:
    {
    _clickDx=x;
    _clickDy=y;
    break;
    }
  case Viewer3D::PART_NW:
    {
    _clickDx=x;
    _clickDy=y;
    break;
    }
  case Viewer3D::PART_NE:
    {
    _clickDx=_clickWidth-x;
    _clickDy=y;
    break;
    }
  case Viewer3D::PART_SW:
    {
    _clickDx=x;
    _clickDy=_clickHeight-y;
    break;
    }
  case Viewer3D::PART_SE:
    {
    _clickDx=_clickWidth-x;
    _clickDy=_clickHeight-y;
    break;
    }
  case Viewer3D::PART_N:
    {
    _clickDx=x;
    _clickDy=y;
    break;
    }
  case Viewer3D::PART_S:
    {
    _clickDx=x;
    _clickDy=_clickHeight-y;
    break;
    }
  case Viewer3D::PART_W:
    {
    _clickDx=x;
    _clickDy=y;
    break;
    }
  case Viewer3D::PART_E:
    {
    _clickDx=_clickWidth-x;
    _clickDy=y;
    break;
    }
  default:
    break;
  }
}

void
Viewer3D::_manageMotion(int xScreen,
                        int yScreen) // Protected
{
switch(_clickPart)
  {
  case Viewer3D::PART_BAR:
    {
    setWindowPosition(xScreen-_clickDx,yScreen-_clickDy);
    break;
    }
  case Viewer3D::PART_NW:
    {
    unsigned int minW,minH;
    LLW::getWindowMinSize(_window,minW,minH);
    int newW=_clickWidth-(xScreen-(_clickX+_clickDx));
    if(newW<(int)minW)
      {
      newW=minW;
      }
    int newH=_clickHeight-(yScreen-(_clickY+_clickDy));
    if(newH<(int)minH)
      {
      newH=minH;
      }
    int newX=_clickX+(_clickWidth-newW);
    int newY=_clickY+(_clickHeight-newH);
    setWindowGeometry(newX,newY,newW,newH);
    break;
    }
  case Viewer3D::PART_NE:
    {
    unsigned int minW,minH;
    LLW::getWindowMinSize(_window,minW,minH);
    int newH=_clickHeight-(yScreen-(_clickY+_clickDy));
    if(newH<(int)minH)
      {
      newH=minH;
      }
    int newY=_clickY+(_clickHeight-newH);
    int newW=(xScreen-_clickX)+_clickDx;
    if(newW<(int)minW)
      {
      newW=minW;
      }
    setWindowGeometry(_clickX,newY,newW,newH);
    break;
    }
  case Viewer3D::PART_SW:
    {
    unsigned int minW,minH;
    LLW::getWindowMinSize(_window,minW,minH);
    int newW=_clickWidth-(xScreen-(_clickX+_clickDx));
    if(newW<(int)minW)
      {
      newW=minW;
      }
    int newX=_clickX+(_clickWidth-newW);
    int newH=(yScreen-_clickY)+_clickDy;
    if(newH<(int)minH)
      {
      newH=minH;
      }
    setWindowGeometry(newX,_clickY,newW,newH);
    break;
    }
  case Viewer3D::PART_SE:
    {
    int newW=(xScreen-_clickX)+_clickDx;
    int newH=(yScreen-_clickY)+_clickDy;
    if(newW<0)
      {
      newW=0;
      }
    if(newH<0)
      {
      newH=0;
      }
    setWindowSize(newW,newH);
    break;
    }
  case Viewer3D::PART_N:
    {
    unsigned int minW,minH;
    LLW::getWindowMinSize(_window,minW,minH);
    int newH=_clickHeight-(yScreen-(_clickY+_clickDy));
    if(newH<(int)minH)
      {
      newH=minH;
      }
    int newY=_clickY+(_clickHeight-newH);
    setWindowGeometry(_clickX,newY,_clickWidth,newH);
    break;
    }
  case Viewer3D::PART_S:
    {
    int newH=(yScreen-_clickY)+_clickDy;
    if(newH<0)
      {
      newH=0;
      }
    setWindowSize(_clickWidth,newH);
    break;
    }
  case Viewer3D::PART_W:
    {
    unsigned int minW,minH;
    LLW::getWindowMinSize(_window,minW,minH);
    int newW=_clickWidth-(xScreen-(_clickX+_clickDx));
    if(newW<(int)minW)
      {
      newW=minW;
      }
    int newX=_clickX+(_clickWidth-newW);
    setWindowGeometry(newX,_clickY,newW,_clickHeight);
    break;
    }
  case Viewer3D::PART_E:
    {
    int newW=(xScreen-_clickX)+_clickDx;
    if(newW<0)
      {
      newW=0;
      }
    setWindowSize(newW,_clickHeight);
    break;
    }
  default:
    break;
  }
}

void
Viewer3D::_shade(void) // Protected
{
unsigned int minW,minH;
LLW::getWindowMinSize(_window,minW,minH);
minH=_dy1;
LLW::setWindowMinSize(_window,minW,minH);
unsigned int w,h;
LLW::getWindowSize(_window,w,h);
LLW::setWindowSize(_window,w,minH);
LLW::unmapWindow(_renderArea);
_shaded=true;
}

void
Viewer3D::_unshade(void) // Protected
{
LLW::mapWindow(_renderArea);
unsigned int w,h;
LLW::getWindowSize(_renderArea,w,h);
_toOuterSize(w,h);
LLW::setWindowSize(_window,w,h);
unsigned int minW,minH;
LLW::getWindowMinSize(_window,minW,minH);
minH=minW;
LLW::setWindowMinSize(_window,minW,minH);
_shaded=false;
}

bool // event processed
Viewer3D::_processEvent(const LLW::Event * evt) // Protected
{
if(evt->area==_window)
  {
  switch(evt->type)
    {
    case LLW::EVT_GEOMETRY:
      {
      unsigned int w,h;
      LLW::getWindowSize(_window,w,h);
      if(_decoration)
        {
        if(_shaded)
          {
          unsigned int inW,inH;
          LLW::getWindowSize(_renderArea,inW,inH);
          if(((int)h!=_dy1)||     // External change !!! (window manager ?)
             (w!=inW+_dx1+_dx2))
            {
            LLW::setWindowSize(_window,w,_dy1);
            LLW::setWindowSize(_renderArea,w-_dx1-_dx2,inH);
            }
          }
        else
          {
          _toInnerSize(w,h);
          LLW::setWindowSize(_renderArea,w,h);
          }
        _redrawDecoration();
        }
      else
        {
        LLW::setWindowSize(_renderArea,w,h);
        }
      if(!_geometryCB.empty())
        {
        Window3D::GeometryEvent event;
        _geometryCB.fire(event);
        }
      break;
      }
    case LLW::EVT_MOUSE_BUTTON:
      {
      if(_decoration)
        {
        Viewer3D::WindowPart part=_findWindowPart(evt->modifiers,
                                                  evt->x,evt->y);
        _chooseCursor(evt->modifiers,part);
        if((part==Viewer3D::PART_MAX)&&_shaded)
          {
          part=Viewer3D::PART_NONE;
          }
        switch(evt->button)
          {
          case LLW::BTN_LEFT:
            {
            if(evt->onOff)
              {
              if((part==Viewer3D::PART_BAR)&&
                 (evt->time-_clickTime<=_clickDelay))
                {
                if(_shaded)
                  {
                  _unshade();
                  }
                else
                  {
                  _shade();
                  }
                }
              else
                {
                _clickTime=evt->time;
                }
              _clickPart=part;
              _manageClick(evt->x,evt->y);
              }
            else
              {
              bool perform=(_clickPart==part);
              _clickPart=Viewer3D::PART_NONE;
              if(perform)
                {
                switch(part)
                  {
                  case Viewer3D::PART_CLOSE:
                    {
                    _manageClose();
                    break;
                    }
                  case Viewer3D::PART_MIN:
                    {
                    LLW::iconifyTopWindow(_window);
                    break;
                    }
                  case Viewer3D::PART_MAX:
                    {
                    if(_max)
                      {
                      LLW::setWindowGeometry(_window,
                                             _saveX,_saveY,
                                             _saveWidth,_saveHeight);
                      _max=false;
                      }
                    else
                      {
                      LLW::getWindowGeometry(_window,
                                             _saveX,_saveY,
                                             _saveWidth,_saveHeight);
                      unsigned int w,h;
                      LLW::getScreenSize(w,h);
                      LLW::setWindowGeometry(_window,0,0,w,h);
                      _max=true;
                      }
                    break;
                    }
                  default:
                    {
                    if(evt->time-_clickTime<=_clickDelay)
                      {
                      LLW::mapWindow(_window);
                      }
                    break;
                    }
                  }
                }
              }
            break;
            }
          case LLW::BTN_MIDDLE:
          case LLW::BTN_RIGHT:
            {
            if(evt->onOff)
              {
              _clickPart=part;
              _manageClick(evt->x,evt->y);
              }
            else
              {
              _clickPart=Viewer3D::PART_NONE;
              }
            break;
            }
          default:
            break;
          }
        }
      else
        {
        _clickPart=Viewer3D::PART_NONE;
        }
      break;
      }
    case LLW::EVT_MOUSE_MOTION:
      {
      if(_decoration)
        {
        if(_clickPart)
          {
          _manageMotion(evt->xScreen,evt->yScreen);
          }
        else
          {
          _chooseCursor(evt->modifiers,
                        _findWindowPart(evt->modifiers,evt->x,evt->y));
          }
        }
      break;
      }
    case LLW::EVT_FOCUS:
      {
      _focus=evt->onOff;
      if(_decoration)
        {
        if(!_clickPart)
          {
          _chooseCursor(evt->modifiers,
                        _findWindowPart(evt->modifiers,evt->x,evt->y));
          }
        _redrawDecoration();
        }
      if(!_focusCB.empty())
        {
        Viewer3D::FocusEvent event;
        event.hasFocus=_focus;
        _focusCB.fire(event);
        }
      break;
      }
    case LLW::EVT_CLOSE:
      {
      _manageClose();
      break;
      }
    default:
      break;
    }
  return(true);
  }
else
  {
  return(Window3D::_processEvent(evt));
  }
}

} // namespace AReVi

//----------------------------------------------------------------------------
