// LI2/ENIB - AReVi - 3D shapes
//----------------------------------------------------------------------------
// file        : AReVi/Shapes/box3D.cpp
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/Shapes/box3D.h"

#include "AReVi/Utils/abstractStream.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class Box3D
//----------------------------------------------------------------------------

AR_CLASS_DEF(Box3D,Surface3D)

//-------- Construction / Destruction ----------------------------------------

Box3D::Box3D(ArCW & arCW) // Protected
: Surface3D(arCW),
  _sx(1.0),
  _sy(1.0),
  _sz(1.0),
  _ccw(true)
{
_normals.push_back(Util3D::Dbl3(+0.0,-1.0,+0.0));
_normals.push_back(Util3D::Dbl3(+1.0,+0.0,+0.0));
_normals.push_back(Util3D::Dbl3(+0.0,+1.0,+0.0));
_normals.push_back(Util3D::Dbl3(-1.0,+0.0,+0.0));
_normals.push_back(Util3D::Dbl3(+0.0,+0.0,+1.0));
_normals.push_back(Util3D::Dbl3(+0.0,+0.0,-1.0));
_textures.push_back(Util3D::Dbl2(0.0,0.0));
_textures.push_back(Util3D::Dbl2(1.0,0.0));
_textures.push_back(Util3D::Dbl2(1.0,1.0));
_textures.push_back(Util3D::Dbl2(0.0,1.0));
}

Box3D::~Box3D(void) // Protected
{
  // Nothing to be done
}

//-------- Geometry ----------------------------------------------------------

bool // success
Box3D::setSize(double sizeX,
               double sizeY,
               double sizeZ)
{
if((sizeX>0.0)&&(sizeY>0.0)&&(sizeZ>0.0))
  {
  _sx=0.5*sizeX;
  _sy=0.5*sizeY;
  _sz=0.5*sizeZ;
  _postGeometryRebuild();
  return(true);
  }
else
  {
  return(false);
  }
}

void
Box3D::getSize(double & sizeXOut,
               double & sizeYOut,
               double & sizeZOut) const
{
sizeXOut=2.0*_sx;
sizeYOut=2.0*_sy;
sizeZOut=2.0*_sz;
}

void
Box3D::setCCW(bool ccw)
{
_ccw=ccw;
_postGeometryRebuild();
}

bool // counter clock wise surface
Box3D::getCCW(void) const
{
return(_ccw);
}

//-------- Serialization operations ------------------------------------------

void
Box3D::serialize(SerializationDependencies & dependenciesOut,
                 ArRef<AbstractOStream> output) const
{
Surface3D::serialize(dependenciesOut,output);
output->writeDouble(_sx);
output->writeDouble(_sy);
output->writeDouble(_sz);
unsigned int flags=(_ccw ? (1<<0) : 0);
output->writeInt(flags);
}

bool // success
Box3D::unserialize(const SerializationDependencies & dependencies,
                   ArRef<AbstractIStream> input)
{
double sx,sy,sz;
unsigned int flags;
if(!Surface3D::unserialize(dependencies,input)||
   !input->readDouble(sx)||
   !input->readDouble(sy)||
   !input->readDouble(sz)||
   !input->readUInt(flags))
  {
  return(false);
  }
_sx=sx;
_sy=sy;
_sz=sz;
_ccw=(flags&(1<<0));
_postGeometryRebuild();
return(true);
}

//-------- Forbidden operations ----------------------------------------------

void
Box3D::_rebuildIfNeeded(void) // Protected
{
if(_needsGeometryRebuild)
  {
  _vertices.clear();
  _vertices.push_back(Util3D::Dbl3(-_sx,-_sy,-_sz));
  _vertices.push_back(Util3D::Dbl3(+_sx,-_sy,-_sz));
  _vertices.push_back(Util3D::Dbl3(+_sx,+_sy,-_sz));
  _vertices.push_back(Util3D::Dbl3(-_sx,+_sy,-_sz));
  _vertices.push_back(Util3D::Dbl3(-_sx,-_sy,+_sz));
  _vertices.push_back(Util3D::Dbl3(+_sx,-_sy,+_sz));
  _vertices.push_back(Util3D::Dbl3(+_sx,+_sy,+_sz));
  _vertices.push_back(Util3D::Dbl3(-_sx,+_sy,+_sz));
  _facesV.clear();
  _facesN.clear();
  _facesT.clear();
  if(_ccw)
    {
    _facesV.push_back(Util3D::Face(0,1,5)); // Y-
    _facesN.push_back(Util3D::Face(0,0,0));
    _facesT.push_back(Util3D::Face(0,1,2));
    _facesV.push_back(Util3D::Face(0,5,4));
    _facesN.push_back(Util3D::Face(0,0,0));
    _facesT.push_back(Util3D::Face(0,2,3));
    _facesV.push_back(Util3D::Face(1,2,6)); // X+
    _facesN.push_back(Util3D::Face(1,1,1));
    _facesT.push_back(Util3D::Face(0,1,2));
    _facesV.push_back(Util3D::Face(1,6,5));
    _facesN.push_back(Util3D::Face(1,1,1));
    _facesT.push_back(Util3D::Face(0,2,3));
    _facesV.push_back(Util3D::Face(2,3,7)); // Y+
    _facesN.push_back(Util3D::Face(2,2,2));
    _facesT.push_back(Util3D::Face(0,1,2));
    _facesV.push_back(Util3D::Face(2,7,6));
    _facesN.push_back(Util3D::Face(2,2,2));
    _facesT.push_back(Util3D::Face(0,2,3));
    _facesV.push_back(Util3D::Face(3,0,4)); // X-
    _facesN.push_back(Util3D::Face(3,3,3));
    _facesT.push_back(Util3D::Face(0,1,2));
    _facesV.push_back(Util3D::Face(3,4,7));
    _facesN.push_back(Util3D::Face(3,3,3));
    _facesT.push_back(Util3D::Face(0,2,3));
    _facesV.push_back(Util3D::Face(5,6,7)); // Z+
    _facesN.push_back(Util3D::Face(4,4,4));
    _facesT.push_back(Util3D::Face(0,1,2));
    _facesV.push_back(Util3D::Face(5,7,4));
    _facesN.push_back(Util3D::Face(4,4,4));
    _facesT.push_back(Util3D::Face(0,2,3));
    _facesV.push_back(Util3D::Face(0,3,2)); // Z-
    _facesN.push_back(Util3D::Face(5,5,5));
    _facesT.push_back(Util3D::Face(0,1,2));
    _facesV.push_back(Util3D::Face(0,2,1));
    _facesN.push_back(Util3D::Face(5,5,5));
    _facesT.push_back(Util3D::Face(0,2,3));
    }
  else
    {
    _facesV.push_back(Util3D::Face(0,5,1)); // Y-
    _facesN.push_back(Util3D::Face(2,2,2));
    _facesT.push_back(Util3D::Face(1,3,0));
    _facesV.push_back(Util3D::Face(0,4,5));
    _facesN.push_back(Util3D::Face(2,2,2));
    _facesT.push_back(Util3D::Face(1,2,3));
    _facesV.push_back(Util3D::Face(1,6,2)); // X+
    _facesN.push_back(Util3D::Face(3,3,3));
    _facesT.push_back(Util3D::Face(1,3,0));
    _facesV.push_back(Util3D::Face(1,5,6));
    _facesN.push_back(Util3D::Face(3,3,3));
    _facesT.push_back(Util3D::Face(1,2,3));
    _facesV.push_back(Util3D::Face(2,7,3)); // Y+
    _facesN.push_back(Util3D::Face(0,0,0));
    _facesT.push_back(Util3D::Face(1,3,0));
    _facesV.push_back(Util3D::Face(2,6,7));
    _facesN.push_back(Util3D::Face(0,0,0));
    _facesT.push_back(Util3D::Face(1,2,3));
    _facesV.push_back(Util3D::Face(3,4,0)); // X-
    _facesN.push_back(Util3D::Face(1,1,1));
    _facesT.push_back(Util3D::Face(1,3,0));
    _facesV.push_back(Util3D::Face(3,7,4));
    _facesN.push_back(Util3D::Face(1,1,1));
    _facesT.push_back(Util3D::Face(1,2,3));
    _facesV.push_back(Util3D::Face(7,5,4)); // Z+
    _facesN.push_back(Util3D::Face(5,5,5));
    _facesT.push_back(Util3D::Face(1,3,0));
    _facesV.push_back(Util3D::Face(7,6,5));
    _facesN.push_back(Util3D::Face(5,5,5));
    _facesT.push_back(Util3D::Face(1,2,3));
    _facesV.push_back(Util3D::Face(2,0,1)); // Z-
    _facesN.push_back(Util3D::Face(4,4,4));
    _facesT.push_back(Util3D::Face(1,3,0));
    _facesV.push_back(Util3D::Face(2,3,0));
    _facesN.push_back(Util3D::Face(4,4,4));
    _facesT.push_back(Util3D::Face(1,2,3));
    }
  }
Surface3D::_rebuildIfNeeded();
}

} // namespace AReVi

//----------------------------------------------------------------------------
