// LI2/ENIB - AReVi - 3D shapes
//----------------------------------------------------------------------------
// file        : AReVi/Shapes/cylinder3D.cpp
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/Shapes/cylinder3D.h"

#include "AReVi/Utils/abstractStream.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class Cylinder3D
//----------------------------------------------------------------------------

AR_CLASS_DEF(Cylinder3D,Surface3D)

//-------- Construction / Destruction ----------------------------------------

Cylinder3D::Cylinder3D(ArCW & arCW) // Protected
: Surface3D(arCW),
  _radius(1.0),
  _height(2.0),
  _side(true),
  _top(true),
  _bottom(true),
  _ccw(true),
  _precision(16)
{
  // Nothing to be done
}

Cylinder3D::~Cylinder3D(void) // Protected
{
  // Nothing to be done
}

//-------- Geometry ----------------------------------------------------------

bool // success
Cylinder3D::setRadius(double radius)
{
if(radius>0.0)
  {
  _radius=radius;
  _postGeometryRebuild();
  return(true);
  }
else
  {
  return(false);
  }
}

double // radius
Cylinder3D::getRadius(void) const
{
return(_radius);
}

bool // success
Cylinder3D::setHeight(double height)
{
if(height>0.0)
  {
  _height=height;
  _postGeometryRebuild();
  return(true);
  }
else
  {
  return(false);
  }
}

double // height
Cylinder3D::getHeight(void) const
{
return(_height);
}

void
Cylinder3D::setSide(bool buildSide)
{
_side=buildSide;
_postGeometryRebuild();
}

bool // side is built
Cylinder3D::getSide(void) const
{
return(_side);
}

void
Cylinder3D::setTop(bool buildTop)
{
_top=buildTop;
_postGeometryRebuild();
}

bool // top is built
Cylinder3D::getTop(void) const
{
return(_top);
}

void
Cylinder3D::setBottom(bool buildBottom)
{
_bottom=buildBottom;
_postGeometryRebuild();
}

bool // bottom is built
Cylinder3D::getBottom(void) const
{
return(_bottom);
}

void
Cylinder3D::setCCW(bool ccw)
{
_ccw=ccw;
_postGeometryRebuild();
}

bool // counter clock wise surface
Cylinder3D::getCCW(void) const
{
return(_ccw);
}

bool // success
Cylinder3D::setPrecision(unsigned int precision)
{
if(precision>0)
  {
  _precision=precision;
  _postGeometryRebuild();
  return(true);
  }
else
  {
  return(false);
  }
}

unsigned int // precision
Cylinder3D::getPrecision(void) const
{
return(_precision);
}

//-------- Serialization operations ------------------------------------------

void
Cylinder3D::serialize(SerializationDependencies & dependenciesOut,
                      ArRef<AbstractOStream> output) const
{
Surface3D::serialize(dependenciesOut,output);
output->writeDouble(_radius);
output->writeDouble(_height);
output->writeUInt(_precision);
unsigned int flags=(_side   ? 1<<0 : 0)+
                   (_top    ? 1<<1 : 0)+
                   (_bottom ? 1<<2 : 0)+
                   (_ccw    ? 1<<3 : 0);
output->writeInt(flags);
}

bool // success
Cylinder3D::unserialize(const SerializationDependencies & dependencies,
                        ArRef<AbstractIStream> input)
{
double radius,height;
unsigned int precision,flags;
if(!Surface3D::unserialize(dependencies,input)||
   !input->readDouble(radius)||
   !input->readDouble(height)||
   !input->readUInt(precision)||
   !input->readUInt(flags))
  {
  return(false);
  }
_radius=radius;
_height=height;
_precision=precision;
_side=flags&(1<<0);
_top=flags&(1<<1);
_bottom=flags&(1<<2);
_ccw=flags&(1<<3);
_postGeometryRebuild();
return(true);
}

//-------- Forbidden operations ----------------------------------------------

void
Cylinder3D::_rebuildIfNeeded(void) // Protected
{
if(_needsGeometryRebuild)
  {
  _vertices.clear();
  _facesV.clear();
  _normals.clear();
  _facesN.clear();
  _textures.clear();
  _facesT.clear();
  double height=0.5*_height;
  if(_side)
    {
    if(_bottom)
      {
      if(_top) // side, bottom and top
        {
        unsigned int i;
        for(i=0;i<_precision;i++)
          {
          double xt=(double)i/(double)_precision;
          double angle=M_2PI*(0.5+xt);
          double c=cos(angle);
          double s=sin(angle);
          double x=_radius*c;
          double y=_radius*s;
          if(_ccw)
            {
            _vertices.push_back(Util3D::Dbl3(x,y,height));
            _vertices.push_back(Util3D::Dbl3(x,y,-height));
            _normals.push_back(Util3D::Dbl3(x,y,0.0));
            _textures.push_back(Util3D::Dbl2(xt,1.0));
            _textures.push_back(Util3D::Dbl2(xt,0.0));
            _textures.push_back(Util3D::Dbl2(0.5*(1.0+s),0.5*(1.0-c)));
            _textures.push_back(Util3D::Dbl2(0.5*(1.0+s),0.5*(1.0+c)));
            }
          else
            {
            _vertices.push_back(Util3D::Dbl3(-x,-y,height));
            _vertices.push_back(Util3D::Dbl3(-x,-y,-height));
            _normals.push_back(Util3D::Dbl3(x,y,0.0));
            _textures.push_back(Util3D::Dbl2(1.0-xt,1.0));
            _textures.push_back(Util3D::Dbl2(1.0-xt,0.0));
            _textures.push_back(Util3D::Dbl2(0.5*(1.0-s),0.5*(1.0-c)));
            _textures.push_back(Util3D::Dbl2(0.5*(1.0-s),0.5*(1.0+c)));
            }
          }
        if(_ccw)
          {
          _textures.push_back(Util3D::Dbl2(1.0,1.0)); // side top end
          _textures.push_back(Util3D::Dbl2(1.0,0.0)); // side bottom end
          _textures.push_back(Util3D::Dbl2(0.5,1.0)); // top end
          _textures.push_back(Util3D::Dbl2(0.5,0.0)); // bottom end
          for(i=0;i<_precision;i++)
            {
            unsigned int next=(i+1)%_precision;
            _facesV.push_back(Util3D::Face(2*i,2*i+1,2*next));
            _facesN.push_back(Util3D::Face(i,i,next));
            _facesT.push_back(Util3D::Face(4*i,4*i+1,4*(i+1)));
            _facesV.push_back(Util3D::Face(2*i+1,2*next+1,2*next));
            _facesN.push_back(Util3D::Face(i,next,next));
            _facesT.push_back(Util3D::Face(4*i+1,4*(i+1)+1,4*(i+1)));
            }
          }
        else
          {
          _textures.push_back(Util3D::Dbl2(0.0,1.0)); // side top end
          _textures.push_back(Util3D::Dbl2(0.0,0.0)); // side bottom end
          _textures.push_back(Util3D::Dbl2(0.5,1.0)); // top end
          _textures.push_back(Util3D::Dbl2(0.5,0.0)); // bottom end
          for(i=0;i<_precision;i++)
            {
            unsigned int next=(i+1)%_precision;
            _facesV.push_back(Util3D::Face(2*i,2*next,2*i+1));
            _facesN.push_back(Util3D::Face(i,next,i));
            _facesT.push_back(Util3D::Face(4*i,4*(i+1),4*i+1));
            _facesV.push_back(Util3D::Face(2*i+1,2*next,2*next+1));
            _facesN.push_back(Util3D::Face(i,next,next));
            _facesT.push_back(Util3D::Face(4*i+1,4*(i+1),4*(i+1)+1));
            }
          }
        unsigned int center=_vertices.size();
        _vertices.push_back(Util3D::Dbl3(0.0,0.0,height));
        unsigned int centerT=_textures.size();
        _textures.push_back(Util3D::Dbl2(0.5,0.5));
        unsigned int normal=_normals.size();
        if(_ccw)
          {
          _normals.push_back(Util3D::Dbl3(0.0,0.0,1.0));
          for(i=0;i<_precision;i++)
            {
            unsigned int next=(i+1)%_precision;
            _facesV.push_back(Util3D::Face(center,2*i,2*next));
            _facesN.push_back(Util3D::Face(normal,normal,normal));
            _facesT.push_back(Util3D::Face(centerT,4*i+2,4*(i+1)+2));
            }
          }
        else
          {
          _normals.push_back(Util3D::Dbl3(0.0,0.0,-1.0));
          for(i=0;i<_precision;i++)
            {
            unsigned int next=(i+1)%_precision;
            _facesV.push_back(Util3D::Face(center,2*next,2*i));
            _facesN.push_back(Util3D::Face(normal,normal,normal));
            _facesT.push_back(Util3D::Face(centerT,4*(i+1)+2,4*i+2));
            }
          }
        center=_vertices.size();
        _vertices.push_back(Util3D::Dbl3(0.0,0.0,-height));
        normal=_normals.size();
        if(_ccw)
          {
          _normals.push_back(Util3D::Dbl3(0.0,0.0,-1.0));
          for(i=0;i<_precision;i++)
            {
            unsigned int next=(i+1)%_precision;
            _facesV.push_back(Util3D::Face(2*i+1,center,2*next+1));
            _facesN.push_back(Util3D::Face(normal,normal,normal));
            _facesT.push_back(Util3D::Face(4*i+3,centerT,4*(i+1)+3));
            }
          }
        else
          {
          _normals.push_back(Util3D::Dbl3(0.0,0.0,1.0));
          for(i=0;i<_precision;i++)
            {
            unsigned int next=(i+1)%_precision;
            _facesV.push_back(Util3D::Face(2*i+1,2*next+1,center));
            _facesN.push_back(Util3D::Face(normal,normal,normal));
            _facesT.push_back(Util3D::Face(4*i+3,4*(i+1)+3,centerT));
            }
          }
        }
      else // side and bottom
        {
        unsigned int i;
        for(i=0;i<_precision;i++)
          {
          double xt=(double)i/(double)_precision;
          double angle=M_2PI*(0.5+xt);
          double c=cos(angle);
          double s=sin(angle);
          double x=_radius*c;
          double y=_radius*s;
          if(_ccw)
            {
            _vertices.push_back(Util3D::Dbl3(x,y,height));
            _vertices.push_back(Util3D::Dbl3(x,y,-height));
            _normals.push_back(Util3D::Dbl3(x,y,0.0));
            _textures.push_back(Util3D::Dbl2(xt,1.0));
            _textures.push_back(Util3D::Dbl2(xt,0.0));
            _textures.push_back(Util3D::Dbl2(0.5*(1.0+s),0.5*(1.0+c)));
            }
          else
            {
            _vertices.push_back(Util3D::Dbl3(-x,-y,height));
            _vertices.push_back(Util3D::Dbl3(-x,-y,-height));
            _normals.push_back(Util3D::Dbl3(x,y,0.0));
            _textures.push_back(Util3D::Dbl2(1.0-xt,1.0));
            _textures.push_back(Util3D::Dbl2(1.0-xt,0.0));
            _textures.push_back(Util3D::Dbl2(0.5*(1.0-s),0.5*(1.0+c)));
            }
          }
        if(_ccw)
          {
          _textures.push_back(Util3D::Dbl2(1.0,1.0)); // side top end
          _textures.push_back(Util3D::Dbl2(1.0,0.0)); // side bottom end
          _textures.push_back(Util3D::Dbl2(0.5,0.0)); // bottom end
          for(i=0;i<_precision;i++)
            {
            unsigned int next=(i+1)%_precision;
            _facesV.push_back(Util3D::Face(2*i,2*i+1,2*next));
            _facesN.push_back(Util3D::Face(i,i,next));
            _facesT.push_back(Util3D::Face(3*i,3*i+1,3*(i+1)));
            _facesV.push_back(Util3D::Face(2*i+1,2*next+1,2*next));
            _facesN.push_back(Util3D::Face(i,next,next));
            _facesT.push_back(Util3D::Face(3*i+1,3*(i+1)+1,3*(i+1)));
            }
          }
        else
          {
          _textures.push_back(Util3D::Dbl2(0.0,1.0)); // side top end
          _textures.push_back(Util3D::Dbl2(0.0,0.0)); // side bottom end
          _textures.push_back(Util3D::Dbl2(0.5,0.0)); // bottom end
          for(i=0;i<_precision;i++)
            {
            unsigned int next=(i+1)%_precision;
            _facesV.push_back(Util3D::Face(2*i,2*next,2*i+1));
            _facesN.push_back(Util3D::Face(i,next,i));
            _facesT.push_back(Util3D::Face(3*i,3*(i+1),3*i+1));
            _facesV.push_back(Util3D::Face(2*i+1,2*next,2*next+1));
            _facesN.push_back(Util3D::Face(i,next,next));
            _facesT.push_back(Util3D::Face(3*i+1,3*(i+1),3*(i+1)+1));
            }
          }
        unsigned int center=_vertices.size();
        _vertices.push_back(Util3D::Dbl3(0.0,0.0,-height));
        unsigned int centerT=_textures.size();
        _textures.push_back(Util3D::Dbl2(0.5,0.5));
        unsigned int normal=_normals.size();
        if(_ccw)
          {
          _normals.push_back(Util3D::Dbl3(0.0,0.0,-1.0));
          for(i=0;i<_precision;i++)
            {
            unsigned int next=(i+1)%_precision;
            _facesV.push_back(Util3D::Face(2*i+1,center,2*next+1));
            _facesN.push_back(Util3D::Face(normal,normal,normal));
            _facesT.push_back(Util3D::Face(3*i+2,centerT,3*(i+1)+2));
            }
          }
        else
          {
          _normals.push_back(Util3D::Dbl3(0.0,0.0,1.0));
          for(i=0;i<_precision;i++)
            {
            unsigned int next=(i+1)%_precision;
            _facesV.push_back(Util3D::Face(2*i+1,2*next+1,center));
            _facesN.push_back(Util3D::Face(normal,normal,normal));
            _facesT.push_back(Util3D::Face(3*i+2,3*(i+1)+2,centerT));
            }
          }
        }
      }
    else
      {
      if(_top) // side and top
        {
        unsigned int i;
        for(i=0;i<_precision;i++)
          {
          double xt=(double)i/(double)_precision;
          double angle=M_2PI*(0.5+xt);
          double c=cos(angle);
          double s=sin(angle);
          double x=_radius*c;
          double y=_radius*s;
          if(_ccw)
            {
            _vertices.push_back(Util3D::Dbl3(x,y,height));
            _vertices.push_back(Util3D::Dbl3(x,y,-height));
            _normals.push_back(Util3D::Dbl3(x,y,0.0));
            _textures.push_back(Util3D::Dbl2(xt,1.0));
            _textures.push_back(Util3D::Dbl2(xt,0.0));
            _textures.push_back(Util3D::Dbl2(0.5*(1.0+s),0.5*(1.0-c)));
            }
          else
            {
            _vertices.push_back(Util3D::Dbl3(-x,-y,height));
            _vertices.push_back(Util3D::Dbl3(-x,-y,-height));
            _normals.push_back(Util3D::Dbl3(x,y,0.0));
            _textures.push_back(Util3D::Dbl2(1.0-xt,1.0));
            _textures.push_back(Util3D::Dbl2(1.0-xt,0.0));
            _textures.push_back(Util3D::Dbl2(0.5*(1.0-s),0.5*(1.0-c)));
            }
          }
        if(_ccw)
          {
          _textures.push_back(Util3D::Dbl2(1.0,1.0)); // side top end
          _textures.push_back(Util3D::Dbl2(1.0,0.0)); // side bottom end
          _textures.push_back(Util3D::Dbl2(0.5,1.0)); // top end
          for(i=0;i<_precision;i++)
            {
            unsigned int next=(i+1)%_precision;
            _facesV.push_back(Util3D::Face(2*i,2*i+1,2*next));
            _facesN.push_back(Util3D::Face(i,i,next));
            _facesT.push_back(Util3D::Face(3*i,3*i+1,3*(i+1)));
            _facesV.push_back(Util3D::Face(2*i+1,2*next+1,2*next));
            _facesN.push_back(Util3D::Face(i,next,next));
            _facesT.push_back(Util3D::Face(3*i+1,3*(i+1)+1,3*(i+1)));
            }
          }
        else
          {
          _textures.push_back(Util3D::Dbl2(0.0,1.0)); // side top end
          _textures.push_back(Util3D::Dbl2(0.0,0.0)); // side bottom end
          _textures.push_back(Util3D::Dbl2(0.5,1.0)); // top end
          for(i=0;i<_precision;i++)
            {
            unsigned int next=(i+1)%_precision;
            _facesV.push_back(Util3D::Face(2*i,2*next,2*i+1));
            _facesN.push_back(Util3D::Face(i,next,i));
            _facesT.push_back(Util3D::Face(3*i,3*(i+1),3*i+1));
            _facesV.push_back(Util3D::Face(2*i+1,2*next,2*next+1));
            _facesN.push_back(Util3D::Face(i,next,next));
            _facesT.push_back(Util3D::Face(3*i+1,3*(i+1),3*(i+1)+1));
            }
          }
        unsigned int center=_vertices.size();
        _vertices.push_back(Util3D::Dbl3(0.0,0.0,height));
        unsigned int centerT=_textures.size();
        _textures.push_back(Util3D::Dbl2(0.5,0.5));
        unsigned int normal=_normals.size();
        if(_ccw)
          {
          _normals.push_back(Util3D::Dbl3(0.0,0.0,1.0));
          for(i=0;i<_precision;i++)
            {
            unsigned int next=(i+1)%_precision;
            _facesV.push_back(Util3D::Face(center,2*i,2*next));
            _facesN.push_back(Util3D::Face(normal,normal,normal));
            _facesT.push_back(Util3D::Face(centerT,3*i+2,3*(i+1)+2));
            }
          }
        else
          {
          _normals.push_back(Util3D::Dbl3(0.0,0.0,-1.0));
          for(i=0;i<_precision;i++)
            {
            unsigned int next=(i+1)%_precision;
            _facesV.push_back(Util3D::Face(center,2*next,2*i));
            _facesN.push_back(Util3D::Face(normal,normal,normal));
            _facesT.push_back(Util3D::Face(centerT,3*(i+1)+2,3*i+2));
            }
          }
        }
      else // side only
        {
        unsigned int i;
        for(i=0;i<_precision;i++)
          {
          double xt=(double)i/(double)_precision;
          double angle=M_2PI*(0.5+xt);
          double c=cos(angle);
          double s=sin(angle);
          double x=_radius*c;
          double y=_radius*s;
          if(_ccw)
            {
            _vertices.push_back(Util3D::Dbl3(x,y,height));
            _vertices.push_back(Util3D::Dbl3(x,y,-height));
            _normals.push_back(Util3D::Dbl3(x,y,0.0));
            _textures.push_back(Util3D::Dbl2(xt,1.0));
            _textures.push_back(Util3D::Dbl2(xt,0.0));
            }
          else
            {
            _vertices.push_back(Util3D::Dbl3(-x,-y,height));
            _vertices.push_back(Util3D::Dbl3(-x,-y,-height));
            _normals.push_back(Util3D::Dbl3(x,y,0.0));
            _textures.push_back(Util3D::Dbl2(1.0-xt,1.0));
            _textures.push_back(Util3D::Dbl2(1.0-xt,0.0));
            }
          }
        if(_ccw)
          {
          _textures.push_back(Util3D::Dbl2(1.0,1.0)); // side top end
          _textures.push_back(Util3D::Dbl2(1.0,0.0)); // side bottom end
          for(i=0;i<_precision;i++)
            {
            unsigned int next=(i+1)%_precision;
            _facesV.push_back(Util3D::Face(2*i,2*i+1,2*next));
            _facesN.push_back(Util3D::Face(i,i,next));
            _facesT.push_back(Util3D::Face(2*i,2*i+1,2*(i+1)));
            _facesV.push_back(Util3D::Face(2*i+1,2*next+1,2*next));
            _facesN.push_back(Util3D::Face(i,next,next));
            _facesT.push_back(Util3D::Face(2*i+1,2*(i+1)+1,2*(i+1)));
            }
          }
        else
          {
          _textures.push_back(Util3D::Dbl2(0.0,1.0)); // side top end
          _textures.push_back(Util3D::Dbl2(0.0,0.0)); // side bottom end
          for(i=0;i<_precision;i++)
            {
            unsigned int next=(i+1)%_precision;
            _facesV.push_back(Util3D::Face(2*i,2*next,2*i+1));
            _facesN.push_back(Util3D::Face(i,next,i));
            _facesT.push_back(Util3D::Face(2*i,2*(i+1),2*i+1));
            _facesV.push_back(Util3D::Face(2*i+1,2*next,2*next+1));
            _facesN.push_back(Util3D::Face(i,next,next));
            _facesT.push_back(Util3D::Face(2*i+1,2*(i+1),2*(i+1)+1));
            }
          }
        }
      }
    }
  else
    {
    if(_bottom)
      {
      if(_top) // bottom and top
        {
        unsigned int i;
        for(i=0;i<_precision;i++)
          {
          double xt=(double)i/(double)_precision;
          double angle=M_2PI*(0.5+xt);
          double c=cos(angle);
          double s=sin(angle);
          double x=_radius*c;
          double y=_radius*s;
          if(_ccw)
            {
            _vertices.push_back(Util3D::Dbl3(x,y,height));
            _vertices.push_back(Util3D::Dbl3(x,y,-height));
            _textures.push_back(Util3D::Dbl2(0.5*(1.0+s),0.5*(1.0-c)));
            _textures.push_back(Util3D::Dbl2(0.5*(1.0+s),0.5*(1.0+c)));
            }
          else
            {
            _vertices.push_back(Util3D::Dbl3(-x,-y,height));
            _vertices.push_back(Util3D::Dbl3(-x,-y,-height));
            _textures.push_back(Util3D::Dbl2(0.5*(1.0-s),0.5*(1.0-c)));
            _textures.push_back(Util3D::Dbl2(0.5*(1.0-s),0.5*(1.0+c)));
            }
          }
        _textures.push_back(Util3D::Dbl2(0.5,1.0)); // top end
        _textures.push_back(Util3D::Dbl2(0.5,0.0)); // bottom end
        unsigned int center=_vertices.size();
        _vertices.push_back(Util3D::Dbl3(0.0,0.0,height));
        unsigned int centerT=_textures.size();
        _textures.push_back(Util3D::Dbl2(0.5,0.5));
        unsigned int normal=_normals.size();
        if(_ccw)
          {
          _normals.push_back(Util3D::Dbl3(0.0,0.0,1.0));
          for(i=0;i<_precision;i++)
            {
            unsigned int next=(i+1)%_precision;
            _facesV.push_back(Util3D::Face(center,2*i,2*next));
            _facesN.push_back(Util3D::Face(normal,normal,normal));
            _facesT.push_back(Util3D::Face(centerT,2*i,2*(i+1)));
            }
          }
        else
          {
          _normals.push_back(Util3D::Dbl3(0.0,0.0,-1.0));
          for(i=0;i<_precision;i++)
            {
            unsigned int next=(i+1)%_precision;
            _facesV.push_back(Util3D::Face(center,2*next,2*i));
            _facesN.push_back(Util3D::Face(normal,normal,normal));
            _facesT.push_back(Util3D::Face(centerT,2*(i+1),2*i));
            }
          }
        center=_vertices.size();
        _vertices.push_back(Util3D::Dbl3(0.0,0.0,-height));
        normal=_normals.size();
        if(_ccw)
          {
          _normals.push_back(Util3D::Dbl3(0.0,0.0,-1.0));
          for(i=0;i<_precision;i++)
            {
            unsigned int next=(i+1)%_precision;
            _facesV.push_back(Util3D::Face(2*i+1,center,2*next+1));
            _facesN.push_back(Util3D::Face(normal,normal,normal));
            _facesT.push_back(Util3D::Face(2*i+1,centerT,2*(i+1)+1));
            }
          }
        else
          {
          _normals.push_back(Util3D::Dbl3(0.0,0.0,1.0));
          for(i=0;i<_precision;i++)
            {
            unsigned int next=(i+1)%_precision;
            _facesV.push_back(Util3D::Face(2*i+1,2*next+1,center));
            _facesN.push_back(Util3D::Face(normal,normal,normal));
            _facesT.push_back(Util3D::Face(2*i+1,2*(i+1)+1,centerT));
            }
          }
        }
      else // bottom only
        {
        unsigned int i;
        for(i=0;i<_precision;i++)
          {
          double xt=(double)i/(double)_precision;
          double angle=M_2PI*(0.5+xt);
          double c=cos(angle);
          double s=sin(angle);
          double x=_radius*c;
          double y=_radius*s;
          if(_ccw)
            {
            _vertices.push_back(Util3D::Dbl3(x,y,-height));
            _textures.push_back(Util3D::Dbl2(0.5*(1.0+s),0.5*(1.0+c)));
            }
          else
            {
            _vertices.push_back(Util3D::Dbl3(-x,-y,-height));
            _textures.push_back(Util3D::Dbl2(0.5*(1.0-s),0.5*(1.0+c)));
            }
          }
        _textures.push_back(Util3D::Dbl2(0.5,0.0)); // bottom end
        unsigned int center=_vertices.size();
        _vertices.push_back(Util3D::Dbl3(0.0,0.0,-height));
        unsigned int centerT=_textures.size();
        _textures.push_back(Util3D::Dbl2(0.5,0.5));
        unsigned int normal=_normals.size();
        if(_ccw)
          {
          _normals.push_back(Util3D::Dbl3(0.0,0.0,-1.0));
          for(i=0;i<_precision;i++)
            {
            unsigned int next=(i+1)%_precision;
            _facesV.push_back(Util3D::Face(i,center,next));
            _facesN.push_back(Util3D::Face(normal,normal,normal));
            _facesT.push_back(Util3D::Face(i,centerT,(i+1)));
            }
          }
        else
          {
          _normals.push_back(Util3D::Dbl3(0.0,0.0,1.0));
          for(i=0;i<_precision;i++)
            {
            unsigned int next=(i+1)%_precision;
            _facesV.push_back(Util3D::Face(i,next,center));
            _facesN.push_back(Util3D::Face(normal,normal,normal));
            _facesT.push_back(Util3D::Face(i,(i+1),centerT));
            }
          }
        }
      }
    else
      {
      if(_top) // top only
        {
        unsigned int i;
        for(i=0;i<_precision;i++)
          {
          double xt=(double)i/(double)_precision;
          double angle=M_2PI*(0.5+xt);
          double c=cos(angle);
          double s=sin(angle);
          double x=_radius*c;
          double y=_radius*s;
          if(_ccw)
            {
            _vertices.push_back(Util3D::Dbl3(x,y,height));
            _textures.push_back(Util3D::Dbl2(0.5*(1.0+s),0.5*(1.0-c)));
            }
          else
            {
            _vertices.push_back(Util3D::Dbl3(-x,-y,height));
            _textures.push_back(Util3D::Dbl2(0.5*(1.0-s),0.5*(1.0-c)));
            }
          }
        _textures.push_back(Util3D::Dbl2(0.5,1.0)); // top end
        unsigned int center=_vertices.size();
        _vertices.push_back(Util3D::Dbl3(0.0,0.0,height));
        unsigned int centerT=_textures.size();
        _textures.push_back(Util3D::Dbl2(0.5,0.5));
        unsigned int normal=_normals.size();
        if(_ccw)
          {
          _normals.push_back(Util3D::Dbl3(0.0,0.0,1.0));
          for(i=0;i<_precision;i++)
            {
            unsigned int next=(i+1)%_precision;
            _facesV.push_back(Util3D::Face(center,i,next));
            _facesN.push_back(Util3D::Face(normal,normal,normal));
            _facesT.push_back(Util3D::Face(centerT,i,(i+1)));
            }
          }
        else
          {
          _normals.push_back(Util3D::Dbl3(0.0,0.0,-1.0));
          for(i=0;i<_precision;i++)
            {
            unsigned int next=(i+1)%_precision;
            _facesV.push_back(Util3D::Face(center,next,i));
            _facesN.push_back(Util3D::Face(normal,normal,normal));
            _facesT.push_back(Util3D::Face(centerT,(i+1),i));
            }
          }
        }
      }
    }
  }
Surface3D::_rebuildIfNeeded();
}

} // namespace AReVi

//----------------------------------------------------------------------------
