// LI2/ENIB - AReVi - 3D shapes
//----------------------------------------------------------------------------
// file        : AReVi/Shapes/trihedron.cpp
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/Shapes/trihedron.h"

#include "AReVi/Shapes/box3D.h"
#include "AReVi/Shapes/cone3D.h"

#include "AReVi/Lib3D/transform3D.h"
#include "AReVi/Lib3D/material3D.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class Trihedron
//----------------------------------------------------------------------------

AR_CLASS_DEF(Trihedron,Shape3D)

//-------- Construction / Destruction ----------------------------------------

Trihedron::Trihedron(ArCW & arCW) // Protected
: Shape3D(arCW)
{
ArRef<Transform3D> t=Transform3D::NEW();
ArRef<Material3D> m=Material3D::NEW();
ArRef<Box3D> b;
ArRef<Cone3D> c;
// X axis
m->setDiffuseColor(0.8,0.0,0.0);
m->setEmissiveColor(0.1,0.0,0.0);
b=Box3D::NEW();
b->writeMaterial(m);
b->setSize(0.7,0.05,0.05);
t->postTranslate(0.35,0.0,0.0);
b->writeTransformation(t);
addRootPart(b);
c=Cone3D::NEW();
c->writeMaterial(m);
c->setHeight(0.3);
c->setRadius(0.1);
c->setPrecision(4);
t->identity();
t->postPitch(M_PI_2);
t->postTranslate(0.0,0.0,0.85);
c->writeTransformation(t);
addRootPart(c);
// Y axis
m->setDiffuseColor(0.0,0.8,0.0);
m->setEmissiveColor(0.0,0.1,0.0);
b=Box3D::NEW();
b->writeMaterial(m);
b->setSize(0.05,0.7,0.05);
t->identity();
t->postTranslate(0.0,0.35,0.0);
b->writeTransformation(t);
addRootPart(b);
c=Cone3D::NEW();
c->writeMaterial(m);
c->setHeight(0.3);
c->setRadius(0.1);
c->setPrecision(4);
t->identity();
t->postRoll(-M_PI_2);
t->postTranslate(0.0,0.0,0.85);
c->writeTransformation(t);
addRootPart(c);
// Z axis
m->setDiffuseColor(0.0,0.0,0.8);
m->setEmissiveColor(0.0,0.0,0.1);
b=Box3D::NEW();
b->writeMaterial(m);
b->setSize(0.05,0.05,0.7);
t->identity();
t->postTranslate(0.0,0.0,0.35);
b->writeTransformation(t);
addRootPart(b);
c=Cone3D::NEW();
c->writeMaterial(m);
c->setHeight(0.3);
c->setRadius(0.1);
c->setPrecision(4);
t->identity();
t->postTranslate(0.0,0.0,0.85);
c->writeTransformation(t);
addRootPart(c);
}

Trihedron::~Trihedron(void) // Protected
{
}

} // namespace AReVi

//----------------------------------------------------------------------------
