// LI2/ENIB - AReVi - Utility services
//----------------------------------------------------------------------------
// file        : AReVi/Utils/fdStream.cpp
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/Utils/fdStream.h"

#include "AReVi/arSystem.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class FDIStream
//----------------------------------------------------------------------------

AR_CLASS_NOVOID_DEF(FDIStream,AbstractIStream)

//-------- Construction / Destruction ----------------------------------------

FDIStream::FDIStream(ArCW & arCW,
                     int fileDescriptor,
                     bool closeOnDestruction) // Protected
: AbstractIStream(arCW),
  _fd(fileDescriptor),
  _close(closeOnDestruction)
{
if(_fd==-1)
  {
  setErrorMessage("Invalid file descriptor `"+intToStr(_fd)+"'");
  _stream.setstate(ios::badbit);
  }
}

FDIStream::~FDIStream(void) // Protected
{
if(_close&&(_fd!=-1))
  {
  ArSystem::closeFD(_fd);
  }
}

//-------- File descriptor ---------------------------------------------------

int // file descriptor
FDIStream::getFileDescriptor(void) const
{
return(_fd);
}

bool // close file descriptor on destruction
FDIStream::closeOnDestruction(void) const
{
return(_close);
}

//-------- Forbidden operations ----------------------------------------------

int // >0: nb read   <=0: error
FDIStream::_readBuffer(char * buffer,
                       unsigned int size) // Protected
{
return(ArSystem::partialReadFD(_fd,buffer,size));
}


//----------------------------------------------------------------------------
// Class FDOStream
//----------------------------------------------------------------------------

AR_CLASS_NOVOID_DEF(FDOStream,AbstractOStream)

//-------- Construction / Destruction ----------------------------------------

FDOStream::FDOStream(ArCW & arCW,
                     int fileDescriptor,
                     bool closeOnDestruction) // Protected
: AbstractOStream(arCW),
  _fd(fileDescriptor),
  _close(closeOnDestruction)
{
if(_fd==-1)
  {
  setErrorMessage("Invalid file descriptor `"+intToStr(_fd)+"'");
  _stream.setstate(ios::badbit);
  }
}

FDOStream::~FDOStream(void) // Protected
{
_stream.flush();
if(_close&&(_fd!=-1))
  {
  ArSystem::closeFD(_fd);
  }
}

//-------- File descriptor ---------------------------------------------------

int // file descriptor
FDOStream::getFileDescriptor(void) const
{
return(_fd);
}

bool // close file descriptor on destruction
FDOStream::closeOnDestruction(void) const
{
return(_close);
}

//-------- Forbidden operations ----------------------------------------------

int // >=0: nb written   -1: error
FDOStream::_writeBuffer(const char * buffer,
                        unsigned int size) // Protected
{
return(ArSystem::writeFD(_fd,buffer,size));
}

} // namespace AReVi

//----------------------------------------------------------------------------
