// LI2/ENIB - AReVi - Utility services
//----------------------------------------------------------------------------
// file        : AReVi/Utils/fileStream.cpp
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/Utils/fileStream.h"

#include "AReVi/arSystem.h"

#include <unistd.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>

namespace AReVi {

//----------------------------------------------------------------------------
// Class FileIStream
//----------------------------------------------------------------------------

AR_CLASS_NOVOID_DEF(FileIStream,FDIStream)

//-------- Construction / Destruction ----------------------------------------

FileIStream::FileIStream(ArCW & arCW,
                         const StlString & fileName) // Protected
: FDIStream(arCW,-1,true),
  _file(fileName)
{
_fd=ArSystem::openInFile(_file.c_str());
if(_fd==-1)
  {
  setErrorMessage("Cannot open input file `"+fileName+"' --- "+
                  ArSystem::printError(errno));
  }
else
  {
  setErrorMessage("");
  _stream.clear();
  }
}

FileIStream::~FileIStream(void) // Protected
{
  // Nothing to be done
}

//-------- File informations -------------------------------------------------

const StlString & // file name
FileIStream::getFileName(void) const
{
return(_file);
}

unsigned int // file size
FileIStream::getFileSize(void) const
{
struct stat st;
if((_fd!=-1)&&(fstat(_fd,&st)!=-1))
  {
  return(static_cast<unsigned int>(st.st_size));
  }
else return(0);
}


//----------------------------------------------------------------------------
// Class FileOStream
//----------------------------------------------------------------------------

AR_CLASS_NOVOID_DEF(FileOStream,FDOStream)

//-------- Construction / Destruction ----------------------------------------

FileOStream::FileOStream(ArCW & arCW,
                         const StlString & fileName,
                         bool append) // Protected
: FDOStream(arCW,-1,true),
  _file(fileName),
  _append(append)
{
_fd=ArSystem::openOutFile(_file.c_str(),_append);
if(_fd==-1)
  {
  setErrorMessage("Cannot open output file `"+fileName+"' --- "+
                  ArSystem::printError(errno));
  }
else
  {
  setErrorMessage("");
  _stream.clear();
  }
}

FileOStream::~FileOStream(void) // Protected
{
  // Nothing to be done
}

//-------- File informations -------------------------------------------------

const StlString & // file name
FileOStream::getFileName(void) const
{
return(_file);
}

bool // write at the end of the file
FileOStream::append(void) const
{
return(_append);
}

} // namespace AReVi

//----------------------------------------------------------------------------
