// LI2/ENIB - AReVi - Utility services
//----------------------------------------------------------------------------
// file        : AReVi/Utils/javaObject.cpp
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/Utils/javaObject.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class JavaData
//----------------------------------------------------------------------------

AR_CLASS_NOVOID_DEF(JavaData,ArObject)

//-------- Construction / Destruction ----------------------------------------

JavaData::JavaData(ArCW & arCW) // Protected
: ArObject(arCW),
  _values()
{
  // Nothing to be done
}

JavaData::~JavaData(void) // Protected
{
while(_values.size())
  {
  if(_values.back().t==JavaData::J_STRING)
    {
    delete _values.back().v.str;
    }
  _values.pop_back();
  }
}

//-------- Value management --------------------------------------------------

unsigned int
JavaData::getNbValues(void) const
{
return(_values.size());
}

JavaData::ValueType
JavaData::getType(unsigned int index) const
{
return(_values[index].t);
}

void
JavaData::addJBoolean(unsigned char value)
{
JavaData::Value val;
val.t=JavaData::J_BOOLEAN;
val.v.Z=value;
_values.push_back(val);
}

unsigned char // java boolean value
JavaData::readJBoolean(unsigned int index) const
{
return(_values[index].v.Z);
}

void
JavaData::addJByte(char value)
{
JavaData::Value val;
val.t=JavaData::J_BYTE;
val.v.B=value;
_values.push_back(val);
}

char // java byte value
JavaData::readJByte(unsigned int index) const
{
return(_values[index].v.B);
}

void
JavaData::addJChar(unsigned short value)
{
JavaData::Value val;
val.t=JavaData::J_CHAR;
val.v.C=value;
_values.push_back(val);
}

unsigned short // java char value
JavaData::readJChar(unsigned int index) const
{
return(_values[index].v.C);
}

void
JavaData::addJShort(short value)
{
JavaData::Value val;
val.t=JavaData::J_SHORT;
val.v.S=value;
_values.push_back(val);
}

short // java short value
JavaData::readJShort(unsigned int index) const
{
return(_values[index].v.S);
}

void
JavaData::addJInt(int value)
{
JavaData::Value val;
val.t=JavaData::J_INT;
val.v.I=value;
_values.push_back(val);
}

int // java int value
JavaData::readJInt(unsigned int index) const
{
return(_values[index].v.I);
}

void
JavaData::addJLong(long long value)
{
JavaData::Value val;
val.t=JavaData::J_LONG;
val.v.J=value;
_values.push_back(val);
}

long long // java long value
JavaData::readJLong(unsigned int index) const
{
return(_values[index].v.J);
}

void
JavaData::addJFloat(float value)
{
JavaData::Value val;
val.t=JavaData::J_FLOAT;
val.v.F=value;
_values.push_back(val);
}

float // java float value
JavaData::readJFloat(unsigned int index) const
{
return(_values[index].v.F);
}

void
JavaData::addJDouble(double value)
{
JavaData::Value val;
val.t=JavaData::J_DOUBLE;
val.v.D=value;
_values.push_back(val);
}

double // java double value
JavaData::readJDouble(unsigned int index) const
{
return(_values[index].v.D);
}

void
JavaData::addJString(const StlString & value)
{
JavaData::Value val;
val.t=JavaData::J_STRING;
val.v.str=new StlString(value);
_values.push_back(val);
}

const StlString & // java string value
JavaData::readJString(unsigned int index) const
{
return(*(_values[index].v.str));
}


//----------------------------------------------------------------------------
// Class JavaObject
//----------------------------------------------------------------------------

AR_CLASS_DEF(JavaObject,ArObject)

bool JavaObject::_initSuccess=false;

void * (*JavaObject::_newFnct)(JavaObject *)=
       (void * (*)(JavaObject *))0;

void (*JavaObject::_deleteFnct)(void *)=
     (void (*)(void *))0;

ArRef<JavaData> (*JavaObject::_toJavaFnct)(void *,ArConstRef<JavaData>)=
                (ArRef<JavaData> (*)(void *,ArConstRef<JavaData>))0;

//-------- Construction / Destruction ----------------------------------------

JavaObject::JavaObject(ArCW & arCW) // Protected
: ArObject(arCW),
  _javaRef((void *)0)
{
if(JavaObject::_initSuccess)
  {
  _javaRef=(*JavaObject::_newFnct)(this);
  if(!_javaRef)
    {
    setErrorMessage("Java failed to instanciate `"+getAlias()+"'");
    }
  }
else
  {
  setErrorMessage("Not within a Java application");
  }
}

JavaObject::~JavaObject(void) // Protected
{
if(_javaRef)
  {
  (*JavaObject::_deleteFnct)(_javaRef);
  }
}

//-------- Communication -----------------------------------------------------

ArRef<JavaData> // result
JavaObject::toJava(ArConstRef<JavaData> data)
{
if(_javaRef)
  {
  return((*JavaObject::_toJavaFnct)(_javaRef,data));
  }
else
  {
  return(JavaData::nullRef());
  }
}

ArRef<JavaData> // result
JavaObject::fromJava(ArConstRef<JavaData> data)
{
cerr << "Reading data for " << getAlias() << "(" << this << ")" << endl;
for(unsigned int i=0;i<data->getNbValues();i++)
  {
  switch(data->getType(i))
    {
    case JavaData::J_BOOLEAN:
      cerr << "    J_BOOLEAN [" << (data->readJBoolean(i)
                                    ? "true" : "false") << "]" << endl;
      break;
    case JavaData::J_BYTE:
      cerr << "    J_BYTE [" << data->readJByte(i) << "] (int:"
                             << (int)data->readJByte(i) << ")" << endl;
      break;
    case JavaData::J_CHAR:
      cerr << "    J_CHAR [" << data->readJChar(i) << "]" << endl;
      break;
    case JavaData::J_SHORT:
      cerr << "    J_SHORT [" << data->readJShort(i) << "]" << endl;
      break;
    case JavaData::J_INT:
      cerr << "    J_INT [" << data->readJInt(i) << "]" << endl;
      break;
    case JavaData::J_LONG:
      cerr << "    J_LONG [" << data->readJLong(i) << "]" << endl;
      break;
    case JavaData::J_FLOAT:
      cerr << "    J_FLOAT [" << data->readJFloat(i) << "]" << endl;
      break;
    case JavaData::J_DOUBLE:
      cerr << "    J_DOUBLE [" << data->readJDouble(i) << "]" << endl;
      break;
    case JavaData::J_STRING:
      cerr << "    J_STRING [" << data->readJString(i) << "]" << endl;
      break;
    }
  }
return(JavaData::nullRef());
}

bool // application is embeded in Java
JavaObject::inJavaApp(void) // Static
{
return(JavaObject::_initSuccess);
}

} // namespace AReVi

//----------------------------------------------------------------------------
