// LI2/ENIB - AReVi - Utility services
//----------------------------------------------------------------------------
// file        : AReVi/Utils/joystick.cpp
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/Utils/joystick.h"

inline static
double Joystick_roundAxis(double axis)
{
return(rint(axis*1000.0)*0.001);
}

#define EMPTY_JOYSTICK

#ifdef __linux
//----------------------------------------------------------------------------
// Class Joystick - Linux specific part
//----------------------------------------------------------------------------
#undef EMPTY_JOYSTICK
#include "joystick_linux_priv.cpp"
#endif // not __linux

#ifdef __CYGWIN__
//----------------------------------------------------------------------------
// Class Joystick - Win32 specific part
//----------------------------------------------------------------------------
#undef EMPTY_JOYSTICK
#include "joystick_w32_priv.cpp"
#endif // not __CYGWIN__

#ifdef EMPTY_JOYSTICK

namespace AReVi {

//----------------------------------------------------------------------------
// Class Joystick - Default (empty) implementation
//----------------------------------------------------------------------------

AR_CLASS_NOVOID_DEF(Joystick,InputDevice)

//-------- Construction / Destruction ----------------------------------------

Joystick::Joystick(ArCW & arCW,
                   const StlString & device) // Protected
: InputDevice(arCW,device),
  _jsData((JoystickData *)0)
{
reset(_device);
}

Joystick::~Joystick(void) // Protected
{
  // Nothing to be done
}

//-------- Device management -------------------------------------------------

bool // success
Joystick::reset(const StlString & device)
{
_device=device;
_buttons.clear();
_axes.clear();
setErrorMessage("No implementation for this system");
return(false);
}

void
Joystick::update(void)
{
  // Nothing to be done
}

} // namespace AReVi

#endif // EMPTY_JOYSTICK

//----------------------------------------------------------------------------
