// LI2/ENIB - AReVi - Utility services
//----------------------------------------------------------------------------
// file        : AReVi/Utils/joystick_linux_priv.cpp
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/arSystem.h"

#include <linux/joystick.h>
#include <sys/ioctl.h>
#include <errno.h>

namespace AReVi {

struct JoystickData
{
int fd;
JoystickData(void) : fd(-1) {}
};

//----------------------------------------------------------------------------
// Class Joystick - Linux specific part
//----------------------------------------------------------------------------

AR_CLASS_NOVOID_DEF(Joystick,InputDevice)

//-------- Construction / Destruction ----------------------------------------

Joystick::Joystick(ArCW & arCW,
                   const StlString & device) // Protected
: InputDevice(arCW,device),
  _jsData(new JoystickData)
{
reset(_device);
}

Joystick::~Joystick(void) // Protected
{
reset("");
delete _jsData;
}

//-------- Device management -------------------------------------------------

bool // success
Joystick::reset(const StlString & device)
{
_device=device;
_buttons.clear();
_axes.clear();
setErrorMessage("");
if(_jsData->fd>=0)
  {
  ArSystem::closeFD(_jsData->fd);
  _jsData->fd=-1;
  }
_jsData->fd=ArSystem::openInFile(_device);
if(_jsData->fd<0)
  {
  setErrorMessage("Cannot open device '"+_device+"' --- "+
                  ArSystem::printError(errno));
  ArSystem::closeFD(_jsData->fd);
  _jsData->fd=-1;
  return(false);
  }
unsigned int i;
char result;
if(ioctl(_jsData->fd,JSIOCGAXES,&result)==-1)
  {
  setErrorMessage("Cannot determine number of axes on device '"+_device+
                  "' --- "+ArSystem::printError(errno));
  ArSystem::closeFD(_jsData->fd);
  _jsData->fd=-1;
  return(false);
  }
for(i=result;i--;)
  {
  _axes.push_back(0.0);
  }
if(ioctl(_jsData->fd,JSIOCGBUTTONS,&result)==-1)
  {
  setErrorMessage("Cannot determine number of buttons on device '"+_device+
                  "' --- "+ArSystem::printError(errno));
  ArSystem::closeFD(_jsData->fd);
  _jsData->fd=-1;
  return(false);
  }
for(i=result;i--;)
  {
  _buttons.push_back(false);
  }
if(!ArSystem::setBlockingFD(_jsData->fd,false))
  {
  setErrorMessage("Cannot use non-blocking mode for device '"+_device+
                  "' --- "+ArSystem::printError(errno));
  ArSystem::closeFD(_jsData->fd);
  _jsData->fd=-1;
  return(false);
  }
return(true);
}

void
Joystick::update(void)
{
bool done=false;
while(!fail()&&!done)
  {
  struct js_event evt;
  int r=ArSystem::partialReadFD(_jsData->fd,&evt,sizeof(struct js_event));
  switch(r)
    {
    case -2:
      {
      done=true; // non-blocking
      break;
      }
    case 0:
      {
      setErrorMessage("EOF reached while reading from device '"+
                      _device+"'");
      ArSystem::closeFD(_jsData->fd);
      _jsData->fd=-1;
      break;
      }
    case -1:
      {
      setErrorMessage("Error while reading from device '"+
                      _device+"' --- "+
                      ArSystem::printError(errno));
      ArSystem::closeFD(_jsData->fd);
      _jsData->fd=-1;
      break;
      }
    default:
      {
      if(r==sizeof(struct js_event))
        {
        switch(evt.type&~JS_EVENT_INIT)
          {
          case JS_EVENT_BUTTON:
            {
            _buttons[evt.number]=(evt.value ? true : false);
            break;
            }
          case JS_EVENT_AXIS:
            {
            _axes[evt.number]=Joystick_roundAxis(((double)evt.value)/32767.0);
            break;
            }
          }
        }
      }
    }
  }
}

} // namespace AReVi

//----------------------------------------------------------------------------
