// LI2/ENIB - AReVi - Utility services
//----------------------------------------------------------------------------
// file        : AReVi/Utils/joystick_w32_priv.cpp
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include <windows.h>

namespace AReVi {

struct JoystickData
{
UINT joyID;
StlVector<double> center;
StlVector<double> range;
StlVector<int> offset;
};

//----------------------------------------------------------------------------
// Class Joystick - Win32 specific part
//----------------------------------------------------------------------------

AR_CLASS_NOVOID_DEF(Joystick,InputDevice)

//-------- Construction / Destruction ----------------------------------------

Joystick::Joystick(ArCW & arCW,
                   const StlString & device) // Protected
: InputDevice(arCW,device),
  _jsData(new JoystickData)
{
reset(_device);
}

Joystick::~Joystick(void) // Protected
{
reset("");
delete _jsData;
}

//-------- Device management -------------------------------------------------

bool // success
Joystick::reset(const StlString & device)
{
_device=device;
_buttons.clear();
_axes.clear();
setErrorMessage("");
_jsData->center.clear();
_jsData->range.clear();
_jsData->offset.clear();
if((_device=="1")||(_device=="JOYSTICKID1")||
   (_device=="/dev/js0")||(_device=="/dev/input/js0"))
  {
  _jsData->joyID=JOYSTICKID1;
  }
else if((_device=="2")||(_device=="JOYSTICKID2")||
        (_device=="/dev/js1")||(_device=="/dev/input/js1"))
  {
  _jsData->joyID=JOYSTICKID2;
  }
else
  {
  setErrorMessage("Unknown joystick device '"+_device+"'");
  return(false);
  }
JOYCAPS joycaps;
memset(&joycaps,0,sizeof(JOYCAPS));
if(joyGetDevCaps(_jsData->joyID,&joycaps,sizeof(JOYCAPS))!=JOYERR_NOERROR)
  {
  setErrorMessage("Cannot read properties of device '"+_device+"'");
  return(false);
  }
_axes.push_back(0.0);
_jsData->center.push_back(0.5*(joycaps.wXmax+joycaps.wXmin));
_jsData->range.push_back(2.0/(joycaps.wXmax-joycaps.wXmin));
_jsData->offset.push_back(0);
_axes.push_back(0.0);
_jsData->center.push_back(0.5*(joycaps.wYmax+joycaps.wYmin));
_jsData->range.push_back(2.0/(joycaps.wYmax-joycaps.wYmin));
_jsData->offset.push_back(1);
if(joycaps.wCaps&JOYCAPS_HASZ)
  {
  _axes.push_back(0.0);
  _jsData->center.push_back(0.5*(joycaps.wZmax+joycaps.wZmin));
  _jsData->range.push_back(2.0/(joycaps.wZmax-joycaps.wZmin));
  _jsData->offset.push_back(2);
  }
if(joycaps.wCaps&JOYCAPS_HASR)
  {
  _axes.push_back(0.0);
  _jsData->center.push_back(0.5*(joycaps.wRmax+joycaps.wRmin));
  _jsData->range.push_back(2.0/(joycaps.wRmax-joycaps.wRmin));
  _jsData->offset.push_back(3);
  }
if(joycaps.wCaps&JOYCAPS_HASU)
  {
  _axes.push_back(0.0);
  _jsData->center.push_back(0.5*(joycaps.wUmax+joycaps.wUmin));
  _jsData->range.push_back(2.0/(joycaps.wUmax-joycaps.wUmin));
  _jsData->offset.push_back(4);
  }
if(joycaps.wCaps&JOYCAPS_HASV)
  {
  _axes.push_back(0.0);
  _jsData->center.push_back(0.5*(joycaps.wVmax+joycaps.wVmin));
  _jsData->range.push_back(2.0/(joycaps.wVmax-joycaps.wVmin));
  _jsData->offset.push_back(5);
  }
if(joycaps.wCaps&JOYCAPS_HASPOV)
  {
  _axes.push_back(0.0);
  _jsData->center.push_back(0.0);
  _jsData->range.push_back(1.0);
  _jsData->offset.push_back(-1);
  _axes.push_back(0.0);
  _jsData->center.push_back(0.0);
  _jsData->range.push_back(1.0);
  _jsData->offset.push_back(-2);
  }
for(unsigned int i=joycaps.wNumButtons;i--;)
  {
  _buttons.push_back(false);
  }
return(true);
}

void
Joystick::update(void)
{
if(!fail())
  {
  JOYINFOEX joyinfo;
  joyinfo.dwFlags=JOY_RETURNALL;
  joyinfo.dwSize=sizeof(JOYINFOEX);
  if(joyGetPosEx(_jsData->joyID,&joyinfo)!=JOYERR_NOERROR)
    {
    setErrorMessage("Error while reading from device '"+_device+"'");
    }
  else
    {
    unsigned int i;
    for(i=_buttons.size();i--;)
      {
      unsigned int b;
      switch(i)
        {
        case 7: b=9; break;
        case 8: b=7; break;
        case 9: b=8; break;
        default: b=i;
        }
      _buttons[i]=(joyinfo.dwButtons&(1<<b) ? true : false);
      }
    for(i=_axes.size();i--;)
      {
      switch(_jsData->offset[i])
        {
        case -2: // will be updated in -1
          break;
        case -1:
          {
          if((joyinfo.dwPOV&0x0000FFFF)==0x0000FFFF)
            {
            _axes[i]=_axes[i+1]=0.0;
            }
          else
            {
            double angle=(joyinfo.dwPOV&0x0000FFFF)*M_PI/18000.0;
            double s=sin(angle);
            double c=cos(angle);
            if(fabs(s)>fabs(c))
              {
              if(s>0.0)
                {
                _axes[i]=1.0;
                _axes[i+1]=Joystick_roundAxis(-c/s);
                }
              else
                {
                _axes[i]=-1.0;
                _axes[i+1]=Joystick_roundAxis(c/s);
                }
              }
            else
              {
              if(c>0.0)
                {
                _axes[i]=Joystick_roundAxis(s/c);
                _axes[i+1]=1.0;
                }
              else
                {
                _axes[i]=Joystick_roundAxis(-s/c);
                _axes[i+1]=-1.0;
                }
              }
            }
          break;
          }
        default:
          {
          DWORD * ptr=(&joyinfo.dwXpos)+_jsData->offset[i];
          _axes[i]=Joystick_roundAxis((*ptr-_jsData->center[i])*
                                      _jsData->range[i]);
          }
        }
      }
    }
  }
}

} // namespace AReVi

//----------------------------------------------------------------------------
