// LI2/ENIB - AReVi - Utility services
//----------------------------------------------------------------------------
// file        : AReVi/Utils/mappedFile.cpp
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/Utils/mappedFile.h"

#include "AReVi/arSystem.h"

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <errno.h>
#include <sys/mman.h>

namespace AReVi {

//----------------------------------------------------------------------------
// Class MappedFile
//----------------------------------------------------------------------------

AR_CLASS_NOVOID_DEF(MappedFile,RigidMemoryBlock)

//-------- Construction / Destruction ----------------------------------------

MappedFile::MappedFile(ArCW & arCW,
                       bool writable,
                       const StlString & fileName,
                       bool shared) // Protected
: RigidMemoryBlock(arCW,(char *)0,0,writable),
  _fileName(fileName),
  _shared(shared)
{
int fd=(_writable ? ArSystem::openInOutFile(fileName)
                  : ArSystem::openInFile(fileName));
if(fd==-1)
  {
  setErrorMessage("Cannot open file `"+fileName+"' --- "+
                  ArSystem::printError(errno));
  return;
  }
struct stat infos;
if(fstat(fd,&infos)==-1)
  {
  setErrorMessage("Cannot determine file size --- "+
                  ArSystem::printError(errno));
  ArSystem::closeFD(fd);
  return;
  }
_size=static_cast<unsigned int>(infos.st_size);
do
  {
  _memory=static_cast<char *>(mmap((char *)0,_size,
                             (_writable ? PROT_WRITE : 0)|PROT_READ|PROT_EXEC,
                             (_shared ? MAP_SHARED : MAP_PRIVATE),fd,0));
  } while((_memory==MAP_FAILED)&&(errno==EINTR));
if(_memory==MAP_FAILED)
  {
  setErrorMessage("Cannot map file into memory --- "+
                  ArSystem::printError(errno));
  ArSystem::closeFD(fd);
  _memory=(char *)0;
  _size=0;
  return;
  }
ArSystem::closeFD(fd);
}

MappedFile::~MappedFile(void) // Protected
{
if(_memory)
  {
  int r;
  do
    {
    r=munmap(_memory,_size);
    } while((r==-1)&&(errno==EINTR));
  _memory=(char *)0;
  }
}

//-------- Inspection --------------------------------------------------------

const StlString & // mapped file
MappedFile::getFileName(void) const
{
return(_fileName);
}

bool // shared with processes and disk
MappedFile::isShared(void) const
{
return(_shared);
}

//-------- Memory management -------------------------------------------------

void
MappedFile::synchronize(void)
{
if(_memory&&_shared)
  {
  int r;
  do
    {
    r=msync(_memory,_size,MS_SYNC|MS_INVALIDATE);
    } while((r==-1)&&(errno==EINTR));
  if(r==-1)
    {
    setErrorMessage("Cannot synchronize memory area --- "+
                    ArSystem::printError(errno));
    }
  }
}

} // namespace AReVi

//----------------------------------------------------------------------------
