// LI2/ENIB - AReVi - Utility services
//----------------------------------------------------------------------------
// file        : AReVi/Utils/math_priv.cpp
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/Utils/math_priv.h"

namespace AReVi {

//-------- Angles ------------------------------------------------------------

double
betweenPi(double angle)
{
angle=fmod(angle,M_2PI);
return(angle>M_PI ? angle-M_2PI : (angle<-M_PI ? angle+M_2PI : angle));
}

double
degreeToRadian(double angle)
{
return(angle*M_PI_180);
}

double
radianToDegree(double angle)
{
return(angle*M_180_PI);
}

//-------- Cartesian 2D <---> Polar ------------------------------------------

double
cart2DToAngle(double x,
              double y)
{
return(x||y ? atan2(y,x) : 0.0);
}

double
cart2DToDistance(double x,
                 double y)
{
return(sqrt(x*x+y*y));
}

void
cart2DToPolar(double & distanceOut,
              double & angleOut,
              double x,
              double y)
{
distanceOut=sqrt(x*x+y*y);
angleOut=(distanceOut ? atan2(y,x) : 0.0);
}

void
polarToCart2D(double & xOut,
              double & yOut,
              double distance,
              double angle)
{
xOut=distance*cos(angle);
yOut=distance*sin(angle);
}

//-------- Cartesian 3D <---> Spherical --------------------------------------

void
cart3DToAngles(double & yawOut,
               double & pitchOut,
               double x,
               double y,
               double z)
{
double dxy=sqrt(x*x+y*y);
yawOut=(dxy ? atan2(y,x) : 0.0);
pitchOut=(dxy||z ? -atan2(z,dxy) : 0.0);
}

double
cart3DToDistance(double x,
                 double y,
                 double z)
{
return(sqrt(x*x+y*y+z*z));
}

void
cart3DToSpherical(double & distanceOut,
                  double & yawOut,
                  double & pitchOut,
                  double x,
                  double y,
                  double z)
{
double dxy=sqrt(x*x+y*y);
distanceOut=sqrt(dxy*dxy+z*z);
yawOut=(dxy ? atan2(y,x) : 0.0);
pitchOut=(distanceOut ? -atan2(z,dxy) : 0.0);
}

void
sphericalToCart3D(double & xOut,
                  double & yOut,
                  double & zOut,
                  double distance,
                  double yaw,
                  double pitch)
{
double dxy=distance*cos(pitch);
xOut=dxy*cos(yaw);
yOut=dxy*sin(yaw);
zOut=-distance*sin(pitch);
}

//-------- Sign extensions ---------------------------------------------------

double
signedSquare(double x) // keeps the sign of x
{
return(x>=0.0 ? x*x : -x*x);
}

double
signedSqrt(double x) // keeps the sign of x
{
return(x>=0.0 ? sqrt(x) : -sqrt(-x));
}

//-------- Interpolation -----------------------------------------------------

void
interpolationIndex(unsigned int & previousIndexOut,
                   double & nextPreviousRatioOut,
                   unsigned int nbElems,
                   double cursor)
{
cursor*=(--nbElems);
int index=(int)floor(cursor);
if(index>=(int)nbElems)
  {
  previousIndexOut=nbElems-1;
  nextPreviousRatioOut=1.0;
  }
else if(index<0)
  {
  previousIndexOut=0;
  nextPreviousRatioOut=0.0;
  }
else
  {
  previousIndexOut=index;
  nextPreviousRatioOut=cursor-index;
  }
}

} // namespace AReVi

//----------------------------------------------------------------------------
