// LI2/ENIB - AReVi - Utility services
//----------------------------------------------------------------------------
// file        : AReVi/Utils/memoryBlock.cpp
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/Utils/memoryBlock.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class MemoryBlock
//----------------------------------------------------------------------------

AR_CLASS_DEF(MemoryBlock,ArObject)

bool MemoryBlock::_initSuccess=false;

bool (* MemoryBlock::_compress)(ArConstRef<MemoryBlock>,ArRef<MemoryBlock>)=
    (bool (*)(ArConstRef<MemoryBlock>,ArRef<MemoryBlock>))0;

bool (* MemoryBlock::_compress2)(ArConstRef<MemoryBlock>,ArRef<MemoryBlock>,
                                 int)=
    (bool (*)(ArConstRef<MemoryBlock>,ArRef<MemoryBlock>,int))0;

bool (* MemoryBlock::_uncompress)(ArConstRef<MemoryBlock>,ArRef<MemoryBlock>)=
    (bool (*)(ArConstRef<MemoryBlock>,ArRef<MemoryBlock>))0;

//-------- Construction / Destruction ----------------------------------------

MemoryBlock::MemoryBlock(ArCW & arCW) // Protected
: ArObject(arCW),
  _size(0),
  _memory((char *)0),
  _canSwap(true)
{
  // Nothing to be done
}

MemoryBlock::~MemoryBlock(void) // Protected
{
if(_memory)
  {
  Allocator::deallocate(_memory);
  }
}

//-------- Memory management -------------------------------------------------

const char * // memory area start
MemoryBlock::getMemory(void) const
{
return(_memory);
}

char * // memory area start
MemoryBlock::accessMemory(void)
{
return(_memory);
}

bool // success
MemoryBlock::setSize(unsigned int size)
{
if((_size<size)||!size) // really reallocate only when growing or freeing
  {
  _memory=static_cast<char *>(Allocator::reallocate(_memory,size));
  }
_size=size;
return(true);
}

bool // success
MemoryBlock::ensureMinSize(unsigned int size)
{
if(_size<size)
  {
  _memory=static_cast<char *>(Allocator::reallocate(_memory,size));
  _size=size;
  }
return(true);
}

unsigned int // memory area size
MemoryBlock::getSize(void) const
{
return(_size);
}

bool // success
MemoryBlock::swapBlock(ArRef<MemoryBlock> block)
{
if(_canSwap&&block->_canSwap)
  {
  unsigned int tmpSize=_size;
  _size=block->_size;
  block->_size=tmpSize;
  char * tmpMemory=_memory;
  _memory=block->_memory;
  block->_memory=tmpMemory;
  return(true);
  }
else
  {
  return(false);
  }
}

bool // success
MemoryBlock::compressFrom(ArConstRef<MemoryBlock> uncompressedBlock)
{
if(MemoryBlock::_initSuccess)
  {
  return(MemoryBlock::_compress(uncompressedBlock,thisRef()));
  }
else
  {
  setErrorMessage("ZLib plugin not initialized");
  return(false);
  }
}

bool // success
MemoryBlock::compressFrom(ArConstRef<MemoryBlock> uncompressedBlock,
                          int level)
{
if(MemoryBlock::_initSuccess)
  {
  return(MemoryBlock::_compress2(uncompressedBlock,thisRef(),level));
  }
else
  {
  setErrorMessage("ZLib plugin not initialized");
  return(false);
  }
}

// MemoryBlock size must be large enough to store uncompressed data!!!
bool // success
MemoryBlock::uncompressFrom(ArConstRef<MemoryBlock> compressedBlock)
{
if(MemoryBlock::_initSuccess)
  {
  return(MemoryBlock::_uncompress(compressedBlock,thisRef()));
  }
else
  {
  setErrorMessage("ZLib plugin not initialized");
  return(false);
  }
}

//----------------------------------------------------------------------------
// Class RigidMemoryBlock
//----------------------------------------------------------------------------

AR_CLASS_NOVOID_DEF(RigidMemoryBlock,MemoryBlock)

//-------- Construction / Destruction ----------------------------------------

RigidMemoryBlock::RigidMemoryBlock(ArCW & arCW,
                                   char * memory,
                                   unsigned int size,
                                   bool writable) // Protected
: MemoryBlock(arCW),
  _writable(writable)
{
_size=size;
_memory=memory;
_canSwap=false;
}

RigidMemoryBlock::~RigidMemoryBlock(void) // Protected
{
_memory=(char *)0;
}

//-------- Memory management -------------------------------------------------

char * // memory area start (if writable)
RigidMemoryBlock::accessMemory(void)
{
return(_writable ? _memory : (char *)0);
}

bool // success (always fails)
RigidMemoryBlock::setSize(unsigned int /* size */)
{
return(false); // size cannot be changed
}

bool // success (always fails)
RigidMemoryBlock::ensureMinSize(unsigned int /* size */)
{
return(false); // size cannot be changed
}

bool // can write into memory area
RigidMemoryBlock::isWritable(void) const
{
return(_writable);
}

} // namespace AReVi

//----------------------------------------------------------------------------
