// LI2/ENIB - AReVi - Utility services
//----------------------------------------------------------------------------
// file        : AReVi/Utils/memoryStream.cpp
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/Utils/memoryStream.h"

#include "AReVi/Utils/memoryBlock.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class MemoryIStream
//----------------------------------------------------------------------------

AR_CLASS_NOVOID_DEF(MemoryIStream,AbstractIStream)

//-------- Construction / Destruction ----------------------------------------

MemoryIStream::MemoryIStream(ArCW & arCW,
                             ArConstRef<MemoryBlock> memory,
                             unsigned int offset) // Protected
: AbstractIStream(arCW,memory,offset),
  _memory(memory)
{
  // Nothing to be done
}

MemoryIStream::~MemoryIStream(void) // Protected
{
  // Nothing to be done
}

//-------- Memory informations -----------------------------------------------

ArConstRef<MemoryBlock> // input memory block
MemoryIStream::getMemoryBlock(void) const
{
return(_memory);
}

unsigned int // amount of extracted memory
MemoryIStream::getMemoryUsage(void) const
{
return(_iBuffer->getMemoryUsage());
}

unsigned int // // where extraction begins
MemoryIStream::getOffset(void) const
{
return(_iBuffer->getOffset());
}


//----------------------------------------------------------------------------
// Class MemoryOStream
//----------------------------------------------------------------------------

AR_CLASS_NOVOID_DEF(MemoryOStream,AbstractOStream)

//-------- Construction / Destruction ----------------------------------------

MemoryOStream::MemoryOStream(ArCW & arCW,
                             ArRef<MemoryBlock> memory,
                             unsigned int offset) // Protected
: AbstractOStream(arCW,memory,offset),
  _memory(memory)
{
  // Nothing to be done
}

MemoryOStream::~MemoryOStream(void) // Protected
{
  // Nothing to be done
}

//-------- Memory informations -----------------------------------------------

ArRef<MemoryBlock> // output memory block
MemoryOStream::getMemoryBlock(void) const
{
return(_memory);
}

unsigned int // amount of filled memory
MemoryOStream::getMemoryUsage(void) const
{
return(_oBuffer->getMemoryUsage());
}

unsigned int // // where fill begins
MemoryOStream::getOffset(void) const
{
return(_oBuffer->getOffset());
}

} // namespace AReVi

//----------------------------------------------------------------------------
