// LI2/ENIB - AReVi - Utility services
//----------------------------------------------------------------------------
// file        : AReVi/Utils/stlString_priv.cpp
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/Utils/stlString_priv.h"

#include <stdio.h>
#include <string.h>

namespace AReVi {

//-------- Write like a string constant --------------------------------------

void
strDump(StlString & resultOut,
        const StlString & str)
{
resultOut.erase();
resultOut+='\"';
for(unsigned int i=0;i<str.size();i++)
  {
  switch(str[i])
    {
    case '\a': resultOut+="\\a";  break;
    case '\b': resultOut+="\\b";  break;
    case '\f': resultOut+="\\f";  break;
    case '\n': resultOut+="\\n";  break;
    case '\r': resultOut+="\\r";  break;
    case '\t': resultOut+="\\t";  break;
    case '\v': resultOut+="\\v";  break;
    case '\\': resultOut+="\\\\"; break;
    case '\"': resultOut+="\\\""; break;
    default: resultOut+=str[i];
    }
  }
resultOut+='\"';
}

//-------- UTF8 conversions --------------------------------------------------

StlString // UTF8 string
strToUTF8(const StlString & str)
{
StlString result;
unsigned int sz=str.size();
result.reserve(sz);
for(unsigned int i=0;i<sz;i++)
  {
  unsigned char c=str[i];
  if(c&0x80)
    {
    result+=((c>>6)&0x1F)|0xC0;
    result+=(c&0x3F)|0x80;
    }
  else
    {
    result+=c;
    }
  }
return(result);
}

StlString // ascii string
strFromUTF8(const StlString & utf8str)
{
StlString result;
unsigned int sz=utf8str.size();
result.reserve(sz);
for(unsigned int i=0;i<sz;i++)
  {
  unsigned char c0=utf8str[i];
  if(c0&0x80)
    {
    if((c0&0xF8)==0xF0)
      {
      if(sz-i>3)
        {
        unsigned char c1=utf8str[i+1];
        unsigned char c2=utf8str[i+2];
        unsigned char c3=utf8str[i+3];
        if(((c1&0xC0)==0x80)&&
           ((c2&0xC0)==0x80)&&
           ((c3&0xC0)==0x80))
          {
          unsigned int nc=(((unsigned int)(c0&0x07))<<18)|
                          (((unsigned int)(c1&0x3F))<<12)|
                          (((unsigned int)(c2&0x3F))<<6)|
                          ((unsigned int)(c3&0x3F));
          if(nc<=0x000000FF) // only valid ascii chars !
            {
            result+=(char)nc;
            }
          }
        i+=3;
        }
      }
    else if((c0&0xF0)==0xE0)
      {
      if(sz-i>2)
        {
        unsigned char c1=utf8str[i+1];
        unsigned char c2=utf8str[i+2];
        if(((c1&0xC0)==0x80)&&
           ((c2&0xC0)==0x80))
          {
          unsigned int nc=(((unsigned int)(c0&0x0F))<<12)|
                          (((unsigned int)(c1&0x3F))<<6)|
                          ((unsigned int)(c2&0x3F));
          if(nc<=0x000000FF) // only valid ascii chars !
            {
            result+=(char)nc;
            }
          }
        i+=2;
        }
      }
    else if((c0&0xE0)==0xC0)
      {
      if(sz-i>1)
        {
        unsigned char c1=utf8str[i+1];
        if(((c1&0xC0)==0x80))
          {
          unsigned int nc=(((unsigned int)(c0&0x1F))<<6)|
                          ((unsigned int)(c1&0x3F));
          if(nc<=0x000000FF) // only valid ascii chars !
            {
            result+=(char)nc;
            }
          }
        ++i;
        }
      }
    }
  else
    {
    result+=c0;
    }
  }
return(result);
}

//-------- Suppress spaces ---------------------------------------------------

StlString // corrected string
strNoTrailingSpace(const StlString & str)
{
for(unsigned int i=str.size();i--;)
  {
  switch(str[i])
    {
    case '\n':
    case ' ':
    case '\t':
    case '\r':
      break;
    default:
      return(str.substr(0,i+1));
    }
  }
return(StlString(""));
}

StlString // corrected string
strNoLeadingSpace(const StlString & str)
{
unsigned int size=str.size();
for(unsigned int i=0;i<size;i++)
  {
  switch(str[i])
    {
    case '\n':
    case ' ':
    case '\t':
    case '\r':
      break;
    default:
      return(str.substr(i,size-i));
    }
  }
return(StlString(""));
}

//-------- File name operations ----------------------------------------------

StlString // extracted base name
strBasename(const StlString & fileName)
{
StlString result;
if(fileName.size())
  {
  const char separ='/';
  const char * begin=fileName.c_str();
  const char * end=begin+(fileName.size()-1);
  while(*end==separ) // avoid ending separators
    {
    if(end==begin) return(result);
    end--;
    }
  const char * limit=end;
  while((*limit!=separ)&&(limit!=begin)) limit--;
  if(*limit==separ) limit++;
  result=StlString(limit,(end-limit)+1);
  }
return(result);
}

StlString // extracted directory name
strDirname(const StlString & fileName)
{
StlString result;
if(fileName.size())
  {
  const char separ='/';
  const char * begin=fileName.c_str();
  const char * end=begin+(fileName.size()-1);
  while(*end==separ) // avoid ending separators
    {
    if(end==begin)
      {
      result=StlString(1,separ);
      return(result);
      }
    end--;
    }
  const char * limit=end;
  while((*limit!=separ)&&(limit!=begin)) limit--;
  if(*limit!=separ) return(result);
  if(limit==begin)
    {
    result=StlString(1,separ);
    return(result);
    }
  result=StlString(begin,limit-begin);
  result+=separ;
  }
return(result);
}

StlString // extracted extension
strExtension(const StlString & fileName)
{
StlString result;
if(fileName.size())
  {
  const char separ='/';
  const char * begin=fileName.c_str();
  const char * end=begin+(fileName.size()-1);
  while(*end==separ) // avoid ending separators
    {
    if(end==begin) return(result);
    end--;
    }
  const char * limit=end;
  while((*limit!='.')&&(limit!=begin)) limit--;
  if(*limit!='.') return(result);
  result=StlString(limit,(end-limit)+1);
  }
return(result);
}

//-------- StlString <---> pointer -------------------------------------------

bool // success
strToPtr(void * & pOut,
         const StlString & str)
{
return(sscanf(str.c_str(),"%p",&pOut)==1);
}

StlString  // formated string
ptrToStr(const void * p)
{
char buffer[0x20];
switch(sizeof(void *))
  {
  case sizeof(unsigned long long):
    {
    unsigned long long tmp;
    memcpy(&tmp,&p,sizeof(unsigned long long));
    sprintf(buffer,"0x%16.16llx",tmp);
    break;
    }
  case sizeof(unsigned int):
    {
    unsigned int tmp;
    memcpy(&tmp,&p,sizeof(unsigned int));
    sprintf(buffer,"0x%8.8x",tmp);
    break;
    }
  default:
    sprintf(buffer,"%p",p);
  }
return(StlString(buffer));
}

//-------- StlString <---> char ----------------------------------------------

bool // success
strToChar(char & cOut,
          const StlString & str)
{
int c;
return((sscanf(str.c_str(),"%d",&c)==1)&&
       ((cOut=(char)c)==c));
}

StlString // formated string
charToStr(char c)
{
char buffer[0x10];
sprintf(buffer,"%d",(int)c);
return(StlString(buffer));
}

StlString // formated string
charToHexaStr(char c)
{
char buffer[0x10];
sprintf(buffer,"%2.2x",(unsigned int)(unsigned char)c);
return(StlString(buffer));
}

//-------- StlString <---> unsigned char -------------------------------------

bool // success
strToUchar(unsigned char & ucOut,
           const StlString & str)
{
unsigned int uc;
return((sscanf(str.c_str(),"%u",&uc)==1)&&
       ((ucOut=(unsigned char)uc)==uc));
}

StlString // formated string
ucharToStr(unsigned char uc)
{
char buffer[0x10];
sprintf(buffer,"%u",(unsigned int)uc);
return(StlString(buffer));
}

StlString // formated string
ucharToHexaStr(unsigned char uc)
{
char buffer[0x10];
sprintf(buffer,"%2.2x",(unsigned int)uc);
return(StlString(buffer));
}

//-------- StlString <---> short ---------------------------------------------

bool // success
strToShort(short & sOut,
           const StlString & str)
{
return(sscanf(str.c_str(),"%hd",&sOut)==1);
}

StlString // formated string
shortToStr(short s)
{
char buffer[0x10];
sprintf(buffer,"%hd",s);
return(StlString(buffer));
}

StlString // formated string
shortToHexaStr(short s)
{
char buffer[0x10];
sprintf(buffer,"%4.4hx",(unsigned short)s);
return(StlString(buffer));
}

//-------- StlString <---> unsigned short ------------------------------------

bool // success
strToUshort(unsigned short & usOut,
            const StlString & str)
{
return(sscanf(str.c_str(),"%hu",&usOut)==1);
}

StlString // formated string
ushortToStr(unsigned short us)
{
char buffer[0x10];
sprintf(buffer,"%hu",us);
return(StlString(buffer));
}

StlString // formated string
ushortToHexaStr(unsigned short us)
{
char buffer[0x10];
sprintf(buffer,"%4.4hx",us);
return(StlString(buffer));
}

//-------- StlString <---> int -----------------------------------------------

bool // success
strToInt(int & iOut,
         const StlString & str)
{
return(sscanf(str.c_str(),"%d",&iOut)==1);
}

StlString // formated string
intToStr(int i)
{
char buffer[0x10];
sprintf(buffer,"%d",i);
return(StlString(buffer));
}

StlString // formated string
intToHexaStr(int i)
{
char buffer[0x10];
sprintf(buffer,"%8.8x",(unsigned int)i);
return(StlString(buffer));
}

//-------- StlString <---> unsigned int --------------------------------------

bool // success
strToUint(unsigned int & uiOut,
          const StlString & str)
{
return(sscanf(str.c_str(),"%u",&uiOut)==1);
}

StlString // formated string
uintToStr(unsigned int ui)
{
char buffer[0x10];
sprintf(buffer,"%u",ui);
return(StlString(buffer));
}

StlString // formated string
uintToHexaStr(unsigned int ui)
{
char buffer[0x10];
sprintf(buffer,"%8.8x",ui);
return(StlString(buffer));
}

//-------- StlString <---> long long -----------------------------------------

bool // success
strToLlong(long long & llOut,
           const StlString & str)
{
return(sscanf(str.c_str(),"%lld",&llOut)==1);
}

StlString // formated string
llongToStr(long long ll)
{
char buffer[0x10];
sprintf(buffer,"%lld",ll);
return(StlString(buffer));
}

StlString // formated string
llongToHexaStr(long long ll)
{
char buffer[0x20];
sprintf(buffer,"%16.16llx",(unsigned long long)ll);
return(StlString(buffer));
}

//-------- StlString <---> unsigned long long --------------------------------

bool // success
strToUllong(unsigned long long & ullOut,
            const StlString & str)
{
return(sscanf(str.c_str(),"%llu",&ullOut)==1);
}

StlString // formated string
ullongToStr(unsigned long long ull)
{
char buffer[0x10];
sprintf(buffer,"%llu",ull);
return(StlString(buffer));
}

StlString // formated string
ullongToHexaStr(unsigned long long ull)
{
char buffer[0x20];
sprintf(buffer,"%16.16llx",ull);
return(StlString(buffer));
}

//-------- StlString <---> float ---------------------------------------------

bool // success
strToFloat(float & fOut,
           const StlString & str)
{
return(sscanf(str.c_str(),"%g",&fOut)==1);
}

StlString // formated string
floatToStr(float f,
           bool showReal)
{
char buffer[0x20];
sprintf(buffer,"%.12g",f);
if(showReal)
  {
  char * ptr;
  bool found=false;
  for(ptr=buffer;*ptr;ptr++)
    {
    if((*ptr=='.')||(*ptr=='e'))
      found=true;
    }
  if(!found)
    {
    *(ptr++)='.';
    *(ptr++)='0';
    *ptr='\0';
    }
  }
return(StlString(buffer));
}

//-------- StlString <---> double --------------------------------------------

bool // success
strToDouble(double & dOut,
            const StlString & str)
{
return(sscanf(str.c_str(),"%lg",&dOut)==1);
}

StlString // formated string
doubleToStr(double d,
            bool showReal)
{
char buffer[0x20];
sprintf(buffer,"%.12g",d);
if(showReal)
  {
  char * ptr;
  bool found=false;
  for(ptr=buffer;*ptr;ptr++)
    {
    if((*ptr=='.')||(*ptr=='e'))
      found=true;
    }
  if(!found)
    {
    *(ptr++)='.';
    *(ptr++)='0';
    *ptr='\0';
    }
  }
return(StlString(buffer));
}

//-------- StlString <---> IP Address ----------------------------------------

bool // success
strToIPAddr(unsigned int & addrOut,
            const StlString & str)
{
unsigned int u1,u2,u3,u4;
if((sscanf(str.c_str(),"%u.%u.%u.%u",&u1,&u2,&u3,&u4)!=4)||
   (u1>0x000000FF)||(u2>0x000000FF)||(u3>0x000000FF)||(u4>0x000000FF))
  {
  return(false);
  }
addrOut=((u1<<24)|(u2<<16)|(u3<<8)|u4);
return(true);
}

StlString // formated string
ipAddrToStr(unsigned int addr)
{
char buffer[0x10];
sprintf(buffer,"%u.%u.%u.%u",(addr>>24)&0x000000FF,
                             (addr>>16)&0x000000FF,
                             (addr>>8)&0x000000FF,
                             addr&0x000000FF);
return(StlString(buffer));
}

} // namespace AReVi

//----------------------------------------------------------------------------
