// LI2/ENIB - AReVi - Utility services
//----------------------------------------------------------------------------
// file        : AReVi/Utils/tclCommand.cpp
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/Utils/tclCommand.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class TclCommand (abstract)
//----------------------------------------------------------------------------

AR_CLASS_NOVOID_DEF(TclCommand,ArObject)

bool TclCommand::_initSuccess=false;

StlVector<_TclCommandInfo *> * TclCommand::_commands=
    (StlVector<_TclCommandInfo *> *)0;

bool (*TclCommand::_isCompleteFnct)(const StlString &)=
     (bool (*)(const StlString &))0;

bool (*TclCommand::_evalFnct)(const StlString &,StlString &)=
     (bool (*)(const StlString &,StlString &))0;

bool (*TclCommand::_evalURLFnct)(const StlString &,StlString &)=
     (bool (*)(const StlString &,StlString &))0;

void (*TclCommand::_addFnct)(_TclCommandInfo *)=
     (void (*)(_TclCommandInfo *))0;

void (*TclCommand::_removeFnct)(_TclCommandInfo *)=
     (void (*)(_TclCommandInfo *))0;

//-------- Construction / Destruction ----------------------------------------

TclCommand::TclCommand(ArCW & arCW) // Protected
: ArObject(arCW)
{
  // Nothing to be done
}

TclCommand::~TclCommand(void) // Protected
{
  // Nothing to be done
}

//-------- Tcl invocation ----------------------------------------------------

bool // command is complete
TclCommand::isComplete(const StlString & command) // Static
{
return(TclCommand::_initSuccess&&(*TclCommand::_isCompleteFnct)(command));
}

bool // success
TclCommand::eval(const StlString & command,
                 StlString & resultOut) // Static
{
if(TclCommand::_initSuccess)
  {
  return((*TclCommand::_evalFnct)(command,resultOut));
  }
else
  {
  resultOut="Not within a TCL application";
  return(false);
  }
}

bool // success
TclCommand::evalURL(const StlString & url,
                    StlString & resultOut) // Static
{
if(TclCommand::_initSuccess)
  {
  return((*TclCommand::_evalURLFnct)(url,resultOut));
  }
else
  {
  resultOut="Not within a TCL application";
  return(false);
  }
}

//-------- Command management ------------------------------------------------

void
TclCommand::remove(const StlString & commandName) // Static
{
if(TclCommand::_initSuccess)
  {
  for(unsigned int i=TclCommand::_commands->size();i--;)
    {
    _TclCommandInfo * cmd=(*TclCommand::_commands)[i];
    if(cmd->name==commandName)
      {
      (*TclCommand::_removeFnct)(cmd);
      StlVectorFastErase(*TclCommand::_commands,i);
      break;
      }
    }
  }
}

void
TclCommand::getCommandNames(StlVector<StlString> & commandNamesOut) // Static
{
commandNamesOut.clear();
if(TclCommand::_initSuccess)
  {
  for(unsigned int i=TclCommand::_commands->size();i--;)
    {
    commandNamesOut.push_back((*TclCommand::_commands)[i]->name);
    }
  }
}

bool // application is embeded in TCL
TclCommand::inTclApp(void) // Static
{
return(TclCommand::_initSuccess);
}

} // namespace AReVi

//----------------------------------------------------------------------------
