// LI2/ENIB - AReVi - Utility services
//----------------------------------------------------------------------------
// file        : AReVi/Utils/urlStream.cpp
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/Utils/urlStream.h"

#include "AReVi/arSystem.h"

#include <errno.h>
#include <stdio.h>

namespace AReVi {

//----------------------------------------------------------------------------
// Class URLStream
//----------------------------------------------------------------------------

AR_CLASS_NOVOID_DEF(URLStream,FDIStream)

//-------- Construction / Destruction ----------------------------------------

URLStream::URLStream(ArCW & arCW,
                     const StlString & url) // Protected
: FDIStream(arCW,-1,true),
  _url(url),
  _protocol(),
  _host(),
  _port(-1),
  _file()
{
if(!ArSystem::decodeURL(_url,_protocol,_host,_port,_file))
  {
  setErrorMessage("Cannot decode url `"+url+"'");
  return;
  }
if(_protocol=="http")
  {
  StlString host;
  int port;
  StlString getArgument;
  ArSystem::getHTTPProxy(host,port);
  if(host.size())
    {
    getArgument=_url;
    }
  else
    {
    getArgument=_file;
    host=_host;
    port=_port;
    }
  unsigned int ipAddr=ArSystem::getIPAddress(host);
  if(!ipAddr)
    {
    setErrorMessage("Cannot find IP address of `"+host+"' --- "+
                    ArSystem::printError(errno));
    return;
    }
  _fd=ArSystem::connectTCP(ipAddr,port);
  if(_fd==-1)
    {
    setErrorMessage("Cannot connect to `"+host+":"+intToStr(port)+"' --- "+
                    ArSystem::printError(errno));
    return;
    }
  else
    {
    ArRef<FDOStream> out=FDOStream::NEW(_fd,false);
    out->stream() << "GET " << getArgument << " HTTP/1.0\r\n"
                  << "Host: " << _host << ":" << _port << "\r\n\r\n";
    }
  setErrorMessage("");
  _stream.clear();
  StlString line;
  if(!getline(_stream,line)||_stream.fail())
    {
    setErrorMessage("Cannot read HTTP header");
    return;
    }
  int status;
  if((sscanf(line.c_str(),"HTTP/%*[^ \t]%*[ \t]%d",&status)!=1)||
     (status!=200))
    {
    setErrorMessage("Bad HTTP header `"+line+"'");
    _stream.setstate(ios::badbit);
    return;
    }
  do
    {
    if(!getline(_stream,line)||_stream.fail())
      {
      setErrorMessage("Cannot read HTTP header");
      return;
      }
    } while(line.size()&&(line!="\r")); // end of header
  }
else if(_protocol=="file")
  {
  _fd=ArSystem::openInFile(_file.c_str());
  if(_fd==-1)
    {
    setErrorMessage("Cannot open file `"+_file+"' --- "+
                    ArSystem::printError(errno));
    }
  else
    {
    setErrorMessage("");
    _stream.clear();
    }
  }
else
  {
  setErrorMessage("Protocol `"+_protocol+"' is not supported");
  }
}

URLStream::~URLStream(void) // Protected
{
  // Nothing to be done
}

//-------- URL informations --------------------------------------------------

const StlString & // URL
URLStream::getURL(void) const
{
return(_url);
}

const StlString & // protocol ("file","http")
URLStream::getProtocol(void) const
{
return(_protocol);
}

const StlString & // host name
URLStream::getHostName(void) const
{
return(_host);
}

int // port number
URLStream::getPortNumber(void) const
{
return(_port);
}

const StlString & // file name
URLStream::getFileName(void) const
{
return(_file);
}

} // namespace AReVi

//----------------------------------------------------------------------------
