// LI2/ENIB - AReVi - VRML services
//----------------------------------------------------------------------------
// file        : AReVi/VRML/anchorNode_priv.cpp
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/VRML/anchorNode_priv.h"

#include "AReVi/VRML/vrmlValue_priv.h"
#include "AReVi/VRML/vrmlShape3D.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class AnchorNode
//----------------------------------------------------------------------------

//-------- Construction / Destruction ----------------------------------------

AnchorNode::~AnchorNode(void)
{
  // Nothing to be done
}

//-------- Field management --------------------------------------------------

void
AnchorNode::addField(const StlString & fieldName,
                     VrmlValue * fieldValue)
{
bool ok=false;
if(fieldName=="description")
  {
  if(fieldValue->checkSFString())
    {
    _descriptionField=fieldValue->getString(0);
    ok=true;
    }
  }
else if(fieldName=="parameter")
  {
  if(fieldValue->checkMFString())
    {
    _parameterField.clear();
    unsigned int nb=fieldValue->getSize();
    for(unsigned int i=0;i<nb;i++)
      {
      _parameterField.push_back(fieldValue->getString(i));
      }
    ok=true;
    }
  }
else if(fieldName=="url")
  {
  if(fieldValue->checkMFString())
    {
    _urlField.clear();
    unsigned int nb=fieldValue->getSize();
    for(unsigned int i=0;i<nb;i++)
      {
      _urlField.push_back(fieldValue->getString(i));
      }
    ok=true;
    }
  }
else
  {
  GroupNode::addField(fieldName,fieldValue);
  ok=true;
  }
if(!ok)
  {
  _shape->_badFieldMsg(fieldName,_type);
  }
}

void
AnchorNode::check(void)
{
_fail=false;
StlString msg="Node type '";
msg+=_type;
msg+="' is considered as 'Group'";
_shape->_errorMsg(msg);
}

//-------- Forbidden operations ----------------------------------------------

void
AnchorNode::_generateFields(ostream & output,
                            unsigned int indent,
                            bool showDefaultValues,
                            StlVector<const VrmlNode *> & defNodesInOut)
                                                            const // Protected
{
if(showDefaultValues||
   _descriptionField.size())
  {
  _generateSFString(output,indent,"description",_descriptionField);
  }
if(showDefaultValues||
   !_parameterField.empty())
  {
  _generateMFString(output,indent,"parameter",_parameterField);
  }
if(showDefaultValues||
   !_urlField.empty())
  {
  _generateMFString(output,indent,"url",_urlField);
  }
GroupNode::_generateFields(output,indent,showDefaultValues,defNodesInOut);
}

AnchorNode::AnchorNode(const StlString & nodeType,
                       VrmlShape3D * shape) // Protected
: GroupNode(nodeType,shape),
  _descriptionField(),
  _parameterField(),
  _urlField()
{
  // Nothing to be done
}

} // namespace AReVi

//----------------------------------------------------------------------------
