// LI2/ENIB - AReVi - VRML services
//----------------------------------------------------------------------------
// file        : AReVi/VRML/appearanceNode_priv.cpp
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/VRML/appearanceNode_priv.h"

#include "AReVi/VRML/vrmlValue_priv.h"
#include "AReVi/VRML/vrmlShape3D.h"

#include "AReVi/VRML/shapeNode_priv.h"
#include "AReVi/VRML/materialNode_priv.h"
#include "AReVi/VRML/textureNode_priv.h"
#include "AReVi/VRML/textureTransformNode_priv.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class AppearanceNode
//----------------------------------------------------------------------------

//-------- Construction / Destruction ----------------------------------------

AppearanceNode::~AppearanceNode(void)
{
if(_materialField)
  {
  _materialField->removeParent(this);
  }
if(_textureField)
  {
  _textureField->removeParent(this);
  }
if(_textureTransformField)
  {
  _textureTransformField->removeParent(this);
  }
if(_texture2Field)
  {
  _texture2Field->removeParent(this);
  }
if(_textureTransform2Field)
  {
  _textureTransform2Field->removeParent(this);
  }
if(_texture3Field)
  {
  _texture3Field->removeParent(this);
  }
if(_textureTransform3Field)
  {
  _textureTransform3Field->removeParent(this);
  }
if(_texture4Field)
  {
  _texture4Field->removeParent(this);
  }
if(_textureTransform4Field)
  {
  _textureTransform4Field->removeParent(this);
  }
}

//-------- Field management --------------------------------------------------

void
AppearanceNode::getNodeFieldNames(StlVector<StlString> & nodeFieldNamesOut)
                                                                         const
{
nodeFieldNamesOut.clear();
nodeFieldNamesOut.push_back("material");
nodeFieldNamesOut.push_back("texture");
nodeFieldNamesOut.push_back("textureTransform");
nodeFieldNamesOut.push_back("texture2");
nodeFieldNamesOut.push_back("textureTransform2");
nodeFieldNamesOut.push_back("texture3");
nodeFieldNamesOut.push_back("textureTransform3");
nodeFieldNamesOut.push_back("texture4");
nodeFieldNamesOut.push_back("textureTransform4");
}

const VrmlNode * // node at field
AppearanceNode::getNodeField(const StlString & fieldName) const
{
if(fieldName=="material")
  {
  return(_materialField);
  }
else if(fieldName=="texture")
  {
  return(_textureField);
  }
else if(fieldName=="textureTransform")
  {
  return(_textureTransformField);
  }
else if(fieldName=="texture2")
  {
  return(_texture2Field);
  }
else if(fieldName=="textureTransform2")
  {
  return(_textureTransform2Field);
  }
else if(fieldName=="texture3")
  {
  return(_texture3Field);
  }
else if(fieldName=="textureTransform3")
  {
  return(_textureTransform3Field);
  }
else if(fieldName=="texture4")
  {
  return(_texture4Field);
  }
else if(fieldName=="textureTransform4")
  {
  return(_textureTransform4Field);
  }
else
  {
  return((const VrmlNode *)0);
  }
}

void
AppearanceNode::addField(const StlString & fieldName,
                         VrmlValue * fieldValue)
{
bool ok=false;
if(fieldName=="material")
  {
  if(fieldValue->checkSFNode())
    {
    VrmlNode * n=fieldValue->accessNode(0);
    if(!n||(n->getType()=="Material"))
      {
      if(!(n&&n->fail()))
        {
        if(_materialField)
          {
          _materialField->removeParent(this);
          }
        _materialField=(MaterialNode *)n;
        if(_materialField)
          {
          _materialField->addParent(this);
          }
        }
      ok=true;
      }
    }
  }
else if(fieldName=="texture")
  {
  if(fieldValue->checkSFNode())
    {
    VrmlNode * n=fieldValue->accessNode(0);
    if(!n||n->isTexture())
      {
      if(!(n&&n->fail()))
        {
        if(_textureField)
          {
          _textureField->removeParent(this);
          }
        _textureField=(TextureNode *)n;
        if(_textureField)
          {
          _textureField->addParent(this);
          }
        }
      ok=true;
      }
    }
  }
else if(fieldName=="textureTransform")
  {
  if(fieldValue->checkSFNode())
    {
    VrmlNode * n=fieldValue->accessNode(0);
    if(!n||(n->getType()=="TextureTransform"))
      {
      if(!(n&&n->fail()))
        {
        if(_textureTransformField)
          {
          _textureTransformField->removeParent(this);
          }
        _textureTransformField=(TextureTransformNode *)n;
        if(_textureTransformField)
          {
          _textureTransformField->addParent(this);
          }
        }
      ok=true;
      }
    }
  }
else if(fieldName=="texture2")
  {
  if(fieldValue->checkSFNode())
    {
    VrmlNode * n=fieldValue->accessNode(0);
    if(!n||n->isTexture())
      {
      if(!(n&&n->fail()))
        {
        if(_texture2Field)
          {
          _texture2Field->removeParent(this);
          }
        _texture2Field=(TextureNode *)n;
        if(_texture2Field)
          {
          _texture2Field->addParent(this);
          }
        }
      ok=true;
      }
    }
  }
else if(fieldName=="textureTransform2")
  {
  if(fieldValue->checkSFNode())
    {
    VrmlNode * n=fieldValue->accessNode(0);
    if(!n||(n->getType()=="TextureTransform"))
      {
      if(!(n&&n->fail()))
        {
        if(_textureTransform2Field)
          {
          _textureTransform2Field->removeParent(this);
          }
        _textureTransform2Field=(TextureTransformNode *)n;
        if(_textureTransform2Field)
          {
          _textureTransform2Field->addParent(this);
          }
        }
      ok=true;
      }
    }
  }
else if(fieldName=="texture3")
  {
  if(fieldValue->checkSFNode())
    {
    VrmlNode * n=fieldValue->accessNode(0);
    if(!n||n->isTexture())
      {
      if(!(n&&n->fail()))
        {
        if(_texture3Field)
          {
          _texture3Field->removeParent(this);
          }
        _texture3Field=(TextureNode *)n;
        if(_texture3Field)
          {
          _texture3Field->addParent(this);
          }
        }
      ok=true;
      }
    }
  }
else if(fieldName=="textureTransform3")
  {
  if(fieldValue->checkSFNode())
    {
    VrmlNode * n=fieldValue->accessNode(0);
    if(!n||(n->getType()=="TextureTransform"))
      {
      if(!(n&&n->fail()))
        {
        if(_textureTransform3Field)
          {
          _textureTransform3Field->removeParent(this);
          }
        _textureTransform3Field=(TextureTransformNode *)n;
        if(_textureTransform3Field)
          {
          _textureTransform3Field->addParent(this);
          }
        }
      ok=true;
      }
    }
  }
else if(fieldName=="texture4")
  {
  if(fieldValue->checkSFNode())
    {
    VrmlNode * n=fieldValue->accessNode(0);
    if(!n||n->isTexture())
      {
      if(!(n&&n->fail()))
        {
        if(_texture4Field)
          {
          _texture4Field->removeParent(this);
          }
        _texture4Field=(TextureNode *)n;
        if(_texture4Field)
          {
          _texture4Field->addParent(this);
          }
        }
      ok=true;
      }
    }
  }
else if(fieldName=="textureTransform4")
  {
  if(fieldValue->checkSFNode())
    {
    VrmlNode * n=fieldValue->accessNode(0);
    if(!n||(n->getType()=="TextureTransform"))
      {
      if(!(n&&n->fail()))
        {
        if(_textureTransform4Field)
          {
          _textureTransform4Field->removeParent(this);
          }
        _textureTransform4Field=(TextureTransformNode *)n;
        if(_textureTransform4Field)
          {
          _textureTransform4Field->addParent(this);
          }
        }
      ok=true;
      }
    }
  }
else
  {
  VrmlNode::addField(fieldName,fieldValue);
  ok=true;
  }
if(!ok)
  {
  _shape->_badFieldMsg(fieldName,_type);
  }
}

void
AppearanceNode::check(void)
{
_fail=false;
}

//-------- Rendering ---------------------------------------------------------

void
AppearanceNode::postMaterialRebuild(void)
{
for(unsigned int i=_parents.size();i--;)
  {
  static_cast<ShapeNode *>(_parents[i])->postMaterialRebuild();
  }
}

void
AppearanceNode::postTextureRebuild(const TextureNode * from)
{
unsigned int which=(from==_textureField ? 1 : 2);
for(unsigned int i=_parents.size();i--;)
  {
  static_cast<ShapeNode *>(_parents[i])->postTextureRebuild(which);
  }
}

void
AppearanceNode::postTextureTransformRebuild(const TextureTransformNode * from)
{
unsigned int which=(from==_textureTransformField ? 1 : 2);
for(unsigned int i=_parents.size();i--;)
  {
  static_cast<ShapeNode *>(_parents[i])->postTextureTransformRebuild(which);
  }
}

//-------- Forbidden operations ----------------------------------------------

void
AppearanceNode::_generateFields(ostream & output,
                                unsigned int indent,
                                bool showDefaultValues,
                                StlVector<const VrmlNode *> & defNodesInOut)
                                                            const // Protected
{
if(showDefaultValues||
   _materialField)
  {
  _generateSFNode(output,indent,"material",_materialField,
                  showDefaultValues,defNodesInOut);
  }
if(showDefaultValues||
   _textureField)
  {
  _generateSFNode(output,indent,"texture",_textureField,
                  showDefaultValues,defNodesInOut);
  }
if(showDefaultValues||
   _textureTransformField)
  {
  _generateSFNode(output,indent,"textureTransform",_textureTransformField,
                  showDefaultValues,defNodesInOut);
  }
if(showDefaultValues||
   _texture2Field)
  {
  _generateSFNode(output,indent,"texture2",_texture2Field,
                  showDefaultValues,defNodesInOut);
  }
if(showDefaultValues||
   _textureTransform2Field)
  {
  _generateSFNode(output,indent,"textureTransform2",_textureTransform2Field,
                  showDefaultValues,defNodesInOut);
  }
if(showDefaultValues||
   _texture3Field)
  {
  _generateSFNode(output,indent,"texture3",_texture3Field,
                  showDefaultValues,defNodesInOut);
  }
if(showDefaultValues||
   _textureTransform3Field)
  {
  _generateSFNode(output,indent,"textureTransform3",_textureTransform3Field,
                  showDefaultValues,defNodesInOut);
  }
if(showDefaultValues||
   _texture4Field)
  {
  _generateSFNode(output,indent,"texture4",_texture4Field,
                  showDefaultValues,defNodesInOut);
  }
if(showDefaultValues||
   _textureTransform4Field)
  {
  _generateSFNode(output,indent,"textureTransform4",_textureTransform4Field,
                  showDefaultValues,defNodesInOut);
  }
}

AppearanceNode::AppearanceNode(const StlString & nodeType,
                               VrmlShape3D * shape) // Protected
: VrmlNode(nodeType,shape),
  _materialField((MaterialNode *)0),
  _textureField((TextureNode *)0),
  _textureTransformField((TextureTransformNode *)0),
  _texture2Field((TextureNode *)0),
  _textureTransform2Field((TextureTransformNode *)0),
  _texture3Field((TextureNode *)0),
  _textureTransform3Field((TextureTransformNode *)0),
  _texture4Field((TextureNode *)0),
  _textureTransform4Field((TextureTransformNode *)0)
{
  // Nothing to be done
}

} // namespace AReVi

//----------------------------------------------------------------------------
