// LI2/ENIB - AReVi - VRML services
//----------------------------------------------------------------------------
// file        : AReVi/VRML/billboardNode_priv.cpp
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/VRML/billboardNode_priv.h"

#include "AReVi/VRML/vrmlValue_priv.h"
#include "AReVi/VRML/vrmlShape3D.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class BillboardNode
//----------------------------------------------------------------------------

//-------- Construction / Destruction ----------------------------------------

BillboardNode::~BillboardNode(void)
{
  // Nothing to be done
}

//-------- Field management --------------------------------------------------

void
BillboardNode::addField(const StlString & fieldName,
                        VrmlValue * fieldValue)
{
bool ok=false;
if(fieldName=="axisOfRotation")
  {
  if(fieldValue->checkSFVec3f())
    {
    _axisOfRotationField.x=fieldValue->getReal(0);
    _axisOfRotationField.y=fieldValue->getReal(1);
    _axisOfRotationField.z=fieldValue->getReal(2);
    ok=true;
    }
  }
else
  {
  GroupNode::addField(fieldName,fieldValue);
  ok=true;
  }
if(!ok)
  {
  _shape->_badFieldMsg(fieldName,_type);
  }
}

void
BillboardNode::check(void)
{
_fail=false;
StlString msg="Node type '";
msg+=_type;
msg+="' is considered as 'Group'";
_shape->_errorMsg(msg);
}

//-------- Forbidden operations ----------------------------------------------

void
BillboardNode::_generateFields(ostream & output,
                               unsigned int indent,
                               bool showDefaultValues,
                               StlVector<const VrmlNode *> & defNodesInOut)
                                                            const // Protected
{
if(showDefaultValues||
   (_axisOfRotationField.x!=0.0f)||
   (_axisOfRotationField.y!=1.0f)||
   (_axisOfRotationField.z!=0.0f))
  {
  _generateSFVec3f(output,indent,"axisOfRotation",_axisOfRotationField);
  }
GroupNode::_generateFields(output,indent,showDefaultValues,defNodesInOut);
}

BillboardNode::BillboardNode(const StlString & nodeType,
                             VrmlShape3D * shape) // Protected
: GroupNode(nodeType,shape),
  _axisOfRotationField(0.0,1.0,0.0)
{
  // Nothing to be done
}

} // namespace AReVi

//----------------------------------------------------------------------------
