// LI2/ENIB - AReVi - VRML services
//----------------------------------------------------------------------------
// file        : AReVi/VRML/boxNode_priv.cpp
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/VRML/boxNode_priv.h"

#include "AReVi/VRML/vrmlValue_priv.h"
#include "AReVi/VRML/vrmlShape3D.h"

#include "AReVi/Shapes/box3D.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class BoxNode
//----------------------------------------------------------------------------

//-------- Construction / Destruction ----------------------------------------

BoxNode::~BoxNode(void)
{
  // Nothing to be done
}

//-------- Field management --------------------------------------------------

void
BoxNode::addField(const StlString & fieldName,
                  VrmlValue * fieldValue)
{
bool ok=false;
if(fieldName=="size")
  {
  if(fieldValue->checkSFVec3f()&&
     (fieldValue->getReal(0)>0.0)&&
     (fieldValue->getReal(1)>0.0)&&
     (fieldValue->getReal(2)>0.0))
    {
    _sizeField.x=fieldValue->getReal(0);
    _sizeField.y=fieldValue->getReal(1);
    _sizeField.z=fieldValue->getReal(2);
    ok=true;
    }
  }
else if(fieldName=="ccw")
  {
  if(fieldValue->checkSFBool())
    {
    _ccwField=fieldValue->getBool(0);
    ok=true;
    }
  }
else if(fieldName=="solid")
  {
  if(fieldValue->checkSFBool())
    {
    _solidField=fieldValue->getBool(0);
    ok=true;
    }
  }
else
  {
  GeometryNode::addField(fieldName,fieldValue);
  ok=true;
  }
if(!ok)
  {
  _shape->_badFieldMsg(fieldName,_type);
  }
}

void
BoxNode::check(void)
{
_fail=false;
if(!_fail)
  {
  rebuild();
  }
}

//-------- Rendering ---------------------------------------------------------

ArRef<ShapePart3D> // built part
BoxNode::buildPart(Util3D::Transform & initTransformOut) const
{
initTransformOut.identity();
initTransformOut.postYaw(-M_PI_2);
initTransformOut.postPitch(-M_PI_2);
return(Box3D::NEW());
}

void
BoxNode::updatePart(ArRef<ShapePart3D> part,
                    const MaterialNode * /* material */) const
{
ArRef<Box3D> box=ar_down_cast<Box3D>(part);
box->setSize(_sizeField.z,_sizeField.x,_sizeField.y); // zxy due to rotation
box->setCCW(_ccwField);
box->setDoubleSided(!_solidField);
}

//-------- Forbidden operations ----------------------------------------------

void
BoxNode::_generateFields(ostream & output,
                         unsigned int indent,
                         bool showDefaultValues,
                         StlVector<const VrmlNode *> & /* defNodesInOut */)
                                                            const // Protected
{
if(showDefaultValues||
   (_sizeField.x!=2.0f)||
   (_sizeField.y!=2.0f)||
   (_sizeField.z!=2.0f))
  {
  _generateSFVec3f(output,indent,"size",_sizeField);
  }
if(showDefaultValues||
   !_ccwField)
  {
  _generateSFBool(output,indent,"ccw",_ccwField);
  }
if(showDefaultValues||
   !_solidField)
  {
  _generateSFBool(output,indent,"solid",_solidField);
  }
}

BoxNode::BoxNode(const StlString & nodeType,
                 VrmlShape3D * shape) // Protected
: GeometryNode(nodeType,shape),
  _sizeField(2.0,2.0,2.0),
  _ccwField(true),
  _solidField(true)
{
  // Nothing to be done
}

} // namespace AReVi

//----------------------------------------------------------------------------
