// LI2/ENIB - AReVi - VRML services
//----------------------------------------------------------------------------
// file        : AReVi/VRML/collisionNode_priv.cpp
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/VRML/collisionNode_priv.h"

#include "AReVi/VRML/vrmlValue_priv.h"
#include "AReVi/VRML/vrmlShape3D.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class CollisionNode
//----------------------------------------------------------------------------

//-------- Construction / Destruction ----------------------------------------

CollisionNode::~CollisionNode(void)
{
if(_proxyField)
  {
  _proxyField->removeParent(this);
  }
}

//-------- Field management --------------------------------------------------

void
CollisionNode::addField(const StlString & fieldName,
                        VrmlValue * fieldValue)
{
bool ok=false;
if(fieldName=="collide")
  {
  if(fieldValue->checkSFBool())
    {
    _collideField=fieldValue->getBool(0);
    ok=true;
    }
  }
else if(fieldName=="proxy")
  {
  if(fieldValue->checkSFNode())
    {
    VrmlNode * n=fieldValue->accessNode(0);
    if(!n||n->isGroupChild())
      {
      if(!(n&&n->fail()))
        {
        if(_proxyField)
          {
          _proxyField->removeParent(this);
          }
        _proxyField=(GroupChildNode *)n;
        if(_proxyField)
          {
          _proxyField->addParent(this);
          }
        }
      ok=true;
      }
    }
  }
else
  {
  GroupNode::addField(fieldName,fieldValue);
  ok=true;
  }
if(!ok)
  {
  _shape->_badFieldMsg(fieldName,_type);
  }
}

void
CollisionNode::check(void)
{
_fail=false;
StlString msg="Node type '";
msg+=_type;
msg+="' is considered as 'Group'";
_shape->_errorMsg(msg);
}

//-------- Forbidden operations ----------------------------------------------

void
CollisionNode::_generateFields(ostream & output,
                               unsigned int indent,
                               bool showDefaultValues,
                               StlVector<const VrmlNode *> & defNodesInOut)
                                                            const // Protected
{
if(showDefaultValues||
   !_collideField)
  {
  _generateSFBool(output,indent,"collide",_collideField);
  }
if(showDefaultValues||
   _proxyField)
  {
  _generateSFNode(output,indent,"proxy",_proxyField,
                  showDefaultValues,defNodesInOut);
  }
GroupNode::_generateFields(output,indent,showDefaultValues,defNodesInOut);
}

CollisionNode::CollisionNode(const StlString & nodeType,
                             VrmlShape3D * shape) // Protected
: GroupNode(nodeType,shape),
  _collideField(true),
  _proxyField((GroupChildNode *)0)
{
  // Nothing to be done
}

} // namespace AReVi

//----------------------------------------------------------------------------
