// LI2/ENIB - AReVi - VRML services
//----------------------------------------------------------------------------
// file        : AReVi/VRML/cylinderNode_priv.cpp
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/VRML/cylinderNode_priv.h"

#include "AReVi/VRML/vrmlValue_priv.h"
#include "AReVi/VRML/vrmlShape3D.h"

#include "AReVi/Shapes/cylinder3D.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class CylinderNode
//----------------------------------------------------------------------------

//-------- Construction / Destruction ----------------------------------------

CylinderNode::~CylinderNode(void)
{
  // Nothing to be done
}

//-------- Field management --------------------------------------------------

void
CylinderNode::addField(const StlString & fieldName,
                       VrmlValue * fieldValue)
{
bool ok=false;
if(fieldName=="height")
  {
  if(fieldValue->checkSFFloat()&&
     (fieldValue->getReal(0)>0.0))
    {
    _heightField=fieldValue->getReal(0);
    ok=true;
    }
  }
else if(fieldName=="radius")
  {
  if(fieldValue->checkSFFloat()&&
     (fieldValue->getReal(0)>0.0))
    {
    _radiusField=fieldValue->getReal(0);
    ok=true;
    }
  }
else if(fieldName=="bottom")
  {
  if(fieldValue->checkSFBool())
    {
    _bottomField=fieldValue->getBool(0);
    ok=true;
    }
  }
else if(fieldName=="side")
  {
  if(fieldValue->checkSFBool())
    {
    _sideField=fieldValue->getBool(0);
    ok=true;
    }
  }
else if(fieldName=="top")
  {
  if(fieldValue->checkSFBool())
    {
    _topField=fieldValue->getBool(0);
    ok=true;
    }
  }
else if(fieldName=="ccw")
  {
  if(fieldValue->checkSFBool())
    {
    _ccwField=fieldValue->getBool(0);
    ok=true;
    }
  }
else if(fieldName=="solid")
  {
  if(fieldValue->checkSFBool())
    {
    _solidField=fieldValue->getBool(0);
    ok=true;
    }
  }
else
  {
  GeometryNode::addField(fieldName,fieldValue);
  ok=true;
  }
if(!ok)
  {
  _shape->_badFieldMsg(fieldName,_type);
  }
}

void
CylinderNode::check(void)
{
_fail=false;
if(!_fail)
  {
  rebuild();
  }
}

//-------- Rendering ---------------------------------------------------------

ArRef<ShapePart3D> // built part
CylinderNode::buildPart(Util3D::Transform & initTransformOut) const
{
initTransformOut.identity();
initTransformOut.postYaw(-M_PI_2);
initTransformOut.postPitch(-M_PI_2);
return(Cylinder3D::NEW());
}

void
CylinderNode::updatePart(ArRef<ShapePart3D> part,
                         const MaterialNode * /* material */) const
{
ArRef<Cylinder3D> cylinder=ar_down_cast<Cylinder3D>(part);
cylinder->setRadius(_radiusField);
cylinder->setHeight(_heightField);
cylinder->setBottom(_bottomField);
cylinder->setSide(_sideField);
cylinder->setTop(_topField);
cylinder->setCCW(_ccwField);
cylinder->setDoubleSided(!_solidField);
}

//-------- Forbidden operations ----------------------------------------------

void
CylinderNode::_generateFields(ostream & output,
                              unsigned int indent,
                              bool showDefaultValues,
                            StlVector<const VrmlNode *> & /* defNodesInOut */)
                                                            const // Protected
{
if(showDefaultValues||
   (_heightField!=2.0f))
  {
  _generateSFFloat(output,indent,"height",_heightField);
  }
if(showDefaultValues||
   (_radiusField!=1.0f))
  {
  _generateSFFloat(output,indent,"radius",_radiusField);
  }
if(showDefaultValues||
   !_bottomField)
  {
  _generateSFBool(output,indent,"bottom",_bottomField);
  }
if(showDefaultValues||
   !_sideField)
  {
  _generateSFBool(output,indent,"side",_sideField);
  }
if(showDefaultValues||
   !_topField)
  {
  _generateSFBool(output,indent,"top",_topField);
  }
if(showDefaultValues||
   !_ccwField)
  {
  _generateSFBool(output,indent,"ccw",_ccwField);
  }
if(showDefaultValues||
   !_solidField)
  {
  _generateSFBool(output,indent,"solid",_solidField);
  }
}

CylinderNode::CylinderNode(const StlString & nodeType,
                           VrmlShape3D * shape) // Protected
: GeometryNode(nodeType,shape),
  _heightField(2.0),
  _radiusField(1.0),
  _bottomField(true),
  _sideField(true),
  _topField(true),
  _ccwField(true),
  _solidField(true)
{
  // Nothing to be done
}

} // namespace AReVi

//----------------------------------------------------------------------------
