// LI2/ENIB - AReVi - VRML services
//----------------------------------------------------------------------------
// file        : AReVi/VRML/elevationGridNode_priv.cpp
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/VRML/elevationGridNode_priv.h"

#include "AReVi/VRML/vrmlValue_priv.h"
#include "AReVi/VRML/vrmlShape3D.h"

#include "AReVi/arSystem.h"

#include "AReVi/VRML/colorNode_priv.h"
#include "AReVi/VRML/normalNode_priv.h"
#include "AReVi/VRML/textureCoordinateNode_priv.h"

#include "AReVi/Shapes/mesh3D.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class ElevationGridNode
//----------------------------------------------------------------------------

//-------- Construction / Destruction ----------------------------------------

ElevationGridNode::~ElevationGridNode(void)
{
if(_colorField)
  {
  _colorField->removeParent(this);
  }
if(_normalField)
  {
  _normalField->removeParent(this);
  }
if(_texCoordField)
  {
  _texCoordField->removeParent(this);
  }
}

//-------- Field management --------------------------------------------------

void
ElevationGridNode::getNodeFieldNames(StlVector<StlString> & nodeFieldNamesOut)
                                                                         const
{
nodeFieldNamesOut.clear();
nodeFieldNamesOut.push_back("color");
nodeFieldNamesOut.push_back("normal");
nodeFieldNamesOut.push_back("texCoord");
}

const VrmlNode * // node at field
ElevationGridNode::getNodeField(const StlString & fieldName) const
{
if(fieldName=="color")
  {
  return(_colorField);
  }
else if(fieldName=="normal")
  {
  return(_normalField);
  }
else if(fieldName=="texCoord")
  {
  return(_texCoordField);
  }
else if(fieldName=="texCoord2")
  {
  return(_texCoord2Field);
  }
else if(fieldName=="texCoord3")
  {
  return(_texCoord3Field);
  }
else if(fieldName=="texCoord4")
  {
  return(_texCoord4Field);
  }
else
  {
  return((const VrmlNode *)0);
  }
}

void
ElevationGridNode::addField(const StlString & fieldName,
                            VrmlValue * fieldValue)
{
bool ok=false;
if(fieldName=="color")
  {
  if(fieldValue->checkSFNode())
    {
    VrmlNode * n=fieldValue->accessNode(0);
    if(!n||(n->getType()=="Color"))
      {
      if(!(n&&n->fail()))
        {
        if(_colorField)
          {
          _colorField->removeParent(this);
          }
        _colorField=(ColorNode *)n;
        if(_colorField)
          {
          _colorField->addParent(this);
          }
        }
      ok=true;
      }
    }
  }
else if(fieldName=="normal")
  {
  if(fieldValue->checkSFNode())
    {
    VrmlNode * n=fieldValue->accessNode(0);
    if(!n||(n->getType()=="Normal"))
      {
      if(!(n&&n->fail()))
        {
        if(_normalField)
          {
          _normalField->removeParent(this);
          }
        _normalField=(NormalNode *)n;
        if(_normalField)
          {
          _normalField->addParent(this);
          }
        }
      ok=true;
      }
    }
  }
else if(fieldName=="texCoord")
  {
  if(fieldValue->checkSFNode())
    {
    VrmlNode * n=fieldValue->accessNode(0);
    if(!n||(n->getType()=="TextureCoordinate"))
      {
      if(!(n&&n->fail()))
        {
        if(_texCoordField)
          {
          _texCoordField->removeParent(this);
          }
        _texCoordField=(TextureCoordinateNode *)n;
        if(_texCoordField)
          {
          _texCoordField->addParent(this);
          }
        }
      ok=true;
      }
    }
  }
else if(fieldName=="texCoord2")
  {
  if(fieldValue->checkSFNode())
    {
    VrmlNode * n=fieldValue->accessNode(0);
    if(!n||(n->getType()=="TextureCoordinate"))
      {
      if(!(n&&n->fail()))
        {
        if(_texCoord2Field)
          {
          _texCoord2Field->removeParent(this);
          }
        _texCoord2Field=(TextureCoordinateNode *)n;
        if(_texCoord2Field)
          {
          _texCoord2Field->addParent(this);
          }
        }
      ok=true;
      }
    }
  }
else if(fieldName=="texCoord3")
  {
  if(fieldValue->checkSFNode())
    {
    VrmlNode * n=fieldValue->accessNode(0);
    if(!n||(n->getType()=="TextureCoordinate"))
      {
      if(!(n&&n->fail()))
        {
        if(_texCoord3Field)
          {
          _texCoord3Field->removeParent(this);
          }
        _texCoord3Field=(TextureCoordinateNode *)n;
        if(_texCoord3Field)
          {
          _texCoord3Field->addParent(this);
          }
        }
      ok=true;
      }
    }
  }
else if(fieldName=="texCoord4")
  {
  if(fieldValue->checkSFNode())
    {
    VrmlNode * n=fieldValue->accessNode(0);
    if(!n||(n->getType()=="TextureCoordinate"))
      {
      if(!(n&&n->fail()))
        {
        if(_texCoord4Field)
          {
          _texCoord4Field->removeParent(this);
          }
        _texCoord4Field=(TextureCoordinateNode *)n;
        if(_texCoord4Field)
          {
          _texCoord4Field->addParent(this);
          }
        }
      ok=true;
      }
    }
  }
else if(fieldName=="height")
  {
  if(fieldValue->checkMFFloat())
    {
    _heightField.clear();
    unsigned int nb=fieldValue->getSize();
    for(unsigned int i=0;i<nb;i++)
      {
      _heightField.push_back(fieldValue->getReal(i));
      }
    ok=true;
    }
  }
else if(fieldName=="ccw")
  {
  if(fieldValue->checkSFBool())
    {
    _ccwField=fieldValue->getBool(0);
    ok=true;
    }
  }
else if(fieldName=="colorPerVertex")
  {
  if(fieldValue->checkSFBool())
    {
    _colorPerVertexField=fieldValue->getBool(0);
    ok=true;
    }
  }
else if(fieldName=="creaseAngle")
  {
  if(fieldValue->checkSFFloat()&&
     (fieldValue->getReal(0)>=0.0))
    {
    _creaseAngleField=fieldValue->getReal(0);
    ok=true;
    }
  }
else if(fieldName=="normalPerVertex")
  {
  if(fieldValue->checkSFBool())
    {
    _normalPerVertexField=fieldValue->getBool(0);
    ok=true;
    }
  }
else if(fieldName=="solid")
  {
  if(fieldValue->checkSFBool())
    {
    _solidField=fieldValue->getBool(0);
    ok=true;
    }
  }
else if(fieldName=="xDimension")
  {
  if(fieldValue->checkSFInt32()&&
     (fieldValue->getInteger(0)>=0))
    {
    _xDimensionField=fieldValue->getInteger(0);
    ok=true;
    }
  }
else if(fieldName=="xSpacing")
  {
  if(fieldValue->checkSFFloat()&&
     (fieldValue->getReal(0)>0.0))
    {
    _xSpacingField=fieldValue->getReal(0);
    ok=true;
    }
  }
else if(fieldName=="zDimension")
  {
  if(fieldValue->checkSFInt32()&&
     (fieldValue->getInteger(0)>=0))
    {
    _zDimensionField=fieldValue->getInteger(0);
    ok=true;
    }
  }
else if(fieldName=="zSpacing")
  {
  if(fieldValue->checkSFFloat()&&
     (fieldValue->getReal(0)>0.0))
    {
    _zSpacingField=fieldValue->getReal(0);
    ok=true;
    }
  }
else
  {
  GeometryNode::addField(fieldName,fieldValue);
  ok=true;
  }
if(!ok)
  {
  _shape->_badFieldMsg(fieldName,_type);
  }
}

void
ElevationGridNode::check(void)
{
_fail=false;
if((_xDimensionField<2)||(_zDimensionField<2))
  {
  StlString msg="Both 'xDimension' and 'zDimension' fields should be";
  msg+=" greater than 2 in node type '";
  msg+=_type;
  msg+="'";
  _shape->_errorMsg(msg);
  _fail=true;
  return; // can't check other properties ...
  }
if(_heightField.size()<_xDimensionField*_zDimensionField)
  {
  StlString msg="Field 'height' should contain ";
  msg+=uintToStr(_xDimensionField);
  msg+="x";
  msg+=uintToStr(_zDimensionField);
  msg+=" values in node type '";
  msg+=_type;
  msg+="'";
  _shape->_errorMsg(msg);
  _fail=true;
  }
if(_colorField)
  {
  unsigned int xDim=_xDimensionField;
  unsigned int zDim=_zDimensionField;
  if(!_colorPerVertexField)
    {
    xDim--;
    zDim--;
    }
  if(_colorField->_colorField.size()<xDim*zDim)
    {
    StlString msg="Field 'color' should contain ";
    msg+=uintToStr(xDim);
    msg+="x";
    msg+=uintToStr(zDim);
    msg+=" colors in node type '";
    msg+=_type;
    msg+="'";
    _shape->_errorMsg(msg);
    _fail=true;
    }
  }
if(_normalField)
  {
  unsigned int xDim=_xDimensionField;
  unsigned int zDim=_zDimensionField;
  if(!_normalPerVertexField)
    {
    xDim--;
    zDim--;
    }
  if(_normalField->_vectorField.size()<xDim*zDim)
    {
    StlString msg="Field 'normal' should contain ";
    msg+=uintToStr(xDim);
    msg+="x";
    msg+=uintToStr(zDim);
    msg+=" normals in node type '";
    msg+=_type;
    msg+="'";
    _shape->_errorMsg(msg);
    _fail=true;
    }
  }
if(_texCoordField)
  {
  if(_texCoordField->_pointField.size()<_xDimensionField*_zDimensionField)
    {
    StlString msg="Field 'texCoord' should contain ";
    msg+=uintToStr(_xDimensionField);
    msg+="x";
    msg+=uintToStr(_zDimensionField);
    msg+=" points in node type '";
    msg+=_type;
    msg+="'";
    _shape->_errorMsg(msg);
    _fail=true;
    }
  }
if(_texCoord2Field)
  {
  if(_texCoord2Field->_pointField.size()<_xDimensionField*_zDimensionField)
    {
    StlString msg="Field 'texCoord2' should contain ";
    msg+=uintToStr(_xDimensionField);
    msg+="x";
    msg+=uintToStr(_zDimensionField);
    msg+=" points in node type '";
    msg+=_type;
    msg+="'";
    _shape->_errorMsg(msg);
    _fail=true;
    }
  }
if(_texCoord3Field)
  {
  if(_texCoord3Field->_pointField.size()<_xDimensionField*_zDimensionField)
    {
    StlString msg="Field 'texCoord3' should contain ";
    msg+=uintToStr(_xDimensionField);
    msg+="x";
    msg+=uintToStr(_zDimensionField);
    msg+=" points in node type '";
    msg+=_type;
    msg+="'";
    _shape->_errorMsg(msg);
    _fail=true;
    }
  }
if(_texCoord4Field)
  {
  if(_texCoord4Field->_pointField.size()<_xDimensionField*_zDimensionField)
    {
    StlString msg="Field 'texCoord4' should contain ";
    msg+=uintToStr(_xDimensionField);
    msg+="x";
    msg+=uintToStr(_zDimensionField);
    msg+=" points in node type '";
    msg+=_type;
    msg+="'";
    _shape->_errorMsg(msg);
    _fail=true;
    }
  }
if(!_fail)
  {
  rebuild();
  }
}

//-------- Rendering ---------------------------------------------------------

ArRef<ShapePart3D> // built part
ElevationGridNode::buildPart(Util3D::Transform & initTransformOut) const
{
initTransformOut.identity();
return(Mesh3D::NEW());
}

void
ElevationGridNode::updatePart(ArRef<ShapePart3D> part,
                              const MaterialNode * /* material */) const
{
ArRef<Mesh3D> mesh=ar_down_cast<Mesh3D>(part);
StlVector<Util3D::Dbl3> & v=mesh->accessVertices();
v.clear();
StlVector<Util3D::Face> & vf=mesh->accessVertexFaces();
vf.clear();
StlVector<Util3D::Dbl3> & n=mesh->accessNormals();
n.clear();
StlVector<Util3D::Face> & nf=mesh->accessNormalFaces();
nf.clear();
StlVector<Util3D::Dbl2> & t=mesh->accessTextures();
t.clear();
StlVector<Util3D::Face> & tf=mesh->accessTextureFaces();
tf.clear();
StlVector<Util3D::Dbl2> & t2=mesh->accessSecondTextures();
t2.clear();
StlVector<Util3D::Face> & tf2=mesh->accessSecondTextureFaces();
tf2.clear();
StlVector<Util3D::Dbl2> & t3=mesh->accessThirdTextures();
t3.clear();
StlVector<Util3D::Face> & tf3=mesh->accessThirdTextureFaces();
tf3.clear();
StlVector<Util3D::Dbl2> & t4=mesh->accessFourthTextures();
t4.clear();
StlVector<Util3D::Face> & tf4=mesh->accessFourthTextureFaces();
tf4.clear();
StlVector<Util3D::Dbl3> & c=mesh->accessColors();
c.clear();
StlVector<Util3D::Face> & cf=mesh->accessColorFaces();
cf.clear();
unsigned int i,j;
for(i=0;i<_zDimensionField;i++)
  {
  for(j=0;j<_xDimensionField;j++)
    {
    v.push_back(Util3D::Dbl3(_xSpacingField*j,
                             _heightField[j+i*_xDimensionField],
                             _zSpacingField*i));
    }
  }
if(_ccwField)
  {
  for(i=1;i<_zDimensionField;i++)
    {
    for(j=1;j<_xDimensionField;j++)
      {
      vf.push_back(Util3D::Face((j-1)+(i-1)*_xDimensionField,
                                (j-1)+(i)*_xDimensionField,
                                (j)+(i)*_xDimensionField));
      vf.push_back(Util3D::Face((j-1)+(i-1)*_xDimensionField,
                                (j)+(i)*_xDimensionField,
                                (j)+(i-1)*_xDimensionField));
      }
    }
  }
else
  {
  for(i=1;i<_zDimensionField;i++)
    {
    for(j=1;j<_xDimensionField;j++)
      {
      vf.push_back(Util3D::Face((j-1)+(i-1)*_xDimensionField,
                                (j)+(i)*_xDimensionField,
                                (j-1)+(i)*_xDimensionField));
      vf.push_back(Util3D::Face((j-1)+(i-1)*_xDimensionField,
                                (j)+(i-1)*_xDimensionField,
                                (j)+(i)*_xDimensionField));
      }
    }
  }
if(_colorField)
  {
  c=_colorField->_colorField;
  if(!_colorPerVertexField)
    {
    for(i=1;i<_zDimensionField;i++)
      {
      for(j=1;j<_xDimensionField;j++)
        {
        unsigned int index=(j-1)+(i-1)*_xDimensionField;
        cf.push_back(Util3D::Face(index,index,index));
        cf.push_back(Util3D::Face(index,index,index));
        }
      }
    }
  // else --> vertex faces will be reused
  }
if(_normalField)
  {
  n=_normalField->_vectorField;
  if(!_normalPerVertexField)
    {
    for(i=1;i<_zDimensionField;i++)
      {
      for(j=1;j<_xDimensionField;j++)
        {
        unsigned int index=(j-1)+(i-1)*_xDimensionField;
        nf.push_back(Util3D::Face(index,index,index));
        nf.push_back(Util3D::Face(index,index,index));
        }
      }
    }
  // else --> vertex faces will be reused
  }
else
  {
  mesh->computeDefaultNormals(_normalPerVertexField);
  }
if(_texCoordField)
  {
  t=_texCoordField->_pointField;
  // --> vertex faces will be reused
  }
else // compute default texture coordinates
  {
  double xStep=1.0/(_xDimensionField-1);
  double zStep=1.0/(_zDimensionField-1);
  for(i=0;i<_zDimensionField;i++)
    {
    for(j=0;j<_xDimensionField;j++)
      {
      t.push_back(Util3D::Dbl2(j*xStep,i*zStep));
      }
    }
  // --> vertex faces will be reused
  }
if(_texCoord2Field)
  {
  t2=_texCoord2Field->_pointField;
  // --> vertex faces will be reused
  }
// else --> first texture coordinates will be reused
if(_texCoord3Field)
  {
  t3=_texCoord3Field->_pointField;
  // --> vertex faces will be reused
  }
// else --> first texture coordinates will be reused
if(_texCoord4Field)
  {
  t4=_texCoord4Field->_pointField;
  // --> vertex faces will be reused
  }
// else --> first texture coordinates will be reused
mesh->setDoubleSided(!_solidField);
mesh->applyChanges(false);
}

//-------- Forbidden operations ----------------------------------------------

void
ElevationGridNode::_generateFields(ostream & output,
                                   unsigned int indent,
                                   bool showDefaultValues,
                                  StlVector<const VrmlNode *> & defNodesInOut)
                                                            const // Protected
{
if(showDefaultValues||
   _colorField)
  {
  _generateSFNode(output,indent,"color",_colorField,
                  showDefaultValues,defNodesInOut);
  }
if(showDefaultValues||
   _normalField)
  {
  _generateSFNode(output,indent,"normal",_normalField,
                  showDefaultValues,defNodesInOut);
  }
if(showDefaultValues||
   _texCoordField)
  {
  _generateSFNode(output,indent,"texCoord",_texCoordField,
                  showDefaultValues,defNodesInOut);
  }
if(showDefaultValues||
   _texCoord2Field)
  {
  _generateSFNode(output,indent,"texCoord2",_texCoord2Field,
                  showDefaultValues,defNodesInOut);
  }
if(showDefaultValues||
   _texCoord3Field)
  {
  _generateSFNode(output,indent,"texCoord3",_texCoord3Field,
                  showDefaultValues,defNodesInOut);
  }
if(showDefaultValues||
   _texCoord4Field)
  {
  _generateSFNode(output,indent,"texCoord4",_texCoord4Field,
                  showDefaultValues,defNodesInOut);
  }
if(showDefaultValues||
   !_heightField.empty())
  {
  _generateMFFloat(output,indent,"height",_heightField);
  }
if(showDefaultValues||
   !_ccwField)
  {
  _generateSFBool(output,indent,"ccw",_ccwField);
  }
if(showDefaultValues||
   !_colorPerVertexField)
  {
  _generateSFBool(output,indent,"colorPerVertex",_colorPerVertexField);
  }
if(showDefaultValues||
   _creaseAngleField)
  {
  _generateSFFloat(output,indent,"creaseAngle",_creaseAngleField);
  }
if(showDefaultValues||
   !_normalPerVertexField)
  {
  _generateSFBool(output,indent,"normalPerVertex",_normalPerVertexField);
  }
if(showDefaultValues||
   !_solidField)
  {
  _generateSFBool(output,indent,"solid",_solidField);
  }
if(showDefaultValues||
   _xDimensionField)
  {
  _generateSFInt32(output,indent,"xDimension",(int)_xDimensionField);
  }
if(showDefaultValues||
   (_xSpacingField!=1.0f))
  {
  _generateSFFloat(output,indent,"xSpacing",_xSpacingField);
  }
if(showDefaultValues||
   _zDimensionField)
  {
  _generateSFInt32(output,indent,"zDimension",(int)_zDimensionField);
  }
if(showDefaultValues||
   (_zSpacingField!=1.0f))
  {
  _generateSFFloat(output,indent,"zSpacing",_zSpacingField);
  }
}

ElevationGridNode::ElevationGridNode(const StlString & nodeType,
                                     VrmlShape3D * shape) // Protected
: GeometryNode(nodeType,shape),
  _colorField((ColorNode *)0),
  _normalField((NormalNode *)0),
  _texCoordField((TextureCoordinateNode *)0),
  _texCoord2Field((TextureCoordinateNode *)0),
  _texCoord3Field((TextureCoordinateNode *)0),
  _texCoord4Field((TextureCoordinateNode *)0),
  _heightField(),
  _ccwField(true),
  _colorPerVertexField(true),
  _creaseAngleField(0.0),
  _normalPerVertexField(true),
  _solidField(true),
  _xDimensionField(0),
  _xSpacingField(1.0),
  _zDimensionField(0),
  _zSpacingField(1.0)
{
  // Nothing to be done
}

} // namespace AReVi

//----------------------------------------------------------------------------
