// LI2/ENIB - AReVi - VRML services
//----------------------------------------------------------------------------
// file        : AReVi/VRML/groupNode_priv.cpp
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/VRML/groupNode_priv.h"

#include "AReVi/VRML/vrmlValue_priv.h"
#include "AReVi/VRML/vrmlShape3D.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class GroupNode
//----------------------------------------------------------------------------

//-------- Construction / Destruction ----------------------------------------

GroupNode::~GroupNode(void)
{
clearChildren();
}

//-------- Inspection --------------------------------------------------------

bool // is a group
GroupNode::isGroup(void) const
{
return(true);
}

//-------- Field management --------------------------------------------------

void
GroupNode::addField(const StlString & fieldName,
                    VrmlValue * fieldValue)
{
bool ok=false;
if(fieldName=="children")
  {
  if(fieldValue->checkMFNode())
    {
    clearChildren();
    unsigned int nb=fieldValue->getSize();
    ok=true;
    for(unsigned int i=0;i<nb;i++)
      {
      const VrmlNode * n=fieldValue->getNode(i);
      if(n&&!n->isGroupChild())
        {
        ok=false;
        }
      else
        {
        VrmlNode * n=fieldValue->accessNode(i);
        if(n&&!n->fail())
          {
          _childrenField.push_back((GroupChildNode *)n);
          n->addParent(this);
          }
        }
      }
    }
  }
else if(fieldName=="bboxCenter")
  {
  if(fieldValue->checkSFVec3f())
    {
    _bboxCenterField.x=fieldValue->getReal(0);
    _bboxCenterField.y=fieldValue->getReal(1);
    _bboxCenterField.z=fieldValue->getReal(2);
    ok=true;
    }
  }
else if(fieldName=="bboxSize")
  {
  if(fieldValue->checkSFVec3f()&&
     (((fieldValue->getReal(0)>0.0)&&
       (fieldValue->getReal(1)>0.0)&&
       (fieldValue->getReal(2)>0.0))||
      ((fieldValue->getReal(0)==-1.0)&&
       (fieldValue->getReal(1)==-1.0)&&
       (fieldValue->getReal(2)==-1.0))))
    {
    _bboxSizeField.x=fieldValue->getReal(0);
    _bboxSizeField.y=fieldValue->getReal(1);
    _bboxSizeField.z=fieldValue->getReal(2);
    ok=true;
    }
  }
else
  {
  GroupChildNode::addField(fieldName,fieldValue);
  ok=true;
  }
if(!ok)
  {
  _shape->_badFieldMsg(fieldName,_type);
  }
}

void
GroupNode::check(void)
{
_fail=false;
}

void
GroupNode::clearChildren(void)
{
while(_childrenField.size())
  {
  _childrenField.back()->removeParent(this);
  _childrenField.pop_back();
  }
}

unsigned int
GroupNode::getNbChildren(void) const
{
return(_childrenField.size());
}

const GroupChildNode *
GroupNode::getChild(unsigned int childIndex) const
{
return(_childrenField[childIndex]);
}

GroupChildNode *
GroupNode::accessChild(unsigned int childIndex)
{
return(_childrenField[childIndex]);
}

//-------- Rendering ---------------------------------------------------------

void
GroupNode::buildParts(const StlVector<TransformNode *> & transformPath)
{
for(unsigned int i=_childrenField.size();i--;)
  {
  _childrenField[i]->buildParts(transformPath);
  }
}

void
GroupNode::rebuild(void)
{
_needsRebuild=false;
if(_needsRebuildUnder)
  {
  for(unsigned int i=_childrenField.size();i--;)
    {
    _childrenField[i]->rebuild();
    }
  _needsRebuildUnder=false;
  }
}

void
GroupNode::postTransformation(void)
{
for(unsigned int i=_childrenField.size();i--;)
  {
  _childrenField[i]->postTransformation();
  }
}

//-------- Forbidden operations ----------------------------------------------

void
GroupNode::_generateFields(ostream & output,
                           unsigned int indent,
                           bool showDefaultValues,
                           StlVector<const VrmlNode *> & defNodesInOut)
                                                            const // Protected
{
if(showDefaultValues||
   _bboxCenterField.x||
   _bboxCenterField.y||
   _bboxCenterField.z)
  {
  _generateSFVec3f(output,indent,"bboxCenter",_bboxCenterField);
  }
if(showDefaultValues||
   (_bboxSizeField.x!=-1.0f)||
   (_bboxSizeField.y!=-1.0f)||
   (_bboxSizeField.z!=-1.0f))
  {
  _generateSFVec3f(output,indent,"bboxSize",_bboxSizeField);
  }
if(showDefaultValues||
   !_childrenField.empty())
  {
  _generateMFNode(output,indent,"children",
                  *(StlVector<VrmlNode *> *)(void *)&_childrenField,
                  showDefaultValues,defNodesInOut);
  }
}

GroupNode::GroupNode(const StlString & nodeType,
                     VrmlShape3D * shape) // Protected
: GroupChildNode(nodeType,shape),
  _childrenField(),
  _bboxCenterField(0.0,0.0,0.0),
  _bboxSizeField(-1.0,-1.0,-1.0)
{
  // Nothing to be done
}

} // namespace AReVi

//----------------------------------------------------------------------------
