// LI2/ENIB - AReVi - VRML services
//----------------------------------------------------------------------------
// file        : AReVi/VRML/imageTextureNode_priv.cpp
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/VRML/imageTextureNode_priv.h"

#include "AReVi/VRML/vrmlValue_priv.h"
#include "AReVi/VRML/vrmlShape3D.h"

#include "AReVi/Lib3D/urlTexture.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class ImageTextureNode
//----------------------------------------------------------------------------

//-------- Construction / Destruction ----------------------------------------

ImageTextureNode::~ImageTextureNode(void)
{
  // Nothing to be done
}

//-------- Field management --------------------------------------------------

void
ImageTextureNode::addField(const StlString & fieldName,
                           VrmlValue * fieldValue)
{
bool ok=false;
if(fieldName=="url")
  {
  if(fieldValue->checkMFString())
    {
    _urlField.clear();
    unsigned int nb=fieldValue->getSize();
    for(unsigned int i=0;i<nb;i++)
      {
      _urlField.push_back(fieldValue->getString(i));
      }
    ok=true;
    }
  }
else
  {
  TextureNode::addField(fieldName,fieldValue);
  ok=true;
  }
if(!ok)
  {
  _shape->_badFieldMsg(fieldName,_type);
  }
}

void
ImageTextureNode::check(void)
{
_fail=false;
if(!_fail)
  {
  rebuild();
  }
}

//-------- Rendering ---------------------------------------------------------

void
ImageTextureNode::rebuild(void)
{
if(_needsRebuild)
  {
  _texture.clear();
  StlString errMsg;
  for(unsigned int i=0;i<_urlField.size();i++)
    {
    StlString url=_urlField[i];
    if(url.size())
      {
      if(url[0]!='/')
        {
        url=_shape->getURLDirectory()+url;
        }
      _texture=URLTexture::NEW(url,true,true);
      if(_texture->fail())
        {
        if(errMsg.size())
          {
          errMsg+="\n";
          }
        errMsg+="Cannot load texture `";
        errMsg+=url;
        errMsg+="'";
        _texture.clear();
        }
      else
        {
        break;
        }
      }
    }
  if(!_texture)
    {
    _shape->_errorMsg(errMsg);
    }
  _needsRebuild=false;
  }
}

//-------- Forbidden operations ----------------------------------------------

void
ImageTextureNode::_generateFields(ostream & output,
                                  unsigned int indent,
                                  bool showDefaultValues,
                                  StlVector<const VrmlNode *> & defNodesInOut)
                                                            const // Protected
{
TextureNode::_generateFields(output,indent,showDefaultValues,defNodesInOut);
if(showDefaultValues||
   !_urlField.empty())
  {
  _generateMFString(output,indent,"url",_urlField);
  }
}

ImageTextureNode::ImageTextureNode(const StlString & nodeType,
                                   VrmlShape3D * shape) // Protected
: TextureNode(nodeType,shape),
  _urlField()
{
  // Nothing to be done
}

} // namespace AReVi

//----------------------------------------------------------------------------
