// LI2/ENIB - AReVi - VRML services
//----------------------------------------------------------------------------
// file        : AReVi/VRML/indexedFaceSetNode_priv.cpp
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/VRML/indexedFaceSetNode_priv.h"

#include "AReVi/VRML/vrmlValue_priv.h"
#include "AReVi/VRML/vrmlShape3D.h"

#include "AReVi/arSystem.h"

#include "AReVi/VRML/colorNode_priv.h"
#include "AReVi/VRML/coordinateNode_priv.h"
#include "AReVi/VRML/normalNode_priv.h"
#include "AReVi/VRML/textureCoordinateNode_priv.h"

#include "AReVi/Shapes/mesh3D.h"
#include "AReVi/Lib3D/tesselator.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class IndexedFaceSetNode
//----------------------------------------------------------------------------

//-------- Construction / Destruction ----------------------------------------

IndexedFaceSetNode::~IndexedFaceSetNode(void)
{
if(_colorField)
  {
  _colorField->removeParent(this);
  }
if(_coordField)
  {
  _coordField->removeParent(this);
  }
if(_normalField)
  {
  _normalField->removeParent(this);
  }
if(_texCoordField)
  {
  _texCoordField->removeParent(this);
  }
if(_texCoord2Field)
  {
  _texCoord2Field->removeParent(this);
  }
if(_texCoord3Field)
  {
  _texCoord3Field->removeParent(this);
  }
if(_texCoord4Field)
  {
  _texCoord4Field->removeParent(this);
  }
}

//-------- Field management --------------------------------------------------

void
IndexedFaceSetNode::getNodeFieldNames(StlVector<StlString> & nodeFieldNamesOut)
                                                                         const
{
nodeFieldNamesOut.clear();
nodeFieldNamesOut.push_back("color");
nodeFieldNamesOut.push_back("coord");
nodeFieldNamesOut.push_back("normal");
nodeFieldNamesOut.push_back("texCoord");
nodeFieldNamesOut.push_back("texCoord2");
nodeFieldNamesOut.push_back("texCoord3");
nodeFieldNamesOut.push_back("texCoord4");
}

const VrmlNode * // node at field
IndexedFaceSetNode::getNodeField(const StlString & fieldName) const
{
if(fieldName=="color")
  {
  return(_colorField);
  }
else if(fieldName=="coord")
  {
  return(_coordField);
  }
else if(fieldName=="normal")
  {
  return(_normalField);
  }
else if(fieldName=="texCoord")
  {
  return(_texCoordField);
  }
else if(fieldName=="texCoord2")
  {
  return(_texCoord2Field);
  }
else if(fieldName=="texCoord3")
  {
  return(_texCoord3Field);
  }
else if(fieldName=="texCoord4")
  {
  return(_texCoord4Field);
  }
else
  {
  return((const VrmlNode *)0);
  }
}

void
IndexedFaceSetNode::addField(const StlString & fieldName,
                             VrmlValue * fieldValue)
{
bool ok=false;
if(fieldName=="color")
  {
  if(fieldValue->checkSFNode())
    {
    VrmlNode * n=fieldValue->accessNode(0);
    if(!n||(n->getType()=="Color"))
      {
      if(!(n&&n->fail()))
        {
        if(_colorField)
          {
          _colorField->removeParent(this);
          }
        _colorField=(ColorNode *)n;
        if(_colorField)
          {
          _colorField->addParent(this);
          }
        }
      ok=true;
      }
    }
  }
else if(fieldName=="coord")
  {
  if(fieldValue->checkSFNode())
    {
    VrmlNode * n=fieldValue->accessNode(0);
    if(!n||(n->getType()=="Coordinate"))
      {
      if(!(n&&n->fail()))
        {
        if(_coordField)
          {
          _coordField->removeParent(this);
          }
        _coordField=(CoordinateNode *)n;
        if(_coordField)
          {
          _coordField->addParent(this);
          }
        }
      ok=true;
      }
    }
  }
else if(fieldName=="normal")
  {
  if(fieldValue->checkSFNode())
    {
    VrmlNode * n=fieldValue->accessNode(0);
    if(!n||(n->getType()=="Normal"))
      {
      if(!(n&&n->fail()))
        {
        if(_normalField)
          {
          _normalField->removeParent(this);
          }
        _normalField=(NormalNode *)n;
        if(_normalField)
          {
          _normalField->addParent(this);
          }
        }
      ok=true;
      }
    }
  }
else if(fieldName=="texCoord")
  {
  if(fieldValue->checkSFNode())
    {
    VrmlNode * n=fieldValue->accessNode(0);
    if(!n||(n->getType()=="TextureCoordinate"))
      {
      if(!(n&&n->fail()))
        {
        if(_texCoordField)
          {
          _texCoordField->removeParent(this);
          }
        _texCoordField=(TextureCoordinateNode *)n;
        if(_texCoordField)
          {
          _texCoordField->addParent(this);
          }
        }
      ok=true;
      }
    }
  }
else if(fieldName=="texCoord2")
  {
  if(fieldValue->checkSFNode())
    {
    VrmlNode * n=fieldValue->accessNode(0);
    if(!n||(n->getType()=="TextureCoordinate"))
      {
      if(!(n&&n->fail()))
        {
        if(_texCoord2Field)
          {
          _texCoord2Field->removeParent(this);
          }
        _texCoord2Field=(TextureCoordinateNode *)n;
        if(_texCoord2Field)
          {
          _texCoord2Field->addParent(this);
          }
        }
      ok=true;
      }
    }
  }
else if(fieldName=="texCoord3")
  {
  if(fieldValue->checkSFNode())
    {
    VrmlNode * n=fieldValue->accessNode(0);
    if(!n||(n->getType()=="TextureCoordinate"))
      {
      if(!(n&&n->fail()))
        {
        if(_texCoord3Field)
          {
          _texCoord3Field->removeParent(this);
          }
        _texCoord3Field=(TextureCoordinateNode *)n;
        if(_texCoord3Field)
          {
          _texCoord3Field->addParent(this);
          }
        }
      ok=true;
      }
    }
  }
else if(fieldName=="texCoord4")
  {
  if(fieldValue->checkSFNode())
    {
    VrmlNode * n=fieldValue->accessNode(0);
    if(!n||(n->getType()=="TextureCoordinate"))
      {
      if(!(n&&n->fail()))
        {
        if(_texCoord4Field)
          {
          _texCoord4Field->removeParent(this);
          }
        _texCoord4Field=(TextureCoordinateNode *)n;
        if(_texCoord4Field)
          {
          _texCoord4Field->addParent(this);
          }
        }
      ok=true;
      }
    }
  }
else if(fieldName=="ccw")
  {
  if(fieldValue->checkSFBool())
    {
    _ccwField=fieldValue->getBool(0);
    ok=true;
    }
  }
else if(fieldName=="colorIndex")
  {
  if(fieldValue->checkMFInt32())
    {
    unsigned int nb=fieldValue->getSize();
    ok=true;
    for(unsigned int i=nb;i--;)
      {
      if(fieldValue->getInteger(i)<-1)
        {
        ok=false;
        break;
        }
      }
    if(ok)
      {
      _colorIndexField.clear();
      if(nb)
        {
        _colorIndexField.push_back(StlVector<int>());
        }
      for(unsigned int i=0;i<nb;i++)
        {
        if(fieldValue->getInteger(i)==-1)
          {
          if(i<nb-1)
            {
            _colorIndexField.push_back(StlVector<int>());
            }
          }
        else
          {
          _colorIndexField.back().push_back(fieldValue->getInteger(i));
          }
        }
      }
    }
  }
else if(fieldName=="colorPerVertex")
  {
  if(fieldValue->checkSFBool())
    {
    _colorPerVertexField=fieldValue->getBool(0);
    ok=true;
    }
  }
else if(fieldName=="convex")
  {
  if(fieldValue->checkSFBool())
    {
    _convexField=fieldValue->getBool(0);
    ok=true;
    }
  }
else if(fieldName=="coordIndex")
  {
  if(fieldValue->checkMFInt32())
    {
    unsigned int nb=fieldValue->getSize();
    ok=true;
    for(unsigned int i=nb;i--;)
      {
      if(fieldValue->getInteger(i)<-1)
        {
        ok=false;
        break;
        }
      }
    if(ok)
      {
      _coordIndexField.clear();
      if(nb)
        {
        _coordIndexField.push_back(StlVector<int>());
        }
      for(unsigned int i=0;i<nb;i++)
        {
        if(fieldValue->getInteger(i)==-1)
          {
          if(i<nb-1)
            {
            _coordIndexField.push_back(StlVector<int>());
            }
          }
        else
          {
          _coordIndexField.back().push_back(fieldValue->getInteger(i));
          }
        }
      }
    }
  }
else if(fieldName=="creaseAngle")
  {
  if(fieldValue->checkSFFloat()&&
     (fieldValue->getReal(0)>=0.0))
    {
    _creaseAngleField=fieldValue->getReal(0);
    ok=true;
    }
  }
else if(fieldName=="normalIndex")
  {
  if(fieldValue->checkMFInt32())
    {
    unsigned int nb=fieldValue->getSize();
    ok=true;
    for(unsigned int i=nb;i--;)
      {
      if(fieldValue->getInteger(i)<-1)
        {
        ok=false;
        break;
        }
      }
    if(ok)
      {
      _normalIndexField.clear();
      if(nb)
        {
        _normalIndexField.push_back(StlVector<int>());
        }
      for(unsigned int i=0;i<nb;i++)
        {
        if(fieldValue->getInteger(i)==-1)
          {
          if(i<nb-1)
            {
            _normalIndexField.push_back(StlVector<int>());
            }
          }
        else
          {
          _normalIndexField.back().push_back(fieldValue->getInteger(i));
          }
        }
      }
    }
  }
else if(fieldName=="normalPerVertex")
  {
  if(fieldValue->checkSFBool())
    {
    _normalPerVertexField=fieldValue->getBool(0);
    ok=true;
    }
  }
else if(fieldName=="solid")
  {
  if(fieldValue->checkSFBool())
    {
    _solidField=fieldValue->getBool(0);
    ok=true;
    }
  }
else if(fieldName=="texCoordIndex")
  {
  if(fieldValue->checkMFInt32())
    {
    unsigned int nb=fieldValue->getSize();
    ok=true;
    for(unsigned int i=nb;i--;)
      {
      if(fieldValue->getInteger(i)<-1)
        {
        ok=false;
        break;
        }
      }
    if(ok)
      {
      _texCoordIndexField.clear();
      if(nb)
        {
        _texCoordIndexField.push_back(StlVector<int>());
        }
      for(unsigned int i=0;i<nb;i++)
        {
        if(fieldValue->getInteger(i)==-1)
          {
          if(i<nb-1)
            {
            _texCoordIndexField.push_back(StlVector<int>());
            }
          }
        else
          {
          _texCoordIndexField.back().push_back(fieldValue->getInteger(i));
          }
        }
      }
    }
  }
else if(fieldName=="texCoordIndex2")
  {
  if(fieldValue->checkMFInt32())
    {
    unsigned int nb=fieldValue->getSize();
    ok=true;
    for(unsigned int i=nb;i--;)
      {
      if(fieldValue->getInteger(i)<-1)
        {
        ok=false;
        break;
        }
      }
    if(ok)
      {
      _texCoordIndex2Field.clear();
      if(nb)
        {
        _texCoordIndex2Field.push_back(StlVector<int>());
        }
      for(unsigned int i=0;i<nb;i++)
        {
        if(fieldValue->getInteger(i)==-1)
          {
          if(i<nb-1)
            {
            _texCoordIndex2Field.push_back(StlVector<int>());
            }
          }
        else
          {
          _texCoordIndex2Field.back().push_back(fieldValue->getInteger(i));
          }
        }
      }
    }
  }
else if(fieldName=="texCoordIndex3")
  {
  if(fieldValue->checkMFInt32())
    {
    unsigned int nb=fieldValue->getSize();
    ok=true;
    for(unsigned int i=nb;i--;)
      {
      if(fieldValue->getInteger(i)<-1)
        {
        ok=false;
        break;
        }
      }
    if(ok)
      {
      _texCoordIndex3Field.clear();
      if(nb)
        {
        _texCoordIndex3Field.push_back(StlVector<int>());
        }
      for(unsigned int i=0;i<nb;i++)
        {
        if(fieldValue->getInteger(i)==-1)
          {
          if(i<nb-1)
            {
            _texCoordIndex3Field.push_back(StlVector<int>());
            }
          }
        else
          {
          _texCoordIndex3Field.back().push_back(fieldValue->getInteger(i));
          }
        }
      }
    }
  }
else if(fieldName=="texCoordIndex4")
  {
  if(fieldValue->checkMFInt32())
    {
    unsigned int nb=fieldValue->getSize();
    ok=true;
    for(unsigned int i=nb;i--;)
      {
      if(fieldValue->getInteger(i)<-1)
        {
        ok=false;
        break;
        }
      }
    if(ok)
      {
      _texCoordIndex4Field.clear();
      if(nb)
        {
        _texCoordIndex4Field.push_back(StlVector<int>());
        }
      for(unsigned int i=0;i<nb;i++)
        {
        if(fieldValue->getInteger(i)==-1)
          {
          if(i<nb-1)
            {
            _texCoordIndex4Field.push_back(StlVector<int>());
            }
          }
        else
          {
          _texCoordIndex4Field.back().push_back(fieldValue->getInteger(i));
          }
        }
      }
    }
  }
else
  {
  GeometryNode::addField(fieldName,fieldValue);
  ok=true;
  }
if(!ok)
  {
  _shape->_badFieldMsg(fieldName,_type);
  }
}

void
IndexedFaceSetNode::check(void)
{
_fail=false;
unsigned int nbVertex=0;
if(_coordField)
  {
  nbVertex=_coordField->_pointField.size();
  }
for(unsigned int i=_coordIndexField.size();i--;)
  {
  unsigned int j=_coordIndexField[i].size();
  if(j<3)
    {
    StlString msg="Bad face size (";
    msg+=uintToStr(j);
    msg+="<3) for field 'coordIndex' in node type '";
    msg+=_type;
    msg+="'";
    _shape->_errorMsg(msg);
    _fail=true;
    break;
    }
  while(j--)
    {
    unsigned int index=_coordIndexField[i][j];
    if(index>=nbVertex)
      {
      StlString msg="Bad vertex index (";
      msg+=uintToStr(index);
      msg+=">";
      msg+=uintToStr(nbVertex-1);
      msg+=") for field 'coordIndex' in node type '";
      msg+=_type;
      msg+="'";
      _shape->_errorMsg(msg);
      _fail=true;
      i=j=0;
      }
    }
  }
if(_colorField)
  {
  unsigned int nbColor=_colorField->_colorField.size();
  if(_colorPerVertexField)
    {
    if(_colorIndexField.size())
      {
      bool ok=true;
      if(_coordIndexField.size()>_colorIndexField.size())
        {
        ok=false;
        }
      else
        {
        for(unsigned int i=_coordIndexField.size();i--;)
          {
          unsigned int j=_colorIndexField[i].size();
          if(_coordIndexField[i].size()!=j)
            {
            ok=false;
            break;
            }
          while(j--)
            {
            unsigned int index=_colorIndexField[i][j];
            if(index>=nbColor)
              {
              StlString msg="Bad color index (";
              msg+=uintToStr(index);
              msg+=">";
              msg+=uintToStr(nbColor-1);
              msg+=") for field 'colorIndex' in node type '";
              msg+=_type;
              msg+="'";
              _shape->_errorMsg(msg);
              _fail=true;
              i=j=0;
              }
            }
          }
        }
      if(!ok)
        {
        StlString msg="Field 'colorIndex' and field 'coordIndex' ";
        msg+="do not match in node type '";
        msg+=_type;
        msg+="'";
        _shape->_errorMsg(msg);
        _fail=true;
        }
      }
    else // coordIndex instead of colorIndex
      {
      for(unsigned int i=_coordIndexField.size();i--;)
        {
        for(unsigned int j=_coordIndexField[i].size();j--;)
          {
          unsigned int index=_coordIndexField[i][j];
          if(index>=nbColor)
            {
            StlString msg="Bad color index (";
            msg+=uintToStr(index);
            msg+=">";
            msg+=uintToStr(nbColor-1);
            msg+=") for field 'coordIndex' in node type '";
            msg+=_type;
            msg+="'";
            _shape->_errorMsg(msg);
            _fail=true;
            i=j=0;
            }
          }
        }
      }
    }
  else
    {
    if(_colorIndexField.size())
      {
      if((_colorIndexField.size()!=1)||
         (_colorIndexField.back().size()<_coordIndexField.size()))
        {
        StlString msg="Field 'colorIndex' and field 'coordIndex' ";
        msg+="do not match in node type '";
        msg+=_type;
        msg+="'";
        _shape->_errorMsg(msg);
        _fail=true;
        }
      else
        {
        for(unsigned int i=_colorIndexField.back().size();i--;)
          {
          unsigned int index=_colorIndexField.back()[i];
          if(index>=nbColor)
            {
            StlString msg="Bad color index (";
            msg+=uintToStr(index);
            msg+=">";
            msg+=uintToStr(nbColor-1);
            msg+=") for field 'coordIndex' in node type '";
            msg+=_type;
            msg+="'";
            _shape->_errorMsg(msg);
            _fail=true;
            break;
            }
          }
        }
      }
    else
      {
      if(nbColor<_coordIndexField.size())
        {
        StlString msg="Field 'color' and field 'coordIndex' ";
        msg+="do not match in node type '";
        msg+=_type;
        msg+="'";
        _shape->_errorMsg(msg);
        _fail=true;
        }
      }
    }
  }
if(_normalField)
  {
  unsigned int nbNormal=_normalField->_vectorField.size();
  if(_normalPerVertexField)
    {
    if(_normalIndexField.size())
      {
      bool ok=true;
      if(_coordIndexField.size()>_normalIndexField.size())
        {
        ok=false;
        }
      else
        {
        for(unsigned int i=_coordIndexField.size();i--;)
          {
          unsigned int j=_normalIndexField[i].size();
          if(_coordIndexField[i].size()!=j)
            {
            ok=false;
            break;
            }
          while(j--)
            {
            unsigned int index=_normalIndexField[i][j];
            if(index>=nbNormal)
              {
              StlString msg="Bad normal index (";
              msg+=uintToStr(index);
              msg+=">";
              msg+=uintToStr(nbNormal-1);
              msg+=") for field 'normalIndex' in node type '";
              msg+=_type;
              msg+="'";
              _shape->_errorMsg(msg);
              _fail=true;
              i=j=0;
              }
            }
          }
        }
      if(!ok)
        {
        StlString msg="Field 'normalIndex' and field 'coordIndex' ";
        msg+="do not match in node type '";
        msg+=_type;
        msg+="'";
        _shape->_errorMsg(msg);
        _fail=true;
        }
      }
    else // coordIndex instead of normalIndex
      {
      for(unsigned int i=_coordIndexField.size();i--;)
        {
        for(unsigned int j=_coordIndexField[i].size();j--;)
          {
          unsigned int index=_coordIndexField[i][j];
          if(index>=nbNormal)
            {
            StlString msg="Bad normal index (";
            msg+=uintToStr(index);
            msg+=">";
            msg+=uintToStr(nbNormal-1);
            msg+=") for field 'coordIndex' in node type '";
            msg+=_type;
            msg+="'";
            _shape->_errorMsg(msg);
            _fail=true;
            i=j=0;
            }
          }
        }
      }
    }
  else
    {
    if(_normalIndexField.size())
      {
      if((_normalIndexField.size()!=1)||
         (_normalIndexField.back().size()<_coordIndexField.size()))
        {
        StlString msg="Field 'normalIndex' and field 'coordIndex' ";
        msg+="do not match in node type '";
        msg+=_type;
        msg+="'";
        _shape->_errorMsg(msg);
        _fail=true;
        }
      else
        {
        for(unsigned int i=_normalIndexField.back().size();i--;)
          {
          unsigned int index=_normalIndexField.back()[i];
          if(index>=nbNormal)
            {
            StlString msg="Bad normal index (";
            msg+=uintToStr(index);
            msg+=">";
            msg+=uintToStr(nbNormal-1);
            msg+=") for field 'coordIndex' in node type '";
            msg+=_type;
            msg+="'";
            _shape->_errorMsg(msg);
            _fail=true;
            break;
            }
          }
        }
      }
    else
      {
      if(nbNormal<_coordIndexField.size())
        {
        StlString msg="Field 'normal' and field 'coordIndex' ";
        msg+="do not match in node type '";
        msg+=_type;
        msg+="'";
        _shape->_errorMsg(msg);
        _fail=true;
        }
      }
    }
  }
if(_texCoordField)
  {
  unsigned int nbTexture=_texCoordField->_pointField.size();
  if(_texCoordIndexField.size())
    {
    bool ok=true;
    if(_coordIndexField.size()>_texCoordIndexField.size())
      {
      ok=false;
      }
    else
      {
      for(unsigned int i=_coordIndexField.size();i--;)
        {
        unsigned int j=_texCoordIndexField[i].size();
        if(_coordIndexField[i].size()!=j)
          {
          ok=false;
          break;
          }
        while(j--)
          {
          unsigned int index=_texCoordIndexField[i][j];
          if(index>=nbTexture)
            {
            StlString msg="Bad texture index (";
            msg+=uintToStr(index);
            msg+=">";
            msg+=uintToStr(nbTexture-1);
            msg+=") for field 'texCoordIndex' in node type '";
            msg+=_type;
            msg+="'";
            _shape->_errorMsg(msg);
            _fail=true;
            i=j=0;
            }
          }
        }
      }
    if(!ok)
      {
      StlString msg="Field 'texCoordIndex' and field 'coordIndex' ";
      msg+="do not match in node type '";
      msg+=_type;
      msg+="'";
      _shape->_errorMsg(msg);
      _fail=true;
      }
    }
  else // coordIndex instead of texCoordIndex
    {
    for(unsigned int i=_coordIndexField.size();i--;)
      {
      for(unsigned int j=_coordIndexField[i].size();j--;)
        {
        unsigned int index=_coordIndexField[i][j];
        if(index>=nbTexture)
          {
          StlString msg="Bad texture index (";
          msg+=uintToStr(index);
          msg+=">";
          msg+=uintToStr(nbTexture-1);
          msg+=") for field 'coordIndex' in node type '";
          msg+=_type;
          msg+="'";
          _shape->_errorMsg(msg);
          _fail=true;
          i=j=0;
          }
        }
      }
    }
  }
if(_texCoord2Field)
  {
  unsigned int nbTexture=_texCoord2Field->_pointField.size();
  if(_texCoordIndex2Field.size())
    {
    bool ok=true;
    if(_coordIndexField.size()>_texCoordIndex2Field.size())
      {
      ok=false;
      }
    else
      {
      for(unsigned int i=_coordIndexField.size();i--;)
        {
        unsigned int j=_texCoordIndex2Field[i].size();
        if(_coordIndexField[i].size()!=j)
          {
          ok=false;
          break;
          }
        while(j--)
          {
          unsigned int index=_texCoordIndex2Field[i][j];
          if(index>=nbTexture)
            {
            StlString msg="Bad texture2 index (";
            msg+=uintToStr(index);
            msg+=">";
            msg+=uintToStr(nbTexture-1);
            msg+=") for field 'texCoordIndex2' in node type '";
            msg+=_type;
            msg+="'";
            _shape->_errorMsg(msg);
            _fail=true;
            i=j=0;
            }
          }
        }
      }
    if(!ok)
      {
      StlString msg="Field 'texCoordIndex2' and field 'coordIndex' ";
      msg+="do not match in node type '";
      msg+=_type;
      msg+="'";
      _shape->_errorMsg(msg);
      _fail=true;
      }
    }
  else // coordIndex instead of texCoordIndex2
    {
    for(unsigned int i=_coordIndexField.size();i--;)
      {
      for(unsigned int j=_coordIndexField[i].size();j--;)
        {
        unsigned int index=_coordIndexField[i][j];
        if(index>=nbTexture)
          {
          StlString msg="Bad texture2 index (";
          msg+=uintToStr(index);
          msg+=">";
          msg+=uintToStr(nbTexture-1);
          msg+=") for field 'coordIndex' in node type '";
          msg+=_type;
          msg+="'";
          _shape->_errorMsg(msg);
          _fail=true;
          i=j=0;
          }
        }
      }
    }
  }
if(_texCoord3Field)
  {
  unsigned int nbTexture=_texCoord3Field->_pointField.size();
  if(_texCoordIndex3Field.size())
    {
    bool ok=true;
    if(_coordIndexField.size()>_texCoordIndex3Field.size())
      {
      ok=false;
      }
    else
      {
      for(unsigned int i=_coordIndexField.size();i--;)
        {
        unsigned int j=_texCoordIndex3Field[i].size();
        if(_coordIndexField[i].size()!=j)
          {
          ok=false;
          break;
          }
        while(j--)
          {
          unsigned int index=_texCoordIndex3Field[i][j];
          if(index>=nbTexture)
            {
            StlString msg="Bad texture3 index (";
            msg+=uintToStr(index);
            msg+=">";
            msg+=uintToStr(nbTexture-1);
            msg+=") for field 'texCoordIndex3' in node type '";
            msg+=_type;
            msg+="'";
            _shape->_errorMsg(msg);
            _fail=true;
            i=j=0;
            }
          }
        }
      }
    if(!ok)
      {
      StlString msg="Field 'texCoordIndex3' and field 'coordIndex' ";
      msg+="do not match in node type '";
      msg+=_type;
      msg+="'";
      _shape->_errorMsg(msg);
      _fail=true;
      }
    }
  else // coordIndex instead of texCoordIndex3
    {
    for(unsigned int i=_coordIndexField.size();i--;)
      {
      for(unsigned int j=_coordIndexField[i].size();j--;)
        {
        unsigned int index=_coordIndexField[i][j];
        if(index>=nbTexture)
          {
          StlString msg="Bad texture3 index (";
          msg+=uintToStr(index);
          msg+=">";
          msg+=uintToStr(nbTexture-1);
          msg+=") for field 'coordIndex' in node type '";
          msg+=_type;
          msg+="'";
          _shape->_errorMsg(msg);
          _fail=true;
          i=j=0;
          }
        }
      }
    }
  }
if(_texCoord4Field)
  {
  unsigned int nbTexture=_texCoord4Field->_pointField.size();
  if(_texCoordIndex4Field.size())
    {
    bool ok=true;
    if(_coordIndexField.size()>_texCoordIndex4Field.size())
      {
      ok=false;
      }
    else
      {
      for(unsigned int i=_coordIndexField.size();i--;)
        {
        unsigned int j=_texCoordIndex4Field[i].size();
        if(_coordIndexField[i].size()!=j)
          {
          ok=false;
          break;
          }
        while(j--)
          {
          unsigned int index=_texCoordIndex4Field[i][j];
          if(index>=nbTexture)
            {
            StlString msg="Bad texture4 index (";
            msg+=uintToStr(index);
            msg+=">";
            msg+=uintToStr(nbTexture-1);
            msg+=") for field 'texCoordIndex4' in node type '";
            msg+=_type;
            msg+="'";
            _shape->_errorMsg(msg);
            _fail=true;
            i=j=0;
            }
          }
        }
      }
    if(!ok)
      {
      StlString msg="Field 'texCoordIndex4' and field 'coordIndex' ";
      msg+="do not match in node type '";
      msg+=_type;
      msg+="'";
      _shape->_errorMsg(msg);
      _fail=true;
      }
    }
  else // coordIndex instead of texCoordIndex4
    {
    for(unsigned int i=_coordIndexField.size();i--;)
      {
      for(unsigned int j=_coordIndexField[i].size();j--;)
        {
        unsigned int index=_coordIndexField[i][j];
        if(index>=nbTexture)
          {
          StlString msg="Bad texture4 index (";
          msg+=uintToStr(index);
          msg+=">";
          msg+=uintToStr(nbTexture-1);
          msg+=") for field 'coordIndex' in node type '";
          msg+=_type;
          msg+="'";
          _shape->_errorMsg(msg);
          _fail=true;
          i=j=0;
          }
        }
      }
    }
  }
if(!_fail)
  {
  rebuild();
  }
}

//-------- Rendering ---------------------------------------------------------

ArRef<ShapePart3D> // built part
IndexedFaceSetNode::buildPart(Util3D::Transform & initTransformOut) const
{
initTransformOut.identity();
return(Mesh3D::NEW());
}

void
IndexedFaceSetNode::updatePart(ArRef<ShapePart3D> part,
                               const MaterialNode * /* material */) const
{
ArRef<Mesh3D> mesh=ar_down_cast<Mesh3D>(part);
StlVector<Util3D::Dbl3> & v=mesh->accessVertices();
v.clear();
StlVector<Util3D::Face> & vf=mesh->accessVertexFaces();
vf.clear();
StlVector<Util3D::Dbl3> & n=mesh->accessNormals();
n.clear();
StlVector<Util3D::Face> & nf=mesh->accessNormalFaces();
nf.clear();
StlVector<Util3D::Dbl2> & t=mesh->accessTextures();
t.clear();
StlVector<Util3D::Face> & tf=mesh->accessTextureFaces();
tf.clear();
StlVector<Util3D::Dbl2> & t2=mesh->accessSecondTextures();
t2.clear();
StlVector<Util3D::Face> & tf2=mesh->accessSecondTextureFaces();
tf2.clear();
StlVector<Util3D::Dbl2> & t3=mesh->accessThirdTextures();
t3.clear();
StlVector<Util3D::Face> & tf3=mesh->accessThirdTextureFaces();
tf3.clear();
StlVector<Util3D::Dbl2> & t4=mesh->accessFourthTextures();
t4.clear();
StlVector<Util3D::Face> & tf4=mesh->accessFourthTextureFaces();
tf4.clear();
StlVector<Util3D::Dbl3> & c=mesh->accessColors();
c.clear();
StlVector<Util3D::Face> & cf=mesh->accessColorFaces();
cf.clear();
StlVector<StlVector<Util3D::Face> > tessInfo;
Util3D::BoundingBox bbox;
if(_coordField)
  {
  ArRef<Tesselator> tess=Tesselator::NEW();
  unsigned int nb=_coordIndexField.size();
  if(!_convexField)
    {
    tessInfo.reserve(nb);
    }
  v=_coordField->_pointField;
  StlVector<Util3D::Face> emptyFace;
  for(unsigned int i=0;i<nb;i++)
    {
    if(_convexField)
      {
      Util3D::Face f;
      unsigned int faceSize=_coordIndexField[i].size();
      for(unsigned int j=2;j<faceSize;j++)
        {
        if(_ccwField)
          {
          f.p1=_coordIndexField[i][0];
          f.p2=_coordIndexField[i][j-1];
          f.p3=_coordIndexField[i][j];
          }
        else
          {
          f.p1=_coordIndexField[i][0];
          f.p2=_coordIndexField[i][j];
          f.p3=_coordIndexField[i][j-1];
          }
        vf.push_back(f);
        bbox.addXYZ(v[f.p1].x,v[f.p1].y,v[f.p1].z);
        bbox.addXYZ(v[f.p2].x,v[f.p2].y,v[f.p2].z);
        bbox.addXYZ(v[f.p3].x,v[f.p3].y,v[f.p3].z);
        }
      }
    else
      {
      tessInfo.push_back(emptyFace);
      tess->tesselation3D(v,_coordIndexField[i],tessInfo[i]);
      Util3D::Face f;
      unsigned int faceSize=tessInfo[i].size();
      for(unsigned int j=0;j<faceSize;j++)
        {
        const Util3D::Face & face=tessInfo[i][j];
        if(_ccwField)
          {
          f.p1=_coordIndexField[i][face.p1];
          f.p2=_coordIndexField[i][face.p2];
          f.p3=_coordIndexField[i][face.p3];
          }
        else
          {
          f.p1=_coordIndexField[i][face.p1];
          f.p2=_coordIndexField[i][face.p3];
          f.p3=_coordIndexField[i][face.p2];
          }
        vf.push_back(f);
        bbox.addXYZ(v[f.p1].x,v[f.p1].y,v[f.p1].z);
        bbox.addXYZ(v[f.p2].x,v[f.p2].y,v[f.p2].z);
        bbox.addXYZ(v[f.p3].x,v[f.p3].y,v[f.p3].z);
        }
      }
    }
  }
if(_colorField)
  {
  c=_colorField->_colorField;
  if(_colorPerVertexField)
    {
    if(_colorIndexField.size())
      {
      unsigned int nb=_coordIndexField.size();
      for(unsigned int i=0;i<nb;i++)
        {
        if(_convexField)
          {
          unsigned int faceSize=_coordIndexField[i].size();
          for(unsigned int j=2;j<faceSize;j++)
            {
            if(_ccwField)
              {
              cf.push_back(Util3D::Face(_colorIndexField[i][0],
                                        _colorIndexField[i][j-1],
                                        _colorIndexField[i][j]));
              }
            else
              {
              cf.push_back(Util3D::Face(_colorIndexField[i][0],
                                        _colorIndexField[i][j],
                                        _colorIndexField[i][j-1]));
              }
            }
          }
        else
          {
          unsigned int faceSize=tessInfo[i].size();
          for(unsigned int j=0;j<faceSize;j++)
            {
            const Util3D::Face & face=tessInfo[i][j];
            if(_ccwField)
              {
              cf.push_back(Util3D::Face(_colorIndexField[i][face.p1],
                                        _colorIndexField[i][face.p2],
                                        _colorIndexField[i][face.p3]));
              }
            else
              {
              cf.push_back(Util3D::Face(_colorIndexField[i][face.p1],
                                        _colorIndexField[i][face.p3],
                                        _colorIndexField[i][face.p2]));
              }
            }
          }
        }
      }
    // else --> vertex faces will be reused
    }
  else
    {
    if(_colorIndexField.size())
      {
      unsigned int nb=_coordIndexField.size();
      for(unsigned int i=0;i<nb;i++)
        {
        unsigned int index=_colorIndexField.back()[i];
        unsigned int faceSize=_coordIndexField[i].size();
        for(unsigned int j=2;j<faceSize;j++)
          {
          cf.push_back(Util3D::Face(index,index,index));
          }
        }
      }
    else
      {
      unsigned int nb=_coordIndexField.size();
      for(unsigned int i=0;i<nb;i++)
        {
        unsigned int faceSize=_coordIndexField[i].size();
        for(unsigned int j=2;j<faceSize;j++)
          {
          cf.push_back(Util3D::Face(i,i,i));
          }
        }
      }
    }
  }
if(_normalField)
  {
  n=_normalField->_vectorField;
  if(_normalPerVertexField)
    {
    if(_normalIndexField.size())
      {
      unsigned int nb=_coordIndexField.size();
      for(unsigned int i=0;i<nb;i++)
        {
        if(_convexField)
          {
          unsigned int faceSize=_coordIndexField[i].size();
          for(unsigned int j=2;j<faceSize;j++)
            {
            if(_ccwField)
              {
              nf.push_back(Util3D::Face(_normalIndexField[i][0],
                                        _normalIndexField[i][j-1],
                                        _normalIndexField[i][j]));
              }
            else
              {
              nf.push_back(Util3D::Face(_normalIndexField[i][0],
                                        _normalIndexField[i][j],
                                        _normalIndexField[i][j-1]));
              }
            }
          }
        else
          {
          unsigned int faceSize=tessInfo[i].size();
          for(unsigned int j=0;j<faceSize;j++)
            {
            const Util3D::Face & face=tessInfo[i][j];
            if(_ccwField)
              {
              nf.push_back(Util3D::Face(_normalIndexField[i][face.p1],
                                        _normalIndexField[i][face.p2],
                                        _normalIndexField[i][face.p3]));
              }
            else
              {
              nf.push_back(Util3D::Face(_normalIndexField[i][face.p1],
                                        _normalIndexField[i][face.p3],
                                        _normalIndexField[i][face.p2]));
              }
            }
          }
        }
      }
    // else --> vertex faces will be reused
    }
  else
    {
    if(_normalIndexField.size())
      {
      unsigned int nb=_coordIndexField.size();
      for(unsigned int i=0;i<nb;i++)
        {
        unsigned int index=_normalIndexField.back()[i];
        unsigned int faceSize=_coordIndexField[i].size();
        for(unsigned int j=2;j<faceSize;j++)
          {
          nf.push_back(Util3D::Face(index,index,index));
          }
        }
      }
    else
      {
      unsigned int nb=_coordIndexField.size();
      for(unsigned int i=0;i<nb;i++)
        {
        unsigned int faceSize=_coordIndexField[i].size();
        for(unsigned int j=2;j<faceSize;j++)
          {
          nf.push_back(Util3D::Face(i,i,i));
          }
        }
      }
    }
  }
else
  {
  mesh->computeDefaultNormals(_normalPerVertexField);
  }
if(_texCoordField)
  {
  t=_texCoordField->_pointField;
  if(_texCoordIndexField.size())
    {
    unsigned int nb=_coordIndexField.size();
    for(unsigned int i=0;i<nb;i++)
      {
      if(_convexField)
        {
        unsigned int faceSize=_coordIndexField[i].size();
        for(unsigned int j=2;j<faceSize;j++)
          {
          if(_ccwField)
            {
            tf.push_back(Util3D::Face(_texCoordIndexField[i][0],
                                      _texCoordIndexField[i][j-1],
                                      _texCoordIndexField[i][j]));
            }
          else
            {
            tf.push_back(Util3D::Face(_texCoordIndexField[i][0],
                                      _texCoordIndexField[i][j],
                                      _texCoordIndexField[i][j-1]));
            }
          }
        }
      else
        {
        unsigned int faceSize=tessInfo[i].size();
        for(unsigned int j=0;j<faceSize;j++)
          {
          const Util3D::Face & face=tessInfo[i][j];
          if(_ccwField)
            {
            tf.push_back(Util3D::Face(_texCoordIndexField[i][face.p1],
                                      _texCoordIndexField[i][face.p2],
                                      _texCoordIndexField[i][face.p3]));
            }
          else
            {
            tf.push_back(Util3D::Face(_texCoordIndexField[i][face.p1],
                                      _texCoordIndexField[i][face.p3],
                                      _texCoordIndexField[i][face.p2]));
            }
          }
        }
      }
    }
  // else --> vertex faces will be reused
  }
else // compute default texture coordinates
  {
  if(bbox.valid)
    {
    double dx=bbox.sizeX();
    double dy=bbox.sizeY();
    double dz=bbox.sizeZ();
    unsigned int nb=v.size();
    if((dx>=dy)&&(dx>=dz))
      {
      if(dy>=dz)
        {
        if(!dx)
          {
          dx=1.0;
          }
        for(unsigned int i=0;i<nb;i++)
          {
          t.push_back(Util3D::Dbl2((v[i].x-bbox.xmin)/dx,
                                   (v[i].y-bbox.ymin)/dx));
          }
        }
      else
        {
        if(!dx)
          {
          dx=1.0;
          }
        for(unsigned int i=0;i<nb;i++)
          {
          t.push_back(Util3D::Dbl2((v[i].x-bbox.xmin)/dx,
                                   (v[i].z-bbox.zmin)/dx));
          }
        }
      }
    else if((dy>=dx)&&(dy>=dz))
      {
      if(dx>=dz)
        {
        if(!dy)
          {
          dy=1.0;
          }
        for(unsigned int i=0;i<nb;i++)
          {
          t.push_back(Util3D::Dbl2((v[i].y-bbox.ymin)/dy,
                                   (v[i].x-bbox.xmin)/dy));
          }
        }
      else
        {
        if(!dy)
          {
          dy=1.0;
          }
        for(unsigned int i=0;i<nb;i++)
          {
          t.push_back(Util3D::Dbl2((v[i].y-bbox.ymin)/dy,
                                   (v[i].z-bbox.zmin)/dy));
          }
        }
      }
    else // (dz>=dx)&&(dz>=dy)
      {
      if(dx>=dy)
        {
        if(!dz)
          {
          dz=1.0;
          }
        for(unsigned int i=0;i<nb;i++)
          {
          t.push_back(Util3D::Dbl2((v[i].z-bbox.zmin)/dz,
                                   (v[i].x-bbox.xmin)/dz));
          }
        }
      else
        {
        if(!dz)
          {
          dz=1.0;
          }
        for(unsigned int i=0;i<nb;i++)
          {
          t.push_back(Util3D::Dbl2((v[i].z-bbox.zmin)/dz,
                                   (v[i].y-bbox.ymin)/dz));
          }
        }
      }
    // --> vertex faces will be reused
    }
  }
if(_texCoord2Field)
  {
  t2=_texCoord2Field->_pointField;
  if(_texCoordIndex2Field.size())
    {
    unsigned int nb=_coordIndexField.size();
    for(unsigned int i=0;i<nb;i++)
      {
      if(_convexField)
        {
        unsigned int faceSize=_coordIndexField[i].size();
        for(unsigned int j=2;j<faceSize;j++)
          {
          if(_ccwField)
            {
            tf2.push_back(Util3D::Face(_texCoordIndex2Field[i][0],
                                       _texCoordIndex2Field[i][j-1],
                                       _texCoordIndex2Field[i][j]));
            }
          else
            {
            tf2.push_back(Util3D::Face(_texCoordIndex2Field[i][0],
                                       _texCoordIndex2Field[i][j],
                                       _texCoordIndex2Field[i][j-1]));
            }
          }
        }
      else
        {
        unsigned int faceSize=tessInfo[i].size();
        for(unsigned int j=0;j<faceSize;j++)
          {
          const Util3D::Face & face=tessInfo[i][j];
          if(_ccwField)
            {
            tf2.push_back(Util3D::Face(_texCoordIndex2Field[i][face.p1],
                                       _texCoordIndex2Field[i][face.p2],
                                       _texCoordIndex2Field[i][face.p3]));
            }
          else
            {
            tf2.push_back(Util3D::Face(_texCoordIndex2Field[i][face.p1],
                                       _texCoordIndex2Field[i][face.p3],
                                       _texCoordIndex2Field[i][face.p2]));
            }
          }
        }
      }
    }
  // else --> vertex faces will be reused
  }
// else --> first texture coordinates will be reused
if(_texCoord3Field)
  {
  t3=_texCoord3Field->_pointField;
  if(_texCoordIndex3Field.size())
    {
    unsigned int nb=_coordIndexField.size();
    for(unsigned int i=0;i<nb;i++)
      {
      if(_convexField)
        {
        unsigned int faceSize=_coordIndexField[i].size();
        for(unsigned int j=2;j<faceSize;j++)
          {
          if(_ccwField)
            {
            tf3.push_back(Util3D::Face(_texCoordIndex3Field[i][0],
                                       _texCoordIndex3Field[i][j-1],
                                       _texCoordIndex3Field[i][j]));
            }
          else
            {
            tf3.push_back(Util3D::Face(_texCoordIndex3Field[i][0],
                                       _texCoordIndex3Field[i][j],
                                       _texCoordIndex3Field[i][j-1]));
            }
          }
        }
      else
        {
        unsigned int faceSize=tessInfo[i].size();
        for(unsigned int j=0;j<faceSize;j++)
          {
          const Util3D::Face & face=tessInfo[i][j];
          if(_ccwField)
            {
            tf3.push_back(Util3D::Face(_texCoordIndex3Field[i][face.p1],
                                       _texCoordIndex3Field[i][face.p2],
                                       _texCoordIndex3Field[i][face.p3]));
            }
          else
            {
            tf3.push_back(Util3D::Face(_texCoordIndex3Field[i][face.p1],
                                       _texCoordIndex3Field[i][face.p3],
                                       _texCoordIndex3Field[i][face.p2]));
            }
          }
        }
      }
    }
  // else --> vertex faces will be reused
  }
// else --> first texture coordinates will be reused
if(_texCoord4Field)
  {
  t4=_texCoord4Field->_pointField;
  if(_texCoordIndex4Field.size())
    {
    unsigned int nb=_coordIndexField.size();
    for(unsigned int i=0;i<nb;i++)
      {
      if(_convexField)
        {
        unsigned int faceSize=_coordIndexField[i].size();
        for(unsigned int j=2;j<faceSize;j++)
          {
          if(_ccwField)
            {
            tf4.push_back(Util3D::Face(_texCoordIndex4Field[i][0],
                                       _texCoordIndex4Field[i][j-1],
                                       _texCoordIndex4Field[i][j]));
            }
          else
            {
            tf4.push_back(Util3D::Face(_texCoordIndex4Field[i][0],
                                       _texCoordIndex4Field[i][j],
                                       _texCoordIndex4Field[i][j-1]));
            }
          }
        }
      else
        {
        unsigned int faceSize=tessInfo[i].size();
        for(unsigned int j=0;j<faceSize;j++)
          {
          const Util3D::Face & face=tessInfo[i][j];
          if(_ccwField)
            {
            tf4.push_back(Util3D::Face(_texCoordIndex4Field[i][face.p1],
                                       _texCoordIndex4Field[i][face.p2],
                                       _texCoordIndex4Field[i][face.p3]));
            }
          else
            {
            tf4.push_back(Util3D::Face(_texCoordIndex4Field[i][face.p1],
                                       _texCoordIndex4Field[i][face.p3],
                                       _texCoordIndex4Field[i][face.p2]));
            }
          }
        }
      }
    }
  // else --> vertex faces will be reused
  }
// else --> first texture coordinates will be reused
mesh->setDoubleSided(!_solidField);
mesh->applyChanges(false);
}

//-------- Forbidden operations ----------------------------------------------

void
IndexedFaceSetNode::_generateFields(ostream & output,
                                    unsigned int indent,
                                    bool showDefaultValues,
                                  StlVector<const VrmlNode *> & defNodesInOut)
                                                            const // Protected
{
if(showDefaultValues||
   _colorField)
  {
  _generateSFNode(output,indent,"color",_colorField,
                  showDefaultValues,defNodesInOut);
  }
if(showDefaultValues||
   _coordField)
  {
  _generateSFNode(output,indent,"coord",_coordField,
                  showDefaultValues,defNodesInOut);
  }
if(showDefaultValues||
   _normalField)
  {
  _generateSFNode(output,indent,"normal",_normalField,
                  showDefaultValues,defNodesInOut);
  }
if(showDefaultValues||
   _texCoordField)
  {
  _generateSFNode(output,indent,"texCoord",_texCoordField,
                  showDefaultValues,defNodesInOut);
  }
if(showDefaultValues||
   _texCoord2Field)
  {
  _generateSFNode(output,indent,"texCoord2",_texCoord2Field,
                  showDefaultValues,defNodesInOut);
  }
if(showDefaultValues||
   _texCoord3Field)
  {
  _generateSFNode(output,indent,"texCoord3",_texCoord3Field,
                  showDefaultValues,defNodesInOut);
  }
if(showDefaultValues||
   _texCoord4Field)
  {
  _generateSFNode(output,indent,"texCoord4",_texCoord4Field,
                  showDefaultValues,defNodesInOut);
  }
if(showDefaultValues||
   !_ccwField)
  {
  _generateSFBool(output,indent,"ccw",_ccwField);
  }
if(showDefaultValues||
   _colorIndexField.size())
  {
  _generateMFInt32(output,indent,"colorIndex",_colorIndexField);
  }
if(showDefaultValues||
   !_colorPerVertexField)
  {
  _generateSFBool(output,indent,"colorPerVertex",_colorPerVertexField);
  }
if(showDefaultValues||
   !_convexField)
  {
  _generateSFBool(output,indent,"convex",_convexField);
  }
if(showDefaultValues||
   _coordIndexField.size())
  {
  _generateMFInt32(output,indent,"coordIndex",_coordIndexField);
  }
if(showDefaultValues||
   _creaseAngleField)
  {
  _generateSFFloat(output,indent,"creaseAngle",_creaseAngleField);
  }
if(showDefaultValues||
   _normalIndexField.size())
  {
  _generateMFInt32(output,indent,"normalIndex",_normalIndexField);
  }
if(showDefaultValues||
   !_normalPerVertexField)
  {
  _generateSFBool(output,indent,"normalPerVertex",_normalPerVertexField);
  }
if(showDefaultValues||
   !_solidField)
  {
  _generateSFBool(output,indent,"solid",_solidField);
  }
if(showDefaultValues||
   _texCoordIndexField.size())
  {
  _generateMFInt32(output,indent,"texCoordIndex",_texCoordIndexField);
  }
if(showDefaultValues||
   _texCoordIndex2Field.size())
  {
  _generateMFInt32(output,indent,"texCoordIndex2",_texCoordIndex2Field);
  }
if(showDefaultValues||
   _texCoordIndex3Field.size())
  {
  _generateMFInt32(output,indent,"texCoordIndex3",_texCoordIndex3Field);
  }
if(showDefaultValues||
   _texCoordIndex4Field.size())
  {
  _generateMFInt32(output,indent,"texCoordIndex4",_texCoordIndex4Field);
  }
}

IndexedFaceSetNode::IndexedFaceSetNode(const StlString & nodeType,
                                       VrmlShape3D * shape) // Protected
: GeometryNode(nodeType,shape),
  _colorField((ColorNode *)0),
  _coordField((CoordinateNode *)0),
  _normalField((NormalNode *)0),
  _texCoordField((TextureCoordinateNode *)0),
  _texCoord2Field((TextureCoordinateNode *)0),
  _texCoord3Field((TextureCoordinateNode *)0),
  _texCoord4Field((TextureCoordinateNode *)0),
  _ccwField(true),
  _colorIndexField(),
  _colorPerVertexField(true),
  _convexField(true),
  _coordIndexField(),
  _creaseAngleField(0.0),
  _normalIndexField(),
  _normalPerVertexField(true),
  _solidField(true),
  _texCoordIndexField(),
  _texCoordIndex2Field(),
  _texCoordIndex3Field(),
  _texCoordIndex4Field()
{
  // Nothing to be done
}

} // namespace AReVi

//----------------------------------------------------------------------------
