// LI2/ENIB - AReVi - VRML services
//----------------------------------------------------------------------------
// file        : AReVi/VRML/indexedLineSetNode_priv.cpp
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/VRML/indexedLineSetNode_priv.h"

#include "AReVi/VRML/vrmlValue_priv.h"
#include "AReVi/VRML/vrmlShape3D.h"

#include "AReVi/VRML/colorNode_priv.h"
#include "AReVi/VRML/coordinateNode_priv.h"
#include "AReVi/VRML/materialNode_priv.h"

#include "AReVi/Shapes/lineSet3D.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class IndexedLineSetNode
//----------------------------------------------------------------------------

//-------- Construction / Destruction ----------------------------------------

IndexedLineSetNode::~IndexedLineSetNode(void)
{
if(_colorField)
  {
  _colorField->removeParent(this);
  }
if(_coordField)
  {
  _coordField->removeParent(this);
  }
}

//-------- Inspection --------------------------------------------------------

bool // made of triangles
IndexedLineSetNode::isSurface(void) const
{
return(false);
}

//-------- Field management --------------------------------------------------

void
IndexedLineSetNode::getNodeFieldNames(StlVector<StlString> & nodeFieldNamesOut)
                                                                         const
{
nodeFieldNamesOut.clear();
nodeFieldNamesOut.push_back("color");
nodeFieldNamesOut.push_back("coord");
}

const VrmlNode * // node at field
IndexedLineSetNode::getNodeField(const StlString & fieldName) const
{
if(fieldName=="color")
  {
  return(_colorField);
  }
else if(fieldName=="coord")
  {
  return(_coordField);
  }
else
  {
  return((const VrmlNode *)0);
  }
}

void
IndexedLineSetNode::addField(const StlString & fieldName,
                             VrmlValue * fieldValue)
{
bool ok=false;
if(fieldName=="color")
  {
  if(fieldValue->checkSFNode())
    {
    VrmlNode * n=fieldValue->accessNode(0);
    if(!n||(n->getType()=="Color"))
      {
      if(!(n&&n->fail()))
        {
        if(_colorField)
          {
          _colorField->removeParent(this);
          }
        _colorField=(ColorNode *)n;
        if(_colorField)
          {
          _colorField->addParent(this);
          }
        }
      ok=true;
      }
    }
  }
else if(fieldName=="coord")
  {
  if(fieldValue->checkSFNode())
    {
    VrmlNode * n=fieldValue->accessNode(0);
    if(!n||(n->getType()=="Coordinate"))
      {
      if(!(n&&n->fail()))
        {
        if(_coordField)
          {
          _coordField->removeParent(this);
          }
        _coordField=(CoordinateNode *)n;
        if(_coordField)
          {
          _coordField->addParent(this);
          }
        }
      ok=true;
      }
    }
  }
else if(fieldName=="colorIndex")
  {
  if(fieldValue->checkMFInt32())
    {
    unsigned int nb=fieldValue->getSize();
    ok=true;
    for(unsigned int i=nb;i--;)
      {
      if(fieldValue->getInteger(i)<-1)
        {
        ok=false;
        break;
        }
      }
    if(ok)
      {
      _colorIndexField.clear();
      if(nb)
        {
        _colorIndexField.push_back(StlVector<int>());
        }
      for(unsigned int i=0;i<nb;i++)
        {
        if(fieldValue->getInteger(i)==-1)
          {
          if(i<nb-1)
            {
            _colorIndexField.push_back(StlVector<int>());
            }
          }
        else
          {
          _colorIndexField.back().push_back(fieldValue->getInteger(i));
          }
        }
      }
    }
  }
else if(fieldName=="colorPerVertex")
  {
  if(fieldValue->checkSFBool())
    {
    _colorPerVertexField=fieldValue->getBool(0);
    ok=true;
    }
  }
else if(fieldName=="coordIndex")
  {
  if(fieldValue->checkMFInt32())
    {
    unsigned int nb=fieldValue->getSize();
    ok=true;
    for(unsigned int i=nb;i--;)
      {
      if(fieldValue->getInteger(i)<-1)
        {
        ok=false;
        break;
        }
      }
    if(ok)
      {
      _coordIndexField.clear();
      if(nb)
        {
        _coordIndexField.push_back(StlVector<int>());
        }
      for(unsigned int i=0;i<nb;i++)
        {
        if(fieldValue->getInteger(i)==-1)
          {
          if(i<nb-1)
            {
            _coordIndexField.push_back(StlVector<int>());
            }
          }
        else
          {
          _coordIndexField.back().push_back(fieldValue->getInteger(i));
          }
        }
      }
    }
  }
else
  {
  GeometryNode::addField(fieldName,fieldValue);
  ok=true;
  }
if(!ok)
  {
  _shape->_badFieldMsg(fieldName,_type);
  }
}

void
IndexedLineSetNode::check(void)
{
_fail=false;
unsigned int nbVertex=0;
if(_coordField)
  {
  nbVertex=_coordField->_pointField.size();
  }
for(unsigned int i=_coordIndexField.size();i--;)
  {
  unsigned int j=_coordIndexField[i].size();
  while(j--)
    {
    unsigned int index=_coordIndexField[i][j];
    if(index>=nbVertex)
      {
      StlString msg="Bad vertex index (";
      msg+=uintToStr(index);
      msg+=">";
      msg+=uintToStr(nbVertex-1);
      msg+=") for field 'coordIndex' in node type '";
      msg+=_type;
      msg+="'";
      _shape->_errorMsg(msg);
      _fail=true;
      i=j=0;
      }
    }
  }
if(_colorField)
  {
  unsigned int nbColor=_colorField->_colorField.size();
  if(_colorPerVertexField)
    {
    if(_colorIndexField.size())
      {
      bool ok=true;
      if(_coordIndexField.size()>_colorIndexField.size())
        {
        ok=false;
        }
      else
        {
        for(unsigned int i=_coordIndexField.size();i--;)
          {
          unsigned int j=_colorIndexField[i].size();
          if(_coordIndexField[i].size()!=j)
            {
            ok=false;
            break;
            }
          while(j--)
            {
            unsigned int index=_colorIndexField[i][j];
            if(index>=nbColor)
              {
              StlString msg="Bad color index (";
              msg+=uintToStr(index);
              msg+=">";
              msg+=uintToStr(nbColor-1);
              msg+=") for field 'colorIndex' in node type '";
              msg+=_type;
              msg+="'";
              _shape->_errorMsg(msg);
              _fail=true;
              i=j=0;
              }
            }
          }
        }
      if(!ok)
        {
        StlString msg="Field 'colorIndex' and field 'coordIndex' ";
        msg+="do not match in node type '";
        msg+=_type;
        msg+="'";
        _shape->_errorMsg(msg);
        _fail=true;
        }
      }
    else // coordIndex instead of colorIndex
      {
      for(unsigned int i=_coordIndexField.size();i--;)
        {
        for(unsigned int j=_coordIndexField[i].size();j--;)
          {
          unsigned int index=_coordIndexField[i][j];
          if(index>=nbColor)
            {
            StlString msg="Bad color index (";
            msg+=uintToStr(index);
            msg+=">";
            msg+=uintToStr(nbColor-1);
            msg+=") for field 'coordIndex' in node type '";
            msg+=_type;
            msg+="'";
            _shape->_errorMsg(msg);
            _fail=true;
            i=j=0;
            }
          }
        }
      }
    }
  else
    {
    if(_colorIndexField.size())
      {
      if((_colorIndexField.size()!=1)||
         (_colorIndexField.back().size()<_coordIndexField.size()))
        {
        StlString msg="Field 'colorIndex' and field 'coordIndex' ";
        msg+="do not match in node type '";
        msg+=_type;
        msg+="'";
        _shape->_errorMsg(msg);
        _fail=true;
        }
      else
        {
        for(unsigned int i=_colorIndexField.back().size();i--;)
          {
          unsigned int index=_colorIndexField.back()[i];
          if(index>=nbColor)
            {
            StlString msg="Bad color index (";
            msg+=uintToStr(index);
            msg+=">";
            msg+=uintToStr(nbColor-1);
            msg+=") for field 'coordIndex' in node type '";
            msg+=_type;
            msg+="'";
            _shape->_errorMsg(msg);
            _fail=true;
            break;
            }
          }
        }
      }
    else
      {
      if(nbColor<_coordIndexField.size())
        {
        StlString msg="Field 'color' and field 'coordIndex' ";
        msg+="do not match in node type '";
        msg+=_type;
        msg+="'";
        _shape->_errorMsg(msg);
        _fail=true;
        }
      }
    }
  }
if(!_fail)
  {
  rebuild();
  }
}

//-------- Rendering ---------------------------------------------------------

ArRef<ShapePart3D> // built part
IndexedLineSetNode::buildPart(Util3D::Transform & initTransformOut) const
{
initTransformOut.identity();
return(LineSet3D::NEW());
}

void
IndexedLineSetNode::updatePart(ArRef<ShapePart3D> part,
                               const MaterialNode * material) const
{
ArRef<LineSet3D> lines=ar_down_cast<LineSet3D>(part);
StlVector<StlVector<Util3D::Dbl3> > & v=lines->accessVertices();
v.clear();
StlVector<StlVector<Util3D::Dbl3> > & c=lines->accessColors();
c.clear();
StlVector<Util3D::Dbl3> emptyVector;
unsigned int i;
unsigned int nbL=_coordIndexField.size();
for(i=0;i<nbL;i++)
  {
  v.push_back(emptyVector);
  c.push_back(emptyVector);
  unsigned int nbV=_coordIndexField[i].size();
  for(unsigned int j=0;j<nbV;j++)
    {
    v[i].push_back(_coordField->_pointField[_coordIndexField[i][j]]);
    }
  }
if(_colorField&&_colorField->_colorField.size())
  {
  if(_colorPerVertexField)
    {
    const StlVector<StlVector<int> > & index=
      (_colorIndexField.empty() ? _coordIndexField : _colorIndexField);
    for(i=0;i<nbL;i++)
      {
      unsigned int nbV=v[i].size();
      for(unsigned int j=0;j<nbV;j++)
        {
        c[i].push_back(_colorField->_colorField[index[i][j]]);
        }
      }
    }
  else
    {
    if(_colorIndexField.empty())
      {
      for(i=0;i<nbL;i++)
        {
        c[i].push_back(_colorField->_colorField[i]);
        }
      }
    else
      {
      for(i=0;i<nbL;i++)
        {
        c[i].push_back(_colorField->_colorField[_colorIndexField.back()[i]]);
        }
      }
    }
  }
else if(material)
  {
  for(i=c.size();i--;)
    {
    c[i].push_back(material->_emissiveColorField);
    }
  }
lines->applyChanges(false);
}

//-------- Forbidden operations ----------------------------------------------

void
IndexedLineSetNode::_generateFields(ostream & output,
                                    unsigned int indent,
                                    bool showDefaultValues,
                                  StlVector<const VrmlNode *> & defNodesInOut)
                                                            const // Protected
{
if(showDefaultValues||
   _colorField)
  {
  _generateSFNode(output,indent,"color",_colorField,
                  showDefaultValues,defNodesInOut);
  }
if(showDefaultValues||
   _coordField)
  {
  _generateSFNode(output,indent,"coord",_coordField,
                  showDefaultValues,defNodesInOut);
  }
if(showDefaultValues||
   !_colorIndexField.empty())
  {
  _generateMFInt32(output,indent,"colorIndex",_colorIndexField);
  }
if(showDefaultValues||
   !_colorPerVertexField)
  {
  _generateSFBool(output,indent,"colorPerVertex",_colorPerVertexField);
  }
if(showDefaultValues||
   !_coordIndexField.empty())
  {
  _generateMFInt32(output,indent,"coordIndex",_coordIndexField);
  }
}

IndexedLineSetNode::IndexedLineSetNode(const StlString & nodeType,
                                       VrmlShape3D * shape) // Protected
: GeometryNode(nodeType,shape),
  _colorField((ColorNode *)0),
  _coordField((CoordinateNode *)0),
  _colorIndexField(),
  _colorPerVertexField(true),
  _coordIndexField()
{
  // Nothing to be done
}

} // namespace AReVi

//----------------------------------------------------------------------------
