// LI2/ENIB - AReVi - VRML services
//----------------------------------------------------------------------------
// file        : AReVi/VRML/inlineNode_priv.cpp
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/VRML/inlineNode_priv.h"

#include "AReVi/VRML/vrmlValue_priv.h"
#include "AReVi/VRML/vrmlShape3D.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class InlineNode
//----------------------------------------------------------------------------

//-------- Construction / Destruction ----------------------------------------

InlineNode::~InlineNode(void)
{
  // Nothing to be done
}

//-------- Field management --------------------------------------------------

void
InlineNode::addField(const StlString & fieldName,
                     VrmlValue * fieldValue)
{
bool ok=false;
if(fieldName=="url")
  {
  if(fieldValue->checkMFString())
    {
    _urlField.clear();
    unsigned int nb=fieldValue->getSize();
    for(unsigned int i=0;i<nb;i++)
      {
      _urlField.push_back(fieldValue->getString(i));
      }
    ok=true;
    }
  }
else if(fieldName=="bboxCenter")
  {
  if(fieldValue->checkSFVec3f())
    {
    _bboxCenterField.x=fieldValue->getReal(0);
    _bboxCenterField.y=fieldValue->getReal(1);
    _bboxCenterField.z=fieldValue->getReal(2);
    ok=true;
    }
  }
else if(fieldName=="bboxSize")
  {
  if(fieldValue->checkSFVec3f()&&
     (((fieldValue->getReal(0)>0.0)&&
       (fieldValue->getReal(1)>0.0)&&
       (fieldValue->getReal(2)>0.0))||
      ((fieldValue->getReal(0)==-1.0)&&
       (fieldValue->getReal(1)==-1.0)&&
       (fieldValue->getReal(2)==-1.0))))
    {
    _bboxSizeField.x=fieldValue->getReal(0);
    _bboxSizeField.y=fieldValue->getReal(1);
    _bboxSizeField.z=fieldValue->getReal(2);
    ok=true;
    }
  }
else
  {
  GroupChildNode::addField(fieldName,fieldValue);
  ok=true;
  }
if(!ok)
  {
  _shape->_badFieldMsg(fieldName,_type);
  }
}

void
InlineNode::check(void)
{
_fail=false;
}

//-------- Forbidden operations ----------------------------------------------

void
InlineNode::_generateFields(ostream & output,
                            unsigned int indent,
                            bool showDefaultValues,
                            StlVector<const VrmlNode *> & /* defNodesInOut */)
                                                            const // Protected
{
if(showDefaultValues||
   !_urlField.empty())
  {
  _generateMFString(output,indent,"url",_urlField);
  }
if(showDefaultValues||
   _bboxCenterField.x||
   _bboxCenterField.y||
   _bboxCenterField.z)
  {
  _generateSFVec3f(output,indent,"bboxCenter",_bboxCenterField);
  }
if(showDefaultValues||
   (_bboxSizeField.x!=-1.0f)||
   (_bboxSizeField.y!=-1.0f)||
   (_bboxSizeField.z!=-1.0f))
  {
  _generateSFVec3f(output,indent,"bboxSize",_bboxSizeField);
  }
}

InlineNode::InlineNode(const StlString & nodeType,
                       VrmlShape3D * shape) // Protected
: GroupChildNode(nodeType,shape),
  _urlField(),
  _bboxCenterField(0.0,0.0,0.0),
  _bboxSizeField(-1.0,-1.0,-1.0)
{
  // Nothing to be done
}

} // namespace AReVi

//----------------------------------------------------------------------------
