// LI2/ENIB - AReVi - VRML services
//----------------------------------------------------------------------------
// file        : AReVi/VRML/materialNode_priv.cpp
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/VRML/materialNode_priv.h"

#include "AReVi/VRML/vrmlValue_priv.h"
#include "AReVi/VRML/vrmlShape3D.h"

#include "AReVi/VRML/appearanceNode_priv.h"

#include "AReVi/Lib3D/material3D.h"

#include "AReVi/Shapes/surface3D.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class MaterialNode
//----------------------------------------------------------------------------

//-------- Construction / Destruction ----------------------------------------

MaterialNode::~MaterialNode(void)
{
  // Nothing to be done
}

//-------- Field management --------------------------------------------------

void
MaterialNode::addField(const StlString & fieldName,
                       VrmlValue * fieldValue)
{
bool ok=false;
if(fieldName=="ambientIntensity")
  {
  if(fieldValue->checkSFFloat()&&
     (fieldValue->getReal(0)>=0.0)&&
     (fieldValue->getReal(0)<=1.0))
    {
    _ambientIntensityField=fieldValue->getReal(0);
    ok=true;
    }
  }
else if(fieldName=="diffuseColor")
  {
  if(fieldValue->checkSFColor())
    {
    _diffuseColorField.x=fieldValue->getReal(0);
    _diffuseColorField.y=fieldValue->getReal(1);
    _diffuseColorField.z=fieldValue->getReal(2);
    ok=true;
    }
  }
else if(fieldName=="emissiveColor")
  {
  if(fieldValue->checkSFColor())
    {
    _emissiveColorField.x=fieldValue->getReal(0);
    _emissiveColorField.y=fieldValue->getReal(1);
    _emissiveColorField.z=fieldValue->getReal(2);
    ok=true;
    }
  }
else if(fieldName=="shininess")
  {
  if(fieldValue->checkSFFloat()&&
     (fieldValue->getReal(0)>=0.0)&&
     (fieldValue->getReal(0)<=1.0))
    {
    _shininessField=fieldValue->getReal(0);
    ok=true;
    }
  }
else if(fieldName=="specularColor")
  {
  if(fieldValue->checkSFColor())
    {
    _specularColorField.x=fieldValue->getReal(0);
    _specularColorField.y=fieldValue->getReal(1);
    _specularColorField.z=fieldValue->getReal(2);
    ok=true;
    }
  }
else if(fieldName=="transparency")
  {
  if(fieldValue->checkSFFloat()&&
     (fieldValue->getReal(0)>=0.0)&&
     (fieldValue->getReal(0)<=1.0))
    {
    _transparencyField=fieldValue->getReal(0);
    ok=true;
    }
  }
else
  {
  VrmlNode::addField(fieldName,fieldValue);
  ok=true;
  }
if(!ok)
  {
  _shape->_badFieldMsg(fieldName,_type);
  }
}

void
MaterialNode::check(void)
{
_fail=false;
}

//-------- Rendering ---------------------------------------------------------

void
MaterialNode::postRebuild(void)
{
for(unsigned int i=_parents.size();i--;)
  {
  static_cast<AppearanceNode *>(_parents[i])->postMaterialRebuild();
  }
}

void
MaterialNode::rebuild(void)
{
if(!_material)
  {
  _material=Material3D::NEW();
  }
Util3D::Material & m=_material->accessMaterial();
m.setLighting(true);
m.setShininess(_shininessField);
m.setAmbientIntensity(_ambientIntensityField);
m.setDiffuseColor(_diffuseColorField.x,
                  _diffuseColorField.y,
                  _diffuseColorField.z);
m.setEmissiveColor(_emissiveColorField.x,
                   _emissiveColorField.y,
                   _emissiveColorField.z);
m.setSpecularColor(_specularColorField.x,
                   _specularColorField.y,
                   _specularColorField.z);
m.setTransparency(_transparencyField>0.01 ? _transparencyField : 0.0);
}

void
MaterialNode::updatePart(ArRef<Surface3D> part) const
{
part->writeMaterial(_material);
}

//-------- Forbidden operations ----------------------------------------------

void
MaterialNode::_generateFields(ostream & output,
                              unsigned int indent,
                              bool showDefaultValues,
                            StlVector<const VrmlNode *> & /* defNodesInOut */)
                                                            const // Protected
{
if(showDefaultValues||
   (_ambientIntensityField!=0.2f))
  {
  _generateSFFloat(output,indent,"ambientIntensity",_ambientIntensityField);
  }
if(showDefaultValues||
   (_diffuseColorField.x!=0.8f)||
   (_diffuseColorField.y!=0.8f)||
   (_diffuseColorField.z!=0.8f))
  {
  _generateSFColor(output,indent,"diffuseColor",_diffuseColorField);
  }
if(showDefaultValues||
   _emissiveColorField.x||
   _emissiveColorField.y||
   _emissiveColorField.z)
  {
  _generateSFColor(output,indent,"emissiveColor",_emissiveColorField);
  }
if(showDefaultValues||
   (_shininessField!=0.2f))
  {
  _generateSFFloat(output,indent,"shininess",_shininessField);
  }
if(showDefaultValues||
   _specularColorField.x||
   _specularColorField.y||
   _specularColorField.z)
  {
  _generateSFColor(output,indent,"specularColor",_specularColorField);
  }
if(showDefaultValues||
   _transparencyField)
  {
  _generateSFFloat(output,indent,"transparency",_transparencyField);
  }
}

MaterialNode::MaterialNode(const StlString & nodeType,
                           VrmlShape3D * shape) // Protected
: VrmlNode(nodeType,shape),
  _ambientIntensityField(0.2),
  _diffuseColorField(0.8,0.8,0.8),
  _emissiveColorField(0.0,0.0,0.0),
  _shininessField(0.2),
  _specularColorField(0.0,0.0,0.0),
  _transparencyField(0.0),
  _material()
{
  // Nothing to be done
}

} // namespace AReVi

//----------------------------------------------------------------------------
