// LI2/ENIB - AReVi - VRML services
//----------------------------------------------------------------------------
// file        : AReVi/VRML/pixelTextureNode_priv.cpp
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/VRML/pixelTextureNode_priv.h"

#include "AReVi/VRML/vrmlValue_priv.h"
#include "AReVi/VRML/vrmlShape3D.h"

#include "AReVi/Lib3D/dataTexture.h"

#include "AReVi/Utils/memoryBlock.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class PixelTextureNode
//----------------------------------------------------------------------------

//-------- Construction / Destruction ----------------------------------------

PixelTextureNode::~PixelTextureNode(void)
{
  // Nothing to be done
}

//-------- Field management --------------------------------------------------

void
PixelTextureNode::addField(const StlString & fieldName,
                           VrmlValue * fieldValue)
{
bool ok=false;
if(fieldName=="image")
  {
  if(fieldValue->checkSFImage())
    {
    _imageField.clear();
    unsigned int nb=fieldValue->getSize();
    for(unsigned int i=0;i<nb;i++)
      {
      _imageField.push_back(fieldValue->getInteger(i));
      }
    ok=true;
    }
  }
else
  {
  TextureNode::addField(fieldName,fieldValue);
  ok=true;
  }
if(!ok)
  {
  _shape->_badFieldMsg(fieldName,_type);
  }
}

void
PixelTextureNode::check(void)
{
_fail=false;
if(!_fail)
  {
  rebuild();
  }
}

//-------- Rendering ---------------------------------------------------------

void
PixelTextureNode::rebuild(void)
{
if(_needsRebuild)
  {
  _texture.clear();
  if(_imageField.size()>=3)
    {
    unsigned int width=_imageField[0];
    unsigned int height=_imageField[1];
    unsigned int components=_imageField[2];
    unsigned int size=width*height;
    ArRef<MemoryBlock> data=MemoryBlock::NEW();
    if(components&&(components<=4)&&
       (_imageField.size()>=size+3)&&
       data->ensureMinSize(size*components))
      {
      unsigned char * ptr=(unsigned char *)data->accessMemory();
      const unsigned int * pixels=&_imageField[3];
      switch(components)
        {
        case 1:
          {
          for(unsigned int i=width*height;i--;)
            {
            ptr[i]=(unsigned char)((pixels[i])&0x000000FF);
            }
          break;
          }
        case 2:
          {
          for(unsigned int i=width*height;i--;)
            {
            ptr[2*i]=(unsigned char)((pixels[i]>>8)&0x000000FF);
            ptr[2*i+1]=(unsigned char)((pixels[i])&0x000000FF);
            }
          break;
          }
        case 3:
          {
          for(unsigned int i=width*height;i--;)
            {
            ptr[3*i]=(unsigned char)((pixels[i]>>16)&0x000000FF);
            ptr[3*i+1]=(unsigned char)((pixels[i]>>8)&0x000000FF);
            ptr[3*i+2]=(unsigned char)((pixels[i])&0x000000FF);
            }
          break;
          }
        case 4:
          {
          for(unsigned int i=width*height;i--;)
            {
            ptr[4*i]=(unsigned char)((pixels[i]>>24)&0x000000FF);
            ptr[4*i+1]=(unsigned char)((pixels[i]>>16)&0x000000FF);
            ptr[4*i+2]=(unsigned char)((pixels[i]>>8)&0x000000FF);
            ptr[4*i+3]=(unsigned char)((pixels[i])&0x000000FF);
            }
          break;
          }
        }
      _texture=DataTexture::NEW(width,height,components,data,true,true);
      if(_texture->fail())
        {
        _texture.clear();
        }
      }
    }
  _needsRebuild=false;
  }
}

//-------- Forbidden operations ----------------------------------------------

void
PixelTextureNode::_generateFields(ostream & output,
                                  unsigned int indent,
                                  bool showDefaultValues,
                                  StlVector<const VrmlNode *> & defNodesInOut)
                                                            const // Protected
{
TextureNode::_generateFields(output,indent,showDefaultValues,defNodesInOut);
if(showDefaultValues||
   _imageField[0]||
   _imageField[1]||
   _imageField[2])
  {
  _generateSFImage(output,indent,"image",_imageField);
  }
}

PixelTextureNode::PixelTextureNode(const StlString & nodeType,
                                   VrmlShape3D * shape) // Protected
: TextureNode(nodeType,shape),
  _imageField()
{
_imageField.push_back(0);
_imageField.push_back(0);
_imageField.push_back(0);
}

} // namespace AReVi

//----------------------------------------------------------------------------
