// LI2/ENIB - AReVi - VRML services
//----------------------------------------------------------------------------
// file        : AReVi/VRML/pointSetNode_priv.cpp
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/VRML/pointSetNode_priv.h"

#include "AReVi/VRML/vrmlValue_priv.h"
#include "AReVi/VRML/vrmlShape3D.h"

#include "AReVi/VRML/colorNode_priv.h"
#include "AReVi/VRML/coordinateNode_priv.h"
#include "AReVi/VRML/materialNode_priv.h"

#include "AReVi/Shapes/pointSet3D.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class PointSetNode
//----------------------------------------------------------------------------

//-------- Construction / Destruction ----------------------------------------

PointSetNode::~PointSetNode(void)
{
if(_colorField)
  {
  _colorField->removeParent(this);
  }
if(_coordField)
  {
  _coordField->removeParent(this);
  }
}

//-------- Inspection --------------------------------------------------------

bool // made of triangles
PointSetNode::isSurface(void) const
{
return(false);
}

//-------- Field management --------------------------------------------------

void
PointSetNode::getNodeFieldNames(StlVector<StlString> & nodeFieldNamesOut)
                                                                         const
{
nodeFieldNamesOut.clear();
nodeFieldNamesOut.push_back("color");
nodeFieldNamesOut.push_back("coord");
}

const VrmlNode * // node at field
PointSetNode::getNodeField(const StlString & fieldName) const
{
if(fieldName=="color")
  {
  return(_colorField);
  }
else if(fieldName=="coord")
  {
  return(_coordField);
  }
else
  {
  return((const VrmlNode *)0);
  }
}

void
PointSetNode::addField(const StlString & fieldName,
                       VrmlValue * fieldValue)
{
bool ok=false;
if(fieldName=="color")
  {
  if(fieldValue->checkSFNode())
    {
    VrmlNode * n=fieldValue->accessNode(0);
    if(!n||(n->getType()=="Color"))
      {
      if(!(n&&n->fail()))
        {
        if(_colorField)
          {
          _colorField->removeParent(this);
          }
        _colorField=(ColorNode *)n;
        if(_colorField)
          {
          _colorField->addParent(this);
          }
        }
      ok=true;
      }
    }
  }
else if(fieldName=="coord")
  {
  if(fieldValue->checkSFNode())
    {
    VrmlNode * n=fieldValue->accessNode(0);
    if(!n||(n->getType()=="Coordinate"))
      {
      if(!(n&&n->fail()))
        {
        if(_coordField)
          {
          _coordField->removeParent(this);
          }
        _coordField=(CoordinateNode *)n;
        if(_coordField)
          {
          _coordField->addParent(this);
          }
        }
      ok=true;
      }
    }
  }
else
  {
  GeometryNode::addField(fieldName,fieldValue);
  ok=true;
  }
if(!ok)
  {
  _shape->_badFieldMsg(fieldName,_type);
  }
}

void
PointSetNode::check(void)
{
_fail=false;
unsigned int nbVertex=0;
if(_coordField)
  {
  nbVertex=_coordField->_pointField.size();
  }
unsigned int nbColor=0;
if(_colorField)
  {
  nbColor=_colorField->_colorField.size();
  }
if(_colorField&&(nbVertex>nbColor))
  {
  StlString msg="Field 'colorIndex' and field 'coordIndex' ";
  msg+="do not match in node type '";
  msg+=_type;
  msg+="'";
  _shape->_errorMsg(msg);
  _fail=true;
  }
if(!_fail)
  {
  rebuild();
  }
}

//-------- Rendering ---------------------------------------------------------

ArRef<ShapePart3D> // built part
PointSetNode::buildPart(Util3D::Transform & initTransformOut) const
{
initTransformOut.identity();
return(PointSet3D::NEW());
}

void
PointSetNode::updatePart(ArRef<ShapePart3D> part,
                         const MaterialNode * material) const
{
ArRef<PointSet3D> points=ar_down_cast<PointSet3D>(part);
StlVector<Util3D::Dbl3> & v=points->accessVertices();
if(_coordField)
  {
  v=_coordField->_pointField;
  }
else
  {
  v.clear();
  }
StlVector<Util3D::Dbl3> & c=points->accessColors();
if(_colorField)
  {
  c=_colorField->_colorField;
  }
else
  {
  c.clear();
  }
if(c.empty()&&material)
  {
  c.push_back(material->_emissiveColorField);
  }
points->applyChanges(false);
}

//-------- Forbidden operations ----------------------------------------------

void
PointSetNode::_generateFields(ostream & output,
                              unsigned int indent,
                              bool showDefaultValues,
                              StlVector<const VrmlNode *> & defNodesInOut)
                                                            const // Protected
{
if(showDefaultValues||
   _colorField)
  {
  _generateSFNode(output,indent,"color",_colorField,
                  showDefaultValues,defNodesInOut);
  }
if(showDefaultValues||
   _coordField)
  {
  _generateSFNode(output,indent,"coord",_coordField,
                  showDefaultValues,defNodesInOut);
  }
}

PointSetNode::PointSetNode(const StlString & nodeType,
                           VrmlShape3D * shape) // Protected
: GeometryNode(nodeType,shape),
  _colorField((ColorNode *)0),
  _coordField((CoordinateNode *)0)
{
  // Nothing to be done
}

} // namespace AReVi

//----------------------------------------------------------------------------
