// LI2/ENIB - AReVi - VRML services
//----------------------------------------------------------------------------
// file        : AReVi/VRML/shapeNode_priv.cpp
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/VRML/shapeNode_priv.h"

#include "AReVi/VRML/vrmlValue_priv.h"
#include "AReVi/VRML/vrmlShape3D.h"
#include "AReVi/Lib3D/transform3D.h"
#include "AReVi/Lib3D/material3D.h"
#include "AReVi/Shapes/surface3D.h"

#include "AReVi/VRML/appearanceNode_priv.h"
#include "AReVi/VRML/geometryNode_priv.h"
#include "AReVi/VRML/materialNode_priv.h"
#include "AReVi/VRML/textureTransformNode_priv.h"
#include "AReVi/VRML/textureNode_priv.h"
#include "AReVi/VRML/transformNode_priv.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class ShapeNode
//----------------------------------------------------------------------------

//-------- Construction / Destruction ----------------------------------------

ShapeNode::~ShapeNode(void)
{
while(!_partInfos.empty())
  {
  _shape->removeRootPart(_partInfos.back().part);
  _partInfos.pop_back();
  }
if(_appearanceField)
  {
  _appearanceField->removeParent(this);
  }
if(_geometryField)
  {
  _geometryField->removeParent(this);
  }
}

//-------- Field management --------------------------------------------------

void
ShapeNode::getNodeFieldNames(StlVector<StlString> & nodeFieldNamesOut) const
{
nodeFieldNamesOut.clear();
nodeFieldNamesOut.push_back("appearance");
nodeFieldNamesOut.push_back("geometry");
}

const VrmlNode * // node at field
ShapeNode::getNodeField(const StlString & fieldName) const
{
if(fieldName=="appearance")
  {
  return(_appearanceField);
  }
else if(fieldName=="geometry")
  {
  return(_geometryField);
  }
else
  {
  return((const VrmlNode *)0);
  }
}

void
ShapeNode::addField(const StlString & fieldName,
                    VrmlValue * fieldValue)
{
bool ok=false;
if(fieldName=="appearance")
  {
  if(fieldValue->checkSFNode())
    {
    VrmlNode * n=fieldValue->accessNode(0);
    if(!n||(n->getType()=="Appearance"))
      {
      if(!(n&&n->fail()))
        {
        if(_appearanceField)
          {
          _appearanceField->removeParent(this);
          }
        _appearanceField=(AppearanceNode *)n;
        if(_appearanceField)
          {
          _appearanceField->addParent(this);
          }
        }
      ok=true;
      }
    }
  }
else if(fieldName=="geometry")
  {
  if(fieldValue->checkSFNode())
    {
    VrmlNode * n=fieldValue->accessNode(0);
    if(!n||n->isGeometry())
      {
      if(!(n&&n->fail()))
        {
        if(_geometryField)
          {
          _geometryField->removeParent(this);
          }
        _geometryField=(GeometryNode *)n;
        if(_geometryField)
          {
          _geometryField->addParent(this);
          }
        }
      ok=true;
      }
    }
  }
else
  {
  GroupChildNode::addField(fieldName,fieldValue);
  ok=true;
  }
if(!ok)
  {
  _shape->_badFieldMsg(fieldName,_type);
  }
}

void
ShapeNode::check(void)
{
_fail=false;
}

//-------- Rendering ---------------------------------------------------------

void
ShapeNode::buildParts(const StlVector<TransformNode *> & transformPath)
{
if(_geometryField)
  {
  Util3D::Transform initTransform(true);
  ArRef<ShapePart3D> part=_geometryField->buildPart(initTransform);
  _partInfos.push_back(ShapeNode::PartInfo(part,initTransform,transformPath));
  _shape->addRootPart(part);
  postTransformation();
  }
}

void
ShapeNode::getParts(StlVector<ArConstRef<ShapePart3D> > & partsOut) const
{
for(unsigned int i=_partInfos.size();i--;)
  {
  ArConstRef<ShapePart3D> part=_partInfos[i].part;
  if(part&&(part->getShape().c_ptr()==_shape))
    {
    partsOut.push_back(part);
    }
  }
}

void
ShapeNode::accessParts(StlVector<ArRef<ShapePart3D> > & partsOut)
{
for(unsigned int i=_partInfos.size();i--;)
  {
  ArRef<ShapePart3D> part=_partInfos[i].part;
  if(part&&(part->getShape().c_ptr()==_shape))
    {
    partsOut.push_back(part);
    }
  }
}

void
ShapeNode::postMaterialRebuild(void)
{
postRebuild();
_mustBuildMaterial=true;
}

void
ShapeNode::postTextureRebuild(unsigned int which)
{
postRebuild();
switch(which)
  {
  case 1:
    _mustBuildTexture=true;
    break;
  case 2:
    _mustBuildTexture2=true;
    break;
  case 3:
    _mustBuildTexture3=true;
    break;
  case 4:
    _mustBuildTexture4=true;
    break;
  }
}

void
ShapeNode::postTextureTransformRebuild(unsigned int which)
{
postRebuild();
switch(which)
  {
  case 1:
    _mustBuildTextureTransform=true;
    break;
  case 2:
    _mustBuildTextureTransform2=true;
    break;
  case 3:
    _mustBuildTextureTransform3=true;
    break;
  case 4:
    _mustBuildTextureTransform4=true;
    break;
  }
}

void
ShapeNode::postGeometryRebuild(void)
{
postRebuild();
_mustBuildGeometry=true;
}

void
ShapeNode::rebuild(void)
{
if(_needsRebuild)
  {
  if(_geometryField)
    {
    StlVector<ArRef<ShapePart3D> > parts;
    accessParts(parts);
    unsigned int nbParts=parts.size();
    if(_geometryField->isSurface())
      {
      if(_mustBuildMaterial)
        {
        if(_appearanceField&&_appearanceField->_materialField)
          {
          _appearanceField->_materialField->rebuild();
          for(unsigned int i=nbParts;i--;)
            {
            _appearanceField->_materialField
                            ->updatePart(ar_down_cast<Surface3D>(parts[i]));
            }
          }
        else
          {
          if(!_lightOff)
            {
            _lightOff=Material3D::NEW();
            _lightOff->setEmissiveColor(0.8,0.8,0.8);
            _lightOff->setLighting(false);
            }
          for(unsigned int i=nbParts;i--;)
            {
            ar_down_cast<Surface3D>(parts[i])->writeMaterial(_lightOff);
            }
          }
        _mustBuildMaterial=false;
        }
      if(_mustBuildTexture)
        {
        if(_appearanceField&&_appearanceField->_textureField)
          {
          _appearanceField->_textureField->rebuild();
          for(unsigned int i=nbParts;i--;)
            {
            _appearanceField->_textureField
                            ->updatePart(ar_down_cast<Surface3D>(parts[i]),1);
            }
          }
        _mustBuildTexture=false;
        }
      if(_mustBuildTextureTransform)
        {
        if(_appearanceField&&_appearanceField->_textureTransformField)
          {
          _appearanceField->_textureTransformField->rebuild();
          for(unsigned int i=nbParts;i--;)
            {
            _appearanceField->_textureTransformField
                            ->updatePart(ar_down_cast<Surface3D>(parts[i]),1);
            }
          }
        _mustBuildTextureTransform=false;
        }
      if(_mustBuildTexture2)
        {
        if(_appearanceField&&_appearanceField->_texture2Field)
          {
          _appearanceField->_texture2Field->rebuild();
          for(unsigned int i=nbParts;i--;)
            {
            _appearanceField->_texture2Field
                            ->updatePart(ar_down_cast<Surface3D>(parts[i]),2);
            }
          }
        _mustBuildTexture2=false;
        }
      if(_mustBuildTextureTransform2)
        {
        if(_appearanceField&&_appearanceField->_textureTransform2Field)
          {
          _appearanceField->_textureTransform2Field->rebuild();
          for(unsigned int i=nbParts;i--;)
            {
            _appearanceField->_textureTransform2Field
                            ->updatePart(ar_down_cast<Surface3D>(parts[i]),2);
            }
          }
        _mustBuildTextureTransform2=false;
        }
      if(_mustBuildTexture3)
        {
        if(_appearanceField&&_appearanceField->_texture3Field)
          {
          _appearanceField->_texture3Field->rebuild();
          for(unsigned int i=nbParts;i--;)
            {
            _appearanceField->_texture3Field
                            ->updatePart(ar_down_cast<Surface3D>(parts[i]),3);
            }
          }
        _mustBuildTexture3=false;
        }
      if(_mustBuildTextureTransform3)
        {
        if(_appearanceField&&_appearanceField->_textureTransform3Field)
          {
          _appearanceField->_textureTransform3Field->rebuild();
          for(unsigned int i=nbParts;i--;)
            {
            _appearanceField->_textureTransform3Field
                            ->updatePart(ar_down_cast<Surface3D>(parts[i]),3);
            }
          }
        _mustBuildTextureTransform3=false;
        }
      if(_mustBuildTexture4)
        {
        if(_appearanceField&&_appearanceField->_texture4Field)
          {
          _appearanceField->_texture4Field->rebuild();
          for(unsigned int i=nbParts;i--;)
            {
            _appearanceField->_texture4Field
                            ->updatePart(ar_down_cast<Surface3D>(parts[i]),4);
            }
          }
        _mustBuildTexture4=false;
        }
      if(_mustBuildTextureTransform4)
        {
        if(_appearanceField&&_appearanceField->_textureTransform4Field)
          {
          _appearanceField->_textureTransform4Field->rebuild();
          for(unsigned int i=nbParts;i--;)
            {
            _appearanceField->_textureTransform4Field
                            ->updatePart(ar_down_cast<Surface3D>(parts[i]),4);
            }
          }
        _mustBuildTextureTransform4=false;
        }
      }
    if(_mustBuildGeometry)
      {
      _geometryField->rebuild();
      const MaterialNode * material=
        (_appearanceField&&_appearanceField->_materialField
         ? _appearanceField->_materialField : (const MaterialNode *)0);
      for(unsigned int i=nbParts;i--;)
        {
        _geometryField->updatePart(parts[i],material);
        }
      _mustBuildGeometry=false;
      }
    }
  _needsRebuild=false;
  }
_needsRebuildUnder=false;
}

void
ShapeNode::postTransformation(void)
{
if(!_mustTransform)
  {
  _shape->_postShapeTransformation(this);
  _mustTransform=true;
  }
}

void
ShapeNode::applyTransformations(void)
{
if(_mustTransform)
  {
  if(!_transform)
    {
    _transform=Transform3D::NEW();
    }
  Util3D::Transform & t=_transform->accessTransformation();
  for(unsigned int i=_partInfos.size();i--;)
    {
    ShapeNode::PartInfo & info=_partInfos[i];
    if(info.part)
      {
      t=_partInfos[i].initTransform;
      for(unsigned int j=info.transformPath.size();j--;)
        {
        t.preTransform(info.transformPath[j]->_transform);
        }
      info.part->writeTransformation(_transform);
      }
    }
  _mustTransform=false;
  }
}

//-------- Forbidden operations ----------------------------------------------

void
ShapeNode::_generateFields(ostream & output,
                           unsigned int indent,
                           bool showDefaultValues,
                           StlVector<const VrmlNode *> & defNodesInOut) const
{                                                                 // Protected
if(showDefaultValues||
   _appearanceField)
  {
  _generateSFNode(output,indent,"appearance",_appearanceField,
                  showDefaultValues,defNodesInOut);
  }
if(showDefaultValues||
   _geometryField)
  {
  _generateSFNode(output,indent,"geometry",_geometryField,
                  showDefaultValues,defNodesInOut);
  }
}

ShapeNode::ShapeNode(const StlString & nodeType,
                     VrmlShape3D * shape) // Protected
: GroupChildNode(nodeType,shape),
  _appearanceField((AppearanceNode *)0),
  _geometryField((GeometryNode *)0),
  _partInfos(),
  _transform(),
  _lightOff(),
  _mustBuildMaterial(true),
  _mustBuildTexture(true),
  _mustBuildTextureTransform(true),
  _mustBuildTexture2(true),
  _mustBuildTextureTransform2(true),
  _mustBuildTexture3(true),
  _mustBuildTextureTransform3(true),
  _mustBuildTexture4(true),
  _mustBuildTextureTransform4(true),
  _mustBuildGeometry(true),
  _mustTransform(false)
{
  // Nothing to be done
}

} // namespace AReVi

//----------------------------------------------------------------------------
