// LI2/ENIB - AReVi - VRML services
//----------------------------------------------------------------------------
// file        : AReVi/VRML/sphereNode_priv.cpp
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/VRML/sphereNode_priv.h"

#include "AReVi/VRML/vrmlValue_priv.h"
#include "AReVi/VRML/vrmlShape3D.h"

#include "AReVi/Shapes/sphere3D.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class SphereNode
//----------------------------------------------------------------------------

//-------- Construction / Destruction ----------------------------------------

SphereNode::~SphereNode(void)
{
  // Nothing to be done
}

//-------- Field management --------------------------------------------------

void
SphereNode::addField(const StlString & fieldName,
                     VrmlValue * fieldValue)
{
bool ok=false;
if(fieldName=="radius")
  {
  if(fieldValue->checkSFFloat()&&
     (fieldValue->getReal(0)>0.0))
    {
    _radiusField=fieldValue->getReal(0);
    ok=true;
    }
  }
else if(fieldName=="ccw")
  {
  if(fieldValue->checkSFBool())
    {
    _ccwField=fieldValue->getBool(0);
    ok=true;
    }
  }
else if(fieldName=="solid")
  {
  if(fieldValue->checkSFBool())
    {
    _solidField=fieldValue->getBool(0);
    ok=true;
    }
  }
else
  {
  GeometryNode::addField(fieldName,fieldValue);
  ok=true;
  }
if(!ok)
  {
  _shape->_badFieldMsg(fieldName,_type);
  }
}

void
SphereNode::check(void)
{
_fail=false;
if(!_fail)
  {
  rebuild();
  }
}

//-------- Rendering ---------------------------------------------------------

ArRef<ShapePart3D> // built part
SphereNode::buildPart(Util3D::Transform & initTransformOut) const
{
initTransformOut.identity();
initTransformOut.postYaw(-M_PI_2);
initTransformOut.postPitch(-M_PI_2);
return(Sphere3D::NEW());
}

void
SphereNode::updatePart(ArRef<ShapePart3D> part,
                       const MaterialNode * /* material */) const
{
ArRef<Sphere3D> sphere=ar_down_cast<Sphere3D>(part);
sphere->setRadius(_radiusField);
sphere->setCCW(_ccwField);
sphere->setDoubleSided(!_solidField);
}

//-------- Forbidden operations ----------------------------------------------

void
SphereNode::_generateFields(ostream & output,
                            unsigned int indent,
                            bool showDefaultValues,
                            StlVector<const VrmlNode *> & /* defNodesInOut */)
                                                            const // Protected
{
if(showDefaultValues||
   (_radiusField!=1.0f))
  {
  _generateSFFloat(output,indent,"radius",_radiusField);
  }
if(showDefaultValues||
   !_ccwField)
  {
  _generateSFBool(output,indent,"ccw",_ccwField);
  }
if(showDefaultValues||
   !_solidField)
  {
  _generateSFBool(output,indent,"solid",_solidField);
  }
}

SphereNode::SphereNode(const StlString & nodeType,
                       VrmlShape3D * shape) // Protected
: GeometryNode(nodeType,shape),
  _radiusField(1.0),
  _ccwField(true),
  _solidField(true)
{
  // Nothing to be done
}

} // namespace AReVi

//----------------------------------------------------------------------------
