// LI2/ENIB - AReVi - VRML services
//----------------------------------------------------------------------------
// file        : AReVi/VRML/textureCoordinateNode_priv.cpp
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/VRML/textureCoordinateNode_priv.h"

#include "AReVi/VRML/vrmlValue_priv.h"
#include "AReVi/VRML/vrmlShape3D.h"

#include "AReVi/VRML/geometryNode_priv.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class TextureCoordinateNode
//----------------------------------------------------------------------------

//-------- Construction / Destruction ----------------------------------------

TextureCoordinateNode::~TextureCoordinateNode(void)
{
  // Nothing to be done
}

//-------- Field management --------------------------------------------------

void
TextureCoordinateNode::addField(const StlString & fieldName,
                                VrmlValue * fieldValue)
{
bool ok=false;
if(fieldName=="point")
  {
  if(fieldValue->checkMFVec2f())
    {
    _pointField.clear();
    unsigned int nb=fieldValue->getSize();
    for(unsigned int i=0;i<nb;i+=2)
      {
      _pointField.push_back(Util3D::Dbl2(fieldValue->getReal(i),
                                         fieldValue->getReal(i+1)));
      }
    ok=true;
    }
  }
else
  {
  VrmlNode::addField(fieldName,fieldValue);
  ok=true;
  }
if(!ok)
  {
  _shape->_badFieldMsg(fieldName,_type);
  }
}

void
TextureCoordinateNode::check(void)
{
_fail=false;
}

//-------- Rendering ---------------------------------------------------------

void
TextureCoordinateNode::postRebuild(void)
{
for(unsigned int i=_parents.size();i--;)
  {
  static_cast<GeometryNode *>(_parents[i])->postRebuild();
  }
}

//-------- Forbidden operations ----------------------------------------------

void
TextureCoordinateNode::_generateFields(ostream & output,
                                       unsigned int indent,
                                       bool showDefaultValues,
                            StlVector<const VrmlNode *> & /* defNodesInOut */)
                                                            const // Protected
{
if(showDefaultValues||
   !_pointField.empty())
  {
  _generateMFVec2f(output,indent,"point",_pointField);
  }
}

TextureCoordinateNode::TextureCoordinateNode(const StlString & nodeType,
                                             VrmlShape3D * shape)
: VrmlNode(nodeType,shape),                                     // Protected
  _pointField()
{
  // Nothing to be done
}

} // namespace AReVi

//----------------------------------------------------------------------------
