// LI2/ENIB - AReVi - VRML services
//----------------------------------------------------------------------------
// file        : AReVi/VRML/textureNode_priv.cpp
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/VRML/textureNode_priv.h"

#include "AReVi/VRML/vrmlValue_priv.h"
#include "AReVi/VRML/vrmlShape3D.h"

#include "AReVi/VRML/appearanceNode_priv.h"

#include "AReVi/Shapes/surface3D.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class TextureNode
//----------------------------------------------------------------------------

//-------- Construction / Destruction ----------------------------------------

TextureNode::~TextureNode(void)
{
  // Nothing to be done
}

//-------- Inspection --------------------------------------------------------

bool // is a texture
TextureNode::isTexture(void) const
{
return(true);
}

//-------- Field management --------------------------------------------------

void
TextureNode::addField(const StlString & fieldName,
                      VrmlValue * fieldValue)
{
bool ok=false;
if(fieldName=="repeatS")
  {
  if(fieldValue->checkSFBool())
    {
    _repeatSField=fieldValue->getBool(0);
    ok=true;
    }
  }
else if(fieldName=="repeatT")
  {
  if(fieldValue->checkSFBool())
    {
    _repeatTField=fieldValue->getBool(0);
    ok=true;
    }
  }
else
  {
  VrmlNode::addField(fieldName,fieldValue);
  ok=true;
  }
if(!ok)
  {
  _shape->_badFieldMsg(fieldName,_type);
  }
}

//-------- Rendering ---------------------------------------------------------

void
TextureNode::postTextureRebuild(bool regenerate)
{
if(regenerate)
  {
  _needsRebuild=true;
  }
for(unsigned int i=_parents.size();i--;)
  {
  static_cast<AppearanceNode *>(_parents[i])->postTextureRebuild(this);
  }
}

// void
// TextureNode::rebuild(void)=0;

void
TextureNode::updatePart(ArRef<Surface3D> part,
                        unsigned int which) const
{
switch(which)
  {
  case 1:
    {
    part->setTextureRepeat(_repeatSField,_repeatTField);
    part->setTexture(_texture);
    break;
    }
  case 2:
    {
    part->setSecondTextureRepeat(_repeatSField,_repeatTField);
    part->setSecondTexture(_texture);
    break;
    }
  case 3:
    {
    part->setThirdTextureRepeat(_repeatSField,_repeatTField);
    part->setThirdTexture(_texture);
    break;
    }
  case 4:
    {
    part->setFourthTextureRepeat(_repeatSField,_repeatTField);
    part->setFourthTexture(_texture);
    break;
    }
  }
}

//-------- Forbidden operations ----------------------------------------------

void
TextureNode::_generateFields(ostream & output,
                             unsigned int indent,
                             bool showDefaultValues,
                             StlVector<const VrmlNode *> & /* defNodesInOut */)
                                                            const // Protected
{
if(showDefaultValues||
   !_repeatSField)
  {
  _generateSFBool(output,indent,"repeatS",_repeatSField);
  }
if(showDefaultValues||
   !_repeatTField)
  {
  _generateSFBool(output,indent,"repeatT",_repeatTField);
  }
}

TextureNode::TextureNode(const StlString & nodeType,
                         VrmlShape3D * shape) // Protected
: VrmlNode(nodeType,shape),
  _repeatSField(true),
  _repeatTField(true),
  _texture()
{
  // Nothing to be done
}

} // namespace AReVi

//----------------------------------------------------------------------------
