// LI2/ENIB - AReVi - VRML services
//----------------------------------------------------------------------------
// file        : AReVi/VRML/textureTransformNode_priv.cpp
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/VRML/textureTransformNode_priv.h"

#include "AReVi/VRML/vrmlValue_priv.h"
#include "AReVi/VRML/vrmlShape3D.h"

#include "AReVi/VRML/appearanceNode_priv.h"

#include "AReVi/Lib3D/transform2D.h"

#include "AReVi/Shapes/surface3D.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class TextureTransformNode
//----------------------------------------------------------------------------

//-------- Construction / Destruction ----------------------------------------

TextureTransformNode::~TextureTransformNode(void)
{
  // Nothing to be done
}

//-------- Field management --------------------------------------------------

void
TextureTransformNode::addField(const StlString & fieldName,
                               VrmlValue * fieldValue)
{
bool ok=false;
if(fieldName=="center")
  {
  if(fieldValue->checkSFVec2f())
    {
    _centerField.x=fieldValue->getReal(0);
    _centerField.y=fieldValue->getReal(1);
    ok=true;
    }
  }
else if(fieldName=="rotation")
  {
  if(fieldValue->checkSFFloat())
    {
    _rotationField=fieldValue->getReal(0);
    ok=true;
    }
  }
else if(fieldName=="scale")
  {
  if(fieldValue->checkSFVec2f())
    {
    _scaleField.x=fieldValue->getReal(0);
    _scaleField.y=fieldValue->getReal(1);
    ok=true;
    }
  }
else if(fieldName=="translation")
  {
  if(fieldValue->checkSFVec2f())
    {
    _translationField.x=fieldValue->getReal(0);
    _translationField.y=fieldValue->getReal(1);
    ok=true;
    }
  }
else
  {
  VrmlNode::addField(fieldName,fieldValue);
  ok=true;
  }
if(!ok)
  {
  _shape->_badFieldMsg(fieldName,_type);
  }
}

void
TextureTransformNode::check(void)
{
_fail=false;
}

//-------- Rendering ---------------------------------------------------------

void
TextureTransformNode::postRebuild(void)
{
for(unsigned int i=_parents.size();i--;)
  {
  static_cast<AppearanceNode *>(_parents[i])
                               ->postTextureTransformRebuild(this);
  }
}

void
TextureTransformNode::rebuild(void)
{
if(!_transform)
  {
  _transform=Transform2D::NEW();
  }
Util3D::Transform & t=_transform->accessTransformation();
t.identity();
t.postTranslate(-_centerField.x,-_centerField.y,0.0);
t.postScale(_scaleField.x,_scaleField.y,1.0);
t.postYaw(_rotationField);
t.postTranslate(_centerField.x+_translationField.x,
                _centerField.y+_translationField.y,0.0);
}

void
TextureTransformNode::updatePart(ArRef<Surface3D> part,
                                 unsigned int which) const
{
switch(which)
  {
  case 1:
    {
    part->writeTextureTransformation(_transform);
    break;
    }
  case 2:
    {
    part->writeSecondTextureTransformation(_transform);
    break;
    }
  case 3:
    {
    part->writeThirdTextureTransformation(_transform);
    break;
    }
  case 4:
    {
    part->writeFourthTextureTransformation(_transform);
    break;
    }
  }
}

//-------- Forbidden operations ----------------------------------------------

void
TextureTransformNode::_generateFields(ostream & output,
                                      unsigned int indent,
                                      bool showDefaultValues,
                            StlVector<const VrmlNode *> & /* defNodesInOut */)
                                                            const // Protected
{
if(showDefaultValues||
   _centerField.x||
   _centerField.y)
  {
  _generateSFVec2f(output,indent,"center",_centerField);
  }
if(showDefaultValues||
   _rotationField)
  {
  _generateSFFloat(output,indent,"rotation",_rotationField);
  }
if(showDefaultValues||
   (_scaleField.x!=1.0f)||
   (_scaleField.y!=1.0f))
  {
  _generateSFVec2f(output,indent,"scale",_scaleField);
  }
if(showDefaultValues||
   _translationField.x||
   _translationField.y)
  {
  _generateSFVec2f(output,indent,"translation",_translationField);
  }
}

TextureTransformNode::TextureTransformNode(const StlString & nodeType,
                                           VrmlShape3D * shape) // Protected
: VrmlNode(nodeType,shape),
  _centerField(0.0,0.0),
  _rotationField(0.0),
  _scaleField(1.0,1.0),
  _translationField(0.0,0.0),
  _transform()
{
  // Nothing to be done
}

} // namespace AReVi

//----------------------------------------------------------------------------
