// LI2/ENIB - AReVi - VRML services
//----------------------------------------------------------------------------
// file        : AReVi/VRML/transformNode_priv.cpp
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/VRML/transformNode_priv.h"

#include "AReVi/VRML/vrmlValue_priv.h"
#include "AReVi/VRML/vrmlShape3D.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class TransformNode
//----------------------------------------------------------------------------

//-------- Construction / Destruction ----------------------------------------

TransformNode::~TransformNode(void)
{
  // Nothing to be done
}

//-------- Field management --------------------------------------------------

void
TransformNode::addField(const StlString & fieldName,
                        VrmlValue * fieldValue)
{
bool ok=false;
if(fieldName=="center")
  {
  if(fieldValue->checkSFVec3f())
    {
    _centerField.x=fieldValue->getReal(0);
    _centerField.y=fieldValue->getReal(1);
    _centerField.z=fieldValue->getReal(2);
    ok=true;
    }
  }
else if(fieldName=="rotation")
  {
  if(fieldValue->checkSFRotation())
    {
    _rotationField.x=fieldValue->getReal(0);
    _rotationField.y=fieldValue->getReal(1);
    _rotationField.z=fieldValue->getReal(2);
    _rotationField.a=fieldValue->getReal(3);
    ok=true;
    }
  }
else if(fieldName=="scale")
  {
  if(fieldValue->checkSFVec3f()&&
     (fieldValue->getReal(0)>0.0)&&
     (fieldValue->getReal(1)>0.0)&&
     (fieldValue->getReal(2)>0.0))
    {
    _scaleField.x=fieldValue->getReal(0);
    _scaleField.y=fieldValue->getReal(1);
    _scaleField.z=fieldValue->getReal(2);
    ok=true;
    }
  }
else if(fieldName=="scaleOrientation")
  {
  if(fieldValue->checkSFRotation())
    {
    _scaleOrientationField.x=fieldValue->getReal(0);
    _scaleOrientationField.y=fieldValue->getReal(1);
    _scaleOrientationField.z=fieldValue->getReal(2);
    _scaleOrientationField.a=fieldValue->getReal(3);
    ok=true;
    }
  }
else if(fieldName=="translation")
  {
  if(fieldValue->checkSFVec3f())
    {
    _translationField.x=fieldValue->getReal(0);
    _translationField.y=fieldValue->getReal(1);
    _translationField.z=fieldValue->getReal(2);
    ok=true;
    }
  }
else
  {
  GroupNode::addField(fieldName,fieldValue);
  ok=true;
  }
if(!ok)
  {
  _shape->_badFieldMsg(fieldName,_type);
  }
}

void
TransformNode::check(void)
{
_fail=false;
}

//-------- Rendering ---------------------------------------------------------

void
TransformNode::buildParts(const StlVector<TransformNode *> & transformPath)
{
StlVector<TransformNode *> newPath(transformPath);
newPath.push_back(this);
for(unsigned int i=_childrenField.size();i--;)
  {
  _childrenField[i]->buildParts(newPath);
  }
}

void
TransformNode::rebuild(void)
{
if(_needsRebuild)
  {
  _transform.identity();
  _transform.postTranslate(_translationField.x+_centerField.x,
                           _translationField.y+_centerField.y,
                           _translationField.z+_centerField.z);
  _transform.postRotate(_rotationField.x,
                        _rotationField.y,
                        _rotationField.z,
                        _rotationField.a);
  _transform.postRotate(_scaleOrientationField.x,
                        _scaleOrientationField.y,
                        _scaleOrientationField.z,
                        _scaleOrientationField.a);
  _transform.postScale(_scaleField.x,
                       _scaleField.y,
                       _scaleField.z);
  _transform.postRotate(_scaleOrientationField.x,
                        _scaleOrientationField.y,
                        _scaleOrientationField.z,
                        -_scaleOrientationField.a);
  _transform.postTranslate(-_centerField.x,
                           -_centerField.y,
                           -_centerField.z);
  postTransformation();
  _needsRebuild=false;
  }
if(_needsRebuildUnder)
  {
  for(unsigned int i=_childrenField.size();i--;)
    {
    _childrenField[i]->rebuild();
    }
  _needsRebuildUnder=false;
  }
}

//-------- Forbidden operations ----------------------------------------------

void
TransformNode::_generateFields(ostream & output,
                               unsigned int indent,
                               bool showDefaultValues,
                               StlVector<const VrmlNode *> & defNodesInOut)
                                                            const // Protected
{
if(showDefaultValues||
   _centerField.x||
   _centerField.y||
   _centerField.z)
  {
  _generateSFVec3f(output,indent,"center",_centerField);
  }
if(showDefaultValues||
   _rotationField.x||
   _rotationField.y||
   (_rotationField.z!=1.0f)||
   _rotationField.a)
  {
  _generateSFRotation(output,indent,"rotation",_rotationField);
  }
if(showDefaultValues||
   (_scaleField.x!=1.0f)||
   (_scaleField.y!=1.0f)||
   (_scaleField.z!=1.0f))
  {
  _generateSFVec3f(output,indent,"scale",_scaleField);
  }
if(showDefaultValues||
   _scaleOrientationField.x||
   _scaleOrientationField.y||
   (_scaleOrientationField.z!=1.0f)||
   _scaleOrientationField.a)
  {
  _generateSFRotation(output,indent,"scaleOrientation",_scaleOrientationField);
  }
if(showDefaultValues||
   _translationField.x||
   _translationField.y||
   _translationField.z)
  {
  _generateSFVec3f(output,indent,"translation",_translationField);
  }
GroupNode::_generateFields(output,indent,showDefaultValues,defNodesInOut);
}

TransformNode::TransformNode(const StlString & nodeType,
                             VrmlShape3D * shape) // Protected
: GroupNode(nodeType,shape),
  _centerField(0.0,0.0,0.0),
  _rotationField(0.0,0.0,1.0,0.0),
  _scaleField(1.0,1.0,1.0),
  _scaleOrientationField(0.0,0.0,1.0,0.0),
  _translationField(0.0,0.0,0.0),
  _transform(true)
{
  // Nothing to be done
}

} // namespace AReVi

//----------------------------------------------------------------------------
