// LI2/ENIB - AReVi - VRML services
//----------------------------------------------------------------------------
// file        : AReVi/VRML/vrmlBoxInteractor.cpp
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/VRML/vrmlBoxInteractor.h"

#include "AReVi/VRML/boxNode_priv.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class VrmlBoxInteractor
//----------------------------------------------------------------------------

AR_CLASS_DEF(VrmlBoxInteractor,VrmlInteractor)

//-------- Construction / Destruction ----------------------------------------

VrmlBoxInteractor::VrmlBoxInteractor(ArCW & arCW) // Protected
: VrmlInteractor(arCW)
{
  // Nothing to be done
}

VrmlBoxInteractor::~VrmlBoxInteractor(void) // Protected
{
  // Nothing to be done
}

//-------- Connection --------------------------------------------------------

bool // success
VrmlBoxInteractor::connectToNode(ArRef<VrmlShape3D> shape,
                                 const StlString & nodeName,
                                 int occurence)
{
if(VrmlInteractor::connectToNode(shape,nodeName,occurence)&&
   (_node->getType()=="Box"))
  {
  return(true);
  }
else
  {
  disconnect();
  return(false);
  }
}

bool // success
VrmlBoxInteractor::connectTo(ArConstRef<VrmlInteractor> interactor)
{
if(VrmlInteractor::connectTo(interactor)&&
   (_node->getType()=="Box"))
  {
  return(true);
  }
else
  {
  disconnect();
  return(false);
  }
}

//-------- Parameters --------------------------------------------------------

bool // success
VrmlBoxInteractor::readSizeField(double & sxOut,
                                 double & syOut,
                                 double & szOut) const
{
const BoxNode * node=static_cast<const BoxNode *>(_node);
if(node)
  {
  sxOut=node->_sizeField.x;
  syOut=node->_sizeField.y;
  szOut=node->_sizeField.z;
  return(true);
  }
else
  {
  return(false);
  }
}

bool // success
VrmlBoxInteractor::writeSizeField(double sx,
                                  double sy,
                                  double sz)
{
BoxNode * node=static_cast<BoxNode *>(_node);
if(node)
  {
  if((sx>0.0)&&(sy>0.0)&&(sz>0.0))
    {
    node->_sizeField.x=sx;
    node->_sizeField.y=sy;
    node->_sizeField.z=sz;
    node->postRebuild();
    return(true);
    }
  else
    {
    return(false);
    }
  }
else
  {
  return(false);
  }
}

bool // success
VrmlBoxInteractor::readSolidField(bool & solidOut) const
{
const BoxNode * node=static_cast<const BoxNode *>(_node);
if(node)
  {
  solidOut=node->_solidField;
  return(true);
  }
else
  {
  return(false);
  }
}

bool // success
VrmlBoxInteractor::writeSolidField(bool solid)
{
BoxNode * node=static_cast<BoxNode *>(_node);
if(node)
  {
  node->_solidField=solid;
  node->postRebuild();
  return(true);
  }
else
  {
  return(false);
  }
}

} // namespace AReVi

//----------------------------------------------------------------------------
