// LI2/ENIB - AReVi - VRML services
//----------------------------------------------------------------------------
// file        : AReVi/VRML/vrmlColorInteractor.cpp
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/VRML/vrmlColorInteractor.h"

#include "AReVi/VRML/colorNode_priv.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class VrmlColorInteractor
//----------------------------------------------------------------------------

AR_CLASS_DEF(VrmlColorInteractor,VrmlInteractor)

//-------- Construction / Destruction ----------------------------------------

VrmlColorInteractor::VrmlColorInteractor(ArCW & arCW) // Protected
: VrmlInteractor(arCW)
{
  // Nothing to be done
}

VrmlColorInteractor::~VrmlColorInteractor(void) // Protected
{
  // Nothing to be done
}

//-------- Connection --------------------------------------------------------

bool // success
VrmlColorInteractor::connectToNode(ArRef<VrmlShape3D> shape,
                                   const StlString & nodeName,
                                   int occurence)
{
if(VrmlInteractor::connectToNode(shape,nodeName,occurence)&&
   (_node->getType()=="Color"))
  {
  return(true);
  }
else
  {
  disconnect();
  return(false);
  }
}

bool // success
VrmlColorInteractor::connectTo(ArConstRef<VrmlInteractor> interactor)
{
if(VrmlInteractor::connectTo(interactor)&&
   (_node->getType()=="Color"))
  {
  return(true);
  }
else
  {
  disconnect();
  return(false);
  }
}

//-------- Parameters --------------------------------------------------------

bool // success
VrmlColorInteractor::readColorFieldSize(unsigned int & sizeOut) const
{
const ColorNode * node=static_cast<const ColorNode *>(_node);
if(node)
  {
  sizeOut=node->_colorField.size();
  return(true);
  }
else
  {
  return(false);
  }
}

bool // success
VrmlColorInteractor::readColorField(unsigned int index,
                                    double & rOut,
                                    double & gOut,
                                    double & bOut) const
{
const ColorNode * node=static_cast<const ColorNode *>(_node);
if(node)
  {
  if(index<node->_colorField.size())
    {
    rOut=node->_colorField[index].x;
    gOut=node->_colorField[index].y;
    bOut=node->_colorField[index].z;
    return(true);
    }
  else
    {
    return(false);
    }
  }
else
  {
  return(false);
  }
}

bool // success
VrmlColorInteractor::writeColorField(unsigned int index,
                                     double r,
                                     double g,
                                     double b)
{
ColorNode * node=static_cast<ColorNode *>(_node);
if(node)
  {
  if(index<node->_colorField.size()&&
     (r>=0.0)&&(r<=1.0)&&
     (g>=0.0)&&(g<=1.0)&&
     (b>=0.0)&&(b<=1.0))
    {
    node->_colorField[index].x=r;
    node->_colorField[index].y=g;
    node->_colorField[index].z=b;
    node->postRebuild();
    return(true);
    }
  else
    {
    return(false);
    }
  }
else
  {
  return(false);
  }
}

bool // success
VrmlColorInteractor::interpolate(ArRef<VrmlColorInteractor> previous,
                                 ArRef<VrmlColorInteractor> next,
                                 double nextPreviousRatio)
{
ColorNode * node=static_cast<ColorNode *>(_node);
const ColorNode * previousNode=static_cast<const ColorNode *>(previous->_node);
const ColorNode * nextNode=static_cast<const ColorNode *>(next->_node);
if(node&&previousNode&&nextNode)
  {
  unsigned int nb=node->_colorField.size();
  if((nb<=previousNode->_colorField.size())&&
     (nb<=nextNode->_colorField.size()))
    {
    if(nextPreviousRatio<0.0)
      {
      nextPreviousRatio=0.0;
      }
    else if(nextPreviousRatio>1.0)
      {
      nextPreviousRatio=1.0;
      }
    double previousNextRatio=1.0-nextPreviousRatio;
    while(nb--)
      {
      node->_colorField[nb].x=
            previousNode->_colorField[nb].x*previousNextRatio+
            nextNode->_colorField[nb].x*nextPreviousRatio;
      node->_colorField[nb].y=
            previousNode->_colorField[nb].y*previousNextRatio+
            nextNode->_colorField[nb].y*nextPreviousRatio;
      node->_colorField[nb].z=
            previousNode->_colorField[nb].z*previousNextRatio+
            nextNode->_colorField[nb].z*nextPreviousRatio;
      }
    node->postRebuild();
    return(true);
    }
  else
    {
    return(false);
    }
  }
else
  {
  return(false);
  }
}

} // namespace AReVi

//----------------------------------------------------------------------------
