// LI2/ENIB - AReVi - VRML services
//----------------------------------------------------------------------------
// file        : AReVi/VRML/vrmlConeInteractor.cpp
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/VRML/vrmlConeInteractor.h"

#include "AReVi/VRML/coneNode_priv.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class VrmlConeInteractor
//----------------------------------------------------------------------------

AR_CLASS_DEF(VrmlConeInteractor,VrmlInteractor)

//-------- Construction / Destruction ----------------------------------------

VrmlConeInteractor::VrmlConeInteractor(ArCW & arCW) // Protected
: VrmlInteractor(arCW)
{
  // Nothing to be done
}

VrmlConeInteractor::~VrmlConeInteractor(void) // Protected
{
  // Nothing to be done
}

//-------- Connection --------------------------------------------------------

bool // success
VrmlConeInteractor::connectToNode(ArRef<VrmlShape3D> shape,
                                  const StlString & nodeName,
                                  int occurence)
{
if(VrmlInteractor::connectToNode(shape,nodeName,occurence)&&
   (_node->getType()=="Cone"))
  {
  return(true);
  }
else
  {
  disconnect();
  return(false);
  }
}

bool // success
VrmlConeInteractor::connectTo(ArConstRef<VrmlInteractor> interactor)
{
if(VrmlInteractor::connectTo(interactor)&&
   (_node->getType()=="Cone"))
  {
  return(true);
  }
else
  {
  disconnect();
  return(false);
  }
}

//-------- Parameters --------------------------------------------------------

bool // success
VrmlConeInteractor::readBottomRadiusField(double & bottomRadiusOut) const
{
const ConeNode * node=static_cast<const ConeNode *>(_node);
if(node)
  {
  bottomRadiusOut=node->_bottomRadiusField;
  return(true);
  }
else
  {
  return(false);
  }
}

bool // success
VrmlConeInteractor::writeBottomRadiusField(double bottomRadius)
{
ConeNode * node=static_cast<ConeNode *>(_node);
if(node)
  {
  if(bottomRadius>0.0)
    {
    node->_bottomRadiusField=bottomRadius;
    node->postRebuild();
    return(true);
    }
  else
    {
    return(false);
    }
  }
else
  {
  return(false);
  }
}

bool // success
VrmlConeInteractor::readHeightField(double & heightOut) const
{
const ConeNode * node=static_cast<const ConeNode *>(_node);
if(node)
  {
  heightOut=node->_heightField;
  return(true);
  }
else
  {
  return(false);
  }
}

bool // success
VrmlConeInteractor::writeHeightField(double height)
{
ConeNode * node=static_cast<ConeNode *>(_node);
if(node)
  {
  if(height>0.0)
    {
    node->_heightField=height;
    node->postRebuild();
    return(true);
    }
  else
    {
    return(false);
    }
  }
else
  {
  return(false);
  }
}

bool // success
VrmlConeInteractor::readSideField(bool & sideOut) const
{
const ConeNode * node=static_cast<const ConeNode *>(_node);
if(node)
  {
  sideOut=node->_sideField;
  return(true);
  }
else
  {
  return(false);
  }
}

bool // success
VrmlConeInteractor::writeSideField(bool side)
{
ConeNode * node=static_cast<ConeNode *>(_node);
if(node)
  {
  node->_sideField=side;
  node->postRebuild();
  return(true);
  }
else
  {
  return(false);
  }
}

bool // success
VrmlConeInteractor::readBottomField(bool & bottomOut) const
{
const ConeNode * node=static_cast<const ConeNode *>(_node);
if(node)
  {
  bottomOut=node->_bottomField;
  return(true);
  }
else
  {
  return(false);
  }
}

bool // success
VrmlConeInteractor::writeBottomField(bool bottom)
{
ConeNode * node=static_cast<ConeNode *>(_node);
if(node)
  {
  node->_bottomField=bottom;
  node->postRebuild();
  return(true);
  }
else
  {
  return(false);
  }
}

bool // success
VrmlConeInteractor::readSolidField(bool & solidOut) const
{
const ConeNode * node=static_cast<const ConeNode *>(_node);
if(node)
  {
  solidOut=node->_solidField;
  return(true);
  }
else
  {
  return(false);
  }
}

bool // success
VrmlConeInteractor::writeSolidField(bool solid)
{
ConeNode * node=static_cast<ConeNode *>(_node);
if(node)
  {
  node->_solidField=solid;
  node->postRebuild();
  return(true);
  }
else
  {
  return(false);
  }
}

} // namespace AReVi

//----------------------------------------------------------------------------
