// LI2/ENIB - AReVi - VRML services
//----------------------------------------------------------------------------
// file        : AReVi/VRML/vrmlCylinderInteractor.cpp
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/VRML/vrmlCylinderInteractor.h"

#include "AReVi/VRML/cylinderNode_priv.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class VrmlCylinderInteractor
//----------------------------------------------------------------------------

AR_CLASS_DEF(VrmlCylinderInteractor,VrmlInteractor)

//-------- Construction / Destruction ----------------------------------------

VrmlCylinderInteractor::VrmlCylinderInteractor(ArCW & arCW) // Protected
: VrmlInteractor(arCW)
{
  // Nothing to be done
}

VrmlCylinderInteractor::~VrmlCylinderInteractor(void) // Protected
{
  // Nothing to be done
}

//-------- Connection --------------------------------------------------------

bool // success
VrmlCylinderInteractor::connectToNode(ArRef<VrmlShape3D> shape,
                                      const StlString & nodeName,
                                      int occurence)
{
if(VrmlInteractor::connectToNode(shape,nodeName,occurence)&&
   (_node->getType()=="Cylinder"))
  {
  return(true);
  }
else
  {
  disconnect();
  return(false);
  }
}

bool // success
VrmlCylinderInteractor::connectTo(ArConstRef<VrmlInteractor> interactor)
{
if(VrmlInteractor::connectTo(interactor)&&
   (_node->getType()=="Cylinder"))
  {
  return(true);
  }
else
  {
  disconnect();
  return(false);
  }
}

//-------- Parameters --------------------------------------------------------

bool // success
VrmlCylinderInteractor::readBottomField(bool & bottomOut) const
{
const CylinderNode * node=static_cast<const CylinderNode *>(_node);
if(node)
  {
  bottomOut=node->_bottomField;
  return(true);
  }
else
  {
  return(false);
  }
}

bool // success
VrmlCylinderInteractor::writeBottomField(bool bottom)
{
CylinderNode * node=static_cast<CylinderNode *>(_node);
if(node)
  {
  node->_bottomField=bottom;
  node->postRebuild();
  return(true);
  }
else
  {
  return(false);
  }
}

bool // success
VrmlCylinderInteractor::readHeightField(double & heightOut) const
{
const CylinderNode * node=static_cast<const CylinderNode *>(_node);
if(node)
  {
  heightOut=node->_heightField;
  return(true);
  }
else
  {
  return(false);
  }
}

bool // success
VrmlCylinderInteractor::writeHeightField(double height)
{
CylinderNode * node=static_cast<CylinderNode *>(_node);
if(node)
  {
  if(height>0.0)
    {
    node->_heightField=height;
    node->postRebuild();
    return(true);
    }
  else
    {
    return(false);
    }
  }
else
  {
  return(false);
  }
}

bool // success
VrmlCylinderInteractor::readRadiusField(double & radiusOut) const
{
const CylinderNode * node=static_cast<const CylinderNode *>(_node);
if(node)
  {
  radiusOut=node->_radiusField;
  return(true);
  }
else
  {
  return(false);
  }
}

bool // success
VrmlCylinderInteractor::writeRadiusField(double radius)
{
CylinderNode * node=static_cast<CylinderNode *>(_node);
if(node)
  {
  if(radius>0.0)
    {
    node->_radiusField=radius;
    node->postRebuild();
    return(true);
    }
  else
    {
    return(false);
    }
  }
else
  {
  return(false);
  }
}

bool // success
VrmlCylinderInteractor::readSideField(bool & sideOut) const
{
const CylinderNode * node=static_cast<const CylinderNode *>(_node);
if(node)
  {
  sideOut=node->_sideField;
  return(true);
  }
else
  {
  return(false);
  }
}

bool // success
VrmlCylinderInteractor::writeSideField(bool side)
{
CylinderNode * node=static_cast<CylinderNode *>(_node);
if(node)
  {
  node->_sideField=side;
  node->postRebuild();
  return(true);
  }
else
  {
  return(false);
  }
}

bool // success
VrmlCylinderInteractor::readTopField(bool & topOut) const
{
const CylinderNode * node=static_cast<const CylinderNode *>(_node);
if(node)
  {
  topOut=node->_topField;
  return(true);
  }
else
  {
  return(false);
  }
}

bool // success
VrmlCylinderInteractor::writeTopField(bool top)
{
CylinderNode * node=static_cast<CylinderNode *>(_node);
if(node)
  {
  node->_topField=top;
  node->postRebuild();
  return(true);
  }
else
  {
  return(false);
  }
}

bool // success
VrmlCylinderInteractor::readSolidField(bool & solidOut) const
{
const CylinderNode * node=static_cast<const CylinderNode *>(_node);
if(node)
  {
  solidOut=node->_solidField;
  return(true);
  }
else
  {
  return(false);
  }
}

bool // success
VrmlCylinderInteractor::writeSolidField(bool solid)
{
CylinderNode * node=static_cast<CylinderNode *>(_node);
if(node)
  {
  node->_solidField=solid;
  node->postRebuild();
  return(true);
  }
else
  {
  return(false);
  }
}

} // namespace AReVi

//----------------------------------------------------------------------------
