// LI2/ENIB - AReVi - VRML services
//----------------------------------------------------------------------------
// file        : AReVi/VRML/vrmlGenericInteractor.cpp
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/VRML/vrmlGenericInteractor.h"

#include "AReVi/VRML/vrmlShape3D.h"
#include "AReVi/VRML/groupNode_priv.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class VrmlGenericInteractor
//----------------------------------------------------------------------------

AR_CLASS_DEF(VrmlGenericInteractor,VrmlInteractor)

//-------- Construction / Destruction ----------------------------------------

VrmlGenericInteractor::VrmlGenericInteractor(ArCW & arCW) // Protected
: VrmlInteractor(arCW)
{
  // Nothing to be done
}

VrmlGenericInteractor::~VrmlGenericInteractor(void) // Protected
{
  // Nothing to be done
}

//-------- Connection --------------------------------------------------------

bool // a shape/node is connected
VrmlGenericInteractor::isConnected(void) const
{
return(_shape ? true : false);
}

bool // success
VrmlGenericInteractor::connectTo(ArConstRef<VrmlInteractor> interactor)
{
disconnect();
_shape=interactor->_shape;
if(_shape)
  {
  _node=interactor->_node;
  _occurence=interactor->_occurence;
  _shape->_addInteractor(this);
  return(true);
  }
else
  {
  return(false);
  }
}

bool // success
VrmlGenericInteractor::connectToShape(ArRef<VrmlShape3D> shape)
{
disconnect();
_shape=shape;
if(_shape)
  {
  _shape->_addInteractor(this);
  return(true);
  }
else
  {
  return(false);
  }
}

unsigned int // nb nodes in ``children'' field
VrmlGenericInteractor::getNbChildren() const
{
if(_node)
  {
  if(_node->isGroup())
    {
    const GroupNode * node=static_cast<const GroupNode *>(_node);
    return(node->getNbChildren());
    }
  else
    {
    return(0);
    }
  }
else if(_shape)
  {
  return(_shape->_getNbRootNodes());
  }
else
  {
  return(0);
  }
}

bool // success
VrmlGenericInteractor::moveToChild(unsigned int childIndex)
{
if(_node)
  {
  if(_node->isGroup())
    {
    GroupNode * node=static_cast<GroupNode *>(_node);
    if(childIndex<node->getNbChildren())
      {
      _node=node->accessChild(childIndex);
      return(true);
      }
    else
      {
      return(false);
      }
    }
  else
    {
    return(false);
    }
  }
else if(_shape)
  {
  if(childIndex<_shape->_getNbRootNodes())
    {
    _node=_shape->_accessRootNode(childIndex);
    return(true);
    }
  else
    {
    return(false);
    }
  }
else
  {
  return(false);
  }
}

void
VrmlGenericInteractor::getChildFieldNames(
                       StlVector<StlString> & childFieldNamesOut) const
{
childFieldNamesOut.clear();
if(_node)
  {
  _node->getNodeFieldNames(childFieldNamesOut);
  }
}

bool // success
VrmlGenericInteractor::moveToField(const StlString & fieldName)
{
if(_node)
  {
  VrmlNode * node=_node->accessNodeField(fieldName);
  if(node)
    {
    _node=node;
    return(true);
    }
  else
    {
    return(false);
    }
  }
else
  {
  return(false);
  }
}

} // namespace AReVi

//----------------------------------------------------------------------------
