// LI2/ENIB - AReVi - VRML services
//----------------------------------------------------------------------------
// file        : AReVi/VRML/vrmlImageTextureInteractor.cpp
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/VRML/vrmlImageTextureInteractor.h"

#include "AReVi/VRML/imageTextureNode_priv.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class VrmlImageTextureInteractor
//----------------------------------------------------------------------------

AR_CLASS_DEF(VrmlImageTextureInteractor,VrmlInteractor)

//-------- Construction / Destruction ----------------------------------------

VrmlImageTextureInteractor::VrmlImageTextureInteractor(ArCW & arCW)
: VrmlInteractor(arCW)                                            // Protected
{
  // Nothing to be done
}

VrmlImageTextureInteractor::~VrmlImageTextureInteractor(void) // Protected
{
  // Nothing to be done
}

//-------- Connection --------------------------------------------------------

bool // success
VrmlImageTextureInteractor::connectToNode(ArRef<VrmlShape3D> shape,
                                          const StlString & nodeName,
                                          int occurence)
{
if(VrmlInteractor::connectToNode(shape,nodeName,occurence)&&
   (_node->getType()=="ImageTexture"))
  {
  return(true);
  }
else
  {
  disconnect();
  return(false);
  }
}

bool // success
VrmlImageTextureInteractor::connectTo(ArConstRef<VrmlInteractor> interactor)
{
if(VrmlInteractor::connectTo(interactor)&&
   (_node->getType()=="ImageTexture"))
  {
  return(true);
  }
else
  {
  disconnect();
  return(false);
  }
}

//-------- Parameters --------------------------------------------------------

bool // success
VrmlImageTextureInteractor::readUrlField(StlVector<StlString> & urlOut) const
{
const ImageTextureNode * node=static_cast<const ImageTextureNode *>(_node);
if(node)
  {
  urlOut=node->_urlField;
  return(true);
  }
else
  {
  return(false);
  }
}

bool // success
VrmlImageTextureInteractor::writeUrlField(const StlVector<StlString> & url)
{
ImageTextureNode * node=static_cast<ImageTextureNode *>(_node);
if(node)
  {
  node->_urlField=url;
  node->postTextureRebuild(true);
  return(true);
  }
else
  {
  return(false);
  }
}

bool // success
VrmlImageTextureInteractor::readRepeatSField(bool & repeatSOut) const
{
const ImageTextureNode * node=static_cast<const ImageTextureNode *>(_node);
if(node)
  {
  repeatSOut=node->_repeatSField;
  return(true);
  }
else
  {
  return(false);
  }
}

bool // success
VrmlImageTextureInteractor::writeRepeatSField(bool repeatS)
{
ImageTextureNode * node=static_cast<ImageTextureNode *>(_node);
if(node)
  {
  node->_repeatSField=repeatS;
  node->postTextureRebuild(false);
  return(true);
  }
else
  {
  return(false);
  }
}

bool // success
VrmlImageTextureInteractor::readRepeatTField(bool & repeatTOut) const
{
const ImageTextureNode * node=static_cast<const ImageTextureNode *>(_node);
if(node)
  {
  repeatTOut=node->_repeatTField;
  return(true);
  }
else
  {
  return(false);
  }
}

bool // success
VrmlImageTextureInteractor::writeRepeatTField(bool repeatT)
{
ImageTextureNode * node=static_cast<ImageTextureNode *>(_node);
if(node)
  {
  node->_repeatTField=repeatT;
  node->postTextureRebuild(false);
  return(true);
  }
else
  {
  return(false);
  }
}

} // namespace AReVi

//----------------------------------------------------------------------------
