// LI2/ENIB - AReVi - VRML services
//----------------------------------------------------------------------------
// file        : AReVi/VRML/vrmlIndexedFaceSetInteractor.cpp
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/VRML/vrmlIndexedFaceSetInteractor.h"

#include "AReVi/VRML/indexedFaceSetNode_priv.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class VrmlIndexedFaceSetInteractor
//----------------------------------------------------------------------------

AR_CLASS_DEF(VrmlIndexedFaceSetInteractor,VrmlInteractor)

//-------- Construction / Destruction ----------------------------------------

VrmlIndexedFaceSetInteractor::VrmlIndexedFaceSetInteractor(ArCW & arCW)
: VrmlInteractor(arCW)                                            // Protected
{
  // Nothing to be done
}

VrmlIndexedFaceSetInteractor::~VrmlIndexedFaceSetInteractor(void) // Protected
{
  // Nothing to be done
}

//-------- Connection --------------------------------------------------------

bool // success
VrmlIndexedFaceSetInteractor::connectToNode(ArRef<VrmlShape3D> shape,
                                            const StlString & nodeName,
                                            int occurence)
{
if(VrmlInteractor::connectToNode(shape,nodeName,occurence)&&
   (_node->getType()=="IndexedFaceSet"))
  {
  return(true);
  }
else
  {
  disconnect();
  return(false);
  }
}

bool // success
VrmlIndexedFaceSetInteractor::connectTo(ArConstRef<VrmlInteractor> interactor)
{
if(VrmlInteractor::connectTo(interactor)&&
   (_node->getType()=="IndexedFaceSet"))
  {
  return(true);
  }
else
  {
  disconnect();
  return(false);
  }
}

//-------- Parameters --------------------------------------------------------

bool // success
VrmlIndexedFaceSetInteractor::readCcwField(bool & ccwOut) const
{
const IndexedFaceSetNode * node=static_cast<const IndexedFaceSetNode *>(_node);
if(node)
  {
  ccwOut=node->_ccwField;
  return(true);
  }
else
  {
  return(false);
  }
}

bool // success
VrmlIndexedFaceSetInteractor::writeCcwField(bool ccw)
{
IndexedFaceSetNode * node=static_cast<IndexedFaceSetNode *>(_node);
if(node)
  {
  node->_ccwField=ccw;
  node->postRebuild();
  return(true);
  }
else
  {
  return(false);
  }
}

bool // success
VrmlIndexedFaceSetInteractor::readSolidField(bool & solidOut) const
{
const IndexedFaceSetNode * node=static_cast<const IndexedFaceSetNode *>(_node);
if(node)
  {
  solidOut=node->_solidField;
  return(true);
  }
else
  {
  return(false);
  }
}

bool // success
VrmlIndexedFaceSetInteractor::writeSolidField(bool solid)
{
IndexedFaceSetNode * node=static_cast<IndexedFaceSetNode *>(_node);
if(node)
  {
  node->_solidField=solid;
  node->postRebuild();
  return(true);
  }
else
  {
  return(false);
  }
}

} // namespace AReVi

//----------------------------------------------------------------------------
