// LI2/ENIB - AReVi - VRML services
//----------------------------------------------------------------------------
// file        : AReVi/VRML/vrmlInteractor.cpp
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/VRML/vrmlInteractor.h"

#include "AReVi/VRML/vrmlShape3D.h"
#include "AReVi/VRML/vrmlNode_priv.h"

#include "AReVi/Utils/memoryStream.h"
#include "AReVi/Utils/memoryBlock.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class VrmlInteractor
//----------------------------------------------------------------------------

AR_CLASS_DEF(VrmlInteractor,ArObject)

//-------- Construction / Destruction ----------------------------------------

VrmlInteractor::VrmlInteractor(ArCW & arCW) // Protected
: ArObject(arCW),
  _shape(),
  _occurence(0),
  _node((VrmlNode *)0),
  _dummyStr()
{
  // Nothing to be done
}

VrmlInteractor::~VrmlInteractor(void) // Protected
{
disconnect();
}

//-------- Inspection --------------------------------------------------------

ArConstRef<VrmlShape3D> // connected shape
VrmlInteractor::getShape(void) const
{
return(_shape);
}

ArRef<VrmlShape3D> // connected shape
VrmlInteractor::accessShape(void)
{
return(_shape);
}

const StlString & // name of connected node
VrmlInteractor::getNodeName(void) const
{
return(_node ? _node->getName() : _dummyStr);
}

int // occurence of named node
VrmlInteractor::getNodeOccurence(void) const
{
return(_occurence);
}

const StlString & // type of connected node
VrmlInteractor::getNodeType(void) const
{
return(_node ? _node->getType() : _dummyStr);
}

//-------- Connection --------------------------------------------------------

bool // a shape/node is connected
VrmlInteractor::isConnected(void) const
{
return(_node ? true : false);
}

bool // success
VrmlInteractor::connectToNode(ArRef<VrmlShape3D> shape,
                              const StlString & nodeName,
                              int occurence)
{
disconnect();
_node=shape->_accessNamedNode(nodeName,occurence);
if(_node)
  {
  _shape=shape;
  _occurence=occurence;
  _shape->_addInteractor(this);
  return(true);
  }
else
  {
  return(false);
  }
}

bool // success
VrmlInteractor::connectTo(ArConstRef<VrmlInteractor> interactor)
{
disconnect();
_node=interactor->_node;
if(_node)
  {
  _shape=interactor->_shape;
  _occurence=interactor->_occurence;
  _shape->_addInteractor(this);
  return(true);
  }
else
  {
  return(false);
  }
}

void
VrmlInteractor::disconnect(void)
{
if(_shape)
  {
  _shape->_removeInteractor(this);
  }
}

//-------- Code generation ---------------------------------------------------

void
VrmlInteractor::generateString(StlString & vrmlStringOut,
                               unsigned int indent,
                               bool showDefaultValues) const
{
ArRef<MemoryBlock> memory=MemoryBlock::NEW();
ArRef<MemoryOStream> stream=MemoryOStream::NEW(memory,0);
generateStream(stream,indent,showDefaultValues);
vrmlStringOut=StlString(memory->getMemory(),stream->getMemoryUsage());
}

void
VrmlInteractor::generateStream(ArRef<AbstractOStream> output,
                               unsigned int indent,
                               bool showDefaultValues) const
{
if(_node)
  {
  StlVector<const VrmlNode *> defNodes;
  _node->generateCode(output->stream(),indent,
                      showDefaultValues,defNodes);
  }
}

//-------- Forbidden operations ----------------------------------------------

void
VrmlInteractor::_shapeRemoved(void) // Protected
{
_shape.clear();
_occurence=0;
_node=(VrmlNode *)0;
}

} // namespace AReVi

//----------------------------------------------------------------------------
