// LI2/ENIB - AReVi - VRML services
//----------------------------------------------------------------------------
// file        : AReVi/VRML/vrmlMaterialInteractor.cpp
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/VRML/vrmlMaterialInteractor.h"

#include "AReVi/VRML/materialNode_priv.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class VrmlMaterialInteractor
//----------------------------------------------------------------------------

AR_CLASS_DEF(VrmlMaterialInteractor,VrmlInteractor)

//-------- Construction / Destruction ----------------------------------------

VrmlMaterialInteractor::VrmlMaterialInteractor(ArCW & arCW) // Protected
: VrmlInteractor(arCW)
{
  // Nothing to be done
}

VrmlMaterialInteractor::~VrmlMaterialInteractor(void) // Protected
{
  // Nothing to be done
}

//-------- Connection --------------------------------------------------------

bool // success
VrmlMaterialInteractor::connectToNode(ArRef<VrmlShape3D> shape,
                                      const StlString & nodeName,
                                      int occurence)
{
if(VrmlInteractor::connectToNode(shape,nodeName,occurence)&&
   (_node->getType()=="Material"))
  {
  return(true);
  }
else
  {
  disconnect();
  return(false);
  }
}

bool // success
VrmlMaterialInteractor::connectTo(ArConstRef<VrmlInteractor> interactor)
{
if(VrmlInteractor::connectTo(interactor)&&
   (_node->getType()=="Material"))
  {
  return(true);
  }
else
  {
  disconnect();
  return(false);
  }
}

//-------- Parameters --------------------------------------------------------

bool // success
VrmlMaterialInteractor::readAmbientIntensityField(double & ambientIntensityOut)
                                                                         const
{
const MaterialNode * node=static_cast<const MaterialNode *>(_node);
if(node)
  {
  ambientIntensityOut=node->_ambientIntensityField;
  return(true);
  }
else
  {
  return(false);
  }
}

bool // success
VrmlMaterialInteractor::writeAmbientIntensityField(double ambientIntensity)
{
MaterialNode * node=static_cast<MaterialNode *>(_node);
if(node)
  {
  if((ambientIntensity>=0.0)&&(ambientIntensity<=1.0))
    {
    node->_ambientIntensityField=ambientIntensity;
    node->postRebuild();
    return(true);
    }
  else
    {
    return(false);
    }
  }
else
  {
  return(false);
  }
}

bool // success
VrmlMaterialInteractor::readDiffuseColorField(double & rOut,
                                              double & gOut,
                                              double & bOut) const
{
const MaterialNode * node=static_cast<const MaterialNode *>(_node);
if(node)
  {
  rOut=node->_diffuseColorField.x;
  gOut=node->_diffuseColorField.y;
  bOut=node->_diffuseColorField.z;
  return(true);
  }
else
  {
  return(false);
  }
}

bool // success
VrmlMaterialInteractor::writeDiffuseColorField(double r,
                                               double g,
                                               double b)
{
MaterialNode * node=static_cast<MaterialNode *>(_node);
if(node)
  {
  if((r>=0.0)&&(r<=1.0)&&(g>=0.0)&&(g<=1.0)&&(b>=0.0)&&(b<=1.0))
    {
    node->_diffuseColorField.x=r;
    node->_diffuseColorField.y=g;
    node->_diffuseColorField.z=b;
    node->postRebuild();
    return(true);
    }
  else
    {
    return(false);
    }
  }
else
  {
  return(false);
  }
}

bool // success
VrmlMaterialInteractor::readEmissiveColorField(double & rOut,
                                               double & gOut,
                                               double & bOut) const
{
const MaterialNode * node=static_cast<const MaterialNode *>(_node);
if(node)
  {
  rOut=node->_emissiveColorField.x;
  gOut=node->_emissiveColorField.y;
  bOut=node->_emissiveColorField.z;
  return(true);
  }
else
  {
  return(false);
  }
}

bool // success
VrmlMaterialInteractor::writeEmissiveColorField(double r,
                                                double g,
                                                double b)
{
MaterialNode * node=static_cast<MaterialNode *>(_node);
if(node)
  {
  if((r>=0.0)&&(r<=1.0)&&(g>=0.0)&&(g<=1.0)&&(b>=0.0)&&(b<=1.0))
    {
    node->_emissiveColorField.x=r;
    node->_emissiveColorField.y=g;
    node->_emissiveColorField.z=b;
    node->postRebuild();
    return(true);
    }
  else
    {
    return(false);
    }
  }
else
  {
  return(false);
  }
}

bool // success
VrmlMaterialInteractor::readShininessField(double & shininessOut) const
{
const MaterialNode * node=static_cast<const MaterialNode *>(_node);
if(node)
  {
  shininessOut=node->_shininessField;
  return(true);
  }
else
  {
  return(false);
  }
}

bool // success
VrmlMaterialInteractor::writeShininessField(double shininess)
{
MaterialNode * node=static_cast<MaterialNode *>(_node);
if(node)
  {
  if((shininess>=0.0)&&(shininess<=1.0))
    {
    node->_shininessField=shininess;
    node->postRebuild();
    return(true);
    }
  else
    {
    return(false);
    }
  }
else
  {
  return(false);
  }
}

bool // success
VrmlMaterialInteractor::readSpecularColorField(double & rOut,
                                               double & gOut,
                                               double & bOut) const
{
const MaterialNode * node=static_cast<const MaterialNode *>(_node);
if(node)
  {
  rOut=node->_specularColorField.x;
  gOut=node->_specularColorField.y;
  bOut=node->_specularColorField.z;
  return(true);
  }
else
  {
  return(false);
  }
}

bool // success
VrmlMaterialInteractor::writeSpecularColorField(double r,
                                                double g,
                                                double b)
{
MaterialNode * node=static_cast<MaterialNode *>(_node);
if(node)
  {
  if((r>=0.0)&&(r<=1.0)&&(g>=0.0)&&(g<=1.0)&&(b>=0.0)&&(b<=1.0))
    {
    node->_specularColorField.x=r;
    node->_specularColorField.y=g;
    node->_specularColorField.z=b;
    node->postRebuild();
    return(true);
    }
  else
    {
    return(false);
    }
  }
else
  {
  return(false);
  }
}

bool // success
VrmlMaterialInteractor::readTransparencyField(double & transparencyOut) const
{
const MaterialNode * node=static_cast<const MaterialNode *>(_node);
if(node)
  {
  transparencyOut=node->_transparencyField;
  return(true);
  }
else
  {
  return(false);
  }
}

bool // success
VrmlMaterialInteractor::writeTransparencyField(double transparency)
{
MaterialNode * node=static_cast<MaterialNode *>(_node);
if(node)
  {
  if((transparency>=0.0)&&(transparency<=1.0))
    {
    node->_transparencyField=transparency;
    node->postRebuild();
    return(true);
    }
  else
    {
    return(false);
    }
  }
else
  {
  return(false);
  }
}

bool // success
VrmlMaterialInteractor::interpolate(ArRef<VrmlMaterialInteractor> previous,
                                    ArRef<VrmlMaterialInteractor> next,
                                    double nextPreviousRatio)
{
MaterialNode * node=static_cast<MaterialNode *>(_node);
const MaterialNode * previousNode=static_cast<const MaterialNode *>
                                  (previous->_node);
const MaterialNode * nextNode=static_cast<const MaterialNode *>
                              (next->_node);
if(node&&previousNode&&nextNode)
  {
  if(nextPreviousRatio<0.0)
    {
    nextPreviousRatio=0.0;
    }
  else if(nextPreviousRatio>1.0)
    {
    nextPreviousRatio=1.0;
    }
  double previousNextRatio=1.0-nextPreviousRatio;
  node->_ambientIntensityField=
        previousNode->_ambientIntensityField*previousNextRatio+
        nextNode->_ambientIntensityField*nextPreviousRatio;
  node->_diffuseColorField.x=
        previousNode->_diffuseColorField.x*previousNextRatio+
        nextNode->_diffuseColorField.x*nextPreviousRatio;
  node->_diffuseColorField.y=
        previousNode->_diffuseColorField.y*previousNextRatio+
        nextNode->_diffuseColorField.y*nextPreviousRatio;
  node->_diffuseColorField.z=
        previousNode->_diffuseColorField.z*previousNextRatio+
        nextNode->_diffuseColorField.z*nextPreviousRatio;
  node->_emissiveColorField.x=
        previousNode->_emissiveColorField.x*previousNextRatio+
        nextNode->_emissiveColorField.x*nextPreviousRatio;
  node->_emissiveColorField.y=
        previousNode->_emissiveColorField.y*previousNextRatio+
        nextNode->_emissiveColorField.y*nextPreviousRatio;
  node->_emissiveColorField.z=
        previousNode->_emissiveColorField.z*previousNextRatio+
        nextNode->_emissiveColorField.z*nextPreviousRatio;
  node->_shininessField=
        previousNode->_shininessField*previousNextRatio+
        nextNode->_shininessField*nextPreviousRatio;
  node->_specularColorField.x=
        previousNode->_specularColorField.x*previousNextRatio+
        nextNode->_specularColorField.x*nextPreviousRatio;
  node->_specularColorField.y=
        previousNode->_specularColorField.y*previousNextRatio+
        nextNode->_specularColorField.y*nextPreviousRatio;
  node->_specularColorField.z=
        previousNode->_specularColorField.z*previousNextRatio+
        nextNode->_specularColorField.z*nextPreviousRatio;
  node->_transparencyField=
        previousNode->_transparencyField*previousNextRatio+
        nextNode->_transparencyField*nextPreviousRatio;
  node->postRebuild();
  return(true);
  }
else
  {
  return(false);
  }
}

} // namespace AReVi

//----------------------------------------------------------------------------
