// LI2/ENIB - AReVi - VRML services
//----------------------------------------------------------------------------
// file        : AReVi/VRML/vrmlNode_priv.cpp
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/VRML/vrmlNode_priv.h"

#include "AReVi/VRML/vrmlValue_priv.h"
#include "AReVi/VRML/vrmlShape3D.h"

#include "AReVi/VRML/anchorNode_priv.h"
#include "AReVi/VRML/appearanceNode_priv.h"
#include "AReVi/VRML/billboardNode_priv.h"
#include "AReVi/VRML/boxNode_priv.h"
#include "AReVi/VRML/collisionNode_priv.h"
#include "AReVi/VRML/colorNode_priv.h"
#include "AReVi/VRML/coneNode_priv.h"
#include "AReVi/VRML/coordinateNode_priv.h"
#include "AReVi/VRML/cylinderNode_priv.h"
#include "AReVi/VRML/elevationGridNode_priv.h"
#include "AReVi/VRML/extrusionNode_priv.h"
#include "AReVi/VRML/groupNode_priv.h"
#include "AReVi/VRML/imageTextureNode_priv.h"
#include "AReVi/VRML/indexedFaceSetNode_priv.h"
#include "AReVi/VRML/indexedLineSetNode_priv.h"
#include "AReVi/VRML/inlineNode_priv.h"
#include "AReVi/VRML/materialNode_priv.h"
#include "AReVi/VRML/normalNode_priv.h"
#include "AReVi/VRML/pixelTextureNode_priv.h"
#include "AReVi/VRML/pointSetNode_priv.h"
#include "AReVi/VRML/shapeNode_priv.h"
#include "AReVi/VRML/sphereNode_priv.h"
#include "AReVi/VRML/textureCoordinateNode_priv.h"
#include "AReVi/VRML/textureTransformNode_priv.h"
#include "AReVi/VRML/transformNode_priv.h"
#include "AReVi/VRML/worldInfoNode_priv.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class VrmlNode
//----------------------------------------------------------------------------

//-------- Construction / Destruction ----------------------------------------

VrmlNode * // new node
VrmlNode::create(const StlString & nodeType,
                 VrmlShape3D * shape) // Static
{
if(nodeType=="Anchor")
  {
  return(new AnchorNode(nodeType,shape));
  }
if(nodeType=="Appearance")
  {
  return(new AppearanceNode(nodeType,shape));
  }
if(nodeType=="Billboard")
  {
  return(new BillboardNode(nodeType,shape));
  }
if(nodeType=="Box")
  {
  return(new BoxNode(nodeType,shape));
  }
if(nodeType=="Collision")
  {
  return(new CollisionNode(nodeType,shape));
  }
if(nodeType=="Color")
  {
  return(new ColorNode(nodeType,shape));
  }
if(nodeType=="Cone")
  {
  return(new ConeNode(nodeType,shape));
  }
if(nodeType=="Coordinate")
  {
  return(new CoordinateNode(nodeType,shape));
  }
if(nodeType=="Cylinder")
  {
  return(new CylinderNode(nodeType,shape));
  }
if(nodeType=="ElevationGrid")
  {
  return(new ElevationGridNode(nodeType,shape));
  }
if(nodeType=="Extrusion")
  {
  return(new ExtrusionNode(nodeType,shape));
  }
if(nodeType=="Group")
  {
  return(new GroupNode(nodeType,shape));
  }
if(nodeType=="ImageTexture")
  {
  return(new ImageTextureNode(nodeType,shape));
  }
if(nodeType=="IndexedFaceSet")
  {
  return(new IndexedFaceSetNode(nodeType,shape));
  }
if(nodeType=="IndexedLineSet")
  {
  return(new IndexedLineSetNode(nodeType,shape));
  }
if(nodeType=="Inline")
  {
  return(new InlineNode(nodeType,shape));
  }
if(nodeType=="Material")
  {
  return(new MaterialNode(nodeType,shape));
  }
if(nodeType=="Normal")
  {
  return(new NormalNode(nodeType,shape));
  }
if(nodeType=="PixelTexture")
  {
  return(new PixelTextureNode(nodeType,shape));
  }
if(nodeType=="PointSet")
  {
  return(new PointSetNode(nodeType,shape));
  }
if(nodeType=="Shape")
  {
  return(new ShapeNode(nodeType,shape));
  }
if(nodeType=="Sphere")
  {
  return(new SphereNode(nodeType,shape));
  }
if(nodeType=="TextureCoordinate")
  {
  return(new TextureCoordinateNode(nodeType,shape));
  }
if(nodeType=="TextureTransform")
  {
  return(new TextureTransformNode(nodeType,shape));
  }
if(nodeType=="Transform")
  {
  return(new TransformNode(nodeType,shape));
  }
if(nodeType=="WorldInfo")
  {
  return(new WorldInfoNode(nodeType,shape));
  }
return(new VrmlNode(nodeType,shape));
}

VrmlNode::~VrmlNode(void)
{
if(_name.size())
  {
  _shape->_removeNamedNode(this);
  }
}

//-------- Inspection --------------------------------------------------------

bool // problem occured
VrmlNode::fail(void) const
{
return(_fail);
}

bool // valid child for grouping node
VrmlNode::isGroupChild(void) const
{
return(false);
}

bool // is a group
VrmlNode::isGroup(void) const
{
return(false);
}

bool // is a texture
VrmlNode::isTexture(void) const
{
return(false);
}

bool // is a geometry
VrmlNode::isGeometry(void) const
{
return(false);
}

//-------- Naming ------------------------------------------------------------

const StlString & // node type
VrmlNode::getType(void) const
{
return(_type);
}

void
VrmlNode::setName(const StlString & nodeName)
{
if(!_name.size())
  {
  _name=nodeName;
  if(_name.size())
    {
    _shape->_addNamedNode(this);
    }
  }
}

const StlString & // node name
VrmlNode::getName(void) const
{
return(_name);
}

//-------- Parent management -------------------------------------------------

void
VrmlNode::addParent(VrmlNode * parent)
{
_parents.push_back(parent);
}

void
VrmlNode::removeParent(VrmlNode * parent)
{
for(unsigned int i=_parents.size();i--;)
  {
  if(_parents[i]==parent)
    {
    StlVectorFastErase(_parents,i);
    break;
    }
  }
destroyIfNoParent();
}

void
VrmlNode::destroyIfNoParent(void)
{
if(_parents.empty())
  {
  delete this;
  }
}

//-------- Field management --------------------------------------------------

void
VrmlNode::getNodeFieldNames(StlVector<StlString> & nodeFieldNamesOut) const
{
nodeFieldNamesOut.clear();
}

const VrmlNode * // node at field
VrmlNode::getNodeField(const StlString & /* fieldName */) const
{
return((const VrmlNode *)0);
}

VrmlNode * // node at field
VrmlNode::accessNodeField(const StlString & fieldName) const
{
// avoid code duplication for getFieldNode() and accessFieldNode()
return(const_cast<VrmlNode *>(getNodeField(fieldName)));
}

void
VrmlNode::addField(const StlString & fieldName,
                   VrmlValue * /* fieldValue */)
{
_shape->_ignoreFieldMsg(fieldName,_type);
}

void
VrmlNode::check(void)
{
_shape->_ignoreNodeMsg(_type);
_fail=true;
}

//-------- Code generation ---------------------------------------------------

void
VrmlNode::generateCode(ostream & output,
                       unsigned int indent,
                       bool showDefaultValues,
                       StlVector<const VrmlNode *> & defNodesInOut) const
{
unsigned int i;
if(_name.size())
  {
  for(i=defNodesInOut.size();i--;)
    {
    if(defNodesInOut[i]==this)
      {
      output << "USE " << _name << '\n';
      return;
      }
    }
  defNodesInOut.push_back(this);
  output << "DEF " << _name << ' ';
  }
output << _type << " {\n";
_generateFields(output,indent+2,showDefaultValues,defNodesInOut);
_indent(output,indent);
output << "} # ";
if(_name.size())
  {
  output << _name << ' ';
  }
output << _type << '\n';
}

//-------- Rendering ---------------------------------------------------------

void
VrmlNode::postRebuild(void)
{
if(!_needsRebuild)
  {
  _needsRebuild=true;
  postRebuildToTop();
  }
}

void
VrmlNode::postRebuildToTop(void)
{
for(unsigned int i=_parents.size();i--;)
  {
  if(_parents[i])
    {
    if(!_parents[i]->_needsRebuildUnder)
      {
      _parents[i]->_needsRebuildUnder=true;
      _parents[i]->postRebuildToTop();
      }
    }
  else
    {
    _shape->_postRebuild();
    }
  }
}

void
VrmlNode::rebuild(void)
{
_needsRebuild=false;
_needsRebuildUnder=false;
}

//-------- Forbidden operations ----------------------------------------------

void
VrmlNode::_generateFields(ostream & /* output */,
                          unsigned int /* indent */,
                          bool /* showDefaultValues */,
                          StlVector<const VrmlNode *> & /* defNodesInOut */)
                                                            const // Protected
{
  // Nothing to be done
}

void
VrmlNode::_indent(ostream & output,
                  unsigned int indent) const // Protected
{
while(indent--)
  {
  output << ' ';
  }
}

void
VrmlNode::_generateSFBool(ostream & output,
                          unsigned int indent,
                          const StlString & fieldName,
                          bool field) const // Protected
{
_indent(output,indent);
output << fieldName << ' ' << (field ? "TRUE" : "FALSE") << '\n';
}

void
VrmlNode::_generateSFColor(ostream & output,
                           unsigned int indent,
                           const StlString & fieldName,
                           const Util3D::Dbl3 & field) const // Protected
{
_indent(output,indent);
output << fieldName << ' ' << field.x << ' '
                           << field.y << ' '
                           << field.z << '\n';
}

void
VrmlNode::_generateMFColor(ostream & output,
                           unsigned int indent,
                           const StlString & fieldName,
                           const StlVector<Util3D::Dbl3> & field) const
{                                                                 // Protected
_indent(output,indent);
output << fieldName << " [\n";
for(unsigned int i=0;i<field.size();i++)
  {
  _indent(output,indent+2);
  output << field[i].x << ' ' << field[i].y << ' ' << field[i].z << '\n';
  }
_indent(output,indent);
output << "]\n";
}

void
VrmlNode::_generateSFFloat(ostream & output,
                           unsigned int indent,
                           const StlString & fieldName,
                           double field) const // Protected
{
_indent(output,indent);
output << fieldName << ' ' << field << '\n';
}

void
VrmlNode::_generateMFFloat(ostream & output,
                           unsigned int indent,
                           const StlString & fieldName,
                           const StlVector<double> & field) const // protected
{
_indent(output,indent);
output << fieldName << " [\n";
for(unsigned int i=0;i<field.size();i++)
  {
  _indent(output,indent+2);
  output << field[i] << '\n';
  }
_indent(output,indent);
output << "]\n";
}

void
VrmlNode::_generateSFImage(ostream & output,
                           unsigned int indent,
                           const StlString & fieldName,
                           const StlVector<unsigned int> & field) const
{                                                                 // protected
_indent(output,indent);
output << fieldName << ' ' << field[0] << ' '
                           << field[1] << ' '
                           << field[2] << '\n';
unsigned int subIndent=indent+fieldName.size();
bool mustIndent=true;
for(unsigned int i=3;i<field.size();i++)
  {
  if(mustIndent)
    {
    _indent(output,subIndent);
    }
  output << ' ' << field[i];
  if(((i-3)%4)==3)
    {
    output << '\n';
    mustIndent=true;
    }
  else
    {
    mustIndent=false;
    }
  }
if(!mustIndent)
  {
  output << '\n';
  }
}

void
VrmlNode::_generateSFInt32(ostream & output,
                           unsigned int indent,
                           const StlString & fieldName,
                           int field) const // Protected
{
_indent(output,indent);
output << fieldName << ' ' << field << '\n';
}

void
VrmlNode::_generateMFInt32(ostream & output,
                           unsigned int indent,
                           const StlString & fieldName,
                           const StlVector<StlVector<int> > & field) const
{                                                                 // Protected
_indent(output,indent);
output << fieldName << " [\n";
if(field.size()==1)
  {
  for(unsigned int i=0;i<field[0].size();i++)
    {
    _indent(output,indent+2);
    output << field[0][i] << '\n';
    }
  }
else
  {
  for(unsigned int i=0;i<field.size();i++)
    {
    _indent(output,indent+2);
    for(unsigned int j=0;j<field[i].size();j++)
      {
      output << field[i][j] << ' ';
      }
    output << "-1\n";
    }
  }
_indent(output,indent);
output << "]\n";
}

void
VrmlNode::_generateSFNode(ostream & output,
                          unsigned int indent,
                          const StlString & fieldName,
                          const VrmlNode * field,
                          bool showDefaultValues,
                          StlVector<const VrmlNode *> & defNodesInOut) const
{                                                                 // Protected
_indent(output,indent);
output << fieldName << ' ';
if(field)
  {
  field->generateCode(output,indent,showDefaultValues,defNodesInOut);
  }
else
  {
  output << "NULL\n";
  }
}

void
VrmlNode::_generateMFNode(ostream & output,
                          unsigned int indent,
                          const StlString & fieldName,
                          const StlVector<VrmlNode *> & field,
                          bool showDefaultValues,
                          StlVector<const VrmlNode *> & defNodesInOut) const
{                                                                 // Protected
_indent(output,indent);
output << fieldName << " [\n";
for(unsigned int i=0;i<field.size();i++)
  {
  _indent(output,indent+2);
  if(field[i])
    {
    field[i]->generateCode(output,indent+2,showDefaultValues,defNodesInOut);
    }
  else
    {
    output << "NULL\n";
    }
  }
_indent(output,indent);
output << "]\n";
}

void
VrmlNode::_generateSFRotation(ostream & output,
                              unsigned int indent,
                              const StlString & fieldName,
                              const Util3D::Dbl4 & field) const // Protected
{
_indent(output,indent);
output << fieldName << ' ' << field.x << ' '
                           << field.y << ' '
                           << field.z << ' '
                           << field.a << '\n';
}

void
VrmlNode::_generateMFRotation(ostream & output,
                              unsigned int indent,
                              const StlString & fieldName,
                              const StlVector<Util3D::Dbl4> & field) const
{                                                                 // Protected
_indent(output,indent);
output << fieldName << " [\n";
for(unsigned int i=0;i<field.size();i++)
  {
  _indent(output,indent+2);
  output << field[i].x << ' '
         << field[i].y << ' '
         << field[i].z << ' '
         << field[i].a << '\n';
  }
_indent(output,indent);
output << "]\n";
}

void
VrmlNode::_generateSFString(ostream & output,
                            unsigned int indent,
                            const StlString & fieldName,
                            const StlString & field) const // Protected
{
_indent(output,indent);
StlString dump;
strDump(dump,field);
output << fieldName << ' ' << dump << '\n';
}

void
VrmlNode::_generateMFString(ostream & output,
                            unsigned int indent,
                            const StlString & fieldName,
                            const StlVector<StlString> & field) const
{                                                                 // Protected
_indent(output,indent);
output << fieldName << " [\n";
for(unsigned int i=0;i<field.size();i++)
  {
  _indent(output,indent+2);
  StlString dump;
  strDump(dump,field[i]);
  output << dump << '\n';
  }
_indent(output,indent);
output << "]\n";
}

void
VrmlNode::_generateSFTime(ostream & output,
                          unsigned int indent,
                          const StlString & fieldName,
                          double field) const // Protected
{
_indent(output,indent);
output << fieldName << ' ' << field << '\n';
}

void
VrmlNode::_generateMFTime(ostream & output,
                          unsigned int indent,
                          const StlString & fieldName,
                          const StlVector<double> & field) const // Protected
{
_indent(output,indent);
output << fieldName << " [\n";
for(unsigned int i=0;i<field.size();i++)
  {
  _indent(output,indent+2);
  output << field[i] << '\n';
  }
_indent(output,indent);
output << "]\n";
}

void
VrmlNode::_generateSFVec2f(ostream & output,
                           unsigned int indent,
                           const StlString & fieldName,
                           const Util3D::Dbl2 & field) const // Protected
{
_indent(output,indent);
output << fieldName << ' ' << field.x << ' ' << field.y << '\n';
}

void
VrmlNode::_generateMFVec2f(ostream & output,
                           unsigned int indent,
                           const StlString & fieldName,
                           const StlVector<Util3D::Dbl2> & field) const
{                                                                 // Protected
_indent(output,indent);
output << fieldName << " [\n";
for(unsigned int i=0;i<field.size();i++)
  {
  _indent(output,indent+2);
  output << field[i].x << ' ' << field[i].y << '\n';
  }
_indent(output,indent);
output << "]\n";
}

void
VrmlNode::_generateSFVec3f(ostream & output,
                           unsigned int indent,
                           const StlString & fieldName,
                           const Util3D::Dbl3 & field) const // Protected
{
_indent(output,indent);
output << fieldName << ' ' << field.x << ' '
                           << field.y << ' '
                           << field.z << '\n';
}

void
VrmlNode::_generateMFVec3f(ostream & output,
                           unsigned int indent,
                           const StlString & fieldName,
                           const StlVector<Util3D::Dbl3> & field) const
{                                                                 // Protected
_indent(output,indent);
output << fieldName << " [\n";
for(unsigned int i=0;i<field.size();i++)
  {
  _indent(output,indent+2);
  output << field[i].x << ' ' << field[i].y << ' ' << field[i].z << '\n';
  }
_indent(output,indent);
output << "]\n";
}

VrmlNode::VrmlNode(const StlString & nodeType,
                   VrmlShape3D * shape) // Protected
: _type(nodeType),
  _shape(shape),
  _parents(),
  _name(),
  _fail(false),
  _needsRebuild(true),
  _needsRebuildUnder(true)
{
  // Nothing to be done
}

} // namespace AReVi

//----------------------------------------------------------------------------
