// LI2/ENIB - AReVi - VRML services
//----------------------------------------------------------------------------
// file        : AReVi/VRML/vrmlNormalInteractor.cpp
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/VRML/vrmlNormalInteractor.h"

#include "AReVi/VRML/normalNode_priv.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class VrmlNormalInteractor
//----------------------------------------------------------------------------

AR_CLASS_DEF(VrmlNormalInteractor,VrmlInteractor)

//-------- Construction / Destruction ----------------------------------------

VrmlNormalInteractor::VrmlNormalInteractor(ArCW & arCW) // Protected
: VrmlInteractor(arCW)
{
  // Nothing to be done
}

VrmlNormalInteractor::~VrmlNormalInteractor(void) // Protected
{
  // Nothing to be done
}

//-------- Connection --------------------------------------------------------

bool // success
VrmlNormalInteractor::connectToNode(ArRef<VrmlShape3D> shape,
                                    const StlString & nodeName,
                                    int occurence)
{
if(VrmlInteractor::connectToNode(shape,nodeName,occurence)&&
   (_node->getType()=="Normal"))
  {
  return(true);
  }
else
  {
  disconnect();
  return(false);
  }
}

bool // success
VrmlNormalInteractor::connectTo(ArConstRef<VrmlInteractor> interactor)
{
if(VrmlInteractor::connectTo(interactor)&&
   (_node->getType()=="Normal"))
  {
  return(true);
  }
else
  {
  disconnect();
  return(false);
  }
}

//-------- Parameters --------------------------------------------------------

bool // success
VrmlNormalInteractor::readVectorFieldSize(unsigned int & sizeOut) const
{
const NormalNode * node=static_cast<const NormalNode *>(_node);
if(node)
  {
  sizeOut=node->_vectorField.size();
  return(true);
  }
else
  {
  return(false);
  }
}

bool // success
VrmlNormalInteractor::readVectorField(unsigned int index,
                                      double & vxOut,
                                      double & vyOut,
                                      double & vzOut) const
{
const NormalNode * node=static_cast<const NormalNode *>(_node);
if(node)
  {
  if(index<node->_vectorField.size())
    {
    vxOut=node->_vectorField[index].x;
    vyOut=node->_vectorField[index].y;
    vzOut=node->_vectorField[index].z;
    return(true);
    }
  else
    {
    return(false);
    }
  }
else
  {
  return(false);
  }
}

bool // success
VrmlNormalInteractor::writeVectorField(unsigned int index,
                                       double vx,
                                       double vy,
                                       double vz)
{
NormalNode * node=static_cast<NormalNode *>(_node);
if(node)
  {
  if(index<node->_vectorField.size())
    {
    node->_vectorField[index].x=vx;
    node->_vectorField[index].y=vy;
    node->_vectorField[index].z=vz;
    node->postRebuild();
    return(true);
    }
  else
    {
    return(false);
    }
  }
else
  {
  return(false);
  }
}

bool // success
VrmlNormalInteractor::interpolate(ArRef<VrmlNormalInteractor> previous,
                                  ArRef<VrmlNormalInteractor> next,
                                  double nextPreviousRatio)
{
NormalNode * node=static_cast<NormalNode *>(_node);
const NormalNode * previousNode=static_cast<const NormalNode *>
                                (previous->_node);
const NormalNode * nextNode=static_cast<const NormalNode *>
                            (next->_node);
if(node&&previousNode&&nextNode)
  {
  unsigned int nb=node->_vectorField.size();
  if((nb<=previousNode->_vectorField.size())&&
     (nb<=nextNode->_vectorField.size()))
    {
    if(nextPreviousRatio<0.0) nextPreviousRatio=0.0;
    else if(nextPreviousRatio>1.0) nextPreviousRatio=1.0;
    double previousNextRatio=1.0-nextPreviousRatio;
    while(nb--)
      {
      node->_vectorField[nb].x=
            previousNode->_vectorField[nb].x*previousNextRatio+
            nextNode->_vectorField[nb].x*nextPreviousRatio;
      node->_vectorField[nb].y=
            previousNode->_vectorField[nb].y*previousNextRatio+
            nextNode->_vectorField[nb].y*nextPreviousRatio;
      node->_vectorField[nb].z=
            previousNode->_vectorField[nb].z*previousNextRatio+
            nextNode->_vectorField[nb].z*nextPreviousRatio;
      }
    node->postRebuild();
    return(true);
    }
  else
    {
    return(false);
    }
  }
else
  {
  return(false);
  }
}

} // namespace AReVi

//----------------------------------------------------------------------------
