/* LI2/ENIB - AReVi - VRML services
//----------------------------------------------------------------------------
// file        : AReVi/VRML/vrmlParser_priv.l
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
// Class VrmlParser
//--------------------------------------------------------------------------*/

%{
#undef YY_DECL
#define YY_DECL int VrmlParser::yylex(void * arg)
#define yylval (*((YYSTYPE*)arg))
%}

%option c++
%option 8bit
%option noyywrap
%option prefix="_Vrml"
%option yyclass="VrmlParser"

%x strEnv

space		[ \t\r]+
nl		[\n]
integer		[0-9]+
hexaInteger	0[Xx][0-9A-Fa-f]+
r1		[0-9]*\.[0-9]+
r2		[0-9]+\.[0-9]*
r_exp		[eE][+-]?[0-9]+
real		({r1}|{r2}){r_exp}?|{integer}{r_exp}
id1		[^\x00-\x20"#',.[\\\]{}\x7f+\-0-9]
id2		[^\x00-\x20"#',.[\\\]{}\x7f]
ident		{id1}{id2}*

%%

  /*---- Punctuation ----*/

"."		{
                _store(yytext);
                return(PERIOD);
                }

"{"		{
                _store(yytext);
                return(LEFT_BRACE);
                }

"}"		{
                _store(yytext);
                return(RIGHT_BRACE);
                }

"["		{
                _store(yytext);
                return(LEFT_BRACKET);
                }

"]"		{
                _store(yytext);
                return(RIGHT_BRACKET);
                }

"+"		{
                _store(yytext);
                return(PLUS);
                }

"-"		{
                _store(yytext);
                return(MINUS);
                }

  /*---- Keywords ----*/

"DEF"		{
                _store(yytext);
                return(DEF_KW);
                }

"EXTERNPROTO"	{
                _store(yytext);
                return(EXTERNPROTO_KW);
                }

"FALSE"		{
                _store(yytext);
                return(FALSE_KW);
                }

"IS"		{
                _store(yytext);
                return(IS_KW);
                }

"NULL"		{
                _store(yytext);
                return(NULL_KW);
                }

"PROTO"		{
                _store(yytext);
                return(PROTO_KW);
                }

"ROUTE"		{
                _store(yytext);
                return(ROUTE_KW);
                }

"TO"		{
                _store(yytext);
                return(TO_KW);
                }

"TRUE"		{
                _store(yytext);
                return(TRUE_KW);
                }

"USE"		{
                _store(yytext);
                return(USE_KW);
                }

"eventIn"	{
                _store(yytext);
                return(eventIn_KW);
                }

"eventOut"	{
                _store(yytext);
                return(eventOut_KW);
                }

"exposedField"	{
                _store(yytext);
                return(exposedField_KW);
                }

"field"		{
                _store(yytext);
                return(field_KW);
                }

  /*---- Field types ----*/

"SFBool"	{
                _store(yytext);
                return(SFBool_KW);
                }

"SFColor"	{
                _store(yytext);
                return(SFColor_KW);
                }

"MFColor"	{
                _store(yytext);
                return(MFColor_KW);
                }

"SFFloat"	{
                _store(yytext);
                return(SFFloat_KW);
                }

"MFFloat"	{
                _store(yytext);
                return(MFFloat_KW);
                }

"SFImage"	{
                _store(yytext);
                return(SFImage_KW);
                }

"SFInt32"	{
                _store(yytext);
                return(SFInt32_KW);
                }

"MFInt32"	{
                _store(yytext);
                return(MFInt32_KW);
                }

"SFNode"	{
                _store(yytext);
                return(SFNode_KW);
                }

"MFNode"	{
                _store(yytext);
                return(MFNode_KW);
                }

"SFRotation"	{
                _store(yytext);
                return(SFRotation_KW);
                }

"MFRotation"	{
                _store(yytext);
                return(MFRotation_KW);
                }

"SFString"	{
                _store(yytext);
                return(SFString_KW);
                }

"MFString"	{
                _store(yytext);
                return(MFString_KW);
                }

"SFTime"	{
                _store(yytext);
                return(SFTime_KW);
                }

"SFVec2f"	{
                _store(yytext);
                return(SFVec2f_KW);
                }

"MFVec2f"	{
                _store(yytext);
                return(MFVec2f_KW);
                }

"SFVec3f"	{
                _store(yytext);
                return(SFVec3f_KW);
                }

"MFVec3f"	{
                _store(yytext);
                return(MFVec3f_KW);
                }

  /*---- Values ----*/

"Script"	{
                _store(yytext);
                yylval.str=_saveString(new StlString(yytext));
                return(Script_KW);
                }

{integer}	{
                _store(yytext);
                sscanf(yytext,"%d",&yylval.integer);
                return(INTEGER_VAL);
                }

{hexaInteger}	{
                _store(yytext);
                sscanf(yytext,"%x",(unsigned int *)&yylval.integer);
                return(INTEGER_VAL);
                }

{real}		{
                _store(yytext);
                sscanf(yytext,"%lg",&yylval.real);
                return(REAL_VAL);
                }

{ident}		{
                _store(yytext);
                yylval.str=_saveString(new StlString(yytext));
                return(IDENT);
                }

"\""		{
                _store(yytext);
                _saveString(new StlString);
                BEGIN(strEnv);
                }
<strEnv>"\""	{
                _store(yytext);
                BEGIN(INITIAL);
                yylval.str=_curStrings.back();
                return(STRING_VAL);
                }
<strEnv>"\\a"	{ _store(yytext); (*_curStrings.back())+="\a"; }
<strEnv>"\\b"	{ _store(yytext); (*_curStrings.back())+="\b"; }
<strEnv>"\\f"	{ _store(yytext); (*_curStrings.back())+="\f"; }
<strEnv>"\\n"	{ _store(yytext); (*_curStrings.back())+="\n"; }
<strEnv>"\\r"	{ _store(yytext); (*_curStrings.back())+="\r"; }
<strEnv>"\\t"	{ _store(yytext); (*_curStrings.back())+="\t"; }
<strEnv>"\\v"	{ _store(yytext); (*_curStrings.back())+="\v"; }
<strEnv>"\\\\"	{ _store(yytext); (*_curStrings.back())+="\\"; }
<strEnv>"\\\""	{ _store(yytext); (*_curStrings.back())+="\""; }
<strEnv>"\n"	{ _nextLine(); (*_curStrings.back())+="\n"; }
<strEnv>"\\\n"	{ (*_curStrings.back())+="\n"; }
<strEnv>"\\".	{ _store(yytext); (*_curStrings.back())+=yytext[1]; }
<strEnv><<EOF>>	{
                BEGIN(INITIAL);
                yylval.str=_curStrings.back();
                return(STRING_VAL);
                }
<strEnv>.       {
                _store(yytext);
                (*_curStrings.back())+=yytext;
                }

  /*---- Others ----*/

"#".*    	{ /* comment */; }

{space}		{ _store(yytext); }

","		{
                _store(yytext);
                // a comma is considered as a white space !
                }

{nl}		{ _nextLine(); }

.		{
                _store(yytext);
                yyerror("lexical error");
                }

%%

/*--------------------------------------------------------------------------*/
