// LI2/ENIB - AReVi - VRML services
//----------------------------------------------------------------------------
// file        : AReVi/VRML/vrmlParser_priv.y
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// Class VrmlParser
//----------------------------------------------------------------------------

%{
#include "AReVi/VRML/vrmlParser_priv.h"
using namespace AReVi;

#include "AReVi/VRML/vrmlShape3D.h"
#include "AReVi/VRML/vrmlValue_priv.h"

#include <stdio.h>
#include <stdlib.h>

#define YYINCLUDED_STDLIB_H
#define yyparse VrmlParser::parse
%}

%pure_parser

//----------------------------------------------------------------------------

//---- Punctuation ----
%token PERIOD LEFT_BRACE RIGHT_BRACE LEFT_BRACKET RIGHT_BRACKET PLUS MINUS

//---- Keywords ----
%token DEF_KW EXTERNPROTO_KW FALSE_KW IS_KW NULL_KW PROTO_KW
%token ROUTE_KW TO_KW TRUE_KW USE_KW
%token eventIn_KW eventOut_KW exposedField_KW field_KW

//---- Field types ----
%token SFBool_KW
%token SFColor_KW MFColor_KW
%token SFFloat_KW MFFloat_KW
%token SFImage_KW
%token SFInt32_KW MFInt32_KW
%token SFNode_KW MFNode_KW
%token SFRotation_KW MFRotation_KW
%token SFString_KW MFString_KW
%token SFTime_KW
%token SFVec2f_KW MFVec2f_KW
%token SFVec3f_KW MFVec3f_KW

//---- Value type ----

%union
  {
  int integer;
  double real;
  StlString * str;
  VrmlValue * val;
  VrmlNode * node;
  };

//---- Values ----
%token<integer> INTEGER_VAL
%token<real>    REAL_VAL
%token<str>     STRING_VAL IDENT Script_KW

//---- Non-terminal values ----
%type<val>  fieldValue intRealValueList nodeStatementList stringList
%type<node> nodeStatement node nodeBegin scriptBegin

//----------------------------------------------------------------------------

%start vrml

%%

vrml
	: vrmlStatements
;

vrmlStatements
	: // empty
	| vrmlStatements vrmlStatement
;

vrmlStatement
	: routeStatement
            {
            // Ignore routes for now
            }
	| protoStatement
            {
            // Ignore prototypes for now
            }
	| nodeStatement
            {
            if($1)
              {
              if($1->isGroupChild())
                {
                if($1->fail())
                  {
                  $1->destroyIfNoParent();
                  }
                else
                  {
                  _shape->_addRootNode((GroupChildNode *)$1);
                  }
                }
              else
                {
                StlString msg="Node type '";
                msg+=$1->getType();
                msg+="' is not a valid root node";
                _shape->_errorMsg(msg);
                $1->destroyIfNoParent();
                }
              }
            }
;

//-------- Routes --------

routeStatement
	: ROUTE_KW IDENT PERIOD IDENT TO_KW IDENT PERIOD IDENT
            {
            delete _relaxString($2);
            delete _relaxString($4);
            delete _relaxString($6);
            delete _relaxString($8);
            _shape->_errorMsg("Ignoring ROUTE statement");
            }
;

//-------- Prototypes --------

protoStatement
	: proto_kw IDENT
          LEFT_BRACKET interfaceDeclarations RIGHT_BRACKET
          LEFT_BRACE protoBody RIGHT_BRACE
            {
            _insideProto--;
            delete _relaxString($2);
            _shape->_errorMsg("Ignoring PROTO statement");
            }
	| externproto_kw IDENT
          LEFT_BRACKET externInterfaceDeclarations RIGHT_BRACKET
          urlList
            {
            _insideProto--;
            delete _relaxString($2);
            _shape->_errorMsg("Ignoring EXTERNPROTO statement");
            }
;

proto_kw
	: PROTO_KW
            {
            _insideProto++;
            }
;

interfaceDeclarations
	: // empty
	| interfaceDeclarations interfaceDeclaration
;

interfaceDeclaration
	: eventIn_KW fieldType IDENT
            {
            // Ignore prototypes for now
            delete _relaxString($3);
            }
	| eventOut_KW fieldType IDENT
            {
            // Ignore prototypes for now
            delete _relaxString($3);
            }
	| field_KW fieldType IDENT fieldValue
            {
            // Ignore prototypes for now
            delete _relaxString($3);
            delete _relaxValue($4);
            }
	| exposedField_KW fieldType IDENT fieldValue
            {
            // Ignore prototypes for now
            delete _relaxString($3);
            delete _relaxValue($4);
            }
;

protoBody
	: // empty
	| protoBody protoElem
;

protoElem
	: routeStatement
            {
            // Ignore routes for now
            }
	| protoStatement
            {
            // Ignore prototypes for now
            }
	| nodeStatement
            {
            // Ignore prototypes for now
            }
;

externproto_kw
        : EXTERNPROTO_KW
            {
            _insideProto++;
            }
;

externInterfaceDeclarations
	: // empty
	| externInterfaceDeclarations externInterfaceDeclaration
;

externInterfaceDeclaration
	: eventIn_KW fieldType IDENT
            {
            // Ignore prototypes for now
            delete _relaxString($3);
            }
	| eventOut_KW fieldType IDENT
            {
            // Ignore prototypes for now
            delete _relaxString($3);
            }
	| field_KW fieldType IDENT
            {
            // Ignore prototypes for now
            delete _relaxString($3);
            }
	| exposedField_KW fieldType IDENT
            {
            // Ignore prototypes for now
            delete _relaxString($3);
            }
;

fieldType
	: SFBool_KW
	| SFColor_KW
	| MFColor_KW
	| SFFloat_KW
	| MFFloat_KW
	| SFImage_KW
	| SFInt32_KW
	| MFInt32_KW
	| SFNode_KW
	| MFNode_KW
	| SFRotation_KW
	| MFRotation_KW
	| SFString_KW
	| MFString_KW
	| SFTime_KW
	| SFVec2f_KW
	| MFVec2f_KW
	| SFVec3f_KW
	| MFVec3f_KW
;

urlList
	: STRING_VAL
            {
            // Ignore prototypes for now
            delete _relaxString($1);
            }
	| LEFT_BRACKET RIGHT_BRACKET
            {
            // Ignore prototypes for now
            }
	| LEFT_BRACKET stringList RIGHT_BRACKET
            {
            // Ignore prototypes for now
            delete _relaxValue($2);
            }
;

//-------- Nodes --------

nodeStatement
	: node
            {
            if(_insideProto)
              {
              _relaxNode($1)->destroyIfNoParent();
              $$=(VrmlNode *)0;
              }
            else
              {
              $$=_relaxNode($1);
              $$->check();
              }
            }
	| DEF_KW IDENT node
            {
            if(_insideProto)
              {
              delete _relaxString($2);
              _relaxNode($3)->destroyIfNoParent();
              $$=(VrmlNode *)0;
              }
            else
              {
              $$=_relaxNode($3);
              $$->check();
              $$->setName(*$2);
              delete _relaxString($2);
              }
            }
	| USE_KW IDENT
            {
            if(_insideProto)
              {
              delete _relaxString($2);
              $$=(VrmlNode *)0;
              }
            else
              {
              $$=_shape->_accessNamedNode(*$2);
              if(!$$)
                {
                StlString msg="Can't find a node which name is '";
                msg+=*$2;
                msg+="'";
                _shape->_errorMsg(msg);
                }
              delete _relaxString($2);
              }
            }
;

node
	: nodeBegin nodeBody RIGHT_BRACE
	| scriptBegin scriptBody RIGHT_BRACE
;

nodeBegin
	: IDENT LEFT_BRACE
            {
            $$=_saveNode(VrmlNode::create(*$1,_shape));
            delete _relaxString($1);
            }
;

scriptBegin
	: Script_KW LEFT_BRACE
            {
            $$=_saveNode(VrmlNode::create(*$1,_shape));
            delete _relaxString($1);
            }
;

nodeBody
	: // empty
	| nodeBody nodeElem
;

nodeElem
	: IDENT fieldValue
            {
            _curNodes.back()->addField(*$1,$2);
            delete _relaxString($1);
            delete _relaxValue($2);
            }
 	| IDENT IS_KW IDENT
            {
            // Ignore prototypes for now
            delete _relaxString($1);
            delete _relaxString($3);
            }
 	| routeStatement
            {
            // Ignore routes for now
            }
 	| protoStatement
            {
            // Ignore prototypes for now
            }
;

scriptBody
	: // empty
	| scriptBody scriptElem
;

scriptElem
	: IDENT fieldValue
            {
            // Ignore scripts for now
            delete _relaxString($1);
            delete _relaxValue($2);
            }
	| IDENT IS_KW IDENT
            {
            // Ignore scripts for now
            delete _relaxString($1);
            delete _relaxString($3);
            }
	| routeStatement
            {
            // Ignore scripts for now
            }
	| protoStatement
            {
            // Ignore scripts for now
            }
	| eventIn_KW fieldType IDENT
            {
            // Ignore scripts for now
            delete _relaxString($3);
            }
	| eventIn_KW fieldType IDENT IS_KW IDENT
            {
            // Ignore scripts for now
            delete _relaxString($3);
            delete _relaxString($5);
            }
	| eventOut_KW fieldType IDENT
            {
            // Ignore scripts for now
            delete _relaxString($3);
            }
	| eventOut_KW fieldType IDENT IS_KW IDENT
            {
            // Ignore scripts for now
            delete _relaxString($3);
            delete _relaxString($5);
            }
	| field_KW fieldType IDENT fieldValue
            {
            // Ignore scripts for now
            delete _relaxString($3);
            delete _relaxValue($4);
            }
	| field_KW fieldType IDENT IS_KW IDENT
            {
            // Ignore scripts for now
            delete _relaxString($3);
            delete _relaxString($5);
            }
;

fieldValue
	: LEFT_BRACKET RIGHT_BRACKET
          // MF...
            {
            $$=_saveValue(new VrmlValue);
            $$->setBrackets();
            }
	| TRUE_KW
          // SFBool
            {
            $$=_saveValue(new VrmlValue);
            $$->addBool(true);
            }
	| FALSE_KW
          // SFBool
            {
            $$=_saveValue(new VrmlValue);
            $$->addBool(false);
            }
	| intRealValueList
          // SFColor SFFloat SFImage SFInt32 SFRotation SFTime SFVec2f SFVec3f
            {
            $$=$1;
            }
	| LEFT_BRACKET intRealValueList RIGHT_BRACKET
          // MFColor MFFloat MFInt32 MFRotation MFVec2f MFVec3f
            {
            $$=$2;
            $$->setBrackets();
            }
	| NULL_KW
          // SFNode
            {
            $$=_saveValue(new VrmlValue);
            $$->addNode((VrmlNode *)0);
            }
	| nodeStatement
          // SFNode
            {
            $$=_saveValue(new VrmlValue);
            $$->addNode($1);
            }
	| LEFT_BRACKET nodeStatementList RIGHT_BRACKET
          // MFNode
            {
            $$=$2;
            $$->setBrackets();
            }
	| STRING_VAL
          // SFString
            {
            $$=_saveValue(new VrmlValue);
            $$->addString(*$1);
            delete _relaxString($1);
            }
	| LEFT_BRACKET stringList RIGHT_BRACKET
          // MFString
            {
            $$=$2;
            $$->setBrackets();
            }
;

intRealValueList
	: INTEGER_VAL
            {
            $$=_saveValue(new VrmlValue);
            $$->addInteger($1);
            }
	| PLUS INTEGER_VAL
            {
            $$=_saveValue(new VrmlValue);
            $$->addInteger($2);
            }
	| MINUS INTEGER_VAL
            {
            $$=_saveValue(new VrmlValue);
            $$->addInteger(-$2);
            }
	| REAL_VAL
            {
            $$=_saveValue(new VrmlValue);
            $$->addReal($1);
            }
	| PLUS REAL_VAL
            {
            $$=_saveValue(new VrmlValue);
            $$->addReal($2);
            }
	| MINUS REAL_VAL
            {
            $$=_saveValue(new VrmlValue);
            $$->addReal(-$2);
            }
	| intRealValueList INTEGER_VAL
            {
            $$=$1;
            $$->addInteger($2);
            }
	| intRealValueList PLUS INTEGER_VAL
            {
            $$=$1;
            $$->addInteger($3);
            }
	| intRealValueList MINUS INTEGER_VAL
            {
            $$=$1;
            $$->addInteger(-$3);
            }
	| intRealValueList REAL_VAL
            {
            $$=$1;
            $$->addReal($2);
            }
	| intRealValueList PLUS REAL_VAL
            {
            $$=$1;
            $$->addReal($3);
            }
	| intRealValueList MINUS REAL_VAL
            {
            $$=$1;
            $$->addReal(-$3);
            }
;

nodeStatementList
	: nodeStatement
            {
            $$=_saveValue(new VrmlValue);
            $$->addNode($1);
            }
	| nodeStatementList nodeStatement
            {
            $$=$1;
            $$->addNode($2);
            }
;

stringList
	: STRING_VAL
            {
            $$=_saveValue(new VrmlValue);
            $$->addString(*$1);
            delete _relaxString($1);
            }
	| stringList STRING_VAL
            {
            $$=$1;
            $$->addString(*$2);
            delete _relaxString($2);
            }
;

%%

//----------------------------------------------------------------------------

namespace AReVi {

//-------- Construction / Destruction ----------------------------------------

VrmlParser::VrmlParser(istream * input,
                       VrmlShape3D * shape)
: _VrmlFlexLexer(input),
  _nbErrors(0),
  _nbLines(1),
  _line(),
  _shape(shape),
  _insideProto(0),
  _curStrings(),
  _curValues(),
  _curNodes()
{
  // Nothing to be done
}


VrmlParser::~VrmlParser(void)
{
if(!_curStrings.empty())
  {
  cerr << "VrmlParser::~VrmlParser(): "
       << _curStrings.size() << " string(s) left !!!" << endl;
  do
    {
    delete _curStrings.back();
    _curStrings.pop_back();
    } while(!_curStrings.empty());
  }
if(!_curValues.empty())
  {
  cerr << "VrmlParser::~VrmlParser(): "
       << _curValues.size() << " value(s) left !!!" << endl;
  do
    {
    delete _curValues.back();
    _curValues.pop_back();
    } while(!_curValues.empty());
  }
if(!_curNodes.empty())
  {
  cerr << "VrmlParser::~VrmlParser(): "
       << _curNodes.size() << " node(s) left !!!" << endl;
  do
    {
    delete _curNodes.back();
    _curNodes.pop_back();
    } while(!_curNodes.empty());
  }
}

//-------- Error messages ----------------------------------------------------

void
VrmlParser::yyerror(const char * text)
{
StlString msg=text;
if(_line.size())
  {
  msg+='\n';
  msg+=_line;
  msg+=" <---";
  }
_shape->_errorMsg(msg);
}

//-------- Utilities ---------------------------------------------------------

StlString *
VrmlParser::_saveString(StlString * str)
{
_curStrings.push_back(str);
return(str);
}

StlString *
VrmlParser::_relaxString(StlString * str)
{
for(unsigned int i=_curStrings.size();i--;)
  {
  if(_curStrings[i]==str)
    {
    _curStrings.erase(_curStrings.begin()+i);
    return(str);
    }
  }
return(str);
}


VrmlValue *
VrmlParser::_saveValue(VrmlValue * val)
{
_curValues.push_back(val);
return(val);
}

VrmlValue *
VrmlParser::_relaxValue(VrmlValue * val)
{
for(unsigned int i=_curValues.size();i--;)
  {
  if(_curValues[i]==val)
    {
    _curValues.erase(_curValues.begin()+i);
    return(val);
    }
  }
return(val);
}

VrmlNode *
VrmlParser::_saveNode(VrmlNode * node)
{
_curNodes.push_back(node);
return(node);
}

VrmlNode *
VrmlParser::_relaxNode(VrmlNode * node)
{
for(unsigned int i=_curNodes.size();i--;)
  {
  if(_curNodes[i]==node)
    {
    _curNodes.erase(_curNodes.begin()+i);
    return(node);
    }
  }
return(node);
}

void
VrmlParser::_nextLine(void)
{
_nbLines++;
_line.erase();
}

void
VrmlParser::_store(const char * text)
{
_line+=text;
}

} // namespace AReVi

//----------------------------------------------------------------------------
