// LI2/ENIB - AReVi - VRML services
//----------------------------------------------------------------------------
// file        : AReVi/VRML/vrmlSphereInteractor.cpp
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/VRML/vrmlSphereInteractor.h"

#include "AReVi/VRML/sphereNode_priv.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class VrmlSphereInteractor
//----------------------------------------------------------------------------

AR_CLASS_DEF(VrmlSphereInteractor,VrmlInteractor)

//-------- Construction / Destruction ----------------------------------------

VrmlSphereInteractor::VrmlSphereInteractor(ArCW & arCW) // Protected
: VrmlInteractor(arCW)
{
  // Nothing to be done
}

VrmlSphereInteractor::~VrmlSphereInteractor(void) // Protected
{
  // Nothing to be done
}

//-------- Connection --------------------------------------------------------

bool // success
VrmlSphereInteractor::connectToNode(ArRef<VrmlShape3D> shape,
                                    const StlString & nodeName,
                                    int occurence)
{
if(VrmlInteractor::connectToNode(shape,nodeName,occurence)&&
   (_node->getType()=="Sphere"))
  {
  return(true);
  }
else
  {
  disconnect();
  return(false);
  }
}

bool // success
VrmlSphereInteractor::connectTo(ArConstRef<VrmlInteractor> interactor)
{
if(VrmlInteractor::connectTo(interactor)&&
   (_node->getType()=="Sphere"))
  {
  return(true);
  }
else
  {
  disconnect();
  return(false);
  }
}

//-------- Parameters --------------------------------------------------------

bool // success
VrmlSphereInteractor::readRadiusField(double & radiusOut) const
{
const SphereNode * node=static_cast<const SphereNode *>(_node);
if(node)
  {
  radiusOut=node->_radiusField;
  return(true);
  }
else
  {
  return(false);
  }
}

bool // success
VrmlSphereInteractor::writeRadiusField(double radius)
{
SphereNode * node=static_cast<SphereNode *>(_node);
if(node)
  {
  if(radius>0.0)
    {
    node->_radiusField=radius;
    node->postRebuild();
    return(true);
    }
  else
    {
    return(false);
    }
  }
else
  {
  return(false);
  }
}

bool // success
VrmlSphereInteractor::readSolidField(bool & solidOut) const
{
const SphereNode * node=static_cast<const SphereNode *>(_node);
if(node)
  {
  solidOut=node->_solidField;
  return(true);
  }
else
  {
  return(false);
  }
}

bool // success
VrmlSphereInteractor::writeSolidField(bool solid)
{
SphereNode * node=static_cast<SphereNode *>(_node);
if(node)
  {
  node->_solidField=solid;
  node->postRebuild();
  return(true);
  }
else
  {
  return(false);
  }
}

} // namespace AReVi

//----------------------------------------------------------------------------
