// LI2/ENIB - AReVi - VRML services
//----------------------------------------------------------------------------
// file        : AReVi/VRML/vrmlTextureCoordinateInteractor.cpp
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/VRML/vrmlTextureCoordinateInteractor.h"

#include "AReVi/VRML/textureCoordinateNode_priv.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class VrmlTextureCoordinateInteractor
//----------------------------------------------------------------------------

AR_CLASS_DEF(VrmlTextureCoordinateInteractor,VrmlInteractor)

//-------- Construction / Destruction ----------------------------------------

VrmlTextureCoordinateInteractor::VrmlTextureCoordinateInteractor(ArCW & arCW)
: VrmlInteractor(arCW)                                            // Protected
{
  // Nothing to be done
}

VrmlTextureCoordinateInteractor::~VrmlTextureCoordinateInteractor(void)
{                                                                 // Protected
  // Nothing to be done
}

//-------- Connection --------------------------------------------------------

bool // success
VrmlTextureCoordinateInteractor::connectToNode(ArRef<VrmlShape3D> shape,
                                               const StlString & nodeName,
                                               int occurence)
{
if(VrmlInteractor::connectToNode(shape,nodeName,occurence)&&
   (_node->getType()=="TextureCoordinate"))
  {
  return(true);
  }
else
  {
  disconnect();
  return(false);
  }
}

bool // success
VrmlTextureCoordinateInteractor::connectTo(ArConstRef<VrmlInteractor>
                                           interactor)
{
if(VrmlInteractor::connectTo(interactor)&&
   (_node->getType()=="TextureCoordinate"))
  {
  return(true);
  }
else
  {
  disconnect();
  return(false);
  }
}

//-------- Parameters --------------------------------------------------------

bool // success
VrmlTextureCoordinateInteractor::readPointFieldSize(unsigned int & sizeOut)
                                                                         const
{
const TextureCoordinateNode * node=static_cast<const TextureCoordinateNode *>
                                   (_node);
if(node)
  {
  sizeOut=node->_pointField.size();
  return(true);
  }
else
  {
  return(false);
  }
}

bool // success
VrmlTextureCoordinateInteractor::readPointField(unsigned int index,
                                                double & pxOut,
                                                double & pyOut) const
{
const TextureCoordinateNode * node=static_cast<const TextureCoordinateNode *>
                                   (_node);
if(node)
  {
  if(index<node->_pointField.size())
    {
    pxOut=node->_pointField[index].x;
    pyOut=node->_pointField[index].y;
    return(true);
    }
  else
    {
    return(false);
    }
  }
else
  {
  return(false);
  }
}

bool // success
VrmlTextureCoordinateInteractor::writePointField(unsigned int index,
                                                 double px,
                                                 double py)
{
TextureCoordinateNode * node=static_cast<TextureCoordinateNode *>(_node);
if(node)
  {
  if(index<node->_pointField.size())
    {
    node->_pointField[index].x=px;
    node->_pointField[index].y=py;
    node->postRebuild();
    return(true);
    }
  else
    {
    return(false);
    }
  }
else
  {
  return(false);
  }
}

bool // success
VrmlTextureCoordinateInteractor::interpolate(
                              ArRef<VrmlTextureCoordinateInteractor> previous,
                              ArRef<VrmlTextureCoordinateInteractor> next,
                              double nextPreviousRatio)
{
TextureCoordinateNode * node=static_cast<TextureCoordinateNode *>(_node);
const TextureCoordinateNode * previousNode=
      static_cast<const TextureCoordinateNode *>(previous->_node);
const TextureCoordinateNode * nextNode=
      static_cast<const TextureCoordinateNode *>(next->_node);
if(node&&previousNode&&nextNode)
  {
  unsigned int nb=node->_pointField.size();
  if((nb<=previousNode->_pointField.size())&&
     (nb<=nextNode->_pointField.size()))
    {
    if(nextPreviousRatio<0.0) nextPreviousRatio=0.0;
    else if(nextPreviousRatio>1.0) nextPreviousRatio=1.0;
    double previousNextRatio=1.0-nextPreviousRatio;
    while(nb--)
      {
      node->_pointField[nb].x=
            previousNode->_pointField[nb].x*previousNextRatio+
            nextNode->_pointField[nb].x*nextPreviousRatio;
      node->_pointField[nb].y=
            previousNode->_pointField[nb].y*previousNextRatio+
            nextNode->_pointField[nb].y*nextPreviousRatio;
      }
    node->postRebuild();
    return(true);
    }
  else
    {
    return(false);
    }
  }
else
  {
  return(false);
  }
}

} // namespace AReVi

//----------------------------------------------------------------------------
