// LI2/ENIB - AReVi - VRML services
//----------------------------------------------------------------------------
// file        : AReVi/VRML/vrmlTextureTransformInteractor.cpp
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/VRML/vrmlTextureTransformInteractor.h"

#include "AReVi/VRML/textureTransformNode_priv.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class VrmlTextureTransformInteractor
//----------------------------------------------------------------------------

AR_CLASS_DEF(VrmlTextureTransformInteractor,VrmlInteractor)

//-------- Construction / Destruction ----------------------------------------

VrmlTextureTransformInteractor::VrmlTextureTransformInteractor(ArCW & arCW)
: VrmlInteractor(arCW)                                            // Protected
{
  // Nothing to be done
}

VrmlTextureTransformInteractor::~VrmlTextureTransformInteractor(void)
{                                                                 // Protected
  // Nothing to be done
}

//-------- Connection --------------------------------------------------------

bool // success
VrmlTextureTransformInteractor::connectToNode(ArRef<VrmlShape3D> shape,
                                              const StlString & nodeName,
                                              int occurence)
{
if(VrmlInteractor::connectToNode(shape,nodeName,occurence)&&
   (_node->getType()=="TextureTransform"))
  {
  return(true);
  }
else
  {
  disconnect();
  return(false);
  }
}

bool // success
VrmlTextureTransformInteractor::connectTo(ArConstRef<VrmlInteractor>
                                          interactor)
{
if(VrmlInteractor::connectTo(interactor)&&
   (_node->getType()=="TextureTransform"))
  {
  return(true);
  }
else
  {
  disconnect();
  return(false);
  }
}

//-------- Parameters --------------------------------------------------------

bool // success
VrmlTextureTransformInteractor::readCenterField(double & cxOut,
                                                double & cyOut) const
{
const TextureTransformNode * node=static_cast<const TextureTransformNode *>
                                  (_node);
if(node)
  {
  cxOut=node->_centerField.x;
  cyOut=node->_centerField.y;
  return(true);
  }
else
  {
  return(false);
  }
}

bool // success
VrmlTextureTransformInteractor::writeCenterField(double cx,
                                                 double cy)
{
TextureTransformNode * node=static_cast<TextureTransformNode *>(_node);
if(node)
  {
  node->_centerField.x=cx;
  node->_centerField.y=cy;
  node->postRebuild();
  return(true);
  }
else
  {
  return(false);
  }
}

bool // success
VrmlTextureTransformInteractor::readRotationField(double & angleOut) const
{
const TextureTransformNode * node=static_cast<const TextureTransformNode *>
                                  (_node);
if(node)
  {
  angleOut=node->_rotationField;
  return(true);
  }
else
  {
  return(false);
  }
}

bool // success
VrmlTextureTransformInteractor::writeRotationField(double angle)
{
TextureTransformNode * node=static_cast<TextureTransformNode *>(_node);
if(node)
  {
  node->_rotationField=angle;
  node->postRebuild();
  return(true);
  }
else
  {
  return(false);
  }
}

bool // success
VrmlTextureTransformInteractor::readScaleField(double & sxOut,
                                               double & syOut) const
{
const TextureTransformNode * node=static_cast<const TextureTransformNode *>
                                  (_node);
if(node)
  {
  sxOut=node->_scaleField.x;
  syOut=node->_scaleField.y;
  return(true);
  }
else
  {
  return(false);
  }
}

bool // success
VrmlTextureTransformInteractor::writeScaleField(double sx,
                                                double sy)
{
TextureTransformNode * node=static_cast<TextureTransformNode *>(_node);
if(node)
  {
  node->_scaleField.x=sx;
  node->_scaleField.y=sy;
  node->postRebuild();
  return(true);
  }
else
  {
  return(false);
  }
}

bool // success
VrmlTextureTransformInteractor::readTranslationField(double & txOut,
                                                     double & tyOut) const
{
const TextureTransformNode * node=static_cast<const TextureTransformNode *>
                                  (_node);
if(node)
  {
  txOut=node->_translationField.x;
  tyOut=node->_translationField.y;
  return(true);
  }
else
  {
  return(false);
  }
}

bool // success
VrmlTextureTransformInteractor::writeTranslationField(double tx,
                                                      double ty)
{
TextureTransformNode * node=static_cast<TextureTransformNode *>(_node);
if(node)
  {
  node->_translationField.x=tx;
  node->_translationField.y=ty;
  node->postRebuild();
  return(true);
  }
else
  {
  return(false);
  }
}

} // namespace AReVi

//----------------------------------------------------------------------------
