// LI2/ENIB - AReVi - VRML services
//----------------------------------------------------------------------------
// file        : AReVi/VRML/vrmlTransformInteractor.cpp
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/VRML/vrmlTransformInteractor.h"

#include "AReVi/VRML/transformNode_priv.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class VrmlTransformInteractor
//----------------------------------------------------------------------------

AR_CLASS_DEF(VrmlTransformInteractor,VrmlInteractor)

//-------- Construction / Destruction ----------------------------------------

VrmlTransformInteractor::VrmlTransformInteractor(ArCW & arCW) // Protected
: VrmlInteractor(arCW)
{
  // Nothing to be done
}

VrmlTransformInteractor::~VrmlTransformInteractor(void) // Protected
{
  // Nothing to be done
}

//-------- Connection --------------------------------------------------------

bool // success
VrmlTransformInteractor::connectToNode(ArRef<VrmlShape3D> shape,
                                       const StlString & nodeName,
                                       int occurence)
{
if(VrmlInteractor::connectToNode(shape,nodeName,occurence)&&
   (_node->getType()=="Transform"))
  {
  return(true);
  }
else
  {
  disconnect();
  return(false);
  }
}

bool // success
VrmlTransformInteractor::connectTo(ArConstRef<VrmlInteractor> interactor)
{
if(VrmlInteractor::connectTo(interactor)&&
   (_node->getType()=="Transform"))
  {
  return(true);
  }
else
  {
  disconnect();
  return(false);
  }
}

//-------- Parameters --------------------------------------------------------

bool // success
VrmlTransformInteractor::readCenterField(double & cxOut,
                                         double & cyOut,
                                         double & czOut) const
{
const TransformNode * node=static_cast<const TransformNode *>(_node);
if(node)
  {
  cxOut=node->_centerField.x;
  cyOut=node->_centerField.y;
  czOut=node->_centerField.z;
  return(true);
  }
else
  {
  return(false);
  }
}

bool // success
VrmlTransformInteractor::writeCenterField(double cx,
                                          double cy,
                                          double cz)
{
TransformNode * node=static_cast<TransformNode *>(_node);
if(node)
  {
  node->_centerField.x=cx;
  node->_centerField.y=cy;
  node->_centerField.z=cz;
  node->postRebuild();
  return(true);
  }
else
  {
  return(false);
  }
}

bool // success
VrmlTransformInteractor::readRotationField(double & vxOut,
                                           double & vyOut,
                                           double & vzOut,
                                           double & angleOut) const
{
const TransformNode * node=static_cast<const TransformNode *>(_node);
if(node)
  {
  vxOut=node->_rotationField.x;
  vyOut=node->_rotationField.y;
  vzOut=node->_rotationField.z;
  angleOut=node->_rotationField.a;
  return(true);
  }
else
  {
  return(false);
  }
}

bool // success
VrmlTransformInteractor::writeRotationField(double vx,
                                            double vy,
                                            double vz,
                                            double angle)
{
TransformNode * node=static_cast<TransformNode *>(_node);
if(node)
  {
  double r=vx*vx+vy*vy+vz*vz;
  if(r)
    {
    r=sqrt(r);
    node->_rotationField.x=vx/r;
    node->_rotationField.y=vy/r;
    node->_rotationField.z=vz/r;
    node->_rotationField.a=angle/r;
    node->postRebuild();
    return(true);
    }
  else
    {
    return(false);
    }
  }
else
  {
  return(false);
  }
}

bool // success
VrmlTransformInteractor::readScaleField(double & sxOut,
                                        double & syOut,
                                        double & szOut) const
{
const TransformNode * node=static_cast<const TransformNode *>(_node);
if(node)
  {
  sxOut=node->_scaleField.x;
  syOut=node->_scaleField.y;
  szOut=node->_scaleField.z;
  return(true);
  }
else
  {
  return(false);
  }
}

bool // success
VrmlTransformInteractor::writeScaleField(double sx,
                                         double sy,
                                         double sz)
{
TransformNode * node=static_cast<TransformNode *>(_node);
if(node)
  {
  if((sx>0.0)&&(sy>0.0)&&(sz>0.0))
    {
    node->_scaleField.x=sx;
    node->_scaleField.y=sy;
    node->_scaleField.z=sz;
    node->postRebuild();
    return(true);
    }
  else
    {
    return(false);
    }
  }
else
  {
  return(false);
  }
}

bool // success
VrmlTransformInteractor::readScaleOrientationField(double & vxOut,
                                                   double & vyOut,
                                                   double & vzOut,
                                                   double & angleOut) const
{
const TransformNode * node=static_cast<const TransformNode *>(_node);
if(node)
  {
  vxOut=node->_scaleOrientationField.x;
  vyOut=node->_scaleOrientationField.y;
  vzOut=node->_scaleOrientationField.z;
  angleOut=node->_scaleOrientationField.a;
  return(true);
  }
else
  {
  return(false);
  }
}

bool // success
VrmlTransformInteractor::writeScaleOrientationField(double vx,
                                                    double vy,
                                                    double vz,
                                                    double angle)
{
TransformNode * node=static_cast<TransformNode *>(_node);
if(node)
  {
  double r=vx*vx+vy*vy+vz*vz;
  if(r)
    {
    r=sqrt(r);
    node->_scaleOrientationField.x=vx/r;
    node->_scaleOrientationField.y=vy/r;
    node->_scaleOrientationField.z=vz/r;
    node->_scaleOrientationField.a=angle/r;
    node->postRebuild();
    return(true);
    }
  else
    {
    return(false);
    }
  }
else
  {
  return(false);
  }
}

bool // success
VrmlTransformInteractor::readTranslationField(double & txOut,
                                              double & tyOut,
                                              double & tzOut) const
{
const TransformNode * node=static_cast<const TransformNode *>(_node);
if(node)
  {
  txOut=node->_translationField.x;
  tyOut=node->_translationField.y;
  tzOut=node->_translationField.z;
  return(true);
  }
else
  {
  return(false);
  }
}

bool // success
VrmlTransformInteractor::writeTranslationField(double tx,
                                               double ty,
                                               double tz)
{
TransformNode * node=static_cast<TransformNode *>(_node);
if(node)
  {
  node->_translationField.x=tx;
  node->_translationField.y=ty;
  node->_translationField.z=tz;
  node->postRebuild();
  return(true);
  }
else
  {
  return(false);
  }
}

} // namespace AReVi

//----------------------------------------------------------------------------
