// LI2/ENIB - AReVi - VRML services
//----------------------------------------------------------------------------
// file        : AReVi/VRML/vrmlValue_priv.cpp
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/VRML/vrmlValue_priv.h"

#include "AReVi/VRML/vrmlNode_priv.h"

#include "AReVi/Utils/math_priv.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class VrmlValue
//----------------------------------------------------------------------------

//-------- Construction / Destruction ----------------------------------------

VrmlValue::VrmlValue(void)
: _type(VrmlValue::UNKNOWN_T),
  _bracket(false)
{
  // _val is not initialized !
}

VrmlValue::~VrmlValue(void)
{
_destroy();
}

//-------- Type checking -----------------------------------------------------

bool // success
VrmlValue::checkSFBool(void)
{
if(!_bracket)
  {
  if(_type==VrmlValue::BOOL_T)
    {
    if(_val.b->size()==1)
      {
      return(true);
      }
    }
  }
return(false);
}

bool // success
VrmlValue::checkSFColor(void)
{
if(!_bracket)
  {
  if(_type==VrmlValue::INT_T)
    {
    _switchTo(VrmlValue::REAL_T);
    }
  if(_type==VrmlValue::REAL_T)
    {
    unsigned int size=_val.r->size();
    if(size==3)
      {
      while(size--)
        {
        if((*_val.r)[size]>1.0) return(false);
        if((*_val.r)[size]<0.0) return(false);
        }
      return(true);
      }
    }
  }
return(false);
}

bool // success
VrmlValue::checkMFColor(void)
{
if((_type==VrmlValue::UNKNOWN_T)&&_bracket)
  {
  return(true);
  }
if(_type==VrmlValue::INT_T)
  {
  _switchTo(VrmlValue::REAL_T);
  }
if(_type==VrmlValue::REAL_T)
  {
  unsigned int size=_val.r->size();
  if(size%3)
    {
    return(false);
    }
  unsigned int nb=size;
  while(size--)
    {
    if((*_val.r)[size]>1.0) return(false);
    if((*_val.r)[size]<0.0) return(false);
    }
  if(nb==3)
    {
    return(true);
    }
  if(!_bracket)
    {
    return(false);
    }
  return(true);
  }
return(false);
}

bool // success
VrmlValue::checkSFFloat(void)
{
if(!_bracket)
  {
  if(_type==VrmlValue::INT_T)
    {
    _switchTo(VrmlValue::REAL_T);
    }
  if(_type==VrmlValue::REAL_T)
    {
    if(_val.r->size()==1)
      {
      return(true);
      }
    }
  }
return(false);
}

bool // success
VrmlValue::checkMFFloat(void)
{
if((_type==VrmlValue::UNKNOWN_T)&&_bracket)
  {
  return(true);
  }
if(_type==VrmlValue::INT_T)
  {
  _switchTo(VrmlValue::REAL_T);
  }
if(_type==VrmlValue::REAL_T)
  {
  if(_val.r->size()==1)
    {
    return(true);
    }
  if(!_bracket)
    {
    return(false);
    }
  return(true);
  }
return(false);
}

bool // success
VrmlValue::checkSFImage(void)
{
if(!_bracket)
  {
  if(_type==VrmlValue::INT_T)
    {
    unsigned int size=_val.i->size();
    if(size>=3)
      {
      int width=(*_val.i)[0];
      if(width<0)
        {
        return(false);
        }
      int height=(*_val.i)[1];
      if(height<0)
        {
        return(false);
        }
      int color=(*_val.i)[2];
      if((color<0)||(color>4))
        {
        return(false);
        }
      if(size!=3+((unsigned int)width*(unsigned int)height))
        {
        return(false);
        }
      return(true);
      }
    }
  }
return(false);
}

bool // success
VrmlValue::checkSFInt32(void)
{
if(!_bracket)
  {
  if(_type==VrmlValue::INT_T)
    {
    if(_val.i->size()==1)
      {
      return(true);
      }
    }
  }
return(false);
}

bool // success
VrmlValue::checkMFInt32(void)
{
if((_type==VrmlValue::UNKNOWN_T)&&_bracket)
  {
  return(true);
  }
if(_type==VrmlValue::INT_T)
  {
  if(_val.i->size()==1)
    {
    return(true);
    }
  if(!_bracket)
    {
    return(false);
    }
  return(true);
  }
return(false);
}

bool // success
VrmlValue::checkSFNode(void)
{
if(!_bracket)
  {
  if(_type==VrmlValue::NODE_T)
    {
    if(_val.n->size()==1)
      {
      return(true);
      }
    }
  }
return(false);
}

bool // success
VrmlValue::checkMFNode(void)
{
if((_type==VrmlValue::UNKNOWN_T)&&_bracket)
  {
  return(true);
  }
if(_type==VrmlValue::NODE_T)
  {
  if(!_bracket&&(_val.n->size()!=1))
    {
    return(false);
    }
  return(true);
  }
return(false);
}

bool // success
VrmlValue::checkSFRotation(void)
{
if(!_bracket)
  {
  if(_type==VrmlValue::INT_T)
    {
    _switchTo(VrmlValue::REAL_T);
    }
  if(_type==VrmlValue::REAL_T)
    {
    if(_val.r->size()==4)
      {
      double x=(*_val.r)[0];
      double y=(*_val.r)[1];
      double z=(*_val.r)[2];
      double r=x*x+y*y+z*z;
      if(r)
        {
        if(r!=1.0)
          {
          r=::sqrt(r);
          (*_val.r)[0]=x/r;
          (*_val.r)[1]=y/r;
          (*_val.r)[2]=z/r;
          }
        return(true);
        }
      }
    }
  }
return(false);
}

bool // success
VrmlValue::checkMFRotation(void)
{
if((_type==VrmlValue::UNKNOWN_T)&&_bracket)
  {
  return(true);
  }
if(_type==VrmlValue::INT_T)
  {
  _switchTo(VrmlValue::REAL_T);
  }
if(_type==VrmlValue::REAL_T)
  {
  unsigned int size=_val.r->size();
  if(size%4)
    {
    return(false);
    }
  if(!_bracket&&(size!=4))
    {
    return(false);
    }
  while(size)
    {
    size-=4;
    double x=(*_val.r)[size];
    double y=(*_val.r)[size+1];
    double z=(*_val.r)[size+2];
    double r=x*x+y*y+z*z;
    if(!r)
      {
      return(false);
      }
    if(r!=1.0)
      {
      r=sqrt(r);
      (*_val.r)[size]=x/r;
      (*_val.r)[size+1]=y/r;
      (*_val.r)[size+2]=z/r;
      }
    }
  return(true);
  }
return(false);
}

bool // success
VrmlValue::checkSFString(void)
{
if(!_bracket)
  {
  if(_type==VrmlValue::STRING_T)
    {
    if(_val.s->size()==1)
      {
      return(true);
      }
    }
  }
return(false);
}

bool // success
VrmlValue::checkMFString(void)
{
if((_type==VrmlValue::UNKNOWN_T)&&_bracket)
  {
  return(true);
  }
if(_type==VrmlValue::STRING_T)
  {
  if(_val.s->size()==1)
    {
    return(true);
    }
  if(!_bracket)
    {
    return(false);
    }
  return(true);
  }
return(false);
}

bool // success
VrmlValue::checkSFTime(void)
{
if(!_bracket)
  {
  if(_type==VrmlValue::INT_T)
    {
    _switchTo(VrmlValue::REAL_T);
    }
  if(_type==VrmlValue::REAL_T)
    {
    if(_val.r->size()==1)
      {
      return(true);
      }
    }
  }
return(false);
}

bool // success
VrmlValue::checkSFVec2f(void)
{
if(!_bracket)
  {
  if(_type==VrmlValue::INT_T)
    {
    _switchTo(VrmlValue::REAL_T);
    }
  if(_type==VrmlValue::REAL_T)
    {
    if(_val.r->size()==2)
      {
      return(true);
      }
    }
  }
return(false);
}

bool // success
VrmlValue::checkMFVec2f(void)
{
if((_type==VrmlValue::UNKNOWN_T)&&_bracket)
  {
  return(true);
  }
if(_type==VrmlValue::INT_T)
  {
  _switchTo(VrmlValue::REAL_T);
  }
if(_type==VrmlValue::REAL_T)
  {
  unsigned int size=_val.r->size();
  if(size%2)
    {
    return(false);
    }
  if(size==2)
    {
    return(true);
    }
  if(!_bracket)
    {
    return(false);
    }
  return(true);
  }
return(false);
}

bool // success
VrmlValue::checkSFVec3f(void)
{
if(!_bracket)
  {
  if(_type==VrmlValue::INT_T)
    {
    _switchTo(VrmlValue::REAL_T);
    }
  if(_type==VrmlValue::REAL_T)
    {
    if(_val.r->size()==3)
      {
      return(true);
      }
    }
  }
return(false);
}

bool // success
VrmlValue::checkMFVec3f(void)
{
if((_type==VrmlValue::UNKNOWN_T)&&_bracket)
  {
  return(true);
  }
if(_type==VrmlValue::INT_T)
  {
  _switchTo(VrmlValue::REAL_T);
  }
if(_type==VrmlValue::REAL_T)
  {
  unsigned int size=_val.r->size();
  if(size%3)
    {
    return(false);
    }
  if(size==3)
    {
    return(true);
    }
  if(!_bracket)
    {
    return(false);
    }
  return(true);
  }
return(false);
}

//-------- Boolean values ----------------------------------------------------

void
VrmlValue::addBool(bool boolVal)
{
_switchTo(VrmlValue::BOOL_T);
_val.b->push_back(boolVal);
}

bool // bool at index
VrmlValue::getBool(int index) const
{
return((*_val.b)[index]);
}

//-------- Integer values ----------------------------------------------------

void
VrmlValue::addInteger(int integerVal)
{
if(_type==VrmlValue::REAL_T)
  {
  _val.r->push_back((double)integerVal);
  }
else
  {
  _switchTo(VrmlValue::INT_T);
  _val.i->push_back(integerVal);
  }
}

int // integer at index
VrmlValue::getInteger(int index) const
{
return((*_val.i)[index]);
}

//-------- Real values -------------------------------------------------------

void
VrmlValue::addReal(double realVal)
{
_switchTo(VrmlValue::REAL_T);
_val.r->push_back(realVal);
}

double // real at index
VrmlValue::getReal(int index) const
{
return((*_val.r)[index]);
}

//-------- Node values -------------------------------------------------------

void
VrmlValue::addNode(VrmlNode * nodeVal)
{
_switchTo(VrmlValue::NODE_T);
_val.n->push_back(nodeVal);
}

const VrmlNode * // node at index
VrmlValue::getNode(int index) const
{
return((*_val.n)[index]);
}

VrmlNode * // node at index
VrmlValue::accessNode(int index)
{
return((*_val.n)[index]);
}

//-------- String values -----------------------------------------------------

void
VrmlValue::addString(const StlString & stringVal)
{
_switchTo(VrmlValue::STRING_T);
_val.s->push_back(stringVal);
}

const StlString & // string at index
VrmlValue::getString(int index) const
{
return((*_val.s)[index]);
}

//-------- Multiple values ---------------------------------------------------

unsigned int // number of contained values
VrmlValue::getSize(void) const
{
switch(_type)
  {
  case VrmlValue::BOOL_T:
    {
    return(_val.b->size());
    }
  case VrmlValue::INT_T:
    {
    return(_val.i->size());
    }
  case VrmlValue::REAL_T:
    {
    return(_val.r->size());
    }
  case VrmlValue::NODE_T:
    {
    return(_val.n->size());
    }
  case VrmlValue::STRING_T:
    {
    return(_val.s->size());
    }
  case VrmlValue::UNKNOWN_T:
    {
    return(0);
    }
  }
return(0);
}

void
VrmlValue::setBrackets(void)
{
_bracket=true;
}

//-------- Private methods ---------------------------------------------------

void
VrmlValue::_init(VrmlValue::Type type) // Protected
{
switch(type)
  {
  case VrmlValue::BOOL_T:
    {
    _val.b=new StlVector<bool>;
    break;
    }
  case VrmlValue::INT_T:
    {
    _val.i=new StlVector<int>;
    break;
    }
  case VrmlValue::REAL_T:
    {
    _val.r=new StlVector<double>;
    break;
    }
  case VrmlValue::NODE_T:
    {
    _val.n=new StlVector<VrmlNode *>;
    break;
    }
  case VrmlValue::STRING_T:
    {
    _val.s=new StlVector<StlString>;
    break;
    }
  case VrmlValue::UNKNOWN_T:
    break;
  }
_type=type;
}

void
VrmlValue::_destroy(void) // Protected
{
switch(_type)
  {
  case VrmlValue::BOOL_T:
    {
    delete _val.b;
    break;
    }
  case VrmlValue::INT_T:
    {
    delete _val.i;
    break;
    }
  case VrmlValue::REAL_T:
    {
    delete _val.r;
    break;
    }
  case VrmlValue::NODE_T:
    {
    for(unsigned int i=_val.n->size();i--;)
      {
      if((*_val.n)[i])
        {
        (*_val.n)[i]->destroyIfNoParent();
        }
      }
    delete _val.n;
    break;
    }
  case VrmlValue::STRING_T:
    {
    delete _val.s;
    break;
    }
  case VrmlValue::UNKNOWN_T:
    break;
  }
_type=VrmlValue::UNKNOWN_T;
}

void
VrmlValue::_switchTo(VrmlValue::Type type) // Protected
{
if(_type==type)
  {
  return;
  }
else if((_type==VrmlValue::INT_T)&&(type==VrmlValue::REAL_T))
  {
  StlVector<double> * v=new StlVector<double>;
  v->reserve(_val.i->size());
  for(unsigned int i=0;i<_val.i->size();i++)
    {
    v->push_back((double)(*_val.i)[i]);
    }
  delete _val.i;
  _type=VrmlValue::REAL_T;
  _val.r=v;
  }
else
  {
  if(_type!=VrmlValue::UNKNOWN_T)
    {
    cerr << "Destroying VrmlValue content !" << endl;
    _destroy();
    }
  _init(type);
  }
}

} // namespace AReVi

//----------------------------------------------------------------------------
