// LI2/ENIB - AReVi - VRML services
//----------------------------------------------------------------------------
// file        : AReVi/VRML/worldInfoNode_priv.cpp
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/VRML/worldInfoNode_priv.h"

#include "AReVi/VRML/vrmlValue_priv.h"
#include "AReVi/VRML/vrmlShape3D.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class WorldInfoNode
//----------------------------------------------------------------------------

//-------- Construction / Destruction ----------------------------------------

WorldInfoNode::~WorldInfoNode(void)
{
  // Nothing to be done
}

//-------- Field management --------------------------------------------------

void
WorldInfoNode::addField(const StlString & fieldName,
                        VrmlValue * fieldValue)
{
bool ok=false;
if(fieldName=="info")
  {
  if(fieldValue->checkMFString())
    {
    _infoField.clear();
    unsigned int nb=fieldValue->getSize();
    for(unsigned int i=0;i<nb;i++)
      {
      _infoField.push_back(fieldValue->getString(i));
      }
    ok=true;
    }
  }
else if(fieldName=="title")
  {
  if(fieldValue->checkSFString())
    {
    _titleField=fieldValue->getString(0);
    ok=true;
    }
  }
else
  {
  GroupChildNode::addField(fieldName,fieldValue);
  ok=true;
  }
if(!ok)
  {
  _shape->_badFieldMsg(fieldName,_type);
  }
}

void
WorldInfoNode::check(void)
{
_fail=false;
}

//-------- Forbidden operations ----------------------------------------------

void
WorldInfoNode::_generateFields(ostream & output,
                               unsigned int indent,
                               bool showDefaultValues,
                            StlVector<const VrmlNode *> & /* defNodesInOut */)
                                                            const // Protected
{
if(showDefaultValues||
   _titleField.size())
  {
  _generateSFString(output,indent,"title",_titleField);
  }
if(showDefaultValues||
   !_infoField.empty())
  {
  _generateMFString(output,indent,"info",_infoField);
  }
}

WorldInfoNode::WorldInfoNode(const StlString & nodeType,
                             VrmlShape3D * shape) // Protected
: GroupChildNode(nodeType,shape),
  _infoField(),
  _titleField()
{
  // Nothing to be done
}

} // namespace AReVi

//----------------------------------------------------------------------------
