// LI2/ENIB - AReVi - Core services
//----------------------------------------------------------------------------
// file        : AReVi/activity.cpp
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/activity.h"

#include "AReVi/scheduler.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class Activity
//----------------------------------------------------------------------------

AR_CLASS_NOVOID_DEF(Activity,ArObject)

//-------- Construction / Destruction ----------------------------------------

Activity::Activity(ArCW & arCW,
                   double interval) // Protected
: ArObject(arCW),
  _obj(),
  _method((bool (ArObject::*)(ArRef<Activity>,double))0),
  _methodArgClass(),
  _interval(interval<=0.0 ? 0.0
                          : Scheduler::_roundTime(Scheduler::get().c_ptr(),
                                                  interval)),
  _priority(1),
  _lastTime(Scheduler::get()->_time),
  _nextTime(_lastTime),
  _suspended(false),
  _ended(false)
{
if(Scheduler::get()->fail())
  {
  setErrorMessage(Scheduler::get()->getErrorMessage());
  _ended=true;
  return;
  }
Scheduler::access()->_insertActivity(thisRef());
}

Activity::~Activity(void) // Protected
{
abort();
}

//-------- Scheduling properties ---------------------------------------------

double // current time
Activity::getTime(void) const
{
return(Scheduler::get()->getTime());
}

void
Activity::setInterval(double interval)
{
interval=(interval<=0.0 ? 0.0
                        : Scheduler::_roundTime(Scheduler::get().c_ptr(),
                                                interval));
if(interval!=_interval)
  {
  Scheduler::access()->_changeInterval(thisRef(),interval);
  }
}

double // activation interval
Activity::getInterval(void) const
{
return(_interval);
}

void
Activity::setPriority(int priority)
{
if(priority<0)
  {
  priority=-1;
  }
else if(priority>0)
  {
  priority=1;
  }
if(++priority!=_priority)
  {
  Scheduler::access()->_changePriority(thisRef(),priority);
  }
}

int // priority ( -1 0 +1 )
Activity::getPriority(void) const
{
return(_priority-1);
}

//-------- Scheduling control ------------------------------------------------

void
Activity::setSuspended(bool suspended)
{
if(!_ended&&(suspended!=_suspended))
  {
  _suspended=suspended;
  if(_suspended)
    {
    Scheduler::access()->_removeActivity(thisRef());
    }
  else
    {
    _lastTime=Scheduler::get()->getTime();
    Scheduler::access()->_insertActivity(thisRef());
    }
  }
}

bool // suspended
Activity::isSuspended(void) const
{
return(_suspended);
}

void
Activity::abort(void)
{
if(!_ended)
  {
  _ended=true;
  if(!_suspended)
    {
    Scheduler::access()->_removeActivity(thisRef());
    }
  }
}

bool // ended
Activity::isEnded(void) const
{
return(_ended);
}

//-------- Forbidden operations ----------------------------------------------

bool // re-activate
Activity::_activate(double dt)
{
return(_obj ? (_obj.c_ptr()->*_method)(thisRef(),dt) : false);
}

} // namespace AReVi

//----------------------------------------------------------------------------
