// LI2/ENIB - AReVi - Core services
//----------------------------------------------------------------------------
// file        : AReVi/arClass.cpp
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/arClass.h"

#include "AReVi/arMethod.h"
#include "AReVi/arConstant.h"
#include "AReVi/arValue.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class ArClass
//----------------------------------------------------------------------------

AR_CLASS_NOVOID_DEF(ArClass,ArObject)

StlVector<ArClass *> *
ArClass::_toBeManaged=(StlVector<ArClass *> *)0;

StlMap<StlString,ArClass *> *
ArClass::_classMap=(StlMap<StlString,ArClass *> *)0;

#define ARSTUB_INIT(fnct)                                            \
  if(fnct)                                                           \
    {                                                                \
    (*fnct)(thisRef());                                              \
    ar_const_cast(thisRef())->fnct=(void (*)(ArConstRef<ArClass>))0; \
    }

//-------- Construction / Destruction ----------------------------------------

void
ArClass::registerClass(ArCW & wrapper,
                       const StlString & className,
                       const StlString & parentName,
                       ArObject * (*instantiateFnct)(void)) // Static
{
if(!ArClass::_classMap) // FIXME: never destroyed
  {
  ArClass::_classMap=new StlMap<StlString,ArClass *>;
  }
ArConstRef<ArClass> cls;
if(&wrapper!=&ArObject::_CLS)
  {
  cls=ArClass::find(className);
  if(cls)
    {
    cerr << "ArClass(`" << className << "') derived from `" << parentName
         << "' already exists as a derived from `"
         << cls->_super->_className << "' !" << endl;
    exit(-1);
    }
  cls=ArClass::find(parentName);
  if(!cls)
    {
    cerr << "Cannot find ArClass(`" << parentName
         << "') to make derived ArClass(`" << className << "') !" << endl;
    exit(-1);
    }
  }
ArClass::_classMap->insert(StlMap<StlString,ArClass *>::value_type(
                           className,
                           new ArClass(wrapper,className,cls,instantiateFnct)));
}

ArClass::ArClass(ArCW & wrapper,
                 const StlString & className,
                 ArConstRef<ArClass> superClass,
                 ArObject * (*instantiateFnct)(void)) // Private
: ArObject(),
  _className(className),
  _instantiateFnct(instantiateFnct),
  _super(superClass),
  _sub(),
  _count(1),
  _countLoop(false),
  _instances(),
  _initMethodsFnct((void (*)(ArConstRef<ArClass>))0),
  _methods(),
  _initConstantsFnct((void (*)(ArConstRef<ArClass>))0),
  _constants()
{
if(&wrapper==&ArClass::_CLS) // the ``ArClass'' ArClass is coming
  {
  if(ArClass::_toBeManaged)
    {
    while(!ArClass::_toBeManaged->empty())
      {
      ArClass * c=ArClass::_toBeManaged->back();
      _manageObject(c);
      ArClass::_toBeManaged->pop_back();
      }
    delete ArClass::_toBeManaged;
    ArClass::_toBeManaged=(StlVector<ArClass *> *)0;
    }
  }
wrapper=this;
if(ArClass::_CLS) // the ``ArClass'' ArClass exists
  {
  ArClass::_CLS->_manageObject(this);
  }
else // wait until the ``ArClass'' ArClass comes
  {
  if(!ArClass::_toBeManaged)
    {
    ArClass::_toBeManaged=new StlVector<ArClass *>;
    }
  ArClass::_toBeManaged->push_back(this);
  }
if(_super)
  {
  bool done=false;
  for(unsigned int i=0;i<_super->_sub.size();i++)
    {
    if(_super->_sub[i]->_className>_className)
      {
      ar_const_cast(_super)->_sub.insert(ar_const_cast(_super)->_sub.begin()+i,
                                         thisRef());
      done=true;
      break;
      }
    }
  if(!done)
    {
    ar_const_cast(_super)->_sub.push_back(thisRef());
    }
  }
}

ArClass::~ArClass(void)
{
cerr << "Explicit destruction of ArClass(`"
     << _className << "') is forbidden !" << endl;
exit(-1);
}

//-------- Class hierarchy ---------------------------------------------------

const StlString & // class name
ArClass::getName(void) const
{
return(_className);
}

bool // equals to or derived from arClass
ArClass::isA(ArConstRef<ArClass> arClass) const
{
ArConstRef<ArClass> c(this);
do
  {
  if(c==arClass)
    {
    return(true);
    }
  c=c->_super;
  } while(c);
return(false);
}

ArConstRef<ArClass> // parent class or 0
ArClass::getParent(void) const
{
return(_super);
}

void
ArClass::getAncestors(StlVector<ArConstRef<ArClass> > & ancestorsOut) const
{
ancestorsOut.clear();
ArConstRef<ArClass> c=_super;
while(c)
  {
  ancestorsOut.push_back(c);
  c=c->_super;
  }
}

void
ArClass::getChildren(StlVector<ArConstRef<ArClass> > & childrenOut) const
{
childrenOut=_sub;
}

void
ArClass::getDescendants(StlVector<ArConstRef<ArClass> > & descendantsOut) const
{
descendantsOut.clear();
_getDescendants(descendantsOut);
}

ArConstRef<ArClass> // found class or 0
ArClass::find(const StlString & className) // Static
{
StlMap<StlString,ArClass *>::const_iterator it=
    ArClass::_classMap->find(className);
return(it==ArClass::_classMap->end() ? ArClass::nullRef()
                                     : ArConstRef<ArClass>((*it).second));
}

//-------- Instance management -----------------------------------------------

unsigned int
ArClass::getNbInstances(bool subClasses) const
{
unsigned int nb=_instances.size();
if(subClasses)
  {
  for(unsigned int i=_sub.size();i--;)
    {
    nb+=_sub[i]->getNbInstances(true);
    }
  }
return(nb);
}

void
ArClass::getAliases(StlVector<StlString> & aliasesOut,
                    bool subClasses) const
{
aliasesOut.clear();
_getAliases(aliasesOut,subClasses);
}

//-------- Reflection --------------------------------------------------------

unsigned int // number of methods in this class only
ArClass::getNbMethods(void) const
{
ARSTUB_INIT(_initMethodsFnct)
return(_methods.size());
}

ArConstRef<ArMethod> // index'th method in this class only
ArClass::getMethod(unsigned int index) const
{
ARSTUB_INIT(_initMethodsFnct)
return(_methods[index]);
}

ArConstRef<ArMethod> // found constructor-method in this class only
ArClass::findConstructor(const StlVector<ArRef<ArValue> > & args) const
{
ARSTUB_INIT(_initMethodsFnct)
for(unsigned int i=_methods.size();i--;)
  {
  ArConstRef<ArMethod> mth=_methods[i];
  if(mth->isConstructor()&&
     (mth->getNbArgs()==args.size()))
    {
    bool match=true;
    for(unsigned int j=args.size();j--;)
      {
      if(!mth->getArgType(j)->isSameType(args[j]->getType()))
        {
        match=false;
        break;
        }
      }
    if(match)
      {
      return(mth);
      }
    }
  }
return(ArMethod::nullRef());
}

ArConstRef<ArMethod> // found constructor-method in this class only
ArClass::findConstructorWithSignature(const StlString & argSignature) const
{
ARSTUB_INIT(_initMethodsFnct)
for(unsigned int i=_methods.size();i--;)
  {
  ArConstRef<ArMethod> mth=_methods[i];
  if(mth->isConstructor()&&
     mth->matchesArgSignature(argSignature))
    {
    return(mth);
    }
  }
return(ArMethod::nullRef());
}

bool // found constructor-methods in this class only
ArClass::findConstructors(StlVector<ArConstRef<ArMethod> > & methodsOut) const
{
ARSTUB_INIT(_initMethodsFnct)
methodsOut.clear();
for(unsigned int i=_methods.size();i--;)
  {
  ArConstRef<ArMethod> mth=_methods[i];
  if(mth->isConstructor())
    {
    methodsOut.push_back(mth);
    }
  }
return(!methodsOut.empty());
}

ArConstRef<ArMethod> // found static-method in base classes
ArClass::findStaticMethod(const StlString & methodName,
                          const StlVector<ArRef<ArValue> > & args) const
{
ARSTUB_INIT(_initMethodsFnct)
for(unsigned int i=_methods.size();i--;)
  {
  ArConstRef<ArMethod> mth=_methods[i];
  if(mth->isStaticMethod()&&
     (mth->getNbArgs()==args.size())&&
     (mth->getName()==methodName))
    {
    bool match=true;
    for(unsigned int j=args.size();j--;)
      {
      if(!mth->getArgType(j)->isSameType(args[j]->getType()))
        {
        match=false;
        break;
        }
      }
    if(match)
      {
      return(mth);
      }
    }
  }
return(_super ? _super->findStaticMethod(methodName,args)
              : ArMethod::nullRef());
}

ArConstRef<ArMethod> // found static-method in base classes
ArClass::findStaticMethodWithSignature(const StlString & methodName,
                                       const StlString & argSignature) const
{
ARSTUB_INIT(_initMethodsFnct)
for(unsigned int i=_methods.size();i--;)
  {
  ArConstRef<ArMethod> mth=_methods[i];
  if(mth->isStaticMethod()&&
     (mth->getName()==methodName)&&mth->matchesArgSignature(argSignature))
    {
    return(mth);
    }
  }
return(_super ? _super->findStaticMethodWithSignature(methodName,argSignature)
              : ArMethod::nullRef());
}

bool // found static-methods in base classes
ArClass::findStaticMethods(StlVector<ArConstRef<ArMethod> > & methodsOut,
                           const StlString & methodName) const
{
ARSTUB_INIT(_initMethodsFnct)
if(_super)
  {
  _super->findStaticMethods(methodsOut,methodName);
  }
else
  {
  methodsOut.clear();
  }
for(unsigned int i=_methods.size();i--;)
  {
  ArConstRef<ArMethod> mth=_methods[i];
  if(mth->isStaticMethod()&&
     (mth->getName()==methodName))
    {
    methodsOut.push_back(mth);
    for(unsigned int j=methodsOut.size()-1;j--;)
      {
      if(methodsOut[j]->matchesArgSignature(mth->getArgSignature()))
        {
        methodsOut.erase(methodsOut.begin()+j);
        break;
        }
      }
    }
  }
return(!methodsOut.empty());
}

ArConstRef<ArMethod> // found method in base classes
ArClass::findMethod(const StlString & methodName,
                    const StlVector<ArRef<ArValue> > & args) const
{
ARSTUB_INIT(_initMethodsFnct)
for(unsigned int i=_methods.size();i--;)
  {
  ArConstRef<ArMethod> mth=_methods[i];
  if(!mth->isStaticMethod()&&
     (mth->getNbArgs()==args.size())&&
     (mth->getName()==methodName))
    {
    bool match=true;
    for(unsigned int j=args.size();j--;)
      {
      if(!mth->getArgType(j)->isSameType(args[j]->getType()))
        {
        match=false;
        break;
        }
      }
    if(match)
      {
      return(mth);
      }
    }
  }
return(_super ? _super->findMethod(methodName,args)
              : ArMethod::nullRef());
}

ArConstRef<ArMethod> // found method in base classes
ArClass::findMethodWithSignature(const StlString & methodName,
                                 const StlString & argSignature) const
{
ARSTUB_INIT(_initMethodsFnct)
for(unsigned int i=_methods.size();i--;)
  {
  ArConstRef<ArMethod> mth=_methods[i];
  if(!mth->isStaticMethod()&&
     (mth->getName()==methodName)&&mth->matchesArgSignature(argSignature))
    {
    return(mth);
    }
  }
return(_super ? _super->findMethodWithSignature(methodName,argSignature)
              : ArMethod::nullRef());
}

bool // found methods in base classes
ArClass::findMethods(StlVector<ArConstRef<ArMethod> > & methodsOut,
                     const StlString & methodName) const
{
ARSTUB_INIT(_initMethodsFnct)
if(_super)
  {
  _super->findMethods(methodsOut,methodName);
  }
else
  {
  methodsOut.clear();
  }
for(unsigned int i=_methods.size();i--;)
  {
  ArConstRef<ArMethod> mth=_methods[i];
  if(!mth->isStaticMethod()&&
     (mth->getName()==methodName))
    {
    methodsOut.push_back(mth);
    for(unsigned int j=methodsOut.size()-1;j--;)
      {
      if(methodsOut[j]->matchesArgSignature(mth->getArgSignature()))
        {
        methodsOut.erase(methodsOut.begin()+j);
        break;
        }
      }
    }
  }
return(!methodsOut.empty());
}

ArConstRef<ArMethod> // found const-method in base classes
ArClass::findConstMethod(const StlString & methodName,
                         const StlVector<ArRef<ArValue> > & args) const
{
ARSTUB_INIT(_initMethodsFnct)
for(unsigned int i=_methods.size();i--;)
  {
  ArConstRef<ArMethod> mth=_methods[i];
  if(!mth->isStaticMethod()&&mth->isConstMethod()&&
     (mth->getNbArgs()==args.size())&&
     (mth->getName()==methodName))
    {
    bool match=true;
    for(unsigned int j=args.size();j--;)
      {
      if(!mth->getArgType(j)->isSameType(args[j]->getType()))
        {
        match=false;
        break;
        }
      }
    if(match)
      {
      return(mth);
      }
    }
  }
return(_super ? _super->findConstMethod(methodName,args)
              : ArMethod::nullRef());
}

ArConstRef<ArMethod> // found const-method in base classes
ArClass::findConstMethodWithSignature(const StlString & methodName,
                                      const StlString & argSignature) const
{
ARSTUB_INIT(_initMethodsFnct)
for(unsigned int i=_methods.size();i--;)
  {
  ArConstRef<ArMethod> mth=_methods[i];
  if(!mth->isStaticMethod()&&mth->isConstMethod()&&
     (mth->getName()==methodName)&&mth->matchesArgSignature(argSignature))
    {
    return(mth);
    }
  }
return(_super ? _super->findConstMethodWithSignature(methodName,argSignature)
              : ArMethod::nullRef());
}

bool // found const-methods in base classes
ArClass::findConstMethods(StlVector<ArConstRef<ArMethod> > & methodsOut,
                          const StlString & methodName) const
{
ARSTUB_INIT(_initMethodsFnct)
if(_super)
  {
  _super->findConstMethods(methodsOut,methodName);
  }
else
  {
  methodsOut.clear();
  }
for(unsigned int i=_methods.size();i--;)
  {
  ArConstRef<ArMethod> mth=_methods[i];
  if(!mth->isStaticMethod()&&mth->isConstMethod()&&
     (mth->getName()==methodName))
    {
    methodsOut.push_back(mth);
    for(unsigned int j=methodsOut.size()-1;j--;)
      {
      if(methodsOut[j]->matchesArgSignature(mth->getArgSignature()))
        {
        methodsOut.erase(methodsOut.begin()+j);
        break;
        }
      }
    }
  }
return(!methodsOut.empty());
}

unsigned int // number of constants in this class only
ArClass::getNbConstants(void) const
{
ARSTUB_INIT(_initConstantsFnct)
return(_constants.size());
}

ArConstRef<ArConstant> // index'th constant in this class only
ArClass::getConstant(unsigned int index) const
{
ARSTUB_INIT(_initConstantsFnct)
return(_constants[index]);
}

ArConstRef<ArConstant> // found constant in base classes
ArClass::findConstant(const StlString & constantName) const
{
ARSTUB_INIT(_initConstantsFnct)
for(unsigned int i=_constants.size();i--;)
  {
  ArConstRef<ArConstant> cst=_constants[i];
  if(cst->getName()==constantName)
    {
    return(cst);
    }
  }
return(_super ? _super->findConstant(constantName)
              : ArConstant::nullRef());
}

//-------- Forbidden operations ----------------------------------------------

void
ArClass::_manageObject(ArObject * obj) // Private
{
if(_countLoop)
  {
  bool found;
  do
    {
    found=false;
    for(unsigned int i=_instances.size();i--;)
      {
      if(_instances[i]->_id==_count)
        {
        found=true;
        if(!(++_count))
          {
          _count=1;
          }
        break;
        }
      }
    } while(found);
  }
obj->_id=_count++;
if(!_count)
  {
  _countLoop=true;
  _count=1;
  }
obj->_alias=_className;
obj->_alias+=".";
obj->_alias+=uintToStr(obj->_id);
_instances.push_back(obj->thisPtr());
}

void
ArClass::_unmanageObject(ArObject * obj) // Private
{
for(unsigned int i=_instances.size();i--;)
  {
  if(_instances[i].c_ptr()==obj)
    {
    StlVectorFastErase(_instances,i);
    break;
    }
  }
}

void
ArClass::_getDescendants(StlVector<ArConstRef<ArClass> > & descendantsOut)
                                                              const // Private
{
for(unsigned int i=_sub.size();i--;)
  {
  descendantsOut.push_back(_sub[i]);
  _sub[i]->_getDescendants(descendantsOut);
  }
}

void
ArClass::_getInstances(StlVector<ArRef<ArObject> > & instancesOut,
                       bool subClasses) const // Private
{
unsigned int i;
for(i=_instances.size();i--;)
  {
  instancesOut.push_back(_instances[i]);
  }
if(subClasses)
  {
  for(i=_sub.size();i--;)
    {
    _sub[i]->_getInstances(instancesOut,true);
    }
  }
}

void
ArClass::_getAliases(StlVector<StlString> & aliasesOut,
                     bool subClasses) const // Private
{
unsigned int i;
for(i=_instances.size();i--;)
  {
  aliasesOut.push_back(_instances[i]->getAlias());
  }
if(subClasses)
  {
  for(unsigned int i=_sub.size();i--;)
    {
    _sub[i]->_getAliases(aliasesOut,true);
    }
  }
}

bool // success
ArClass::_findAlias(ArConstRef<ArClass> cls,
                    const StlString & alias,
                    ArRef<ArObject> & instanceOut) const // Private
{
if(isA(cls))
  {
  unsigned int i;
  for(i=_instances.size();i--;)
    {
    if(_instances[i]->_alias==alias)
      {
      instanceOut=_instances[i];
      return(true);
      }
    }
  for(i=_sub.size();i--;)
    {
    if(_sub[i]->_findAlias(cls,alias,instanceOut))
      {
      return(true);
      }
    }
  }
return(false);
}

} // namespace AReVi

//----------------------------------------------------------------------------
