// LI2/ENIB - AReVi - Core services
//----------------------------------------------------------------------------
// file        : AReVi/arStubHelper_priv.cpp
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/arStubHelper_priv.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class ArStubHelper
//----------------------------------------------------------------------------

void
ArStubHelper::initCls(const char * clsName,
                      void (*mthFnct)(ArConstRef<ArClass>),
                      void (*cstFnct)(ArConstRef<ArClass>))
{
ArConstRef<ArClass> cls=ArClass::find(clsName);
if(cls)
  {
  ar_const_cast(cls)->_initMethodsFnct=mthFnct;
  ar_const_cast(cls)->_initConstantsFnct=cstFnct;
  }
else
  {
  cerr << "Stub for "
       << clsName << ": class not found !" << endl;
  }
}

void
ArStubHelper::constant(ArConstRef<ArClass> cls,
                       const char * declFile,
                       unsigned int declLine,
                       const char * cstName,
                       ArConstRef<ArValue> v)
{
new ArConstant(ArConstant::_CLS,
               cls,declFile,declLine,cstName,v);
}

void
ArStubHelper::method(ArMethod * & m,
                     ArConstRef<ArClass> cls,
                     const char * declFile,
                     unsigned int declLine,
                     const char * mthName,
                     bool isStatic,
                     bool isConst,
                     ArMethod::Stub stub)
{
m=new ArMethod(ArMethod::_CLS,
               cls,declFile,declLine,
               mthName,isStatic,isConst,
               stub);
}

void
ArStubHelper::result(ArMethod * & m,
                     ArType::DirectType dType,
                     unsigned int dimension,
                     ArType::PassingMode pMode)
{
if(m)
  {
  m->_setResult(dType,dimension,pMode);
  }
}

void
ArStubHelper::result(ArMethod * & m,
                     ArType::IndirectType iType,
                     const char * clsName,
                     unsigned int dimension,
                     ArType::PassingMode pMode)
{
if(m)
  {
  ArConstRef<ArClass> cls=ArClass::find(clsName);
  if(cls)
    {
    m->_setResult(iType,cls,dimension,pMode);
    }
  else
    {
    cerr << "Stub for "
         << m->getDeclarationClass()->getName()
         << "::" << m->getName() << "(): class `"
         << clsName << "' not found !" << endl;
    m->thisRef().destroy(); // UGLY !
    m=(ArMethod *)0;
    }
  }
}

void
ArStubHelper::arg(ArMethod * & m,
                  const char * name,
                  ArType::DirectType dType,
                  unsigned int dimension,
                  ArType::PassingMode pMode)
{
if(m)
  {
  m->_addArg(name,dType,dimension,pMode);
  }
}

void
ArStubHelper::arg(ArMethod * & m,
                  const char * name,
                  ArType::IndirectType iType,
                  const char * clsName,
                  unsigned int dimension,
                  ArType::PassingMode pMode)
{
if(m)
  {
  ArConstRef<ArClass> cls=ArClass::find(clsName);
  if(cls)
    {
    m->_addArg(name,iType,cls,dimension,pMode);
    }
  else
    {
    cerr << "Stub for "
         << m->getDeclarationClass()->getName()
         << "::" << m->getName() << "(): class `"
         << clsName << "' not found !" << endl;
    m->thisRef().destroy(); // UGLY !
    m=(ArMethod *)0;
    }
  }
}

} // namespace AReVi

//----------------------------------------------------------------------------
