// LI2/ENIB - AReVi - Core services
//----------------------------------------------------------------------------
// file        : AReVi/arType.cpp
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/arType.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class ArType
//----------------------------------------------------------------------------

AR_CLASS_NOVOID_DEF(ArType,ArObject)

//-------- Construction / Destruction ----------------------------------------

ArType::ArType(ArCW & arCW,
               ArType::DirectType type,
               unsigned int dimension,
               ArType::PassingMode passing) // Private
: ArObject(arCW),
  _isDirect(true),
  _type(type),
  _indirectClass(),
  _dimension(dimension),
  _passingMode(passing),
  _cppType()
{
switch(_type)
  {
  case ArType::DT_BOOL:
    {
    _cppType="bool";
    break;
    }
  case ArType::DT_CHAR:
    {
    _cppType="char";
    break;
    }
  case ArType::DT_UCHAR:
    {
    _cppType="unsigned char";
    break;
    }
  case ArType::DT_SHORT:
    {
    _cppType="short";
    break;
    }
  case ArType::DT_USHORT:
    {
    _cppType="unsigned short";
    break;
    }
  case ArType::DT_INT:
    {
    _cppType="int";
    break;
    }
  case ArType::DT_UINT:
    {
    _cppType="unsigned int";
    break;
    }
  case ArType::DT_LLONG:
    {
    _cppType="long long";
    break;
    }
  case ArType::DT_ULLONG:
    {
    _cppType="unsigned long long";
    break;
    }
  case ArType::DT_FLOAT:
    {
    _cppType="float";
    break;
    }
  case ArType::DT_DOUBLE:
    {
    _cppType="double";
    break;
    }
  case ArType::DT_STRING:
    {
    _cppType="StlString";
    break;
    }
  default:
    {
    _cppType="void"; // should not happen
    break;
    }
  }
bool addSpace=false;
for(unsigned int i=_dimension;i--;)
  {
  _cppType="StlVector<"+_cppType+(addSpace ? " >" : ">");
  addSpace=true;
  }
switch(_passingMode)
  {
  case ArType::PM_COPY:
    {
    break;
    }
  case ArType::PM_REF:
    {
    _cppType+=" &";
    break;
    }
  case ArType::PM_CONST_REF:
    {
    _cppType="const "+_cppType+" &";
    break;
    }
  }
}

ArType::ArType(ArCW & arCW,
               ArType::IndirectType type,
               ArConstRef<ArClass> cls,
               unsigned int dimension,
               ArType::PassingMode passing) // Private
: ArObject(arCW),
  _isDirect(false),
  _type(type),
  _indirectClass(cls),
  _dimension(dimension),
  _passingMode(passing)
{
switch(_type)
  {
  case ArType::IT_REF:
    {
    _cppType="ArRef<";
    break;
    }
  case ArType::IT_CONST_REF:
    {
    _cppType="ArConstRef<";
    break;
    }
  case ArType::IT_PTR:
    {
    _cppType="ArPtr<";
    break;
    }
  case ArType::IT_CONST_PTR:
    {
    _cppType="ArConstPtr<";
    break;
    }
  default:
    {
    _cppType="<"; // should not happen
    break;
    }
  }
_cppType+=_indirectClass->getName();
_cppType+=">";
for(unsigned int i=_dimension;i--;)
  {
  _cppType="StlVector<"+_cppType+" >";
  }
switch(_passingMode)
  {
  case ArType::PM_COPY:
    {
    break;
    }
  case ArType::PM_REF:
    {
    _cppType+=" &";
    break;
    }
  case ArType::PM_CONST_REF:
    {
    _cppType="const "+_cppType+" &";
    break;
    }
  }
}

ArType::~ArType(void) // Private
{
  // Nothing to be done
}

} // namespace AReVi

//----------------------------------------------------------------------------
