// LI2/ENIB - AReVi - Core services
//----------------------------------------------------------------------------
// file        : AReVi/arValue.cpp
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReVi/arValue.h"

namespace AReVi {

//----------------------------------------------------------------------------
// Class ArValue
//----------------------------------------------------------------------------

AR_CLASS_NOVOID_DEF(ArValue,ArObject)

//-------- Construction / Destruction ----------------------------------------

ArValue::ArValue(ArCW & arCW,
                 ArConstRef<ArType> type) // Protected
: ArObject(arCW),
  _t(type),
  _v((void *)0)
{
// FIXME: optimize tests
if(isBool())
  {
  _v=new bool(0);
  }
else if(isBoolV())
  {
  _v=new StlVector<bool>();
  }
else if(isBoolVV())
  {
  _v=new StlVector<StlVector<bool> >();
  }
else if(isChar())
  {
  _v=new char(0);
  }
else if(isCharV())
  {
  _v=new StlVector<char>();
  }
else if(isCharVV())
  {
  _v=new StlVector<StlVector<char> >();
  }
else if(isUChar())
  {
  _v=new unsigned char(0);
  }
else if(isUCharV())
  {
  _v=new StlVector<unsigned char>();
  }
else if(isUCharVV())
  {
  _v=new StlVector<StlVector<unsigned char> >();
  }
else if(isShort())
  {
  _v=new short(0);
  }
else if(isShortV())
  {
  _v=new StlVector<short>();
  }
else if(isShortVV())
  {
  _v=new StlVector<StlVector<short> >();
  }
else if(isUShort())
  {
  _v=new unsigned short(0);
  }
else if(isUShortV())
  {
  _v=new StlVector<unsigned short>();
  }
else if(isUShortVV())
  {
  _v=new StlVector<StlVector<unsigned short> >();
  }
else if(isInt())
  {
  _v=new int(0);
  }
else if(isIntV())
  {
  _v=new StlVector<int>();
  }
else if(isIntVV())
  {
  _v=new StlVector<StlVector<int> >();
  }
else if(isUInt())
  {
  _v=new unsigned int(0);
  }
else if(isUIntV())
  {
  _v=new StlVector<unsigned int>();
  }
else if(isUIntVV())
  {
  _v=new StlVector<StlVector<unsigned int> >();
  }
else if(isLLong())
  {
  _v=new long long(0);
  }
else if(isLLongV())
  {
  _v=new StlVector<long long>();
  }
else if(isLLongVV())
  {
  _v=new StlVector<StlVector<long long> >();
  }
else if(isULLong())
  {
  _v=new unsigned long long(0);
  }
else if(isULLongV())
  {
  _v=new StlVector<unsigned long long>();
  }
else if(isULLongVV())
  {
  _v=new StlVector<StlVector<unsigned long long> >();
  }
else if(isFloat())
  {
  _v=new float(0.0);
  }
else if(isFloatV())
  {
  _v=new StlVector<float>();
  }
else if(isFloatVV())
  {
  _v=new StlVector<StlVector<float> >();
  }
else if(isDouble())
  {
  _v=new double(0.0);
  }
else if(isDoubleV())
  {
  _v=new StlVector<double>();
  }
else if(isDoubleVV())
  {
  _v=new StlVector<StlVector<double> >();
  }
else if(isString())
  {
  _v=new StlString();
  }
else if(isStringV())
  {
  _v=new StlVector<StlString>();
  }
else if(isStringVV())
  {
  _v=new StlVector<StlVector<StlString> >();
  }
else if(isRef())
  {
  _v=new ArRef<ArObject>();
  }
else if(isRefV())
  {
  _v=new StlVector<ArRef<ArObject> >();
  }
else if(isRefVV())
  {
  _v=new StlVector<StlVector<ArRef<ArObject> > >();
  }
else if(isConstRef())
  {
  _v=new ArConstRef<ArObject>();
  }
else if(isConstRefV())
  {
  _v=new StlVector<ArConstRef<ArObject> >();
  }
else if(isConstRefVV())
  {
  _v=new StlVector<StlVector<ArConstRef<ArObject> > >();
  }
else if(isPtr())
  {
  _v=new ArPtr<ArObject>();
  }
else if(isPtrV())
  {
  _v=new StlVector<ArPtr<ArObject> >();
  }
else if(isPtrVV())
  {
  _v=new StlVector<StlVector<ArPtr<ArObject> > >();
  }
else if(isConstPtr())
  {
  _v=new ArConstPtr<ArObject>();
  }
else if(isConstPtrV())
  {
  _v=new StlVector<ArConstPtr<ArObject> >();
  }
else if(isConstPtrVV())
  {
  _v=new StlVector<StlVector<ArConstPtr<ArObject> > >();
  }
}

ArValue::ArValue(ArCW & arCW,
                 ArType::DirectType type,
                 unsigned int dimension) // Protected
: ArObject(arCW),
  _t(new ArType(ArType::_CLS,type,dimension,ArType::PM_COPY)),
  _v((void *)0)
{
ar_const_cast(_t)->setTransient(true);
// FIXME: optimize tests
if(isBool())
  {
  _v=new bool(0);
  }
else if(isBoolV())
  {
  _v=new StlVector<bool>();
  }
else if(isBoolVV())
  {
  _v=new StlVector<StlVector<bool> >();
  }
else if(isChar())
  {
  _v=new char(0);
  }
else if(isCharV())
  {
  _v=new StlVector<char>();
  }
else if(isCharVV())
  {
  _v=new StlVector<StlVector<char> >();
  }
else if(isUChar())
  {
  _v=new unsigned char(0);
  }
else if(isUCharV())
  {
  _v=new StlVector<unsigned char>();
  }
else if(isUCharVV())
  {
  _v=new StlVector<StlVector<unsigned char> >();
  }
else if(isShort())
  {
  _v=new short(0);
  }
else if(isShortV())
  {
  _v=new StlVector<short>();
  }
else if(isShortVV())
  {
  _v=new StlVector<StlVector<short> >();
  }
else if(isUShort())
  {
  _v=new unsigned short(0);
  }
else if(isUShortV())
  {
  _v=new StlVector<unsigned short>();
  }
else if(isUShortVV())
  {
  _v=new StlVector<StlVector<unsigned short> >();
  }
else if(isInt())
  {
  _v=new int(0);
  }
else if(isIntV())
  {
  _v=new StlVector<int>();
  }
else if(isIntVV())
  {
  _v=new StlVector<StlVector<int> >();
  }
else if(isUInt())
  {
  _v=new unsigned int(0);
  }
else if(isUIntV())
  {
  _v=new StlVector<unsigned int>();
  }
else if(isUIntVV())
  {
  _v=new StlVector<StlVector<unsigned int> >();
  }
else if(isLLong())
  {
  _v=new long long(0);
  }
else if(isLLongV())
  {
  _v=new StlVector<long long>();
  }
else if(isLLongVV())
  {
  _v=new StlVector<StlVector<long long> >();
  }
else if(isULLong())
  {
  _v=new unsigned long long(0);
  }
else if(isULLongV())
  {
  _v=new StlVector<unsigned long long>();
  }
else if(isULLongVV())
  {
  _v=new StlVector<StlVector<unsigned long long> >();
  }
else if(isFloat())
  {
  _v=new float(0.0);
  }
else if(isFloatV())
  {
  _v=new StlVector<float>();
  }
else if(isFloatVV())
  {
  _v=new StlVector<StlVector<float> >();
  }
else if(isDouble())
  {
  _v=new double(0.0);
  }
else if(isDoubleV())
  {
  _v=new StlVector<double>();
  }
else if(isDoubleVV())
  {
  _v=new StlVector<StlVector<double> >();
  }
else if(isString())
  {
  _v=new StlString();
  }
else if(isStringV())
  {
  _v=new StlVector<StlString>();
  }
else if(isStringVV())
  {
  _v=new StlVector<StlVector<StlString> >();
  }
}

ArValue::ArValue(ArCW & arCW,
                 ArType::IndirectType type,
                 ArConstRef<ArClass> cls,
                 unsigned int dimension) // Protected
: ArObject(arCW),
  _t(new ArType(ArType::_CLS,type,cls,dimension,ArType::PM_COPY)),
  _v((void *)0)
{
ar_const_cast(_t)->setTransient(true);
// FIXME: optimize tests
if(isRef())
  {
  _v=new ArRef<ArObject>();
  }
else if(isRefV())
  {
  _v=new StlVector<ArRef<ArObject> >();
  }
else if(isRefVV())
  {
  _v=new StlVector<StlVector<ArRef<ArObject> > >();
  }
else if(isConstRef())
  {
  _v=new ArConstRef<ArObject>();
  }
else if(isConstRefV())
  {
  _v=new StlVector<ArConstRef<ArObject> >();
  }
else if(isConstRefVV())
  {
  _v=new StlVector<StlVector<ArConstRef<ArObject> > >();
  }
else if(isPtr())
  {
  _v=new ArPtr<ArObject>();
  }
else if(isPtrV())
  {
  _v=new StlVector<ArPtr<ArObject> >();
  }
else if(isPtrVV())
  {
  _v=new StlVector<StlVector<ArPtr<ArObject> > >();
  }
else if(isConstPtr())
  {
  _v=new ArConstPtr<ArObject>();
  }
else if(isConstPtrV())
  {
  _v=new StlVector<ArConstPtr<ArObject> >();
  }
else if(isConstPtrVV())
  {
  _v=new StlVector<StlVector<ArConstPtr<ArObject> > >();
  }
}

ArValue::~ArValue(void) // Protected
{
// FIXME: optimize tests
if(isBool())
  {
  delete (bool *)_v;
  }
else if(isBoolV())
  {
  delete (StlVector<bool> *)_v;
  }
else if(isBoolVV())
  {
  delete (StlVector<StlVector<bool> > *)_v;
  }
else if(isChar())
  {
  delete (char *)_v;
  }
else if(isCharV())
  {
  delete (StlVector<char> *)_v;
  }
else if(isCharVV())
  {
  delete (StlVector<StlVector<char> > *)_v;
  }
else if(isUChar())
  {
  delete (unsigned char *)_v;
  }
else if(isUCharV())
  {
  delete (StlVector<unsigned char> *)_v;
  }
else if(isUCharVV())
  {
  delete (StlVector<StlVector<unsigned char> > *)_v;
  }
else if(isShort())
  {
  delete (short *)_v;
  }
else if(isShortV())
  {
  delete (StlVector<short> *)_v;
  }
else if(isShortVV())
  {
  delete (StlVector<StlVector<short> > *)_v;
  }
else if(isUShort())
  {
  delete (unsigned short *)_v;
  }
else if(isUShortV())
  {
  delete (StlVector<unsigned short> *)_v;
  }
else if(isUShortVV())
  {
  delete (StlVector<StlVector<unsigned short> > *)_v;
  }
else if(isInt())
  {
  delete (int *)_v;
  }
else if(isIntV())
  {
  delete (StlVector<int> *)_v;
  }
else if(isIntVV())
  {
  delete (StlVector<StlVector<int> > *)_v;
  }
else if(isUInt())
  {
  delete (unsigned int *)_v;
  }
else if(isUIntV())
  {
  delete (StlVector<unsigned int> *)_v;
  }
else if(isUIntVV())
  {
  delete (StlVector<StlVector<unsigned int> > *)_v;
  }
else if(isLLong())
  {
  delete (long long *)_v;
  }
else if(isLLongV())
  {
  delete (StlVector<long long> *)_v;
  }
else if(isLLongVV())
  {
  delete (StlVector<StlVector<long long> > *)_v;
  }
else if(isULLong())
  {
  delete (unsigned long long *)_v;
  }
else if(isULLongV())
  {
  delete (StlVector<unsigned long long> *)_v;
  }
else if(isULLongVV())
  {
  delete (StlVector<StlVector<unsigned long long> > *)_v;
  }
else if(isFloat())
  {
  delete (float *)_v;
  }
else if(isFloatV())
  {
  delete (StlVector<float> *)_v;
  }
else if(isFloatVV())
  {
  delete (StlVector<StlVector<float> > *)_v;
  }
else if(isDouble())
  {
  delete (double *)_v;
  }
else if(isDoubleV())
  {
  delete (StlVector<double> *)_v;
  }
else if(isDoubleVV())
  {
  delete (StlVector<StlVector<double> > *)_v;
  }
else if(isString())
  {
  delete (StlString *)_v;
  }
else if(isStringV())
  {
  delete (StlVector<StlString> *)_v;
  }
else if(isStringVV())
  {
  delete (StlVector<StlVector<StlString> > *)_v;
  }
else if(isRef())
  {
  delete (ArRef<ArObject> *)_v;
  }
else if(isRefV())
  {
  delete (StlVector<ArRef<ArObject> > *)_v;
  }
else if(isRefVV())
  {
  delete (StlVector<StlVector<ArRef<ArObject> > > *)_v;
  }
else if(isConstRef())
  {
  delete (ArConstRef<ArObject> *)_v;
  }
else if(isConstRefV())
  {
  delete (StlVector<ArConstRef<ArObject> > *)_v;
  }
else if(isConstRefVV())
  {
  delete (StlVector<StlVector<ArConstRef<ArObject> > > *)_v;
  }
else if(isPtr())
  {
  delete (ArPtr<ArObject> *)_v;
  }
else if(isPtrV())
  {
  delete (StlVector<ArPtr<ArObject> > *)_v;
  }
else if(isPtrVV())
  {
  delete (StlVector<StlVector<ArPtr<ArObject> > > *)_v;
  }
else if(isConstPtr())
  {
  delete (ArConstPtr<ArObject> *)_v;
  }
else if(isConstPtrV())
  {
  delete (StlVector<ArConstPtr<ArObject> > *)_v;
  }
else if(isConstPtrVV())
  {
  delete (StlVector<StlVector<ArConstPtr<ArObject> > > *)_v;
  }
}

} // namespace AReVi

//----------------------------------------------------------------------------
