// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/Controlers/nsControler.cpp
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReViNS/Controlers/nsControler.h"
#include "AReViNS/Events/nsEvent.h"
#include "AReViNS/localDepService.h"
#include "AReViNS/remoteDepService.h"

namespace AReVi {
//----------------------------------------------------------------------------
// CLASS       : NSListener
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSListener,ArObject)
//---------------------- Constructors / Destructor ---------------------------
NSListener::NSListener(ArCW & arCW) // Protected
  : ArObject(arCW),_object(),_depService()
{
  // Nothing to be done
}

NSListener::~NSListener(void)
{
  // Nothing to be done
}
//---------------------------- Public methods --------------------------------
// Controler will sign to all object events and will be able to transmit
// them. (getPendingEvents());
void
NSListener::init(ArRef<LocalDepService> depService, ArRef<ArObject> object)
{
  _depService=depService;
  _object=object;
}

//------------------------ Controler is Watcher ------------------------------
// Return events backup (eg : motion, setup change ...)
void
NSListener::getPendingEvents(StlList<ArRef<NSEvent> > &/*events*/)
{
  // Nothing to be done
}

//----------------------------------------------------------------------------
// CLASS       : NSInteractor
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSInteractor,ArObject)
//---------------------- Constructors / Destructor ---------------------------
NSInteractor::NSInteractor(ArCW & arCW) // Protected
  : ArObject(arCW),
    _eventsTable(),
    _object(),
    _depService(),
    _waitingForEvent(),
    _ready(false)
{
  // Nothing to be done
}

NSInteractor::~NSInteractor(void)
{
  // Nothing to be done
}
//---------------------------- Public methods --------------------------------

// Controler will sign to all object network events and will be able to
// affect remote object with.
void
NSInteractor::init(ArRef<RemoteDepService> depService, ArRef<ArObject> object)
{
  _object=object;
  _depService=depService;
}

// Return events backup (eg : motion, setup change ...)
NSEvent::EventProcess
NSInteractor::fireEvent(ArConstRef<NSEvent> event)
{
  if (!_ready)
  {
    if (_depService.valid() && _object.valid())
      if (find(_depService->getWaitingRefs().begin(),
               _depService->getWaitingRefs().end(),
               _object->getAlias())==_depService->getWaitingRefs().end())
         _ready=true; // DepService is no more waiting ref=>so ref is valid

    if (!_ready)
      return NSEvent::EP_OK; // Ignore all events until Interactor is ready
  }
  else if (!_object.valid())
    return NSEvent::EP_FAIL;

  if (_waitingForEvent.valid())
  {
    if (_waitingForEvent!=event)
      return NSEvent::EP_POST; // Interactor is waiting for an event post all overs
    else
      _waitingForEvent.clear();
  }

  StlMap<StlString, NetworkService::EventMethod>::iterator it=_eventsTable.find(event->getClass()->getName());
  //cerr << getAlias() << "=>NSInteractor::fireEvent event=" << event->getAlias() << endl;
  if (it==_eventsTable.end())
  {
    setErrorMessage("No callback mapped to controler event of type '"+event->getClass()->getName()+"'");
    return NSEvent::EP_FAIL;
  }
  else
  {
    NetworkService::EventMethod method=it->second;
    NSEvent::EventProcess ret((thisRef().c_ptr()->*method)(event));

    if (ret!=NSEvent::EP_LATER)
    {
      return ret;
    }
    _waitingForEvent=event;
    //cerr << "WAITING FOR EVENT " << _waitingForEvent->getAlias() << endl;
    return NSEvent::EP_POST;
  }
  return NSEvent::EP_OK;
}

//------------------------- Forbiden operations ------------------------------
// Register and unregister events
bool // virtual
NSInteractor::_registerEvent(const StlString &eventClass,
                            NetworkService::EventMethod method)
{
  if (_eventsTable.find(eventClass)!=_eventsTable.end())
    return false;

  //dcerr(getAlias() + " register to "+eventClass);

  _eventsTable[eventClass]=method;
  return true;
}

// unregister events
bool // virtual
NSInteractor::_unregisterEvent(const StlString &eventClass)
{
  if (_eventsTable.find(eventClass)==_eventsTable.end())
    return false;
  _eventsTable.erase(_eventsTable.find(eventClass));
  return true;
}

} // namespace AReVi
//---End-Of-File---------------------------------------------------------------
