// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/Controlers/nsCtrlArConsole.cpp
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReViNS/Controlers/nsCtrlArConsole.h"
#include "AReViNS/Events/nsEventArConsole.h"
//#include "AReViNS/remoteDepService.h"

namespace AReVi {
//----------------------------------------------------------------------------
// CLASS       : NSListenerArConsole
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSListenerArConsole,NSListenerOSDText)
//---------------------- Constructors / Destructor ---------------------------
NSListenerArConsole::NSListenerArConsole(ArCW & arCW) // Protected
  : NSListenerOSDText(arCW),
    _arConsole(),
    _events(),
    _backgroundFlag(false)
{
  // Nothing to do
}

NSListenerArConsole::~NSListenerArConsole(void)
{
  // Nothing to do
}

//---------------------------- Public methods --------------------------------
// Controler will sign to all object events and will be able to transmit
// them. (getPendingEvents());
void
NSListenerArConsole::init(ArRef<LocalDepService> service, ArRef<ArObject> object)
{
if (object->getClass()->isA(ArConsole::CLASS()))
  _arConsole=ar_down_cast<ArConsole>(object);
else
  return;

NSListenerOSDText::init(service,object);
_arConsole->addBackgroundChangedCB(thisRef(),&NSListenerArConsole::_onBackgroundChangedCB);
_arConsole->addBufferSetupCB(thisRef(),&NSListenerArConsole::_onBufferSetupCB);
_arConsole->addClearTextCB(thisRef(),&NSListenerArConsole::_onClearTextCB);
_arConsole->addScrollTextCB(thisRef(),&NSListenerArConsole::_onScrollTextCB);
_arConsole->addForegroundChangedCB(thisRef(),&NSListenerArConsole::_onForegroundChangedCB);
_arConsole->addPushTextCB(thisRef(),&NSListenerArConsole::_onPushTextCB);
_arConsole->addPushAttribCB(thisRef(),&NSListenerArConsole::_onPushAttribCB);
_arConsole->addPopAttribCB(thisRef(),&NSListenerArConsole::_onPopAttribCB);
_arConsole->addFlushAttribsCB(thisRef(),&NSListenerArConsole::_onFlushAttribCB);
_arConsole->addStoreCharacterCB(thisRef(),&NSListenerArConsole::_onStoreCharacterCB);
}

//------------------------ Controler is Watcher ------------------------------
// Return events backup (eg : motion, setup change ...)
void
NSListenerArConsole::getPendingEvents(StlList<ArRef<NSEvent> > &events)
{
if(_backgroundFlag)
  {
  ArRef<NSEventArConsoleBackground> event=NSEventArConsoleBackground::NEW();
  double r,g,b,a;
  _arConsole->getBackgroundColor(r,g,b,a);
  event->writeColor(r,g,b,a);
  events.push_back(event);
  _backgroundFlag=false;
  }

StlList<ArRef<NSEvent> >::iterator it=_events.begin();
StlList<ArRef<NSEvent> >::iterator it_end=_events.end();
while(it!=it_end)
  {
  events.push_back(*it);
  it++;
  }
_events.clear();
}

//------------------------- Forbiden operations ------------------------------
// Background changed (setBackground).
void
NSListenerArConsole::_onBackgroundChangedCB(const ArConsole::BackgroundChangedEvent &/*evt*/)
{
_backgroundFlag=true;
}

// Foreground changed (setColor, set Font).
void
NSListenerArConsole::_onForegroundChangedCB(const ArConsole::ForegroundChangedEvent &/*evt*/)
{
ArRef<NSEventArConsoleForeground> event=NSEventArConsoleForeground::NEW();
double r,g,b,a;
_arConsole->getColor(r,g,b,a);
event->writeColor(r,g,b,a);
event->writeFont(_arConsole->getFont());
_events.push_back(event);
}

// Text Scrolled (scroll)
void
NSListenerArConsole::_onScrollTextCB(const ArConsole::ScrollTextEvent &evt)
{
ArRef<NSEventArConsoleScrollText> event=NSEventArConsoleScrollText::NEW();
event->setNbLines(evt.nbLines);
_events.push_back(event);
}

// Buffer setup(setSize, setBufferSize, clear)
void
NSListenerArConsole::_onBufferSetupCB(const ArConsole::BufferSetupEvent &/*evt*/)
{
ArRef<NSEventArConsoleBufferSetup> event=NSEventArConsoleBufferSetup::NEW();
event->writeArConsole(_arConsole);
_events.push_back(event);
}

// Push Text (push)
void
NSListenerArConsole::_onPushTextCB(const ArConsole::PushTextEvent &evt)
{
ArRef<NSEventArConsolePushText> event=NSEventArConsolePushText::NEW();
event->setText(evt.text);
_events.push_back(event);
}


// Store character (storeCharacter)
void
NSListenerArConsole::_onStoreCharacterCB(const ArConsole::StoreCharacterEvent &evt)
{
ArRef<NSEventArConsoleStoreCharacter> event=NSEventArConsoleStoreCharacter::NEW();
event->setChar(evt.c);
_events.push_back(event);

}

// Clear  Text (clear )
void
NSListenerArConsole::_onClearTextCB(const ArConsole::ClearTextEvent &/*evt*/)
{
ArRef<NSEventArConsoleClearText> event=NSEventArConsoleClearText::NEW();
_events.push_back(event);
}


// Push attrib (pushAttrib)
void
NSListenerArConsole::_onPushAttribCB(const ArConsole::PushAttribEvent &evt)
{
ArRef<NSEventArConsoleAttrib> event=NSEventArConsoleAttrib::NEW();
event->writeAttrib(evt.attr);
_events.push_back(event);
}

// Pop attrib (popAttrib)
void
NSListenerArConsole::_onPopAttribCB(const ArConsole::PopAttribEvent &)
{
ArRef<NSEventArConsolePopAttrib> event=NSEventArConsolePopAttrib::NEW();
_events.push_back(event);
}

// Flush attrib (flushAttrib)
void
NSListenerArConsole::_onFlushAttribCB(const ArConsole::FlushAttribsEvent &)
{
ArRef<NSEventArConsoleFlushAttribs> event=NSEventArConsoleFlushAttribs::NEW();
_events.push_back(event);
}

//----------------------------------------------------------------------------
// CLASS       : NSInteractorArConsole
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSInteractorArConsole,NSInteractor)
//---------------------- Constructors / Destructor ---------------------------
NSInteractorArConsole::NSInteractorArConsole(ArCW & arCW) // Protected
  : NSInteractorOSDText(arCW),_arConsole()
{}

NSInteractorArConsole::~NSInteractorArConsole(void)
{
}
//---------------------------- Public methods --------------------------------

// Controler will sign to all object network events and will be able to
// affect remote object with.
void
NSInteractorArConsole::init(ArRef<RemoteDepService> service, ArRef<ArObject> object)
{
  if (object->getClass()->isA(ArConsole::CLASS()))
    _arConsole=ar_down_cast<ArConsole>(object);
  else
    return;

  NSInteractorOSDText::init(service,object);

  _registerEvent(NSEventArConsoleBackground::CLASS()->getName(),
                (NetworkService::EventMethod)&NSInteractorArConsole::_onBackgroundChangedEvent);
  _registerEvent(NSEventArConsoleBufferSetup::CLASS()->getName(),
                (NetworkService::EventMethod)&NSInteractorArConsole::_onBufferSetupEvent);
  _registerEvent(NSEventArConsoleClearText::CLASS()->getName(),
                (NetworkService::EventMethod)&NSInteractorArConsole::_onClearTextEvent);
  _registerEvent(NSEventArConsoleScrollText::CLASS()->getName(),
                (NetworkService::EventMethod)&NSInteractorArConsole::_onScrollTextEvent);
  _registerEvent(NSEventArConsoleForeground::CLASS()->getName(),
                (NetworkService::EventMethod)&NSInteractorArConsole::_onForegroundChangedEvent);
  _registerEvent(NSEventArConsoleStoreCharacter::CLASS()->getName(),
                (NetworkService::EventMethod)&NSInteractorArConsole::_onStoreCharacterEvent);
  _registerEvent(NSEventArConsolePushText::CLASS()->getName(),
                (NetworkService::EventMethod)&NSInteractorArConsole::_onPushTextEvent);
  _registerEvent(NSEventArConsoleAttrib::CLASS()->getName(),
                (NetworkService::EventMethod)&NSInteractorArConsole::_onPushAttribEvent);
  _registerEvent(NSEventArConsolePopAttrib::CLASS()->getName(),
                (NetworkService::EventMethod)&NSInteractorArConsole::_onPopAttribEvent);
  _registerEvent(NSEventArConsoleFlushAttribs::CLASS()->getName(),
                (NetworkService::EventMethod)&NSInteractorArConsole::_onFlushAttribEvent);

}

//------------------------- Forbiden operations ------------------------------

//! Background changed (setBackground).
NSEvent::EventProcess
NSInteractorArConsole::_onBackgroundChangedEvent(ArConstRef<NSEvent> eventObj)
{
ArConstRef<NSEventArConsoleBackground> event=ar_down_cast<NSEventArConsoleBackground>(eventObj);
ArConsoleAttr attrib;

event->readAttrib(attrib);
_arConsole->setBackgroundColor(attrib.r,attrib.g,attrib.b,attrib.a);
return NSEvent::EP_OK;
}

//! Foreground changed (setColor, set Font).
NSEvent::EventProcess
NSInteractorArConsole::_onForegroundChangedEvent(ArConstRef<NSEvent> eventObj)
{
ArConstRef<NSEventArConsoleForeground> event=ar_down_cast<NSEventArConsoleForeground>(eventObj);
ArConsoleAttr attrib;

event->readAttrib(attrib);
_arConsole->setColor(attrib.r,attrib.g,attrib.b,attrib.a);
_arConsole->setFont(attrib.font);
return NSEvent::EP_OK;
}

//! Text Scrolled (scroll)
NSEvent::EventProcess
NSInteractorArConsole::_onScrollTextEvent(ArConstRef<NSEvent> eventObj)
{
ArConstRef<NSEventArConsoleScrollText> event=ar_down_cast<NSEventArConsoleScrollText>(eventObj);
event->readArConsole(_arConsole);
return NSEvent::EP_OK;
}

//! Buffer setup(setSize, setBufferSize, clear)
NSEvent::EventProcess
NSInteractorArConsole::_onBufferSetupEvent(ArConstRef<NSEvent> eventObj)
{
ArConstRef<NSEventArConsoleBufferSetup> event=ar_down_cast<NSEventArConsoleBufferSetup>(eventObj);
event->readArConsole(_arConsole);
return NSEvent::EP_OK;
}

//! Push Text (push)
NSEvent::EventProcess
NSInteractorArConsole::_onPushTextEvent(ArConstRef<NSEvent> eventObj)
{
ArConstRef<NSEventArConsolePushText> event=ar_down_cast<NSEventArConsolePushText>(eventObj);
event->readArConsole(_arConsole);
return NSEvent::EP_OK;
}

//! Store character (storeCharacter)
NSEvent::EventProcess
NSInteractorArConsole::_onStoreCharacterEvent(ArConstRef<NSEvent> eventObj)
{
ArConstRef<NSEventArConsoleStoreCharacter> event=ar_down_cast<NSEventArConsoleStoreCharacter>(eventObj);
event->readArConsole(_arConsole);
return NSEvent::EP_OK;
}

//! clear Text (clear)
NSEvent::EventProcess
NSInteractorArConsole::_onClearTextEvent(ArConstRef<NSEvent>)
{
_arConsole->clear();
return NSEvent::EP_OK;
}

//! Push attrib (pushAttrib)
NSEvent::EventProcess
NSInteractorArConsole::_onPushAttribEvent(ArConstRef<NSEvent> eventObj)
{
ArConstRef<NSEventArConsoleAttrib> event=ar_down_cast<NSEventArConsoleAttrib>(eventObj);
ArConsoleAttr attrib;
event->readAttrib(attrib);
_arConsole->pushAttrib(attrib);
return NSEvent::EP_OK;
}

//! Pop attrib (popAttrib)
NSEvent::EventProcess
NSInteractorArConsole::_onPopAttribEvent(ArConstRef<NSEvent>)
{
_arConsole->popAttrib();
return NSEvent::EP_OK;
}

//! Flush attrib (flushAttrib)
NSEvent::EventProcess
NSInteractorArConsole::_onFlushAttribEvent(ArConstRef<NSEvent>)
{
_arConsole->flushAttrib();
return NSEvent::EP_OK;
}


} // namespace NetworkServices
//---End-Of-File---------------------------------------------------------------
