// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/Controlers/nsCtrlBase3D.cpp
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReViNS/Controlers/nsCtrlBase3D.h"
#include "AReViNS/Events/nsEventBase3D.h"

#include "AReVi/Lib3D/base3D.h"

namespace AReVi {
//----------------------------------------------------------------------------
// CLASS       : NSListenerBase3D
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSListenerBase3D,NSListener)
//---------------------- Constructors / Destructor ---------------------------
NSListenerBase3D::NSListenerBase3D(ArCW & arCW) // Protected
  : NSListener(arCW),_base(),_motionFlag(false)
{}

NSListenerBase3D::~NSListenerBase3D(void)
{
}
//---------------------------- Public methods --------------------------------
// Controler will sign to all object events and will be able to transmit
// them. (getPendingEvents());
void
NSListenerBase3D::init(ArRef<LocalDepService> service, ArRef<ArObject> object)
{
  if (object->getClass()->isA(Base3D::CLASS()))
    _base=ar_down_cast<Base3D>(object);
  else
    return;

  NSListener::init(service,object);
  _base->addMotionCB(thisRef(),&NSListenerBase3D::_onMotionCB);
}

//------------------------ Controler is Watcher ------------------------------
// Return events backup (eg : motion, setup change ...)
void
NSListenerBase3D::getPendingEvents(StlList<ArRef<NSEvent> > &events)
{
  if (_motionFlag)
  {
  //  cerr << "NSListenerBase3D::getPendingEvents _motionFlag=" << _motionFlag <<endl;
    ArRef<NSEventBase3DMotion> event=NSEventBase3DMotion::NEW();
    event->setBase(_base->getBase());
    events.push_back(event);
    _motionFlag=false;
  }
}

//------------------------- Forbiden operations ------------------------------
void
NSListenerBase3D::_onMotionCB(const Base3D::MotionEvent &)
{
  _motionFlag=true;
}

//----------------------------------------------------------------------------
// CLASS       : NSInteractorBase3D
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSInteractorBase3D,NSInteractor)
//---------------------- Constructors / Destructor ---------------------------
NSInteractorBase3D::NSInteractorBase3D(ArCW & arCW) // Protected
  : NSInteractor(arCW),_base()
{}

NSInteractorBase3D::~NSInteractorBase3D(void)
{}
//---------------------------- Public methods --------------------------------

// Controler will sign to all object network events and will be able to
// affect remote object with.
void
NSInteractorBase3D::init(ArRef<RemoteDepService> service, ArRef<ArObject> object)
{
  if (object->getClass()->isA(Base3D::CLASS()))
    _base=ar_down_cast<Base3D>(object);
  else
    return;

  NSInteractor::init(service,object);

  _registerEvent(NSEventBase3DMotion::CLASS()->getName(),
                (NetworkService::EventMethod)&NSInteractorBase3D::_onMotionEvent);

}

//------------------------- Forbiden operations ------------------------------
NSEvent::EventProcess
NSInteractorBase3D::_onMotionEvent(ArConstRef<NSEvent> eventObj)
{
  ArConstRef<NSEventBase3DMotion> event=ar_down_cast<NSEventBase3DMotion>(eventObj);

  if (_base.valid())
  {
    _base->accessBase()=event->getBase();
    _base->maskMotion();
    _base->unmaskMotion();
  }
  else
    cerr << "OBJ => no _base "<< endl;
  return NSEvent::EP_OK;
}


} // namespace NetworkServices

//---End-Of-File---------------------------------------------------------------
