// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/Controlers/nsCtrlCylinder3D.cpp
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReViNS/Controlers/nsCtrlCylinder3D.h"
#include "AReViNS/Events/nsEventCylinder3D.h"
#include "AReVi/Shapes/cylinder3D.h"

namespace AReVi {
//----------------------------------------------------------------------------
// CLASS       : NSListenerCylinder3D
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSListenerCylinder3D,NSListenerSurface3D)
//---------------------- Constructors / Destructor ---------------------------
NSListenerCylinder3D::NSListenerCylinder3D(ArCW & arCW) // Protected
  : NSListenerSurface3D(arCW),_cylinder()
{}

NSListenerCylinder3D::~NSListenerCylinder3D(void)
{}
//---------------------------- Public methods --------------------------------
// Controler will sign to all object events and will be able to transmit
// them. (getPendingEvents());
void
NSListenerCylinder3D::init(ArRef<LocalDepService> service, ArRef<ArObject> object)
{
  if (object->getClass()->isA(Cylinder3D::CLASS()))
    _cylinder=ar_down_cast<Cylinder3D>(object);
  else
    return;

  NSListenerSurface3D::init(service,object);
}

//------------------------ Controler is Watcher ------------------------------
// Return events backup (eg : motion, setup change ...)
void
NSListenerCylinder3D::getPendingEvents(StlList<ArRef<NSEvent> > &events)
{
  if (_geometryRebuildFlag)
  {
    ArRef<NSEventCylinder3DGeometryRebuild> event=NSEventCylinder3DGeometryRebuild::NEW();
    event->writeGeometry(_cylinder);
    events.push_back(event);
    _geometryRebuildFlag=false;
  }

  NSListenerSurface3D::getPendingEvents(events);
}

//----------------------------------------------------------------------------
// CLASS       : NSInteractorCylinder3D
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSInteractorCylinder3D,NSInteractorSurface3D)
//---------------------- Constructors / Destructor ---------------------------
NSInteractorCylinder3D::NSInteractorCylinder3D(ArCW & arCW) // Protected
  : NSInteractorSurface3D(arCW),_cylinder()
{}

NSInteractorCylinder3D::~NSInteractorCylinder3D(void)
{}
//---------------------------- Public methods --------------------------------
// Controler will sign to all object network events and will be able to
// affect remote object with.
void
NSInteractorCylinder3D::init(ArRef<RemoteDepService> service, ArRef<ArObject> object)
{
  if (object->getClass()->isA(Cylinder3D::CLASS()))
    _cylinder=ar_down_cast<Cylinder3D>(object);
  else
    return;

  NSInteractorSurface3D::init(service,object);
  _unregisterEvent(NSEventShapePart3DGeometryRebuild::CLASS()->getName());

  _registerEvent(NSEventCylinder3DGeometryRebuild::CLASS()->getName(),
                (NetworkService::EventMethod)&NSInteractorCylinder3D::_onGeometryRebuildEvent);
}

//------------------------- Forbiden operations ------------------------------
NSEvent::EventProcess
NSInteractorCylinder3D::_onGeometryRebuildEvent(ArConstRef<NSEvent> eventObj)
{
  NSInteractorSurface3D::_onGeometryRebuildEvent(eventObj);
  ArConstRef<NSEventCylinder3DGeometryRebuild> event=ar_down_cast<NSEventCylinder3DGeometryRebuild>(eventObj);
  event->readGeometry(_cylinder);
  return NSEvent::EP_OK;
}

} // namespace NetworkServices
//---End-Of-File---------------------------------------------------------------
