// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/Controlers/nsCtrlDataSound3D.cpp
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReViNS/Controlers/nsCtrlDataSound3D.h"
#include "AReViNS/Events/nsEventDataSound3D.h"

#include "AReVi/Lib3D/dataSound3D.h"
#include "AReVi/Utils/abstractStream.h"

namespace AReVi {
//----------------------------------------------------------------------------
// CLASS       : NSListenerDataSound3D
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSListenerDataSound3D,NSListenerSound3D)
//---------------------- Constructors / Destructor ---------------------------
NSListenerDataSound3D::NSListenerDataSound3D(ArCW & arCW) // Protected
  : NSListenerSound3D(arCW),
    _dataSnd()
{}

NSListenerDataSound3D::~NSListenerDataSound3D(void)
{
}
//---------------------------- Public methods --------------------------------
// Controler will sign to all object events and will be able to transmit
// them. (getPendingEvents());
void
NSListenerDataSound3D::init(ArRef<LocalDepService> service, ArRef<ArObject> object)
{
  if (object->getClass()->isA(DataSound3D::CLASS()))
    _dataSnd=ar_down_cast<DataSound3D>(object);
  else
    return;

  NSListenerSound3D::init(service,object);
  _dataSnd->addPushBufferCB(thisRef(),&NSListenerDataSound3D::_onPushBufferCB);
  _dataSnd->addPopBufferCB(thisRef(),&NSListenerDataSound3D::_onPopBufferCB);
}

//------------------------ Controler is Watcher ------------------------------
// Return events backup (eg : motion, setup change ...)
void
NSListenerDataSound3D::getPendingEvents(StlList<ArRef<NSEvent> > &events)
{
  NSListenerSound3D::getPendingEvents(events);
/*
  if (_popBufferFlag)
  {
  //  cerr << "NSEventDataSound3DPushBuffer::getPendingEvents" << endl;
     ArRef<NSEventDataSound3DPopBuffer> event=NSEventDataSound3DPopBuffer::NEW();
    events.push_back(event);
    _popBufferFlag=false;
  }*/
}

//------------------------- Forbiden operations ------------------------------
void
NSListenerDataSound3D::_onPushBufferCB(const DataSound3D::PushBufferEvent &evt)
{
ArRef<NSEventDataSound3DPushBuffer> event=NSEventDataSound3DPushBuffer::NEW();
event->writeDataSound3D(evt.frequency,evt.format,evt.buffer);
_events.push_back(event);
}

void
NSListenerDataSound3D::_onPopBufferCB(const DataSound3D::PopBufferEvent &)
{
  //_popBufferFlag=true;
  ArRef<NSEventDataSound3DPopBuffer> event=NSEventDataSound3DPopBuffer::NEW();
  _events.push_back(event);
}

//----------------------------------------------------------------------------
// CLASS       : NSInteractorDataSound3D
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSInteractorDataSound3D,NSInteractorSound3D)
//---------------------- Constructors / Destructor ---------------------------
NSInteractorDataSound3D::NSInteractorDataSound3D(ArCW & arCW) // Protected
  : NSInteractorSound3D(arCW),
    _dataSnd()
{}

NSInteractorDataSound3D::~NSInteractorDataSound3D(void)
{}
//---------------------------- Public methods --------------------------------

// Controler will sign to all object network events and will be able to
// affect remote object with.
void
NSInteractorDataSound3D::init(ArRef<RemoteDepService> service, ArRef<ArObject> object)
{
  if (object->getClass()->isA(DataSound3D::CLASS()))
    _dataSnd=ar_down_cast<DataSound3D>(object);
  else
  {
    cerr << "Object " << object->getAlias() << " not a DataSound3D CLASS" << endl;
    return;
  }
  NSInteractorSound3D::init(service,object);

  _registerEvent(NSEventDataSound3DPushBuffer::CLASS()->getName(),
                (NetworkService::EventMethod)&NSInteractorDataSound3D::_onPushBufferEvent);
  _registerEvent(NSEventDataSound3DPopBuffer::CLASS()->getName(),
                (NetworkService::EventMethod)&NSInteractorDataSound3D::_onPopBufferEvent);

}

//------------------------- Forbiden operations ------------------------------
NSEvent::EventProcess
NSInteractorDataSound3D::_onPushBufferEvent(ArConstRef<NSEvent> eventObj)
{
  ArConstRef<NSEventDataSound3DPushBuffer> event=ar_down_cast<NSEventDataSound3DPushBuffer>(eventObj);

  if (_dataSnd.valid())
    event->readDataSound3D(_dataSnd);

  return NSEvent::EP_OK;
}

NSEvent::EventProcess
NSInteractorDataSound3D::_onPopBufferEvent(ArConstRef<NSEvent> eventObj)
{
  ArConstRef<NSEventDataSound3DPopBuffer> event=ar_down_cast<NSEventDataSound3DPopBuffer>(eventObj);

  if (_dataSnd.valid())
    _dataSnd->popBuffers();

  return NSEvent::EP_OK;
}
} // namespace NetworkServices

//---End-Of-File---------------------------------------------------------------
