// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/Controlers/nsCtrlDataTexture.cpp
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReViNS/Controlers/nsCtrlDataTexture.h"
#include "AReViNS/Events/nsEventDataTexture.h"

namespace AReVi {
//----------------------------------------------------------------------------
// CLASS       : NSListenerDataTexture
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSListenerDataTexture,NSListener)
//---------------------- Constructors / Destructor ---------------------------
NSListenerDataTexture::NSListenerDataTexture(ArCW & arCW) // Protected
  : NSListener(arCW),
    _dataTexture(),
    _reloadFlag(false),
    _setupFlag(false)
{}

NSListenerDataTexture::~NSListenerDataTexture(void)
{
}
//---------------------------- Public methods --------------------------------
// Controler will sign to all object events and will be able to transmit
// them. (getPendingEvents());
void
NSListenerDataTexture::init(ArRef<LocalDepService> service, ArRef<ArObject> object)
{
  if (object->getClass()->isA(DataTexture::CLASS()))
    _dataTexture=ar_down_cast<DataTexture>(object);
  else
    return;

  NSListener::init(service,object);
  _dataTexture->addReloadCB(thisRef(),&NSListenerDataTexture::_onReloadCB);
  _dataTexture->addSetupCB(thisRef(),&NSListenerDataTexture::_onSetupCB);
}

//------------------------ Controler is Watcher ------------------------------
// Return events backup (eg : motion, setup change ...)
void
NSListenerDataTexture::getPendingEvents(StlList<ArRef<NSEvent> > &events)
{
  if (_reloadFlag)
  {
    ArRef<NSEventDataTextureReload> event=NSEventDataTextureReload::NEW();
    event->writeDataTexture(_dataTexture);
    events.push_back(event);
    _reloadFlag=false;
  }
  if (_setupFlag)
  {
    ArRef<NSEventDataTextureSetup> event=NSEventDataTextureSetup::NEW();
    event->writeDataTexture(_dataTexture);
    events.push_back(event);
    _setupFlag=false;
  }
}

//------------------------- Forbiden operations ------------------------------
void
NSListenerDataTexture::_onReloadCB(const DataTexture::ReloadEvent &)
{
  _reloadFlag=true;
}

void
NSListenerDataTexture::_onSetupCB(const DataTexture::SetupEvent &)
{
  _setupFlag=true;
}


//----------------------------------------------------------------------------
// CLASS       : NSInteractorDataTexture
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSInteractorDataTexture,NSInteractor)
//---------------------- Constructors / Destructor ---------------------------
NSInteractorDataTexture::NSInteractorDataTexture(ArCW & arCW) // Protected
  : NSInteractor(arCW),
    _dataTexture()
{}

NSInteractorDataTexture::~NSInteractorDataTexture(void)
{}
//---------------------------- Public methods --------------------------------

// Controler will sign to all object network events and will be able to
// affect remote object with.
void
NSInteractorDataTexture::init(ArRef<RemoteDepService> service, ArRef<ArObject> object)
{
  if (object->getClass()->isA(DataTexture::CLASS()))
    _dataTexture=ar_down_cast<DataTexture>(object);
  else
    return;

  NSInteractor::init(service,object);

  _registerEvent(NSEventDataTextureReload::CLASS()->getName(),
                (NetworkService::EventMethod)&NSInteractorDataTexture::_onReloadEvent);
  _registerEvent(NSEventDataTextureSetup::CLASS()->getName(),
                (NetworkService::EventMethod)&NSInteractorDataTexture::_onSetupEvent);
}

//------------------------- Forbiden operations ------------------------------
NSEvent::EventProcess
NSInteractorDataTexture::_onReloadEvent(ArConstRef<NSEvent> eventObj)
{
  ArConstRef<NSEventDataTextureReload> event=ar_down_cast<NSEventDataTextureReload>(eventObj);

  if (_dataTexture.valid())
  {
    event->readDataTexture(_dataTexture);
  }
  return NSEvent::EP_OK;
}

NSEvent::EventProcess
NSInteractorDataTexture::_onSetupEvent(ArConstRef<NSEvent> eventObj)
{
  ArConstRef<NSEventDataTextureSetup> event=ar_down_cast<NSEventDataTextureSetup>(eventObj);

  if (_dataTexture.valid())
    event->readDataTexture(_dataTexture);

  return NSEvent::EP_OK;
}


} // namespace NetworkServices

//---End-Of-File---------------------------------------------------------------
