// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/Controlers/nsCtrlLight3D.cpp
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReViNS/Controlers/nsCtrlLight3D.h"
#include "AReViNS/Events/nsEventLight3D.h"

#include "AReVi/Utils/abstractStream.h"



namespace AReVi {
//----------------------------------------------------------------------------
// CLASS       : NSListenerLight3D
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSListenerLight3D,NSListenerBase3D)
//---------------------- Constructors / Destructor ---------------------------
NSListenerLight3D::NSListenerLight3D(ArCW & arCW) // Protected
  : NSListenerBase3D(arCW),_light(),_setupFlag(false)
{}

NSListenerLight3D::~NSListenerLight3D(void)
{
}
//---------------------------- Public methods --------------------------------
// Controler will sign to all object events and will be able to transmit
// them. (getPendingEvents());
void
NSListenerLight3D::init(ArRef<LocalDepService> service, ArRef<ArObject> object)
{
  if (object->getClass()->isA(Light3D::CLASS()))
    _light=ar_down_cast<Light3D>(object);
  else
    return;

  NSListenerBase3D::init(service,object);
  _light->addSetupCB(thisRef(),&NSListenerLight3D::_onSetupCB);
}

//------------------------ Controler is Watcher ------------------------------
// Return events backup (eg : motion, setup change ...)
void
NSListenerLight3D::getPendingEvents(StlList<ArRef<NSEvent> > &events)
{
  NSListenerBase3D::getPendingEvents(events);

  if (_setupFlag)
  {
  //  cerr << "NSListenerLight3D::getPendingEvents _motionFlag=" << _motionFlag <<endl;
    ArRef<NSEventLight3DSetup> event=NSEventLight3DSetup::NEW();
    event->writeLight3DSetup(_light);
    events.push_back(event);
    _setupFlag=false;
  }
}

//------------------------- Forbiden operations ------------------------------
void
NSListenerLight3D::_onSetupCB(const Light3D::SetupEvent &)
{
  _setupFlag=true;
}

//----------------------------------------------------------------------------
// CLASS       : NSInteractorLight3D
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSInteractorLight3D,NSInteractorBase3D)
//---------------------- Constructors / Destructor ---------------------------
NSInteractorLight3D::NSInteractorLight3D(ArCW & arCW) // Protected
  : NSInteractorBase3D(arCW),_light()
{}

NSInteractorLight3D::~NSInteractorLight3D(void)
{}
//---------------------------- Public methods --------------------------------

// Controler will sign to all object network events and will be able to
// affect remote object with.
void
NSInteractorLight3D::init(ArRef<RemoteDepService> service, ArRef<ArObject> object)
{
  if (object->getClass()->isA(Light3D::CLASS()))
    _light=ar_down_cast<Light3D>(object);
  else
    return;

  NSInteractorBase3D::init(service,object);

  _registerEvent(NSEventLight3DSetup::CLASS()->getName(),
                (NetworkService::EventMethod)&NSInteractorLight3D::_onSetupEvent);
}

//------------------------- Forbiden operations ------------------------------
NSEvent::EventProcess
NSInteractorLight3D::_onSetupEvent(ArConstRef<NSEvent> eventObj)
{
  ArConstRef<NSEventLight3DSetup> event=ar_down_cast<NSEventLight3DSetup>(eventObj);

  if (_light.valid())
    event->readLight3DSetup(_light);
  return NSEvent::EP_OK;
}
} // namespace NetworkServices

//---End-Of-File---------------------------------------------------------------
