// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/Controlers/nsCtrlListener3D.cpp
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReViNS/Controlers/nsCtrlListener3D.h"
#include "AReViNS/Events/nsEventListener3D.h"

#include "AReViNS/remoteDepService.h"
#include "AReVi/Lib3D/sound3D.h"
#include "AReVi/Utils/abstractStream.h"



namespace AReVi {
//----------------------------------------------------------------------------
// CLASS       : NSListenerListener3D
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSListenerListener3D,NSListenerBase3D)
//---------------------- Constructors / Destructor ---------------------------
NSListenerListener3D::NSListenerListener3D(ArCW & arCW) // Protected
  : NSListenerBase3D(arCW),_listener(),_setupFlag(false)
{}

NSListenerListener3D::~NSListenerListener3D(void)
{}
//---------------------------- Public methods --------------------------------
// Controler will sign to all object events and will be able to transmit
// them. (getPendingEvents());
void
NSListenerListener3D::init(ArRef<LocalDepService> service, ArRef<ArObject> object)
{
  if (object->getClass()->isA(Listener3D::CLASS()))
    _listener=ar_down_cast<Listener3D>(object);
  else
    return;

  NSListenerBase3D::init(service,object);
  _listener->addSetupCB(thisRef(),&NSListenerListener3D::_onSetupCB);
}

//------------------------ Controler is Watcher ------------------------------
// Return events backup (eg : motion, setup change ...)
void
NSListenerListener3D::getPendingEvents(StlList<ArRef<NSEvent> > &events)
{
  NSListenerBase3D::getPendingEvents(events);

  if (_setupFlag)
  {
  //  cerr << "NSListenerListener3D::getPendingEvents _motionFlag=" << _motionFlag <<endl;
    ArRef<NSEventListener3DSetup> event=NSEventListener3DSetup::NEW();
    event->writeListener3D(_listener);
    events.push_back(event);
    _setupFlag=false;
  }
}

//------------------------- Forbiden operations ------------------------------
void
NSListenerListener3D::_onSetupCB(const Listener3D::SetupEvent &)
{
  _setupFlag=true;
}

//----------------------------------------------------------------------------
// CLASS       : NSInteractorListener3D
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSInteractorListener3D,NSInteractorBase3D)
//---------------------- Constructors / Destructor ---------------------------
NSInteractorListener3D::NSInteractorListener3D(ArCW & arCW) // Protected
  : NSInteractorBase3D(arCW),_listener()
{}

NSInteractorListener3D::~NSInteractorListener3D(void)
{}
//---------------------------- Public methods --------------------------------

// Controler will sign to all object network events and will be able to
// affect remote object with.
void
NSInteractorListener3D::init(ArRef<RemoteDepService> service, ArRef<ArObject> object)
{
  if (object->getClass()->isA(Listener3D::CLASS()))
    _listener=ar_down_cast<Listener3D>(object);
  else
  {
    cerr << "Object " << object->getAlias() << " not a listener3D CLASS" << endl;
    return;
  }
  NSInteractorBase3D::init(service,object);

  _registerEvent(NSEventListener3DSetup::CLASS()->getName(),
                (NetworkService::EventMethod)&NSInteractorListener3D::_onSetupEvent);
}

//------------------------- Forbiden operations ------------------------------
NSEvent::EventProcess
NSInteractorListener3D::_onSetupEvent(ArConstRef<NSEvent> eventObj)
{
  ArConstRef<NSEventListener3DSetup> event=ar_down_cast<NSEventListener3DSetup>(eventObj);

  if (_listener.valid())
  {
    if (!event->readListener3D(_listener,_depService))
      return NSEvent::EP_FAIL;
  }
  return NSEvent::EP_OK;
}

} // namespace NetworkServices

//---End-Of-File---------------------------------------------------------------
