// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/Controlers/nsCtrlMesh3D.cpp
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReViNS/Controlers/nsCtrlMesh3D.h"
#include "AReViNS/Events/nsEventMesh3D.h"
#include "AReVi/Shapes/mesh3D.h"

namespace AReVi {
//----------------------------------------------------------------------------
// CLASS       : NSListenerMesh3D
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSListenerMesh3D,NSListenerSurface3D)
//---------------------- Constructors / Destructor ---------------------------
NSListenerMesh3D::NSListenerMesh3D(ArCW & arCW) // Protected
  : NSListenerSurface3D(arCW),_mesh()
{}

NSListenerMesh3D::~NSListenerMesh3D(void)
{}
//---------------------------- Public methods --------------------------------
// Controler will sign to all object events and will be able to transmit
// them. (getPendingEvents());
void
NSListenerMesh3D::init(ArRef<LocalDepService> service, ArRef<ArObject> object)
{
  if (object->getClass()->isA(Mesh3D::CLASS()))
    _mesh=ar_down_cast<Mesh3D>(object);
  else
    return;

  NSListenerSurface3D::init(service,object);
}

//------------------------ Controler is Watcher ------------------------------
// Return events backup (eg : motion, setup change ...)
void
NSListenerMesh3D::getPendingEvents(StlList<ArRef<NSEvent> > &events)
{
  if (_geometryRebuildFlag)
  {
    ArRef<NSEventMesh3DGeometryRebuild> event=NSEventMesh3DGeometryRebuild::NEW();
    event->writeGeometry(_mesh);
    events.push_back(event);
    _geometryRebuildFlag=false;
  }

  NSListenerSurface3D::getPendingEvents(events);
}

//----------------------------------------------------------------------------
// CLASS       : NSInteractorMesh3D
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSInteractorMesh3D,NSInteractorSurface3D)
//---------------------- Constructors / Destructor ---------------------------
NSInteractorMesh3D::NSInteractorMesh3D(ArCW & arCW) // Protected
  : NSInteractorSurface3D(arCW),_mesh()
{}

NSInteractorMesh3D::~NSInteractorMesh3D(void)
{}
//---------------------------- Public methods --------------------------------
// Controler will sign to all object network events and will be able to
// affect remote object with.
void
NSInteractorMesh3D::init(ArRef<RemoteDepService> service, ArRef<ArObject> object)
{
  if (object->getClass()->isA(Mesh3D::CLASS()))
    _mesh=ar_down_cast<Mesh3D>(object);
  else
    return;

  NSInteractorSurface3D::init(service,object);
  _unregisterEvent(NSEventShapePart3DGeometryRebuild::CLASS()->getName());

  _registerEvent(NSEventMesh3DGeometryRebuild::CLASS()->getName(),
                (NetworkService::EventMethod)&NSInteractorMesh3D::_onGeometryRebuildEvent);
}

//------------------------- Forbiden operations ------------------------------
NSEvent::EventProcess
NSInteractorMesh3D::_onGeometryRebuildEvent(ArConstRef<NSEvent> eventObj)
{
  NSInteractorSurface3D::_onGeometryRebuildEvent(eventObj);
  ArConstRef<NSEventMesh3DGeometryRebuild> event=ar_down_cast<NSEventMesh3DGeometryRebuild>(eventObj);
  event->readGeometry(_mesh);
  return NSEvent::EP_OK;
}

} // namespace NetworkServices
//---End-Of-File---------------------------------------------------------------
