// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/Controlers/nsCtrlMetaball.cpp
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReViNS/Controlers/nsCtrlMetaball.h"
#include "AReViNS/Events/nsEventMetaball.h"
#include "AReVi/Shapes/metaball.h"

namespace AReVi {
//----------------------------------------------------------------------------
// CLASS       : NSListenerMetaball
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSListenerMetaball,NSListenerSurface3D)
//---------------------- Constructors / Destructor ---------------------------
NSListenerMetaball::NSListenerMetaball(ArCW & arCW) // Protected
  : NSListenerSurface3D(arCW),_metaball()
{}

NSListenerMetaball::~NSListenerMetaball(void)
{}
//---------------------------- Public methods --------------------------------
// Controler will sign to all object events and will be able to transmit
// them. (getPendingEvents());
void
NSListenerMetaball::init(ArRef<LocalDepService> service, ArRef<ArObject> object)
{
  if (object->getClass()->isA(Metaball::CLASS()))
    _metaball=ar_down_cast<Metaball>(object);
  else
    return;

  NSListenerSurface3D::init(service,object);
}

//------------------------ Controler is Watcher ------------------------------
// Return events backup (eg : motion, setup change ...)
void
NSListenerMetaball::getPendingEvents(StlList<ArRef<NSEvent> > &events)
{
  if (_geometryRebuildFlag)
  {
    ArRef<NSEventMetaballGeometryRebuild> event=NSEventMetaballGeometryRebuild::NEW();
    event->writeGeometry(_metaball);
    events.push_back(event);
    _geometryRebuildFlag=false;
  }

  NSListenerSurface3D::getPendingEvents(events);
}

//----------------------------------------------------------------------------
// CLASS       : NSInteractorMetaball
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSInteractorMetaball,NSInteractorSurface3D)
//---------------------- Constructors / Destructor ---------------------------
NSInteractorMetaball::NSInteractorMetaball(ArCW & arCW) // Protected
  : NSInteractorSurface3D(arCW),_metaball()
{}

NSInteractorMetaball::~NSInteractorMetaball(void)
{}
//---------------------------- Public methods --------------------------------
// Controler will sign to all object network events and will be able to
// affect remote object with.
void
NSInteractorMetaball::init(ArRef<RemoteDepService> service, ArRef<ArObject> object)
{
  if (object->getClass()->isA(Metaball::CLASS()))
    _metaball=ar_down_cast<Metaball>(object);
  else
    return;

  NSInteractorSurface3D::init(service,object);
  _unregisterEvent(NSEventShapePart3DGeometryRebuild::CLASS()->getName());

  _registerEvent(NSEventMetaballGeometryRebuild::CLASS()->getName(),
                (NetworkService::EventMethod)&NSInteractorMetaball::_onGeometryRebuildEvent);
}

//------------------------- Forbiden operations ------------------------------
NSEvent::EventProcess
NSInteractorMetaball::_onGeometryRebuildEvent(ArConstRef<NSEvent> eventObj)
{
  NSInteractorSurface3D::_onGeometryRebuildEvent(eventObj);
  ArConstRef<NSEventMetaballGeometryRebuild> event=ar_down_cast<NSEventMetaballGeometryRebuild>(eventObj);
  event->readGeometry(_metaball);
  return NSEvent::EP_OK;
}

} // namespace NetworkServices
//---End-Of-File---------------------------------------------------------------
