// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/Controlers/nsCtrlNurbsCurve3D.cpp
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReViNS/Controlers/nsCtrlNurbsCurve3D.h"
#include "AReViNS/Events/nsEventNurbsCurve3D.h"
#include "AReVi/Contrib/nurbsCurve3D.h"

namespace AReVi {
//----------------------------------------------------------------------------
// CLASS       : NSListenerNurbsCurve3D
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSListenerNurbsCurve3D,NSListenerShapePart3D)
//---------------------- Constructors / Destructor ---------------------------
NSListenerNurbsCurve3D::NSListenerNurbsCurve3D(ArCW & arCW) // Protected
  : NSListenerShapePart3D(arCW),_nurbsCurve()
{}

NSListenerNurbsCurve3D::~NSListenerNurbsCurve3D(void)
{}
//---------------------------- Public methods --------------------------------
// Controler will sign to all object events and will be able to transmit
// them. (getPendingEvents());
void
NSListenerNurbsCurve3D::init(ArRef<LocalDepService> service, ArRef<ArObject> object)
{
  if (object->getClass()->isA(NurbsCurve3D::CLASS()))
    _nurbsCurve=ar_down_cast<NurbsCurve3D>(object);
  else
    return;

  NSListenerShapePart3D::init(service,object);
}

//------------------------ Controler is Watcher ------------------------------
// Return events backup (eg : motion, setup change ...)
void
NSListenerNurbsCurve3D::getPendingEvents(StlList<ArRef<NSEvent> > &events)
{
  if (_geometryRebuildFlag)
  {
    ArRef<NSEventNurbsCurve3DGeometryRebuild> event=NSEventNurbsCurve3DGeometryRebuild::NEW();
    event->writeGeometry(_nurbsCurve);
    events.push_back(event);
    _geometryRebuildFlag=false;
  }

  NSListenerShapePart3D::getPendingEvents(events);
}

//----------------------------------------------------------------------------
// CLASS       : NSInteractorNurbsCurve3D
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSInteractorNurbsCurve3D,NSInteractorShapePart3D)
//---------------------- Constructors / Destructor ---------------------------
NSInteractorNurbsCurve3D::NSInteractorNurbsCurve3D(ArCW & arCW) // Protected
  : NSInteractorShapePart3D(arCW),_nurbsCurve()
{}

NSInteractorNurbsCurve3D::~NSInteractorNurbsCurve3D(void)
{}
//---------------------------- Public methods --------------------------------
// Controler will sign to all object network events and will be able to
// affect remote object with.
void
NSInteractorNurbsCurve3D::init(ArRef<RemoteDepService> service, ArRef<ArObject> object)
{
  if (object->getClass()->isA(NurbsCurve3D::CLASS()))
    _nurbsCurve=ar_down_cast<NurbsCurve3D>(object);
  else
    return;

  NSInteractorShapePart3D::init(service,object);
  _unregisterEvent(NSEventShapePart3DGeometryRebuild::CLASS()->getName());

  _registerEvent(NSEventNurbsCurve3DGeometryRebuild::CLASS()->getName(),
                (NetworkService::EventMethod)&NSInteractorNurbsCurve3D::_onGeometryRebuildEvent);
}

//------------------------- Forbiden operations ------------------------------
NSEvent::EventProcess
NSInteractorNurbsCurve3D::_onGeometryRebuildEvent(ArConstRef<NSEvent> eventObj)
{
  NSInteractorShapePart3D::_onGeometryRebuildEvent(eventObj);
  ArConstRef<NSEventNurbsCurve3DGeometryRebuild> event=ar_down_cast<NSEventNurbsCurve3DGeometryRebuild>(eventObj);
  if (event->readGeometry(_nurbsCurve))
    return NSEvent::EP_OK;
  else
    return NSEvent::EP_FAIL;
}

} // namespace NetworkServices
//---End-Of-File---------------------------------------------------------------
