// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/Controlers/nsCtrlObject3D.cpp
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReViNS/Controlers/nsCtrlObject3D.h"
#include "AReViNS/Events/nsEventObject3D.h"
#include "AReViNS/remoteDepService.h"

#include "AReVi/Utils/abstractStream.h"
#include "AReVi/Shapes/shape3D.h"



namespace AReVi {
//----------------------------------------------------------------------------
// CLASS       : NSListenerObject3D
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSListenerObject3D,NSListenerBase3D)
//---------------------- Constructors / Destructor ---------------------------
NSListenerObject3D::NSListenerObject3D(ArCW & arCW) // Protected
  : NSListenerBase3D(arCW),
    _object3D(),
    _shapeChangeFlag(false),
    _appearanceFlag(false)
{}

NSListenerObject3D::~NSListenerObject3D(void)
{
}
//---------------------------- Public methods --------------------------------
// Controler will sign to all object events and will be able to transmit
// them. (getPendingEvents());
void
NSListenerObject3D::init(ArRef<LocalDepService> service, ArRef<ArObject> object)
{
  if (object->getClass()->isA(Object3D::CLASS()))
    _object3D=ar_down_cast<Object3D>(object);
  else
    return;

  NSListenerBase3D::init(service,object);
  _object3D->addShapeChangeCB(thisRef(),&NSListenerObject3D::_onShapeChangeCB);
  _object3D->addAppearanceCB(thisRef(),&NSListenerObject3D::_onAppearanceCB);
}

//------------------------ Controler is Watcher ------------------------------
// Return events backup (eg : motion, setup change ...)
void
NSListenerObject3D::getPendingEvents(StlList<ArRef<NSEvent> > &events)
{
  NSListenerBase3D::getPendingEvents(events);

  if (_shapeChangeFlag)
  {
    ArRef<NSEventObject3DShapeChange> event=NSEventObject3DShapeChange::NEW();
    event->writeObject3D(_object3D);
    events.push_back(event);
    _shapeChangeFlag=false;
  }
  if (_appearanceFlag)
  {
    ArRef<NSEventObject3DAppearance> event=NSEventObject3DAppearance::NEW();
    event->writeObject3D(_object3D);
    events.push_back(event);
    _appearanceFlag=false;
  }
}

//------------------------- Forbiden operations ------------------------------
void
NSListenerObject3D::_onShapeChangeCB(const Object3D::ShapeChangeEvent &)
{
  _shapeChangeFlag=true;
}

void
NSListenerObject3D::_onAppearanceCB(const Object3D::AppearanceEvent &)
{
  _appearanceFlag=true;
}

//----------------------------------------------------------------------------
// CLASS       : NSInteractorObject3D
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSInteractorObject3D,NSInteractorBase3D)
//---------------------- Constructors / Destructor ---------------------------
NSInteractorObject3D::NSInteractorObject3D(ArCW & arCW) // Protected
  : NSInteractorBase3D(arCW),_object3D()
{}

NSInteractorObject3D::~NSInteractorObject3D(void)
{}
//---------------------------- Public methods --------------------------------

// Controler will sign to all object network events and will be able to
// affect remote object with.
void
NSInteractorObject3D::init(ArRef<RemoteDepService> service, ArRef<ArObject> object)
{
  if (object->getClass()->isA(Object3D::CLASS()))
    _object3D=ar_down_cast<Object3D>(object);
  else
    return;

  NSInteractorBase3D::init(service,object);

  _registerEvent(NSEventObject3DShapeChange::CLASS()->getName(),
                (NetworkService::EventMethod)&NSInteractorObject3D::_onShapeChangeEvent);
  _registerEvent(NSEventObject3DAppearance::CLASS()->getName(),
                (NetworkService::EventMethod)&NSInteractorObject3D::_onAppearanceEvent);
}

//------------------------- Forbiden operations ------------------------------
NSEvent::EventProcess
NSInteractorObject3D::_onShapeChangeEvent(ArConstRef<NSEvent> eventObj)
{
  ArConstRef<NSEventObject3DShapeChange> event=ar_down_cast<NSEventObject3DShapeChange>(eventObj);

  if (_object3D.valid())
  {
    if (!event->readObject3D(_object3D,_depService))
    {
      setErrorMessage(_object3D->getErrorMessage());
      _object3D->setErrorMessage("");
      return NSEvent::EP_FAIL;
    }
  }
  return NSEvent::EP_OK;
}

NSEvent::EventProcess
NSInteractorObject3D::_onAppearanceEvent(ArConstRef<NSEvent> eventObj)
{
  ArConstRef<NSEventObject3DAppearance> event=ar_down_cast<NSEventObject3DAppearance>(eventObj);

  if (_object3D.valid())
    event->readObject3D(_object3D);

  return NSEvent::EP_OK;
}
} // namespace NetworkServices

//---End-Of-File---------------------------------------------------------------
