// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/Controlers/nsCtrlOSD.cpp
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReViNS/Controlers/nsCtrlOsd.h"
#include "AReViNS/Events/nsEventOsd.h"
//#include "AReViNS/remoteDepService.h"

namespace AReVi {
//----------------------------------------------------------------------------
// CLASS       : NSListenerOSD
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSListenerOSD,NSListener)
//---------------------- Constructors / Destructor ---------------------------
NSListenerOSD::NSListenerOSD(ArCW & arCW) // Protected
  : NSListener(arCW),
    _osd(),
    _setupFlag(false)
{
  // Nothing to do
}

NSListenerOSD::~NSListenerOSD(void)
{
  // Nothing to do
}

//---------------------------- Public methods --------------------------------
// Controler will sign to all object events and will be able to transmit
// them. (getPendingEvents());
void
NSListenerOSD::init(ArRef<LocalDepService> service, ArRef<ArObject> object)
{
  if (object->getClass()->isA(OSD::CLASS()))
    _osd=ar_down_cast<OSD>(object);
  else
    return;

  NSListener::init(service,object);
  _osd->addSetupCB(thisRef(),&NSListenerOSD::_onSetupCB);
  _osd->addMotionCB(thisRef(),&NSListenerOSD::_onMotionCB);
}

//------------------------ Controler is Watcher ------------------------------
// Return events backup (eg : motion, setup change ...)
void
NSListenerOSD::getPendingEvents(StlList<ArRef<NSEvent> > &events)
{
  if(_setupFlag)
  {
    ArRef<NSEventOSDSetup> event=NSEventOSDSetup::NEW();
    event->writeOSD(_osd);
    events.push_back(event);
    _setupFlag=false;
  }
}

//------------------------- Forbiden operations ------------------------------
// OSD Setup (set padding, size ...)
void
NSListenerOSD::_onSetupCB(const OSD::SetupEvent &)
{
  _setupFlag=true;
}

// OSD Motion (set location, position, orientation ...)
void
NSListenerOSD::_onMotionCB(const OSD::MotionEvent &)
{
  _setupFlag=true;
}

//----------------------------------------------------------------------------
// CLASS       : NSInteractorOSD
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSInteractorOSD,NSInteractor)
//---------------------- Constructors / Destructor ---------------------------
NSInteractorOSD::NSInteractorOSD(ArCW & arCW) // Protected
  : NSInteractor(arCW),_osd()
{}

NSInteractorOSD::~NSInteractorOSD(void)
{
}
//---------------------------- Public methods --------------------------------

// Controler will sign to all object network events and will be able to
// affect remote object with.
void
NSInteractorOSD::init(ArRef<RemoteDepService> service, ArRef<ArObject> object)
{
  if (object->getClass()->isA(OSD::CLASS()))
    _osd=ar_down_cast<OSD>(object);
  else
    return;

  NSInteractor::init(service,object);

  _registerEvent(NSEventOSDSetup::CLASS()->getName(),
                (NetworkService::EventMethod)&NSInteractorOSD::_onSetupEvent);
}

//------------------------- Forbiden operations ------------------------------
// Viewer Setup (fog, background, backTransform...)
NSEvent::EventProcess
NSInteractorOSD::_onSetupEvent(ArConstRef<NSEvent> eventObj)
{
  ArConstRef<NSEventOSDSetup> event=ar_down_cast<NSEventOSDSetup>(eventObj);
  event->readOSD(_osd);
  return NSEvent::EP_OK;
}

} // namespace NetworkServices
//---End-Of-File---------------------------------------------------------------
