// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/Controlers/nsCtrlOSDSprite.cpp
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReViNS/Controlers/nsCtrlOsdSprite.h"
#include "AReViNS/Events/nsEventOsd.h"
#include "AReViNS/remoteDepService.h"




namespace AReVi {
//----------------------------------------------------------------------------
// CLASS       : NSListenerOSDSprite
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSListenerOSDSprite,NSListenerOSD)
//---------------------- Constructors / Destructor ---------------------------
NSListenerOSDSprite::NSListenerOSDSprite(ArCW & arCW) // Protected
  : NSListenerOSD(arCW),
    _osdSprite()
{
  // Nothing to do
}

NSListenerOSDSprite::~NSListenerOSDSprite(void)
{
  // Nothing to do
}

//---------------------------- Public methods --------------------------------
// Controler will sign to all object events and will be able to transmit
// them. (getPendingEvents());
void
NSListenerOSDSprite::init(ArRef<LocalDepService> service, ArRef<ArObject> object)
{
  if (object->getClass()->isA(OSDSprite::CLASS()))
    _osdSprite=ar_down_cast<OSDSprite>(object);
  else
    return;

  NSListenerOSD::init(service,object);
}

//------------------------ Controler is Watcher ------------------------------
// Return events backup (eg : motion, setup change ...)
void
NSListenerOSDSprite::getPendingEvents(StlList<ArRef<NSEvent> > &events)
{
  if(_setupFlag)
  {
    ArRef<NSEventOSDSpriteSetup> event=NSEventOSDSpriteSetup::NEW();
    event->writeOSDSprite(_osdSprite);
    events.push_back(event);
    _setupFlag=false;
  }
}

//------------------------- Forbiden operations ------------------------------

//----------------------------------------------------------------------------
// CLASS       : NSInteractorOSDSprite
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSInteractorOSDSprite,NSInteractor)
//---------------------- Constructors / Destructor ---------------------------
NSInteractorOSDSprite::NSInteractorOSDSprite(ArCW & arCW) // Protected
  : NSInteractorOSD(arCW),_osdSprite()
{}

NSInteractorOSDSprite::~NSInteractorOSDSprite(void)
{
}
//---------------------------- Public methods --------------------------------

// Controler will sign to all object network events and will be able to
// affect remote object with.
void
NSInteractorOSDSprite::init(ArRef<RemoteDepService> service, ArRef<ArObject> object)
{
  if (object->getClass()->isA(OSDSprite::CLASS()))
    _osdSprite=ar_down_cast<OSDSprite>(object);
  else
    return;

  NSInteractorOSD::init(service,object);

  _unregisterEvent(NSEventOSDSetup::CLASS()->getName());

  _registerEvent(NSEventOSDSpriteSetup::CLASS()->getName(),
                (NetworkService::EventMethod)&NSInteractorOSDSprite::_onSetupEvent);
}

//------------------------- Forbiden operations ------------------------------
// OSD Setup (set padding, size ...)
NSEvent::EventProcess
NSInteractorOSDSprite::_onSetupEvent(ArConstRef<NSEvent> eventObj)
{
  ArConstRef<NSEventOSDSpriteSetup> event=ar_down_cast<NSEventOSDSpriteSetup>(eventObj);

  if (!event->readOSDSprite(_osdSprite,_depService))
  {
    setErrorMessage("Error on OSDSpriteSetupEvent : Non valid message");
    return NSEvent::EP_FAIL;
  }
  return NSEvent::EP_OK;
}

} // namespace NetworkServices
//---End-Of-File---------------------------------------------------------------
