// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/Controlers/nsCtrlOSDText.cpp
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReViNS/Controlers/nsCtrlOsdText.h"
#include "AReViNS/Events/nsEventOsd.h"
//#include "AReViNS/remoteDepService.h"

namespace AReVi {
//----------------------------------------------------------------------------
// CLASS       : NSListenerOSDText
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSListenerOSDText,NSListenerOSD)
//---------------------- Constructors / Destructor ---------------------------
NSListenerOSDText::NSListenerOSDText(ArCW & arCW) // Protected
  : NSListenerOSD(arCW),
    _osdText(),
    _textFlag(false)
{
  // Nothing to do
}

NSListenerOSDText::~NSListenerOSDText(void)
{
  // Nothing to do
}

//---------------------------- Public methods --------------------------------
// Controler will sign to all object events and will be able to transmit
// them. (getPendingEvents());
void
NSListenerOSDText::init(ArRef<LocalDepService> service, ArRef<ArObject> object)
{
  if (object->getClass()->isA(OSDText::CLASS()))
    _osdText=ar_down_cast<OSDText>(object);
  else
    return;

  NSListenerOSD::init(service,object);
  _osdText->addTextChangedCB(thisRef(),&NSListenerOSDText::_onTextChangedCB);
}

//------------------------ Controler is Watcher ------------------------------
// Return events backup (eg : motion, setup change ...)
void
NSListenerOSDText::getPendingEvents(StlList<ArRef<NSEvent> > &events)
{
  if(_setupFlag)
  {
    ArRef<NSEventOSDTextSetup> event=NSEventOSDTextSetup::NEW();
    event->writeOSDText(_osdText);
    events.push_back(event);
    _setupFlag=false;
  }
  if(_textFlag)
  {
    ArRef<NSEventOSDText> event=NSEventOSDText::NEW();
    event->writeOSDText(_osdText);
    events.push_back(event);
    _textFlag=false;
  }
}

//------------------------- Forbiden operations ------------------------------
// Text changed (setText).
void
NSListenerOSDText::_onTextChangedCB(const OSDText::TextChangedEvent &)
{
  _textFlag=true;
}

//----------------------------------------------------------------------------
// CLASS       : NSInteractorOSDText
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSInteractorOSDText,NSInteractor)
//---------------------- Constructors / Destructor ---------------------------
NSInteractorOSDText::NSInteractorOSDText(ArCW & arCW) // Protected
  : NSInteractorOSD(arCW),_osdText()
{}

NSInteractorOSDText::~NSInteractorOSDText(void)
{
}
//---------------------------- Public methods --------------------------------

// Controler will sign to all object network events and will be able to
// affect remote object with.
void
NSInteractorOSDText::init(ArRef<RemoteDepService> service, ArRef<ArObject> object)
{
  if (object->getClass()->isA(OSDText::CLASS()))
    _osdText=ar_down_cast<OSDText>(object);
  else
    return;

  NSInteractorOSD::init(service,object);

  _unregisterEvent(NSEventOSDSetup::CLASS()->getName());

  _registerEvent(NSEventOSDTextSetup::CLASS()->getName(),
                (NetworkService::EventMethod)&NSInteractorOSDText::_onSetupEvent);
  _registerEvent(NSEventOSDText::CLASS()->getName(),
                (NetworkService::EventMethod)&NSInteractorOSDText::_onTextChangedEvent);
}

//------------------------- Forbiden operations ------------------------------
// OSD Setup (set padding, size ...)
NSEvent::EventProcess
NSInteractorOSDText::_onSetupEvent(ArConstRef<NSEvent> eventObj)
{
  ArConstRef<NSEventOSDTextSetup> event=ar_down_cast<NSEventOSDTextSetup>(eventObj);
  event->readOSDText(_osdText);
  return NSEvent::EP_OK;
}

//! Text changed (setText).
NSEvent::EventProcess
NSInteractorOSDText::_onTextChangedEvent(ArConstRef<NSEvent> eventObj)
{
  ArConstRef<NSEventOSDText> event=ar_down_cast<NSEventOSDText>(eventObj);
  event->readOSDText(_osdText);
  return NSEvent::EP_OK;
}

} // namespace NetworkServices
//---End-Of-File---------------------------------------------------------------
