// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/Controlers/nsCtrlParticleCollider.cpp
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReViNS/Controlers/nsCtrlParticleCollider.h"
//#include "AReViNS/Events/nsEventParticleCollider.h"
#include "AReViNS/localDepService.h"

namespace AReVi {
//----------------------------------------------------------------------------
// CLASS       : NSListenerParticleCollider
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSListenerParticleCollider,NSListener)
//---------------------- Constructors / Destructor ---------------------------
NSListenerParticleCollider::NSListenerParticleCollider(ArCW & arCW) // Protected
  : NSListener(arCW),
    _particleCollider(),
    _setupFlag(false)
{}

NSListenerParticleCollider::~NSListenerParticleCollider(void)
{
}
//---------------------------- Public methods --------------------------------
// Controler will sign to all object events and will be able to transmit
// them. (getPendingEvents());
void
NSListenerParticleCollider::init(ArRef<LocalDepService> service, ArRef<ArObject> object)
{
  if (object->getClass()->isA(ParticleCollider::CLASS()))
    _particleCollider=ar_down_cast<ParticleCollider>(object);
  else
    return;

  NSListener::init(service,object);
  _particleCollider->addSetupCB(thisRef(),&NSListenerParticleCollider::_onSetupCB);
}

//------------------------ Controler is Watcher ------------------------------
// Return events backup (eg : motion, setup change ...)
void
NSListenerParticleCollider::getPendingEvents(StlList<ArRef<NSEvent> > &/*events*/)
{
  if (_setupFlag)
  {
    _depService->shareObject(_particleCollider); // TODO : Ne plus sérialiser l'objet!!!
    /*
    ArRef<NSEventParticleColliderSetup> event=NSEventParticleColliderSetup::NEW();
    event->writeParticleCollider(_particleCollider);
    events.push_back(event);
    */
    _setupFlag=false;
  }
}

//------------------------- Forbiden operations ------------------------------
void
NSListenerParticleCollider::_onSetupCB(const ParticleCollider::SetupEvent &)
{
  _setupFlag=true;
}


//----------------------------------------------------------------------------
// CLASS       : NSInteractorParticleCollider
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSInteractorParticleCollider,NSInteractor)
//---------------------- Constructors / Destructor ---------------------------
NSInteractorParticleCollider::NSInteractorParticleCollider(ArCW & arCW) // Protected
  : NSInteractor(arCW),
    _particleCollider()
{}

NSInteractorParticleCollider::~NSInteractorParticleCollider(void)
{}
//---------------------------- Public methods --------------------------------

// Controler will sign to all object network events and will be able to
// affect remote object with.
void
NSInteractorParticleCollider::init(ArRef<RemoteDepService> service, ArRef<ArObject> object)
{
  if (object->getClass()->isA(ParticleCollider::CLASS()))
    _particleCollider=ar_down_cast<ParticleCollider>(object);
  else
    return;

  NSInteractor::init(service,object);
 // _registerEvent(NSEventParticleColliderSetup::CLASS()->getName(),
 //               (NetworkService::EventMethod)&NSInteractorParticleCollider::_onSetupEvent);
}

//------------------------- Forbiden operations ------------------------------
NSEvent::EventProcess
NSInteractorParticleCollider::_onSetupEvent(ArConstRef<NSEvent> /*eventObj*/)
{
// TODO :!!! Ctrl for all ParticleColliders!!!
  //ArConstRef<NSEventParticleColliderSetup> event=ar_down_cast<NSEventParticleColliderSetup>(eventObj);
 //if (_particleCollider.valid())
    //event->readParticleCollider(_particleCollider);

// => TO SIMPLIFY CONTROLER SEND SERIALIZATION
  return NSEvent::EP_OK;
}


} // namespace NetworkServices

//---End-Of-File---------------------------------------------------------------
