// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/Controlers/nsCtrlParticleSystem.cpp
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReViNS/Controlers/nsCtrlParticleSystem.h"
#include "AReViNS/Events/nsEventParticleSystem.h"
#include "AReViNS/remoteDepService.h"

#include "AReVi/Lib3D/particleSystem.h"
#include "AReVi/Utils/abstractStream.h"



namespace AReVi {

//----------------------------------------------------------------------------
// CLASS       : NSListenerParticleSystem
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSListenerParticleSystem,NSListenerBase3D)
//---------------------- Constructors / Destructor ---------------------------
NSListenerParticleSystem::NSListenerParticleSystem(ArCW & arCW) // Protected
  : NSListenerBase3D(arCW),_part(),
    _renderingFlag(false),
    _emissionFlag(false),
    _evolutionFlag(false),
    _events()
{}

NSListenerParticleSystem::~NSListenerParticleSystem(void)
{
}
//---------------------------- Public methods --------------------------------
// Controler will sign to all object events and will be able to transmit
// them. (getPendingEvents());
void
NSListenerParticleSystem::init(ArRef<LocalDepService> service, ArRef<ArObject> object)
{
  if (object->getClass()->isA(ParticleSystem::CLASS()))
    _part=ar_down_cast<ParticleSystem>(object);
  else
    return;

  NSListenerBase3D::init(service,object);
  _part->addRenderingCB(thisRef(),&NSListenerParticleSystem::_onRenderingCB);
  _part->addUpdateCB(thisRef(),&NSListenerParticleSystem::_onUpdateCB);
}

//------------------------ Controler is Watcher ------------------------------
// Return events backup (eg : motion, setup change ...)
void
NSListenerParticleSystem::getPendingEvents(StlList<ArRef<NSEvent> > &events)
{
  NSListenerBase3D::getPendingEvents(events);

  if (_renderingFlag)
  {
  //  cerr << "NSListenerParticleSystem::getPendingEvents _motionFlag=" << _motionFlag <<endl;
    ArRef<NSEventParticleSystemRendering> event=NSEventParticleSystemRendering::NEW();
    event->writeParticleSystem(_part);
    events.push_back(event);
    _renderingFlag=false;
  }
  if (_emissionFlag)
  {
  //  cerr << "NSListenerParticleSystem::getPendingEvents _motionFlag=" << _motionFlag <<endl;
    ArRef<NSEventParticleSystemEmission> event=NSEventParticleSystemEmission::NEW();
    event->writeParticleSystem(_part);
    events.push_back(event);
    _emissionFlag=false;
  }
  if (_evolutionFlag)
  {
  //  cerr << "NSListenerParticleSystem::getPendingEvents _motionFlag=" << _motionFlag <<endl;
    ArRef<NSEventParticleSystemEvolution> event=NSEventParticleSystemEvolution::NEW();
    event->writeParticleSystem(_part);
    events.push_back(event);
    _evolutionFlag=false;
  }
  //cerr << "SIZE=" << _events.size() << endl;
  StlList<ArRef<NSEvent> >::iterator it=_events.begin();
  StlList<ArRef<NSEvent> >::iterator it_end=_events.end();
  while(it!=it_end)
  {
    events.push_back(*it);
    it++;
  }
  _events.clear();
}

//------------------------- Forbiden operations ------------------------------
void
NSListenerParticleSystem::_onRenderingCB(const ParticleSystem::RenderingEvent &)
{
  _renderingFlag=true;
}

void
NSListenerParticleSystem::_onUpdateCB(const ParticleSystem::UpdateEvent &evt)
{
  ArRef<NSEventParticleSystemUpdate> event=NSEventParticleSystemUpdate::NEW();
  event->dt=evt.dt;
  _events.push_back(event);
}

void
NSListenerParticleSystem::_onEmissionCB(const ParticleSystem::EmissionEvent &)
{
  _emissionFlag=true;
}

void
NSListenerParticleSystem::_onEvolutionCB(const ParticleSystem::EvolutionEvent &)
{
  _evolutionFlag=true;
}
void
NSListenerParticleSystem::_onAddColliderCB(const ParticleSystem::AddColliderEvent &event)
{
  ArRef<NSEventParticleSystemAddCollider> nsEvent=NSEventParticleSystemAddCollider::NEW();

  nsEvent->setRef(event.collider);
  _events.push_back(nsEvent);
}
void
NSListenerParticleSystem::_onRemoveColliderCB(const ParticleSystem::RemoveColliderEvent &event)
{
  if (event.collider.valid())
    {
    ArRef<NSEventParticleSystemRemoveCollider> nsEvent=NSEventParticleSystemRemoveCollider::NEW();
    nsEvent->setID(event.collider->getAlias());
    _events.push_back(nsEvent);
    }
}

//----------------------------------------------------------------------------
// CLASS       : NSInteractorParticleSystem
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSInteractorParticleSystem,NSInteractorBase3D)
//---------------------- Constructors / Destructor ---------------------------
NSInteractorParticleSystem::NSInteractorParticleSystem(ArCW & arCW) // Protected
  : NSInteractorBase3D(arCW),_part()
{}

NSInteractorParticleSystem::~NSInteractorParticleSystem(void)
{}
//---------------------------- Public methods --------------------------------

// Controler will sign to all object network events and will be able to
// affect remote object with.
void
NSInteractorParticleSystem::init(ArRef<RemoteDepService> service, ArRef<ArObject> object)
{
  if (object->getClass()->isA(ParticleSystem::CLASS()))
    _part=ar_down_cast<ParticleSystem>(object);
  else
    return;

  NSInteractorBase3D::init(service,object);

  _registerEvent(NSEventParticleSystemRendering::CLASS()->getName(),
                (NetworkService::EventMethod)&NSInteractorParticleSystem::_onRenderingEvent);
  _registerEvent(NSEventParticleSystemUpdate::CLASS()->getName(),
                (NetworkService::EventMethod)&NSInteractorParticleSystem::_onUpdateEvent);
  _registerEvent(NSEventParticleSystemEmission::CLASS()->getName(),
                (NetworkService::EventMethod)&NSInteractorParticleSystem::_onEmissionEvent);
  _registerEvent(NSEventParticleSystemEvolution::CLASS()->getName(),
                (NetworkService::EventMethod)&NSInteractorParticleSystem::_onEvolutionEvent);
  _registerEvent(NSEventParticleSystemAddCollider::CLASS()->getName(),
                (NetworkService::EventMethod)&NSInteractorParticleSystem::_onAddColliderEvent);
  _registerEvent(NSEventParticleSystemRemoveCollider::CLASS()->getName(),
                (NetworkService::EventMethod)&NSInteractorParticleSystem::_onRemoveColliderEvent);
}

//------------------------- Forbiden operations ------------------------------
NSEvent::EventProcess
NSInteractorParticleSystem::_onRenderingEvent(ArConstRef<NSEvent> eventObj)
{
  ArConstRef<NSEventParticleSystemRendering> event=ar_down_cast<NSEventParticleSystemRendering>(eventObj);

  if (_part.valid())
    event->readParticleSystem(_part,_depService);
  return NSEvent::EP_OK;
}

NSEvent::EventProcess
NSInteractorParticleSystem::_onUpdateEvent(ArConstRef<NSEvent> eventObj)
{
  ArConstRef<NSEventParticleSystemUpdate> event=ar_down_cast<NSEventParticleSystemUpdate>(eventObj);

  if (_part.valid())
    _part->update(event->dt);
  //cerr << _part->getNbParticles() << endl;
  return NSEvent::EP_OK;
}

NSEvent::EventProcess
NSInteractorParticleSystem::_onEmissionEvent(ArConstRef<NSEvent> eventObj)
{
  ArConstRef<NSEventParticleSystemEmission> event=ar_down_cast<NSEventParticleSystemEmission>(eventObj);
  event->readParticleSystem(_part);
  return NSEvent::EP_OK;
}

NSEvent::EventProcess
NSInteractorParticleSystem::_onEvolutionEvent(ArConstRef<NSEvent> eventObj)
{
  ArConstRef<NSEventParticleSystemEvolution> event=ar_down_cast<NSEventParticleSystemEvolution>(eventObj);
  event->readParticleSystem(_part);
  return NSEvent::EP_OK;
}

NSEvent::EventProcess
NSInteractorParticleSystem::_onAddColliderEvent(ArConstRef<NSEvent> eventObj)
{
  ArConstRef<NSEventParticleSystemAddCollider> event=ar_down_cast<NSEventParticleSystemAddCollider>(eventObj);
  ArRef<ParticleCollider> collider;
  _depService->requestRemoteObject(event->getID(),event->getClassName(), collider);
  if (!collider.valid())
  {
    setErrorMessage(_depService->getErrorMessage());
    return NSEvent::EP_FAIL;
  }
  _part->addCollider(collider);
  return NSEvent::EP_OK;
}

NSEvent::EventProcess
NSInteractorParticleSystem::_onRemoveColliderEvent(ArConstRef<NSEvent> eventObj)
{
  ArConstRef<NSEventParticleSystemRemoveCollider> event=ar_down_cast<NSEventParticleSystemRemoveCollider>(eventObj);
  ArRef<ParticleCollider> collider;
  _depService->findRemoteObject(event->getID(), collider);
  if (collider.valid())
    _part->removeCollider(collider);
  // else ... no matter!

  return NSEvent::EP_OK;
}

} // namespace NetworkServices

//---End-Of-File---------------------------------------------------------------
