// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/Controlers/nsCtrlRenderTexture.cpp
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReViNS/Controlers/nsCtrlRenderTexture.h"
#include "AReViNS/Events/nsEventRenderTexture.h"


namespace AReVi {
//----------------------------------------------------------------------------
// CLASS       : NSListenerRenderTexture
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSListenerRenderTexture,NSListener)
//---------------------- Constructors / Destructor ---------------------------
NSListenerRenderTexture::NSListenerRenderTexture(ArCW & arCW) // Protected
  : NSListener(arCW),
    _renderTexture(),
    _setupFlag(false)
{}

NSListenerRenderTexture::~NSListenerRenderTexture(void)
{
}
//---------------------------- Public methods --------------------------------
// Controler will sign to all object events and will be able to transmit
// them. (getPendingEvents());
void
NSListenerRenderTexture::init(ArRef<LocalDepService> service, ArRef<ArObject> object)
{
  if (object->getClass()->isA(RenderTexture::CLASS()))
    _renderTexture=ar_down_cast<RenderTexture>(object);
  else
    return;

  NSListener::init(service,object);
  _renderTexture->addSetupCB(thisRef(),&NSListenerRenderTexture::_onSetupCB);
}

//------------------------ Controler is Watcher ------------------------------
// Return events backup (eg : motion, setup change ...)
void
NSListenerRenderTexture::getPendingEvents(StlList<ArRef<NSEvent> > &events)
{
  if (_setupFlag)
  {
    ArRef<NSEventRenderTextureSetup> event=NSEventRenderTextureSetup::NEW();
    event->writeRenderTexture(_renderTexture);
    events.push_back(event);
    _setupFlag=false;
  }
}

//------------------------- Forbiden operations ------------------------------
void
NSListenerRenderTexture::_onSetupCB(const RenderTexture::SetupEvent &)
{
  _setupFlag=true;
}


//----------------------------------------------------------------------------
// CLASS       : NSInteractorRenderTexture
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSInteractorRenderTexture,NSInteractor)
//---------------------- Constructors / Destructor ---------------------------
NSInteractorRenderTexture::NSInteractorRenderTexture(ArCW & arCW) // Protected
  : NSInteractor(arCW),
    _renderTexture()
{}

NSInteractorRenderTexture::~NSInteractorRenderTexture(void)
{}
//---------------------------- Public methods --------------------------------

// Controler will sign to all object network events and will be able to
// affect remote object with.
void
NSInteractorRenderTexture::init(ArRef<RemoteDepService> service, ArRef<ArObject> object)
{
  if (object->getClass()->isA(RenderTexture::CLASS()))
    _renderTexture=ar_down_cast<RenderTexture>(object);
  else
    return;

  NSInteractor::init(service,object);
  _registerEvent(NSEventRenderTextureSetup::CLASS()->getName(),
                (NetworkService::EventMethod)&NSInteractorRenderTexture::_onSetupEvent);
}

//------------------------- Forbiden operations ------------------------------
NSEvent::EventProcess
NSInteractorRenderTexture::_onSetupEvent(ArConstRef<NSEvent> eventObj)
{
  ArConstRef<NSEventRenderTextureSetup> event=ar_down_cast<NSEventRenderTextureSetup>(eventObj);

  if (_renderTexture.valid())
    event->readRenderTexture(_renderTexture);

  return NSEvent::EP_OK;
}


} // namespace NetworkServices

//---End-Of-File---------------------------------------------------------------
