// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/Controlers/nsCtrlRenderer3D.cpp
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReViNS/Controlers/nsCtrlRenderer3D.h"
#include "AReViNS/Events/nsEventRenderer3D.h"

#include "AReViNS/remoteDepService.h"

#include "AReVi/Utils/abstractStream.h"
#include "AReVi/Lib3D/scene3D.h"
#include "AReVi/Lib3D/renderer3D.h"
#include "AReVi/Lib3D/osd.h"



namespace AReVi {
//----------------------------------------------------------------------------
// CLASS       : NSListenerRenderer3D
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSListenerRenderer3D,NSListenerBase3D)
//---------------------- Constructors / Destructor ---------------------------
NSListenerRenderer3D::NSListenerRenderer3D(ArCW & arCW) // Protected
  : NSListenerBase3D(arCW),
    _renderer(),
    _viewParametersFlag(false),
    _renderingSetupFlag(false),
    _appearanceSetupFlag(false),
    _events()
{}

NSListenerRenderer3D::~NSListenerRenderer3D(void)
{
}
//---------------------------- Public methods --------------------------------
// Controler will sign to all object events and will be able to transmit
// them. (getPendingEvents());
void
NSListenerRenderer3D::init(ArRef<LocalDepService> service, ArRef<ArObject> object)
{
  if (object->getClass()->isA(Renderer3D::CLASS()))
    _renderer=ar_down_cast<Renderer3D>(object);
  else
    return;

  NSListenerBase3D::init(service,object);
  _renderer->addViewParametersCB(thisRef(),&NSListenerRenderer3D::_onViewParametersCB);
  _renderer->addRenderingSetupCB(thisRef(),&NSListenerRenderer3D::_onRenderingSetupCB);
  _renderer->addAppearanceSetupCB(thisRef(),&NSListenerRenderer3D::_onAppearanceSetupCB);
  _renderer->addAddOSDCB(thisRef(),&NSListenerRenderer3D::_onAddOSDCB);
  _renderer->addRemoveOSDCB(thisRef(),&NSListenerRenderer3D::_onRemoveOSDCB);
}

//------------------------ Controler is Watcher ------------------------------
// Return events backup (eg : motion, setup change ...)
void
NSListenerRenderer3D::getPendingEvents(StlList<ArRef<NSEvent> > &events)
{
  NSListenerBase3D::getPendingEvents(events);

  if (_viewParametersFlag)
  {
  //  cerr << "NSListenerRenderer3D::getPendingEvents _motionFlag=" << _motionFlag <<endl;
    ArRef<NSEventRenderer3DViewParameters> event=NSEventRenderer3DViewParameters::NEW();
    event->writeRendererViewsParameters(_renderer);
    events.push_back(event);
    _viewParametersFlag=false;
  }
  if (_renderingSetupFlag)
  {
  //  cerr << "NSListenerRenderer3D::getPendingEvents _motionFlag=" << _motionFlag <<endl;
    ArRef<NSEventRenderer3DRenderingSetup> event=NSEventRenderer3DRenderingSetup::NEW();

    ArConstRef<Scene3D> scene=_renderer->getScene();
    if (scene.valid())
      event->setMainScene(scene->getAlias(),scene->getClass()->getName());
    else
      event->setMainScene("","");

    scene=_renderer->getFrontScene();
    if (scene.valid())
      event->setFrontScene(scene->getAlias(),scene->getClass()->getName());
    else
      event->setFrontScene("","");

    scene=_renderer->getBackScene();
    if (scene.valid())
      event->setBackScene(scene->getAlias(),scene->getClass()->getName());
    else
      event->setBackScene("","");

    events.push_back(event);
    _renderingSetupFlag=false;
  }
  if (_appearanceSetupFlag)
  {
  //  cerr << "NSListenerRenderer3D::getPendingEvents _motionFlag=" << _motionFlag <<endl;
    ArRef<NSEventRenderer3DAppearanceSetup> event=NSEventRenderer3DAppearanceSetup::NEW();
    event->writeRendererAppearanceSetup(_renderer);
    events.push_back(event);
    _appearanceSetupFlag=false;
  }

  StlList<ArRef<NSEvent> >::iterator it=_events.begin();
  StlList<ArRef<NSEvent> >::iterator it_end=_events.end();
  while(it!=it_end)
  {
    events.push_back(*it);
    it++;
  }
  _events.clear();
}

//------------------------- Forbiden operations ------------------------------
void
NSListenerRenderer3D::_onViewParametersCB(const Renderer3D::ViewParametersEvent &)
{
  _viewParametersFlag=true;
}

void
NSListenerRenderer3D::_onRenderingSetupCB(const Renderer3D::RenderingSetupEvent &)
{
  _renderingSetupFlag=true;
}

void
NSListenerRenderer3D::_onAppearanceSetupCB(const Renderer3D::AppearanceSetupEvent &)
{
  _appearanceSetupFlag=true;
}

void
NSListenerRenderer3D::_onAddOSDCB(const Renderer3D::AddOSDEvent &evt)
{
  if (evt.osd.valid())
  {
    ArRef<NSEventRenderer3DAddOSD> event=NSEventRenderer3DAddOSD::NEW();
    event->setRef(evt.osd);
    _events.push_back(event);
  }
}

void
NSListenerRenderer3D::_onRemoveOSDCB(const Renderer3D::RemoveOSDEvent &evt)
{
  if (evt.osd.valid())
  {
    ArRef<NSEventRenderer3DRemoveOSD> event=NSEventRenderer3DRemoveOSD::NEW();
    event->setID(evt.osd->getAlias());
    _events.push_back(event);
  }
}

//----------------------------------------------------------------------------
// CLASS       : NSInteractorRenderer3D
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSInteractorRenderer3D,NSInteractorBase3D)
//---------------------- Constructors / Destructor ---------------------------
NSInteractorRenderer3D::NSInteractorRenderer3D(ArCW & arCW) // Protected
  : NSInteractorBase3D(arCW),_renderer()
{}

NSInteractorRenderer3D::~NSInteractorRenderer3D(void)
{}
//---------------------------- Public methods --------------------------------

// Controler will sign to all object network events and will be able to
// affect remote object with.
void
NSInteractorRenderer3D::init(ArRef<RemoteDepService> service, ArRef<ArObject> object)
{
  if (object->getClass()->isA(Renderer3D::CLASS()))
    _renderer=ar_down_cast<Renderer3D>(object);
  else
    return;

  NSInteractorBase3D::init(service,object);

  _registerEvent(NSEventRenderer3DViewParameters::CLASS()->getName(),
                (NetworkService::EventMethod)&NSInteractorRenderer3D::_onViewParametersEvent);
  _registerEvent(NSEventRenderer3DRenderingSetup::CLASS()->getName(),
                (NetworkService::EventMethod)&NSInteractorRenderer3D::_onRenderingSetupEvent);
  _registerEvent(NSEventRenderer3DAppearanceSetup::CLASS()->getName(),
                (NetworkService::EventMethod)&NSInteractorRenderer3D::_onAppearanceSetupEvent);
  _registerEvent(NSEventRenderer3DAddOSD::CLASS()->getName(),
                (NetworkService::EventMethod)&NSInteractorRenderer3D::_onAddOSDEvent);
  _registerEvent(NSEventRenderer3DRemoveOSD::CLASS()->getName(),
                (NetworkService::EventMethod)&NSInteractorRenderer3D::_onRemoveOSDEvent);

}

//------------------------- Forbiden operations ------------------------------
NSEvent::EventProcess
NSInteractorRenderer3D::_onViewParametersEvent(ArConstRef<NSEvent> eventObj)
{
  //cerr << "NSInteractorRenderer3D::_onViewParametersEvent"<< endl;
  ArConstRef<NSEventRenderer3DViewParameters> event=ar_down_cast<NSEventRenderer3DViewParameters>(eventObj);
  event->readRendererViewsParameters(_renderer);
  return NSEvent::EP_OK;
}

NSEvent::EventProcess
NSInteractorRenderer3D::_onAppearanceSetupEvent(ArConstRef<NSEvent> eventObj)
{
  ArConstRef<NSEventRenderer3DAppearanceSetup> event=ar_down_cast<NSEventRenderer3DAppearanceSetup>(eventObj);
  event->readRendererAppearanceSetup(_renderer);
  return NSEvent::EP_OK;
}

NSEvent::EventProcess
NSInteractorRenderer3D::_onRenderingSetupEvent(ArConstRef<NSEvent> eventObj)
{
//  cerr << "NSInteractorRenderer3D::_onRenderingSetupEvent"<< endl;
  ArConstRef<NSEventRenderer3DRenderingSetup> event=ar_down_cast<NSEventRenderer3DRenderingSetup>(eventObj);

  StlString main,back,front;
  event->getScenes(main,front,back);

  ArRef<Scene3D> scene;
  //cerr << "SET SCENES =" << main << endl;

  if (main!="")
  {
    _depService->findRemoteObject(main,scene);
    if (!scene)
      if (!_depService->addRemoteReference(main,event->getMainSceneClass()))
        return NSEvent::EP_FAIL;
      else
        _depService->findRemoteObject(main,scene);

    //cerr << "SELECT SCENE" << scene << "  serverAlias=" << main << endl;
    _renderer->selectScene(scene);
  }

  if (front!="")
  {
    _depService->findRemoteObject(front,scene);
    if (!scene)
      if (!_depService->addRemoteReference(front,event->getFrontSceneClass()))
        return NSEvent::EP_FAIL;
      else
        _depService->findRemoteObject(front,scene);
    _renderer->setFrontScene(scene);
  }

  if (back!="")
  {
    _depService->findRemoteObject(back,scene);
    if (!scene)
      if (!_depService->addRemoteReference(back,event->getFrontSceneClass()))
        return NSEvent::EP_FAIL;
      else
        _depService->findRemoteObject(back,scene);
    _renderer->setBackScene(scene);
  }

  return NSEvent::EP_OK;
}

// ADD OSD
NSEvent::EventProcess
NSInteractorRenderer3D::_onAddOSDEvent(ArConstRef<NSEvent> eventObj)
{
  ArConstRef<NSEventRenderer3DAddOSD> event=ar_down_cast<NSEventRenderer3DAddOSD>(eventObj);
  ArRef<OSD> osd;
  //cerr << "Add OSD => " << event->getID() << endl;
  _depService->requestRemoteObject(event->getID(),
                                   event->getClassName(),
                                   osd);
  if (!osd.valid())
  {
    setErrorMessage("Error on addOSDEvent : (" + event->getID() + " , " + event->getClassName()+")");
    return NSEvent::EP_FAIL;
  }
  _renderer->addOSD(osd);
  return NSEvent::EP_OK;
}

// REMOVE OSD
NSEvent::EventProcess
NSInteractorRenderer3D::_onRemoveOSDEvent(ArConstRef<NSEvent> eventObj)
{
  ArConstRef<NSEventRenderer3DRemoveOSD> event=ar_down_cast<NSEventRenderer3DRemoveOSD>(eventObj);
  ArRef<OSD> osd;
  _depService->findRemoteObject(event->getID(),osd);
  if (osd.valid())
    _renderer->removeOSD(osd);
  return NSEvent::EP_OK;
}



} // namespace NetworkServices

//---End-Of-File---------------------------------------------------------------
