// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/Controlers/nsCtrlScene3D.cpp
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReViNS/Controlers/nsCtrlScene3D.h"
#include "AReViNS/Events/nsEventScene3D.h"

#include "AReViNS/remoteDepService.h"

#include "AReVi/Lib3D/object3D.h"
#include "AReVi/Lib3D/particleSystem.h"
#include "AReVi/Lib3D/light3D.h"
#include "AReVi/Lib3D/sound3D.h"

namespace AReVi {
//----------------------------------------------------------------------------
// CLASS       : NSListenerScene3D
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSListenerScene3D,NSListener)
//---------------------- Constructors / Destructor ---------------------------
NSListenerScene3D::NSListenerScene3D(ArCW & arCW) // Protected
  : NSListener(arCW),_scene(), _setupFlag(false), _events()
{
  // Nothing to do
}

NSListenerScene3D::~NSListenerScene3D(void)
{
  // Nothing to do
}
//---------------------------- Public methods --------------------------------
// Controler will sign to all object events and will be able to transmit
// them. (getPendingEvents());
void
NSListenerScene3D::init(ArRef<LocalDepService> service, ArRef<ArObject> object)
{
  if (object->getClass()->isA(Scene3D::CLASS()))
    _scene=ar_down_cast<Scene3D>(object);
  else
    return;

  NSListener::init(service,object);
  _scene->addSceneSetupCB(thisRef(),&NSListenerScene3D::_onSceneSetupCB);
  _scene->addAddObjectCB(thisRef(),&NSListenerScene3D::_onAddObjectCB);
  _scene->addRemoveObjectCB(thisRef(),&NSListenerScene3D::_onRemoveObjectCB);
  _scene->addSelectObjectCB(thisRef(),&NSListenerScene3D::_onSelectObjectCB);
  _scene->addUnselectObjectCB(thisRef(),&NSListenerScene3D::_onUnselectObjectCB);
  _scene->addAddParticleSystemCB(thisRef(),&NSListenerScene3D::_onAddParticleSystemCB);
  _scene->addRemoveParticleSystemCB(thisRef(),&NSListenerScene3D::_onRemoveParticleSystemCB);
  _scene->addAddSoundCB(thisRef(),&NSListenerScene3D::_onAddSoundCB);
  _scene->addRemoveSoundCB(thisRef(),&NSListenerScene3D::_onRemoveSoundCB);
  _scene->addAddLightCB(thisRef(),&NSListenerScene3D::_onAddLightCB);
  _scene->addRemoveLightCB(thisRef(),&NSListenerScene3D::_onRemoveLightCB);
}

//------------------------ Controler is Watcher ------------------------------
// Return events backup (eg : motion, setup change ...)
void
NSListenerScene3D::getPendingEvents(StlList<ArRef<NSEvent> > &events)
{
  if(_setupFlag)
  {
    ArRef<NSEventScene3DSetup> event=NSEventScene3DSetup::NEW();
    event->writeScene3D(_scene);
    events.push_back(event);
    _setupFlag=false;
  }

  StlList<ArRef<NSEvent> >::iterator it=_events.begin();
  StlList<ArRef<NSEvent> >::iterator it_end=_events.end();
  while(it!=it_end)
  {
    events.push_back(*it);
    it++;
  }
  _events.clear();
}

//------------------------- Forbiden operations ------------------------------
// Viewer Setup (fog, background, backTransform...)
void
NSListenerScene3D::_onSceneSetupCB(const Scene3D::SceneSetupEvent &)
{
  _setupFlag=true;
}

// Add Object
void
NSListenerScene3D::_onAddObjectCB(const Scene3D::AddObjectEvent &evt)
{
  if (evt.object.valid())
  {
    ArRef<NSEventScene3DAddObject> event=NSEventScene3DAddObject::NEW();
    event->setRef(evt.object);
    _events.push_back(event);
  }
}

// Remove Object
void
NSListenerScene3D::_onRemoveObjectCB(const Scene3D::RemoveObjectEvent &evt)
{
  if (evt.object.valid())
  {
    ArRef<NSEventScene3DRemoveObject> event=NSEventScene3DRemoveObject::NEW();
    event->setID(evt.object->getAlias());
    _events.push_back(event);
  }
}

// Select Object
void
NSListenerScene3D::_onSelectObjectCB(const Scene3D::SelectObjectEvent &evt)
{
  if (evt.object.valid())
  {
    ArRef<NSEventScene3DSelectObject> event=NSEventScene3DSelectObject::NEW();
    event->setRef(evt.object);
    _events.push_back(event);
  }
}

// Unselect Object
void
NSListenerScene3D::_onUnselectObjectCB(const Scene3D::UnselectObjectEvent &evt)
{
  if (evt.object.valid())
  {
    ArRef<NSEventScene3DUnselectObject> event=NSEventScene3DUnselectObject::NEW();
    event->setID(evt.object->getAlias());
    _events.push_back(event);
  }
}

// Add Light
void
NSListenerScene3D::_onAddLightCB(const Scene3D::AddLightEvent &evt)
{
  if (evt.light.valid())
  {
    ArRef<NSEventScene3DAddLight> event=NSEventScene3DAddLight::NEW();
    event->setRef(evt.light);
    _events.push_back(event);
  }
}

// Remove Light
void
NSListenerScene3D::_onRemoveLightCB(const Scene3D::RemoveLightEvent &evt)
{
  if (evt.light.valid())
  {
    ArRef<NSEventScene3DRemoveLight> event=NSEventScene3DRemoveLight::NEW();
    event->setID(evt.light->getAlias());
    _events.push_back(event);
  }
}

// Add ParticleSystem
void
NSListenerScene3D::_onAddParticleSystemCB(const Scene3D::AddParticleSystemEvent &evt)
{
  if (evt.particleSystem.valid())
  {
    ArRef<NSEventScene3DAddParticleSystem> event=NSEventScene3DAddParticleSystem::NEW();
    event->setRef(evt.particleSystem);
    _events.push_back(event);
  }
}

// Remove ParticleSystem
void
NSListenerScene3D::_onRemoveParticleSystemCB(const Scene3D::RemoveParticleSystemEvent &evt)
{
  if (evt.particleSystem.valid())
  {
    ArRef<NSEventScene3DRemoveParticleSystem> event=NSEventScene3DRemoveParticleSystem::NEW();
    event->setID(evt.particleSystem->getAlias());
    _events.push_back(event);
  }
}

// Add Sound
void
NSListenerScene3D::_onAddSoundCB(const Scene3D::AddSoundEvent &evt)
{
  if (evt.sound.valid())
  {
    ArRef<NSEventScene3DAddSound> event=NSEventScene3DAddSound::NEW();
    event->setRef(evt.sound);
    _events.push_back(event);
  }
}

// Remove Sound
void
NSListenerScene3D::_onRemoveSoundCB(const Scene3D::RemoveSoundEvent &evt)
{
  if (evt.sound.valid())
  {
    ArRef<NSEventScene3DRemoveSound> event=NSEventScene3DRemoveSound::NEW();
    event->setID(evt.sound->getAlias());
    _events.push_back(event);
  }
}

//----------------------------------------------------------------------------
// CLASS       : NSInteractorScene3D
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSInteractorScene3D,NSInteractor)
//---------------------- Constructors / Destructor ---------------------------
NSInteractorScene3D::NSInteractorScene3D(ArCW & arCW) // Protected
  : NSInteractor(arCW),_scene()
{}

NSInteractorScene3D::~NSInteractorScene3D(void)
{
}
//---------------------------- Public methods --------------------------------

// Controler will sign to all object network events and will be able to
// affect remote object with.
void
NSInteractorScene3D::init(ArRef<RemoteDepService> service, ArRef<ArObject> object)
{
  if (object->getClass()->isA(Scene3D::CLASS()))
    _scene=ar_down_cast<Scene3D>(object);
  else
    return;

  NSInteractor::init(service,object);

  _registerEvent(NSEventScene3DSetup::CLASS()->getName(),
                (NetworkService::EventMethod)&NSInteractorScene3D::_onSceneSetupEvent);

  _registerEvent(NSEventScene3DAddObject::CLASS()->getName(),
                (NetworkService::EventMethod)&NSInteractorScene3D::_onAddObjectEvent);
  _registerEvent(NSEventScene3DRemoveObject::CLASS()->getName(),
                (NetworkService::EventMethod)&NSInteractorScene3D::_onRemoveObjectEvent);

  _registerEvent(NSEventScene3DSelectObject::CLASS()->getName(),
                (NetworkService::EventMethod)&NSInteractorScene3D::_onSelectObjectEvent);
  _registerEvent(NSEventScene3DUnselectObject::CLASS()->getName(),
                (NetworkService::EventMethod)&NSInteractorScene3D::_onUnselectObjectEvent);

  _registerEvent(NSEventScene3DAddParticleSystem::CLASS()->getName(),
                (NetworkService::EventMethod)&NSInteractorScene3D::_onAddParticleSystemEvent);
  _registerEvent(NSEventScene3DRemoveParticleSystem::CLASS()->getName(),
                (NetworkService::EventMethod)&NSInteractorScene3D::_onRemoveParticleSystemEvent);

  _registerEvent(NSEventScene3DAddLight::CLASS()->getName(),
                (NetworkService::EventMethod)&NSInteractorScene3D::_onAddLightEvent);
  _registerEvent(NSEventScene3DRemoveLight::CLASS()->getName(),
                (NetworkService::EventMethod)&NSInteractorScene3D::_onRemoveLightEvent);

  _registerEvent(NSEventScene3DAddSound::CLASS()->getName(),
                (NetworkService::EventMethod)&NSInteractorScene3D::_onAddSoundEvent);
  _registerEvent(NSEventScene3DRemoveSound::CLASS()->getName(),
                (NetworkService::EventMethod)&NSInteractorScene3D::_onRemoveSoundEvent);
}

//------------------------- Forbiden operations ------------------------------
// Viewer Setup (fog, background, backTransform...)
NSEvent::EventProcess
NSInteractorScene3D::_onSceneSetupEvent(ArConstRef<NSEvent> eventObj)
{
  ArConstRef<NSEventScene3DSetup> event=ar_down_cast<NSEventScene3DSetup>(eventObj);
  if (!event->readScene3D(_scene,_depService))
  {
    setErrorMessage("Error on SceneSetupEvent : Can't resolve dependencies (Background Texture)");
    return NSEvent::EP_FAIL;
  }
  return NSEvent::EP_OK;
}

// Add Object
NSEvent::EventProcess
NSInteractorScene3D::_onAddObjectEvent(ArConstRef<NSEvent> eventObj)
{
  ArConstRef<NSEventScene3DAddObject> event=ar_down_cast<NSEventScene3DAddObject>(eventObj);
  ArRef<Object3D> object;
  _depService->requestRemoteObject(event->getID(),
                                   event->getClassName(),
                                   object);
  if (!object.valid())
  {
    setErrorMessage("Error on addObjectEvent : (" + event->getID() + " , " + event->getClassName()+")");
    return NSEvent::EP_FAIL;
  }
  _scene->addObject(object);
  return NSEvent::EP_OK;
}

// Remove Object
NSEvent::EventProcess
NSInteractorScene3D::_onRemoveObjectEvent(ArConstRef<NSEvent> eventObj)
{
  ArConstRef<NSEventScene3DRemoveObject> event=ar_down_cast<NSEventScene3DRemoveObject>(eventObj);
  ArRef<Object3D> object;
  _depService->findRemoteObject(event->getID(),object);
  if (object.valid())
    _scene->removeObject(object);
  return NSEvent::EP_OK;
}

// Select Object
NSEvent::EventProcess
NSInteractorScene3D::_onSelectObjectEvent(ArConstRef<NSEvent> eventObj)
{
  ArConstRef<NSEventScene3DSelectObject> event=ar_down_cast<NSEventScene3DSelectObject>(eventObj);
  ArRef<Object3D> object;
  _depService->requestRemoteObject(event->getID(),
                                   event->getClassName(),
                                   object);
  if (!object.valid())
  {
    setErrorMessage("Error on SelectObjectEvent : (" + event->getID() + " , " + event->getClassName()+")");
    return NSEvent::EP_FAIL;
  }
  _scene->selectObject(object);
  return NSEvent::EP_OK;
}

// Unselect Object
NSEvent::EventProcess
NSInteractorScene3D::_onUnselectObjectEvent(ArConstRef<NSEvent> eventObj)
{
  ArConstRef<NSEventScene3DUnselectObject> event=ar_down_cast<NSEventScene3DUnselectObject>(eventObj);
  ArRef<Object3D> object;
  _depService->findRemoteObject(event->getID(),object);
  if (object.valid())
    _scene->unselectObject(object);
  return NSEvent::EP_OK;
}

// Add Light
NSEvent::EventProcess
NSInteractorScene3D::_onAddLightEvent(ArConstRef<NSEvent> eventObj)
{
  ArConstRef<NSEventScene3DAddLight> event=ar_down_cast<NSEventScene3DAddLight>(eventObj);
  ArRef<Light3D> light;
  _depService->requestRemoteObject(event->getID(),
                                   event->getClassName(),
                                   light);
  if (!light.valid())
  {
    setErrorMessage("Error on addLightEvent : (" + event->getID() + " , " + event->getClassName()+")");
    return NSEvent::EP_FAIL;
  }
  _scene->addLight(light);
  return NSEvent::EP_OK;
}

// Remove Light
NSEvent::EventProcess
NSInteractorScene3D::_onRemoveLightEvent(ArConstRef<NSEvent> eventObj)
{
  ArConstRef<NSEventScene3DRemoveLight> event=ar_down_cast<NSEventScene3DRemoveLight>(eventObj);
  ArRef<Light3D> light;
  _depService->findRemoteObject(event->getID(),light);
  if (light.valid())
    _scene->removeLight(light);
  return NSEvent::EP_OK;
}


// Add ParticleSystem
NSEvent::EventProcess
NSInteractorScene3D::_onAddParticleSystemEvent(ArConstRef<NSEvent> eventObj)
{
  ArConstRef<NSEventScene3DAddParticleSystem> event=ar_down_cast<NSEventScene3DAddParticleSystem>(eventObj);
  ArRef<ParticleSystem> particleSystem;
  _depService->requestRemoteObject(event->getID(),
                                   event->getClassName(),
                                   particleSystem);
  if (!particleSystem.valid())
  {
    setErrorMessage("Error on addParticleSystemEvent : (" + event->getID() + " , " + event->getClassName()+")");
    return NSEvent::EP_FAIL;
  }
  _scene->addParticleSystem(particleSystem);
  return NSEvent::EP_OK;
}

// Remove ParticleSystem
NSEvent::EventProcess
NSInteractorScene3D::_onRemoveParticleSystemEvent(ArConstRef<NSEvent> eventObj)
{
  ArConstRef<NSEventScene3DRemoveParticleSystem> event=ar_down_cast<NSEventScene3DRemoveParticleSystem>(eventObj);
  ArRef<ParticleSystem> particleSystem;
  _depService->findRemoteObject(event->getID(),particleSystem);
  if (particleSystem.valid())
    _scene->removeParticleSystem(particleSystem);
  return NSEvent::EP_OK;
}


// Add Sound
NSEvent::EventProcess
NSInteractorScene3D::_onAddSoundEvent(ArConstRef<NSEvent> eventObj)
{
  ArConstRef<NSEventScene3DAddSound> event=ar_down_cast<NSEventScene3DAddSound>(eventObj);
  ArRef<Sound3D> sound;
  _depService->requestRemoteObject(event->getID(),
                                   event->getClassName(),
                                   sound);
  if (!sound.valid())
  {
    setErrorMessage("Error on addSoundEvent : (" + event->getID() + " , " + event->getClassName()+")");
    return NSEvent::EP_FAIL;
  }
  _scene->addSound(sound);
  return NSEvent::EP_OK;
}

// Remove Sound
NSEvent::EventProcess
NSInteractorScene3D::_onRemoveSoundEvent(ArConstRef<NSEvent> eventObj)
{
  ArConstRef<NSEventScene3DRemoveSound> event=ar_down_cast<NSEventScene3DRemoveSound>(eventObj);
  ArRef<Sound3D> sound;
  _depService->findRemoteObject(event->getID(),sound);
  if (sound.valid())
    _scene->removeSound(sound);
  return NSEvent::EP_OK;
}
} // namespace NetworkServices
//---End-Of-File---------------------------------------------------------------
