// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/Controlers/nsCtrlShape3D.cpp
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReViNS/Controlers/nsCtrlShape3D.h"
#include "AReViNS/Events/nsEventShape3D.h"
#include "AReViNS/remoteDepService.h"

#include "AReVi/Lib3D/transform3D.h"
#include "AReVi/Shapes/shapePart3D.h"



namespace AReVi {
//----------------------------------------------------------------------------
// CLASS       : NSListenerShape3D
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSListenerShape3D,NSListener)
//---------------------- Constructors / Destructor ---------------------------
NSListenerShape3D::NSListenerShape3D(ArCW & arCW) // Protected
  : NSListener(arCW),_shape(),
  _transformationFlag(false),
  _structureChangeFlag(false)
{
}

NSListenerShape3D::~NSListenerShape3D(void)
{}
//---------------------------- Public methods --------------------------------
// Controler will sign to all object events and will be able to transmit
// them. (getPendingEvents());
void
NSListenerShape3D::init(ArRef<LocalDepService> service, ArRef<ArObject> object)
{
  if (object->getClass()->isA(Shape3D::CLASS()))
    _shape=ar_down_cast<Shape3D>(object);
  else
    return;

  NSListener::init(service,object);
  _shape->addTransformationCB(thisRef(),&NSListenerShape3D::_onTransformationCB);
  _shape->addStructureChangeCB(thisRef(),&NSListenerShape3D::_onStructureChangeCB);
}

//------------------------ Controler is Watcher ------------------------------
// Return events backup (eg : motion, setup change ...)
void
NSListenerShape3D::getPendingEvents(StlList<ArRef<NSEvent> > &events)
{
  if (_structureChangeFlag)
  {
    ArRef<NSEventShapePart3DList> event=NSEventShapePart3DList::NEW();
    for (unsigned i=_shape->getNbRootParts();i--;)
      event->addRef(_shape->getRootPart(i));
    events.push_back(event);
    _structureChangeFlag=false;
  }

  if (_transformationFlag)
  {
    ArRef<NSEventTransform3D> event=NSEventTransform3D::NEW();
    ArRef<Transform3D> tr=Transform3D::NEW();
    _shape->readTransformation(tr);
    event->setTransform(tr->getTransformation());
    events.push_back(event);
    _transformationFlag=false;
  }
}

//------------------------- Forbiden operations ------------------------------
void
NSListenerShape3D::_onTransformationCB(const Shape3D::TransformationEvent &)
{
  _transformationFlag=true;
}

void
NSListenerShape3D::_onStructureChangeCB(const Shape3D::StructureChangeEvent &)
{
  _structureChangeFlag=true;
}

//----------------------------------------------------------------------------
// CLASS       : NSInteractorShape3D
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSInteractorShape3D,NSInteractor)
//---------------------- Constructors / Destructor ---------------------------
NSInteractorShape3D::NSInteractorShape3D(ArCW & arCW) // Protected
  : NSInteractor(arCW),_shape()
{}

NSInteractorShape3D::~NSInteractorShape3D(void)
{}
//---------------------------- Public methods --------------------------------
// Controler will sign to all object network events and will be able to
// affect remote object with.
void
NSInteractorShape3D::init(ArRef<RemoteDepService> service, ArRef<ArObject> object)
{
  if (object->getClass()->isA(Shape3D::CLASS()))
    _shape=ar_down_cast<Shape3D>(object);
  else
    return;

  NSInteractor::init(service,object);
  _registerEvent(NSEventTransform3D::CLASS()->getName(),
                (NetworkService::EventMethod)&NSInteractorShape3D::_onTransformationEvent);
  _registerEvent(NSEventShapePart3DList::CLASS()->getName(),
                (NetworkService::EventMethod)&NSInteractorShape3D::_onStructureChangeEvent);
}

//------------------------- Forbiden operations ------------------------------
NSEvent::EventProcess
NSInteractorShape3D::_onTransformationEvent(ArConstRef<NSEvent> eventObj)
{
  ArConstRef<NSEventTransform3D> event=ar_down_cast<NSEventTransform3D>(eventObj);
  ArRef<Transform3D> tr=Transform3D::NEW();
  tr->accessTransformation()=event->getTransform();
  _shape->writeTransformation(tr);
  return NSEvent::EP_OK;
}

NSEvent::EventProcess
NSInteractorShape3D::_onStructureChangeEvent(ArConstRef<NSEvent> eventObj)
{
  ArConstRef<NSEventShapePart3DList> event=ar_down_cast<NSEventShapePart3DList>(eventObj);
  ArRef<ShapePart3D> part;

  for (unsigned i=_shape->getNbRootParts();i--;)
    _shape->removeRootPart(_shape->accessRootPart(i));

  for (unsigned i=event->getRefs().size();i--;)
  {
    _depService->requestRemoteObject(event->getRefs()[i],
                                     event->getClassRefs()[i],
                                     part);
    if (part)
      _shape->addRootPart(part);
    else if (event->getRefs()[i]!="")
    {
      setErrorMessage("Cant't add root part " + event->getRefs()[i] + " class=" + event->getClassRefs()[i]);
      cerr << getErrorMessage() << endl;
    }
  }
  return NSEvent::EP_OK;
}

} // namespace NetworkServices
//---End-Of-File---------------------------------------------------------------
