// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/Controlers/nsCtrlShapePart3D.cpp
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReViNS/Controlers/nsCtrlShapePart3D.h"
#include "AReViNS/Events/nsEventShapePart3D.h"
#include "AReViNS/Events/nsEventShape3D.h"
#include "AReViNS/remoteDepService.h"
#include "AReViNS/localDepService.h"

#include "AReVi/Lib3D/transform3D.h"



namespace AReVi {
//----------------------------------------------------------------------------
// CLASS       : NSListenerShapePart3D
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSListenerShapePart3D,NSListener)
//---------------------- Constructors / Destructor ---------------------------
NSListenerShapePart3D::NSListenerShapePart3D(ArCW & arCW) // Protected
  : NSListener(arCW),_part(),
  _geometryRebuildFlag(false),
  _appearanceRebuildFlag(false),
  _transformationFlag(false),
  _structureChangeFlag(false)
{

}

NSListenerShapePart3D::~NSListenerShapePart3D(void)
{
}
//---------------------------- Public methods --------------------------------
// Controler will sign to all object events and will be able to transmit
// them. (getPendingEvents());
void
NSListenerShapePart3D::init(ArRef<LocalDepService> service, ArRef<ArObject> object)
{
  if (object->getClass()->isA(ShapePart3D::CLASS()))
    _part=ar_down_cast<ShapePart3D>(object);
  else
    return;

  NSListener::init(service,object);
 // cerr << "addPostRebuildCB to " << _part->getAlias() << endl;
  _part->addTransformationCB(thisRef(),&NSListenerShapePart3D::_onTransformationCB);
  _part->addStructureChangeCB(thisRef(),&NSListenerShapePart3D::_onStructureChangeCB);
  _part->addAppearanceRebuildCB(thisRef(),&NSListenerShapePart3D::_onAppearanceRebuildCB);
  _part->addGeometryRebuildCB(thisRef(),&NSListenerShapePart3D::_onGeometryRebuildCB);
}

//------------------------ Controler is Watcher ------------------------------
// Return events backup (eg : motion, setup change ...)
void
NSListenerShapePart3D::getPendingEvents(StlList<ArRef<NSEvent> > &events)
{
  if (_geometryRebuildFlag)
  {
    _depService->shareObject(_part); // TODO : Ne plus sérialiser l'objet!!!
    ArRef<NSEventShapePart3DGeometryRebuild> event=NSEventShapePart3DGeometryRebuild::NEW();
    //event->setID(_part->getAlias());
    events.push_back(event);
    _geometryRebuildFlag=false;

    _appearanceRebuildFlag=false; // tempo : tant qu'il sérialise tout c'est pas la peine de faire le reste!
    _transformationFlag=false;
    _structureChangeFlag=false;
  }

  if (_appearanceRebuildFlag)
  {
    _depService->shareObject(_part); // TODO : Ne plus sérialiser l'objet!!!
    ArRef<NSEventShapePart3DAppearanceRebuild> event=NSEventShapePart3DAppearanceRebuild::NEW();
    //event->setID(_part->getAlias());
    events.push_back(event);
    _appearanceRebuildFlag=false;

    _geometryRebuildFlag=false; // tempo : tant qu'il sérialise tout c'est pas la peine de faire le reste!
    _transformationFlag=false;
    _structureChangeFlag=false;
  }
  if (_transformationFlag)
  {
    ArRef<NSEventTransform3D> event=NSEventTransform3D::NEW();
    ArRef<Transform3D> tr=Transform3D::NEW();
      _part->readTransformation(tr);
      event->setTransform(tr->getTransformation());
      events.push_back(event);
      _transformationFlag=false;
  }
  if (_structureChangeFlag)
  {
    ArRef<NSEventShapePart3DList> event=NSEventShapePart3DList::NEW();
    for (unsigned i=_part->getNbSubParts();i--;)
      event->addRef(_part->getSubPart(i));
    events.push_back(event);
    _structureChangeFlag=false;
  }
  NSListener::getPendingEvents(events);
}

//------------------------- Forbiden operations ------------------------------
void
NSListenerShapePart3D::_onTransformationCB(const ShapePart3D::TransformationEvent &)
{
  _transformationFlag=true;
}

void
NSListenerShapePart3D::_onStructureChangeCB(const ShapePart3D::StructureChangeEvent &)
{
//cerr << "PostRebuild" << endl;
  _structureChangeFlag=true;
}

void
NSListenerShapePart3D::_onAppearanceRebuildCB(const ShapePart3D::AppearanceRebuildEvent &)
{
  _appearanceRebuildFlag=true;
}

void
NSListenerShapePart3D::_onGeometryRebuildCB(const ShapePart3D::GeometryRebuildEvent &)
{
//cerr << "PostRebuild" << endl;
  _geometryRebuildFlag=true;
}

//----------------------------------------------------------------------------
// CLASS       : NSInteractorShapePart3D
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSInteractorShapePart3D,NSInteractor)
//---------------------- Constructors / Destructor ---------------------------
NSInteractorShapePart3D::NSInteractorShapePart3D(ArCW & arCW) // Protected
  : NSInteractor(arCW),_part()
{

}

NSInteractorShapePart3D::~NSInteractorShapePart3D(void)
{
}
//---------------------------- Public methods --------------------------------
// Controler will sign to all object network events and will be able to
// affect remote object with.
void
NSInteractorShapePart3D::init(ArRef<RemoteDepService> service, ArRef<ArObject> object)
{
  if (object->getClass()->isA(ShapePart3D::CLASS()))
    _part=ar_down_cast<ShapePart3D>(object);
  else
    return;

  NSInteractor::init(service,object);
  _registerEvent(NSEventTransform3D::CLASS()->getName(),
                (NetworkService::EventMethod)&NSInteractorShapePart3D::_onTransformationEvent);
  _registerEvent(NSEventShapePart3DList::CLASS()->getName(),
                (NetworkService::EventMethod)&NSInteractorShapePart3D::_onStructureChangeEvent);
  _registerEvent(NSEventShapePart3DAppearanceRebuild::CLASS()->getName(),
                (NetworkService::EventMethod)&NSInteractorShapePart3D::_onAppearanceRebuildEvent);
  _registerEvent(NSEventShapePart3DGeometryRebuild::CLASS()->getName(),
                (NetworkService::EventMethod)&NSInteractorShapePart3D::_onGeometryRebuildEvent);


}

//------------------------- Forbiden operations ------------------------------
NSEvent::EventProcess
NSInteractorShapePart3D::_onTransformationEvent(ArConstRef<NSEvent> eventObj)
{
  ArConstRef<NSEventTransform3D> event=ar_down_cast<NSEventTransform3D>(eventObj);
  ArRef<Transform3D> tr=Transform3D::NEW();
  tr->accessTransformation()=event->getTransform();
  _part->writeTransformation(tr);
  return NSEvent::EP_OK;
}

NSEvent::EventProcess
NSInteractorShapePart3D::_onStructureChangeEvent(ArConstRef<NSEvent> eventObj)
{
  ArConstRef<NSEventShapePart3DList> event=ar_down_cast<NSEventShapePart3DList>(eventObj);
  ArRef<ShapePart3D> part;

  for (unsigned i=_part->getNbSubParts();i--;)
    _part->removeSubPart(_part->accessSubPart(i));

  for (unsigned i=event->getRefs().size();i--;)
  {
    _depService->requestRemoteObject(event->getRefs()[i],
                                     event->getClassRefs()[i],
                                     part);
    if (part)
      _part->addSubPart(part);
    else if (event->getRefs()[i]!="")
    {
      setErrorMessage("Cant't add Sub part " + event->getRefs()[i] + " class=" + event->getClassRefs()[i]);
      cerr << getErrorMessage() << endl;
    }
  }
  return NSEvent::EP_OK;
}

NSEvent::EventProcess
NSInteractorShapePart3D::_onAppearanceRebuildEvent(ArConstRef<NSEvent> /*eventObj*/)
{
  /*ArConstRef<NSEventShapePart3DAppearanceRebuild> event=ar_down_cast<NSEventShapePart3DAppearanceRebuild>(eventObj);
  _depService->updateRemoteReference(_part->getAlias());
  */
  return NSEvent::EP_OK;
}

NSEvent::EventProcess
NSInteractorShapePart3D::_onGeometryRebuildEvent(ArConstRef<NSEvent> /*eventObj*/)
{
  /*ArConstRef<NSEventShapePart3DGeometryRebuild> event=ar_down_cast<NSEventShapePart3DGeometryRebuild>(eventObj);
  _depService->updateRemoteReference(_part->getAlias());*/
  return NSEvent::EP_OK;
}

} // namespace NetworkServices
//---End-Of-File---------------------------------------------------------------
