// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/Controlers/nsCtrlSound3D.cpp
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReViNS/Controlers/nsCtrlSound3D.h"
#include "AReViNS/Events/nsEventSound3D.h"

#include "AReViNS/remoteDepService.h"
#include "AReVi/Lib3D/sound3D.h"
#include "AReVi/Utils/abstractStream.h"



namespace AReVi {
//----------------------------------------------------------------------------
// CLASS       : NSListenerSound3D
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSListenerSound3D,NSListenerBase3D)
//---------------------- Constructors / Destructor ---------------------------
NSListenerSound3D::NSListenerSound3D(ArCW & arCW) // Protected
  : NSListenerBase3D(arCW),
    _snd(),
    _setupFlag(false),
    _events()
{}

NSListenerSound3D::~NSListenerSound3D(void)
{
}
//---------------------------- Public methods --------------------------------
// Controler will sign to all object events and will be able to transmit
// them. (getPendingEvents());
void
NSListenerSound3D::init(ArRef<LocalDepService> service, ArRef<ArObject> object)
{
  if (object->getClass()->isA(Sound3D::CLASS()))
    _snd=ar_down_cast<Sound3D>(object);
  else
    return;

  NSListenerBase3D::init(service,object);
  _snd->addSetupCB(thisRef(),&NSListenerSound3D::_onSetupCB);
  _snd->addControlCB(thisRef(),&NSListenerSound3D::_onControlCB);
}

//------------------------ Controler is Watcher ------------------------------
// Return events backup (eg : motion, setup change ...)
void
NSListenerSound3D::getPendingEvents(StlList<ArRef<NSEvent> > &events)
{
  NSListenerBase3D::getPendingEvents(events);

  if (_setupFlag)
  {
  //  cerr << "NSListenerSound3D::getPendingEvents _motionFlag=" << _motionFlag <<endl;
    ArRef<NSEventSound3DSetup> event=NSEventSound3DSetup::NEW();
    event->writeSound3D(_snd);
    events.push_back(event);
    _setupFlag=false;
  }
  /*if (_controlFlag)
  {
  cerr << "ON CONTROL EVENT" << endl;
  //  cerr << "NSListenerSound3D::getPendingEvents _motionFlag=" << _motionFlag <<endl;
    ArRef<NSEventSound3DControl> event=NSEventSound3DControl::NEW();
    event->writeSound3D(_snd);
    events.push_back(event);
    _controlFlag=false;
  }*/
  StlList<ArRef<NSEvent> >::iterator it=_events.begin();
  StlList<ArRef<NSEvent> >::iterator it_end=_events.end();
  while(it!=it_end)
  {
    events.push_back(*it);
    it++;
  }
  _events.clear();
}

//------------------------- Forbiden operations ------------------------------
void
NSListenerSound3D::_onSetupCB(const Sound3D::SetupEvent &)
{
  _setupFlag=true;
}

void
NSListenerSound3D::_onControlCB(const Sound3D::ControlEvent &)
{
  //_controlFlag=true;
  //cerr << "ON CONTROL EVENT" << endl;
  //  cerr << "NSListenerSound3D::getPendingEvents _motionFlag=" << _motionFlag <<endl;
  ArRef<NSEventSound3DControl> event=NSEventSound3DControl::NEW();
  event->writeSound3D(_snd);
  _events.push_back(event);
}

//----------------------------------------------------------------------------
// CLASS       : NSInteractorSound3D
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSInteractorSound3D,NSInteractorBase3D)
//---------------------- Constructors / Destructor ---------------------------
NSInteractorSound3D::NSInteractorSound3D(ArCW & arCW) // Protected
  : NSInteractorBase3D(arCW),_snd()
{}

NSInteractorSound3D::~NSInteractorSound3D(void)
{}
//---------------------------- Public methods --------------------------------

// Controler will sign to all object network events and will be able to
// affect remote object with.
void
NSInteractorSound3D::init(ArRef<RemoteDepService> service, ArRef<ArObject> object)
{
  if (object->getClass()->isA(Sound3D::CLASS()))
    _snd=ar_down_cast<Sound3D>(object);
  else
  {
    cerr << "Object " << object->getAlias() << " not a sound3D CLASS" << endl;
    return;
  }
  NSInteractorBase3D::init(service,object);

  _registerEvent(NSEventSound3DSetup::CLASS()->getName(),
                (NetworkService::EventMethod)&NSInteractorSound3D::_onSetupEvent);
  _registerEvent(NSEventSound3DControl::CLASS()->getName(),
                (NetworkService::EventMethod)&NSInteractorSound3D::_onControlEvent);

}

//------------------------- Forbiden operations ------------------------------
NSEvent::EventProcess
NSInteractorSound3D::_onSetupEvent(ArConstRef<NSEvent> eventObj)
{
  ArConstRef<NSEventSound3DSetup> event=ar_down_cast<NSEventSound3DSetup>(eventObj);

  if (_snd.valid())
  {
    event->readSound3D(_snd);

  /*  StlString localURL;
    if (_depService->findRemoteURL(event->getURL(),localURL)) // check URL
      event->readSound3D(_snd,localURL);
    else
    {
      //cerr << "SOUND CHANGED REQUEST SOUND URL=" << event->getURL() << endl;
    // Request URL on server and wait for it.
      _depService->addRemoteURL(event->getURL());
      return NSEvent::EP_LATER;// lock sounds event until sound is not arrived.
    }*/
  }
  return NSEvent::EP_OK;
}

NSEvent::EventProcess
NSInteractorSound3D::_onControlEvent(ArConstRef<NSEvent> eventObj)
{
  ArConstRef<NSEventSound3DControl> event=ar_down_cast<NSEventSound3DControl>(eventObj);

  if (_snd.valid())
    {
    //cerr << "ON CONTROL EVENT" << endl;
    event->readSound3D(_snd);
    }

  return NSEvent::EP_OK;
}
} // namespace NetworkServices

//---End-Of-File---------------------------------------------------------------
