// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/Controlers/nsCtrlSphere3D.cpp
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReViNS/Controlers/nsCtrlSphere3D.h"
#include "AReViNS/Events/nsEventSphere3D.h"
#include "AReVi/Shapes/sphere3D.h"

namespace AReVi {
//----------------------------------------------------------------------------
// CLASS       : NSListenerSphere3D
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSListenerSphere3D,NSListenerSurface3D)
//---------------------- Constructors / Destructor ---------------------------
NSListenerSphere3D::NSListenerSphere3D(ArCW & arCW) // Protected
  : NSListenerSurface3D(arCW),_sphere()
{}

NSListenerSphere3D::~NSListenerSphere3D(void)
{}
//---------------------------- Public methods --------------------------------
// Controler will sign to all object events and will be able to transmit
// them. (getPendingEvents());
void
NSListenerSphere3D::init(ArRef<LocalDepService> service, ArRef<ArObject> object)
{
  if (object->getClass()->isA(Sphere3D::CLASS()))
    _sphere=ar_down_cast<Sphere3D>(object);
  else
    return;

  NSListenerSurface3D::init(service,object);
}

//------------------------ Controler is Watcher ------------------------------
// Return events backup (eg : motion, setup change ...)
void
NSListenerSphere3D::getPendingEvents(StlList<ArRef<NSEvent> > &events)
{
  if (_geometryRebuildFlag)
  {
    ArRef<NSEventSphere3DGeometryRebuild> event=NSEventSphere3DGeometryRebuild::NEW();
    event->writeGeometry(_sphere);
    events.push_back(event);
    _geometryRebuildFlag=false;
  }

  NSListenerSurface3D::getPendingEvents(events);
}

//----------------------------------------------------------------------------
// CLASS       : NSInteractorSphere3D
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSInteractorSphere3D,NSInteractorSurface3D)
//---------------------- Constructors / Destructor ---------------------------
NSInteractorSphere3D::NSInteractorSphere3D(ArCW & arCW) // Protected
  : NSInteractorSurface3D(arCW),_sphere()
{}

NSInteractorSphere3D::~NSInteractorSphere3D(void)
{}
//---------------------------- Public methods --------------------------------
// Controler will sign to all object network events and will be able to
// affect remote object with.
void
NSInteractorSphere3D::init(ArRef<RemoteDepService> service, ArRef<ArObject> object)
{
  if (object->getClass()->isA(Sphere3D::CLASS()))
    _sphere=ar_down_cast<Sphere3D>(object);
  else
    return;

  NSInteractorSurface3D::init(service,object);
  _unregisterEvent(NSEventShapePart3DGeometryRebuild::CLASS()->getName());

  _registerEvent(NSEventSphere3DGeometryRebuild::CLASS()->getName(),
                (NetworkService::EventMethod)&NSInteractorSphere3D::_onGeometryRebuildEvent);
}

//------------------------- Forbiden operations ------------------------------
NSEvent::EventProcess
NSInteractorSphere3D::_onGeometryRebuildEvent(ArConstRef<NSEvent> eventObj)
{
  NSInteractorSurface3D::_onGeometryRebuildEvent(eventObj);
  ArConstRef<NSEventSphere3DGeometryRebuild> event=ar_down_cast<NSEventSphere3DGeometryRebuild>(eventObj);
  event->readGeometry(_sphere);
  return NSEvent::EP_OK;
}

} // namespace NetworkServices
//---End-Of-File---------------------------------------------------------------
