// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/Controlers/nsCtrlSurface3D.cpp
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------


#include "AReViNS/Controlers/nsCtrlSurface3D.h"
#include "AReViNS/Events/nsEventSurface3D.h"

namespace AReVi {
//----------------------------------------------------------------------------
// CLASS       : NSListenerSurface3D
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSListenerSurface3D,NSListenerShapePart3D)
//---------------------- Constructors / Destructor ---------------------------
NSListenerSurface3D::NSListenerSurface3D(ArCW & arCW) // Protected
  : NSListenerShapePart3D(arCW),_surface()
{}

NSListenerSurface3D::~NSListenerSurface3D(void)
{}
//---------------------------- Public methods --------------------------------
// Controler will sign to all object events and will be able to transmit
// them. (getPendingEvents());
void
NSListenerSurface3D::init(ArRef<LocalDepService> service, ArRef<ArObject> object)
{
  if (object->getClass()->isA(Surface3D::CLASS()))
    _surface=ar_down_cast<Surface3D>(object);
  else
    return;

  NSListenerShapePart3D::init(service,object);
}

//------------------------ Controler is Watcher ------------------------------
// Return events backup (eg : motion, setup change ...)
void
NSListenerSurface3D::getPendingEvents(StlList<ArRef<NSEvent> > &events)
{
  if (_appearanceRebuildFlag)
  {
    ArRef<NSEventSurface3DAppearanceRebuild> event=NSEventSurface3DAppearanceRebuild::NEW();
    event->writeAppearance(_surface);
    events.push_back(event);
    _appearanceRebuildFlag=false;
  }

  NSListenerShapePart3D::getPendingEvents(events);
}

//----------------------------------------------------------------------------
// CLASS       : NSInteractorSurface3D
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSInteractorSurface3D,NSInteractorShapePart3D)
//---------------------- Constructors / Destructor ---------------------------
NSInteractorSurface3D::NSInteractorSurface3D(ArCW & arCW) // Protected
  : NSInteractorShapePart3D(arCW),_surface()
{}

NSInteractorSurface3D::~NSInteractorSurface3D(void)
{}
//---------------------------- Public methods --------------------------------
// Controler will sign to all object network events and will be able to
// affect remote object with.
void
NSInteractorSurface3D::init(ArRef<RemoteDepService> service, ArRef<ArObject> object)
{
  if (object->getClass()->isA(Surface3D::CLASS()))
    _surface=ar_down_cast<Surface3D>(object);
  else
    return;

  NSInteractorShapePart3D::init(service,object);
  _unregisterEvent(NSEventShapePart3DAppearanceRebuild::CLASS()->getName());

  _registerEvent(NSEventSurface3DAppearanceRebuild::CLASS()->getName(),
                (NetworkService::EventMethod)&NSInteractorSurface3D::_onAppearanceRebuildEvent);
}

//------------------------- Forbiden operations ------------------------------
NSEvent::EventProcess
NSInteractorSurface3D::_onAppearanceRebuildEvent(ArConstRef<NSEvent> eventObj)
{
  NSInteractorShapePart3D::_onAppearanceRebuildEvent(eventObj);
  ArConstRef<NSEventSurface3DAppearanceRebuild> event=ar_down_cast<NSEventSurface3DAppearanceRebuild>(eventObj);
  event->readAppearance(_surface,_depService);
  //_depService->updateRemoteReference(_surface->getAlias());

  return NSEvent::EP_OK;
}

} // namespace NetworkServices
//---End-Of-File---------------------------------------------------------------
