// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/Controlers/nsCtrlText3D.cpp
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReViNS/Controlers/nsCtrlText3D.h"
#include "AReViNS/Events/nsEventText3D.h"
#include "AReVi/Shapes/text3D.h"

namespace AReVi {
//----------------------------------------------------------------------------
// CLASS       : NSListenerText3D
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSListenerText3D,NSListenerSurface3D)
//---------------------- Constructors / Destructor ---------------------------
NSListenerText3D::NSListenerText3D(ArCW & arCW) // Protected
  : NSListenerSurface3D(arCW),_text()
{}

NSListenerText3D::~NSListenerText3D(void)
{}
//---------------------------- Public methods --------------------------------
// Controler will sign to all object events and will be able to transmit
// them. (getPendingEvents());
void
NSListenerText3D::init(ArRef<LocalDepService> service, ArRef<ArObject> object)
{
  if (object->getClass()->isA(Text3D::CLASS()))
    _text=ar_down_cast<Text3D>(object);
  else
    return;

  NSListenerSurface3D::init(service,object);
}

//------------------------ Controler is Watcher ------------------------------
// Return events backup (eg : motion, setup change ...)
void
NSListenerText3D::getPendingEvents(StlList<ArRef<NSEvent> > &events)
{
  if (_geometryRebuildFlag)
  {
    ArRef<NSEventText3DGeometryRebuild> event=NSEventText3DGeometryRebuild::NEW();
    event->writeGeometry(_text);
    events.push_back(event);
    _geometryRebuildFlag=false;
  }

  NSListenerSurface3D::getPendingEvents(events);
}

//----------------------------------------------------------------------------
// CLASS       : NSInteractorText3D
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSInteractorText3D,NSInteractorSurface3D)
//---------------------- Constructors / Destructor ---------------------------
NSInteractorText3D::NSInteractorText3D(ArCW & arCW) // Protected
  : NSInteractorSurface3D(arCW),_text()
{}

NSInteractorText3D::~NSInteractorText3D(void)
{
//cerr << "NSInteractorText3D::~NSInteractorText3D" << endl;
}
//---------------------------- Public methods --------------------------------
// Controler will sign to all object network events and will be able to
// affect remote object with.
void
NSInteractorText3D::init(ArRef<RemoteDepService> service, ArRef<ArObject> object)
{
  if (object->getClass()->isA(Text3D::CLASS()))
    _text=ar_down_cast<Text3D>(object);
  else
    return;

  NSInteractorSurface3D::init(service,object);
  _unregisterEvent(NSEventShapePart3DGeometryRebuild::CLASS()->getName());

  _registerEvent(NSEventText3DGeometryRebuild::CLASS()->getName(),
                (NetworkService::EventMethod)&NSInteractorText3D::_onGeometryRebuildEvent);
}

//------------------------- Forbiden operations ------------------------------
NSEvent::EventProcess
NSInteractorText3D::_onGeometryRebuildEvent(ArConstRef<NSEvent> eventObj)
{
  NSInteractorSurface3D::_onGeometryRebuildEvent(eventObj);
  ArConstRef<NSEventText3DGeometryRebuild> event=ar_down_cast<NSEventText3DGeometryRebuild>(eventObj);
  if (!event->readGeometry(_text,_depService))
  {
    //cerr << "REQUEST MISSING URL" << endl;
    return NSEvent::EP_LATER;
    // NOTE : => no other message must be sent to this controler until dependance is resolved
    //        => no EP_POST in this case!!
  }
  return NSEvent::EP_OK;
}

} // namespace NetworkServices
//---End-Of-File---------------------------------------------------------------
