// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/Controlers/nsCtrlTextureRenderer3D.cpp
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReViNS/Controlers/nsCtrlTextureRenderer3D.h"
#include "AReViNS/Events/nsEventTextureRenderer3D.h"

#include "AReViNS/remoteDepService.h"

#include "AReVi/Utils/abstractStream.h"
#include "AReVi/Lib3D/scene3D.h"
#include "AReVi/Lib3D/textureRenderer3D.h"



namespace AReVi {
//----------------------------------------------------------------------------
// CLASS       : NSListenerTextureRenderer3D
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSListenerTextureRenderer3D,NSListenerRenderer3D)
//---------------------- Constructors / Destructor ---------------------------
NSListenerTextureRenderer3D::NSListenerTextureRenderer3D(ArCW & arCW) // Protected
  : NSListenerRenderer3D(arCW),
    _textureRenderer(),
    _renderFlag(false)
{}

NSListenerTextureRenderer3D::~NSListenerTextureRenderer3D(void)
{
}
//---------------------------- Public methods --------------------------------
// Controler will sign to all object events and will be able to transmit
// them. (getPendingEvents());
void
NSListenerTextureRenderer3D::init(ArRef<LocalDepService> service, ArRef<ArObject> object)
{
  if (object->getClass()->isA(TextureRenderer3D::CLASS()))
    _textureRenderer=ar_down_cast<TextureRenderer3D>(object);
  else
    return;

  NSListenerRenderer3D::init(service,object);
  _textureRenderer->addRenderCB(thisRef(),&NSListenerTextureRenderer3D::_onRenderCB);
}

//------------------------ Controler is Watcher ------------------------------
// Return events backup (eg : motion, setup change ...)
void
NSListenerTextureRenderer3D::getPendingEvents(StlList<ArRef<NSEvent> > &events)
{
  if (_renderFlag)
  {
  //  cerr << "NSListenerTextureRenderer3D::getPendingEvents _motionFlag=" << _motionFlag <<endl;
    ArRef<NSEventTextureRenderer3DRender> event=NSEventTextureRenderer3DRender::NEW();
    events.push_back(event);
    _renderFlag=false;
  }
  if (_renderingSetupFlag)
  {
  //  cerr << "NSListenerTextureRenderer3D::getPendingEvents _motionFlag=" << _motionFlag <<endl;
    ArRef<NSEventTextureRenderer3DRenderingSetup> event=NSEventTextureRenderer3DRenderingSetup::NEW();

    ArConstRef<Scene3D> scene=_textureRenderer->getScene();
    if (scene.valid())
      event->setMainScene(scene->getAlias(),scene->getClass()->getName());
    else
      event->setMainScene("","");

    scene=_textureRenderer->getFrontScene();
    if (scene.valid())
      event->setFrontScene(scene->getAlias(),scene->getClass()->getName());
    else
      event->setFrontScene("","");

    scene=_textureRenderer->getBackScene();
    if (scene.valid())
      event->setBackScene(scene->getAlias(),scene->getClass()->getName());
    else
      event->setBackScene("","");

    events.push_back(event);

    event->writeTextureRenderer3D(_textureRenderer);
    _renderingSetupFlag=false;
  }
  NSListenerRenderer3D::getPendingEvents(events);
}

//------------------------- Forbiden operations ------------------------------
void
NSListenerTextureRenderer3D::_onRenderCB(const TextureRenderer3D::RenderEvent &)
{
  _renderFlag=true;
}


//----------------------------------------------------------------------------
// CLASS       : NSInteractorTextureRenderer3D
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSInteractorTextureRenderer3D,NSInteractorRenderer3D)
//---------------------- Constructors / Destructor ---------------------------
NSInteractorTextureRenderer3D::NSInteractorTextureRenderer3D(ArCW & arCW) // Protected
  : NSInteractorRenderer3D(arCW),_textureRenderer()
{}

NSInteractorTextureRenderer3D::~NSInteractorTextureRenderer3D(void)
{}
//---------------------------- Public methods --------------------------------

// Controler will sign to all object network events and will be able to
// affect remote object with.
void
NSInteractorTextureRenderer3D::init(ArRef<RemoteDepService> service, ArRef<ArObject> object)
{
  if (object->getClass()->isA(TextureRenderer3D::CLASS()))
    _textureRenderer=ar_down_cast<TextureRenderer3D>(object);
  else
    return;

  NSInteractorRenderer3D::init(service,object);

  _unregisterEvent(NSEventRenderer3DRenderingSetup::CLASS()->getName());

  _registerEvent(NSEventTextureRenderer3DRenderingSetup::CLASS()->getName(),
                (NetworkService::EventMethod)&NSInteractorTextureRenderer3D::_onRenderingSetupEvent);
  _registerEvent(NSEventTextureRenderer3DRender::CLASS()->getName(),
                (NetworkService::EventMethod)&NSInteractorTextureRenderer3D::_onRenderEvent);
}

//------------------------- Forbiden operations ------------------------------
NSEvent::EventProcess
NSInteractorTextureRenderer3D::_onRenderEvent(ArConstRef<NSEvent>)
{
  _textureRenderer->render();
  return NSEvent::EP_OK;
}

NSEvent::EventProcess
NSInteractorTextureRenderer3D::_onRenderingSetupEvent(ArConstRef<NSEvent> eventObj)
{
  NSEvent::EventProcess ret=NSInteractorRenderer3D::_onRenderingSetupEvent(eventObj);
  if (ret!=NSEvent::EP_OK)
    return ret;

//  cerr << "NSInteractorTextureRenderer3D::_onRenderingSetupEvent"<< endl;
  ArConstRef<NSEventTextureRenderer3DRenderingSetup> event=ar_down_cast<NSEventTextureRenderer3DRenderingSetup>(eventObj);
  event->readTextureRenderer3D(_textureRenderer);
  return NSEvent::EP_OK;
}


} // namespace NetworkServices

//---End-Of-File---------------------------------------------------------------
