// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/Controlers/nsCtrlViewer3D.cpp
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReViNS/Controlers/nsCtrlViewer3D.h"
#include "AReViNS/Events/nsEventViewer3D.h"

#include "AReVi/Utils/abstractStream.h"



namespace AReVi {
//----------------------------------------------------------------------------
// CLASS       : NSListenerViewer3D
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSListenerViewer3D,NSListenerWindow3D)
//---------------------- Constructors / Destructor ---------------------------
NSListenerViewer3D::NSListenerViewer3D(ArCW & arCW) // Protected
  : NSListenerWindow3D(arCW),
    _viewer(),
    _appearanceFlag(false)
{}

NSListenerViewer3D::~NSListenerViewer3D(void)
{
}
//---------------------------- Public methods --------------------------------
// Controler will sign to all object events and will be able to transmit
// them. (getPendingEvents());
void
NSListenerViewer3D::init(ArRef<LocalDepService> service, ArRef<ArObject> object)
{
  if (object->getClass()->isA(Viewer3D::CLASS()))
    _viewer=ar_down_cast<Viewer3D>(object);
  else
    return;

  NSListenerWindow3D::init(service,object);
  _viewer->addAppearanceCB(thisRef(),&NSListenerViewer3D::_onAppearanceCB);

  // NOTICE : Windows3D and derived objets are particulars objects.
  _appearanceFlag=true;
}

//------------------------ Controler is Watcher ------------------------------
// Return events backup (eg : motion, setup change ...)
void
NSListenerViewer3D::getPendingEvents(StlList<ArRef<NSEvent> > &events)
{
  NSListenerWindow3D::getPendingEvents(events);

  if (_appearanceFlag)
  {
  //  cerr << "NSListenerViewer3D::getPendingEvents _motionFlag=" << _motionFlag <<endl;
    ArRef<NSEventViewer3DAppearance> event=NSEventViewer3DAppearance::NEW();
    event->writeViewerAppearance(_viewer);
    events.push_back(event);
    _appearanceFlag=false;
  }
}

//------------------------- Forbiden operations ------------------------------
void
NSListenerViewer3D::_onAppearanceCB(const Viewer3D::AppearanceEvent &)
{
  //cerr <<"NSListenerViewer3D::_onAppearanceCB" << endl;
  _appearanceFlag=true;
}

//----------------------------------------------------------------------------
// CLASS       : NSInteractorViewer3D
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSInteractorViewer3D,NSInteractorWindow3D)
//---------------------- Constructors / Destructor ---------------------------
NSInteractorViewer3D::NSInteractorViewer3D(ArCW & arCW) // Protected
  : NSInteractorWindow3D(arCW),_viewer()
{}

NSInteractorViewer3D::~NSInteractorViewer3D(void)
{}
//---------------------------- Public methods --------------------------------

// Controler will sign to all object network events and will be able to
// affect remote object with.
void
NSInteractorViewer3D::init(ArRef<RemoteDepService> service, ArRef<ArObject> object)
{
  if (object->getClass()->isA(Viewer3D::CLASS()))
    _viewer=ar_down_cast<Viewer3D>(object);
  else
    return;

  NSInteractorWindow3D::init(service,object);

  _registerEvent(NSEventViewer3DAppearance::CLASS()->getName(),
                (NetworkService::EventMethod)&NSInteractorViewer3D::_onAppearanceEvent);
}

//------------------------- Forbiden operations ------------------------------
NSEvent::EventProcess
NSInteractorViewer3D::_onAppearanceEvent(ArConstRef<NSEvent> eventObj)
{
 // cerr << "NSInteractorViewer3D::_onAppearanceEvent"<< endl;
  ArConstRef<NSEventViewer3DAppearance> event=ar_down_cast<NSEventViewer3DAppearance>(eventObj);
  event->readViewerAppearance(_viewer);
  return NSEvent::EP_OK;
}

} // namespace NetworkServices

//---End-Of-File---------------------------------------------------------------
