// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/Controlers/nsCtrlWindow3D.cpp
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReViNS/Controlers/nsCtrlWindow3D.h"
#include "AReViNS/Events/nsEventWindow3D.h"
#include "AReViNS/Events/nsEventRenderer3D.h"
#include "AReVi/Lib3D/osd.h"

#include "AReVi/Utils/abstractStream.h"



namespace AReVi {
//----------------------------------------------------------------------------
// CLASS       : NSListenerWindow3D
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSListenerWindow3D,NSListenerRenderer3D)
//---------------------- Constructors / Destructor ---------------------------
NSListenerWindow3D::NSListenerWindow3D(ArCW & arCW) // Protected
  : NSListenerRenderer3D(arCW),
    _window(),
    _cursorFlag(false)
{}

NSListenerWindow3D::~NSListenerWindow3D(void)
{
}
//---------------------------- Public methods --------------------------------
// Controler will sign to all object events and will be able to transmit
// them. (getPendingEvents());
void
NSListenerWindow3D::init(ArRef<LocalDepService> service, ArRef<ArObject> object)
{
  if (object->getClass()->isA(Window3D::CLASS()))
    _window=ar_down_cast<Window3D>(object);
  else
    return;

  NSListenerRenderer3D::init(service,object);
  _window->addCursorCB(thisRef(),&NSListenerWindow3D::_onCursorCB);

  // NOTICE : Windows3D and derived objets are particulars objects.
  _motionFlag=true;           // Base3D
  _appearanceSetupFlag=true;  // Renderer3D
  _viewParametersFlag=true;   // Renderer3D
  _renderingSetupFlag=true;   // Renderer3D
  _cursorFlag=true;           // Window3D

  // Add AllOSD objects
  for (unsigned int i=0;i<_window->getNbOSD();i++) // Renderer3D
  {
    ArRef<NSEventRenderer3DAddOSD> event=NSEventRenderer3DAddOSD::NEW();
    event->setRef(_window->accessOSD(i));
    _events.push_back(event);
  }
}

//------------------------ Controler is Watcher ------------------------------
// Return events backup (eg : motion, setup change ...)
void
NSListenerWindow3D::getPendingEvents(StlList<ArRef<NSEvent> > &events)
{
  NSListenerRenderer3D::getPendingEvents(events);

  if (_cursorFlag)
  {
  //  cerr << "NSListenerWindow3D::getPendingEvents _motionFlag=" << _motionFlag <<endl;
    ArRef<NSEventWindow3DCursor> event=NSEventWindow3DCursor::NEW();
    event->setCursor(_window->getWindowCursor());
    events.push_back(event);
    _cursorFlag=false;
  }
}

//------------------------- Forbiden operations ------------------------------
void
NSListenerWindow3D::_onCursorCB(const Window3D::CursorEvent &)
{
  _cursorFlag=true;
}

//----------------------------------------------------------------------------
// CLASS       : NSInteractorWindow3D
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSInteractorWindow3D,NSInteractorRenderer3D)
//---------------------- Constructors / Destructor ---------------------------
NSInteractorWindow3D::NSInteractorWindow3D(ArCW & arCW) // Protected
  : NSInteractorRenderer3D(arCW),_window()
{}

NSInteractorWindow3D::~NSInteractorWindow3D(void)
{}
//---------------------------- Public methods --------------------------------

// Controler will sign to all object network events and will be able to
// affect remote object with.
void
NSInteractorWindow3D::init(ArRef<RemoteDepService> service, ArRef<ArObject> object)
{
  if (object->getClass()->isA(Window3D::CLASS()))
    _window=ar_down_cast<Window3D>(object);
  else
    return;

  NSInteractorRenderer3D::init(service,object);

  _registerEvent(NSEventWindow3DCursor::CLASS()->getName(),
                (NetworkService::EventMethod)&NSInteractorWindow3D::_onCursorEvent);
}

//------------------------- Forbiden operations ------------------------------
NSEvent::EventProcess
NSInteractorWindow3D::_onCursorEvent(ArConstRef<NSEvent> eventObj)
{
  //cerr << "NSInteractorWindow3D::_onCursorEvent"<< endl;
  ArConstRef<NSEventWindow3DCursor> event=ar_down_cast<NSEventWindow3DCursor>(eventObj);
  _window->setWindowCursor((Window3D::Cursor)event->getCursor());
  return NSEvent::EP_OK;
}

} // namespace NetworkServices

//---End-Of-File---------------------------------------------------------------
