// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/Events/nsEvent.cpp
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReViNS/Events/nsEvent.h"
#include "AReVi/Utils/abstractStream.h"

namespace AReVi {
//----------------------------------------------------------------------------
// CLASS       : NSEvent
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSEvent,ArObject)
//---------------------- Constructors / Destructor ---------------------------
NSEvent::NSEvent(ArCW & arCW) // Protected
  : ArObject(arCW),_service(),_serviceId(-1),_compress(true)
{
  // Nothing to be done
}

NSEvent::~NSEvent(void)
{
  // Nothing to be done
}
//---------------------------- Public methods --------------------------------
//-------------------------- Stream operations -------------------------------
bool // success
NSEvent::readFromStream(ArRef<AbstractIStream> input)
{
  return ArObject::readFromStream(input);
}

void
NSEvent::writeToStream(ArRef<AbstractOStream> output) const
{
  ArObject::writeToStream(output);
}

//-------------------------- Set / Get / Access ------------------------------

//------------------------- Forbiden operations ------------------------------
void
NSEvent::_setNetworkService(ArRef<NetworkService> service,const int &serviceId)
{
  _service=service;
  _serviceId=serviceId;
}


//----------------------------------------------------------------------------
// CLASS       : NSEventConnection
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSEventConnection,NSEvent)
//---------------------- Constructors / Destructor ---------------------------
NSEventConnection::NSEventConnection(ArCW & arCW) // Protected
  : NSEvent(arCW)
{
  // Nothing to be done
}

NSEventConnection::~NSEventConnection(void)
{
  // Nothing to be done
}

//----------------------------------------------------------------------------
// CLASS       : NSEventPingRequest
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSEventPingRequest,NSEvent)
//---------------------- Constructors / Destructor ---------------------------
NSEventPingRequest::NSEventPingRequest(ArCW & arCW) // Protected
  : NSEvent(arCW)
{
  // Nothing to be done
}

NSEventPingRequest::~NSEventPingRequest(void)
{
  // Nothing to be done
}


//----------------------------------------------------------------------------
// CLASS       : NSEventPing
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSEventPing,NSEvent)
//---------------------- Constructors / Destructor ---------------------------
NSEventPing::NSEventPing(ArCW & arCW) // Protected
  : NSEvent(arCW)
{
  // Nothing to be done
}

NSEventPing::~NSEventPing(void)
{
  // Nothing to be done
}



//----------------------------------------------------------------------------
// CLASS       : NSEventRef
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSEventRef,NSEvent)
//---------------------- Constructors / Destructor ---------------------------
NSEventRef::NSEventRef(ArCW & arCW) // Protected
  : NSEvent(arCW),_ID("")
{
  // Nothing to be done
}

NSEventRef::~NSEventRef(void)
{
  // Nothing to be done
}
//---------------------------- Public methods --------------------------------
//-------------------------- Stream operations -------------------------------
bool // success
NSEventRef::readFromStream(ArRef<AbstractIStream> input)
{
  if (!NSEvent::readFromStream(input))
    return false;

  return input->readString(_ID);
}

void
NSEventRef::writeToStream(ArRef<AbstractOStream> output) const
{
  NSEvent::writeToStream(output);

  output->writeString(_ID);
}

//-------------------------- Set / Get / Access ------------------------------
// Get Set ID/alias of removed object3D in Scene3D
const StlString &
NSEventRef::getID(void) const
{
  return _ID;
}

void
NSEventRef::setID(const StlString &ID)
{
  _ID=ID;
}

//----------------------------------------------------------------------------
// CLASS       : NSEventSessionName
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSEventSessionName,NSEventRef)
//---------------------- Constructors / Destructor ---------------------------
NSEventSessionName::NSEventSessionName(ArCW & arCW) // Protected
  : NSEventRef(arCW)
{
  // Nothing to be done
}

NSEventSessionName::~NSEventSessionName(void)
{
  // Nothing to be done
}



//----------------------------------------------------------------------------
// CLASS       : NSEventArRef
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSEventArRef,NSEventRef)
//---------------------- Constructors / Destructor ---------------------------
NSEventArRef::NSEventArRef(ArCW & arCW) // Protected
  : NSEventRef(arCW),_className("")
{
  // Nothing to be done
}

NSEventArRef::~NSEventArRef(void)
{
  // Nothing to be done
}
//---------------------------- Public methods --------------------------------
//-------------------------- Stream operations -------------------------------
bool // success
NSEventArRef::readFromStream(ArRef<AbstractIStream> input)
{
  if (!NSEventRef::readFromStream(input))
    return false;

  return input->readString(_className);
}

void
NSEventArRef::writeToStream(ArRef<AbstractOStream> output) const
{
  NSEventRef::writeToStream(output);

  output->writeString(_className);
}

//-------------------------- Set / Get / Access ------------------------------
// Get Set ID/alias of removed object3D in Scene3D
const StlString &
NSEventArRef::getClassName(void) const
{
  return _className;
}

void
NSEventArRef::setRef(ArConstRef<ArObject> ref)
{
  if (ref.valid())
  {
    _className=ref->getClass()->getName();
    setID(ref->getAlias());
  }
  else
  {
    _className="";
    setID("");
  }
}


} // namespace AReVi
//---End-Of-File---------------------------------------------------------------
