// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/Events/nsEventArConsole.cpp
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReViNS/Events/nsEventArConsole.h"
#include "AReVi/Utils/abstractStream.h"

#include "AReVi/Contrib/arConsole.h"

namespace AReVi {

//----------------------------------------------------------------------------
// CLASS       : NSEventArConsoleAttrib
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSEventArConsoleAttrib,NSEvent)
//---------------------- Constructors / Destructor ---------------------------
NSEventArConsoleAttrib::NSEventArConsoleAttrib(ArCW & arCW) // Protected
  : NSEvent(arCW),
    _r(),
    _g(),
    _b(),
    _a(),
    _font()
{
// Nothing to be done
}

NSEventArConsoleAttrib::~NSEventArConsoleAttrib(void)
{
// Nothing to be done
}

//---------------------------- Public methods --------------------------------
// Read/Write views parameters
void
NSEventArConsoleAttrib::readAttrib(ArConsoleAttr& attribOut) const
{
attribOut.r=_r;
attribOut.g=_g;
attribOut.b=_b;
attribOut.a=_a;
attribOut.font=_font;
}

void
NSEventArConsoleAttrib::writeAttrib(const ArConsoleAttr& attrib)
{
_r=attrib.r;
_g=attrib.g;
_b=attrib.b;
_a=attrib.a;
_font=attrib.font;
}

void
NSEventArConsoleAttrib::writeColor(double r, double g,double b,double a)
{
_r=r;
_g=g;
_b=b;
_a=a;
}

void
NSEventArConsoleAttrib::writeFont(const StlString &font)
{
_font=font;
}

//-------------------------- Stream operations -------------------------------
bool // success
NSEventArConsoleAttrib::readFromStream(ArRef<AbstractIStream> input)
{
//cerr << "NSEventArConsoleBackground::readFromStream" << endl;
if (!NSEvent::readFromStream(input) ||
    !input->readDouble(_r) ||
    !input->readDouble(_g) ||
    !input->readDouble(_b) ||
    !input->readDouble(_a) ||
    !input->readString(_font))
  {
  return false;
  }

return true;
}

void
NSEventArConsoleAttrib::writeToStream(ArRef<AbstractOStream> output) const
{
NSEvent::writeToStream(output);

output->writeDouble(_r);
output->writeDouble(_g);
output->writeDouble(_b);
output->writeDouble(_a);
output->writeString(_font);
}



//----------------------------------------------------------------------------
// CLASS       : NSEventArConsoleBackground
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSEventArConsoleBackground,NSEventArConsoleAttrib)
//---------------------- Constructors / Destructor ---------------------------
NSEventArConsoleBackground::NSEventArConsoleBackground(ArCW & arCW) // Protected
  : NSEventArConsoleAttrib(arCW)
{
// Nothing to be done
}

NSEventArConsoleBackground::~NSEventArConsoleBackground(void)
{
// Nothing to be done
}


//----------------------------------------------------------------------------
// CLASS       : NSEventArConsoleForeground
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSEventArConsoleForeground,NSEventArConsoleAttrib)
//---------------------- Constructors / Destructor ---------------------------
NSEventArConsoleForeground::NSEventArConsoleForeground(ArCW & arCW) // Protected
  : NSEventArConsoleAttrib(arCW)
{
// Nothing to be done
}

NSEventArConsoleForeground::~NSEventArConsoleForeground(void)
{
// Nothing to be done
}


//----------------------------------------------------------------------------
// CLASS       : NSEventArConsolePopAttrib
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSEventArConsolePopAttrib,NSEvent)
//---------------------- Constructors / Destructor ---------------------------
NSEventArConsolePopAttrib::NSEventArConsolePopAttrib(ArCW & arCW) // Protected
  : NSEvent(arCW)
{
// Nothing to be done
}

NSEventArConsolePopAttrib::~NSEventArConsolePopAttrib(void)
{
// Nothing to be done
}


//----------------------------------------------------------------------------
// CLASS       : NSEventArConsoleFlushAttribs
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSEventArConsoleFlushAttribs,NSEvent)
//---------------------- Constructors / Destructor ---------------------------
NSEventArConsoleFlushAttribs::NSEventArConsoleFlushAttribs(ArCW & arCW) // Protected
  : NSEvent(arCW)
{
// Nothing to be done
}

NSEventArConsoleFlushAttribs::~NSEventArConsoleFlushAttribs(void)
{
// Nothing to be done
}


//----------------------------------------------------------------------------
// CLASS       : NSEventArConsoleClearText
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSEventArConsoleClearText,NSEvent)
//---------------------- Constructors / Destructor ---------------------------
NSEventArConsoleClearText::NSEventArConsoleClearText(ArCW & arCW) // Protected
  : NSEvent(arCW)
{
// Nothing to be done
}

NSEventArConsoleClearText::~NSEventArConsoleClearText(void)
{
// Nothing to be done
}



//----------------------------------------------------------------------------
// CLASS       : NSEventArConsoleBufferSetup
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSEventArConsoleBufferSetup,NSEvent)
//---------------------- Constructors / Destructor ---------------------------
NSEventArConsoleBufferSetup::NSEventArConsoleBufferSetup(ArCW & arCW) // Protected
  : NSEvent(arCW),
    _bufferSize(),
    _displaySize()
{
// Nothing to be done
}

NSEventArConsoleBufferSetup::~NSEventArConsoleBufferSetup(void)
{
// Nothing to be done
}

//---------------------------- Public methods --------------------------------
// Read/Write views parameters
void
NSEventArConsoleBufferSetup::readArConsole(ArRef<ArConsole> arConsole) const
{
arConsole->setSize(_displaySize);
arConsole->setBufferSize(_bufferSize);
}

void
NSEventArConsoleBufferSetup::writeArConsole(ArConstRef<ArConsole> arConsole)
{
_bufferSize=arConsole->getBufferSize();
_displaySize=arConsole->getSize();
}

//-------------------------- Stream operations -------------------------------
bool // success
NSEventArConsoleBufferSetup::readFromStream(ArRef<AbstractIStream> input)
{
//cerr << "NSEventArConsoleBackground::readFromStream" << endl;
if (!NSEvent::readFromStream(input) ||
    !input->readInt(_displaySize) ||
    !input->readInt(_bufferSize))
  {
  return false;
  }

return true;
}

void
NSEventArConsoleBufferSetup::writeToStream(ArRef<AbstractOStream> output) const
{
NSEvent::writeToStream(output);

output->writeInt(_bufferSize);
output->writeInt(_displaySize);
}



//----------------------------------------------------------------------------
// CLASS       : NSEventArConsoleScrollText
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSEventArConsoleScrollText,NSEvent)
//---------------------- Constructors / Destructor ---------------------------
NSEventArConsoleScrollText::NSEventArConsoleScrollText(ArCW & arCW) // Protected
  : NSEvent(arCW),
    _nbLines()
{
// Nothing to be done
}

NSEventArConsoleScrollText::~NSEventArConsoleScrollText(void)
{
// Nothing to be done
}

//---------------------------- Public methods --------------------------------
// Read/Write views parameters
void
NSEventArConsoleScrollText::readArConsole(ArRef<ArConsole> arConsole) const
{
arConsole->scroll(_nbLines);
}

void
NSEventArConsoleScrollText::setNbLines(int nbLines)
{
_nbLines=nbLines;
}

//-------------------------- Stream operations -------------------------------
bool // success
NSEventArConsoleScrollText::readFromStream(ArRef<AbstractIStream> input)
{
if (!NSEvent::readFromStream(input) ||
    !input->readInt(_nbLines))
  {
  return false;
  }

return true;
}

void
NSEventArConsoleScrollText::writeToStream(ArRef<AbstractOStream> output) const
{
NSEvent::writeToStream(output);

output->writeInt(_nbLines);
}

//----------------------------------------------------------------------------
// CLASS       : NSEventArConsolePushText
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSEventArConsolePushText,NSEvent)
//---------------------- Constructors / Destructor ---------------------------
NSEventArConsolePushText::NSEventArConsolePushText(ArCW & arCW) // Protected
  : NSEvent(arCW),
    _text()
{
// Nothing to be done
}

NSEventArConsolePushText::~NSEventArConsolePushText(void)
{
// Nothing to be done
}

//---------------------------- Public methods --------------------------------
// Read/Write views parameters
void
NSEventArConsolePushText::readArConsole(ArRef<ArConsole> arConsole) const
{
arConsole->push(_text);
}

void
NSEventArConsolePushText::setText(const StlString &text)
{
_text=text;
}

//-------------------------- Stream operations -------------------------------
bool // success
NSEventArConsolePushText::readFromStream(ArRef<AbstractIStream> input)
{
//cerr << "NSEventArConsoleBackground::readFromStream" << endl;
if (!NSEvent::readFromStream(input) ||
    !input->readString(_text))
  {
  return false;
  }

return true;
}

void
NSEventArConsolePushText::writeToStream(ArRef<AbstractOStream> output) const
{
NSEvent::writeToStream(output);

output->writeString(_text);
}

//----------------------------------------------------------------------------
// CLASS       : NSEventArConsoleStoreCharacter
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSEventArConsoleStoreCharacter,NSEvent)
//---------------------- Constructors / Destructor ---------------------------
NSEventArConsoleStoreCharacter::NSEventArConsoleStoreCharacter(ArCW & arCW) // Protected
  : NSEvent(arCW),
    _c()
{
// Nothing to be done
}

NSEventArConsoleStoreCharacter::~NSEventArConsoleStoreCharacter(void)
{
// Nothing to be done
}

//---------------------------- Public methods --------------------------------
// Read/Write views parameters
void
NSEventArConsoleStoreCharacter::readArConsole(ArRef<ArConsole> arConsole) const
{
arConsole->storeCharacter(_c);
}

void
NSEventArConsoleStoreCharacter::setChar(char c)
{
_c=c;
}

//-------------------------- Stream operations -------------------------------
bool // success
NSEventArConsoleStoreCharacter::readFromStream(ArRef<AbstractIStream> input)
{
//cerr << "NSEventArConsoleBackground::readFromStream" << endl;
if (!NSEvent::readFromStream(input) ||
    !input->readChar(_c))
  {
  return false;
  }

return true;
}

void
NSEventArConsoleStoreCharacter::writeToStream(ArRef<AbstractOStream> output) const
{
NSEvent::writeToStream(output);

output->writeChar(_c);
}


} // namespace NetworkServices
//---End-Of-File---------------------------------------------------------------
