// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/Events/nsEventBox3D.cpp
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReViNS/Events/nsEventBox3D.h"
#include "AReVi/Utils/abstractStream.h"
#include "AReVi/Shapes/box3D.h"

namespace AReVi {
//----------------------------------------------------------------------------
// CLASS       : NSEventBox3DGeometryRebuild
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSEventBox3DGeometryRebuild,NSEventShapePart3DGeometryRebuild)
//---------------------- Constructors / Destructor ---------------------------
NSEventBox3DGeometryRebuild::NSEventBox3DGeometryRebuild(ArCW & arCW) // Protected
  : NSEventShapePart3DGeometryRebuild(arCW)
{
}

NSEventBox3DGeometryRebuild::~NSEventBox3DGeometryRebuild(void)
{}

void
NSEventBox3DGeometryRebuild::writeGeometry(ArConstRef<Box3D> box)
{
  box->getSize(_sx,_sy,_sz);
  _ccw=box->getCCW();
}


void
NSEventBox3DGeometryRebuild::readGeometry(ArRef<Box3D> box) const
{
  box->setSize(_sx,_sy,_sz);
  box->setCCW(_ccw);
}

//-------------------------- Stream operations -------------------------------
void
NSEventBox3DGeometryRebuild::writeToStream(ArRef<AbstractOStream> output) const
{
  NSEventShapePart3DGeometryRebuild::writeToStream(output);

  output->writeDouble(_sx);
  output->writeDouble(_sy);
  output->writeDouble(_sz);
  output->writeChar((_ccw));
}

bool // success
NSEventBox3DGeometryRebuild::readFromStream(ArRef<AbstractIStream> input)
{
  if (!NSEventShapePart3DGeometryRebuild::readFromStream(input))
    return false;

  char flags;

  if (!input->readDouble(_sx))
    return false;
  if (!input->readDouble(_sy))
    return false;
  if (!input->readDouble(_sz))
    return false;

  if (!input->readChar(flags))
    return false;

  _ccw=flags;
  return true;
}

} // namespace NetworkServices
//---End-Of-File---------------------------------------------------------------
