// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/Events/nsEventCone3D.cpp
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReViNS/Events/nsEventCone3D.h"
#include "AReVi/Utils/abstractStream.h"
#include "AReVi/Shapes/cone3D.h"

namespace AReVi {
//----------------------------------------------------------------------------
// CLASS       : NSEventCone3DGeometryRebuild
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSEventCone3DGeometryRebuild,NSEventShapePart3DGeometryRebuild)
//---------------------- Constructors / Destructor ---------------------------
NSEventCone3DGeometryRebuild::NSEventCone3DGeometryRebuild(ArCW & arCW) // Protected
  : NSEventShapePart3DGeometryRebuild(arCW)
{
}

NSEventCone3DGeometryRebuild::~NSEventCone3DGeometryRebuild(void)
{}

void
NSEventCone3DGeometryRebuild::writeGeometry(ArConstRef<Cone3D> cone)
{
  _radius=cone->getRadius();
  _height=cone->getHeight();
  _side=cone->getSide();
  _bottom=cone->getBottom();
  _ccw=cone->getCCW();
  _precision=cone->getPrecision();
}


void
NSEventCone3DGeometryRebuild::readGeometry(ArRef<Cone3D> cone) const
{
  cone->setRadius(_radius);
  cone->setHeight(_height);
  cone->setSide(_side);
  cone->setBottom(_bottom);
  cone->setCCW(_ccw);
  cone->setPrecision(_precision);
}

//-------------------------- Stream operations -------------------------------
void
NSEventCone3DGeometryRebuild::writeToStream(ArRef<AbstractOStream> output) const
{
  NSEventShapePart3DGeometryRebuild::writeToStream(output);

  output->writeDouble(_radius);
  output->writeDouble(_height);
  output->writeUInt(_precision);
  output->writeInt(_side+(_bottom<<1)+(_ccw<<2));
}

bool // success
NSEventCone3DGeometryRebuild::readFromStream(ArRef<AbstractIStream> input)
{
  if (!NSEventShapePart3DGeometryRebuild::readFromStream(input))
    return false;

  int flags;

  if (!input->readDouble(_radius))
    return false;

  if (!input->readDouble(_height))
    return false;

  if (!input->readUInt(_precision))
    return false;

  if (!input->readInt(flags))
    return false;

  _side=flags&1;
  _bottom=flags&2;
  _ccw=flags&4;
  return true;
}

} // namespace NetworkServices
//---End-Of-File---------------------------------------------------------------
