// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/Events/nsEventControler.cpp
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReViNS/Events/nsEventControler.h"
#include "AReVi/Utils/abstractStream.h"

namespace AReVi {
//----------------------------------------------------------------------------
// CLASS       : NSEventControler
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSEventControler,NSEventRef)
//---------------------- Constructors / Destructor ---------------------------
NSEventControler::NSEventControler(ArCW & arCW) // Protected
  : NSEventRef(arCW),
    _event()
{}

NSEventControler::~NSEventControler(void)
{}

//---------------------------- Public methods --------------------------------
//-------------------------- Stream operations -------------------------------
bool // success
NSEventControler::readFromStream(ArRef<AbstractIStream> input)
{
  if (!NSEventRef::readFromStream(input))
    return false;

  StlString className;

// Class Name
  if (!input->readString(className))
    return false;


  ArConstRef<ArClass> eventClass=ArClass::find(className);

  if(eventClass)
  {
    eventClass->newInstance(_event);

    if(!_event)
    {
      cerr << "NSEventControler::Can't create event object of type =>'" << className << "' => FAIL" << endl ;
      return false;
    }
  }
  else
  {
    cerr << "NSEventControler::Can't find event of type =>'" << className << "' => FAIL" << endl ;
    return false;
  }

  return _event->readFromStream(input);
}

void
NSEventControler::writeToStream(ArRef<AbstractOStream> output) const
{
  NSEventRef::writeToStream(output);

// Class Name
  output->writeString(_event->getClass()->getName());

// Write event
  _event->writeToStream(output);
}

//-------------------------- Set / Get / Access ------------------------------
// fillData from object
void
NSEventControler::setEvent(ArRef<NSEvent> event)
{
  _event=event;
// compress event only if embedded event allow it.
  setCompress(event->getCompress());
}

ArConstRef<NSEvent>
NSEventControler::getEvent(void) const
{
  return _event;
}


//----------------------------------------------------------------------------
// CLASS       : NSEventControlerPack
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSEventControlerPack,NSEvent)
//---------------------- Constructors / Destructor ---------------------------
NSEventControlerPack::NSEventControlerPack(ArCW & arCW) // Protected
  : NSEvent(arCW),
    _events()
{}

NSEventControlerPack::~NSEventControlerPack(void)
{}

//---------------------------- Public methods --------------------------------
//-------------------------- Stream operations -------------------------------
bool // success
NSEventControlerPack::readFromStream(ArRef<AbstractIStream> input)
{
  if (!NSEvent::readFromStream(input))
    return false;

  unsigned nbEvents;

// Class Name
  if (!input->readUInt(nbEvents))
    return false;

  ArRef<NSEvent> event;

  for (unsigned cpt=nbEvents;cpt--;)
    {
    NSEventControler::CLASS()->newInstance(event);

    if(!event)
      {
        cerr << "NSEventControlerPack::Can't create event controloer object => FAIL" << endl ;
        return false;
      }

    if (!event->readFromStream(input))
      {
      return false;
      }

    _events.push_back(event);
    }
  return true;

}

void
NSEventControlerPack::writeToStream(ArRef<AbstractOStream> output) const
{
  NSEvent::writeToStream(output);


// Nb Of Events
  output->writeUInt(_events.size());

// Write events
  StlList<ArRef<NSEvent> >::const_iterator it=_events.begin();
  StlList<ArRef<NSEvent> >::const_iterator it_end=_events.end();
  while(it!=it_end)
    {
    (*it)->writeToStream(output);
    it++;
    }
}

//-------------------------- Set / Get / Access ------------------------------
// fillData from object
void
NSEventControlerPack::setEvents(StlList<ArRef<NSEvent> > &events)
{
  _events=events;
// compress event only if embedded event allow it.
//   setCompress(event->getCompress());
}

const StlList<ArRef<NSEvent> > &
NSEventControlerPack::getEvents(void) const
{
  return _events;
}


//----------------------------------------------------------------------------
// CLASS       : NSEventRemoteDepReady
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSEventRemoteDepReady,NSEvent)
//---------------------- Constructors / Destructor ---------------------------
NSEventRemoteDepReady::NSEventRemoteDepReady(ArCW & arCW) // Protected
  : NSEvent(arCW)
{}

NSEventRemoteDepReady::~NSEventRemoteDepReady(void)
{}


} // namespace NetworkServices
//---End-Of-File---------------------------------------------------------------
