// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/Events/nsEventCylinder3D.cpp
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReViNS/Events/nsEventCylinder3D.h"
#include "AReVi/Utils/abstractStream.h"
#include "AReVi/Shapes/cylinder3D.h"

namespace AReVi {
//----------------------------------------------------------------------------
// CLASS       : NSEventCylinder3DGeometryRebuild
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSEventCylinder3DGeometryRebuild,NSEventShapePart3DGeometryRebuild)
//---------------------- Constructors / Destructor ---------------------------
NSEventCylinder3DGeometryRebuild::NSEventCylinder3DGeometryRebuild(ArCW & arCW) // Protected
  : NSEventShapePart3DGeometryRebuild(arCW)
{
}

NSEventCylinder3DGeometryRebuild::~NSEventCylinder3DGeometryRebuild(void)
{}

void
NSEventCylinder3DGeometryRebuild::writeGeometry(ArConstRef<Cylinder3D> cylinder)
{
  _radius=cylinder->getRadius();
  _height=cylinder->getHeight();
  _side=cylinder->getSide();
  _top=cylinder->getTop();
  _bottom=cylinder->getBottom();
  _ccw=cylinder->getCCW();
  _precision=cylinder->getPrecision();
}


void
NSEventCylinder3DGeometryRebuild::readGeometry(ArRef<Cylinder3D> cylinder) const
{
  cylinder->setRadius(_radius);
  cylinder->setHeight(_height);
  cylinder->setSide(_side);
  cylinder->setTop(_top);
  cylinder->setBottom(_bottom);
  cylinder->setCCW(_ccw);
  cylinder->setPrecision(_precision);
}

//-------------------------- Stream operations -------------------------------
void
NSEventCylinder3DGeometryRebuild::writeToStream(ArRef<AbstractOStream> output) const
{
  NSEventShapePart3DGeometryRebuild::writeToStream(output);

  output->writeDouble(_radius);
  output->writeDouble(_height);
  output->writeUInt(_precision);
  output->writeInt(_side+(_top<<1)+(_bottom<<2)+(_ccw<<3));
}

bool // success
NSEventCylinder3DGeometryRebuild::readFromStream(ArRef<AbstractIStream> input)
{
  if (!NSEventShapePart3DGeometryRebuild::readFromStream(input))
    return false;

  int flags;

  if (!input->readDouble(_radius))
    return false;

  if (!input->readDouble(_height))
    return false;

  if (!input->readUInt(_precision))
    return false;

  if (!input->readInt(flags))
    return false;

  _side=flags&1;
  _top=flags&2;
  _bottom=flags&4;
  _ccw=flags&8;
  return true;
}

} // namespace NetworkServices
//---End-Of-File---------------------------------------------------------------
