// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/Events/nsEventDataSound3D.cpp
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReViNS/Events/nsEventDataSound3D.h"
#include "AReVi/Utils/abstractStream.h"
#include "AReVi/Utils/memoryBlock.h"
#include "AReVi/Lib3D/dataSound3D.h"

namespace AReVi {
//----------------------------------------------------------------------------
// CLASS       : NSEventDataSound3DPushBuffer
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSEventDataSound3DPushBuffer,NSEvent)
//---------------------- Constructors / Destructor ---------------------------
NSEventDataSound3DPushBuffer::NSEventDataSound3DPushBuffer(ArCW & arCW) // Protected
  : NSEvent(arCW)
{
}

NSEventDataSound3DPushBuffer::~NSEventDataSound3DPushBuffer(void)
{
}
//---------------------------- Public methods --------------------------------
// Read/Write views parameters
bool
NSEventDataSound3DPushBuffer::readDataSound3D(ArRef<DataSound3D> sound) const
{
sound->pushBuffer(_buffer,_frequency,(UtilSound::SoundFormat)_format);
  return false;
}

void
NSEventDataSound3DPushBuffer::writeDataSound3D(unsigned int frequency,unsigned int format, ArConstRef<MemoryBlock> buffer)
{
_frequency=frequency;
_format=format;
_buffer=buffer;
}

//-------------------------- Stream operations -------------------------------
bool // success
NSEventDataSound3DPushBuffer::readFromStream(ArRef<AbstractIStream> input)
{
  unsigned size;

  if (!NSEvent::readFromStream(input) ||
      !input->readUInt(_frequency) ||
      !input->readUInt(_format) ||
      !input->readUInt(size))
    return false;

  ArRef<MemoryBlock> buffer=MemoryBlock::NEW();
  buffer->setSize(size);
  if (!input->readData(buffer->accessMemory(),buffer->getSize()))
    return false;

  _buffer=buffer;
  return true;
}

void
NSEventDataSound3DPushBuffer::writeToStream(ArRef<AbstractOStream> output) const
{
NSEvent::writeToStream(output);

output->writeUInt(_frequency);
output->writeUInt(_format);
if (_buffer.valid())
  {
  output->writeUInt(_buffer->getSize());
  output->writeData(_buffer->getMemory(),_buffer->getSize());
  }
else
  {
  output->writeUInt(0);
  cerr << getAlias() << "::writeToStream() => Invalid buffer" << endl;
  }
}

//----------------------------------------------------------------------------
// CLASS       : NSEventDataSound3DPopBuffer
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSEventDataSound3DPopBuffer,NSEvent)
//---------------------- Constructors / Destructor ---------------------------
NSEventDataSound3DPopBuffer::NSEventDataSound3DPopBuffer(ArCW & arCW) // Protected
  : NSEvent(arCW)
{}

NSEventDataSound3DPopBuffer::~NSEventDataSound3DPopBuffer(void)
{
}
} // namespace NetworkServices
//---End-Of-File---------------------------------------------------------------
