// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/Events/nsEventFile.cpp
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReViNS/Events/nsEventFile.h"
#include "AReVi/Utils/urlStream.h"
#include "AReVi/Utils/memoryStream.h"
#include "AReVi/Utils/memoryBlock.h"

namespace AReVi {
//----------------------------------------------------------------------------
// CLASS       : NSEventFileRequest
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSEventFileRequest,NSEvent)
//---------------------- Constructors / Destructor ---------------------------
NSEventFileRequest::NSEventFileRequest(ArCW & arCW) // Protected
  : NSEvent(arCW),_url("")
{
  setCompress(false); // by default don't compress files!
}

NSEventFileRequest::~NSEventFileRequest(void)
{
}
//---------------------------- Public methods --------------------------------
//-------------------------- Stream operations -------------------------------
bool // success
NSEventFileRequest::readFromStream(ArRef<AbstractIStream> input)
{
  if (!NSEvent::readFromStream(input))
  {
    setErrorMessage("Fail reading NSEvent");
    return false;
  }

  if (!input->readString(_url))
  {
    setErrorMessage("Fail reading fileName");
    return false;
  }
  else
    return true;
}

void
NSEventFileRequest::writeToStream(ArRef<AbstractOStream> output) const
{
  NSEvent::writeToStream(output);

  //cerr << "EVENT FILE=" << _url << endl;
  output->writeString(_url);
}

//-------------------------- Set / Get / Access ------------------------------
// Get Set ID/alias of removed object3D in Scene3D
const StlString &
NSEventFileRequest::getURL(void) const
{
  return _url;
}

void
NSEventFileRequest::setURL(const StlString &url)
{
  _url=url;
}


//----------------------------------------------------------------------------
// CLASS       : NSEventFile
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSEventFile,NSEventFileRequest)
//---------------------- Constructors / Destructor ---------------------------
NSEventFile::NSEventFile(ArCW & arCW) // Protected
  : NSEventFileRequest(arCW),
    _data(MemoryBlock::NEW())
{
}

NSEventFile::~NSEventFile(void)
{
}
//---------------------------- Public methods --------------------------------
//-------------------------- Stream operations -------------------------------
bool // success
NSEventFile::readFromStream(ArRef<AbstractIStream> input)
{
  if (!NSEventFileRequest::readFromStream(input))
    return false;

  unsigned int size;
  if (!input->readUInt(size))
  {
    setErrorMessage("Fail reading fileSize for "+_url);
    return false;
  }

  _data->setSize(size);
  if (size)
  {
    if (!input->readData(_data->accessMemory(),size))
    {
      setErrorMessage("Fail reading data for file '"+_url + "' of size=" + uintToStr(size));
      return false;
    }
  }

  return true;
}

void
NSEventFile::writeToStream(ArRef<AbstractOStream> output) const
{
  NSEventFileRequest::writeToStream(output);

  output->writeUInt(_data->getSize());
  output->writeData(_data->getMemory(),_data->getSize());
}

//-------------------------- Set / Get / Access ------------------------------
// get Data
ArConstRef<MemoryBlock>
NSEventFile::getData(void) const
{
  return _data;
}

// set url
void
NSEventFile::setURL(const StlString &url)
{
  NSEventFileRequest::setURL(url);

  ArRef<URLStream> stream=URLStream::NEW(url);
  if (stream->fail())
  {
    setErrorMessage(stream->getErrorMessage());
    _data->setSize(0);
  }

  ArRef<MemoryOStream> output=MemoryOStream::NEW(_data,0);
  stream->copyDataToOutput(output);
  _data->setSize(output->getMemoryUsage());
}

} // namespace NetworkServices
//---End-Of-File---------------------------------------------------------------
