// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/Events/nsEventLight3D.cpp
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReViNS/Events/nsEventLight3D.h"
#include "AReVi/Utils/abstractStream.h"
#include "AReVi/Lib3D/light3D.h"

namespace AReVi {
//----------------------------------------------------------------------------
// CLASS       : NSEventLight3DSetup
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSEventLight3DSetup,NSEvent)
//---------------------- Constructors / Destructor ---------------------------
NSEventLight3DSetup::NSEventLight3DSetup(ArCW & arCW) // Protected
  : NSEvent(arCW)
{
}

NSEventLight3DSetup::~NSEventLight3DSetup(void)
{
}
//---------------------------- Public methods --------------------------------
// Read/Write views parameters
void
NSEventLight3DSetup::readLight3DSetup(ArRef<Light3D> light) const
{
  light->setDirectional(_directional);
  light->setAttenuation(_constantAtt,_linearAtt,_quadraticAtt);
  light->setSpotParameters(_spotExponent,_spotCutOff);
  light->setColor(_color[0],_color[1],_color[2]);
  light->setSwitch(_on);
}

void
NSEventLight3DSetup::writeLight3DSetup(ArConstRef<Light3D> light)
{
  _directional=light->getDirectional();
  light->getAttenuation(_constantAtt,_linearAtt,_quadraticAtt);
  light->getSpotParameters(_spotExponent,_spotCutOff);
  light->getColor(_color[0],_color[1],_color[2]);
  _on=light->getSwitch();
}

//-------------------------- Stream operations -------------------------------
bool // success
NSEventLight3DSetup::readFromStream(ArRef<AbstractIStream> input)
{
//cerr << "NSEventLight3DSetup::readFromStream" << endl;
  if (!NSEvent::readFromStream(input))
    return false;

  // Read fog setup
  input->readDouble(_constantAtt);
  input->readDouble(_linearAtt);
  input->readDouble(_quadraticAtt);
  input->readDouble(_spotExponent);
  input->readDouble(_spotCutOff);
  input->readDouble(_color[0]);
  input->readDouble(_color[1]);
  input->readDouble(_color[2]);

  unsigned int flags;
  input->readUInt(flags);
  _on=flags&1;
  _directional=flags&2;

  return true;
}

void
NSEventLight3DSetup::writeToStream(ArRef<AbstractOStream> output) const
{
  NSEvent::writeToStream(output);

   // Write setup
  output->writeDouble(_constantAtt);
  output->writeDouble(_linearAtt);
  output->writeDouble(_quadraticAtt);
  output->writeDouble(_spotExponent);
  output->writeDouble(_spotCutOff);
  output->writeDouble(_color[0]);
  output->writeDouble(_color[1]);
  output->writeDouble(_color[2]);

  unsigned int flags(((unsigned )_on)+((unsigned )_directional<<1));
  output->writeUInt(flags);
}

} // namespace NetworkServices
//---End-Of-File---------------------------------------------------------------
