// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/Events/nsEventListener3D.cpp
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReViNS/Events/nsEventListener3D.h"

#include "AReVi/Utils/abstractStream.h"
#include "AReVi/Lib3D/sound3D.h"
#include "AReVi/Lib3D/scene3D.h"

namespace AReVi {
//----------------------------------------------------------------------------
// CLASS       : NSEventListener3DSetup
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSEventListener3DSetup,NSEvent)
//---------------------- Constructors / Destructor ---------------------------
NSEventListener3DSetup::NSEventListener3DSetup(ArCW & arCW) // Protected
  : NSEvent(arCW),_refScene()
{}

NSEventListener3DSetup::~NSEventListener3DSetup(void)
{
}
//---------------------------- Public methods --------------------------------
// Read/Write views parameters
bool
NSEventListener3DSetup::readListener3D(ArRef<Listener3D> listener,
                                       ArRef<RemoteDepService> depService) const
{
  ArRef<Scene3D> scene;

  if (!_refScene.request(depService,scene))
    return false;

  /*  if (scene.valid())
    {
      cerr << "Change scene to " << scene->getAlias() << endl;
      cerr << "NbSound=" << scene->getNbSounds() << endl;
    }
    else
      cerr << "select scene to null!" << endl;*/
  listener->setGain(_gain);
  listener->selectScene(scene);
  return true;
}

void
NSEventListener3DSetup::writeListener3D(ArConstRef<Listener3D> listener)
{
  _gain=listener->getGain();
  _refScene=RemoteRef(listener->getScene());
}

//-------------------------- Stream operations -------------------------------
bool // success
NSEventListener3DSetup::readFromStream(ArRef<AbstractIStream> input)
{
//cerr << "NSEventListener3DSetup::readFromStream" << endl;
  if (!NSEvent::readFromStream(input))
    return false;

// Read listener setup
  input->readDouble(_gain);
  return _refScene.readFromStream(input);
}

void
NSEventListener3DSetup::writeToStream(ArRef<AbstractOStream> output) const
{
  NSEvent::writeToStream(output);

// Write setup
  output->writeDouble(_gain);
  _refScene.writeToStream(output);
}

} // namespace NetworkServices
//---End-Of-File---------------------------------------------------------------
