// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/Events/nsEventMesh3D.cpp
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReViNS/Events/nsEventMesh3D.h"
#include "AReVi/Utils/memoryStream.h"
#include "AReVi/Utils/memoryBlock.h"
#include "AReVi/Shapes/mesh3D.h"

namespace AReVi {
//----------------------------------------------------------------------------
// CLASS       : NSEventMesh3DGeometryRebuild
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSEventMesh3DGeometryRebuild,NSEventShapePart3DGeometryRebuild)
//---------------------- Constructors / Destructor ---------------------------
NSEventMesh3DGeometryRebuild::NSEventMesh3DGeometryRebuild(ArCW & arCW) // Protected
  : NSEventShapePart3DGeometryRebuild(arCW),_meshData(MemoryBlock::NEW())
{
}

NSEventMesh3DGeometryRebuild::~NSEventMesh3DGeometryRebuild(void)
{}

void
NSEventMesh3DGeometryRebuild::writeGeometry(ArConstRef<Mesh3D> mesh)
{
  ArRef<MemoryOStream> output=MemoryOStream::NEW(_meshData,0);
  const StlVector<Util3D::Dbl3> &vertices(mesh->getVertices());
  const StlVector<Util3D::Face> &facesV(mesh->getVertexFaces());
  const StlVector<Util3D::Dbl3> &normals(mesh->getNormals());
  const StlVector<Util3D::Face> &facesN(mesh->getNormalFaces());
  const StlVector<Util3D::Dbl2> &textures(mesh->getTextures());
  const StlVector<Util3D::Face> &facesT(mesh->getTextureFaces());
  const StlVector<Util3D::Dbl2> &textures2(mesh->getSecondTextures());
  const StlVector<Util3D::Face> &facesT2(mesh->getSecondTextureFaces());
  const StlVector<Util3D::Dbl2> &textures3(mesh->getThirdTextures());
  const StlVector<Util3D::Face> &facesT3(mesh->getThirdTextureFaces());
  const StlVector<Util3D::Dbl2> &textures4(mesh->getFourthTextures());
  const StlVector<Util3D::Face> &facesT4(mesh->getFourthTextureFaces());
  const StlVector<Util3D::Dbl3> &colors(mesh->getColors());
  const StlVector<Util3D::Face> &facesC(mesh->getColorFaces());


  // FACES/TEXTURES
  // textures
  output->writeUInt(textures.size());

  for (unsigned i=textures.size();i--;)
    {
    output->writeDouble(textures[i].x);
    output->writeDouble(textures[i].y);
    }
  // facesT
  output->writeUInt(facesT.size());
  for (unsigned i=facesT.size();i--;)
    {
    output->writeUInt(facesT[i].p1);
    output->writeUInt(facesT[i].p2);
    output->writeUInt(facesT[i].p3);
    }

  // textures2
  output->writeUInt(textures2.size());
  for (unsigned i=textures2.size();i--;)
    {
    output->writeDouble(textures2[i].x);
    output->writeDouble(textures2[i].y);
    }
  // facesT2
  output->writeUInt(facesT2.size());
  for (unsigned i=facesT2.size();i--;)
    {
    output->writeUInt(facesT2[i].p1);
    output->writeUInt(facesT2[i].p2);
    output->writeUInt(facesT2[i].p3);
    }

  // textures3
  output->writeUInt(textures3.size());
  for (unsigned i=textures3.size();i--;)
    {
    output->writeDouble(textures3[i].x);
    output->writeDouble(textures3[i].y);
    }
  // facesT3
  output->writeUInt(facesT3.size());
  for (unsigned i=facesT3.size();i--;)
    {
    output->writeUInt(facesT3[i].p1);
    output->writeUInt(facesT3[i].p2);
    output->writeUInt(facesT3[i].p3);
    }

  // textures4
  output->writeUInt(textures4.size());
  for (unsigned i=textures4.size();i--;)
    {
    output->writeDouble(textures4[i].x);
    output->writeDouble(textures4[i].y);
    }
  // facesT4
  output->writeUInt(facesT4.size());
  for (unsigned i=facesT4.size();i--;)
    {
    output->writeUInt(facesT4[i].p1);
    output->writeUInt(facesT4[i].p2);
    output->writeUInt(facesT4[i].p3);
    }

  // colors
  output->writeUInt(colors.size());
  for (unsigned i=colors.size();i--;)
    {
    output->writeDouble(colors[i].x);
    output->writeDouble(colors[i].y);
    output->writeDouble(colors[i].z);
    }
  // facesC
  output->writeUInt(facesC.size());
  for (unsigned i=facesC.size();i--;)
    {
    output->writeUInt(facesC[i].p1);
    output->writeUInt(facesC[i].p2);
    output->writeUInt(facesC[i].p3);
    }

  // vertices
  output->writeUInt(vertices.size());
  for (unsigned i=vertices.size();i--;)
    {
    output->writeDouble(vertices[i].x);
    output->writeDouble(vertices[i].y);
    output->writeDouble(vertices[i].z);
    }
  // faceV
  output->writeUInt(facesV.size());
  for (unsigned i=facesV.size();i--;)
    {
    output->writeUInt(facesV[i].p1);
    output->writeUInt(facesV[i].p2);
    output->writeUInt(facesV[i].p3);
    }
  // normals
  output->writeUInt(normals.size());
  for (unsigned i=normals.size();i--;)
    {
    output->writeDouble(normals[i].x);
    output->writeDouble(normals[i].y);
    output->writeDouble(normals[i].z);
    }
  // faceN
  output->writeUInt(facesN.size());
  for (unsigned i=facesN.size();i--;)
    {
    output->writeUInt(facesN[i].p1);
    output->writeUInt(facesN[i].p2);
    output->writeUInt(facesN[i].p3);
    }

  output->flush();
  _meshData->setSize(output->getMemoryUsage());
}


void
NSEventMesh3DGeometryRebuild::readGeometry(ArRef<Mesh3D> mesh) const
{
  ArRef<MemoryIStream> input=MemoryIStream::NEW(_meshData,0);
  StlVector<Util3D::Dbl3> &vertices(mesh->accessVertices());
  StlVector<Util3D::Face> &facesV(mesh->accessVertexFaces());
  StlVector<Util3D::Dbl3> &normals(mesh->accessNormals());
  StlVector<Util3D::Face> &facesN(mesh->accessNormalFaces());
  StlVector<Util3D::Dbl2> &textures(mesh->accessTextures());
  StlVector<Util3D::Face> &facesT(mesh->accessTextureFaces());
  StlVector<Util3D::Dbl2> &textures2(mesh->accessSecondTextures());
  StlVector<Util3D::Face> &facesT2(mesh->accessSecondTextureFaces());
  StlVector<Util3D::Dbl2> &textures3(mesh->accessThirdTextures());
  StlVector<Util3D::Face> &facesT3(mesh->accessThirdTextureFaces());
  StlVector<Util3D::Dbl2> &textures4(mesh->accessFourthTextures());
  StlVector<Util3D::Face> &facesT4(mesh->accessFourthTextureFaces());
  StlVector<Util3D::Dbl3> &colors(mesh->accessColors());
  StlVector<Util3D::Face> &facesC(mesh->accessColorFaces());

  // FACES/TEXTURES
  // textures
  unsigned nbElts;

  input->readUInt(nbElts);
  textures.resize(nbElts);
  for (unsigned i=nbElts;i--;)
    {
    input->readDouble(textures[i].x);
    input->readDouble(textures[i].y);
    }
  // facesT
  input->readUInt(nbElts);
  facesT.resize(nbElts);
  for (unsigned i=nbElts;i--;)
    {
    input->readUInt(facesT[i].p1);
    input->readUInt(facesT[i].p2);
    input->readUInt(facesT[i].p3);
    }

  // textures2
  input->readUInt(nbElts);
  textures2.resize(nbElts);
  for (unsigned i=nbElts;i--;)
    {
    input->readDouble(textures2[i].x);
    input->readDouble(textures2[i].y);
    }
  // facesT2
  input->readUInt(nbElts);
  facesT2.resize(nbElts);
  for (unsigned i=nbElts;i--;)
    {
    input->readUInt(facesT2[i].p1);
    input->readUInt(facesT2[i].p2);
    input->readUInt(facesT2[i].p3);
    }

  // textures3
  input->readUInt(nbElts);
  textures3.resize(nbElts);
  for (unsigned i=nbElts;i--;)
    {
    input->readDouble(textures3[i].x);
    input->readDouble(textures3[i].y);
    }
  // facesT3
  input->readUInt(nbElts);
  facesT3.resize(nbElts);
  for (unsigned i=nbElts;i--;)
    {
    input->readUInt(facesT3[i].p1);
    input->readUInt(facesT3[i].p2);
    input->readUInt(facesT3[i].p3);
    }

  // textures4
  input->readUInt(nbElts);
  textures4.resize(nbElts);
  for (unsigned i=nbElts;i--;)
    {
    input->readDouble(textures4[i].x);
    input->readDouble(textures4[i].y);
    }
  // facesT4
  input->readUInt(nbElts);
  facesT4.resize(nbElts);
  for (unsigned i=nbElts;i--;)
    {
    input->readUInt(facesT4[i].p1);
    input->readUInt(facesT4[i].p2);
    input->readUInt(facesT4[i].p3);
    }

  // colors
  input->readUInt(nbElts);
  colors.resize(nbElts);
  for (unsigned i=nbElts;i--;)
    {
    input->readDouble(colors[i].x);
    input->readDouble(colors[i].y);
    input->readDouble(colors[i].z);
    }
  // facesC
  input->readUInt(nbElts);
  facesC.resize(nbElts);
  for (unsigned i=nbElts;i--;)
    {
    input->readUInt(facesC[i].p1);
    input->readUInt(facesC[i].p2);
    input->readUInt(facesC[i].p3);
    }

  // vertices
  input->readUInt(nbElts);
  vertices.resize(nbElts);
  for (unsigned i=nbElts;i--;)
    {
    input->readDouble(vertices[i].x);
    input->readDouble(vertices[i].y);
    input->readDouble(vertices[i].z);
    }
  // faceV
  input->readUInt(nbElts);
  facesV.resize(nbElts);
  for (unsigned i=facesV.size();i--;)
    {
    input->readUInt(facesV[i].p1);
    input->readUInt(facesV[i].p2);
    input->readUInt(facesV[i].p3);
    }

  // normals
  input->readUInt(nbElts);
  normals.resize(nbElts);
  for (unsigned i=normals.size();i--;)
  {
    input->readDouble(normals[i].x);
    input->readDouble(normals[i].y);
    input->readDouble(normals[i].z);
    }
  // faceN
  input->readUInt(nbElts);
  facesN.resize(nbElts);
  for (unsigned i=facesN.size();i--;)
    {
    input->readUInt(facesN[i].p1);
    input->readUInt(facesN[i].p2);
    input->readUInt(facesN[i].p3);
    }

  mesh->applyChanges(true);
}

//-------------------------- Stream operations -------------------------------
void
NSEventMesh3DGeometryRebuild::writeToStream(ArRef<AbstractOStream> output) const
{
  NSEventShapePart3DGeometryRebuild::writeToStream(output);

  output->writeUInt(_meshData->getSize());
  output->writeData(_meshData->getMemory(),_meshData->getSize());
}

bool // success
NSEventMesh3DGeometryRebuild::readFromStream(ArRef<AbstractIStream> input)
{
  if (!NSEventShapePart3DGeometryRebuild::readFromStream(input))
    return false;
  unsigned size;

  if (!input->readUInt(size))
    return false;

  _meshData->setSize(size);
  return input->readData(_meshData->accessMemory(),_meshData->getSize());
}

} // namespace NetworkServices
//---End-Of-File---------------------------------------------------------------
