// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/Events/nsEventMetaball.cpp
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReViNS/Events/nsEventMetaball.h"
#include "AReVi/Utils/abstractStream.h"

namespace AReVi {

//----------------------------------------------------------------------------
// CLASS       : NSEventMetaballGeometryRebuild
//----------------------------------------------------------------------------

AR_CLASS_DEF(NSEventMetaballGeometryRebuild,NSEventShapePart3DGeometryRebuild)

//---------------------- Constructors / Destructor ---------------------------

NSEventMetaballGeometryRebuild::NSEventMetaballGeometryRebuild(ArCW & arCW) // Protected
  : NSEventShapePart3DGeometryRebuild(arCW)
{
}

NSEventMetaballGeometryRebuild::~NSEventMetaballGeometryRebuild(void)
{}

void
NSEventMetaballGeometryRebuild::writeGeometry(ArConstRef<Metaball> metaball)
{
_isolevel=metaball->getIsoLevel();
_ccw=metaball->getCCW();
_precision=metaball->getPrecision();
_smooth=metaball->getSmoothed();
_fieldFunction=metaball->getFieldFunction();

_metaballs.resize(metaball->getNbMetaballs());

for (unsigned i=_metaballs.size();i--;)
  _metaballs[i]=metaball->getMetaball(i);
}


void
NSEventMetaballGeometryRebuild::readGeometry(ArRef<Metaball> metaball) const
{
metaball->setIsoLevel(_isolevel);
metaball->setCCW(_ccw);
metaball->setPrecision(_precision);
metaball->setSmoothed(_smooth);
metaball->setFieldFunction(_fieldFunction);

while(metaball->getNbMetaballs())
  metaball->removeMetaball(0);

for (unsigned i=_metaballs.size();i--;)
  metaball->addMetaball(_metaballs[i]);
}

//-------------------------- Stream operations -------------------------------
void
NSEventMetaballGeometryRebuild::writeToStream(ArRef<AbstractOStream> output) const
{
NSEventShapePart3DGeometryRebuild::writeToStream(output);

output->writeDouble(_isolevel);
output->writeUInt(_precision);
output->writeUInt((int)_fieldFunction);
unsigned int flags=(_smooth ? 1<<0 : 0)+
                   (_ccw    ? 1<<1 : 0);
output->writeInt(flags);

// Write metaballs ...
output->writeUInt(_metaballs.size());

for (unsigned i=_metaballs.size();i--;)
  {
  output->writeDouble(_metaballs[i].radius);
  output->writeDouble(_metaballs[i].innerSize);
  output->writeDouble(_metaballs[i].potential);
  output->writeDouble(_metaballs[i].position.x);
  output->writeDouble(_metaballs[i].position.y);
  output->writeDouble(_metaballs[i].position.z);
  output->writeUInt((int)_metaballs[i].type);
  }
}

bool // success
NSEventMetaballGeometryRebuild::readFromStream(ArRef<AbstractIStream> input)
{
if (!NSEventShapePart3DGeometryRebuild::readFromStream(input))
  return false;

double isolevel;
unsigned int precision,flags, fieldFunction, nbMetaballs;
if(!input->readDouble(isolevel)||
   !input->readUInt(precision)||
   !input->readUInt(fieldFunction)||
   !input->readUInt(flags) ||
   !input->readUInt(nbMetaballs))
  {
  return(false);
  }

// Read metaballs ...
_metaballs.resize(nbMetaballs);

for (unsigned i=_metaballs.size();i--;)
  {
  unsigned int type;
  if (!input->readDouble(_metaballs[i].radius) ||
      !input->readDouble(_metaballs[i].innerSize) ||
      !input->readDouble(_metaballs[i].potential) ||
      !input->readDouble(_metaballs[i].position.x) ||
      !input->readDouble(_metaballs[i].position.y) ||
      !input->readDouble(_metaballs[i].position.z) ||
      !input->readUInt(type))
    {
    return(false);
    }
  _metaballs[i].type=(Metaball::MetaballType)type;
  }

_isolevel=isolevel;
_precision=precision;
_smooth=flags&(1<<0);
_ccw=flags&(1<<1);
_fieldFunction=(Metaball::FieldFunction)fieldFunction;

return(true);
}

} // namespace NetworkServices
//---End-Of-File---------------------------------------------------------------
