// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/Events/nsEventNurbsCurve3D.cpp
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReViNS/Events/nsEventNurbsCurve3D.h"
#include "AReVi/Utils/memoryStream.h"
#include "AReVi/Utils/memoryBlock.h"
#include "AReVi/Contrib/nurbsCurve3D.h"

namespace AReVi {
//----------------------------------------------------------------------------
// CLASS       : NSEventNurbsCurve3DGeometryRebuild
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSEventNurbsCurve3DGeometryRebuild,NSEventShapePart3DGeometryRebuild)
//---------------------- Constructors / Destructor ---------------------------
NSEventNurbsCurve3DGeometryRebuild::NSEventNurbsCurve3DGeometryRebuild(ArCW & arCW) // Protected
  : NSEventShapePart3DGeometryRebuild(arCW),_nurbsCurveData(MemoryBlock::NEW())
{
}

NSEventNurbsCurve3DGeometryRebuild::~NSEventNurbsCurve3DGeometryRebuild(void)
{}

void
NSEventNurbsCurve3DGeometryRebuild::writeGeometry(ArConstRef<NurbsCurve3D> curve)
{
ArRef<MemoryOStream> output=MemoryOStream::NEW(_nurbsCurveData,0);

output->writeUInt(curve->getPrecision());
double r,g,b;
curve->getColor(r,g,b);
output->writeDouble(r);
output->writeDouble(g);
output->writeDouble(b);
curve->getCurve().writeToStream(output);

output->flush();
_nurbsCurveData->setSize(output->getMemoryUsage());
}


bool
NSEventNurbsCurve3DGeometryRebuild::readGeometry(ArRef<NurbsCurve3D> curve) const
{
ArRef<MemoryIStream> input=MemoryIStream::NEW(_nurbsCurveData,0);

ArNurbs::Curve3d nurbs;
unsigned int precision;
Util3D::Dbl3 color;
if (!input->readUInt(precision) ||
    !input->readDouble(color.x)||
    !input->readDouble(color.y)||
    !input->readDouble(color.z)||
    !nurbs.readFromStream(input))
  return false;

curve->setPrecision(precision);
curve->setColor(color.x,color.y,color.z);
curve->accessCurve()=nurbs;
return curve->applyChanges(true);
}

//-------------------------- Stream operations -------------------------------
void
NSEventNurbsCurve3DGeometryRebuild::writeToStream(ArRef<AbstractOStream> output) const
{
  NSEventShapePart3DGeometryRebuild::writeToStream(output);

  output->writeUInt(_nurbsCurveData->getSize());
  output->writeData(_nurbsCurveData->getMemory(),_nurbsCurveData->getSize());
}

bool // success
NSEventNurbsCurve3DGeometryRebuild::readFromStream(ArRef<AbstractIStream> input)
{
  if (!NSEventShapePart3DGeometryRebuild::readFromStream(input))
    return false;
  unsigned size;

  if (!input->readUInt(size))
    return false;

  _nurbsCurveData->setSize(size);
  return input->readData(_nurbsCurveData->accessMemory(),_nurbsCurveData->getSize());
}

} // namespace NetworkServices
//---End-Of-File---------------------------------------------------------------
