// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/Events/nsEventNurbsSurface3D.cpp
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReViNS/Events/nsEventNurbsSurface3D.h"
#include "AReVi/Utils/memoryStream.h"
#include "AReVi/Utils/memoryBlock.h"
#include "AReVi/Contrib/nurbsSurface3D.h"

namespace AReVi {
//----------------------------------------------------------------------------
// CLASS       : NSEventNurbsSurface3DGeometryRebuild
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSEventNurbsSurface3DGeometryRebuild,NSEventShapePart3DGeometryRebuild)
//---------------------- Constructors / Destructor ---------------------------
NSEventNurbsSurface3DGeometryRebuild::NSEventNurbsSurface3DGeometryRebuild(ArCW & arCW) // Protected
  : NSEventShapePart3DGeometryRebuild(arCW),
    _nurbsSurfaceData(MemoryBlock::NEW())
{
}

NSEventNurbsSurface3DGeometryRebuild::~NSEventNurbsSurface3DGeometryRebuild(void)
{
}

void
NSEventNurbsSurface3DGeometryRebuild::writeGeometry(ArConstRef<NurbsSurface3D> surface)
{
ArRef<MemoryOStream> output=MemoryOStream::NEW(_nurbsSurfaceData,0);
unsigned int u,v;

surface->getPrecision(u,v);
output->writeUInt(u);
output->writeUInt(v);
surface->getSurface().writeToStream(output);

output->flush();
_nurbsSurfaceData->setSize(output->getMemoryUsage());
}


bool
NSEventNurbsSurface3DGeometryRebuild::readGeometry(ArRef<NurbsSurface3D> surface) const
{
ArRef<MemoryIStream> input=MemoryIStream::NEW(_nurbsSurfaceData,0);

ArNurbs::Surface3d nurbs;
unsigned int u,v;
if (!input->readUInt(u) ||
    !input->readUInt(v) ||
    !nurbs.readFromStream(input))
  return false;

if (nurbs.valid())
  surface->accessSurface()=nurbs;
else
  return false;

surface->setPrecision(u,v);

return surface->applyChanges(true);
}

//-------------------------- Stream operations -------------------------------
void
NSEventNurbsSurface3DGeometryRebuild::writeToStream(ArRef<AbstractOStream> output) const
{
  NSEventShapePart3DGeometryRebuild::writeToStream(output);

  output->writeUInt(_nurbsSurfaceData->getSize());
  output->writeData(_nurbsSurfaceData->getMemory(),_nurbsSurfaceData->getSize());
}

bool // success
NSEventNurbsSurface3DGeometryRebuild::readFromStream(ArRef<AbstractIStream> input)
{
  if (!NSEventShapePart3DGeometryRebuild::readFromStream(input))
    return false;
  unsigned size;

  if (!input->readUInt(size))
    return false;

  _nurbsSurfaceData->setSize(size);
  return input->readData(_nurbsSurfaceData->accessMemory(),_nurbsSurfaceData->getSize());
}

} // namespace NetworkServices
//---End-Of-File---------------------------------------------------------------
