// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/Events/nsEventObjectRef.cpp
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReViNS/Events/nsEventObjectRef.h"
#include "AReVi/Utils/abstractStream.h"
#include "AReVi/Utils/memoryStream.h"
#include "AReVi/Utils/memoryBlock.h"

namespace AReVi {
//----------------------------------------------------------------------------
// CLASS       : NSEventObjectRefRequest
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSEventObjectRefRequest,NSEventRef)
//---------------------- Constructors / Destructor ---------------------------
NSEventObjectRefRequest::NSEventObjectRefRequest(ArCW & arCW) // Protected
  : NSEventRef(arCW)
{}

NSEventObjectRefRequest::~NSEventObjectRefRequest(void)
{}


//----------------------------------------------------------------------------
// CLASS       : NSEventObjectRef
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSEventObjectRef,NSEventArRef)
//---------------------- Constructors / Destructor ---------------------------
NSEventObjectRef::NSEventObjectRef(ArCW & arCW) // Protected
  : NSEventArRef(arCW),
    _urls(),
    _refs(),
    _classRefs(),
    _objectData(MemoryBlock::NEW())

{
}

NSEventObjectRef::~NSEventObjectRef(void)
{
}
//---------------------------- Public methods --------------------------------
//-------------------------- Stream operations -------------------------------
bool // success
NSEventObjectRef::readFromStream(ArRef<AbstractIStream> input)
{
  if (!NSEventArRef::readFromStream(input))
    return false;

  unsigned int size;

// Object deps
  if (!input->readUInt(size))
    return false;
  _urls.resize(size);
  for (unsigned int i=size;i--;)
    input->readString(_urls[i]);


  if (!input->readUInt(size))
    return false;
  _refs.resize(size);
  for (unsigned int i=size;i--;)
    input->readString(_refs[i]);

  if (!input->readUInt(size))
    return false;
  _classRefs.resize(size);
  for (unsigned int i=size;i--;)
    input->readString(_classRefs[i]);

// Object data
  if (!input->readUInt(size))
    return false;
  _objectData->setSize(size);
  if (size)
    if (!input->readData(_objectData->accessMemory(),size))
      return false;

  return true;
}

void
NSEventObjectRef::writeToStream(ArRef<AbstractOStream> output) const
{
  NSEventArRef::writeToStream(output);

// Object deps
  output->writeUInt(_urls.size());
  for (unsigned int i=_urls.size();i--;)
    output->writeString(_urls[i]);

  output->writeUInt(_refs.size());
  for (unsigned int i=_refs.size();i--;)
    output->writeString(_refs[i]);

  output->writeUInt(_classRefs.size());
  for (unsigned int i=_classRefs.size();i--;)
    output->writeString(_classRefs[i]);

// Object data
  output->writeUInt(_objectData->getSize());
  output->writeData(_objectData->getMemory(),_objectData->getSize());
}

//-------------------------- Set / Get / Access ------------------------------
// fillData from object
void
NSEventObjectRef::setRef(ArConstRef<ArObject> object)
{
  if (object)
  {
    setID(object->getAlias());
    _className=object->getClass()->getName();
    ArRef<MemoryOStream> stream=MemoryOStream::NEW(_objectData,0);

    //cerr << "setREf => " << object->getAlias() << endl;
    SerializationDependencies dependencies;
    object->serialize(dependencies,stream);
    _objectData->setSize(stream->getMemoryUsage());

    //cerr << "NBREf=" << dependencies.getReferences().size() << endl;
    //cerr << "NBURL=" << dependencies.getURLs().size() << endl;

    _urls=dependencies.getURLs();
    for (unsigned int i=0;i<dependencies.getReferences().size();i++)
    {
      _refs.push_back(dependencies.getReferences()[i]->getAlias());
      _classRefs.push_back(dependencies.getReferences()[i]->getClass()->getName());
    }
  }
  else
  {
    // Keep alias to identify object ...
    _urls.clear();
    _refs.clear();
    _classRefs.clear();
    _objectData->setSize(0);
    _className="";
  }
}

const StlVector<StlString> &
NSEventObjectRef::getURLs(void) const
{
  return _urls;
}

const StlVector<StlString> &
NSEventObjectRef::getRefs(void) const
{
  return _refs;
}

const StlVector<StlString> &
NSEventObjectRef::getClassRefs(void) const
{
  return _classRefs;
}

ArConstRef<MemoryBlock>
NSEventObjectRef::getData(void) const
{
  return _objectData;
}

//----------------------------------------------------------------------------
// CLASS       : NSEventArRefList
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSEventArRefList,NSEvent)
//---------------------- Constructors / Destructor ---------------------------
NSEventArRefList::NSEventArRefList(ArCW & arCW) // Protected
  : NSEvent(arCW),
    _refs(),
    _classRefs()
{
}

NSEventArRefList::~NSEventArRefList(void)
{
}
//---------------------------- Public methods --------------------------------
//-------------------------- Stream operations -------------------------------
bool // success
NSEventArRefList::readFromStream(ArRef<AbstractIStream> input)
{
  if (!NSEvent::readFromStream(input))
    return false;

  unsigned int size;

// Object deps
  if (!input->readUInt(size))
    return false;
  _refs.resize(size);
  for (unsigned int i=size;i--;)
    input->readString(_refs[i]);

  if (!input->readUInt(size))
    return false;
  _classRefs.resize(size);
  for (unsigned int i=size;i--;)
    input->readString(_classRefs[i]);

  return true;
}

void
NSEventArRefList::writeToStream(ArRef<AbstractOStream> output) const
{
  NSEvent::writeToStream(output);

// Object deps
  output->writeUInt(_refs.size());
  for (unsigned int i=_refs.size();i--;)
    output->writeString(_refs[i]);

  output->writeUInt(_classRefs.size());
  for (unsigned int i=_classRefs.size();i--;)
    output->writeString(_classRefs[i]);
}

//-------------------------- Set / Get / Access ------------------------------
// add List of references
void
NSEventArRefList::setRefs(const StlVector<ArRef<ArObject> > &refs)
{
  _refs.clear();
  _classRefs.clear();

  for (unsigned cpt=refs.size();cpt--;)
    addRef(refs[cpt]);
}

void
NSEventArRefList::addRef(ArConstRef<ArObject> object)
{
  if (object.valid())
  {
    _refs.push_back(object->getAlias());
    _classRefs.push_back(object->getClass()->getName());
  }
  else
  {
    _refs.push_back("");
    _classRefs.push_back("");
  }
}

const StlVector<StlString> &
NSEventArRefList::getRefs(void) const
{
  return _refs;
}

const StlVector<StlString> &
NSEventArRefList::getClassRefs(void) const
{
  return _classRefs;
}

//----------------------------------------------------------------------------
// CLASS       : NSEventDownCastMap
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSEventDownCastMap,NSEvent)
//---------------------- Constructors / Destructor ---------------------------
NSEventDownCastMap::NSEventDownCastMap(ArCW & arCW) // Protected
  : NSEvent(arCW),
    _map()
{}

NSEventDownCastMap::~NSEventDownCastMap(void)
{}
//---------------------------- Public methods --------------------------------
//-------------------------- Stream operations -------------------------------
bool // success
NSEventDownCastMap::readFromStream(ArRef<AbstractIStream> input)
{
  if (!NSEvent::readFromStream(input))
    return false;

  unsigned int size;
// Map
  if (!input->readUInt(size))
    return false;

//  cerr << "REEAD SIZE=" << size << endl;

  for (unsigned int i=size;i--;)
  {
    StlString key,value;
 //   cerr << "MAP SIZE=" << _map.size() << endl;
    if (!input->readString(key))
      return false;
 //   _map[key]="dddd";
 //   cerr << "read=" << key << endl;
    if (!input->readString(value))
      return false;
    _map[key]=value;
  }

 /* cerr << "=> MAP SIZE=" << _map.size() << endl;
  StlMap<StlString, StlString>::const_iterator it=_map.begin();
  while(it!=_map.end())
  {
    cerr << "    + " << it->first << "=>" << it->second << endl;
    it++;
  }*/

  return true;
}

void
NSEventDownCastMap::writeToStream(ArRef<AbstractOStream> output) const
{
  NSEvent::writeToStream(output);

// Map
  output->writeUInt(_map.size());
 // cerr << "SIZE=" << _map.size() << endl;

  StlMap<StlString,StlString>::const_iterator it=_map.begin();
  StlMap<StlString,StlString>::const_iterator it_end=_map.end();
  while(it!=it_end)
  {
 // cerr << "wirte=" << it->first << endl;
    output->writeString(it->first);
    output->writeString(it->second);
    it++;
  }
}

//-------------------------- Set / Get / Access ------------------------------
// fillData from object
void
NSEventDownCastMap::setMap(const StlMap<StlString,StlString> &map)
{
  _map=map;
}

const StlMap<StlString,StlString> &
NSEventDownCastMap::getMap(void) const
{
  return _map;
}

//----------------------------------------------------------------------------
// CLASS       : NSEventObjectListenerRequest
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSEventObjectListenerRequest,NSEventRef)
//---------------------- Constructors / Destructor ---------------------------
NSEventObjectListenerRequest::NSEventObjectListenerRequest(ArCW & arCW) // Protected
  : NSEventRef(arCW)
{}

NSEventObjectListenerRequest::~NSEventObjectListenerRequest(void)
{}

} // namespace NetworkServices
//---End-Of-File---------------------------------------------------------------
