// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/Events/nsEventOSD.cpp
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReViNS/Events/nsEventOsd.h"
#include "AReVi/Utils/abstractStream.h"

#include "AReVi/Lib3D/renderer3D.h"
#include "AReVi/Lib3D/texture.h"

#include "AReVi/Lib3D/osd.h"
#include "AReVi/Lib3D/osdText.h"
#include "AReVi/Lib3D/osdSprite.h"

namespace AReVi {
//----------------------------------------------------------------------------
// CLASS       : NSEventOSDSetup
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSEventOSDSetup,NSEvent)
//---------------------- Constructors / Destructor ---------------------------
NSEventOSDSetup::NSEventOSDSetup(ArCW & arCW) // Protected
  : NSEvent(arCW)
{
}

NSEventOSDSetup::~NSEventOSDSetup(void)
{
}
//---------------------------- Public methods --------------------------------
// Read/Write views parameters
void
NSEventOSDSetup::readOSD(ArRef<OSD> osd) const
{
osd->setLocation((OSD::Location)_location);
osd->setDepth(_depth);
osd->setAlpha(_alpha);
osd->setPadding(_padding[0], _padding[1]);
osd->setPosition(_px,_py);
osd->setOrientation(_orientation);
}

void
NSEventOSDSetup::writeOSD(ArConstRef<OSD> osd)
{
_location=(int)osd->getLocation();
_depth=osd->getDepth();
_alpha=osd->getAlpha();
osd->getPadding(_padding[0], _padding[1]);
if(!osd->getPosition(Renderer3D::nullRef(),_px,_py))
  {
  _px=_py=0;
  }
_orientation=osd->getOrientation();
}

//-------------------------- Stream operations -------------------------------
bool // success
NSEventOSDSetup::readFromStream(ArRef<AbstractIStream> input)
{
//cerr << "NSEventOSDSetup::readFromStream" << endl;
if (!NSEvent::readFromStream(input) ||
    !input->readInt(_location) ||
    !input->readDouble(_offsetX) ||
    !input->readDouble(_offsetY) ||
    !input->readDouble(_depth) ||
    !input->readDouble(_alpha) ||
    !input->readDouble(_padding[0]) ||
    !input->readDouble(_padding[1]) ||
    !input->readInt(_px) ||
    !input->readInt(_py) ||
    !input->readDouble(_orientation))
  {
  return false;
  }
return true;
}

void
NSEventOSDSetup::writeToStream(ArRef<AbstractOStream> output) const
{
NSEvent::writeToStream(output);

output->writeInt(_location);
output->writeDouble(_offsetX);
output->writeDouble(_offsetY);
output->writeDouble(_depth);
output->writeDouble(_alpha);
output->writeDouble(_padding[0]);
output->writeDouble(_padding[1]);
output->writeInt(_px);
output->writeInt(_py);
output->writeDouble(_orientation);
}

//----------------------------------------------------------------------------
// CLASS       : NSEventOSDTextSetup
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSEventOSDTextSetup,NSEventOSDSetup)
//---------------------- Constructors / Destructor ---------------------------
NSEventOSDTextSetup::NSEventOSDTextSetup(ArCW & arCW) // Protected
  : NSEventOSDSetup(arCW)
{
}

NSEventOSDTextSetup::~NSEventOSDTextSetup(void)
{
}
//---------------------------- Public methods --------------------------------
// Read/Write views parameters
void
NSEventOSDTextSetup::readOSDText(ArRef<OSDText> osd) const
{
NSEventOSDSetup::readOSD(osd);

osd->setFont(_fontName);
osd->setJustification((OSDText::Justification)_justification);
osd->setColor(_color[0], _color[1], _color[2], _color[3]);
}

void
NSEventOSDTextSetup::writeOSDText(ArConstRef<OSDText> osd)
{
NSEventOSDSetup::writeOSD(osd);
_justification=(int)osd->getJustification();
_fontName=osd->getFont();
osd->getColor(_color[0], _color[1], _color[2], _color[3]);
}

//-------------------------- Stream operations -------------------------------
bool // success
NSEventOSDTextSetup::readFromStream(ArRef<AbstractIStream> input)
{
//cerr << "NSEventOSDTextSetup::readFromStream" << endl;
if (!NSEventOSDSetup::readFromStream(input) ||
    !input->readInt(_justification) ||
    !input->readString(_fontName) ||
    !input->readDouble(_color[0]) ||
    !input->readDouble(_color[1]) ||
    !input->readDouble(_color[2]) ||
    !input->readDouble(_color[3]))
  {
  return false;
  }
return true;
}

void
NSEventOSDTextSetup::writeToStream(ArRef<AbstractOStream> output) const
{
NSEventOSDSetup::writeToStream(output);

output->writeInt(_justification);
output->writeString(_fontName);
output->writeDouble(_color[0]);
output->writeDouble(_color[1]);
output->writeDouble(_color[2]);
output->writeDouble(_color[3]);
}

//----------------------------------------------------------------------------
// CLASS       : NSEventOSDText
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSEventOSDText,NSEvent)
//---------------------- Constructors / Destructor ---------------------------
NSEventOSDText::NSEventOSDText(ArCW & arCW) // Protected
  : NSEvent(arCW)
{
}

NSEventOSDText::~NSEventOSDText(void)
{
}
//---------------------------- Public methods --------------------------------
// Read/Write views parameters
void
NSEventOSDText::readOSDText(ArRef<OSDText> osd) const
{
osd->setText(_text);
}

void
NSEventOSDText::writeOSDText(ArConstRef<OSDText> osd)
{
_text=osd->getText();
}

//-------------------------- Stream operations -------------------------------
bool // success
NSEventOSDText::readFromStream(ArRef<AbstractIStream> input)
{
//cerr << "NSEventOSDText::readFromStream" << endl;
if (!NSEvent::readFromStream(input) ||
    !input->readString(_text))
  {
  return false;
  }
return true;
}

void
NSEventOSDText::writeToStream(ArRef<AbstractOStream> output) const
{
NSEvent::writeToStream(output);
output->writeString(_text);
}

//----------------------------------------------------------------------------
// CLASS       : NSEventOSDSpriteSetup
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSEventOSDSpriteSetup,NSEventOSDSetup)
//---------------------- Constructors / Destructor ---------------------------
NSEventOSDSpriteSetup::NSEventOSDSpriteSetup(ArCW & arCW) // Protected
  : NSEventOSDSetup(arCW)
{
}

NSEventOSDSpriteSetup::~NSEventOSDSpriteSetup(void)
{
}
//---------------------------- Public methods --------------------------------
// Read/Write views parameters
bool
NSEventOSDSpriteSetup::readOSDSprite(ArRef<OSDSprite> osd,
                               ArRef<RemoteDepService> depService) const
{
ArRef<Texture> texture;
if (!_texture.request(depService,texture))
  {
  osd->setErrorMessage("Can not resolve dependencies => NSEventOSDSpriteSetup fail");
  return false;
  }

NSEventOSDSetup::readOSD(osd);
osd->setScale(_scale[0],_scale[1]);
osd->setRatio(_ratio);
osd->setColorFilter(_color[0], _color[1], _color[2]);
osd->setSize(_w,_h);
osd->setTexture(texture);
return true;
}

void
NSEventOSDSpriteSetup::writeOSDSprite(ArConstRef<OSDSprite> osd)
{
NSEventOSDSetup::writeOSD(osd);
osd->getScale(_scale[0],_scale[1]);
_ratio=osd->getRatio();
osd->getColorFilter(_color[0], _color[1], _color[2]);
if (!ar_const_cast(osd)->getSize(Renderer3D::nullRef(),_w,_h))
  {
  _w=_h=0;
  }
_texture=RemoteRef(osd->getTexture());
}

//-------------------------- Stream operations -------------------------------
bool // success
NSEventOSDSpriteSetup::readFromStream(ArRef<AbstractIStream> input)
{
//cerr << "NSEventOSDSpriteSetup::readFromStream" << endl;
if (!NSEventOSDSetup::readFromStream(input) ||
    !_texture.readFromStream(input) ||
    !input->readDouble(_ratio) ||
    !input->readDouble(_scale[0]) ||
    !input->readDouble(_scale[1]) ||
    !input->readDouble(_color[0]) ||
    !input->readDouble(_color[1]) ||
    !input->readDouble(_color[2]) ||
    !input->readUInt(_w) ||
    !input->readUInt(_h))
  {
  return false;
  }
return true;
}

void
NSEventOSDSpriteSetup::writeToStream(ArRef<AbstractOStream> output) const
{
NSEventOSDSetup::writeToStream(output);

_texture.writeToStream(output);
output->writeDouble(_ratio);
output->writeDouble(_scale[0]);
output->writeDouble(_scale[1]);
output->writeDouble(_color[0]);
output->writeDouble(_color[1]);
output->writeDouble(_color[2]);
output->writeUInt(_w);
output->writeUInt(_h);
}

} // namespace NetworkServices
//---End-Of-File---------------------------------------------------------------
