// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/Events/nsEventParticleSystem.cpp
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReViNS/Events/nsEventParticleSystem.h"
#include "AReVi/Utils/abstractStream.h"
#include "AReVi/Lib3D/particleSystem.h"
#include "AReVi/Lib3D/texture.h"

namespace AReVi {
//----------------------------------------------------------------------------
// CLASS       : NSEventParticleSystemUpdate
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSEventParticleSystemUpdate,NSEvent)
//---------------------- Constructors / Destructor ---------------------------
NSEventParticleSystemUpdate::NSEventParticleSystemUpdate(ArCW & arCW) // Protected
  : NSEvent(arCW)
{}

NSEventParticleSystemUpdate::~NSEventParticleSystemUpdate(void)
{
}

void
NSEventParticleSystemUpdate::writeToStream(ArRef<AbstractOStream> output) const
{
  NSEvent::writeToStream(output);
  output->writeDouble(dt);
}

bool // success
NSEventParticleSystemUpdate::readFromStream(ArRef<AbstractIStream> input)
{
  if (!NSEvent::readFromStream(input))
    return false;
  return input->readDouble(dt);
}


//----------------------------------------------------------------------------
// CLASS       : NSEventParticleSystemRendering
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSEventParticleSystemRendering,NSEvent)
//---------------------- Constructors / Destructor ---------------------------
NSEventParticleSystemRendering::NSEventParticleSystemRendering(ArCW & arCW) // Protected
  : NSEvent(arCW)
{}

NSEventParticleSystemRendering::~NSEventParticleSystemRendering(void)
{
}

// Read/Write views parameters
void
NSEventParticleSystemRendering::readParticleSystem(ArRef<ParticleSystem> part,
                                     ArRef<RemoteDepService> depService) const
{
part->setLineAppearance(_lines);
part->setThickness(_thickness);
ArRef<Texture> texture;
_refTexture.request(depService,texture); // ignore fail
part->setTexture(texture);
part->setInitialAngle(_initAngle);
part->setInitialAngleDispersion(_initAngleDisp);
part->setAngularSpeed(_angularSpeed);
part->setAngularSpeedDispersion(_angularSpeedDisp);
part->setSize(_size[0],_size[1]);
part->setSizeDispersion(_sizeDisp[0],_sizeDisp[1]);
part->setColor(_color[0],_color[1],_color[2],_color[3]);
part->setLighting(_lighting);
}

void
NSEventParticleSystemRendering::writeParticleSystem(ArConstRef<ParticleSystem> part)
{
_lines=part->getLineAppearance();
_thickness=part->getThickness();
_refTexture=RemoteRef(part->getTexture());
_initAngle=part->getInitialAngle();
_initAngleDisp=part->getInitialAngleDispersion();
_angularSpeed=part->getAngularSpeed();
_angularSpeedDisp=part->getAngularSpeedDispersion();
part->getSize(_size[0],_size[1]);
part->getSizeDispersion(_sizeDisp[0],_sizeDisp[1]);
part->getColor(_color[0],_color[1],_color[2],_color[3]);
_lighting=part->getLighting();
}

void
NSEventParticleSystemRendering::writeToStream(ArRef<AbstractOStream> output) const
{
  NSEvent::writeToStream(output);
  output->writeUInt(_thickness);
  output->writeDouble(_initAngle);
  output->writeDouble(_initAngleDisp);
  output->writeDouble(_angularSpeed);
  output->writeDouble(_angularSpeedDisp);
  output->writeDouble(_size[0]);
  output->writeDouble(_size[1]);
  output->writeDouble(_sizeDisp[0]);
  output->writeDouble(_sizeDisp[1]);
  output->writeDouble(_color[0]);
  output->writeDouble(_color[1]);
  output->writeDouble(_color[2]);
  output->writeDouble(_color[3]);
  unsigned int flags=(_lines              ? (1<<0) : 0)+
                     (_refTexture.valid() ? (1<<1) : 0)+
                     (_lighting           ? (1<<2) : 0);
  output->writeUInt(flags);
  if(_refTexture.valid())
    {
    _refTexture.writeToStream(output);
    }
}

bool // success
NSEventParticleSystemRendering::readFromStream(ArRef<AbstractIStream> input)
{
  unsigned int flags;
  if (!NSEvent::readFromStream(input)||
      !input->readUInt(_thickness)||
      !input->readDouble(_initAngle)||
      !input->readDouble(_initAngleDisp)||
      !input->readDouble(_angularSpeed)||
      !input->readDouble(_angularSpeedDisp)||
      !input->readDouble(_size[0])||
      !input->readDouble(_size[1])||
      !input->readDouble(_sizeDisp[0])||
      !input->readDouble(_sizeDisp[1])||
      !input->readDouble(_color[0])||
      !input->readDouble(_color[1])||
      !input->readDouble(_color[2])||
      !input->readDouble(_color[3])||
      !input->readUInt(flags))
    {
    return(false);
    }
  _lines=flags&(1<<0);
  if(flags&(1<<1))
    {
    if(!_refTexture.readFromStream(input))
      {
      return(false);
      }
    }
  _lighting=flags&(1<<2);
  return true;
}

//----------------------------------------------------------------------------
// CLASS       : NSEventParticleSystemEmission
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSEventParticleSystemEmission,NSEvent)
//---------------------- Constructors / Destructor ---------------------------
NSEventParticleSystemEmission::NSEventParticleSystemEmission(ArCW & arCW) // Protected
  : NSEvent(arCW)
{}

NSEventParticleSystemEmission::~NSEventParticleSystemEmission(void)
{}


// Read/Write views parameters
void
NSEventParticleSystemEmission::readParticleSystem(ArRef<ParticleSystem> part) const
{
  part->setSection(_width,_height,_widthAngle,_heightAngle,_rounded);
  part->setParticleRate(_particleRate);
  part->setEmissionSpeed(_emissionSpeed);
  part->setEmissionSpeedDispersion(_emissionSpeedDisp);
}

void
NSEventParticleSystemEmission::writeParticleSystem(ArConstRef<ParticleSystem> part)
{
  part->getSection(_width,_height,_widthAngle,_heightAngle,_rounded);
  _particleRate=part->getParticleRate();
  _emissionSpeed=part->getEmissionSpeed();
  _emissionSpeedDisp=part->getEmissionSpeedDispersion();
}


void
NSEventParticleSystemEmission::writeToStream(ArRef<AbstractOStream> output) const
{
  NSEvent::writeToStream(output);
  output->writeDouble(_width);
  output->writeDouble(_height);
  output->writeDouble(_widthAngle);
  output->writeDouble(_heightAngle);
  output->writeDouble(_particleRate);
  output->writeDouble(_emissionSpeed);
  output->writeDouble(_emissionSpeedDisp);
  unsigned int flags=(_rounded ? (1<<0) : 0);
  output->writeUInt(flags);
}

bool // success
NSEventParticleSystemEmission::readFromStream(ArRef<AbstractIStream> input)
{
  unsigned int flags;
  if (!NSEvent::readFromStream(input)||
      !input->readDouble(_width)||
      !input->readDouble(_height)||
      !input->readDouble(_widthAngle)||
      !input->readDouble(_heightAngle)||
      !input->readDouble(_particleRate)||
      !input->readDouble(_emissionSpeed)||
      !input->readDouble(_emissionSpeedDisp)||
      !input->readUInt(flags))
    {
    return(false);
    }
  _rounded=flags&(1<<0);
  return true;
}

//----------------------------------------------------------------------------
// CLASS       : NSEventParticleSystemEvolution
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSEventParticleSystemEvolution,NSEvent)
//---------------------- Constructors / Destructor ---------------------------
NSEventParticleSystemEvolution::NSEventParticleSystemEvolution(ArCW & arCW) // Protected
  : NSEvent(arCW)
{}

NSEventParticleSystemEvolution::~NSEventParticleSystemEvolution(void)
{}


// Read/Write views parameters
void
NSEventParticleSystemEvolution::readParticleSystem(ArRef<ParticleSystem> part) const
{
  part->setParticleDuration(_particleDuration);
  part->setGravity(_gravity[0],_gravity[1],_gravity[2]);
}

void
NSEventParticleSystemEvolution::writeParticleSystem(ArConstRef<ParticleSystem> part)
{
  _particleDuration=part->getParticleDuration();
  part->getGravity(_gravity[0],_gravity[1],_gravity[2]);
}


void
NSEventParticleSystemEvolution::writeToStream(ArRef<AbstractOStream> output) const
{
  NSEvent::writeToStream(output);
  output->writeDouble(_particleDuration);
  output->writeDouble(_gravity[0]);
  output->writeDouble(_gravity[1]);
  output->writeDouble(_gravity[2]);
}

bool // success
NSEventParticleSystemEvolution::readFromStream(ArRef<AbstractIStream> input)
{
  if (!NSEvent::readFromStream(input)||
      !input->readDouble(_particleDuration)||
      !input->readDouble(_gravity[0])||
      !input->readDouble(_gravity[1])||
      !input->readDouble(_gravity[2]))
    {
    return(false);
    }
  return true;
}


//----------------------------------------------------------------------------
// CLASS       : NSEventParticleSystemAddCollider
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSEventParticleSystemAddCollider,NSEventArRef)
//---------------------- Constructors / Destructor ---------------------------
NSEventParticleSystemAddCollider::NSEventParticleSystemAddCollider(ArCW & arCW) // Protected
  : NSEventArRef(arCW)
{}

NSEventParticleSystemAddCollider::~NSEventParticleSystemAddCollider(void)
{}

//----------------------------------------------------------------------------
// CLASS       : NSEventParticleSystemRemoveCollider
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSEventParticleSystemRemoveCollider,NSEventRef)
//---------------------- Constructors / Destructor ---------------------------
NSEventParticleSystemRemoveCollider::NSEventParticleSystemRemoveCollider(ArCW & arCW) // Protected
  : NSEventRef(arCW)
{}

NSEventParticleSystemRemoveCollider::~NSEventParticleSystemRemoveCollider(void)
{}

} // namespace NetworkServices
//---End-Of-File---------------------------------------------------------------
