// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/Events/nsEventRenderTexture.cpp
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReViNS/Events/nsEventRenderTexture.h"
#include "AReVi/Utils/abstractStream.h"
#include "AReVi/Lib3D/renderTexture.h"

namespace AReVi {

//----------------------------------------------------------------------------
// CLASS       : NSEventRenderTextureSetup
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSEventRenderTextureSetup,NSEvent)
//---------------------- Constructors / Destructor ---------------------------
NSEventRenderTextureSetup::NSEventRenderTextureSetup(ArCW & arCW) // Protected
  : NSEvent(arCW),
    _needsBlending(false),
    _needsAlphaTest(false)
{}

NSEventRenderTextureSetup::~NSEventRenderTextureSetup(void)
{}

//---------------------------- Public methods --------------------------------
// Read/Write data parameters
void
NSEventRenderTextureSetup::writeRenderTexture(ArConstRef<RenderTexture> renderTexture)
{
  _needsBlending=renderTexture->getBlending();
  _needsAlphaTest=renderTexture->getAlphaTest();
}

void
NSEventRenderTextureSetup::readRenderTexture(ArRef<RenderTexture> renderTexture) const
{
  renderTexture->setBlending(_needsBlending);
  renderTexture->setAlphaTest(_needsAlphaTest);
}

//-------------------------- Stream operations -------------------------------
void
NSEventRenderTextureSetup::writeToStream(ArRef<AbstractOStream> output) const
{
  NSEvent::writeToStream(output);

  unsigned int flags=(_needsBlending  ? 1<<0 : 0)+
                     (_needsAlphaTest ? 1<<1 : 0);
  output->writeUInt(flags);
}

bool // success
NSEventRenderTextureSetup::readFromStream(ArRef<AbstractIStream> input)
{
  unsigned int flags;

  if (!NSEvent::readFromStream(input)||
      !input->readUInt(flags))
  {
  return(false);
  }

  _needsBlending=flags&(1<<0);
  _needsAlphaTest=flags&(1<<1);
  return true;
}


} // namespace NetworkServices
//---End-Of-File---------------------------------------------------------------
