// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/Events/nsEventRenderer3D.cpp
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReViNS/Events/nsEventRenderer3D.h"
#include "AReVi/Utils/abstractStream.h"
#include "AReVi/Lib3D/renderer3D.h"

namespace AReVi {
//----------------------------------------------------------------------------
// CLASS       : NSEventRenderer3DViewParameters
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSEventRenderer3DViewParameters,NSEvent)
//---------------------- Constructors / Destructor ---------------------------
NSEventRenderer3DViewParameters::NSEventRenderer3DViewParameters(ArCW & arCW) // Protected
  : NSEvent(arCW)
{
}

NSEventRenderer3DViewParameters::~NSEventRenderer3DViewParameters(void)
{
}
//---------------------------- Public methods --------------------------------
// Read/Write views parameters
void
NSEventRenderer3DViewParameters::readRendererViewsParameters(ArRef<Renderer3D> renderer) const
{
  renderer->setFieldOfView(_fov);
  renderer->setNearDistance(_near);
  renderer->setFarDistance(_far);
  renderer->setNearYOffset(_nearYOffset);
  renderer->setNearZOffset(_nearZOffset);
  renderer->setFixedAspectRatio(_fixedRatio);

  renderer->setWireFrame(_wireFrame);
  renderer->setHeadLight(_headLight);
  renderer->setOriginVisible(_originVisible);
  renderer->setShapeBoundingBox(_drawShapeBBox);
  renderer->setShapePartBoundingBox(_drawShapePartBBox);
  renderer->setStereoMode((Renderer3D::StereoMode)_stereoMode);
  renderer->setStereoLeftColor(_leftColor[0],_leftColor[1],_leftColor[2]);
  renderer->setStereoRightColor(_rightColor[0],_rightColor[1],_rightColor[2]);
  renderer->setColorStereoMaxResolution(_stereoMaxWidth,_stereoMaxHeight);
  renderer->setStereoGlance(_drawStereoGlance);
  renderer->setStereoViewDistance(_stereoViewDistance);
  renderer->setStereoEyeSeparation(_stereoEyeSeparation);
}

void
NSEventRenderer3DViewParameters::writeRendererViewsParameters(ArConstRef<Renderer3D> renderer)
{
  _fov=renderer->getFieldOfView();
  _near=renderer->getNearDistance();
  _far=renderer->getFarDistance();
  _nearYOffset=renderer->getNearYOffset();
  _nearZOffset=renderer->getNearZOffset();
  _fixedRatio=renderer->getFixedAspectRatio();

  _wireFrame=renderer->getWireFrame();
  _headLight=renderer->getHeadLight();
  _originVisible=renderer->getOriginVisible();
  _drawShapeBBox=renderer->getShapeBoundingBox();
  _drawShapePartBBox=renderer->getShapePartBoundingBox();
  _stereoMode=renderer->getStereoMode();
  renderer->getStereoLeftColor(_leftColor[0],_leftColor[1],_leftColor[2]);
  renderer->getStereoRightColor(_rightColor[0],_rightColor[1],_rightColor[2]);
  renderer->getColorStereoMaxResolution(_stereoMaxWidth,_stereoMaxHeight);
  _drawStereoGlance=renderer->getStereoGlance();
  _stereoViewDistance=renderer->getStereoViewDistance();
  _stereoEyeSeparation=renderer->getStereoEyeSeparation();
}

//-------------------------- Stream operations -------------------------------
bool // success
NSEventRenderer3DViewParameters::readFromStream(ArRef<AbstractIStream> input)
{
//cerr << "NSEventRenderer3DViewParameters::readFromStream" << endl;
  if (!NSEvent::readFromStream(input))
    return false;

  if(!input->readDouble(_fov))
      return false;
  if(!input->readDouble(_near))
      return false;
  if(!input->readDouble(_far))
      return false;
  if(!input->readDouble(_nearYOffset))
      return false;
  if(!input->readDouble(_nearZOffset))
      return false;
  if(!input->readDouble(_fixedRatio))
      return false;
  if(!input->readUInt(_stereoMode))
      return false;
  if(!input->readDouble(_leftColor[0]))
      return false;
  if(!input->readDouble(_leftColor[1]))
      return false;
  if(!input->readDouble(_leftColor[2]))
      return false;
  if(!input->readDouble(_rightColor[0]))
      return false;
  if(!input->readDouble(_rightColor[1]))
      return false;
  if(!input->readDouble(_rightColor[2]))
      return false;
  if(!input->readUInt(_stereoMaxWidth))
      return false;
  if(!input->readUInt(_stereoMaxHeight))
      return false;
  if(!input->readDouble(_stereoViewDistance))
      return false;
  if(!input->readDouble(_stereoEyeSeparation))
      return false;

// Settings (wireFrame, Headlight, originVisible, draw Shape/shapePart BBox
  unsigned setupFlags;
  if (!input->readUInt(setupFlags))
    return false;

  _wireFrame=setupFlags&1;
  _headLight=setupFlags&2;
  _originVisible=setupFlags&4;
  _drawShapeBBox=setupFlags&8;
  _drawShapePartBBox=setupFlags&16;
  _drawStereoGlance=setupFlags&32;

  return true;
}

void
NSEventRenderer3DViewParameters::writeToStream(ArRef<AbstractOStream> output) const
{
  NSEvent::writeToStream(output);

  output->writeDouble(_fov);
  output->writeDouble(_near);
  output->writeDouble(_far);
  output->writeDouble(_nearYOffset);
  output->writeDouble(_nearZOffset);
  output->writeDouble(_fixedRatio);
  output->writeUInt(_stereoMode);
  output->writeDouble(_leftColor[0]);
  output->writeDouble(_leftColor[1]);
  output->writeDouble(_leftColor[2]);
  output->writeDouble(_rightColor[0]);
  output->writeDouble(_rightColor[1]);
  output->writeDouble(_rightColor[2]);
  output->writeUInt(_stereoMaxWidth);
  output->writeUInt(_stereoMaxHeight);
  output->writeDouble(_stereoViewDistance);
  output->writeDouble(_stereoEyeSeparation);

// Settings (wireFrame, Headlight, originVisible, draw Shape/shapePart BBox
  unsigned setupFlags((_wireFrame&1)+
                      ((_headLight&1)<<1)+
                      ((_originVisible&1)<<2)+
                      ((_drawShapeBBox&1)<<3)+
                      ((_drawShapePartBBox&1)<<4)+
                      ((_drawStereoGlance&1)<<5));
  output->writeUInt(setupFlags);
}


//----------------------------------------------------------------------------
// CLASS       : NSEventRenderer3DAppearanceSetup
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSEventRenderer3DAppearanceSetup,NSEvent)
//---------------------- Constructors / Destructor ---------------------------
NSEventRenderer3DAppearanceSetup::NSEventRenderer3DAppearanceSetup(ArCW & arCW) // Protected
  : NSEvent(arCW)
{
}

NSEventRenderer3DAppearanceSetup::~NSEventRenderer3DAppearanceSetup(void)
{
}
//---------------------------- Public methods --------------------------------
// Read/Write views parameters
void
NSEventRenderer3DAppearanceSetup::readRendererAppearanceSetup(ArRef<Renderer3D> renderer) const
{
  renderer->setBackgroundColor(_backgroundColor[0],
                               _backgroundColor[1],
                               _backgroundColor[2]);
  renderer->setHeadLightColor(_headLightColor[0],
                              _headLightColor[1],
                              _headLightColor[2]);
  renderer->setBoundingBoxColor(_bboxColor[0],
                                _bboxColor[1],
                                _bboxColor[2]);
}

void
NSEventRenderer3DAppearanceSetup::writeRendererAppearanceSetup(ArConstRef<Renderer3D> renderer)
{
  renderer->getBackgroundColor(_backgroundColor[0],                                                            _backgroundColor[1],
                               _backgroundColor[2]);
  renderer->getHeadLightColor(_headLightColor[0],                                                             _headLightColor[1],
                              _headLightColor[2]);
  renderer->getBoundingBoxColor(_bboxColor[0],
                                _bboxColor[1],
                                _bboxColor[2]);
}

//-------------------------- Stream operations -------------------------------
bool // success
NSEventRenderer3DAppearanceSetup::readFromStream(ArRef<AbstractIStream> input)
{
  if (!NSEvent::readFromStream(input))
    return false;
// Colors (Back, Light, BBox)
  if (!input->readDouble(_backgroundColor[0]))
    return false;
  if (!input->readDouble(_backgroundColor[1]))
    return false;
  if (!input->readDouble(_backgroundColor[2]))
    return false;

  if (!input->readDouble(_headLightColor[0]))
    return false;
  if (!input->readDouble(_headLightColor[1]))
    return false;
  if (!input->readDouble(_headLightColor[2]))
    return false;

  if (!input->readDouble(_bboxColor[0]))
    return false;
  if (!input->readDouble(_bboxColor[1]))
    return false;
  if (!input->readDouble(_bboxColor[2]))
    return false;
  return true;
}

void
NSEventRenderer3DAppearanceSetup::writeToStream(ArRef<AbstractOStream> output) const
{
  NSEvent::writeToStream(output);
// Colors (Back, Light, BBox)
  output->writeDouble(_backgroundColor[0]);
  output->writeDouble(_backgroundColor[1]);
  output->writeDouble(_backgroundColor[2]);

  output->writeDouble(_headLightColor[0]);
  output->writeDouble(_headLightColor[1]);
  output->writeDouble(_headLightColor[2]);

  output->writeDouble(_bboxColor[0]);
  output->writeDouble(_bboxColor[1]);
  output->writeDouble(_bboxColor[2]);
}


//----------------------------------------------------------------------------
// CLASS       : NSEventRenderer3DRenderingSetup
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSEventRenderer3DRenderingSetup,NSEvent)
//---------------------- Constructors / Destructor ---------------------------
NSEventRenderer3DRenderingSetup::NSEventRenderer3DRenderingSetup(ArCW & arCW) // Protected
  : NSEvent(arCW),
    _mainScene(""),_mainClass(""),
    _frontScene(""),_frontClass(""),
    _backScene(""),_backClass("")
{
}

NSEventRenderer3DRenderingSetup::~NSEventRenderer3DRenderingSetup(void)
{
}
//---------------------------- Public methods --------------------------------
//-------------------------- Stream operations -------------------------------
bool // success
NSEventRenderer3DRenderingSetup::readFromStream(ArRef<AbstractIStream> input)
{
  if (!NSEvent::readFromStream(input))
    return false;

  if (!input->readString(_mainScene))
    return false;
  if (!input->readString(_mainClass))
    return false;
  if (!input->readString(_frontScene))
    return false;
  if (!input->readString(_frontClass))
    return false;
  if (!input->readString(_backScene))
    return false;
  if (!input->readString(_backClass))
    return false;
  return true;
}

void
NSEventRenderer3DRenderingSetup::writeToStream(ArRef<AbstractOStream> output) const
{
  NSEvent::writeToStream(output);

  output->writeString(_mainScene);
  output->writeString(_mainClass);
  output->writeString(_frontScene);
  output->writeString(_frontClass);
  output->writeString(_backScene);
  output->writeString(_backClass);
}

//----------------------------------------------------------------------------
// CLASS       : NSEventRenderer3DAddOSD
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSEventRenderer3DAddOSD,NSEventArRef)
//---------------------- Constructors / Destructor ---------------------------
NSEventRenderer3DAddOSD::NSEventRenderer3DAddOSD(ArCW & arCW) // Protected
  : NSEventArRef(arCW)
{}

NSEventRenderer3DAddOSD::~NSEventRenderer3DAddOSD(void)
{}

//----------------------------------------------------------------------------
// CLASS       : NSEventRenderer3DRemoveOSD
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSEventRenderer3DRemoveOSD,NSEventRef)
//---------------------- Constructors / Destructor ---------------------------
NSEventRenderer3DRemoveOSD::NSEventRenderer3DRemoveOSD(ArCW & arCW) // Protected
  : NSEventRef(arCW)
{}

NSEventRenderer3DRemoveOSD::~NSEventRenderer3DRemoveOSD(void)
{}

} // namespace NetworkServices
//---End-Of-File---------------------------------------------------------------
