// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/Events/nsEventScene3D.cpp
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReViNS/Events/nsEventScene3D.h"

#include "AReVi/Lib3D/scene3D.h"
#include "AReVi/Lib3D/transform2D.h"
#include "AReVi/Lib3D/texture.h"
#include "AReVi/Utils/abstractStream.h"

namespace AReVi {
//----------------------------------------------------------------------------
// CLASS       : NSEventScene3DSetup
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSEventScene3DSetup,NSEvent)
//---------------------- Constructors / Destructor ---------------------------
NSEventScene3DSetup::NSEventScene3DSetup(ArCW & arCW) // Protected
  : NSEvent(arCW),
    _transform(Transform2D::NEW())
{}

NSEventScene3DSetup::~NSEventScene3DSetup(void)
{}

//---------------------------- Public methods --------------------------------
// Read/Write sound parameters
bool
NSEventScene3DSetup::readScene3D(ArRef<Scene3D> scene,
                                 ArRef<RemoteDepService> depService) const
{
  ArRef<Texture> texture;
  if (!_backgroundTex.request(depService,texture))
    return false;

  scene->setBackground(texture);
  scene->setFogColor(_fogColor[0],_fogColor[1],_fogColor[2]);
  scene->setFog(_fogDegree,_fogDensity);
  scene->writeBackgroundTransformation(_transform);
  return true;
}

void
NSEventScene3DSetup::writeScene3D(ArConstRef<Scene3D> scene)
{
  _backgroundTex=RemoteRef(scene->getBackground());
  scene->getFogColor(_fogColor[0],_fogColor[1],_fogColor[2]);
  scene->getFog(_fogDegree,_fogDensity);
  scene->readBackgroundTransformation(_transform);
}

//-------------------------- Stream operations -------------------------------
void
NSEventScene3DSetup::writeToStream(ArRef<AbstractOStream> output) const
{
  NSEvent::writeToStream(output);

  output->writeDouble(_fogColor[0]);
  output->writeDouble(_fogColor[1]);
  output->writeDouble(_fogColor[2]);

  output->writeUInt(_fogDegree);
  output->writeDouble(_fogDensity);
  _backgroundTex.writeToStream(output);

  for(unsigned int i=16;i--;)
    output->writeDouble(_transform->getTransformation().matrix[i]);
}

bool // success
NSEventScene3DSetup::readFromStream(ArRef<AbstractIStream> input)
{
  if (!NSEvent::readFromStream(input) ||
      !input->readDouble(_fogColor[0]) ||
      !input->readDouble(_fogColor[1]) ||
      !input->readDouble(_fogColor[2]) ||
      !input->readUInt(_fogDegree) ||
      !input->readDouble(_fogDensity) ||
      !_backgroundTex.readFromStream(input))
    {
    return false;
    }

  double matrix[16];
  for(unsigned int i=16;i--;)
  {
  if(!input->readDouble(matrix[i]))
    {
    return(false);
    }
  }
  _transform->accessTransformation().setMatrix(matrix);
  _transform->accessTransformation().isIdentity(true);
  return true;
}


//----------------------------------------------------------------------------
// CLASS       : NSEventScene3DAddObject
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSEventScene3DAddObject,NSEventArRef)
//---------------------- Constructors / Destructor ---------------------------
NSEventScene3DAddObject::NSEventScene3DAddObject(ArCW & arCW) // Protected
  : NSEventArRef(arCW)
{}

NSEventScene3DAddObject::~NSEventScene3DAddObject(void)
{}

//----------------------------------------------------------------------------
// CLASS       : NSEventScene3DRemoveObject
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSEventScene3DRemoveObject,NSEventRef)
//---------------------- Constructors / Destructor ---------------------------
NSEventScene3DRemoveObject::NSEventScene3DRemoveObject(ArCW & arCW) // Protected
  : NSEventRef(arCW)
{}

NSEventScene3DRemoveObject::~NSEventScene3DRemoveObject(void)
{}

//----------------------------------------------------------------------------
// CLASS       : NSEventScene3DSelectObject
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSEventScene3DSelectObject,NSEventArRef)
//---------------------- Constructors / Destructor ---------------------------
NSEventScene3DSelectObject::NSEventScene3DSelectObject(ArCW & arCW) // Protected
  : NSEventArRef(arCW)
{}

NSEventScene3DSelectObject::~NSEventScene3DSelectObject(void)
{}

//----------------------------------------------------------------------------
// CLASS       : NSEventScene3DUnselectObject
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSEventScene3DUnselectObject,NSEventRef)
//---------------------- Constructors / Destructor ---------------------------
NSEventScene3DUnselectObject::NSEventScene3DUnselectObject(ArCW & arCW) // Protected
  : NSEventRef(arCW)
{}

NSEventScene3DUnselectObject::~NSEventScene3DUnselectObject(void)
{}


//----------------------------------------------------------------------------
// CLASS       : NSEventScene3DAddParticleSystem
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSEventScene3DAddParticleSystem,NSEventArRef)
//---------------------- Constructors / Destructor ---------------------------
NSEventScene3DAddParticleSystem::NSEventScene3DAddParticleSystem(ArCW & arCW) // Protected
  : NSEventArRef(arCW)
{}

NSEventScene3DAddParticleSystem::~NSEventScene3DAddParticleSystem(void)
{}

//----------------------------------------------------------------------------
// CLASS       : NSEventScene3DRemoveParticleSystem
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSEventScene3DRemoveParticleSystem,NSEventRef)
//---------------------- Constructors / Destructor ---------------------------
NSEventScene3DRemoveParticleSystem::NSEventScene3DRemoveParticleSystem(ArCW & arCW) // Protected
  : NSEventRef(arCW)
{}

NSEventScene3DRemoveParticleSystem::~NSEventScene3DRemoveParticleSystem(void)
{}

//----------------------------------------------------------------------------
// CLASS       : NSEventScene3DAddLight
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSEventScene3DAddLight,NSEventArRef)
//---------------------- Constructors / Destructor ---------------------------
NSEventScene3DAddLight::NSEventScene3DAddLight(ArCW & arCW) // Protected
  : NSEventArRef(arCW)
{}

NSEventScene3DAddLight::~NSEventScene3DAddLight(void)
{}

//----------------------------------------------------------------------------
// CLASS       : NSEventScene3DRemoveLight
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSEventScene3DRemoveLight,NSEventRef)
//---------------------- Constructors / Destructor ---------------------------
NSEventScene3DRemoveLight::NSEventScene3DRemoveLight(ArCW & arCW) // Protected
  : NSEventRef(arCW)
{}

NSEventScene3DRemoveLight::~NSEventScene3DRemoveLight(void)
{}

//----------------------------------------------------------------------------
// CLASS       : NSEventScene3DAddSound
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSEventScene3DAddSound,NSEventArRef)
//---------------------- Constructors / Destructor ---------------------------
NSEventScene3DAddSound::NSEventScene3DAddSound(ArCW & arCW) // Protected
  : NSEventArRef(arCW)
{}

NSEventScene3DAddSound::~NSEventScene3DAddSound(void)
{}

//----------------------------------------------------------------------------
// CLASS       : NSEventScene3DRemoveSound
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSEventScene3DRemoveSound,NSEventRef)
//---------------------- Constructors / Destructor ---------------------------
NSEventScene3DRemoveSound::NSEventScene3DRemoveSound(ArCW & arCW) // Protected
  : NSEventRef(arCW)
{}

NSEventScene3DRemoveSound::~NSEventScene3DRemoveSound(void)
{}

} // namespace NetworkServices
//---End-Of-File---------------------------------------------------------------
