// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/Events/nsEventShape3D.cpp
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReViNS/Events/nsEventShape3D.h"
#include "AReVi/Utils/abstractStream.h"
#include "AReVi/Shapes/shapePart3D.h"

namespace AReVi {
//----------------------------------------------------------------------------
// CLASS       : NSEventTransform3D
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSEventTransform3D,NSEvent)
//---------------------- Constructors / Destructor ---------------------------
NSEventTransform3D::NSEventTransform3D(ArCW & arCW) // Protected
  : NSEvent(arCW),_transform(false)
{
}

NSEventTransform3D::~NSEventTransform3D(void)
{}

// Get Set Transform3D attrib
const Util3D::Transform &
NSEventTransform3D::getTransform(void) const
{
  return _transform;
}

void
NSEventTransform3D::setTransform(const Util3D::Transform &transform)
{
  _transform=transform;
}

//-------------------------- Stream operations -------------------------------
void
NSEventTransform3D::writeToStream(ArRef<AbstractOStream> output) const
{
  NSEvent::writeToStream(output);

  for (unsigned cpt=16;cpt--;)
    output->writeDouble(_transform.matrix[cpt]);
}

bool // success
NSEventTransform3D::readFromStream(ArRef<AbstractIStream> input)
{
  if (!NSEvent::readFromStream(input))
    return false;

  for (unsigned cpt=16;cpt--;)
    if (!input->readDouble(_transform.matrix[cpt]))
      return false;

  _transform.isIdentity(true);

  return true;
}

//----------------------------------------------------------------------------
// CLASS       : NSEventShapePart3DList
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSEventShapePart3DList,NSEventArRefList)
//---------------------- Constructors / Destructor ---------------------------
NSEventShapePart3DList::NSEventShapePart3DList(ArCW & arCW) // Protected
  : NSEventArRefList(arCW)
{
}

NSEventShapePart3DList::~NSEventShapePart3DList(void)
{}

// add ShapePart 3D reference
void
NSEventShapePart3DList::addRef(ArConstRef<ArObject> object)
{
  if (object.valid()) // add only if is a valid ShapePart3D ref.
  {
    if (object->getClass()->isA(ShapePart3D::CLASS()))
      NSEventArRefList::addRef(object);
  }
}

} // namespace NetworkServices
//---End-Of-File---------------------------------------------------------------
