// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/Events/nsEventSound3D.cpp
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReViNS/Events/nsEventSound3D.h"
#include "AReVi/Utils/abstractStream.h"
#include "AReVi/Lib3D/sound3D.h"

namespace AReVi {
//----------------------------------------------------------------------------
// CLASS       : NSEventSound3DSetup
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSEventSound3DSetup,NSEvent)
//---------------------- Constructors / Destructor ---------------------------
NSEventSound3DSetup::NSEventSound3DSetup(ArCW & arCW) // Protected
  : NSEvent(arCW)
{
}

NSEventSound3DSetup::~NSEventSound3DSetup(void)
{
}
//---------------------------- Public methods --------------------------------
// Get URL name => check existance!
/*const StlString &
NSEventSound3DSetup::getURL(void) const
{
  return _url;
}
*/
// Read/Write views parameters
bool
NSEventSound3DSetup::readSound3D(ArRef<Sound3D> sound/*,
                                 const  StlString &localURL*/) const
{
  sound->setGain(_gain);
  sound->setPitch(_pitch);
  sound->setDistanceAttenuation(_minDistance,_maxDistance);
  sound->setDirectionalAttenuation(_innerAngle,_outerAngle,_outerGain);

  sound->setDirectional(_directional);
  sound->setRepeat(_repeat);
  sound->setAttenuated(_attenuated);
  //cerr << "LOAD URL " << localURL << endl;
  //return sound->loadURL(localURL);
  return false;
}

void
NSEventSound3DSetup::writeSound3D(ArConstRef<Sound3D> sound)
{
  _repeat=sound->getRepeat();
  _gain=sound->getGain();
  _pitch=sound->getPitch();
  sound->getDistanceAttenuation(_minDistance,_maxDistance);
  _directional=sound->getDirectional();
  sound->getDirectionalAttenuation(_innerAngle,_outerAngle,_outerGain);
  _attenuated=sound->getAttenuated();
  //_url=getURL();
}

//-------------------------- Stream operations -------------------------------
bool // success
NSEventSound3DSetup::readFromStream(ArRef<AbstractIStream> input)
{
//cerr << "NSEventSound3DSetup::readFromStream" << endl;
  if (!NSEvent::readFromStream(input))
    return false;

  // Read sound setup
  input->readDouble(_gain);
  input->readDouble(_pitch);
  input->readDouble(_minDistance);
  input->readDouble(_maxDistance);
  input->readDouble(_innerAngle);
  input->readDouble(_outerAngle);
  input->readDouble(_outerGain);

  unsigned int flags;
  input->readUInt(flags);
  _repeat=flags&1;
  _directional=flags&2;
  _attenuated=flags&4;

  return true;
}

void
NSEventSound3DSetup::writeToStream(ArRef<AbstractOStream> output) const
{
  NSEvent::writeToStream(output);

   // Write setup
  output->writeDouble(_gain);
  output->writeDouble(_pitch);
  output->writeDouble(_minDistance);
  output->writeDouble(_maxDistance);
  output->writeDouble(_innerAngle);
  output->writeDouble(_outerAngle);
  output->writeDouble(_outerGain);

  unsigned int flags(((unsigned )_repeat)+((unsigned )_directional<<1)+((unsigned )_attenuated<<2));
  output->writeUInt(flags);
}

//----------------------------------------------------------------------------
// CLASS       : NSEventSound3DControl
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSEventSound3DControl,NSEvent)
//---------------------- Constructors / Destructor ---------------------------
NSEventSound3DControl::NSEventSound3DControl(ArCW & arCW) // Protected
  : NSEvent(arCW)
{}

NSEventSound3DControl::~NSEventSound3DControl(void)
{
}

// Read/Write sound controls
void
NSEventSound3DControl::readSound3D(ArRef<Sound3D> sound) const
{
  if (_playMode)
    sound->play();
  else if (_paused)
    sound->pause();
  else
    sound->stop();

}

void
NSEventSound3DControl::writeSound3D(ArConstRef<Sound3D> sound)
{
  _paused=sound->isPaused();
  _stopped=sound->isStopped();
  _playMode=sound->getPlayMode();
}

void
NSEventSound3DControl::writeToStream(ArRef<AbstractOStream> output) const
{
  NSEvent::writeToStream(output);
  unsigned int flags=(_stopped     ? 1<<0 : 0)+
                     (_paused      ? 1<<1 : 0)+
                     (_playMode    ? 1<<2 : 0);
  output->writeUInt(flags);

  //cerr << "WRITE COMMAND PAUSED=" << _paused <<  "    STOOPED=" << _stopped << endl;
}

bool // success
NSEventSound3DControl::readFromStream(ArRef<AbstractIStream> input)
{
  if (!NSEvent::readFromStream(input))
    return false;

  unsigned int flags;
  input->readUInt(flags);

  _stopped=flags&1;
  _paused=flags&2;
  _playMode=flags&4;

  //cerr << "READ COMMAND PAUSED=" << _paused <<  "    STOOPED=" << _stopped << endl;
  return true;
}

} // namespace NetworkServices
//---End-Of-File---------------------------------------------------------------
