// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/Events/nsEventSurface3D.cpp
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReViNS/Events/nsEventSurface3D.h"
#include "AReVi/Utils/abstractStream.h"
#include "AReVi/Lib3D/material3D.h"
#include "AReVi/Lib3D/texture.h"
#include "AReVi/Lib3D/transform2D.h"
#include "AReVi/Shapes/surface3D.h"

namespace AReVi {
//----------------------------------------------------------------------------
// CLASS       : NSEventSurface3DAppearanceRebuild
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSEventSurface3DAppearanceRebuild,NSEventShapePart3DAppearanceRebuild)
//---------------------- Constructors / Destructor ---------------------------
NSEventSurface3DAppearanceRebuild::NSEventSurface3DAppearanceRebuild(ArCW & arCW) // Protected
  : NSEventShapePart3DAppearanceRebuild(arCW),
    _material(Material3D::NEW()),
    _doubleSided(false),
    _refTexture(),
    _refTexture2(),
    _refTexture3(),
    _refTexture4(),
    _texTransform(Transform2D::NEW()),
    _texTransform2(Transform2D::NEW()),
    _texTransform3(Transform2D::NEW()),
    _texTransform4(Transform2D::NEW())
{
}

NSEventSurface3DAppearanceRebuild::~NSEventSurface3DAppearanceRebuild(void)
{}

void
NSEventSurface3DAppearanceRebuild::writeAppearance(ArConstRef<Surface3D> surface)
{
  surface->readMaterial(_material);
  _doubleSided=surface->getDoubleSided();
  _refTexture=RemoteRef(surface->getTexture());
  surface->readTextureTransformation(_texTransform);
  surface->getTextureRepeat(_repeatX,_repeatY);
  _cubeMapReflect=surface->getTextureReflection();

  _refTexture2=RemoteRef(surface->getSecondTexture());
  surface->readSecondTextureTransformation(_texTransform2);
  surface->getSecondTextureRepeat(_repeatX2,_repeatY2);
  _cubeMapReflect2=surface->getSecondTextureReflection();

  _refTexture3=RemoteRef(surface->getThirdTexture());
  surface->readThirdTextureTransformation(_texTransform3);
  surface->getThirdTextureRepeat(_repeatX3,_repeatY3);
  _cubeMapReflect3=surface->getThirdTextureReflection();

  _refTexture4=RemoteRef(surface->getFourthTexture());
  surface->readFourthTextureTransformation(_texTransform4);
  surface->getFourthTextureRepeat(_repeatX4,_repeatY4);
  _cubeMapReflect4=surface->getFourthTextureReflection();
}


void
NSEventSurface3DAppearanceRebuild::readAppearance(ArRef<Surface3D> surfaceOut,
                                                  ArRef<RemoteDepService> depService) const
{
  surfaceOut->writeMaterial(_material);
  surfaceOut->setDoubleSided(_doubleSided);

  ArRef<Texture> texture;
  _refTexture.request(depService,texture); // ignore fail
  surfaceOut->setTexture(texture);
  surfaceOut->writeTextureTransformation(_texTransform);
  surfaceOut->setTextureRepeat(_repeatX,_repeatY);
  surfaceOut->setTextureReflection(_cubeMapReflect);

  _refTexture2.request(depService,texture); // ignore fail
  surfaceOut->writeSecondTextureTransformation(_texTransform2);
  surfaceOut->setSecondTexture(texture);
  surfaceOut->setSecondTextureRepeat(_repeatX2,_repeatY2);
  surfaceOut->setSecondTextureReflection(_cubeMapReflect2);

  _refTexture3.request(depService,texture); // ignore fail
  surfaceOut->writeThirdTextureTransformation(_texTransform3);
  surfaceOut->setThirdTexture(texture);
  surfaceOut->setThirdTextureRepeat(_repeatX3,_repeatY3);
  surfaceOut->setThirdTextureReflection(_cubeMapReflect3);

  _refTexture4.request(depService,texture); // ignore fail
  surfaceOut->writeFourthTextureTransformation(_texTransform4);
  surfaceOut->setFourthTexture(texture);
  surfaceOut->setFourthTextureRepeat(_repeatX4,_repeatY4);
  surfaceOut->setFourthTextureReflection(_cubeMapReflect4);
}

//-------------------------- Stream operations -------------------------------
void
NSEventSurface3DAppearanceRebuild::writeToStream(ArRef<AbstractOStream> output) const
{
  NSEventShapePart3DAppearanceRebuild::writeToStream(output);
  _material->writeToStream(output);

  bool hasTexTransform=!_texTransform->isIdentity(false);
  bool hasTexTransform2=!_texTransform2->isIdentity(false);
  bool hasTexTransform3=!_texTransform3->isIdentity(false);
  bool hasTexTransform4=!_texTransform4->isIdentity(false);

  unsigned int appearanceFlags=(_doubleSided         ? 1<<0  : 0)+
                               (_refTexture.valid()  ? 1<<1  : 0)+
                               (_refTexture2.valid() ? 1<<2  : 0)+
                               (_refTexture3.valid() ? 1<<3  : 0)+
                               (_refTexture4.valid() ? 1<<4  : 0)+
                               (hasTexTransform      ? 1<<5  : 0)+
                               (hasTexTransform2     ? 1<<6  : 0)+
                               (hasTexTransform3     ? 1<<7  : 0)+
                               (hasTexTransform4     ? 1<<8  : 0)+
                               (_repeatX             ? 1<<9  : 0)+
                               (_repeatY             ? 1<<10 : 0)+
                               (_repeatX2            ? 1<<11 : 0)+
                               (_repeatY2            ? 1<<12 : 0)+
                               (_repeatX3            ? 1<<13 : 0)+
                               (_repeatY3            ? 1<<14 : 0)+
                               (_repeatX4            ? 1<<15 : 0)+
                               (_repeatY4            ? 1<<16 : 0)+
                               (_cubeMapReflect      ? 1<<17 : 0)+
                               (_cubeMapReflect2     ? 1<<18 : 0)+
                               (_cubeMapReflect3     ? 1<<19 : 0)+
                               (_cubeMapReflect4     ? 1<<20 : 0);

  output->writeUInt(appearanceFlags);

  if(_refTexture.valid())
    {
    _refTexture.writeToStream(output);
    }
  if(_refTexture2.valid())
    {
    _refTexture2.writeToStream(output);
    }
  if(_refTexture3.valid())
    {
    _refTexture3.writeToStream(output);
    }
  if(_refTexture4.valid())
    {
    _refTexture4.writeToStream(output);
    }
  unsigned int i;
  if(hasTexTransform)
    {
    for(i=16;i--;)
      {
      output->writeDouble(_texTransform->getTransformation().matrix[i]);
      }
    }
  if(hasTexTransform2)
    {
    for(i=16;i--;)
      {
      output->writeDouble(_texTransform2->getTransformation().matrix[i]);
      }
    }
  if(hasTexTransform3)
    {
    for(i=16;i--;)
      {
      output->writeDouble(_texTransform3->getTransformation().matrix[i]);
      }
    }
  if(hasTexTransform4)
    {
    for(i=16;i--;)
      {
      output->writeDouble(_texTransform4->getTransformation().matrix[i]);
      }
    }
}

bool // success
NSEventSurface3DAppearanceRebuild::readFromStream(ArRef<AbstractIStream> input)
{
  unsigned int flags;
  if(!NSEventShapePart3DAppearanceRebuild::readFromStream(input)||
     !_material->readFromStream(input)||
     !input->readUInt(flags))
    {
    return false;
    }
  _doubleSided=flags&(1<<0);
  _repeatX=flags&(1<<9);
  _repeatY=flags&(1<<10);
  _cubeMapReflect=flags&(1<<17);
  _repeatX2=flags&(1<<11);
  _repeatY2=flags&(1<<12);
  _cubeMapReflect2=flags&(1<<18);
  _repeatX3=flags&(1<<13);
  _repeatY3=flags&(1<<14);
  _cubeMapReflect3=flags&(1<<19);
  _repeatX4=flags&(1<<15);
  _repeatY4=flags&(1<<16);
  _cubeMapReflect4=flags&(1<<20);

  if(flags&(1<<1))
    {
    if(!_refTexture.readFromStream(input))
      {
      return(false);
      }
    }
  else
    {
    _refTexture=RemoteRef();
    }
  if(flags&(1<<2))
    {
    if(!_refTexture2.readFromStream(input))
      {
      return(false);
      }
    }
  else
    {
    _refTexture2=RemoteRef();
    }
  if(flags&(1<<3))
    {
    if(!_refTexture3.readFromStream(input))
      {
      return(false);
      }
    }
  else
    {
    _refTexture3=RemoteRef();
    }
  if(flags&(1<<4))
    {
    if(!_refTexture4.readFromStream(input))
      {
      return(false);
      }
    }
  else
    {
    _refTexture4=RemoteRef();
    }
  unsigned int i;
  bool hasTexTransform=flags&(1<<5);
  bool hasTexTransform2=flags&(1<<6);
  bool hasTexTransform3=flags&(1<<7);
  bool hasTexTransform4=flags&(1<<8);
  if(hasTexTransform)
    {
    for(i=16;i--;)
      {
      if(!input->readDouble(_texTransform->accessTransformation().matrix[i]))
        {
        return(false);
        }
      }
    _texTransform->accessTransformation().isIdentity(true);
    }
  else
    {
    _texTransform->accessTransformation().identity();
    }
  if(hasTexTransform2)
    {
    for(i=16;i--;)
      {
      if(!input->readDouble(_texTransform2->accessTransformation().matrix[i]))
        {
        return(false);
        }
      }
    _texTransform2->accessTransformation().isIdentity(true);
    }
  else
    {
    _texTransform2->accessTransformation().identity();
    }
  if(hasTexTransform3)
    {
    for(i=16;i--;)
      {
      if(!input->readDouble(_texTransform3->accessTransformation().matrix[i]))
        {
        return(false);
        }
      }
    _texTransform3->accessTransformation().isIdentity(true);
    }
  else
    {
    _texTransform3->accessTransformation().identity();
    }
  if(hasTexTransform4)
    {
    for(i=16;i--;)
      {
      if(!input->readDouble(_texTransform4->accessTransformation().matrix[i]))
        {
        return(false);
        }
      }
    _texTransform4->accessTransformation().isIdentity(true);
    }
  else
    {
    _texTransform4->accessTransformation().identity();
    }

   return true;
}

} // namespace NetworkServices
//---End-Of-File---------------------------------------------------------------
