// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/Events/nsEventText3D.cpp
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReViNS/Events/nsEventText3D.h"
#include "AReViNS/remoteDepService.h"

#include "AReVi/Utils/abstractStream.h"
#include "AReVi/Shapes/text3D.h"

namespace AReVi {
//----------------------------------------------------------------------------
// CLASS       : NSEventText3DGeometryRebuild
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSEventText3DGeometryRebuild,NSEventShapePart3DGeometryRebuild)
//---------------------- Constructors / Destructor ---------------------------
NSEventText3DGeometryRebuild::NSEventText3DGeometryRebuild(ArCW & arCW) // Protected
  : NSEventShapePart3DGeometryRebuild(arCW)
{
}

NSEventText3DGeometryRebuild::~NSEventText3DGeometryRebuild(void)
{}

void
NSEventText3DGeometryRebuild::writeGeometry(ArConstRef<Text3D> text)
{
  text->getSpaces(_symbolSpace,_whiteSpace);
  _fontURL=text->getFont();
  _extrusion=text->getExtrusion();
  _text=text->getText();
}


bool
NSEventText3DGeometryRebuild::readGeometry(ArRef<Text3D> text,
                                           ArRef<RemoteDepService> depService) const
{
  StlString clientURL; // check url.
  if (!depService->requestRemoteURL(_fontURL,clientURL))
    return false;

  text->setSpaces(_symbolSpace,_whiteSpace);
  text->setFont(clientURL);
  text->setExtrusion(_extrusion);
  text->setText(_text);
  return true;
}

//-------------------------- Stream operations -------------------------------
void
NSEventText3DGeometryRebuild::writeToStream(ArRef<AbstractOStream> output) const
{
  NSEventShapePart3DGeometryRebuild::writeToStream(output);

  output->writeDouble(_symbolSpace);
  output->writeDouble(_whiteSpace);
  output->writeDouble(_extrusion);
  output->writeString(_fontURL);
  output->writeString(_text);
}

bool // success
NSEventText3DGeometryRebuild::readFromStream(ArRef<AbstractIStream> input)
{
  if (!NSEventShapePart3DGeometryRebuild::readFromStream(input))
    return false;

  if (!input->readDouble(_symbolSpace))
    return false;

  if (!input->readDouble(_whiteSpace))
    return false;

  if (!input->readDouble(_extrusion))
    return false;

  if (!input->readString(_fontURL))
    return false;

  if (!input->readString(_text))
    return false;
  return true;
}

} // namespace NetworkServices
//---End-Of-File---------------------------------------------------------------
