// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/Events/nsEventTextureRenderer3D.cpp
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReViNS/Events/nsEventTextureRenderer3D.h"
#include "AReVi/Utils/abstractStream.h"
#include "AReVi/Lib3D/textureRenderer3D.h"

namespace AReVi {

//----------------------------------------------------------------------------
// CLASS       : NSEventTextureRenderer3DRenderingSetup
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSEventTextureRenderer3DRenderingSetup,NSEventRenderer3DRenderingSetup)
//---------------------- Constructors / Destructor ---------------------------
NSEventTextureRenderer3DRenderingSetup::NSEventTextureRenderer3DRenderingSetup(ArCW & arCW) // Protected
  : NSEventRenderer3DRenderingSetup(arCW)
{}

NSEventTextureRenderer3DRenderingSetup::~NSEventTextureRenderer3DRenderingSetup(void)
{}
//---------------------------- Public methods --------------------------------
void
NSEventTextureRenderer3DRenderingSetup::writeTextureRenderer3D(ArConstRef<TextureRenderer3D> textureRenderer)
{
  textureRenderer->getRenderSize(_width,_height);
  _autoRender=textureRenderer->getAutoRender();
}

void
NSEventTextureRenderer3DRenderingSetup::readTextureRenderer3D(ArRef<TextureRenderer3D> textureRenderer) const
{
  textureRenderer->setRenderSize(_width,_height);
  textureRenderer->setAutoRender(_autoRender);
}

//-------------------------- Stream operations -------------------------------
bool // success
NSEventTextureRenderer3DRenderingSetup::readFromStream(ArRef<AbstractIStream> input)
{
  if (!NSEventRenderer3DRenderingSetup::readFromStream(input))
    return false;

  char autoRender;
  if (!input->readUInt(_width))
    return false;
  if (!input->readUInt(_height))
    return false;
  if (!input->readChar(autoRender))
    return false;
  _autoRender=autoRender;

  return true;
}

void
NSEventTextureRenderer3DRenderingSetup::writeToStream(ArRef<AbstractOStream> output) const
{
  NSEventRenderer3DRenderingSetup::writeToStream(output);

  output->writeUInt(_width);
  output->writeUInt(_height);
  output->writeChar((char)_autoRender);
}

//----------------------------------------------------------------------------
// CLASS       : NSEventTextureRenderer3DRender
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSEventTextureRenderer3DRender,NSEvent)
//---------------------- Constructors / Destructor ---------------------------
NSEventTextureRenderer3DRender::NSEventTextureRenderer3DRender(ArCW & arCW) // Protected
  : NSEvent(arCW)
{}

NSEventTextureRenderer3DRender::~NSEventTextureRenderer3DRender(void)
{}


} // namespace NetworkServices
//---End-Of-File---------------------------------------------------------------
