// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/Events/nsEventViewer3D.cpp
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReViNS/Events/nsEventViewer3D.h"
#include "AReVi/Utils/abstractStream.h"
#include "AReVi/Lib3D/viewer3D.h"

namespace AReVi {

//----------------------------------------------------------------------------
// CLASS       : NSEventViewer3DListener
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSEventViewer3DListener,NSEventArRef)
//---------------------- Constructors / Destructor ---------------------------
NSEventViewer3DListener::NSEventViewer3DListener(ArCW & arCW) // Protected
  : NSEventArRef(arCW)
{}

NSEventViewer3DListener::~NSEventViewer3DListener(void)
{}

//----------------------------------------------------------------------------
// CLASS       : NSEventViewer3DAppearance
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSEventViewer3DAppearance,NSEvent)
//---------------------- Constructors / Destructor ---------------------------
NSEventViewer3DAppearance::NSEventViewer3DAppearance(ArCW & arCW) // Protected
  : NSEvent(arCW), _title(""),_decoration(false)
{}

NSEventViewer3DAppearance::~NSEventViewer3DAppearance(void)
{}

//---------------------------- Public methods --------------------------------
// Read/Write Viewer setup
void
NSEventViewer3DAppearance::readViewerAppearance(ArRef<Viewer3D> viewer) const
{
  viewer->setTitle(_title);
  viewer->setDecoration(_decoration);
}

void
NSEventViewer3DAppearance::writeViewerAppearance(ArConstRef<Viewer3D> viewer)
{
  _title=viewer->getTitle();
  _decoration=viewer->getDecoration();
}


//-------------------------- Stream operations -------------------------------
bool // success
NSEventViewer3DAppearance::readFromStream(ArRef<AbstractIStream> input)
{
  if (!NSEvent::readFromStream(input))
    return false;

  if (!input->readString(_title))
    return false;
  int deco;
  if (!input->readInt(deco))
    return false;
  _decoration=deco;
  return true;
}

void
NSEventViewer3DAppearance::writeToStream(ArRef<AbstractOStream> output) const
{
  NSEvent::writeToStream(output);
  output->writeString(_title);
  output->writeInt((int)_decoration);
}
} // namespace NetworkServices
//---End-Of-File---------------------------------------------------------------

