// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/Events/nsEventWindow3D.cpp
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReViNS/Events/nsEventWindow3D.h"
#include "AReVi/Utils/abstractStream.h"

namespace AReVi {
//----------------------------------------------------------------------------
// CLASS       : NSEventWindow3DGeometry
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSEventWindow3DGeometry,NSEvent)
//---------------------- Constructors / Destructor ---------------------------
NSEventWindow3DGeometry::NSEventWindow3DGeometry(ArCW & arCW) // Protected
  : NSEvent(arCW), _x(0),_y(0),_w(0),_h(0)
{}

NSEventWindow3DGeometry::~NSEventWindow3DGeometry(void)
{}
//---------------------------- Public methods --------------------------------
//-------------------------- Stream operations -------------------------------
bool // success
NSEventWindow3DGeometry::readFromStream(ArRef<AbstractIStream> input)
{
  if (!NSEvent::readFromStream(input))
    return false;

  if (!input->readInt(_x))
    return false;
  if (!input->readInt(_y))
    return false;
  if (!input->readUInt(_w))
    return false;
  if (!input->readUInt(_h))
    return false;
  return true;
}

void
NSEventWindow3DGeometry::writeToStream(ArRef<AbstractOStream> output) const
{
  NSEvent::writeToStream(output);
  output->writeInt(_x);
  output->writeInt(_y);
  output->writeUInt(_w);
  output->writeUInt(_h);
}

//-------------------------- Set / Get / Access ------------------------------
//! Get Set Window Geometry
void
NSEventWindow3DGeometry::setWindowGeometry(int x,
                                           int y,
                                           unsigned int width,
                                           unsigned int height)
{
  _x=x;
  _h=height;
  _y=y;
  _w=width;
}


void
NSEventWindow3DGeometry::getWindowGeometry(int & xOut,
                                           int & yOut,
                                           unsigned int & widthOut,
                                           unsigned int & heightOut) const
{
  xOut=_x;
  yOut=_y;
  widthOut=_w;
  heightOut=_h;
}

//----------------------------------------------------------------------------
// CLASS       : NSEventWindow3DMouseMotion
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSEventWindow3DMouseMotion,NSEvent)
//---------------------- Constructors / Destructor ---------------------------
NSEventWindow3DMouseMotion::NSEventWindow3DMouseMotion(ArCW & arCW) // Protected
  : NSEvent(arCW), _x(0),_y(0),_modifiers(0)
{}

NSEventWindow3DMouseMotion::~NSEventWindow3DMouseMotion(void)
{}
//---------------------------- Public methods --------------------------------
//-------------------------- Stream operations -------------------------------
bool // success
NSEventWindow3DMouseMotion::readFromStream(ArRef<AbstractIStream> input)
{
  if (!NSEvent::readFromStream(input))
    return false;

  if (!input->readInt(_x))
    return false;
  if (!input->readInt(_y))
    return false;
  if (!input->readUInt(_modifiers))
    return false;
  return true;
}

void
NSEventWindow3DMouseMotion::writeToStream(ArRef<AbstractOStream> output) const
{
  NSEvent::writeToStream(output);
  output->writeInt(_x);
  output->writeInt(_y);
  output->writeUInt(_modifiers);
}

//-------------------------- Set / Get / Access ------------------------------
// Get Set Scenes
void
NSEventWindow3DMouseMotion::setPosition(int x,int y)
{
  _x=x;
  _y=y;
}

void
NSEventWindow3DMouseMotion::setModifiers(unsigned modifiers)
{
  _modifiers=modifiers;
}


//----------------------------------------------------------------------------
// CLASS       : NSEventWindow3DMouseButton
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSEventWindow3DMouseButton,NSEventWindow3DMouseMotion)
//---------------------- Constructors / Destructor ---------------------------
NSEventWindow3DMouseButton::NSEventWindow3DMouseButton(ArCW & arCW) // Protected
  : NSEventWindow3DMouseMotion(arCW), _button(0),_pressed(false)
{}

NSEventWindow3DMouseButton::~NSEventWindow3DMouseButton(void)
{}
//---------------------------- Public methods --------------------------------
//-------------------------- Stream operations -------------------------------
bool // success
NSEventWindow3DMouseButton::readFromStream(ArRef<AbstractIStream> input)
{
  if (!NSEventWindow3DMouseMotion::readFromStream(input))
    return false;

  if (!input->readInt(_button))
    return false;
  char pressed;
  if (!input->readChar(pressed))
    return false;
  _pressed=pressed;
  return true;
}

void
NSEventWindow3DMouseButton::writeToStream(ArRef<AbstractOStream> output) const
{
  NSEventWindow3DMouseMotion::writeToStream(output);
  output->writeInt(_button);
  output->writeChar((char)_pressed);
}

//-------------------------- Set / Get / Access ------------------------------
// Get Set Scenes
void
NSEventWindow3DMouseButton::setButton(int btn,bool pressed)
{
  _button=btn;
  _pressed=pressed;
}


//----------------------------------------------------------------------------
// CLASS       : NSEventWindow3DKeyboard
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSEventWindow3DKeyboard,NSEventWindow3DMouseMotion)
//---------------------- Constructors / Destructor ---------------------------
NSEventWindow3DKeyboard::NSEventWindow3DKeyboard(ArCW & arCW) // Protected
  : NSEventWindow3DMouseMotion(arCW), _key(""),_pressed(false)
{}

NSEventWindow3DKeyboard::~NSEventWindow3DKeyboard(void)
{}
//---------------------------- Public methods --------------------------------
//-------------------------- Stream operations -------------------------------
bool // success
NSEventWindow3DKeyboard::readFromStream(ArRef<AbstractIStream> input)
{
  if (!NSEventWindow3DMouseMotion::readFromStream(input))
    return false;

  if (!input->readString(_key))
    return false;
  int pressed;
  if (!input->readInt(pressed))
    return false;
  _pressed=pressed;
  return true;
}

void
NSEventWindow3DKeyboard::writeToStream(ArRef<AbstractOStream> output) const
{
  NSEventWindow3DMouseMotion::writeToStream(output);
  output->writeString(_key);
  output->writeInt((int)_pressed);
}

//-------------------------- Set / Get / Access ------------------------------
// Get Set Scenes
void
NSEventWindow3DKeyboard::setKey(const StlString &key,bool pressed)
{
  _key=key;
  _pressed=pressed;
}

//----------------------------------------------------------------------------
// CLASS       : NSEventWindow3DClose
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSEventWindow3DClose,NSEvent)
//---------------------- Constructors / Destructor ---------------------------
NSEventWindow3DClose::NSEventWindow3DClose(ArCW & arCW) // Protected
  : NSEvent(arCW)
{}

NSEventWindow3DClose::~NSEventWindow3DClose(void)
{}


//----------------------------------------------------------------------------
// CLASS       : NSEventWindow3DCursor
//----------------------------------------------------------------------------
AR_CLASS_DEF(NSEventWindow3DCursor,NSEvent)

//---------------------- Constructors / Destructor ---------------------------
NSEventWindow3DCursor::NSEventWindow3DCursor(ArCW & arCW) // Protected
  : NSEvent(arCW), _cursorID(0)
{}

NSEventWindow3DCursor::~NSEventWindow3DCursor(void)
{}
//---------------------------- Public methods --------------------------------
//-------------------------- Stream operations -------------------------------
bool // success
NSEventWindow3DCursor::readFromStream(ArRef<AbstractIStream> input)
{
  if (!NSEvent::readFromStream(input))
    return false;
  return input->readUInt(_cursorID);
}

void
NSEventWindow3DCursor::writeToStream(ArRef<AbstractOStream> output) const
{
  NSEvent::writeToStream(output);
  output->writeUInt(_cursorID);
}

//-------------------------- Set / Get / Access ------------------------------
void
NSEventWindow3DCursor::setCursor(unsigned cursorID)
{
  _cursorID=cursorID;
}

//---------------------------- Public methods --------------------------------

} // namespace NetworkServices
//---End-Of-File---------------------------------------------------------------

