// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/remoteDepService.cpp
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReViNS/remoteDepService.h"
#include "AReViNS/Events/nsEventObjectRef.h"
#include "AReViNS/Events/nsEventFile.h"
#include "AReViNS/Events/nsEventControler.h"
#include "AReViNS/Controlers/nsControler.h"

#include "AReVi/arSystem.h"
#include "AReVi/Utils/memoryBlock.h"
#include "AReVi/Utils/memoryStream.h"
#include "AReVi/Utils/fileStream.h"
#include "AReVi/Shapes/shape3D.h"
#include "AReVi/Shapes/shapePart3D.h"

namespace AReVi {

//----------------------------------------------------------------------------
// CLASS       : RemoteDepService
//----------------------------------------------------------------------------

AR_CLASS_NOVOID_DEF(RemoteDepService,RemoteService)

//---------------------- Constructors / Destructor ---------------------------

RemoteDepService::RemoteDepService(ArCW & arCW,
                             unsigned int distantIPAddress,
                             int distantPort) // Protected
  : RemoteService(arCW),
    _remoteRefs(),
    _waitRefs(),
    _path("./RemoteFile"),
    _remoteUrls(),
    _waitUrls(),
    _interactorsMap(),
    _interactors(),
    _processingControlerPacks(false),
    _remoteRefReceptionCB(thisRef()),
    _remoteFileReceptionCB(thisRef())
{
  _initRemoteDepService(distantIPAddress,distantPort);
}

RemoteDepService::~RemoteDepService(void)
{
// remove all Temporary Files
  StlMap<StlString,StlString >::const_iterator it=_remoteUrls.begin();
  while (it!=_remoteUrls.end())
  {
    ArSystem::removeFile(it->second);
    it++;
  }
}
//---------------------------- Public methods --------------------------------
const StlMap<StlString, ArRef<ArObject> > &
RemoteDepService::getRemoteRefs(void) const
{
  return _remoteRefs;
}

bool
RemoteDepService::findRemoteURL(const StlString & serverUrl,
                                StlString &remoteUrlOut) const
{
  StlMap<StlString,StlString >::const_iterator it=_remoteUrls.find(serverUrl);
  if (it==_remoteUrls.end())
  {
    remoteUrlOut="";
    return false;
  }
  else
  {
    remoteUrlOut=it->second;
    return true;
  }
}

// Same that findRemoteURL exept that add request to server if url not
// already exist (and return false).
bool
RemoteDepService::requestRemoteURL(const StlString & serverUrl,
                                    StlString &remoteUrlOut)
{
  if (findRemoteURL(serverUrl,remoteUrlOut))
    return true;

  addRemoteURL(serverUrl);
  return false;
}

const StlMap<StlString, StlString> &
RemoteDepService::getRemoteURLs(void) const
{
  return _remoteUrls;
}

//----------------------- Remote objects manager -----------------------------
// Request serialisation of specified object.
bool // request success
RemoteDepService::addRemoteReference(const StlString &serverRef, const StlString &className) //virtual
{
  if (!_addRemoteReference(serverRef,className))
    return false;

  _waitRefs.push_back(serverRef);

  return updateRemoteReference(serverRef);
}

// Request serialisation of specified object.
bool // request success
RemoteDepService::updateRemoteReference(const StlString &serverRef) //virtual
{
  // Send serialisation request to server.
  ArRef<NSEventObjectRefRequest> event=NSEventObjectRefRequest::NEW();
  event->setID(serverRef);
 // cerr << "SEND REQEUST FOR " << remoteRef << "=";
  sendEvent(event);
  //cerr << endl;
  return true;
}

// Request suppression of element
bool // success
RemoteDepService::removeRemoteReference(const StlString &serverRef)
{
  //cerr << "Remove remote ref=>" << serverRef << endl;
  StlMap<StlString ,ArRef<ArObject> >::iterator it=_remoteRefs.find(serverRef);
  if (it==_remoteRefs.end())
    return false;
  else
  {
    if (it->second.valid())
      it->second.destroy(); // UGLY !
    _remoteRefs.erase(it);
  }

  StlMap<StlString ,ArRef<NSInteractor> >::iterator itCtrl=_interactors.find(serverRef);
  if (itCtrl==_interactors.end())
    return false;
  else
  {
    if (itCtrl->second.valid())
      itCtrl->second.destroy(); // UGLY !
    _interactors.erase(itCtrl);
  }

  return true;
}

// Comment
bool
RemoteDepService::linkRemoteObjectToServer(const StlString & serverAlias,
                                           ArRef<ArObject> remoteObject)
{
StlMap<StlString ,ArRef<ArObject> >::iterator itRef=_remoteRefs.find(serverAlias);
if (itRef!=_remoteRefs.end())
  return false;

// Add Ref in remoteRefs:
_remoteRefs[serverAlias]=remoteObject;

// request listener instanciation on server side
ArRef<NSEventObjectListenerRequest> event=NSEventObjectListenerRequest::NEW();
event->setID(serverAlias);
sendEvent(event);

// Add Object controler if any
StlString refClass(remoteObject->getClass()->getName());

// Look if specified
StlMap<StlString, StlString>::const_iterator it=_interactorsMap.find(refClass);
if (it!=_interactorsMap.end())
  {
  ArConstRef<ArClass> ctrlClass=ArClass::find(it->second);
  ArRef<NSInteractor> ctrlObject;

  if(ctrlClass)
    {
    ctrlClass->newInstance(ctrlObject);

    if(!ctrlObject)
      {
      setErrorMessage("Can't create controler of type =>'" + it->second + "' => FAIL");
      _onError(NS_ERROR_INTERACTOR_FAIL);
      return false;
      }
    else
      {
      ctrlObject->init(thisRef(), remoteObject);
      _interactors[event->getID()]=ctrlObject;
      }
    }
    else
      {
      setErrorMessage("Can't find object controler of type =>'" + it->second + "' for object class '" + it->first + "' => FAIL");
      _onError(NS_ERROR_INTERACTOR_FAIL);
      return false;
      }
  }
else // else try auto
  {
  StlString itClassName="NSInteractor"+refClass;
  ArConstRef<ArClass> ctrlClass=ArClass::find(itClassName);
  ArRef<NSInteractor> ctrlObject;

  if(ctrlClass)
    {
    ctrlClass->newInstance(ctrlObject);

    if(ctrlObject)
      {
      ctrlObject->init(thisRef(), remoteObject);
      _interactors[event->getID()]=ctrlObject;
      }
    }// else => no interactor.
  }
return true;
}

// Set remote temps url prefix
// eg : /tmp/remoteFile => /tmp/remoteFile02d0e
//                      => /tmp/remoteFilefeggr ...
void
RemoteDepService::setRemoteURLsPrefix(const StlString &prefix)
{
  if (!prefix.empty())
    _path=prefix;
}

// Request serialisation of specified object.
bool // success
RemoteDepService::addRemoteURL(const StlString &url) //virtual
{
  if (find(_waitUrls.begin(),_waitUrls.end(),url)==_waitUrls.end())
  {
    _waitUrls.push_back(url);
    ArRef<NSEventFileRequest> event=NSEventFileRequest::NEW();
    event->setURL(url);
    sendEvent(event);
  }

  return true;
}

bool // true if all is resolved
RemoteDepService::resolveDependencies(const StlVector<StlString> &refs,
                                      const StlVector<StlString> &urls,
                                      SerializationDependencies &depsOut,
                                      StlVector<unsigned int> &missingRefsOut,
                                      StlVector<unsigned int> &missingUrlsOut)
{
  bool allResolved(true);
  ArRef<ArObject> object;
  StlString localURL;

  for (unsigned int cpt=0;cpt<refs.size();cpt++)
  {
    findRemoteObject<ArObject>(refs[cpt],object);
    if (object)
      depsOut.addReference(object);
    else
    {
      missingRefsOut.push_back(cpt);
      allResolved=false;
    }
  }
  for (unsigned int cpt=0;cpt<urls.size();cpt++)
  {

    if (findRemoteURL(urls[cpt],localURL))
      depsOut.addURL(localURL);
    else
    {
      missingUrlsOut.push_back(cpt);
      allResolved=false;
    }
  }
  return allResolved;
}

//------------------------  Controlers management ----------------------------
// Register controlers
bool
RemoteDepService::registerInteractor(const StlString &objectClass,
                                     const StlString &ctrlClass,
                                     bool subclass)
{
  ArConstRef<ArClass> object(ArClass::find(objectClass)),
                      ctrl(ArClass::find(ctrlClass));

  if (!object.valid() || !ctrl.valid()) // object and controler must exist!!
  {
    if (!object.valid())
      setErrorMessage("objectClass=" + objectClass + " must exist!");
    if (!ctrl.valid())
      setErrorMessage("ctrlClass=" + ctrlClass + " must exist!");
    return false;
  }

  _interactorsMap[objectClass]=ctrlClass;

  if (subclass) // All subclass of 'objectClass' are also registered
                // as using the same interactor
  {
    StlVector<ArConstRef<ArClass> > descendantsOut;
    object->getDescendants(descendantsOut);
    for (unsigned int i=descendantsOut.size();i--;)
      _interactorsMap[descendantsOut[i]->getName()]=ctrlClass;
  }

  return true;
}

//------------------------- Forbiden operations ------------------------------
void
RemoteDepService::_initRemoteDepService(unsigned int  distantIPAddress,
                                        int            port)
{
  _setConnectionMessage("SharedObjects");

  if (!connect(distantIPAddress,port))
    return;

  if(fail())
    return;

// register non automatics interactors (default are NSInteractor+ClassName).
  registerInteractor("LineSet3D","NSInteractorShapePart3D",true);
  registerInteractor("PointSet3D","NSInteractorShapePart3D",true);
  registerInteractor("URLSound3D","NSInteractorSound3D",true);

  //registerInteractor("TextureRenderer3D","NSInteractorRenderer3D");

  registerEvent(NSEventObjectRef::CLASS()->getName(),
                thisRef(),
                (NetworkService::EventMethod)&RemoteDepService::_onSetRemoteObject);

  registerEvent(NSEventDownCastMap::CLASS()->getName(),
                thisRef(),
                (NetworkService::EventMethod)&RemoteDepService::_onSetDownCastMap);

  registerEvent(NSEventFile::CLASS()->getName(),
                thisRef(),
                (NetworkService::EventMethod)&RemoteDepService::_onSetRemoteURL);

  registerEvent(NSEventControler::CLASS()->getName(),
                thisRef(),
                (NetworkService::EventMethod)&RemoteDepService::_onControlerEvent);

  registerEvent(NSEventControlerPack::CLASS()->getName(),
                thisRef(),
                (NetworkService::EventMethod)&RemoteDepService::_onControlerEventPack);
}


bool
RemoteDepService::_addRemoteReference(const StlString &remoteRef,const StlString &className)
{
  if (_remoteRefs.find(remoteRef)!=_remoteRefs.end())
    return false;

  StlString refClass;

  // Look if class need to be downcasted first.
  if (_downCastMap.find(className)!=_downCastMap.end())
  {
    refClass=_downCastMap[className];
  }
  else
    refClass=className;


  ArConstRef<ArClass> remoteClass=ArClass::find(refClass);
  ArRef<ArObject> remoteObject;

  if(remoteClass) // check class existance
    remoteClass->newInstance(remoteObject);
  else
  {
    setErrorMessage("Can't find class of type =>'" + refClass + "' => FAIL");
    _onError(NS_ERROR_REMOTEINSTANCIATION_FAIL);
    return false;
  }

  if(!remoteObject)
  {
    setErrorMessage("Can't create object of type =>'" + refClass + "' => FAIL");
    _onError(NS_ERROR_REMOTEINSTANCIATION_FAIL);
    return false;
  }
  _remoteRefs[remoteRef]=remoteObject;
  return true;
}

// CB : inform that a object is sent by server
NSEvent::EventProcess
RemoteDepService::_onSetRemoteObject(ArConstRef<NSEvent> eventObj)
{
  ArConstRef<NSEventObjectRef> event=ar_down_cast<NSEventObjectRef>(eventObj);

  if (event->getClassName()=="") // Object Removed on server
  {
    removeRemoteReference(event->getID());
  }
  else
  {
    // Create reference
    _addRemoteReference(event->getID(),event->getClassName());

    ArRef<ArObject> object;
    findRemoteObject<ArObject>(event->getID(),object);

    if (!object.valid()) // check success
    {
      // Remove object from waiting list.
      StlList<StlString>::iterator listIt=find(_waitRefs.begin(),_waitRefs.end(),event->getID());
      if (listIt!=_waitRefs.end())
        _waitRefs.erase(listIt);
      return NSEvent::EP_FAIL;
    }

  // Resolve dependencies
    SerializationDependencies deps;

    StlVector<unsigned int> missingRefs;
    StlVector<unsigned int> missingUrls;

    if (!resolveDependencies(event->getRefs(),event->getURLs(),deps,missingRefs,missingUrls))
    {
      if (missingUrls.size()) // URL are missing post message
      {
        // Look if their are requested
        for (unsigned int i=missingUrls.size();i--;)
        {
          if (find(_waitUrls.begin(),_waitUrls.end(),event->getURLs()[missingUrls[i]])==_waitUrls.end())
            addRemoteURL(event->getURLs()[missingUrls[i]]);
        }

        return NSEvent::EP_POST;
      }

      // Ask all missing refs
      for (unsigned int i=missingRefs.size();i--;)
      {
        if (!addRemoteReference(event->getRefs()[missingRefs[i]],event->getClassRefs()[missingRefs[i]]))
        {
          setErrorMessage("[RemoteDepService] ERROR => Can't resolve reference for '" + event->getID() +"'");
          _onError(NS_ERROR_RESOLVEDEPENDENCIES_FAIL);

          // Remove object from waiting list.
          StlList<StlString>::iterator listIt=find(_waitRefs.begin(),_waitRefs.end(),event->getID());
          if (listIt!=_waitRefs.end())
            _waitRefs.erase(listIt);

          return NSEvent::EP_FAIL;
        }
      }

      deps.clear();
      resolveDependencies(event->getRefs(),event->getURLs(),deps,missingRefs,missingUrls);
    }

  // Unserialize data => fill object informations
    ArRef<MemoryIStream> stream=MemoryIStream::NEW(event->getData(),0);
    if (!object->unserialize(deps,stream))
    {
      if (object->fail())
        setErrorMessage("[RemoteDepService] WARNING : Error during deserialization of '" + object->getAlias() +"'\n" +
                        "                             => " + object->getErrorMessage());
      else
        setErrorMessage("[RemoteDepService] WARNING : Error during deserialization of '" + object->getAlias() +"'");
      _onError(NS_ERROR_UNSERIALIZE_OBJECT);
    }


 // Add Object controler if any
    ArRef<ArObject> remoteObject(object);
    StlString refClass(object->getClass()->getName());

  // Look if specified
    StlMap<StlString, StlString>::const_iterator it=_interactorsMap.find(refClass);
    if (it!=_interactorsMap.end())
    {
      ArConstRef<ArClass> ctrlClass=ArClass::find(it->second);
      ArRef<NSInteractor> ctrlObject;

      if(ctrlClass)
      {
        ctrlClass->newInstance(ctrlObject);

        if(!ctrlObject)
        {
          setErrorMessage("Can't create controler of type =>'" + it->second + "' => FAIL");
          _onError(NS_ERROR_INTERACTOR_FAIL);
        }
        else
        {
          ctrlObject->init(thisRef(), remoteObject);
          _interactors[event->getID()]=ctrlObject;
        }
      }
      else
      {
        setErrorMessage("Can't find object controler of type =>'" + it->second + "' for object class '" + it->first + "' => FAIL");
        _onError(NS_ERROR_INTERACTOR_FAIL);
      }
    }
    else // else try auto
    {
      StlString itClassName="NSInteractor"+refClass;
      ArConstRef<ArClass> ctrlClass=ArClass::find(itClassName);
      ArRef<NSInteractor> ctrlObject;

      if(ctrlClass)
      {
        ctrlClass->newInstance(ctrlObject);

        if(ctrlObject)
        {
          ctrlObject->init(thisRef(), remoteObject);
          _interactors[event->getID()]=ctrlObject;
        }
      }// else => no interactor.
    }
  }

  // Remove object from waiting list.
  StlList<StlString>::iterator listIt=find(_waitRefs.begin(),_waitRefs.end(),event->getID());
  if (listIt!=_waitRefs.end())
    _waitRefs.erase(listIt);

  if(!_remoteRefReceptionCB.empty())
  {
    RemoteDepService::RemoteRefReceptionEvent eventCB;
    eventCB.serverAlias=event->getID();
    _remoteRefReceptionCB.fire(eventCB);
  }
  return NSEvent::EP_OK;
}

// CB : set downcast map
NSEvent::EventProcess
RemoteDepService::_onSetDownCastMap(ArConstRef<NSEvent> eventObj)
{
  ArConstRef<NSEventDownCastMap> event=ar_down_cast<NSEventDownCastMap>(eventObj);
  _downCastMap=event->getMap();
  return NSEvent::EP_OK;
}

// CB : inform that an url is sent by server
NSEvent::EventProcess
RemoteDepService::_onSetRemoteURL(ArConstRef<NSEvent> eventObj)
{
  //cerr << "RemoteDepService::_onSetRemoteURL" << endl;
  ArConstRef<NSEventFile> event=ar_down_cast<NSEventFile>(eventObj);

  StlList<StlString>::iterator listIt=find(_waitUrls.begin(),_waitUrls.end(),event->getURL());

  if (listIt!=_waitUrls.end())
    _waitUrls.erase(listIt);

  StlString fileName;
  if (!ArSystem::createTmpFile(_path,fileName))
  {
    setErrorMessage("Can't create tempory name based on '" + _path + "'");
    _onError(NS_ERROR_CREATETEMPFILE);
    return NSEvent::EP_FAIL;
  }

  ArRef<FileOStream> file=FileOStream::NEW(fileName,false);
  if(file->fail())
  {
    cerr << file->getErrorMessage() << endl;
    return NSEvent::EP_OK;
  }

  ArRef<MemoryIStream> input=MemoryIStream::NEW(event->getData(),0);
  file->copyDataFromInput(input);
  if(file->fail())
  {
    setErrorMessage("WARNING : Error during writing '"+fileName+"'\n"
                    "          =>" + file->getErrorMessage());
    _onError(NS_ERROR_WRITINGTEMPFILE);
  }

  //cerr << "RECEPTION OF '" << event->getURL() << "' => " << fileName << "(" << (event->getData()->getSize()/1024.0) << "ko)...done!" <<  endl;

  _remoteUrls[event->getURL()]=fileName;

  if(!_remoteFileReceptionCB.empty())
  {
    RemoteDepService::RemoteFileReceptionEvent eventCB;
    eventCB.serverUrl=event->getURL();
    _remoteFileReceptionCB.fire(eventCB);
  }

  return NSEvent::EP_OK;
}

// CB : inform of message from a server controler
NSEvent::EventProcess
RemoteDepService::_onControlerEvent(ArConstRef<NSEvent> eventObj)
{
  ArConstRef<NSEventControler> event=ar_down_cast<NSEventControler>(eventObj);

  StlMap<StlString, ArRef<NSInteractor> >::iterator it=_interactors.find(event->getID());
  if (it!=_interactors.end())
  {
    NSEvent::EventProcess ret(it->second->fireEvent(event->getEvent()));
    if (ret==NSEvent::EP_FAIL)
    {
      setErrorMessage("Error on event with controler : \n   =>" + it->second->getErrorMessage());
      _onError(NS_ERROR_EVENTCONTROLER);
    }
    return ret;
  }
  return NSEvent::EP_OK;
}

// CB : inform of message from a server controler
NSEvent::EventProcess
RemoteDepService::_onControlerEventPack(ArConstRef<NSEvent> eventObj)
{
  ArConstRef<NSEventControlerPack> event=ar_down_cast<NSEventControlerPack>(eventObj);
  _processingControlerPacks=true;
//   _eventsReceived.insert(
  std::copy(event->getEvents().begin(), event->getEvents().end(), std::front_inserter(_eventsReceived));
  return NSEvent::EP_OK;
}


bool
RemoteDepService::_updateNetwork(ArRef<Activity> , double )
{
return _networkAct.valid();
}

void
RemoteDepService::_onPolling(const Scheduler::PollingEvent &evt)
{
static ArRef<NSEventRemoteDepReady> event(NSEventRemoteDepReady::NEW());

NetworkService::_onPolling(evt);

// cerr << _eventsReceived.empty() << "    Posted=" << _eventsPosted.empty() << endl;
if (_processingControlerPacks && _eventsReceived.empty() && _eventsPosted.empty()) //Inform that it's ready
  {
//   cerr << "READY SEND OK" << endl;
  _processingControlerPacks=false;
  sendEvent(event);
  }
}


//----------------------------------------------------------------------------
// CLASS       : RemoteRef
//----------------------------------------------------------------------------
RemoteRef::RemoteRef(ArConstRef<ArObject> ref)  : _alias(""),_className("")
{
  if (ref.valid())
  {
    _alias=ref->getAlias();
    _className=ref->getClass()->getName();
  }
}

//-------------------------- Stream operations -------------------------------
void
RemoteRef::writeToStream(ArRef<AbstractOStream> output) const
{
  output->writeString(_alias);
  output->writeString(_className);
}

bool // success
RemoteRef::readFromStream(ArRef<AbstractIStream> input)
{
  if (!input->readString(_alias))
    return false;
  if (!input->readString(_className))
    return false;
  return true;
}

} // namespace NetworkServices
//---End-Of-File---------------------------------------------------------------
