// LI2/ENIB - AReVi - Network services
//----------------------------------------------------------------------------
// file        : AReViNS/remoteViewer3D.cpp
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

#include "AReViNS/remoteViewer3D.h"
#include "AReViNS/Controlers/nsCtrlViewer3D.h"
#include "AReViNS/Events/nsEventViewer3D.h"
#include "AReViNS/Events/nsEventWindow3D.h"
#include "AReViNS/Events/nsEventBase3D.h"
#include "AReViNS/Events/nsEventControler.h"

#include "AReVi/activity.h"
#include "AReVi/Lib3D/scene3D.h"
#include "AReVi/Lib3D/sound3D.h"

namespace AReVi {
//----------------------------------------------------------------------------
// CLASS       : RemoteViewer3D
//----------------------------------------------------------------------------
AR_CLASS_NOVOID_DEF(RemoteViewer3D,RemoteService)

//---------------------- Constructors / Destructor ---------------------------
RemoteViewer3D::RemoteViewer3D(ArCW & arCW,
                               ArRef<RemoteDepService> depService) // Protected
  : RemoteService(arCW),
    _viewer(Viewer3D::NEW()),
    _viewerInteractor(),
    _depService(depService),
    _geometryFlag(false),
    _closeFlag(false)
{
_viewer->addDestroyCB(thisRef(),&RemoteViewer3D::_onDestroyCB);
}

RemoteViewer3D::RemoteViewer3D(ArCW & arCW,
                               ArRef<RemoteDepService> depService,
                               ArRef<Viewer3D> viewer) // Protected
  : RemoteService(arCW),
    _viewer(viewer),
    _viewerInteractor(),
    _depService(depService),
    _geometryFlag(false),
    _closeFlag(false)
{
if (_viewer.valid())
  _viewer->addDestroyCB(thisRef(),&RemoteViewer3D::_onDestroyCB);
}


RemoteViewer3D::~RemoteViewer3D(void)
{
}

// Connection method
bool
RemoteViewer3D::connect(unsigned int distantIPAddress,
                        int           distantPort)
{
  _setConnectionMessage("SharedViewer3D");

  if (!RemoteService::connect(distantIPAddress,distantPort))
    return false;

  if (fail() && !_viewer.valid())
    return false;

  registerEvent(NSEventControler::CLASS()->getName(),
                thisRef(),
                (NetworkService::EventMethod)&RemoteViewer3D::_onViewerInteractorEvent);
  registerEvent(NSEventWindow3DClose::CLASS()->getName(),
                thisRef(),
                (NetworkService::EventMethod)&RemoteViewer3D::_onCloseEvent);
  registerEvent(NSEventViewer3DListener::CLASS()->getName(),
                thisRef(),
                (NetworkService::EventMethod)&RemoteViewer3D::_onListenerEvent);

  _viewer->addGeometryCB(thisRef(),&RemoteViewer3D::_onGeometryCB);
  _viewer->addMouseMotionCB(thisRef(),&RemoteViewer3D::_onMouseMotionCB);
  _viewer->addMouseButtonCB(thisRef(),&RemoteViewer3D::_onMouseButtonCB);
  _viewer->addKeyboardCB(thisRef(),&RemoteViewer3D::_onKeyboardCB);
  _viewer->addCloseCB(thisRef(),&RemoteViewer3D::_onCloseCB);

  _viewerInteractor=NSInteractorViewer3D::NEW();
  _viewerInteractor->init(_depService,_viewer);

  unsigned int w,h;
  Viewer3D::getScreenSize(w,h);
  _viewer->setWindowGeometry(w-400,0,400,400);
  //_viewer->setMapped(true);

  //setUpdateInterval(0.001);
  return true;
}

//---------------------------- Public methods --------------------------------
ArRef<Viewer3D>
RemoteViewer3D::accessViewer(void)
{
  return _viewer;
}

ArRef<RemoteDepService>
RemoteViewer3D::accessDepService(void)
{
  if (_depService.valid())
    return _depService;
  else
    return RemoteDepService::nullRef();
}


//------------------------- Forbiden operations ------------------------------
NSEvent::EventProcess
RemoteViewer3D::_onViewerInteractorEvent(ArConstRef<NSEvent> eventObj)
{
  ArConstRef<NSEventControler> event=ar_down_cast<NSEventControler>(eventObj);
  return _viewerInteractor->fireEvent(event->getEvent());
}

NSEvent::EventProcess
RemoteViewer3D::_onCloseEvent(ArConstRef<NSEvent>)
{
  if (_viewer.valid())
    _viewer.destroy(); // UGLY !

  return NSEvent::EP_OK;
}

NSEvent::EventProcess
RemoteViewer3D::_onListenerEvent(ArConstRef<NSEvent> eventObj)
{
  ArConstRef<NSEventViewer3DListener> event=ar_down_cast<NSEventViewer3DListener>(eventObj);

  ArRef<Listener3D> listener;
  _depService->findRemoteObject(event->getID(),listener);
  if (!listener && event->getID()!="")
  {
    if (!_depService->addRemoteReference(event->getID(),event->getClassName()))
      return NSEvent::EP_FAIL;
    else
     _depService->findRemoteObject(event->getID(),listener);
  }
  Listener3D::setCurrent(listener);

  return NSEvent::EP_OK;
}

bool //virtual
RemoteViewer3D::_updateNetwork(ArRef<Activity>, double )
{
  //if(!_getNetworkEvents())
  if(!_networkAct.valid())
    return false;
/*
  if (!processEvents())
    return false;
*/
// Send client events.
  if (!_viewer.valid())
    return false;

  if (!_events.empty())
    {
    StlList<ArRef<NSEvent> >::iterator it=_events.begin();
    StlList<ArRef<NSEvent> >::iterator it_end=_events.end();

    while(it!=it_end)
      {
      sendEvent(*it);
      it++;
      }
    _events.clear();
    }

  if (_geometryFlag)
  {
    unsigned int w,h;
    int x,y;
    _viewer->getWindowGeometry(x,y,w,h);
    ArRef<NSEventWindow3DGeometry> event=NSEventWindow3DGeometry::NEW();
    event->setWindowGeometry(x,y,w,h);
    _geometryFlag=false;

    sendEvent(event);
  }
  if (_closeFlag)
  {
    ArRef<NSEventWindow3DClose> event=NSEventWindow3DClose::NEW();
    _closeFlag=false;
    sendEvent(event);
  }

  return _networkAct.valid();
}

// Geometry (Resize/Move widget?)
void
RemoteViewer3D::_onGeometryCB(const Viewer3D::GeometryEvent &)
{
  _geometryFlag=true;
}

// On Keyboards events
void
RemoteViewer3D::_onKeyboardCB(const Viewer3D::KeyboardEvent &event)
{
  ArRef<NSEventWindow3DKeyboard> nsEvent=NSEventWindow3DKeyboard::NEW();
  nsEvent->setPosition(event.xMouse,event.yMouse);
  nsEvent->setKey(event.key,event.pressed);
  nsEvent->setModifiers(_viewer->getModifiers());
  _events.push_back(nsEvent);
}

// On MOUSE events
void
RemoteViewer3D::_onMouseMotionCB(const Viewer3D::MouseMotionEvent &event)
{
  ArRef<NSEventWindow3DMouseMotion> nsEvent=NSEventWindow3DMouseMotion::NEW();
  nsEvent->setPosition(event.xMouse,event.yMouse);
  nsEvent->setModifiers(_viewer->getModifiers());
  _events.push_back(nsEvent);
}

// On MOUSE Buttons events
void
RemoteViewer3D::_onMouseButtonCB(const Viewer3D::MouseButtonEvent &event)
{
  ArRef<NSEventWindow3DMouseButton> nsEvent=NSEventWindow3DMouseButton::NEW();
  nsEvent->setPosition(event.xMouse,event.yMouse);
  nsEvent->setButton(event.button,event.pressed);
  nsEvent->setModifiers(_viewer->getModifiers());
  _events.push_back(nsEvent);
}

// VIEWER CLOSE
void
RemoteViewer3D::_onCloseCB(const Viewer3D::CloseEvent &)
{
  _closeFlag=true;
  _networkAct.clear();
}

// VIEWER DESTROY
void
RemoteViewer3D::_onDestroyCB(const ArObject::DestroyEvent &)
{
  _networkAct.clear();
  close();
}

} // namespace NetworkServices
//---End-Of-File---------------------------------------------------------------
