// LI2/ENIB - AReVi - Image loader plugin
//----------------------------------------------------------------------------
// file        : Ext/Imlib2ImageLoader/imlib2ImageLoader.cpp
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
//
//  This plugin uses the Imlib2 API (http://www.enlightenment.org).
//
//  The resulting dynamic library can be loaded at run-time
//  (fails if a dependency problem occurs).
//
//  These functions uses a `width x height x components' byte array
//  with rows ordered from bottom to top.
//
//----------------------------------------------------------------------------

#include "AReVi/Lib3D/llw_priv.h"

#include <string.h>

#define X_DISPLAY_MISSING
#include <Imlib2.h>

namespace AReVi {

//----------------------------------------------------------------------------

static
bool // success (dataOut filled from bottom to top)
Imlib2_imageLoader(const char * fileName,
                   unsigned int & widthOut,
                   unsigned int & heightOut,
                   unsigned int & componentsOut,
                   ArRef<MemoryBlock> dataOut)
{
bool result=false;
Imlib_Image image=imlib_load_image_immediately(fileName);
if(image)
  {
  imlib_context_set_image(image);
  widthOut=imlib_image_get_width();
  heightOut=imlib_image_get_height();
  unsigned int nbPixels=widthOut*heightOut;
  if(imlib_image_has_alpha())
    {
    bool rgb=false;
    componentsOut=4; // RED - GREEN - BLUE - ALPHA
    if(dataOut->ensureMinSize(componentsOut*nbPixels))
      {
      char * ptr=dataOut->accessMemory();
      for(unsigned int y=heightOut;y--;)
        {
        unsigned int h=heightOut-1-y;
        for(unsigned int x=widthOut;x--;)
          {
          Imlib_Color color;
          imlib_image_query_pixel(x,h,&color);
          unsigned int index=4*(x+(widthOut*y));
          ptr[index]=color.red;
          ptr[index+1]=color.green;
          ptr[index+2]=color.blue;
          ptr[index+3]=color.alpha;
          if(!rgb&&((color.red!=color.green)||(color.red!=color.blue)))
            {
            rgb=true;
            }
          }
        }
      if(!rgb)
        {
        componentsOut=2; // GRAY - ALPHA
        for(unsigned int i=0;i<nbPixels;i++)
          {
          ptr[2*i]=ptr[4*i];
          ptr[2*i+1]=ptr[4*i+3];
          }
        }
      result=true;
      }
    }
  else
    {
    bool rgb=false;
    componentsOut=3; // RED - GREEN - BLUE
    if(dataOut->ensureMinSize(componentsOut*nbPixels))
      {
      char * ptr=dataOut->accessMemory();
      for(unsigned int y=heightOut;y--;)
        {
        unsigned int h=heightOut-1-y;
        for(unsigned int x=widthOut;x--;)
          {
          Imlib_Color color;
          imlib_image_query_pixel(x,h,&color);
          unsigned int index=3*(x+(widthOut*y));
          ptr[index]=color.red;
          ptr[index+1]=color.green;
          ptr[index+2]=color.blue;
          if(!rgb&&((color.red!=color.green)||(color.red!=color.blue)))
            {
            rgb=true;
            }
          }
        }
      if(!rgb)
        {
        componentsOut=1; // GRAY
        for(unsigned int i=0;i<nbPixels;i++)
          {
          ptr[i]=ptr[3*i];
          }
        }
      result=true;
      }
    }
  imlib_free_image();
  }
return(result);
}

static
bool // success
Imlib2_imageSaver(const char * fileName,
                  unsigned int width,
                  unsigned int height,
                  unsigned int components,
                  const char * data)
{
Imlib_Image image=imlib_create_image(width,height);
if(image)
  {
  imlib_context_set_image(image);
  switch(components)
    {
    case 1:
      {
      imlib_image_set_has_alpha(0);
      for(unsigned int y=height;y--;)
        {
        unsigned int h=height-1-y;
        for(unsigned int x=width;x--;)
          {
          unsigned int index=x+(width*y);
          imlib_context_set_color(data[index],
                                  data[index],
                                  data[index],
                                  255);
          imlib_image_draw_pixel(x,h,0);
          }
        }
      break;
      }
    case 2:
      {
      imlib_image_set_has_alpha(1);
      for(unsigned int y=height;y--;)
        {
        unsigned int h=height-1-y;
        for(unsigned int x=width;x--;)
          {
          unsigned int index=2*(x+(width*y));
          imlib_context_set_color(data[index],
                                  data[index],
                                  data[index],
                                  data[index+1]);
          imlib_image_draw_pixel(x,h,0);
          }
        }
      break;
      }
    case 3:
      {
      imlib_image_set_has_alpha(0);
      for(unsigned int y=height;y--;)
        {
        unsigned int h=height-1-y;
        for(unsigned int x=width;x--;)
          {
          unsigned int index=3*(x+(width*y));
          imlib_context_set_color(data[index],
                                  data[index+1],
                                  data[index+2],
                                  255);
          imlib_image_draw_pixel(x,h,0);
          }
        }
      break;
      }
    case 4:
      {
      imlib_image_set_has_alpha(1);
      for(unsigned int y=height;y--;)
        {
        unsigned int h=height-1-y;
        for(unsigned int x=width;x--;)
          {
          unsigned int index=4*(x+(width*y));
          imlib_context_set_color(data[index],
                                  data[index+1],
                                  data[index+2],
                                  data[index+3]);
          imlib_image_draw_pixel(x,h,0);
          }
        }
      break;
      }
    }
  char * dot=strrchr(fileName,'.');
  if(dot)
    {
    imlib_image_set_format(dot+1);
    }
  Imlib_Load_Error err;
  imlib_save_image_with_error_return(fileName,&err);
  imlib_free_image();
  return(err==IMLIB_LOAD_ERROR_NONE);
  }
return(false);
}

//-------- Initialization ----------------------------------------------------

bool // success
ImageLoader_init(const StlVector<StlString> * /* params */)
{
LLW::_imageLoaderFncts->push_back(&Imlib2_imageLoader);
LLW::_imageSaverFncts->push_back(&Imlib2_imageSaver);
return(true);
}

extern "C"
bool // success
Imlib2ImageLoader_init(const StlVector<StlString> * params)
{
return(ImageLoader_init(params)); // friend of LLW
}

void
ImageLoader_destroy(void)
{
unsigned int i;
for(i=LLW::_imageLoaderFncts->size();i--;)
  {
  LLW::ImageLoaderFnct fnct=(*LLW::_imageLoaderFncts)[i];
  if(fnct==&Imlib2_imageLoader)
    {
    StlVectorFastErase(*LLW::_imageLoaderFncts,i);
    break;
    }
  }
for(i=LLW::_imageSaverFncts->size();i--;)
  {
  LLW::ImageSaverFnct fnct=(*LLW::_imageSaverFncts)[i];
  if(fnct==&Imlib2_imageSaver)
    {
    StlVectorFastErase(*LLW::_imageSaverFncts,i);
    break;
    }
  }
}

extern "C"
void
Imlib2ImageLoader_destroy(void)
{
ImageLoader_destroy(); // friend of LLW
}

} // namespace AReVi

//----------------------------------------------------------------------------
