// LI2/ENIB - AReVi - Java wrapper library
//----------------------------------------------------------------------------
// file        : Ext/JavaWrapper/javaWrapper.cpp
// author(s)   : F. Harrouet
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
//
//  This library uses the JNI API (http://java.sun.com).
//
//  The resulting dynamic library can be loaded at run-time from Java.
//
//  NB: under Window$/CYGWIN the jvm.dll.a and jawt.dll.a files can be
//  generated as follows:
//    $ dlltool -k --def jvm.def --dllname jvm.dll --output-lib jvm.dll.a
//    $ dlltool -U --def jawt.def --dllname jawt.dll --output-lib jawt.dll.a
//
//----------------------------------------------------------------------------

#include "AReVi/Utils/javaObject.h"

#include "AReVi/arSystem.h"

#ifdef __CYGWIN__
typedef long long __int64;
#endif // __CYGWIN__

#include <jni.h>

// very tricky and platform dependant ! do not use for now !
// #define USE_JAWT

#ifdef USE_JAWT
#include <jawt.h>
#include <jawt_md.h>
#endif

namespace AReVi {

static JavaVM * JAVA_jvm=(JavaVM *)0;
static JNIEnv * JAVA_env=(JNIEnv *)0;

static bool (*JAVA_AReViAppInitFnct)(int,char **)=(bool (*)(int,char **))0;
static bool (*JAVA_AReViAppUpdateFnct)(void)=(bool (*)(void))0;
static void (*JAVA_AReViAppDestroyFnct)(void)=(void (*)(void))0;

static jclass JAVA_objectClass=(jclass)0;
static jclass JAVA_stringClass=(jclass)0;

static jclass JAVA_AReViObjectClass=(jclass)0;
static jmethodID JAVA_deleteMethod=(jmethodID)0;
static jmethodID JAVA_fromAReViMethod=(jmethodID)0;
static jfieldID JAVA_addressField=(jfieldID)0;

static jclass JAVA_booleanClass=(jclass)0;
static jmethodID JAVA_booleanConstr=(jmethodID)0;
static jmethodID JAVA_booleanMethod=(jmethodID)0;

static jclass JAVA_byteClass=(jclass)0;
static jmethodID JAVA_byteConstr=(jmethodID)0;
static jmethodID JAVA_byteMethod=(jmethodID)0;

static jclass JAVA_charClass=(jclass)0;
static jmethodID JAVA_charConstr=(jmethodID)0;
static jmethodID JAVA_charMethod=(jmethodID)0;

static jclass JAVA_shortClass=(jclass)0;
static jmethodID JAVA_shortConstr=(jmethodID)0;
static jmethodID JAVA_shortMethod=(jmethodID)0;

static jclass JAVA_intClass=(jclass)0;
static jmethodID JAVA_intConstr=(jmethodID)0;
static jmethodID JAVA_intMethod=(jmethodID)0;

static jclass JAVA_longClass=(jclass)0;
static jmethodID JAVA_longConstr=(jmethodID)0;
static jmethodID JAVA_longMethod=(jmethodID)0;

static jclass JAVA_floatClass=(jclass)0;
static jmethodID JAVA_floatConstr=(jmethodID)0;
static jmethodID JAVA_floatMethod=(jmethodID)0;

static jclass JAVA_doubleClass=(jclass)0;
static jmethodID JAVA_doubleConstr=(jmethodID)0;
static jmethodID JAVA_doubleMethod=(jmethodID)0;

//-------- Utilities ---------------------------------------------------------

inline static
void
JAVA_beginNative(JNIEnv * env)
{
if(env->MonitorEnter(JAVA_AReViObjectClass))
  {
  if(env->ExceptionOccurred())
    {
    env->ExceptionDescribe();
    }
  }
JAVA_env=env;
}

inline static
void
JAVA_endNative(void)
{
if(JAVA_AReViObjectClass) // avoid access after destruction
  {
  if(JAVA_env->MonitorExit(JAVA_AReViObjectClass))
    {
    if(JAVA_env->ExceptionOccurred())
      {
      JAVA_env->ExceptionDescribe();
      }
    }
  }
}

static
bool // success
loadClass(jclass & classOut,
          const char * className)
{
jclass cls=JAVA_env->FindClass(className);
if(cls)
  {
  classOut=(jclass)JAVA_env->NewGlobalRef(cls);
  JAVA_env->DeleteLocalRef(cls);
  return(true);
  }
else
  {
  JAVA_destroy();
  return(false);
  }
}

static
bool // success
loadMethod(jmethodID & methodOut,
           jclass cls,
           const char * methodName,
           const char * signature)
{
jmethodID m=JAVA_env->GetMethodID(cls,methodName,signature);
if(m)
  {
  methodOut=m;
  return(true);
  }
else
  {
  JAVA_destroy();
  return(false);
  }
}

static
bool // success
loadField(jfieldID & fieldOut,
          jclass cls,
          const char * fieldName,
          const char * signature)
{
jfieldID f=JAVA_env->GetFieldID(cls,fieldName,signature);
if(f)
  {
  fieldOut=f;
  return(true);
  }
else
  {
  JAVA_destroy();
  return(false);
  }
}

//-------- Construction / Destruction ----------------------------------------

static
void * // java ref
JAVA_new(JavaObject * obj)
{
jobject javaRef=(jobject)0;
jclass cls;
if(obj->getClass()==JavaObject::CLASS())
  {
  cls=JAVA_AReViObjectClass;
  }
else
  {
  cls=JAVA_env->FindClass(obj->getClass()->getName().c_str());
  if(JAVA_env->ExceptionOccurred())
    {
    JAVA_env->ExceptionDescribe();
    }
  }
if(cls)
  {
  jmethodID constr=JAVA_env->GetMethodID(cls,"<init>","(I)V");
  if(constr)
    {
    jobject ref=JAVA_env->NewObject(cls,constr,(jint)obj);
    if(JAVA_env->ExceptionOccurred())
      {
      JAVA_env->ExceptionDescribe();
      }
    if(ref)
      {
      javaRef=JAVA_env->NewGlobalRef(ref);
      JAVA_env->DeleteLocalRef(ref);
      }
    }
  if(obj->getClass()!=JavaObject::CLASS())
    {
    JAVA_env->DeleteLocalRef(cls);
    }
  }
return((void *)javaRef);
}

static
void
JAVA_delete(void * javaRef)
{
jobject ref=(jobject)javaRef;
JAVA_env->CallVoidMethod(ref,JAVA_deleteMethod);
if(JAVA_env->ExceptionOccurred())
  {
  JAVA_env->ExceptionDescribe();
  }
JAVA_env->DeleteGlobalRef(ref);
}

//-------- Communication -----------------------------------------------------

static
ArRef<JavaData> // result
translateJavaToAReVi(jobjectArray array)
{
unsigned int nb=JAVA_env->GetArrayLength(array);
ArRef<JavaData> data=JavaData::NEW();
for(unsigned int i=0;i<nb;i++)
  {
  jobject obj=JAVA_env->GetObjectArrayElement(array,i);
  if(obj)
    {
    if(JAVA_env->IsInstanceOf(obj,JAVA_booleanClass))
      {
      data->addJBoolean(JAVA_env->CallBooleanMethod(obj,JAVA_booleanMethod));
      }
    else if(JAVA_env->IsInstanceOf(obj,JAVA_byteClass))
      {
      data->addJByte(JAVA_env->CallByteMethod(obj,JAVA_byteMethod));
      }
    else if(JAVA_env->IsInstanceOf(obj,JAVA_charClass))
      {
      data->addJChar(JAVA_env->CallCharMethod(obj,JAVA_charMethod));
      }
    else if(JAVA_env->IsInstanceOf(obj,JAVA_shortClass))
      {
      data->addJShort(JAVA_env->CallShortMethod(obj,JAVA_shortMethod));
      }
    else if(JAVA_env->IsInstanceOf(obj,JAVA_intClass))
      {
      data->addJInt(JAVA_env->CallIntMethod(obj,JAVA_intMethod));
      }
    else if(JAVA_env->IsInstanceOf(obj,JAVA_longClass))
      {
      data->addJLong(JAVA_env->CallLongMethod(obj,JAVA_longMethod));
      }
    else if(JAVA_env->IsInstanceOf(obj,JAVA_floatClass))
      {
      data->addJFloat(JAVA_env->CallFloatMethod(obj,JAVA_floatMethod));
      }
    else if(JAVA_env->IsInstanceOf(obj,JAVA_doubleClass))
      {
      data->addJDouble(JAVA_env->CallDoubleMethod(obj,JAVA_doubleMethod));
      }
    else if(JAVA_env->IsInstanceOf(obj,JAVA_stringClass))
      {
      const char * value=JAVA_env->GetStringUTFChars((jstring)obj,
                                                     (jboolean *)0);
      data->addJString(value);
      JAVA_env->ReleaseStringUTFChars((jstring)obj,value);
      }
    JAVA_env->DeleteLocalRef(obj);
    }
  }
return(data);
}

static
jobjectArray // result
translateAReViToJava(ArConstRef<JavaData> data)
{
unsigned int nb=data->getNbValues();
jobject obj=(jobject)0;
jobjectArray array=JAVA_env->NewObjectArray(nb,JAVA_objectClass,obj);
if(JAVA_env->ExceptionOccurred())
  {
  JAVA_env->ExceptionDescribe();
  }
if(array)
  {
  for(unsigned int i=nb;i--;)
    {
    switch(data->getType(i))
      {
      case JavaData::J_BOOLEAN:
        obj=JAVA_env->NewObject(JAVA_booleanClass,JAVA_booleanConstr,
                                data->readJBoolean(i));
        break;
      case JavaData::J_BYTE:
        obj=JAVA_env->NewObject(JAVA_byteClass,JAVA_byteConstr,
                                data->readJByte(i));
        break;
      case JavaData::J_CHAR:
        obj=JAVA_env->NewObject(JAVA_charClass,JAVA_charConstr,
                                data->readJChar(i));
        break;
      case JavaData::J_SHORT:
        obj=JAVA_env->NewObject(JAVA_shortClass,JAVA_shortConstr,
                                data->readJShort(i));
        break;
      case JavaData::J_INT:
        obj=JAVA_env->NewObject(JAVA_intClass,JAVA_intConstr,
                                data->readJInt(i));
        break;
      case JavaData::J_LONG:
        obj=JAVA_env->NewObject(JAVA_longClass,JAVA_longConstr,
                                data->readJLong(i));
        break;
      case JavaData::J_FLOAT:
        obj=JAVA_env->NewObject(JAVA_floatClass,JAVA_floatConstr,
                                data->readJFloat(i));
        break;
      case JavaData::J_DOUBLE:
        obj=JAVA_env->NewObject(JAVA_doubleClass,JAVA_doubleConstr,
                                data->readJDouble(i));
        break;
      case JavaData::J_STRING:
        obj=JAVA_env->NewStringUTF(data->readJString(i).c_str());
        break;
      }
    if(JAVA_env->ExceptionOccurred())
      {
      JAVA_env->ExceptionDescribe();
      }
    if(obj)
      {
      JAVA_env->SetObjectArrayElement(array,i,obj);
      JAVA_env->DeleteLocalRef(obj);
      }
    }
  }
return(array);
}

static
ArRef<JavaData> // result
JAVA_toJava(void * obj,
            ArConstRef<JavaData> data)
{
ArRef<JavaData> result;
jobjectArray array=translateAReViToJava(data);
if(!array)
  {
  return(result);
  }
jobjectArray r=(jobjectArray)JAVA_env->CallObjectMethod((jobject)obj,
                                                        JAVA_fromAReViMethod,
                                                        array);
if(JAVA_env->ExceptionOccurred())
  {
  JAVA_env->ExceptionDescribe();
  }
JAVA_env->DeleteLocalRef(array);
if(r)
  {
  result=translateJavaToAReVi(r);
  JAVA_env->DeleteLocalRef(r);
  }
return(result);
}

extern "C"
jobjectArray // result
JAVA_native_toAReVi(JNIEnv * env,
                    jobject self,
                    jobjectArray array)
{
JAVA_beginNative(env);
jobjectArray result=(jobjectArray)0;
JavaObject * target=reinterpret_cast<JavaObject *>
                    (JAVA_env->GetIntField(self,JAVA_addressField));
if(target)
  {
  ArRef<JavaData> data=translateJavaToAReVi(array);
  ArRef<JavaData> r=target->fromJava(data);
  if(r)
    {
    result=translateAReViToJava(r);
    }
  }
JAVA_endNative();
return(result);
}

extern "C"
jint // native window ID
JAVA_native_windowID(JNIEnv * env,
                     jclass /* cls */,
                     jobject /* component */)
{
JAVA_beginNative(env);
int winID=0;
#ifdef USE_JAWT
JAWT awt;
awt.version=JAWT_VERSION_1_3;
if(JAWT_GetAWT(JAVA_env,&awt)!=JNI_FALSE)
  {
  JAWT_DrawingSurface * ds=awt.GetDrawingSurface(JAVA_env,component);
  if(ds)
    {
    if(!(ds->Lock(ds)&JAWT_LOCK_ERROR))
      {
      JAWT_DrawingSurfaceInfo * dsi=ds->GetDrawingSurfaceInfo(ds);
#ifdef __CYGWIN__
      winID=(int)((JAWT_Win32DrawingSurfaceInfo*)dsi->platformInfo)->hwnd;
#else // not __CYGWIN__
#  ifdef __APPLE__
      winID=(int)((JAWT_MacDrawingSurfaceInfo*)dsi->platformInfo)->fQDWindow;
#  else // not __APPLE__
      winID=(int)((JAWT_X11DrawingSurfaceInfo*)dsi->platformInfo)->drawable;
#  endif // not __APPLE__
#endif // not __CYGWIN__
      ds->FreeDrawingSurfaceInfo(dsi);
      ds->Unlock(ds);
      }
    awt.FreeDrawingSurface(ds);
    }
  }
#endif // USE_JAWT
JAVA_endNative();
return(winID);
}

//-------- Initialization ----------------------------------------------------

bool // success
JAVA_init(void)
{
JavaObject::_initSuccess=false;
if(!loadClass(JAVA_objectClass,"java/lang/Object")||
   !loadClass(JAVA_stringClass,"java/lang/String")||
   !loadClass(JAVA_AReViObjectClass,"AReViObject"))
  {
  return(false);
  }
if(!loadMethod(JAVA_deleteMethod,JAVA_AReViObjectClass,"delete","()V")||
   !loadMethod(JAVA_fromAReViMethod,JAVA_AReViObjectClass,
               "fromAReVi","([Ljava/lang/Object;)[Ljava/lang/Object;")||
   !loadField(JAVA_addressField,JAVA_AReViObjectClass,"_address","I"))
  {
  return(false);
  }
if(!loadClass(JAVA_booleanClass,"java/lang/Boolean")||
   !loadMethod(JAVA_booleanConstr,JAVA_booleanClass,"<init>","(Z)V")||
   !loadMethod(JAVA_booleanMethod,JAVA_booleanClass,"booleanValue","()Z"))
  {
  return(false);
  }
if(!loadClass(JAVA_byteClass,"java/lang/Byte")||
   !loadMethod(JAVA_byteConstr,JAVA_byteClass,"<init>","(B)V")||
   !loadMethod(JAVA_byteMethod,JAVA_byteClass,"byteValue","()B"))
  {
  return(false);
  }
if(!loadClass(JAVA_charClass,"java/lang/Character")||
   !loadMethod(JAVA_charConstr,JAVA_charClass,"<init>","(C)V")||
   !loadMethod(JAVA_charMethod,JAVA_charClass,"charValue","()C"))
  {
  return(false);
  }
if(!loadClass(JAVA_shortClass,"java/lang/Short")||
   !loadMethod(JAVA_shortConstr,JAVA_shortClass,"<init>","(S)V")||
   !loadMethod(JAVA_shortMethod,JAVA_shortClass,"shortValue","()S"))
  {
  return(false);
  }
if(!loadClass(JAVA_intClass,"java/lang/Integer")||
   !loadMethod(JAVA_intConstr,JAVA_intClass,"<init>","(I)V")||
   !loadMethod(JAVA_intMethod,JAVA_intClass,"intValue","()I"))
  {
  return(false);
  }
if(!loadClass(JAVA_longClass,"java/lang/Long")||
   !loadMethod(JAVA_longConstr,JAVA_longClass,"<init>","(J)V")||
   !loadMethod(JAVA_longMethod,JAVA_longClass,"longValue","()J"))
  {
  return(false);
  }
if(!loadClass(JAVA_floatClass,"java/lang/Float")||
   !loadMethod(JAVA_floatConstr,JAVA_floatClass,"<init>","(F)V")||
   !loadMethod(JAVA_floatMethod,JAVA_floatClass,"floatValue","()F"))
  {
  return(false);
  }
if(!loadClass(JAVA_doubleClass,"java/lang/Double")||
   !loadMethod(JAVA_doubleConstr,JAVA_doubleClass,"<init>","(D)V")||
   !loadMethod(JAVA_doubleMethod,JAVA_doubleClass,"doubleValue","()D"))
  {
  return(false);
  }
JavaObject::_newFnct=&JAVA_new;
JavaObject::_deleteFnct=&JAVA_delete;
JavaObject::_toJavaFnct=&JAVA_toJava;
JavaObject::_initSuccess=true;
return(true);
}

void
JAVA_destroy(void)
{
if(JavaObject::_initSuccess)
  {
  StlVector<ArRef<JavaObject> > objects;
  JavaObject::CLASS()->getInstances(objects,true);
  while(!objects.empty())
    {
    objects.back().destroy(); // UGLY !
    objects.pop_back();
    }
  }
if(JAVA_env)
  {
  if(JAVA_env->ExceptionOccurred())
    {
    JAVA_env->ExceptionDescribe();
    }
  }
JavaObject::_initSuccess=false;
JavaObject::_newFnct=(void * (*)(JavaObject *))0;
JavaObject::_deleteFnct=(void (*)(void *))0;
JavaObject::_toJavaFnct=(ArRef<JavaData> (*)(void *,ArConstRef<JavaData>))0;
JAVA_jvm=(JavaVM *)0;
JAVA_env=(JNIEnv *)0;
JAVA_AReViAppInitFnct=(bool (*)(int,char **))0;
JAVA_AReViAppUpdateFnct=(bool (*)(void))0;
JAVA_AReViAppDestroyFnct=(void (*)(void))0;
JAVA_objectClass=(jclass)0;
JAVA_stringClass=(jclass)0;
JAVA_AReViObjectClass=(jclass)0;
JAVA_deleteMethod=(jmethodID)0;
JAVA_fromAReViMethod=(jmethodID)0;
JAVA_addressField=(jfieldID)0;
JAVA_booleanClass=(jclass)0;
JAVA_booleanConstr=(jmethodID)0;
JAVA_booleanMethod=(jmethodID)0;
JAVA_byteClass=(jclass)0;
JAVA_byteConstr=(jmethodID)0;
JAVA_byteMethod=(jmethodID)0;
JAVA_charClass=(jclass)0;
JAVA_charConstr=(jmethodID)0;
JAVA_charMethod=(jmethodID)0;
JAVA_shortClass=(jclass)0;
JAVA_shortConstr=(jmethodID)0;
JAVA_shortMethod=(jmethodID)0;
JAVA_intClass=(jclass)0;
JAVA_intConstr=(jmethodID)0;
JAVA_intMethod=(jmethodID)0;
JAVA_longClass=(jclass)0;
JAVA_longConstr=(jmethodID)0;
JAVA_longMethod=(jmethodID)0;
JAVA_floatClass=(jclass)0;
JAVA_floatConstr=(jmethodID)0;
JAVA_floatMethod=(jmethodID)0;
JAVA_doubleClass=(jclass)0;
JAVA_doubleConstr=(jmethodID)0;
JAVA_doubleMethod=(jmethodID)0;
}

extern "C"
jboolean // go on
JAVA_native_initApp(JNIEnv * env,
                    jclass /* cls */,
                    jobjectArray args)
{
JAVA_env=env;
if(!JAVA_init())
  {
  return(JNI_FALSE);
  }
JAVA_beginNative(env);
unsigned int i,nb=env->GetArrayLength(args);
StlVector<char *> argv;
argv.reserve(nb+1);
for(i=0;i<nb;i++)
  {
  jobject obj=env->GetObjectArrayElement(args,i);
  const char * arg=env->GetStringUTFChars((jstring)obj,(jboolean *)0);
  argv.push_back(const_cast<char *>(arg));
  }
argv.push_back((char *)0);
bool result=(*JAVA_AReViAppInitFnct)(argv.size()-1,&argv.front());
for(i=0;i<nb;i++)
  {
  jobject obj=env->GetObjectArrayElement(args,i);
  env->ReleaseStringUTFChars((jstring)obj,argv[i]);
  }
JAVA_endNative();
return(result ? JNI_TRUE : JNI_FALSE);
}

extern "C"
jboolean // go on
JAVA_native_updateApp(JNIEnv * env,
                      jclass /* cls */)
{
JAVA_beginNative(env);
bool result=(*JAVA_AReViAppUpdateFnct)();
JAVA_endNative();
return(result ? JNI_TRUE : JNI_FALSE);
}

extern "C"
void
JAVA_native_destroyApp(JNIEnv * env,
                       jclass /* cls */)
{
JAVA_beginNative(env);
(*JAVA_AReViAppDestroyFnct)();
JAVA_destroy();
JAVA_endNative();
}

int
JAVA_InitAReViApp(void * javaVM,
                  bool (*initAppFnct)(int,char **),
                  bool (*updateAppFnct)(void),
                  void (*destroyAppFnct)(void))
{
JAVA_jvm=reinterpret_cast<JavaVM *>(javaVM);
JAVA_AReViAppInitFnct=initAppFnct;
JAVA_AReViAppUpdateFnct=updateAppFnct;
JAVA_AReViAppDestroyFnct=destroyAppFnct;
return(JNI_VERSION_1_2);
}

} // namespace AReVi

//----------------------------------------------------------------------------
