// LI2/ENIB - AReVi-Contrib - Jpeg compression plugin
//----------------------------------------------------------------------------
// file        : ExtContrib/JpegLib/jpegLib.cpp
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
//
//  This plugin uses the JpegLib API (http://www.ijg.org).
//
//  The resulting dynamic library can be loaded at run-time
//  (fails if a dependency problem occurs).
//
//----------------------------------------------------------------------------

#include <stdio.h>
#include <string.h>
#include <sys/types.h>

#ifdef __cplusplus // for bad jpeglib.h files
extern "C" {
#endif

#include <jpeglib.h>

#ifdef __cplusplus
}
#endif


#include "AReVi/Utils/memoryBlock.h"
#include "AReVi/Utils/memoryStream.h"
#include "AReVi/Lib3D/llw_priv.h"

namespace AReVi {

//----------------------------------------------------------------------------
// JPEG COMPRESSION TOOLS
//----------------------------------------------------------------------------
// Init destination data for jpeg compression
GLOBAL(void)
jpeg_stream_dest(j_compress_ptr cinfo, ArRef<AbstractOStream> outStream);

// Init source data for jpeg compression
GLOBAL(void)
jpeg_stream_src(j_decompress_ptr cinfo, ArRef<AbstractIStream> inStream);


// compress input data to output data using jpeg compression

static
bool // success
JpegLib_compressRGBToJPEG(ArConstRef<MemoryBlock> uncompressedBuffer,
                          ArRef<MemoryBlock> compressedBufferOut,
                          unsigned int width,
                          unsigned int height,
                          unsigned int quality)//[0-100]
{
  struct jpeg_compress_struct cinfo;
  struct jpeg_error_mgr jerr;
  JSAMPROW row_pointer[1];	// pointer to JSAMPLE row[s]
  int row_stride;		// physical row width in image buffer

  // Step 1: allocate and initialize JPEG compression object
  cinfo.err = jpeg_std_error(&jerr);
  jpeg_create_compress(&cinfo);


  // Step 2: specify data destination (here memoryBlock)
  //  compressedBufferOut.size(uncompressedBuffer.getSize());
  ArRef<MemoryOStream> stream=MemoryOStream::NEW(compressedBufferOut,0);
  jpeg_stream_dest(&cinfo, stream);

  // Step 3: set parameters for compression
  cinfo.image_width = width; 	// image width and height, in pixels
  cinfo.image_height = height;
  cinfo.input_components = 3; // # of color components per pixel
  cinfo.in_color_space = JCS_RGB; 	// colorspace of input image
  jpeg_set_defaults(&cinfo);
  jpeg_set_quality(&cinfo, quality, TRUE ); // limit to baseline-JPEG values

  // Step 4: Start compressor
  jpeg_start_compress(&cinfo, TRUE);
  row_stride = width * 3; // JSAMPLEs per row in image_buffer
  while (cinfo.next_scanline < cinfo.image_height)
  {
    row_pointer[0] = (JSAMPLE *)& uncompressedBuffer->getMemory()[cinfo.next_scanline * row_stride];
      (void) jpeg_write_scanlines(&cinfo, row_pointer, 1);
  }

  // Step 6: Finish compression
  jpeg_finish_compress(&cinfo);
  // Step 7: release JPEG compression object
  jpeg_destroy_compress(&cinfo);

  return true;
}

// uncompress input data to output data using jpeg compression
static
bool // success
JpegLib_uncompressRGBFromJPEG(ArConstRef<MemoryBlock> compressedBuffer,
                   ArRef<MemoryBlock> uncompressedBufferOut,
                   unsigned int &widthOut,
                   unsigned int &heightOut
                   )
{
  // This struct contains the JPEG decompression parameters and pointers to
  // working space (which is allocated as needed by the JPEG library).
  struct jpeg_decompress_struct cinfo;
  struct jpeg_error_mgr jerr;

  JSAMPARRAY buffer;		/* Output row buffer */
  int row_stride;		/* physical row width in output buffer */

  ArRef<MemoryIStream> stream=MemoryIStream::NEW(compressedBuffer,0);
  ArRef<MemoryOStream> uncompressedStream=MemoryOStream::NEW(uncompressedBufferOut,0);

  /* Step 1: allocate and initialize JPEG decompression object */
  /* We set up the normal JPEG error routines, then override error_exit. */
  cinfo.err = jpeg_std_error(&jerr);
  // Now we can initialize the JPEG decompression object.
  jpeg_create_decompress(&cinfo);

  // Step 2: specify data source (a stream)
  jpeg_stream_src(&cinfo, stream);

  // Step 3: read file parameters with jpeg_read_header()
  (void) jpeg_read_header(&cinfo, TRUE);

  // Step 4: set parameters for decompression
  // In this example, we don't need to change any of the defaults set by
  // jpeg_read_header(), so we do nothing here.

  // Step 5: Start decompressor
  (void) jpeg_start_decompress(&cinfo);

  /* We may need to do some setup of our own at this point before reading
   * the data.  After jpeg_start_decompress() we have the correct scaled
   * output image dimensions available, as well as the output colormap
   * if we asked for color quantization.
   * In this example, we need to make an output work buffer of the right size.
   */
  /* JSAMPLEs per row in output buffer */
  row_stride = cinfo.output_width * cinfo.output_components;
  /* Make a one-row-high sample array that will go away when done with image */
  buffer = (*cinfo.mem->alloc_sarray)
    ((j_common_ptr) &cinfo, JPOOL_IMAGE, row_stride, 1);

  // Step 6: while (scan lines remain to be read)
  //           jpeg_read_scanlines(...);

  /* Here we use the library's state variable cinfo.output_scanline as the
   * loop counter, so that we don't have to keep track ourselves.
   */
  while (cinfo.output_scanline < cinfo.output_height)
  {
    /* jpeg_read_scanlines expects an array of pointers to scanlines.
     * Here the array is only one element long, but you could ask for
     * more than one scanline at a time if that's more convenient.
     */
    (void) jpeg_read_scanlines(&cinfo, buffer, 1);
    /* Assume put_scanline_someplace wants a pointer and sample count. */
    //cerr << "scanline =" << cinfo.output_scanline << "/" << cinfo.output_height << endl;
    uncompressedStream->writeData(buffer[0],row_stride);
    //put_scanline_someplace(buffer[0], row_stride);
  }

  // Step 7: Finish decompression
  (void) jpeg_finish_decompress(&cinfo);

  // Step 8: Release JPEG decompression object

  // This is an important step since it will release a good deal of memory.
  jpeg_destroy_decompress(&cinfo);

  widthOut=cinfo.image_width;
  heightOut=cinfo.image_height;

  //cerr << "w=" << widthOut << "  h=" << heightOut << endl <<
          //"=>rowstride=" << row_stride << "      info width*3=" << widthOut*3 << endl;
  return true;
}


//-------- Initialization ----------------------------------------------------
extern "C"
bool // success
JpegLib_init(const StlVector<StlString> * )
{
LLW::_initJpegLibSuccess=false;
LLW::_compressRGBImageToJPEG=JpegLib_compressRGBToJPEG;
LLW::_uncompressRGBImageFromJPEG=JpegLib_uncompressRGBFromJPEG;
LLW::_initJpegLibSuccess=true;
return true;
}

extern "C"
void
JpegLib_destroy(void)
{
LLW::_initJpegLibSuccess=false;
}

} // namespace AReVi

//----------------------------------------------------------------------------
