// LI2/ENIB - AReVi-Contrib - Lua plugin
//----------------------------------------------------------------------------
// file        : ExtContrib/LuaWrapper/luaWrapper.cpp
// author(s)   : D. Herviou
//----------------------------------------------------------------------------
// Copyright (C) 2002-2005  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
//
//  This plugin uses the Lua API (http://www.lua.org).
//
//  The resulting dynamic library can be loaded at run-time
//  (fails if a dependency problem occurs).
//
//----------------------------------------------------------------------------

#include "AReVi/Contrib/arLua.h"
#include "AReVi/arClass.h"

extern "C"{
#include "lua.h"
#include "lauxlib.h"
#include "lualib.h"
void luaC_collectgarbage (lua_State *L); // #include "lgc.h"
}

namespace AReVi{


  //lua basic function
static  void* luaWrapper_lua_open(void){
  return lua_open();
}

static  void* luaWrapper_lua_newstate(Lua::lua_Allocator alloc, void* ud){
  return lua_newstate((lua_Alloc)alloc,ud);
}


static  void luaWrapper_lua_close(void* L){
    lua_close((lua_State*)L);
}

  //opening function
static  int luaWrapper_luaopen_base(void* L){
    return luaopen_base((lua_State*)L);
  }

static  int luaWrapper_luaopen_table(void* L){
    return luaopen_table((lua_State*)L);
  }

static  int luaWrapper_luaopen_io(void* L){
    return luaopen_io((lua_State*)L);
  }
static  int luaWrapper_luaopen_string(void* L){
    return luaopen_string((lua_State*)L);
  }
static  int luaWrapper_luaopen_math(void* L){
    return luaopen_math((lua_State*)L);
  }
// static  int luaWrapper_luaopen_loadlib(void* L){
//     return luaopen_loadlib((lua_State*)L);
//   }
static  int luaWrapper_luaopen_debug(void* L){
    return luaopen_debug((lua_State*)L);
  }


  //execution stack
static  void luaWrapper_lua_call (void *L, int nargs, int nresults){
    lua_call((lua_State*)L, nargs, nresults);
  }
static  int   luaWrapper_lua_pcall (void *L, int nargs, int nresults, int errfunc){
    return lua_pcall((lua_State*)L, nargs, nresults, errfunc);
  }
static  int luaWrapper_lua_cpcall (void *L, int(*func)(void*), void *ud){
    return lua_cpcall((lua_State*)L, (lua_CFunction)func, ud);
  }

  //error
static  int luaWrapper_lua_error (void* L){
    return lua_error((lua_State *)L);
  }


static  void luaWrapper_lua_concat (void *L, int n){
    lua_concat((lua_State*)L,n);
  }

  //divers
static  void luaWrapper_lua_getglobal(void* L,const char* s){
    lua_getglobal((lua_State*)L, s);
  }

static  void luaWrapper_lua_getfield(void* L,int index, const char* s){
    lua_getfield((lua_State*)L,index, s);
  }

static  void luaWrapper_lua_setglobal(void* L,const char* s){
    lua_setglobal((lua_State*)L, s);
  }

static  void luaWrapper_lua_getregistry(void* L){
    lua_getregistry((lua_State*)L);
  }


  //auxiliar library
static  void luaWrapper_luaL_openlib(void* L,
			      const char* libname,
			      const void *l,
			      int nup){
    luaL_openlib((lua_State*)L, libname, (luaL_reg*)l, nup);
}

static void luaWrapper_luaL_openlibs(void *L){
   luaL_openlibs((lua_State*)L);
}

#ifdef LUA_JIT
static int luaWrapper_luaopen_jit(void* L){
  return luaopen_jit((lua_State*)L);
}
#endif

static  int luaWrapper_luaL_loadfile (void *L, const char *filename){
    return luaL_loadfile((lua_State*)L, filename);
  }

static  int luaWrapper_luaL_loadbuffer (void *L,
				  const char *buff,
				  size_t sz,
				  const char *name){
    return luaL_loadbuffer((lua_State*)L,buff, sz,name);
  }

static int luaWrapper_luaL_error(void* L,
				   const char* fmt,...){
  va_list argp;
  va_start(argp, fmt);
  luaL_where((lua_State*)L, 1);
  lua_pushvfstring((lua_State*)L, fmt, argp);
  va_end(argp);
  lua_concat((lua_State*)L, 2);
  //  cerr<<"lua_concat return "<<lua_tostring((lua_State*)L,-1)<<endl;

  return lua_error((lua_State*)L);


//  va_list argp;
//   va_start(argp,fmt);
//   return luaL_error((lua_State*)L,fmt,argp);
}

  static  int luaWrapper_luaL_dofile(void* L, const char* filename){
    return luaL_dofile((lua_State*)L, filename);
  }

  static  int luaWrapper_luaL_dostring(void* L, const char* str){
    return luaL_dostring((lua_State*)L, str);
  }
// static  int luaWrapper_lua_dobuffer(void* L, const char* buff,
// 			      size_t sz, const char* n){
//     return lua_dobuffer((lua_State*)L, buff, sz, n);
//   }





static  int luaWrapper_luaL_newmetatable(void* L, const char* name){
    return luaL_newmetatable((lua_State*)L,name);
  }

static  void luaWrapper_luaL_getmetatable(void* L, const char* name){
    luaL_getmetatable((lua_State*)L,name);
  }



  //table manipulation
static  void luaWrapper_lua_setmetatable(void* L, int index){
    lua_setmetatable((lua_State*)L,index);
  }

static  void luaWrapper_lua_newtable(void* L){
    lua_newtable((lua_State*)L);
  }

static  void luaWrapper_lua_createtable(void* L, int narr, int nrec){
    lua_createtable((lua_State*)L, narr, nrec);
  }

static  void luaWrapper_lua_gettable(void* L,int index){
    lua_gettable((lua_State*)L,index);
  }

static  void luaWrapper_lua_settable(void* L,int index){
    lua_settable((lua_State*)L,index);
  }

static  void  luaWrapper_lua_rawget(void *L, int idx){
    lua_rawget((lua_State*)L, idx);
  }

static  void  luaWrapper_lua_rawgeti(void *L, int idx, int n){
    lua_rawgeti((lua_State*)L, idx, n);
  }

static  void  luaWrapper_lua_rawset(void *L, int idx){
    lua_rawset((lua_State*)L, idx);
  }

static  void  luaWrapper_lua_rawseti(void *L, int idx, int n){
    lua_rawseti((lua_State*)L, idx, n);
  }

static  int  luaWrapper_lua_next(void *L, int idx){
    return lua_next((lua_State*)L, idx);
  }


  //pushing element on stack
static  void luaWrapper_lua_pushnil(void * L){
    lua_pushnil ((lua_State *)L);
  }

static  void luaWrapper_lua_pushboolean(void* L, int b){
    lua_pushboolean((lua_State *)L, b);
  }

static  void luaWrapper_lua_pushnumber(void* L , double n){
    lua_pushnumber((lua_State*)L,n);
  }

static  void luaWrapper_lua_pushlstring(void* L,const char *s,size_t length){
    lua_pushlstring((lua_State*)L, s, length);
  }

static  void luaWrapper_lua_pushstring (void * L, const char *s){
    lua_pushstring((lua_State*)L, s);
  }

//   void luaWrapper_lua_pushliteral (void * L, const char *s){
//     lua_pushlstring((lua_State*)L, "" s, (sizeof(s)/sizeof(char))-1);
//   }

static  void luaWrapper_lua_pushlightuserdata(void* L, void* data){
    lua_pushlightuserdata((lua_State*)L,data);
  }

static  void luaWrapper_lua_pushcfunction(void* L, int(*cfunction)(void*) ){
    lua_pushcfunction((lua_State*)L, (lua_CFunction)cfunction);
  }


static  void* luaWrapper_lua_newuserdata(void* L, int nBytes){
    return lua_newuserdata((lua_State*)L,nBytes);
  }

  //query stack
static  int luaWrapper_lua_toboolean(void * L, int index){
    return lua_toboolean((lua_State*)L,index);
  }

static  double luaWrapper_lua_tonumber(void * L, int index){
    return lua_tonumber((lua_State*)L,index);
  }

static  const char* luaWrapper_lua_tostring(void * L, int index){
    return lua_tostring((lua_State*)L,index);
  }

static  void* luaWrapper_lua_touserdata(void * L, int index){
    return lua_touserdata((lua_State*)L,index);
  }

static  size_t luaWrapper_lua_strlen(void * L, int index){
    return lua_strlen((lua_State*)L,index);
  }

static  arlua_CFunction luaWrapper_lua_tocfunction(void* L, int index){
    return (arlua_CFunction)lua_tocfunction((lua_State*)L, index);
  }



  //testing element
static  int luaWrapper_lua_isnumber(void* L, int index){
    return lua_isnumber((lua_State*)L,index);
  }

static  int luaWrapper_lua_iscfunction(void* L, int index){
    return lua_iscfunction((lua_State*)L,index);
  }

static  int luaWrapper_lua_isfunction(void* L, int index){
    return lua_isfunction((lua_State*)L,index);
  }

static  int luaWrapper_lua_isuserdata(void* L, int index){
    return lua_isuserdata((lua_State*)L,index);
  }

static  int luaWrapper_lua_islightuserdata(void* L, int index){
    return lua_islightuserdata((lua_State*)L,index);
  }

static  int luaWrapper_lua_isstring(void* L, int index){
    return lua_isstring((lua_State*)L,index);
  }

static  int luaWrapper_lua_isboolean(void* L, int index){
    return lua_isboolean((lua_State*)L,index);
  }

static  int luaWrapper_lua_istable(void* L, int index){
    return lua_istable((lua_State*)L,index);
  }

static  int luaWrapper_lua_isnil(void* L, int index){
    return lua_isnil((lua_State*)L,index);
  }

static  int luaWrapper_lua_isnone(void* L, int index){
    return lua_isnone((lua_State*)L,index);
  }

static  int luaWrapper_lua_isnoneornil(void* L, int index){
    return lua_isnoneornil((lua_State*)L,index);
  }

static  int luaWrapper_lua_type(void* L, int index){
    return lua_type((lua_State*)L,index);
  }


  //stack manipulation
static  int luaWrapper_lua_gettop(void* L){
    return lua_gettop((lua_State*)L);
  }

static  void luaWrapper_lua_settop(void* L, int index){
    lua_settop((lua_State*)L,index);
  }

static  void luaWrapper_lua_pop(void* L, int index){
    lua_pop((lua_State*)L,index);
  }

static  void luaWrapper_lua_pushvalue(void * L, int index){
    lua_pushvalue((lua_State*)L, index);
  }

static  void luaWrapper_lua_remove(void* L, int index){
    lua_remove((lua_State*)L, index);
  }

static  void luaWrapper_lua_insert(void* L, int index){
    lua_insert((lua_State*)L, index);
  }

static  void luaWrapper_lua_replace(void* L, int index){
    lua_replace((lua_State*)L, index);
  }

static  int luaWrapper_lua_checkstack(void* L, int sz){
    return lua_checkstack((lua_State*)L,sz);
  }

  //garbage manipulation
static void luaWrapper_luaC_collectgarbage(void* L){
  luaC_collectgarbage((lua_State*)L);
}

static int luaWrapper_lua_gc(void* L, int what, int data){
  return lua_gc((lua_State*)L, what, data);
}



extern "C"
bool LuaWrapper_init(const StlVector<StlString> *){
  cerr<<"initialization plugin LUA"<<endl;
  Lua::_arlua_lua_open        = &luaWrapper_lua_open ;
  Lua::_arlua_lua_newstate    = &luaWrapper_lua_newstate ;
  Lua::_arlua_lua_close       = &luaWrapper_lua_close ;
  Lua::_arlua_luaopen_base    = &luaWrapper_luaopen_base ;
  Lua::_arlua_luaopen_table   = &luaWrapper_luaopen_table ;
  Lua::_arlua_luaopen_io      = &luaWrapper_luaopen_io ;
  Lua::_arlua_luaopen_string  = &luaWrapper_luaopen_string ;
  Lua::_arlua_luaopen_math    = &luaWrapper_luaopen_math ;
  //  Lua::_arlua_luaopen_loadlib = &luaWrapper_luaopen_loadlib ;
  Lua::_arlua_luaopen_debug   = &luaWrapper_luaopen_debug ;
  Lua::_arlua_lua_call        = &luaWrapper_lua_call ;
  Lua::_arlua_lua_pcall       = &luaWrapper_lua_pcall ;
  Lua::_arlua_lua_cpcall      = &luaWrapper_lua_cpcall ;
  Lua::_arlua_lua_error       = &luaWrapper_lua_error ;
  Lua::_arlua_lua_concat      = &luaWrapper_lua_concat ;
  Lua::_arlua_lua_getglobal   = &luaWrapper_lua_getglobal ;
  Lua::_arlua_lua_getfield    = &luaWrapper_lua_getfield ;
  Lua::_arlua_lua_setglobal   = &luaWrapper_lua_setglobal ;
  Lua::_arlua_lua_getregistry = &luaWrapper_lua_getregistry ;
  Lua::_arlua_luaL_openlib    = &luaWrapper_luaL_openlib ;
  Lua::_arlua_luaL_openlibs   = &luaWrapper_luaL_openlibs ;
  Lua::_arlua_luaL_loadfile   = &luaWrapper_luaL_loadfile ;
  Lua::_arlua_luaL_loadbuffer = &luaWrapper_luaL_loadbuffer ;
  Lua::_arlua_luaL_error      = &luaWrapper_luaL_error ;
  Lua::_arlua_luaL_dofile      = &luaWrapper_luaL_dofile ;
  Lua::_arlua_luaL_dostring    = &luaWrapper_luaL_dostring ;
#ifdef LUA_JIT
  Lua::_arlua_luaopen_jit     = &luaWrapper_luaopen_jit;
#endif
//   Lua::_arlua_lua_dobuffer    = &luaWrapper_lua_dobuffer ;

  Lua::_arlua_luaL_newmetatable= &luaWrapper_luaL_newmetatable ;
  Lua::_arlua_luaL_getmetatable= &luaWrapper_luaL_getmetatable ;
  Lua::_arlua_lua_setmetatable = &luaWrapper_lua_setmetatable ;
  Lua::_arlua_lua_newtable    = &luaWrapper_lua_newtable ;
  Lua::_arlua_lua_createtable = &luaWrapper_lua_createtable ;
  Lua::_arlua_lua_gettable    = &luaWrapper_lua_gettable ;
  Lua::_arlua_lua_settable    = &luaWrapper_lua_settable ;
  Lua::_arlua_lua_rawget      = &luaWrapper_lua_rawget ;
  Lua::_arlua_lua_rawgeti     = &luaWrapper_lua_rawgeti ;
  Lua::_arlua_lua_rawset      = &luaWrapper_lua_rawset ;
  Lua::_arlua_lua_rawseti     = &luaWrapper_lua_rawseti ;
  Lua::_arlua_lua_next        = &luaWrapper_lua_next ;

  Lua::_arlua_lua_pushnil     = &luaWrapper_lua_pushnil ;
  Lua::_arlua_lua_pushboolean = &luaWrapper_lua_pushboolean ;
  Lua::_arlua_lua_pushnumber  = &luaWrapper_lua_pushnumber ;
  Lua::_arlua_lua_pushlstring = &luaWrapper_lua_pushlstring ;
  Lua::_arlua_lua_pushstring  = &luaWrapper_lua_pushstring ;
  Lua::_arlua_lua_pushlightuserdata  = &luaWrapper_lua_pushlightuserdata ;
  Lua::_arlua_lua_pushcfunction  = &luaWrapper_lua_pushcfunction ;
  Lua::_arlua_lua_newuserdata = &luaWrapper_lua_newuserdata ;
  Lua::_arlua_lua_toboolean   = &luaWrapper_lua_toboolean ;
  Lua::_arlua_lua_tonumber    = &luaWrapper_lua_tonumber ;
  Lua::_arlua_lua_tostring    = &luaWrapper_lua_tostring ;
  Lua::_arlua_lua_touserdata  = &luaWrapper_lua_touserdata ;
  Lua::_arlua_lua_tocfunction = &luaWrapper_lua_tocfunction ;
  Lua::_arlua_lua_strlen      = &luaWrapper_lua_strlen ;
  Lua::_arlua_lua_isnumber    = &luaWrapper_lua_isnumber ;
  Lua::_arlua_lua_iscfunction = &luaWrapper_lua_iscfunction ;
  Lua::_arlua_lua_isfunction  = &luaWrapper_lua_isfunction ;
  Lua::_arlua_lua_isuserdata  = &luaWrapper_lua_isuserdata ;
  Lua::_arlua_lua_islightuserdata = &luaWrapper_lua_islightuserdata ;
  Lua::_arlua_lua_isstring    = &luaWrapper_lua_isstring ;
  Lua::_arlua_lua_isboolean   = &luaWrapper_lua_isboolean ;
  Lua::_arlua_lua_istable     = &luaWrapper_lua_istable ;
  Lua::_arlua_lua_isnil       = &luaWrapper_lua_isnil ;
  Lua::_arlua_lua_isnone      = &luaWrapper_lua_isnone ;
  Lua::_arlua_lua_isnoneornil = &luaWrapper_lua_isnoneornil ;
  Lua::_arlua_lua_type        = &luaWrapper_lua_type ;
  Lua::_arlua_lua_gettop      = &luaWrapper_lua_gettop ;
  Lua::_arlua_lua_settop      = &luaWrapper_lua_settop ;
  Lua::_arlua_lua_pop         = &luaWrapper_lua_pop ;
  Lua::_arlua_lua_pushvalue   = &luaWrapper_lua_pushvalue;
  Lua::_arlua_lua_remove      = &luaWrapper_lua_remove ;
  Lua::_arlua_lua_insert      = &luaWrapper_lua_insert ;
  Lua::_arlua_lua_replace     = &luaWrapper_lua_replace ;
  Lua::_arlua_lua_checkstack  = &luaWrapper_lua_checkstack ;
  Lua::_arlua_luaC_collectgarbage  = &luaWrapper_luaC_collectgarbage ;
  Lua::_arlua_lua_gc          = &luaWrapper_lua_gc ;



  Lua::_initSuccess=true;
  LuaBind::_initSuccess=true;
  LuaInfo::_initSuccess=true;

  cerr<<"fin initialization plugin LUA"<<endl;
  return true;
}



extern "C"
void LuaWrapper_destroy(void){
   Lua::_initSuccess=false;
   LuaBind::_initSuccess=false;
   LuaInfo::_initSuccess=false;

}




}//namespace AReVi
