// LI2/ENIB - AReVi-Contrib - sound plugin using ESD
//----------------------------------------------------------------------------
// file        : ExtContrib/SoundESD/soundESD.cpp
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
//
//  This plugin uses the ESD API (http://www.tux.org/~ricdude/EsounD.html).
//
//  The resulting dynamic library can be loaded at run-time
//  (fails if a dependency problem occurs).
//
//----------------------------------------------------------------------------

#include "AReVi/Contrib/soundRecorder.h"
#include "AReVi/Contrib/soundPlayer.h"

#include "soundPlayerESD.h"
#include "soundRecorderESD.h"


#include "AReVi/arSystem.h"

#include <esd.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <linux/soundcard.h>


namespace AReVi {

//----------------------------------------------------------------------------
// Class SoundPlayerESD
//----------------------------------------------------------------------------

// Playing
extern "C"
void *
SoundPlayer_openSoundOutput(SoundPlayer *player, const StlString &deviceName, int &deviceID)
{
SoundPlayerESD *playerESD=new SoundPlayerESD(player,deviceName);
deviceID=playerESD->getDeviceID();

if (deviceID!=-1)
  return(playerESD);
else
  {
  delete playerESD;
  return(NULL);
  }
}

extern "C"
bool
SoundPlayer_setOutputVolume(void *playerData)
{
if (playerData)
  return ((SoundPlayerESD *)playerData)->setVolume();
else
  return(false);
}

extern "C"
void
SoundPlayer_closeSoundPlayer(void *playerData)
{
if (playerData)
  delete ((SoundPlayerESD *)playerData);
}

extern "C"
bool SoundPlayer_setPlay(void *playerData, bool play)
{
if (playerData)
  return ((SoundPlayerESD *)playerData)->setPlay(play);
else
  return(false);
}

extern "C"
bool
SoundPlayer_isPlaying(const void *playerData)
{
if (playerData)
  return ((SoundPlayerESD *)playerData)->isPlaying();
else
  return(false);
}



extern "C"
int // bufferID
SoundPlayer_pushBuffer(void *playerData, ArConstRef<MemoryBlock> buffer)
{
if (playerData)
  return ((SoundPlayerESD *)playerData)->pushBuffer(buffer);
else
  return(-1);
}

extern "C"
unsigned
SoundPlayer_popBuffers(void *playerData, bool all)
{
if (playerData)
  return ((SoundPlayerESD *)playerData)->popBuffers(all);
else
  return(0);
}

extern "C"
unsigned
SoundPlayer_getNbBuffers(const void *playerData)
{
if (playerData)
  return ((SoundPlayerESD *)playerData)->getNbBuffers();
else
  return(0);
}

extern "C"
bool
SoundPlayer_setCurrentBuffer(void *playerData, int bufferID)
{
if (playerData)
  return ((SoundPlayerESD *)playerData)->setCurrentBuffer(bufferID);
else
  return(false);
}

extern "C"
int // bufferID
SoundPlayer_getCurrentBuffer(const void *playerData)
{
if (playerData)
  return ((SoundPlayerESD *)playerData)->getCurrentBuffer();
else
  return(-1);
}

extern "C"
unsigned int  // milliseconds
SoundPlayer_getDuration(const void *playerData)
{
if (playerData)
  return ((SoundPlayerESD *)playerData)->getDuration();
else
  return(0);
}

extern "C"
bool
SoundPlayer_setTime(void *playerData, unsigned int newTime)
{
if (playerData)
  return ((SoundPlayerESD *)playerData)->setTime(newTime);
else
  return(false);
}

extern "C"
unsigned int  // time in milliseconds
SoundPlayer_getTime(const void *playerData)
{
if (playerData)
  return ((const SoundPlayerESD *)playerData)->getTime();
else
  return(0);
}

//----------------------------------------------------------------------------
// Class SoundRecorderESD
//----------------------------------------------------------------------------

// Recording
extern "C"
void *
SoundRecorder_openSoundInput(SoundRecorder *recorder, const StlString &deviceName, int &deviceID)
{
SoundRecorderESD *recorderESD=new SoundRecorderESD(recorder,deviceName);
deviceID=recorderESD->getDeviceID();

if (deviceID!=-1)
  return(recorderESD);
else
  {
  delete recorderESD;
  return(NULL);
  }
}

extern "C"
void
SoundRecorder_closeSoundInput(void *recorderData)
{
if (recorderData)
  delete ((SoundRecorderESD *)recorderData);
}

extern "C"
bool SoundRecorder_setRecord(void *recorderData, bool record)
{
if (recorderData)
  return ((SoundRecorderESD *)recorderData)->setRecord(record);
else
  return(false);
}

extern "C"
bool // success
SoundRecorder_popPendingBuffer(void *recorderData, ArRef<MemoryBlock> bufferOut)
{
if (recorderData)
  return ((SoundRecorderESD *)recorderData)->popPendingBuffer(bufferOut);
else
  return(false);
}

extern "C"
unsigned
SoundRecorder_getNbPendingBuffers(const void *recorderData)
{
if (recorderData)
  return ((const SoundRecorderESD *)recorderData)->getNbPendingBuffers();
else
  return(0);
}


//-------- Initialization ----------------------------------------------------


bool // success
SoundPlayer_init(const StlVector<StlString> * /* params */)
{
SoundPlayer::_initSuccess=false;
SoundRecorder::_initSuccess=false;

SoundRecorder::_openSoundInputFnct=SoundRecorder_openSoundInput;
SoundRecorder::_closeSoundInputFnct=SoundRecorder_closeSoundInput;
SoundRecorder::_setRecordFnct=SoundRecorder_setRecord;
SoundRecorder::_popPendingBufferFnct=SoundRecorder_popPendingBuffer;
SoundRecorder::_getNbPendingBuffersFnct=SoundRecorder_getNbPendingBuffers;

// ------ Player  ------ //

SoundPlayer::_openSoundOutputFnct=&SoundPlayer_openSoundOutput;
SoundPlayer::_closeSoundOutputFnct=&SoundPlayer_closeSoundPlayer;
SoundPlayer::_setPlayFnct=&SoundPlayer_setPlay;
SoundPlayer::_isPlayingFnct=&SoundPlayer_isPlaying;

SoundPlayer::_setOutputVolumeFnct=&SoundPlayer_setOutputVolume;

SoundPlayer::_pushBufferFnct=&SoundPlayer_pushBuffer;
SoundPlayer::_popBuffersFnct=&SoundPlayer_popBuffers;
SoundPlayer::_getNbBuffersFnct=&SoundPlayer_getNbBuffers;
SoundPlayer::_setCurrentBufferFnct=&SoundPlayer_setCurrentBuffer;
SoundPlayer::_getCurrentBufferFnct=&SoundPlayer_getCurrentBuffer;
SoundPlayer::_getDurationFnct=&SoundPlayer_getDuration;
SoundPlayer::_setTimeFnct=&SoundPlayer_setTime;
SoundPlayer::_getTimeFnct=&SoundPlayer_getTime;



SoundRecorder::_initSuccess=true;
SoundPlayer::_initSuccess=true;
return(true);
}

void
SoundPlayer_destroy(void)
{
// Nothing to be done here!
}


//-------- Plugin Initialization ----------------------------------------------------


extern "C"
bool // success
SoundESD_init(const StlVector<StlString> *  params )
{
return(SoundPlayer_init(params));
}

extern "C"
void
SoundESD_destroy(void)
{
SoundPlayer_destroy();
}


} // namespace AReVi

//----------------------------------------------------------------------------
