// LI2/ENIB - AReVi-Contrib - sound plugin using ESD
//----------------------------------------------------------------------------
// file        : ExtContrib/SoundESD/soundPlayerESD.h
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
//
//  This plugin uses the ESD API (http://www.tux.org/~ricdude/EsounD.html).
//
//  The resulting dynamic library can be loaded at run-time
//  (fails if a dependency problem occurs).
//
//----------------------------------------------------------------------------

#ifndef AREVI_PLUGINS_CONTRIB_SOUNDPLAYERESD_H
#define AREVI_PLUGINS_CONTRIB_SOUNDPLAYERESD_H 1

#include "AReVi/arSystem.h"

#include <esd.h>

namespace AReVi {

class SoundPlayer;

#define BUFFER_MIN_SIZE_SCALE (256.0/11025) // Seems to be a good value!

//----------------------------------------------------------------------------
// Class SoundPlayerESD
//----------------------------------------------------------------------------

class SoundPlayerESD
{

public:
//-------- Construction / Destruction ----------------------------------------

  SoundPlayerESD(SoundPlayer *player, const StlString &/*deviceName*/);

  ~SoundPlayerESD(void); // not virtual

//-------- SoundPlayerESD inspectors -----------------------------------------
  int
  getDeviceID(void) const;

  int
  getMixerID(void) const;

  bool
  setVolume(void);

  bool
  setPlay(bool play);

  bool
  isPlaying(void) const;


//-------- Buffers control --------------------------------------------------

  int // bufferID
  pushBuffer(ArConstRef<MemoryBlock> buffer);

 // if allBuffers = true, clear all buffers
 // if allBuffers = false, remove only buffers already played
  unsigned
  popBuffers(bool allBuffers);

  int
  getCurrentBuffer(void) const;

  bool
  setCurrentBuffer(int bufferID);

  unsigned
  getNbBuffers(void) const;

//-------- Time control --------------------------------------------------

  unsigned int // time in millisecond
  getTime(void) const;

  bool //success
  setTime(unsigned int newTimePosition);

  unsigned int // total buffers duration
  getDuration(void) const;


// Forbiden
protected:

  void
  _pushBuffer(const char *data,unsigned dataSize);

  void
  _startThread(void);

  void //Blocking!
  _endThread(void);

  void
  _lock(void) const;

  void
  _unlock(void) const;

  void
  _wait(void) const;


//-------- Thread -----------------------------------------------------------
  void
  _playingThread(void);

  static
  void *
  _threadTask(void *);

  void
  _informPosition(void) const;

//-------- Member attributes ------------------------------------------------
protected:
  SoundPlayer *_player;
  int _mixerID;

  int _pipeID[2];

  // Buffers
  struct SoundData
  {
  int bufferID;
  char *bufferData;
  unsigned bufferSize;
  };

  int _bufferIDRef;

  // <---- Shared With Thread
  int _deviceID; // Only when is playing

  list<SoundData> _buffers;
  list<SoundData>::iterator _currentBuffer;
  unsigned _currentBufferPosition;

  // ----->

  // Thread & Mutex data
  pthread_t _th;
  pthread_mutex_t _mtx;
  pthread_cond_t _cond;
  bool _threadEnded;

  // State variables
  bool _playing;
  bool _paused;

  bool _autoErase;

  //! Buffers must be of fixed size. So data in pendingBufferData is
  //! is the rest lastFilled buffer.
  ArRef<MemoryBlock> _pendingBufferData;
  unsigned _pendingBufferSize;

  unsigned _bufferMinSize;
};

#endif // AREVI_PLUGINS_CONTRIB_SOUNDPLAYERESD_H

} // namespace AReVi

//----------------------------------------------------------------------------
