// LI2/ENIB - AReVi-Contrib - sound plugin using ESD
//----------------------------------------------------------------------------
// file        : ExtContrib/SoundESD/soundRecorderESD.h
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
//
//  This plugin uses the ESD API (http://www.tux.org/~ricdude/EsounD.html).
//
//  The resulting dynamic library can be loaded at run-time
//  (fails if a dependency problem occurs).
//
//----------------------------------------------------------------------------

#ifndef AREVI_EXTCONTRIB_SOUNDRECORDERESD_H
#define AREVI_EXTCONTRIB_SOUNDRECORDERESD_H 1

#include "AReVi/arSystem.h"

#include <esd.h>

namespace AReVi {

class SoundRecorder;

//----------------------------------------------------------------------------
// Class SoundRecorderESD
//----------------------------------------------------------------------------

class SoundRecorderESD
{

public:
//-------- Construction / Destruction ----------------------------------------

  SoundRecorderESD(SoundRecorder *recorder, const StlString &/*deviceName*/);

  ~SoundRecorderESD(void); // not virtual

//-------- SoundRecorderESD inspectors -----------------------------------------
  int
  getDeviceID(void) const;

  bool
  setRecord(bool record);

  bool
  isRecording(void) const;


//-------- Buffers control --------------------------------------------------

  bool // success
  popPendingBuffer(ArRef<MemoryBlock> bufferOut);

  unsigned
  getNbPendingBuffers(void) const;

// Forbiden
protected:

  void
  _startThread(void);

  void //Blocking!
  _endThread(void);

  void
  _lock(void) const;

  void
  _unlock(void) const;

  void
  _wait(void) const;


//-------- Thread -----------------------------------------------------------
  void
  _recordingThread(void);

  static
  void *
  _threadTask(void *);

  void
  _informPendingBuffer(void) const;

//-------- Member attributes ------------------------------------------------
protected:
  SoundRecorder *_recorder;

  int _pipeID[2];
  bool _esdRecord; // Record only that ESD play

  // Buffers
  struct SoundData
  {
  int bufferID;
  char *bufferData;
  unsigned bufferSize;
  };

  int _bufferIDRef;

  // <---- Shared With Thread
  int _deviceID; // Only when is recording

  list<SoundData> _buffers;
  unsigned _currentBufferPosition;
  SoundData _tempBuffer;
  // ----->

  // Thread & Mutex data
  pthread_t _th;
  pthread_mutex_t _mtx;
  pthread_cond_t _cond;
  bool _threadEnded;

  // State variables
  bool _recording;
};

#endif // AREVI_EXTCONTRIB_SOUNDRECORDERESD_H

} // namespace AReVi

//----------------------------------------------------------------------------
