// LI2/ENIB - AReVi-Contrib - Text To Speech plugin
//----------------------------------------------------------------------------
// file        : ExtContrib/Text2Speech/text2Speech.cpp
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
//
//  This plugin uses the Elan Speech API (http://www.elanspeech.com).
//
//  The resulting dynamic library can be loaded at run-time
//  (fails if a dependency problem occurs).
//
//----------------------------------------------------------------------------

#include "AReVi/Contrib/text2Speech.h"

#include "AReVi/arSystem.h"

#include <unistd.h>
#include <signal.h>
#include "nscube.h"
#include <fcntl.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <linux/soundcard.h>


#define TTS_SOUNDFORMAT  (UtilSound::MONO16)
// Uncomment the following line to show DEBUG messages.
//#define DEBUG_TTS   1

using namespace std;
namespace AReVi {

typedef struct AcapelaVoiceInfo
{
StlString voiceName;
int frequency;
int language;
};

static StlMap<StlString, AcapelaVoiceInfo> _voicesInfos;
StlMap<TTSCHANELID, nscHANDLE> _acapelaChannels;


// - Say Text (Block until all text is processed)
extern "C" ArRef<MemoryBlock>
Text2Speech_sayTextToMemory(TTSCHANELID channelID,
                            const StlString & str,const StlString & encoding,unsigned int  &freqOut,UtilSound::SoundFormat &formatOut);

// - Say Text (Callbacks are called during processing)
extern "C" bool
Text2Speech_sayTextToMemoryAsync(void *appAsyncData, const StlString & str,const StlString & encoding);

extern "C" void *
Text2Speech_sayTextToMemoryAsyncInit(TTSCHANELID channelID,
                                     Text2Speech::TTSSayTextToMemoryPerformer performer,
                                     Text2Speech::TTSSayTextToMemoryMethod method,
                                     void *oldAppAsyncData);
// - Abort processing
extern "C" bool
Text2Speech_abortProcessing(void *pInstanceData);
// - Manual update of NSCUBE loop process
extern "C" bool
Text2Speech_continueProcessing(void * dispatchHandle,int timeOut);


// ---------------------------------------------------
// - Get List Of Availables voices
// ---------------------------------------------------
extern "C" bool
Text2Speech_getVoices(StlVector<StlString> & voicesOut, const int &specificLanguage,const int &specificFrequency)
{
StlMap<StlString, AcapelaVoiceInfo>::const_iterator  it=_voicesInfos.begin();
while(it!=_voicesInfos.end())
  {
  if ((specificLanguage==-1 || specificLanguage==it->second.language) &&
          (specificFrequency==-1 || specificFrequency==it->second.frequency))
      voicesOut.push_back(it->first);

  it++;
  }
  return true;
}


// ---------------------------------------------------
// - Init List Of Availables voices
// ---------------------------------------------------
// NB : Pour le moment seul les voix au format PCM(1)
//      sont utilisables ...
static bool
Text2Speech_initVoicesInfos(void *serverHandle)
{
nscHSRV             hSrv=serverHandle;
nscRESULT           Result;     // Result of nsc functions
NSC_SRVINFO_DATA    SrvInfo;    // This structure describes informations about server
NSC_FINDVOICE_DATA  FindVoice;  //This structure describes a voice found by nscFindFirstVoice
nscHANDLE           hVoice;     //Handle to be used in subsequent calls to nscFindNextVoice

// Enumeration
if( (Result = nscGetServerInfo(hSrv, &SrvInfo))!=NSC_OK)
  {
  cerr << "Text2Speech_getVoices ERROR: nscGetServerInfo return " << Result << endl;
  return false;
  }

#ifdef DEBUG_TTS
  printf("Searching voice...   ");
#endif

if( (Result = nscFindFirstVoice(hSrv,NULL,0,0,0,&FindVoice,&hVoice))==NSC_OK)
  {
  if (FindVoice.nOutputCoding==1)
    {
    AcapelaVoiceInfo infos;
    infos.voiceName=FindVoice.cVoiceName;
    infos.frequency=FindVoice.nOutputSampleFreq;
    infos.language=FindVoice.nLanguage;
    _voicesInfos[FindVoice.cVoiceName]=infos;
    }

  while((Result = nscFindNextVoice(hVoice,&FindVoice))==NSC_OK)
    {
    // Check voice format
    #ifdef DEBUG_TTS
      cerr << "VOICE NAME =" << FindVoice.cVoiceName << endl;
      cerr << "  - output format =" << FindVoice.nOutputCoding << endl;
      cerr << "  - initial format =" << FindVoice.nInitialCoding << endl;
    #endif
    if (FindVoice.nOutputCoding==1)
      {
      AcapelaVoiceInfo infos;
      infos.voiceName=FindVoice.cVoiceName;
      infos.frequency=FindVoice.nOutputSampleFreq;
      infos.language=FindVoice.nLanguage;
      _voicesInfos[FindVoice.cVoiceName]=infos;
      }
    }
  }
else
  {
  cerr << "Text2Speech_getVoices ERROR: nscFindFirstVoice return  " << Result << endl;
  return false;
  }
nscCloseFindVoice(hVoice);
return true;
}

// ---------------------------------------------------
// - Get Current Voice Settings
// ---------------------------------------------------
extern "C" bool
Text2Speech_getVoiceSettings(const StlString &voiceName, unsigned int &freqOut, UtilSound::SoundFormat &formatOut, unsigned int &langIdOut)
{
StlMap<StlString, AcapelaVoiceInfo>::const_iterator it=_voicesInfos.find(voiceName);
if (it==_voicesInfos.end()) return(false);
formatOut=TTS_SOUNDFORMAT;
freqOut=it->second.frequency;
langIdOut=it->second.language;
return true;
}


// ---------------------------------------------------
// - Open a voice channel
// ---------------------------------------------------
extern "C" TTSCHANELID
Text2Speech_openVoice(void *serverHandle,void * dispatchHandle,const StlString& voiceName)
{
nscRESULT     Result;   /* Result of nsc functions */
nscCHANID ChId;         // Id Of channel returned by nscInitChannel
nscHANDLE     hTTS;     // Handle of channel returned by nscLockChannel

// Init channel
#ifdef DEBUG_TTS
  printf("Channel initialising with %s ... ",voiceName.c_str());
#endif

unsigned nbTry(10);

while(1)
  {
  if( (Result = nscInitChannel(serverHandle,voiceName.c_str(),0,0,dispatchHandle,&ChId))==NSC_OK)
    {
    #ifdef DEBUG_TTS
      printf("Channel initialised\n");
    #endif
    break;
    }
  else if (Result == NSC_ERR_CONNREJECT) // Hack cause in case of close/reopen channel, server has no the time to refresh...
    {
    ArSystem::sleep(300);
    nbTry--;
    if (!nbTry)
      {
      cerr << "Text2Speech_openVoice ERROR: Too many canals (licence limitation) : " << Result << endl;
      return (TTSCHANELID)-1;
      }
    }
  else
    {
    cerr << "Text2Speech_openVoice ERROR: nscInitChannel return  " << Result << endl;
    return (TTSCHANELID)-1;
    }
  }

// ----------------  Lock channel --------------------
#ifdef DEBUG_TTS
  printf("Channel locking...   ");
#endif
if( (Result = nscLockChannel(serverHandle,ChId,dispatchHandle,&hTTS))==NSC_OK)
  {
  #ifdef DEBUG_TTS
    printf("Channel locked\n");
  #endif
  }
else
  {
  cerr << "Text2Speech_openVoice ERROR: nscLockChannel return  " << Result << endl;
  return false;
  }
_acapelaChannels[ChId]=hTTS;
#ifdef DEBUG_TTS
  cerr << "Open  NbAcive=" << _acapelaChannels.size() << endl;
#endif
return(ChId);
}

// ---------------------------------------------------
// - Close a voice channel
// ---------------------------------------------------
extern "C" bool
Text2Speech_closeVoice(void *serverHandle,TTSCHANELID channelID)
{
nscRESULT     Result;   /* Result of nsc functions */
nscHANDLE     hTTS;           // Handle of channel returned by nscLockChannel

if (_acapelaChannels.find(channelID)==_acapelaChannels.end())
  {
  cerr << "Text2Speech_setVoiceParams ERROR: invalid channel ID " << channelID << endl;
  return(false);
  }
else
  {
  hTTS=_acapelaChannels[channelID];
  _acapelaChannels.erase(_acapelaChannels.find(channelID));
  }

#ifdef DEBUG_TTS
  cerr << "Close NbAcive=" << _acapelaChannels.size() << endl;
#endif

// ----------------  Unlock channel -------------------
#ifdef DEBUG_TTS
  printf("Channel unlocking...   ");
#endif

if( (Result = nscUnlockChannel(hTTS))==NSC_OK)
  {
  #ifdef DEBUG_TTS
    printf("Channel unlocked\n");
  #endif
  }
else
  {
  cerr << "Text2Speech_closeVoice ERROR: nscUnlockChannel return  " << Result << endl;
  return false;
  }


// Close channel
#ifdef DEBUG_TTS
  printf("Channel closing...   ");
#endif


if( (Result = nscCloseChannel(serverHandle,channelID))==NSC_OK)
  {
  #ifdef DEBUG_TTS
    printf("Channel closed\n");
  #endif

  return true;
  }
else
  {
  cerr << "Text2Speech_closeVoice ERROR: nscCloseChannel return  " << Result << endl;
  return false;
  }
}

// ---------------------------------------------------
// - Set voice Params (Volume, pitch, speed)
// ---------------------------------------------------
extern "C" bool
Text2Speech_setVoiceParams(TTSCHANELID channelID,
                           double pitch,double volume,double speed)
{
nscRESULT     Result;   // Result of nsc functions
nscCHANID     ChId=channelID; // Id Of channel returned by nscInitChannel
nscHANDLE     hTTS;           // Handle of channel returned by nscLockChannel

if (_acapelaChannels.find(ChId)==_acapelaChannels.end())
  {
  cerr << "Text2Speech_setVoiceParams ERROR: invalid channel ID " << channelID << endl;
  return(false);
  }
else
  hTTS=_acapelaChannels[ChId];


// ----------------  Set voice params --------------------
if (volume>-1)
  {
      if( (Result = nscSetParamChannel(hTTS, NSC_PRM_VOL, (int)(volume*32767)))==NSC_OK)
    {
      #ifdef DEBUG_TTS
        printf("nscSetParamChannel volume\n");
      #endif
    }
    else
    {
      cerr << "Text2Speech_setVoiceParams ERROR: nscSetParamChannel(volume) return  " << Result << endl;
      return false;
    }
  }
if (pitch>-1)
  {
    if( (Result = nscSetParamChannel(hTTS, NSC_PRM_PITCH, (int)(pitch*100)))==NSC_OK)
    {
      #ifdef DEBUG_TTS
        printf("nscSetParamChannel Pitch\n");
      #endif
    }
    else
    {
      cerr << "Text2Speech_setVoiceParams ERROR: nscSetParamChannel(Pitch) return  " << Result << endl;
      return false;
    }
  }
if (speed>-1)
  {
    if( (Result = nscSetParamChannel(hTTS, NSC_PRM_SPEED, (int)(speed*100)))==NSC_OK)
    {
      #ifdef DEBUG_TTS
        printf("nscSetParamChannel speed\n");
      #endif
    }
    else
    {
      cerr << "Text2Speech_setVoiceParams ERROR: nscSetParamChannel(speed) return  " << Result << endl;
      return false;
    }
  }

  return true;
}

// ---------------------------------------------------
// - Get voice Params (Volume, pitch, speed)
// ---------------------------------------------------
extern "C" bool
Text2Speech_getVoiceParams(TTSCHANELID channelID,
                            double &pitchOut,double &volumeOut,double &speedOut)
{
nscRESULT     Result;   // Result of nsc functions
nscCHANID     ChId=channelID; // Id Of channel returned by nscInitChannel
nscHANDLE     hTTS;           // Handle of channel returned by nscLockChannel

if (_acapelaChannels.find(ChId)==_acapelaChannels.end())
  {
  cerr << "Text2Speech_setVoiceParams ERROR: invalid channel ID " << channelID << endl;
  return(false);
  }
else
  hTTS=_acapelaChannels[ChId];

// ----------------  Set voice params --------------------
int pitch, volume, speed;

if( (Result = nscGetParamChannel(hTTS, NSC_PRM_VOL, &volume))==NSC_OK)
{
  #ifdef DEBUG_TTS
    printf("nscGetParamChannel volume\n");
  #endif
}
else
{
  cerr << "Text2Speech_setVoiceParams ERROR: nscGetParamChannel(volume) return  " << Result << endl;
  return false;
}

if( (Result = nscGetParamChannel(hTTS, NSC_PRM_PITCH, &pitch))==NSC_OK)
{
  #ifdef DEBUG_TTS
    printf("nscGetParamChannel Pitch\n");
  #endif
}
else
{
  cerr << "Text2Speech_setVoiceParams ERROR: nscGetParamChannel(Pitch) return  " << Result << endl;
  return false;
}

if( (Result = nscGetParamChannel(hTTS, NSC_PRM_SPEED, &speed))==NSC_OK)
{
  #ifdef DEBUG_TTS
    printf("nscGetParamChannel speed\n");
  #endif
}
else
{
  cerr << "Text2Speech_setVoiceParams ERROR: nscGetParamChannel(speed) return  " << Result << endl;
  return false;
}

volumeOut=volume/32767.0;
pitchOut=pitch/100.0;
speedOut=speed/100.0;

  return true;
}

// ---------------------------------------------------
// - Return list of supported encoding formats
// ---------------------------------------------------
extern "C"
void
Text2Speech_getEncodingSupportList(StlVector<StlString> &encodingFormatsOut)
{
encodingFormatsOut.push_back("ISO-8859-1");
encodingFormatsOut.push_back("ISO-8859-15");
encodingFormatsOut.push_back("UTF-8");
encodingFormatsOut.push_back("UCS-2");
encodingFormatsOut.push_back("UCS-2BE");
encodingFormatsOut.push_back("UCS-2LE");
encodingFormatsOut.push_back("UCS-4");
encodingFormatsOut.push_back("UTF-16");
encodingFormatsOut.push_back("UTF-16BE");
encodingFormatsOut.push_back("UTF-16LE");
encodingFormatsOut.push_back("UTF-7");
encodingFormatsOut.push_back("UTF-32");
encodingFormatsOut.push_back("UTF-32BE");
encodingFormatsOut.push_back("UTF-32LE");
}

//-------- Initialization ----------------------------------------------------

extern "C"
bool // success
Text2Speech_init(const StlVector<StlString> *params)
{
  Text2Speech::_initSuccess=false;

  #ifdef DEBUG_TTS
    cerr << endl << "Initializing elanTTS fonctions...";
  #endif

  Text2Speech::_sayTextToMemory       = &Text2Speech_sayTextToMemory;
  Text2Speech::_sayTextToMemoryAsync  = &Text2Speech_sayTextToMemoryAsync;
  Text2Speech::_sayTextToMemoryAsyncInit = &Text2Speech_sayTextToMemoryAsyncInit;
  Text2Speech::_getVoices             = &Text2Speech_getVoices;
  Text2Speech::_openVoice             = &Text2Speech_openVoice;
  Text2Speech::_closeVoice            = &Text2Speech_closeVoice;
  Text2Speech::_getVoiceSettings      = &Text2Speech_getVoiceSettings;
  Text2Speech::_getVoiceParams        = &Text2Speech_getVoiceParams;
  Text2Speech::_setVoiceParams        = &Text2Speech_setVoiceParams;
  Text2Speech::_continueProcessing    = &Text2Speech_continueProcessing;
  Text2Speech::_abortProcessing       = &Text2Speech_abortProcessing;
  Text2Speech::_getEncodingSupportList= &Text2Speech_getEncodingSupportList;

  _voicesInfos.clear();

  #ifdef DEBUG_TTS
    cerr << "OK" << endl;
  #endif

  StlString     IPadr="127.0.0.1";
  int           port = 0 ; // Let NSCube select default port (6666...)
  nscHSRV       hSrv;     /* Handle of a Speech Cube server context returned by nscCreateServerContext */
  nscRESULT     Result;   /* Result of nsc functions */
  nscHANDLE     hDispatch;/* Handle will be passed to the event dispatching functions and returned by nscCreateDispatcher */

// ---  Lecture des parametres ---
  if (params)
  {
    if (params->size()==1) // Juste le port
    {
      strToInt(port, (*params)[0]);
      #ifdef DEBUG_TTS
        cerr << "Text2Speech::Custom port=" << port << endl;
      #endif
    }
    else if (params->size()==2) // adress[0] + port[1]
    {
      strToInt(port, (*params)[1]);
      IPadr=(*params)[0];
      #ifdef DEBUG_TTS
        cerr << "Text2Speech::Custom server adress =" << IPadr << endl;
        cerr << "Text2Speech::Custom port=" << port << endl;
      #endif
    }
  }

  #ifdef DEBUG_TTS
    fprintf(stderr, "Connecting to %s ...   ",IPadr.c_str());
  #endif
  if( (Result = nscCreateServerContext(NSC_AF_INET,port,IPadr.c_str(),&hSrv))==NSC_OK)
  {
    #ifdef DEBUG_TTS
      fprintf(stderr, "Connected to %s\n",IPadr.c_str());
    #endif
  }
  else
  {
//----------------- Cas o le serveur 'nscube' n'est pas encore lanc�----------------------------//
    if (Result==NSC_SRV_NOTRUNNING)
    {
      cerr << endl << "Text2Speech_init ERROR: nscube server must be started before." << endl;
      return false;
    }
    else if (Result!=NSC_OK)
    {
      cerr << "Text2Speech_init ERROR: nscCreateServerContext return  " << Result << endl;
      return false;
    }
    #ifdef DEBUG_TTS
      else fprintf(stderr, "Connected to %s\n",IPadr.c_str());
    #endif
  }
//----------------- CONNECTION AU SERVEUR OK ... ----------------------------//
  /* Dispatch */
  #ifdef DEBUG_TTS
    printf("Creating dispatcher...   ");
  #endif
  if( (Result = nscCreateDispatcher(&hDispatch))==NSC_OK)
  {
    #ifdef DEBUG_TTS
      printf("Dispatcher created\n");
    #endif
  }
  else
  {
    cerr << "Text2Speech_init ERROR: nscCreateDispatcher return  " << Result << endl;
    return false;
  }
  Text2Speech::_serverHandle=hSrv;
  Text2Speech::_dispatchHandle=hDispatch;

  #ifdef DEBUG_TTS
    cerr << endl << "End of initialisation ==> TTS PLUGIN SUCCESS" << endl;
  #endif

  _voicesInfos.clear();

  Text2Speech::_initSuccess=Text2Speech_initVoicesInfos(Text2Speech::_serverHandle);


  NSC_SRVINFO_DATA    SrvInfo;    // This structure describes informations about server

if( (Result = nscGetServerInfo(hSrv, &SrvInfo))!=NSC_OK)
  {
  cerr << "Text2Speech_getVoices ERROR: nscGetServerInfo return " << Result << endl;
  return false;
  }

#ifdef DEBUG_TTS
cerr << "-------- TTS INIT(INFOS)--------- " << endl;
cerr << "nMaxNbVoice=" << SrvInfo.nMaxNbVoice << endl;
cerr << "nAuthRateCtrl=" << SrvInfo.nAuthRateCtrl<< endl;
cerr << "nAutMaxNbChannel=" << SrvInfo.nAutMaxNbChannel<< endl;
cerr << "nServerVersion=" << SrvInfo.nServerVersion<< endl;
cerr << "nServerType=" << SrvInfo.nServerType<< endl;
#endif


  return(Text2Speech::_initSuccess);
}

extern "C"
void
Text2Speech_destroy(void)
{
  if (Text2Speech::_initSuccess)
  {
// Ferme tous les canaux ouverts.
    StlVector<ArRef<Text2Speech> > listTTS;
    Text2Speech::CLASS()->getInstances(listTTS,true);

    for (int cpt=listTTS.size()-1;cpt>=0;cpt--)
      listTTS[cpt]->_closeAll();

    Text2Speech::_initSuccess=false;

    nscHSRV       hSrv=Text2Speech::_serverHandle;
    nscRESULT     Result;   /* Result of nsc functions */
    nscHANDLE     hDispatch=Text2Speech::_dispatchHandle;

// DeleteDispatcher
    #ifdef DEBUG_TTS
      fprintf(stderr, "Dispatcher deleting...   ");
    #endif
    if( (Result = nscDeleteDispatcher(hDispatch))==NSC_OK)
    {
      #ifdef DEBUG_TTS
        fprintf(stderr, "Dispatcher Deleted\n");
      #endif
    }
    else
    {
      cerr << "Text2Speech_destroy ERROR: nscDeleteDispatcher return  " << Result << endl;
      return;
    }

    _voicesInfos.clear();

  // ReleaseServer
    #ifdef DEBUG_TTS
      fprintf(stderr, "Disconnecting...   ");
    #endif
    if( (Result = nscReleaseServerContext(hSrv))==NSC_OK)
    {
      #ifdef DEBUG_TTS
        fprintf(stderr, "Disconnected\n");
      #endif
    }
    else
    {
      cerr << "Text2Speech_destroy ERROR: nscReleaseServerContext return  " << Result << endl;
      return;
    }
  }
}

} // end of namespace AReVi
//----------------------------------------------------------------------------
