// LI2/ENIB - AReVi-Contrib - Text To Speech plugin
//----------------------------------------------------------------------------
// file        : ExtContrib/Text2Speech/text2SpeechASync.cpp
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
//
//  This plugin uses the Elan Speech API (http://www.elanspeech.com).
//
//  The resulting dynamic library can be loaded at run-time
//  (fails if a dependency problem occurs).
//
//----------------------------------------------------------------------------

#include "AReVi/Contrib/text2Speech.h"

#include "AReVi/arSystem.h"

#include <unistd.h>
#include <signal.h>
#include "nscube.h"
#include <fcntl.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <unistd.h>

#define TTS_SOUNDFORMAT  (UtilSound::MONO16)
// Uncomment the following line to show DEBUG messages.
// #define DEBUG_TTS   1

using namespace std;
namespace AReVi {

extern StlMap<TTSCHANELID, nscHANDLE> _acapelaChannels;

typedef struct Text2Speech_AppInstanceDataAsync_Struct
{
  unsigned int _frequency;
  nscHANDLE     _hTTS;           // Handle of channel returned by nscLockChannel
  Text2Speech::TTSSayTextToMemoryMethod _asyncMethod;
  Text2Speech::TTSSayTextToMemoryPerformer _asyncPerformer;

} Text2Speech_AppInstanceDataASync;

// CALLBACKS
static int Text2Speech_CallBackSpeechData(const unsigned char *pData,
                       unsigned int cbDataSize,
                       PNSC_SOUND_DATA pSoundData,
                       void *pAppInstanceData);
static int Text2Speech_CallBackSpeechEvent(unsigned int nEventID,
                        unsigned int cbEventDataSize,
                        PNSC_EVENT_DATA pEventData,
                        void *pAppInstanceData);

extern "C" void *
Text2Speech_sayTextToMemoryAsyncInit(TTSCHANELID channelID,
                                     Text2Speech::TTSSayTextToMemoryPerformer performer,
                                     Text2Speech::TTSSayTextToMemoryMethod method,
                                     void *pData)
{
Text2Speech_AppInstanceDataASync *pAppInstanceData=(Text2Speech_AppInstanceDataASync *)pData;

if (channelID==(TTSCHANELID)-1)
  {
  if (pAppInstanceData) delete pAppInstanceData;
  return(NULL);
  }

nscHANDLE     hTTS;           // Handle of channel returned by nscLockChannel
NSC_FINDVOICE_DATA infosVoice; // This structure describes the current voice
nscRESULT     Result;   // Result of nsc functions

if (_acapelaChannels.find(channelID)==_acapelaChannels.end())
  {
  cerr << "Text2Speech_sayTextToMemoryAsyncInit ERROR: invalid channel ID " << channelID << endl;
  return(false);
  }
else
  hTTS=_acapelaChannels[channelID];

// ----------------  Get infos about the voice (Freq!) --------------------
unsigned int freqOut;
if( (Result = nscInfoVoice(hTTS,0,&infosVoice))==NSC_OK)
  {
  #ifdef DEBUG_TTS
  cerr <<  "---------------------" << endl << " INFOS VOICE  : " << endl;
  cerr << "Freq= " << infosVoice.nOutputSampleFreq << endl;
  cerr << "Language= " << infosVoice.nLanguage << endl;
  cerr << "SpeakerName= " << infosVoice.cSpeakerName << endl;
  #endif
  freqOut=infosVoice.nOutputSampleFreq;
  }
  else
  {
  cerr << "Text2Speech_sayTextToMemoryAsync ERROR: nscInfoVoiceText return  " << Result << endl;
  freqOut=0;
  return NULL;
  }

if (!pAppInstanceData)
  pAppInstanceData=new Text2Speech_AppInstanceDataASync();

pAppInstanceData->_asyncMethod=method;
pAppInstanceData->_asyncPerformer=performer;
pAppInstanceData->_frequency=freqOut;
pAppInstanceData->_hTTS=hTTS;
return(pAppInstanceData);
}

extern "C" bool
Text2Speech_sayTextToMemoryAsync(void *pData, const StlString & str, const StlString & encoding)
{
nscRESULT     Result;   // Result of nsc functions
NSC_EXEC_DATA ExecData;       // Informations using by a call to nscExecChannel

Text2Speech_AppInstanceDataASync *pAppInstanceData=(Text2Speech_AppInstanceDataASync *)pData;
if (!pAppInstanceData)
 return(false);

nscHANDLE     hTTS=pAppInstanceData->_hTTS;

// ----------------  Add Text to channel --------------------
  static int IdText = 1;
//   IdText ++;
  #ifdef DEBUG_TTS
  printf("Text %i adding...   ", IdText);
  #endif

  // If begin by ISO-8859-1 ... standard call
  if (encoding.substr(0,10)=="ISO-8859-1")
    {
    #ifdef DEBUG_TTS
    cerr << "Use default fonction" << endl;
    #endif
    Result = nscAddText(hTTS,str.c_str(),(void *)IdText);
    }
  else
    {
    #ifdef DEBUG_TTS
    cerr << "Use encoding : " << encoding << endl;
    #endif
    Result = nscAddTextEx(hTTS,encoding.c_str(),str.c_str(),str.size(), (void *)IdText);
    }

  if( Result ==NSC_OK)
  {
    #ifdef DEBUG_TTS
    printf("Text %i added\n", IdText);
    #endif
  }
  else
  {
    if (Result!=NSC_ALREADY_STARTED)
    {
      cerr << "Text2Speech_sayTextToMemoryAsync ERROR: nscAddText return  " << Result << endl;
      return(false);
    }
    #ifdef DEBUG_TTS
    else fprintf(stderr,"\nWARNING : TTS already started...\n");
    #endif
  }

// ----------------  ExecChannel Setup -------------------
  ExecData.pfnSpeechData=Text2Speech_CallBackSpeechData;
  ExecData.pfnSpeechEvent=Text2Speech_CallBackSpeechEvent;
  //ExecData.ulEventFilter=NSC_EVTBIT_TEXT | NSC_EVTBIT_STOP_DONE | NSC_EVTBIT_PHO_SYNCH | NSC_EVTBIT_WORD_SYNCH | NSC_EVTBIT_TTSERROR;
  ExecData.ulEventFilter=NSC_EVTBIT_TEXT | NSC_EVTBIT_STOP_DONE | NSC_EVTBIT_TTSERROR;
  ExecData.bEventSynchroReq=0; // 1 For realtime
  ExecData.vsSoundData.uiSize=0;
  ExecData.vsSoundData.pSoundBuffer=NULL;
  ExecData.pAppInstanceData=pAppInstanceData;


  #ifdef DEBUG_TTS
  printf("Channel stating...\n");
  #endif
// ----------------  StartChannel -------------------
  if( (Result = nscStartChannel(hTTS,&ExecData))==NSC_OK)
  {
    #ifdef DEBUG_TTS
    cerr << "... nscStartChannel OK." << endl;
    #endif
  }
  else
  {
    if (Result!=NSC_ALREADY_STARTED)
    {
      cerr << "Text2Speech_sayTextToMemoryAsync ERROR: nscStartChannel return  " << Result << endl;
      return false;
    }
    #ifdef DEBUG_TTS
      cerr << "Text2Speech_sayTextToMemoryAsync WARNING: nscStartChannel already started" << Result << endl;
    #endif
  }

return true;
}

// ---------------------------------------------------
// - Abort processing
// ---------------------------------------------------
extern "C" bool
Text2Speech_abortProcessing(void *pInstanceData)
{
  nscRESULT           Result;     // Result of nsc functions
  Text2Speech_AppInstanceDataASync * data=(Text2Speech_AppInstanceDataASync *)pInstanceData;
  // ----------------  Exit channel exec channel -------------------
  #ifdef DEBUG_TTS
    printf("Exiting channel execution ...   ");
  #endif

  if( (Result = nscExitChannel(data->_hTTS))==NSC_OK)
  {
    #ifdef DEBUG_TTS
      printf("Text2Speech_abortProcessing :: Channel exec stopped\n");
    #endif
  }
  else
  {
    if (Result<0)
    {
      cerr << "Text2Speech_abortProcessing ERROR: nscExitChannel return  " << Result << endl;
      return false;
    }
    #ifdef DEBUG_TTS
    else
      cerr << "Text2Speech_abortProcessing WARNING: nscExitChannel return  " << Result << endl;
    #endif
    return false;
  }
  return true;
}

// ---------------------------------------------------
// - Manual update of NSCUBE loop process
// ---------------------------------------------------
extern "C" bool
Text2Speech_continueProcessing(void * dispatchHandle,int timeOut)
{
   nscRESULT           Result;     // Result of nsc functions

// NSC_TIMEOUT_EVENT n'est pas une erreur
  do
  {
    Result=nscGetandProcess((nscHANDLE)dispatchHandle, timeOut);
    //if (Result==NSC_ERR_INVALID_HANDLE) return false;
    if (Result!=NSC_TIMEOUT_EVENT && Result!=NSC_OK)
      return false;

  }
  while(Result==NSC_OK);
  return true;
}

int
Text2Speech_CallBackSpeechData(const unsigned char *pData,
                       unsigned int cbDataSize,
                       PNSC_SOUND_DATA /* pSoundData */,
                       void *pAppInstanceData)
{
  Text2Speech_AppInstanceDataASync * data=(Text2Speech_AppInstanceDataASync *)pAppInstanceData;

  if (data->_asyncMethod && data->_asyncPerformer)
  {
    #ifdef DEBUG_TTS
    if (cbDataSize<=128)
      { // openAL ....
      cerr << "    WARNING :  DATA BUFFER SAMPLE SIZE < 128 =" << cbDataSize << endl;
      }
    #endif
    ArRef<RigidMemoryBlock> memoryBlock=RigidMemoryBlock::NEW((char *)pData,cbDataSize,false);

    #ifdef DEBUG_TTS
    cerr << "- --- -- --ASYNC MODE=> CALL PERFORMER::METHOD " << data->_asyncPerformer << endl;
    #endif
    (data->_asyncPerformer.c_ptr()->*(data->_asyncMethod))(memoryBlock,data->_frequency,TTS_SOUNDFORMAT,false);
  }

  return 0;
}



int
Text2Speech_CallBackSpeechEvent(unsigned int nEventID,
                        unsigned int ,
#ifdef DEBUG_TTS
                        PNSC_EVENT_DATA  pEventData ,
#else
                        PNSC_EVENT_DATA,
#endif
                        void *pAppInstanceData)
{
//  cerr << "EventID=" << nEventID << endl;
  switch(nEventID)
  {
    #ifdef DEBUG_TTS
    case NSC_EVID_TEXT_STARTED :
    {
        NSC_EVENT_DATA_TextStarted *pEVTS = (NSC_EVENT_DATA_TextStarted *)pEventData;
        printf("IdText %i : Synthesis started.\n", (int)pEVTS->pUserData);
        break;
    }
    #endif
    case NSC_EVID_TEXT_DONE :
    {
//       Text2Speech_AppInstanceDataASync * data=(Text2Speech_AppInstanceDataASync *)pAppInstanceData;
//
//       if (data->_asyncMethod && data->_asyncPerformer)
//       {
//         nscRESULT     Result;   // Result of nsc functions
//
//         // ----------------  Exit channel exec channel -------------------
//         #ifdef DEBUG_TTS
//           printf("NSC_EVID_TEXT_DONE : Exiting channel execution ...   ");
//         #endif
//
//         if( (Result = nscExitChannel(data->_hTTS))==NSC_OK)
//         {
//           #ifdef DEBUG_TTS
//             printf("Channel exec stopped\n");
//           #endif
//         }
//         else
//         {
//           if (Result<0)
//           {
//             cerr << "Text2Speech_CallBackSpeechEvent ERROR: nscExitChannel return  " << Result << endl;
//             return false;
//           }
//           #ifdef DEBUG_TTS
//           else
//             cerr << "Text2Speech_CallBackSpeechEvent WARNING: nscExitChannel return  " << Result << endl;
//           #endif
//         }
//       }
//       break;
      Text2Speech_AppInstanceDataASync * data=(Text2Speech_AppInstanceDataASync *)pAppInstanceData;


        if (data->_asyncMethod && data->_asyncPerformer)
        {
          #ifdef DEBUG_TTS
            cerr << "NSC_EVID_TEXT_DONE =>- --- -- --ASYNC MODE=> CALL PERFORMER::METHOD " << data->_asyncPerformer << endl;
          #endif
          //(data->_asyncPerformer.c_ptr()->*(data->_asyncMethod))(memoryBlock,data->_frequency,TTS_SOUNDFORMAT,true);
          (data->_asyncPerformer.c_ptr()->*(data->_asyncMethod))(MemoryBlock::nullRef(),0,TTS_SOUNDFORMAT,true);

        }
      break;

    }
    case NSC_EVID_PHO_SYNCH :
    {
//         cerr << "NSC_PHO_SYNCH " << endl;
//         Text2Speech_AppInstanceDataASync * data=(Text2Speech_AppInstanceDataASync *)pAppInstanceData;
//         NSC_EVENT_DATA_PhoSynch *phoEvent=(NSC_EVENT_DATA_PhoSynch *)pEventData;

//         if (data->_asyncMethod && data->_asyncPerformer)
//         {
//         cerr << "ELAN Pho=" << phoEvent->cEngine_Phoneme << "   IPA=" << phoEvent->uiIpa_Phoneme << endl;
         // delete data;
//         }
        break;
    }
    case NSC_EVID_WORD_SYNCH :
    {
//       NSC_EVENT_DATA_WordSynch *wordEvent=(NSC_EVENT_DATA_WordSynch *)pEventData;
//         cerr << "NSC_EVID_WORD_SYNCH  (Index=" << wordEvent->uiWordPos << ")" << endl;
        break;
    }

    case NSC_EVID_TTSERROR:
    {
//         cerr << "NSC_EVID_TTSERROR" << endl;
        break;
    }
    case NSC_EVID_STOP_DONE :
    {
//         cerr << "NSC_EVID_STOP_DONE" << endl;
        break;
    }
  }
  return 0;
}

} // end of namespace AReVi
//----------------------------------------------------------------------------
