// LI2/ENIB - AReVi-Contrib - Text To Speech plugin
//----------------------------------------------------------------------------
// file        : ExtContrib/Text2Speech/text2SpeechSync.cpp
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
//
//  This plugin uses the Elan Speech API (http://www.elanspeech.com).
//
//  The resulting dynamic library can be loaded at run-time
//  (fails if a dependency problem occurs).
//
//----------------------------------------------------------------------------

#include "AReVi/Contrib/text2Speech.h"

#include "AReVi/arSystem.h"

#include <unistd.h>
#include <signal.h>
#include "nscube.h"
#include <fcntl.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <unistd.h>

#define TTS_SOUNDFORMAT  (UtilSound::MONO16)
// Uncomment the following line to show DEBUG messages.
//#define DEBUG_TTS   1

using namespace std;
namespace AReVi {

extern StlMap<TTSCHANELID, nscHANDLE> _acapelaChannels;

typedef struct Text2Speech_AppInstanceDataSync_Struct
{
  // Stocke les buffers.
  StlVector<ArRef<MemoryBlock> > _buffers;
  ArRef<MemoryBlock> _mainBuffer;
  unsigned int _frequency;
} Text2Speech_AppInstanceDataSync;

// CALLBACKS
static int Text2Speech_CallBackSpeechData(const unsigned char *pData,
                       unsigned int cbDataSize,
                       PNSC_SOUND_DATA pSoundData,
                       void *pAppInstanceData);
static int Text2Speech_CallBackSpeechEvent(unsigned int nEventID,
                        unsigned int cbEventDataSize,
                        PNSC_EVENT_DATA pEventData,
                        void *pAppInstanceData);

//---------------------------------------------------------
extern "C" ArRef<MemoryBlock>
Text2Speech_sayTextToMemory(TTSCHANELID channelID,
                            const StlString & str, const StlString & encoding ,
                            unsigned int  &freqOut,UtilSound::SoundFormat &formatOut)
{
  nscRESULT     Result;   // Result of nsc functions
  nscCHANID     ChId=channelID; // Id Of channel returned by nscInitChannel
  nscHANDLE     hTTS;           // Handle of channel returned by nscLockChannel
  NSC_EXEC_DATA ExecData;       // Informations using by a call to nscExecChannel
  NSC_FINDVOICE_DATA infosVoice; // This structure describes the current voice

  ArRef<MemoryBlock> memoryBlock=MemoryBlock::nullRef(); // Stockage m�oire du son

if (_acapelaChannels.find(ChId)==_acapelaChannels.end())
  {
  cerr << "Text2Speech_sayTextToMemory ERROR: invalid channel ID " << channelID << endl;
  return(memoryBlock);
  }
else
  hTTS=_acapelaChannels[ChId];



// ----------------  Add Text to channel --------------------
  int IdText = 1;
  #ifdef DEBUG_TTS
  printf("Text %i adding...   ", IdText);
  #endif
  // If begin by ISO-8859-1 ... standard call
  if (encoding.substr(0,10)=="ISO-8859-1")
    {
    #ifdef DEBUG_TTS
    cerr << "Use default fonction" << endl;
    #endif
    Result = nscAddText(hTTS,str.c_str(),(void *)IdText);
    }
  else
    {
    #ifdef DEBUG_TTS
    cerr << "Use encoding : " << encoding << endl;
    #endif
    Result = nscAddTextEx(hTTS,encoding.c_str(),str.c_str(),str.size(), (void *)IdText);
    }

  if( Result ==NSC_OK)
  {
    #ifdef DEBUG_TTS
    printf("Text %i added\n", IdText);
    #endif
  }
  else
  {
    cerr << "Text2Speech_sayTextToMemory ERROR: nscAddText return  " << Result << endl;
    return memoryBlock;
  }

// ----------------  Get infos about the voice (Freq!) --------------------
  if( (Result = nscInfoVoice(hTTS,0,&infosVoice))==NSC_OK)
  {
    #ifdef DEBUG_TTS
    cerr <<  "---------------------" << endl << " INFOS VOICE  : " << endl;
    cerr << "Freq= " << infosVoice.nOutputSampleFreq << endl;
    cerr << "Language= " << infosVoice.nLanguage << endl;
    cerr << "SpeakerName= " << infosVoice.cSpeakerName << endl;
    #endif
    freqOut=infosVoice.nOutputSampleFreq;
    formatOut=TTS_SOUNDFORMAT;
  }
  else
  {
    cerr << "Text2Speech_sayTextToMemory ERROR: nscInfoVoiceText return  " << Result << endl;
    freqOut=0;
    formatOut=TTS_SOUNDFORMAT;
    return memoryBlock;
  }

// ----------------  ExecChannel Setup -------------------
  ExecData.pfnSpeechData=Text2Speech_CallBackSpeechData;
  ExecData.pfnSpeechEvent=Text2Speech_CallBackSpeechEvent;
  ExecData.ulEventFilter=NSC_EVTBIT_TEXT;
//   ExecData.ulEventFilter=NSC_EVTBIT_TEXT | NSC_EVTBIT_STOP_DONE | NSC_EVTBIT_PHO_SYNCH | NSC_EVTBIT_WORD_SYNCH | NSC_EVTBIT_TTSERROR;
  ExecData.bEventSynchroReq=0; // (1 For realtime)
  ExecData.vsSoundData.uiSize=0;
  ExecData.vsSoundData.pSoundBuffer=NULL;

  Text2Speech_AppInstanceDataSync appInstanceData;
  appInstanceData._frequency=freqOut;
  appInstanceData._buffers.clear();
  ExecData.pAppInstanceData=&appInstanceData;


  #ifdef DEBUG_TTS
  printf("Channel executing...\n");
  #endif
// ----------------  ExecChannel -------------------
  if( (Result = nscExecChannel(hTTS,&ExecData))==NSC_OK)
  {
    memoryBlock=appInstanceData._mainBuffer;
  }
  else
    cerr << "Text2Speech_sayTextToMemory ERROR: nscExecChannel return  " << Result << endl;
  return memoryBlock;
}

int
Text2Speech_CallBackSpeechData(const unsigned char *pData,
                       unsigned int cbDataSize,
                       PNSC_SOUND_DATA /* pSoundData */,
                       void *pAppInstanceData)
{
  StlVector<ArRef<MemoryBlock> > *pBuffers=&((Text2Speech_AppInstanceDataSync *)pAppInstanceData)->_buffers;
  ArRef<MemoryBlock> memoryBlock=MemoryBlock::NEW();
  memoryBlock->setSize(cbDataSize); // 1s Buffer
  pBuffers->push_back(memoryBlock);

  memcpy(memoryBlock->accessMemory(),pData, cbDataSize);

  return 0;
}



int Text2Speech_CallBackSpeechEvent(unsigned int nEventID,
                        unsigned int ,
                        PNSC_EVENT_DATA,
                        void *pAppInstanceData)
{
  switch(nEventID)
  {
    case NSC_EVID_TEXT_DONE :
    {
      Text2Speech_AppInstanceDataSync * data=(Text2Speech_AppInstanceDataSync *)pAppInstanceData;

      unsigned totalSize=0,currentPos=0;
      for (unsigned cpt=0;cpt<data->_buffers.size();cpt++)
        totalSize+=data->_buffers[cpt]->getSize();


      ArRef<MemoryBlock> memoryBlock=MemoryBlock::NEW();
      memoryBlock->setSize(totalSize); // 1s Buffer

      for (unsigned cpt=0;cpt<data->_buffers.size();cpt++)
      {
        memcpy(memoryBlock->accessMemory()+currentPos,
              data->_buffers[cpt]->accessMemory(),
              data->_buffers[cpt]->getSize());
        currentPos+=data->_buffers[cpt]->getSize();
      }
      data->_mainBuffer=memoryBlock;
      break;
    }
    case NSC_EVID_PHO_SYNCH :
    {
//         cerr << "NSC_PHO_SYNCH " << endl;
        break;
    }
    case NSC_EVID_WORD_SYNCH :
    {
//         cerr << "NSC_EVID_WORD_SYNCH  " << endl;
        break;
    }
     case NSC_EVID_TTSERROR:
    {
//         cerr << "NSC_EVID_TTSERROR" << endl;
        break;
    }
    case NSC_EVID_STOP_DONE :
    {
//       cerr << "NSC_EVID_STOP_DONE" << endl;
        break;
    }
  }
  return 0;
}

} // end of namespace AReVi
//----------------------------------------------------------------------------
