// LI2/ENIB - AReVi-Contrib - XML management plugin
//----------------------------------------------------------------------------
// file        : ExtContrib/XmlParser/xmlParser.cpp
// author(s)   : E. Cazeaux, E. Maffre
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
//
//  This plugin uses the libxml2 API (http://xmlsoft.org).
//
//  The resulting dynamic library can be loaded at run-time
//  (fails if a dependency problem occurs).
//
//----------------------------------------------------------------------------

#include "AReVi/Contrib/xmlParser.h"
#include "AReVi/Utils/memoryStream.h"
#include "AReVi/Utils/memoryBlock.h"
#include "AReVi/arSystem.h"

#include <iostream>
#include <libxml/parser.h>
#include <libxml/tree.h>
#include <libxml/encoding.h>
#include <libxml/xinclude.h>
#include <libxml/nanohttp.h>
#include <libxml/xpath.h>

#ifdef XMLPARSER_ENABLEXSLT
  #include <libxslt/xsltInternals.h>
  #include <libxslt/transform.h>
  #include <libxslt/xsltutils.h>
#endif


namespace AReVi {


extern "C"
bool
XmlParser_newDocument(void ** docPtr){
    *docPtr = xmlNewDoc ((xmlChar *)"1.0");
    if ( *docPtr )
      return true;

    return false;
}


static
void xmlStructuredErrorFunc(void * userData, xmlErrorPtr pError)
{
StlVector<XmlParser::XmlParseError> *errors((StlVector<XmlParser::XmlParseError> *)userData);
XmlParser::XmlParseError error;
error.line=pError->line;
error.message=pError->message?pError->message:"";
error.file=pError->file?pError->file:"Stream";
error.level=pError->level;
errors->push_back(error);
}

extern "C"
bool
XmlParser_parseFile(StlString fileName,void ** docPtr, bool processInclude, StlVector<XmlParser::XmlParseError> &errors )
{
// Initilasation du proxy
  StlString proxy;
  int port;
  ArSystem::getHTTPProxy(proxy,port);
  if (proxy.size())
  {
    xmlNanoHTTPScanProxy(("http://"+proxy+":"+intToStr(port)+"/").c_str());
  }
  else
    xmlNanoHTTPScanProxy(NULL); // supprime les info sur le proxy.

  xmlSetStructuredErrorFunc(&errors,xmlStructuredErrorFunc);

// On parse le document
  *docPtr = xmlParseFile(fileName.c_str());
  xmlSetStructuredErrorFunc(NULL,NULL);

  if ( *docPtr )
  {
    if (processInclude)
    {
      xmlXIncludeProcess((xmlDocPtr)(*docPtr));
    }
    return true;
  }
  return false;
}

extern "C"
bool
XmlParser_parseDoc(char *buffer,void ** docPtr, bool processInclude, StlVector<XmlParser::XmlParseError> &errors )
{
  // Initilasation du proxy
  StlString proxy;
  int port;
  ArSystem::getHTTPProxy(proxy,port);
  if (proxy.size())
  {
    xmlNanoHTTPScanProxy(("http://"+proxy+":"+intToStr(port)+"/").c_str());
  }
  else
    xmlNanoHTTPScanProxy(NULL); // supprime les info sur le proxy.


  xmlSetStructuredErrorFunc(&errors,xmlStructuredErrorFunc);
// On parse le document
  *docPtr = xmlParseDoc((xmlChar *)buffer);
  xmlSetStructuredErrorFunc(NULL,NULL);
  if ( *docPtr )
  {
    if (processInclude)
    {
      xmlXIncludeProcess((xmlDocPtr)(*docPtr));
    }
    return true;
  }
  return false;
}

extern "C"
bool
XmlParser_saveFile(StlString fileName,void *docPtr, const char *encoding)
{
  if (!docPtr) return false;

  xmlIndentTreeOutput = 1;

  if (encoding!=NULL) xmlSaveFormatFileEnc(fileName.c_str(), (xmlDocPtr)docPtr,encoding,1);
  else
  {
    xmlSaveFormatFile(fileName.c_str(), (xmlDocPtr)docPtr, 1);
  }

	return true;
}

extern "C"
bool
XmlParser_saveDoc(ArRef<AbstractOStream> stream,void *docPtr, const char *encoding)
{
  if (!docPtr) return false;

  xmlIndentTreeOutput = 1;
  int doc_txt_len;
  char *doc_txt_ptr=NULL;

  if (encoding!=NULL)
  {
    xmlDocDumpFormatMemoryEnc((xmlDocPtr)docPtr, // Doc to dump
                              (xmlChar **)(void *)&doc_txt_ptr, // mem to store
                              &doc_txt_len, // size of allocation
                              encoding,1);
  }
  else
  {
    xmlDocDumpFormatMemory((xmlDocPtr)docPtr, // Doc to dump
                            (xmlChar **)(void *)&doc_txt_ptr, // mem to store
                            &doc_txt_len, // size of allocation
                            1);
  }

  if (doc_txt_ptr && doc_txt_len>0)
  {
    bool ret=stream->writeData(doc_txt_ptr,doc_txt_len);
    xmlFree(doc_txt_ptr);
    doc_txt_ptr=NULL;
    return ret;
  }
  else
    return false;
}

extern "C"
void
XmlParser_freeDoc(void * docPtr)
{
xmlFreeDoc((xmlDocPtr)docPtr);
}

extern "C"
void *
XmlParser_getRoot(void * docPtr)
{
return (void*)xmlDocGetRootElement((xmlDocPtr)docPtr);
}

extern "C"
bool
XmlParser_setRoot(void * docPtr, void * node)
{
xmlDocSetRootElement((xmlDocPtr)docPtr, (xmlNodePtr)node);
return(true);
}

extern "C"
bool XmlParser_matchPath(void * docPtr, const StlString &expre, StlList<XmlParser::XmlContent> &nodesOut)
{
xmlXPathContextPtr xpathCtx;
xmlXPathObjectPtr xpathObj;

/* Create xpath evaluation context */
xpathCtx = xmlXPathNewContext((xmlDocPtr)docPtr);
if(xpathCtx == NULL)
  {
  fprintf(stderr,"Error: unable to create new XPath context\n");
  return(false);
  }

/* Evaluate xpath expression */
xpathObj = xmlXPathEvalExpression((const xmlChar*)expre.data(), xpathCtx);
if(xpathObj == NULL)
  {
  fprintf(stderr,"Error: unable to evaluate xpath expression \"%s\"\n", expre.c_str());
  xmlXPathFreeContext(xpathCtx);
  return(false);
  }

xmlNodeSetPtr nodes=xpathObj->nodesetval;
XmlParser::XmlContent content;

// Backup all finded nodes.
int size;
int i;
size = (nodes) ? nodes->nodeNr : 0;

/*
  * NOTE: the nodes are processed in reverse order, i.e. reverse document
  *       order because xmlNodeSetContent can actually free up descendant
  *       of the node and such nodes may have been selected too ! Handling
  *       in reverse order ensure that descendant are accessed first, before
  *       they get removed. Mixing XPath and modifications on a tree must be
  *       done carefully ! => No Prob here cause no modif ...
  */
for(i = size - 1; i >= 0; i--)
  {
  assert(nodes->nodeTab[i]);

  if (nodes->nodeTab[i]->type==XML_ATTRIBUTE_NODE) // is an  attrib
    {
    content.node=ArRef<XmlNode>((XmlNode *)(nodes->nodeTab[i]->parent->_private));
    content.property=(const char *)nodes->nodeTab[i]->name;
    }
  else
    {
    content.node=ArRef<XmlNode>((XmlNode *)(nodes->nodeTab[i]->_private));
    content.property="";
    }

  nodesOut.push_back(content);
  }

/* Cleanup of XPath data */
xmlXPathFreeObject(xpathObj);
xmlXPathFreeContext(xpathCtx);

return(true);
}


// XSLT
// Always fail if lib is not available
extern "C"
bool
XmlParser_applyXSLT(ArRef<AbstractIStream> xmlInput,
                    ArRef<AbstractIStream> styleSheet,
                    ArRef<AbstractOStream> output,
                    bool processInclude,
                    StlVector<XmlParser::XmlParseError> &errors)
{
#ifdef XMLPARSER_ENABLEXSLT
xmlDocPtr *xmlInputDoc,styleSheetDoc;

// Parse input xml file
ArRef<MemoryBlock> block=MemoryBlock::NEW();
ArRef<MemoryOStream> memStream=MemoryOStream::NEW(block,0);
memStream->copyDataFromInput(xmlInput);
memStream->stream() << '\0';

if (!XmlParser_parseDoc((char *)block->getMemory(),
                        (void **)(void *)&xmlInputDoc,processInclude,errors))
  {
  output->setErrorMessage("Fail to parse xmlInput");
  return(false);
  }


// Parse input stylesheet file
block=MemoryBlock::NEW();
memStream=MemoryOStream::NEW(block,0);
memStream->copyDataFromInput(styleSheet);
memStream->stream() << '\0';


if (!XmlParser_parseDoc((char *)block->getMemory(),
                        (void **)(void *)&styleSheetDoc,processInclude,errors))
  {
  output->setErrorMessage("Fail to parse styleSheet structure");
  xmlFreeDoc((xmlDocPtr)xmlInputDoc);
  return(false);
  }

// Apply transform
xsltStylesheetPtr styleSheetPtr=xsltParseStylesheetDoc((xmlDocPtr)styleSheetDoc);

if (!styleSheetPtr)
  {
  output->setErrorMessage("Fail to parse styleSheet, invalid data");
  xmlFreeDoc((xmlDocPtr)xmlInputDoc);
  xmlFreeDoc((xmlDocPtr)styleSheetDoc);
  return(false);
  }

int oldSubstituteEntitiesDefault = xmlSubstituteEntitiesDefault(1);
int oldLoadExtDtdDefaultValue = xmlLoadExtDtdDefaultValue;
xmlLoadExtDtdDefaultValue =1;

const char *params[16 + 1];
params[0]=NULL;
xmlDocPtr resultDoc = xsltApplyStylesheet(styleSheetPtr, (xmlDocPtr)xmlInputDoc, params); // FIXME : Use params?

bool result;
if (resultDoc)
  {
  //result=XmlParser_saveDoc(output,(void *)resultDoc, outputEncoding);
  xmlChar *doc_txt_ptr=NULL;
  int doc_txt_len=0;

  result=(xsltSaveResultToString(&doc_txt_ptr, &doc_txt_len, resultDoc,styleSheetPtr)!=-1);

  if (!result || !doc_txt_ptr || doc_txt_len<=0)
    output->setErrorMessage("Fail to save result of transformation");
  else
    {
    result=output->writeData(doc_txt_ptr,doc_txt_len);
    xmlFree(doc_txt_ptr);
    doc_txt_ptr=NULL;
    }
  }
else
  {
  output->setErrorMessage("Fail to apply style sheet");
  result=false;
  }

// Free All
xmlFreeDoc((xmlDocPtr)xmlInputDoc);
xsltFreeStylesheet(styleSheetPtr);
// xmlFreeDoc((xmlDocPtr)styleSheetDoc); // Not to free=>cleaned by xsltFreeStylesheet
xmlFreeDoc(resultDoc);
xmlLoadExtDtdDefaultValue = oldLoadExtDtdDefaultValue;
xmlSubstituteEntitiesDefault(oldSubstituteEntitiesDefault);
return(result);

#else // libxslt is not available ...
  cerr << "XmlParser::XSLT is not available (you may get xslt library)" << endl;
  return(false);
#endif
}


//--------------------
// Construit une reference sur un XmlNode
extern "C"
ArRef<XmlNode>
XmlNode_newRef(ArRef<XmlNode> node, void *nodePtr)
{
// Use private data to store AReVi XmlNode reference.
((xmlNode *)nodePtr)->_private=node.c_ptr();
return(node);
}


extern "C"
void
XmlNode_freeString(void * charPtr)
{
    xmlFree(charPtr);
}

extern "C"
char *
XmlNode_getName(void * node){
    return (char*)((xmlNode*)node)->name;
}

extern "C"
char *
XmlNode_getString(void * node){
    return (char*)xmlNodeListGetString(((xmlNode*)node)->doc, (xmlNode*)node, 1);
}

extern "C"
char *
XmlNode_getTextString(void * node){
    return (char*)xmlNodeGetContent((xmlNode*)node);
}

extern "C"
char *
XmlNode_getProperty(void * node,  StlString propName){
    return (char*)xmlGetProp((xmlNode*)node,(const xmlChar*)propName.data()) ;
}

extern "C"
void *
XmlNode_getNext(void * node){
    return (void*)((xmlNode*)node)->next;
}


extern "C"
void *
XmlNode_getPrevious(void * node){
    return (void*)((xmlNode*)node)->prev;
}


extern "C"
void *
XmlNode_getFirstChild(void * node){
    return (void*)((xmlNode*)node)->children;
}

extern "C"
void *
XmlNode_getLastChild(void * node){
    return (void*)xmlGetLastChild((xmlNode*)node);
}


extern "C"
void *
XmlNode_getParent(void * node){
    return (void*)((xmlNode*)node)->parent;
}

extern "C"
void *
XmlNode_getDoc(void * node){
    return (void*)((xmlNode*)node)->doc;
}

extern "C"
void XmlNode_setName(void * node, const char *name){
  xmlNodeSetName((xmlNode*)node,(const xmlChar *)name);
}

extern "C"
void * XmlNode_newNode(void * ns, const char *name){
  return xmlNewNode((xmlNs *)ns,(const xmlChar *)name);
}

extern "C"
void * XmlNode_copyNode(const void * node, int recursive)
{
  return xmlCopyNode((const xmlNodePtr)node,recursive);
}

extern "C"
void   XmlNode_freeNode(void * node, bool unlinkFirst)
{
  if (unlinkFirst)  xmlUnlinkNode((xmlNode *)node);
  xmlFreeNode((xmlNode *)node);
}

extern "C"
void * XmlNode_addChild(void * parent, void *node){
  return xmlAddChild((xmlNode*)parent,(xmlNode*)node);
}

extern "C"
void * XmlNode_addText(void * parent, const char *text){
  return xmlAddChild((xmlNode*)parent,(xmlNode*)xmlNewText((const xmlChar*)text));
}

extern "C"
void * XmlNode_addComment(void * parent, const char *text){
  return xmlAddChild((xmlNode*)parent,(xmlNode*)xmlNewComment((const xmlChar*)text));
}

extern "C"
bool XmlNode_isText(void * node){
  return xmlNodeIsText((xmlNode*)node);
}

extern "C"
bool XmlNode_isComment(void * node){
  return (((xmlNode*)node)->type==XML_COMMENT_NODE);
}

extern "C"
void
XmlNode_setString(void * node, const char *texte){

// cerr << "Is text=" << xmlNodeIsText((xmlNode*)node) << endl;
// xmlAddChild((xmlNode*)node,xmlNewText((const xmlChar*)texte));
xmlNodeSetContent((xmlNode*)node, (const xmlChar*)texte);
}


extern "C"
void XmlNode_setProperty(void * node, const StlString &propName, const StlString &value)
{
  xmlSetProp((xmlNode*)node, (const xmlChar*)propName.c_str(), (const xmlChar*)value.c_str());
}

extern "C"
void XmlNode_addProperty(void * node, const StlString &propName, const StlString &value)
{
  xmlNewProp((xmlNode*)node,(const xmlChar*)propName.c_str(),(const xmlChar*)value.c_str());
}

extern "C"
bool XmlNode_hasProperty(void * node, const StlString &propName)
{
  return xmlHasProp((xmlNode*)node, (const xmlChar*)propName.c_str())!=NULL;
}

extern "C"
bool XmlNode_getPropertyList(void *node,StlVector<StlString> &propList)
{
  if (!node) return false;
  xmlNode *n=(xmlNode*)node;
  struct _xmlAttr *attrib=n->properties;
  while(attrib)
  {
    propList.push_back((const char *)attrib->name);
    attrib=attrib->next;
  }
  return true;
}


extern "C"
StlString XmlNode_UTF8ToISO(const StlString &utf8String)
{
  int lenUTF = utf8String.length();
  int lenISO = lenUTF; // lenISO<=lenUTF!
  unsigned char *stringISO=(unsigned char *)malloc(lenISO+1);
  UTF8Toisolat1(stringISO, &lenISO, (unsigned char *)utf8String.c_str(), &lenUTF);

  stringISO[lenISO]='\0';
  StlString out=(char *)stringISO;
  free(stringISO);
  return out;
}

extern "C"
StlString XmlNode_ISOToUTF8(const StlString &isoString)
{
  int lenISO = isoString.length();
  int lenUTF = lenISO*2; // lenUTF<=lenISO*2!
  unsigned char *stringUTF=(unsigned char *)malloc(lenUTF+1);
  isolat1ToUTF8(stringUTF, &lenUTF, (unsigned char *)isoString.c_str(), &lenISO);

  stringUTF[lenUTF]='\0';
  StlString out=(char *)stringUTF;
  free(stringUTF);
  return out;
}


//--------------------------------------------------
// FLOW OUT
//--------------------------------------------------
extern "C"
int XmlFlowOut_xmlNodeDump(void * bufferPtr, void *nodePtr,const int &level, const char *encoding)
{
  xmlNodeDumpOutput((xmlOutputBufferPtr)bufferPtr,
                      ((xmlNode*)nodePtr)->doc,
                      (xmlNode*)nodePtr,
                      level, // Level for indententation
                      1, // Format
                      encoding); // Encoding

  return 1;
}

extern "C"
int XmlFlowOut_xmlSaveFileTo(void * bufferPtr, void * docPtr, const char *encoding)
{
  return xmlSaveFileTo((xmlOutputBufferPtr)bufferPtr,(xmlDocPtr)docPtr,encoding);
}


extern "C"
void *XmlFlowOut_xmlOutputBufferCreateFile(FILE *file)
{
  return xmlOutputBufferCreateFile(file,NULL); // NULL=> Encoding Handler
}

extern "C"
int XmlFlowOut_xmlOutputBufferFlush(void *bufferPtr)
{
  return xmlOutputBufferFlush((xmlOutputBufferPtr)bufferPtr);
}

extern "C"
int XmlFlowOut_xmlOutputBufferClose(void *bufferPtr)
{
  return xmlOutputBufferClose((xmlOutputBufferPtr)bufferPtr);
}

//-------- Initialization ----------------------------------------------------
extern "C"
bool // success
XmlParser_init(const StlVector<StlString> * /* params */)
{
  XmlParser::_initSuccess=false;

// XMLNode
  XmlNode::_newRef=&XmlNode_newRef;
  XmlNode::_freeString=&XmlNode_freeString;
  XmlNode::_getName=&XmlNode_getName;
  XmlNode::_getString=&XmlNode_getString;
  XmlNode::_getTextString=&XmlNode_getTextString;
  XmlNode::_getProperty=&XmlNode_getProperty;
  XmlNode::_getNext=&XmlNode_getNext;
  XmlNode::_getPrevious=&XmlNode_getPrevious;
  XmlNode::_getFirstChild=&XmlNode_getFirstChild;
  XmlNode::_getLastChild=&XmlNode_getLastChild;
  XmlNode::_getParent=&XmlNode_getParent;
  XmlNode::_getDoc=&XmlNode_getDoc;

  XmlNode::_setName=&XmlNode_setName;
  XmlNode::_newNode=&XmlNode_newNode;
  XmlNode::_copyNode=&XmlNode_copyNode;
  XmlNode::_freeNode=&XmlNode_freeNode;
  XmlNode::_addChild=&XmlNode_addChild;
  XmlNode::_addText=&XmlNode_addText;
  XmlNode::_addComment=&XmlNode_addComment;
  XmlNode::_isText=&XmlNode_isText;
  XmlNode::_isComment=&XmlNode_isComment;
  XmlNode::_setProperty=&XmlNode_setProperty;
  XmlNode::_setString=&XmlNode_setString;
  XmlNode::_addProperty=&XmlNode_addProperty;
  XmlNode::_hasProperty=&XmlNode_hasProperty;
  XmlNode::_getPropertyList=&XmlNode_getPropertyList;

  XmlNode::_UTF8ToISO=XmlNode_UTF8ToISO;
  XmlNode::_ISOToUTF8=XmlNode_ISOToUTF8;

// XMLParser
  XmlParser::_newDocument=&XmlParser_newDocument;
  XmlParser::_parseFile=&XmlParser_parseFile;
  XmlParser::_parseDoc=&XmlParser_parseDoc;
  XmlParser::_saveFile=&XmlParser_saveFile;
  XmlParser::_saveDoc=&XmlParser_saveDoc;
  XmlParser::_freeDoc=&XmlParser_freeDoc;
  XmlParser::_getRoot=&XmlParser_getRoot;
  XmlParser::_setRoot=&XmlParser_setRoot;
  XmlParser::_matchPath=&XmlParser_matchPath;
  XmlParser::_applyXSLT=&XmlParser_applyXSLT;

// XMLFlowOut
  XmlFlowOut::_xmlNodeDump=&XmlFlowOut_xmlNodeDump;
  XmlFlowOut::_xmlSaveFileTo=&XmlFlowOut_xmlSaveFileTo;
  XmlFlowOut::_xmlOutputBufferCreateFile=&XmlFlowOut_xmlOutputBufferCreateFile;
  XmlFlowOut::_xmlOutputBufferFlush=&XmlFlowOut_xmlOutputBufferFlush;
  XmlFlowOut::_xmlOutputBufferClose=&XmlFlowOut_xmlOutputBufferClose;

// XML lib init
  xmlInitParser();
  xmlKeepBlanksDefault(0);
  XmlParser::_initSuccess=true;

// Initilasation du proxy
  xmlNanoHTTPInit();

  return(true);
}


extern "C"
void
XmlParser_destroy(void)
{
#ifdef XMLPARSER_ENABLEXSLT
  // Cleanup local variable from libxslt
  xsltCleanupGlobals();
#endif

// Cleanup local variable from libxml2
xmlNanoHTTPCleanup();
xmlCleanupParser();
}
//-------- Event management --------------------------------------------------

}
//----------------------------------------------------------------------------

