// LI2/ENIB - AReVi - ZLib compression plugin
//----------------------------------------------------------------------------
// file        : ExtContrib/ZLib/zlibTools.cpp
// author(s)   : E. Cazeaux
//----------------------------------------------------------------------------
// Copyright (C) 2002-2006  LI2/ENIB
//   Laboratoire d'Ingenierie Informatique (LI2)
//   Ecole Nationale d'Ingenieurs de Brest (ENIB)
//   CS 73862 - 29238 BREST Cedex 3 - France
//   Tel: +33(0)298 05 89 89, Fax: +33(0)298 05 89 79, e-mail: li2@enib.fr
//
// This software package is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
// This software package is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
// General Public License (file LICENSE-LGPL) for more details.
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
//
//  This plugin uses the ZLib API (http://www.gzip.org/zlib/).
//
//  The resulting dynamic library can be loaded at run-time
//  (fails if a dependency problem occurs).
//
//----------------------------------------------------------------------------

#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <zlib.h>
#include <bzlib.h>

#include "AReVi/Utils/memoryBlock.h"

namespace AReVi {
//----------------------------------------------------------------------------

static
bool // success
ZLib_compress(ArConstRef<MemoryBlock> uncompressedBuffer,ArRef<MemoryBlock> compressedBuffer)
{
// compressed buffer must be 0.1% larger than uncompressed + 12bytes
if (compressedBuffer->getSize()<(uncompressedBuffer->getSize()*1.1+12))
  {
  compressedBuffer->setSize((unsigned int)(uncompressedBuffer->getSize()*1.1)+12);
  }



unsigned long compressedSize=compressedBuffer->getSize();
if (compress((Bytef *)compressedBuffer->accessMemory(),&compressedSize,
             (const Bytef *)uncompressedBuffer->getMemory(),uncompressedBuffer->getSize())==Z_OK)
  {
  compressedBuffer->setSize(compressedSize);
  return true;
  }
/*
unsigned int compressedSize=compressedBuffer->getSize();
if (BZ2_bzBuffToBuffCompress((char *)compressedBuffer->accessMemory(),&compressedSize,
             (char *)uncompressedBuffer->getMemory(),uncompressedBuffer->getSize(),
             9,0,30)==0)
  {
  compressedBuffer->setSize(compressedSize);
  return true;
  }*/
else
  {
  compressedBuffer->setErrorMessage("Zlib compression error");
  return false;
  }
}

static
bool // success
ZLib_compress2(ArConstRef<MemoryBlock> uncompressedBuffer,
              ArRef<MemoryBlock> compressedBuffer,
              int level)
{
if (level==Z_DEFAULT_COMPRESSION || (level>=0 && level<=9))
  {
  // compressed buffer must be 0.1% larger than uncompressed + 12bytes
  if (compressedBuffer->getSize()<(uncompressedBuffer->getSize()*1.1+12))
    {
    compressedBuffer->setSize((unsigned int)(uncompressedBuffer->getSize()*1.1)+12);
    }

  unsigned long compressedSize=compressedBuffer->getSize();

  if (compress2((Bytef *)compressedBuffer->accessMemory(),&compressedSize,
              (const Bytef *)uncompressedBuffer->getMemory(),uncompressedBuffer->getSize(),level)==Z_OK)
    {
    compressedBuffer->setSize(compressedSize);
    return true;
    }
  else
    {
    compressedBuffer->setErrorMessage("Zlib compression error");
    return false;
    }
  }
else // bad compression level
  {
  compressedBuffer->setErrorMessage("Compression level must between [-1,9]");
  return false;
  }
}

static
bool // success (uncompressedBuffer must be large enough to store uncompressed data)
ZLib_uncompress(ArConstRef<MemoryBlock> compressedBuffer,ArRef<MemoryBlock> uncompressedBuffer)
{

unsigned long uncompressedSize=uncompressedBuffer->getSize();
if (uncompress((Bytef *)uncompressedBuffer->accessMemory(),&uncompressedSize,
             (const Bytef *)compressedBuffer->getMemory(),compressedBuffer->getSize())==Z_OK)
  {
  uncompressedBuffer->setSize(uncompressedSize);
  return true;
  }/*
unsigned int uncompressedSize=uncompressedBuffer->getSize();
if (BZ2_bzBuffToBuffDecompress((char *)uncompressedBuffer->accessMemory(),&uncompressedSize,
             (char *)compressedBuffer->getMemory(),compressedBuffer->getSize(),
             0,0)==0)
  {
  uncompressedBuffer->setSize(uncompressedSize);
  return true;
  }*/
else
  {
  uncompressedBuffer->setErrorMessage("Zlib decompression error");
  return false;
  }
}

//-------- Initialization ----------------------------------------------------
extern "C"
bool // success
ZLib_init(const StlVector<StlString> * )
{
MemoryBlock::_initSuccess=false;
MemoryBlock::_compress=ZLib_compress;
MemoryBlock::_compress2=ZLib_compress2;
MemoryBlock::_uncompress=ZLib_uncompress;
MemoryBlock::_initSuccess=true;
return true;
}

extern "C"
void
ZLib_destroy(void)
{
MemoryBlock::_initSuccess=false;
}

} // namespace AReVi

//----------------------------------------------------------------------------
